var embedMethods = {
    getProjectFromReactModalStore: (taskId) => {
        if (!taskId) {
            // console.log('[getProjectFromReactModalStore] taskId is empty')
            return null;
        }

        try {
            let linksRootElement = document.querySelector('.ReactModalPortal .simplebar-content div[class*="List-root"]');

            if (!linksRootElement) {
                return null;
            }

            let dataMap = {
                Task: {},
                List: {},
                Folder: {},
                Space: {},
                Human: {}
            }
            let linksStateKeys = Object.keys(linksRootElement);
            // console.log(linksRootElement[linksStateKeys[0]].child.memoizedProps.children);
            linksRootElement[linksStateKeys[0]].child.memoizedProps.children.forEach((item) => {
                let ent = item.props.option?.option?.ent;

                if (ent) {
                    dataMap[ent.typeName][ent.externalID.split(':')[1]] = ent;
                }

                // Map for CustomIds
                if (ent && ent.typeName === 'Task') {
                    dataMap.Human[ent.humanID] = ent;
                }
            });

            let isCustom = !!~taskId.indexOf('-');
            let task = isCustom ? dataMap.Human[taskId] : dataMap.Task[taskId];

            // if (!task && !isCustom) {
            //     let ids = Object.keys(dataMap.Task);
            //     let matchParentTaskIndex = Object.keys(dataMap.Task).indexOf(taskId.slice(0, 4));
            //
            //     if (matchParentTaskIndex > -1) {
            //         task = dataMap.Task[ids[matchParentTaskIndex]];
            //     }
            // }

            if (!task) {
                return null;
            }

            const getParent = (id, type) => {
                return dataMap[type][id];
            }

            let dataset = {category: null, subcategory: null};
            let parent = null;
            let parentAsset = task.parentAssets[0];
            let parentTask = null;
            let parseId = (id) => {
                return id.split(':')[1];
            }

            do {
                parent = getParent(parseId(parentAsset.externalID), parentAsset.typeName);
                parentAsset = parent ? parent.parentAssets[0] : null;

                if (parent && parent.typeName === 'List') {
                    dataset.subcategory = parseId(parent.externalID);
                }

                if (parent && parent.typeName === 'Folder') {
                    dataset.category = parseId(parent.externalID);
                }

                if (parent && parent.typeName === 'Task') {
                    parentTask = parent;
                }
            } while (parent && !~['Folder', 'Space'].indexOf(parent.typeName));

            // Board case
            const id = parentTask && parentTask.externalID ? parentTask.externalID : task.externalID;
            if (!dataset.category && dataset.subcategory && id) {
                let boardTaskElement = document.querySelector(`.cu-panel-board__task[data-task="${parseId(id)}"]`);
                let boardTaskChildElement = boardTaskElement && boardTaskElement.querySelector('.cu-dashboard-board-card');

                dataset.category = boardTaskChildElement && boardTaskChildElement.dataset.category ? boardTaskChildElement.dataset.category : null;
            }

            // List case
            if (!dataset.category && dataset.subcategory && id) {
                let listRowElement = document.querySelector(`.cu-task-row[data-test="task-list__task-row"][data-id="${parseId(id)}"]`);
                let listRowChildrenElement = listRowElement && listRowElement.querySelector('.cu-task-row__container');
                let listRow2Element = document.querySelector(`.cu-task-row__container[data-subcategory="${dataset.subcategory}"]`);
                let set = listRowChildrenElement && listRowChildrenElement.dataset.category ? listRowChildrenElement.dataset : null;
                let set2 = listRow2Element && listRow2Element.dataset.category ? listRow2Element.dataset : null;
                let data = (set || set2 || {});

                dataset.category = data.category ? data.category : null;
            }

            // console.log('Parent: ', parent, dataset, parentTask);

            return dataset.category && dataset.subcategory ? dataset : null;
        } catch (err) {
            // console.log(err);

            return null;
        }
    },
    getClickupApiSettings: () => {
        let API_PATH = null;
        let team = null;

        for (const key in window.dataLayer) {
            const item = window.dataLayer[key];
            if (!API_PATH && item.hasOwnProperty('workspaceID') && item.hasOwnProperty('shardId')) {
                API_PATH = item.shardId;
                team = item.workspaceID
            }
        }

        return {
            API_PATH,
            token: localStorage.getItem('id_token'),
            team
        };
    }
};

/**
 * We are needing create embeds file for integration(asana, clickup, ...)
 * with name <integration_name>EmbedScripts.js on src/app/contentScript/v2/adaptor folder
 *
 * Inside we are declaring: var = embedMethods = {} and setting some methods for integration.
 * Gulp will build <integration_name>EmbedMethods.js + embedMethodScript.js to <integration_name>EmbedMethods.js
 * and then content script will be append this file in page.
 *
 *
 * We can call some embed method: embedMethod('someNameMethod', [someArgs])
 *
 * 'someNameMethod' is required
 * [someArgs] is optional
 *
 * */

var dictionary = null;
var DIC = (key) => {
    // console.log('Dictionary: ', dictionary, ' Key: ', key);
    return dictionary[key] || key;
};

document.addEventListener('embedMethod', (event) => {
    if ((!event.detail) || (event.detail && !event.detail.name)) {
        return;
    }

    try {
        if (!dictionary) {
            dictionary = event.detail.dict ? JSON.parse(event.detail.dict) : {};
        }

        const result = embedMethods[event.detail.name].apply(this, event.detail.args ? JSON.parse(event.detail.args) : null);

        document.documentElement.setAttribute(event.detail.name.toLowerCase(), JSON.stringify(result));
    } catch (err) {
        console.log(`[EV][embedMethod] ${event.detail.name} is fail `, err);
    }
});
