var embedMethods = {
    getProjectId: () => {
        try {
            let taskElement = document.querySelector('div[role="row"][data-testid*="TableRow"][data-hovercard-subject-tag], ' +
                'div[data-testid="board-view-column-card"][data-hovercard-subject-tag]');

            if (!taskElement && document.querySelector('div.roadmap-table-cell')) {
                let taskRowSvgIndicator = document.querySelector('div[role="row"][data-testid*="TableRow"] svg[aria-label="Open issue"]');
                let parentTaskElement = taskRowSvgIndicator && taskRowSvgIndicator.closest('div[role="row"][data-testid*="TableRow"]');
                taskElement = parentTaskElement && parentTaskElement.querySelector('div.roadmap-table-cell:nth-child(2)');
            }

            if (!taskElement) {
                let taskRowSvgIndicator = document.querySelector('div[role="row"][data-testid*="TableRow"] svg[aria-label="Open issue"]')
                taskElement = taskRowSvgIndicator && taskRowSvgIndicator.closest('div[role="row"][data-testid*="TableRow"]');
            }

            if (!taskElement) {
                return null;
            }

            let memoizedProps = taskElement[Object.keys(taskElement)[0]].child.memoizedProps;
            if (!memoizedProps) {
                return null;
            }

            let taskState = memoizedProps.hasOwnProperty('row') ?
                memoizedProps.row.original : memoizedProps.hasOwnProperty('item') ?
                    memoizedProps.item : null;
            if (!taskState) {
                return null;
            }

            return taskState.contentRepositoryId ? 'gh:' + taskState.contentRepositoryId : null;
        } catch (err) {
            return null;
        }
    },
    getTask: (selector) => {
        try {
            if (!selector) {
                return null;
            }

            let taskRowElement = document.querySelector(selector);

            if (!taskRowElement) {
                return null;
            }

            let memoizedProps = taskRowElement[Object.keys(taskRowElement)[0]].child.memoizedProps;

            if (!memoizedProps) {
                return null;
            }

            let taskState = memoizedProps.hasOwnProperty('row') ?
                memoizedProps.row.original : memoizedProps.hasOwnProperty('item') ?
                    memoizedProps.item : null;

            if (!taskState || taskState.contentType === "DraftIssue") {
                return null;
            }

            return {
                name: taskState.columnData.Title.value.title.raw,
                taskId: 'gh:' + taskState.content.id,
                url: taskState.content.url,
                projectId: 'gh:' + taskState.contentRepositoryId
            }
        } catch (err) {
            return null;
        }
    },
    getDetailsTask: () => {
        try {
            let taskHeaderChildElement = document.querySelector('div[class*="Overlay__StyledOverlay"] span[data-testid="side-panel-focus-target"]')
                || document.querySelector('aside span[data-testid="side-panel-focus-target"]')
                || document.querySelector('div[role="dialog"] header[class*="Box"], aside span[data-testid="side-panel-focus-target"] + div > header');
            let taskHeaderElement = taskHeaderChildElement && taskHeaderChildElement.parentElement;

            if (!taskHeaderElement) {
                return null;
            }

            let taskState = taskHeaderElement[Object.keys(taskHeaderElement)[0]].child?.memoizedProps?.item
                || taskHeaderElement[Object.keys(taskHeaderElement)[0]].memoizedProps.children[1]?.props?.sidePanelState?.item
                || taskHeaderElement[Object.keys(taskHeaderElement)[0]].memoizedProps.children[1]?.props?.children[1]?.props?.sidePanelState?.item


            if (!taskState) {
                return null;
            }

            return {
                name: taskState.columnData.Title.value.title.raw,
                taskId: 'gh:' + taskState.content.id,
                url: taskState.content.url,
                projectId: 'gh:' + taskState.contentRepositoryId,
                contentType: taskState.contentType
            }
        } catch (err) {
            console.log(err);
            return null;
        }
    }
}

/**
 * We are needing create embeds file for integration(asana, clickup, ...)
 * with name <integration_name>EmbedScripts.js on src/app/contentScript/v2/adaptor folder
 *
 * Inside we are declaring: var = embedMethods = {} and setting some methods for integration.
 * Gulp will build <integration_name>EmbedMethods.js + embedMethodScript.js to <integration_name>EmbedMethods.js
 * and then content script will be append this file in page.
 *
 *
 * We can call some embed method: embedMethod('someNameMethod', [someArgs])
 *
 * 'someNameMethod' is required
 * [someArgs] is optional
 *
 * */

var dictionary = null;
var DIC = (key) => {
    // console.log('Dictionary: ', dictionary, ' Key: ', key);
    return dictionary[key] || key;
};

document.addEventListener('embedMethod', (event) => {
    if ((!event.detail) || (event.detail && !event.detail.name)) {
        return;
    }

    try {
        if (!dictionary) {
            dictionary = event.detail.dict ? JSON.parse(event.detail.dict) : {};
        }

        const result = embedMethods[event.detail.name].apply(this, event.detail.args ? JSON.parse(event.detail.args) : null);

        document.documentElement.setAttribute(event.detail.name.toLowerCase(), JSON.stringify(result));
    } catch (err) {
        console.log(`[EV][embedMethod] ${event.detail.name} is fail `, err);
    }
});
