var embedMethods = {
    getInlineTask: (href) => {
        try {
            if (!href) {
                return null;
            }

            const issueRowElement = document.querySelector(`main a[href="${href}"]`);
            const issueModel = issueRowElement[Object.keys(issueRowElement)[0]].child.memoizedProps.model;

            return {
                id: issueModel.id,
                title: issueModel.title,
                projectId: issueModel.projectId || issueModel.teamId,
                isArchived: issueModel.isArchived
            }
        } catch (err) {
            console.log('[LinearEmbedMethods][getSectionRowTask] err: ', err);
            return null;
        }
    },
    getDetailsTask: () => {
        try {
            const detailsElement = document.querySelector('#reach-skip-nav + div div:not([style]) header > div div + span, #reach-skip-nav + div div:not([style]) header > div > span');
            const issueModel = detailsElement[Object.keys(detailsElement)[0]].child.memoizedProps.issue;

            return {
                id: issueModel.id,
                title: issueModel.title,
                projectId: issueModel.projectId || issueModel.teamId,
                isArchived: issueModel.isArchived
            }
        } catch (err) {
            console.log('[LinearEmbedMethods][getDetailsTask] err: ', err);
            return null;
        }
    },
    getConnectedAccountId: () => {
        try {
            const applicationStore = JSON.parse(sessionStorage.getItem('ApplicationStore'));
            return 'li:' + applicationStore.currentUserId;
        } catch (err) {
            console.log('[LinearEmbedMethods][getConnectedAccountId] err: ', err);
            return null;
        }
    },
    getProjectIdFromProjectRow: (url) => {
        try {
            if (!url) {
                return null;
            }

            const projectRowElement = document.querySelector(`a[data-dnd-group-targeted][href="${url}"], a[data-board-item][href="${url}"]`);
            return 'li:' + projectRowElement[Object.keys(projectRowElement)[0]].child.memoizedProps.model.id;
        } catch (err) {
            console.log('[LinearEmbedMethods][getProjectIdFromProjectRow] err: ', err);
            return null;
        }
    }
}

/**
 * We are needing create embeds file for integration(asana, clickup, ...)
 * with name <integration_name>EmbedScripts.js on src/app/contentScript/v2/adaptor folder
 *
 * Inside we are declaring: var = embedMethods = {} and setting some methods for integration.
 * Gulp will build <integration_name>EmbedMethods.js + embedMethodScript.js to <integration_name>EmbedMethods.js
 * and then content script will be append this file in page.
 *
 *
 * We can call some embed method: embedMethod('someNameMethod', [someArgs])
 *
 * 'someNameMethod' is required
 * [someArgs] is optional
 *
 * */

var dictionary = null;
var DIC = (key) => {
    // console.log('Dictionary: ', dictionary, ' Key: ', key);
    return dictionary[key] || key;
};

document.addEventListener('embedMethod', (event) => {
    if ((!event.detail) || (event.detail && !event.detail.name)) {
        return;
    }

    try {
        if (!dictionary) {
            dictionary = event.detail.dict ? JSON.parse(event.detail.dict) : {};
        }

        const result = embedMethods[event.detail.name].apply(this, event.detail.args ? JSON.parse(event.detail.args) : null);

        document.documentElement.setAttribute(event.detail.name.toLowerCase(), JSON.stringify(result));
    } catch (err) {
        console.log(`[EV][embedMethod] ${event.detail.name} is fail `, err);
    }
});
