var embedMethods = {
  getIdFromMapping: function(id, type) {
    const idFromMapping = embedMethods.getTodoistStore()['mapping'][type]['byV2Id'][id];

    return idFromMapping ? idFromMapping : id;
  },
  getTodoistStore: function() {
    const fakeStore = {items: {}, projects: {}, user: {id: null}};

    if (!document.querySelector('#todoist_app')._reactRootContainer) {
      return fakeStore;
    }

    try {
      window.todoist = document.querySelector('#todoist_app')._reactRootContainer.current.child.child.child.pendingProps.children.props;
    } catch (err) {
      console.log('[TodoistEmbedMethods.js][setTodoistStore] error: ', err);

      return fakeStore;
    }

    return todoist.store.getState();
  },
  findTaskProjectId: function(composedTaskId) {
    try {
      return embedMethods.getIdFromMapping(embedMethods.getTodoistStore()['items'][composedTaskId.split(':')[1]].project_id, 'projects');
    } catch(err) {
      console.log('[TodoistEmbedMethods.js][findTaskProjectId] error: ', err);
      return null;
    }
  },
  getTodoistTask: function(taskId) {
    if (!taskId) {
      return null;
    }

    if (!!~taskId.search('_')) {
      return null;
    }

    if (!isFinite(taskId) && !!~taskId.search(':')) {
      taskId = taskId.split(':')[1];
    }

    try {
      const task = JSON.parse(JSON.stringify(embedMethods.getTodoistStore()['items'][taskId]));
      const isV2 = task._id_version && task._id_version === 'v2';
      task.idV2 = isV2 ? embedMethods.getIdFromMapping(task.id, 'tasks') : null;
      task.project_idV2 = isV2 ? embedMethods.getIdFromMapping(task.project_id, 'projects') : null;
      return task;
    } catch(err) {
      console.log('[TodoistEmbedMethods.js][getTodoistTask] error: ', err);
      return null;
    }
  },
  getTodoistTasks: function() {
    try {
      const items = JSON.parse(JSON.stringify(embedMethods.getTodoistStore().items));
      Object.keys(items).forEach((id) => {
          const item = items[id];
          const isV2 = item._id_version && item._id_version === 'v2';
          item.idV2 = isV2 ? embedMethods.getIdFromMapping(item.id, 'tasks') : null;
          item.project_idV2 = isV2 ? embedMethods.getIdFromMapping(item.project_id, 'projects') : null;
      });

      return items;
    } catch (err) {
      console.log('[TodoistEmbedMethods.js][getTodoistTasks] error: ', err);
      return {};
    }
  },
  getTodoistParent: function(taskId) {
    try {
      if (!isFinite(taskId)) {
        taskId = taskId.split(':')[1];
      }

      const store = embedMethods.getTodoistStore();
      const parentId = store['items'][taskId].parent_id;

      return {
        id: parentId,
        name: store['items'][parentId].content,
      };
    } catch(err) {
      console.log('[TodoistEmbedMethods.js][getTodoistParent] error: ', err);
      return null;
    }
  },
  getTodoistProject: function(projectId) {
    try {
      const project = JSON.parse(JSON.stringify(embedMethods.getTodoistStore().projects[projectId]));
      const isV2 = project._id_version && project._id_version === 'v2';
      project.idV2 = isV2 ? embedMethods.getIdFromMapping(project.id, 'projects') : null;
      return project;
    } catch (err) {
      console.log('[TodoistEmbedMethods.js][getTodoistProject] error: ', err);
      return null;
    }
  },
  getTodoistProjects: function() {
    try {
      const projects = JSON.parse(JSON.stringify(embedMethods.getTodoistStore().projects));
      Object.keys(projects).forEach((id) => {
        const project = projects[id];
        const isV2 = project._id_version && project._id_version === 'v2';
        project.idV2 = isV2 ? embedMethods.getIdFromMapping(project.id, 'projects') : null;
      });

      return projects;
    } catch (err) {
      console.log('[TodoistEmbedMethods.js][getTodoistProjects] error: ', err);
      return {};
    }
  },
  getConnectedAccount: function() {
    try {
      const todoistState = embedMethods.getTodoistStore()
      const ravenErrorHandler = window.Raven || null
      let accountId = null;

      if (todoistState) {
        accountId = todoistState.user.id;
      }
      if (!accountId && ravenErrorHandler) {
        accountId = ravenErrorHandler.getContext().user.id;
      }

      return accountId ? 'td:' + accountId : accountId;
    } catch(err) {
      console.log('[TodoistEmbedMethods.js][getConnectedAccount] error: ', err);
      return null;
    }
  }
}

/**
 * We are needing create embeds file for integration(asana, clickup, ...)
 * with name <integration_name>EmbedScripts.js on src/app/contentScript/v2/adaptor folder
 *
 * Inside we are declaring: var = embedMethods = {} and setting some methods for integration.
 * Gulp will build <integration_name>EmbedMethods.js + embedMethodScript.js to <integration_name>EmbedMethods.js
 * and then content script will be append this file in page.
 *
 *
 * We can call some embed method: embedMethod('someNameMethod', [someArgs])
 *
 * 'someNameMethod' is required
 * [someArgs] is optional
 *
 * */

var dictionary = null;
var DIC = (key) => {
    // console.log('Dictionary: ', dictionary, ' Key: ', key);
    return dictionary[key] || key;
};

document.addEventListener('embedMethod', (event) => {
    if ((!event.detail) || (event.detail && !event.detail.name)) {
        return;
    }

    try {
        if (!dictionary) {
            dictionary = event.detail.dict ? JSON.parse(event.detail.dict) : {};
        }

        const result = embedMethods[event.detail.name].apply(this, event.detail.args ? JSON.parse(event.detail.args) : null);

        document.documentElement.setAttribute(event.detail.name.toLowerCase(), JSON.stringify(result));
    } catch (err) {
        console.log(`[EV][embedMethod] ${event.detail.name} is fail `, err);
    }
});
