/*!
 * jQuery JavaScript Library v3.5.1
 * https://jquery.com/
 *
 * Includes Sizzle.js
 * https://sizzlejs.com/
 *
 * Copyright JS Foundation and other contributors
 * Released under the MIT license
 * https://jquery.org/license
 *
 * Date: 2020-05-04T22:49Z
 */
( function( global, factory ) {

    "use strict";

    if ( typeof module === "object" && typeof module.exports === "object" ) {

        // For CommonJS and CommonJS-like environments where a proper `window`
        // is present, execute the factory and get jQuery.
        // For environments that do not have a `window` with a `document`
        // (such as Node.js), expose a factory as module.exports.
        // This accentuates the need for the creation of a real `window`.
        // e.g. var jQuery = require("jquery")(window);
        // See ticket #14549 for more info.
        module.exports = global.document ?
            factory( global, true ) :
            function( w ) {
                if ( !w.document ) {
                    throw new Error( "jQuery requires a window with a document" );
                }
                return factory( w );
            };
    } else {
        factory( global );
    }

// Pass this if window is not defined yet
} )( typeof window !== "undefined" ? window : this, function( window, noGlobal ) {

// Edge <= 12 - 13+, Firefox <=18 - 45+, IE 10 - 11, Safari 5.1 - 9+, iOS 6 - 9.1
// throw exceptions when non-strict code (e.g., ASP.NET 4.5) accesses strict mode
// arguments.callee.caller (trac-13335). But as of jQuery 3.0 (2016), strict mode should be common
// enough that all such attempts are guarded in a try block.
    "use strict";

    var arr = [];

    var getProto = Object.getPrototypeOf;

    var slice = arr.slice;

    var flat = arr.flat ? function( array ) {
        return arr.flat.call( array );
    } : function( array ) {
        return arr.concat.apply( [], array );
    };


    var push = arr.push;

    var indexOf = arr.indexOf;

    var class2type = {};

    var toString = class2type.toString;

    var hasOwn = class2type.hasOwnProperty;

    var fnToString = hasOwn.toString;

    var ObjectFunctionString = fnToString.call( Object );

    var support = {};

    var isFunction = function isFunction( obj ) {

        // Support: Chrome <=57, Firefox <=52
        // In some browsers, typeof returns "function" for HTML <object> elements
        // (i.e., `typeof document.createElement( "object" ) === "function"`).
        // We don't want to classify *any* DOM node as a function.
        return typeof obj === "function" && typeof obj.nodeType !== "number";
    };


    var isWindow = function isWindow( obj ) {
        return obj != null && obj === obj.window;
    };


    var document = window.document;



    var preservedScriptAttributes = {
        type: true,
        src: true,
        nonce: true,
        noModule: true
    };

    function DOMEval( code, node, doc ) {
        doc = doc || document;

        var i, val,
            script = doc.createElement( "script" );

        script.text = code;
        if ( node ) {
            for ( i in preservedScriptAttributes ) {

                // Support: Firefox 64+, Edge 18+
                // Some browsers don't support the "nonce" property on scripts.
                // On the other hand, just using `getAttribute` is not enough as
                // the `nonce` attribute is reset to an empty string whenever it
                // becomes browsing-context connected.
                // See https://github.com/whatwg/html/issues/2369
                // See https://html.spec.whatwg.org/#nonce-attributes
                // The `node.getAttribute` check was added for the sake of
                // `jQuery.globalEval` so that it can fake a nonce-containing node
                // via an object.
                val = node[ i ] || node.getAttribute && node.getAttribute( i );
                if ( val ) {
                    script.setAttribute( i, val );
                }
            }
        }
        doc.head.appendChild( script ).parentNode.removeChild( script );
    }


    function toType( obj ) {
        if ( obj == null ) {
            return obj + "";
        }

        // Support: Android <=2.3 only (functionish RegExp)
        return typeof obj === "object" || typeof obj === "function" ?
            class2type[ toString.call( obj ) ] || "object" :
            typeof obj;
    }
    /* global Symbol */
// Defining this global in .eslintrc.json would create a danger of using the global
// unguarded in another place, it seems safer to define global only for this module



    var
        version = "3.5.1",

        // Define a local copy of jQuery
        jQuery = function( selector, context ) {

            // The jQuery object is actually just the init constructor 'enhanced'
            // Need init if jQuery is called (just allow error to be thrown if not included)
            return new jQuery.fn.init( selector, context );
        };

    jQuery.fn = jQuery.prototype = {

        // The current version of jQuery being used
        jquery: version,

        constructor: jQuery,

        // The default length of a jQuery object is 0
        length: 0,

        toArray: function() {
            return slice.call( this );
        },

        // Get the Nth element in the matched element set OR
        // Get the whole matched element set as a clean array
        get: function( num ) {

            // Return all the elements in a clean array
            if ( num == null ) {
                return slice.call( this );
            }

            // Return just the one element from the set
            return num < 0 ? this[ num + this.length ] : this[ num ];
        },

        // Take an array of elements and push it onto the stack
        // (returning the new matched element set)
        pushStack: function( elems ) {

            // Build a new jQuery matched element set
            var ret = jQuery.merge( this.constructor(), elems );

            // Add the old object onto the stack (as a reference)
            ret.prevObject = this;

            // Return the newly-formed element set
            return ret;
        },

        // Execute a callback for every element in the matched set.
        each: function( callback ) {
            return jQuery.each( this, callback );
        },

        map: function( callback ) {
            return this.pushStack( jQuery.map( this, function( elem, i ) {
                return callback.call( elem, i, elem );
            } ) );
        },

        slice: function() {
            return this.pushStack( slice.apply( this, arguments ) );
        },

        first: function() {
            return this.eq( 0 );
        },

        last: function() {
            return this.eq( -1 );
        },

        even: function() {
            return this.pushStack( jQuery.grep( this, function( _elem, i ) {
                return ( i + 1 ) % 2;
            } ) );
        },

        odd: function() {
            return this.pushStack( jQuery.grep( this, function( _elem, i ) {
                return i % 2;
            } ) );
        },

        eq: function( i ) {
            var len = this.length,
                j = +i + ( i < 0 ? len : 0 );
            return this.pushStack( j >= 0 && j < len ? [ this[ j ] ] : [] );
        },

        end: function() {
            return this.prevObject || this.constructor();
        },

        // For internal use only.
        // Behaves like an Array's method, not like a jQuery method.
        push: push,
        sort: arr.sort,
        splice: arr.splice
    };

    jQuery.extend = jQuery.fn.extend = function() {
        var options, name, src, copy, copyIsArray, clone,
            target = arguments[ 0 ] || {},
            i = 1,
            length = arguments.length,
            deep = false;

        // Handle a deep copy situation
        if ( typeof target === "boolean" ) {
            deep = target;

            // Skip the boolean and the target
            target = arguments[ i ] || {};
            i++;
        }

        // Handle case when target is a string or something (possible in deep copy)
        if ( typeof target !== "object" && !isFunction( target ) ) {
            target = {};
        }

        // Extend jQuery itself if only one argument is passed
        if ( i === length ) {
            target = this;
            i--;
        }

        for ( ; i < length; i++ ) {

            // Only deal with non-null/undefined values
            if ( ( options = arguments[ i ] ) != null ) {

                // Extend the base object
                for ( name in options ) {
                    copy = options[ name ];

                    // Prevent Object.prototype pollution
                    // Prevent never-ending loop
                    if ( name === "__proto__" || target === copy ) {
                        continue;
                    }

                    // Recurse if we're merging plain objects or arrays
                    if ( deep && copy && ( jQuery.isPlainObject( copy ) ||
                        ( copyIsArray = Array.isArray( copy ) ) ) ) {
                        src = target[ name ];

                        // Ensure proper type for the source value
                        if ( copyIsArray && !Array.isArray( src ) ) {
                            clone = [];
                        } else if ( !copyIsArray && !jQuery.isPlainObject( src ) ) {
                            clone = {};
                        } else {
                            clone = src;
                        }
                        copyIsArray = false;

                        // Never move original objects, clone them
                        target[ name ] = jQuery.extend( deep, clone, copy );

                        // Don't bring in undefined values
                    } else if ( copy !== undefined ) {
                        target[ name ] = copy;
                    }
                }
            }
        }

        // Return the modified object
        return target;
    };

    jQuery.extend( {

        // Unique for each copy of jQuery on the page
        expando: "jQuery" + ( version + Math.random() ).replace( /\D/g, "" ),

        // Assume jQuery is ready without the ready module
        isReady: true,

        error: function( msg ) {
            throw new Error( msg );
        },

        noop: function() {},

        isPlainObject: function( obj ) {
            var proto, Ctor;

            // Detect obvious negatives
            // Use toString instead of jQuery.type to catch host objects
            if ( !obj || toString.call( obj ) !== "[object Object]" ) {
                return false;
            }

            proto = getProto( obj );

            // Objects with no prototype (e.g., `Object.create( null )`) are plain
            if ( !proto ) {
                return true;
            }

            // Objects with prototype are plain iff they were constructed by a global Object function
            Ctor = hasOwn.call( proto, "constructor" ) && proto.constructor;
            return typeof Ctor === "function" && fnToString.call( Ctor ) === ObjectFunctionString;
        },

        isEmptyObject: function( obj ) {
            var name;

            for ( name in obj ) {
                return false;
            }
            return true;
        },

        // Evaluates a script in a provided context; falls back to the global one
        // if not specified.
        globalEval: function( code, options, doc ) {
            DOMEval( code, { nonce: options && options.nonce }, doc );
        },

        each: function( obj, callback ) {
            var length, i = 0;

            if ( isArrayLike( obj ) ) {
                length = obj.length;
                for ( ; i < length; i++ ) {
                    if ( callback.call( obj[ i ], i, obj[ i ] ) === false ) {
                        break;
                    }
                }
            } else {
                for ( i in obj ) {
                    if ( callback.call( obj[ i ], i, obj[ i ] ) === false ) {
                        break;
                    }
                }
            }

            return obj;
        },

        // results is for internal usage only
        makeArray: function( arr, results ) {
            var ret = results || [];

            if ( arr != null ) {
                if ( isArrayLike( Object( arr ) ) ) {
                    jQuery.merge( ret,
                        typeof arr === "string" ?
                            [ arr ] : arr
                    );
                } else {
                    push.call( ret, arr );
                }
            }

            return ret;
        },

        inArray: function( elem, arr, i ) {
            return arr == null ? -1 : indexOf.call( arr, elem, i );
        },

        // Support: Android <=4.0 only, PhantomJS 1 only
        // push.apply(_, arraylike) throws on ancient WebKit
        merge: function( first, second ) {
            var len = +second.length,
                j = 0,
                i = first.length;

            for ( ; j < len; j++ ) {
                first[ i++ ] = second[ j ];
            }

            first.length = i;

            return first;
        },

        grep: function( elems, callback, invert ) {
            var callbackInverse,
                matches = [],
                i = 0,
                length = elems.length,
                callbackExpect = !invert;

            // Go through the array, only saving the items
            // that pass the validator function
            for ( ; i < length; i++ ) {
                callbackInverse = !callback( elems[ i ], i );
                if ( callbackInverse !== callbackExpect ) {
                    matches.push( elems[ i ] );
                }
            }

            return matches;
        },

        // arg is for internal usage only
        map: function( elems, callback, arg ) {
            var length, value,
                i = 0,
                ret = [];

            // Go through the array, translating each of the items to their new values
            if ( isArrayLike( elems ) ) {
                length = elems.length;
                for ( ; i < length; i++ ) {
                    value = callback( elems[ i ], i, arg );

                    if ( value != null ) {
                        ret.push( value );
                    }
                }

                // Go through every key on the object,
            } else {
                for ( i in elems ) {
                    value = callback( elems[ i ], i, arg );

                    if ( value != null ) {
                        ret.push( value );
                    }
                }
            }

            // Flatten any nested arrays
            return flat( ret );
        },

        // A global GUID counter for objects
        guid: 1,

        // jQuery.support is not used in Core but other projects attach their
        // properties to it so it needs to exist.
        support: support
    } );

    if ( typeof Symbol === "function" ) {
        jQuery.fn[ Symbol.iterator ] = arr[ Symbol.iterator ];
    }

// Populate the class2type map
    jQuery.each( "Boolean Number String Function Array Date RegExp Object Error Symbol".split( " " ),
        function( _i, name ) {
            class2type[ "[object " + name + "]" ] = name.toLowerCase();
        } );

    function isArrayLike( obj ) {

        // Support: real iOS 8.2 only (not reproducible in simulator)
        // `in` check used to prevent JIT error (gh-2145)
        // hasOwn isn't used here due to false negatives
        // regarding Nodelist length in IE
        var length = !!obj && "length" in obj && obj.length,
            type = toType( obj );

        if ( isFunction( obj ) || isWindow( obj ) ) {
            return false;
        }

        return type === "array" || length === 0 ||
            typeof length === "number" && length > 0 && ( length - 1 ) in obj;
    }
    var Sizzle =
        /*!
 * Sizzle CSS Selector Engine v2.3.5
 * https://sizzlejs.com/
 *
 * Copyright JS Foundation and other contributors
 * Released under the MIT license
 * https://js.foundation/
 *
 * Date: 2020-03-14
 */
        ( function( window ) {
            var i,
                support,
                Expr,
                getText,
                isXML,
                tokenize,
                compile,
                select,
                outermostContext,
                sortInput,
                hasDuplicate,

                // Local document vars
                setDocument,
                document,
                docElem,
                documentIsHTML,
                rbuggyQSA,
                rbuggyMatches,
                matches,
                contains,

                // Instance-specific data
                expando = "sizzle" + 1 * new Date(),
                preferredDoc = window.document,
                dirruns = 0,
                done = 0,
                classCache = createCache(),
                tokenCache = createCache(),
                compilerCache = createCache(),
                nonnativeSelectorCache = createCache(),
                sortOrder = function( a, b ) {
                    if ( a === b ) {
                        hasDuplicate = true;
                    }
                    return 0;
                },

                // Instance methods
                hasOwn = ( {} ).hasOwnProperty,
                arr = [],
                pop = arr.pop,
                pushNative = arr.push,
                push = arr.push,
                slice = arr.slice,

                // Use a stripped-down indexOf as it's faster than native
                // https://jsperf.com/thor-indexof-vs-for/5
                indexOf = function( list, elem ) {
                    var i = 0,
                        len = list.length;
                    for ( ; i < len; i++ ) {
                        if ( list[ i ] === elem ) {
                            return i;
                        }
                    }
                    return -1;
                },

                booleans = "checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|" +
                    "ismap|loop|multiple|open|readonly|required|scoped",

                // Regular expressions

                // http://www.w3.org/TR/css3-selectors/#whitespace
                whitespace = "[\\x20\\t\\r\\n\\f]",

                // https://www.w3.org/TR/css-syntax-3/#ident-token-diagram
                identifier = "(?:\\\\[\\da-fA-F]{1,6}" + whitespace +
                    "?|\\\\[^\\r\\n\\f]|[\\w-]|[^\0-\\x7f])+",

                // Attribute selectors: http://www.w3.org/TR/selectors/#attribute-selectors
                attributes = "\\[" + whitespace + "*(" + identifier + ")(?:" + whitespace +

                    // Operator (capture 2)
                    "*([*^$|!~]?=)" + whitespace +

                    // "Attribute values must be CSS identifiers [capture 5]
                    // or strings [capture 3 or capture 4]"
                    "*(?:'((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\"|(" + identifier + "))|)" +
                    whitespace + "*\\]",

                pseudos = ":(" + identifier + ")(?:\\((" +

                    // To reduce the number of selectors needing tokenize in the preFilter, prefer arguments:
                    // 1. quoted (capture 3; capture 4 or capture 5)
                    "('((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\")|" +

                    // 2. simple (capture 6)
                    "((?:\\\\.|[^\\\\()[\\]]|" + attributes + ")*)|" +

                    // 3. anything else (capture 2)
                    ".*" +
                    ")\\)|)",

                // Leading and non-escaped trailing whitespace, capturing some non-whitespace characters preceding the latter
                rwhitespace = new RegExp( whitespace + "+", "g" ),
                rtrim = new RegExp( "^" + whitespace + "+|((?:^|[^\\\\])(?:\\\\.)*)" +
                    whitespace + "+$", "g" ),

                rcomma = new RegExp( "^" + whitespace + "*," + whitespace + "*" ),
                rcombinators = new RegExp( "^" + whitespace + "*([>+~]|" + whitespace + ")" + whitespace +
                    "*" ),
                rdescend = new RegExp( whitespace + "|>" ),

                rpseudo = new RegExp( pseudos ),
                ridentifier = new RegExp( "^" + identifier + "$" ),

                matchExpr = {
                    "ID": new RegExp( "^#(" + identifier + ")" ),
                    "CLASS": new RegExp( "^\\.(" + identifier + ")" ),
                    "TAG": new RegExp( "^(" + identifier + "|[*])" ),
                    "ATTR": new RegExp( "^" + attributes ),
                    "PSEUDO": new RegExp( "^" + pseudos ),
                    "CHILD": new RegExp( "^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\(" +
                        whitespace + "*(even|odd|(([+-]|)(\\d*)n|)" + whitespace + "*(?:([+-]|)" +
                        whitespace + "*(\\d+)|))" + whitespace + "*\\)|)", "i" ),
                    "bool": new RegExp( "^(?:" + booleans + ")$", "i" ),

                    // For use in libraries implementing .is()
                    // We use this for POS matching in `select`
                    "needsContext": new RegExp( "^" + whitespace +
                        "*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\(" + whitespace +
                        "*((?:-\\d)?\\d*)" + whitespace + "*\\)|)(?=[^-]|$)", "i" )
                },

                rhtml = /HTML$/i,
                rinputs = /^(?:input|select|textarea|button)$/i,
                rheader = /^h\d$/i,

                rnative = /^[^{]+\{\s*\[native \w/,

                // Easily-parseable/retrievable ID or TAG or CLASS selectors
                rquickExpr = /^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,

                rsibling = /[+~]/,

                // CSS escapes
                // http://www.w3.org/TR/CSS21/syndata.html#escaped-characters
                runescape = new RegExp( "\\\\[\\da-fA-F]{1,6}" + whitespace + "?|\\\\([^\\r\\n\\f])", "g" ),
                funescape = function( escape, nonHex ) {
                    var high = "0x" + escape.slice( 1 ) - 0x10000;

                    return nonHex ?

                        // Strip the backslash prefix from a non-hex escape sequence
                        nonHex :

                        // Replace a hexadecimal escape sequence with the encoded Unicode code point
                        // Support: IE <=11+
                        // For values outside the Basic Multilingual Plane (BMP), manually construct a
                        // surrogate pair
                        high < 0 ?
                            String.fromCharCode( high + 0x10000 ) :
                            String.fromCharCode( high >> 10 | 0xD800, high & 0x3FF | 0xDC00 );
                },

                // CSS string/identifier serialization
                // https://drafts.csswg.org/cssom/#common-serializing-idioms
                rcssescape = /([\0-\x1f\x7f]|^-?\d)|^-$|[^\0-\x1f\x7f-\uFFFF\w-]/g,
                fcssescape = function( ch, asCodePoint ) {
                    if ( asCodePoint ) {

                        // U+0000 NULL becomes U+FFFD REPLACEMENT CHARACTER
                        if ( ch === "\0" ) {
                            return "\uFFFD";
                        }

                        // Control characters and (dependent upon position) numbers get escaped as code points
                        return ch.slice( 0, -1 ) + "\\" +
                            ch.charCodeAt( ch.length - 1 ).toString( 16 ) + " ";
                    }

                    // Other potentially-special ASCII characters get backslash-escaped
                    return "\\" + ch;
                },

                // Used for iframes
                // See setDocument()
                // Removing the function wrapper causes a "Permission Denied"
                // error in IE
                unloadHandler = function() {
                    setDocument();
                },

                inDisabledFieldset = addCombinator(
                    function( elem ) {
                        return elem.disabled === true && elem.nodeName.toLowerCase() === "fieldset";
                    },
                    { dir: "parentNode", next: "legend" }
                );

// Optimize for push.apply( _, NodeList )
            try {
                push.apply(
                    ( arr = slice.call( preferredDoc.childNodes ) ),
                    preferredDoc.childNodes
                );

                // Support: Android<4.0
                // Detect silently failing push.apply
                // eslint-disable-next-line no-unused-expressions
                arr[ preferredDoc.childNodes.length ].nodeType;
            } catch ( e ) {
                push = { apply: arr.length ?

                        // Leverage slice if possible
                        function( target, els ) {
                            pushNative.apply( target, slice.call( els ) );
                        } :

                        // Support: IE<9
                        // Otherwise append directly
                        function( target, els ) {
                            var j = target.length,
                                i = 0;

                            // Can't trust NodeList.length
                            while ( ( target[ j++ ] = els[ i++ ] ) ) {}
                            target.length = j - 1;
                        }
                };
            }

            function Sizzle( selector, context, results, seed ) {
                var m, i, elem, nid, match, groups, newSelector,
                    newContext = context && context.ownerDocument,

                    // nodeType defaults to 9, since context defaults to document
                    nodeType = context ? context.nodeType : 9;

                results = results || [];

                // Return early from calls with invalid selector or context
                if ( typeof selector !== "string" || !selector ||
                    nodeType !== 1 && nodeType !== 9 && nodeType !== 11 ) {

                    return results;
                }

                // Try to shortcut find operations (as opposed to filters) in HTML documents
                if ( !seed ) {
                    setDocument( context );
                    context = context || document;

                    if ( documentIsHTML ) {

                        // If the selector is sufficiently simple, try using a "get*By*" DOM method
                        // (excepting DocumentFragment context, where the methods don't exist)
                        if ( nodeType !== 11 && ( match = rquickExpr.exec( selector ) ) ) {

                            // ID selector
                            if ( ( m = match[ 1 ] ) ) {

                                // Document context
                                if ( nodeType === 9 ) {
                                    if ( ( elem = context.getElementById( m ) ) ) {

                                        // Support: IE, Opera, Webkit
                                        // TODO: identify versions
                                        // getElementById can match elements by name instead of ID
                                        if ( elem.id === m ) {
                                            results.push( elem );
                                            return results;
                                        }
                                    } else {
                                        return results;
                                    }

                                    // Element context
                                } else {

                                    // Support: IE, Opera, Webkit
                                    // TODO: identify versions
                                    // getElementById can match elements by name instead of ID
                                    if ( newContext && ( elem = newContext.getElementById( m ) ) &&
                                        contains( context, elem ) &&
                                        elem.id === m ) {

                                        results.push( elem );
                                        return results;
                                    }
                                }

                                // Type selector
                            } else if ( match[ 2 ] ) {
                                push.apply( results, context.getElementsByTagName( selector ) );
                                return results;

                                // Class selector
                            } else if ( ( m = match[ 3 ] ) && support.getElementsByClassName &&
                                context.getElementsByClassName ) {

                                push.apply( results, context.getElementsByClassName( m ) );
                                return results;
                            }
                        }

                        // Take advantage of querySelectorAll
                        if ( support.qsa &&
                            !nonnativeSelectorCache[ selector + " " ] &&
                            ( !rbuggyQSA || !rbuggyQSA.test( selector ) ) &&

                            // Support: IE 8 only
                            // Exclude object elements
                            ( nodeType !== 1 || context.nodeName.toLowerCase() !== "object" ) ) {

                            newSelector = selector;
                            newContext = context;

                            // qSA considers elements outside a scoping root when evaluating child or
                            // descendant combinators, which is not what we want.
                            // In such cases, we work around the behavior by prefixing every selector in the
                            // list with an ID selector referencing the scope context.
                            // The technique has to be used as well when a leading combinator is used
                            // as such selectors are not recognized by querySelectorAll.
                            // Thanks to Andrew Dupont for this technique.
                            if ( nodeType === 1 &&
                                ( rdescend.test( selector ) || rcombinators.test( selector ) ) ) {

                                // Expand context for sibling selectors
                                newContext = rsibling.test( selector ) && testContext( context.parentNode ) ||
                                    context;

                                // We can use :scope instead of the ID hack if the browser
                                // supports it & if we're not changing the context.
                                if ( newContext !== context || !support.scope ) {

                                    // Capture the context ID, setting it first if necessary
                                    if ( ( nid = context.getAttribute( "id" ) ) ) {
                                        nid = nid.replace( rcssescape, fcssescape );
                                    } else {
                                        context.setAttribute( "id", ( nid = expando ) );
                                    }
                                }

                                // Prefix every selector in the list
                                groups = tokenize( selector );
                                i = groups.length;
                                while ( i-- ) {
                                    groups[ i ] = ( nid ? "#" + nid : ":scope" ) + " " +
                                        toSelector( groups[ i ] );
                                }
                                newSelector = groups.join( "," );
                            }

                            try {
                                push.apply( results,
                                    newContext.querySelectorAll( newSelector )
                                );
                                return results;
                            } catch ( qsaError ) {
                                nonnativeSelectorCache( selector, true );
                            } finally {
                                if ( nid === expando ) {
                                    context.removeAttribute( "id" );
                                }
                            }
                        }
                    }
                }

                // All others
                return select( selector.replace( rtrim, "$1" ), context, results, seed );
            }

            /**
             * Create key-value caches of limited size
             * @returns {function(string, object)} Returns the Object data after storing it on itself with
             *	property name the (space-suffixed) string and (if the cache is larger than Expr.cacheLength)
             *	deleting the oldest entry
             */
            function createCache() {
                var keys = [];

                function cache( key, value ) {

                    // Use (key + " ") to avoid collision with native prototype properties (see Issue #157)
                    if ( keys.push( key + " " ) > Expr.cacheLength ) {

                        // Only keep the most recent entries
                        delete cache[ keys.shift() ];
                    }
                    return ( cache[ key + " " ] = value );
                }
                return cache;
            }

            /**
             * Mark a function for special use by Sizzle
             * @param {Function} fn The function to mark
             */
            function markFunction( fn ) {
                fn[ expando ] = true;
                return fn;
            }

            /**
             * Support testing using an element
             * @param {Function} fn Passed the created element and returns a boolean result
             */
            function assert( fn ) {
                var el = document.createElement( "fieldset" );

                try {
                    return !!fn( el );
                } catch ( e ) {
                    return false;
                } finally {

                    // Remove from its parent by default
                    if ( el.parentNode ) {
                        el.parentNode.removeChild( el );
                    }

                    // release memory in IE
                    el = null;
                }
            }

            /**
             * Adds the same handler for all of the specified attrs
             * @param {String} attrs Pipe-separated list of attributes
             * @param {Function} handler The method that will be applied
             */
            function addHandle( attrs, handler ) {
                var arr = attrs.split( "|" ),
                    i = arr.length;

                while ( i-- ) {
                    Expr.attrHandle[ arr[ i ] ] = handler;
                }
            }

            /**
             * Checks document order of two siblings
             * @param {Element} a
             * @param {Element} b
             * @returns {Number} Returns less than 0 if a precedes b, greater than 0 if a follows b
             */
            function siblingCheck( a, b ) {
                var cur = b && a,
                    diff = cur && a.nodeType === 1 && b.nodeType === 1 &&
                        a.sourceIndex - b.sourceIndex;

                // Use IE sourceIndex if available on both nodes
                if ( diff ) {
                    return diff;
                }

                // Check if b follows a
                if ( cur ) {
                    while ( ( cur = cur.nextSibling ) ) {
                        if ( cur === b ) {
                            return -1;
                        }
                    }
                }

                return a ? 1 : -1;
            }

            /**
             * Returns a function to use in pseudos for input types
             * @param {String} type
             */
            function createInputPseudo( type ) {
                return function( elem ) {
                    var name = elem.nodeName.toLowerCase();
                    return name === "input" && elem.type === type;
                };
            }

            /**
             * Returns a function to use in pseudos for buttons
             * @param {String} type
             */
            function createButtonPseudo( type ) {
                return function( elem ) {
                    var name = elem.nodeName.toLowerCase();
                    return ( name === "input" || name === "button" ) && elem.type === type;
                };
            }

            /**
             * Returns a function to use in pseudos for :enabled/:disabled
             * @param {Boolean} disabled true for :disabled; false for :enabled
             */
            function createDisabledPseudo( disabled ) {

                // Known :disabled false positives: fieldset[disabled] > legend:nth-of-type(n+2) :can-disable
                return function( elem ) {

                    // Only certain elements can match :enabled or :disabled
                    // https://html.spec.whatwg.org/multipage/scripting.html#selector-enabled
                    // https://html.spec.whatwg.org/multipage/scripting.html#selector-disabled
                    if ( "form" in elem ) {

                        // Check for inherited disabledness on relevant non-disabled elements:
                        // * listed form-associated elements in a disabled fieldset
                        //   https://html.spec.whatwg.org/multipage/forms.html#category-listed
                        //   https://html.spec.whatwg.org/multipage/forms.html#concept-fe-disabled
                        // * option elements in a disabled optgroup
                        //   https://html.spec.whatwg.org/multipage/forms.html#concept-option-disabled
                        // All such elements have a "form" property.
                        if ( elem.parentNode && elem.disabled === false ) {

                            // Option elements defer to a parent optgroup if present
                            if ( "label" in elem ) {
                                if ( "label" in elem.parentNode ) {
                                    return elem.parentNode.disabled === disabled;
                                } else {
                                    return elem.disabled === disabled;
                                }
                            }

                            // Support: IE 6 - 11
                            // Use the isDisabled shortcut property to check for disabled fieldset ancestors
                            return elem.isDisabled === disabled ||

                                // Where there is no isDisabled, check manually
                                /* jshint -W018 */
                                elem.isDisabled !== !disabled &&
                                inDisabledFieldset( elem ) === disabled;
                        }

                        return elem.disabled === disabled;

                        // Try to winnow out elements that can't be disabled before trusting the disabled property.
                        // Some victims get caught in our net (label, legend, menu, track), but it shouldn't
                        // even exist on them, let alone have a boolean value.
                    } else if ( "label" in elem ) {
                        return elem.disabled === disabled;
                    }

                    // Remaining elements are neither :enabled nor :disabled
                    return false;
                };
            }

            /**
             * Returns a function to use in pseudos for positionals
             * @param {Function} fn
             */
            function createPositionalPseudo( fn ) {
                return markFunction( function( argument ) {
                    argument = +argument;
                    return markFunction( function( seed, matches ) {
                        var j,
                            matchIndexes = fn( [], seed.length, argument ),
                            i = matchIndexes.length;

                        // Match elements found at the specified indexes
                        while ( i-- ) {
                            if ( seed[ ( j = matchIndexes[ i ] ) ] ) {
                                seed[ j ] = !( matches[ j ] = seed[ j ] );
                            }
                        }
                    } );
                } );
            }

            /**
             * Checks a node for validity as a Sizzle context
             * @param {Element|Object=} context
             * @returns {Element|Object|Boolean} The input node if acceptable, otherwise a falsy value
             */
            function testContext( context ) {
                return context && typeof context.getElementsByTagName !== "undefined" && context;
            }

// Expose support vars for convenience
            support = Sizzle.support = {};

            /**
             * Detects XML nodes
             * @param {Element|Object} elem An element or a document
             * @returns {Boolean} True iff elem is a non-HTML XML node
             */
            isXML = Sizzle.isXML = function( elem ) {
                var namespace = elem.namespaceURI,
                    docElem = ( elem.ownerDocument || elem ).documentElement;

                // Support: IE <=8
                // Assume HTML when documentElement doesn't yet exist, such as inside loading iframes
                // https://bugs.jquery.com/ticket/4833
                return !rhtml.test( namespace || docElem && docElem.nodeName || "HTML" );
            };

            /**
             * Sets document-related variables once based on the current document
             * @param {Element|Object} [doc] An element or document object to use to set the document
             * @returns {Object} Returns the current document
             */
            setDocument = Sizzle.setDocument = function( node ) {
                var hasCompare, subWindow,
                    doc = node ? node.ownerDocument || node : preferredDoc;

                // Return early if doc is invalid or already selected
                // Support: IE 11+, Edge 17 - 18+
                // IE/Edge sometimes throw a "Permission denied" error when strict-comparing
                // two documents; shallow comparisons work.
                // eslint-disable-next-line eqeqeq
                if ( doc == document || doc.nodeType !== 9 || !doc.documentElement ) {
                    return document;
                }

                // Update global variables
                document = doc;
                docElem = document.documentElement;
                documentIsHTML = !isXML( document );

                // Support: IE 9 - 11+, Edge 12 - 18+
                // Accessing iframe documents after unload throws "permission denied" errors (jQuery #13936)
                // Support: IE 11+, Edge 17 - 18+
                // IE/Edge sometimes throw a "Permission denied" error when strict-comparing
                // two documents; shallow comparisons work.
                // eslint-disable-next-line eqeqeq
                if ( preferredDoc != document &&
                    ( subWindow = document.defaultView ) && subWindow.top !== subWindow ) {

                    // Support: IE 11, Edge
                    if ( subWindow.addEventListener ) {
                        subWindow.addEventListener( "unload", unloadHandler, false );

                        // Support: IE 9 - 10 only
                    } else if ( subWindow.attachEvent ) {
                        subWindow.attachEvent( "onunload", unloadHandler );
                    }
                }

                // Support: IE 8 - 11+, Edge 12 - 18+, Chrome <=16 - 25 only, Firefox <=3.6 - 31 only,
                // Safari 4 - 5 only, Opera <=11.6 - 12.x only
                // IE/Edge & older browsers don't support the :scope pseudo-class.
                // Support: Safari 6.0 only
                // Safari 6.0 supports :scope but it's an alias of :root there.
                support.scope = assert( function( el ) {
                    docElem.appendChild( el ).appendChild( document.createElement( "div" ) );
                    return typeof el.querySelectorAll !== "undefined" &&
                        !el.querySelectorAll( ":scope fieldset div" ).length;
                } );

                /* Attributes
	---------------------------------------------------------------------- */

                // Support: IE<8
                // Verify that getAttribute really returns attributes and not properties
                // (excepting IE8 booleans)
                support.attributes = assert( function( el ) {
                    el.className = "i";
                    return !el.getAttribute( "className" );
                } );

                /* getElement(s)By*
	---------------------------------------------------------------------- */

                // Check if getElementsByTagName("*") returns only elements
                support.getElementsByTagName = assert( function( el ) {
                    el.appendChild( document.createComment( "" ) );
                    return !el.getElementsByTagName( "*" ).length;
                } );

                // Support: IE<9
                support.getElementsByClassName = rnative.test( document.getElementsByClassName );

                // Support: IE<10
                // Check if getElementById returns elements by name
                // The broken getElementById methods don't pick up programmatically-set names,
                // so use a roundabout getElementsByName test
                support.getById = assert( function( el ) {
                    docElem.appendChild( el ).id = expando;
                    return !document.getElementsByName || !document.getElementsByName( expando ).length;
                } );

                // ID filter and find
                if ( support.getById ) {
                    Expr.filter[ "ID" ] = function( id ) {
                        var attrId = id.replace( runescape, funescape );
                        return function( elem ) {
                            return elem.getAttribute( "id" ) === attrId;
                        };
                    };
                    Expr.find[ "ID" ] = function( id, context ) {
                        if ( typeof context.getElementById !== "undefined" && documentIsHTML ) {
                            var elem = context.getElementById( id );
                            return elem ? [ elem ] : [];
                        }
                    };
                } else {
                    Expr.filter[ "ID" ] =  function( id ) {
                        var attrId = id.replace( runescape, funescape );
                        return function( elem ) {
                            var node = typeof elem.getAttributeNode !== "undefined" &&
                                elem.getAttributeNode( "id" );
                            return node && node.value === attrId;
                        };
                    };

                    // Support: IE 6 - 7 only
                    // getElementById is not reliable as a find shortcut
                    Expr.find[ "ID" ] = function( id, context ) {
                        if ( typeof context.getElementById !== "undefined" && documentIsHTML ) {
                            var node, i, elems,
                                elem = context.getElementById( id );

                            if ( elem ) {

                                // Verify the id attribute
                                node = elem.getAttributeNode( "id" );
                                if ( node && node.value === id ) {
                                    return [ elem ];
                                }

                                // Fall back on getElementsByName
                                elems = context.getElementsByName( id );
                                i = 0;
                                while ( ( elem = elems[ i++ ] ) ) {
                                    node = elem.getAttributeNode( "id" );
                                    if ( node && node.value === id ) {
                                        return [ elem ];
                                    }
                                }
                            }

                            return [];
                        }
                    };
                }

                // Tag
                Expr.find[ "TAG" ] = support.getElementsByTagName ?
                    function( tag, context ) {
                        if ( typeof context.getElementsByTagName !== "undefined" ) {
                            return context.getElementsByTagName( tag );

                            // DocumentFragment nodes don't have gEBTN
                        } else if ( support.qsa ) {
                            return context.querySelectorAll( tag );
                        }
                    } :

                    function( tag, context ) {
                        var elem,
                            tmp = [],
                            i = 0,

                            // By happy coincidence, a (broken) gEBTN appears on DocumentFragment nodes too
                            results = context.getElementsByTagName( tag );

                        // Filter out possible comments
                        if ( tag === "*" ) {
                            while ( ( elem = results[ i++ ] ) ) {
                                if ( elem.nodeType === 1 ) {
                                    tmp.push( elem );
                                }
                            }

                            return tmp;
                        }
                        return results;
                    };

                // Class
                Expr.find[ "CLASS" ] = support.getElementsByClassName && function( className, context ) {
                    if ( typeof context.getElementsByClassName !== "undefined" && documentIsHTML ) {
                        return context.getElementsByClassName( className );
                    }
                };

                /* QSA/matchesSelector
	---------------------------------------------------------------------- */

                // QSA and matchesSelector support

                // matchesSelector(:active) reports false when true (IE9/Opera 11.5)
                rbuggyMatches = [];

                // qSa(:focus) reports false when true (Chrome 21)
                // We allow this because of a bug in IE8/9 that throws an error
                // whenever `document.activeElement` is accessed on an iframe
                // So, we allow :focus to pass through QSA all the time to avoid the IE error
                // See https://bugs.jquery.com/ticket/13378
                rbuggyQSA = [];

                if ( ( support.qsa = rnative.test( document.querySelectorAll ) ) ) {

                    // Build QSA regex
                    // Regex strategy adopted from Diego Perini
                    assert( function( el ) {

                        var input;

                        // Select is set to empty string on purpose
                        // This is to test IE's treatment of not explicitly
                        // setting a boolean content attribute,
                        // since its presence should be enough
                        // https://bugs.jquery.com/ticket/12359
                        docElem.appendChild( el ).innerHTML = "<a id='" + expando + "'></a>" +
                            "<select id='" + expando + "-\r\\' msallowcapture=''>" +
                            "<option selected=''></option></select>";

                        // Support: IE8, Opera 11-12.16
                        // Nothing should be selected when empty strings follow ^= or $= or *=
                        // The test attribute must be unknown in Opera but "safe" for WinRT
                        // https://msdn.microsoft.com/en-us/library/ie/hh465388.aspx#attribute_section
                        if ( el.querySelectorAll( "[msallowcapture^='']" ).length ) {
                            rbuggyQSA.push( "[*^$]=" + whitespace + "*(?:''|\"\")" );
                        }

                        // Support: IE8
                        // Boolean attributes and "value" are not treated correctly
                        if ( !el.querySelectorAll( "[selected]" ).length ) {
                            rbuggyQSA.push( "\\[" + whitespace + "*(?:value|" + booleans + ")" );
                        }

                        // Support: Chrome<29, Android<4.4, Safari<7.0+, iOS<7.0+, PhantomJS<1.9.8+
                        if ( !el.querySelectorAll( "[id~=" + expando + "-]" ).length ) {
                            rbuggyQSA.push( "~=" );
                        }

                        // Support: IE 11+, Edge 15 - 18+
                        // IE 11/Edge don't find elements on a `[name='']` query in some cases.
                        // Adding a temporary attribute to the document before the selection works
                        // around the issue.
                        // Interestingly, IE 10 & older don't seem to have the issue.
                        input = document.createElement( "input" );
                        input.setAttribute( "name", "" );
                        el.appendChild( input );
                        if ( !el.querySelectorAll( "[name='']" ).length ) {
                            rbuggyQSA.push( "\\[" + whitespace + "*name" + whitespace + "*=" +
                                whitespace + "*(?:''|\"\")" );
                        }

                        // Webkit/Opera - :checked should return selected option elements
                        // http://www.w3.org/TR/2011/REC-css3-selectors-20110929/#checked
                        // IE8 throws error here and will not see later tests
                        if ( !el.querySelectorAll( ":checked" ).length ) {
                            rbuggyQSA.push( ":checked" );
                        }

                        // Support: Safari 8+, iOS 8+
                        // https://bugs.webkit.org/show_bug.cgi?id=136851
                        // In-page `selector#id sibling-combinator selector` fails
                        if ( !el.querySelectorAll( "a#" + expando + "+*" ).length ) {
                            rbuggyQSA.push( ".#.+[+~]" );
                        }

                        // Support: Firefox <=3.6 - 5 only
                        // Old Firefox doesn't throw on a badly-escaped identifier.
                        el.querySelectorAll( "\\\f" );
                        rbuggyQSA.push( "[\\r\\n\\f]" );
                    } );

                    assert( function( el ) {
                        el.innerHTML = "<a href='' disabled='disabled'></a>" +
                            "<select disabled='disabled'><option/></select>";

                        // Support: Windows 8 Native Apps
                        // The type and name attributes are restricted during .innerHTML assignment
                        var input = document.createElement( "input" );
                        input.setAttribute( "type", "hidden" );
                        el.appendChild( input ).setAttribute( "name", "D" );

                        // Support: IE8
                        // Enforce case-sensitivity of name attribute
                        if ( el.querySelectorAll( "[name=d]" ).length ) {
                            rbuggyQSA.push( "name" + whitespace + "*[*^$|!~]?=" );
                        }

                        // FF 3.5 - :enabled/:disabled and hidden elements (hidden elements are still enabled)
                        // IE8 throws error here and will not see later tests
                        if ( el.querySelectorAll( ":enabled" ).length !== 2 ) {
                            rbuggyQSA.push( ":enabled", ":disabled" );
                        }

                        // Support: IE9-11+
                        // IE's :disabled selector does not pick up the children of disabled fieldsets
                        docElem.appendChild( el ).disabled = true;
                        if ( el.querySelectorAll( ":disabled" ).length !== 2 ) {
                            rbuggyQSA.push( ":enabled", ":disabled" );
                        }

                        // Support: Opera 10 - 11 only
                        // Opera 10-11 does not throw on post-comma invalid pseudos
                        el.querySelectorAll( "*,:x" );
                        rbuggyQSA.push( ",.*:" );
                    } );
                }

                if ( ( support.matchesSelector = rnative.test( ( matches = docElem.matches ||
                    docElem.webkitMatchesSelector ||
                    docElem.mozMatchesSelector ||
                    docElem.oMatchesSelector ||
                    docElem.msMatchesSelector ) ) ) ) {

                    assert( function( el ) {

                        // Check to see if it's possible to do matchesSelector
                        // on a disconnected node (IE 9)
                        support.disconnectedMatch = matches.call( el, "*" );

                        // This should fail with an exception
                        // Gecko does not error, returns false instead
                        matches.call( el, "[s!='']:x" );
                        rbuggyMatches.push( "!=", pseudos );
                    } );
                }

                rbuggyQSA = rbuggyQSA.length && new RegExp( rbuggyQSA.join( "|" ) );
                rbuggyMatches = rbuggyMatches.length && new RegExp( rbuggyMatches.join( "|" ) );

                /* Contains
	---------------------------------------------------------------------- */
                hasCompare = rnative.test( docElem.compareDocumentPosition );

                // Element contains another
                // Purposefully self-exclusive
                // As in, an element does not contain itself
                contains = hasCompare || rnative.test( docElem.contains ) ?
                    function( a, b ) {
                        var adown = a.nodeType === 9 ? a.documentElement : a,
                            bup = b && b.parentNode;
                        return a === bup || !!( bup && bup.nodeType === 1 && (
                            adown.contains ?
                                adown.contains( bup ) :
                                a.compareDocumentPosition && a.compareDocumentPosition( bup ) & 16
                        ) );
                    } :
                    function( a, b ) {
                        if ( b ) {
                            while ( ( b = b.parentNode ) ) {
                                if ( b === a ) {
                                    return true;
                                }
                            }
                        }
                        return false;
                    };

                /* Sorting
	---------------------------------------------------------------------- */

                // Document order sorting
                sortOrder = hasCompare ?
                    function( a, b ) {

                        // Flag for duplicate removal
                        if ( a === b ) {
                            hasDuplicate = true;
                            return 0;
                        }

                        // Sort on method existence if only one input has compareDocumentPosition
                        var compare = !a.compareDocumentPosition - !b.compareDocumentPosition;
                        if ( compare ) {
                            return compare;
                        }

                        // Calculate position if both inputs belong to the same document
                        // Support: IE 11+, Edge 17 - 18+
                        // IE/Edge sometimes throw a "Permission denied" error when strict-comparing
                        // two documents; shallow comparisons work.
                        // eslint-disable-next-line eqeqeq
                        compare = ( a.ownerDocument || a ) == ( b.ownerDocument || b ) ?
                            a.compareDocumentPosition( b ) :

                            // Otherwise we know they are disconnected
                            1;

                        // Disconnected nodes
                        if ( compare & 1 ||
                            ( !support.sortDetached && b.compareDocumentPosition( a ) === compare ) ) {

                            // Choose the first element that is related to our preferred document
                            // Support: IE 11+, Edge 17 - 18+
                            // IE/Edge sometimes throw a "Permission denied" error when strict-comparing
                            // two documents; shallow comparisons work.
                            // eslint-disable-next-line eqeqeq
                            if ( a == document || a.ownerDocument == preferredDoc &&
                                contains( preferredDoc, a ) ) {
                                return -1;
                            }

                            // Support: IE 11+, Edge 17 - 18+
                            // IE/Edge sometimes throw a "Permission denied" error when strict-comparing
                            // two documents; shallow comparisons work.
                            // eslint-disable-next-line eqeqeq
                            if ( b == document || b.ownerDocument == preferredDoc &&
                                contains( preferredDoc, b ) ) {
                                return 1;
                            }

                            // Maintain original order
                            return sortInput ?
                                ( indexOf( sortInput, a ) - indexOf( sortInput, b ) ) :
                                0;
                        }

                        return compare & 4 ? -1 : 1;
                    } :
                    function( a, b ) {

                        // Exit early if the nodes are identical
                        if ( a === b ) {
                            hasDuplicate = true;
                            return 0;
                        }

                        var cur,
                            i = 0,
                            aup = a.parentNode,
                            bup = b.parentNode,
                            ap = [ a ],
                            bp = [ b ];

                        // Parentless nodes are either documents or disconnected
                        if ( !aup || !bup ) {

                            // Support: IE 11+, Edge 17 - 18+
                            // IE/Edge sometimes throw a "Permission denied" error when strict-comparing
                            // two documents; shallow comparisons work.
                            /* eslint-disable eqeqeq */
                            return a == document ? -1 :
                                b == document ? 1 :
                                    /* eslint-enable eqeqeq */
                                    aup ? -1 :
                                        bup ? 1 :
                                            sortInput ?
                                                ( indexOf( sortInput, a ) - indexOf( sortInput, b ) ) :
                                                0;

                            // If the nodes are siblings, we can do a quick check
                        } else if ( aup === bup ) {
                            return siblingCheck( a, b );
                        }

                        // Otherwise we need full lists of their ancestors for comparison
                        cur = a;
                        while ( ( cur = cur.parentNode ) ) {
                            ap.unshift( cur );
                        }
                        cur = b;
                        while ( ( cur = cur.parentNode ) ) {
                            bp.unshift( cur );
                        }

                        // Walk down the tree looking for a discrepancy
                        while ( ap[ i ] === bp[ i ] ) {
                            i++;
                        }

                        return i ?

                            // Do a sibling check if the nodes have a common ancestor
                            siblingCheck( ap[ i ], bp[ i ] ) :

                            // Otherwise nodes in our document sort first
                            // Support: IE 11+, Edge 17 - 18+
                            // IE/Edge sometimes throw a "Permission denied" error when strict-comparing
                            // two documents; shallow comparisons work.
                            /* eslint-disable eqeqeq */
                            ap[ i ] == preferredDoc ? -1 :
                                bp[ i ] == preferredDoc ? 1 :
                                    /* eslint-enable eqeqeq */
                                    0;
                    };

                return document;
            };

            Sizzle.matches = function( expr, elements ) {
                return Sizzle( expr, null, null, elements );
            };

            Sizzle.matchesSelector = function( elem, expr ) {
                setDocument( elem );

                if ( support.matchesSelector && documentIsHTML &&
                    !nonnativeSelectorCache[ expr + " " ] &&
                    ( !rbuggyMatches || !rbuggyMatches.test( expr ) ) &&
                    ( !rbuggyQSA     || !rbuggyQSA.test( expr ) ) ) {

                    try {
                        var ret = matches.call( elem, expr );

                        // IE 9's matchesSelector returns false on disconnected nodes
                        if ( ret || support.disconnectedMatch ||

                            // As well, disconnected nodes are said to be in a document
                            // fragment in IE 9
                            elem.document && elem.document.nodeType !== 11 ) {
                            return ret;
                        }
                    } catch ( e ) {
                        nonnativeSelectorCache( expr, true );
                    }
                }

                return Sizzle( expr, document, null, [ elem ] ).length > 0;
            };

            Sizzle.contains = function( context, elem ) {

                // Set document vars if needed
                // Support: IE 11+, Edge 17 - 18+
                // IE/Edge sometimes throw a "Permission denied" error when strict-comparing
                // two documents; shallow comparisons work.
                // eslint-disable-next-line eqeqeq
                if ( ( context.ownerDocument || context ) != document ) {
                    setDocument( context );
                }
                return contains( context, elem );
            };

            Sizzle.attr = function( elem, name ) {

                // Set document vars if needed
                // Support: IE 11+, Edge 17 - 18+
                // IE/Edge sometimes throw a "Permission denied" error when strict-comparing
                // two documents; shallow comparisons work.
                // eslint-disable-next-line eqeqeq
                if ( ( elem.ownerDocument || elem ) != document ) {
                    setDocument( elem );
                }

                var fn = Expr.attrHandle[ name.toLowerCase() ],

                    // Don't get fooled by Object.prototype properties (jQuery #13807)
                    val = fn && hasOwn.call( Expr.attrHandle, name.toLowerCase() ) ?
                        fn( elem, name, !documentIsHTML ) :
                        undefined;

                return val !== undefined ?
                    val :
                    support.attributes || !documentIsHTML ?
                        elem.getAttribute( name ) :
                        ( val = elem.getAttributeNode( name ) ) && val.specified ?
                            val.value :
                            null;
            };

            Sizzle.escape = function( sel ) {
                return ( sel + "" ).replace( rcssescape, fcssescape );
            };

            Sizzle.error = function( msg ) {
                throw new Error( "Syntax error, unrecognized expression: " + msg );
            };

            /**
             * Document sorting and removing duplicates
             * @param {ArrayLike} results
             */
            Sizzle.uniqueSort = function( results ) {
                var elem,
                    duplicates = [],
                    j = 0,
                    i = 0;

                // Unless we *know* we can detect duplicates, assume their presence
                hasDuplicate = !support.detectDuplicates;
                sortInput = !support.sortStable && results.slice( 0 );
                results.sort( sortOrder );

                if ( hasDuplicate ) {
                    while ( ( elem = results[ i++ ] ) ) {
                        if ( elem === results[ i ] ) {
                            j = duplicates.push( i );
                        }
                    }
                    while ( j-- ) {
                        results.splice( duplicates[ j ], 1 );
                    }
                }

                // Clear input after sorting to release objects
                // See https://github.com/jquery/sizzle/pull/225
                sortInput = null;

                return results;
            };

            /**
             * Utility function for retrieving the text value of an array of DOM nodes
             * @param {Array|Element} elem
             */
            getText = Sizzle.getText = function( elem ) {
                var node,
                    ret = "",
                    i = 0,
                    nodeType = elem.nodeType;

                if ( !nodeType ) {

                    // If no nodeType, this is expected to be an array
                    while ( ( node = elem[ i++ ] ) ) {

                        // Do not traverse comment nodes
                        ret += getText( node );
                    }
                } else if ( nodeType === 1 || nodeType === 9 || nodeType === 11 ) {

                    // Use textContent for elements
                    // innerText usage removed for consistency of new lines (jQuery #11153)
                    if ( typeof elem.textContent === "string" ) {
                        return elem.textContent;
                    } else {

                        // Traverse its children
                        for ( elem = elem.firstChild; elem; elem = elem.nextSibling ) {
                            ret += getText( elem );
                        }
                    }
                } else if ( nodeType === 3 || nodeType === 4 ) {
                    return elem.nodeValue;
                }

                // Do not include comment or processing instruction nodes

                return ret;
            };

            Expr = Sizzle.selectors = {

                // Can be adjusted by the user
                cacheLength: 50,

                createPseudo: markFunction,

                match: matchExpr,

                attrHandle: {},

                find: {},

                relative: {
                    ">": { dir: "parentNode", first: true },
                    " ": { dir: "parentNode" },
                    "+": { dir: "previousSibling", first: true },
                    "~": { dir: "previousSibling" }
                },

                preFilter: {
                    "ATTR": function( match ) {
                        match[ 1 ] = match[ 1 ].replace( runescape, funescape );

                        // Move the given value to match[3] whether quoted or unquoted
                        match[ 3 ] = ( match[ 3 ] || match[ 4 ] ||
                            match[ 5 ] || "" ).replace( runescape, funescape );

                        if ( match[ 2 ] === "~=" ) {
                            match[ 3 ] = " " + match[ 3 ] + " ";
                        }

                        return match.slice( 0, 4 );
                    },

                    "CHILD": function( match ) {

                        /* matches from matchExpr["CHILD"]
				1 type (only|nth|...)
				2 what (child|of-type)
				3 argument (even|odd|\d*|\d*n([+-]\d+)?|...)
				4 xn-component of xn+y argument ([+-]?\d*n|)
				5 sign of xn-component
				6 x of xn-component
				7 sign of y-component
				8 y of y-component
			*/
                        match[ 1 ] = match[ 1 ].toLowerCase();

                        if ( match[ 1 ].slice( 0, 3 ) === "nth" ) {

                            // nth-* requires argument
                            if ( !match[ 3 ] ) {
                                Sizzle.error( match[ 0 ] );
                            }

                            // numeric x and y parameters for Expr.filter.CHILD
                            // remember that false/true cast respectively to 0/1
                            match[ 4 ] = +( match[ 4 ] ?
                                match[ 5 ] + ( match[ 6 ] || 1 ) :
                                2 * ( match[ 3 ] === "even" || match[ 3 ] === "odd" ) );
                            match[ 5 ] = +( ( match[ 7 ] + match[ 8 ] ) || match[ 3 ] === "odd" );

                            // other types prohibit arguments
                        } else if ( match[ 3 ] ) {
                            Sizzle.error( match[ 0 ] );
                        }

                        return match;
                    },

                    "PSEUDO": function( match ) {
                        var excess,
                            unquoted = !match[ 6 ] && match[ 2 ];

                        if ( matchExpr[ "CHILD" ].test( match[ 0 ] ) ) {
                            return null;
                        }

                        // Accept quoted arguments as-is
                        if ( match[ 3 ] ) {
                            match[ 2 ] = match[ 4 ] || match[ 5 ] || "";

                            // Strip excess characters from unquoted arguments
                        } else if ( unquoted && rpseudo.test( unquoted ) &&

                            // Get excess from tokenize (recursively)
                            ( excess = tokenize( unquoted, true ) ) &&

                            // advance to the next closing parenthesis
                            ( excess = unquoted.indexOf( ")", unquoted.length - excess ) - unquoted.length ) ) {

                            // excess is a negative index
                            match[ 0 ] = match[ 0 ].slice( 0, excess );
                            match[ 2 ] = unquoted.slice( 0, excess );
                        }

                        // Return only captures needed by the pseudo filter method (type and argument)
                        return match.slice( 0, 3 );
                    }
                },

                filter: {

                    "TAG": function( nodeNameSelector ) {
                        var nodeName = nodeNameSelector.replace( runescape, funescape ).toLowerCase();
                        return nodeNameSelector === "*" ?
                            function() {
                                return true;
                            } :
                            function( elem ) {
                                return elem.nodeName && elem.nodeName.toLowerCase() === nodeName;
                            };
                    },

                    "CLASS": function( className ) {
                        var pattern = classCache[ className + " " ];

                        return pattern ||
                            ( pattern = new RegExp( "(^|" + whitespace +
                                ")" + className + "(" + whitespace + "|$)" ) ) && classCache(
                                className, function( elem ) {
                                    return pattern.test(
                                        typeof elem.className === "string" && elem.className ||
                                        typeof elem.getAttribute !== "undefined" &&
                                        elem.getAttribute( "class" ) ||
                                        ""
                                    );
                                } );
                    },

                    "ATTR": function( name, operator, check ) {
                        return function( elem ) {
                            var result = Sizzle.attr( elem, name );

                            if ( result == null ) {
                                return operator === "!=";
                            }
                            if ( !operator ) {
                                return true;
                            }

                            result += "";

                            /* eslint-disable max-len */

                            return operator === "=" ? result === check :
                                operator === "!=" ? result !== check :
                                    operator === "^=" ? check && result.indexOf( check ) === 0 :
                                        operator === "*=" ? check && result.indexOf( check ) > -1 :
                                            operator === "$=" ? check && result.slice( -check.length ) === check :
                                                operator === "~=" ? ( " " + result.replace( rwhitespace, " " ) + " " ).indexOf( check ) > -1 :
                                                    operator === "|=" ? result === check || result.slice( 0, check.length + 1 ) === check + "-" :
                                                        false;
                            /* eslint-enable max-len */

                        };
                    },

                    "CHILD": function( type, what, _argument, first, last ) {
                        var simple = type.slice( 0, 3 ) !== "nth",
                            forward = type.slice( -4 ) !== "last",
                            ofType = what === "of-type";

                        return first === 1 && last === 0 ?

                            // Shortcut for :nth-*(n)
                            function( elem ) {
                                return !!elem.parentNode;
                            } :

                            function( elem, _context, xml ) {
                                var cache, uniqueCache, outerCache, node, nodeIndex, start,
                                    dir = simple !== forward ? "nextSibling" : "previousSibling",
                                    parent = elem.parentNode,
                                    name = ofType && elem.nodeName.toLowerCase(),
                                    useCache = !xml && !ofType,
                                    diff = false;

                                if ( parent ) {

                                    // :(first|last|only)-(child|of-type)
                                    if ( simple ) {
                                        while ( dir ) {
                                            node = elem;
                                            while ( ( node = node[ dir ] ) ) {
                                                if ( ofType ?
                                                    node.nodeName.toLowerCase() === name :
                                                    node.nodeType === 1 ) {

                                                    return false;
                                                }
                                            }

                                            // Reverse direction for :only-* (if we haven't yet done so)
                                            start = dir = type === "only" && !start && "nextSibling";
                                        }
                                        return true;
                                    }

                                    start = [ forward ? parent.firstChild : parent.lastChild ];

                                    // non-xml :nth-child(...) stores cache data on `parent`
                                    if ( forward && useCache ) {

                                        // Seek `elem` from a previously-cached index

                                        // ...in a gzip-friendly way
                                        node = parent;
                                        outerCache = node[ expando ] || ( node[ expando ] = {} );

                                        // Support: IE <9 only
                                        // Defend against cloned attroperties (jQuery gh-1709)
                                        uniqueCache = outerCache[ node.uniqueID ] ||
                                            ( outerCache[ node.uniqueID ] = {} );

                                        cache = uniqueCache[ type ] || [];
                                        nodeIndex = cache[ 0 ] === dirruns && cache[ 1 ];
                                        diff = nodeIndex && cache[ 2 ];
                                        node = nodeIndex && parent.childNodes[ nodeIndex ];

                                        while ( ( node = ++nodeIndex && node && node[ dir ] ||

                                            // Fallback to seeking `elem` from the start
                                            ( diff = nodeIndex = 0 ) || start.pop() ) ) {

                                            // When found, cache indexes on `parent` and break
                                            if ( node.nodeType === 1 && ++diff && node === elem ) {
                                                uniqueCache[ type ] = [ dirruns, nodeIndex, diff ];
                                                break;
                                            }
                                        }

                                    } else {

                                        // Use previously-cached element index if available
                                        if ( useCache ) {

                                            // ...in a gzip-friendly way
                                            node = elem;
                                            outerCache = node[ expando ] || ( node[ expando ] = {} );

                                            // Support: IE <9 only
                                            // Defend against cloned attroperties (jQuery gh-1709)
                                            uniqueCache = outerCache[ node.uniqueID ] ||
                                                ( outerCache[ node.uniqueID ] = {} );

                                            cache = uniqueCache[ type ] || [];
                                            nodeIndex = cache[ 0 ] === dirruns && cache[ 1 ];
                                            diff = nodeIndex;
                                        }

                                        // xml :nth-child(...)
                                        // or :nth-last-child(...) or :nth(-last)?-of-type(...)
                                        if ( diff === false ) {

                                            // Use the same loop as above to seek `elem` from the start
                                            while ( ( node = ++nodeIndex && node && node[ dir ] ||
                                                ( diff = nodeIndex = 0 ) || start.pop() ) ) {

                                                if ( ( ofType ?
                                                    node.nodeName.toLowerCase() === name :
                                                    node.nodeType === 1 ) &&
                                                    ++diff ) {

                                                    // Cache the index of each encountered element
                                                    if ( useCache ) {
                                                        outerCache = node[ expando ] ||
                                                            ( node[ expando ] = {} );

                                                        // Support: IE <9 only
                                                        // Defend against cloned attroperties (jQuery gh-1709)
                                                        uniqueCache = outerCache[ node.uniqueID ] ||
                                                            ( outerCache[ node.uniqueID ] = {} );

                                                        uniqueCache[ type ] = [ dirruns, diff ];
                                                    }

                                                    if ( node === elem ) {
                                                        break;
                                                    }
                                                }
                                            }
                                        }
                                    }

                                    // Incorporate the offset, then check against cycle size
                                    diff -= last;
                                    return diff === first || ( diff % first === 0 && diff / first >= 0 );
                                }
                            };
                    },

                    "PSEUDO": function( pseudo, argument ) {

                        // pseudo-class names are case-insensitive
                        // http://www.w3.org/TR/selectors/#pseudo-classes
                        // Prioritize by case sensitivity in case custom pseudos are added with uppercase letters
                        // Remember that setFilters inherits from pseudos
                        var args,
                            fn = Expr.pseudos[ pseudo ] || Expr.setFilters[ pseudo.toLowerCase() ] ||
                                Sizzle.error( "unsupported pseudo: " + pseudo );

                        // The user may use createPseudo to indicate that
                        // arguments are needed to create the filter function
                        // just as Sizzle does
                        if ( fn[ expando ] ) {
                            return fn( argument );
                        }

                        // But maintain support for old signatures
                        if ( fn.length > 1 ) {
                            args = [ pseudo, pseudo, "", argument ];
                            return Expr.setFilters.hasOwnProperty( pseudo.toLowerCase() ) ?
                                markFunction( function( seed, matches ) {
                                    var idx,
                                        matched = fn( seed, argument ),
                                        i = matched.length;
                                    while ( i-- ) {
                                        idx = indexOf( seed, matched[ i ] );
                                        seed[ idx ] = !( matches[ idx ] = matched[ i ] );
                                    }
                                } ) :
                                function( elem ) {
                                    return fn( elem, 0, args );
                                };
                        }

                        return fn;
                    }
                },

                pseudos: {

                    // Potentially complex pseudos
                    "not": markFunction( function( selector ) {

                        // Trim the selector passed to compile
                        // to avoid treating leading and trailing
                        // spaces as combinators
                        var input = [],
                            results = [],
                            matcher = compile( selector.replace( rtrim, "$1" ) );

                        return matcher[ expando ] ?
                            markFunction( function( seed, matches, _context, xml ) {
                                var elem,
                                    unmatched = matcher( seed, null, xml, [] ),
                                    i = seed.length;

                                // Match elements unmatched by `matcher`
                                while ( i-- ) {
                                    if ( ( elem = unmatched[ i ] ) ) {
                                        seed[ i ] = !( matches[ i ] = elem );
                                    }
                                }
                            } ) :
                            function( elem, _context, xml ) {
                                input[ 0 ] = elem;
                                matcher( input, null, xml, results );

                                // Don't keep the element (issue #299)
                                input[ 0 ] = null;
                                return !results.pop();
                            };
                    } ),

                    "has": markFunction( function( selector ) {
                        return function( elem ) {
                            return Sizzle( selector, elem ).length > 0;
                        };
                    } ),

                    "contains": markFunction( function( text ) {
                        text = text.replace( runescape, funescape );
                        return function( elem ) {
                            return ( elem.textContent || getText( elem ) ).indexOf( text ) > -1;
                        };
                    } ),

                    // "Whether an element is represented by a :lang() selector
                    // is based solely on the element's language value
                    // being equal to the identifier C,
                    // or beginning with the identifier C immediately followed by "-".
                    // The matching of C against the element's language value is performed case-insensitively.
                    // The identifier C does not have to be a valid language name."
                    // http://www.w3.org/TR/selectors/#lang-pseudo
                    "lang": markFunction( function( lang ) {

                        // lang value must be a valid identifier
                        if ( !ridentifier.test( lang || "" ) ) {
                            Sizzle.error( "unsupported lang: " + lang );
                        }
                        lang = lang.replace( runescape, funescape ).toLowerCase();
                        return function( elem ) {
                            var elemLang;
                            do {
                                if ( ( elemLang = documentIsHTML ?
                                    elem.lang :
                                    elem.getAttribute( "xml:lang" ) || elem.getAttribute( "lang" ) ) ) {

                                    elemLang = elemLang.toLowerCase();
                                    return elemLang === lang || elemLang.indexOf( lang + "-" ) === 0;
                                }
                            } while ( ( elem = elem.parentNode ) && elem.nodeType === 1 );
                            return false;
                        };
                    } ),

                    // Miscellaneous
                    "target": function( elem ) {
                        var hash = window.location && window.location.hash;
                        return hash && hash.slice( 1 ) === elem.id;
                    },

                    "root": function( elem ) {
                        return elem === docElem;
                    },

                    "focus": function( elem ) {
                        return elem === document.activeElement &&
                            ( !document.hasFocus || document.hasFocus() ) &&
                            !!( elem.type || elem.href || ~elem.tabIndex );
                    },

                    // Boolean properties
                    "enabled": createDisabledPseudo( false ),
                    "disabled": createDisabledPseudo( true ),

                    "checked": function( elem ) {

                        // In CSS3, :checked should return both checked and selected elements
                        // http://www.w3.org/TR/2011/REC-css3-selectors-20110929/#checked
                        var nodeName = elem.nodeName.toLowerCase();
                        return ( nodeName === "input" && !!elem.checked ) ||
                            ( nodeName === "option" && !!elem.selected );
                    },

                    "selected": function( elem ) {

                        // Accessing this property makes selected-by-default
                        // options in Safari work properly
                        if ( elem.parentNode ) {
                            // eslint-disable-next-line no-unused-expressions
                            elem.parentNode.selectedIndex;
                        }

                        return elem.selected === true;
                    },

                    // Contents
                    "empty": function( elem ) {

                        // http://www.w3.org/TR/selectors/#empty-pseudo
                        // :empty is negated by element (1) or content nodes (text: 3; cdata: 4; entity ref: 5),
                        //   but not by others (comment: 8; processing instruction: 7; etc.)
                        // nodeType < 6 works because attributes (2) do not appear as children
                        for ( elem = elem.firstChild; elem; elem = elem.nextSibling ) {
                            if ( elem.nodeType < 6 ) {
                                return false;
                            }
                        }
                        return true;
                    },

                    "parent": function( elem ) {
                        return !Expr.pseudos[ "empty" ]( elem );
                    },

                    // Element/input types
                    "header": function( elem ) {
                        return rheader.test( elem.nodeName );
                    },

                    "input": function( elem ) {
                        return rinputs.test( elem.nodeName );
                    },

                    "button": function( elem ) {
                        var name = elem.nodeName.toLowerCase();
                        return name === "input" && elem.type === "button" || name === "button";
                    },

                    "text": function( elem ) {
                        var attr;
                        return elem.nodeName.toLowerCase() === "input" &&
                            elem.type === "text" &&

                            // Support: IE<8
                            // New HTML5 attribute values (e.g., "search") appear with elem.type === "text"
                            ( ( attr = elem.getAttribute( "type" ) ) == null ||
                                attr.toLowerCase() === "text" );
                    },

                    // Position-in-collection
                    "first": createPositionalPseudo( function() {
                        return [ 0 ];
                    } ),

                    "last": createPositionalPseudo( function( _matchIndexes, length ) {
                        return [ length - 1 ];
                    } ),

                    "eq": createPositionalPseudo( function( _matchIndexes, length, argument ) {
                        return [ argument < 0 ? argument + length : argument ];
                    } ),

                    "even": createPositionalPseudo( function( matchIndexes, length ) {
                        var i = 0;
                        for ( ; i < length; i += 2 ) {
                            matchIndexes.push( i );
                        }
                        return matchIndexes;
                    } ),

                    "odd": createPositionalPseudo( function( matchIndexes, length ) {
                        var i = 1;
                        for ( ; i < length; i += 2 ) {
                            matchIndexes.push( i );
                        }
                        return matchIndexes;
                    } ),

                    "lt": createPositionalPseudo( function( matchIndexes, length, argument ) {
                        var i = argument < 0 ?
                            argument + length :
                            argument > length ?
                                length :
                                argument;
                        for ( ; --i >= 0; ) {
                            matchIndexes.push( i );
                        }
                        return matchIndexes;
                    } ),

                    "gt": createPositionalPseudo( function( matchIndexes, length, argument ) {
                        var i = argument < 0 ? argument + length : argument;
                        for ( ; ++i < length; ) {
                            matchIndexes.push( i );
                        }
                        return matchIndexes;
                    } )
                }
            };

            Expr.pseudos[ "nth" ] = Expr.pseudos[ "eq" ];

// Add button/input type pseudos
            for ( i in { radio: true, checkbox: true, file: true, password: true, image: true } ) {
                Expr.pseudos[ i ] = createInputPseudo( i );
            }
            for ( i in { submit: true, reset: true } ) {
                Expr.pseudos[ i ] = createButtonPseudo( i );
            }

// Easy API for creating new setFilters
            function setFilters() {}
            setFilters.prototype = Expr.filters = Expr.pseudos;
            Expr.setFilters = new setFilters();

            tokenize = Sizzle.tokenize = function( selector, parseOnly ) {
                var matched, match, tokens, type,
                    soFar, groups, preFilters,
                    cached = tokenCache[ selector + " " ];

                if ( cached ) {
                    return parseOnly ? 0 : cached.slice( 0 );
                }

                soFar = selector;
                groups = [];
                preFilters = Expr.preFilter;

                while ( soFar ) {

                    // Comma and first run
                    if ( !matched || ( match = rcomma.exec( soFar ) ) ) {
                        if ( match ) {

                            // Don't consume trailing commas as valid
                            soFar = soFar.slice( match[ 0 ].length ) || soFar;
                        }
                        groups.push( ( tokens = [] ) );
                    }

                    matched = false;

                    // Combinators
                    if ( ( match = rcombinators.exec( soFar ) ) ) {
                        matched = match.shift();
                        tokens.push( {
                            value: matched,

                            // Cast descendant combinators to space
                            type: match[ 0 ].replace( rtrim, " " )
                        } );
                        soFar = soFar.slice( matched.length );
                    }

                    // Filters
                    for ( type in Expr.filter ) {
                        if ( ( match = matchExpr[ type ].exec( soFar ) ) && ( !preFilters[ type ] ||
                            ( match = preFilters[ type ]( match ) ) ) ) {
                            matched = match.shift();
                            tokens.push( {
                                value: matched,
                                type: type,
                                matches: match
                            } );
                            soFar = soFar.slice( matched.length );
                        }
                    }

                    if ( !matched ) {
                        break;
                    }
                }

                // Return the length of the invalid excess
                // if we're just parsing
                // Otherwise, throw an error or return tokens
                return parseOnly ?
                    soFar.length :
                    soFar ?
                        Sizzle.error( selector ) :

                        // Cache the tokens
                        tokenCache( selector, groups ).slice( 0 );
            };

            function toSelector( tokens ) {
                var i = 0,
                    len = tokens.length,
                    selector = "";
                for ( ; i < len; i++ ) {
                    selector += tokens[ i ].value;
                }
                return selector;
            }

            function addCombinator( matcher, combinator, base ) {
                var dir = combinator.dir,
                    skip = combinator.next,
                    key = skip || dir,
                    checkNonElements = base && key === "parentNode",
                    doneName = done++;

                return combinator.first ?

                    // Check against closest ancestor/preceding element
                    function( elem, context, xml ) {
                        while ( ( elem = elem[ dir ] ) ) {
                            if ( elem.nodeType === 1 || checkNonElements ) {
                                return matcher( elem, context, xml );
                            }
                        }
                        return false;
                    } :

                    // Check against all ancestor/preceding elements
                    function( elem, context, xml ) {
                        var oldCache, uniqueCache, outerCache,
                            newCache = [ dirruns, doneName ];

                        // We can't set arbitrary data on XML nodes, so they don't benefit from combinator caching
                        if ( xml ) {
                            while ( ( elem = elem[ dir ] ) ) {
                                if ( elem.nodeType === 1 || checkNonElements ) {
                                    if ( matcher( elem, context, xml ) ) {
                                        return true;
                                    }
                                }
                            }
                        } else {
                            while ( ( elem = elem[ dir ] ) ) {
                                if ( elem.nodeType === 1 || checkNonElements ) {
                                    outerCache = elem[ expando ] || ( elem[ expando ] = {} );

                                    // Support: IE <9 only
                                    // Defend against cloned attroperties (jQuery gh-1709)
                                    uniqueCache = outerCache[ elem.uniqueID ] ||
                                        ( outerCache[ elem.uniqueID ] = {} );

                                    if ( skip && skip === elem.nodeName.toLowerCase() ) {
                                        elem = elem[ dir ] || elem;
                                    } else if ( ( oldCache = uniqueCache[ key ] ) &&
                                        oldCache[ 0 ] === dirruns && oldCache[ 1 ] === doneName ) {

                                        // Assign to newCache so results back-propagate to previous elements
                                        return ( newCache[ 2 ] = oldCache[ 2 ] );
                                    } else {

                                        // Reuse newcache so results back-propagate to previous elements
                                        uniqueCache[ key ] = newCache;

                                        // A match means we're done; a fail means we have to keep checking
                                        if ( ( newCache[ 2 ] = matcher( elem, context, xml ) ) ) {
                                            return true;
                                        }
                                    }
                                }
                            }
                        }
                        return false;
                    };
            }

            function elementMatcher( matchers ) {
                return matchers.length > 1 ?
                    function( elem, context, xml ) {
                        var i = matchers.length;
                        while ( i-- ) {
                            if ( !matchers[ i ]( elem, context, xml ) ) {
                                return false;
                            }
                        }
                        return true;
                    } :
                    matchers[ 0 ];
            }

            function multipleContexts( selector, contexts, results ) {
                var i = 0,
                    len = contexts.length;
                for ( ; i < len; i++ ) {
                    Sizzle( selector, contexts[ i ], results );
                }
                return results;
            }

            function condense( unmatched, map, filter, context, xml ) {
                var elem,
                    newUnmatched = [],
                    i = 0,
                    len = unmatched.length,
                    mapped = map != null;

                for ( ; i < len; i++ ) {
                    if ( ( elem = unmatched[ i ] ) ) {
                        if ( !filter || filter( elem, context, xml ) ) {
                            newUnmatched.push( elem );
                            if ( mapped ) {
                                map.push( i );
                            }
                        }
                    }
                }

                return newUnmatched;
            }

            function setMatcher( preFilter, selector, matcher, postFilter, postFinder, postSelector ) {
                if ( postFilter && !postFilter[ expando ] ) {
                    postFilter = setMatcher( postFilter );
                }
                if ( postFinder && !postFinder[ expando ] ) {
                    postFinder = setMatcher( postFinder, postSelector );
                }
                return markFunction( function( seed, results, context, xml ) {
                    var temp, i, elem,
                        preMap = [],
                        postMap = [],
                        preexisting = results.length,

                        // Get initial elements from seed or context
                        elems = seed || multipleContexts(
                            selector || "*",
                            context.nodeType ? [ context ] : context,
                            []
                        ),

                        // Prefilter to get matcher input, preserving a map for seed-results synchronization
                        matcherIn = preFilter && ( seed || !selector ) ?
                            condense( elems, preMap, preFilter, context, xml ) :
                            elems,

                        matcherOut = matcher ?

                            // If we have a postFinder, or filtered seed, or non-seed postFilter or preexisting results,
                            postFinder || ( seed ? preFilter : preexisting || postFilter ) ?

                                // ...intermediate processing is necessary
                                [] :

                                // ...otherwise use results directly
                                results :
                            matcherIn;

                    // Find primary matches
                    if ( matcher ) {
                        matcher( matcherIn, matcherOut, context, xml );
                    }

                    // Apply postFilter
                    if ( postFilter ) {
                        temp = condense( matcherOut, postMap );
                        postFilter( temp, [], context, xml );

                        // Un-match failing elements by moving them back to matcherIn
                        i = temp.length;
                        while ( i-- ) {
                            if ( ( elem = temp[ i ] ) ) {
                                matcherOut[ postMap[ i ] ] = !( matcherIn[ postMap[ i ] ] = elem );
                            }
                        }
                    }

                    if ( seed ) {
                        if ( postFinder || preFilter ) {
                            if ( postFinder ) {

                                // Get the final matcherOut by condensing this intermediate into postFinder contexts
                                temp = [];
                                i = matcherOut.length;
                                while ( i-- ) {
                                    if ( ( elem = matcherOut[ i ] ) ) {

                                        // Restore matcherIn since elem is not yet a final match
                                        temp.push( ( matcherIn[ i ] = elem ) );
                                    }
                                }
                                postFinder( null, ( matcherOut = [] ), temp, xml );
                            }

                            // Move matched elements from seed to results to keep them synchronized
                            i = matcherOut.length;
                            while ( i-- ) {
                                if ( ( elem = matcherOut[ i ] ) &&
                                    ( temp = postFinder ? indexOf( seed, elem ) : preMap[ i ] ) > -1 ) {

                                    seed[ temp ] = !( results[ temp ] = elem );
                                }
                            }
                        }

                        // Add elements to results, through postFinder if defined
                    } else {
                        matcherOut = condense(
                            matcherOut === results ?
                                matcherOut.splice( preexisting, matcherOut.length ) :
                                matcherOut
                        );
                        if ( postFinder ) {
                            postFinder( null, results, matcherOut, xml );
                        } else {
                            push.apply( results, matcherOut );
                        }
                    }
                } );
            }

            function matcherFromTokens( tokens ) {
                var checkContext, matcher, j,
                    len = tokens.length,
                    leadingRelative = Expr.relative[ tokens[ 0 ].type ],
                    implicitRelative = leadingRelative || Expr.relative[ " " ],
                    i = leadingRelative ? 1 : 0,

                    // The foundational matcher ensures that elements are reachable from top-level context(s)
                    matchContext = addCombinator( function( elem ) {
                        return elem === checkContext;
                    }, implicitRelative, true ),
                    matchAnyContext = addCombinator( function( elem ) {
                        return indexOf( checkContext, elem ) > -1;
                    }, implicitRelative, true ),
                    matchers = [ function( elem, context, xml ) {
                        var ret = ( !leadingRelative && ( xml || context !== outermostContext ) ) || (
                            ( checkContext = context ).nodeType ?
                                matchContext( elem, context, xml ) :
                                matchAnyContext( elem, context, xml ) );

                        // Avoid hanging onto element (issue #299)
                        checkContext = null;
                        return ret;
                    } ];

                for ( ; i < len; i++ ) {
                    if ( ( matcher = Expr.relative[ tokens[ i ].type ] ) ) {
                        matchers = [ addCombinator( elementMatcher( matchers ), matcher ) ];
                    } else {
                        matcher = Expr.filter[ tokens[ i ].type ].apply( null, tokens[ i ].matches );

                        // Return special upon seeing a positional matcher
                        if ( matcher[ expando ] ) {

                            // Find the next relative operator (if any) for proper handling
                            j = ++i;
                            for ( ; j < len; j++ ) {
                                if ( Expr.relative[ tokens[ j ].type ] ) {
                                    break;
                                }
                            }
                            return setMatcher(
                                i > 1 && elementMatcher( matchers ),
                                i > 1 && toSelector(

                                // If the preceding token was a descendant combinator, insert an implicit any-element `*`
                                tokens
                                    .slice( 0, i - 1 )
                                    .concat( { value: tokens[ i - 2 ].type === " " ? "*" : "" } )
                                ).replace( rtrim, "$1" ),
                                matcher,
                                i < j && matcherFromTokens( tokens.slice( i, j ) ),
                                j < len && matcherFromTokens( ( tokens = tokens.slice( j ) ) ),
                                j < len && toSelector( tokens )
                            );
                        }
                        matchers.push( matcher );
                    }
                }

                return elementMatcher( matchers );
            }

            function matcherFromGroupMatchers( elementMatchers, setMatchers ) {
                var bySet = setMatchers.length > 0,
                    byElement = elementMatchers.length > 0,
                    superMatcher = function( seed, context, xml, results, outermost ) {
                        var elem, j, matcher,
                            matchedCount = 0,
                            i = "0",
                            unmatched = seed && [],
                            setMatched = [],
                            contextBackup = outermostContext,

                            // We must always have either seed elements or outermost context
                            elems = seed || byElement && Expr.find[ "TAG" ]( "*", outermost ),

                            // Use integer dirruns iff this is the outermost matcher
                            dirrunsUnique = ( dirruns += contextBackup == null ? 1 : Math.random() || 0.1 ),
                            len = elems.length;

                        if ( outermost ) {

                            // Support: IE 11+, Edge 17 - 18+
                            // IE/Edge sometimes throw a "Permission denied" error when strict-comparing
                            // two documents; shallow comparisons work.
                            // eslint-disable-next-line eqeqeq
                            outermostContext = context == document || context || outermost;
                        }

                        // Add elements passing elementMatchers directly to results
                        // Support: IE<9, Safari
                        // Tolerate NodeList properties (IE: "length"; Safari: <number>) matching elements by id
                        for ( ; i !== len && ( elem = elems[ i ] ) != null; i++ ) {
                            if ( byElement && elem ) {
                                j = 0;

                                // Support: IE 11+, Edge 17 - 18+
                                // IE/Edge sometimes throw a "Permission denied" error when strict-comparing
                                // two documents; shallow comparisons work.
                                // eslint-disable-next-line eqeqeq
                                if ( !context && elem.ownerDocument != document ) {
                                    setDocument( elem );
                                    xml = !documentIsHTML;
                                }
                                while ( ( matcher = elementMatchers[ j++ ] ) ) {
                                    if ( matcher( elem, context || document, xml ) ) {
                                        results.push( elem );
                                        break;
                                    }
                                }
                                if ( outermost ) {
                                    dirruns = dirrunsUnique;
                                }
                            }

                            // Track unmatched elements for set filters
                            if ( bySet ) {

                                // They will have gone through all possible matchers
                                if ( ( elem = !matcher && elem ) ) {
                                    matchedCount--;
                                }

                                // Lengthen the array for every element, matched or not
                                if ( seed ) {
                                    unmatched.push( elem );
                                }
                            }
                        }

                        // `i` is now the count of elements visited above, and adding it to `matchedCount`
                        // makes the latter nonnegative.
                        matchedCount += i;

                        // Apply set filters to unmatched elements
                        // NOTE: This can be skipped if there are no unmatched elements (i.e., `matchedCount`
                        // equals `i`), unless we didn't visit _any_ elements in the above loop because we have
                        // no element matchers and no seed.
                        // Incrementing an initially-string "0" `i` allows `i` to remain a string only in that
                        // case, which will result in a "00" `matchedCount` that differs from `i` but is also
                        // numerically zero.
                        if ( bySet && i !== matchedCount ) {
                            j = 0;
                            while ( ( matcher = setMatchers[ j++ ] ) ) {
                                matcher( unmatched, setMatched, context, xml );
                            }

                            if ( seed ) {

                                // Reintegrate element matches to eliminate the need for sorting
                                if ( matchedCount > 0 ) {
                                    while ( i-- ) {
                                        if ( !( unmatched[ i ] || setMatched[ i ] ) ) {
                                            setMatched[ i ] = pop.call( results );
                                        }
                                    }
                                }

                                // Discard index placeholder values to get only actual matches
                                setMatched = condense( setMatched );
                            }

                            // Add matches to results
                            push.apply( results, setMatched );

                            // Seedless set matches succeeding multiple successful matchers stipulate sorting
                            if ( outermost && !seed && setMatched.length > 0 &&
                                ( matchedCount + setMatchers.length ) > 1 ) {

                                Sizzle.uniqueSort( results );
                            }
                        }

                        // Override manipulation of globals by nested matchers
                        if ( outermost ) {
                            dirruns = dirrunsUnique;
                            outermostContext = contextBackup;
                        }

                        return unmatched;
                    };

                return bySet ?
                    markFunction( superMatcher ) :
                    superMatcher;
            }

            compile = Sizzle.compile = function( selector, match /* Internal Use Only */ ) {
                var i,
                    setMatchers = [],
                    elementMatchers = [],
                    cached = compilerCache[ selector + " " ];

                if ( !cached ) {

                    // Generate a function of recursive functions that can be used to check each element
                    if ( !match ) {
                        match = tokenize( selector );
                    }
                    i = match.length;
                    while ( i-- ) {
                        cached = matcherFromTokens( match[ i ] );
                        if ( cached[ expando ] ) {
                            setMatchers.push( cached );
                        } else {
                            elementMatchers.push( cached );
                        }
                    }

                    // Cache the compiled function
                    cached = compilerCache(
                        selector,
                        matcherFromGroupMatchers( elementMatchers, setMatchers )
                    );

                    // Save selector and tokenization
                    cached.selector = selector;
                }
                return cached;
            };

            /**
             * A low-level selection function that works with Sizzle's compiled
             *  selector functions
             * @param {String|Function} selector A selector or a pre-compiled
             *  selector function built with Sizzle.compile
             * @param {Element} context
             * @param {Array} [results]
             * @param {Array} [seed] A set of elements to match against
             */
            select = Sizzle.select = function( selector, context, results, seed ) {
                var i, tokens, token, type, find,
                    compiled = typeof selector === "function" && selector,
                    match = !seed && tokenize( ( selector = compiled.selector || selector ) );

                results = results || [];

                // Try to minimize operations if there is only one selector in the list and no seed
                // (the latter of which guarantees us context)
                if ( match.length === 1 ) {

                    // Reduce context if the leading compound selector is an ID
                    tokens = match[ 0 ] = match[ 0 ].slice( 0 );
                    if ( tokens.length > 2 && ( token = tokens[ 0 ] ).type === "ID" &&
                        context.nodeType === 9 && documentIsHTML && Expr.relative[ tokens[ 1 ].type ] ) {

                        context = ( Expr.find[ "ID" ]( token.matches[ 0 ]
                            .replace( runescape, funescape ), context ) || [] )[ 0 ];
                        if ( !context ) {
                            return results;

                            // Precompiled matchers will still verify ancestry, so step up a level
                        } else if ( compiled ) {
                            context = context.parentNode;
                        }

                        selector = selector.slice( tokens.shift().value.length );
                    }

                    // Fetch a seed set for right-to-left matching
                    i = matchExpr[ "needsContext" ].test( selector ) ? 0 : tokens.length;
                    while ( i-- ) {
                        token = tokens[ i ];

                        // Abort if we hit a combinator
                        if ( Expr.relative[ ( type = token.type ) ] ) {
                            break;
                        }
                        if ( ( find = Expr.find[ type ] ) ) {

                            // Search, expanding context for leading sibling combinators
                            if ( ( seed = find(
                                token.matches[ 0 ].replace( runescape, funescape ),
                                rsibling.test( tokens[ 0 ].type ) && testContext( context.parentNode ) ||
                                context
                            ) ) ) {

                                // If seed is empty or no tokens remain, we can return early
                                tokens.splice( i, 1 );
                                selector = seed.length && toSelector( tokens );
                                if ( !selector ) {
                                    push.apply( results, seed );
                                    return results;
                                }

                                break;
                            }
                        }
                    }
                }

                // Compile and execute a filtering function if one is not provided
                // Provide `match` to avoid retokenization if we modified the selector above
                ( compiled || compile( selector, match ) )(
                    seed,
                    context,
                    !documentIsHTML,
                    results,
                    !context || rsibling.test( selector ) && testContext( context.parentNode ) || context
                );
                return results;
            };

// One-time assignments

// Sort stability
            support.sortStable = expando.split( "" ).sort( sortOrder ).join( "" ) === expando;

// Support: Chrome 14-35+
// Always assume duplicates if they aren't passed to the comparison function
            support.detectDuplicates = !!hasDuplicate;

// Initialize against the default document
            setDocument();

// Support: Webkit<537.32 - Safari 6.0.3/Chrome 25 (fixed in Chrome 27)
// Detached nodes confoundingly follow *each other*
            support.sortDetached = assert( function( el ) {

                // Should return 1, but returns 4 (following)
                return el.compareDocumentPosition( document.createElement( "fieldset" ) ) & 1;
            } );

// Support: IE<8
// Prevent attribute/property "interpolation"
// https://msdn.microsoft.com/en-us/library/ms536429%28VS.85%29.aspx
            if ( !assert( function( el ) {
                el.innerHTML = "<a href='#'></a>";
                return el.firstChild.getAttribute( "href" ) === "#";
            } ) ) {
                addHandle( "type|href|height|width", function( elem, name, isXML ) {
                    if ( !isXML ) {
                        return elem.getAttribute( name, name.toLowerCase() === "type" ? 1 : 2 );
                    }
                } );
            }

// Support: IE<9
// Use defaultValue in place of getAttribute("value")
            if ( !support.attributes || !assert( function( el ) {
                el.innerHTML = "<input/>";
                el.firstChild.setAttribute( "value", "" );
                return el.firstChild.getAttribute( "value" ) === "";
            } ) ) {
                addHandle( "value", function( elem, _name, isXML ) {
                    if ( !isXML && elem.nodeName.toLowerCase() === "input" ) {
                        return elem.defaultValue;
                    }
                } );
            }

// Support: IE<9
// Use getAttributeNode to fetch booleans when getAttribute lies
            if ( !assert( function( el ) {
                return el.getAttribute( "disabled" ) == null;
            } ) ) {
                addHandle( booleans, function( elem, name, isXML ) {
                    var val;
                    if ( !isXML ) {
                        return elem[ name ] === true ? name.toLowerCase() :
                            ( val = elem.getAttributeNode( name ) ) && val.specified ?
                                val.value :
                                null;
                    }
                } );
            }

            return Sizzle;

        } )( window );



    jQuery.find = Sizzle;
    jQuery.expr = Sizzle.selectors;

// Deprecated
    jQuery.expr[ ":" ] = jQuery.expr.pseudos;
    jQuery.uniqueSort = jQuery.unique = Sizzle.uniqueSort;
    jQuery.text = Sizzle.getText;
    jQuery.isXMLDoc = Sizzle.isXML;
    jQuery.contains = Sizzle.contains;
    jQuery.escapeSelector = Sizzle.escape;




    var dir = function( elem, dir, until ) {
        var matched = [],
            truncate = until !== undefined;

        while ( ( elem = elem[ dir ] ) && elem.nodeType !== 9 ) {
            if ( elem.nodeType === 1 ) {
                if ( truncate && jQuery( elem ).is( until ) ) {
                    break;
                }
                matched.push( elem );
            }
        }
        return matched;
    };


    var siblings = function( n, elem ) {
        var matched = [];

        for ( ; n; n = n.nextSibling ) {
            if ( n.nodeType === 1 && n !== elem ) {
                matched.push( n );
            }
        }

        return matched;
    };


    var rneedsContext = jQuery.expr.match.needsContext;



    function nodeName( elem, name ) {

        return elem.nodeName && elem.nodeName.toLowerCase() === name.toLowerCase();

    };
    var rsingleTag = ( /^<([a-z][^\/\0>:\x20\t\r\n\f]*)[\x20\t\r\n\f]*\/?>(?:<\/\1>|)$/i );



// Implement the identical functionality for filter and not
    function winnow( elements, qualifier, not ) {
        if ( isFunction( qualifier ) ) {
            return jQuery.grep( elements, function( elem, i ) {
                return !!qualifier.call( elem, i, elem ) !== not;
            } );
        }

        // Single element
        if ( qualifier.nodeType ) {
            return jQuery.grep( elements, function( elem ) {
                return ( elem === qualifier ) !== not;
            } );
        }

        // Arraylike of elements (jQuery, arguments, Array)
        if ( typeof qualifier !== "string" ) {
            return jQuery.grep( elements, function( elem ) {
                return ( indexOf.call( qualifier, elem ) > -1 ) !== not;
            } );
        }

        // Filtered directly for both simple and complex selectors
        return jQuery.filter( qualifier, elements, not );
    }

    jQuery.filter = function( expr, elems, not ) {
        var elem = elems[ 0 ];

        if ( not ) {
            expr = ":not(" + expr + ")";
        }

        if ( elems.length === 1 && elem.nodeType === 1 ) {
            return jQuery.find.matchesSelector( elem, expr ) ? [ elem ] : [];
        }

        return jQuery.find.matches( expr, jQuery.grep( elems, function( elem ) {
            return elem.nodeType === 1;
        } ) );
    };

    jQuery.fn.extend( {
        find: function( selector ) {
            var i, ret,
                len = this.length,
                self = this;

            if ( typeof selector !== "string" ) {
                return this.pushStack( jQuery( selector ).filter( function() {
                    for ( i = 0; i < len; i++ ) {
                        if ( jQuery.contains( self[ i ], this ) ) {
                            return true;
                        }
                    }
                } ) );
            }

            ret = this.pushStack( [] );

            for ( i = 0; i < len; i++ ) {
                jQuery.find( selector, self[ i ], ret );
            }

            return len > 1 ? jQuery.uniqueSort( ret ) : ret;
        },
        filter: function( selector ) {
            return this.pushStack( winnow( this, selector || [], false ) );
        },
        not: function( selector ) {
            return this.pushStack( winnow( this, selector || [], true ) );
        },
        is: function( selector ) {
            return !!winnow(
                this,

                // If this is a positional/relative selector, check membership in the returned set
                // so $("p:first").is("p:last") won't return true for a doc with two "p".
                typeof selector === "string" && rneedsContext.test( selector ) ?
                    jQuery( selector ) :
                    selector || [],
                false
            ).length;
        }
    } );


// Initialize a jQuery object


// A central reference to the root jQuery(document)
    var rootjQuery,

        // A simple way to check for HTML strings
        // Prioritize #id over <tag> to avoid XSS via location.hash (#9521)
        // Strict HTML recognition (#11290: must start with <)
        // Shortcut simple #id case for speed
        rquickExpr = /^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]+))$/,

        init = jQuery.fn.init = function( selector, context, root ) {
            var match, elem;

            // HANDLE: $(""), $(null), $(undefined), $(false)
            if ( !selector ) {
                return this;
            }

            // Method init() accepts an alternate rootjQuery
            // so migrate can support jQuery.sub (gh-2101)
            root = root || rootjQuery;

            // Handle HTML strings
            if ( typeof selector === "string" ) {
                if ( selector[ 0 ] === "<" &&
                    selector[ selector.length - 1 ] === ">" &&
                    selector.length >= 3 ) {

                    // Assume that strings that start and end with <> are HTML and skip the regex check
                    match = [ null, selector, null ];

                } else {
                    match = rquickExpr.exec( selector );
                }

                // Match html or make sure no context is specified for #id
                if ( match && ( match[ 1 ] || !context ) ) {

                    // HANDLE: $(html) -> $(array)
                    if ( match[ 1 ] ) {
                        context = context instanceof jQuery ? context[ 0 ] : context;

                        // Option to run scripts is true for back-compat
                        // Intentionally let the error be thrown if parseHTML is not present
                        jQuery.merge( this, jQuery.parseHTML(
                            match[ 1 ],
                            context && context.nodeType ? context.ownerDocument || context : document,
                            true
                        ) );

                        // HANDLE: $(html, props)
                        if ( rsingleTag.test( match[ 1 ] ) && jQuery.isPlainObject( context ) ) {
                            for ( match in context ) {

                                // Properties of context are called as methods if possible
                                if ( isFunction( this[ match ] ) ) {
                                    this[ match ]( context[ match ] );

                                    // ...and otherwise set as attributes
                                } else {
                                    this.attr( match, context[ match ] );
                                }
                            }
                        }

                        return this;

                        // HANDLE: $(#id)
                    } else {
                        elem = document.getElementById( match[ 2 ] );

                        if ( elem ) {

                            // Inject the element directly into the jQuery object
                            this[ 0 ] = elem;
                            this.length = 1;
                        }
                        return this;
                    }

                    // HANDLE: $(expr, $(...))
                } else if ( !context || context.jquery ) {
                    return ( context || root ).find( selector );

                    // HANDLE: $(expr, context)
                    // (which is just equivalent to: $(context).find(expr)
                } else {
                    return this.constructor( context ).find( selector );
                }

                // HANDLE: $(DOMElement)
            } else if ( selector.nodeType ) {
                this[ 0 ] = selector;
                this.length = 1;
                return this;

                // HANDLE: $(function)
                // Shortcut for document ready
            } else if ( isFunction( selector ) ) {
                return root.ready !== undefined ?
                    root.ready( selector ) :

                    // Execute immediately if ready is not present
                    selector( jQuery );
            }

            return jQuery.makeArray( selector, this );
        };

// Give the init function the jQuery prototype for later instantiation
    init.prototype = jQuery.fn;

// Initialize central reference
    rootjQuery = jQuery( document );


    var rparentsprev = /^(?:parents|prev(?:Until|All))/,

        // Methods guaranteed to produce a unique set when starting from a unique set
        guaranteedUnique = {
            children: true,
            contents: true,
            next: true,
            prev: true
        };

    jQuery.fn.extend( {
        has: function( target ) {
            var targets = jQuery( target, this ),
                l = targets.length;

            return this.filter( function() {
                var i = 0;
                for ( ; i < l; i++ ) {
                    if ( jQuery.contains( this, targets[ i ] ) ) {
                        return true;
                    }
                }
            } );
        },

        closest: function( selectors, context ) {
            var cur,
                i = 0,
                l = this.length,
                matched = [],
                targets = typeof selectors !== "string" && jQuery( selectors );

            // Positional selectors never match, since there's no _selection_ context
            if ( !rneedsContext.test( selectors ) ) {
                for ( ; i < l; i++ ) {
                    for ( cur = this[ i ]; cur && cur !== context; cur = cur.parentNode ) {

                        // Always skip document fragments
                        if ( cur.nodeType < 11 && ( targets ?
                            targets.index( cur ) > -1 :

                            // Don't pass non-elements to Sizzle
                            cur.nodeType === 1 &&
                            jQuery.find.matchesSelector( cur, selectors ) ) ) {

                            matched.push( cur );
                            break;
                        }
                    }
                }
            }

            return this.pushStack( matched.length > 1 ? jQuery.uniqueSort( matched ) : matched );
        },

        // Determine the position of an element within the set
        index: function( elem ) {

            // No argument, return index in parent
            if ( !elem ) {
                return ( this[ 0 ] && this[ 0 ].parentNode ) ? this.first().prevAll().length : -1;
            }

            // Index in selector
            if ( typeof elem === "string" ) {
                return indexOf.call( jQuery( elem ), this[ 0 ] );
            }

            // Locate the position of the desired element
            return indexOf.call( this,

                // If it receives a jQuery object, the first element is used
                elem.jquery ? elem[ 0 ] : elem
            );
        },

        add: function( selector, context ) {
            return this.pushStack(
                jQuery.uniqueSort(
                    jQuery.merge( this.get(), jQuery( selector, context ) )
                )
            );
        },

        addBack: function( selector ) {
            return this.add( selector == null ?
                this.prevObject : this.prevObject.filter( selector )
            );
        }
    } );

    function sibling( cur, dir ) {
        while ( ( cur = cur[ dir ] ) && cur.nodeType !== 1 ) {}
        return cur;
    }

    jQuery.each( {
        parent: function( elem ) {
            var parent = elem.parentNode;
            return parent && parent.nodeType !== 11 ? parent : null;
        },
        parents: function( elem ) {
            return dir( elem, "parentNode" );
        },
        parentsUntil: function( elem, _i, until ) {
            return dir( elem, "parentNode", until );
        },
        next: function( elem ) {
            return sibling( elem, "nextSibling" );
        },
        prev: function( elem ) {
            return sibling( elem, "previousSibling" );
        },
        nextAll: function( elem ) {
            return dir( elem, "nextSibling" );
        },
        prevAll: function( elem ) {
            return dir( elem, "previousSibling" );
        },
        nextUntil: function( elem, _i, until ) {
            return dir( elem, "nextSibling", until );
        },
        prevUntil: function( elem, _i, until ) {
            return dir( elem, "previousSibling", until );
        },
        siblings: function( elem ) {
            return siblings( ( elem.parentNode || {} ).firstChild, elem );
        },
        children: function( elem ) {
            return siblings( elem.firstChild );
        },
        contents: function( elem ) {
            if ( elem.contentDocument != null &&

                // Support: IE 11+
                // <object> elements with no `data` attribute has an object
                // `contentDocument` with a `null` prototype.
                getProto( elem.contentDocument ) ) {

                return elem.contentDocument;
            }

            // Support: IE 9 - 11 only, iOS 7 only, Android Browser <=4.3 only
            // Treat the template element as a regular one in browsers that
            // don't support it.
            if ( nodeName( elem, "template" ) ) {
                elem = elem.content || elem;
            }

            return jQuery.merge( [], elem.childNodes );
        }
    }, function( name, fn ) {
        jQuery.fn[ name ] = function( until, selector ) {
            var matched = jQuery.map( this, fn, until );

            if ( name.slice( -5 ) !== "Until" ) {
                selector = until;
            }

            if ( selector && typeof selector === "string" ) {
                matched = jQuery.filter( selector, matched );
            }

            if ( this.length > 1 ) {

                // Remove duplicates
                if ( !guaranteedUnique[ name ] ) {
                    jQuery.uniqueSort( matched );
                }

                // Reverse order for parents* and prev-derivatives
                if ( rparentsprev.test( name ) ) {
                    matched.reverse();
                }
            }

            return this.pushStack( matched );
        };
    } );
    var rnothtmlwhite = ( /[^\x20\t\r\n\f]+/g );



// Convert String-formatted options into Object-formatted ones
    function createOptions( options ) {
        var object = {};
        jQuery.each( options.match( rnothtmlwhite ) || [], function( _, flag ) {
            object[ flag ] = true;
        } );
        return object;
    }

    /*
 * Create a callback list using the following parameters:
 *
 *	options: an optional list of space-separated options that will change how
 *			the callback list behaves or a more traditional option object
 *
 * By default a callback list will act like an event callback list and can be
 * "fired" multiple times.
 *
 * Possible options:
 *
 *	once:			will ensure the callback list can only be fired once (like a Deferred)
 *
 *	memory:			will keep track of previous values and will call any callback added
 *					after the list has been fired right away with the latest "memorized"
 *					values (like a Deferred)
 *
 *	unique:			will ensure a callback can only be added once (no duplicate in the list)
 *
 *	stopOnFalse:	interrupt callings when a callback returns false
 *
 */
    jQuery.Callbacks = function( options ) {

        // Convert options from String-formatted to Object-formatted if needed
        // (we check in cache first)
        options = typeof options === "string" ?
            createOptions( options ) :
            jQuery.extend( {}, options );

        var // Flag to know if list is currently firing
            firing,

            // Last fire value for non-forgettable lists
            memory,

            // Flag to know if list was already fired
            fired,

            // Flag to prevent firing
            locked,

            // Actual callback list
            list = [],

            // Queue of execution data for repeatable lists
            queue = [],

            // Index of currently firing callback (modified by add/remove as needed)
            firingIndex = -1,

            // Fire callbacks
            fire = function() {

                // Enforce single-firing
                locked = locked || options.once;

                // Execute callbacks for all pending executions,
                // respecting firingIndex overrides and runtime changes
                fired = firing = true;
                for ( ; queue.length; firingIndex = -1 ) {
                    memory = queue.shift();
                    while ( ++firingIndex < list.length ) {

                        // Run callback and check for early termination
                        if ( list[ firingIndex ].apply( memory[ 0 ], memory[ 1 ] ) === false &&
                            options.stopOnFalse ) {

                            // Jump to end and forget the data so .add doesn't re-fire
                            firingIndex = list.length;
                            memory = false;
                        }
                    }
                }

                // Forget the data if we're done with it
                if ( !options.memory ) {
                    memory = false;
                }

                firing = false;

                // Clean up if we're done firing for good
                if ( locked ) {

                    // Keep an empty list if we have data for future add calls
                    if ( memory ) {
                        list = [];

                        // Otherwise, this object is spent
                    } else {
                        list = "";
                    }
                }
            },

            // Actual Callbacks object
            self = {

                // Add a callback or a collection of callbacks to the list
                add: function() {
                    if ( list ) {

                        // If we have memory from a past run, we should fire after adding
                        if ( memory && !firing ) {
                            firingIndex = list.length - 1;
                            queue.push( memory );
                        }

                        ( function add( args ) {
                            jQuery.each( args, function( _, arg ) {
                                if ( isFunction( arg ) ) {
                                    if ( !options.unique || !self.has( arg ) ) {
                                        list.push( arg );
                                    }
                                } else if ( arg && arg.length && toType( arg ) !== "string" ) {

                                    // Inspect recursively
                                    add( arg );
                                }
                            } );
                        } )( arguments );

                        if ( memory && !firing ) {
                            fire();
                        }
                    }
                    return this;
                },

                // Remove a callback from the list
                remove: function() {
                    jQuery.each( arguments, function( _, arg ) {
                        var index;
                        while ( ( index = jQuery.inArray( arg, list, index ) ) > -1 ) {
                            list.splice( index, 1 );

                            // Handle firing indexes
                            if ( index <= firingIndex ) {
                                firingIndex--;
                            }
                        }
                    } );
                    return this;
                },

                // Check if a given callback is in the list.
                // If no argument is given, return whether or not list has callbacks attached.
                has: function( fn ) {
                    return fn ?
                        jQuery.inArray( fn, list ) > -1 :
                        list.length > 0;
                },

                // Remove all callbacks from the list
                empty: function() {
                    if ( list ) {
                        list = [];
                    }
                    return this;
                },

                // Disable .fire and .add
                // Abort any current/pending executions
                // Clear all callbacks and values
                disable: function() {
                    locked = queue = [];
                    list = memory = "";
                    return this;
                },
                disabled: function() {
                    return !list;
                },

                // Disable .fire
                // Also disable .add unless we have memory (since it would have no effect)
                // Abort any pending executions
                lock: function() {
                    locked = queue = [];
                    if ( !memory && !firing ) {
                        list = memory = "";
                    }
                    return this;
                },
                locked: function() {
                    return !!locked;
                },

                // Call all callbacks with the given context and arguments
                fireWith: function( context, args ) {
                    if ( !locked ) {
                        args = args || [];
                        args = [ context, args.slice ? args.slice() : args ];
                        queue.push( args );
                        if ( !firing ) {
                            fire();
                        }
                    }
                    return this;
                },

                // Call all the callbacks with the given arguments
                fire: function() {
                    self.fireWith( this, arguments );
                    return this;
                },

                // To know if the callbacks have already been called at least once
                fired: function() {
                    return !!fired;
                }
            };

        return self;
    };


    function Identity( v ) {
        return v;
    }
    function Thrower( ex ) {
        throw ex;
    }

    function adoptValue( value, resolve, reject, noValue ) {
        var method;

        try {

            // Check for promise aspect first to privilege synchronous behavior
            if ( value && isFunction( ( method = value.promise ) ) ) {
                method.call( value ).done( resolve ).fail( reject );

                // Other thenables
            } else if ( value && isFunction( ( method = value.then ) ) ) {
                method.call( value, resolve, reject );

                // Other non-thenables
            } else {

                // Control `resolve` arguments by letting Array#slice cast boolean `noValue` to integer:
                // * false: [ value ].slice( 0 ) => resolve( value )
                // * true: [ value ].slice( 1 ) => resolve()
                resolve.apply( undefined, [ value ].slice( noValue ) );
            }

            // For Promises/A+, convert exceptions into rejections
            // Since jQuery.when doesn't unwrap thenables, we can skip the extra checks appearing in
            // Deferred#then to conditionally suppress rejection.
        } catch ( value ) {

            // Support: Android 4.0 only
            // Strict mode functions invoked without .call/.apply get global-object context
            reject.apply( undefined, [ value ] );
        }
    }

    jQuery.extend( {

        Deferred: function( func ) {
            var tuples = [

                    // action, add listener, callbacks,
                    // ... .then handlers, argument index, [final state]
                    [ "notify", "progress", jQuery.Callbacks( "memory" ),
                        jQuery.Callbacks( "memory" ), 2 ],
                    [ "resolve", "done", jQuery.Callbacks( "once memory" ),
                        jQuery.Callbacks( "once memory" ), 0, "resolved" ],
                    [ "reject", "fail", jQuery.Callbacks( "once memory" ),
                        jQuery.Callbacks( "once memory" ), 1, "rejected" ]
                ],
                state = "pending",
                promise = {
                    state: function() {
                        return state;
                    },
                    always: function() {
                        deferred.done( arguments ).fail( arguments );
                        return this;
                    },
                    "catch": function( fn ) {
                        return promise.then( null, fn );
                    },

                    // Keep pipe for back-compat
                    pipe: function( /* fnDone, fnFail, fnProgress */ ) {
                        var fns = arguments;

                        return jQuery.Deferred( function( newDefer ) {
                            jQuery.each( tuples, function( _i, tuple ) {

                                // Map tuples (progress, done, fail) to arguments (done, fail, progress)
                                var fn = isFunction( fns[ tuple[ 4 ] ] ) && fns[ tuple[ 4 ] ];

                                // deferred.progress(function() { bind to newDefer or newDefer.notify })
                                // deferred.done(function() { bind to newDefer or newDefer.resolve })
                                // deferred.fail(function() { bind to newDefer or newDefer.reject })
                                deferred[ tuple[ 1 ] ]( function() {
                                    var returned = fn && fn.apply( this, arguments );
                                    if ( returned && isFunction( returned.promise ) ) {
                                        returned.promise()
                                            .progress( newDefer.notify )
                                            .done( newDefer.resolve )
                                            .fail( newDefer.reject );
                                    } else {
                                        newDefer[ tuple[ 0 ] + "With" ](
                                            this,
                                            fn ? [ returned ] : arguments
                                        );
                                    }
                                } );
                            } );
                            fns = null;
                        } ).promise();
                    },
                    then: function( onFulfilled, onRejected, onProgress ) {
                        var maxDepth = 0;
                        function resolve( depth, deferred, handler, special ) {
                            return function() {
                                var that = this,
                                    args = arguments,
                                    mightThrow = function() {
                                        var returned, then;

                                        // Support: Promises/A+ section 2.3.3.3.3
                                        // https://promisesaplus.com/#point-59
                                        // Ignore double-resolution attempts
                                        if ( depth < maxDepth ) {
                                            return;
                                        }

                                        returned = handler.apply( that, args );

                                        // Support: Promises/A+ section 2.3.1
                                        // https://promisesaplus.com/#point-48
                                        if ( returned === deferred.promise() ) {
                                            throw new TypeError( "Thenable self-resolution" );
                                        }

                                        // Support: Promises/A+ sections 2.3.3.1, 3.5
                                        // https://promisesaplus.com/#point-54
                                        // https://promisesaplus.com/#point-75
                                        // Retrieve `then` only once
                                        then = returned &&

                                            // Support: Promises/A+ section 2.3.4
                                            // https://promisesaplus.com/#point-64
                                            // Only check objects and functions for thenability
                                            ( typeof returned === "object" ||
                                                typeof returned === "function" ) &&
                                            returned.then;

                                        // Handle a returned thenable
                                        if ( isFunction( then ) ) {

                                            // Special processors (notify) just wait for resolution
                                            if ( special ) {
                                                then.call(
                                                    returned,
                                                    resolve( maxDepth, deferred, Identity, special ),
                                                    resolve( maxDepth, deferred, Thrower, special )
                                                );

                                                // Normal processors (resolve) also hook into progress
                                            } else {

                                                // ...and disregard older resolution values
                                                maxDepth++;

                                                then.call(
                                                    returned,
                                                    resolve( maxDepth, deferred, Identity, special ),
                                                    resolve( maxDepth, deferred, Thrower, special ),
                                                    resolve( maxDepth, deferred, Identity,
                                                        deferred.notifyWith )
                                                );
                                            }

                                            // Handle all other returned values
                                        } else {

                                            // Only substitute handlers pass on context
                                            // and multiple values (non-spec behavior)
                                            if ( handler !== Identity ) {
                                                that = undefined;
                                                args = [ returned ];
                                            }

                                            // Process the value(s)
                                            // Default process is resolve
                                            ( special || deferred.resolveWith )( that, args );
                                        }
                                    },

                                    // Only normal processors (resolve) catch and reject exceptions
                                    process = special ?
                                        mightThrow :
                                        function() {
                                            try {
                                                mightThrow();
                                            } catch ( e ) {

                                                if ( jQuery.Deferred.exceptionHook ) {
                                                    jQuery.Deferred.exceptionHook( e,
                                                        process.stackTrace );
                                                }

                                                // Support: Promises/A+ section 2.3.3.3.4.1
                                                // https://promisesaplus.com/#point-61
                                                // Ignore post-resolution exceptions
                                                if ( depth + 1 >= maxDepth ) {

                                                    // Only substitute handlers pass on context
                                                    // and multiple values (non-spec behavior)
                                                    if ( handler !== Thrower ) {
                                                        that = undefined;
                                                        args = [ e ];
                                                    }

                                                    deferred.rejectWith( that, args );
                                                }
                                            }
                                        };

                                // Support: Promises/A+ section 2.3.3.3.1
                                // https://promisesaplus.com/#point-57
                                // Re-resolve promises immediately to dodge false rejection from
                                // subsequent errors
                                if ( depth ) {
                                    process();
                                } else {

                                    // Call an optional hook to record the stack, in case of exception
                                    // since it's otherwise lost when execution goes async
                                    if ( jQuery.Deferred.getStackHook ) {
                                        process.stackTrace = jQuery.Deferred.getStackHook();
                                    }
                                    window.setTimeout( process );
                                }
                            };
                        }

                        return jQuery.Deferred( function( newDefer ) {

                            // progress_handlers.add( ... )
                            tuples[ 0 ][ 3 ].add(
                                resolve(
                                    0,
                                    newDefer,
                                    isFunction( onProgress ) ?
                                        onProgress :
                                        Identity,
                                    newDefer.notifyWith
                                )
                            );

                            // fulfilled_handlers.add( ... )
                            tuples[ 1 ][ 3 ].add(
                                resolve(
                                    0,
                                    newDefer,
                                    isFunction( onFulfilled ) ?
                                        onFulfilled :
                                        Identity
                                )
                            );

                            // rejected_handlers.add( ... )
                            tuples[ 2 ][ 3 ].add(
                                resolve(
                                    0,
                                    newDefer,
                                    isFunction( onRejected ) ?
                                        onRejected :
                                        Thrower
                                )
                            );
                        } ).promise();
                    },

                    // Get a promise for this deferred
                    // If obj is provided, the promise aspect is added to the object
                    promise: function( obj ) {
                        return obj != null ? jQuery.extend( obj, promise ) : promise;
                    }
                },
                deferred = {};

            // Add list-specific methods
            jQuery.each( tuples, function( i, tuple ) {
                var list = tuple[ 2 ],
                    stateString = tuple[ 5 ];

                // promise.progress = list.add
                // promise.done = list.add
                // promise.fail = list.add
                promise[ tuple[ 1 ] ] = list.add;

                // Handle state
                if ( stateString ) {
                    list.add(
                        function() {

                            // state = "resolved" (i.e., fulfilled)
                            // state = "rejected"
                            state = stateString;
                        },

                        // rejected_callbacks.disable
                        // fulfilled_callbacks.disable
                        tuples[ 3 - i ][ 2 ].disable,

                        // rejected_handlers.disable
                        // fulfilled_handlers.disable
                        tuples[ 3 - i ][ 3 ].disable,

                        // progress_callbacks.lock
                        tuples[ 0 ][ 2 ].lock,

                        // progress_handlers.lock
                        tuples[ 0 ][ 3 ].lock
                    );
                }

                // progress_handlers.fire
                // fulfilled_handlers.fire
                // rejected_handlers.fire
                list.add( tuple[ 3 ].fire );

                // deferred.notify = function() { deferred.notifyWith(...) }
                // deferred.resolve = function() { deferred.resolveWith(...) }
                // deferred.reject = function() { deferred.rejectWith(...) }
                deferred[ tuple[ 0 ] ] = function() {
                    deferred[ tuple[ 0 ] + "With" ]( this === deferred ? undefined : this, arguments );
                    return this;
                };

                // deferred.notifyWith = list.fireWith
                // deferred.resolveWith = list.fireWith
                // deferred.rejectWith = list.fireWith
                deferred[ tuple[ 0 ] + "With" ] = list.fireWith;
            } );

            // Make the deferred a promise
            promise.promise( deferred );

            // Call given func if any
            if ( func ) {
                func.call( deferred, deferred );
            }

            // All done!
            return deferred;
        },

        // Deferred helper
        when: function( singleValue ) {
            var

                // count of uncompleted subordinates
                remaining = arguments.length,

                // count of unprocessed arguments
                i = remaining,

                // subordinate fulfillment data
                resolveContexts = Array( i ),
                resolveValues = slice.call( arguments ),

                // the master Deferred
                master = jQuery.Deferred(),

                // subordinate callback factory
                updateFunc = function( i ) {
                    return function( value ) {
                        resolveContexts[ i ] = this;
                        resolveValues[ i ] = arguments.length > 1 ? slice.call( arguments ) : value;
                        if ( !( --remaining ) ) {
                            master.resolveWith( resolveContexts, resolveValues );
                        }
                    };
                };

            // Single- and empty arguments are adopted like Promise.resolve
            if ( remaining <= 1 ) {
                adoptValue( singleValue, master.done( updateFunc( i ) ).resolve, master.reject,
                    !remaining );

                // Use .then() to unwrap secondary thenables (cf. gh-3000)
                if ( master.state() === "pending" ||
                    isFunction( resolveValues[ i ] && resolveValues[ i ].then ) ) {

                    return master.then();
                }
            }

            // Multiple arguments are aggregated like Promise.all array elements
            while ( i-- ) {
                adoptValue( resolveValues[ i ], updateFunc( i ), master.reject );
            }

            return master.promise();
        }
    } );


// These usually indicate a programmer mistake during development,
// warn about them ASAP rather than swallowing them by default.
    var rerrorNames = /^(Eval|Internal|Range|Reference|Syntax|Type|URI)Error$/;

    jQuery.Deferred.exceptionHook = function( error, stack ) {

        // Support: IE 8 - 9 only
        // Console exists when dev tools are open, which can happen at any time
        if ( window.console && window.console.warn && error && rerrorNames.test( error.name ) ) {
            window.console.warn( "jQuery.Deferred exception: " + error.message, error.stack, stack );
        }
    };




    jQuery.readyException = function( error ) {
        window.setTimeout( function() {
            throw error;
        } );
    };




// The deferred used on DOM ready
    var readyList = jQuery.Deferred();

    jQuery.fn.ready = function( fn ) {

        readyList
            .then( fn )

            // Wrap jQuery.readyException in a function so that the lookup
            // happens at the time of error handling instead of callback
            // registration.
            .catch( function( error ) {
                jQuery.readyException( error );
            } );

        return this;
    };

    jQuery.extend( {

        // Is the DOM ready to be used? Set to true once it occurs.
        isReady: false,

        // A counter to track how many items to wait for before
        // the ready event fires. See #6781
        readyWait: 1,

        // Handle when the DOM is ready
        ready: function( wait ) {

            // Abort if there are pending holds or we're already ready
            if ( wait === true ? --jQuery.readyWait : jQuery.isReady ) {
                return;
            }

            // Remember that the DOM is ready
            jQuery.isReady = true;

            // If a normal DOM Ready event fired, decrement, and wait if need be
            if ( wait !== true && --jQuery.readyWait > 0 ) {
                return;
            }

            // If there are functions bound, to execute
            readyList.resolveWith( document, [ jQuery ] );
        }
    } );

    jQuery.ready.then = readyList.then;

// The ready event handler and self cleanup method
    function completed() {
        document.removeEventListener( "DOMContentLoaded", completed );
        window.removeEventListener( "load", completed );
        jQuery.ready();
    }

// Catch cases where $(document).ready() is called
// after the browser event has already occurred.
// Support: IE <=9 - 10 only
// Older IE sometimes signals "interactive" too soon
    if ( document.readyState === "complete" ||
        ( document.readyState !== "loading" && !document.documentElement.doScroll ) ) {

        // Handle it asynchronously to allow scripts the opportunity to delay ready
        window.setTimeout( jQuery.ready );

    } else {

        // Use the handy event callback
        document.addEventListener( "DOMContentLoaded", completed );

        // A fallback to window.onload, that will always work
        window.addEventListener( "load", completed );
    }




// Multifunctional method to get and set values of a collection
// The value/s can optionally be executed if it's a function
    var access = function( elems, fn, key, value, chainable, emptyGet, raw ) {
        var i = 0,
            len = elems.length,
            bulk = key == null;

        // Sets many values
        if ( toType( key ) === "object" ) {
            chainable = true;
            for ( i in key ) {
                access( elems, fn, i, key[ i ], true, emptyGet, raw );
            }

            // Sets one value
        } else if ( value !== undefined ) {
            chainable = true;

            if ( !isFunction( value ) ) {
                raw = true;
            }

            if ( bulk ) {

                // Bulk operations run against the entire set
                if ( raw ) {
                    fn.call( elems, value );
                    fn = null;

                    // ...except when executing function values
                } else {
                    bulk = fn;
                    fn = function( elem, _key, value ) {
                        return bulk.call( jQuery( elem ), value );
                    };
                }
            }

            if ( fn ) {
                for ( ; i < len; i++ ) {
                    fn(
                        elems[ i ], key, raw ?
                            value :
                            value.call( elems[ i ], i, fn( elems[ i ], key ) )
                    );
                }
            }
        }

        if ( chainable ) {
            return elems;
        }

        // Gets
        if ( bulk ) {
            return fn.call( elems );
        }

        return len ? fn( elems[ 0 ], key ) : emptyGet;
    };


// Matches dashed string for camelizing
    var rmsPrefix = /^-ms-/,
        rdashAlpha = /-([a-z])/g;

// Used by camelCase as callback to replace()
    function fcamelCase( _all, letter ) {
        return letter.toUpperCase();
    }

// Convert dashed to camelCase; used by the css and data modules
// Support: IE <=9 - 11, Edge 12 - 15
// Microsoft forgot to hump their vendor prefix (#9572)
    function camelCase( string ) {
        return string.replace( rmsPrefix, "ms-" ).replace( rdashAlpha, fcamelCase );
    }
    var acceptData = function( owner ) {

        // Accepts only:
        //  - Node
        //    - Node.ELEMENT_NODE
        //    - Node.DOCUMENT_NODE
        //  - Object
        //    - Any
        return owner.nodeType === 1 || owner.nodeType === 9 || !( +owner.nodeType );
    };




    function Data() {
        this.expando = jQuery.expando + Data.uid++;
    }

    Data.uid = 1;

    Data.prototype = {

        cache: function( owner ) {

            // Check if the owner object already has a cache
            var value = owner[ this.expando ];

            // If not, create one
            if ( !value ) {
                value = {};

                // We can accept data for non-element nodes in modern browsers,
                // but we should not, see #8335.
                // Always return an empty object.
                if ( acceptData( owner ) ) {

                    // If it is a node unlikely to be stringify-ed or looped over
                    // use plain assignment
                    if ( owner.nodeType ) {
                        owner[ this.expando ] = value;

                        // Otherwise secure it in a non-enumerable property
                        // configurable must be true to allow the property to be
                        // deleted when data is removed
                    } else {
                        Object.defineProperty( owner, this.expando, {
                            value: value,
                            configurable: true
                        } );
                    }
                }
            }

            return value;
        },
        set: function( owner, data, value ) {
            var prop,
                cache = this.cache( owner );

            // Handle: [ owner, key, value ] args
            // Always use camelCase key (gh-2257)
            if ( typeof data === "string" ) {
                cache[ camelCase( data ) ] = value;

                // Handle: [ owner, { properties } ] args
            } else {

                // Copy the properties one-by-one to the cache object
                for ( prop in data ) {
                    cache[ camelCase( prop ) ] = data[ prop ];
                }
            }
            return cache;
        },
        get: function( owner, key ) {
            return key === undefined ?
                this.cache( owner ) :

                // Always use camelCase key (gh-2257)
                owner[ this.expando ] && owner[ this.expando ][ camelCase( key ) ];
        },
        access: function( owner, key, value ) {

            // In cases where either:
            //
            //   1. No key was specified
            //   2. A string key was specified, but no value provided
            //
            // Take the "read" path and allow the get method to determine
            // which value to return, respectively either:
            //
            //   1. The entire cache object
            //   2. The data stored at the key
            //
            if ( key === undefined ||
                ( ( key && typeof key === "string" ) && value === undefined ) ) {

                return this.get( owner, key );
            }

            // When the key is not a string, or both a key and value
            // are specified, set or extend (existing objects) with either:
            //
            //   1. An object of properties
            //   2. A key and value
            //
            this.set( owner, key, value );

            // Since the "set" path can have two possible entry points
            // return the expected data based on which path was taken[*]
            return value !== undefined ? value : key;
        },
        remove: function( owner, key ) {
            var i,
                cache = owner[ this.expando ];

            if ( cache === undefined ) {
                return;
            }

            if ( key !== undefined ) {

                // Support array or space separated string of keys
                if ( Array.isArray( key ) ) {

                    // If key is an array of keys...
                    // We always set camelCase keys, so remove that.
                    key = key.map( camelCase );
                } else {
                    key = camelCase( key );

                    // If a key with the spaces exists, use it.
                    // Otherwise, create an array by matching non-whitespace
                    key = key in cache ?
                        [ key ] :
                        ( key.match( rnothtmlwhite ) || [] );
                }

                i = key.length;

                while ( i-- ) {
                    delete cache[ key[ i ] ];
                }
            }

            // Remove the expando if there's no more data
            if ( key === undefined || jQuery.isEmptyObject( cache ) ) {

                // Support: Chrome <=35 - 45
                // Webkit & Blink performance suffers when deleting properties
                // from DOM nodes, so set to undefined instead
                // https://bugs.chromium.org/p/chromium/issues/detail?id=378607 (bug restricted)
                if ( owner.nodeType ) {
                    owner[ this.expando ] = undefined;
                } else {
                    delete owner[ this.expando ];
                }
            }
        },
        hasData: function( owner ) {
            var cache = owner[ this.expando ];
            return cache !== undefined && !jQuery.isEmptyObject( cache );
        }
    };
    var dataPriv = new Data();

    var dataUser = new Data();



//	Implementation Summary
//
//	1. Enforce API surface and semantic compatibility with 1.9.x branch
//	2. Improve the module's maintainability by reducing the storage
//		paths to a single mechanism.
//	3. Use the same single mechanism to support "private" and "user" data.
//	4. _Never_ expose "private" data to user code (TODO: Drop _data, _removeData)
//	5. Avoid exposing implementation details on user objects (eg. expando properties)
//	6. Provide a clear path for implementation upgrade to WeakMap in 2014

    var rbrace = /^(?:\{[\w\W]*\}|\[[\w\W]*\])$/,
        rmultiDash = /[A-Z]/g;

    function getData( data ) {
        if ( data === "true" ) {
            return true;
        }

        if ( data === "false" ) {
            return false;
        }

        if ( data === "null" ) {
            return null;
        }

        // Only convert to a number if it doesn't change the string
        if ( data === +data + "" ) {
            return +data;
        }

        if ( rbrace.test( data ) ) {
            return JSON.parse( data );
        }

        return data;
    }

    function dataAttr( elem, key, data ) {
        var name;

        // If nothing was found internally, try to fetch any
        // data from the HTML5 data-* attribute
        if ( data === undefined && elem.nodeType === 1 ) {
            name = "data-" + key.replace( rmultiDash, "-$&" ).toLowerCase();
            data = elem.getAttribute( name );

            if ( typeof data === "string" ) {
                try {
                    data = getData( data );
                } catch ( e ) {}

                // Make sure we set the data so it isn't changed later
                dataUser.set( elem, key, data );
            } else {
                data = undefined;
            }
        }
        return data;
    }

    jQuery.extend( {
        hasData: function( elem ) {
            return dataUser.hasData( elem ) || dataPriv.hasData( elem );
        },

        data: function( elem, name, data ) {
            return dataUser.access( elem, name, data );
        },

        removeData: function( elem, name ) {
            dataUser.remove( elem, name );
        },

        // TODO: Now that all calls to _data and _removeData have been replaced
        // with direct calls to dataPriv methods, these can be deprecated.
        _data: function( elem, name, data ) {
            return dataPriv.access( elem, name, data );
        },

        _removeData: function( elem, name ) {
            dataPriv.remove( elem, name );
        }
    } );

    jQuery.fn.extend( {
        data: function( key, value ) {
            var i, name, data,
                elem = this[ 0 ],
                attrs = elem && elem.attributes;

            // Gets all values
            if ( key === undefined ) {
                if ( this.length ) {
                    data = dataUser.get( elem );

                    if ( elem.nodeType === 1 && !dataPriv.get( elem, "hasDataAttrs" ) ) {
                        i = attrs.length;
                        while ( i-- ) {

                            // Support: IE 11 only
                            // The attrs elements can be null (#14894)
                            if ( attrs[ i ] ) {
                                name = attrs[ i ].name;
                                if ( name.indexOf( "data-" ) === 0 ) {
                                    name = camelCase( name.slice( 5 ) );
                                    dataAttr( elem, name, data[ name ] );
                                }
                            }
                        }
                        dataPriv.set( elem, "hasDataAttrs", true );
                    }
                }

                return data;
            }

            // Sets multiple values
            if ( typeof key === "object" ) {
                return this.each( function() {
                    dataUser.set( this, key );
                } );
            }

            return access( this, function( value ) {
                var data;

                // The calling jQuery object (element matches) is not empty
                // (and therefore has an element appears at this[ 0 ]) and the
                // `value` parameter was not undefined. An empty jQuery object
                // will result in `undefined` for elem = this[ 0 ] which will
                // throw an exception if an attempt to read a data cache is made.
                if ( elem && value === undefined ) {

                    // Attempt to get data from the cache
                    // The key will always be camelCased in Data
                    data = dataUser.get( elem, key );
                    if ( data !== undefined ) {
                        return data;
                    }

                    // Attempt to "discover" the data in
                    // HTML5 custom data-* attrs
                    data = dataAttr( elem, key );
                    if ( data !== undefined ) {
                        return data;
                    }

                    // We tried really hard, but the data doesn't exist.
                    return;
                }

                // Set the data...
                this.each( function() {

                    // We always store the camelCased key
                    dataUser.set( this, key, value );
                } );
            }, null, value, arguments.length > 1, null, true );
        },

        removeData: function( key ) {
            return this.each( function() {
                dataUser.remove( this, key );
            } );
        }
    } );


    jQuery.extend( {
        queue: function( elem, type, data ) {
            var queue;

            if ( elem ) {
                type = ( type || "fx" ) + "queue";
                queue = dataPriv.get( elem, type );

                // Speed up dequeue by getting out quickly if this is just a lookup
                if ( data ) {
                    if ( !queue || Array.isArray( data ) ) {
                        queue = dataPriv.access( elem, type, jQuery.makeArray( data ) );
                    } else {
                        queue.push( data );
                    }
                }
                return queue || [];
            }
        },

        dequeue: function( elem, type ) {
            type = type || "fx";

            var queue = jQuery.queue( elem, type ),
                startLength = queue.length,
                fn = queue.shift(),
                hooks = jQuery._queueHooks( elem, type ),
                next = function() {
                    jQuery.dequeue( elem, type );
                };

            // If the fx queue is dequeued, always remove the progress sentinel
            if ( fn === "inprogress" ) {
                fn = queue.shift();
                startLength--;
            }

            if ( fn ) {

                // Add a progress sentinel to prevent the fx queue from being
                // automatically dequeued
                if ( type === "fx" ) {
                    queue.unshift( "inprogress" );
                }

                // Clear up the last queue stop function
                delete hooks.stop;
                fn.call( elem, next, hooks );
            }

            if ( !startLength && hooks ) {
                hooks.empty.fire();
            }
        },

        // Not public - generate a queueHooks object, or return the current one
        _queueHooks: function( elem, type ) {
            var key = type + "queueHooks";
            return dataPriv.get( elem, key ) || dataPriv.access( elem, key, {
                empty: jQuery.Callbacks( "once memory" ).add( function() {
                    dataPriv.remove( elem, [ type + "queue", key ] );
                } )
            } );
        }
    } );

    jQuery.fn.extend( {
        queue: function( type, data ) {
            var setter = 2;

            if ( typeof type !== "string" ) {
                data = type;
                type = "fx";
                setter--;
            }

            if ( arguments.length < setter ) {
                return jQuery.queue( this[ 0 ], type );
            }

            return data === undefined ?
                this :
                this.each( function() {
                    var queue = jQuery.queue( this, type, data );

                    // Ensure a hooks for this queue
                    jQuery._queueHooks( this, type );

                    if ( type === "fx" && queue[ 0 ] !== "inprogress" ) {
                        jQuery.dequeue( this, type );
                    }
                } );
        },
        dequeue: function( type ) {
            return this.each( function() {
                jQuery.dequeue( this, type );
            } );
        },
        clearQueue: function( type ) {
            return this.queue( type || "fx", [] );
        },

        // Get a promise resolved when queues of a certain type
        // are emptied (fx is the type by default)
        promise: function( type, obj ) {
            var tmp,
                count = 1,
                defer = jQuery.Deferred(),
                elements = this,
                i = this.length,
                resolve = function() {
                    if ( !( --count ) ) {
                        defer.resolveWith( elements, [ elements ] );
                    }
                };

            if ( typeof type !== "string" ) {
                obj = type;
                type = undefined;
            }
            type = type || "fx";

            while ( i-- ) {
                tmp = dataPriv.get( elements[ i ], type + "queueHooks" );
                if ( tmp && tmp.empty ) {
                    count++;
                    tmp.empty.add( resolve );
                }
            }
            resolve();
            return defer.promise( obj );
        }
    } );
    var pnum = ( /[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/ ).source;

    var rcssNum = new RegExp( "^(?:([+-])=|)(" + pnum + ")([a-z%]*)$", "i" );


    var cssExpand = [ "Top", "Right", "Bottom", "Left" ];

    var documentElement = document.documentElement;



    var isAttached = function( elem ) {
            return jQuery.contains( elem.ownerDocument, elem );
        },
        composed = { composed: true };

    // Support: IE 9 - 11+, Edge 12 - 18+, iOS 10.0 - 10.2 only
    // Check attachment across shadow DOM boundaries when possible (gh-3504)
    // Support: iOS 10.0-10.2 only
    // Early iOS 10 versions support `attachShadow` but not `getRootNode`,
    // leading to errors. We need to check for `getRootNode`.
    if ( documentElement.getRootNode ) {
        isAttached = function( elem ) {
            return jQuery.contains( elem.ownerDocument, elem ) ||
                elem.getRootNode( composed ) === elem.ownerDocument;
        };
    }
    var isHiddenWithinTree = function( elem, el ) {

        // isHiddenWithinTree might be called from jQuery#filter function;
        // in that case, element will be second argument
        elem = el || elem;

        // Inline style trumps all
        return elem.style.display === "none" ||
            elem.style.display === "" &&

            // Otherwise, check computed style
            // Support: Firefox <=43 - 45
            // Disconnected elements can have computed display: none, so first confirm that elem is
            // in the document.
            isAttached( elem ) &&

            jQuery.css( elem, "display" ) === "none";
    };



    function adjustCSS( elem, prop, valueParts, tween ) {
        var adjusted, scale,
            maxIterations = 20,
            currentValue = tween ?
                function() {
                    return tween.cur();
                } :
                function() {
                    return jQuery.css( elem, prop, "" );
                },
            initial = currentValue(),
            unit = valueParts && valueParts[ 3 ] || ( jQuery.cssNumber[ prop ] ? "" : "px" ),

            // Starting value computation is required for potential unit mismatches
            initialInUnit = elem.nodeType &&
                ( jQuery.cssNumber[ prop ] || unit !== "px" && +initial ) &&
                rcssNum.exec( jQuery.css( elem, prop ) );

        if ( initialInUnit && initialInUnit[ 3 ] !== unit ) {

            // Support: Firefox <=54
            // Halve the iteration target value to prevent interference from CSS upper bounds (gh-2144)
            initial = initial / 2;

            // Trust units reported by jQuery.css
            unit = unit || initialInUnit[ 3 ];

            // Iteratively approximate from a nonzero starting point
            initialInUnit = +initial || 1;

            while ( maxIterations-- ) {

                // Evaluate and update our best guess (doubling guesses that zero out).
                // Finish if the scale equals or crosses 1 (making the old*new product non-positive).
                jQuery.style( elem, prop, initialInUnit + unit );
                if ( ( 1 - scale ) * ( 1 - ( scale = currentValue() / initial || 0.5 ) ) <= 0 ) {
                    maxIterations = 0;
                }
                initialInUnit = initialInUnit / scale;

            }

            initialInUnit = initialInUnit * 2;
            jQuery.style( elem, prop, initialInUnit + unit );

            // Make sure we update the tween properties later on
            valueParts = valueParts || [];
        }

        if ( valueParts ) {
            initialInUnit = +initialInUnit || +initial || 0;

            // Apply relative offset (+=/-=) if specified
            adjusted = valueParts[ 1 ] ?
                initialInUnit + ( valueParts[ 1 ] + 1 ) * valueParts[ 2 ] :
                +valueParts[ 2 ];
            if ( tween ) {
                tween.unit = unit;
                tween.start = initialInUnit;
                tween.end = adjusted;
            }
        }
        return adjusted;
    }


    var defaultDisplayMap = {};

    function getDefaultDisplay( elem ) {
        var temp,
            doc = elem.ownerDocument,
            nodeName = elem.nodeName,
            display = defaultDisplayMap[ nodeName ];

        if ( display ) {
            return display;
        }

        temp = doc.body.appendChild( doc.createElement( nodeName ) );
        display = jQuery.css( temp, "display" );

        temp.parentNode.removeChild( temp );

        if ( display === "none" ) {
            display = "block";
        }
        defaultDisplayMap[ nodeName ] = display;

        return display;
    }

    function showHide( elements, show ) {
        var display, elem,
            values = [],
            index = 0,
            length = elements.length;

        // Determine new display value for elements that need to change
        for ( ; index < length; index++ ) {
            elem = elements[ index ];
            if ( !elem.style ) {
                continue;
            }

            display = elem.style.display;
            if ( show ) {

                // Since we force visibility upon cascade-hidden elements, an immediate (and slow)
                // check is required in this first loop unless we have a nonempty display value (either
                // inline or about-to-be-restored)
                if ( display === "none" ) {
                    values[ index ] = dataPriv.get( elem, "display" ) || null;
                    if ( !values[ index ] ) {
                        elem.style.display = "";
                    }
                }
                if ( elem.style.display === "" && isHiddenWithinTree( elem ) ) {
                    values[ index ] = getDefaultDisplay( elem );
                }
            } else {
                if ( display !== "none" ) {
                    values[ index ] = "none";

                    // Remember what we're overwriting
                    dataPriv.set( elem, "display", display );
                }
            }
        }

        // Set the display of the elements in a second loop to avoid constant reflow
        for ( index = 0; index < length; index++ ) {
            if ( values[ index ] != null ) {
                elements[ index ].style.display = values[ index ];
            }
        }

        return elements;
    }

    jQuery.fn.extend( {
        show: function() {
            return showHide( this, true );
        },
        hide: function() {
            return showHide( this );
        },
        toggle: function( state ) {
            if ( typeof state === "boolean" ) {
                return state ? this.show() : this.hide();
            }

            return this.each( function() {
                if ( isHiddenWithinTree( this ) ) {
                    jQuery( this ).show();
                } else {
                    jQuery( this ).hide();
                }
            } );
        }
    } );
    var rcheckableType = ( /^(?:checkbox|radio)$/i );

    var rtagName = ( /<([a-z][^\/\0>\x20\t\r\n\f]*)/i );

    var rscriptType = ( /^$|^module$|\/(?:java|ecma)script/i );



    ( function() {
        var fragment = document.createDocumentFragment(),
            div = fragment.appendChild( document.createElement( "div" ) ),
            input = document.createElement( "input" );

        // Support: Android 4.0 - 4.3 only
        // Check state lost if the name is set (#11217)
        // Support: Windows Web Apps (WWA)
        // `name` and `type` must use .setAttribute for WWA (#14901)
        input.setAttribute( "type", "radio" );
        input.setAttribute( "checked", "checked" );
        input.setAttribute( "name", "t" );

        div.appendChild( input );

        // Support: Android <=4.1 only
        // Older WebKit doesn't clone checked state correctly in fragments
        support.checkClone = div.cloneNode( true ).cloneNode( true ).lastChild.checked;

        // Support: IE <=11 only
        // Make sure textarea (and checkbox) defaultValue is properly cloned
        div.innerHTML = "<textarea>x</textarea>";
        support.noCloneChecked = !!div.cloneNode( true ).lastChild.defaultValue;

        // Support: IE <=9 only
        // IE <=9 replaces <option> tags with their contents when inserted outside of
        // the select element.
        div.innerHTML = "<option></option>";
        support.option = !!div.lastChild;
    } )();


// We have to close these tags to support XHTML (#13200)
    var wrapMap = {

        // XHTML parsers do not magically insert elements in the
        // same way that tag soup parsers do. So we cannot shorten
        // this by omitting <tbody> or other required elements.
        thead: [ 1, "<table>", "</table>" ],
        col: [ 2, "<table><colgroup>", "</colgroup></table>" ],
        tr: [ 2, "<table><tbody>", "</tbody></table>" ],
        td: [ 3, "<table><tbody><tr>", "</tr></tbody></table>" ],

        _default: [ 0, "", "" ]
    };

    wrapMap.tbody = wrapMap.tfoot = wrapMap.colgroup = wrapMap.caption = wrapMap.thead;
    wrapMap.th = wrapMap.td;

// Support: IE <=9 only
    if ( !support.option ) {
        wrapMap.optgroup = wrapMap.option = [ 1, "<select multiple='multiple'>", "</select>" ];
    }


    function getAll( context, tag ) {

        // Support: IE <=9 - 11 only
        // Use typeof to avoid zero-argument method invocation on host objects (#15151)
        var ret;

        if ( typeof context.getElementsByTagName !== "undefined" ) {
            ret = context.getElementsByTagName( tag || "*" );

        } else if ( typeof context.querySelectorAll !== "undefined" ) {
            ret = context.querySelectorAll( tag || "*" );

        } else {
            ret = [];
        }

        if ( tag === undefined || tag && nodeName( context, tag ) ) {
            return jQuery.merge( [ context ], ret );
        }

        return ret;
    }


// Mark scripts as having already been evaluated
    function setGlobalEval( elems, refElements ) {
        var i = 0,
            l = elems.length;

        for ( ; i < l; i++ ) {
            dataPriv.set(
                elems[ i ],
                "globalEval",
                !refElements || dataPriv.get( refElements[ i ], "globalEval" )
            );
        }
    }


    var rhtml = /<|&#?\w+;/;

    function buildFragment( elems, context, scripts, selection, ignored ) {
        var elem, tmp, tag, wrap, attached, j,
            fragment = context.createDocumentFragment(),
            nodes = [],
            i = 0,
            l = elems.length;

        for ( ; i < l; i++ ) {
            elem = elems[ i ];

            if ( elem || elem === 0 ) {

                // Add nodes directly
                if ( toType( elem ) === "object" ) {

                    // Support: Android <=4.0 only, PhantomJS 1 only
                    // push.apply(_, arraylike) throws on ancient WebKit
                    jQuery.merge( nodes, elem.nodeType ? [ elem ] : elem );

                    // Convert non-html into a text node
                } else if ( !rhtml.test( elem ) ) {
                    nodes.push( context.createTextNode( elem ) );

                    // Convert html into DOM nodes
                } else {
                    tmp = tmp || fragment.appendChild( context.createElement( "div" ) );

                    // Deserialize a standard representation
                    tag = ( rtagName.exec( elem ) || [ "", "" ] )[ 1 ].toLowerCase();
                    wrap = wrapMap[ tag ] || wrapMap._default;
                    tmp.innerHTML = wrap[ 1 ] + jQuery.htmlPrefilter( elem ) + wrap[ 2 ];

                    // Descend through wrappers to the right content
                    j = wrap[ 0 ];
                    while ( j-- ) {
                        tmp = tmp.lastChild;
                    }

                    // Support: Android <=4.0 only, PhantomJS 1 only
                    // push.apply(_, arraylike) throws on ancient WebKit
                    jQuery.merge( nodes, tmp.childNodes );

                    // Remember the top-level container
                    tmp = fragment.firstChild;

                    // Ensure the created nodes are orphaned (#12392)
                    tmp.textContent = "";
                }
            }
        }

        // Remove wrapper from fragment
        fragment.textContent = "";

        i = 0;
        while ( ( elem = nodes[ i++ ] ) ) {

            // Skip elements already in the context collection (trac-4087)
            if ( selection && jQuery.inArray( elem, selection ) > -1 ) {
                if ( ignored ) {
                    ignored.push( elem );
                }
                continue;
            }

            attached = isAttached( elem );

            // Append to fragment
            tmp = getAll( fragment.appendChild( elem ), "script" );

            // Preserve script evaluation history
            if ( attached ) {
                setGlobalEval( tmp );
            }

            // Capture executables
            if ( scripts ) {
                j = 0;
                while ( ( elem = tmp[ j++ ] ) ) {
                    if ( rscriptType.test( elem.type || "" ) ) {
                        scripts.push( elem );
                    }
                }
            }
        }

        return fragment;
    }


    var
        rkeyEvent = /^key/,
        rmouseEvent = /^(?:mouse|pointer|contextmenu|drag|drop)|click/,
        rtypenamespace = /^([^.]*)(?:\.(.+)|)/;

    function returnTrue() {
        return true;
    }

    function returnFalse() {
        return false;
    }

// Support: IE <=9 - 11+
// focus() and blur() are asynchronous, except when they are no-op.
// So expect focus to be synchronous when the element is already active,
// and blur to be synchronous when the element is not already active.
// (focus and blur are always synchronous in other supported browsers,
// this just defines when we can count on it).
    function expectSync( elem, type ) {
        return ( elem === safeActiveElement() ) === ( type === "focus" );
    }

// Support: IE <=9 only
// Accessing document.activeElement can throw unexpectedly
// https://bugs.jquery.com/ticket/13393
    function safeActiveElement() {
        try {
            return document.activeElement;
        } catch ( err ) { }
    }

    function on( elem, types, selector, data, fn, one ) {
        var origFn, type;

        // Types can be a map of types/handlers
        if ( typeof types === "object" ) {

            // ( types-Object, selector, data )
            if ( typeof selector !== "string" ) {

                // ( types-Object, data )
                data = data || selector;
                selector = undefined;
            }
            for ( type in types ) {
                on( elem, type, selector, data, types[ type ], one );
            }
            return elem;
        }

        if ( data == null && fn == null ) {

            // ( types, fn )
            fn = selector;
            data = selector = undefined;
        } else if ( fn == null ) {
            if ( typeof selector === "string" ) {

                // ( types, selector, fn )
                fn = data;
                data = undefined;
            } else {

                // ( types, data, fn )
                fn = data;
                data = selector;
                selector = undefined;
            }
        }
        if ( fn === false ) {
            fn = returnFalse;
        } else if ( !fn ) {
            return elem;
        }

        if ( one === 1 ) {
            origFn = fn;
            fn = function( event ) {

                // Can use an empty set, since event contains the info
                jQuery().off( event );
                return origFn.apply( this, arguments );
            };

            // Use same guid so caller can remove using origFn
            fn.guid = origFn.guid || ( origFn.guid = jQuery.guid++ );
        }
        return elem.each( function() {
            jQuery.event.add( this, types, fn, data, selector );
        } );
    }

    /*
 * Helper functions for managing events -- not part of the public interface.
 * Props to Dean Edwards' addEvent library for many of the ideas.
 */
    jQuery.event = {

        global: {},

        add: function( elem, types, handler, data, selector ) {

            var handleObjIn, eventHandle, tmp,
                events, t, handleObj,
                special, handlers, type, namespaces, origType,
                elemData = dataPriv.get( elem );

            // Only attach events to objects that accept data
            if ( !acceptData( elem ) ) {
                return;
            }

            // Caller can pass in an object of custom data in lieu of the handler
            if ( handler.handler ) {
                handleObjIn = handler;
                handler = handleObjIn.handler;
                selector = handleObjIn.selector;
            }

            // Ensure that invalid selectors throw exceptions at attach time
            // Evaluate against documentElement in case elem is a non-element node (e.g., document)
            if ( selector ) {
                jQuery.find.matchesSelector( documentElement, selector );
            }

            // Make sure that the handler has a unique ID, used to find/remove it later
            if ( !handler.guid ) {
                handler.guid = jQuery.guid++;
            }

            // Init the element's event structure and main handler, if this is the first
            if ( !( events = elemData.events ) ) {
                events = elemData.events = Object.create( null );
            }
            if ( !( eventHandle = elemData.handle ) ) {
                eventHandle = elemData.handle = function( e ) {

                    // Discard the second event of a jQuery.event.trigger() and
                    // when an event is called after a page has unloaded
                    return typeof jQuery !== "undefined" && jQuery.event.triggered !== e.type ?
                        jQuery.event.dispatch.apply( elem, arguments ) : undefined;
                };
            }

            // Handle multiple events separated by a space
            types = ( types || "" ).match( rnothtmlwhite ) || [ "" ];
            t = types.length;
            while ( t-- ) {
                tmp = rtypenamespace.exec( types[ t ] ) || [];
                type = origType = tmp[ 1 ];
                namespaces = ( tmp[ 2 ] || "" ).split( "." ).sort();

                // There *must* be a type, no attaching namespace-only handlers
                if ( !type ) {
                    continue;
                }

                // If event changes its type, use the special event handlers for the changed type
                special = jQuery.event.special[ type ] || {};

                // If selector defined, determine special event api type, otherwise given type
                type = ( selector ? special.delegateType : special.bindType ) || type;

                // Update special based on newly reset type
                special = jQuery.event.special[ type ] || {};

                // handleObj is passed to all event handlers
                handleObj = jQuery.extend( {
                    type: type,
                    origType: origType,
                    data: data,
                    handler: handler,
                    guid: handler.guid,
                    selector: selector,
                    needsContext: selector && jQuery.expr.match.needsContext.test( selector ),
                    namespace: namespaces.join( "." )
                }, handleObjIn );

                // Init the event handler queue if we're the first
                if ( !( handlers = events[ type ] ) ) {
                    handlers = events[ type ] = [];
                    handlers.delegateCount = 0;

                    // Only use addEventListener if the special events handler returns false
                    if ( !special.setup ||
                        special.setup.call( elem, data, namespaces, eventHandle ) === false ) {

                        if ( elem.addEventListener ) {
                            elem.addEventListener( type, eventHandle );
                        }
                    }
                }

                if ( special.add ) {
                    special.add.call( elem, handleObj );

                    if ( !handleObj.handler.guid ) {
                        handleObj.handler.guid = handler.guid;
                    }
                }

                // Add to the element's handler list, delegates in front
                if ( selector ) {
                    handlers.splice( handlers.delegateCount++, 0, handleObj );
                } else {
                    handlers.push( handleObj );
                }

                // Keep track of which events have ever been used, for event optimization
                jQuery.event.global[ type ] = true;
            }

        },

        // Detach an event or set of events from an element
        remove: function( elem, types, handler, selector, mappedTypes ) {

            var j, origCount, tmp,
                events, t, handleObj,
                special, handlers, type, namespaces, origType,
                elemData = dataPriv.hasData( elem ) && dataPriv.get( elem );

            if ( !elemData || !( events = elemData.events ) ) {
                return;
            }

            // Once for each type.namespace in types; type may be omitted
            types = ( types || "" ).match( rnothtmlwhite ) || [ "" ];
            t = types.length;
            while ( t-- ) {
                tmp = rtypenamespace.exec( types[ t ] ) || [];
                type = origType = tmp[ 1 ];
                namespaces = ( tmp[ 2 ] || "" ).split( "." ).sort();

                // Unbind all events (on this namespace, if provided) for the element
                if ( !type ) {
                    for ( type in events ) {
                        jQuery.event.remove( elem, type + types[ t ], handler, selector, true );
                    }
                    continue;
                }

                special = jQuery.event.special[ type ] || {};
                type = ( selector ? special.delegateType : special.bindType ) || type;
                handlers = events[ type ] || [];
                tmp = tmp[ 2 ] &&
                    new RegExp( "(^|\\.)" + namespaces.join( "\\.(?:.*\\.|)" ) + "(\\.|$)" );

                // Remove matching events
                origCount = j = handlers.length;
                while ( j-- ) {
                    handleObj = handlers[ j ];

                    if ( ( mappedTypes || origType === handleObj.origType ) &&
                        ( !handler || handler.guid === handleObj.guid ) &&
                        ( !tmp || tmp.test( handleObj.namespace ) ) &&
                        ( !selector || selector === handleObj.selector ||
                            selector === "**" && handleObj.selector ) ) {
                        handlers.splice( j, 1 );

                        if ( handleObj.selector ) {
                            handlers.delegateCount--;
                        }
                        if ( special.remove ) {
                            special.remove.call( elem, handleObj );
                        }
                    }
                }

                // Remove generic event handler if we removed something and no more handlers exist
                // (avoids potential for endless recursion during removal of special event handlers)
                if ( origCount && !handlers.length ) {
                    if ( !special.teardown ||
                        special.teardown.call( elem, namespaces, elemData.handle ) === false ) {

                        jQuery.removeEvent( elem, type, elemData.handle );
                    }

                    delete events[ type ];
                }
            }

            // Remove data and the expando if it's no longer used
            if ( jQuery.isEmptyObject( events ) ) {
                dataPriv.remove( elem, "handle events" );
            }
        },

        dispatch: function( nativeEvent ) {

            var i, j, ret, matched, handleObj, handlerQueue,
                args = new Array( arguments.length ),

                // Make a writable jQuery.Event from the native event object
                event = jQuery.event.fix( nativeEvent ),

                handlers = (
                    dataPriv.get( this, "events" ) || Object.create( null )
                )[ event.type ] || [],
                special = jQuery.event.special[ event.type ] || {};

            // Use the fix-ed jQuery.Event rather than the (read-only) native event
            args[ 0 ] = event;

            for ( i = 1; i < arguments.length; i++ ) {
                args[ i ] = arguments[ i ];
            }

            event.delegateTarget = this;

            // Call the preDispatch hook for the mapped type, and let it bail if desired
            if ( special.preDispatch && special.preDispatch.call( this, event ) === false ) {
                return;
            }

            // Determine handlers
            handlerQueue = jQuery.event.handlers.call( this, event, handlers );

            // Run delegates first; they may want to stop propagation beneath us
            i = 0;
            while ( ( matched = handlerQueue[ i++ ] ) && !event.isPropagationStopped() ) {
                event.currentTarget = matched.elem;

                j = 0;
                while ( ( handleObj = matched.handlers[ j++ ] ) &&
                !event.isImmediatePropagationStopped() ) {

                    // If the event is namespaced, then each handler is only invoked if it is
                    // specially universal or its namespaces are a superset of the event's.
                    if ( !event.rnamespace || handleObj.namespace === false ||
                        event.rnamespace.test( handleObj.namespace ) ) {

                        event.handleObj = handleObj;
                        event.data = handleObj.data;

                        ret = ( ( jQuery.event.special[ handleObj.origType ] || {} ).handle ||
                            handleObj.handler ).apply( matched.elem, args );

                        if ( ret !== undefined ) {
                            if ( ( event.result = ret ) === false ) {
                                event.preventDefault();
                                event.stopPropagation();
                            }
                        }
                    }
                }
            }

            // Call the postDispatch hook for the mapped type
            if ( special.postDispatch ) {
                special.postDispatch.call( this, event );
            }

            return event.result;
        },

        handlers: function( event, handlers ) {
            var i, handleObj, sel, matchedHandlers, matchedSelectors,
                handlerQueue = [],
                delegateCount = handlers.delegateCount,
                cur = event.target;

            // Find delegate handlers
            if ( delegateCount &&

                // Support: IE <=9
                // Black-hole SVG <use> instance trees (trac-13180)
                cur.nodeType &&

                // Support: Firefox <=42
                // Suppress spec-violating clicks indicating a non-primary pointer button (trac-3861)
                // https://www.w3.org/TR/DOM-Level-3-Events/#event-type-click
                // Support: IE 11 only
                // ...but not arrow key "clicks" of radio inputs, which can have `button` -1 (gh-2343)
                !( event.type === "click" && event.button >= 1 ) ) {

                for ( ; cur !== this; cur = cur.parentNode || this ) {

                    // Don't check non-elements (#13208)
                    // Don't process clicks on disabled elements (#6911, #8165, #11382, #11764)
                    if ( cur.nodeType === 1 && !( event.type === "click" && cur.disabled === true ) ) {
                        matchedHandlers = [];
                        matchedSelectors = {};
                        for ( i = 0; i < delegateCount; i++ ) {
                            handleObj = handlers[ i ];

                            // Don't conflict with Object.prototype properties (#13203)
                            sel = handleObj.selector + " ";

                            if ( matchedSelectors[ sel ] === undefined ) {
                                matchedSelectors[ sel ] = handleObj.needsContext ?
                                    jQuery( sel, this ).index( cur ) > -1 :
                                    jQuery.find( sel, this, null, [ cur ] ).length;
                            }
                            if ( matchedSelectors[ sel ] ) {
                                matchedHandlers.push( handleObj );
                            }
                        }
                        if ( matchedHandlers.length ) {
                            handlerQueue.push( { elem: cur, handlers: matchedHandlers } );
                        }
                    }
                }
            }

            // Add the remaining (directly-bound) handlers
            cur = this;
            if ( delegateCount < handlers.length ) {
                handlerQueue.push( { elem: cur, handlers: handlers.slice( delegateCount ) } );
            }

            return handlerQueue;
        },

        addProp: function( name, hook ) {
            Object.defineProperty( jQuery.Event.prototype, name, {
                enumerable: true,
                configurable: true,

                get: isFunction( hook ) ?
                    function() {
                        if ( this.originalEvent ) {
                            return hook( this.originalEvent );
                        }
                    } :
                    function() {
                        if ( this.originalEvent ) {
                            return this.originalEvent[ name ];
                        }
                    },

                set: function( value ) {
                    Object.defineProperty( this, name, {
                        enumerable: true,
                        configurable: true,
                        writable: true,
                        value: value
                    } );
                }
            } );
        },

        fix: function( originalEvent ) {
            return originalEvent[ jQuery.expando ] ?
                originalEvent :
                new jQuery.Event( originalEvent );
        },

        special: {
            load: {

                // Prevent triggered image.load events from bubbling to window.load
                noBubble: true
            },
            click: {

                // Utilize native event to ensure correct state for checkable inputs
                setup: function( data ) {

                    // For mutual compressibility with _default, replace `this` access with a local var.
                    // `|| data` is dead code meant only to preserve the variable through minification.
                    var el = this || data;

                    // Claim the first handler
                    if ( rcheckableType.test( el.type ) &&
                        el.click && nodeName( el, "input" ) ) {

                        // dataPriv.set( el, "click", ... )
                        leverageNative( el, "click", returnTrue );
                    }

                    // Return false to allow normal processing in the caller
                    return false;
                },
                trigger: function( data ) {

                    // For mutual compressibility with _default, replace `this` access with a local var.
                    // `|| data` is dead code meant only to preserve the variable through minification.
                    var el = this || data;

                    // Force setup before triggering a click
                    if ( rcheckableType.test( el.type ) &&
                        el.click && nodeName( el, "input" ) ) {

                        leverageNative( el, "click" );
                    }

                    // Return non-false to allow normal event-path propagation
                    return true;
                },

                // For cross-browser consistency, suppress native .click() on links
                // Also prevent it if we're currently inside a leveraged native-event stack
                _default: function( event ) {
                    var target = event.target;
                    return rcheckableType.test( target.type ) &&
                        target.click && nodeName( target, "input" ) &&
                        dataPriv.get( target, "click" ) ||
                        nodeName( target, "a" );
                }
            },

            beforeunload: {
                postDispatch: function( event ) {

                    // Support: Firefox 20+
                    // Firefox doesn't alert if the returnValue field is not set.
                    if ( event.result !== undefined && event.originalEvent ) {
                        event.originalEvent.returnValue = event.result;
                    }
                }
            }
        }
    };

// Ensure the presence of an event listener that handles manually-triggered
// synthetic events by interrupting progress until reinvoked in response to
// *native* events that it fires directly, ensuring that state changes have
// already occurred before other listeners are invoked.
    function leverageNative( el, type, expectSync ) {

        // Missing expectSync indicates a trigger call, which must force setup through jQuery.event.add
        if ( !expectSync ) {
            if ( dataPriv.get( el, type ) === undefined ) {
                jQuery.event.add( el, type, returnTrue );
            }
            return;
        }

        // Register the controller as a special universal handler for all event namespaces
        dataPriv.set( el, type, false );
        jQuery.event.add( el, type, {
            namespace: false,
            handler: function( event ) {
                var notAsync, result,
                    saved = dataPriv.get( this, type );

                if ( ( event.isTrigger & 1 ) && this[ type ] ) {

                    // Interrupt processing of the outer synthetic .trigger()ed event
                    // Saved data should be false in such cases, but might be a leftover capture object
                    // from an async native handler (gh-4350)
                    if ( !saved.length ) {

                        // Store arguments for use when handling the inner native event
                        // There will always be at least one argument (an event object), so this array
                        // will not be confused with a leftover capture object.
                        saved = slice.call( arguments );
                        dataPriv.set( this, type, saved );

                        // Trigger the native event and capture its result
                        // Support: IE <=9 - 11+
                        // focus() and blur() are asynchronous
                        notAsync = expectSync( this, type );
                        this[ type ]();
                        result = dataPriv.get( this, type );
                        if ( saved !== result || notAsync ) {
                            dataPriv.set( this, type, false );
                        } else {
                            result = {};
                        }
                        if ( saved !== result ) {

                            // Cancel the outer synthetic event
                            event.stopImmediatePropagation();
                            event.preventDefault();
                            return result.value;
                        }

                        // If this is an inner synthetic event for an event with a bubbling surrogate
                        // (focus or blur), assume that the surrogate already propagated from triggering the
                        // native event and prevent that from happening again here.
                        // This technically gets the ordering wrong w.r.t. to `.trigger()` (in which the
                        // bubbling surrogate propagates *after* the non-bubbling base), but that seems
                        // less bad than duplication.
                    } else if ( ( jQuery.event.special[ type ] || {} ).delegateType ) {
                        event.stopPropagation();
                    }

                    // If this is a native event triggered above, everything is now in order
                    // Fire an inner synthetic event with the original arguments
                } else if ( saved.length ) {

                    // ...and capture the result
                    dataPriv.set( this, type, {
                        value: jQuery.event.trigger(

                            // Support: IE <=9 - 11+
                            // Extend with the prototype to reset the above stopImmediatePropagation()
                            jQuery.extend( saved[ 0 ], jQuery.Event.prototype ),
                            saved.slice( 1 ),
                            this
                        )
                    } );

                    // Abort handling of the native event
                    event.stopImmediatePropagation();
                }
            }
        } );
    }

    jQuery.removeEvent = function( elem, type, handle ) {

        // This "if" is needed for plain objects
        if ( elem.removeEventListener ) {
            elem.removeEventListener( type, handle );
        }
    };

    jQuery.Event = function( src, props ) {

        // Allow instantiation without the 'new' keyword
        if ( !( this instanceof jQuery.Event ) ) {
            return new jQuery.Event( src, props );
        }

        // Event object
        if ( src && src.type ) {
            this.originalEvent = src;
            this.type = src.type;

            // Events bubbling up the document may have been marked as prevented
            // by a handler lower down the tree; reflect the correct value.
            this.isDefaultPrevented = src.defaultPrevented ||
            src.defaultPrevented === undefined &&

            // Support: Android <=2.3 only
            src.returnValue === false ?
                returnTrue :
                returnFalse;

            // Create target properties
            // Support: Safari <=6 - 7 only
            // Target should not be a text node (#504, #13143)
            this.target = ( src.target && src.target.nodeType === 3 ) ?
                src.target.parentNode :
                src.target;

            this.currentTarget = src.currentTarget;
            this.relatedTarget = src.relatedTarget;

            // Event type
        } else {
            this.type = src;
        }

        // Put explicitly provided properties onto the event object
        if ( props ) {
            jQuery.extend( this, props );
        }

        // Create a timestamp if incoming event doesn't have one
        this.timeStamp = src && src.timeStamp || Date.now();

        // Mark it as fixed
        this[ jQuery.expando ] = true;
    };

// jQuery.Event is based on DOM3 Events as specified by the ECMAScript Language Binding
// https://www.w3.org/TR/2003/WD-DOM-Level-3-Events-20030331/ecma-script-binding.html
    jQuery.Event.prototype = {
        constructor: jQuery.Event,
        isDefaultPrevented: returnFalse,
        isPropagationStopped: returnFalse,
        isImmediatePropagationStopped: returnFalse,
        isSimulated: false,

        preventDefault: function() {
            var e = this.originalEvent;

            this.isDefaultPrevented = returnTrue;

            if ( e && !this.isSimulated ) {
                e.preventDefault();
            }
        },
        stopPropagation: function() {
            var e = this.originalEvent;

            this.isPropagationStopped = returnTrue;

            if ( e && !this.isSimulated ) {
                e.stopPropagation();
            }
        },
        stopImmediatePropagation: function() {
            var e = this.originalEvent;

            this.isImmediatePropagationStopped = returnTrue;

            if ( e && !this.isSimulated ) {
                e.stopImmediatePropagation();
            }

            this.stopPropagation();
        }
    };

// Includes all common event props including KeyEvent and MouseEvent specific props
    jQuery.each( {
        altKey: true,
        bubbles: true,
        cancelable: true,
        changedTouches: true,
        ctrlKey: true,
        detail: true,
        eventPhase: true,
        metaKey: true,
        pageX: true,
        pageY: true,
        shiftKey: true,
        view: true,
        "char": true,
        code: true,
        charCode: true,
        key: true,
        keyCode: true,
        button: true,
        buttons: true,
        clientX: true,
        clientY: true,
        offsetX: true,
        offsetY: true,
        pointerId: true,
        pointerType: true,
        screenX: true,
        screenY: true,
        targetTouches: true,
        toElement: true,
        touches: true,

        which: function( event ) {
            var button = event.button;

            // Add which for key events
            if ( event.which == null && rkeyEvent.test( event.type ) ) {
                return event.charCode != null ? event.charCode : event.keyCode;
            }

            // Add which for click: 1 === left; 2 === middle; 3 === right
            if ( !event.which && button !== undefined && rmouseEvent.test( event.type ) ) {
                if ( button & 1 ) {
                    return 1;
                }

                if ( button & 2 ) {
                    return 3;
                }

                if ( button & 4 ) {
                    return 2;
                }

                return 0;
            }

            return event.which;
        }
    }, jQuery.event.addProp );

    jQuery.each( { focus: "focusin", blur: "focusout" }, function( type, delegateType ) {
        jQuery.event.special[ type ] = {

            // Utilize native event if possible so blur/focus sequence is correct
            setup: function() {

                // Claim the first handler
                // dataPriv.set( this, "focus", ... )
                // dataPriv.set( this, "blur", ... )
                leverageNative( this, type, expectSync );

                // Return false to allow normal processing in the caller
                return false;
            },
            trigger: function() {

                // Force setup before trigger
                leverageNative( this, type );

                // Return non-false to allow normal event-path propagation
                return true;
            },

            delegateType: delegateType
        };
    } );

// Create mouseenter/leave events using mouseover/out and event-time checks
// so that event delegation works in jQuery.
// Do the same for pointerenter/pointerleave and pointerover/pointerout
//
// Support: Safari 7 only
// Safari sends mouseenter too often; see:
// https://bugs.chromium.org/p/chromium/issues/detail?id=470258
// for the description of the bug (it existed in older Chrome versions as well).
    jQuery.each( {
        mouseenter: "mouseover",
        mouseleave: "mouseout",
        pointerenter: "pointerover",
        pointerleave: "pointerout"
    }, function( orig, fix ) {
        jQuery.event.special[ orig ] = {
            delegateType: fix,
            bindType: fix,

            handle: function( event ) {
                var ret,
                    target = this,
                    related = event.relatedTarget,
                    handleObj = event.handleObj;

                // For mouseenter/leave call the handler if related is outside the target.
                // NB: No relatedTarget if the mouse left/entered the browser window
                if ( !related || ( related !== target && !jQuery.contains( target, related ) ) ) {
                    event.type = handleObj.origType;
                    ret = handleObj.handler.apply( this, arguments );
                    event.type = fix;
                }
                return ret;
            }
        };
    } );

    jQuery.fn.extend( {

        on: function( types, selector, data, fn ) {
            return on( this, types, selector, data, fn );
        },
        one: function( types, selector, data, fn ) {
            return on( this, types, selector, data, fn, 1 );
        },
        off: function( types, selector, fn ) {
            var handleObj, type;
            if ( types && types.preventDefault && types.handleObj ) {

                // ( event )  dispatched jQuery.Event
                handleObj = types.handleObj;
                jQuery( types.delegateTarget ).off(
                    handleObj.namespace ?
                        handleObj.origType + "." + handleObj.namespace :
                        handleObj.origType,
                    handleObj.selector,
                    handleObj.handler
                );
                return this;
            }
            if ( typeof types === "object" ) {

                // ( types-object [, selector] )
                for ( type in types ) {
                    this.off( type, selector, types[ type ] );
                }
                return this;
            }
            if ( selector === false || typeof selector === "function" ) {

                // ( types [, fn] )
                fn = selector;
                selector = undefined;
            }
            if ( fn === false ) {
                fn = returnFalse;
            }
            return this.each( function() {
                jQuery.event.remove( this, types, fn, selector );
            } );
        }
    } );


    var

        // Support: IE <=10 - 11, Edge 12 - 13 only
        // In IE/Edge using regex groups here causes severe slowdowns.
        // See https://connect.microsoft.com/IE/feedback/details/1736512/
        rnoInnerhtml = /<script|<style|<link/i,

        // checked="checked" or checked
        rchecked = /checked\s*(?:[^=]|=\s*.checked.)/i,
        rcleanScript = /^\s*<!(?:\[CDATA\[|--)|(?:\]\]|--)>\s*$/g;

// Prefer a tbody over its parent table for containing new rows
    function manipulationTarget( elem, content ) {
        if ( nodeName( elem, "table" ) &&
            nodeName( content.nodeType !== 11 ? content : content.firstChild, "tr" ) ) {

            return jQuery( elem ).children( "tbody" )[ 0 ] || elem;
        }

        return elem;
    }

// Replace/restore the type attribute of script elements for safe DOM manipulation
    function disableScript( elem ) {
        elem.type = ( elem.getAttribute( "type" ) !== null ) + "/" + elem.type;
        return elem;
    }
    function restoreScript( elem ) {
        if ( ( elem.type || "" ).slice( 0, 5 ) === "true/" ) {
            elem.type = elem.type.slice( 5 );
        } else {
            elem.removeAttribute( "type" );
        }

        return elem;
    }

    function cloneCopyEvent( src, dest ) {
        var i, l, type, pdataOld, udataOld, udataCur, events;

        if ( dest.nodeType !== 1 ) {
            return;
        }

        // 1. Copy private data: events, handlers, etc.
        if ( dataPriv.hasData( src ) ) {
            pdataOld = dataPriv.get( src );
            events = pdataOld.events;

            if ( events ) {
                dataPriv.remove( dest, "handle events" );

                for ( type in events ) {
                    for ( i = 0, l = events[ type ].length; i < l; i++ ) {
                        jQuery.event.add( dest, type, events[ type ][ i ] );
                    }
                }
            }
        }

        // 2. Copy user data
        if ( dataUser.hasData( src ) ) {
            udataOld = dataUser.access( src );
            udataCur = jQuery.extend( {}, udataOld );

            dataUser.set( dest, udataCur );
        }
    }

// Fix IE bugs, see support tests
    function fixInput( src, dest ) {
        var nodeName = dest.nodeName.toLowerCase();

        // Fails to persist the checked state of a cloned checkbox or radio button.
        if ( nodeName === "input" && rcheckableType.test( src.type ) ) {
            dest.checked = src.checked;

            // Fails to return the selected option to the default selected state when cloning options
        } else if ( nodeName === "input" || nodeName === "textarea" ) {
            dest.defaultValue = src.defaultValue;
        }
    }

    function domManip( collection, args, callback, ignored ) {

        // Flatten any nested arrays
        args = flat( args );

        var fragment, first, scripts, hasScripts, node, doc,
            i = 0,
            l = collection.length,
            iNoClone = l - 1,
            value = args[ 0 ],
            valueIsFunction = isFunction( value );

        // We can't cloneNode fragments that contain checked, in WebKit
        if ( valueIsFunction ||
            ( l > 1 && typeof value === "string" &&
                !support.checkClone && rchecked.test( value ) ) ) {
            return collection.each( function( index ) {
                var self = collection.eq( index );
                if ( valueIsFunction ) {
                    args[ 0 ] = value.call( this, index, self.html() );
                }
                domManip( self, args, callback, ignored );
            } );
        }

        if ( l ) {
            fragment = buildFragment( args, collection[ 0 ].ownerDocument, false, collection, ignored );
            first = fragment.firstChild;

            if ( fragment.childNodes.length === 1 ) {
                fragment = first;
            }

            // Require either new content or an interest in ignored elements to invoke the callback
            if ( first || ignored ) {
                scripts = jQuery.map( getAll( fragment, "script" ), disableScript );
                hasScripts = scripts.length;

                // Use the original fragment for the last item
                // instead of the first because it can end up
                // being emptied incorrectly in certain situations (#8070).
                for ( ; i < l; i++ ) {
                    node = fragment;

                    if ( i !== iNoClone ) {
                        node = jQuery.clone( node, true, true );

                        // Keep references to cloned scripts for later restoration
                        if ( hasScripts ) {

                            // Support: Android <=4.0 only, PhantomJS 1 only
                            // push.apply(_, arraylike) throws on ancient WebKit
                            jQuery.merge( scripts, getAll( node, "script" ) );
                        }
                    }

                    callback.call( collection[ i ], node, i );
                }

                if ( hasScripts ) {
                    doc = scripts[ scripts.length - 1 ].ownerDocument;

                    // Reenable scripts
                    jQuery.map( scripts, restoreScript );

                    // Evaluate executable scripts on first document insertion
                    for ( i = 0; i < hasScripts; i++ ) {
                        node = scripts[ i ];
                        if ( rscriptType.test( node.type || "" ) &&
                            !dataPriv.access( node, "globalEval" ) &&
                            jQuery.contains( doc, node ) ) {

                            if ( node.src && ( node.type || "" ).toLowerCase()  !== "module" ) {

                                // Optional AJAX dependency, but won't run scripts if not present
                                if ( jQuery._evalUrl && !node.noModule ) {
                                    jQuery._evalUrl( node.src, {
                                        nonce: node.nonce || node.getAttribute( "nonce" )
                                    }, doc );
                                }
                            } else {
                                DOMEval( node.textContent.replace( rcleanScript, "" ), node, doc );
                            }
                        }
                    }
                }
            }
        }

        return collection;
    }

    function remove( elem, selector, keepData ) {
        var node,
            nodes = selector ? jQuery.filter( selector, elem ) : elem,
            i = 0;

        for ( ; ( node = nodes[ i ] ) != null; i++ ) {
            if ( !keepData && node.nodeType === 1 ) {
                jQuery.cleanData( getAll( node ) );
            }

            if ( node.parentNode ) {
                if ( keepData && isAttached( node ) ) {
                    setGlobalEval( getAll( node, "script" ) );
                }
                node.parentNode.removeChild( node );
            }
        }

        return elem;
    }

    jQuery.extend( {
        htmlPrefilter: function( html ) {
            return html;
        },

        clone: function( elem, dataAndEvents, deepDataAndEvents ) {
            var i, l, srcElements, destElements,
                clone = elem.cloneNode( true ),
                inPage = isAttached( elem );

            // Fix IE cloning issues
            if ( !support.noCloneChecked && ( elem.nodeType === 1 || elem.nodeType === 11 ) &&
                !jQuery.isXMLDoc( elem ) ) {

                // We eschew Sizzle here for performance reasons: https://jsperf.com/getall-vs-sizzle/2
                destElements = getAll( clone );
                srcElements = getAll( elem );

                for ( i = 0, l = srcElements.length; i < l; i++ ) {
                    fixInput( srcElements[ i ], destElements[ i ] );
                }
            }

            // Copy the events from the original to the clone
            if ( dataAndEvents ) {
                if ( deepDataAndEvents ) {
                    srcElements = srcElements || getAll( elem );
                    destElements = destElements || getAll( clone );

                    for ( i = 0, l = srcElements.length; i < l; i++ ) {
                        cloneCopyEvent( srcElements[ i ], destElements[ i ] );
                    }
                } else {
                    cloneCopyEvent( elem, clone );
                }
            }

            // Preserve script evaluation history
            destElements = getAll( clone, "script" );
            if ( destElements.length > 0 ) {
                setGlobalEval( destElements, !inPage && getAll( elem, "script" ) );
            }

            // Return the cloned set
            return clone;
        },

        cleanData: function( elems ) {
            var data, elem, type,
                special = jQuery.event.special,
                i = 0;

            for ( ; ( elem = elems[ i ] ) !== undefined; i++ ) {
                if ( acceptData( elem ) ) {
                    if ( ( data = elem[ dataPriv.expando ] ) ) {
                        if ( data.events ) {
                            for ( type in data.events ) {
                                if ( special[ type ] ) {
                                    jQuery.event.remove( elem, type );

                                    // This is a shortcut to avoid jQuery.event.remove's overhead
                                } else {
                                    jQuery.removeEvent( elem, type, data.handle );
                                }
                            }
                        }

                        // Support: Chrome <=35 - 45+
                        // Assign undefined instead of using delete, see Data#remove
                        elem[ dataPriv.expando ] = undefined;
                    }
                    if ( elem[ dataUser.expando ] ) {

                        // Support: Chrome <=35 - 45+
                        // Assign undefined instead of using delete, see Data#remove
                        elem[ dataUser.expando ] = undefined;
                    }
                }
            }
        }
    } );

    jQuery.fn.extend( {
        detach: function( selector ) {
            return remove( this, selector, true );
        },

        remove: function( selector ) {
            return remove( this, selector );
        },

        text: function( value ) {
            return access( this, function( value ) {
                return value === undefined ?
                    jQuery.text( this ) :
                    this.empty().each( function() {
                        if ( this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9 ) {
                            this.textContent = value;
                        }
                    } );
            }, null, value, arguments.length );
        },

        append: function() {
            return domManip( this, arguments, function( elem ) {
                if ( this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9 ) {
                    var target = manipulationTarget( this, elem );
                    target.appendChild( elem );
                }
            } );
        },

        prepend: function() {
            return domManip( this, arguments, function( elem ) {
                if ( this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9 ) {
                    var target = manipulationTarget( this, elem );
                    target.insertBefore( elem, target.firstChild );
                }
            } );
        },

        before: function() {
            return domManip( this, arguments, function( elem ) {
                if ( this.parentNode ) {
                    this.parentNode.insertBefore( elem, this );
                }
            } );
        },

        after: function() {
            return domManip( this, arguments, function( elem ) {
                if ( this.parentNode ) {
                    this.parentNode.insertBefore( elem, this.nextSibling );
                }
            } );
        },

        empty: function() {
            var elem,
                i = 0;

            for ( ; ( elem = this[ i ] ) != null; i++ ) {
                if ( elem.nodeType === 1 ) {

                    // Prevent memory leaks
                    jQuery.cleanData( getAll( elem, false ) );

                    // Remove any remaining nodes
                    elem.textContent = "";
                }
            }

            return this;
        },

        clone: function( dataAndEvents, deepDataAndEvents ) {
            dataAndEvents = dataAndEvents == null ? false : dataAndEvents;
            deepDataAndEvents = deepDataAndEvents == null ? dataAndEvents : deepDataAndEvents;

            return this.map( function() {
                return jQuery.clone( this, dataAndEvents, deepDataAndEvents );
            } );
        },

        html: function( value ) {
            return access( this, function( value ) {
                var elem = this[ 0 ] || {},
                    i = 0,
                    l = this.length;

                if ( value === undefined && elem.nodeType === 1 ) {
                    return elem.innerHTML;
                }

                // See if we can take a shortcut and just use innerHTML
                if ( typeof value === "string" && !rnoInnerhtml.test( value ) &&
                    !wrapMap[ ( rtagName.exec( value ) || [ "", "" ] )[ 1 ].toLowerCase() ] ) {

                    value = jQuery.htmlPrefilter( value );

                    try {
                        for ( ; i < l; i++ ) {
                            elem = this[ i ] || {};

                            // Remove element nodes and prevent memory leaks
                            if ( elem.nodeType === 1 ) {
                                jQuery.cleanData( getAll( elem, false ) );
                                elem.innerHTML = value;
                            }
                        }

                        elem = 0;

                        // If using innerHTML throws an exception, use the fallback method
                    } catch ( e ) {}
                }

                if ( elem ) {
                    this.empty().append( value );
                }
            }, null, value, arguments.length );
        },

        replaceWith: function() {
            var ignored = [];

            // Make the changes, replacing each non-ignored context element with the new content
            return domManip( this, arguments, function( elem ) {
                var parent = this.parentNode;

                if ( jQuery.inArray( this, ignored ) < 0 ) {
                    jQuery.cleanData( getAll( this ) );
                    if ( parent ) {
                        parent.replaceChild( elem, this );
                    }
                }

                // Force callback invocation
            }, ignored );
        }
    } );

    jQuery.each( {
        appendTo: "append",
        prependTo: "prepend",
        insertBefore: "before",
        insertAfter: "after",
        replaceAll: "replaceWith"
    }, function( name, original ) {
        jQuery.fn[ name ] = function( selector ) {
            var elems,
                ret = [],
                insert = jQuery( selector ),
                last = insert.length - 1,
                i = 0;

            for ( ; i <= last; i++ ) {
                elems = i === last ? this : this.clone( true );
                jQuery( insert[ i ] )[ original ]( elems );

                // Support: Android <=4.0 only, PhantomJS 1 only
                // .get() because push.apply(_, arraylike) throws on ancient WebKit
                push.apply( ret, elems.get() );
            }

            return this.pushStack( ret );
        };
    } );
    var rnumnonpx = new RegExp( "^(" + pnum + ")(?!px)[a-z%]+$", "i" );

    var getStyles = function( elem ) {

        // Support: IE <=11 only, Firefox <=30 (#15098, #14150)
        // IE throws on elements created in popups
        // FF meanwhile throws on frame elements through "defaultView.getComputedStyle"
        var view = elem.ownerDocument.defaultView;

        if ( !view || !view.opener ) {
            view = window;
        }

        return view.getComputedStyle( elem );
    };

    var swap = function( elem, options, callback ) {
        var ret, name,
            old = {};

        // Remember the old values, and insert the new ones
        for ( name in options ) {
            old[ name ] = elem.style[ name ];
            elem.style[ name ] = options[ name ];
        }

        ret = callback.call( elem );

        // Revert the old values
        for ( name in options ) {
            elem.style[ name ] = old[ name ];
        }

        return ret;
    };


    var rboxStyle = new RegExp( cssExpand.join( "|" ), "i" );



    ( function() {

        // Executing both pixelPosition & boxSizingReliable tests require only one layout
        // so they're executed at the same time to save the second computation.
        function computeStyleTests() {

            // This is a singleton, we need to execute it only once
            if ( !div ) {
                return;
            }

            container.style.cssText = "position:absolute;left:-11111px;width:60px;" +
                "margin-top:1px;padding:0;border:0";
            div.style.cssText =
                "position:relative;display:block;box-sizing:border-box;overflow:scroll;" +
                "margin:auto;border:1px;padding:1px;" +
                "width:60%;top:1%";
            documentElement.appendChild( container ).appendChild( div );

            var divStyle = window.getComputedStyle( div );
            pixelPositionVal = divStyle.top !== "1%";

            // Support: Android 4.0 - 4.3 only, Firefox <=3 - 44
            reliableMarginLeftVal = roundPixelMeasures( divStyle.marginLeft ) === 12;

            // Support: Android 4.0 - 4.3 only, Safari <=9.1 - 10.1, iOS <=7.0 - 9.3
            // Some styles come back with percentage values, even though they shouldn't
            div.style.right = "60%";
            pixelBoxStylesVal = roundPixelMeasures( divStyle.right ) === 36;

            // Support: IE 9 - 11 only
            // Detect misreporting of content dimensions for box-sizing:border-box elements
            boxSizingReliableVal = roundPixelMeasures( divStyle.width ) === 36;

            // Support: IE 9 only
            // Detect overflow:scroll screwiness (gh-3699)
            // Support: Chrome <=64
            // Don't get tricked when zoom affects offsetWidth (gh-4029)
            div.style.position = "absolute";
            scrollboxSizeVal = roundPixelMeasures( div.offsetWidth / 3 ) === 12;

            documentElement.removeChild( container );

            // Nullify the div so it wouldn't be stored in the memory and
            // it will also be a sign that checks already performed
            div = null;
        }

        function roundPixelMeasures( measure ) {
            return Math.round( parseFloat( measure ) );
        }

        var pixelPositionVal, boxSizingReliableVal, scrollboxSizeVal, pixelBoxStylesVal,
            reliableTrDimensionsVal, reliableMarginLeftVal,
            container = document.createElement( "div" ),
            div = document.createElement( "div" );

        // Finish early in limited (non-browser) environments
        if ( !div.style ) {
            return;
        }

        // Support: IE <=9 - 11 only
        // Style of cloned element affects source element cloned (#8908)
        div.style.backgroundClip = "content-box";
        div.cloneNode( true ).style.backgroundClip = "";
        support.clearCloneStyle = div.style.backgroundClip === "content-box";

        jQuery.extend( support, {
            boxSizingReliable: function() {
                computeStyleTests();
                return boxSizingReliableVal;
            },
            pixelBoxStyles: function() {
                computeStyleTests();
                return pixelBoxStylesVal;
            },
            pixelPosition: function() {
                computeStyleTests();
                return pixelPositionVal;
            },
            reliableMarginLeft: function() {
                computeStyleTests();
                return reliableMarginLeftVal;
            },
            scrollboxSize: function() {
                computeStyleTests();
                return scrollboxSizeVal;
            },

            // Support: IE 9 - 11+, Edge 15 - 18+
            // IE/Edge misreport `getComputedStyle` of table rows with width/height
            // set in CSS while `offset*` properties report correct values.
            // Behavior in IE 9 is more subtle than in newer versions & it passes
            // some versions of this test; make sure not to make it pass there!
            reliableTrDimensions: function() {
                var table, tr, trChild, trStyle;
                if ( reliableTrDimensionsVal == null ) {
                    table = document.createElement( "table" );
                    tr = document.createElement( "tr" );
                    trChild = document.createElement( "div" );

                    table.style.cssText = "position:absolute;left:-11111px";
                    tr.style.height = "1px";
                    trChild.style.height = "9px";

                    documentElement
                        .appendChild( table )
                        .appendChild( tr )
                        .appendChild( trChild );

                    trStyle = window.getComputedStyle( tr );
                    reliableTrDimensionsVal = parseInt( trStyle.height ) > 3;

                    documentElement.removeChild( table );
                }
                return reliableTrDimensionsVal;
            }
        } );
    } )();


    function curCSS( elem, name, computed ) {
        var width, minWidth, maxWidth, ret,

            // Support: Firefox 51+
            // Retrieving style before computed somehow
            // fixes an issue with getting wrong values
            // on detached elements
            style = elem.style;

        computed = computed || getStyles( elem );

        // getPropertyValue is needed for:
        //   .css('filter') (IE 9 only, #12537)
        //   .css('--customProperty) (#3144)
        if ( computed ) {
            ret = computed.getPropertyValue( name ) || computed[ name ];

            if ( ret === "" && !isAttached( elem ) ) {
                ret = jQuery.style( elem, name );
            }

            // A tribute to the "awesome hack by Dean Edwards"
            // Android Browser returns percentage for some values,
            // but width seems to be reliably pixels.
            // This is against the CSSOM draft spec:
            // https://drafts.csswg.org/cssom/#resolved-values
            if ( !support.pixelBoxStyles() && rnumnonpx.test( ret ) && rboxStyle.test( name ) ) {

                // Remember the original values
                width = style.width;
                minWidth = style.minWidth;
                maxWidth = style.maxWidth;

                // Put in the new values to get a computed value out
                style.minWidth = style.maxWidth = style.width = ret;
                ret = computed.width;

                // Revert the changed values
                style.width = width;
                style.minWidth = minWidth;
                style.maxWidth = maxWidth;
            }
        }

        return ret !== undefined ?

            // Support: IE <=9 - 11 only
            // IE returns zIndex value as an integer.
            ret + "" :
            ret;
    }


    function addGetHookIf( conditionFn, hookFn ) {

        // Define the hook, we'll check on the first run if it's really needed.
        return {
            get: function() {
                if ( conditionFn() ) {

                    // Hook not needed (or it's not possible to use it due
                    // to missing dependency), remove it.
                    delete this.get;
                    return;
                }

                // Hook needed; redefine it so that the support test is not executed again.
                return ( this.get = hookFn ).apply( this, arguments );
            }
        };
    }


    var cssPrefixes = [ "Webkit", "Moz", "ms" ],
        emptyStyle = document.createElement( "div" ).style,
        vendorProps = {};

// Return a vendor-prefixed property or undefined
    function vendorPropName( name ) {

        // Check for vendor prefixed names
        var capName = name[ 0 ].toUpperCase() + name.slice( 1 ),
            i = cssPrefixes.length;

        while ( i-- ) {
            name = cssPrefixes[ i ] + capName;
            if ( name in emptyStyle ) {
                return name;
            }
        }
    }

// Return a potentially-mapped jQuery.cssProps or vendor prefixed property
    function finalPropName( name ) {
        var final = jQuery.cssProps[ name ] || vendorProps[ name ];

        if ( final ) {
            return final;
        }
        if ( name in emptyStyle ) {
            return name;
        }
        return vendorProps[ name ] = vendorPropName( name ) || name;
    }


    var

        // Swappable if display is none or starts with table
        // except "table", "table-cell", or "table-caption"
        // See here for display values: https://developer.mozilla.org/en-US/docs/CSS/display
        rdisplayswap = /^(none|table(?!-c[ea]).+)/,
        rcustomProp = /^--/,
        cssShow = { position: "absolute", visibility: "hidden", display: "block" },
        cssNormalTransform = {
            letterSpacing: "0",
            fontWeight: "400"
        };

    function setPositiveNumber( _elem, value, subtract ) {

        // Any relative (+/-) values have already been
        // normalized at this point
        var matches = rcssNum.exec( value );
        return matches ?

            // Guard against undefined "subtract", e.g., when used as in cssHooks
            Math.max( 0, matches[ 2 ] - ( subtract || 0 ) ) + ( matches[ 3 ] || "px" ) :
            value;
    }

    function boxModelAdjustment( elem, dimension, box, isBorderBox, styles, computedVal ) {
        var i = dimension === "width" ? 1 : 0,
            extra = 0,
            delta = 0;

        // Adjustment may not be necessary
        if ( box === ( isBorderBox ? "border" : "content" ) ) {
            return 0;
        }

        for ( ; i < 4; i += 2 ) {

            // Both box models exclude margin
            if ( box === "margin" ) {
                delta += jQuery.css( elem, box + cssExpand[ i ], true, styles );
            }

            // If we get here with a content-box, we're seeking "padding" or "border" or "margin"
            if ( !isBorderBox ) {

                // Add padding
                delta += jQuery.css( elem, "padding" + cssExpand[ i ], true, styles );

                // For "border" or "margin", add border
                if ( box !== "padding" ) {
                    delta += jQuery.css( elem, "border" + cssExpand[ i ] + "Width", true, styles );

                    // But still keep track of it otherwise
                } else {
                    extra += jQuery.css( elem, "border" + cssExpand[ i ] + "Width", true, styles );
                }

                // If we get here with a border-box (content + padding + border), we're seeking "content" or
                // "padding" or "margin"
            } else {

                // For "content", subtract padding
                if ( box === "content" ) {
                    delta -= jQuery.css( elem, "padding" + cssExpand[ i ], true, styles );
                }

                // For "content" or "padding", subtract border
                if ( box !== "margin" ) {
                    delta -= jQuery.css( elem, "border" + cssExpand[ i ] + "Width", true, styles );
                }
            }
        }

        // Account for positive content-box scroll gutter when requested by providing computedVal
        if ( !isBorderBox && computedVal >= 0 ) {

            // offsetWidth/offsetHeight is a rounded sum of content, padding, scroll gutter, and border
            // Assuming integer scroll gutter, subtract the rest and round down
            delta += Math.max( 0, Math.ceil(
                elem[ "offset" + dimension[ 0 ].toUpperCase() + dimension.slice( 1 ) ] -
                computedVal -
                delta -
                extra -
                0.5

                // If offsetWidth/offsetHeight is unknown, then we can't determine content-box scroll gutter
                // Use an explicit zero to avoid NaN (gh-3964)
            ) ) || 0;
        }

        return delta;
    }

    function getWidthOrHeight( elem, dimension, extra ) {

        // Start with computed style
        var styles = getStyles( elem ),

            // To avoid forcing a reflow, only fetch boxSizing if we need it (gh-4322).
            // Fake content-box until we know it's needed to know the true value.
            boxSizingNeeded = !support.boxSizingReliable() || extra,
            isBorderBox = boxSizingNeeded &&
                jQuery.css( elem, "boxSizing", false, styles ) === "border-box",
            valueIsBorderBox = isBorderBox,

            val = curCSS( elem, dimension, styles ),
            offsetProp = "offset" + dimension[ 0 ].toUpperCase() + dimension.slice( 1 );

        // Support: Firefox <=54
        // Return a confounding non-pixel value or feign ignorance, as appropriate.
        if ( rnumnonpx.test( val ) ) {
            if ( !extra ) {
                return val;
            }
            val = "auto";
        }


        // Support: IE 9 - 11 only
        // Use offsetWidth/offsetHeight for when box sizing is unreliable.
        // In those cases, the computed value can be trusted to be border-box.
        if ( ( !support.boxSizingReliable() && isBorderBox ||

            // Support: IE 10 - 11+, Edge 15 - 18+
            // IE/Edge misreport `getComputedStyle` of table rows with width/height
            // set in CSS while `offset*` properties report correct values.
            // Interestingly, in some cases IE 9 doesn't suffer from this issue.
            !support.reliableTrDimensions() && nodeName( elem, "tr" ) ||

            // Fall back to offsetWidth/offsetHeight when value is "auto"
            // This happens for inline elements with no explicit setting (gh-3571)
            val === "auto" ||

            // Support: Android <=4.1 - 4.3 only
            // Also use offsetWidth/offsetHeight for misreported inline dimensions (gh-3602)
            !parseFloat( val ) && jQuery.css( elem, "display", false, styles ) === "inline" ) &&

            // Make sure the element is visible & connected
            elem.getClientRects().length ) {

            isBorderBox = jQuery.css( elem, "boxSizing", false, styles ) === "border-box";

            // Where available, offsetWidth/offsetHeight approximate border box dimensions.
            // Where not available (e.g., SVG), assume unreliable box-sizing and interpret the
            // retrieved value as a content box dimension.
            valueIsBorderBox = offsetProp in elem;
            if ( valueIsBorderBox ) {
                val = elem[ offsetProp ];
            }
        }

        // Normalize "" and auto
        val = parseFloat( val ) || 0;

        // Adjust for the element's box model
        return ( val +
            boxModelAdjustment(
                elem,
                dimension,
                extra || ( isBorderBox ? "border" : "content" ),
                valueIsBorderBox,
                styles,

                // Provide the current computed size to request scroll gutter calculation (gh-3589)
                val
            )
        ) + "px";
    }

    jQuery.extend( {

        // Add in style property hooks for overriding the default
        // behavior of getting and setting a style property
        cssHooks: {
            opacity: {
                get: function( elem, computed ) {
                    if ( computed ) {

                        // We should always get a number back from opacity
                        var ret = curCSS( elem, "opacity" );
                        return ret === "" ? "1" : ret;
                    }
                }
            }
        },

        // Don't automatically add "px" to these possibly-unitless properties
        cssNumber: {
            "animationIterationCount": true,
            "columnCount": true,
            "fillOpacity": true,
            "flexGrow": true,
            "flexShrink": true,
            "fontWeight": true,
            "gridArea": true,
            "gridColumn": true,
            "gridColumnEnd": true,
            "gridColumnStart": true,
            "gridRow": true,
            "gridRowEnd": true,
            "gridRowStart": true,
            "lineHeight": true,
            "opacity": true,
            "order": true,
            "orphans": true,
            "widows": true,
            "zIndex": true,
            "zoom": true
        },

        // Add in properties whose names you wish to fix before
        // setting or getting the value
        cssProps: {},

        // Get and set the style property on a DOM Node
        style: function( elem, name, value, extra ) {

            // Don't set styles on text and comment nodes
            if ( !elem || elem.nodeType === 3 || elem.nodeType === 8 || !elem.style ) {
                return;
            }

            // Make sure that we're working with the right name
            var ret, type, hooks,
                origName = camelCase( name ),
                isCustomProp = rcustomProp.test( name ),
                style = elem.style;

            // Make sure that we're working with the right name. We don't
            // want to query the value if it is a CSS custom property
            // since they are user-defined.
            if ( !isCustomProp ) {
                name = finalPropName( origName );
            }

            // Gets hook for the prefixed version, then unprefixed version
            hooks = jQuery.cssHooks[ name ] || jQuery.cssHooks[ origName ];

            // Check if we're setting a value
            if ( value !== undefined ) {
                type = typeof value;

                // Convert "+=" or "-=" to relative numbers (#7345)
                if ( type === "string" && ( ret = rcssNum.exec( value ) ) && ret[ 1 ] ) {
                    value = adjustCSS( elem, name, ret );

                    // Fixes bug #9237
                    type = "number";
                }

                // Make sure that null and NaN values aren't set (#7116)
                if ( value == null || value !== value ) {
                    return;
                }

                // If a number was passed in, add the unit (except for certain CSS properties)
                // The isCustomProp check can be removed in jQuery 4.0 when we only auto-append
                // "px" to a few hardcoded values.
                if ( type === "number" && !isCustomProp ) {
                    value += ret && ret[ 3 ] || ( jQuery.cssNumber[ origName ] ? "" : "px" );
                }

                // background-* props affect original clone's values
                if ( !support.clearCloneStyle && value === "" && name.indexOf( "background" ) === 0 ) {
                    style[ name ] = "inherit";
                }

                // If a hook was provided, use that value, otherwise just set the specified value
                if ( !hooks || !( "set" in hooks ) ||
                    ( value = hooks.set( elem, value, extra ) ) !== undefined ) {

                    if ( isCustomProp ) {
                        style.setProperty( name, value );
                    } else {
                        style[ name ] = value;
                    }
                }

            } else {

                // If a hook was provided get the non-computed value from there
                if ( hooks && "get" in hooks &&
                    ( ret = hooks.get( elem, false, extra ) ) !== undefined ) {

                    return ret;
                }

                // Otherwise just get the value from the style object
                return style[ name ];
            }
        },

        css: function( elem, name, extra, styles ) {
            var val, num, hooks,
                origName = camelCase( name ),
                isCustomProp = rcustomProp.test( name );

            // Make sure that we're working with the right name. We don't
            // want to modify the value if it is a CSS custom property
            // since they are user-defined.
            if ( !isCustomProp ) {
                name = finalPropName( origName );
            }

            // Try prefixed name followed by the unprefixed name
            hooks = jQuery.cssHooks[ name ] || jQuery.cssHooks[ origName ];

            // If a hook was provided get the computed value from there
            if ( hooks && "get" in hooks ) {
                val = hooks.get( elem, true, extra );
            }

            // Otherwise, if a way to get the computed value exists, use that
            if ( val === undefined ) {
                val = curCSS( elem, name, styles );
            }

            // Convert "normal" to computed value
            if ( val === "normal" && name in cssNormalTransform ) {
                val = cssNormalTransform[ name ];
            }

            // Make numeric if forced or a qualifier was provided and val looks numeric
            if ( extra === "" || extra ) {
                num = parseFloat( val );
                return extra === true || isFinite( num ) ? num || 0 : val;
            }

            return val;
        }
    } );

    jQuery.each( [ "height", "width" ], function( _i, dimension ) {
        jQuery.cssHooks[ dimension ] = {
            get: function( elem, computed, extra ) {
                if ( computed ) {

                    // Certain elements can have dimension info if we invisibly show them
                    // but it must have a current display style that would benefit
                    return rdisplayswap.test( jQuery.css( elem, "display" ) ) &&

                    // Support: Safari 8+
                    // Table columns in Safari have non-zero offsetWidth & zero
                    // getBoundingClientRect().width unless display is changed.
                    // Support: IE <=11 only
                    // Running getBoundingClientRect on a disconnected node
                    // in IE throws an error.
                    ( !elem.getClientRects().length || !elem.getBoundingClientRect().width ) ?
                        swap( elem, cssShow, function() {
                            return getWidthOrHeight( elem, dimension, extra );
                        } ) :
                        getWidthOrHeight( elem, dimension, extra );
                }
            },

            set: function( elem, value, extra ) {
                var matches,
                    styles = getStyles( elem ),

                    // Only read styles.position if the test has a chance to fail
                    // to avoid forcing a reflow.
                    scrollboxSizeBuggy = !support.scrollboxSize() &&
                        styles.position === "absolute",

                    // To avoid forcing a reflow, only fetch boxSizing if we need it (gh-3991)
                    boxSizingNeeded = scrollboxSizeBuggy || extra,
                    isBorderBox = boxSizingNeeded &&
                        jQuery.css( elem, "boxSizing", false, styles ) === "border-box",
                    subtract = extra ?
                        boxModelAdjustment(
                            elem,
                            dimension,
                            extra,
                            isBorderBox,
                            styles
                        ) :
                        0;

                // Account for unreliable border-box dimensions by comparing offset* to computed and
                // faking a content-box to get border and padding (gh-3699)
                if ( isBorderBox && scrollboxSizeBuggy ) {
                    subtract -= Math.ceil(
                        elem[ "offset" + dimension[ 0 ].toUpperCase() + dimension.slice( 1 ) ] -
                        parseFloat( styles[ dimension ] ) -
                        boxModelAdjustment( elem, dimension, "border", false, styles ) -
                        0.5
                    );
                }

                // Convert to pixels if value adjustment is needed
                if ( subtract && ( matches = rcssNum.exec( value ) ) &&
                    ( matches[ 3 ] || "px" ) !== "px" ) {

                    elem.style[ dimension ] = value;
                    value = jQuery.css( elem, dimension );
                }

                return setPositiveNumber( elem, value, subtract );
            }
        };
    } );

    jQuery.cssHooks.marginLeft = addGetHookIf( support.reliableMarginLeft,
        function( elem, computed ) {
            if ( computed ) {
                return ( parseFloat( curCSS( elem, "marginLeft" ) ) ||
                    elem.getBoundingClientRect().left -
                    swap( elem, { marginLeft: 0 }, function() {
                        return elem.getBoundingClientRect().left;
                    } )
                ) + "px";
            }
        }
    );

// These hooks are used by animate to expand properties
    jQuery.each( {
        margin: "",
        padding: "",
        border: "Width"
    }, function( prefix, suffix ) {
        jQuery.cssHooks[ prefix + suffix ] = {
            expand: function( value ) {
                var i = 0,
                    expanded = {},

                    // Assumes a single number if not a string
                    parts = typeof value === "string" ? value.split( " " ) : [ value ];

                for ( ; i < 4; i++ ) {
                    expanded[ prefix + cssExpand[ i ] + suffix ] =
                        parts[ i ] || parts[ i - 2 ] || parts[ 0 ];
                }

                return expanded;
            }
        };

        if ( prefix !== "margin" ) {
            jQuery.cssHooks[ prefix + suffix ].set = setPositiveNumber;
        }
    } );

    jQuery.fn.extend( {
        css: function( name, value ) {
            return access( this, function( elem, name, value ) {
                var styles, len,
                    map = {},
                    i = 0;

                if ( Array.isArray( name ) ) {
                    styles = getStyles( elem );
                    len = name.length;

                    for ( ; i < len; i++ ) {
                        map[ name[ i ] ] = jQuery.css( elem, name[ i ], false, styles );
                    }

                    return map;
                }

                return value !== undefined ?
                    jQuery.style( elem, name, value ) :
                    jQuery.css( elem, name );
            }, name, value, arguments.length > 1 );
        }
    } );


    function Tween( elem, options, prop, end, easing ) {
        return new Tween.prototype.init( elem, options, prop, end, easing );
    }
    jQuery.Tween = Tween;

    Tween.prototype = {
        constructor: Tween,
        init: function( elem, options, prop, end, easing, unit ) {
            this.elem = elem;
            this.prop = prop;
            this.easing = easing || jQuery.easing._default;
            this.options = options;
            this.start = this.now = this.cur();
            this.end = end;
            this.unit = unit || ( jQuery.cssNumber[ prop ] ? "" : "px" );
        },
        cur: function() {
            var hooks = Tween.propHooks[ this.prop ];

            return hooks && hooks.get ?
                hooks.get( this ) :
                Tween.propHooks._default.get( this );
        },
        run: function( percent ) {
            var eased,
                hooks = Tween.propHooks[ this.prop ];

            if ( this.options.duration ) {
                this.pos = eased = jQuery.easing[ this.easing ](
                    percent, this.options.duration * percent, 0, 1, this.options.duration
                );
            } else {
                this.pos = eased = percent;
            }
            this.now = ( this.end - this.start ) * eased + this.start;

            if ( this.options.step ) {
                this.options.step.call( this.elem, this.now, this );
            }

            if ( hooks && hooks.set ) {
                hooks.set( this );
            } else {
                Tween.propHooks._default.set( this );
            }
            return this;
        }
    };

    Tween.prototype.init.prototype = Tween.prototype;

    Tween.propHooks = {
        _default: {
            get: function( tween ) {
                var result;

                // Use a property on the element directly when it is not a DOM element,
                // or when there is no matching style property that exists.
                if ( tween.elem.nodeType !== 1 ||
                    tween.elem[ tween.prop ] != null && tween.elem.style[ tween.prop ] == null ) {
                    return tween.elem[ tween.prop ];
                }

                // Passing an empty string as a 3rd parameter to .css will automatically
                // attempt a parseFloat and fallback to a string if the parse fails.
                // Simple values such as "10px" are parsed to Float;
                // complex values such as "rotate(1rad)" are returned as-is.
                result = jQuery.css( tween.elem, tween.prop, "" );

                // Empty strings, null, undefined and "auto" are converted to 0.
                return !result || result === "auto" ? 0 : result;
            },
            set: function( tween ) {

                // Use step hook for back compat.
                // Use cssHook if its there.
                // Use .style if available and use plain properties where available.
                if ( jQuery.fx.step[ tween.prop ] ) {
                    jQuery.fx.step[ tween.prop ]( tween );
                } else if ( tween.elem.nodeType === 1 && (
                    jQuery.cssHooks[ tween.prop ] ||
                    tween.elem.style[ finalPropName( tween.prop ) ] != null ) ) {
                    jQuery.style( tween.elem, tween.prop, tween.now + tween.unit );
                } else {
                    tween.elem[ tween.prop ] = tween.now;
                }
            }
        }
    };

// Support: IE <=9 only
// Panic based approach to setting things on disconnected nodes
    Tween.propHooks.scrollTop = Tween.propHooks.scrollLeft = {
        set: function( tween ) {
            if ( tween.elem.nodeType && tween.elem.parentNode ) {
                tween.elem[ tween.prop ] = tween.now;
            }
        }
    };

    jQuery.easing = {
        linear: function( p ) {
            return p;
        },
        swing: function( p ) {
            return 0.5 - Math.cos( p * Math.PI ) / 2;
        },
        _default: "swing"
    };

    jQuery.fx = Tween.prototype.init;

// Back compat <1.8 extension point
    jQuery.fx.step = {};




    var
        fxNow, inProgress,
        rfxtypes = /^(?:toggle|show|hide)$/,
        rrun = /queueHooks$/;

    function schedule() {
        if ( inProgress ) {
            if ( document.hidden === false && window.requestAnimationFrame ) {
                window.requestAnimationFrame( schedule );
            } else {
                window.setTimeout( schedule, jQuery.fx.interval );
            }

            jQuery.fx.tick();
        }
    }

// Animations created synchronously will run synchronously
    function createFxNow() {
        window.setTimeout( function() {
            fxNow = undefined;
        } );
        return ( fxNow = Date.now() );
    }

// Generate parameters to create a standard animation
    function genFx( type, includeWidth ) {
        var which,
            i = 0,
            attrs = { height: type };

        // If we include width, step value is 1 to do all cssExpand values,
        // otherwise step value is 2 to skip over Left and Right
        includeWidth = includeWidth ? 1 : 0;
        for ( ; i < 4; i += 2 - includeWidth ) {
            which = cssExpand[ i ];
            attrs[ "margin" + which ] = attrs[ "padding" + which ] = type;
        }

        if ( includeWidth ) {
            attrs.opacity = attrs.width = type;
        }

        return attrs;
    }

    function createTween( value, prop, animation ) {
        var tween,
            collection = ( Animation.tweeners[ prop ] || [] ).concat( Animation.tweeners[ "*" ] ),
            index = 0,
            length = collection.length;
        for ( ; index < length; index++ ) {
            if ( ( tween = collection[ index ].call( animation, prop, value ) ) ) {

                // We're done with this property
                return tween;
            }
        }
    }

    function defaultPrefilter( elem, props, opts ) {
        var prop, value, toggle, hooks, oldfire, propTween, restoreDisplay, display,
            isBox = "width" in props || "height" in props,
            anim = this,
            orig = {},
            style = elem.style,
            hidden = elem.nodeType && isHiddenWithinTree( elem ),
            dataShow = dataPriv.get( elem, "fxshow" );

        // Queue-skipping animations hijack the fx hooks
        if ( !opts.queue ) {
            hooks = jQuery._queueHooks( elem, "fx" );
            if ( hooks.unqueued == null ) {
                hooks.unqueued = 0;
                oldfire = hooks.empty.fire;
                hooks.empty.fire = function() {
                    if ( !hooks.unqueued ) {
                        oldfire();
                    }
                };
            }
            hooks.unqueued++;

            anim.always( function() {

                // Ensure the complete handler is called before this completes
                anim.always( function() {
                    hooks.unqueued--;
                    if ( !jQuery.queue( elem, "fx" ).length ) {
                        hooks.empty.fire();
                    }
                } );
            } );
        }

        // Detect show/hide animations
        for ( prop in props ) {
            value = props[ prop ];
            if ( rfxtypes.test( value ) ) {
                delete props[ prop ];
                toggle = toggle || value === "toggle";
                if ( value === ( hidden ? "hide" : "show" ) ) {

                    // Pretend to be hidden if this is a "show" and
                    // there is still data from a stopped show/hide
                    if ( value === "show" && dataShow && dataShow[ prop ] !== undefined ) {
                        hidden = true;

                        // Ignore all other no-op show/hide data
                    } else {
                        continue;
                    }
                }
                orig[ prop ] = dataShow && dataShow[ prop ] || jQuery.style( elem, prop );
            }
        }

        // Bail out if this is a no-op like .hide().hide()
        propTween = !jQuery.isEmptyObject( props );
        if ( !propTween && jQuery.isEmptyObject( orig ) ) {
            return;
        }

        // Restrict "overflow" and "display" styles during box animations
        if ( isBox && elem.nodeType === 1 ) {

            // Support: IE <=9 - 11, Edge 12 - 15
            // Record all 3 overflow attributes because IE does not infer the shorthand
            // from identically-valued overflowX and overflowY and Edge just mirrors
            // the overflowX value there.
            opts.overflow = [ style.overflow, style.overflowX, style.overflowY ];

            // Identify a display type, preferring old show/hide data over the CSS cascade
            restoreDisplay = dataShow && dataShow.display;
            if ( restoreDisplay == null ) {
                restoreDisplay = dataPriv.get( elem, "display" );
            }
            display = jQuery.css( elem, "display" );
            if ( display === "none" ) {
                if ( restoreDisplay ) {
                    display = restoreDisplay;
                } else {

                    // Get nonempty value(s) by temporarily forcing visibility
                    showHide( [ elem ], true );
                    restoreDisplay = elem.style.display || restoreDisplay;
                    display = jQuery.css( elem, "display" );
                    showHide( [ elem ] );
                }
            }

            // Animate inline elements as inline-block
            if ( display === "inline" || display === "inline-block" && restoreDisplay != null ) {
                if ( jQuery.css( elem, "float" ) === "none" ) {

                    // Restore the original display value at the end of pure show/hide animations
                    if ( !propTween ) {
                        anim.done( function() {
                            style.display = restoreDisplay;
                        } );
                        if ( restoreDisplay == null ) {
                            display = style.display;
                            restoreDisplay = display === "none" ? "" : display;
                        }
                    }
                    style.display = "inline-block";
                }
            }
        }

        if ( opts.overflow ) {
            style.overflow = "hidden";
            anim.always( function() {
                style.overflow = opts.overflow[ 0 ];
                style.overflowX = opts.overflow[ 1 ];
                style.overflowY = opts.overflow[ 2 ];
            } );
        }

        // Implement show/hide animations
        propTween = false;
        for ( prop in orig ) {

            // General show/hide setup for this element animation
            if ( !propTween ) {
                if ( dataShow ) {
                    if ( "hidden" in dataShow ) {
                        hidden = dataShow.hidden;
                    }
                } else {
                    dataShow = dataPriv.access( elem, "fxshow", { display: restoreDisplay } );
                }

                // Store hidden/visible for toggle so `.stop().toggle()` "reverses"
                if ( toggle ) {
                    dataShow.hidden = !hidden;
                }

                // Show elements before animating them
                if ( hidden ) {
                    showHide( [ elem ], true );
                }

                /* eslint-disable no-loop-func */

                anim.done( function() {

                    /* eslint-enable no-loop-func */

                    // The final step of a "hide" animation is actually hiding the element
                    if ( !hidden ) {
                        showHide( [ elem ] );
                    }
                    dataPriv.remove( elem, "fxshow" );
                    for ( prop in orig ) {
                        jQuery.style( elem, prop, orig[ prop ] );
                    }
                } );
            }

            // Per-property setup
            propTween = createTween( hidden ? dataShow[ prop ] : 0, prop, anim );
            if ( !( prop in dataShow ) ) {
                dataShow[ prop ] = propTween.start;
                if ( hidden ) {
                    propTween.end = propTween.start;
                    propTween.start = 0;
                }
            }
        }
    }

    function propFilter( props, specialEasing ) {
        var index, name, easing, value, hooks;

        // camelCase, specialEasing and expand cssHook pass
        for ( index in props ) {
            name = camelCase( index );
            easing = specialEasing[ name ];
            value = props[ index ];
            if ( Array.isArray( value ) ) {
                easing = value[ 1 ];
                value = props[ index ] = value[ 0 ];
            }

            if ( index !== name ) {
                props[ name ] = value;
                delete props[ index ];
            }

            hooks = jQuery.cssHooks[ name ];
            if ( hooks && "expand" in hooks ) {
                value = hooks.expand( value );
                delete props[ name ];

                // Not quite $.extend, this won't overwrite existing keys.
                // Reusing 'index' because we have the correct "name"
                for ( index in value ) {
                    if ( !( index in props ) ) {
                        props[ index ] = value[ index ];
                        specialEasing[ index ] = easing;
                    }
                }
            } else {
                specialEasing[ name ] = easing;
            }
        }
    }

    function Animation( elem, properties, options ) {
        var result,
            stopped,
            index = 0,
            length = Animation.prefilters.length,
            deferred = jQuery.Deferred().always( function() {

                // Don't match elem in the :animated selector
                delete tick.elem;
            } ),
            tick = function() {
                if ( stopped ) {
                    return false;
                }
                var currentTime = fxNow || createFxNow(),
                    remaining = Math.max( 0, animation.startTime + animation.duration - currentTime ),

                    // Support: Android 2.3 only
                    // Archaic crash bug won't allow us to use `1 - ( 0.5 || 0 )` (#12497)
                    temp = remaining / animation.duration || 0,
                    percent = 1 - temp,
                    index = 0,
                    length = animation.tweens.length;

                for ( ; index < length; index++ ) {
                    animation.tweens[ index ].run( percent );
                }

                deferred.notifyWith( elem, [ animation, percent, remaining ] );

                // If there's more to do, yield
                if ( percent < 1 && length ) {
                    return remaining;
                }

                // If this was an empty animation, synthesize a final progress notification
                if ( !length ) {
                    deferred.notifyWith( elem, [ animation, 1, 0 ] );
                }

                // Resolve the animation and report its conclusion
                deferred.resolveWith( elem, [ animation ] );
                return false;
            },
            animation = deferred.promise( {
                elem: elem,
                props: jQuery.extend( {}, properties ),
                opts: jQuery.extend( true, {
                    specialEasing: {},
                    easing: jQuery.easing._default
                }, options ),
                originalProperties: properties,
                originalOptions: options,
                startTime: fxNow || createFxNow(),
                duration: options.duration,
                tweens: [],
                createTween: function( prop, end ) {
                    var tween = jQuery.Tween( elem, animation.opts, prop, end,
                        animation.opts.specialEasing[ prop ] || animation.opts.easing );
                    animation.tweens.push( tween );
                    return tween;
                },
                stop: function( gotoEnd ) {
                    var index = 0,

                        // If we are going to the end, we want to run all the tweens
                        // otherwise we skip this part
                        length = gotoEnd ? animation.tweens.length : 0;
                    if ( stopped ) {
                        return this;
                    }
                    stopped = true;
                    for ( ; index < length; index++ ) {
                        animation.tweens[ index ].run( 1 );
                    }

                    // Resolve when we played the last frame; otherwise, reject
                    if ( gotoEnd ) {
                        deferred.notifyWith( elem, [ animation, 1, 0 ] );
                        deferred.resolveWith( elem, [ animation, gotoEnd ] );
                    } else {
                        deferred.rejectWith( elem, [ animation, gotoEnd ] );
                    }
                    return this;
                }
            } ),
            props = animation.props;

        propFilter( props, animation.opts.specialEasing );

        for ( ; index < length; index++ ) {
            result = Animation.prefilters[ index ].call( animation, elem, props, animation.opts );
            if ( result ) {
                if ( isFunction( result.stop ) ) {
                    jQuery._queueHooks( animation.elem, animation.opts.queue ).stop =
                        result.stop.bind( result );
                }
                return result;
            }
        }

        jQuery.map( props, createTween, animation );

        if ( isFunction( animation.opts.start ) ) {
            animation.opts.start.call( elem, animation );
        }

        // Attach callbacks from options
        animation
            .progress( animation.opts.progress )
            .done( animation.opts.done, animation.opts.complete )
            .fail( animation.opts.fail )
            .always( animation.opts.always );

        jQuery.fx.timer(
            jQuery.extend( tick, {
                elem: elem,
                anim: animation,
                queue: animation.opts.queue
            } )
        );

        return animation;
    }

    jQuery.Animation = jQuery.extend( Animation, {

        tweeners: {
            "*": [ function( prop, value ) {
                var tween = this.createTween( prop, value );
                adjustCSS( tween.elem, prop, rcssNum.exec( value ), tween );
                return tween;
            } ]
        },

        tweener: function( props, callback ) {
            if ( isFunction( props ) ) {
                callback = props;
                props = [ "*" ];
            } else {
                props = props.match( rnothtmlwhite );
            }

            var prop,
                index = 0,
                length = props.length;

            for ( ; index < length; index++ ) {
                prop = props[ index ];
                Animation.tweeners[ prop ] = Animation.tweeners[ prop ] || [];
                Animation.tweeners[ prop ].unshift( callback );
            }
        },

        prefilters: [ defaultPrefilter ],

        prefilter: function( callback, prepend ) {
            if ( prepend ) {
                Animation.prefilters.unshift( callback );
            } else {
                Animation.prefilters.push( callback );
            }
        }
    } );

    jQuery.speed = function( speed, easing, fn ) {
        var opt = speed && typeof speed === "object" ? jQuery.extend( {}, speed ) : {
            complete: fn || !fn && easing ||
                isFunction( speed ) && speed,
            duration: speed,
            easing: fn && easing || easing && !isFunction( easing ) && easing
        };

        // Go to the end state if fx are off
        if ( jQuery.fx.off ) {
            opt.duration = 0;

        } else {
            if ( typeof opt.duration !== "number" ) {
                if ( opt.duration in jQuery.fx.speeds ) {
                    opt.duration = jQuery.fx.speeds[ opt.duration ];

                } else {
                    opt.duration = jQuery.fx.speeds._default;
                }
            }
        }

        // Normalize opt.queue - true/undefined/null -> "fx"
        if ( opt.queue == null || opt.queue === true ) {
            opt.queue = "fx";
        }

        // Queueing
        opt.old = opt.complete;

        opt.complete = function() {
            if ( isFunction( opt.old ) ) {
                opt.old.call( this );
            }

            if ( opt.queue ) {
                jQuery.dequeue( this, opt.queue );
            }
        };

        return opt;
    };

    jQuery.fn.extend( {
        fadeTo: function( speed, to, easing, callback ) {

            // Show any hidden elements after setting opacity to 0
            return this.filter( isHiddenWithinTree ).css( "opacity", 0 ).show()

                // Animate to the value specified
                .end().animate( { opacity: to }, speed, easing, callback );
        },
        animate: function( prop, speed, easing, callback ) {
            var empty = jQuery.isEmptyObject( prop ),
                optall = jQuery.speed( speed, easing, callback ),
                doAnimation = function() {

                    // Operate on a copy of prop so per-property easing won't be lost
                    var anim = Animation( this, jQuery.extend( {}, prop ), optall );

                    // Empty animations, or finishing resolves immediately
                    if ( empty || dataPriv.get( this, "finish" ) ) {
                        anim.stop( true );
                    }
                };
            doAnimation.finish = doAnimation;

            return empty || optall.queue === false ?
                this.each( doAnimation ) :
                this.queue( optall.queue, doAnimation );
        },
        stop: function( type, clearQueue, gotoEnd ) {
            var stopQueue = function( hooks ) {
                var stop = hooks.stop;
                delete hooks.stop;
                stop( gotoEnd );
            };

            if ( typeof type !== "string" ) {
                gotoEnd = clearQueue;
                clearQueue = type;
                type = undefined;
            }
            if ( clearQueue ) {
                this.queue( type || "fx", [] );
            }

            return this.each( function() {
                var dequeue = true,
                    index = type != null && type + "queueHooks",
                    timers = jQuery.timers,
                    data = dataPriv.get( this );

                if ( index ) {
                    if ( data[ index ] && data[ index ].stop ) {
                        stopQueue( data[ index ] );
                    }
                } else {
                    for ( index in data ) {
                        if ( data[ index ] && data[ index ].stop && rrun.test( index ) ) {
                            stopQueue( data[ index ] );
                        }
                    }
                }

                for ( index = timers.length; index--; ) {
                    if ( timers[ index ].elem === this &&
                        ( type == null || timers[ index ].queue === type ) ) {

                        timers[ index ].anim.stop( gotoEnd );
                        dequeue = false;
                        timers.splice( index, 1 );
                    }
                }

                // Start the next in the queue if the last step wasn't forced.
                // Timers currently will call their complete callbacks, which
                // will dequeue but only if they were gotoEnd.
                if ( dequeue || !gotoEnd ) {
                    jQuery.dequeue( this, type );
                }
            } );
        },
        finish: function( type ) {
            if ( type !== false ) {
                type = type || "fx";
            }
            return this.each( function() {
                var index,
                    data = dataPriv.get( this ),
                    queue = data[ type + "queue" ],
                    hooks = data[ type + "queueHooks" ],
                    timers = jQuery.timers,
                    length = queue ? queue.length : 0;

                // Enable finishing flag on private data
                data.finish = true;

                // Empty the queue first
                jQuery.queue( this, type, [] );

                if ( hooks && hooks.stop ) {
                    hooks.stop.call( this, true );
                }

                // Look for any active animations, and finish them
                for ( index = timers.length; index--; ) {
                    if ( timers[ index ].elem === this && timers[ index ].queue === type ) {
                        timers[ index ].anim.stop( true );
                        timers.splice( index, 1 );
                    }
                }

                // Look for any animations in the old queue and finish them
                for ( index = 0; index < length; index++ ) {
                    if ( queue[ index ] && queue[ index ].finish ) {
                        queue[ index ].finish.call( this );
                    }
                }

                // Turn off finishing flag
                delete data.finish;
            } );
        }
    } );

    jQuery.each( [ "toggle", "show", "hide" ], function( _i, name ) {
        var cssFn = jQuery.fn[ name ];
        jQuery.fn[ name ] = function( speed, easing, callback ) {
            return speed == null || typeof speed === "boolean" ?
                cssFn.apply( this, arguments ) :
                this.animate( genFx( name, true ), speed, easing, callback );
        };
    } );

// Generate shortcuts for custom animations
    jQuery.each( {
        slideDown: genFx( "show" ),
        slideUp: genFx( "hide" ),
        slideToggle: genFx( "toggle" ),
        fadeIn: { opacity: "show" },
        fadeOut: { opacity: "hide" },
        fadeToggle: { opacity: "toggle" }
    }, function( name, props ) {
        jQuery.fn[ name ] = function( speed, easing, callback ) {
            return this.animate( props, speed, easing, callback );
        };
    } );

    jQuery.timers = [];
    jQuery.fx.tick = function() {
        var timer,
            i = 0,
            timers = jQuery.timers;

        fxNow = Date.now();

        for ( ; i < timers.length; i++ ) {
            timer = timers[ i ];

            // Run the timer and safely remove it when done (allowing for external removal)
            if ( !timer() && timers[ i ] === timer ) {
                timers.splice( i--, 1 );
            }
        }

        if ( !timers.length ) {
            jQuery.fx.stop();
        }
        fxNow = undefined;
    };

    jQuery.fx.timer = function( timer ) {
        jQuery.timers.push( timer );
        jQuery.fx.start();
    };

    jQuery.fx.interval = 13;
    jQuery.fx.start = function() {
        if ( inProgress ) {
            return;
        }

        inProgress = true;
        schedule();
    };

    jQuery.fx.stop = function() {
        inProgress = null;
    };

    jQuery.fx.speeds = {
        slow: 600,
        fast: 200,

        // Default speed
        _default: 400
    };


// Based off of the plugin by Clint Helfers, with permission.
// https://web.archive.org/web/20100324014747/http://blindsignals.com/index.php/2009/07/jquery-delay/
    jQuery.fn.delay = function( time, type ) {
        time = jQuery.fx ? jQuery.fx.speeds[ time ] || time : time;
        type = type || "fx";

        return this.queue( type, function( next, hooks ) {
            var timeout = window.setTimeout( next, time );
            hooks.stop = function() {
                window.clearTimeout( timeout );
            };
        } );
    };


    ( function() {
        var input = document.createElement( "input" ),
            select = document.createElement( "select" ),
            opt = select.appendChild( document.createElement( "option" ) );

        input.type = "checkbox";

        // Support: Android <=4.3 only
        // Default value for a checkbox should be "on"
        support.checkOn = input.value !== "";

        // Support: IE <=11 only
        // Must access selectedIndex to make default options select
        support.optSelected = opt.selected;

        // Support: IE <=11 only
        // An input loses its value after becoming a radio
        input = document.createElement( "input" );
        input.value = "t";
        input.type = "radio";
        support.radioValue = input.value === "t";
    } )();


    var boolHook,
        attrHandle = jQuery.expr.attrHandle;

    jQuery.fn.extend( {
        attr: function( name, value ) {
            return access( this, jQuery.attr, name, value, arguments.length > 1 );
        },

        removeAttr: function( name ) {
            return this.each( function() {
                jQuery.removeAttr( this, name );
            } );
        }
    } );

    jQuery.extend( {
        attr: function( elem, name, value ) {
            var ret, hooks,
                nType = elem.nodeType;

            // Don't get/set attributes on text, comment and attribute nodes
            if ( nType === 3 || nType === 8 || nType === 2 ) {
                return;
            }

            // Fallback to prop when attributes are not supported
            if ( typeof elem.getAttribute === "undefined" ) {
                return jQuery.prop( elem, name, value );
            }

            // Attribute hooks are determined by the lowercase version
            // Grab necessary hook if one is defined
            if ( nType !== 1 || !jQuery.isXMLDoc( elem ) ) {
                hooks = jQuery.attrHooks[ name.toLowerCase() ] ||
                    ( jQuery.expr.match.bool.test( name ) ? boolHook : undefined );
            }

            if ( value !== undefined ) {
                if ( value === null ) {
                    jQuery.removeAttr( elem, name );
                    return;
                }

                if ( hooks && "set" in hooks &&
                    ( ret = hooks.set( elem, value, name ) ) !== undefined ) {
                    return ret;
                }

                elem.setAttribute( name, value + "" );
                return value;
            }

            if ( hooks && "get" in hooks && ( ret = hooks.get( elem, name ) ) !== null ) {
                return ret;
            }

            ret = jQuery.find.attr( elem, name );

            // Non-existent attributes return null, we normalize to undefined
            return ret == null ? undefined : ret;
        },

        attrHooks: {
            type: {
                set: function( elem, value ) {
                    if ( !support.radioValue && value === "radio" &&
                        nodeName( elem, "input" ) ) {
                        var val = elem.value;
                        elem.setAttribute( "type", value );
                        if ( val ) {
                            elem.value = val;
                        }
                        return value;
                    }
                }
            }
        },

        removeAttr: function( elem, value ) {
            var name,
                i = 0,

                // Attribute names can contain non-HTML whitespace characters
                // https://html.spec.whatwg.org/multipage/syntax.html#attributes-2
                attrNames = value && value.match( rnothtmlwhite );

            if ( attrNames && elem.nodeType === 1 ) {
                while ( ( name = attrNames[ i++ ] ) ) {
                    elem.removeAttribute( name );
                }
            }
        }
    } );

// Hooks for boolean attributes
    boolHook = {
        set: function( elem, value, name ) {
            if ( value === false ) {

                // Remove boolean attributes when set to false
                jQuery.removeAttr( elem, name );
            } else {
                elem.setAttribute( name, name );
            }
            return name;
        }
    };

    jQuery.each( jQuery.expr.match.bool.source.match( /\w+/g ), function( _i, name ) {
        var getter = attrHandle[ name ] || jQuery.find.attr;

        attrHandle[ name ] = function( elem, name, isXML ) {
            var ret, handle,
                lowercaseName = name.toLowerCase();

            if ( !isXML ) {

                // Avoid an infinite loop by temporarily removing this function from the getter
                handle = attrHandle[ lowercaseName ];
                attrHandle[ lowercaseName ] = ret;
                ret = getter( elem, name, isXML ) != null ?
                    lowercaseName :
                    null;
                attrHandle[ lowercaseName ] = handle;
            }
            return ret;
        };
    } );




    var rfocusable = /^(?:input|select|textarea|button)$/i,
        rclickable = /^(?:a|area)$/i;

    jQuery.fn.extend( {
        prop: function( name, value ) {
            return access( this, jQuery.prop, name, value, arguments.length > 1 );
        },

        removeProp: function( name ) {
            return this.each( function() {
                delete this[ jQuery.propFix[ name ] || name ];
            } );
        }
    } );

    jQuery.extend( {
        prop: function( elem, name, value ) {
            var ret, hooks,
                nType = elem.nodeType;

            // Don't get/set properties on text, comment and attribute nodes
            if ( nType === 3 || nType === 8 || nType === 2 ) {
                return;
            }

            if ( nType !== 1 || !jQuery.isXMLDoc( elem ) ) {

                // Fix name and attach hooks
                name = jQuery.propFix[ name ] || name;
                hooks = jQuery.propHooks[ name ];
            }

            if ( value !== undefined ) {
                if ( hooks && "set" in hooks &&
                    ( ret = hooks.set( elem, value, name ) ) !== undefined ) {
                    return ret;
                }

                return ( elem[ name ] = value );
            }

            if ( hooks && "get" in hooks && ( ret = hooks.get( elem, name ) ) !== null ) {
                return ret;
            }

            return elem[ name ];
        },

        propHooks: {
            tabIndex: {
                get: function( elem ) {

                    // Support: IE <=9 - 11 only
                    // elem.tabIndex doesn't always return the
                    // correct value when it hasn't been explicitly set
                    // https://web.archive.org/web/20141116233347/http://fluidproject.org/blog/2008/01/09/getting-setting-and-removing-tabindex-values-with-javascript/
                    // Use proper attribute retrieval(#12072)
                    var tabindex = jQuery.find.attr( elem, "tabindex" );

                    if ( tabindex ) {
                        return parseInt( tabindex, 10 );
                    }

                    if (
                        rfocusable.test( elem.nodeName ) ||
                        rclickable.test( elem.nodeName ) &&
                        elem.href
                    ) {
                        return 0;
                    }

                    return -1;
                }
            }
        },

        propFix: {
            "for": "htmlFor",
            "class": "className"
        }
    } );

// Support: IE <=11 only
// Accessing the selectedIndex property
// forces the browser to respect setting selected
// on the option
// The getter ensures a default option is selected
// when in an optgroup
// eslint rule "no-unused-expressions" is disabled for this code
// since it considers such accessions noop
    if ( !support.optSelected ) {
        jQuery.propHooks.selected = {
            get: function( elem ) {

                /* eslint no-unused-expressions: "off" */

                var parent = elem.parentNode;
                if ( parent && parent.parentNode ) {
                    parent.parentNode.selectedIndex;
                }
                return null;
            },
            set: function( elem ) {

                /* eslint no-unused-expressions: "off" */

                var parent = elem.parentNode;
                if ( parent ) {
                    parent.selectedIndex;

                    if ( parent.parentNode ) {
                        parent.parentNode.selectedIndex;
                    }
                }
            }
        };
    }

    jQuery.each( [
        "tabIndex",
        "readOnly",
        "maxLength",
        "cellSpacing",
        "cellPadding",
        "rowSpan",
        "colSpan",
        "useMap",
        "frameBorder",
        "contentEditable"
    ], function() {
        jQuery.propFix[ this.toLowerCase() ] = this;
    } );




    // Strip and collapse whitespace according to HTML spec
    // https://infra.spec.whatwg.org/#strip-and-collapse-ascii-whitespace
    function stripAndCollapse( value ) {
        var tokens = value.match( rnothtmlwhite ) || [];
        return tokens.join( " " );
    }


    function getClass( elem ) {
        return elem.getAttribute && elem.getAttribute( "class" ) || "";
    }

    function classesToArray( value ) {
        if ( Array.isArray( value ) ) {
            return value;
        }
        if ( typeof value === "string" ) {
            return value.match( rnothtmlwhite ) || [];
        }
        return [];
    }

    jQuery.fn.extend( {
        addClass: function( value ) {
            var classes, elem, cur, curValue, clazz, j, finalValue,
                i = 0;

            if ( isFunction( value ) ) {
                return this.each( function( j ) {
                    jQuery( this ).addClass( value.call( this, j, getClass( this ) ) );
                } );
            }

            classes = classesToArray( value );

            if ( classes.length ) {
                while ( ( elem = this[ i++ ] ) ) {
                    curValue = getClass( elem );
                    cur = elem.nodeType === 1 && ( " " + stripAndCollapse( curValue ) + " " );

                    if ( cur ) {
                        j = 0;
                        while ( ( clazz = classes[ j++ ] ) ) {
                            if ( cur.indexOf( " " + clazz + " " ) < 0 ) {
                                cur += clazz + " ";
                            }
                        }

                        // Only assign if different to avoid unneeded rendering.
                        finalValue = stripAndCollapse( cur );
                        if ( curValue !== finalValue ) {
                            elem.setAttribute( "class", finalValue );
                        }
                    }
                }
            }

            return this;
        },

        removeClass: function( value ) {
            var classes, elem, cur, curValue, clazz, j, finalValue,
                i = 0;

            if ( isFunction( value ) ) {
                return this.each( function( j ) {
                    jQuery( this ).removeClass( value.call( this, j, getClass( this ) ) );
                } );
            }

            if ( !arguments.length ) {
                return this.attr( "class", "" );
            }

            classes = classesToArray( value );

            if ( classes.length ) {
                while ( ( elem = this[ i++ ] ) ) {
                    curValue = getClass( elem );

                    // This expression is here for better compressibility (see addClass)
                    cur = elem.nodeType === 1 && ( " " + stripAndCollapse( curValue ) + " " );

                    if ( cur ) {
                        j = 0;
                        while ( ( clazz = classes[ j++ ] ) ) {

                            // Remove *all* instances
                            while ( cur.indexOf( " " + clazz + " " ) > -1 ) {
                                cur = cur.replace( " " + clazz + " ", " " );
                            }
                        }

                        // Only assign if different to avoid unneeded rendering.
                        finalValue = stripAndCollapse( cur );
                        if ( curValue !== finalValue ) {
                            elem.setAttribute( "class", finalValue );
                        }
                    }
                }
            }

            return this;
        },

        toggleClass: function( value, stateVal ) {
            var type = typeof value,
                isValidValue = type === "string" || Array.isArray( value );

            if ( typeof stateVal === "boolean" && isValidValue ) {
                return stateVal ? this.addClass( value ) : this.removeClass( value );
            }

            if ( isFunction( value ) ) {
                return this.each( function( i ) {
                    jQuery( this ).toggleClass(
                        value.call( this, i, getClass( this ), stateVal ),
                        stateVal
                    );
                } );
            }

            return this.each( function() {
                var className, i, self, classNames;

                if ( isValidValue ) {

                    // Toggle individual class names
                    i = 0;
                    self = jQuery( this );
                    classNames = classesToArray( value );

                    while ( ( className = classNames[ i++ ] ) ) {

                        // Check each className given, space separated list
                        if ( self.hasClass( className ) ) {
                            self.removeClass( className );
                        } else {
                            self.addClass( className );
                        }
                    }

                    // Toggle whole class name
                } else if ( value === undefined || type === "boolean" ) {
                    className = getClass( this );
                    if ( className ) {

                        // Store className if set
                        dataPriv.set( this, "__className__", className );
                    }

                    // If the element has a class name or if we're passed `false`,
                    // then remove the whole classname (if there was one, the above saved it).
                    // Otherwise bring back whatever was previously saved (if anything),
                    // falling back to the empty string if nothing was stored.
                    if ( this.setAttribute ) {
                        this.setAttribute( "class",
                            className || value === false ?
                                "" :
                                dataPriv.get( this, "__className__" ) || ""
                        );
                    }
                }
            } );
        },

        hasClass: function( selector ) {
            var className, elem,
                i = 0;

            className = " " + selector + " ";
            while ( ( elem = this[ i++ ] ) ) {
                if ( elem.nodeType === 1 &&
                    ( " " + stripAndCollapse( getClass( elem ) ) + " " ).indexOf( className ) > -1 ) {
                    return true;
                }
            }

            return false;
        }
    } );




    var rreturn = /\r/g;

    jQuery.fn.extend( {
        val: function( value ) {
            var hooks, ret, valueIsFunction,
                elem = this[ 0 ];

            if ( !arguments.length ) {
                if ( elem ) {
                    hooks = jQuery.valHooks[ elem.type ] ||
                        jQuery.valHooks[ elem.nodeName.toLowerCase() ];

                    if ( hooks &&
                        "get" in hooks &&
                        ( ret = hooks.get( elem, "value" ) ) !== undefined
                    ) {
                        return ret;
                    }

                    ret = elem.value;

                    // Handle most common string cases
                    if ( typeof ret === "string" ) {
                        return ret.replace( rreturn, "" );
                    }

                    // Handle cases where value is null/undef or number
                    return ret == null ? "" : ret;
                }

                return;
            }

            valueIsFunction = isFunction( value );

            return this.each( function( i ) {
                var val;

                if ( this.nodeType !== 1 ) {
                    return;
                }

                if ( valueIsFunction ) {
                    val = value.call( this, i, jQuery( this ).val() );
                } else {
                    val = value;
                }

                // Treat null/undefined as ""; convert numbers to string
                if ( val == null ) {
                    val = "";

                } else if ( typeof val === "number" ) {
                    val += "";

                } else if ( Array.isArray( val ) ) {
                    val = jQuery.map( val, function( value ) {
                        return value == null ? "" : value + "";
                    } );
                }

                hooks = jQuery.valHooks[ this.type ] || jQuery.valHooks[ this.nodeName.toLowerCase() ];

                // If set returns undefined, fall back to normal setting
                if ( !hooks || !( "set" in hooks ) || hooks.set( this, val, "value" ) === undefined ) {
                    this.value = val;
                }
            } );
        }
    } );

    jQuery.extend( {
        valHooks: {
            option: {
                get: function( elem ) {

                    var val = jQuery.find.attr( elem, "value" );
                    return val != null ?
                        val :

                        // Support: IE <=10 - 11 only
                        // option.text throws exceptions (#14686, #14858)
                        // Strip and collapse whitespace
                        // https://html.spec.whatwg.org/#strip-and-collapse-whitespace
                        stripAndCollapse( jQuery.text( elem ) );
                }
            },
            select: {
                get: function( elem ) {
                    var value, option, i,
                        options = elem.options,
                        index = elem.selectedIndex,
                        one = elem.type === "select-one",
                        values = one ? null : [],
                        max = one ? index + 1 : options.length;

                    if ( index < 0 ) {
                        i = max;

                    } else {
                        i = one ? index : 0;
                    }

                    // Loop through all the selected options
                    for ( ; i < max; i++ ) {
                        option = options[ i ];

                        // Support: IE <=9 only
                        // IE8-9 doesn't update selected after form reset (#2551)
                        if ( ( option.selected || i === index ) &&

                            // Don't return options that are disabled or in a disabled optgroup
                            !option.disabled &&
                            ( !option.parentNode.disabled ||
                                !nodeName( option.parentNode, "optgroup" ) ) ) {

                            // Get the specific value for the option
                            value = jQuery( option ).val();

                            // We don't need an array for one selects
                            if ( one ) {
                                return value;
                            }

                            // Multi-Selects return an array
                            values.push( value );
                        }
                    }

                    return values;
                },

                set: function( elem, value ) {
                    var optionSet, option,
                        options = elem.options,
                        values = jQuery.makeArray( value ),
                        i = options.length;

                    while ( i-- ) {
                        option = options[ i ];

                        /* eslint-disable no-cond-assign */

                        if ( option.selected =
                            jQuery.inArray( jQuery.valHooks.option.get( option ), values ) > -1
                        ) {
                            optionSet = true;
                        }

                        /* eslint-enable no-cond-assign */
                    }

                    // Force browsers to behave consistently when non-matching value is set
                    if ( !optionSet ) {
                        elem.selectedIndex = -1;
                    }
                    return values;
                }
            }
        }
    } );

// Radios and checkboxes getter/setter
    jQuery.each( [ "radio", "checkbox" ], function() {
        jQuery.valHooks[ this ] = {
            set: function( elem, value ) {
                if ( Array.isArray( value ) ) {
                    return ( elem.checked = jQuery.inArray( jQuery( elem ).val(), value ) > -1 );
                }
            }
        };
        if ( !support.checkOn ) {
            jQuery.valHooks[ this ].get = function( elem ) {
                return elem.getAttribute( "value" ) === null ? "on" : elem.value;
            };
        }
    } );




// Return jQuery for attributes-only inclusion


    support.focusin = "onfocusin" in window;


    var rfocusMorph = /^(?:focusinfocus|focusoutblur)$/,
        stopPropagationCallback = function( e ) {
            e.stopPropagation();
        };

    jQuery.extend( jQuery.event, {

        trigger: function( event, data, elem, onlyHandlers ) {

            var i, cur, tmp, bubbleType, ontype, handle, special, lastElement,
                eventPath = [ elem || document ],
                type = hasOwn.call( event, "type" ) ? event.type : event,
                namespaces = hasOwn.call( event, "namespace" ) ? event.namespace.split( "." ) : [];

            cur = lastElement = tmp = elem = elem || document;

            // Don't do events on text and comment nodes
            if ( elem.nodeType === 3 || elem.nodeType === 8 ) {
                return;
            }

            // focus/blur morphs to focusin/out; ensure we're not firing them right now
            if ( rfocusMorph.test( type + jQuery.event.triggered ) ) {
                return;
            }

            if ( type.indexOf( "." ) > -1 ) {

                // Namespaced trigger; create a regexp to match event type in handle()
                namespaces = type.split( "." );
                type = namespaces.shift();
                namespaces.sort();
            }
            ontype = type.indexOf( ":" ) < 0 && "on" + type;

            // Caller can pass in a jQuery.Event object, Object, or just an event type string
            event = event[ jQuery.expando ] ?
                event :
                new jQuery.Event( type, typeof event === "object" && event );

            // Trigger bitmask: & 1 for native handlers; & 2 for jQuery (always true)
            event.isTrigger = onlyHandlers ? 2 : 3;
            event.namespace = namespaces.join( "." );
            event.rnamespace = event.namespace ?
                new RegExp( "(^|\\.)" + namespaces.join( "\\.(?:.*\\.|)" ) + "(\\.|$)" ) :
                null;

            // Clean up the event in case it is being reused
            event.result = undefined;
            if ( !event.target ) {
                event.target = elem;
            }

            // Clone any incoming data and prepend the event, creating the handler arg list
            data = data == null ?
                [ event ] :
                jQuery.makeArray( data, [ event ] );

            // Allow special events to draw outside the lines
            special = jQuery.event.special[ type ] || {};
            if ( !onlyHandlers && special.trigger && special.trigger.apply( elem, data ) === false ) {
                return;
            }

            // Determine event propagation path in advance, per W3C events spec (#9951)
            // Bubble up to document, then to window; watch for a global ownerDocument var (#9724)
            if ( !onlyHandlers && !special.noBubble && !isWindow( elem ) ) {

                bubbleType = special.delegateType || type;
                if ( !rfocusMorph.test( bubbleType + type ) ) {
                    cur = cur.parentNode;
                }
                for ( ; cur; cur = cur.parentNode ) {
                    eventPath.push( cur );
                    tmp = cur;
                }

                // Only add window if we got to document (e.g., not plain obj or detached DOM)
                if ( tmp === ( elem.ownerDocument || document ) ) {
                    eventPath.push( tmp.defaultView || tmp.parentWindow || window );
                }
            }

            // Fire handlers on the event path
            i = 0;
            while ( ( cur = eventPath[ i++ ] ) && !event.isPropagationStopped() ) {
                lastElement = cur;
                event.type = i > 1 ?
                    bubbleType :
                    special.bindType || type;

                // jQuery handler
                handle = (
                        dataPriv.get( cur, "events" ) || Object.create( null )
                    )[ event.type ] &&
                    dataPriv.get( cur, "handle" );
                if ( handle ) {
                    handle.apply( cur, data );
                }

                // Native handler
                handle = ontype && cur[ ontype ];
                if ( handle && handle.apply && acceptData( cur ) ) {
                    event.result = handle.apply( cur, data );
                    if ( event.result === false ) {
                        event.preventDefault();
                    }
                }
            }
            event.type = type;

            // If nobody prevented the default action, do it now
            if ( !onlyHandlers && !event.isDefaultPrevented() ) {

                if ( ( !special._default ||
                    special._default.apply( eventPath.pop(), data ) === false ) &&
                    acceptData( elem ) ) {

                    // Call a native DOM method on the target with the same name as the event.
                    // Don't do default actions on window, that's where global variables be (#6170)
                    if ( ontype && isFunction( elem[ type ] ) && !isWindow( elem ) ) {

                        // Don't re-trigger an onFOO event when we call its FOO() method
                        tmp = elem[ ontype ];

                        if ( tmp ) {
                            elem[ ontype ] = null;
                        }

                        // Prevent re-triggering of the same event, since we already bubbled it above
                        jQuery.event.triggered = type;

                        if ( event.isPropagationStopped() ) {
                            lastElement.addEventListener( type, stopPropagationCallback );
                        }

                        elem[ type ]();

                        if ( event.isPropagationStopped() ) {
                            lastElement.removeEventListener( type, stopPropagationCallback );
                        }

                        jQuery.event.triggered = undefined;

                        if ( tmp ) {
                            elem[ ontype ] = tmp;
                        }
                    }
                }
            }

            return event.result;
        },

        // Piggyback on a donor event to simulate a different one
        // Used only for `focus(in | out)` events
        simulate: function( type, elem, event ) {
            var e = jQuery.extend(
                new jQuery.Event(),
                event,
                {
                    type: type,
                    isSimulated: true
                }
            );

            jQuery.event.trigger( e, null, elem );
        }

    } );

    jQuery.fn.extend( {

        trigger: function( type, data ) {
            return this.each( function() {
                jQuery.event.trigger( type, data, this );
            } );
        },
        triggerHandler: function( type, data ) {
            var elem = this[ 0 ];
            if ( elem ) {
                return jQuery.event.trigger( type, data, elem, true );
            }
        }
    } );


// Support: Firefox <=44
// Firefox doesn't have focus(in | out) events
// Related ticket - https://bugzilla.mozilla.org/show_bug.cgi?id=687787
//
// Support: Chrome <=48 - 49, Safari <=9.0 - 9.1
// focus(in | out) events fire after focus & blur events,
// which is spec violation - http://www.w3.org/TR/DOM-Level-3-Events/#events-focusevent-event-order
// Related ticket - https://bugs.chromium.org/p/chromium/issues/detail?id=449857
    if ( !support.focusin ) {
        jQuery.each( { focus: "focusin", blur: "focusout" }, function( orig, fix ) {

            // Attach a single capturing handler on the document while someone wants focusin/focusout
            var handler = function( event ) {
                jQuery.event.simulate( fix, event.target, jQuery.event.fix( event ) );
            };

            jQuery.event.special[ fix ] = {
                setup: function() {

                    // Handle: regular nodes (via `this.ownerDocument`), window
                    // (via `this.document`) & document (via `this`).
                    var doc = this.ownerDocument || this.document || this,
                        attaches = dataPriv.access( doc, fix );

                    if ( !attaches ) {
                        doc.addEventListener( orig, handler, true );
                    }
                    dataPriv.access( doc, fix, ( attaches || 0 ) + 1 );
                },
                teardown: function() {
                    var doc = this.ownerDocument || this.document || this,
                        attaches = dataPriv.access( doc, fix ) - 1;

                    if ( !attaches ) {
                        doc.removeEventListener( orig, handler, true );
                        dataPriv.remove( doc, fix );

                    } else {
                        dataPriv.access( doc, fix, attaches );
                    }
                }
            };
        } );
    }
    var location = window.location;

    var nonce = { guid: Date.now() };

    var rquery = ( /\?/ );



// Cross-browser xml parsing
    jQuery.parseXML = function( data ) {
        var xml;
        if ( !data || typeof data !== "string" ) {
            return null;
        }

        // Support: IE 9 - 11 only
        // IE throws on parseFromString with invalid input.
        try {
            xml = ( new window.DOMParser() ).parseFromString( data, "text/xml" );
        } catch ( e ) {
            xml = undefined;
        }

        if ( !xml || xml.getElementsByTagName( "parsererror" ).length ) {
            jQuery.error( "Invalid XML: " + data );
        }
        return xml;
    };


    var
        rbracket = /\[\]$/,
        rCRLF = /\r?\n/g,
        rsubmitterTypes = /^(?:submit|button|image|reset|file)$/i,
        rsubmittable = /^(?:input|select|textarea|keygen)/i;

    function buildParams( prefix, obj, traditional, add ) {
        var name;

        if ( Array.isArray( obj ) ) {

            // Serialize array item.
            jQuery.each( obj, function( i, v ) {
                if ( traditional || rbracket.test( prefix ) ) {

                    // Treat each array item as a scalar.
                    add( prefix, v );

                } else {

                    // Item is non-scalar (array or object), encode its numeric index.
                    buildParams(
                        prefix + "[" + ( typeof v === "object" && v != null ? i : "" ) + "]",
                        v,
                        traditional,
                        add
                    );
                }
            } );

        } else if ( !traditional && toType( obj ) === "object" ) {

            // Serialize object item.
            for ( name in obj ) {
                buildParams( prefix + "[" + name + "]", obj[ name ], traditional, add );
            }

        } else {

            // Serialize scalar item.
            add( prefix, obj );
        }
    }

// Serialize an array of form elements or a set of
// key/values into a query string
    jQuery.param = function( a, traditional ) {
        var prefix,
            s = [],
            add = function( key, valueOrFunction ) {

                // If value is a function, invoke it and use its return value
                var value = isFunction( valueOrFunction ) ?
                    valueOrFunction() :
                    valueOrFunction;

                s[ s.length ] = encodeURIComponent( key ) + "=" +
                    encodeURIComponent( value == null ? "" : value );
            };

        if ( a == null ) {
            return "";
        }

        // If an array was passed in, assume that it is an array of form elements.
        if ( Array.isArray( a ) || ( a.jquery && !jQuery.isPlainObject( a ) ) ) {

            // Serialize the form elements
            jQuery.each( a, function() {
                add( this.name, this.value );
            } );

        } else {

            // If traditional, encode the "old" way (the way 1.3.2 or older
            // did it), otherwise encode params recursively.
            for ( prefix in a ) {
                buildParams( prefix, a[ prefix ], traditional, add );
            }
        }

        // Return the resulting serialization
        return s.join( "&" );
    };

    jQuery.fn.extend( {
        serialize: function() {
            return jQuery.param( this.serializeArray() );
        },
        serializeArray: function() {
            return this.map( function() {

                // Can add propHook for "elements" to filter or add form elements
                var elements = jQuery.prop( this, "elements" );
                return elements ? jQuery.makeArray( elements ) : this;
            } )
                .filter( function() {
                    var type = this.type;

                    // Use .is( ":disabled" ) so that fieldset[disabled] works
                    return this.name && !jQuery( this ).is( ":disabled" ) &&
                        rsubmittable.test( this.nodeName ) && !rsubmitterTypes.test( type ) &&
                        ( this.checked || !rcheckableType.test( type ) );
                } )
                .map( function( _i, elem ) {
                    var val = jQuery( this ).val();

                    if ( val == null ) {
                        return null;
                    }

                    if ( Array.isArray( val ) ) {
                        return jQuery.map( val, function( val ) {
                            return { name: elem.name, value: val.replace( rCRLF, "\r\n" ) };
                        } );
                    }

                    return { name: elem.name, value: val.replace( rCRLF, "\r\n" ) };
                } ).get();
        }
    } );


    var
        r20 = /%20/g,
        rhash = /#.*$/,
        rantiCache = /([?&])_=[^&]*/,
        rheaders = /^(.*?):[ \t]*([^\r\n]*)$/mg,

        // #7653, #8125, #8152: local protocol detection
        rlocalProtocol = /^(?:about|app|app-storage|.+-extension|file|res|widget):$/,
        rnoContent = /^(?:GET|HEAD)$/,
        rprotocol = /^\/\//,

        /* Prefilters
	 * 1) They are useful to introduce custom dataTypes (see ajax/jsonp.js for an example)
	 * 2) These are called:
	 *    - BEFORE asking for a transport
	 *    - AFTER param serialization (s.data is a string if s.processData is true)
	 * 3) key is the dataType
	 * 4) the catchall symbol "*" can be used
	 * 5) execution will start with transport dataType and THEN continue down to "*" if needed
	 */
        prefilters = {},

        /* Transports bindings
	 * 1) key is the dataType
	 * 2) the catchall symbol "*" can be used
	 * 3) selection will start with transport dataType and THEN go to "*" if needed
	 */
        transports = {},

        // Avoid comment-prolog char sequence (#10098); must appease lint and evade compression
        allTypes = "*/".concat( "*" ),

        // Anchor tag for parsing the document origin
        originAnchor = document.createElement( "a" );
    originAnchor.href = location.href;

// Base "constructor" for jQuery.ajaxPrefilter and jQuery.ajaxTransport
    function addToPrefiltersOrTransports( structure ) {

        // dataTypeExpression is optional and defaults to "*"
        return function( dataTypeExpression, func ) {

            if ( typeof dataTypeExpression !== "string" ) {
                func = dataTypeExpression;
                dataTypeExpression = "*";
            }

            var dataType,
                i = 0,
                dataTypes = dataTypeExpression.toLowerCase().match( rnothtmlwhite ) || [];

            if ( isFunction( func ) ) {

                // For each dataType in the dataTypeExpression
                while ( ( dataType = dataTypes[ i++ ] ) ) {

                    // Prepend if requested
                    if ( dataType[ 0 ] === "+" ) {
                        dataType = dataType.slice( 1 ) || "*";
                        ( structure[ dataType ] = structure[ dataType ] || [] ).unshift( func );

                        // Otherwise append
                    } else {
                        ( structure[ dataType ] = structure[ dataType ] || [] ).push( func );
                    }
                }
            }
        };
    }

// Base inspection function for prefilters and transports
    function inspectPrefiltersOrTransports( structure, options, originalOptions, jqXHR ) {

        var inspected = {},
            seekingTransport = ( structure === transports );

        function inspect( dataType ) {
            var selected;
            inspected[ dataType ] = true;
            jQuery.each( structure[ dataType ] || [], function( _, prefilterOrFactory ) {
                var dataTypeOrTransport = prefilterOrFactory( options, originalOptions, jqXHR );
                if ( typeof dataTypeOrTransport === "string" &&
                    !seekingTransport && !inspected[ dataTypeOrTransport ] ) {

                    options.dataTypes.unshift( dataTypeOrTransport );
                    inspect( dataTypeOrTransport );
                    return false;
                } else if ( seekingTransport ) {
                    return !( selected = dataTypeOrTransport );
                }
            } );
            return selected;
        }

        return inspect( options.dataTypes[ 0 ] ) || !inspected[ "*" ] && inspect( "*" );
    }

// A special extend for ajax options
// that takes "flat" options (not to be deep extended)
// Fixes #9887
    function ajaxExtend( target, src ) {
        var key, deep,
            flatOptions = jQuery.ajaxSettings.flatOptions || {};

        for ( key in src ) {
            if ( src[ key ] !== undefined ) {
                ( flatOptions[ key ] ? target : ( deep || ( deep = {} ) ) )[ key ] = src[ key ];
            }
        }
        if ( deep ) {
            jQuery.extend( true, target, deep );
        }

        return target;
    }

    /* Handles responses to an ajax request:
 * - finds the right dataType (mediates between content-type and expected dataType)
 * - returns the corresponding response
 */
    function ajaxHandleResponses( s, jqXHR, responses ) {

        var ct, type, finalDataType, firstDataType,
            contents = s.contents,
            dataTypes = s.dataTypes;

        // Remove auto dataType and get content-type in the process
        while ( dataTypes[ 0 ] === "*" ) {
            dataTypes.shift();
            if ( ct === undefined ) {
                ct = s.mimeType || jqXHR.getResponseHeader( "Content-Type" );
            }
        }

        // Check if we're dealing with a known content-type
        if ( ct ) {
            for ( type in contents ) {
                if ( contents[ type ] && contents[ type ].test( ct ) ) {
                    dataTypes.unshift( type );
                    break;
                }
            }
        }

        // Check to see if we have a response for the expected dataType
        if ( dataTypes[ 0 ] in responses ) {
            finalDataType = dataTypes[ 0 ];
        } else {

            // Try convertible dataTypes
            for ( type in responses ) {
                if ( !dataTypes[ 0 ] || s.converters[ type + " " + dataTypes[ 0 ] ] ) {
                    finalDataType = type;
                    break;
                }
                if ( !firstDataType ) {
                    firstDataType = type;
                }
            }

            // Or just use first one
            finalDataType = finalDataType || firstDataType;
        }

        // If we found a dataType
        // We add the dataType to the list if needed
        // and return the corresponding response
        if ( finalDataType ) {
            if ( finalDataType !== dataTypes[ 0 ] ) {
                dataTypes.unshift( finalDataType );
            }
            return responses[ finalDataType ];
        }
    }

    /* Chain conversions given the request and the original response
 * Also sets the responseXXX fields on the jqXHR instance
 */
    function ajaxConvert( s, response, jqXHR, isSuccess ) {
        var conv2, current, conv, tmp, prev,
            converters = {},

            // Work with a copy of dataTypes in case we need to modify it for conversion
            dataTypes = s.dataTypes.slice();

        // Create converters map with lowercased keys
        if ( dataTypes[ 1 ] ) {
            for ( conv in s.converters ) {
                converters[ conv.toLowerCase() ] = s.converters[ conv ];
            }
        }

        current = dataTypes.shift();

        // Convert to each sequential dataType
        while ( current ) {

            if ( s.responseFields[ current ] ) {
                jqXHR[ s.responseFields[ current ] ] = response;
            }

            // Apply the dataFilter if provided
            if ( !prev && isSuccess && s.dataFilter ) {
                response = s.dataFilter( response, s.dataType );
            }

            prev = current;
            current = dataTypes.shift();

            if ( current ) {

                // There's only work to do if current dataType is non-auto
                if ( current === "*" ) {

                    current = prev;

                    // Convert response if prev dataType is non-auto and differs from current
                } else if ( prev !== "*" && prev !== current ) {

                    // Seek a direct converter
                    conv = converters[ prev + " " + current ] || converters[ "* " + current ];

                    // If none found, seek a pair
                    if ( !conv ) {
                        for ( conv2 in converters ) {

                            // If conv2 outputs current
                            tmp = conv2.split( " " );
                            if ( tmp[ 1 ] === current ) {

                                // If prev can be converted to accepted input
                                conv = converters[ prev + " " + tmp[ 0 ] ] ||
                                    converters[ "* " + tmp[ 0 ] ];
                                if ( conv ) {

                                    // Condense equivalence converters
                                    if ( conv === true ) {
                                        conv = converters[ conv2 ];

                                        // Otherwise, insert the intermediate dataType
                                    } else if ( converters[ conv2 ] !== true ) {
                                        current = tmp[ 0 ];
                                        dataTypes.unshift( tmp[ 1 ] );
                                    }
                                    break;
                                }
                            }
                        }
                    }

                    // Apply converter (if not an equivalence)
                    if ( conv !== true ) {

                        // Unless errors are allowed to bubble, catch and return them
                        if ( conv && s.throws ) {
                            response = conv( response );
                        } else {
                            try {
                                response = conv( response );
                            } catch ( e ) {
                                return {
                                    state: "parsererror",
                                    error: conv ? e : "No conversion from " + prev + " to " + current
                                };
                            }
                        }
                    }
                }
            }
        }

        return { state: "success", data: response };
    }

    jQuery.extend( {

        // Counter for holding the number of active queries
        active: 0,

        // Last-Modified header cache for next request
        lastModified: {},
        etag: {},

        ajaxSettings: {
            url: location.href,
            type: "GET",
            isLocal: rlocalProtocol.test( location.protocol ),
            global: true,
            processData: true,
            async: true,
            contentType: "application/x-www-form-urlencoded; charset=UTF-8",

            /*
		timeout: 0,
		data: null,
		dataType: null,
		username: null,
		password: null,
		cache: null,
		throws: false,
		traditional: false,
		headers: {},
		*/

            accepts: {
                "*": allTypes,
                text: "text/plain",
                html: "text/html",
                xml: "application/xml, text/xml",
                json: "application/json, text/javascript"
            },

            contents: {
                xml: /\bxml\b/,
                html: /\bhtml/,
                json: /\bjson\b/
            },

            responseFields: {
                xml: "responseXML",
                text: "responseText",
                json: "responseJSON"
            },

            // Data converters
            // Keys separate source (or catchall "*") and destination types with a single space
            converters: {

                // Convert anything to text
                "* text": String,

                // Text to html (true = no transformation)
                "text html": true,

                // Evaluate text as a json expression
                "text json": JSON.parse,

                // Parse text as xml
                "text xml": jQuery.parseXML
            },

            // For options that shouldn't be deep extended:
            // you can add your own custom options here if
            // and when you create one that shouldn't be
            // deep extended (see ajaxExtend)
            flatOptions: {
                url: true,
                context: true
            }
        },

        // Creates a full fledged settings object into target
        // with both ajaxSettings and settings fields.
        // If target is omitted, writes into ajaxSettings.
        ajaxSetup: function( target, settings ) {
            return settings ?

                // Building a settings object
                ajaxExtend( ajaxExtend( target, jQuery.ajaxSettings ), settings ) :

                // Extending ajaxSettings
                ajaxExtend( jQuery.ajaxSettings, target );
        },

        ajaxPrefilter: addToPrefiltersOrTransports( prefilters ),
        ajaxTransport: addToPrefiltersOrTransports( transports ),

        // Main method
        ajax: function( url, options ) {

            // If url is an object, simulate pre-1.5 signature
            if ( typeof url === "object" ) {
                options = url;
                url = undefined;
            }

            // Force options to be an object
            options = options || {};

            var transport,

                // URL without anti-cache param
                cacheURL,

                // Response headers
                responseHeadersString,
                responseHeaders,

                // timeout handle
                timeoutTimer,

                // Url cleanup var
                urlAnchor,

                // Request state (becomes false upon send and true upon completion)
                completed,

                // To know if global events are to be dispatched
                fireGlobals,

                // Loop variable
                i,

                // uncached part of the url
                uncached,

                // Create the final options object
                s = jQuery.ajaxSetup( {}, options ),

                // Callbacks context
                callbackContext = s.context || s,

                // Context for global events is callbackContext if it is a DOM node or jQuery collection
                globalEventContext = s.context &&
                ( callbackContext.nodeType || callbackContext.jquery ) ?
                    jQuery( callbackContext ) :
                    jQuery.event,

                // Deferreds
                deferred = jQuery.Deferred(),
                completeDeferred = jQuery.Callbacks( "once memory" ),

                // Status-dependent callbacks
                statusCode = s.statusCode || {},

                // Headers (they are sent all at once)
                requestHeaders = {},
                requestHeadersNames = {},

                // Default abort message
                strAbort = "canceled",

                // Fake xhr
                jqXHR = {
                    readyState: 0,

                    // Builds headers hashtable if needed
                    getResponseHeader: function( key ) {
                        var match;
                        if ( completed ) {
                            if ( !responseHeaders ) {
                                responseHeaders = {};
                                while ( ( match = rheaders.exec( responseHeadersString ) ) ) {
                                    responseHeaders[ match[ 1 ].toLowerCase() + " " ] =
                                        ( responseHeaders[ match[ 1 ].toLowerCase() + " " ] || [] )
                                            .concat( match[ 2 ] );
                                }
                            }
                            match = responseHeaders[ key.toLowerCase() + " " ];
                        }
                        return match == null ? null : match.join( ", " );
                    },

                    // Raw string
                    getAllResponseHeaders: function() {
                        return completed ? responseHeadersString : null;
                    },

                    // Caches the header
                    setRequestHeader: function( name, value ) {
                        if ( completed == null ) {
                            name = requestHeadersNames[ name.toLowerCase() ] =
                                requestHeadersNames[ name.toLowerCase() ] || name;
                            requestHeaders[ name ] = value;
                        }
                        return this;
                    },

                    // Overrides response content-type header
                    overrideMimeType: function( type ) {
                        if ( completed == null ) {
                            s.mimeType = type;
                        }
                        return this;
                    },

                    // Status-dependent callbacks
                    statusCode: function( map ) {
                        var code;
                        if ( map ) {
                            if ( completed ) {

                                // Execute the appropriate callbacks
                                jqXHR.always( map[ jqXHR.status ] );
                            } else {

                                // Lazy-add the new callbacks in a way that preserves old ones
                                for ( code in map ) {
                                    statusCode[ code ] = [ statusCode[ code ], map[ code ] ];
                                }
                            }
                        }
                        return this;
                    },

                    // Cancel the request
                    abort: function( statusText ) {
                        var finalText = statusText || strAbort;
                        if ( transport ) {
                            transport.abort( finalText );
                        }
                        done( 0, finalText );
                        return this;
                    }
                };

            // Attach deferreds
            deferred.promise( jqXHR );

            // Add protocol if not provided (prefilters might expect it)
            // Handle falsy url in the settings object (#10093: consistency with old signature)
            // We also use the url parameter if available
            s.url = ( ( url || s.url || location.href ) + "" )
                .replace( rprotocol, location.protocol + "//" );

            // Alias method option to type as per ticket #12004
            s.type = options.method || options.type || s.method || s.type;

            // Extract dataTypes list
            s.dataTypes = ( s.dataType || "*" ).toLowerCase().match( rnothtmlwhite ) || [ "" ];

            // A cross-domain request is in order when the origin doesn't match the current origin.
            if ( s.crossDomain == null ) {
                urlAnchor = document.createElement( "a" );

                // Support: IE <=8 - 11, Edge 12 - 15
                // IE throws exception on accessing the href property if url is malformed,
                // e.g. http://example.com:80x/
                try {
                    urlAnchor.href = s.url;

                    // Support: IE <=8 - 11 only
                    // Anchor's host property isn't correctly set when s.url is relative
                    urlAnchor.href = urlAnchor.href;
                    s.crossDomain = originAnchor.protocol + "//" + originAnchor.host !==
                        urlAnchor.protocol + "//" + urlAnchor.host;
                } catch ( e ) {

                    // If there is an error parsing the URL, assume it is crossDomain,
                    // it can be rejected by the transport if it is invalid
                    s.crossDomain = true;
                }
            }

            // Convert data if not already a string
            if ( s.data && s.processData && typeof s.data !== "string" ) {
                s.data = jQuery.param( s.data, s.traditional );
            }

            // Apply prefilters
            inspectPrefiltersOrTransports( prefilters, s, options, jqXHR );

            // If request was aborted inside a prefilter, stop there
            if ( completed ) {
                return jqXHR;
            }

            // We can fire global events as of now if asked to
            // Don't fire events if jQuery.event is undefined in an AMD-usage scenario (#15118)
            fireGlobals = jQuery.event && s.global;

            // Watch for a new set of requests
            if ( fireGlobals && jQuery.active++ === 0 ) {
                jQuery.event.trigger( "ajaxStart" );
            }

            // Uppercase the type
            s.type = s.type.toUpperCase();

            // Determine if request has content
            s.hasContent = !rnoContent.test( s.type );

            // Save the URL in case we're toying with the If-Modified-Since
            // and/or If-None-Match header later on
            // Remove hash to simplify url manipulation
            cacheURL = s.url.replace( rhash, "" );

            // More options handling for requests with no content
            if ( !s.hasContent ) {

                // Remember the hash so we can put it back
                uncached = s.url.slice( cacheURL.length );

                // If data is available and should be processed, append data to url
                if ( s.data && ( s.processData || typeof s.data === "string" ) ) {
                    cacheURL += ( rquery.test( cacheURL ) ? "&" : "?" ) + s.data;

                    // #9682: remove data so that it's not used in an eventual retry
                    delete s.data;
                }

                // Add or update anti-cache param if needed
                if ( s.cache === false ) {
                    cacheURL = cacheURL.replace( rantiCache, "$1" );
                    uncached = ( rquery.test( cacheURL ) ? "&" : "?" ) + "_=" + ( nonce.guid++ ) +
                        uncached;
                }

                // Put hash and anti-cache on the URL that will be requested (gh-1732)
                s.url = cacheURL + uncached;

                // Change '%20' to '+' if this is encoded form body content (gh-2658)
            } else if ( s.data && s.processData &&
                ( s.contentType || "" ).indexOf( "application/x-www-form-urlencoded" ) === 0 ) {
                s.data = s.data.replace( r20, "+" );
            }

            // Set the If-Modified-Since and/or If-None-Match header, if in ifModified mode.
            if ( s.ifModified ) {
                if ( jQuery.lastModified[ cacheURL ] ) {
                    jqXHR.setRequestHeader( "If-Modified-Since", jQuery.lastModified[ cacheURL ] );
                }
                if ( jQuery.etag[ cacheURL ] ) {
                    jqXHR.setRequestHeader( "If-None-Match", jQuery.etag[ cacheURL ] );
                }
            }

            // Set the correct header, if data is being sent
            if ( s.data && s.hasContent && s.contentType !== false || options.contentType ) {
                jqXHR.setRequestHeader( "Content-Type", s.contentType );
            }

            // Set the Accepts header for the server, depending on the dataType
            jqXHR.setRequestHeader(
                "Accept",
                s.dataTypes[ 0 ] && s.accepts[ s.dataTypes[ 0 ] ] ?
                    s.accepts[ s.dataTypes[ 0 ] ] +
                    ( s.dataTypes[ 0 ] !== "*" ? ", " + allTypes + "; q=0.01" : "" ) :
                    s.accepts[ "*" ]
            );

            // Check for headers option
            for ( i in s.headers ) {
                jqXHR.setRequestHeader( i, s.headers[ i ] );
            }

            // Allow custom headers/mimetypes and early abort
            if ( s.beforeSend &&
                ( s.beforeSend.call( callbackContext, jqXHR, s ) === false || completed ) ) {

                // Abort if not done already and return
                return jqXHR.abort();
            }

            // Aborting is no longer a cancellation
            strAbort = "abort";

            // Install callbacks on deferreds
            completeDeferred.add( s.complete );
            jqXHR.done( s.success );
            jqXHR.fail( s.error );

            // Get transport
            transport = inspectPrefiltersOrTransports( transports, s, options, jqXHR );

            // If no transport, we auto-abort
            if ( !transport ) {
                done( -1, "No Transport" );
            } else {
                jqXHR.readyState = 1;

                // Send global event
                if ( fireGlobals ) {
                    globalEventContext.trigger( "ajaxSend", [ jqXHR, s ] );
                }

                // If request was aborted inside ajaxSend, stop there
                if ( completed ) {
                    return jqXHR;
                }

                // Timeout
                if ( s.async && s.timeout > 0 ) {
                    timeoutTimer = window.setTimeout( function() {
                        jqXHR.abort( "timeout" );
                    }, s.timeout );
                }

                try {
                    completed = false;
                    transport.send( requestHeaders, done );
                } catch ( e ) {

                    // Rethrow post-completion exceptions
                    if ( completed ) {
                        throw e;
                    }

                    // Propagate others as results
                    done( -1, e );
                }
            }

            // Callback for when everything is done
            function done( status, nativeStatusText, responses, headers ) {
                var isSuccess, success, error, response, modified,
                    statusText = nativeStatusText;

                // Ignore repeat invocations
                if ( completed ) {
                    return;
                }

                completed = true;

                // Clear timeout if it exists
                if ( timeoutTimer ) {
                    window.clearTimeout( timeoutTimer );
                }

                // Dereference transport for early garbage collection
                // (no matter how long the jqXHR object will be used)
                transport = undefined;

                // Cache response headers
                responseHeadersString = headers || "";

                // Set readyState
                jqXHR.readyState = status > 0 ? 4 : 0;

                // Determine if successful
                isSuccess = status >= 200 && status < 300 || status === 304;

                // Get response data
                if ( responses ) {
                    response = ajaxHandleResponses( s, jqXHR, responses );
                }

                // Use a noop converter for missing script
                if ( !isSuccess && jQuery.inArray( "script", s.dataTypes ) > -1 ) {
                    s.converters[ "text script" ] = function() {};
                }

                // Convert no matter what (that way responseXXX fields are always set)
                response = ajaxConvert( s, response, jqXHR, isSuccess );

                // If successful, handle type chaining
                if ( isSuccess ) {

                    // Set the If-Modified-Since and/or If-None-Match header, if in ifModified mode.
                    if ( s.ifModified ) {
                        modified = jqXHR.getResponseHeader( "Last-Modified" );
                        if ( modified ) {
                            jQuery.lastModified[ cacheURL ] = modified;
                        }
                        modified = jqXHR.getResponseHeader( "etag" );
                        if ( modified ) {
                            jQuery.etag[ cacheURL ] = modified;
                        }
                    }

                    // if no content
                    if ( status === 204 || s.type === "HEAD" ) {
                        statusText = "nocontent";

                        // if not modified
                    } else if ( status === 304 ) {
                        statusText = "notmodified";

                        // If we have data, let's convert it
                    } else {
                        statusText = response.state;
                        success = response.data;
                        error = response.error;
                        isSuccess = !error;
                    }
                } else {

                    // Extract error from statusText and normalize for non-aborts
                    error = statusText;
                    if ( status || !statusText ) {
                        statusText = "error";
                        if ( status < 0 ) {
                            status = 0;
                        }
                    }
                }

                // Set data for the fake xhr object
                jqXHR.status = status;
                jqXHR.statusText = ( nativeStatusText || statusText ) + "";

                // Success/Error
                if ( isSuccess ) {
                    deferred.resolveWith( callbackContext, [ success, statusText, jqXHR ] );
                } else {
                    deferred.rejectWith( callbackContext, [ jqXHR, statusText, error ] );
                }

                // Status-dependent callbacks
                jqXHR.statusCode( statusCode );
                statusCode = undefined;

                if ( fireGlobals ) {
                    globalEventContext.trigger( isSuccess ? "ajaxSuccess" : "ajaxError",
                        [ jqXHR, s, isSuccess ? success : error ] );
                }

                // Complete
                completeDeferred.fireWith( callbackContext, [ jqXHR, statusText ] );

                if ( fireGlobals ) {
                    globalEventContext.trigger( "ajaxComplete", [ jqXHR, s ] );

                    // Handle the global AJAX counter
                    if ( !( --jQuery.active ) ) {
                        jQuery.event.trigger( "ajaxStop" );
                    }
                }
            }

            return jqXHR;
        },

        getJSON: function( url, data, callback ) {
            return jQuery.get( url, data, callback, "json" );
        },

        getScript: function( url, callback ) {
            return jQuery.get( url, undefined, callback, "script" );
        }
    } );

    jQuery.each( [ "get", "post" ], function( _i, method ) {
        jQuery[ method ] = function( url, data, callback, type ) {

            // Shift arguments if data argument was omitted
            if ( isFunction( data ) ) {
                type = type || callback;
                callback = data;
                data = undefined;
            }

            // The url can be an options object (which then must have .url)
            return jQuery.ajax( jQuery.extend( {
                url: url,
                type: method,
                dataType: type,
                data: data,
                success: callback
            }, jQuery.isPlainObject( url ) && url ) );
        };
    } );

    jQuery.ajaxPrefilter( function( s ) {
        var i;
        for ( i in s.headers ) {
            if ( i.toLowerCase() === "content-type" ) {
                s.contentType = s.headers[ i ] || "";
            }
        }
    } );


    jQuery._evalUrl = function( url, options, doc ) {
        return jQuery.ajax( {
            url: url,

            // Make this explicit, since user can override this through ajaxSetup (#11264)
            type: "GET",
            dataType: "script",
            cache: true,
            async: false,
            global: false,

            // Only evaluate the response if it is successful (gh-4126)
            // dataFilter is not invoked for failure responses, so using it instead
            // of the default converter is kludgy but it works.
            converters: {
                "text script": function() {}
            },
            dataFilter: function( response ) {
                jQuery.globalEval( response, options, doc );
            }
        } );
    };


    jQuery.fn.extend( {
        wrapAll: function( html ) {
            var wrap;

            if ( this[ 0 ] ) {
                if ( isFunction( html ) ) {
                    html = html.call( this[ 0 ] );
                }

                // The elements to wrap the target around
                wrap = jQuery( html, this[ 0 ].ownerDocument ).eq( 0 ).clone( true );

                if ( this[ 0 ].parentNode ) {
                    wrap.insertBefore( this[ 0 ] );
                }

                wrap.map( function() {
                    var elem = this;

                    while ( elem.firstElementChild ) {
                        elem = elem.firstElementChild;
                    }

                    return elem;
                } ).append( this );
            }

            return this;
        },

        wrapInner: function( html ) {
            if ( isFunction( html ) ) {
                return this.each( function( i ) {
                    jQuery( this ).wrapInner( html.call( this, i ) );
                } );
            }

            return this.each( function() {
                var self = jQuery( this ),
                    contents = self.contents();

                if ( contents.length ) {
                    contents.wrapAll( html );

                } else {
                    self.append( html );
                }
            } );
        },

        wrap: function( html ) {
            var htmlIsFunction = isFunction( html );

            return this.each( function( i ) {
                jQuery( this ).wrapAll( htmlIsFunction ? html.call( this, i ) : html );
            } );
        },

        unwrap: function( selector ) {
            this.parent( selector ).not( "body" ).each( function() {
                jQuery( this ).replaceWith( this.childNodes );
            } );
            return this;
        }
    } );


    jQuery.expr.pseudos.hidden = function( elem ) {
        return !jQuery.expr.pseudos.visible( elem );
    };
    jQuery.expr.pseudos.visible = function( elem ) {
        return !!( elem.offsetWidth || elem.offsetHeight || elem.getClientRects().length );
    };




    jQuery.ajaxSettings.xhr = function() {
        try {
            return new window.XMLHttpRequest();
        } catch ( e ) {}
    };

    var xhrSuccessStatus = {

            // File protocol always yields status code 0, assume 200
            0: 200,

            // Support: IE <=9 only
            // #1450: sometimes IE returns 1223 when it should be 204
            1223: 204
        },
        xhrSupported = jQuery.ajaxSettings.xhr();

    support.cors = !!xhrSupported && ( "withCredentials" in xhrSupported );
    support.ajax = xhrSupported = !!xhrSupported;

    jQuery.ajaxTransport( function( options ) {
        var callback, errorCallback;

        // Cross domain only allowed if supported through XMLHttpRequest
        if ( support.cors || xhrSupported && !options.crossDomain ) {
            return {
                send: function( headers, complete ) {
                    var i,
                        xhr = options.xhr();

                    xhr.open(
                        options.type,
                        options.url,
                        options.async,
                        options.username,
                        options.password
                    );

                    // Apply custom fields if provided
                    if ( options.xhrFields ) {
                        for ( i in options.xhrFields ) {
                            xhr[ i ] = options.xhrFields[ i ];
                        }
                    }

                    // Override mime type if needed
                    if ( options.mimeType && xhr.overrideMimeType ) {
                        xhr.overrideMimeType( options.mimeType );
                    }

                    // X-Requested-With header
                    // For cross-domain requests, seeing as conditions for a preflight are
                    // akin to a jigsaw puzzle, we simply never set it to be sure.
                    // (it can always be set on a per-request basis or even using ajaxSetup)
                    // For same-domain requests, won't change header if already provided.
                    if ( !options.crossDomain && !headers[ "X-Requested-With" ] ) {
                        headers[ "X-Requested-With" ] = "XMLHttpRequest";
                    }

                    // Set headers
                    for ( i in headers ) {
                        xhr.setRequestHeader( i, headers[ i ] );
                    }

                    // Callback
                    callback = function( type ) {
                        return function() {
                            if ( callback ) {
                                callback = errorCallback = xhr.onload =
                                    xhr.onerror = xhr.onabort = xhr.ontimeout =
                                        xhr.onreadystatechange = null;

                                if ( type === "abort" ) {
                                    xhr.abort();
                                } else if ( type === "error" ) {

                                    // Support: IE <=9 only
                                    // On a manual native abort, IE9 throws
                                    // errors on any property access that is not readyState
                                    if ( typeof xhr.status !== "number" ) {
                                        complete( 0, "error" );
                                    } else {
                                        complete(

                                            // File: protocol always yields status 0; see #8605, #14207
                                            xhr.status,
                                            xhr.statusText
                                        );
                                    }
                                } else {
                                    complete(
                                        xhrSuccessStatus[ xhr.status ] || xhr.status,
                                        xhr.statusText,

                                        // Support: IE <=9 only
                                        // IE9 has no XHR2 but throws on binary (trac-11426)
                                        // For XHR2 non-text, let the caller handle it (gh-2498)
                                        ( xhr.responseType || "text" ) !== "text"  ||
                                        typeof xhr.responseText !== "string" ?
                                            { binary: xhr.response } :
                                            { text: xhr.responseText },
                                        xhr.getAllResponseHeaders()
                                    );
                                }
                            }
                        };
                    };

                    // Listen to events
                    xhr.onload = callback();
                    errorCallback = xhr.onerror = xhr.ontimeout = callback( "error" );

                    // Support: IE 9 only
                    // Use onreadystatechange to replace onabort
                    // to handle uncaught aborts
                    if ( xhr.onabort !== undefined ) {
                        xhr.onabort = errorCallback;
                    } else {
                        xhr.onreadystatechange = function() {

                            // Check readyState before timeout as it changes
                            if ( xhr.readyState === 4 ) {

                                // Allow onerror to be called first,
                                // but that will not handle a native abort
                                // Also, save errorCallback to a variable
                                // as xhr.onerror cannot be accessed
                                window.setTimeout( function() {
                                    if ( callback ) {
                                        errorCallback();
                                    }
                                } );
                            }
                        };
                    }

                    // Create the abort callback
                    callback = callback( "abort" );

                    try {

                        // Do send the request (this may raise an exception)
                        xhr.send( options.hasContent && options.data || null );
                    } catch ( e ) {

                        // #14683: Only rethrow if this hasn't been notified as an error yet
                        if ( callback ) {
                            throw e;
                        }
                    }
                },

                abort: function() {
                    if ( callback ) {
                        callback();
                    }
                }
            };
        }
    } );




// Prevent auto-execution of scripts when no explicit dataType was provided (See gh-2432)
    jQuery.ajaxPrefilter( function( s ) {
        if ( s.crossDomain ) {
            s.contents.script = false;
        }
    } );

// Install script dataType
    jQuery.ajaxSetup( {
        accepts: {
            script: "text/javascript, application/javascript, " +
                "application/ecmascript, application/x-ecmascript"
        },
        contents: {
            script: /\b(?:java|ecma)script\b/
        },
        converters: {
            "text script": function( text ) {
                jQuery.globalEval( text );
                return text;
            }
        }
    } );

// Handle cache's special case and crossDomain
    jQuery.ajaxPrefilter( "script", function( s ) {
        if ( s.cache === undefined ) {
            s.cache = false;
        }
        if ( s.crossDomain ) {
            s.type = "GET";
        }
    } );

// Bind script tag hack transport
    jQuery.ajaxTransport( "script", function( s ) {

        // This transport only deals with cross domain or forced-by-attrs requests
        if ( s.crossDomain || s.scriptAttrs ) {
            var script, callback;
            return {
                send: function( _, complete ) {
                    script = jQuery( "<script>" )
                        .attr( s.scriptAttrs || {} )
                        .prop( { charset: s.scriptCharset, src: s.url } )
                        .on( "load error", callback = function( evt ) {
                            script.remove();
                            callback = null;
                            if ( evt ) {
                                complete( evt.type === "error" ? 404 : 200, evt.type );
                            }
                        } );

                    // Use native DOM manipulation to avoid our domManip AJAX trickery
                    document.head.appendChild( script[ 0 ] );
                },
                abort: function() {
                    if ( callback ) {
                        callback();
                    }
                }
            };
        }
    } );




    var oldCallbacks = [],
        rjsonp = /(=)\?(?=&|$)|\?\?/;

// Default jsonp settings
    jQuery.ajaxSetup( {
        jsonp: "callback",
        jsonpCallback: function() {
            var callback = oldCallbacks.pop() || ( jQuery.expando + "_" + ( nonce.guid++ ) );
            this[ callback ] = true;
            return callback;
        }
    } );

// Detect, normalize options and install callbacks for jsonp requests
    jQuery.ajaxPrefilter( "json jsonp", function( s, originalSettings, jqXHR ) {

        var callbackName, overwritten, responseContainer,
            jsonProp = s.jsonp !== false && ( rjsonp.test( s.url ) ?
                    "url" :
                    typeof s.data === "string" &&
                    ( s.contentType || "" )
                        .indexOf( "application/x-www-form-urlencoded" ) === 0 &&
                    rjsonp.test( s.data ) && "data"
            );

        // Handle iff the expected data type is "jsonp" or we have a parameter to set
        if ( jsonProp || s.dataTypes[ 0 ] === "jsonp" ) {

            // Get callback name, remembering preexisting value associated with it
            callbackName = s.jsonpCallback = isFunction( s.jsonpCallback ) ?
                s.jsonpCallback() :
                s.jsonpCallback;

            // Insert callback into url or form data
            if ( jsonProp ) {
                s[ jsonProp ] = s[ jsonProp ].replace( rjsonp, "$1" + callbackName );
            } else if ( s.jsonp !== false ) {
                s.url += ( rquery.test( s.url ) ? "&" : "?" ) + s.jsonp + "=" + callbackName;
            }

            // Use data converter to retrieve json after script execution
            s.converters[ "script json" ] = function() {
                if ( !responseContainer ) {
                    jQuery.error( callbackName + " was not called" );
                }
                return responseContainer[ 0 ];
            };

            // Force json dataType
            s.dataTypes[ 0 ] = "json";

            // Install callback
            overwritten = window[ callbackName ];
            window[ callbackName ] = function() {
                responseContainer = arguments;
            };

            // Clean-up function (fires after converters)
            jqXHR.always( function() {

                // If previous value didn't exist - remove it
                if ( overwritten === undefined ) {
                    jQuery( window ).removeProp( callbackName );

                    // Otherwise restore preexisting value
                } else {
                    window[ callbackName ] = overwritten;
                }

                // Save back as free
                if ( s[ callbackName ] ) {

                    // Make sure that re-using the options doesn't screw things around
                    s.jsonpCallback = originalSettings.jsonpCallback;

                    // Save the callback name for future use
                    oldCallbacks.push( callbackName );
                }

                // Call if it was a function and we have a response
                if ( responseContainer && isFunction( overwritten ) ) {
                    overwritten( responseContainer[ 0 ] );
                }

                responseContainer = overwritten = undefined;
            } );

            // Delegate to script
            return "script";
        }
    } );




// Support: Safari 8 only
// In Safari 8 documents created via document.implementation.createHTMLDocument
// collapse sibling forms: the second one becomes a child of the first one.
// Because of that, this security measure has to be disabled in Safari 8.
// https://bugs.webkit.org/show_bug.cgi?id=137337
    support.createHTMLDocument = ( function() {
        var body = document.implementation.createHTMLDocument( "" ).body;
        body.innerHTML = "<form></form><form></form>";
        return body.childNodes.length === 2;
    } )();


// Argument "data" should be string of html
// context (optional): If specified, the fragment will be created in this context,
// defaults to document
// keepScripts (optional): If true, will include scripts passed in the html string
    jQuery.parseHTML = function( data, context, keepScripts ) {
        if ( typeof data !== "string" ) {
            return [];
        }
        if ( typeof context === "boolean" ) {
            keepScripts = context;
            context = false;
        }

        var base, parsed, scripts;

        if ( !context ) {

            // Stop scripts or inline event handlers from being executed immediately
            // by using document.implementation
            if ( support.createHTMLDocument ) {
                context = document.implementation.createHTMLDocument( "" );

                // Set the base href for the created document
                // so any parsed elements with URLs
                // are based on the document's URL (gh-2965)
                base = context.createElement( "base" );
                base.href = document.location.href;
                context.head.appendChild( base );
            } else {
                context = document;
            }
        }

        parsed = rsingleTag.exec( data );
        scripts = !keepScripts && [];

        // Single tag
        if ( parsed ) {
            return [ context.createElement( parsed[ 1 ] ) ];
        }

        parsed = buildFragment( [ data ], context, scripts );

        if ( scripts && scripts.length ) {
            jQuery( scripts ).remove();
        }

        return jQuery.merge( [], parsed.childNodes );
    };


    /**
     * Load a url into a page
     */
    jQuery.fn.load = function( url, params, callback ) {
        var selector, type, response,
            self = this,
            off = url.indexOf( " " );

        if ( off > -1 ) {
            selector = stripAndCollapse( url.slice( off ) );
            url = url.slice( 0, off );
        }

        // If it's a function
        if ( isFunction( params ) ) {

            // We assume that it's the callback
            callback = params;
            params = undefined;

            // Otherwise, build a param string
        } else if ( params && typeof params === "object" ) {
            type = "POST";
        }

        // If we have elements to modify, make the request
        if ( self.length > 0 ) {
            jQuery.ajax( {
                url: url,

                // If "type" variable is undefined, then "GET" method will be used.
                // Make value of this field explicit since
                // user can override it through ajaxSetup method
                type: type || "GET",
                dataType: "html",
                data: params
            } ).done( function( responseText ) {

                // Save response for use in complete callback
                response = arguments;

                self.html( selector ?

                    // If a selector was specified, locate the right elements in a dummy div
                    // Exclude scripts to avoid IE 'Permission Denied' errors
                    jQuery( "<div>" ).append( jQuery.parseHTML( responseText ) ).find( selector ) :

                    // Otherwise use the full result
                    responseText );

                // If the request succeeds, this function gets "data", "status", "jqXHR"
                // but they are ignored because response was set above.
                // If it fails, this function gets "jqXHR", "status", "error"
            } ).always( callback && function( jqXHR, status ) {
                self.each( function() {
                    callback.apply( this, response || [ jqXHR.responseText, status, jqXHR ] );
                } );
            } );
        }

        return this;
    };




    jQuery.expr.pseudos.animated = function( elem ) {
        return jQuery.grep( jQuery.timers, function( fn ) {
            return elem === fn.elem;
        } ).length;
    };




    jQuery.offset = {
        setOffset: function( elem, options, i ) {
            var curPosition, curLeft, curCSSTop, curTop, curOffset, curCSSLeft, calculatePosition,
                position = jQuery.css( elem, "position" ),
                curElem = jQuery( elem ),
                props = {};

            // Set position first, in-case top/left are set even on static elem
            if ( position === "static" ) {
                elem.style.position = "relative";
            }

            curOffset = curElem.offset();
            curCSSTop = jQuery.css( elem, "top" );
            curCSSLeft = jQuery.css( elem, "left" );
            calculatePosition = ( position === "absolute" || position === "fixed" ) &&
                ( curCSSTop + curCSSLeft ).indexOf( "auto" ) > -1;

            // Need to be able to calculate position if either
            // top or left is auto and position is either absolute or fixed
            if ( calculatePosition ) {
                curPosition = curElem.position();
                curTop = curPosition.top;
                curLeft = curPosition.left;

            } else {
                curTop = parseFloat( curCSSTop ) || 0;
                curLeft = parseFloat( curCSSLeft ) || 0;
            }

            if ( isFunction( options ) ) {

                // Use jQuery.extend here to allow modification of coordinates argument (gh-1848)
                options = options.call( elem, i, jQuery.extend( {}, curOffset ) );
            }

            if ( options.top != null ) {
                props.top = ( options.top - curOffset.top ) + curTop;
            }
            if ( options.left != null ) {
                props.left = ( options.left - curOffset.left ) + curLeft;
            }

            if ( "using" in options ) {
                options.using.call( elem, props );

            } else {
                if ( typeof props.top === "number" ) {
                    props.top += "px";
                }
                if ( typeof props.left === "number" ) {
                    props.left += "px";
                }
                curElem.css( props );
            }
        }
    };

    jQuery.fn.extend( {

        // offset() relates an element's border box to the document origin
        offset: function( options ) {

            // Preserve chaining for setter
            if ( arguments.length ) {
                return options === undefined ?
                    this :
                    this.each( function( i ) {
                        jQuery.offset.setOffset( this, options, i );
                    } );
            }

            var rect, win,
                elem = this[ 0 ];

            if ( !elem ) {
                return;
            }

            // Return zeros for disconnected and hidden (display: none) elements (gh-2310)
            // Support: IE <=11 only
            // Running getBoundingClientRect on a
            // disconnected node in IE throws an error
            if ( !elem.getClientRects().length ) {
                return { top: 0, left: 0 };
            }

            // Get document-relative position by adding viewport scroll to viewport-relative gBCR
            rect = elem.getBoundingClientRect();
            win = elem.ownerDocument.defaultView;
            return {
                top: rect.top + win.pageYOffset,
                left: rect.left + win.pageXOffset
            };
        },

        // position() relates an element's margin box to its offset parent's padding box
        // This corresponds to the behavior of CSS absolute positioning
        position: function() {
            if ( !this[ 0 ] ) {
                return;
            }

            var offsetParent, offset, doc,
                elem = this[ 0 ],
                parentOffset = { top: 0, left: 0 };

            // position:fixed elements are offset from the viewport, which itself always has zero offset
            if ( jQuery.css( elem, "position" ) === "fixed" ) {

                // Assume position:fixed implies availability of getBoundingClientRect
                offset = elem.getBoundingClientRect();

            } else {
                offset = this.offset();

                // Account for the *real* offset parent, which can be the document or its root element
                // when a statically positioned element is identified
                doc = elem.ownerDocument;
                offsetParent = elem.offsetParent || doc.documentElement;
                while ( offsetParent &&
                ( offsetParent === doc.body || offsetParent === doc.documentElement ) &&
                jQuery.css( offsetParent, "position" ) === "static" ) {

                    offsetParent = offsetParent.parentNode;
                }
                if ( offsetParent && offsetParent !== elem && offsetParent.nodeType === 1 ) {

                    // Incorporate borders into its offset, since they are outside its content origin
                    parentOffset = jQuery( offsetParent ).offset();
                    parentOffset.top += jQuery.css( offsetParent, "borderTopWidth", true );
                    parentOffset.left += jQuery.css( offsetParent, "borderLeftWidth", true );
                }
            }

            // Subtract parent offsets and element margins
            return {
                top: offset.top - parentOffset.top - jQuery.css( elem, "marginTop", true ),
                left: offset.left - parentOffset.left - jQuery.css( elem, "marginLeft", true )
            };
        },

        // This method will return documentElement in the following cases:
        // 1) For the element inside the iframe without offsetParent, this method will return
        //    documentElement of the parent window
        // 2) For the hidden or detached element
        // 3) For body or html element, i.e. in case of the html node - it will return itself
        //
        // but those exceptions were never presented as a real life use-cases
        // and might be considered as more preferable results.
        //
        // This logic, however, is not guaranteed and can change at any point in the future
        offsetParent: function() {
            return this.map( function() {
                var offsetParent = this.offsetParent;

                while ( offsetParent && jQuery.css( offsetParent, "position" ) === "static" ) {
                    offsetParent = offsetParent.offsetParent;
                }

                return offsetParent || documentElement;
            } );
        }
    } );

// Create scrollLeft and scrollTop methods
    jQuery.each( { scrollLeft: "pageXOffset", scrollTop: "pageYOffset" }, function( method, prop ) {
        var top = "pageYOffset" === prop;

        jQuery.fn[ method ] = function( val ) {
            return access( this, function( elem, method, val ) {

                // Coalesce documents and windows
                var win;
                if ( isWindow( elem ) ) {
                    win = elem;
                } else if ( elem.nodeType === 9 ) {
                    win = elem.defaultView;
                }

                if ( val === undefined ) {
                    return win ? win[ prop ] : elem[ method ];
                }

                if ( win ) {
                    win.scrollTo(
                        !top ? val : win.pageXOffset,
                        top ? val : win.pageYOffset
                    );

                } else {
                    elem[ method ] = val;
                }
            }, method, val, arguments.length );
        };
    } );

// Support: Safari <=7 - 9.1, Chrome <=37 - 49
// Add the top/left cssHooks using jQuery.fn.position
// Webkit bug: https://bugs.webkit.org/show_bug.cgi?id=29084
// Blink bug: https://bugs.chromium.org/p/chromium/issues/detail?id=589347
// getComputedStyle returns percent when specified for top/left/bottom/right;
// rather than make the css module depend on the offset module, just check for it here
    jQuery.each( [ "top", "left" ], function( _i, prop ) {
        jQuery.cssHooks[ prop ] = addGetHookIf( support.pixelPosition,
            function( elem, computed ) {
                if ( computed ) {
                    computed = curCSS( elem, prop );

                    // If curCSS returns percentage, fallback to offset
                    return rnumnonpx.test( computed ) ?
                        jQuery( elem ).position()[ prop ] + "px" :
                        computed;
                }
            }
        );
    } );


// Create innerHeight, innerWidth, height, width, outerHeight and outerWidth methods
    jQuery.each( { Height: "height", Width: "width" }, function( name, type ) {
        jQuery.each( { padding: "inner" + name, content: type, "": "outer" + name },
            function( defaultExtra, funcName ) {

                // Margin is only for outerHeight, outerWidth
                jQuery.fn[ funcName ] = function( margin, value ) {
                    var chainable = arguments.length && ( defaultExtra || typeof margin !== "boolean" ),
                        extra = defaultExtra || ( margin === true || value === true ? "margin" : "border" );

                    return access( this, function( elem, type, value ) {
                        var doc;

                        if ( isWindow( elem ) ) {

                            // $( window ).outerWidth/Height return w/h including scrollbars (gh-1729)
                            return funcName.indexOf( "outer" ) === 0 ?
                                elem[ "inner" + name ] :
                                elem.document.documentElement[ "client" + name ];
                        }

                        // Get document width or height
                        if ( elem.nodeType === 9 ) {
                            doc = elem.documentElement;

                            // Either scroll[Width/Height] or offset[Width/Height] or client[Width/Height],
                            // whichever is greatest
                            return Math.max(
                                elem.body[ "scroll" + name ], doc[ "scroll" + name ],
                                elem.body[ "offset" + name ], doc[ "offset" + name ],
                                doc[ "client" + name ]
                            );
                        }

                        return value === undefined ?

                            // Get width or height on the element, requesting but not forcing parseFloat
                            jQuery.css( elem, type, extra ) :

                            // Set width or height on the element
                            jQuery.style( elem, type, value, extra );
                    }, type, chainable ? margin : undefined, chainable );
                };
            } );
    } );


    jQuery.each( [
        "ajaxStart",
        "ajaxStop",
        "ajaxComplete",
        "ajaxError",
        "ajaxSuccess",
        "ajaxSend"
    ], function( _i, type ) {
        jQuery.fn[ type ] = function( fn ) {
            return this.on( type, fn );
        };
    } );




    jQuery.fn.extend( {

        bind: function( types, data, fn ) {
            return this.on( types, null, data, fn );
        },
        unbind: function( types, fn ) {
            return this.off( types, null, fn );
        },

        delegate: function( selector, types, data, fn ) {
            return this.on( types, selector, data, fn );
        },
        undelegate: function( selector, types, fn ) {

            // ( namespace ) or ( selector, types [, fn] )
            return arguments.length === 1 ?
                this.off( selector, "**" ) :
                this.off( types, selector || "**", fn );
        },

        hover: function( fnOver, fnOut ) {
            return this.mouseenter( fnOver ).mouseleave( fnOut || fnOver );
        }
    } );

    jQuery.each( ( "blur focus focusin focusout resize scroll click dblclick " +
        "mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave " +
        "change select submit keydown keypress keyup contextmenu" ).split( " " ),
        function( _i, name ) {

            // Handle event binding
            jQuery.fn[ name ] = function( data, fn ) {
                return arguments.length > 0 ?
                    this.on( name, null, data, fn ) :
                    this.trigger( name );
            };
        } );




// Support: Android <=4.0 only
// Make sure we trim BOM and NBSP
    var rtrim = /^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g;

// Bind a function to a context, optionally partially applying any
// arguments.
// jQuery.proxy is deprecated to promote standards (specifically Function#bind)
// However, it is not slated for removal any time soon
    jQuery.proxy = function( fn, context ) {
        var tmp, args, proxy;

        if ( typeof context === "string" ) {
            tmp = fn[ context ];
            context = fn;
            fn = tmp;
        }

        // Quick check to determine if target is callable, in the spec
        // this throws a TypeError, but we will just return undefined.
        if ( !isFunction( fn ) ) {
            return undefined;
        }

        // Simulated bind
        args = slice.call( arguments, 2 );
        proxy = function() {
            return fn.apply( context || this, args.concat( slice.call( arguments ) ) );
        };

        // Set the guid of unique handler to the same of original handler, so it can be removed
        proxy.guid = fn.guid = fn.guid || jQuery.guid++;

        return proxy;
    };

    jQuery.holdReady = function( hold ) {
        if ( hold ) {
            jQuery.readyWait++;
        } else {
            jQuery.ready( true );
        }
    };
    jQuery.isArray = Array.isArray;
    jQuery.parseJSON = JSON.parse;
    jQuery.nodeName = nodeName;
    jQuery.isFunction = isFunction;
    jQuery.isWindow = isWindow;
    jQuery.camelCase = camelCase;
    jQuery.type = toType;

    jQuery.now = Date.now;

    jQuery.isNumeric = function( obj ) {

        // As of jQuery 3.0, isNumeric is limited to
        // strings and numbers (primitives or objects)
        // that can be coerced to finite numbers (gh-2662)
        var type = jQuery.type( obj );
        return ( type === "number" || type === "string" ) &&

            // parseFloat NaNs numeric-cast false positives ("")
            // ...but misinterprets leading-number strings, particularly hex literals ("0x...")
            // subtraction forces infinities to NaN
            !isNaN( obj - parseFloat( obj ) );
    };

    jQuery.trim = function( text ) {
        return text == null ?
            "" :
            ( text + "" ).replace( rtrim, "" );
    };



// Register as a named AMD module, since jQuery can be concatenated with other
// files that may use define, but not via a proper concatenation script that
// understands anonymous AMD modules. A named AMD is safest and most robust
// way to register. Lowercase jquery is used because AMD module names are
// derived from file names, and jQuery is normally delivered in a lowercase
// file name. Do this after creating the global so that if an AMD module wants
// to call noConflict to hide this version of jQuery, it will work.

// Note that for maximum portability, libraries that are not jQuery should
// declare themselves as anonymous modules, and avoid setting a global if an
// AMD loader is present. jQuery is a special case. For more information, see
// https://github.com/jrburke/requirejs/wiki/Updating-existing-libraries#wiki-anon

    if ( typeof define === "function" && define.amd ) {
        define( "jquery", [], function() {
            return jQuery;
        } );
    }




    var

        // Map over jQuery in case of overwrite
        _jQuery = window.jQuery,

        // Map over the $ in case of overwrite
        _$ = window.$;

    jQuery.noConflict = function( deep ) {
        if ( window.$ === jQuery ) {
            window.$ = _$;
        }

        if ( deep && window.jQuery === jQuery ) {
            window.jQuery = _jQuery;
        }

        return jQuery;
    };

// Expose jQuery and $ identifiers, even in AMD
// (#7102#comment:10, https://github.com/jquery/jquery/pull/557)
// and CommonJS for browser emulators (#13566)
    if ( typeof noGlobal === "undefined" ) {
        window.jQuery = window.$ = jQuery;
    }

    return jQuery;
} );

window.BrowserPort = function(id) {
    this.type = typeof(browser) !== 'undefined' ? 'firefox' : 'chrome';
    this.eventDispatcher = id === 'background' ? new HostEventDispatcher() : new PortEventDispatcher(id);
    this.action = new BrowserButtonAction();
    this.settings = new Settings(this.eventDispatcher);
    this.resource = new Resource();
    this.tab = new Tab(this.eventDispatcher);
    this.installer = new Installer();
    this.analytics = new Analytics(this.eventDispatcher);

    this.setUninstallUrl = function(url) {
        // chrome.runtime.setUninstallURL(url, function() {});
    };
};

window.BrowserButtonAction = function() {
    this.hasAlerts = false;
    this.state = null;

    this.icons = {
        offline: {
            "19": './icons/19_offline.png',
            "38": './icons/38_offline.png'
        },
        start: {
            "19": './icons/19_start.png',
            "38": './icons/38_start.png'
        },
        startWarnings: {
            "19": './icons/19_start_warning.png',
            "38": './icons/38_start_warning.png'
        },
        stop: {
            "19": './icons/19_stop.png',
            "38": './icons/38_stop.png'
        },
        stopWarnings: {
            "19": './icons/19_stop_warning.png',
            "38": './icons/38_stop_warning.png'
        }
    };

    if (/^((?!chrome|android).)*safari/i.test(navigator.userAgent)) {
        ico = name => ({
            '16': './icons/safari_' + name + '@16.png',
            '32': './icons/safari_' + name + '@32.png',
            '64': './icons/safari_' + name + '@64.png',
            '128': './icons/safari_' + name + '@128.png'
        });
        
        this.icons = {
            offline: ico('offline'),
            start: ico('start'),
            startWarnings: ico('start_warning'),
            stop: ico('stop'),
            stopWarnings: ico('stop_warning')
        }
    }
};

BrowserButtonAction.prototype = {

    /**
     * Change browser button
     *
     *
     * @param {string|null} state can be: start, stop or null
     * @param {boolean} hasAlerts shows small dot near start/stop icons
     */
    change: function(state, hasAlerts) {
        if (!state) {
            chrome.browserAction.setTitle({title: 'You are not logged in or account suspended in Everhour!'});
            chrome.browserAction.setIcon({
                path: this.icons.offline
            });
            return;
        }

        /**
         * Consider parseTitle method before changing title!
         * The title should definitely outline state and hasAlerts
         */
        chrome.browserAction.setTitle({title: (state === 'start' ? 'Start Timer' : 'Stop Timer') + ' - Everhour' + (hasAlerts ? ' *' : '')});
        chrome.browserAction.setIcon({
            path: this.icons[state + (hasAlerts ? 'Warnings' : '')]
        });
    },

    changeState: function(state) {
        if (!state) {
            this.change(null, false);
            return;
        }

        chrome.browserAction.getTitle({}, title => {
            const {hasAlert, _} = this.parseTitle(title);
            this.change(state, hasAlert);
        });
    },

    changeAlert: function(hasAlert) {
        chrome.browserAction.getTitle({}, title => {
            const {state, _} = this.parseTitle(title);
            this.change(state, state ? hasAlert : false);
        });
    },

    parseTitle: function(title) {
        const hasAlert = title.indexOf('*') !== -1;

        let state = null;

        switch (true) {
            case title.indexOf('Start timer') !== -1:
                state = 'start';
                break;
            case title.indexOf('Stop timer') !== -1:
                state = 'stop';
                break;
        }

        return {hasAlert, state}
    },

    /**
     * Resize action panel
     * @param {int} width
     * @param {int} height
     * @returns {*}
     */
    resize: function(width, height) {
        return;
    },

    /**
     * Get title
     * @returns {*}
     */
    getTitle: function() {
        var promise = $.Deferred();
        chrome.browserAction.getTitle({}, function(title) {
            promise.resolve(title);
        });

        return promise.promise();
    }
};

window.PortEventDispatcher = function(id) {
    /**
     * Dispatcher instance ID
     * We often target events by dispatched ID
     * E.g. send message to background
     *
     * @type {string}
     */
    this.id = id || this.generateId();

    /**
     * Event subscribers namely listeners
     * @type {Object[]}
     */
    this.listeners = [];

    /**
     * @type {object}
     */
    this.port = chrome.runtime.connect({name: this.id});

    this.init();
};

PortEventDispatcher.prototype = {

    init: function() {
        /**
         * Handle messaged sent from chrome.tabs.sendMessage
         */
        chrome.runtime.onMessage.addListener(this.handleMessage.bind(this));

        /**
         * Handle messaged sent from port.postMessage
         */
        this.port.onMessage.addListener(this.handleMessage.bind(this));
    },

    /**
     * Send to yourself or background script, which send it next to other ports
     *
     * @param {string} listeners - !self, self, everhour, content-script, background, browser-button, extension, *
     * @param {string} event
     * @param {object} [data]
     * @param {Function} [responseCallback]
     * @param {boolean} forCurrentTab
     */
    broadcast: function(listeners, event, data, responseCallback, forCurrentTab) {
        const eventData = {
            listeners: listeners.replace(/self/ig, this.id),
            event: event,
            data: data,
            emitter: this.id,
            forCurrentTab: !!forCurrentTab
        };

        try {
            if (this.match(this.id, listeners)) {
                this.dispatchEvent(eventData, responseCallback);
            }

            // send message to host
            // you won't receive it back in chrome.runtime.onMessage
            chrome.runtime.sendMessage(eventData, responseCallback);
        } catch (e) {
            console.log('invalid message', e, eventData);
        }
    },

    /**
     * Subscribe on event
     *
     * @param {string} emitter  - everhour, content-script, background, *
     * @param {string} event
     * @param {function} callback function(data, event)
     * @param {boolean} [once]=false
     */
    on: function(emitter, event, callback, once) {
        this.listeners.push({
            once: !!once,
            emitter: emitter,
            event: event,
            callback: callback
        });
    },

    /**
     * Subscribe on event
     *
     * @param {string} emitter  - everhour, content-script, background, *
     * @param {string} event
     * @param {function} callback function(data, event)
     */
    once: function(emitter, event, callback) {
        this.on(emitter, event, callback, true);
    },


    handleMessage: function(event, sender, callback) {
        if (!event || !event.event || !event.listeners) {
            return;
        }

        if (this.match(this.id, event.listeners)) {
            this.dispatchEvent(event, callback);
        }
    },

    /**
     * @param {string} id
     * @param {string} pattern
     */
    match: function(id, pattern) {
        if (pattern === '*') {
            return true;
        }

        id = id.replace('self', this.id);
        pattern = pattern.replace('self', this.id);

        const conditions = pattern.split('!');

        const includePattern = conditions[0] || null;
        const excludePattern = conditions[1] || null;

        function matchPattern(id, pattern) {
            if (id === pattern) {
                return true;
            }

            const regexp = new RegExp('^' + pattern.replace(/:\*/g, '($|:.+)').replace(/\*/g, '.+') + '$', 'ig');
            return regexp.test(id);
        }

        if (excludePattern && matchPattern(id, excludePattern)) {
            return false;
        }

        if (includePattern && !matchPattern(id, includePattern)) {
            return false;
        }

        return true;

    },

    dispatchEvent: function(event, responseCallback) {
        if (!this.match(this.id, event.listeners)) {
            return;
        }

        this.listeners
            .filter(subscription => subscription.event === event.event)
            .filter(subscription => this.match(event.emitter, subscription.emitter))
            .forEach(subscription => {
                subscription.callback.call(this, event.data, event, responseCallback);

                if (subscription.once) {
                    this.listeners = this.listeners.filter(item => item !== subscription);
                }
            });
    },

    /**
     * Generate unique request token
     * @returns {string}
     */
    generateId: function() {
        function s4() {
            return Math.floor((1 + Math.random()) * 0x10000)
                .toString(16)
                .substring(1);
        }

        return 'content-script:' + s4() + s4() + '-' + s4() + '-' + s4() + '-' + s4() + '-' + s4() + s4() + s4();
    },
};
window.Resource = function() {

};

Resource.prototype = {
    /**
     * Load resource content
     * @param {string} path
     * @returns {string}
     */
    load: function(path) {
        return $.get(this.getUrl(path));
    },

    /**
     * Build resource url
     * @param {string} path
     * @returns {string}
     */
    getUrl: function(path) {
        return chrome.extension.getURL(path);
    }
};
window.Settings = function(eventDispatcher) {
    this.default = {
        accessToken: null
    };

    this.options = {};
    this.eventDispatcher = eventDispatcher;

    // override options if somebody change them
    this.eventDispatcher.on('!self', 'settings:save', $.proxy(this.override, this));
};

Settings.prototype = {
    /**
     * Load options from storage
     *
     * Usage:
     *   browser.settings.load().done(function(options) {
     *      application.start();
     *   });
     *
     * @returns {object}
     */
    load: function() {
        const deferred = $.Deferred();

        chrome.storage.sync.get(null, (options) => {
            if (!options || !options.accessToken) {
                options = Object.assign({}, this.default);
            }
            this.options = options;
            deferred.resolve(options);
        });

        return deferred.promise();
    },

    /**
     * Get single option or all of them
     *
     * Usage:
     *   browser.settings.get('accessToken') -> 1234567890
     *   browser.settings.get() -> {accessToken: 1234567890}
     *
     * @param {null|string} option
     * @returns {*}
     */
    get: function(option) {
        if (!option) {
            return this.options;
        }

        return this.options[option] || null;
    },

    /**
     * Set and save sing option or all of them
     *
     * Usage:
     *   browser.settings.set('accessToken', 1234567890)
     *   browser.settings.set({accessToken: 1234567890})
     *
     * @param {object|string} option
     * @param {null|string} value
     * @returns {*}
     */
    set: function(option, value) {
        var previous = JSON.stringify(this.options);
        if (typeof option == 'object') {
            this.options = option;
        } else {
            this.options[option] = value;
        }

        if (previous != JSON.stringify(this.options)) {
            return this.save();
        }

        var promise = $.Deferred();
        promise.resolve();

        return promise.promise();
    },

    /**
     * Save current options to storage
     * @returns {*}
     */
    save: function() {
        var self = this;
        var promise = $.Deferred();

        chrome.storage.sync.set(this.options, function() {
            promise.resolve();
        });

        return promise.promise().then(function() {
            self.eventDispatcher.broadcast('!self', 'settings:save', self.options);
        });
    },

    /**
     * Override options
     * Useful when we receive options update from outside
     *
     * Usage:
     *    browser.eventDispatcher.on('*', 'settings:save', function(newOptions) {
     *      browser.settings.override(newOptions);
     *    });
     *
     * @param options
     */
    override: function(options) {
        this.options = options;
    }
};

window.Tab = function(eventDispatcher) {
    if (eventDispatcher.id === 'background' && chrome.permissions.onAdded) {
        // firefox does not support this
        // https://bugzilla.mozilla.org/show_bug.cgi?id=1444294
        chrome.permissions.onAdded.addListener(() => {
            eventDispatcher.broadcast('background', 'permissions:changed', null);
        });

        chrome.permissions.onRemoved.addListener(() => {
            eventDispatcher.broadcast('background', 'permissions:changed', null);
        });
    }
};

Tab.prototype = {
    /**
     * Open new tab with specified url
     * @param {string} url
     */
    open: function(url) {
        return chrome.tabs.create({url: url});
    },

    /**
     * @param {string} version
     */
    reloadEverhourTabs(version)
    {
        const comparator = version === 'v1' ? isLegacyEverhourApp : isEverhour2App;
        chrome.tabs.query({}, function (tabs) {
            tabs.forEach(function (tab) {
                if (tab.url && comparator(tab.url)) {
                    chrome.tabs.reload(tab.id, {
                        bypassCache: false
                    });
                }
            });
        });
    },

    reloadIntegrationTabs()
    {
        chrome.tabs.query({}, function (tabs) {
            tabs.forEach(function (tab) {
                if (tab.url && isIntegrationPage(tab.url)) {
                    chrome.tabs.reload(tab.id, {
                        bypassCache: false
                    });
                }
            });
        });
    },

    reloadCustomDomainTabs(domains)
    {
        chrome.tabs.query({url: domains.map(domain => domain + '*')}, function (tabs) {
            tabs.forEach(function (tab) {
                chrome.tabs.reload(tab.id, {
                    bypassCache: false
                });
            });
        });
    },

    _embedContentScriptsForCustomDomainsLocked: {},

    removeContentScriptsForCustomDomains(searchTerm)
    {
        if (!chrome.declarativeContent) {
            console.warn('chrome.declarativeContent is not supported');
            return;
        }

        /**
         * If there are no custom domains
         */
        chrome.declarativeContent.onPageChanged.getRules(undefined, rules => {
            if (rules.length === 0) {
                return;
            }

            const everhourRules = [];
            rules.forEach(rule => {
                if (rule.id.indexOf(searchTerm || 'everhour') === -1) {
                    return;
                }

                everhourRules.push(rule.id);

                rule.conditions.forEach(condition => {
                    if (condition.instanceType === 'declarativeContent.PageStateMatcher' && condition.pageUrl  && condition.pageUrl.urlPrefix) {
                        this.reloadCustomDomainTabs([condition.pageUrl.urlPrefix]);
                    }
                });
            });

            if (everhourRules.length === 0) {
                return;
            }

            chrome.declarativeContent.onPageChanged.removeRules(everhourRules);
        });
    },

    embedContentScriptsForCustomDomains(id, script, domains)
    {
        /**
         * If there are no custom domains
         */
        if (!domains || domains.length === 0) {
            return;
        }

        if (this._embedContentScriptsForCustomDomainsLocked[id]) {
            return;
        }

        if (chrome.declarativeContent) {
            /**
             * Inject content script for custom domains
             */

            const customDomainRule = {
                id: id,
                conditions: domains.map(url => {
                    const pageUrl = {};
                    if (url.indexOf('https://*.') !== 0) {
                        pageUrl.urlPrefix = url;
                    } else {
                        // use regexp for subdomains
                        // escape first, and then replace https://*.somedomain.com to regexp
                        pageUrl.urlMatches = url.replace(/[.*+?^${}()|[\]\\]/g, '\\$&').replace('https://\\*', 'https://.+');
                    }

                    return new chrome.declarativeContent.PageStateMatcher({
                        pageUrl,
                        // make sure we run script after page loaded
                        css: ['body']
                    })
                }),
                actions: [
                    new chrome.declarativeContent.RequestContentScript({
                        js: [script],
                        allFrames: false
                    })
                ]
            };

            if (!!~['everhour_cs_jr', 'everhour_cs_tw', 'everhour_cs_mo', 'everhour_cs_td', 'everhour_cs_no'].indexOf(id)) {
                customDomainRule.conditions = customDomainRule.conditions.map((condition) => {
                    delete condition.css;

                    return condition;
                });
            }

            /**
             * Prevent concurrent content script binding
             * Can happen during login
             */
            this._embedContentScriptsForCustomDomainsLocked[id] = true;
            chrome.declarativeContent.onPageChanged.removeRules([id], () => {
                chrome.declarativeContent.onPageChanged.addRules([customDomainRule], () => {
                    /**
                     * After granting permission for custom domains we are waiting any time and reload tabs because
                     * there are cases when the page is reloaded and extansion scripts is missing
                     * */
                    setTimeout(() => {
                        this.reloadCustomDomainTabs(domains);
                        this._embedContentScriptsForCustomDomainsLocked[id] = false;
                    }, 150);
                });
            });

            return;
        }

        if (typeof(browser) !== 'undefined' && browser.contentScripts && browser.contentScripts.register) {
            chrome.permissions.getAll((permissions) => {
                const bindDomains = [];

                domains.forEach((domain) => {
                    // We are registering only added domains because firefox return error and doesn't register other domains in array
                    if (!!~permissions.origins.indexOf(domain + '*')) {
                        bindDomains.push(domain);
                    }
                });

                const matches = bindDomains.map(url => url + '*');
                // Error: Type error for parameter contentScriptOptions (Error processing matches: Array requires at least 1 items; you have 0) for contentScripts.register.
                if (!matches.length && browserPort.type === 'firefox') {
                    return null;
                }

                browser.contentScripts.register({
                    matches: matches,
                    js: [{file: script}],
                    runAt: "document_idle",
                    allFrames: true
                }).then(() => {
                    this.reloadCustomDomainTabs(bindDomains);
                }).catch((error) => {
                    // console.log('register custom domains error => ', error);
                });
            });
        }
    },

    requestCustomDomainPermissions: function(domains, callback) {
        const request = {origins: domains.map(domain => domain + '*')};
        callback = callback || function () {};

        /**
         * Won't work in FF from browser popup
         * https://bugzilla.mozilla.org/show_bug.cgi?id=1432083
         */
        if (typeof(browser) !== 'undefined' && browser.contentScripts && browser.contentScripts.register) {
            browser.permissions.request(request).then(callback);
            return;
        }

        chrome.permissions.request(request, callback);
    },

    removeWebsitesPermissions: function(domains, callback) {
        const request = {origins: domains.map(domain => domain + '*')};
        callback = callback || function () {};

        chrome.permissions.remove(request, callback);
    },

    isCustomDomainPermitted: function(domain, callback) {
        chrome.permissions.contains({origins: [domain + '*']}, callback);
    }
};

window.Installer = function() {};

Installer.prototype = {

    bind: function(callback) {
        chrome.runtime.onMessageExternal.addListener(function(message, sender, response) {
            if (message.event == 'everhour.token_exchange' && message.accessToken) {
                console.log('installer.incoming_message:ok', 'Processed installation');
                browserPort.settings.set('accessToken', message.accessToken);
                response(true);
                callback();
                return ;
            }

            return ;
        });
    }
};
window.Analytics = function(eventDispatcher) {
    this.eventDispatcher = eventDispatcher;
    this._conditions = {
        _onceByUrl: {},
    };
    this.sessionId = null;
};

Analytics.collectBillingPeriod = function(profile) {
    if (!profile || !profile.team || !profile.team.billing) {
        return 'boarding';
    }

    if (profile.team.createdAt) {
        var teamCreatedHoursAgo = Math.ceil((Date.now() - Date.parse(profile.team.createdAt.replace(" ", "T") + "+00:00")) / (3600 * 1000));
        if (teamCreatedHoursAgo <= 48) {
            return 'boarding';
        }
    }

    var billing = profile.team.billing;
    if (billing.status === 'trial' || billing.status === 'trial_end') {
        return 'trial';
    }

    if (billing.plan === 'free') {
        return 'free';
    }

    return 'paid';
};

Analytics.prototype = {
    event: function(event, params) {
        if (!event) {
            return;
        }

        params = params || {};
        var clientId = params._client || null;
        delete params._client;

        if (!clientId && typeof store !== 'undefined' && store.profile) {
            var profile = store.profile.get();
            clientId = profile.acquisitionVisitorId;
            params.billing_period = Analytics.collectBillingPeriod(profile);
        }

        if (event === 'task_details' && !['trial', 'boarding'].includes(params.billing_period)) {
            return;
        }

        if (!clientId) {
            console.warn('Missing ga client id');
            return;
        }

        if (typeof app !== 'undefined' && app.adaptor) {
            params.component = 'integration';
            params.integration = app.adaptor.platform;
            if (event.slice(-2) !== 'ed') {
                params._onceByUrl = true;
            }
        } else if (this.eventDispatcher.id === 'browser-button') {
            params.component = 'popover';
        } else if (this.eventDispatcher.id === 'background') {
            params.component = 'background';
        }


        if (!this.assertConditions(event, params)) {
            return;
        }

        if (event.slice(4) !== 'ext_') {
            event = 'ext_' + event;
        }

        this.eventDispatcher.broadcast('background', 'analytics:event', {
            event,
            params,
            clientId,
        });
    },

    assertConditions: function(event, conditions) {
        conditions = conditions || {};

        if (conditions._onceByUrl) {
            var url = window.location.href;
            if (this._conditions._onceByUrl[event] === url) {
                return false;
            }
            this._conditions._onceByUrl[event] = url;
            delete conditions._onceByUrl;
        }

        return true;
    },

    sendEvent: function(clientId, event, params) {
        const MEASUREMENT_ID = 'G-JYMQ705V3P';
        const API_SECRET = 'pOia3oj4QM-a63foUwde0g';

        // single sessions starting from ext loaded
        if (!this.sessionId) {
            this.sessionId = Date.now().toString();
        }

        params.session_id = this.sessionId;

        // TODO: engagement skipped since we can't calculate it properly
        // https://support.google.com/analytics/answer/11109416
        // params.engagement_time_msec

        fetch(
            `https://www.google-analytics.com/mp/collect?measurement_id=${MEASUREMENT_ID}&api_secret=${API_SECRET}`,
            {
                method: 'POST',
                body: JSON.stringify({
                    client_id: clientId,
                    events: [
                        {
                            name: event,
                            params
                        }
                    ]
                })
            }
        ).then((response) => {
            if (!response.ok) {
                // console.error('Google Analytics request failed with an exception');
            }
        });
    }
};

/*! jQuery UI - v1.12.1 - 2016-09-14
* http://jqueryui.com
* Includes: widget.js, jquery-1-7.js, keycode.js,  widgets/datepicker.js
* Copyright jQuery Foundation and other contributors; Licensed MIT */

(function( factory ) {
    if ( typeof define === "function" && define.amd ) {

        // AMD. Register as an anonymous module.
        define([ "jquery" ], factory );
    } else {

        // Browser globals
        factory( jQuery );
    }
}(function( $ ) {

    $.ui = $.ui || {};

    var version = $.ui.version = "1.12.1";

    var widgetUuid = 0;

    var safeActiveElement = $.ui.safeActiveElement = function( document ) {
        var activeElement;

        // Support: IE 9 only
        // IE9 throws an "Unspecified error" accessing document.activeElement from an <iframe>
        try {
            activeElement = document.activeElement;
        } catch ( error ) {
            activeElement = document.body;
        }
        // Support: IE 9 - 11 only
        // IE may return null instead of an element
        // Interestingly, this only seems to occur when NOT in an iframe
        if ( !activeElement ) {
            activeElement = document.body;
        }

        // Support: IE 11 only
        // IE11 returns a seemingly empty object in some cases when accessing
        // document.activeElement from an <iframe>
        if ( !activeElement.nodeName ) {
            activeElement = document.body;
        }

        return activeElement;
    };

    var uniqueId = $.fn.extend( {
        uniqueId: ( function() {
            var uuid = 0;

            return function() {
                return this.each( function() {
                    if ( !this.id ) {
                        this.id = "ui-id-" + ( ++uuid );
                    }
                } );
            };
        } )(),

        removeUniqueId: function() {
            return this.each( function() {
                if ( /^ui-id-\d+$/.test( this.id ) ) {
                    $( this ).removeAttr( "id" );
                }
            } );
        }
    } );

    $.widget = function( name, base, prototype ) {
        var existingConstructor, constructor, basePrototype;

        // ProxiedPrototype allows the provided prototype to remain unmodified
        // so that it can be used as a mixin for multiple widgets (#8876)
        var proxiedPrototype = {};

        var namespace = name.split( "." )[ 0 ];
        name = name.split( "." )[ 1 ];
        var fullName = namespace + "-" + name;

        if ( !prototype ) {
            prototype = base;
            base = $.Widget;
        }

        if ( $.isArray( prototype ) ) {
            prototype = $.extend.apply( null, [ {} ].concat( prototype ) );
        }

        // Create selector for plugin
        $.expr[ ":" ][ fullName.toLowerCase() ] = function( elem ) {
            return !!$.data( elem, fullName );
        };

        $[ namespace ] = $[ namespace ] || {};
        existingConstructor = $[ namespace ][ name ];
        constructor = $[ namespace ][ name ] = function( options, element ) {

            // Allow instantiation without "new" keyword
            if ( !this._createWidget ) {
                return new constructor( options, element );
            }

            // Allow instantiation without initializing for simple inheritance
            // must use "new" keyword (the code above always passes args)
            if ( arguments.length ) {
                this._createWidget( options, element );
            }
        };

        // Extend with the existing constructor to carry over any static properties
        $.extend( constructor, existingConstructor, {
            version: prototype.version,

            // Copy the object used to create the prototype in case we need to
            // redefine the widget later
            _proto: $.extend( {}, prototype ),

            // Track widgets that inherit from this widget in case this widget is
            // redefined after a widget inherits from it
            _childConstructors: []
        } );

        basePrototype = new base();

        // We need to make the options hash a property directly on the new instance
        // otherwise we'll modify the options hash on the prototype that we're
        // inheriting from
        basePrototype.options = $.widget.extend( {}, basePrototype.options );
        $.each( prototype, function( prop, value ) {
            if ( !$.isFunction( value ) ) {
                proxiedPrototype[ prop ] = value;
                return;
            }
            proxiedPrototype[ prop ] = ( function() {
                function _super() {
                    return base.prototype[ prop ].apply( this, arguments );
                }

                function _superApply( args ) {
                    return base.prototype[ prop ].apply( this, args );
                }

                return function() {
                    var __super = this._super;
                    var __superApply = this._superApply;
                    var returnValue;

                    this._super = _super;
                    this._superApply = _superApply;

                    returnValue = value.apply( this, arguments );

                    this._super = __super;
                    this._superApply = __superApply;

                    return returnValue;
                };
            } )();
        } );
        constructor.prototype = $.widget.extend( basePrototype, {

            // TODO: remove support for widgetEventPrefix
            // always use the name + a colon as the prefix, e.g., draggable:start
            // don't prefix for widgets that aren't DOM-based
            widgetEventPrefix: existingConstructor ? ( basePrototype.widgetEventPrefix || name ) : name
        }, proxiedPrototype, {
            constructor: constructor,
            namespace: namespace,
            widgetName: name,
            widgetFullName: fullName
        } );

        // If this widget is being redefined then we need to find all widgets that
        // are inheriting from it and redefine all of them so that they inherit from
        // the new version of this widget. We're essentially trying to replace one
        // level in the prototype chain.
        if ( existingConstructor ) {
            $.each( existingConstructor._childConstructors, function( i, child ) {
                var childPrototype = child.prototype;

                // Redefine the child widget using the same prototype that was
                // originally used, but inherit from the new version of the base
                $.widget( childPrototype.namespace + "." + childPrototype.widgetName, constructor,
                    child._proto );
            } );

            // Remove the list of existing child constructors from the old constructor
            // so the old child constructors can be garbage collected
            delete existingConstructor._childConstructors;
        } else {
            base._childConstructors.push( constructor );
        }

        $.widget.bridge( name, constructor );

        return constructor;
    };

    $.widget.extend = function( target ) {
        var input = Array.prototype.slice.call( arguments, 1 );
        var inputIndex = 0;
        var inputLength = input.length;
        var key;
        var value;

        for ( ; inputIndex < inputLength; inputIndex++ ) {
            for ( key in input[ inputIndex ] ) {
                value = input[ inputIndex ][ key ];
                if ( input[ inputIndex ].hasOwnProperty( key ) && value !== undefined ) {

                    // Clone objects
                    if ( $.isPlainObject( value ) ) {
                        target[ key ] = $.isPlainObject( target[ key ] ) ?
                            $.widget.extend( {}, target[ key ], value ) :

                            // Don't extend strings, arrays, etc. with objects
                            $.widget.extend( {}, value );

                        // Copy everything else by reference
                    } else {
                        target[ key ] = value;
                    }
                }
            }
        }
        return target;
    };

    $.widget.bridge = function( name, object ) {
        var fullName = object.prototype.widgetFullName || name;
        $.fn[ name ] = function( options ) {
            var isMethodCall = typeof options === "string";
            var args = Array.prototype.slice.call( arguments, 1 );
            var returnValue = this;

            if ( isMethodCall ) {

                // If this is an empty collection, we need to have the instance method
                // return undefined instead of the jQuery instance
                if ( !this.length && options === "instance" ) {
                    returnValue = undefined;
                } else {
                    this.each( function() {
                        var methodValue;
                        var instance = $.data( this, fullName );

                        if ( options === "instance" ) {
                            returnValue = instance;
                            return false;
                        }

                        if ( !instance ) {
                            return $.error( "cannot call methods on " + name +
                                " prior to initialization; " +
                                "attempted to call method '" + options + "'" );
                        }

                        if ( !$.isFunction( instance[ options ] ) || options.charAt( 0 ) === "_" ) {
                            return $.error( "no such method '" + options + "' for " + name +
                                " widget instance" );
                        }

                        methodValue = instance[ options ].apply( instance, args );

                        if ( methodValue !== instance && methodValue !== undefined ) {
                            returnValue = methodValue && methodValue.jquery ?
                                returnValue.pushStack( methodValue.get() ) :
                                methodValue;
                            return false;
                        }
                    } );
                }
            } else {

                // Allow multiple hashes to be passed on init
                if ( args.length ) {
                    options = $.widget.extend.apply( null, [ options ].concat( args ) );
                }

                this.each( function() {
                    var instance = $.data( this, fullName );
                    if ( instance ) {
                        instance.option( options || {} );
                        if ( instance._init ) {
                            instance._init();
                        }
                    } else {
                        $.data( this, fullName, new object( options, this ) );
                    }
                } );
            }

            return returnValue;
        };
    };

    $.Widget = function( /* options, element */ ) {};
    $.Widget._childConstructors = [];

    $.Widget.prototype = {
        widgetName: "widget",
        widgetEventPrefix: "",
        defaultElement: "<div>",

        options: {
            classes: {},
            disabled: false,

            // Callbacks
            create: null
        },

        _createWidget: function( options, element ) {
            element = $( element || this.defaultElement || this )[ 0 ];
            this.element = $( element );
            this.uuid = widgetUuid++;
            this.eventNamespace = "." + this.widgetName + this.uuid;

            this.bindings = $();
            this.hoverable = $();
            this.focusable = $();
            this.classesElementLookup = {};

            if ( element !== this ) {
                $.data( element, this.widgetFullName, this );
                this._on( true, this.element, {
                    remove: function( event ) {
                        if ( event.target === element ) {
                            this.destroy();
                        }
                    }
                } );
                this.document = $( element.style ?

                    // Element within the document
                    element.ownerDocument :

                    // Element is window or document
                    element.document || element );
                this.window = $( this.document[ 0 ].defaultView || this.document[ 0 ].parentWindow );
            }

            this.options = $.widget.extend( {},
                this.options,
                this._getCreateOptions(),
                options );

            this._create();

            if ( this.options.disabled ) {
                this._setOptionDisabled( this.options.disabled );
            }

            this._trigger( "create", null, this._getCreateEventData() );
            this._init();
        },

        _getCreateOptions: function() {
            return {};
        },

        _getCreateEventData: $.noop,

        _create: $.noop,

        _init: $.noop,

        destroy: function() {
            var that = this;

            this._destroy();
            $.each( this.classesElementLookup, function( key, value ) {
                that._removeClass( value, key );
            } );

            // We can probably remove the unbind calls in 2.0
            // all event bindings should go through this._on()
            this.element
                .off( this.eventNamespace )
                .removeData( this.widgetFullName );
            this.widget()
                .off( this.eventNamespace )
                .removeAttr( "aria-disabled" );

            // Clean up events and states
            this.bindings.off( this.eventNamespace );
        },

        _destroy: $.noop,

        widget: function() {
            return this.element;
        },

        option: function( key, value ) {
            var options = key;
            var parts;
            var curOption;
            var i;

            if ( arguments.length === 0 ) {

                // Don't return a reference to the internal hash
                return $.widget.extend( {}, this.options );
            }

            if ( typeof key === "string" ) {

                // Handle nested keys, e.g., "foo.bar" => { foo: { bar: ___ } }
                options = {};
                parts = key.split( "." );
                key = parts.shift();
                if ( parts.length ) {
                    curOption = options[ key ] = $.widget.extend( {}, this.options[ key ] );
                    for ( i = 0; i < parts.length - 1; i++ ) {
                        curOption[ parts[ i ] ] = curOption[ parts[ i ] ] || {};
                        curOption = curOption[ parts[ i ] ];
                    }
                    key = parts.pop();
                    if ( arguments.length === 1 ) {
                        return curOption[ key ] === undefined ? null : curOption[ key ];
                    }
                    curOption[ key ] = value;
                } else {
                    if ( arguments.length === 1 ) {
                        return this.options[ key ] === undefined ? null : this.options[ key ];
                    }
                    options[ key ] = value;
                }
            }

            this._setOptions( options );

            return this;
        },

        _setOptions: function( options ) {
            var key;

            for ( key in options ) {
                this._setOption( key, options[ key ] );
            }

            return this;
        },

        _setOption: function( key, value ) {
            if ( key === "classes" ) {
                this._setOptionClasses( value );
            }

            this.options[ key ] = value;

            if ( key === "disabled" ) {
                this._setOptionDisabled( value );
            }

            return this;
        },

        _setOptionClasses: function( value ) {
            var classKey, elements, currentElements;

            for ( classKey in value ) {
                currentElements = this.classesElementLookup[ classKey ];
                if ( value[ classKey ] === this.options.classes[ classKey ] ||
                    !currentElements ||
                    !currentElements.length ) {
                    continue;
                }

                // We are doing this to create a new jQuery object because the _removeClass() call
                // on the next line is going to destroy the reference to the current elements being
                // tracked. We need to save a copy of this collection so that we can add the new classes
                // below.
                elements = $( currentElements.get() );
                this._removeClass( currentElements, classKey );

                // We don't use _addClass() here, because that uses this.options.classes
                // for generating the string of classes. We want to use the value passed in from
                // _setOption(), this is the new value of the classes option which was passed to
                // _setOption(). We pass this value directly to _classes().
                elements.addClass( this._classes( {
                    element: elements,
                    keys: classKey,
                    classes: value,
                    add: true
                } ) );
            }
        },

        _setOptionDisabled: function( value ) {
            this._toggleClass( this.widget(), this.widgetFullName + "-disabled", null, !!value );

            // If the widget is becoming disabled, then nothing is interactive
            if ( value ) {
                this._removeClass( this.hoverable, null, "ui-state-hover" );
                this._removeClass( this.focusable, null, "ui-state-focus" );
            }
        },

        enable: function() {
            return this._setOptions( { disabled: false } );
        },

        disable: function() {
            return this._setOptions( { disabled: true } );
        },

        _classes: function( options ) {
            var full = [];
            var that = this;

            options = $.extend( {
                element: this.element,
                classes: this.options.classes || {}
            }, options );

            function processClassString( classes, checkOption ) {
                var current, i;
                for ( i = 0; i < classes.length; i++ ) {
                    current = that.classesElementLookup[ classes[ i ] ] || $();
                    if ( options.add ) {
                        current = $( $.unique( current.get().concat( options.element.get() ) ) );
                    } else {
                        current = $( current.not( options.element ).get() );
                    }
                    that.classesElementLookup[ classes[ i ] ] = current;
                    full.push( classes[ i ] );
                    if ( checkOption && options.classes[ classes[ i ] ] ) {
                        full.push( options.classes[ classes[ i ] ] );
                    }
                }
            }

            this._on( options.element, {
                "remove": "_untrackClassesElement"
            } );

            if ( options.keys ) {
                processClassString( options.keys.match( /\S+/g ) || [], true );
            }
            if ( options.extra ) {
                processClassString( options.extra.match( /\S+/g ) || [] );
            }

            return full.join( " " );
        },

        _untrackClassesElement: function( event ) {
            var that = this;
            $.each( that.classesElementLookup, function( key, value ) {
                if ( $.inArray( event.target, value ) !== -1 ) {
                    that.classesElementLookup[ key ] = $( value.not( event.target ).get() );
                }
            } );
        },

        _removeClass: function( element, keys, extra ) {
            return this._toggleClass( element, keys, extra, false );
        },

        _addClass: function( element, keys, extra ) {
            return this._toggleClass( element, keys, extra, true );
        },

        _toggleClass: function( element, keys, extra, add ) {
            add = ( typeof add === "boolean" ) ? add : extra;
            var shift = ( typeof element === "string" || element === null ),
                options = {
                    extra: shift ? keys : extra,
                    keys: shift ? element : keys,
                    element: shift ? this.element : element,
                    add: add
                };
            options.element.toggleClass( this._classes( options ), add );
            return this;
        },

        _on: function( suppressDisabledCheck, element, handlers ) {
            var delegateElement;
            var instance = this;

            // No suppressDisabledCheck flag, shuffle arguments
            if ( typeof suppressDisabledCheck !== "boolean" ) {
                handlers = element;
                element = suppressDisabledCheck;
                suppressDisabledCheck = false;
            }

            // No element argument, shuffle and use this.element
            if ( !handlers ) {
                handlers = element;
                element = this.element;
                delegateElement = this.widget();
            } else {
                element = delegateElement = $( element );
                this.bindings = this.bindings.add( element );
            }

            $.each( handlers, function( event, handler ) {
                function handlerProxy() {

                    // Allow widgets to customize the disabled handling
                    // - disabled as an array instead of boolean
                    // - disabled class as method for disabling individual parts
                    if ( !suppressDisabledCheck &&
                        ( instance.options.disabled === true ||
                            $( this ).hasClass( "ui-state-disabled" ) ) ) {
                        return;
                    }
                    return ( typeof handler === "string" ? instance[ handler ] : handler )
                        .apply( instance, arguments );
                }

                // Copy the guid so direct unbinding works
                if ( typeof handler !== "string" ) {
                    handlerProxy.guid = handler.guid =
                        handler.guid || handlerProxy.guid || $.guid++;
                }

                var match = event.match( /^([\w:-]*)\s*(.*)$/ );
                var eventName = match[ 1 ] + instance.eventNamespace;
                var selector = match[ 2 ];

                if ( selector ) {
                    delegateElement.on( eventName, selector, handlerProxy );
                } else {
                    element.on( eventName, handlerProxy );
                }
            } );
        },

        _off: function( element, eventName ) {
            eventName = ( eventName || "" ).split( " " ).join( this.eventNamespace + " " ) +
                this.eventNamespace;
            element.off( eventName ).off( eventName );

            // Clear the stack to avoid memory leaks (#10056)
            this.bindings = $( this.bindings.not( element ).get() );
            this.focusable = $( this.focusable.not( element ).get() );
            this.hoverable = $( this.hoverable.not( element ).get() );
        },

        _delay: function( handler, delay ) {
            function handlerProxy() {
                return ( typeof handler === "string" ? instance[ handler ] : handler )
                    .apply( instance, arguments );
            }
            var instance = this;
            return setTimeout( handlerProxy, delay || 0 );
        },

        _hoverable: function( element ) {
            this.hoverable = this.hoverable.add( element );
            this._on( element, {
                mouseenter: function( event ) {
                    this._addClass( $( event.currentTarget ), null, "ui-state-hover" );
                },
                mouseleave: function( event ) {
                    this._removeClass( $( event.currentTarget ), null, "ui-state-hover" );
                }
            } );
        },

        _focusable: function( element ) {
            this.focusable = this.focusable.add( element );
            this._on( element, {
                focusin: function( event ) {
                    this._addClass( $( event.currentTarget ), null, "ui-state-focus" );
                },
                focusout: function( event ) {
                    this._removeClass( $( event.currentTarget ), null, "ui-state-focus" );
                }
            } );
        },

        _trigger: function( type, event, data ) {
            var prop, orig;
            var callback = this.options[ type ];

            data = data || {};
            event = $.Event( event );
            event.type = ( type === this.widgetEventPrefix ?
                type :
                this.widgetEventPrefix + type ).toLowerCase();

            // The original event may come from any element
            // so we need to reset the target on the new event
            event.target = this.element[ 0 ];

            // Copy original event properties over to the new event
            orig = event.originalEvent;
            if ( orig ) {
                for ( prop in orig ) {
                    if ( !( prop in event ) ) {
                        event[ prop ] = orig[ prop ];
                    }
                }
            }

            this.element.trigger( event, data );
            return !( $.isFunction( callback ) &&
                callback.apply( this.element[ 0 ], [ event ].concat( data ) ) === false ||
                event.isDefaultPrevented() );
        }
    };

    $.each( { show: "fadeIn", hide: "fadeOut" }, function( method, defaultEffect ) {
        $.Widget.prototype[ "_" + method ] = function( element, options, callback ) {
            if ( typeof options === "string" ) {
                options = { effect: options };
            }

            var hasOptions;
            var effectName = !options ?
                method :
                options === true || typeof options === "number" ?
                    defaultEffect :
                    options.effect || defaultEffect;

            options = options || {};
            if ( typeof options === "number" ) {
                options = { duration: options };
            }

            hasOptions = !$.isEmptyObject( options );
            options.complete = callback;

            if ( options.delay ) {
                element.delay( options.delay );
            }

            if ( hasOptions && $.effects && $.effects.effect[ effectName ] ) {
                element[ method ]( options );
            } else if ( effectName !== method && element[ effectName ] ) {
                element[ effectName ]( options.duration, options.easing, callback );
            } else {
                element.queue( function( next ) {
                    $( this )[ method ]();
                    if ( callback ) {
                        callback.call( element[ 0 ] );
                    }
                    next();
                } );
            }
        };
    } );

    var widget = $.widget;
    $.widget( "ui.autocomplete", {
        version: "1.12.1",
        defaultElement: "<input>",
        options: {
            appendTo: null,
            autoFocus: false,
            delay: 300,
            minLength: 1,
            position: {
                my: "left top",
                at: "left bottom",
                collision: "none"
            },
            source: null,

            // Callbacks
            change: null,
            close: null,
            focus: null,
            open: null,
            response: null,
            search: null,
            select: null
        },

        requestIndex: 0,
        pending: 0,

        _create: function() {

            // Some browsers only repeat keydown events, not keypress events,
            // so we use the suppressKeyPress flag to determine if we've already
            // handled the keydown event. #7269
            // Unfortunately the code for & in keypress is the same as the up arrow,
            // so we use the suppressKeyPressRepeat flag to avoid handling keypress
            // events when we know the keydown event was used to modify the
            // search term. #7799
            var suppressKeyPress, suppressKeyPressRepeat, suppressInput,
                nodeName = this.element[ 0 ].nodeName.toLowerCase(),
                isTextarea = nodeName === "textarea",
                isInput = nodeName === "input";

            // Textareas are always multi-line
            // Inputs are always single-line, even if inside a contentEditable element
            // IE also treats inputs as contentEditable
            // All other element types are determined by whether or not they're contentEditable
            this.isMultiLine = isTextarea || !isInput && this._isContentEditable( this.element );

            this.valueMethod = this.element[ isTextarea || isInput ? "val" : "text" ];
            this.isNewMenu = true;

            this._addClass( "ui-autocomplete-input" );
            this.element.attr( "autocomplete", "off" );

            this._on( this.element, {
                keydown: function( event ) {
                    if ( this.element.prop( "readOnly" ) ) {
                        suppressKeyPress = true;
                        suppressInput = true;
                        suppressKeyPressRepeat = true;
                        return;
                    }

                    suppressKeyPress = false;
                    suppressInput = false;
                    suppressKeyPressRepeat = false;
                    var keyCode = $.ui.keyCode;
                    switch ( event.keyCode ) {
                        case keyCode.PAGE_UP:
                            suppressKeyPress = true;
                            this._move( "previousPage", event );
                            break;
                        case keyCode.PAGE_DOWN:
                            suppressKeyPress = true;
                            this._move( "nextPage", event );
                            break;
                        case keyCode.UP:
                            suppressKeyPress = true;
                            this._keyEvent( "previous", event );
                            break;
                        case keyCode.DOWN:
                            suppressKeyPress = true;
                            this._keyEvent( "next", event );
                            break;
                        case keyCode.ENTER:

                            // when menu is open and has focus
                            if ( this.menu.active ) {

                                // #6055 - Opera still allows the keypress to occur
                                // which causes forms to submit
                                suppressKeyPress = true;
                                event.preventDefault();
                                this.menu.select( event );
                            }
                            break;
                        case keyCode.TAB:
                            if ( this.menu.active ) {
                                this.menu.select( event );
                            }
                            break;
                        case keyCode.ESCAPE:
                            if ( this.menu.element.is( ":visible" ) ) {
                                if ( !this.isMultiLine ) {
                                    this._value( this.term );
                                }
                                this.close( event );

                                // Different browsers have different default behavior for escape
                                // Single press can mean undo or clear
                                // Double press in IE means clear the whole form
                                event.preventDefault();
                            }
                            break;
                        default:
                            suppressKeyPressRepeat = true;

                            // search timeout should be triggered before the input value is changed
                            this._searchTimeout( event );
                            break;
                    }
                },
                keypress: function( event ) {
                    if ( suppressKeyPress ) {
                        suppressKeyPress = false;
                        if ( !this.isMultiLine || this.menu.element.is( ":visible" ) ) {
                            event.preventDefault();
                        }
                        return;
                    }
                    if ( suppressKeyPressRepeat ) {
                        return;
                    }

                    // Replicate some key handlers to allow them to repeat in Firefox and Opera
                    var keyCode = $.ui.keyCode;
                    switch ( event.keyCode ) {
                        case keyCode.PAGE_UP:
                            this._move( "previousPage", event );
                            break;
                        case keyCode.PAGE_DOWN:
                            this._move( "nextPage", event );
                            break;
                        case keyCode.UP:
                            this._keyEvent( "previous", event );
                            break;
                        case keyCode.DOWN:
                            this._keyEvent( "next", event );
                            break;
                    }
                },
                input: function( event ) {
                    if ( suppressInput ) {
                        suppressInput = false;
                        event.preventDefault();
                        return;
                    }
                    this._searchTimeout( event );
                },
                focus: function() {
                    this.selectedItem = null;
                    this.previous = this._value();
                },
                blur: function( event ) {
                    if ( this.cancelBlur ) {
                        delete this.cancelBlur;
                        return;
                    }

                    clearTimeout( this.searching );
                    this.close( event );
                    this._change( event );
                }
            } );

            this._initSource();
            this.menu = $( "<ul>" )
                .appendTo( this._appendTo() )
                .menu( {

                    // disable ARIA support, the live region takes care of that
                    role: null
                } )
                .hide()
                .menu( "instance" );

            this._addClass( this.menu.element, "ui-autocomplete", "ui-front" );
            this._on( this.menu.element, {
                mousedown: function( event ) {

                    // prevent moving focus out of the text field
                    event.preventDefault();

                    // IE doesn't prevent moving focus even with event.preventDefault()
                    // so we set a flag to know when we should ignore the blur event
                    this.cancelBlur = true;
                    this._delay( function() {
                        delete this.cancelBlur;

                        // Support: IE 8 only
                        // Right clicking a menu item or selecting text from the menu items will
                        // result in focus moving out of the input. However, we've already received
                        // and ignored the blur event because of the cancelBlur flag set above. So
                        // we restore focus to ensure that the menu closes properly based on the user's
                        // next actions.
                        if ( this.element[ 0 ] !== $.ui.safeActiveElement( this.document[ 0 ] ) ) {
                            this.element.trigger( "focus" );
                        }
                    } );
                },
                menufocus: function( event, ui ) {
                    var label, item;

                    // support: Firefox
                    // Prevent accidental activation of menu items in Firefox (#7024 #9118)
                    if ( this.isNewMenu ) {
                        this.isNewMenu = false;
                        if ( event.originalEvent && /^mouse/.test( event.originalEvent.type ) ) {
                            this.menu.blur();

                            this.document.one( "mousemove", function() {
                                $( event.target ).trigger( event.originalEvent );
                            } );

                            return;
                        }
                    }

                    item = ui.item.data( "ui-autocomplete-item" );
                    if ( false !== this._trigger( "focus", event, { item: item } ) ) {

                        // use value to match what will end up in the input, if it was a key event
                        if ( event.originalEvent && /^key/.test( event.originalEvent.type ) ) {
                            this._value( item.value );
                        }
                    }

                    // Announce the value in the liveRegion
                    label = ui.item.attr( "aria-label" ) || item.value;
                    if ( label && $.trim( label ).length ) {
                        this.liveRegion.children().hide();
                        $( "<div>" ).text( label ).appendTo( this.liveRegion );
                    }
                },
                menuselect: function( event, ui ) {
                    var item = ui.item.data( "ui-autocomplete-item" ),
                        previous = this.previous;

                    // Only trigger when focus was lost (click on menu)
                    if ( this.element[ 0 ] !== $.ui.safeActiveElement( this.document[ 0 ] ) ) {
                        this.element.trigger( "focus" );
                        this.previous = previous;

                        // #6109 - IE triggers two focus events and the second
                        // is asynchronous, so we need to reset the previous
                        // term synchronously and asynchronously :-(
                        this._delay( function() {
                            this.previous = previous;
                            this.selectedItem = item;
                        } );
                    }

                    if ( false !== this._trigger( "select", event, { item: item } ) ) {
                        this._value( item.value );
                    }

                    // reset the term after the select event
                    // this allows custom select handling to work properly
                    this.term = this._value();

                    this.close( event );
                    this.selectedItem = item;
                }
            } );

            this.liveRegion = $( "<div>", {
                role: "status",
                "aria-live": "assertive",
                "aria-relevant": "additions"
            } )
                .appendTo( this.document[ 0 ].body );

            this._addClass( this.liveRegion, null, "ui-helper-hidden-accessible" );

            // Turning off autocomplete prevents the browser from remembering the
            // value when navigating through history, so we re-enable autocomplete
            // if the page is unloaded before the widget is destroyed. #7790
            this._on( this.window, {
                beforeunload: function() {
                    this.element.removeAttr( "autocomplete" );
                }
            } );
        },

        _destroy: function() {
            clearTimeout( this.searching );
            this.element.removeAttr( "autocomplete" );
            this.menu.element.remove();
            this.liveRegion.remove();
        },

        _setOption: function( key, value ) {
            this._super( key, value );
            if ( key === "source" ) {
                this._initSource();
            }
            if ( key === "appendTo" ) {
                this.menu.element.appendTo( this._appendTo() );
            }
            if ( key === "disabled" && value && this.xhr ) {
                this.xhr.abort();
            }
        },

        _isEventTargetInWidget: function( event ) {
            var menuElement = this.menu.element[ 0 ];

            return event.target === this.element[ 0 ] ||
                event.target === menuElement ||
                $.contains( menuElement, event.target );
        },

        _closeOnClickOutside: function( event ) {
            if ( !this._isEventTargetInWidget( event ) ) {
                this.close();
            }
        },

        _appendTo: function() {
            var element = this.options.appendTo;

            if ( element ) {
                element = element.jquery || element.nodeType ?
                    $( element ) :
                    this.document.find( element ).eq( 0 );
            }

            if ( !element || !element[ 0 ] ) {
                element = this.element.closest( ".ui-front, dialog" );
            }

            if ( !element.length ) {
                element = this.document[ 0 ].body;
            }

            return element;
        },

        _initSource: function() {
            var array, url,
                that = this;
            if ( $.isArray( this.options.source ) ) {
                array = this.options.source;
                this.source = function( request, response ) {
                    response( $.ui.autocomplete.filter( array, request.term ) );
                };
            } else if ( typeof this.options.source === "string" ) {
                url = this.options.source;
                this.source = function( request, response ) {
                    if ( that.xhr ) {
                        that.xhr.abort();
                    }
                    that.xhr = $.ajax( {
                        url: url,
                        data: request,
                        dataType: "json",
                        success: function( data ) {
                            response( data );
                        },
                        error: function() {
                            response( [] );
                        }
                    } );
                };
            } else {
                this.source = this.options.source;
            }
        },

        _searchTimeout: function( event ) {
            clearTimeout( this.searching );
            this.searching = this._delay( function() {

                // Search if the value has changed, or if the user retypes the same value (see #7434)
                var equalValues = this.term === this._value(),
                    menuVisible = this.menu.element.is( ":visible" ),
                    modifierKey = event.altKey || event.ctrlKey || event.metaKey || event.shiftKey;

                if ( !equalValues || ( equalValues && !menuVisible && !modifierKey ) ) {
                    this.selectedItem = null;
                    this.search( null, event );
                }
            }, this.options.delay );
        },

        search: function( value, event ) {
            value = value != null ? value : this._value();

            // Always save the actual value, not the one passed as an argument
            this.term = this._value();

            if ( value.length < this.options.minLength ) {
                return this.close( event );
            }

            if ( this._trigger( "search", event ) === false ) {
                return;
            }

            return this._search( value );
        },

        _search: function( value ) {
            this.pending++;
            this._addClass( "ui-autocomplete-loading" );
            this.cancelSearch = false;

            this.source( { term: value }, this._response() );
        },

        _response: function() {
            var index = ++this.requestIndex;

            return $.proxy( function( content ) {
                if ( index === this.requestIndex ) {
                    this.__response( content );
                }

                this.pending--;
                if ( !this.pending ) {
                    this._removeClass( "ui-autocomplete-loading" );
                }
            }, this );
        },

        __response: function( content ) {
            if ( content ) {
                content = this._normalize( content );
            }
            this._trigger( "response", null, { content: content } );
            if ( !this.options.disabled && content && content.length && !this.cancelSearch ) {
                this._suggest( content );
                this._trigger( "open" );
            } else {

                // use ._close() instead of .close() so we don't cancel future searches
                this._close();
            }
        },

        close: function( event ) {
            this.cancelSearch = true;
            this._close( event );
        },

        _close: function( event ) {

            // Remove the handler that closes the menu on outside clicks
            this._off( this.document, "mousedown" );

            if ( this.menu.element.is( ":visible" ) ) {
                this.menu.element.hide();
                this.menu.blur();
                this.isNewMenu = true;
                this._trigger( "close", event );
            }
        },

        _change: function( event ) {
            if ( this.previous !== this._value() ) {
                this._trigger( "change", event, { item: this.selectedItem } );
            }
        },

        _normalize: function( items ) {

            // assume all items have the right format when the first item is complete
            if ( items.length && items[ 0 ].label && items[ 0 ].value ) {
                return items;
            }
            return $.map( items, function( item ) {
                if ( typeof item === "string" ) {
                    return {
                        label: item,
                        value: item
                    };
                }
                return $.extend( {}, item, {
                    label: item.label || item.value,
                    value: item.value || item.label
                } );
            } );
        },

        _suggest: function( items ) {
            var ul = this.menu.element.empty();
            this._renderMenu( ul, items );
            this.isNewMenu = true;
            this.menu.refresh();

            // Size and position menu
            ul.show();
            this._resizeMenu();
            ul.position( $.extend( {
                of: this.element
            }, this.options.position ) );

            if ( this.options.autoFocus ) {
                this.menu.next();
            }

            // Listen for interactions outside of the widget (#6642)
            this._on( this.document, {
                mousedown: "_closeOnClickOutside"
            } );
        },

        _resizeMenu: function() {
            var ul = this.menu.element;
            ul.outerWidth( Math.max(

                // Firefox wraps long text (possibly a rounding bug)
                // so we add 1px to avoid the wrapping (#7513)
                ul.width( "" ).outerWidth() + 1,
                this.element.outerWidth()
            ) );
        },

        _renderMenu: function( ul, items ) {
            var that = this;
            $.each( items, function( index, item ) {
                that._renderItemData( ul, item );
            } );
        },

        _renderItemData: function( ul, item ) {
            return this._renderItem( ul, item ).data( "ui-autocomplete-item", item );
        },

        _renderItem: function( ul, item ) {
            return $( "<li>" )
                .append( $( "<div>" ).text( item.label ) )
                .appendTo( ul );
        },

        _move: function( direction, event ) {
            if ( !this.menu.element.is( ":visible" ) ) {
                this.search( null, event );
                return;
            }
            if ( this.menu.isFirstItem() && /^previous/.test( direction ) ||
                this.menu.isLastItem() && /^next/.test( direction ) ) {

                if ( !this.isMultiLine ) {
                    this._value( this.term );
                }

                this.menu.blur();
                return;
            }
            this.menu[ direction ]( event );
        },

        widget: function() {
            return this.menu.element;
        },

        _value: function() {
            return this.valueMethod.apply( this.element, arguments );
        },

        _keyEvent: function( keyEvent, event ) {
            if ( !this.isMultiLine || this.menu.element.is( ":visible" ) ) {
                this._move( keyEvent, event );

                // Prevents moving cursor to beginning/end of the text field in some browsers
                event.preventDefault();
            }
        },

        // Support: Chrome <=50
        // We should be able to just use this.element.prop( "isContentEditable" )
        // but hidden elements always report false in Chrome.
        // https://code.google.com/p/chromium/issues/detail?id=313082
        _isContentEditable: function( element ) {
            if ( !element.length ) {
                return false;
            }

            var editable = element.prop( "contentEditable" );

            if ( editable === "inherit" ) {
                return this._isContentEditable( element.parent() );
            }

            return editable === "true";
        }
    } );

    $.extend( $.ui.autocomplete, {
        escapeRegex: function( value ) {
            return value.replace( /[\-\[\]{}()*+?.,\\\^$|#\s]/g, "\\$&" );
        },
        filter: function( array, term ) {
            var matcher = new RegExp( $.ui.autocomplete.escapeRegex( term ), "i" );
            return $.grep( array, function( value ) {
                return matcher.test( value.label || value.value || value );
            } );
        }
    } );

// Live region extension, adding a `messages` option
// NOTE: This is an experimental API. We are still investigating
// a full solution for string manipulation and internationalization.
    $.widget( "ui.autocomplete", $.ui.autocomplete, {
        options: {
            messages: {
                noResults: "No search results.",
                results: function( amount ) {
                    return amount + ( amount > 1 ? " results are" : " result is" ) +
                        " available, use up and down arrow keys to navigate.";
                }
            }
        },

        __response: function( content ) {
            var message;
            this._superApply( arguments );
            if ( this.options.disabled || this.cancelSearch ) {
                return;
            }
            if ( content && content.length ) {
                message = this.options.messages.results( content.length );
            } else {
                message = this.options.messages.noResults;
            }
            this.liveRegion.children().hide();
            $( "<div>" ).text( message ).appendTo( this.liveRegion );
        }
    } );

    var widgetsAutocomplete = $.ui.autocomplete;

    /*!
     * jQuery UI Menu 1.12.1
     * http://jqueryui.com
     *
     * Copyright jQuery Foundation and other contributors
     * Released under the MIT license.
     * http://jquery.org/license
     */

    var widgetsMenu = $.widget( "ui.menu", {
        version: "1.12.1",
        defaultElement: "<ul>",
        delay: 300,
        options: {
            icons: {
                submenu: "ui-icon-caret-1-e"
            },
            items: "> *",
            menus: "ul",
            position: {
                my: "left top",
                at: "right top"
            },
            role: "menu",

            // Callbacks
            blur: null,
            focus: null,
            select: null
        },

        _create: function() {
            this.activeMenu = this.element;

            // Flag used to prevent firing of the click handler
            // as the event bubbles up through nested menus
            this.mouseHandled = false;
            this.element
                .uniqueId()
                .attr( {
                    role: this.options.role,
                    tabIndex: 0
                } );

            this._addClass( "ui-menu", "ui-widget ui-widget-content" );
            this._on( {

                // Prevent focus from sticking to links inside menu after clicking
                // them (focus should always stay on UL during navigation).
                "mousedown .ui-menu-item": function( event ) {
                    event.preventDefault();
                },
                "click .ui-menu-item": function( event ) {
                    var target = $( event.target );
                    var active = $( $.ui.safeActiveElement( this.document[ 0 ] ) );
                    if ( !this.mouseHandled && target.not( ".ui-state-disabled" ).length ) {
                        this.select( event );

                        // Only set the mouseHandled flag if the event will bubble, see #9469.
                        if ( !event.isPropagationStopped() ) {
                            this.mouseHandled = true;
                        }

                        // Open submenu on click
                        if ( target.has( ".ui-menu" ).length ) {
                            this.expand( event );
                        } else if ( !this.element.is( ":focus" ) &&
                            active.closest( ".ui-menu" ).length ) {

                            // Redirect focus to the menu
                            this.element.trigger( "focus", [ true ] );

                            // If the active item is on the top level, let it stay active.
                            // Otherwise, blur the active item since it is no longer visible.
                            if ( this.active && this.active.parents( ".ui-menu" ).length === 1 ) {
                                clearTimeout( this.timer );
                            }
                        }
                    }
                },
                "mouseenter .ui-menu-item": function( event ) {

                    // Ignore mouse events while typeahead is active, see #10458.
                    // Prevents focusing the wrong item when typeahead causes a scroll while the mouse
                    // is over an item in the menu
                    if ( this.previousFilter ) {
                        return;
                    }

                    var actualTarget = $( event.target ).closest( ".ui-menu-item" ),
                        target = $( event.currentTarget );

                    // Ignore bubbled events on parent items, see #11641
                    if ( actualTarget[ 0 ] !== target[ 0 ] ) {
                        return;
                    }

                    // Remove ui-state-active class from siblings of the newly focused menu item
                    // to avoid a jump caused by adjacent elements both having a class with a border
                    this._removeClass( target.siblings().children( ".ui-state-active" ),
                        null, "ui-state-active" );
                    this.focus( event, target );
                },
                mouseleave: "collapseAll",
                "mouseleave .ui-menu": "collapseAll",
                focus: function( event, keepActiveItem ) {

                    // If there's already an active item, keep it active
                    // If not, activate the first item
                    var item = this.active || this.element.find( this.options.items ).eq( 0 );

                    if ( !keepActiveItem ) {
                        this.focus( event, item );
                    }
                },
                blur: function( event ) {
                    this._delay( function() {
                        var notContained = !$.contains(
                            this.element[ 0 ],
                            $.ui.safeActiveElement( this.document[ 0 ] )
                        );
                        if ( notContained ) {
                            this.collapseAll( event );
                        }
                    } );
                },
                keydown: "_keydown"
            } );

            this.refresh();

            // Clicks outside of a menu collapse any open menus
            this._on( this.document, {
                click: function( event ) {
                    if ( this._closeOnDocumentClick( event ) ) {
                        this.collapseAll( event );
                    }

                    // Reset the mouseHandled flag
                    this.mouseHandled = false;
                }
            } );
        },

        _destroy: function() {
            var items = this.element.find( ".ui-menu-item" )
                    .removeAttr( "role aria-disabled" ),
                submenus = items.children( ".ui-menu-item-wrapper" )
                    .removeUniqueId()
                    .removeAttr( "tabIndex role aria-haspopup" );

            // Destroy (sub)menus
            this.element
                .removeAttr( "aria-activedescendant" )
                .find( ".ui-menu" ).addBack()
                .removeAttr( "role aria-labelledby aria-expanded aria-hidden aria-disabled " +
                    "tabIndex" )
                .removeUniqueId()
                .show();

            submenus.children().each( function() {
                var elem = $( this );
                if ( elem.data( "ui-menu-submenu-caret" ) ) {
                    elem.remove();
                }
            } );
        },

        _keydown: function( event ) {
            var match, prev, character, skip,
                preventDefault = true;

            switch ( event.keyCode ) {
                case $.ui.keyCode.PAGE_UP:
                    this.previousPage( event );
                    break;
                case $.ui.keyCode.PAGE_DOWN:
                    this.nextPage( event );
                    break;
                case $.ui.keyCode.HOME:
                    this._move( "first", "first", event );
                    break;
                case $.ui.keyCode.END:
                    this._move( "last", "last", event );
                    break;
                case $.ui.keyCode.UP:
                    this.previous( event );
                    break;
                case $.ui.keyCode.DOWN:
                    this.next( event );
                    break;
                case $.ui.keyCode.LEFT:
                    this.collapse( event );
                    break;
                case $.ui.keyCode.RIGHT:
                    if ( this.active && !this.active.is( ".ui-state-disabled" ) ) {
                        this.expand( event );
                    }
                    break;
                case $.ui.keyCode.ENTER:
                case $.ui.keyCode.SPACE:
                    this._activate( event );
                    break;
                case $.ui.keyCode.ESCAPE:
                    this.collapse( event );
                    break;
                default:
                    preventDefault = false;
                    prev = this.previousFilter || "";
                    skip = false;

                    // Support number pad values
                    character = event.keyCode >= 96 && event.keyCode <= 105 ?
                        ( event.keyCode - 96 ).toString() : String.fromCharCode( event.keyCode );

                    clearTimeout( this.filterTimer );

                    if ( character === prev ) {
                        skip = true;
                    } else {
                        character = prev + character;
                    }

                    match = this._filterMenuItems( character );
                    match = skip && match.index( this.active.next() ) !== -1 ?
                        this.active.nextAll( ".ui-menu-item" ) :
                        match;

                    // If no matches on the current filter, reset to the last character pressed
                    // to move down the menu to the first item that starts with that character
                    if ( !match.length ) {
                        character = String.fromCharCode( event.keyCode );
                        match = this._filterMenuItems( character );
                    }

                    if ( match.length ) {
                        this.focus( event, match );
                        this.previousFilter = character;
                        this.filterTimer = this._delay( function() {
                            delete this.previousFilter;
                        }, 1000 );
                    } else {
                        delete this.previousFilter;
                    }
            }

            if ( preventDefault ) {
                event.preventDefault();
            }
        },

        _activate: function( event ) {
            if ( this.active && !this.active.is( ".ui-state-disabled" ) ) {
                if ( this.active.children( "[aria-haspopup='true']" ).length ) {
                    this.expand( event );
                } else {
                    this.select( event );
                }
            }
        },

        refresh: function() {
            var menus, items, newSubmenus, newItems, newWrappers,
                that = this,
                icon = this.options.icons.submenu,
                submenus = this.element.find( this.options.menus );

            this._toggleClass( "ui-menu-icons", null, !!this.element.find( ".ui-icon" ).length );

            // Initialize nested menus
            newSubmenus = submenus.filter( ":not(.ui-menu)" )
                .hide()
                .attr( {
                    role: this.options.role,
                    "aria-hidden": "true",
                    "aria-expanded": "false"
                } )
                .each( function() {
                    var menu = $( this ),
                        item = menu.prev(),
                        submenuCaret = $( "<span>" ).data( "ui-menu-submenu-caret", true );

                    that._addClass( submenuCaret, "ui-menu-icon", "ui-icon " + icon );
                    item
                        .attr( "aria-haspopup", "true" )
                        .prepend( submenuCaret );
                    menu.attr( "aria-labelledby", item.attr( "id" ) );
                } );

            this._addClass( newSubmenus, "ui-menu", "ui-widget ui-widget-content ui-front" );

            menus = submenus.add( this.element );
            items = menus.find( this.options.items );

            // Initialize menu-items containing spaces and/or dashes only as dividers
            items.not( ".ui-menu-item" ).each( function() {
                var item = $( this );
                if ( that._isDivider( item ) ) {
                    that._addClass( item, "ui-menu-divider", "ui-widget-content" );
                }
            } );

            // Don't refresh list items that are already adapted
            newItems = items.not( ".ui-menu-item, .ui-menu-divider" );
            newWrappers = newItems.children()
                .not( ".ui-menu" )
                .uniqueId()
                .attr( {
                    tabIndex: -1,
                    role: this._itemRole()
                } );
            this._addClass( newItems, "ui-menu-item" )
                ._addClass( newWrappers, "ui-menu-item-wrapper" );

            // Add aria-disabled attribute to any disabled menu item
            items.filter( ".ui-state-disabled" ).attr( "aria-disabled", "true" );

            // If the active item has been removed, blur the menu
            if ( this.active && !$.contains( this.element[ 0 ], this.active[ 0 ] ) ) {
                this.blur();
            }
        },

        _itemRole: function() {
            return {
                menu: "menuitem",
                listbox: "option"
            }[ this.options.role ];
        },

        _setOption: function( key, value ) {
            if ( key === "icons" ) {
                var icons = this.element.find( ".ui-menu-icon" );
                this._removeClass( icons, null, this.options.icons.submenu )
                    ._addClass( icons, null, value.submenu );
            }
            this._super( key, value );
        },

        _setOptionDisabled: function( value ) {
            this._super( value );

            this.element.attr( "aria-disabled", String( value ) );
            this._toggleClass( null, "ui-state-disabled", !!value );
        },

        focus: function( event, item ) {
            var nested, focused, activeParent;
            this.blur( event, event && event.type === "focus" );

            this._scrollIntoView( item );

            this.active = item.first();

            focused = this.active.children( ".ui-menu-item-wrapper" );
            this._addClass( focused, null, "ui-state-active" );

            // Only update aria-activedescendant if there's a role
            // otherwise we assume focus is managed elsewhere
            if ( this.options.role ) {
                this.element.attr( "aria-activedescendant", focused.attr( "id" ) );
            }

            // Highlight active parent menu item, if any
            activeParent = this.active
                .parent()
                .closest( ".ui-menu-item" )
                .children( ".ui-menu-item-wrapper" );
            this._addClass( activeParent, null, "ui-state-active" );

            if ( event && event.type === "keydown" ) {
                this._close();
            } else {
                this.timer = this._delay( function() {
                    this._close();
                }, this.delay );
            }

            nested = item.children( ".ui-menu" );
            if ( nested.length && event && ( /^mouse/.test( event.type ) ) ) {
                this._startOpening( nested );
            }
            this.activeMenu = item.parent();

            this._trigger( "focus", event, { item: item } );
        },

        _scrollIntoView: function( item ) {
            var borderTop, paddingTop, offset, scroll, elementHeight, itemHeight;
            if ( this._hasScroll() ) {
                borderTop = parseFloat( $.css( this.activeMenu[ 0 ], "borderTopWidth" ) ) || 0;
                paddingTop = parseFloat( $.css( this.activeMenu[ 0 ], "paddingTop" ) ) || 0;
                offset = item.offset().top - this.activeMenu.offset().top - borderTop - paddingTop;
                scroll = this.activeMenu.scrollTop();
                elementHeight = this.activeMenu.height();
                itemHeight = item.outerHeight();

                if ( offset < 0 ) {
                    this.activeMenu.scrollTop( scroll + offset );
                } else if ( offset + itemHeight > elementHeight ) {
                    this.activeMenu.scrollTop( scroll + offset - elementHeight + itemHeight );
                }
            }
        },

        blur: function( event, fromFocus ) {
            if ( !fromFocus ) {
                clearTimeout( this.timer );
            }

            if ( !this.active ) {
                return;
            }

            this._removeClass( this.active.children( ".ui-menu-item-wrapper" ),
                null, "ui-state-active" );

            this._trigger( "blur", event, { item: this.active } );
            this.active = null;
        },

        _startOpening: function( submenu ) {
            clearTimeout( this.timer );

            // Don't open if already open fixes a Firefox bug that caused a .5 pixel
            // shift in the submenu position when mousing over the caret icon
            if ( submenu.attr( "aria-hidden" ) !== "true" ) {
                return;
            }

            this.timer = this._delay( function() {
                this._close();
                this._open( submenu );
            }, this.delay );
        },

        _open: function( submenu ) {
            var position = $.extend( {
                of: this.active
            }, this.options.position );

            clearTimeout( this.timer );
            this.element.find( ".ui-menu" ).not( submenu.parents( ".ui-menu" ) )
                .hide()
                .attr( "aria-hidden", "true" );

            submenu
                .show()
                .removeAttr( "aria-hidden" )
                .attr( "aria-expanded", "true" )
                .position( position );
        },

        collapseAll: function( event, all ) {
            clearTimeout( this.timer );
            this.timer = this._delay( function() {

                // If we were passed an event, look for the submenu that contains the event
                var currentMenu = all ? this.element :
                    $( event && event.target ).closest( this.element.find( ".ui-menu" ) );

                // If we found no valid submenu ancestor, use the main menu to close all
                // sub menus anyway
                if ( !currentMenu.length ) {
                    currentMenu = this.element;
                }

                this._close( currentMenu );

                this.blur( event );

                // Work around active item staying active after menu is blurred
                this._removeClass( currentMenu.find( ".ui-state-active" ), null, "ui-state-active" );

                this.activeMenu = currentMenu;
            }, this.delay );
        },

        // With no arguments, closes the currently active menu - if nothing is active
        // it closes all menus.  If passed an argument, it will search for menus BELOW
        _close: function( startMenu ) {
            if ( !startMenu ) {
                startMenu = this.active ? this.active.parent() : this.element;
            }

            startMenu.find( ".ui-menu" )
                .hide()
                .attr( "aria-hidden", "true" )
                .attr( "aria-expanded", "false" );
        },

        _closeOnDocumentClick: function( event ) {
            return !$( event.target ).closest( ".ui-menu" ).length;
        },

        _isDivider: function( item ) {

            // Match hyphen, em dash, en dash
            return !/[^\-\u2014\u2013\s]/.test( item.text() );
        },

        collapse: function( event ) {
            var newItem = this.active &&
                this.active.parent().closest( ".ui-menu-item", this.element );
            if ( newItem && newItem.length ) {
                this._close();
                this.focus( event, newItem );
            }
        },

        expand: function( event ) {
            var newItem = this.active &&
                this.active
                    .children( ".ui-menu " )
                    .find( this.options.items )
                    .first();

            if ( newItem && newItem.length ) {
                this._open( newItem.parent() );

                // Delay so Firefox will not hide activedescendant change in expanding submenu from AT
                this._delay( function() {
                    this.focus( event, newItem );
                } );
            }
        },

        next: function( event ) {
            this._move( "next", "first", event );
        },

        previous: function( event ) {
            this._move( "prev", "last", event );
        },

        isFirstItem: function() {
            return this.active && !this.active.prevAll( ".ui-menu-item" ).length;
        },

        isLastItem: function() {
            return this.active && !this.active.nextAll( ".ui-menu-item" ).length;
        },

        _move: function( direction, filter, event ) {
            var next;
            if ( this.active ) {
                if ( direction === "first" || direction === "last" ) {
                    next = this.active
                        [ direction === "first" ? "prevAll" : "nextAll" ]( ".ui-menu-item" )
                        .eq( -1 );
                } else {
                    next = this.active
                        [ direction + "All" ]( ".ui-menu-item" )
                        .eq( 0 );
                }
            }
            if ( !next || !next.length || !this.active ) {
                next = this.activeMenu.find( this.options.items )[ filter ]();
            }

            this.focus( event, next );
        },

        nextPage: function( event ) {
            var item, base, height;

            if ( !this.active ) {
                this.next( event );
                return;
            }
            if ( this.isLastItem() ) {
                return;
            }
            if ( this._hasScroll() ) {
                base = this.active.offset().top;
                height = this.element.height();
                this.active.nextAll( ".ui-menu-item" ).each( function() {
                    item = $( this );
                    return item.offset().top - base - height < 0;
                } );

                this.focus( event, item );
            } else {
                this.focus( event, this.activeMenu.find( this.options.items )
                    [ !this.active ? "first" : "last" ]() );
            }
        },

        previousPage: function( event ) {
            var item, base, height;
            if ( !this.active ) {
                this.next( event );
                return;
            }
            if ( this.isFirstItem() ) {
                return;
            }
            if ( this._hasScroll() ) {
                base = this.active.offset().top;
                height = this.element.height();
                this.active.prevAll( ".ui-menu-item" ).each( function() {
                    item = $( this );
                    return item.offset().top - base + height > 0;
                } );

                this.focus( event, item );
            } else {
                this.focus( event, this.activeMenu.find( this.options.items ).first() );
            }
        },

        _hasScroll: function() {
            return this.element.outerHeight() < this.element.prop( "scrollHeight" );
        },

        select: function( event ) {

            // TODO: It should never be possible to not have an active item at this
            // point, but the tests don't trigger mouseenter before click.
            this.active = this.active || $( event.target ).closest( ".ui-menu-item" );
            var ui = { item: this.active };
            if ( !this.active.has( ".ui-menu" ).length ) {
                this.collapseAll( event, true );
            }
            this._trigger( "select", event, ui );
        },

        _filterMenuItems: function( character ) {
            var escapedCharacter = character.replace( /[\-\[\]{}()*+?.,\\\^$|#\s]/g, "\\$&" ),
                regex = new RegExp( "^" + escapedCharacter, "i" );

            return this.activeMenu
                .find( this.options.items )

                // Only match on items, not dividers or other content (#10571)
                .filter( ".ui-menu-item" )
                .filter( function() {
                    return regex.test(
                        $.trim( $( this ).children( ".ui-menu-item-wrapper" ).text() ) );
                } );
        }
    } );

    /*!
     * jQuery UI Keycode 1.12.1
     * http://jqueryui.com
     *
     * Copyright jQuery Foundation and other contributors
     * Released under the MIT license.
     * http://jquery.org/license
     */

    var keycode = $.ui.keyCode = {
        BACKSPACE: 8,
        COMMA: 188,
        DELETE: 46,
        DOWN: 40,
        END: 35,
        ENTER: 13,
        ESCAPE: 27,
        HOME: 36,
        LEFT: 37,
        PAGE_DOWN: 34,
        PAGE_UP: 33,
        PERIOD: 190,
        RIGHT: 39,
        SPACE: 32,
        TAB: 9,
        UP: 38
    };


// jscs:disable maximumLineLength
    /* jscs:disable requireCamelCaseOrUpperCaseIdentifiers */
    /*!
     * jQuery UI Datepicker 1.12.1
     * http://jqueryui.com
     *
     * Copyright jQuery Foundation and other contributors
     * Released under the MIT license.
     * http://jquery.org/license
     */

//>>label: Datepicker
//>>group: Widgets
//>>description: Displays a calendar from an input or inline for selecting dates.
//>>docs: http://api.jqueryui.com/datepicker/
//>>demos: http://jqueryui.com/datepicker/
//>>css.structure: ../../themes/base/core.css
//>>css.structure: ../../themes/base/datepicker.css
//>>css.theme: ../../themes/base/theme.css



    $.extend( $.ui, { datepicker: { version: "1.12.1" } } );

    var datepicker_instActive;

    function datepicker_getZindex( elem ) {
        var position, value;
        while ( elem.length && elem[ 0 ] !== document ) {

            // Ignore z-index if position is set to a value where z-index is ignored by the browser
            // This makes behavior of this function consistent across browsers
            // WebKit always returns auto if the element is positioned
            position = elem.css( "position" );
            if ( position === "absolute" || position === "relative" || position === "fixed" ) {

                // IE returns 0 when zIndex is not specified
                // other browsers return a string
                // we ignore the case of nested elements with an explicit value of 0
                // <div style="z-index: -10;"><div style="z-index: 0;"></div></div>
                value = parseInt( elem.css( "zIndex" ), 10 );
                if ( !isNaN( value ) && value !== 0 ) {
                    return value;
                }
            }
            elem = elem.parent();
        }

        return 0;
    }
    /* Date picker manager.
       Use the singleton instance of this class, $.datepicker, to interact with the date picker.
       Settings for (groups of) date pickers are maintained in an instance object,
       allowing multiple different settings on the same page. */

    function Datepicker() {
        this._curInst = null; // The current instance in use
        this._keyEvent = false; // If the last event was a key event
        this._disabledInputs = []; // List of date picker inputs that have been disabled
        this._datepickerShowing = false; // True if the popup picker is showing , false if not
        this._inDialog = false; // True if showing within a "dialog", false if not
        this._mainDivId = "ui-datepicker-div"; // The ID of the main datepicker division
        this._inlineClass = "ui-datepicker-inline"; // The name of the inline marker class
        this._appendClass = "ui-datepicker-append"; // The name of the append marker class
        this._triggerClass = "ui-datepicker-trigger"; // The name of the trigger marker class
        this._dialogClass = "ui-datepicker-dialog"; // The name of the dialog marker class
        this._disableClass = "ui-datepicker-disabled"; // The name of the disabled covering marker class
        this._unselectableClass = "ui-datepicker-unselectable"; // The name of the unselectable cell marker class
        this._currentClass = "ui-datepicker-current-day"; // The name of the current day marker class
        this._dayOverClass = "ui-datepicker-days-cell-over"; // The name of the day hover marker class
        this.regional = []; // Available regional settings, indexed by language code
        this.regional[ "" ] = { // Default regional settings
            closeText: "Done", // Display text for close link
            prevText: "Prev", // Display text for previous month link
            nextText: "Next", // Display text for next month link
            currentText: "Today", // Display text for current month link
            monthNames: [ "January","February","March","April","May","June",
                "July","August","September","October","November","December" ], // Names of months for drop-down and formatting
            monthNamesShort: [ "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" ], // For formatting
            dayNames: [ "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday" ], // For formatting
            dayNamesShort: [ "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" ], // For formatting
            dayNamesMin: [ "Su","Mo","Tu","We","Th","Fr","Sa" ], // Column headings for days starting at Sunday
            weekHeader: "Wk", // Column header for week of the year
            dateFormat: "mm/dd/yy", // See format options on parseDate
            firstDay: 0, // The first day of the week, Sun = 0, Mon = 1, ...
            isRTL: false, // True if right-to-left language, false if left-to-right
            showMonthAfterYear: false, // True if the year select precedes month, false for month then year
            yearSuffix: "" // Additional text to append to the year in the month headers
        };
        this._defaults = { // Global defaults for all the date picker instances
            showOn: "focus", // "focus" for popup on focus,
            // "button" for trigger button, or "both" for either
            showAnim: "fadeIn", // Name of jQuery animation for popup
            showOptions: {}, // Options for enhanced animations
            defaultDate: null, // Used when field is blank: actual date,
            // +/-number for offset from today, null for today
            appendText: "", // Display text following the input box, e.g. showing the format
            buttonText: "...", // Text for trigger button
            buttonImage: "", // URL for trigger button image
            buttonImageOnly: false, // True if the image appears alone, false if it appears on a button
            hideIfNoPrevNext: false, // True to hide next/previous month links
            // if not applicable, false to just disable them
            navigationAsDateFormat: false, // True if date formatting applied to prev/today/next links
            gotoCurrent: false, // True if today link goes back to current selection instead
            changeMonth: false, // True if month can be selected directly, false if only prev/next
            changeYear: false, // True if year can be selected directly, false if only prev/next
            yearRange: "c-10:c+10", // Range of years to display in drop-down,
            // either relative to today's year (-nn:+nn), relative to currently displayed year
            // (c-nn:c+nn), absolute (nnnn:nnnn), or a combination of the above (nnnn:-n)
            showOtherMonths: false, // True to show dates in other months, false to leave blank
            selectOtherMonths: false, // True to allow selection of dates in other months, false for unselectable
            showWeek: false, // True to show week of the year, false to not show it
            calculateWeek: this.iso8601Week, // How to calculate the week of the year,
            // takes a Date and returns the number of the week for it
            shortYearCutoff: "+10", // Short year values < this are in the current century,
            // > this are in the previous century,
            // string value starting with "+" for current year + value
            minDate: null, // The earliest selectable date, or null for no limit
            maxDate: null, // The latest selectable date, or null for no limit
            duration: "fast", // Duration of display/closure
            beforeShowDay: null, // Function that takes a date and returns an array with
            // [0] = true if selectable, false if not, [1] = custom CSS class name(s) or "",
            // [2] = cell title (optional), e.g. $.datepicker.noWeekends
            beforeShow: null, // Function that takes an input field and
            // returns a set of custom settings for the date picker
            onSelect: null, // Define a callback function when a date is selected
            onChangeMonthYear: null, // Define a callback function when the month or year is changed
            onClose: null, // Define a callback function when the datepicker is closed
            numberOfMonths: 1, // Number of months to show at a time
            showCurrentAtPos: 0, // The position in multipe months at which to show the current month (starting at 0)
            stepMonths: 1, // Number of months to step back/forward
            stepBigMonths: 12, // Number of months to step back/forward for the big links
            altField: "", // Selector for an alternate field to store selected dates into
            altFormat: "", // The date format to use for the alternate field
            constrainInput: true, // The input is constrained by the current date format
            showButtonPanel: false, // True to show button panel, false to not show it
            autoSize: false, // True to size the input for the date format, false to leave as is
            disabled: false // The initial disabled state
        };
        $.extend( this._defaults, this.regional[ "" ] );
        this.regional.en = $.extend( true, {}, this.regional[ "" ] );
        this.regional[ "en-US" ] = $.extend( true, {}, this.regional.en );
        this.dpDiv = datepicker_bindHover( $( "<div id='" + this._mainDivId + "' class='ui-datepicker ui-widget ui-widget-content ui-helper-clearfix ui-corner-all'></div>" ) );
    }

    $.extend( Datepicker.prototype, {
        /* Class name added to elements to indicate already configured with a date picker. */
        markerClassName: "hasDatepicker",

        //Keep track of the maximum number of rows displayed (see #7043)
        maxRows: 4,

        // TODO rename to "widget" when switching to widget factory
        _widgetDatepicker: function() {
            return this.dpDiv;
        },

        /* Override the default settings for all instances of the date picker.
         * @param  settings  object - the new settings to use as defaults (anonymous object)
         * @return the manager object
         */
        setDefaults: function( settings ) {
            datepicker_extendRemove( this._defaults, settings || {} );
            return this;
        },

        /* Attach the date picker to a jQuery selection.
         * @param  target	element - the target input field or division or span
         * @param  settings  object - the new settings to use for this date picker instance (anonymous)
         */
        _attachDatepicker: function( target, settings ) {
            var nodeName, inline, inst;
            nodeName = target.nodeName.toLowerCase();
            inline = ( nodeName === "div" || nodeName === "span" );
            if ( !target.id ) {
                this.uuid += 1;
                target.id = "dp" + this.uuid;
            }
            inst = this._newInst( $( target ), inline );
            inst.settings = $.extend( {}, settings || {} );
            if ( nodeName === "input" ) {
                this._connectDatepicker( target, inst );
            } else if ( inline ) {
                this._inlineDatepicker( target, inst );
            }
        },

        /* Create a new instance object. */
        _newInst: function( target, inline ) {
            var id = target[ 0 ].id.replace( /([^A-Za-z0-9_\-])/g, "\\\\$1" ); // escape jQuery meta chars
            return { id: id, input: target, // associated target
                selectedDay: 0, selectedMonth: 0, selectedYear: 0, // current selection
                drawMonth: 0, drawYear: 0, // month being drawn
                inline: inline, // is datepicker inline or not
                dpDiv: ( !inline ? this.dpDiv : // presentation div
                    datepicker_bindHover( $( "<div class='" + this._inlineClass + " ui-datepicker ui-widget ui-widget-content ui-helper-clearfix ui-corner-all'></div>" ) ) ) };
        },

        /* Attach the date picker to an input field. */
        _connectDatepicker: function( target, inst ) {
            var input = $( target );
            inst.append = $( [] );
            inst.trigger = $( [] );
            if ( input.hasClass( this.markerClassName ) ) {
                return;
            }
            this._attachments( input, inst );
            input.addClass( this.markerClassName ).on( "keydown", this._doKeyDown ).
            on( "keypress", this._doKeyPress ).on( "keyup", this._doKeyUp );
            this._autoSize( inst );
            $.data( target, "datepicker", inst );

            //If disabled option is true, disable the datepicker once it has been attached to the input (see ticket #5665)
            if ( inst.settings.disabled ) {
                this._disableDatepicker( target );
            }
        },

        /* Make attachments based on settings. */
        _attachments: function( input, inst ) {
            var showOn, buttonText, buttonImage,
                appendText = this._get( inst, "appendText" ),
                isRTL = this._get( inst, "isRTL" );

            if ( inst.append ) {
                inst.append.remove();
            }
            if ( appendText ) {
                inst.append = $( "<span class='" + this._appendClass + "'>" + appendText + "</span>" );
                input[ isRTL ? "before" : "after" ]( inst.append );
            }

            input.off( "focus", this._showDatepicker );

            if ( inst.trigger ) {
                inst.trigger.remove();
            }

            showOn = this._get( inst, "showOn" );
            if ( showOn === "focus" || showOn === "both" ) { // pop-up date picker when in the marked field
                input.on( "focus", this._showDatepicker );
            }
            if ( showOn === "button" || showOn === "both" ) { // pop-up date picker when button clicked
                buttonText = this._get( inst, "buttonText" );
                buttonImage = this._get( inst, "buttonImage" );
                inst.trigger = $( this._get( inst, "buttonImageOnly" ) ?
                    $( "<img/>" ).addClass( this._triggerClass ).
                    attr( { src: buttonImage, alt: buttonText, title: buttonText } ) :
                    $( "<button type='button'></button>" ).addClass( this._triggerClass ).
                    html( !buttonImage ? buttonText : $( "<img/>" ).attr(
                        { src:buttonImage, alt:buttonText, title:buttonText } ) ) );
                input[ isRTL ? "before" : "after" ]( inst.trigger );
                inst.trigger.on( "click", function() {
                    if ( $.datepicker._datepickerShowing && $.datepicker._lastInput === input[ 0 ] ) {
                        $.datepicker._hideDatepicker();
                    } else if ( $.datepicker._datepickerShowing && $.datepicker._lastInput !== input[ 0 ] ) {
                        $.datepicker._hideDatepicker();
                        $.datepicker._showDatepicker( input[ 0 ] );
                    } else {
                        $.datepicker._showDatepicker( input[ 0 ] );
                    }
                    return false;
                } );
            }
        },

        /* Apply the maximum length for the date format. */
        _autoSize: function( inst ) {
            if ( this._get( inst, "autoSize" ) && !inst.inline ) {
                var findMax, max, maxI, i,
                    date = new Date( 2009, 12 - 1, 20 ), // Ensure double digits
                    dateFormat = this._get( inst, "dateFormat" );

                if ( dateFormat.match( /[DM]/ ) ) {
                    findMax = function( names ) {
                        max = 0;
                        maxI = 0;
                        for ( i = 0; i < names.length; i++ ) {
                            if ( names[ i ].length > max ) {
                                max = names[ i ].length;
                                maxI = i;
                            }
                        }
                        return maxI;
                    };
                    date.setMonth( findMax( this._get( inst, ( dateFormat.match( /MM/ ) ?
                        "monthNames" : "monthNamesShort" ) ) ) );
                    date.setDate( findMax( this._get( inst, ( dateFormat.match( /DD/ ) ?
                        "dayNames" : "dayNamesShort" ) ) ) + 20 - date.getDay() );
                }
                inst.input.attr( "size", this._formatDate( inst, date ).length );
            }
        },

        /* Attach an inline date picker to a div. */
        _inlineDatepicker: function( target, inst ) {
            var divSpan = $( target );
            if ( divSpan.hasClass( this.markerClassName ) ) {
                return;
            }
            divSpan.addClass( this.markerClassName ).append( inst.dpDiv );
            $.data( target, "datepicker", inst );
            this._setDate( inst, this._getDefaultDate( inst ), true );
            this._updateDatepicker( inst );
            this._updateAlternate( inst );

            //If disabled option is true, disable the datepicker before showing it (see ticket #5665)
            if ( inst.settings.disabled ) {
                this._disableDatepicker( target );
            }

            // Set display:block in place of inst.dpDiv.show() which won't work on disconnected elements
            // http://bugs.jqueryui.com/ticket/7552 - A Datepicker created on a detached div has zero height
            inst.dpDiv.css( "display", "block" );
        },

        /* Pop-up the date picker in a "dialog" box.
         * @param  input element - ignored
         * @param  date	string or Date - the initial date to display
         * @param  onSelect  function - the function to call when a date is selected
         * @param  settings  object - update the dialog date picker instance's settings (anonymous object)
         * @param  pos int[2] - coordinates for the dialog's position within the screen or
         *					event - with x/y coordinates or
         *					leave empty for default (screen centre)
         * @return the manager object
         */
        _dialogDatepicker: function( input, date, onSelect, settings, pos ) {
            var id, browserWidth, browserHeight, scrollX, scrollY,
                inst = this._dialogInst; // internal instance

            if ( !inst ) {
                this.uuid += 1;
                id = "dp" + this.uuid;
                this._dialogInput = $( "<input type='text' id='" + id +
                    "' style='position: absolute; top: -100px; width: 0px;'/>" );
                this._dialogInput.on( "keydown", this._doKeyDown );
                $( "body" ).append( this._dialogInput );
                inst = this._dialogInst = this._newInst( this._dialogInput, false );
                inst.settings = {};
                $.data( this._dialogInput[ 0 ], "datepicker", inst );
            }
            datepicker_extendRemove( inst.settings, settings || {} );
            date = ( date && date.constructor === Date ? this._formatDate( inst, date ) : date );
            this._dialogInput.val( date );

            this._pos = ( pos ? ( pos.length ? pos : [ pos.pageX, pos.pageY ] ) : null );
            if ( !this._pos ) {
                browserWidth = document.documentElement.clientWidth;
                browserHeight = document.documentElement.clientHeight;
                scrollX = document.documentElement.scrollLeft || document.body.scrollLeft;
                scrollY = document.documentElement.scrollTop || document.body.scrollTop;
                this._pos = // should use actual width/height below
                    [ ( browserWidth / 2 ) - 100 + scrollX, ( browserHeight / 2 ) - 150 + scrollY ];
            }

            // Move input on screen for focus, but hidden behind dialog
            this._dialogInput.css( "left", ( this._pos[ 0 ] + 20 ) + "px" ).css( "top", this._pos[ 1 ] + "px" );
            inst.settings.onSelect = onSelect;
            this._inDialog = true;
            this.dpDiv.addClass( this._dialogClass );
            this._showDatepicker( this._dialogInput[ 0 ] );
            if ( $.blockUI ) {
                $.blockUI( this.dpDiv );
            }
            $.data( this._dialogInput[ 0 ], "datepicker", inst );
            return this;
        },

        /* Detach a datepicker from its control.
         * @param  target	element - the target input field or division or span
         */
        _destroyDatepicker: function( target ) {
            var nodeName,
                $target = $( target ),
                inst = $.data( target, "datepicker" );

            if ( !$target.hasClass( this.markerClassName ) ) {
                return;
            }

            nodeName = target.nodeName.toLowerCase();
            $.removeData( target, "datepicker" );
            if ( nodeName === "input" ) {
                inst.append.remove();
                inst.trigger.remove();
                $target.removeClass( this.markerClassName ).
                off( "focus", this._showDatepicker ).
                off( "keydown", this._doKeyDown ).
                off( "keypress", this._doKeyPress ).
                off( "keyup", this._doKeyUp );
            } else if ( nodeName === "div" || nodeName === "span" ) {
                $target.removeClass( this.markerClassName ).empty();
            }

            if ( datepicker_instActive === inst ) {
                datepicker_instActive = null;
            }
        },

        /* Enable the date picker to a jQuery selection.
         * @param  target	element - the target input field or division or span
         */
        _enableDatepicker: function( target ) {
            var nodeName, inline,
                $target = $( target ),
                inst = $.data( target, "datepicker" );

            if ( !$target.hasClass( this.markerClassName ) ) {
                return;
            }

            nodeName = target.nodeName.toLowerCase();
            if ( nodeName === "input" ) {
                target.disabled = false;
                inst.trigger.filter( "button" ).
                each( function() { this.disabled = false; } ).end().
                filter( "img" ).css( { opacity: "1.0", cursor: "" } );
            } else if ( nodeName === "div" || nodeName === "span" ) {
                inline = $target.children( "." + this._inlineClass );
                inline.children().removeClass( "ui-state-disabled" );
                inline.find( "select.ui-datepicker-month, select.ui-datepicker-year" ).
                prop( "disabled", false );
            }
            this._disabledInputs = $.map( this._disabledInputs,
                function( value ) { return ( value === target ? null : value ); } ); // delete entry
        },

        /* Disable the date picker to a jQuery selection.
         * @param  target	element - the target input field or division or span
         */
        _disableDatepicker: function( target ) {
            var nodeName, inline,
                $target = $( target ),
                inst = $.data( target, "datepicker" );

            if ( !$target.hasClass( this.markerClassName ) ) {
                return;
            }

            nodeName = target.nodeName.toLowerCase();
            if ( nodeName === "input" ) {
                target.disabled = true;
                inst.trigger.filter( "button" ).
                each( function() { this.disabled = true; } ).end().
                filter( "img" ).css( { opacity: "0.5", cursor: "default" } );
            } else if ( nodeName === "div" || nodeName === "span" ) {
                inline = $target.children( "." + this._inlineClass );
                inline.children().addClass( "ui-state-disabled" );
                inline.find( "select.ui-datepicker-month, select.ui-datepicker-year" ).
                prop( "disabled", true );
            }
            this._disabledInputs = $.map( this._disabledInputs,
                function( value ) { return ( value === target ? null : value ); } ); // delete entry
            this._disabledInputs[ this._disabledInputs.length ] = target;
        },

        /* Is the first field in a jQuery collection disabled as a datepicker?
         * @param  target	element - the target input field or division or span
         * @return boolean - true if disabled, false if enabled
         */
        _isDisabledDatepicker: function( target ) {
            if ( !target ) {
                return false;
            }
            for ( var i = 0; i < this._disabledInputs.length; i++ ) {
                if ( this._disabledInputs[ i ] === target ) {
                    return true;
                }
            }
            return false;
        },

        /* Retrieve the instance data for the target control.
         * @param  target  element - the target input field or division or span
         * @return  object - the associated instance data
         * @throws  error if a jQuery problem getting data
         */
        _getInst: function( target ) {
            try {
                return $.data( target, "datepicker" );
            }
            catch ( err ) {
                throw "Missing instance data for this datepicker";
            }
        },

        /* Update or retrieve the settings for a date picker attached to an input field or division.
         * @param  target  element - the target input field or division or span
         * @param  name	object - the new settings to update or
         *				string - the name of the setting to change or retrieve,
         *				when retrieving also "all" for all instance settings or
         *				"defaults" for all global defaults
         * @param  value   any - the new value for the setting
         *				(omit if above is an object or to retrieve a value)
         */
        _optionDatepicker: function( target, name, value ) {
            var settings, date, minDate, maxDate,
                inst = this._getInst( target );

            if ( arguments.length === 2 && typeof name === "string" ) {
                return ( name === "defaults" ? $.extend( {}, $.datepicker._defaults ) :
                    ( inst ? ( name === "all" ? $.extend( {}, inst.settings ) :
                        this._get( inst, name ) ) : null ) );
            }

            settings = name || {};
            if ( typeof name === "string" ) {
                settings = {};
                settings[ name ] = value;
            }

            if ( inst ) {
                if ( this._curInst === inst ) {
                    this._hideDatepicker();
                }

                date = this._getDateDatepicker( target, true );
                minDate = this._getMinMaxDate( inst, "min" );
                maxDate = this._getMinMaxDate( inst, "max" );
                datepicker_extendRemove( inst.settings, settings );

                // reformat the old minDate/maxDate values if dateFormat changes and a new minDate/maxDate isn't provided
                if ( minDate !== null && settings.dateFormat !== undefined && settings.minDate === undefined ) {
                    inst.settings.minDate = this._formatDate( inst, minDate );
                }
                if ( maxDate !== null && settings.dateFormat !== undefined && settings.maxDate === undefined ) {
                    inst.settings.maxDate = this._formatDate( inst, maxDate );
                }
                if ( "disabled" in settings ) {
                    if ( settings.disabled ) {
                        this._disableDatepicker( target );
                    } else {
                        this._enableDatepicker( target );
                    }
                }
                this._attachments( $( target ), inst );
                this._autoSize( inst );
                this._setDate( inst, date );
                this._updateAlternate( inst );
                this._updateDatepicker( inst );
            }
        },

        // Change method deprecated
        _changeDatepicker: function( target, name, value ) {
            this._optionDatepicker( target, name, value );
        },

        /* Redraw the date picker attached to an input field or division.
         * @param  target  element - the target input field or division or span
         */
        _refreshDatepicker: function( target ) {
            var inst = this._getInst( target );
            if ( inst ) {
                this._updateDatepicker( inst );
            }
        },

        /* Set the dates for a jQuery selection.
         * @param  target element - the target input field or division or span
         * @param  date	Date - the new date
         */
        _setDateDatepicker: function( target, date ) {
            var inst = this._getInst( target );
            if ( inst ) {
                this._setDate( inst, date );
                this._updateDatepicker( inst );
                this._updateAlternate( inst );
            }
        },

        /* Get the date(s) for the first entry in a jQuery selection.
         * @param  target element - the target input field or division or span
         * @param  noDefault boolean - true if no default date is to be used
         * @return Date - the current date
         */
        _getDateDatepicker: function( target, noDefault ) {
            var inst = this._getInst( target );
            if ( inst && !inst.inline ) {
                this._setDateFromField( inst, noDefault );
            }
            return ( inst ? this._getDate( inst ) : null );
        },

        /* Handle keystrokes. */
        _doKeyDown: function( event ) {
            var onSelect, dateStr, sel,
                inst = $.datepicker._getInst( event.target ),
                handled = true,
                isRTL = inst.dpDiv.is( ".ui-datepicker-rtl" );

            inst._keyEvent = true;
            if ( $.datepicker._datepickerShowing ) {
                switch ( event.keyCode ) {
                    case 9: $.datepicker._hideDatepicker();
                        handled = false;
                        break; // hide on tab out
                    case 13: sel = $( "td." + $.datepicker._dayOverClass + ":not(." +
                        $.datepicker._currentClass + ")", inst.dpDiv );
                        if ( sel[ 0 ] ) {
                            $.datepicker._selectDay( event.target, inst.selectedMonth, inst.selectedYear, sel[ 0 ] );
                        }

                        onSelect = $.datepicker._get( inst, "onSelect" );
                        if ( onSelect ) {
                            dateStr = $.datepicker._formatDate( inst );

                            // Trigger custom callback
                            onSelect.apply( ( inst.input ? inst.input[ 0 ] : null ), [ dateStr, inst ] );
                        } else {
                            $.datepicker._hideDatepicker();
                        }

                        return false; // don't submit the form
                    case 27: $.datepicker._hideDatepicker();
                        break; // hide on escape
                    case 33: $.datepicker._adjustDate( event.target, ( event.ctrlKey ?
                        -$.datepicker._get( inst, "stepBigMonths" ) :
                        -$.datepicker._get( inst, "stepMonths" ) ), "M" );
                        break; // previous month/year on page up/+ ctrl
                    case 34: $.datepicker._adjustDate( event.target, ( event.ctrlKey ?
                        +$.datepicker._get( inst, "stepBigMonths" ) :
                        +$.datepicker._get( inst, "stepMonths" ) ), "M" );
                        break; // next month/year on page down/+ ctrl
                    case 35: if ( event.ctrlKey || event.metaKey ) {
                        $.datepicker._clearDate( event.target );
                    }
                        handled = event.ctrlKey || event.metaKey;
                        break; // clear on ctrl or command +end
                    case 36: if ( event.ctrlKey || event.metaKey ) {
                        $.datepicker._gotoToday( event.target );
                    }
                        handled = event.ctrlKey || event.metaKey;
                        break; // current on ctrl or command +home
                    case 37: if ( event.ctrlKey || event.metaKey ) {
                        $.datepicker._adjustDate( event.target, ( isRTL ? +1 : -1 ), "D" );
                    }
                        handled = event.ctrlKey || event.metaKey;

                        // -1 day on ctrl or command +left
                        if ( event.originalEvent.altKey ) {
                            $.datepicker._adjustDate( event.target, ( event.ctrlKey ?
                                -$.datepicker._get( inst, "stepBigMonths" ) :
                                -$.datepicker._get( inst, "stepMonths" ) ), "M" );
                        }

                        // next month/year on alt +left on Mac
                        break;
                    case 38: if ( event.ctrlKey || event.metaKey ) {
                        $.datepicker._adjustDate( event.target, -7, "D" );
                    }
                        handled = event.ctrlKey || event.metaKey;
                        break; // -1 week on ctrl or command +up
                    case 39: if ( event.ctrlKey || event.metaKey ) {
                        $.datepicker._adjustDate( event.target, ( isRTL ? -1 : +1 ), "D" );
                    }
                        handled = event.ctrlKey || event.metaKey;

                        // +1 day on ctrl or command +right
                        if ( event.originalEvent.altKey ) {
                            $.datepicker._adjustDate( event.target, ( event.ctrlKey ?
                                +$.datepicker._get( inst, "stepBigMonths" ) :
                                +$.datepicker._get( inst, "stepMonths" ) ), "M" );
                        }

                        // next month/year on alt +right
                        break;
                    case 40: if ( event.ctrlKey || event.metaKey ) {
                        $.datepicker._adjustDate( event.target, +7, "D" );
                    }
                        handled = event.ctrlKey || event.metaKey;
                        break; // +1 week on ctrl or command +down
                    default: handled = false;
                }
            } else if ( event.keyCode === 36 && event.ctrlKey ) { // display the date picker on ctrl+home
                $.datepicker._showDatepicker( this );
            } else {
                handled = false;
            }

            if ( handled ) {
                event.preventDefault();
                event.stopPropagation();
            }
        },

        /* Filter entered characters - based on date format. */
        _doKeyPress: function( event ) {
            var chars, chr,
                inst = $.datepicker._getInst( event.target );

            if ( $.datepicker._get( inst, "constrainInput" ) ) {
                chars = $.datepicker._possibleChars( $.datepicker._get( inst, "dateFormat" ) );
                chr = String.fromCharCode( event.charCode == null ? event.keyCode : event.charCode );
                return event.ctrlKey || event.metaKey || ( chr < " " || !chars || chars.indexOf( chr ) > -1 );
            }
        },

        /* Synchronise manual entry and field/alternate field. */
        _doKeyUp: function( event ) {
            var date,
                inst = $.datepicker._getInst( event.target );

            if ( inst.input.val() !== inst.lastVal ) {
                try {
                    date = $.datepicker.parseDate( $.datepicker._get( inst, "dateFormat" ),
                        ( inst.input ? inst.input.val() : null ),
                        $.datepicker._getFormatConfig( inst ) );

                    if ( date ) { // only if valid
                        $.datepicker._setDateFromField( inst );
                        $.datepicker._updateAlternate( inst );
                        $.datepicker._updateDatepicker( inst );
                    }
                }
                catch ( err ) {
                }
            }
            return true;
        },

        /* Pop-up the date picker for a given input field.
         * If false returned from beforeShow event handler do not show.
         * @param  input  element - the input field attached to the date picker or
         *					event - if triggered by focus
         */
        _showDatepicker: function( input ) {
            input = input.target || input;
            if ( input.nodeName.toLowerCase() !== "input" ) { // find from button/image trigger
                input = $( "input", input.parentNode )[ 0 ];
            }

            if ( $.datepicker._isDisabledDatepicker( input ) || $.datepicker._lastInput === input ) { // already here
                return;
            }

            var inst, beforeShow, beforeShowSettings, isFixed,
                offset, showAnim, duration;

            inst = $.datepicker._getInst( input );
            if ( $.datepicker._curInst && $.datepicker._curInst !== inst ) {
                $.datepicker._curInst.dpDiv.stop( true, true );
                if ( inst && $.datepicker._datepickerShowing ) {
                    $.datepicker._hideDatepicker( $.datepicker._curInst.input[ 0 ] );
                }
            }

            beforeShow = $.datepicker._get( inst, "beforeShow" );
            beforeShowSettings = beforeShow ? beforeShow.apply( input, [ input, inst ] ) : {};
            if ( beforeShowSettings === false ) {
                return;
            }
            datepicker_extendRemove( inst.settings, beforeShowSettings );

            inst.lastVal = null;
            $.datepicker._lastInput = input;
            $.datepicker._setDateFromField( inst );

            if ( $.datepicker._inDialog ) { // hide cursor
                input.value = "";
            }
            if ( !$.datepicker._pos ) { // position below input
                $.datepicker._pos = $.datepicker._findPos( input );
                $.datepicker._pos[ 1 ] += input.offsetHeight; // add the height
            }

            isFixed = false;
            $( input ).parents().each( function() {
                isFixed |= $( this ).css( "position" ) === "fixed";
                return !isFixed;
            } );

            offset = { left: $.datepicker._pos[ 0 ], top: $.datepicker._pos[ 1 ] };
            $.datepicker._pos = null;

            //to avoid flashes on Firefox
            inst.dpDiv.empty();

            // determine sizing offscreen
            inst.dpDiv.css( { position: "absolute", display: "block", top: "-1000px" } );
            $.datepicker._updateDatepicker( inst );

            // fix width for dynamic number of date pickers
            // and adjust position before showing
            offset = $.datepicker._checkOffset( inst, offset, isFixed );
            inst.dpDiv.css( { position: ( $.datepicker._inDialog && $.blockUI ?
                    "static" : ( isFixed ? "fixed" : "absolute" ) ), display: "none",
                left: offset.left + "px", top: offset.top + "px" } );

            if ( !inst.inline ) {
                showAnim = $.datepicker._get( inst, "showAnim" );
                duration = $.datepicker._get( inst, "duration" );
                inst.dpDiv.css( "z-index", datepicker_getZindex( $( input ) ) + 1 );
                $.datepicker._datepickerShowing = true;

                if ( $.effects && $.effects.effect[ showAnim ] ) {
                    inst.dpDiv.show( showAnim, $.datepicker._get( inst, "showOptions" ), duration );
                } else {
                    inst.dpDiv[ showAnim || "show" ]( showAnim ? duration : null );
                }

                if ( $.datepicker._shouldFocusInput( inst ) ) {
                    inst.input.trigger( "focus" );
                }

                $.datepicker._curInst = inst;
            }
        },

        /* Generate the date picker content. */
        _updateDatepicker: function( inst ) {
            this.maxRows = 4; //Reset the max number of rows being displayed (see #7043)
            datepicker_instActive = inst; // for delegate hover events
            inst.dpDiv.empty().append( this._generateHTML( inst ) );
            this._attachHandlers( inst );

            var origyearshtml,
                numMonths = this._getNumberOfMonths( inst ),
                cols = numMonths[ 1 ],
                width = 17,
                activeCell = inst.dpDiv.find( "." + this._dayOverClass + " a" );

            if ( activeCell.length > 0 ) {
                datepicker_handleMouseover.apply( activeCell.get( 0 ) );
            }

            inst.dpDiv.removeClass( "ui-datepicker-multi-2 ui-datepicker-multi-3 ui-datepicker-multi-4" ).width( "" );
            if ( cols > 1 ) {
                inst.dpDiv.addClass( "ui-datepicker-multi-" + cols ).css( "width", ( width * cols ) + "em" );
            }
            inst.dpDiv[ ( numMonths[ 0 ] !== 1 || numMonths[ 1 ] !== 1 ? "add" : "remove" ) +
            "Class" ]( "ui-datepicker-multi" );
            inst.dpDiv[ ( this._get( inst, "isRTL" ) ? "add" : "remove" ) +
            "Class" ]( "ui-datepicker-rtl" );

            if ( inst === $.datepicker._curInst && $.datepicker._datepickerShowing && $.datepicker._shouldFocusInput( inst ) ) {
                inst.input.trigger( "focus" );
            }

            // Deffered render of the years select (to avoid flashes on Firefox)
            if ( inst.yearshtml ) {
                origyearshtml = inst.yearshtml;
                setTimeout( function() {

                    //assure that inst.yearshtml didn't change.
                    if ( origyearshtml === inst.yearshtml && inst.yearshtml ) {
                        inst.dpDiv.find( "select.ui-datepicker-year:first" ).replaceWith( inst.yearshtml );
                    }
                    origyearshtml = inst.yearshtml = null;
                }, 0 );
            }
        },

        // #6694 - don't focus the input if it's already focused
        // this breaks the change event in IE
        // Support: IE and jQuery <1.9
        _shouldFocusInput: function( inst ) {
            return inst.input && inst.input.is( ":visible" ) && !inst.input.is( ":disabled" ) && !inst.input.is( ":focus" );
        },

        /* Check positioning to remain on screen. */
        _checkOffset: function( inst, offset, isFixed ) {
            var dpWidth = inst.dpDiv.outerWidth(),
                dpHeight = inst.dpDiv.outerHeight(),
                inputWidth = inst.input ? inst.input.outerWidth() : 0,
                inputHeight = inst.input ? inst.input.outerHeight() : 0,
                viewWidth = document.documentElement.clientWidth + ( isFixed ? 0 : $( document ).scrollLeft() ),
                viewHeight = document.documentElement.clientHeight + ( isFixed ? 0 : $( document ).scrollTop() );

            offset.left -= ( this._get( inst, "isRTL" ) ? ( dpWidth - inputWidth ) : 0 );
            offset.left -= ( isFixed && offset.left === inst.input.offset().left ) ? $( document ).scrollLeft() : 0;
            offset.top -= ( isFixed && offset.top === ( inst.input.offset().top + inputHeight ) ) ? $( document ).scrollTop() : 0;

            // Now check if datepicker is showing outside window viewport - move to a better place if so.
            offset.left -= Math.min( offset.left, ( offset.left + dpWidth > viewWidth && viewWidth > dpWidth ) ?
                Math.abs( offset.left + dpWidth - viewWidth ) : 0 );
            offset.top -= Math.min( offset.top, ( offset.top + dpHeight > viewHeight && viewHeight > dpHeight ) ?
                Math.abs( dpHeight + inputHeight ) : 0 );

            return offset;
        },

        /* Find an object's position on the screen. */
        _findPos: function( obj ) {
            var position,
                inst = this._getInst( obj ),
                isRTL = this._get( inst, "isRTL" );

            while ( obj && ( obj.type === "hidden" || obj.nodeType !== 1 || $.expr.filters.hidden( obj ) ) ) {
                obj = obj[ isRTL ? "previousSibling" : "nextSibling" ];
            }

            position = $( obj ).offset();
            return [ position.left, position.top ];
        },

        /* Hide the date picker from view.
         * @param  input  element - the input field attached to the date picker
         */
        _hideDatepicker: function( input ) {
            var showAnim, duration, postProcess, onClose,
                inst = this._curInst;

            if ( !inst || ( input && inst !== $.data( input, "datepicker" ) ) ) {
                return;
            }

            if ( this._datepickerShowing ) {
                showAnim = this._get( inst, "showAnim" );
                duration = this._get( inst, "duration" );
                postProcess = function() {
                    $.datepicker._tidyDialog( inst );
                };

                // DEPRECATED: after BC for 1.8.x $.effects[ showAnim ] is not needed
                if ( $.effects && ( $.effects.effect[ showAnim ] || $.effects[ showAnim ] ) ) {
                    inst.dpDiv.hide( showAnim, $.datepicker._get( inst, "showOptions" ), duration, postProcess );
                } else {
                    inst.dpDiv[ ( showAnim === "slideDown" ? "slideUp" :
                        ( showAnim === "fadeIn" ? "fadeOut" : "hide" ) ) ]( ( showAnim ? duration : null ), postProcess );
                }

                if ( !showAnim ) {
                    postProcess();
                }
                this._datepickerShowing = false;

                onClose = this._get( inst, "onClose" );
                if ( onClose ) {
                    onClose.apply( ( inst.input ? inst.input[ 0 ] : null ), [ ( inst.input ? inst.input.val() : "" ), inst ] );
                }

                this._lastInput = null;
                if ( this._inDialog ) {
                    this._dialogInput.css( { position: "absolute", left: "0", top: "-100px" } );
                    if ( $.blockUI ) {
                        $.unblockUI();
                        $( "body" ).append( this.dpDiv );
                    }
                }
                this._inDialog = false;
            }
        },

        /* Tidy up after a dialog display. */
        _tidyDialog: function( inst ) {
            inst.dpDiv.removeClass( this._dialogClass ).off( ".ui-datepicker-calendar" );
        },

        /* Close date picker if clicked elsewhere. */
        _checkExternalClick: function( event ) {
            if ( !$.datepicker._curInst ) {
                return;
            }

            var $target = $( event.target ),
                inst = $.datepicker._getInst( $target[ 0 ] );

            if ( ( ( $target[ 0 ].id !== $.datepicker._mainDivId &&
                $target.parents( "#" + $.datepicker._mainDivId ).length === 0 &&
                !$target.hasClass( $.datepicker.markerClassName ) &&
                !$target.closest( "." + $.datepicker._triggerClass ).length &&
                $.datepicker._datepickerShowing && !( $.datepicker._inDialog && $.blockUI ) ) ) ||
                ( $target.hasClass( $.datepicker.markerClassName ) && $.datepicker._curInst !== inst ) ) {
                $.datepicker._hideDatepicker();
            }
        },

        /* Adjust one of the date sub-fields. */
        _adjustDate: function( id, offset, period ) {
            var target = $( id ),
                inst = this._getInst( target[ 0 ] );

            if ( this._isDisabledDatepicker( target[ 0 ] ) ) {
                return;
            }
            this._adjustInstDate( inst, offset +
                ( period === "M" ? this._get( inst, "showCurrentAtPos" ) : 0 ), // undo positioning
                period );
            this._updateDatepicker( inst );
        },

        /* Action for current link. */
        _gotoToday: function( id ) {
            var date,
                target = $( id ),
                inst = this._getInst( target[ 0 ] );

            if ( this._get( inst, "gotoCurrent" ) && inst.currentDay ) {
                inst.selectedDay = inst.currentDay;
                inst.drawMonth = inst.selectedMonth = inst.currentMonth;
                inst.drawYear = inst.selectedYear = inst.currentYear;
            } else {
                date = new Date();
                inst.selectedDay = date.getDate();
                inst.drawMonth = inst.selectedMonth = date.getMonth();
                inst.drawYear = inst.selectedYear = date.getFullYear();
            }
            this._notifyChange( inst );
            this._adjustDate( target );
        },

        /* Action for selecting a new month/year. */
        _selectMonthYear: function( id, select, period ) {
            var target = $( id ),
                inst = this._getInst( target[ 0 ] );

            inst[ "selected" + ( period === "M" ? "Month" : "Year" ) ] =
                inst[ "draw" + ( period === "M" ? "Month" : "Year" ) ] =
                    parseInt( select.options[ select.selectedIndex ].value, 10 );

            this._notifyChange( inst );
            this._adjustDate( target );
        },

        /* Action for selecting a day. */
        _selectDay: function( id, month, year, td ) {
            var inst,
                target = $( id );

            if ( $( td ).hasClass( this._unselectableClass ) || this._isDisabledDatepicker( target[ 0 ] ) ) {
                return;
            }

            inst = this._getInst( target[ 0 ] );
            inst.selectedDay = inst.currentDay = $( "a", td ).html();
            inst.selectedMonth = inst.currentMonth = month;
            inst.selectedYear = inst.currentYear = year;
            this._selectDate( id, this._formatDate( inst,
                inst.currentDay, inst.currentMonth, inst.currentYear ) );
        },

        /* Erase the input field and hide the date picker. */
        _clearDate: function( id ) {
            var target = $( id );
            this._selectDate( target, "" );
        },

        /* Update the input field with the selected date. */
        _selectDate: function( id, dateStr ) {
            var onSelect,
                target = $( id ),
                inst = this._getInst( target[ 0 ] );

            dateStr = ( dateStr != null ? dateStr : this._formatDate( inst ) );
            if ( inst.input ) {
                inst.input.val( dateStr );
            }
            this._updateAlternate( inst );

            onSelect = this._get( inst, "onSelect" );
            if ( onSelect ) {
                onSelect.apply( ( inst.input ? inst.input[ 0 ] : null ), [ dateStr, inst ] );  // trigger custom callback
            } else if ( inst.input ) {
                inst.input.trigger( "change" ); // fire the change event
            }

            if ( inst.inline ) {
                this._updateDatepicker( inst );
            } else {
                this._hideDatepicker();
                this._lastInput = inst.input[ 0 ];
                if ( typeof( inst.input[ 0 ] ) !== "object" ) {
                    inst.input.trigger( "focus" ); // restore focus
                }
                this._lastInput = null;
            }
        },

        /* Update any alternate field to synchronise with the main field. */
        _updateAlternate: function( inst ) {
            var altFormat, date, dateStr,
                altField = this._get( inst, "altField" );

            if ( altField ) { // update alternate field too
                altFormat = this._get( inst, "altFormat" ) || this._get( inst, "dateFormat" );
                date = this._getDate( inst );
                dateStr = this.formatDate( altFormat, date, this._getFormatConfig( inst ) );
                $( altField ).val( dateStr );
            }
        },

        /* Set as beforeShowDay function to prevent selection of weekends.
         * @param  date  Date - the date to customise
         * @return [boolean, string] - is this date selectable?, what is its CSS class?
         */
        noWeekends: function( date ) {
            var day = date.getDay();
            return [ ( day > 0 && day < 6 ), "" ];
        },

        /* Set as calculateWeek to determine the week of the year based on the ISO 8601 definition.
         * @param  date  Date - the date to get the week for
         * @return  number - the number of the week within the year that contains this date
         */
        iso8601Week: function( date ) {
            var time,
                checkDate = new Date( date.getTime() );

            // Find Thursday of this week starting on Monday
            checkDate.setDate( checkDate.getDate() + 4 - ( checkDate.getDay() || 7 ) );

            time = checkDate.getTime();
            checkDate.setMonth( 0 ); // Compare with Jan 1
            checkDate.setDate( 1 );
            return Math.floor( Math.round( ( time - checkDate ) / 86400000 ) / 7 ) + 1;
        },

        /* Parse a string value into a date object.
         * See formatDate below for the possible formats.
         *
         * @param  format string - the expected format of the date
         * @param  value string - the date in the above format
         * @param  settings Object - attributes include:
         *					shortYearCutoff  number - the cutoff year for determining the century (optional)
         *					dayNamesShort	string[7] - abbreviated names of the days from Sunday (optional)
         *					dayNames		string[7] - names of the days from Sunday (optional)
         *					monthNamesShort string[12] - abbreviated names of the months (optional)
         *					monthNames		string[12] - names of the months (optional)
         * @return  Date - the extracted date value or null if value is blank
         */
        parseDate: function( format, value, settings ) {
            if ( format == null || value == null ) {
                throw "Invalid arguments";
            }

            value = ( typeof value === "object" ? value.toString() : value + "" );
            if ( value === "" ) {
                return null;
            }

            var iFormat, dim, extra,
                iValue = 0,
                shortYearCutoffTemp = ( settings ? settings.shortYearCutoff : null ) || this._defaults.shortYearCutoff,
                shortYearCutoff = ( typeof shortYearCutoffTemp !== "string" ? shortYearCutoffTemp :
                    new Date().getFullYear() % 100 + parseInt( shortYearCutoffTemp, 10 ) ),
                dayNamesShort = ( settings ? settings.dayNamesShort : null ) || this._defaults.dayNamesShort,
                dayNames = ( settings ? settings.dayNames : null ) || this._defaults.dayNames,
                monthNamesShort = ( settings ? settings.monthNamesShort : null ) || this._defaults.monthNamesShort,
                monthNames = ( settings ? settings.monthNames : null ) || this._defaults.monthNames,
                year = -1,
                month = -1,
                day = -1,
                doy = -1,
                literal = false,
                date,

                // Check whether a format character is doubled
                lookAhead = function( match ) {
                    var matches = ( iFormat + 1 < format.length && format.charAt( iFormat + 1 ) === match );
                    if ( matches ) {
                        iFormat++;
                    }
                    return matches;
                },

                // Extract a number from the string value
                getNumber = function( match ) {
                    var isDoubled = lookAhead( match ),
                        size = ( match === "@" ? 14 : ( match === "!" ? 20 :
                            ( match === "y" && isDoubled ? 4 : ( match === "o" ? 3 : 2 ) ) ) ),
                        minSize = ( match === "y" ? size : 1 ),
                        digits = new RegExp( "^\\d{" + minSize + "," + size + "}" ),
                        num = value.substring( iValue ).match( digits );
                    if ( !num ) {
                        throw "Missing number at position " + iValue;
                    }
                    iValue += num[ 0 ].length;
                    return parseInt( num[ 0 ], 10 );
                },

                // Extract a name from the string value and convert to an index
                getName = function( match, shortNames, longNames ) {
                    var index = -1,
                        names = $.map( lookAhead( match ) ? longNames : shortNames, function( v, k ) {
                            return [ [ k, v ] ];
                        } ).sort( function( a, b ) {
                            return -( a[ 1 ].length - b[ 1 ].length );
                        } );

                    $.each( names, function( i, pair ) {
                        var name = pair[ 1 ];
                        if ( value.substr( iValue, name.length ).toLowerCase() === name.toLowerCase() ) {
                            index = pair[ 0 ];
                            iValue += name.length;
                            return false;
                        }
                    } );
                    if ( index !== -1 ) {
                        return index + 1;
                    } else {
                        throw "Unknown name at position " + iValue;
                    }
                },

                // Confirm that a literal character matches the string value
                checkLiteral = function() {
                    if ( value.charAt( iValue ) !== format.charAt( iFormat ) ) {
                        throw "Unexpected literal at position " + iValue;
                    }
                    iValue++;
                };

            for ( iFormat = 0; iFormat < format.length; iFormat++ ) {
                if ( literal ) {
                    if ( format.charAt( iFormat ) === "'" && !lookAhead( "'" ) ) {
                        literal = false;
                    } else {
                        checkLiteral();
                    }
                } else {
                    switch ( format.charAt( iFormat ) ) {
                        case "d":
                            day = getNumber( "d" );
                            break;
                        case "D":
                            getName( "D", dayNamesShort, dayNames );
                            break;
                        case "o":
                            doy = getNumber( "o" );
                            break;
                        case "m":
                            month = getNumber( "m" );
                            break;
                        case "M":
                            month = getName( "M", monthNamesShort, monthNames );
                            break;
                        case "y":
                            year = getNumber( "y" );
                            break;
                        case "@":
                            date = new Date( getNumber( "@" ) );
                            year = date.getFullYear();
                            month = date.getMonth() + 1;
                            day = date.getDate();
                            break;
                        case "!":
                            date = new Date( ( getNumber( "!" ) - this._ticksTo1970 ) / 10000 );
                            year = date.getFullYear();
                            month = date.getMonth() + 1;
                            day = date.getDate();
                            break;
                        case "'":
                            if ( lookAhead( "'" ) ) {
                                checkLiteral();
                            } else {
                                literal = true;
                            }
                            break;
                        default:
                            checkLiteral();
                    }
                }
            }

            if ( iValue < value.length ) {
                extra = value.substr( iValue );
                if ( !/^\s+/.test( extra ) ) {
                    throw "Extra/unparsed characters found in date: " + extra;
                }
            }

            if ( year === -1 ) {
                year = new Date().getFullYear();
            } else if ( year < 100 ) {
                year += new Date().getFullYear() - new Date().getFullYear() % 100 +
                    ( year <= shortYearCutoff ? 0 : -100 );
            }

            if ( doy > -1 ) {
                month = 1;
                day = doy;
                do {
                    dim = this._getDaysInMonth( year, month - 1 );
                    if ( day <= dim ) {
                        break;
                    }
                    month++;
                    day -= dim;
                } while ( true );
            }

            date = this._daylightSavingAdjust( new Date( year, month - 1, day ) );
            if ( date.getFullYear() !== year || date.getMonth() + 1 !== month || date.getDate() !== day ) {
                throw "Invalid date"; // E.g. 31/02/00
            }
            return date;
        },

        /* Standard date formats. */
        ATOM: "yy-mm-dd", // RFC 3339 (ISO 8601)
        COOKIE: "D, dd M yy",
        ISO_8601: "yy-mm-dd",
        RFC_822: "D, d M y",
        RFC_850: "DD, dd-M-y",
        RFC_1036: "D, d M y",
        RFC_1123: "D, d M yy",
        RFC_2822: "D, d M yy",
        RSS: "D, d M y", // RFC 822
        TICKS: "!",
        TIMESTAMP: "@",
        W3C: "yy-mm-dd", // ISO 8601

        _ticksTo1970: ( ( ( 1970 - 1 ) * 365 + Math.floor( 1970 / 4 ) - Math.floor( 1970 / 100 ) +
            Math.floor( 1970 / 400 ) ) * 24 * 60 * 60 * 10000000 ),

        /* Format a date object into a string value.
         * The format can be combinations of the following:
         * d  - day of month (no leading zero)
         * dd - day of month (two digit)
         * o  - day of year (no leading zeros)
         * oo - day of year (three digit)
         * D  - day name short
         * DD - day name long
         * m  - month of year (no leading zero)
         * mm - month of year (two digit)
         * M  - month name short
         * MM - month name long
         * y  - year (two digit)
         * yy - year (four digit)
         * @ - Unix timestamp (ms since 01/01/1970)
         * ! - Windows ticks (100ns since 01/01/0001)
         * "..." - literal text
         * '' - single quote
         *
         * @param  format string - the desired format of the date
         * @param  date Date - the date value to format
         * @param  settings Object - attributes include:
         *					dayNamesShort	string[7] - abbreviated names of the days from Sunday (optional)
         *					dayNames		string[7] - names of the days from Sunday (optional)
         *					monthNamesShort string[12] - abbreviated names of the months (optional)
         *					monthNames		string[12] - names of the months (optional)
         * @return  string - the date in the above format
         */
        formatDate: function( format, date, settings ) {
            if ( !date ) {
                return "";
            }

            var iFormat,
                dayNamesShort = ( settings ? settings.dayNamesShort : null ) || this._defaults.dayNamesShort,
                dayNames = ( settings ? settings.dayNames : null ) || this._defaults.dayNames,
                monthNamesShort = ( settings ? settings.monthNamesShort : null ) || this._defaults.monthNamesShort,
                monthNames = ( settings ? settings.monthNames : null ) || this._defaults.monthNames,

                // Check whether a format character is doubled
                lookAhead = function( match ) {
                    var matches = ( iFormat + 1 < format.length && format.charAt( iFormat + 1 ) === match );
                    if ( matches ) {
                        iFormat++;
                    }
                    return matches;
                },

                // Format a number, with leading zero if necessary
                formatNumber = function( match, value, len ) {
                    var num = "" + value;
                    if ( lookAhead( match ) ) {
                        while ( num.length < len ) {
                            num = "0" + num;
                        }
                    }
                    return num;
                },

                // Format a name, short or long as requested
                formatName = function( match, value, shortNames, longNames ) {
                    return ( lookAhead( match ) ? longNames[ value ] : shortNames[ value ] );
                },
                output = "",
                literal = false;

            if ( date ) {
                for ( iFormat = 0; iFormat < format.length; iFormat++ ) {
                    if ( literal ) {
                        if ( format.charAt( iFormat ) === "'" && !lookAhead( "'" ) ) {
                            literal = false;
                        } else {
                            output += format.charAt( iFormat );
                        }
                    } else {
                        switch ( format.charAt( iFormat ) ) {
                            case "d":
                                output += formatNumber( "d", date.getDate(), 2 );
                                break;
                            case "D":
                                output += formatName( "D", date.getDay(), dayNamesShort, dayNames );
                                break;
                            case "o":
                                output += formatNumber( "o",
                                    Math.round( ( new Date( date.getFullYear(), date.getMonth(), date.getDate() ).getTime() - new Date( date.getFullYear(), 0, 0 ).getTime() ) / 86400000 ), 3 );
                                break;
                            case "m":
                                output += formatNumber( "m", date.getMonth() + 1, 2 );
                                break;
                            case "M":
                                output += formatName( "M", date.getMonth(), monthNamesShort, monthNames );
                                break;
                            case "y":
                                output += ( lookAhead( "y" ) ? date.getFullYear() :
                                    ( date.getFullYear() % 100 < 10 ? "0" : "" ) + date.getFullYear() % 100 );
                                break;
                            case "@":
                                output += date.getTime();
                                break;
                            case "!":
                                output += date.getTime() * 10000 + this._ticksTo1970;
                                break;
                            case "'":
                                if ( lookAhead( "'" ) ) {
                                    output += "'";
                                } else {
                                    literal = true;
                                }
                                break;
                            default:
                                output += format.charAt( iFormat );
                        }
                    }
                }
            }
            return output;
        },

        /* Extract all possible characters from the date format. */
        _possibleChars: function( format ) {
            var iFormat,
                chars = "",
                literal = false,

                // Check whether a format character is doubled
                lookAhead = function( match ) {
                    var matches = ( iFormat + 1 < format.length && format.charAt( iFormat + 1 ) === match );
                    if ( matches ) {
                        iFormat++;
                    }
                    return matches;
                };

            for ( iFormat = 0; iFormat < format.length; iFormat++ ) {
                if ( literal ) {
                    if ( format.charAt( iFormat ) === "'" && !lookAhead( "'" ) ) {
                        literal = false;
                    } else {
                        chars += format.charAt( iFormat );
                    }
                } else {
                    switch ( format.charAt( iFormat ) ) {
                        case "d": case "m": case "y": case "@":
                            chars += "0123456789";
                            break;
                        case "D": case "M":
                            return null; // Accept anything
                        case "'":
                            if ( lookAhead( "'" ) ) {
                                chars += "'";
                            } else {
                                literal = true;
                            }
                            break;
                        default:
                            chars += format.charAt( iFormat );
                    }
                }
            }
            return chars;
        },

        /* Get a setting value, defaulting if necessary. */
        _get: function( inst, name ) {
            return inst.settings[ name ] !== undefined ?
                inst.settings[ name ] : this._defaults[ name ];
        },

        /* Parse existing date and initialise date picker. */
        _setDateFromField: function( inst, noDefault ) {
            if ( inst.input.val() === inst.lastVal ) {
                return;
            }

            var dateFormat = this._get( inst, "dateFormat" ),
                dates = inst.lastVal = inst.input ? inst.input.val() : null,
                defaultDate = this._getDefaultDate( inst ),
                date = defaultDate,
                settings = this._getFormatConfig( inst );

            try {
                date = this.parseDate( dateFormat, dates, settings ) || defaultDate;
            } catch ( event ) {
                dates = ( noDefault ? "" : dates );
            }
            inst.selectedDay = date.getDate();
            inst.drawMonth = inst.selectedMonth = date.getMonth();
            inst.drawYear = inst.selectedYear = date.getFullYear();
            inst.currentDay = ( dates ? date.getDate() : 0 );
            inst.currentMonth = ( dates ? date.getMonth() : 0 );
            inst.currentYear = ( dates ? date.getFullYear() : 0 );
            this._adjustInstDate( inst );
        },

        /* Retrieve the default date shown on opening. */
        _getDefaultDate: function( inst ) {
            return this._restrictMinMax( inst,
                this._determineDate( inst, this._get( inst, "defaultDate" ), new Date() ) );
        },

        /* A date may be specified as an exact value or a relative one. */
        _determineDate: function( inst, date, defaultDate ) {
            var offsetNumeric = function( offset ) {
                    var date = new Date();
                    date.setDate( date.getDate() + offset );
                    return date;
                },
                offsetString = function( offset ) {
                    try {
                        return $.datepicker.parseDate( $.datepicker._get( inst, "dateFormat" ),
                            offset, $.datepicker._getFormatConfig( inst ) );
                    }
                    catch ( e ) {

                        // Ignore
                    }

                    var date = ( offset.toLowerCase().match( /^c/ ) ?
                        $.datepicker._getDate( inst ) : null ) || new Date(),
                        year = date.getFullYear(),
                        month = date.getMonth(),
                        day = date.getDate(),
                        pattern = /([+\-]?[0-9]+)\s*(d|D|w|W|m|M|y|Y)?/g,
                        matches = pattern.exec( offset );

                    while ( matches ) {
                        switch ( matches[ 2 ] || "d" ) {
                            case "d" : case "D" :
                                day += parseInt( matches[ 1 ], 10 ); break;
                            case "w" : case "W" :
                                day += parseInt( matches[ 1 ], 10 ) * 7; break;
                            case "m" : case "M" :
                                month += parseInt( matches[ 1 ], 10 );
                                day = Math.min( day, $.datepicker._getDaysInMonth( year, month ) );
                                break;
                            case "y": case "Y" :
                                year += parseInt( matches[ 1 ], 10 );
                                day = Math.min( day, $.datepicker._getDaysInMonth( year, month ) );
                                break;
                        }
                        matches = pattern.exec( offset );
                    }
                    return new Date( year, month, day );
                },
                newDate = ( date == null || date === "" ? defaultDate : ( typeof date === "string" ? offsetString( date ) :
                    ( typeof date === "number" ? ( isNaN( date ) ? defaultDate : offsetNumeric( date ) ) : new Date( date.getTime() ) ) ) );

            newDate = ( newDate && newDate.toString() === "Invalid Date" ? defaultDate : newDate );
            if ( newDate ) {
                newDate.setHours( 0 );
                newDate.setMinutes( 0 );
                newDate.setSeconds( 0 );
                newDate.setMilliseconds( 0 );
            }
            return this._daylightSavingAdjust( newDate );
        },

        /* Handle switch to/from daylight saving.
         * Hours may be non-zero on daylight saving cut-over:
         * > 12 when midnight changeover, but then cannot generate
         * midnight datetime, so jump to 1AM, otherwise reset.
         * @param  date  (Date) the date to check
         * @return  (Date) the corrected date
         */
        _daylightSavingAdjust: function( date ) {
            if ( !date ) {
                return null;
            }
            date.setHours( date.getHours() > 12 ? date.getHours() + 2 : 0 );
            return date;
        },

        /* Set the date(s) directly. */
        _setDate: function( inst, date, noChange ) {
            var clear = !date,
                origMonth = inst.selectedMonth,
                origYear = inst.selectedYear,
                newDate = this._restrictMinMax( inst, this._determineDate( inst, date, new Date() ) );

            inst.selectedDay = inst.currentDay = newDate.getDate();
            inst.drawMonth = inst.selectedMonth = inst.currentMonth = newDate.getMonth();
            inst.drawYear = inst.selectedYear = inst.currentYear = newDate.getFullYear();
            if ( ( origMonth !== inst.selectedMonth || origYear !== inst.selectedYear ) && !noChange ) {
                this._notifyChange( inst );
            }
            this._adjustInstDate( inst );
            if ( inst.input ) {
                inst.input.val( clear ? "" : this._formatDate( inst ) );
            }
        },

        /* Retrieve the date(s) directly. */
        _getDate: function( inst ) {
            var startDate = ( !inst.currentYear || ( inst.input && inst.input.val() === "" ) ? null :
                this._daylightSavingAdjust( new Date(
                    inst.currentYear, inst.currentMonth, inst.currentDay ) ) );
            return startDate;
        },

        /* Attach the onxxx handlers.  These are declared statically so
         * they work with static code transformers like Caja.
         */
        _attachHandlers: function( inst ) {
            var stepMonths = this._get( inst, "stepMonths" ),
                id = "#" + inst.id.replace( /\\\\/g, "\\" );
            inst.dpDiv.find( "[data-handler]" ).map( function() {
                var handler = {
                    prev: function() {
                        $.datepicker._adjustDate( id, -stepMonths, "M" );
                    },
                    next: function() {
                        $.datepicker._adjustDate( id, +stepMonths, "M" );
                    },
                    hide: function() {
                        $.datepicker._hideDatepicker();
                    },
                    today: function() {
                        $.datepicker._gotoToday( id );
                    },
                    selectDay: function() {
                        $.datepicker._selectDay( id, +this.getAttribute( "data-month" ), +this.getAttribute( "data-year" ), this );
                        return false;
                    },
                    selectMonth: function() {
                        $.datepicker._selectMonthYear( id, this, "M" );
                        return false;
                    },
                    selectYear: function() {
                        $.datepicker._selectMonthYear( id, this, "Y" );
                        return false;
                    }
                };
                $( this ).on( this.getAttribute( "data-event" ), handler[ this.getAttribute( "data-handler" ) ] );
            } );
        },

        /* Generate the HTML for the current state of the date picker. */
        _generateHTML: function( inst ) {
            var maxDraw, prevText, prev, nextText, next, currentText, gotoDate,
                controls, buttonPanel, firstDay, showWeek, dayNames, dayNamesMin,
                monthNames, monthNamesShort, beforeShowDay, showOtherMonths,
                selectOtherMonths, defaultDate, html, dow, row, group, col, selectedDate,
                cornerClass, calender, thead, day, daysInMonth, leadDays, curRows, numRows,
                printDate, dRow, tbody, daySettings, otherMonth, unselectable,
                tempDate = new Date(),
                today = this._daylightSavingAdjust(
                    new Date( tempDate.getFullYear(), tempDate.getMonth(), tempDate.getDate() ) ), // clear time
                isRTL = this._get( inst, "isRTL" ),
                showButtonPanel = this._get( inst, "showButtonPanel" ),
                hideIfNoPrevNext = this._get( inst, "hideIfNoPrevNext" ),
                navigationAsDateFormat = this._get( inst, "navigationAsDateFormat" ),
                numMonths = this._getNumberOfMonths( inst ),
                showCurrentAtPos = this._get( inst, "showCurrentAtPos" ),
                stepMonths = this._get( inst, "stepMonths" ),
                isMultiMonth = ( numMonths[ 0 ] !== 1 || numMonths[ 1 ] !== 1 ),
                currentDate = this._daylightSavingAdjust( ( !inst.currentDay ? new Date( 9999, 9, 9 ) :
                    new Date( inst.currentYear, inst.currentMonth, inst.currentDay ) ) ),
                minDate = this._getMinMaxDate( inst, "min" ),
                maxDate = this._getMinMaxDate( inst, "max" ),
                drawMonth = inst.drawMonth - showCurrentAtPos,
                drawYear = inst.drawYear;

            if ( drawMonth < 0 ) {
                drawMonth += 12;
                drawYear--;
            }
            if ( maxDate ) {
                maxDraw = this._daylightSavingAdjust( new Date( maxDate.getFullYear(),
                    maxDate.getMonth() - ( numMonths[ 0 ] * numMonths[ 1 ] ) + 1, maxDate.getDate() ) );
                maxDraw = ( minDate && maxDraw < minDate ? minDate : maxDraw );
                while ( this._daylightSavingAdjust( new Date( drawYear, drawMonth, 1 ) ) > maxDraw ) {
                    drawMonth--;
                    if ( drawMonth < 0 ) {
                        drawMonth = 11;
                        drawYear--;
                    }
                }
            }
            inst.drawMonth = drawMonth;
            inst.drawYear = drawYear;

            prevText = this._get( inst, "prevText" );
            prevText = ( !navigationAsDateFormat ? prevText : this.formatDate( prevText,
                this._daylightSavingAdjust( new Date( drawYear, drawMonth - stepMonths, 1 ) ),
                this._getFormatConfig( inst ) ) );

            prev = ( this._canAdjustMonth( inst, -1, drawYear, drawMonth ) ?
                "<a class='ui-datepicker-prev ui-corner-all' data-handler='prev' data-event='click'" +
                " title='" + prevText + "'><span class='ui-icon ui-icon-circle-triangle-" + ( isRTL ? "e" : "w" ) + "'>" + prevText + "</span></a>" :
                ( hideIfNoPrevNext ? "" : "<a class='ui-datepicker-prev ui-corner-all ui-state-disabled' title='" + prevText + "'><span class='ui-icon ui-icon-circle-triangle-" + ( isRTL ? "e" : "w" ) + "'>" + prevText + "</span></a>" ) );

            nextText = this._get( inst, "nextText" );
            nextText = ( !navigationAsDateFormat ? nextText : this.formatDate( nextText,
                this._daylightSavingAdjust( new Date( drawYear, drawMonth + stepMonths, 1 ) ),
                this._getFormatConfig( inst ) ) );

            next = ( this._canAdjustMonth( inst, +1, drawYear, drawMonth ) ?
                "<a class='ui-datepicker-next ui-corner-all' data-handler='next' data-event='click'" +
                " title='" + nextText + "'><span class='ui-icon ui-icon-circle-triangle-" + ( isRTL ? "w" : "e" ) + "'>" + nextText + "</span></a>" :
                ( hideIfNoPrevNext ? "" : "<a class='ui-datepicker-next ui-corner-all ui-state-disabled' title='" + nextText + "'><span class='ui-icon ui-icon-circle-triangle-" + ( isRTL ? "w" : "e" ) + "'>" + nextText + "</span></a>" ) );

            currentText = this._get( inst, "currentText" );
            gotoDate = ( this._get( inst, "gotoCurrent" ) && inst.currentDay ? currentDate : today );
            currentText = ( !navigationAsDateFormat ? currentText :
                this.formatDate( currentText, gotoDate, this._getFormatConfig( inst ) ) );

            controls = ( !inst.inline ? "<button type='button' class='ui-datepicker-close ui-state-default ui-priority-primary ui-corner-all' data-handler='hide' data-event='click'>" +
                this._get( inst, "closeText" ) + "</button>" : "" );

            buttonPanel = ( showButtonPanel ) ? "<div class='ui-datepicker-buttonpane ui-widget-content'>" + ( isRTL ? controls : "" ) +
                ( this._isInRange( inst, gotoDate ) ? "<button type='button' class='ui-datepicker-current ui-state-default ui-priority-secondary ui-corner-all' data-handler='today' data-event='click'" +
                    ">" + currentText + "</button>" : "" ) + ( isRTL ? "" : controls ) + "</div>" : "";

            firstDay = parseInt( this._get( inst, "firstDay" ), 10 );
            firstDay = ( isNaN( firstDay ) ? 0 : firstDay );

            showWeek = this._get( inst, "showWeek" );
            dayNames = this._get( inst, "dayNames" );
            dayNamesMin = this._get( inst, "dayNamesMin" );
            monthNames = this._get( inst, "monthNames" );
            monthNamesShort = this._get( inst, "monthNamesShort" );
            beforeShowDay = this._get( inst, "beforeShowDay" );
            showOtherMonths = this._get( inst, "showOtherMonths" );
            selectOtherMonths = this._get( inst, "selectOtherMonths" );
            defaultDate = this._getDefaultDate( inst );
            html = "";

            for ( row = 0; row < numMonths[ 0 ]; row++ ) {
                group = "";
                this.maxRows = 4;
                for ( col = 0; col < numMonths[ 1 ]; col++ ) {
                    selectedDate = this._daylightSavingAdjust( new Date( drawYear, drawMonth, inst.selectedDay ) );
                    cornerClass = " ui-corner-all";
                    calender = "";
                    if ( isMultiMonth ) {
                        calender += "<div class='ui-datepicker-group";
                        if ( numMonths[ 1 ] > 1 ) {
                            switch ( col ) {
                                case 0: calender += " ui-datepicker-group-first";
                                    cornerClass = " ui-corner-" + ( isRTL ? "right" : "left" ); break;
                                case numMonths[ 1 ] - 1: calender += " ui-datepicker-group-last";
                                    cornerClass = " ui-corner-" + ( isRTL ? "left" : "right" ); break;
                                default: calender += " ui-datepicker-group-middle"; cornerClass = ""; break;
                            }
                        }
                        calender += "'>";
                    }
                    calender += "<div class='ui-datepicker-header ui-widget-header ui-helper-clearfix" + cornerClass + "'>" +
                        ( /all|left/.test( cornerClass ) && row === 0 ? ( isRTL ? next : prev ) : "" ) +
                        ( /all|right/.test( cornerClass ) && row === 0 ? ( isRTL ? prev : next ) : "" ) +
                        this._generateMonthYearHeader( inst, drawMonth, drawYear, minDate, maxDate,
                            row > 0 || col > 0, monthNames, monthNamesShort ) + // draw month headers
                        "</div><table class='ui-datepicker-calendar'><thead>" +
                        "<tr>";
                    thead = ( showWeek ? "<th class='ui-datepicker-week-col'>" + this._get( inst, "weekHeader" ) + "</th>" : "" );
                    for ( dow = 0; dow < 7; dow++ ) { // days of the week
                        day = ( dow + firstDay ) % 7;
                        thead += "<th scope='col'" + ( ( dow + firstDay + 6 ) % 7 >= 5 ? " class='ui-datepicker-week-end'" : "" ) + ">" +
                            "<span title='" + dayNames[ day ] + "'>" + dayNamesMin[ day ] + "</span></th>";
                    }
                    calender += thead + "</tr></thead><tbody>";
                    daysInMonth = this._getDaysInMonth( drawYear, drawMonth );
                    if ( drawYear === inst.selectedYear && drawMonth === inst.selectedMonth ) {
                        inst.selectedDay = Math.min( inst.selectedDay, daysInMonth );
                    }
                    leadDays = ( this._getFirstDayOfMonth( drawYear, drawMonth ) - firstDay + 7 ) % 7;
                    curRows = Math.ceil( ( leadDays + daysInMonth ) / 7 ); // calculate the number of rows to generate
                    numRows = ( isMultiMonth ? this.maxRows > curRows ? this.maxRows : curRows : curRows ); //If multiple months, use the higher number of rows (see #7043)
                    this.maxRows = numRows;
                    printDate = this._daylightSavingAdjust( new Date( drawYear, drawMonth, 1 - leadDays ) );
                    for ( dRow = 0; dRow < numRows; dRow++ ) { // create date picker rows
                        calender += "<tr>";
                        tbody = ( !showWeek ? "" : "<td class='ui-datepicker-week-col'>" +
                            this._get( inst, "calculateWeek" )( printDate ) + "</td>" );
                        for ( dow = 0; dow < 7; dow++ ) { // create date picker days
                            daySettings = ( beforeShowDay ?
                                beforeShowDay.apply( ( inst.input ? inst.input[ 0 ] : null ), [ printDate ] ) : [ true, "" ] );
                            otherMonth = ( printDate.getMonth() !== drawMonth );
                            unselectable = ( otherMonth && !selectOtherMonths ) || !daySettings[ 0 ] ||
                                ( minDate && printDate < minDate ) || ( maxDate && printDate > maxDate );
                            tbody += "<td class='" +
                                ( ( dow + firstDay + 6 ) % 7 >= 5 ? " ui-datepicker-week-end" : "" ) + // highlight weekends
                                ( otherMonth ? " ui-datepicker-other-month" : "" ) + // highlight days from other months
                                ( ( printDate.getTime() === selectedDate.getTime() && drawMonth === inst.selectedMonth && inst._keyEvent ) || // user pressed key
                                ( defaultDate.getTime() === printDate.getTime() && defaultDate.getTime() === selectedDate.getTime() ) ?

                                    // or defaultDate is current printedDate and defaultDate is selectedDate
                                    " " + this._dayOverClass : "" ) + // highlight selected day
                                ( unselectable ? " " + this._unselectableClass + " ui-state-disabled" : "" ) +  // highlight unselectable days
                                ( otherMonth && !showOtherMonths ? "" : " " + daySettings[ 1 ] + // highlight custom dates
                                    ( printDate.getTime() === currentDate.getTime() ? " " + this._currentClass : "" ) + // highlight selected day
                                    ( printDate.getTime() === today.getTime() ? " ui-datepicker-today" : "" ) ) + "'" + // highlight today (if different)
                                ( ( !otherMonth || showOtherMonths ) && daySettings[ 2 ] ? " title='" + daySettings[ 2 ].replace( /'/g, "&#39;" ) + "'" : "" ) + // cell title
                                ( unselectable ? "" : " data-handler='selectDay' data-event='click' data-month='" + printDate.getMonth() + "' data-year='" + printDate.getFullYear() + "'" ) + ">" + // actions
                                ( otherMonth && !showOtherMonths ? "&#xa0;" : // display for other months
                                    ( unselectable ? "<span class='ui-state-default'>" + printDate.getDate() + "</span>" : "<a class='ui-state-default" +
                                        ( printDate.getTime() === today.getTime() ? " ui-state-highlight" : "" ) +
                                        ( printDate.getTime() === currentDate.getTime() ? " ui-state-active" : "" ) + // highlight selected day
                                        ( otherMonth ? " ui-priority-secondary" : "" ) + // distinguish dates from other months
                                        "' href='#'>" + printDate.getDate() + "</a>" ) ) + "</td>"; // display selectable date
                            printDate.setDate( printDate.getDate() + 1 );
                            printDate = this._daylightSavingAdjust( printDate );
                        }
                        calender += tbody + "</tr>";
                    }
                    drawMonth++;
                    if ( drawMonth > 11 ) {
                        drawMonth = 0;
                        drawYear++;
                    }
                    calender += "</tbody></table>" + ( isMultiMonth ? "</div>" +
                        ( ( numMonths[ 0 ] > 0 && col === numMonths[ 1 ] - 1 ) ? "<div class='ui-datepicker-row-break'></div>" : "" ) : "" );
                    group += calender;
                }
                html += group;
            }
            html += buttonPanel;
            inst._keyEvent = false;
            return html;
        },

        /* Generate the month and year header. */
        _generateMonthYearHeader: function( inst, drawMonth, drawYear, minDate, maxDate,
                                            secondary, monthNames, monthNamesShort ) {

            var inMinYear, inMaxYear, month, years, thisYear, determineYear, year, endYear,
                changeMonth = this._get( inst, "changeMonth" ),
                changeYear = this._get( inst, "changeYear" ),
                showMonthAfterYear = this._get( inst, "showMonthAfterYear" ),
                html = "<div class='ui-datepicker-title'>",
                monthHtml = "";

            // Month selection
            if ( secondary || !changeMonth ) {
                monthHtml += "<span class='ui-datepicker-month'>" + monthNames[ drawMonth ] + "</span>";
            } else {
                inMinYear = ( minDate && minDate.getFullYear() === drawYear );
                inMaxYear = ( maxDate && maxDate.getFullYear() === drawYear );
                monthHtml += "<select class='ui-datepicker-month' data-handler='selectMonth' data-event='change'>";
                for ( month = 0; month < 12; month++ ) {
                    if ( ( !inMinYear || month >= minDate.getMonth() ) && ( !inMaxYear || month <= maxDate.getMonth() ) ) {
                        monthHtml += "<option value='" + month + "'" +
                            ( month === drawMonth ? " selected='selected'" : "" ) +
                            ">" + monthNamesShort[ month ] + "</option>";
                    }
                }
                monthHtml += "</select>";
            }

            if ( !showMonthAfterYear ) {
                html += monthHtml + ( secondary || !( changeMonth && changeYear ) ? "&#xa0;" : "" );
            }

            // Year selection
            if ( !inst.yearshtml ) {
                inst.yearshtml = "";
                if ( secondary || !changeYear ) {
                    html += "<span class='ui-datepicker-year'>" + drawYear + "</span>";
                } else {

                    // determine range of years to display
                    years = this._get( inst, "yearRange" ).split( ":" );
                    thisYear = new Date().getFullYear();
                    determineYear = function( value ) {
                        var year = ( value.match( /c[+\-].*/ ) ? drawYear + parseInt( value.substring( 1 ), 10 ) :
                            ( value.match( /[+\-].*/ ) ? thisYear + parseInt( value, 10 ) :
                                parseInt( value, 10 ) ) );
                        return ( isNaN( year ) ? thisYear : year );
                    };
                    year = determineYear( years[ 0 ] );
                    endYear = Math.max( year, determineYear( years[ 1 ] || "" ) );
                    year = ( minDate ? Math.max( year, minDate.getFullYear() ) : year );
                    endYear = ( maxDate ? Math.min( endYear, maxDate.getFullYear() ) : endYear );
                    inst.yearshtml += "<select class='ui-datepicker-year' data-handler='selectYear' data-event='change'>";
                    for ( ; year <= endYear; year++ ) {
                        inst.yearshtml += "<option value='" + year + "'" +
                            ( year === drawYear ? " selected='selected'" : "" ) +
                            ">" + year + "</option>";
                    }
                    inst.yearshtml += "</select>";

                    html += inst.yearshtml;
                    inst.yearshtml = null;
                }
            }

            html += this._get( inst, "yearSuffix" );
            if ( showMonthAfterYear ) {
                html += ( secondary || !( changeMonth && changeYear ) ? "&#xa0;" : "" ) + monthHtml;
            }
            html += "</div>"; // Close datepicker_header
            return html;
        },

        /* Adjust one of the date sub-fields. */
        _adjustInstDate: function( inst, offset, period ) {
            var year = inst.selectedYear + ( period === "Y" ? offset : 0 ),
                month = inst.selectedMonth + ( period === "M" ? offset : 0 ),
                day = Math.min( inst.selectedDay, this._getDaysInMonth( year, month ) ) + ( period === "D" ? offset : 0 ),
                date = this._restrictMinMax( inst, this._daylightSavingAdjust( new Date( year, month, day ) ) );

            inst.selectedDay = date.getDate();
            inst.drawMonth = inst.selectedMonth = date.getMonth();
            inst.drawYear = inst.selectedYear = date.getFullYear();
            if ( period === "M" || period === "Y" ) {
                this._notifyChange( inst );
            }
        },

        /* Ensure a date is within any min/max bounds. */
        _restrictMinMax: function( inst, date ) {
            var minDate = this._getMinMaxDate( inst, "min" ),
                maxDate = this._getMinMaxDate( inst, "max" ),
                newDate = ( minDate && date < minDate ? minDate : date );
            return ( maxDate && newDate > maxDate ? maxDate : newDate );
        },

        /* Notify change of month/year. */
        _notifyChange: function( inst ) {
            var onChange = this._get( inst, "onChangeMonthYear" );
            if ( onChange ) {
                onChange.apply( ( inst.input ? inst.input[ 0 ] : null ),
                    [ inst.selectedYear, inst.selectedMonth + 1, inst ] );
            }
        },

        /* Determine the number of months to show. */
        _getNumberOfMonths: function( inst ) {
            var numMonths = this._get( inst, "numberOfMonths" );
            return ( numMonths == null ? [ 1, 1 ] : ( typeof numMonths === "number" ? [ 1, numMonths ] : numMonths ) );
        },

        /* Determine the current maximum date - ensure no time components are set. */
        _getMinMaxDate: function( inst, minMax ) {
            return this._determineDate( inst, this._get( inst, minMax + "Date" ), null );
        },

        /* Find the number of days in a given month. */
        _getDaysInMonth: function( year, month ) {
            return 32 - this._daylightSavingAdjust( new Date( year, month, 32 ) ).getDate();
        },

        /* Find the day of the week of the first of a month. */
        _getFirstDayOfMonth: function( year, month ) {
            return new Date( year, month, 1 ).getDay();
        },

        /* Determines if we should allow a "next/prev" month display change. */
        _canAdjustMonth: function( inst, offset, curYear, curMonth ) {
            var numMonths = this._getNumberOfMonths( inst ),
                date = this._daylightSavingAdjust( new Date( curYear,
                    curMonth + ( offset < 0 ? offset : numMonths[ 0 ] * numMonths[ 1 ] ), 1 ) );

            if ( offset < 0 ) {
                date.setDate( this._getDaysInMonth( date.getFullYear(), date.getMonth() ) );
            }
            return this._isInRange( inst, date );
        },

        /* Is the given date in the accepted range? */
        _isInRange: function( inst, date ) {
            var yearSplit, currentYear,
                minDate = this._getMinMaxDate( inst, "min" ),
                maxDate = this._getMinMaxDate( inst, "max" ),
                minYear = null,
                maxYear = null,
                years = this._get( inst, "yearRange" );
            if ( years ) {
                yearSplit = years.split( ":" );
                currentYear = new Date().getFullYear();
                minYear = parseInt( yearSplit[ 0 ], 10 );
                maxYear = parseInt( yearSplit[ 1 ], 10 );
                if ( yearSplit[ 0 ].match( /[+\-].*/ ) ) {
                    minYear += currentYear;
                }
                if ( yearSplit[ 1 ].match( /[+\-].*/ ) ) {
                    maxYear += currentYear;
                }
            }

            return ( ( !minDate || date.getTime() >= minDate.getTime() ) &&
                ( !maxDate || date.getTime() <= maxDate.getTime() ) &&
                ( !minYear || date.getFullYear() >= minYear ) &&
                ( !maxYear || date.getFullYear() <= maxYear ) );
        },

        /* Provide the configuration settings for formatting/parsing. */
        _getFormatConfig: function( inst ) {
            var shortYearCutoff = this._get( inst, "shortYearCutoff" );
            shortYearCutoff = ( typeof shortYearCutoff !== "string" ? shortYearCutoff :
                new Date().getFullYear() % 100 + parseInt( shortYearCutoff, 10 ) );
            return { shortYearCutoff: shortYearCutoff,
                dayNamesShort: this._get( inst, "dayNamesShort" ), dayNames: this._get( inst, "dayNames" ),
                monthNamesShort: this._get( inst, "monthNamesShort" ), monthNames: this._get( inst, "monthNames" ) };
        },

        /* Format the given date for display. */
        _formatDate: function( inst, day, month, year ) {
            if ( !day ) {
                inst.currentDay = inst.selectedDay;
                inst.currentMonth = inst.selectedMonth;
                inst.currentYear = inst.selectedYear;
            }
            var date = ( day ? ( typeof day === "object" ? day :
                this._daylightSavingAdjust( new Date( year, month, day ) ) ) :
                this._daylightSavingAdjust( new Date( inst.currentYear, inst.currentMonth, inst.currentDay ) ) );
            return this.formatDate( this._get( inst, "dateFormat" ), date, this._getFormatConfig( inst ) );
        }
    } );

    /*
     * Bind hover events for datepicker elements.
     * Done via delegate so the binding only occurs once in the lifetime of the parent div.
     * Global datepicker_instActive, set by _updateDatepicker allows the handlers to find their way back to the active picker.
     */
    function datepicker_bindHover( dpDiv ) {
        var selector = "button, .ui-datepicker-prev, .ui-datepicker-next, .ui-datepicker-calendar td a";
        return dpDiv.on( "mouseout", selector, function() {
            $( this ).removeClass( "ui-state-hover" );
            if ( this.className.indexOf( "ui-datepicker-prev" ) !== -1 ) {
                $( this ).removeClass( "ui-datepicker-prev-hover" );
            }
            if ( this.className.indexOf( "ui-datepicker-next" ) !== -1 ) {
                $( this ).removeClass( "ui-datepicker-next-hover" );
            }
        } )
            .on( "mouseover", selector, datepicker_handleMouseover );
    }

    function datepicker_handleMouseover() {
        if ( !$.datepicker._isDisabledDatepicker( datepicker_instActive.inline ? datepicker_instActive.dpDiv.parent()[ 0 ] : datepicker_instActive.input[ 0 ] ) ) {
            $( this ).parents( ".ui-datepicker-calendar" ).find( "a" ).removeClass( "ui-state-hover" );
            $( this ).addClass( "ui-state-hover" );
            if ( this.className.indexOf( "ui-datepicker-prev" ) !== -1 ) {
                $( this ).addClass( "ui-datepicker-prev-hover" );
            }
            if ( this.className.indexOf( "ui-datepicker-next" ) !== -1 ) {
                $( this ).addClass( "ui-datepicker-next-hover" );
            }
        }
    }

    /* jQuery extend now ignores nulls! */
    function datepicker_extendRemove( target, props ) {
        $.extend( target, props );
        for ( var name in props ) {
            if ( props[ name ] == null ) {
                target[ name ] = props[ name ];
            }
        }
        return target;
    }

    /* Invoke the datepicker functionality.
       @param  options  string - a command, optionally followed by additional parameters or
                        Object - settings for attaching new datepicker functionality
       @return  jQuery object */
    $.fn.datepicker = function( options ) {

        /* Verify an empty collection wasn't passed - Fixes #6976 */
        if ( !this.length ) {
            return this;
        }

        /* Initialise the date picker. */
        if ( !$.datepicker.initialized ) {
            $( document ).on( "mousedown", $.datepicker._checkExternalClick );
            $.datepicker.initialized = true;
        }

        /* Append datepicker main container to body if not exist. */
        if ( $( "#" + $.datepicker._mainDivId ).length === 0 ) {
            $( "body" ).append( $.datepicker.dpDiv );
        }

        var otherArgs = Array.prototype.slice.call( arguments, 1 );
        if ( typeof options === "string" && ( options === "isDisabled" || options === "getDate" || options === "widget" ) ) {
            return $.datepicker[ "_" + options + "Datepicker" ].
            apply( $.datepicker, [ this[ 0 ] ].concat( otherArgs ) );
        }
        if ( options === "option" && arguments.length === 2 && typeof arguments[ 1 ] === "string" ) {
            return $.datepicker[ "_" + options + "Datepicker" ].
            apply( $.datepicker, [ this[ 0 ] ].concat( otherArgs ) );
        }
        return this.each( function() {
            typeof options === "string" ?
                $.datepicker[ "_" + options + "Datepicker" ].
                apply( $.datepicker, [ this ].concat( otherArgs ) ) :
                $.datepicker._attachDatepicker( this, options );
        } );
    };

    $.datepicker = new Datepicker(); // singleton instance
    $.datepicker.initialized = false;
    $.datepicker.uuid = new Date().getTime();
    $.datepicker.version = "1.12.1";

    var widgetsDatepicker = $.datepicker;
}));

!function(e,t){"object"==typeof exports&&"undefined"!=typeof module?module.exports=t():"function"==typeof define&&define.amd?define(t):e.moment=t()}(this,function(){"use strict";var e,i;function f(){return e.apply(null,arguments)}function o(e){return e instanceof Array||"[object Array]"===Object.prototype.toString.call(e)}function u(e){return null!=e&&"[object Object]"===Object.prototype.toString.call(e)}function m(e,t){return Object.prototype.hasOwnProperty.call(e,t)}function l(e){if(Object.getOwnPropertyNames)return 0===Object.getOwnPropertyNames(e).length;var t;for(t in e)if(m(e,t))return;return 1}function r(e){return void 0===e}function h(e){return"number"==typeof e||"[object Number]"===Object.prototype.toString.call(e)}function a(e){return e instanceof Date||"[object Date]"===Object.prototype.toString.call(e)}function d(e,t){var n,s=[];for(n=0;n<e.length;++n)s.push(t(e[n],n));return s}function c(e,t){for(var n in t)m(t,n)&&(e[n]=t[n]);return m(t,"toString")&&(e.toString=t.toString),m(t,"valueOf")&&(e.valueOf=t.valueOf),e}function _(e,t,n,s){return xt(e,t,n,s,!0).utc()}function y(e){return null==e._pf&&(e._pf={empty:!1,unusedTokens:[],unusedInput:[],overflow:-2,charsLeftOver:0,nullInput:!1,invalidEra:null,invalidMonth:null,invalidFormat:!1,userInvalidated:!1,iso:!1,parsedDateParts:[],era:null,meridiem:null,rfc2822:!1,weekdayMismatch:!1}),e._pf}function g(e){if(null==e._isValid){var t=y(e),n=i.call(t.parsedDateParts,function(e){return null!=e}),s=!isNaN(e._d.getTime())&&t.overflow<0&&!t.empty&&!t.invalidEra&&!t.invalidMonth&&!t.invalidWeekday&&!t.weekdayMismatch&&!t.nullInput&&!t.invalidFormat&&!t.userInvalidated&&(!t.meridiem||t.meridiem&&n);if(e._strict&&(s=s&&0===t.charsLeftOver&&0===t.unusedTokens.length&&void 0===t.bigHour),null!=Object.isFrozen&&Object.isFrozen(e))return s;e._isValid=s}return e._isValid}function w(e){var t=_(NaN);return null!=e?c(y(t),e):y(t).userInvalidated=!0,t}i=Array.prototype.some?Array.prototype.some:function(e){var t,n=Object(this),s=n.length>>>0;for(t=0;t<s;t++)if(t in n&&e.call(this,n[t],t,n))return!0;return!1};var p=f.momentProperties=[],t=!1;function v(e,t){var n,s,i;if(r(t._isAMomentObject)||(e._isAMomentObject=t._isAMomentObject),r(t._i)||(e._i=t._i),r(t._f)||(e._f=t._f),r(t._l)||(e._l=t._l),r(t._strict)||(e._strict=t._strict),r(t._tzm)||(e._tzm=t._tzm),r(t._isUTC)||(e._isUTC=t._isUTC),r(t._offset)||(e._offset=t._offset),r(t._pf)||(e._pf=y(t)),r(t._locale)||(e._locale=t._locale),0<p.length)for(n=0;n<p.length;n++)r(i=t[s=p[n]])||(e[s]=i);return e}function k(e){v(this,e),this._d=new Date(null!=e._d?e._d.getTime():NaN),this.isValid()||(this._d=new Date(NaN)),!1===t&&(t=!0,f.updateOffset(this),t=!1)}function M(e){return e instanceof k||null!=e&&null!=e._isAMomentObject}function D(e){!1===f.suppressDeprecationWarnings&&"undefined"!=typeof console&&console.warn&&console.warn("Deprecation warning: "+e)}function n(i,r){var a=!0;return c(function(){if(null!=f.deprecationHandler&&f.deprecationHandler(null,i),a){var e,t,n,s=[];for(t=0;t<arguments.length;t++){if(e="","object"==typeof arguments[t]){for(n in e+="\n["+t+"] ",arguments[0])m(arguments[0],n)&&(e+=n+": "+arguments[0][n]+", ");e=e.slice(0,-2)}else e=arguments[t];s.push(e)}D(i+"\nArguments: "+Array.prototype.slice.call(s).join("")+"\n"+(new Error).stack),a=!1}return r.apply(this,arguments)},r)}var s,S={};function Y(e,t){null!=f.deprecationHandler&&f.deprecationHandler(e,t),S[e]||(D(t),S[e]=!0)}function O(e){return"undefined"!=typeof Function&&e instanceof Function||"[object Function]"===Object.prototype.toString.call(e)}function b(e,t){var n,s=c({},e);for(n in t)m(t,n)&&(u(e[n])&&u(t[n])?(s[n]={},c(s[n],e[n]),c(s[n],t[n])):null!=t[n]?s[n]=t[n]:delete s[n]);for(n in e)m(e,n)&&!m(t,n)&&u(e[n])&&(s[n]=c({},s[n]));return s}function x(e){null!=e&&this.set(e)}f.suppressDeprecationWarnings=!1,f.deprecationHandler=null,s=Object.keys?Object.keys:function(e){var t,n=[];for(t in e)m(e,t)&&n.push(t);return n};function T(e,t,n){var s=""+Math.abs(e),i=t-s.length;return(0<=e?n?"+":"":"-")+Math.pow(10,Math.max(0,i)).toString().substr(1)+s}var N=/(\[[^\[]*\])|(\\)?([Hh]mm(ss)?|Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Qo?|N{1,5}|YYYYYY|YYYYY|YYYY|YY|y{2,4}|yo?|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|kk?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g,P=/(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g,R={},W={};function C(e,t,n,s){var i=s;"string"==typeof s&&(i=function(){return this[s]()}),e&&(W[e]=i),t&&(W[t[0]]=function(){return T(i.apply(this,arguments),t[1],t[2])}),n&&(W[n]=function(){return this.localeData().ordinal(i.apply(this,arguments),e)})}function U(e,t){return e.isValid()?(t=H(t,e.localeData()),R[t]=R[t]||function(s){var e,i,t,r=s.match(N);for(e=0,i=r.length;e<i;e++)W[r[e]]?r[e]=W[r[e]]:r[e]=(t=r[e]).match(/\[[\s\S]/)?t.replace(/^\[|\]$/g,""):t.replace(/\\/g,"");return function(e){var t,n="";for(t=0;t<i;t++)n+=O(r[t])?r[t].call(e,s):r[t];return n}}(t),R[t](e)):e.localeData().invalidDate()}function H(e,t){var n=5;function s(e){return t.longDateFormat(e)||e}for(P.lastIndex=0;0<=n&&P.test(e);)e=e.replace(P,s),P.lastIndex=0,--n;return e}var F={};function L(e,t){var n=e.toLowerCase();F[n]=F[n+"s"]=F[t]=e}function V(e){return"string"==typeof e?F[e]||F[e.toLowerCase()]:void 0}function G(e){var t,n,s={};for(n in e)m(e,n)&&(t=V(n))&&(s[t]=e[n]);return s}var E={};function A(e,t){E[e]=t}function j(e){return e%4==0&&e%100!=0||e%400==0}function I(e){return e<0?Math.ceil(e)||0:Math.floor(e)}function Z(e){var t=+e,n=0;return 0!=t&&isFinite(t)&&(n=I(t)),n}function z(t,n){return function(e){return null!=e?(q(this,t,e),f.updateOffset(this,n),this):$(this,t)}}function $(e,t){return e.isValid()?e._d["get"+(e._isUTC?"UTC":"")+t]():NaN}function q(e,t,n){e.isValid()&&!isNaN(n)&&("FullYear"===t&&j(e.year())&&1===e.month()&&29===e.date()?(n=Z(n),e._d["set"+(e._isUTC?"UTC":"")+t](n,e.month(),xe(n,e.month()))):e._d["set"+(e._isUTC?"UTC":"")+t](n))}var B,J=/\d/,Q=/\d\d/,X=/\d{3}/,K=/\d{4}/,ee=/[+-]?\d{6}/,te=/\d\d?/,ne=/\d\d\d\d?/,se=/\d\d\d\d\d\d?/,ie=/\d{1,3}/,re=/\d{1,4}/,ae=/[+-]?\d{1,6}/,oe=/\d+/,ue=/[+-]?\d+/,le=/Z|[+-]\d\d:?\d\d/gi,he=/Z|[+-]\d\d(?::?\d\d)?/gi,de=/[0-9]{0,256}['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFF07\uFF10-\uFFEF]{1,256}|[\u0600-\u06FF\/]{1,256}(\s*?[\u0600-\u06FF]{1,256}){1,2}/i;function ce(e,n,s){B[e]=O(n)?n:function(e,t){return e&&s?s:n}}function fe(e,t){return m(B,e)?B[e](t._strict,t._locale):new RegExp(me(e.replace("\\","").replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g,function(e,t,n,s,i){return t||n||s||i})))}function me(e){return e.replace(/[-\/\\^$*+?.()|[\]{}]/g,"\\$&")}B={};var _e={};function ye(e,n){var t,s=n;for("string"==typeof e&&(e=[e]),h(n)&&(s=function(e,t){t[n]=Z(e)}),t=0;t<e.length;t++)_e[e[t]]=s}function ge(e,i){ye(e,function(e,t,n,s){n._w=n._w||{},i(e,n._w,n,s)})}var we,pe=0,ve=1,ke=2,Me=3,De=4,Se=5,Ye=6,Oe=7,be=8;function xe(e,t){if(isNaN(e)||isNaN(t))return NaN;var n,s=(t%(n=12)+n)%n;return e+=(t-s)/12,1==s?j(e)?29:28:31-s%7%2}we=Array.prototype.indexOf?Array.prototype.indexOf:function(e){var t;for(t=0;t<this.length;++t)if(this[t]===e)return t;return-1},C("M",["MM",2],"Mo",function(){return this.month()+1}),C("MMM",0,0,function(e){return this.localeData().monthsShort(this,e)}),C("MMMM",0,0,function(e){return this.localeData().months(this,e)}),L("month","M"),A("month",8),ce("M",te),ce("MM",te,Q),ce("MMM",function(e,t){return t.monthsShortRegex(e)}),ce("MMMM",function(e,t){return t.monthsRegex(e)}),ye(["M","MM"],function(e,t){t[ve]=Z(e)-1}),ye(["MMM","MMMM"],function(e,t,n,s){var i=n._locale.monthsParse(e,s,n._strict);null!=i?t[ve]=i:y(n).invalidMonth=e});var Te="January_February_March_April_May_June_July_August_September_October_November_December".split("_"),Ne="Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),Pe=/D[oD]?(\[[^\[\]]*\]|\s)+MMMM?/,Re=de,We=de;function Ce(e,t){var n;if(!e.isValid())return e;if("string"==typeof t)if(/^\d+$/.test(t))t=Z(t);else if(!h(t=e.localeData().monthsParse(t)))return e;return n=Math.min(e.date(),xe(e.year(),t)),e._d["set"+(e._isUTC?"UTC":"")+"Month"](t,n),e}function Ue(e){return null!=e?(Ce(this,e),f.updateOffset(this,!0),this):$(this,"Month")}function He(){function e(e,t){return t.length-e.length}var t,n,s=[],i=[],r=[];for(t=0;t<12;t++)n=_([2e3,t]),s.push(this.monthsShort(n,"")),i.push(this.months(n,"")),r.push(this.months(n,"")),r.push(this.monthsShort(n,""));for(s.sort(e),i.sort(e),r.sort(e),t=0;t<12;t++)s[t]=me(s[t]),i[t]=me(i[t]);for(t=0;t<24;t++)r[t]=me(r[t]);this._monthsRegex=new RegExp("^("+r.join("|")+")","i"),this._monthsShortRegex=this._monthsRegex,this._monthsStrictRegex=new RegExp("^("+i.join("|")+")","i"),this._monthsShortStrictRegex=new RegExp("^("+s.join("|")+")","i")}function Fe(e){return j(e)?366:365}C("Y",0,0,function(){var e=this.year();return e<=9999?T(e,4):"+"+e}),C(0,["YY",2],0,function(){return this.year()%100}),C(0,["YYYY",4],0,"year"),C(0,["YYYYY",5],0,"year"),C(0,["YYYYYY",6,!0],0,"year"),L("year","y"),A("year",1),ce("Y",ue),ce("YY",te,Q),ce("YYYY",re,K),ce("YYYYY",ae,ee),ce("YYYYYY",ae,ee),ye(["YYYYY","YYYYYY"],pe),ye("YYYY",function(e,t){t[pe]=2===e.length?f.parseTwoDigitYear(e):Z(e)}),ye("YY",function(e,t){t[pe]=f.parseTwoDigitYear(e)}),ye("Y",function(e,t){t[pe]=parseInt(e,10)}),f.parseTwoDigitYear=function(e){return Z(e)+(68<Z(e)?1900:2e3)};var Le=z("FullYear",!0);function Ve(e){var t,n;return e<100&&0<=e?((n=Array.prototype.slice.call(arguments))[0]=e+400,t=new Date(Date.UTC.apply(null,n)),isFinite(t.getUTCFullYear())&&t.setUTCFullYear(e)):t=new Date(Date.UTC.apply(null,arguments)),t}function Ge(e,t,n){var s=7+t-n;return s-(7+Ve(e,0,s).getUTCDay()-t)%7-1}function Ee(e,t,n,s,i){var r,a,o=1+7*(t-1)+(7+n-s)%7+Ge(e,s,i);return a=o<=0?Fe(r=e-1)+o:o>Fe(e)?(r=e+1,o-Fe(e)):(r=e,o),{year:r,dayOfYear:a}}function Ae(e,t,n){var s,i,r=Ge(e.year(),t,n),a=Math.floor((e.dayOfYear()-r-1)/7)+1;return a<1?s=a+je(i=e.year()-1,t,n):a>je(e.year(),t,n)?(s=a-je(e.year(),t,n),i=e.year()+1):(i=e.year(),s=a),{week:s,year:i}}function je(e,t,n){var s=Ge(e,t,n),i=Ge(e+1,t,n);return(Fe(e)-s+i)/7}C("w",["ww",2],"wo","week"),C("W",["WW",2],"Wo","isoWeek"),L("week","w"),L("isoWeek","W"),A("week",5),A("isoWeek",5),ce("w",te),ce("ww",te,Q),ce("W",te),ce("WW",te,Q),ge(["w","ww","W","WW"],function(e,t,n,s){t[s.substr(0,1)]=Z(e)});function Ie(e,t){return e.slice(t,7).concat(e.slice(0,t))}C("d",0,"do","day"),C("dd",0,0,function(e){return this.localeData().weekdaysMin(this,e)}),C("ddd",0,0,function(e){return this.localeData().weekdaysShort(this,e)}),C("dddd",0,0,function(e){return this.localeData().weekdays(this,e)}),C("e",0,0,"weekday"),C("E",0,0,"isoWeekday"),L("day","d"),L("weekday","e"),L("isoWeekday","E"),A("day",11),A("weekday",11),A("isoWeekday",11),ce("d",te),ce("e",te),ce("E",te),ce("dd",function(e,t){return t.weekdaysMinRegex(e)}),ce("ddd",function(e,t){return t.weekdaysShortRegex(e)}),ce("dddd",function(e,t){return t.weekdaysRegex(e)}),ge(["dd","ddd","dddd"],function(e,t,n,s){var i=n._locale.weekdaysParse(e,s,n._strict);null!=i?t.d=i:y(n).invalidWeekday=e}),ge(["d","e","E"],function(e,t,n,s){t[s]=Z(e)});var Ze="Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),ze="Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),$e="Su_Mo_Tu_We_Th_Fr_Sa".split("_"),qe=de,Be=de,Je=de;function Qe(){function e(e,t){return t.length-e.length}var t,n,s,i,r,a=[],o=[],u=[],l=[];for(t=0;t<7;t++)n=_([2e3,1]).day(t),s=me(this.weekdaysMin(n,"")),i=me(this.weekdaysShort(n,"")),r=me(this.weekdays(n,"")),a.push(s),o.push(i),u.push(r),l.push(s),l.push(i),l.push(r);a.sort(e),o.sort(e),u.sort(e),l.sort(e),this._weekdaysRegex=new RegExp("^("+l.join("|")+")","i"),this._weekdaysShortRegex=this._weekdaysRegex,this._weekdaysMinRegex=this._weekdaysRegex,this._weekdaysStrictRegex=new RegExp("^("+u.join("|")+")","i"),this._weekdaysShortStrictRegex=new RegExp("^("+o.join("|")+")","i"),this._weekdaysMinStrictRegex=new RegExp("^("+a.join("|")+")","i")}function Xe(){return this.hours()%12||12}function Ke(e,t){C(e,0,0,function(){return this.localeData().meridiem(this.hours(),this.minutes(),t)})}function et(e,t){return t._meridiemParse}C("H",["HH",2],0,"hour"),C("h",["hh",2],0,Xe),C("k",["kk",2],0,function(){return this.hours()||24}),C("hmm",0,0,function(){return""+Xe.apply(this)+T(this.minutes(),2)}),C("hmmss",0,0,function(){return""+Xe.apply(this)+T(this.minutes(),2)+T(this.seconds(),2)}),C("Hmm",0,0,function(){return""+this.hours()+T(this.minutes(),2)}),C("Hmmss",0,0,function(){return""+this.hours()+T(this.minutes(),2)+T(this.seconds(),2)}),Ke("a",!0),Ke("A",!1),L("hour","h"),A("hour",13),ce("a",et),ce("A",et),ce("H",te),ce("h",te),ce("k",te),ce("HH",te,Q),ce("hh",te,Q),ce("kk",te,Q),ce("hmm",ne),ce("hmmss",se),ce("Hmm",ne),ce("Hmmss",se),ye(["H","HH"],Me),ye(["k","kk"],function(e,t,n){var s=Z(e);t[Me]=24===s?0:s}),ye(["a","A"],function(e,t,n){n._isPm=n._locale.isPM(e),n._meridiem=e}),ye(["h","hh"],function(e,t,n){t[Me]=Z(e),y(n).bigHour=!0}),ye("hmm",function(e,t,n){var s=e.length-2;t[Me]=Z(e.substr(0,s)),t[De]=Z(e.substr(s)),y(n).bigHour=!0}),ye("hmmss",function(e,t,n){var s=e.length-4,i=e.length-2;t[Me]=Z(e.substr(0,s)),t[De]=Z(e.substr(s,2)),t[Se]=Z(e.substr(i)),y(n).bigHour=!0}),ye("Hmm",function(e,t,n){var s=e.length-2;t[Me]=Z(e.substr(0,s)),t[De]=Z(e.substr(s))}),ye("Hmmss",function(e,t,n){var s=e.length-4,i=e.length-2;t[Me]=Z(e.substr(0,s)),t[De]=Z(e.substr(s,2)),t[Se]=Z(e.substr(i))});var tt=z("Hours",!0);var nt,st={calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},longDateFormat:{LTS:"h:mm:ss A",LT:"h:mm A",L:"MM/DD/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY h:mm A",LLLL:"dddd, MMMM D, YYYY h:mm A"},invalidDate:"Invalid date",ordinal:"%d",dayOfMonthOrdinalParse:/\d{1,2}/,relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",w:"a week",ww:"%d weeks",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},months:Te,monthsShort:Ne,week:{dow:0,doy:6},weekdays:Ze,weekdaysMin:$e,weekdaysShort:ze,meridiemParse:/[ap]\.?m?\.?/i},it={},rt={};function at(e,t){var n,s=Math.min(e.length,t.length);for(n=0;n<s;n+=1)if(e[n]!==t[n])return n;return s}function ot(e){return e?e.toLowerCase().replace("_","-"):e}function ut(t){var e=null;if(void 0===it[t]&&"undefined"!=typeof module&&module&&module.exports)try{e=nt._abbr,require("./locale/"+t),lt(e)}catch(e){it[t]=null}return it[t]}function lt(e,t){var n;return e&&((n=r(t)?dt(e):ht(e,t))?nt=n:"undefined"!=typeof console&&console.warn&&console.warn("Locale "+e+" not found. Did you forget to load it?")),nt._abbr}function ht(e,t){if(null===t)return delete it[e],null;var n,s=st;if(t.abbr=e,null!=it[e])Y("defineLocaleOverride","use moment.updateLocale(localeName, config) to change an existing locale. moment.defineLocale(localeName, config) should only be used for creating a new locale See http://momentjs.com/guides/#/warnings/define-locale/ for more info."),s=it[e]._config;else if(null!=t.parentLocale)if(null!=it[t.parentLocale])s=it[t.parentLocale]._config;else{if(null==(n=ut(t.parentLocale)))return rt[t.parentLocale]||(rt[t.parentLocale]=[]),rt[t.parentLocale].push({name:e,config:t}),null;s=n._config}return it[e]=new x(b(s,t)),rt[e]&&rt[e].forEach(function(e){ht(e.name,e.config)}),lt(e),it[e]}function dt(e){var t;if(e&&e._locale&&e._locale._abbr&&(e=e._locale._abbr),!e)return nt;if(!o(e)){if(t=ut(e))return t;e=[e]}return function(e){for(var t,n,s,i,r=0;r<e.length;){for(t=(i=ot(e[r]).split("-")).length,n=(n=ot(e[r+1]))?n.split("-"):null;0<t;){if(s=ut(i.slice(0,t).join("-")))return s;if(n&&n.length>=t&&at(i,n)>=t-1)break;t--}r++}return nt}(e)}function ct(e){var t,n=e._a;return n&&-2===y(e).overflow&&(t=n[ve]<0||11<n[ve]?ve:n[ke]<1||n[ke]>xe(n[pe],n[ve])?ke:n[Me]<0||24<n[Me]||24===n[Me]&&(0!==n[De]||0!==n[Se]||0!==n[Ye])?Me:n[De]<0||59<n[De]?De:n[Se]<0||59<n[Se]?Se:n[Ye]<0||999<n[Ye]?Ye:-1,y(e)._overflowDayOfYear&&(t<pe||ke<t)&&(t=ke),y(e)._overflowWeeks&&-1===t&&(t=Oe),y(e)._overflowWeekday&&-1===t&&(t=be),y(e).overflow=t),e}var ft=/^\s*((?:[+-]\d{6}|\d{4})-(?:\d\d-\d\d|W\d\d-\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?::\d\d(?::\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,mt=/^\s*((?:[+-]\d{6}|\d{4})(?:\d\d\d\d|W\d\d\d|W\d\d|\d\d\d|\d\d|))(?:(T| )(\d\d(?:\d\d(?:\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,_t=/Z|[+-]\d\d(?::?\d\d)?/,yt=[["YYYYYY-MM-DD",/[+-]\d{6}-\d\d-\d\d/],["YYYY-MM-DD",/\d{4}-\d\d-\d\d/],["GGGG-[W]WW-E",/\d{4}-W\d\d-\d/],["GGGG-[W]WW",/\d{4}-W\d\d/,!1],["YYYY-DDD",/\d{4}-\d{3}/],["YYYY-MM",/\d{4}-\d\d/,!1],["YYYYYYMMDD",/[+-]\d{10}/],["YYYYMMDD",/\d{8}/],["GGGG[W]WWE",/\d{4}W\d{3}/],["GGGG[W]WW",/\d{4}W\d{2}/,!1],["YYYYDDD",/\d{7}/],["YYYYMM",/\d{6}/,!1],["YYYY",/\d{4}/,!1]],gt=[["HH:mm:ss.SSSS",/\d\d:\d\d:\d\d\.\d+/],["HH:mm:ss,SSSS",/\d\d:\d\d:\d\d,\d+/],["HH:mm:ss",/\d\d:\d\d:\d\d/],["HH:mm",/\d\d:\d\d/],["HHmmss.SSSS",/\d\d\d\d\d\d\.\d+/],["HHmmss,SSSS",/\d\d\d\d\d\d,\d+/],["HHmmss",/\d\d\d\d\d\d/],["HHmm",/\d\d\d\d/],["HH",/\d\d/]],wt=/^\/?Date\((-?\d+)/i,pt=/^(?:(Mon|Tue|Wed|Thu|Fri|Sat|Sun),?\s)?(\d{1,2})\s(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s(\d{2,4})\s(\d\d):(\d\d)(?::(\d\d))?\s(?:(UT|GMT|[ECMP][SD]T)|([Zz])|([+-]\d{4}))$/,vt={UT:0,GMT:0,EDT:-240,EST:-300,CDT:-300,CST:-360,MDT:-360,MST:-420,PDT:-420,PST:-480};function kt(e){var t,n,s,i,r,a,o=e._i,u=ft.exec(o)||mt.exec(o);if(u){for(y(e).iso=!0,t=0,n=yt.length;t<n;t++)if(yt[t][1].exec(u[1])){i=yt[t][0],s=!1!==yt[t][2];break}if(null==i)return void(e._isValid=!1);if(u[3]){for(t=0,n=gt.length;t<n;t++)if(gt[t][1].exec(u[3])){r=(u[2]||" ")+gt[t][0];break}if(null==r)return void(e._isValid=!1)}if(!s&&null!=r)return void(e._isValid=!1);if(u[4]){if(!_t.exec(u[4]))return void(e._isValid=!1);a="Z"}e._f=i+(r||"")+(a||""),Ot(e)}else e._isValid=!1}function Mt(e,t,n,s,i,r){var a=[function(e){var t=parseInt(e,10);{if(t<=49)return 2e3+t;if(t<=999)return 1900+t}return t}(e),Ne.indexOf(t),parseInt(n,10),parseInt(s,10),parseInt(i,10)];return r&&a.push(parseInt(r,10)),a}function Dt(e){var t,n,s,i,r=pt.exec(e._i.replace(/\([^)]*\)|[\n\t]/g," ").replace(/(\s\s+)/g," ").replace(/^\s\s*/,"").replace(/\s\s*$/,""));if(r){if(t=Mt(r[4],r[3],r[2],r[5],r[6],r[7]),n=r[1],s=t,i=e,n&&ze.indexOf(n)!==new Date(s[0],s[1],s[2]).getDay()&&(y(i).weekdayMismatch=!0,!void(i._isValid=!1)))return;e._a=t,e._tzm=function(e,t,n){if(e)return vt[e];if(t)return 0;var s=parseInt(n,10),i=s%100;return 60*((s-i)/100)+i}(r[8],r[9],r[10]),e._d=Ve.apply(null,e._a),e._d.setUTCMinutes(e._d.getUTCMinutes()-e._tzm),y(e).rfc2822=!0}else e._isValid=!1}function St(e,t,n){return null!=e?e:null!=t?t:n}function Yt(e){var t,n,s,i,r,a=[];if(!e._d){var o,u;for(o=e,u=new Date(f.now()),s=o._useUTC?[u.getUTCFullYear(),u.getUTCMonth(),u.getUTCDate()]:[u.getFullYear(),u.getMonth(),u.getDate()],e._w&&null==e._a[ke]&&null==e._a[ve]&&function(e){var t,n,s,i,r,a,o,u,l;null!=(t=e._w).GG||null!=t.W||null!=t.E?(r=1,a=4,n=St(t.GG,e._a[pe],Ae(Tt(),1,4).year),s=St(t.W,1),((i=St(t.E,1))<1||7<i)&&(u=!0)):(r=e._locale._week.dow,a=e._locale._week.doy,l=Ae(Tt(),r,a),n=St(t.gg,e._a[pe],l.year),s=St(t.w,l.week),null!=t.d?((i=t.d)<0||6<i)&&(u=!0):null!=t.e?(i=t.e+r,(t.e<0||6<t.e)&&(u=!0)):i=r);s<1||s>je(n,r,a)?y(e)._overflowWeeks=!0:null!=u?y(e)._overflowWeekday=!0:(o=Ee(n,s,i,r,a),e._a[pe]=o.year,e._dayOfYear=o.dayOfYear)}(e),null!=e._dayOfYear&&(r=St(e._a[pe],s[pe]),(e._dayOfYear>Fe(r)||0===e._dayOfYear)&&(y(e)._overflowDayOfYear=!0),n=Ve(r,0,e._dayOfYear),e._a[ve]=n.getUTCMonth(),e._a[ke]=n.getUTCDate()),t=0;t<3&&null==e._a[t];++t)e._a[t]=a[t]=s[t];for(;t<7;t++)e._a[t]=a[t]=null==e._a[t]?2===t?1:0:e._a[t];24===e._a[Me]&&0===e._a[De]&&0===e._a[Se]&&0===e._a[Ye]&&(e._nextDay=!0,e._a[Me]=0),e._d=(e._useUTC?Ve:function(e,t,n,s,i,r,a){var o;return e<100&&0<=e?(o=new Date(e+400,t,n,s,i,r,a),isFinite(o.getFullYear())&&o.setFullYear(e)):o=new Date(e,t,n,s,i,r,a),o}).apply(null,a),i=e._useUTC?e._d.getUTCDay():e._d.getDay(),null!=e._tzm&&e._d.setUTCMinutes(e._d.getUTCMinutes()-e._tzm),e._nextDay&&(e._a[Me]=24),e._w&&void 0!==e._w.d&&e._w.d!==i&&(y(e).weekdayMismatch=!0)}}function Ot(e){if(e._f!==f.ISO_8601)if(e._f!==f.RFC_2822){e._a=[],y(e).empty=!0;var t,n,s,i,r,a,o,u,l,h=""+e._i,d=h.length,c=0;for(s=H(e._f,e._locale).match(N)||[],t=0;t<s.length;t++)i=s[t],(n=(h.match(fe(i,e))||[])[0])&&(0<(r=h.substr(0,h.indexOf(n))).length&&y(e).unusedInput.push(r),h=h.slice(h.indexOf(n)+n.length),c+=n.length),W[i]?(n?y(e).empty=!1:y(e).unusedTokens.push(i),o=i,l=e,null!=(u=n)&&m(_e,o)&&_e[o](u,l._a,l,o)):e._strict&&!n&&y(e).unusedTokens.push(i);y(e).charsLeftOver=d-c,0<h.length&&y(e).unusedInput.push(h),e._a[Me]<=12&&!0===y(e).bigHour&&0<e._a[Me]&&(y(e).bigHour=void 0),y(e).parsedDateParts=e._a.slice(0),y(e).meridiem=e._meridiem,e._a[Me]=function(e,t,n){var s;if(null==n)return t;return null!=e.meridiemHour?e.meridiemHour(t,n):(null!=e.isPM&&((s=e.isPM(n))&&t<12&&(t+=12),s||12!==t||(t=0)),t)}(e._locale,e._a[Me],e._meridiem),null!==(a=y(e).era)&&(e._a[pe]=e._locale.erasConvertYear(a,e._a[pe])),Yt(e),ct(e)}else Dt(e);else kt(e)}function bt(e){var t,n,s=e._i,i=e._f;return e._locale=e._locale||dt(e._l),null===s||void 0===i&&""===s?w({nullInput:!0}):("string"==typeof s&&(e._i=s=e._locale.preparse(s)),M(s)?new k(ct(s)):(a(s)?e._d=s:o(i)?function(e){var t,n,s,i,r,a,o=!1;if(0===e._f.length)return y(e).invalidFormat=!0,e._d=new Date(NaN);for(i=0;i<e._f.length;i++)r=0,a=!1,t=v({},e),null!=e._useUTC&&(t._useUTC=e._useUTC),t._f=e._f[i],Ot(t),g(t)&&(a=!0),r+=y(t).charsLeftOver,r+=10*y(t).unusedTokens.length,y(t).score=r,o?r<s&&(s=r,n=t):(null==s||r<s||a)&&(s=r,n=t,a&&(o=!0));c(e,n||t)}(e):i?Ot(e):r(n=(t=e)._i)?t._d=new Date(f.now()):a(n)?t._d=new Date(n.valueOf()):"string"==typeof n?function(e){var t=wt.exec(e._i);null===t?(kt(e),!1===e._isValid&&(delete e._isValid,Dt(e),!1===e._isValid&&(delete e._isValid,e._strict?e._isValid=!1:f.createFromInputFallback(e)))):e._d=new Date(+t[1])}(t):o(n)?(t._a=d(n.slice(0),function(e){return parseInt(e,10)}),Yt(t)):u(n)?function(e){if(!e._d){var t=G(e._i),n=void 0===t.day?t.date:t.day;e._a=d([t.year,t.month,n,t.hour,t.minute,t.second,t.millisecond],function(e){return e&&parseInt(e,10)}),Yt(e)}}(t):h(n)?t._d=new Date(n):f.createFromInputFallback(t),g(e)||(e._d=null),e))}function xt(e,t,n,s,i){var r,a={};return!0!==t&&!1!==t||(s=t,t=void 0),!0!==n&&!1!==n||(s=n,n=void 0),(u(e)&&l(e)||o(e)&&0===e.length)&&(e=void 0),a._isAMomentObject=!0,a._useUTC=a._isUTC=i,a._l=n,a._i=e,a._f=t,a._strict=s,(r=new k(ct(bt(a))))._nextDay&&(r.add(1,"d"),r._nextDay=void 0),r}function Tt(e,t,n,s){return xt(e,t,n,s,!1)}f.createFromInputFallback=n("value provided is not in a recognized RFC2822 or ISO format. moment construction falls back to js Date(), which is not reliable across all browsers and versions. Non RFC2822/ISO date formats are discouraged and will be removed in an upcoming major release. Please refer to http://momentjs.com/guides/#/warnings/js-date/ for more info.",function(e){e._d=new Date(e._i+(e._useUTC?" UTC":""))}),f.ISO_8601=function(){},f.RFC_2822=function(){};var Nt=n("moment().min is deprecated, use moment.max instead. http://momentjs.com/guides/#/warnings/min-max/",function(){var e=Tt.apply(null,arguments);return this.isValid()&&e.isValid()?e<this?this:e:w()}),Pt=n("moment().max is deprecated, use moment.min instead. http://momentjs.com/guides/#/warnings/min-max/",function(){var e=Tt.apply(null,arguments);return this.isValid()&&e.isValid()?this<e?this:e:w()});function Rt(e,t){var n,s;if(1===t.length&&o(t[0])&&(t=t[0]),!t.length)return Tt();for(n=t[0],s=1;s<t.length;++s)t[s].isValid()&&!t[s][e](n)||(n=t[s]);return n}var Wt=["year","quarter","month","week","day","hour","minute","second","millisecond"];function Ct(e){var t=G(e),n=t.year||0,s=t.quarter||0,i=t.month||0,r=t.week||t.isoWeek||0,a=t.day||0,o=t.hour||0,u=t.minute||0,l=t.second||0,h=t.millisecond||0;this._isValid=function(e){var t,n,s=!1;for(t in e)if(m(e,t)&&(-1===we.call(Wt,t)||null!=e[t]&&isNaN(e[t])))return!1;for(n=0;n<Wt.length;++n)if(e[Wt[n]]){if(s)return!1;parseFloat(e[Wt[n]])!==Z(e[Wt[n]])&&(s=!0)}return!0}(t),this._milliseconds=+h+1e3*l+6e4*u+1e3*o*60*60,this._days=+a+7*r,this._months=+i+3*s+12*n,this._data={},this._locale=dt(),this._bubble()}function Ut(e){return e instanceof Ct}function Ht(e){return e<0?-1*Math.round(-1*e):Math.round(e)}function Ft(e,n){C(e,0,0,function(){var e=this.utcOffset(),t="+";return e<0&&(e=-e,t="-"),t+T(~~(e/60),2)+n+T(~~e%60,2)})}Ft("Z",":"),Ft("ZZ",""),ce("Z",he),ce("ZZ",he),ye(["Z","ZZ"],function(e,t,n){n._useUTC=!0,n._tzm=Vt(he,e)});var Lt=/([\+\-]|\d\d)/gi;function Vt(e,t){var n,s,i=(t||"").match(e);return null===i?null:0===(s=60*(n=((i[i.length-1]||[])+"").match(Lt)||["-",0,0])[1]+Z(n[2]))?0:"+"===n[0]?s:-s}function Gt(e,t){var n,s;return t._isUTC?(n=t.clone(),s=(M(e)||a(e)?e.valueOf():Tt(e).valueOf())-n.valueOf(),n._d.setTime(n._d.valueOf()+s),f.updateOffset(n,!1),n):Tt(e).local()}function Et(e){return-Math.round(e._d.getTimezoneOffset())}function At(){return!!this.isValid()&&(this._isUTC&&0===this._offset)}f.updateOffset=function(){};var jt=/^(-|\+)?(?:(\d*)[. ])?(\d+):(\d+)(?::(\d+)(\.\d*)?)?$/,It=/^(-|\+)?P(?:([-+]?[0-9,.]*)Y)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)W)?(?:([-+]?[0-9,.]*)D)?(?:T(?:([-+]?[0-9,.]*)H)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)S)?)?$/;function Zt(e,t){var n,s,i,r=e,a=null;return Ut(e)?r={ms:e._milliseconds,d:e._days,M:e._months}:h(e)||!isNaN(+e)?(r={},t?r[t]=+e:r.milliseconds=+e):(a=jt.exec(e))?(n="-"===a[1]?-1:1,r={y:0,d:Z(a[ke])*n,h:Z(a[Me])*n,m:Z(a[De])*n,s:Z(a[Se])*n,ms:Z(Ht(1e3*a[Ye]))*n}):(a=It.exec(e))?(n="-"===a[1]?-1:1,r={y:zt(a[2],n),M:zt(a[3],n),w:zt(a[4],n),d:zt(a[5],n),h:zt(a[6],n),m:zt(a[7],n),s:zt(a[8],n)}):null==r?r={}:"object"==typeof r&&("from"in r||"to"in r)&&(i=function(e,t){var n;if(!e.isValid()||!t.isValid())return{milliseconds:0,months:0};t=Gt(t,e),e.isBefore(t)?n=$t(e,t):((n=$t(t,e)).milliseconds=-n.milliseconds,n.months=-n.months);return n}(Tt(r.from),Tt(r.to)),(r={}).ms=i.milliseconds,r.M=i.months),s=new Ct(r),Ut(e)&&m(e,"_locale")&&(s._locale=e._locale),Ut(e)&&m(e,"_isValid")&&(s._isValid=e._isValid),s}function zt(e,t){var n=e&&parseFloat(e.replace(",","."));return(isNaN(n)?0:n)*t}function $t(e,t){var n={};return n.months=t.month()-e.month()+12*(t.year()-e.year()),e.clone().add(n.months,"M").isAfter(t)&&--n.months,n.milliseconds=t-e.clone().add(n.months,"M"),n}function qt(s,i){return function(e,t){var n;return null===t||isNaN(+t)||(Y(i,"moment()."+i+"(period, number) is deprecated. Please use moment()."+i+"(number, period). See http://momentjs.com/guides/#/warnings/add-inverted-param/ for more info."),n=e,e=t,t=n),Bt(this,Zt(e,t),s),this}}function Bt(e,t,n,s){var i=t._milliseconds,r=Ht(t._days),a=Ht(t._months);e.isValid()&&(s=null==s||s,a&&Ce(e,$(e,"Month")+a*n),r&&q(e,"Date",$(e,"Date")+r*n),i&&e._d.setTime(e._d.valueOf()+i*n),s&&f.updateOffset(e,r||a))}Zt.fn=Ct.prototype,Zt.invalid=function(){return Zt(NaN)};var Jt=qt(1,"add"),Qt=qt(-1,"subtract");function Xt(e){return"string"==typeof e||e instanceof String}function Kt(e){return M(e)||a(e)||Xt(e)||h(e)||function(t){var e=o(t),n=!1;e&&(n=0===t.filter(function(e){return!h(e)&&Xt(t)}).length);return e&&n}(e)||function(e){var t,n,s=u(e)&&!l(e),i=!1,r=["years","year","y","months","month","M","days","day","d","dates","date","D","hours","hour","h","minutes","minute","m","seconds","second","s","milliseconds","millisecond","ms"];for(t=0;t<r.length;t+=1)n=r[t],i=i||m(e,n);return s&&i}(e)||null==e}function en(e,t){if(e.date()<t.date())return-en(t,e);var n=12*(t.year()-e.year())+(t.month()-e.month()),s=e.clone().add(n,"months");return-(n+(t-s<0?(t-s)/(s-e.clone().add(n-1,"months")):(t-s)/(e.clone().add(1+n,"months")-s)))||0}function tn(e){var t;return void 0===e?this._locale._abbr:(null!=(t=dt(e))&&(this._locale=t),this)}f.defaultFormat="YYYY-MM-DDTHH:mm:ssZ",f.defaultFormatUtc="YYYY-MM-DDTHH:mm:ss[Z]";var nn=n("moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.",function(e){return void 0===e?this.localeData():this.locale(e)});function sn(){return this._locale}var rn=126227808e5;function an(e,t){return(e%t+t)%t}function on(e,t,n){return e<100&&0<=e?new Date(e+400,t,n)-rn:new Date(e,t,n).valueOf()}function un(e,t,n){return e<100&&0<=e?Date.UTC(e+400,t,n)-rn:Date.UTC(e,t,n)}function ln(e,t){return t.erasAbbrRegex(e)}function hn(){var e,t,n=[],s=[],i=[],r=[],a=this.eras();for(e=0,t=a.length;e<t;++e)s.push(me(a[e].name)),n.push(me(a[e].abbr)),i.push(me(a[e].narrow)),r.push(me(a[e].name)),r.push(me(a[e].abbr)),r.push(me(a[e].narrow));this._erasRegex=new RegExp("^("+r.join("|")+")","i"),this._erasNameRegex=new RegExp("^("+s.join("|")+")","i"),this._erasAbbrRegex=new RegExp("^("+n.join("|")+")","i"),this._erasNarrowRegex=new RegExp("^("+i.join("|")+")","i")}function dn(e,t){C(0,[e,e.length],0,t)}function cn(e,t,n,s,i){var r;return null==e?Ae(this,s,i).year:((r=je(e,s,i))<t&&(t=r),function(e,t,n,s,i){var r=Ee(e,t,n,s,i),a=Ve(r.year,0,r.dayOfYear);return this.year(a.getUTCFullYear()),this.month(a.getUTCMonth()),this.date(a.getUTCDate()),this}.call(this,e,t,n,s,i))}C("N",0,0,"eraAbbr"),C("NN",0,0,"eraAbbr"),C("NNN",0,0,"eraAbbr"),C("NNNN",0,0,"eraName"),C("NNNNN",0,0,"eraNarrow"),C("y",["y",1],"yo","eraYear"),C("y",["yy",2],0,"eraYear"),C("y",["yyy",3],0,"eraYear"),C("y",["yyyy",4],0,"eraYear"),ce("N",ln),ce("NN",ln),ce("NNN",ln),ce("NNNN",function(e,t){return t.erasNameRegex(e)}),ce("NNNNN",function(e,t){return t.erasNarrowRegex(e)}),ye(["N","NN","NNN","NNNN","NNNNN"],function(e,t,n,s){var i=n._locale.erasParse(e,s,n._strict);i?y(n).era=i:y(n).invalidEra=e}),ce("y",oe),ce("yy",oe),ce("yyy",oe),ce("yyyy",oe),ce("yo",function(e,t){return t._eraYearOrdinalRegex||oe}),ye(["y","yy","yyy","yyyy"],pe),ye(["yo"],function(e,t,n,s){var i;n._locale._eraYearOrdinalRegex&&(i=e.match(n._locale._eraYearOrdinalRegex)),n._locale.eraYearOrdinalParse?t[pe]=n._locale.eraYearOrdinalParse(e,i):t[pe]=parseInt(e,10)}),C(0,["gg",2],0,function(){return this.weekYear()%100}),C(0,["GG",2],0,function(){return this.isoWeekYear()%100}),dn("gggg","weekYear"),dn("ggggg","weekYear"),dn("GGGG","isoWeekYear"),dn("GGGGG","isoWeekYear"),L("weekYear","gg"),L("isoWeekYear","GG"),A("weekYear",1),A("isoWeekYear",1),ce("G",ue),ce("g",ue),ce("GG",te,Q),ce("gg",te,Q),ce("GGGG",re,K),ce("gggg",re,K),ce("GGGGG",ae,ee),ce("ggggg",ae,ee),ge(["gggg","ggggg","GGGG","GGGGG"],function(e,t,n,s){t[s.substr(0,2)]=Z(e)}),ge(["gg","GG"],function(e,t,n,s){t[s]=f.parseTwoDigitYear(e)}),C("Q",0,"Qo","quarter"),L("quarter","Q"),A("quarter",7),ce("Q",J),ye("Q",function(e,t){t[ve]=3*(Z(e)-1)}),C("D",["DD",2],"Do","date"),L("date","D"),A("date",9),ce("D",te),ce("DD",te,Q),ce("Do",function(e,t){return e?t._dayOfMonthOrdinalParse||t._ordinalParse:t._dayOfMonthOrdinalParseLenient}),ye(["D","DD"],ke),ye("Do",function(e,t){t[ke]=Z(e.match(te)[0])});var fn=z("Date",!0);C("DDD",["DDDD",3],"DDDo","dayOfYear"),L("dayOfYear","DDD"),A("dayOfYear",4),ce("DDD",ie),ce("DDDD",X),ye(["DDD","DDDD"],function(e,t,n){n._dayOfYear=Z(e)}),C("m",["mm",2],0,"minute"),L("minute","m"),A("minute",14),ce("m",te),ce("mm",te,Q),ye(["m","mm"],De);var mn=z("Minutes",!1);C("s",["ss",2],0,"second"),L("second","s"),A("second",15),ce("s",te),ce("ss",te,Q),ye(["s","ss"],Se);var _n,yn,gn=z("Seconds",!1);for(C("S",0,0,function(){return~~(this.millisecond()/100)}),C(0,["SS",2],0,function(){return~~(this.millisecond()/10)}),C(0,["SSS",3],0,"millisecond"),C(0,["SSSS",4],0,function(){return 10*this.millisecond()}),C(0,["SSSSS",5],0,function(){return 100*this.millisecond()}),C(0,["SSSSSS",6],0,function(){return 1e3*this.millisecond()}),C(0,["SSSSSSS",7],0,function(){return 1e4*this.millisecond()}),C(0,["SSSSSSSS",8],0,function(){return 1e5*this.millisecond()}),C(0,["SSSSSSSSS",9],0,function(){return 1e6*this.millisecond()}),L("millisecond","ms"),A("millisecond",16),ce("S",ie,J),ce("SS",ie,Q),ce("SSS",ie,X),_n="SSSS";_n.length<=9;_n+="S")ce(_n,oe);function wn(e,t){t[Ye]=Z(1e3*("0."+e))}for(_n="S";_n.length<=9;_n+="S")ye(_n,wn);yn=z("Milliseconds",!1),C("z",0,0,"zoneAbbr"),C("zz",0,0,"zoneName");var pn=k.prototype;function vn(e){return e}pn.add=Jt,pn.calendar=function(e,t){1===arguments.length&&(Kt(arguments[0])?(e=arguments[0],t=void 0):function(e){var t,n=u(e)&&!l(e),s=!1,i=["sameDay","nextDay","lastDay","nextWeek","lastWeek","sameElse"];for(t=0;t<i.length;t+=1)s=s||m(e,i[t]);return n&&s}(arguments[0])&&(t=arguments[0],e=void 0));var n=e||Tt(),s=Gt(n,this).startOf("day"),i=f.calendarFormat(this,s)||"sameElse",r=t&&(O(t[i])?t[i].call(this,n):t[i]);return this.format(r||this.localeData().calendar(i,this,Tt(n)))},pn.clone=function(){return new k(this)},pn.diff=function(e,t,n){var s,i,r;if(!this.isValid())return NaN;if(!(s=Gt(e,this)).isValid())return NaN;switch(i=6e4*(s.utcOffset()-this.utcOffset()),t=V(t)){case"year":r=en(this,s)/12;break;case"month":r=en(this,s);break;case"quarter":r=en(this,s)/3;break;case"second":r=(this-s)/1e3;break;case"minute":r=(this-s)/6e4;break;case"hour":r=(this-s)/36e5;break;case"day":r=(this-s-i)/864e5;break;case"week":r=(this-s-i)/6048e5;break;default:r=this-s}return n?r:I(r)},pn.endOf=function(e){var t,n;if(void 0===(e=V(e))||"millisecond"===e||!this.isValid())return this;switch(n=this._isUTC?un:on,e){case"year":t=n(this.year()+1,0,1)-1;break;case"quarter":t=n(this.year(),this.month()-this.month()%3+3,1)-1;break;case"month":t=n(this.year(),this.month()+1,1)-1;break;case"week":t=n(this.year(),this.month(),this.date()-this.weekday()+7)-1;break;case"isoWeek":t=n(this.year(),this.month(),this.date()-(this.isoWeekday()-1)+7)-1;break;case"day":case"date":t=n(this.year(),this.month(),this.date()+1)-1;break;case"hour":t=this._d.valueOf(),t+=36e5-an(t+(this._isUTC?0:6e4*this.utcOffset()),36e5)-1;break;case"minute":t=this._d.valueOf(),t+=6e4-an(t,6e4)-1;break;case"second":t=this._d.valueOf(),t+=1e3-an(t,1e3)-1;break}return this._d.setTime(t),f.updateOffset(this,!0),this},pn.format=function(e){e=e||(this.isUtc()?f.defaultFormatUtc:f.defaultFormat);var t=U(this,e);return this.localeData().postformat(t)},pn.from=function(e,t){return this.isValid()&&(M(e)&&e.isValid()||Tt(e).isValid())?Zt({to:this,from:e}).locale(this.locale()).humanize(!t):this.localeData().invalidDate()},pn.fromNow=function(e){return this.from(Tt(),e)},pn.to=function(e,t){return this.isValid()&&(M(e)&&e.isValid()||Tt(e).isValid())?Zt({from:this,to:e}).locale(this.locale()).humanize(!t):this.localeData().invalidDate()},pn.toNow=function(e){return this.to(Tt(),e)},pn.get=function(e){return O(this[e=V(e)])?this[e]():this},pn.invalidAt=function(){return y(this).overflow},pn.isAfter=function(e,t){var n=M(e)?e:Tt(e);return!(!this.isValid()||!n.isValid())&&("millisecond"===(t=V(t)||"millisecond")?this.valueOf()>n.valueOf():n.valueOf()<this.clone().startOf(t).valueOf())},pn.isBefore=function(e,t){var n=M(e)?e:Tt(e);return!(!this.isValid()||!n.isValid())&&("millisecond"===(t=V(t)||"millisecond")?this.valueOf()<n.valueOf():this.clone().endOf(t).valueOf()<n.valueOf())},pn.isBetween=function(e,t,n,s){var i=M(e)?e:Tt(e),r=M(t)?t:Tt(t);return!!(this.isValid()&&i.isValid()&&r.isValid())&&(("("===(s=s||"()")[0]?this.isAfter(i,n):!this.isBefore(i,n))&&(")"===s[1]?this.isBefore(r,n):!this.isAfter(r,n)))},pn.isSame=function(e,t){var n,s=M(e)?e:Tt(e);return!(!this.isValid()||!s.isValid())&&("millisecond"===(t=V(t)||"millisecond")?this.valueOf()===s.valueOf():(n=s.valueOf(),this.clone().startOf(t).valueOf()<=n&&n<=this.clone().endOf(t).valueOf()))},pn.isSameOrAfter=function(e,t){return this.isSame(e,t)||this.isAfter(e,t)},pn.isSameOrBefore=function(e,t){return this.isSame(e,t)||this.isBefore(e,t)},pn.isValid=function(){return g(this)},pn.lang=nn,pn.locale=tn,pn.localeData=sn,pn.max=Pt,pn.min=Nt,pn.parsingFlags=function(){return c({},y(this))},pn.set=function(e,t){if("object"==typeof e){var n,s=function(e){var t,n=[];for(t in e)m(e,t)&&n.push({unit:t,priority:E[t]});return n.sort(function(e,t){return e.priority-t.priority}),n}(e=G(e));for(n=0;n<s.length;n++)this[s[n].unit](e[s[n].unit])}else if(O(this[e=V(e)]))return this[e](t);return this},pn.startOf=function(e){var t,n;if(void 0===(e=V(e))||"millisecond"===e||!this.isValid())return this;switch(n=this._isUTC?un:on,e){case"year":t=n(this.year(),0,1);break;case"quarter":t=n(this.year(),this.month()-this.month()%3,1);break;case"month":t=n(this.year(),this.month(),1);break;case"week":t=n(this.year(),this.month(),this.date()-this.weekday());break;case"isoWeek":t=n(this.year(),this.month(),this.date()-(this.isoWeekday()-1));break;case"day":case"date":t=n(this.year(),this.month(),this.date());break;case"hour":t=this._d.valueOf(),t-=an(t+(this._isUTC?0:6e4*this.utcOffset()),36e5);break;case"minute":t=this._d.valueOf(),t-=an(t,6e4);break;case"second":t=this._d.valueOf(),t-=an(t,1e3);break}return this._d.setTime(t),f.updateOffset(this,!0),this},pn.subtract=Qt,pn.toArray=function(){var e=this;return[e.year(),e.month(),e.date(),e.hour(),e.minute(),e.second(),e.millisecond()]},pn.toObject=function(){var e=this;return{years:e.year(),months:e.month(),date:e.date(),hours:e.hours(),minutes:e.minutes(),seconds:e.seconds(),milliseconds:e.milliseconds()}},pn.toDate=function(){return new Date(this.valueOf())},pn.toISOString=function(e){if(!this.isValid())return null;var t=!0!==e,n=t?this.clone().utc():this;return n.year()<0||9999<n.year()?U(n,t?"YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYYYY-MM-DD[T]HH:mm:ss.SSSZ"):O(Date.prototype.toISOString)?t?this.toDate().toISOString():new Date(this.valueOf()+60*this.utcOffset()*1e3).toISOString().replace("Z",U(n,"Z")):U(n,t?"YYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYY-MM-DD[T]HH:mm:ss.SSSZ")},pn.inspect=function(){if(!this.isValid())return"moment.invalid(/* "+this._i+" */)";var e,t,n,s="moment",i="";return this.isLocal()||(s=0===this.utcOffset()?"moment.utc":"moment.parseZone",i="Z"),e="["+s+'("]',t=0<=this.year()&&this.year()<=9999?"YYYY":"YYYYYY",n=i+'[")]',this.format(e+t+"-MM-DD[T]HH:mm:ss.SSS"+n)},"undefined"!=typeof Symbol&&null!=Symbol.for&&(pn[Symbol.for("nodejs.util.inspect.custom")]=function(){return"Moment<"+this.format()+">"}),pn.toJSON=function(){return this.isValid()?this.toISOString():null},pn.toString=function(){return this.clone().locale("en").format("ddd MMM DD YYYY HH:mm:ss [GMT]ZZ")},pn.unix=function(){return Math.floor(this.valueOf()/1e3)},pn.valueOf=function(){return this._d.valueOf()-6e4*(this._offset||0)},pn.creationData=function(){return{input:this._i,format:this._f,locale:this._locale,isUTC:this._isUTC,strict:this._strict}},pn.eraName=function(){var e,t,n,s=this.localeData().eras();for(e=0,t=s.length;e<t;++e){if(n=this.startOf("day").valueOf(),s[e].since<=n&&n<=s[e].until)return s[e].name;if(s[e].until<=n&&n<=s[e].since)return s[e].name}return""},pn.eraNarrow=function(){var e,t,n,s=this.localeData().eras();for(e=0,t=s.length;e<t;++e){if(n=this.startOf("day").valueOf(),s[e].since<=n&&n<=s[e].until)return s[e].narrow;if(s[e].until<=n&&n<=s[e].since)return s[e].narrow}return""},pn.eraAbbr=function(){var e,t,n,s=this.localeData().eras();for(e=0,t=s.length;e<t;++e){if(n=this.startOf("day").valueOf(),s[e].since<=n&&n<=s[e].until)return s[e].abbr;if(s[e].until<=n&&n<=s[e].since)return s[e].abbr}return""},pn.eraYear=function(){var e,t,n,s,i=this.localeData().eras();for(e=0,t=i.length;e<t;++e)if(n=i[e].since<=i[e].until?1:-1,s=this.startOf("day").valueOf(),i[e].since<=s&&s<=i[e].until||i[e].until<=s&&s<=i[e].since)return(this.year()-f(i[e].since).year())*n+i[e].offset;return this.year()},pn.year=Le,pn.isLeapYear=function(){return j(this.year())},pn.weekYear=function(e){return cn.call(this,e,this.week(),this.weekday(),this.localeData()._week.dow,this.localeData()._week.doy)},pn.isoWeekYear=function(e){return cn.call(this,e,this.isoWeek(),this.isoWeekday(),1,4)},pn.quarter=pn.quarters=function(e){return null==e?Math.ceil((this.month()+1)/3):this.month(3*(e-1)+this.month()%3)},pn.month=Ue,pn.daysInMonth=function(){return xe(this.year(),this.month())},pn.week=pn.weeks=function(e){var t=this.localeData().week(this);return null==e?t:this.add(7*(e-t),"d")},pn.isoWeek=pn.isoWeeks=function(e){var t=Ae(this,1,4).week;return null==e?t:this.add(7*(e-t),"d")},pn.weeksInYear=function(){var e=this.localeData()._week;return je(this.year(),e.dow,e.doy)},pn.weeksInWeekYear=function(){var e=this.localeData()._week;return je(this.weekYear(),e.dow,e.doy)},pn.isoWeeksInYear=function(){return je(this.year(),1,4)},pn.isoWeeksInISOWeekYear=function(){return je(this.isoWeekYear(),1,4)},pn.date=fn,pn.day=pn.days=function(e){if(!this.isValid())return null!=e?this:NaN;var t,n,s=this._isUTC?this._d.getUTCDay():this._d.getDay();return null!=e?(t=e,n=this.localeData(),e="string"!=typeof t?t:isNaN(t)?"number"==typeof(t=n.weekdaysParse(t))?t:null:parseInt(t,10),this.add(e-s,"d")):s},pn.weekday=function(e){if(!this.isValid())return null!=e?this:NaN;var t=(this.day()+7-this.localeData()._week.dow)%7;return null==e?t:this.add(e-t,"d")},pn.isoWeekday=function(e){if(!this.isValid())return null!=e?this:NaN;if(null==e)return this.day()||7;var t,n,s=(t=e,n=this.localeData(),"string"==typeof t?n.weekdaysParse(t)%7||7:isNaN(t)?null:t);return this.day(this.day()%7?s:s-7)},pn.dayOfYear=function(e){var t=Math.round((this.clone().startOf("day")-this.clone().startOf("year"))/864e5)+1;return null==e?t:this.add(e-t,"d")},pn.hour=pn.hours=tt,pn.minute=pn.minutes=mn,pn.second=pn.seconds=gn,pn.millisecond=pn.milliseconds=yn,pn.utcOffset=function(e,t,n){var s,i=this._offset||0;if(!this.isValid())return null!=e?this:NaN;if(null==e)return this._isUTC?i:Et(this);if("string"==typeof e){if(null===(e=Vt(he,e)))return this}else Math.abs(e)<16&&!n&&(e*=60);return!this._isUTC&&t&&(s=Et(this)),this._offset=e,this._isUTC=!0,null!=s&&this.add(s,"m"),i!==e&&(!t||this._changeInProgress?Bt(this,Zt(e-i,"m"),1,!1):this._changeInProgress||(this._changeInProgress=!0,f.updateOffset(this,!0),this._changeInProgress=null)),this},pn.utc=function(e){return this.utcOffset(0,e)},pn.local=function(e){return this._isUTC&&(this.utcOffset(0,e),this._isUTC=!1,e&&this.subtract(Et(this),"m")),this},pn.parseZone=function(){if(null!=this._tzm)this.utcOffset(this._tzm,!1,!0);else if("string"==typeof this._i){var e=Vt(le,this._i);null!=e?this.utcOffset(e):this.utcOffset(0,!0)}return this},pn.hasAlignedHourOffset=function(e){return!!this.isValid()&&(e=e?Tt(e).utcOffset():0,(this.utcOffset()-e)%60==0)},pn.isDST=function(){return this.utcOffset()>this.clone().month(0).utcOffset()||this.utcOffset()>this.clone().month(5).utcOffset()},pn.isLocal=function(){return!!this.isValid()&&!this._isUTC},pn.isUtcOffset=function(){return!!this.isValid()&&this._isUTC},pn.isUtc=At,pn.isUTC=At,pn.zoneAbbr=function(){return this._isUTC?"UTC":""},pn.zoneName=function(){return this._isUTC?"Coordinated Universal Time":""},pn.dates=n("dates accessor is deprecated. Use date instead.",fn),pn.months=n("months accessor is deprecated. Use month instead",Ue),pn.years=n("years accessor is deprecated. Use year instead",Le),pn.zone=n("moment().zone is deprecated, use moment().utcOffset instead. http://momentjs.com/guides/#/warnings/zone/",function(e,t){return null!=e?("string"!=typeof e&&(e=-e),this.utcOffset(e,t),this):-this.utcOffset()}),pn.isDSTShifted=n("isDSTShifted is deprecated. See http://momentjs.com/guides/#/warnings/dst-shifted/ for more information",function(){if(!r(this._isDSTShifted))return this._isDSTShifted;var e,t={};return v(t,this),(t=bt(t))._a?(e=(t._isUTC?_:Tt)(t._a),this._isDSTShifted=this.isValid()&&0<function(e,t,n){var s,i=Math.min(e.length,t.length),r=Math.abs(e.length-t.length),a=0;for(s=0;s<i;s++)(n&&e[s]!==t[s]||!n&&Z(e[s])!==Z(t[s]))&&a++;return a+r}(t._a,e.toArray())):this._isDSTShifted=!1,this._isDSTShifted});var kn=x.prototype;function Mn(e,t,n,s){var i=dt(),r=_().set(s,t);return i[n](r,e)}function Dn(e,t,n){if(h(e)&&(t=e,e=void 0),e=e||"",null!=t)return Mn(e,t,n,"month");var s,i=[];for(s=0;s<12;s++)i[s]=Mn(e,s,n,"month");return i}function Sn(e,t,n,s){t=("boolean"==typeof e?h(t)&&(n=t,t=void 0):(t=e,e=!1,h(n=t)&&(n=t,t=void 0)),t||"");var i,r=dt(),a=e?r._week.dow:0,o=[];if(null!=n)return Mn(t,(n+a)%7,s,"day");for(i=0;i<7;i++)o[i]=Mn(t,(i+a)%7,s,"day");return o}kn.calendar=function(e,t,n){var s=this._calendar[e]||this._calendar.sameElse;return O(s)?s.call(t,n):s},kn.longDateFormat=function(e){var t=this._longDateFormat[e],n=this._longDateFormat[e.toUpperCase()];return t||!n?t:(this._longDateFormat[e]=n.match(N).map(function(e){return"MMMM"===e||"MM"===e||"DD"===e||"dddd"===e?e.slice(1):e}).join(""),this._longDateFormat[e])},kn.invalidDate=function(){return this._invalidDate},kn.ordinal=function(e){return this._ordinal.replace("%d",e)},kn.preparse=vn,kn.postformat=vn,kn.relativeTime=function(e,t,n,s){var i=this._relativeTime[n];return O(i)?i(e,t,n,s):i.replace(/%d/i,e)},kn.pastFuture=function(e,t){var n=this._relativeTime[0<e?"future":"past"];return O(n)?n(t):n.replace(/%s/i,t)},kn.set=function(e){var t,n;for(n in e)m(e,n)&&(O(t=e[n])?this[n]=t:this["_"+n]=t);this._config=e,this._dayOfMonthOrdinalParseLenient=new RegExp((this._dayOfMonthOrdinalParse.source||this._ordinalParse.source)+"|"+/\d{1,2}/.source)},kn.eras=function(e,t){var n,s,i,r=this._eras||dt("en")._eras;for(n=0,s=r.length;n<s;++n){switch(typeof r[n].since){case"string":i=f(r[n].since).startOf("day"),r[n].since=i.valueOf();break}switch(typeof r[n].until){case"undefined":r[n].until=1/0;break;case"string":i=f(r[n].until).startOf("day").valueOf(),r[n].until=i.valueOf();break}}return r},kn.erasParse=function(e,t,n){var s,i,r,a,o,u=this.eras();for(e=e.toUpperCase(),s=0,i=u.length;s<i;++s)if(r=u[s].name.toUpperCase(),a=u[s].abbr.toUpperCase(),o=u[s].narrow.toUpperCase(),n)switch(t){case"N":case"NN":case"NNN":if(a===e)return u[s];break;case"NNNN":if(r===e)return u[s];break;case"NNNNN":if(o===e)return u[s];break}else if(0<=[r,a,o].indexOf(e))return u[s]},kn.erasConvertYear=function(e,t){var n=e.since<=e.until?1:-1;return void 0===t?f(e.since).year():f(e.since).year()+(t-e.offset)*n},kn.erasAbbrRegex=function(e){return m(this,"_erasAbbrRegex")||hn.call(this),e?this._erasAbbrRegex:this._erasRegex},kn.erasNameRegex=function(e){return m(this,"_erasNameRegex")||hn.call(this),e?this._erasNameRegex:this._erasRegex},kn.erasNarrowRegex=function(e){return m(this,"_erasNarrowRegex")||hn.call(this),e?this._erasNarrowRegex:this._erasRegex},kn.months=function(e,t){return e?o(this._months)?this._months[e.month()]:this._months[(this._months.isFormat||Pe).test(t)?"format":"standalone"][e.month()]:o(this._months)?this._months:this._months.standalone},kn.monthsShort=function(e,t){return e?o(this._monthsShort)?this._monthsShort[e.month()]:this._monthsShort[Pe.test(t)?"format":"standalone"][e.month()]:o(this._monthsShort)?this._monthsShort:this._monthsShort.standalone},kn.monthsParse=function(e,t,n){var s,i,r;if(this._monthsParseExact)return function(e,t,n){var s,i,r,a=e.toLocaleLowerCase();if(!this._monthsParse)for(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[],s=0;s<12;++s)r=_([2e3,s]),this._shortMonthsParse[s]=this.monthsShort(r,"").toLocaleLowerCase(),this._longMonthsParse[s]=this.months(r,"").toLocaleLowerCase();return n?"MMM"===t?-1!==(i=we.call(this._shortMonthsParse,a))?i:null:-1!==(i=we.call(this._longMonthsParse,a))?i:null:"MMM"===t?-1!==(i=we.call(this._shortMonthsParse,a))||-1!==(i=we.call(this._longMonthsParse,a))?i:null:-1!==(i=we.call(this._longMonthsParse,a))||-1!==(i=we.call(this._shortMonthsParse,a))?i:null}.call(this,e,t,n);for(this._monthsParse||(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[]),s=0;s<12;s++){if(i=_([2e3,s]),n&&!this._longMonthsParse[s]&&(this._longMonthsParse[s]=new RegExp("^"+this.months(i,"").replace(".","")+"$","i"),this._shortMonthsParse[s]=new RegExp("^"+this.monthsShort(i,"").replace(".","")+"$","i")),n||this._monthsParse[s]||(r="^"+this.months(i,"")+"|^"+this.monthsShort(i,""),this._monthsParse[s]=new RegExp(r.replace(".",""),"i")),n&&"MMMM"===t&&this._longMonthsParse[s].test(e))return s;if(n&&"MMM"===t&&this._shortMonthsParse[s].test(e))return s;if(!n&&this._monthsParse[s].test(e))return s}},kn.monthsRegex=function(e){return this._monthsParseExact?(m(this,"_monthsRegex")||He.call(this),e?this._monthsStrictRegex:this._monthsRegex):(m(this,"_monthsRegex")||(this._monthsRegex=We),this._monthsStrictRegex&&e?this._monthsStrictRegex:this._monthsRegex)},kn.monthsShortRegex=function(e){return this._monthsParseExact?(m(this,"_monthsRegex")||He.call(this),e?this._monthsShortStrictRegex:this._monthsShortRegex):(m(this,"_monthsShortRegex")||(this._monthsShortRegex=Re),this._monthsShortStrictRegex&&e?this._monthsShortStrictRegex:this._monthsShortRegex)},kn.week=function(e){return Ae(e,this._week.dow,this._week.doy).week},kn.firstDayOfYear=function(){return this._week.doy},kn.firstDayOfWeek=function(){return this._week.dow},kn.weekdays=function(e,t){var n=o(this._weekdays)?this._weekdays:this._weekdays[e&&!0!==e&&this._weekdays.isFormat.test(t)?"format":"standalone"];return!0===e?Ie(n,this._week.dow):e?n[e.day()]:n},kn.weekdaysMin=function(e){return!0===e?Ie(this._weekdaysMin,this._week.dow):e?this._weekdaysMin[e.day()]:this._weekdaysMin},kn.weekdaysShort=function(e){return!0===e?Ie(this._weekdaysShort,this._week.dow):e?this._weekdaysShort[e.day()]:this._weekdaysShort},kn.weekdaysParse=function(e,t,n){var s,i,r;if(this._weekdaysParseExact)return function(e,t,n){var s,i,r,a=e.toLocaleLowerCase();if(!this._weekdaysParse)for(this._weekdaysParse=[],this._shortWeekdaysParse=[],this._minWeekdaysParse=[],s=0;s<7;++s)r=_([2e3,1]).day(s),this._minWeekdaysParse[s]=this.weekdaysMin(r,"").toLocaleLowerCase(),this._shortWeekdaysParse[s]=this.weekdaysShort(r,"").toLocaleLowerCase(),this._weekdaysParse[s]=this.weekdays(r,"").toLocaleLowerCase();return n?"dddd"===t?-1!==(i=we.call(this._weekdaysParse,a))?i:null:"ddd"===t?-1!==(i=we.call(this._shortWeekdaysParse,a))?i:null:-1!==(i=we.call(this._minWeekdaysParse,a))?i:null:"dddd"===t?-1!==(i=we.call(this._weekdaysParse,a))||-1!==(i=we.call(this._shortWeekdaysParse,a))||-1!==(i=we.call(this._minWeekdaysParse,a))?i:null:"ddd"===t?-1!==(i=we.call(this._shortWeekdaysParse,a))||-1!==(i=we.call(this._weekdaysParse,a))||-1!==(i=we.call(this._minWeekdaysParse,a))?i:null:-1!==(i=we.call(this._minWeekdaysParse,a))||-1!==(i=we.call(this._weekdaysParse,a))||-1!==(i=we.call(this._shortWeekdaysParse,a))?i:null}.call(this,e,t,n);for(this._weekdaysParse||(this._weekdaysParse=[],this._minWeekdaysParse=[],this._shortWeekdaysParse=[],this._fullWeekdaysParse=[]),s=0;s<7;s++){if(i=_([2e3,1]).day(s),n&&!this._fullWeekdaysParse[s]&&(this._fullWeekdaysParse[s]=new RegExp("^"+this.weekdays(i,"").replace(".","\\.?")+"$","i"),this._shortWeekdaysParse[s]=new RegExp("^"+this.weekdaysShort(i,"").replace(".","\\.?")+"$","i"),this._minWeekdaysParse[s]=new RegExp("^"+this.weekdaysMin(i,"").replace(".","\\.?")+"$","i")),this._weekdaysParse[s]||(r="^"+this.weekdays(i,"")+"|^"+this.weekdaysShort(i,"")+"|^"+this.weekdaysMin(i,""),this._weekdaysParse[s]=new RegExp(r.replace(".",""),"i")),n&&"dddd"===t&&this._fullWeekdaysParse[s].test(e))return s;if(n&&"ddd"===t&&this._shortWeekdaysParse[s].test(e))return s;if(n&&"dd"===t&&this._minWeekdaysParse[s].test(e))return s;if(!n&&this._weekdaysParse[s].test(e))return s}},kn.weekdaysRegex=function(e){return this._weekdaysParseExact?(m(this,"_weekdaysRegex")||Qe.call(this),e?this._weekdaysStrictRegex:this._weekdaysRegex):(m(this,"_weekdaysRegex")||(this._weekdaysRegex=qe),this._weekdaysStrictRegex&&e?this._weekdaysStrictRegex:this._weekdaysRegex)},kn.weekdaysShortRegex=function(e){return this._weekdaysParseExact?(m(this,"_weekdaysRegex")||Qe.call(this),e?this._weekdaysShortStrictRegex:this._weekdaysShortRegex):(m(this,"_weekdaysShortRegex")||(this._weekdaysShortRegex=Be),this._weekdaysShortStrictRegex&&e?this._weekdaysShortStrictRegex:this._weekdaysShortRegex)},kn.weekdaysMinRegex=function(e){return this._weekdaysParseExact?(m(this,"_weekdaysRegex")||Qe.call(this),e?this._weekdaysMinStrictRegex:this._weekdaysMinRegex):(m(this,"_weekdaysMinRegex")||(this._weekdaysMinRegex=Je),this._weekdaysMinStrictRegex&&e?this._weekdaysMinStrictRegex:this._weekdaysMinRegex)},kn.isPM=function(e){return"p"===(e+"").toLowerCase().charAt(0)},kn.meridiem=function(e,t,n){return 11<e?n?"pm":"PM":n?"am":"AM"},lt("en",{eras:[{since:"0001-01-01",until:1/0,offset:1,name:"Anno Domini",narrow:"AD",abbr:"AD"},{since:"0000-12-31",until:-1/0,offset:1,name:"Before Christ",narrow:"BC",abbr:"BC"}],dayOfMonthOrdinalParse:/\d{1,2}(th|st|nd|rd)/,ordinal:function(e){var t=e%10;return e+(1===Z(e%100/10)?"th":1==t?"st":2==t?"nd":3==t?"rd":"th")}}),f.lang=n("moment.lang is deprecated. Use moment.locale instead.",lt),f.langData=n("moment.langData is deprecated. Use moment.localeData instead.",dt);var Yn=Math.abs;function On(e,t,n,s){var i=Zt(t,n);return e._milliseconds+=s*i._milliseconds,e._days+=s*i._days,e._months+=s*i._months,e._bubble()}function bn(e){return e<0?Math.floor(e):Math.ceil(e)}function xn(e){return 4800*e/146097}function Tn(e){return 146097*e/4800}function Nn(e){return function(){return this.as(e)}}var Pn=Nn("ms"),Rn=Nn("s"),Wn=Nn("m"),Cn=Nn("h"),Un=Nn("d"),Hn=Nn("w"),Fn=Nn("M"),Ln=Nn("Q"),Vn=Nn("y");function Gn(e){return function(){return this.isValid()?this._data[e]:NaN}}var En=Gn("milliseconds"),An=Gn("seconds"),jn=Gn("minutes"),In=Gn("hours"),Zn=Gn("days"),zn=Gn("months"),$n=Gn("years");var qn=Math.round,Bn={ss:44,s:45,m:45,h:22,d:26,w:null,M:11};function Jn(e,t,n,s){var i=Zt(e).abs(),r=qn(i.as("s")),a=qn(i.as("m")),o=qn(i.as("h")),u=qn(i.as("d")),l=qn(i.as("M")),h=qn(i.as("w")),d=qn(i.as("y")),c=(r<=n.ss?["s",r]:r<n.s&&["ss",r])||a<=1&&["m"]||a<n.m&&["mm",a]||o<=1&&["h"]||o<n.h&&["hh",o]||u<=1&&["d"]||u<n.d&&["dd",u];return null!=n.w&&(c=c||h<=1&&["w"]||h<n.w&&["ww",h]),(c=c||l<=1&&["M"]||l<n.M&&["MM",l]||d<=1&&["y"]||["yy",d])[2]=t,c[3]=0<+e,c[4]=s,function(e,t,n,s,i){return i.relativeTime(t||1,!!n,e,s)}.apply(null,c)}var Qn=Math.abs;function Xn(e){return(0<e)-(e<0)||+e}function Kn(){if(!this.isValid())return this.localeData().invalidDate();var e,t,n,s,i,r,a,o,u=Qn(this._milliseconds)/1e3,l=Qn(this._days),h=Qn(this._months),d=this.asSeconds();return d?(e=I(u/60),t=I(e/60),u%=60,e%=60,n=I(h/12),h%=12,s=u?u.toFixed(3).replace(/\.?0+$/,""):"",i=d<0?"-":"",r=Xn(this._months)!==Xn(d)?"-":"",a=Xn(this._days)!==Xn(d)?"-":"",o=Xn(this._milliseconds)!==Xn(d)?"-":"",i+"P"+(n?r+n+"Y":"")+(h?r+h+"M":"")+(l?a+l+"D":"")+(t||e||u?"T":"")+(t?o+t+"H":"")+(e?o+e+"M":"")+(u?o+s+"S":"")):"P0D"}var es=Ct.prototype;return es.isValid=function(){return this._isValid},es.abs=function(){var e=this._data;return this._milliseconds=Yn(this._milliseconds),this._days=Yn(this._days),this._months=Yn(this._months),e.milliseconds=Yn(e.milliseconds),e.seconds=Yn(e.seconds),e.minutes=Yn(e.minutes),e.hours=Yn(e.hours),e.months=Yn(e.months),e.years=Yn(e.years),this},es.add=function(e,t){return On(this,e,t,1)},es.subtract=function(e,t){return On(this,e,t,-1)},es.as=function(e){if(!this.isValid())return NaN;var t,n,s=this._milliseconds;if("month"===(e=V(e))||"quarter"===e||"year"===e)switch(t=this._days+s/864e5,n=this._months+xn(t),e){case"month":return n;case"quarter":return n/3;case"year":return n/12}else switch(t=this._days+Math.round(Tn(this._months)),e){case"week":return t/7+s/6048e5;case"day":return t+s/864e5;case"hour":return 24*t+s/36e5;case"minute":return 1440*t+s/6e4;case"second":return 86400*t+s/1e3;case"millisecond":return Math.floor(864e5*t)+s;default:throw new Error("Unknown unit "+e)}},es.asMilliseconds=Pn,es.asSeconds=Rn,es.asMinutes=Wn,es.asHours=Cn,es.asDays=Un,es.asWeeks=Hn,es.asMonths=Fn,es.asQuarters=Ln,es.asYears=Vn,es.valueOf=function(){return this.isValid()?this._milliseconds+864e5*this._days+this._months%12*2592e6+31536e6*Z(this._months/12):NaN},es._bubble=function(){var e,t,n,s,i,r=this._milliseconds,a=this._days,o=this._months,u=this._data;return 0<=r&&0<=a&&0<=o||r<=0&&a<=0&&o<=0||(r+=864e5*bn(Tn(o)+a),o=a=0),u.milliseconds=r%1e3,e=I(r/1e3),u.seconds=e%60,t=I(e/60),u.minutes=t%60,n=I(t/60),u.hours=n%24,a+=I(n/24),o+=i=I(xn(a)),a-=bn(Tn(i)),s=I(o/12),o%=12,u.days=a,u.months=o,u.years=s,this},es.clone=function(){return Zt(this)},es.get=function(e){return e=V(e),this.isValid()?this[e+"s"]():NaN},es.milliseconds=En,es.seconds=An,es.minutes=jn,es.hours=In,es.days=Zn,es.weeks=function(){return I(this.days()/7)},es.months=zn,es.years=$n,es.humanize=function(e,t){if(!this.isValid())return this.localeData().invalidDate();var n,s,i=!1,r=Bn;return"object"==typeof e&&(t=e,e=!1),"boolean"==typeof e&&(i=e),"object"==typeof t&&(r=Object.assign({},Bn,t),null!=t.s&&null==t.ss&&(r.ss=t.s-1)),n=this.localeData(),s=Jn(this,!i,r,n),i&&(s=n.pastFuture(+this,s)),n.postformat(s)},es.toISOString=Kn,es.toString=Kn,es.toJSON=Kn,es.locale=tn,es.localeData=sn,es.toIsoString=n("toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)",Kn),es.lang=nn,C("X",0,0,"unix"),C("x",0,0,"valueOf"),ce("x",ue),ce("X",/[+-]?\d+(\.\d{1,3})?/),ye("X",function(e,t,n){n._d=new Date(1e3*parseFloat(e))}),ye("x",function(e,t,n){n._d=new Date(Z(e))}),f.version="2.25.3",e=Tt,f.fn=pn,f.min=function(){return Rt("isBefore",[].slice.call(arguments,0))},f.max=function(){return Rt("isAfter",[].slice.call(arguments,0))},f.now=function(){return Date.now?Date.now():+new Date},f.utc=_,f.unix=function(e){return Tt(1e3*e)},f.months=function(e,t){return Dn(e,t,"months")},f.isDate=a,f.locale=lt,f.invalid=w,f.duration=Zt,f.isMoment=M,f.weekdays=function(e,t,n){return Sn(e,t,n,"weekdays")},f.parseZone=function(){return Tt.apply(null,arguments).parseZone()},f.localeData=dt,f.isDuration=Ut,f.monthsShort=function(e,t){return Dn(e,t,"monthsShort")},f.weekdaysMin=function(e,t,n){return Sn(e,t,n,"weekdaysMin")},f.defineLocale=ht,f.updateLocale=function(e,t){if(null!=t){var n,s,i=st;null!=it[e]&&null!=it[e].parentLocale?it[e].set(b(it[e]._config,t)):(null!=(s=ut(e))&&(i=s._config),t=b(i,t),null==s&&(t.abbr=e),(n=new x(t)).parentLocale=it[e],it[e]=n),lt(e)}else null!=it[e]&&(null!=it[e].parentLocale?(it[e]=it[e].parentLocale,e===lt()&&lt(e)):null!=it[e]&&delete it[e]);return it[e]},f.locales=function(){return s(it)},f.weekdaysShort=function(e,t,n){return Sn(e,t,n,"weekdaysShort")},f.normalizeUnits=V,f.relativeTimeRounding=function(e){return void 0===e?qn:"function"==typeof e&&(qn=e,!0)},f.relativeTimeThreshold=function(e,t){return void 0!==Bn[e]&&(void 0===t?Bn[e]:(Bn[e]=t,"s"===e&&(Bn.ss=t-1),!0))},f.calendarFormat=function(e,t){var n=e.diff(t,"days",!0);return n<-6?"sameElse":n<-1?"lastWeek":n<0?"lastDay":n<1?"sameDay":n<2?"nextDay":n<7?"nextWeek":"sameElse"},f.prototype=pn,f.HTML5_FMT={DATETIME_LOCAL:"YYYY-MM-DDTHH:mm",DATETIME_LOCAL_SECONDS:"YYYY-MM-DDTHH:mm:ss",DATETIME_LOCAL_MS:"YYYY-MM-DDTHH:mm:ss.SSS",DATE:"YYYY-MM-DD",TIME:"HH:mm",TIME_SECONDS:"HH:mm:ss",TIME_MS:"HH:mm:ss.SSS",WEEK:"GGGG-[W]WW",MONTH:"YYYY-MM"},f});
//# sourceMappingURL=moment.min.js.map
!function(n,r){var t,e;"object"==typeof exports&&"undefined"!=typeof module?module.exports=r():"function"==typeof define&&define.amd?define("underscore",r):(t=n._,e=r(),(n._=e).noConflict=function(){return n._=t,e})}(this,function(){
//     Underscore.js 1.10.2
//     https://underscorejs.org
//     (c) 2009-2020 Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
//     Underscore may be freely distributed under the MIT license.
var n="object"==typeof self&&self.self===self&&self||"object"==typeof global&&global.global===global&&global||Function("return this")()||{},e=Array.prototype,i=Object.prototype,p="undefined"!=typeof Symbol?Symbol.prototype:null,u=e.push,f=e.slice,s=i.toString,o=i.hasOwnProperty,r=Array.isArray,a=Object.keys,t=Object.create,c=n.isNaN,l=n.isFinite,v=function(){};function h(n){return n instanceof h?n:this instanceof h?void(this._wrapped=n):new h(n)}var g=h.VERSION="1.10.2";function y(u,o,n){if(void 0===o)return u;switch(null==n?3:n){case 1:return function(n){return u.call(o,n)};case 3:return function(n,r,t){return u.call(o,n,r,t)};case 4:return function(n,r,t,e){return u.call(o,n,r,t,e)}}return function(){return u.apply(o,arguments)}}function d(n,r,t){return null==n?ur:Cn(n)?y(n,r,t):Ln(n)&&!Kn(n)?ir(n):or(n)}function m(n,r){return d(n,r,1/0)}function b(n,r,t){return h.iteratee!==m?h.iteratee(n,r):d(n,r,t)}function j(u,o){return o=null==o?u.length-1:+o,function(){for(var n=Math.max(arguments.length-o,0),r=Array(n),t=0;t<n;t++)r[t]=arguments[t+o];switch(o){case 0:return u.call(this,r);case 1:return u.call(this,arguments[0],r);case 2:return u.call(this,arguments[0],arguments[1],r)}var e=Array(o+1);for(t=0;t<o;t++)e[t]=arguments[t];return e[o]=r,u.apply(this,e)}}function _(n){if(!Ln(n))return{};if(t)return t(n);v.prototype=n;var r=new v;return v.prototype=null,r}function w(r){return function(n){return null==n?void 0:n[r]}}function x(n,r){return null!=n&&o.call(n,r)}function S(n,r){for(var t=r.length,e=0;e<t;e++){if(null==n)return;n=n[r[e]]}return t?n:void 0}h.iteratee=m;var A=Math.pow(2,53)-1,O=w("length");function M(n){var r=O(n);return"number"==typeof r&&0<=r&&r<=A}function E(n,r,t){var e,u;if(r=y(r,t),M(n))for(e=0,u=n.length;e<u;e++)r(n[e],e,n);else{var o=Sn(n);for(e=0,u=o.length;e<u;e++)r(n[o[e]],o[e],n)}return n}function N(n,r,t){r=b(r,t);for(var e=!M(n)&&Sn(n),u=(e||n).length,o=Array(u),i=0;i<u;i++){var a=e?e[i]:i;o[i]=r(n[a],a,n)}return o}function k(f){return function(n,r,t,e){var u=3<=arguments.length;return function(n,r,t,e){var u=!M(n)&&Sn(n),o=(u||n).length,i=0<f?0:o-1;for(e||(t=n[u?u[i]:i],i+=f);0<=i&&i<o;i+=f){var a=u?u[i]:i;t=r(t,n[a],a,n)}return t}(n,y(r,e,4),t,u)}}var I=k(1),T=k(-1);function B(n,r,t){var e=(M(n)?on:Tn)(n,r,t);if(void 0!==e&&-1!==e)return n[e]}function R(n,e,r){var u=[];return e=b(e,r),E(n,function(n,r,t){e(n,r,t)&&u.push(n)}),u}function F(n,r,t){r=b(r,t);for(var e=!M(n)&&Sn(n),u=(e||n).length,o=0;o<u;o++){var i=e?e[o]:o;if(!r(n[i],i,n))return!1}return!0}function q(n,r,t){r=b(r,t);for(var e=!M(n)&&Sn(n),u=(e||n).length,o=0;o<u;o++){var i=e?e[o]:o;if(r(n[i],i,n))return!0}return!1}function D(n,r,t,e){return M(n)||(n=On(n)),("number"!=typeof t||e)&&(t=0),0<=ln(n,r,t)}var W=j(function(n,t,e){var u,o;return Cn(t)?o=t:Kn(t)&&(u=t.slice(0,-1),t=t[t.length-1]),N(n,function(n){var r=o;if(!r){if(u&&u.length&&(n=S(n,u)),null==n)return;r=n[t]}return null==r?r:r.apply(n,e)})});function z(n,r){return N(n,or(r))}function P(n,e,r){var t,u,o=-1/0,i=-1/0;if(null==e||"number"==typeof e&&"object"!=typeof n[0]&&null!=n)for(var a=0,f=(n=M(n)?n:On(n)).length;a<f;a++)null!=(t=n[a])&&o<t&&(o=t);else e=b(e,r),E(n,function(n,r,t){u=e(n,r,t),(i<u||u===-1/0&&o===-1/0)&&(o=n,i=u)});return o}function K(n,r,t){if(null==r||t)return M(n)||(n=On(n)),n[ar(n.length-1)];var e=M(n)?Dn(n):On(n),u=O(e);r=Math.max(Math.min(r,u),0);for(var o=u-1,i=0;i<r;i++){var a=ar(i,o),f=e[i];e[i]=e[a],e[a]=f}return e.slice(0,r)}function L(i,r){return function(e,u,n){var o=r?[[],[]]:{};return u=b(u,n),E(e,function(n,r){var t=u(n,r,e);i(o,n,t)}),o}}var V=L(function(n,r,t){x(n,t)?n[t].push(r):n[t]=[r]}),C=L(function(n,r,t){n[t]=r}),J=L(function(n,r,t){x(n,t)?n[t]++:n[t]=1}),U=/[^\ud800-\udfff]|[\ud800-\udbff][\udc00-\udfff]|[\ud800-\udfff]/g;var $=L(function(n,r,t){n[t?0:1].push(r)},!0);function G(n,r,t){return null==n||n.length<1?null==r?void 0:[]:null==r||t?n[0]:H(n,n.length-r)}function H(n,r,t){return f.call(n,0,Math.max(0,n.length-(null==r||t?1:r)))}function Q(n,r,t){return f.call(n,null==r||t?1:r)}function X(n,r,t,e){for(var u=(e=e||[]).length,o=0,i=O(n);o<i;o++){var a=n[o];if(M(a)&&(Kn(a)||Vn(a)))if(r)for(var f=0,c=a.length;f<c;)e[u++]=a[f++];else X(a,r,t,e),u=e.length;else t||(e[u++]=a)}return e}var Y=j(function(n,r){return rn(n,r)});function Z(n,r,t,e){er(r)||(e=t,t=r,r=!1),null!=t&&(t=b(t,e));for(var u=[],o=[],i=0,a=O(n);i<a;i++){var f=n[i],c=t?t(f,i,n):f;r&&!t?(i&&o===c||u.push(f),o=c):t?D(o,c)||(o.push(c),u.push(f)):D(u,f)||u.push(f)}return u}var nn=j(function(n){return Z(X(n,!0,!0))});var rn=j(function(n,r){return r=X(r,!0,!0),R(n,function(n){return!D(r,n)})});function tn(n){for(var r=n&&P(n,O).length||0,t=Array(r),e=0;e<r;e++)t[e]=z(n,e);return t}var en=j(tn);function un(o){return function(n,r,t){r=b(r,t);for(var e=O(n),u=0<o?0:e-1;0<=u&&u<e;u+=o)if(r(n[u],u,n))return u;return-1}}var on=un(1),an=un(-1);function fn(n,r,t,e){for(var u=(t=b(t,e,1))(r),o=0,i=O(n);o<i;){var a=Math.floor((o+i)/2);t(n[a])<u?o=a+1:i=a}return o}function cn(o,i,a){return function(n,r,t){var e=0,u=O(n);if("number"==typeof t)0<o?e=0<=t?t:Math.max(t+u,e):u=0<=t?Math.min(t+1,u):t+u+1;else if(a&&t&&u)return n[t=a(n,r)]===r?t:-1;if(r!=r)return 0<=(t=i(f.call(n,e,u),tr))?t+e:-1;for(t=0<o?e:u-1;0<=t&&t<u;t+=o)if(n[t]===r)return t;return-1}}var ln=cn(1,on,fn),pn=cn(-1,an);function sn(n,r,t,e,u){if(!(e instanceof r))return n.apply(t,u);var o=_(n.prototype),i=n.apply(o,u);return Ln(i)?i:o}var vn=j(function(r,t,e){if(!Cn(r))throw new TypeError("Bind must be called on a function");var u=j(function(n){return sn(r,u,t,this,e.concat(n))});return u}),hn=j(function(u,o){var i=hn.placeholder,a=function(){for(var n=0,r=o.length,t=Array(r),e=0;e<r;e++)t[e]=o[e]===i?arguments[n++]:o[e];for(;n<arguments.length;)t.push(arguments[n++]);return sn(u,a,this,this,t)};return a});hn.placeholder=h;var gn=j(function(n,r){var t=(r=X(r,!1,!1)).length;if(t<1)throw new Error("bindAll must be passed function names");for(;t--;){var e=r[t];n[e]=vn(n[e],n)}});var yn=j(function(n,r,t){return setTimeout(function(){return n.apply(null,t)},r)}),dn=hn(yn,h,1);function mn(n){return function(){return!n.apply(this,arguments)}}function bn(n,r){var t;return function(){return 0<--n&&(t=r.apply(this,arguments)),n<=1&&(r=null),t}}var jn=hn(bn,2),_n=!{toString:null}.propertyIsEnumerable("toString"),wn=["valueOf","isPrototypeOf","toString","propertyIsEnumerable","hasOwnProperty","toLocaleString"];function xn(n,r){var t=wn.length,e=n.constructor,u=Cn(e)&&e.prototype||i,o="constructor";for(x(n,o)&&!D(r,o)&&r.push(o);t--;)(o=wn[t])in n&&n[o]!==u[o]&&!D(r,o)&&r.push(o)}function Sn(n){if(!Ln(n))return[];if(a)return a(n);var r=[];for(var t in n)x(n,t)&&r.push(t);return _n&&xn(n,r),r}function An(n){if(!Ln(n))return[];var r=[];for(var t in n)r.push(t);return _n&&xn(n,r),r}function On(n){for(var r=Sn(n),t=r.length,e=Array(t),u=0;u<t;u++)e[u]=n[r[u]];return e}function Mn(n){for(var r={},t=Sn(n),e=0,u=t.length;e<u;e++)r[n[t[e]]]=t[e];return r}function En(n){var r=[];for(var t in n)Cn(n[t])&&r.push(t);return r.sort()}function Nn(f,c){return function(n){var r=arguments.length;if(c&&(n=Object(n)),r<2||null==n)return n;for(var t=1;t<r;t++)for(var e=arguments[t],u=f(e),o=u.length,i=0;i<o;i++){var a=u[i];c&&void 0!==n[a]||(n[a]=e[a])}return n}}var kn=Nn(An),In=Nn(Sn);function Tn(n,r,t){r=b(r,t);for(var e,u=Sn(n),o=0,i=u.length;o<i;o++)if(r(n[e=u[o]],e,n))return e}function Bn(n,r,t){return r in t}var Rn=j(function(n,r){var t={},e=r[0];if(null==n)return t;Cn(e)?(1<r.length&&(e=y(e,r[1])),r=An(n)):(e=Bn,r=X(r,!1,!1),n=Object(n));for(var u=0,o=r.length;u<o;u++){var i=r[u],a=n[i];e(a,i,n)&&(t[i]=a)}return t}),Fn=j(function(n,t){var r,e=t[0];return Cn(e)?(e=mn(e),1<t.length&&(r=t[1])):(t=N(X(t,!1,!1),String),e=function(n,r){return!D(t,r)}),Rn(n,e,r)}),qn=Nn(An,!0);function Dn(n){return Ln(n)?Kn(n)?n.slice():kn({},n):n}function Wn(n,r){var t=Sn(r),e=t.length;if(null==n)return!e;for(var u=Object(n),o=0;o<e;o++){var i=t[o];if(r[i]!==u[i]||!(i in u))return!1}return!0}function zn(n,r,t,e){if(n===r)return 0!==n||1/n==1/r;if(null==n||null==r)return!1;if(n!=n)return r!=r;var u=typeof n;return("function"===u||"object"===u||"object"==typeof r)&&function(n,r,t,e){n instanceof h&&(n=n._wrapped);r instanceof h&&(r=r._wrapped);var u=s.call(n);if(u!==s.call(r))return!1;switch(u){case"[object RegExp]":case"[object String]":return""+n==""+r;case"[object Number]":return+n!=+n?+r!=+r:0==+n?1/+n==1/r:+n==+r;case"[object Date]":case"[object Boolean]":return+n==+r;case"[object Symbol]":return p.valueOf.call(n)===p.valueOf.call(r)}var o="[object Array]"===u;if(!o){if("object"!=typeof n||"object"!=typeof r)return!1;var i=n.constructor,a=r.constructor;if(i!==a&&!(Cn(i)&&i instanceof i&&Cn(a)&&a instanceof a)&&"constructor"in n&&"constructor"in r)return!1}e=e||[];var f=(t=t||[]).length;for(;f--;)if(t[f]===n)return e[f]===r;if(t.push(n),e.push(r),o){if((f=n.length)!==r.length)return!1;for(;f--;)if(!zn(n[f],r[f],t,e))return!1}else{var c,l=Sn(n);if(f=l.length,Sn(r).length!==f)return!1;for(;f--;)if(c=l[f],!x(r,c)||!zn(n[c],r[c],t,e))return!1}return t.pop(),e.pop(),!0}(n,r,t,e)}function Pn(r){return function(n){return s.call(n)==="[object "+r+"]"}}var Kn=r||Pn("Array");function Ln(n){var r=typeof n;return"function"===r||"object"===r&&!!n}var Vn=Pn("Arguments"),Cn=Pn("Function"),Jn=Pn("String"),Un=Pn("Number"),$n=Pn("Date"),Gn=Pn("RegExp"),Hn=Pn("Error"),Qn=Pn("Symbol"),Xn=Pn("Map"),Yn=Pn("WeakMap"),Zn=Pn("Set"),nr=Pn("WeakSet");!function(){Vn(arguments)||(Vn=function(n){return x(n,"callee")})}();var rr=n.document&&n.document.childNodes;function tr(n){return Un(n)&&c(n)}function er(n){return!0===n||!1===n||"[object Boolean]"===s.call(n)}function ur(n){return n}function or(r){return Kn(r)?function(n){return S(n,r)}:w(r)}function ir(r){return r=In({},r),function(n){return Wn(n,r)}}function ar(n,r){return null==r&&(r=n,n=0),n+Math.floor(Math.random()*(r-n+1))}"function"!=typeof/./&&"object"!=typeof Int8Array&&"function"!=typeof rr&&(Cn=function(n){return"function"==typeof n||!1});var fr=Date.now||function(){return(new Date).getTime()},cr={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#x27;","`":"&#x60;"},lr=Mn(cr);function pr(r){var t=function(n){return r[n]},n="(?:"+Sn(r).join("|")+")",e=RegExp(n),u=RegExp(n,"g");return function(n){return n=null==n?"":""+n,e.test(n)?n.replace(u,t):n}}var sr=pr(cr),vr=pr(lr);var hr=0;var gr=h.templateSettings={evaluate:/<%([\s\S]+?)%>/g,interpolate:/<%=([\s\S]+?)%>/g,escape:/<%-([\s\S]+?)%>/g},yr=/(.)^/,dr={"'":"'","\\":"\\","\r":"r","\n":"n","\u2028":"u2028","\u2029":"u2029"},mr=/\\|'|\r|\n|\u2028|\u2029/g,br=function(n){return"\\"+dr[n]};function jr(n,r){return n._chain?h(r).chain():r}function _r(t){return E(En(t),function(n){var r=h[n]=t[n];h.prototype[n]=function(){var n=[this._wrapped];return u.apply(n,arguments),jr(this,r.apply(h,n))}}),h}E(["pop","push","reverse","shift","sort","splice","unshift"],function(r){var t=e[r];h.prototype[r]=function(){var n=this._wrapped;return t.apply(n,arguments),"shift"!==r&&"splice"!==r||0!==n.length||delete n[0],jr(this,n)}}),E(["concat","join","slice"],function(n){var r=e[n];h.prototype[n]=function(){return jr(this,r.apply(this._wrapped,arguments))}}),h.prototype.valueOf=h.prototype.toJSON=h.prototype.value=function(){return this._wrapped},h.prototype.toString=function(){return String(this._wrapped)};var wr=_r({default:h,VERSION:g,iteratee:m,restArguments:j,each:E,forEach:E,map:N,collect:N,reduce:I,foldl:I,inject:I,reduceRight:T,foldr:T,find:B,detect:B,filter:R,select:R,reject:function(n,r,t){return R(n,mn(b(r)),t)},every:F,all:F,some:q,any:q,contains:D,includes:D,include:D,invoke:W,pluck:z,where:function(n,r){return R(n,ir(r))},findWhere:function(n,r){return B(n,ir(r))},max:P,min:function(n,e,r){var t,u,o=1/0,i=1/0;if(null==e||"number"==typeof e&&"object"!=typeof n[0]&&null!=n)for(var a=0,f=(n=M(n)?n:On(n)).length;a<f;a++)null!=(t=n[a])&&t<o&&(o=t);else e=b(e,r),E(n,function(n,r,t){((u=e(n,r,t))<i||u===1/0&&o===1/0)&&(o=n,i=u)});return o},shuffle:function(n){return K(n,1/0)},sample:K,sortBy:function(n,e,r){var u=0;return e=b(e,r),z(N(n,function(n,r,t){return{value:n,index:u++,criteria:e(n,r,t)}}).sort(function(n,r){var t=n.criteria,e=r.criteria;if(t!==e){if(e<t||void 0===t)return 1;if(t<e||void 0===e)return-1}return n.index-r.index}),"value")},groupBy:V,indexBy:C,countBy:J,toArray:function(n){return n?Kn(n)?f.call(n):Jn(n)?n.match(U):M(n)?N(n,ur):On(n):[]},size:function(n){return null==n?0:M(n)?n.length:Sn(n).length},partition:$,first:G,head:G,take:G,initial:H,last:function(n,r,t){return null==n||n.length<1?null==r?void 0:[]:null==r||t?n[n.length-1]:Q(n,Math.max(0,n.length-r))},rest:Q,tail:Q,drop:Q,compact:function(n){return R(n,Boolean)},flatten:function(n,r){return X(n,r,!1)},without:Y,uniq:Z,unique:Z,union:nn,intersection:function(n){for(var r=[],t=arguments.length,e=0,u=O(n);e<u;e++){var o=n[e];if(!D(r,o)){var i;for(i=1;i<t&&D(arguments[i],o);i++);i===t&&r.push(o)}}return r},difference:rn,unzip:tn,zip:en,object:function(n,r){for(var t={},e=0,u=O(n);e<u;e++)r?t[n[e]]=r[e]:t[n[e][0]]=n[e][1];return t},findIndex:on,findLastIndex:an,sortedIndex:fn,indexOf:ln,lastIndexOf:pn,range:function(n,r,t){null==r&&(r=n||0,n=0),t||(t=r<n?-1:1);for(var e=Math.max(Math.ceil((r-n)/t),0),u=Array(e),o=0;o<e;o++,n+=t)u[o]=n;return u},chunk:function(n,r){if(null==r||r<1)return[];for(var t=[],e=0,u=n.length;e<u;)t.push(f.call(n,e,e+=r));return t},bind:vn,partial:hn,bindAll:gn,memoize:function(e,u){var o=function(n){var r=o.cache,t=""+(u?u.apply(this,arguments):n);return x(r,t)||(r[t]=e.apply(this,arguments)),r[t]};return o.cache={},o},delay:yn,defer:dn,throttle:function(t,e,u){var o,i,a,f,c=0;u||(u={});var l=function(){c=!1===u.leading?0:fr(),o=null,f=t.apply(i,a),o||(i=a=null)},n=function(){var n=fr();c||!1!==u.leading||(c=n);var r=e-(n-c);return i=this,a=arguments,r<=0||e<r?(o&&(clearTimeout(o),o=null),c=n,f=t.apply(i,a),o||(i=a=null)):o||!1===u.trailing||(o=setTimeout(l,r)),f};return n.cancel=function(){clearTimeout(o),c=0,o=i=a=null},n},debounce:function(t,e,u){var o,i,a=function(n,r){o=null,r&&(i=t.apply(n,r))},n=j(function(n){if(o&&clearTimeout(o),u){var r=!o;o=setTimeout(a,e),r&&(i=t.apply(this,n))}else o=yn(a,e,this,n);return i});return n.cancel=function(){clearTimeout(o),o=null},n},wrap:function(n,r){return hn(r,n)},negate:mn,compose:function(){var t=arguments,e=t.length-1;return function(){for(var n=e,r=t[e].apply(this,arguments);n--;)r=t[n].call(this,r);return r}},after:function(n,r){return function(){if(--n<1)return r.apply(this,arguments)}},before:bn,once:jn,keys:Sn,allKeys:An,values:On,mapObject:function(n,r,t){r=b(r,t);for(var e=Sn(n),u=e.length,o={},i=0;i<u;i++){var a=e[i];o[a]=r(n[a],a,n)}return o},pairs:function(n){for(var r=Sn(n),t=r.length,e=Array(t),u=0;u<t;u++)e[u]=[r[u],n[r[u]]];return e},invert:Mn,functions:En,methods:En,extend:kn,extendOwn:In,assign:In,findKey:Tn,pick:Rn,omit:Fn,defaults:qn,create:function(n,r){var t=_(n);return r&&In(t,r),t},clone:Dn,tap:function(n,r){return r(n),n},isMatch:Wn,isEqual:function(n,r){return zn(n,r)},isEmpty:function(n){return null==n||(M(n)&&(Kn(n)||Jn(n)||Vn(n))?0===n.length:0===Sn(n).length)},isElement:function(n){return!(!n||1!==n.nodeType)},isArray:Kn,isObject:Ln,isArguments:Vn,isFunction:Cn,isString:Jn,isNumber:Un,isDate:$n,isRegExp:Gn,isError:Hn,isSymbol:Qn,isMap:Xn,isWeakMap:Yn,isSet:Zn,isWeakSet:nr,isFinite:function(n){return!Qn(n)&&l(n)&&!c(parseFloat(n))},isNaN:tr,isBoolean:er,isNull:function(n){return null===n},isUndefined:function(n){return void 0===n},has:function(n,r){if(!Kn(r))return x(n,r);for(var t=r.length,e=0;e<t;e++){var u=r[e];if(null==n||!o.call(n,u))return!1;n=n[u]}return!!t},identity:ur,constant:function(n){return function(){return n}},noop:function(){},property:or,propertyOf:function(r){return null==r?function(){}:function(n){return Kn(n)?S(r,n):r[n]}},matcher:ir,matches:ir,times:function(n,r,t){var e=Array(Math.max(0,n));r=y(r,t,1);for(var u=0;u<n;u++)e[u]=r(u);return e},random:ar,now:fr,escape:sr,unescape:vr,result:function(n,r,t){Kn(r)||(r=[r]);var e=r.length;if(!e)return Cn(t)?t.call(n):t;for(var u=0;u<e;u++){var o=null==n?void 0:n[r[u]];void 0===o&&(o=t,u=e),n=Cn(o)?o.call(n):o}return n},uniqueId:function(n){var r=++hr+"";return n?n+r:r},templateSettings:gr,template:function(o,n,r){!n&&r&&(n=r),n=qn({},n,h.templateSettings);var t,e=RegExp([(n.escape||yr).source,(n.interpolate||yr).source,(n.evaluate||yr).source].join("|")+"|$","g"),i=0,a="__p+='";o.replace(e,function(n,r,t,e,u){return a+=o.slice(i,u).replace(mr,br),i=u+n.length,r?a+="'+\n((__t=("+r+"))==null?'':_.escape(__t))+\n'":t?a+="'+\n((__t=("+t+"))==null?'':__t)+\n'":e&&(a+="';\n"+e+"\n__p+='"),n}),a+="';\n",n.variable||(a="with(obj||{}){\n"+a+"}\n"),a="var __t,__p='',__j=Array.prototype.join,"+"print=function(){__p+=__j.call(arguments,'');};\n"+a+"return __p;\n";try{t=new Function(n.variable||"obj","_",a)}catch(n){throw n.source=a,n}var u=function(n){return t.call(this,n,h)},f=n.variable||"obj";return u.source="function("+f+"){\n"+a+"}",u},chain:function(n){var r=h(n);return r._chain=!0,r},mixin:_r});return wr._=wr});
/*!
 * Generated using the Bootstrap Customizer (https://getbootstrap.com/docs/3.4/customize/)
 */

/*!
 * Bootstrap v3.4.1 (https://getbootstrap.com/)
 * Copyright 2011-2019 Twitter, Inc.
 * Licensed under the MIT license
 */

if (typeof jQuery === 'undefined') {
    throw new Error('Bootstrap\'s JavaScript requires jQuery')
}
+function ($) {
    'use strict';
    var version = $.fn.jquery.split(' ')[0].split('.')
    if ((version[0] < 2 && version[1] < 9) || (version[0] == 1 && version[1] == 9 && version[2] < 1) || (version[0] > 3)) {
        throw new Error('Bootstrap\'s JavaScript requires jQuery version 1.9.1 or higher, but lower than version 4')
    }
}(jQuery);

/* ========================================================================
 * Bootstrap: tooltip.js v3.4.1
 * https://getbootstrap.com/docs/3.4/javascript/#tooltip
 * Inspired by the original jQuery.tipsy by Jason Frame
 * ========================================================================
 * Copyright 2011-2019 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */

+function ($) {
    'use strict';

    var DISALLOWED_ATTRIBUTES = ['sanitize', 'whiteList', 'sanitizeFn']

    var uriAttrs = [
        'background',
        'cite',
        'href',
        'itemtype',
        'longdesc',
        'poster',
        'src',
        'xlink:href'
    ]

    var ARIA_ATTRIBUTE_PATTERN = /^aria-[\w-]*$/i

    var DefaultWhitelist = {
        // Global attributes allowed on any supplied element below.
        '*': ['class', 'dir', 'id', 'lang', 'role', ARIA_ATTRIBUTE_PATTERN],
        a: ['target', 'href', 'title', 'rel'],
        area: [],
        b: [],
        br: [],
        col: [],
        code: [],
        div: [],
        em: [],
        hr: [],
        h1: [],
        h2: [],
        h3: [],
        h4: [],
        h5: [],
        h6: [],
        i: [],
        img: ['src', 'alt', 'title', 'width', 'height'],
        li: [],
        ol: [],
        p: [],
        pre: [],
        s: [],
        small: [],
        span: [],
        sub: [],
        sup: [],
        strong: [],
        u: [],
        ul: []
    }

    /**
     * A pattern that recognizes a commonly useful subset of URLs that are safe.
     *
     * Shoutout to Angular 7 https://github.com/angular/angular/blob/7.2.4/packages/core/src/sanitization/url_sanitizer.ts
     */
    var SAFE_URL_PATTERN = /^(?:(?:https?|mailto|ftp|tel|file):|[^&:/?#]*(?:[/?#]|$))/gi

    /**
     * A pattern that matches safe data URLs. Only matches image, video and audio types.
     *
     * Shoutout to Angular 7 https://github.com/angular/angular/blob/7.2.4/packages/core/src/sanitization/url_sanitizer.ts
     */
    var DATA_URL_PATTERN = /^data:(?:image\/(?:bmp|gif|jpeg|jpg|png|tiff|webp)|video\/(?:mpeg|mp4|ogg|webm)|audio\/(?:mp3|oga|ogg|opus));base64,[a-z0-9+/]+=*$/i

    function allowedAttribute(attr, allowedAttributeList) {
        var attrName = attr.nodeName.toLowerCase()

        if ($.inArray(attrName, allowedAttributeList) !== -1) {
            if ($.inArray(attrName, uriAttrs) !== -1) {
                return Boolean(attr.nodeValue.match(SAFE_URL_PATTERN) || attr.nodeValue.match(DATA_URL_PATTERN))
            }

            return true
        }

        var regExp = $(allowedAttributeList).filter(function (index, value) {
            return value instanceof RegExp
        })

        // Check if a regular expression validates the attribute.
        for (var i = 0, l = regExp.length; i < l; i++) {
            if (attrName.match(regExp[i])) {
                return true
            }
        }

        return false
    }

    function sanitizeHtml(unsafeHtml, whiteList, sanitizeFn) {
        if (unsafeHtml.length === 0) {
            return unsafeHtml
        }

        if (sanitizeFn && typeof sanitizeFn === 'function') {
            return sanitizeFn(unsafeHtml)
        }

        // IE 8 and below don't support createHTMLDocument
        if (!document.implementation || !document.implementation.createHTMLDocument) {
            return unsafeHtml
        }

        var createdDocument = document.implementation.createHTMLDocument('sanitization')
        createdDocument.body.innerHTML = unsafeHtml

        var whitelistKeys = $.map(whiteList, function (el, i) {
            return i
        })
        var elements = $(createdDocument.body).find('*')

        for (var i = 0, len = elements.length; i < len; i++) {
            var el = elements[i]
            var elName = el.nodeName.toLowerCase()

            if ($.inArray(elName, whitelistKeys) === -1) {
                el.parentNode.removeChild(el)

                continue
            }

            var attributeList = $.map(el.attributes, function (el) {
                return el
            })
            var whitelistedAttributes = [].concat(whiteList['*'] || [], whiteList[elName] || [])

            for (var j = 0, len2 = attributeList.length; j < len2; j++) {
                if (!allowedAttribute(attributeList[j], whitelistedAttributes)) {
                    el.removeAttribute(attributeList[j].nodeName)
                }
            }
        }

        return createdDocument.body.innerHTML
    }

    // TOOLTIP PUBLIC CLASS DEFINITION
    // ===============================

    var Tooltip = function (element, options) {
        this.type = null
        this.options = null
        this.enabled = null
        this.timeout = null
        this.hoverState = null
        this.$element = null
        this.inState = null

        this.init('tooltip', element, options)
    }

    Tooltip.VERSION = '3.4.1'

    Tooltip.TRANSITION_DURATION = 150

    Tooltip.DEFAULTS = {
        animation: true,
        placement: 'top',
        selector: false,
        template: '<div class="tooltip" role="tooltip"><div class="tooltip-arrow"></div><div class="tooltip-inner"></div></div>',
        trigger: 'hover focus',
        title: '',
        delay: 0,
        html: false,
        container: false,
        viewport: {
            selector: 'body',
            padding: 0
        },
        sanitize: true,
        sanitizeFn: null,
        whiteList: DefaultWhitelist
    }

    Tooltip.prototype.init = function (type, element, options) {
        this.enabled = true
        this.type = type
        this.$element = $(element)
        this.options = this.getOptions(options)
        this.$viewport = this.options.viewport && $(document).find($.isFunction(this.options.viewport) ? this.options.viewport.call(this, this.$element) : (this.options.viewport.selector || this.options.viewport))
        this.inState = {click: false, hover: false, focus: false}

        if (this.$element[0] instanceof document.constructor && !this.options.selector) {
            throw new Error('`selector` option must be specified when initializing ' + this.type + ' on the window.document object!')
        }

        var triggers = this.options.trigger.split(' ')

        for (var i = triggers.length; i--;) {
            var trigger = triggers[i]

            if (trigger == 'click') {
                this.$element.on('click.' + this.type, this.options.selector, $.proxy(this.toggle, this))
            } else if (trigger != 'manual') {
                var eventIn = trigger == 'hover' ? 'mouseenter' : 'focusin'
                var eventOut = trigger == 'hover' ? 'mouseleave' : 'focusout'

                this.$element.on(eventIn + '.' + this.type, this.options.selector, $.proxy(this.enter, this))
                this.$element.on(eventOut + '.' + this.type, this.options.selector, $.proxy(this.leave, this))
            }
        }

        this.options.selector ?
            (this._options = $.extend({}, this.options, {trigger: 'manual', selector: ''})) :
            this.fixTitle()
    }

    Tooltip.prototype.getDefaults = function () {
        return Tooltip.DEFAULTS
    }

    Tooltip.prototype.getOptions = function (options) {
        var dataAttributes = this.$element.data()

        for (var dataAttr in dataAttributes) {
            if (dataAttributes.hasOwnProperty(dataAttr) && $.inArray(dataAttr, DISALLOWED_ATTRIBUTES) !== -1) {
                delete dataAttributes[dataAttr]
            }
        }

        options = $.extend({}, this.getDefaults(), dataAttributes, options)

        if (options.delay && typeof options.delay == 'number') {
            options.delay = {
                show: options.delay,
                hide: options.delay
            }
        }

        if (options.sanitize) {
            options.template = sanitizeHtml(options.template, options.whiteList, options.sanitizeFn)
        }

        return options
    }

    Tooltip.prototype.getDelegateOptions = function () {
        var options = {}
        var defaults = this.getDefaults()

        this._options && $.each(this._options, function (key, value) {
            if (defaults[key] != value) options[key] = value
        })

        return options
    }

    Tooltip.prototype.enter = function (obj) {
        var self = obj instanceof this.constructor ?
            obj : $(obj.currentTarget).data('bs.' + this.type)

        if (!self) {
            self = new this.constructor(obj.currentTarget, this.getDelegateOptions())
            $(obj.currentTarget).data('bs.' + this.type, self)
        }

        if (obj instanceof $.Event) {
            self.inState[obj.type == 'focusin' ? 'focus' : 'hover'] = true
        }

        if (self.tip().hasClass('in') || self.hoverState == 'in') {
            self.hoverState = 'in'
            return
        }

        clearTimeout(self.timeout)

        self.hoverState = 'in'

        if (!self.options.delay || !self.options.delay.show) return self.show()

        self.timeout = setTimeout(function () {
            if (self.hoverState == 'in') self.show()
        }, self.options.delay.show)
    }

    Tooltip.prototype.isInStateTrue = function () {
        for (var key in this.inState) {
            if (this.inState[key]) return true
        }

        return false
    }

    Tooltip.prototype.leave = function (obj) {
        var self = obj instanceof this.constructor ?
            obj : $(obj.currentTarget).data('bs.' + this.type)

        if (!self) {
            self = new this.constructor(obj.currentTarget, this.getDelegateOptions())
            $(obj.currentTarget).data('bs.' + this.type, self)
        }

        if (obj instanceof $.Event) {
            self.inState[obj.type == 'focusout' ? 'focus' : 'hover'] = false
        }

        if (self.isInStateTrue()) return

        clearTimeout(self.timeout)

        self.hoverState = 'out'

        if (!self.options.delay || !self.options.delay.hide) return self.hide()

        self.timeout = setTimeout(function () {
            if (self.hoverState == 'out') self.hide()
        }, self.options.delay.hide)
    }

    Tooltip.prototype.show = function () {
        var e = $.Event('show.bs.' + this.type)

        if (this.hasContent() && this.enabled) {
            this.$element.trigger(e)

            var inDom = $.contains(this.$element[0].ownerDocument.documentElement, this.$element[0])
            if (e.isDefaultPrevented() || !inDom) return
            var that = this

            var $tip = this.tip()

            var tipId = this.getUID(this.type)

            this.setContent()
            $tip.attr('id', tipId)
            this.$element.attr('aria-describedby', tipId)

            if (this.options.animation) $tip.addClass('fade')

            var placement = typeof this.options.placement == 'function' ?
                this.options.placement.call(this, $tip[0], this.$element[0]) :
                this.options.placement

            var autoToken = /\s?auto?\s?/i
            var autoPlace = autoToken.test(placement)
            if (autoPlace) placement = placement.replace(autoToken, '') || 'top'

            $tip
                .detach()
                .css({top: 0, left: 0, display: 'block'})
                .addClass(placement)
                .data('bs.' + this.type, this)

            this.options.container ? $tip.appendTo($(document).find(this.options.container)) : $tip.insertAfter(this.$element)
            this.$element.trigger('inserted.bs.' + this.type)

            var pos = this.getPosition()
            var actualWidth = $tip[0].offsetWidth
            var actualHeight = $tip[0].offsetHeight

            if (autoPlace) {
                var orgPlacement = placement
                var viewportDim = this.getPosition(this.$viewport)

                placement = placement == 'bottom' && pos.bottom + actualHeight > viewportDim.bottom ? 'top' :
                    placement == 'top' && pos.top - actualHeight < viewportDim.top ? 'bottom' :
                        placement == 'right' && pos.right + actualWidth > viewportDim.width ? 'left' :
                            placement == 'left' && pos.left - actualWidth < viewportDim.left ? 'right' :
                                placement

                $tip
                    .removeClass(orgPlacement)
                    .addClass(placement)
            }

            var calculatedOffset = this.getCalculatedOffset(placement, pos, actualWidth, actualHeight)

            this.applyPlacement(calculatedOffset, placement)

            var complete = function () {
                var prevHoverState = that.hoverState
                if (that.$element) {
                    that.$element.trigger('shown.bs.' + that.type)
                }
                that.hoverState = null

                if (prevHoverState == 'out') that.leave(that)
            }

            $.support.transition && this.$tip.hasClass('fade') ?
                $tip
                    .one('bsTransitionEnd', complete)
                    .emulateTransitionEnd(Tooltip.TRANSITION_DURATION) :
                complete()
        }
    }

    Tooltip.prototype.applyPlacement = function (offset, placement) {
        var $tip = this.tip()
        var width = $tip[0].offsetWidth
        var height = $tip[0].offsetHeight

        // manually read margins because getBoundingClientRect includes difference
        var marginTop = parseInt($tip.css('margin-top'), 10)
        var marginLeft = parseInt($tip.css('margin-left'), 10)

        // we must check for NaN for ie 8/9
        if (isNaN(marginTop)) marginTop = 0
        if (isNaN(marginLeft)) marginLeft = 0

        offset.top += marginTop
        offset.left += marginLeft

        // $.fn.offset doesn't round pixel values
        // so we use setOffset directly with our own function B-0
        $.offset.setOffset($tip[0], $.extend({
            using: function (props) {
                $tip.css({
                    top: Math.round(props.top),
                    left: Math.round(props.left)
                })
            }
        }, offset), 0)

        $tip.addClass('in')

        // check to see if placing tip in new offset caused the tip to resize itself
        var actualWidth = $tip[0].offsetWidth
        var actualHeight = $tip[0].offsetHeight

        if (placement == 'top' && actualHeight != height) {
            offset.top = offset.top + height - actualHeight
        }

        var delta = this.getViewportAdjustedDelta(placement, offset, actualWidth, actualHeight)

        if (delta.left) offset.left += delta.left
        else offset.top += delta.top

        var isVertical = /top|bottom/.test(placement)
        var arrowDelta = isVertical ? delta.left * 2 - width + actualWidth : delta.top * 2 - height + actualHeight
        var arrowOffsetPosition = isVertical ? 'offsetWidth' : 'offsetHeight'

        $tip.offset(offset)
        this.replaceArrow(arrowDelta, $tip[0][arrowOffsetPosition], isVertical)
    }

    Tooltip.prototype.replaceArrow = function (delta, dimension, isVertical) {
        this.arrow()
            .css(isVertical ? 'left' : 'top', 50 * (1 - delta / dimension) + '%')
            .css(isVertical ? 'top' : 'left', '')
    }

    Tooltip.prototype.setContent = function () {
        var $tip = this.tip()
        var title = this.getTitle()

        if (this.options.html) {
            if (this.options.sanitize) {
                title = sanitizeHtml(title, this.options.whiteList, this.options.sanitizeFn)
            }

            $tip.find('.tooltip-inner').html(title)
        } else {
            $tip.find('.tooltip-inner').text(title)
        }

        $tip.removeClass('fade in top bottom left right')
    }

    Tooltip.prototype.hide = function (callback) {
        var that = this
        var $tip = $(this.$tip)
        var e = $.Event('hide.bs.' + this.type)

        function complete() {
            if (that.hoverState != 'in') $tip.detach()
            if (that.$element) { // TODO: Check whether guarding this code with this `if` is really necessary.
                that.$element
                    .removeAttr('aria-describedby')
                    .trigger('hidden.bs.' + that.type)
            }
            callback && callback()
        }

        this.$element.trigger(e)

        if (e.isDefaultPrevented()) return

        $tip.removeClass('in')

        $.support.transition && $tip.hasClass('fade') ?
            $tip
                .one('bsTransitionEnd', complete)
                .emulateTransitionEnd(Tooltip.TRANSITION_DURATION) :
            complete()

        this.hoverState = null

        return this
    }

    Tooltip.prototype.fixTitle = function () {
        var $e = this.$element
        if ($e.attr('title') || typeof $e.attr('data-original-title') != 'string') {
            $e.attr('data-original-title', $e.attr('title') || '').attr('title', '')
        }
    }

    Tooltip.prototype.hasContent = function () {
        return this.getTitle()
    }

    Tooltip.prototype.getPosition = function ($element) {
        $element = $element || this.$element

        var el = $element[0]
        var isBody = el.tagName == 'BODY'

        var elRect = el.getBoundingClientRect()
        if (elRect.width == null) {
            // width and height are missing in IE8, so compute them manually; see https://github.com/twbs/bootstrap/issues/14093
            elRect = $.extend({}, elRect, {width: elRect.right - elRect.left, height: elRect.bottom - elRect.top})
        }
        var isSvg = window.SVGElement && el instanceof window.SVGElement
        // Avoid using $.offset() on SVGs since it gives incorrect results in jQuery 3.
        // See https://github.com/twbs/bootstrap/issues/20280
        var elOffset = isBody ? {top: 0, left: 0} : (isSvg ? null : $element.offset())
        var scroll = {scroll: isBody ? document.documentElement.scrollTop || document.body.scrollTop : $element.scrollTop()}
        var outerDims = isBody ? {width: $(window).width(), height: $(window).height()} : null

        return $.extend({}, elRect, scroll, outerDims, elOffset)
    }

    Tooltip.prototype.getCalculatedOffset = function (placement, pos, actualWidth, actualHeight) {
        return placement == 'bottom' ? {top: pos.top + pos.height, left: pos.left + pos.width / 2 - actualWidth / 2} :
            placement == 'top' ? {top: pos.top - actualHeight, left: pos.left + pos.width / 2 - actualWidth / 2} :
                placement == 'left' ? {top: pos.top + pos.height / 2 - actualHeight / 2, left: pos.left - actualWidth} :
                    /* placement == 'right' */ {
                        top: pos.top + pos.height / 2 - actualHeight / 2,
                        left: pos.left + pos.width
                    }

    }

    Tooltip.prototype.getViewportAdjustedDelta = function (placement, pos, actualWidth, actualHeight) {
        var delta = {top: 0, left: 0}
        if (!this.$viewport) return delta

        var viewportPadding = this.options.viewport && this.options.viewport.padding || 0
        var viewportDimensions = this.getPosition(this.$viewport)

        if (/right|left/.test(placement)) {
            var topEdgeOffset = pos.top - viewportPadding - viewportDimensions.scroll
            var bottomEdgeOffset = pos.top + viewportPadding - viewportDimensions.scroll + actualHeight
            if (topEdgeOffset < viewportDimensions.top) { // top overflow
                delta.top = viewportDimensions.top - topEdgeOffset
            } else if (bottomEdgeOffset > viewportDimensions.top + viewportDimensions.height) { // bottom overflow
                delta.top = viewportDimensions.top + viewportDimensions.height - bottomEdgeOffset
            }
        } else {
            var leftEdgeOffset = pos.left - viewportPadding
            var rightEdgeOffset = pos.left + viewportPadding + actualWidth
            if (leftEdgeOffset < viewportDimensions.left) { // left overflow
                delta.left = viewportDimensions.left - leftEdgeOffset
            } else if (rightEdgeOffset > viewportDimensions.right) { // right overflow
                delta.left = viewportDimensions.left + viewportDimensions.width - rightEdgeOffset
            }
        }

        return delta
    }

    Tooltip.prototype.getTitle = function () {
        var title
        var $e = this.$element
        var o = this.options

        title = $e.attr('data-original-title')
            || (typeof o.title == 'function' ? o.title.call($e[0]) : o.title)

        return title
    }

    Tooltip.prototype.getUID = function (prefix) {
        do prefix += ~~(Math.random() * 1000000)
        while (document.getElementById(prefix))
        return prefix
    }

    Tooltip.prototype.tip = function () {
        if (!this.$tip) {
            this.$tip = $(this.options.template)
            if (this.$tip.length != 1) {
                throw new Error(this.type + ' `template` option must consist of exactly 1 top-level element!')
            }
        }
        return this.$tip
    }

    Tooltip.prototype.arrow = function () {
        return (this.$arrow = this.$arrow || this.tip().find('.tooltip-arrow'))
    }

    Tooltip.prototype.enable = function () {
        this.enabled = true
    }

    Tooltip.prototype.disable = function () {
        this.enabled = false
    }

    Tooltip.prototype.toggleEnabled = function () {
        this.enabled = !this.enabled
    }

    Tooltip.prototype.toggle = function (e) {
        var self = this
        if (e) {
            self = $(e.currentTarget).data('bs.' + this.type)
            if (!self) {
                self = new this.constructor(e.currentTarget, this.getDelegateOptions())
                $(e.currentTarget).data('bs.' + this.type, self)
            }
        }

        if (e) {
            self.inState.click = !self.inState.click
            if (self.isInStateTrue()) self.enter(self)
            else self.leave(self)
        } else {
            self.tip().hasClass('in') ? self.leave(self) : self.enter(self)
        }
    }

    Tooltip.prototype.destroy = function () {
        var that = this
        clearTimeout(this.timeout)
        this.hide(function () {
            if (!that || !that.$element) {
                return;
            }
            that.$element.off('.' + that.type).removeData('bs.' + that.type)
            if (that.$tip) {
                that.$tip.detach()
            }
            that.$tip = null
            that.$arrow = null
            that.$viewport = null
            that.$element = null
        })
    }

    Tooltip.prototype.sanitizeHtml = function (unsafeHtml) {
        return sanitizeHtml(unsafeHtml, this.options.whiteList, this.options.sanitizeFn)
    }

    // TOOLTIP PLUGIN DEFINITION
    // =========================

    function Plugin(option) {
        return this.each(function () {
            var $this = $(this)
            var data = $this.data('bs.tooltip')
            var options = typeof option == 'object' && option

            if (!data && /destroy|hide/.test(option)) return
            if (!data) $this.data('bs.tooltip', (data = new Tooltip(this, options)))
            if (typeof option == 'string') data[option]()
        })
    }

    var old = $.fn.tooltip

    $.fn.tooltip = Plugin
    $.fn.tooltip.Constructor = Tooltip


    // TOOLTIP NO CONFLICT
    // ===================

    $.fn.tooltip.noConflict = function () {
        $.fn.tooltip = old
        return this
    }

}(jQuery);

/* ========================================================================
 * Bootstrap: transition.js v3.4.1
 * https://getbootstrap.com/docs/3.4/javascript/#transitions
 * ========================================================================
 * Copyright 2011-2019 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
    'use strict';

    // CSS TRANSITION SUPPORT (Shoutout: https://modernizr.com/)
    // ============================================================

    function transitionEnd() {
        var el = document.createElement('bootstrap')

        var transEndEventNames = {
            WebkitTransition: 'webkitTransitionEnd',
            MozTransition: 'transitionend',
            OTransition: 'oTransitionEnd otransitionend',
            transition: 'transitionend'
        }

        for (var name in transEndEventNames) {
            if (el.style[name] !== undefined) {
                return {end: transEndEventNames[name]}
            }
        }

        return false // explicit for ie8 (  ._.)
    }

    // https://blog.alexmaccaw.com/css-transitions
    $.fn.emulateTransitionEnd = function (duration) {
        var called = false
        var $el = this
        $(this).one('bsTransitionEnd', function () {
            called = true
        })
        var callback = function () {
            if (!called) $($el).trigger($.support.transition.end)
        }
        setTimeout(callback, duration)
        return this
    }

    $(function () {
        $.support.transition = transitionEnd()

        if (!$.support.transition) return

        $.event.special.bsTransitionEnd = {
            bindType: $.support.transition.end,
            delegateType: $.support.transition.end,
            handle: function (e) {
                if ($(e.target).is(this)) return e.handleObj.handler.apply(this, arguments)
            }
        }
    })

}(jQuery);

/*!
 * Generated using the Bootstrap Customizer (https://getbootstrap.com/docs/3.4/customize/)
 */

/*!
 * Bootstrap v3.4.1 (https://getbootstrap.com/)
 * Copyright 2011-2019 Twitter, Inc.
 * Licensed under the MIT license
 */

if (typeof jQuery === 'undefined') {
    throw new Error('Bootstrap\'s JavaScript requires jQuery')
}
+function ($) {
    'use strict';
    var version = $.fn.jquery.split(' ')[0].split('.');
    if ((version[0] < 2 && version[1] < 9) || (version[0] === 1 && version[1] === 9 && version[2] < 1) || (version[0] > 3)) {
        throw new Error('Bootstrap\'s JavaScript requires jQuery version 1.9.1 or higher, but lower than version 4')
    }
}(jQuery);

/* ========================================================================
 * Bootstrap: modal.js v3.4.1
 * https://getbootstrap.com/docs/3.4/javascript/#modals
 * ========================================================================
 * Copyright 2011-2019 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
    'use strict';

    // MODAL CLASS DEFINITION
    // ======================

    var Modal = function (element, options) {
        this.options = options;
        this.$body = $(document.body);
        this.$element = $(element);
        this.$dialog = this.$element.find('.modal-dialog');
        this.$backdrop = null;
        this.isShown = null;
        this.originalBodyPad = null;
        this.scrollbarWidth = 0;
        this.ignoreBackdropClick = false;
        this.fixedContent = '.navbar-fixed-top, .navbar-fixed-bottom';

        if (this.options.remote) {
            this.$element
                .find('.modal-content')
                .load(this.options.remote, $.proxy(function () {
                    this.$element.trigger('loaded.bs.modal')
                }, this))
        }
    };

    Modal.VERSION = '3.4.1';

    Modal.TRANSITION_DURATION = 300;
    Modal.BACKDROP_TRANSITION_DURATION = 150;

    Modal.DEFAULTS = {
        backdrop: true,
        keyboard: true,
        show: true
    };

    Modal.prototype.toggle = function (_relatedTarget) {
        return this.isShown ? this.hide() : this.show(_relatedTarget)
    };

    Modal.prototype.show = function (_relatedTarget) {
        var that = this;
        var e = $.Event('show.bs.modal', { relatedTarget: _relatedTarget });

        this.$element.trigger(e);

        if (this.isShown || e.isDefaultPrevented()) return;

        this.isShown = true;

        this.checkScrollbar();
        this.setScrollbar();
        this.$body.addClass('modal-open');

        this.escape();
        this.resize();

        this.$element.on('click.dismiss.bs.modal', '[data-dismiss="modal"]', $.proxy(this.hide, this));

        this.$dialog.on('mousedown.dismiss.bs.modal', function () {
            that.$element.one('mouseup.dismiss.bs.modal', function (e) {
                if ($(e.target).is(that.$element)) that.ignoreBackdropClick = true
            })
        });

        this.backdrop(function () {
            var transition = $.support.transition && that.$element.hasClass('fade');

            if (!that.$element.parent().length) {
                that.$element.appendTo(that.$body) // don't move modals dom position
            }

            that.$element
                .show()
                .scrollTop(0);

            that.adjustDialog();

            if (transition) {
                that.$element[0].offsetWidth // force reflow
            }

            that.$element.addClass('in');

            that.enforceFocus();

            var e = $.Event('shown.bs.modal', { relatedTarget: _relatedTarget });

            transition ?
                that.$dialog // wait for modal to slide in
                    .one('bsTransitionEnd', function () {
                        that.$element.trigger('focus').trigger(e)
                    })
                    .emulateTransitionEnd(Modal.TRANSITION_DURATION) :
                that.$element.trigger('focus').trigger(e)
        })
    };

    Modal.prototype.hide = function (e) {
        if (e) e.preventDefault();

        e = $.Event('hide.bs.modal');

        this.$element.trigger(e);

        if (!this.isShown || e.isDefaultPrevented()) return;

        this.isShown = false;

        this.escape();
        this.resize();

        $(document).off('focusin.bs.modal');

        this.$element
            .removeClass('in')
            .off('click.dismiss.bs.modal')
            .off('mouseup.dismiss.bs.modal');

        this.$dialog.off('mousedown.dismiss.bs.modal');

        $.support.transition && this.$element.hasClass('fade') ?
            this.$element
                .one('bsTransitionEnd', $.proxy(this.hideModal, this))
                .emulateTransitionEnd(Modal.TRANSITION_DURATION) :
            this.hideModal()
    };

    Modal.prototype.enforceFocus = function () {
        $(document)
            .off('focusin.bs.modal') // guard against infinite focus loop
            .on('focusin.bs.modal', $.proxy(function (e) {
                if (document !== e.target &&
                    this.$element[0] !== e.target &&
                    !this.$element.has(e.target).length) {
                    this.$element.trigger('focus')
                }
            }, this))
    };

    Modal.prototype.escape = function () {
        if (this.isShown && this.options.keyboard) {
            this.$element.on('keydown.dismiss.bs.modal', $.proxy(function (e) {
                e.which === 27 && this.hide()
            }, this))
        } else if (!this.isShown) {
            this.$element.off('keydown.dismiss.bs.modal')
        }
    };

    Modal.prototype.resize = function () {
        if (this.isShown) {
            $(window).on('resize.bs.modal', $.proxy(this.handleUpdate, this))
        } else {
            $(window).off('resize.bs.modal')
        }
    };

    Modal.prototype.hideModal = function () {
        var that = this;
        this.$element.hide();
        this.backdrop(function () {
            that.$body.removeClass('modal-open');
            that.resetAdjustments();
            that.resetScrollbar();
            that.$element.trigger('hidden.bs.modal')
        })
    };

    Modal.prototype.removeBackdrop = function () {
        this.$backdrop && this.$backdrop.remove();
        this.$backdrop = null;
    };

    Modal.prototype.backdrop = function (callback) {
        var that = this;
        var animate = this.$element.hasClass('fade') ? 'fade' : '';

        if (this.isShown && this.options.backdrop) {
            var doAnimate = $.support.transition && animate;

            this.$backdrop = $(document.createElement('div'))
                .addClass('modal-backdrop ' + animate)
                .appendTo(this.$body);

            this.$element.on('click.dismiss.bs.modal', $.proxy(function (e) {
                if (this.ignoreBackdropClick) {
                    this.ignoreBackdropClick = false;
                    return
                }
                if (e.target !== e.currentTarget) return;
                this.options.backdrop === 'static'
                    ? this.$element[0].focus()
                    : this.hide()
            }, this));

            if (doAnimate) this.$backdrop[0].offsetWidth; // force reflow

            this.$backdrop.addClass('in');

            if (!callback) return;

            doAnimate ?
                this.$backdrop
                    .one('bsTransitionEnd', callback)
                    .emulateTransitionEnd(Modal.BACKDROP_TRANSITION_DURATION) :
                callback()

        } else if (!this.isShown && this.$backdrop) {
            this.$backdrop.removeClass('in');

            var callbackRemove = function () {
                that.removeBackdrop();
                callback && callback()
            };
            $.support.transition && this.$element.hasClass('fade') ?
                this.$backdrop
                    .one('bsTransitionEnd', callbackRemove)
                    .emulateTransitionEnd(Modal.BACKDROP_TRANSITION_DURATION) :
                callbackRemove()

        } else if (callback) {
            callback()
        }
    };

    // these following methods are used to handle overflowing modals

    Modal.prototype.handleUpdate = function () {
        this.adjustDialog()
    };

    Modal.prototype.adjustDialog = function () {
        var modalIsOverflowing = this.$element[0].scrollHeight > document.documentElement.clientHeight

        this.$element.css({
            paddingLeft: !this.bodyIsOverflowing && modalIsOverflowing ? this.scrollbarWidth : '',
            paddingRight: this.bodyIsOverflowing && !modalIsOverflowing ? this.scrollbarWidth : ''
        })
    };

    Modal.prototype.resetAdjustments = function () {
        this.$element.css({
            paddingLeft: '',
            paddingRight: ''
        })
    };

    Modal.prototype.checkScrollbar = function () {
        var fullWindowWidth = window.innerWidth;
        if (!fullWindowWidth) { // workaround for missing window.innerWidth in IE8
            var documentElementRect = document.documentElement.getBoundingClientRect();
            fullWindowWidth = documentElementRect.right - Math.abs(documentElementRect.left)
        }
        this.bodyIsOverflowing = document.body.clientWidth < fullWindowWidth;
        this.scrollbarWidth = this.measureScrollbar()
    };

    Modal.prototype.setScrollbar = function () {
        var bodyPad = parseInt((this.$body.css('padding-right') || 0), 10);
        this.originalBodyPad = document.body.style.paddingRight || '';
        var scrollbarWidth = this.scrollbarWidth;
        if (this.bodyIsOverflowing) {
            this.$body.css('padding-right', bodyPad + scrollbarWidth);
            $(this.fixedContent).each(function (index, element) {
                var actualPadding = element.style.paddingRight;
                var calculatedPadding = $(element).css('padding-right');
                $(element)
                    .data('padding-right', actualPadding)
                    .css('padding-right', parseFloat(calculatedPadding) + scrollbarWidth + 'px')
            })
        }
    };

    Modal.prototype.resetScrollbar = function () {
        this.$body.css('padding-right', this.originalBodyPad);
        $(this.fixedContent).each(function (index, element) {
            var padding = $(element).data('padding-right');
            $(element).removeData('padding-right');
            element.style.paddingRight = padding ? padding : ''
        })
    };

    Modal.prototype.measureScrollbar = function () { // thx walsh
        var scrollDiv = document.createElement('div');
        scrollDiv.className = 'modal-scrollbar-measure';
        this.$body.append(scrollDiv);
        var scrollbarWidth = scrollDiv.offsetWidth - scrollDiv.clientWidth;
        this.$body[0].removeChild(scrollDiv);
        return scrollbarWidth;
    };


    // MODAL PLUGIN DEFINITION
    // =======================

    function Plugin(option, _relatedTarget) {
        return this.each(function () {
            var $this = $(this);
            var data = $this.data('bs.modal');
            var options = $.extend({}, Modal.DEFAULTS, $this.data(), typeof option == 'object' && option);

            if (!data) $this.data('bs.modal', (data = new Modal(this, options)));
            if (typeof option == 'string') data[option](_relatedTarget);
            else if (options.show) data.show(_relatedTarget)
        })
    }

    var old = $.fn.modal;

    $.fn.modal = Plugin;
    $.fn.modal.Constructor = Modal;


    // MODAL NO CONFLICT
    // =================

    $.fn.modal.noConflict = function () {
        $.fn.modal = old;
        return this
    };


    // MODAL DATA-API
    // ==============

    $(document).on('click.bs.modal.data-api', '[data-toggle="modal"]', function (e) {
        var $this = $(this);
        var href = $this.attr('href');
        var target = $this.attr('data-target') ||
            (href && href.replace(/.*(?=#[^\s]+$)/, '')); // strip for ie7

        var $target = $(document).find(target);
        var option = $target.data('bs.modal') ? 'toggle' : $.extend({ remote: !/#/.test(href) && href }, $target.data(), $this.data());

        if ($this.is('a')) e.preventDefault();

        $target.one('show.bs.modal', function (showEvent) {
            if (showEvent.isDefaultPrevented()) return; // only register focus restorer if modal will actually get shown
            $target.one('hidden.bs.modal', function () {
                $this.is(':visible') && $this.trigger('focus')
            })
        });
        Plugin.call($target, option, this)
    })

}(jQuery);

/* ========================================================================
 * Bootstrap: transition.js v3.4.1
 * https://getbootstrap.com/docs/3.4/javascript/#transitions
 * ========================================================================
 * Copyright 2011-2019 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
    'use strict';

    // CSS TRANSITION SUPPORT (Shoutout: https://modernizr.com/)
    // ============================================================

    function transitionEnd() {
        var el = document.createElement('bootstrap');

        var transEndEventNames = {
            WebkitTransition : 'webkitTransitionEnd',
            MozTransition    : 'transitionend',
            OTransition      : 'oTransitionEnd otransitionend',
            transition       : 'transitionend'
        };

        for (var name in transEndEventNames) {
            if (el.style[name] !== undefined) {
                return { end: transEndEventNames[name] }
            }
        }

        return false // explicit for ie8 (  ._.)
    }

    // https://blog.alexmaccaw.com/css-transitions
    $.fn.emulateTransitionEnd = function (duration) {
        var called = false;
        var $el = this;
        $(this).one('bsTransitionEnd', function () { called = true });
        var callback = function () { if (!called) $($el).trigger($.support.transition.end) };
        setTimeout(callback, duration);
        return this;
    };

    $(function () {
        $.support.transition = transitionEnd();

        if (!$.support.transition) return;

        $.event.special.bsTransitionEnd = {
            bindType: $.support.transition.end,
            delegateType: $.support.transition.end,
            handle: function (e) {
                if ($(e.target).is(this)) return e.handleObj.handler.apply(this, arguments)
            }
        }
    })

}(jQuery);

/**
 * @license Copyright 2018, Ably
 *
 * Ably JavaScript Library v1.0.18
 * https://github.com/ably/ably-js
 *
 * Ably Realtime Messaging
 * https://www.ably.io
 *
 * Released under the Apache Licence v2.0
 */

;(function() {
	"use strict";
	var Ably = window.Ably = this;

  /*
    Prevent libraries such as msgpack plugging into AMD or CommonJS
    as the libraries loaded are expected in the `this` context.
    `require` is only used within the Node.js library, the ably-js browser library
    is built as a single Javascript file.
  */
  var define, exports, require;


/**
 * CryptoJS core components.
 */
var CryptoJS = CryptoJS || (function (Math, undefined) {
    /**
     * CryptoJS namespace.
     */
    var C = {};

    /**
     * Library namespace.
     */
    var C_lib = C.lib = {};

    /**
     * Base object for prototypal inheritance.
     */
    var Base = C_lib.Base = (function () {
        function F() {}

        return {
            /**
             * Creates a new object that inherits from this object.
             *
             * @param {Object} overrides Properties to copy into the new object.
             *
             * @return {Object} The new object.
             *
             * @static
             *
             * @example
             *
             *     var MyType = CryptoJS.lib.Base.extend({
             *         field: 'value',
             *
             *         method: function () {
             *         }
             *     });
             */
            extend: function (overrides) {
                // Spawn
                F.prototype = this;
                var subtype = new F();

                // Augment
                if (overrides) {
                    subtype.mixIn(overrides);
                }

                // Create default initializer
                if (!subtype.hasOwnProperty('init')) {
                    subtype.init = function () {
                        subtype.$super.init.apply(this, arguments);
                    };
                }

                // Initializer's prototype is the subtype object
                subtype.init.prototype = subtype;

                // Reference supertype
                subtype.$super = this;

                return subtype;
            },

            /**
             * Extends this object and runs the init method.
             * Arguments to create() will be passed to init().
             *
             * @return {Object} The new object.
             *
             * @static
             *
             * @example
             *
             *     var instance = MyType.create();
             */
            create: function () {
                var instance = this.extend();
                instance.init.apply(instance, arguments);

                return instance;
            },

            /**
             * Initializes a newly created object.
             * Override this method to add some logic when your objects are created.
             *
             * @example
             *
             *     var MyType = CryptoJS.lib.Base.extend({
             *         init: function () {
             *             // ...
             *         }
             *     });
             */
            init: function () {
            },

            /**
             * Copies properties into this object.
             *
             * @param {Object} properties The properties to mix in.
             *
             * @example
             *
             *     MyType.mixIn({
             *         field: 'value'
             *     });
             */
            mixIn: function (properties) {
                for (var propertyName in properties) {
                    if (properties.hasOwnProperty(propertyName)) {
                        this[propertyName] = properties[propertyName];
                    }
                }

                // IE won't copy toString using the loop above
                if (properties.hasOwnProperty('toString')) {
                    this.toString = properties.toString;
                }
            },

            /**
             * Creates a copy of this object.
             *
             * @return {Object} The clone.
             *
             * @example
             *
             *     var clone = instance.clone();
             */
            clone: function () {
                return this.init.prototype.extend(this);
            }
        };
    }());

    /**
     * An array of 32-bit words.
     *
     * @property {Array} words The array of 32-bit words.
     * @property {number} sigBytes The number of significant bytes in this word array.
     */
    var WordArray = C_lib.WordArray = Base.extend({
        /**
         * Initializes a newly created word array.
         *
         * @param {Array} words (Optional) An array of 32-bit words.
         * @param {number} sigBytes (Optional) The number of significant bytes in the words.
         *
         * @example
         *
         *     var wordArray = CryptoJS.lib.WordArray.create();
         *     var wordArray = CryptoJS.lib.WordArray.create([0x00010203, 0x04050607]);
         *     var wordArray = CryptoJS.lib.WordArray.create([0x00010203, 0x04050607], 6);
         */
        init: function (words, sigBytes) {
            words = this.words = words || [];

            if (sigBytes != undefined) {
                this.sigBytes = sigBytes;
            } else {
                this.sigBytes = words.length * 4;
            }
        },

        /**
         * Converts this word array to a string.
         *
         * @param {Encoder} encoder (Optional) The encoding strategy to use. Default: CryptoJS.enc.Hex
         *
         * @return {string} The stringified word array.
         *
         * @example
         *
         *     var string = wordArray + '';
         *     var string = wordArray.toString();
         *     var string = wordArray.toString(CryptoJS.enc.Utf8);
         */
        toString: function (encoder) {
            return (encoder || Hex).stringify(this);
        },

        /**
         * Concatenates a word array to this word array.
         *
         * @param {WordArray} wordArray The word array to append.
         *
         * @return {WordArray} This word array.
         *
         * @example
         *
         *     wordArray1.concat(wordArray2);
         */
        concat: function (wordArray) {
            // Shortcuts
            var thisWords = this.words;
            var thatWords = wordArray.words;
            var thisSigBytes = this.sigBytes;
            var thatSigBytes = wordArray.sigBytes;

            // Clamp excess bits
            this.clamp();

            // Concat
            if (thisSigBytes % 4) {
                // Copy one byte at a time
                for (var i = 0; i < thatSigBytes; i++) {
                    var thatByte = (thatWords[i >>> 2] >>> (24 - (i % 4) * 8)) & 0xff;
                    thisWords[(thisSigBytes + i) >>> 2] |= thatByte << (24 - ((thisSigBytes + i) % 4) * 8);
                }
            } else if (thatWords.length > 0xffff) {
                // Copy one word at a time
                for (var i = 0; i < thatSigBytes; i += 4) {
                    thisWords[(thisSigBytes + i) >>> 2] = thatWords[i >>> 2];
                }
            } else {
                // Copy all words at once
                thisWords.push.apply(thisWords, thatWords);
            }
            this.sigBytes += thatSigBytes;

            // Chainable
            return this;
        },

        /**
         * Removes insignificant bits.
         *
         * @example
         *
         *     wordArray.clamp();
         */
        clamp: function () {
            // Shortcuts
            var words = this.words;
            var sigBytes = this.sigBytes;

            // Clamp
            words[sigBytes >>> 2] &= 0xffffffff << (32 - (sigBytes % 4) * 8);
            words.length = Math.ceil(sigBytes / 4);
        },

        /**
         * Creates a copy of this word array.
         *
         * @return {WordArray} The clone.
         *
         * @example
         *
         *     var clone = wordArray.clone();
         */
        clone: function () {
            var clone = Base.clone.call(this);
            clone.words = this.words.slice(0);

            return clone;
        },

        /**
         * Creates a word array filled with random bytes.
         *
         * @param {number} nBytes The number of random bytes to generate.
         *
         * @return {WordArray} The random word array.
         *
         * @static
         *
         * @example
         *
         *     var wordArray = CryptoJS.lib.WordArray.random(16);
         */
        random: function (nBytes) {
            var words = [];

            var r = (function (m_w) {
                var m_w = m_w;
                var m_z = 0x3ade68b1;
                var mask = 0xffffffff;

                return function () {
                    m_z = (0x9069 * (m_z & 0xFFFF) + (m_z >> 0x10)) & mask;
                    m_w = (0x4650 * (m_w & 0xFFFF) + (m_w >> 0x10)) & mask;
                    var result = ((m_z << 0x10) + m_w) & mask;
                    result /= 0x100000000;
                    result += 0.5;
                    return result * (Math.random() > .5 ? 1 : -1);
                }
            });

            for (var i = 0, rcache; i < nBytes; i += 4) {
                var _r = r((rcache || Math.random()) * 0x100000000);

                rcache = _r() * 0x3ade67b7;
                words.push((_r() * 0x100000000) | 0);
            }

            return new WordArray.init(words, nBytes);
        }
    });

    /**
     * Encoder namespace.
     */
    var C_enc = C.enc = {};

    /**
     * Hex encoding strategy.
     */
    var Hex = C_enc.Hex = {
        /**
         * Converts a word array to a hex string.
         *
         * @param {WordArray} wordArray The word array.
         *
         * @return {string} The hex string.
         *
         * @static
         *
         * @example
         *
         *     var hexString = CryptoJS.enc.Hex.stringify(wordArray);
         */
        stringify: function (wordArray) {
            // Shortcuts
            var words = wordArray.words;
            var sigBytes = wordArray.sigBytes;

            // Convert
            var hexChars = [];
            for (var i = 0; i < sigBytes; i++) {
                var bite = (words[i >>> 2] >>> (24 - (i % 4) * 8)) & 0xff;
                hexChars.push((bite >>> 4).toString(16));
                hexChars.push((bite & 0x0f).toString(16));
            }

            return hexChars.join('');
        },

        /**
         * Converts a hex string to a word array.
         *
         * @param {string} hexStr The hex string.
         *
         * @return {WordArray} The word array.
         *
         * @static
         *
         * @example
         *
         *     var wordArray = CryptoJS.enc.Hex.parse(hexString);
         */
        parse: function (hexStr) {
            // Shortcut
            var hexStrLength = hexStr.length;

            // Convert
            var words = [];
            for (var i = 0; i < hexStrLength; i += 2) {
                words[i >>> 3] |= parseInt(hexStr.substr(i, 2), 16) << (24 - (i % 8) * 4);
            }

            return new WordArray.init(words, hexStrLength / 2);
        }
    };

    /**
     * Latin1 encoding strategy.
     */
    var Latin1 = C_enc.Latin1 = {
        /**
         * Converts a word array to a Latin1 string.
         *
         * @param {WordArray} wordArray The word array.
         *
         * @return {string} The Latin1 string.
         *
         * @static
         *
         * @example
         *
         *     var latin1String = CryptoJS.enc.Latin1.stringify(wordArray);
         */
        stringify: function (wordArray) {
            // Shortcuts
            var words = wordArray.words;
            var sigBytes = wordArray.sigBytes;

            // Convert
            var latin1Chars = [];
            for (var i = 0; i < sigBytes; i++) {
                var bite = (words[i >>> 2] >>> (24 - (i % 4) * 8)) & 0xff;
                latin1Chars.push(String.fromCharCode(bite));
            }

            return latin1Chars.join('');
        },

        /**
         * Converts a Latin1 string to a word array.
         *
         * @param {string} latin1Str The Latin1 string.
         *
         * @return {WordArray} The word array.
         *
         * @static
         *
         * @example
         *
         *     var wordArray = CryptoJS.enc.Latin1.parse(latin1String);
         */
        parse: function (latin1Str) {
            // Shortcut
            var latin1StrLength = latin1Str.length;

            // Convert
            var words = [];
            for (var i = 0; i < latin1StrLength; i++) {
                words[i >>> 2] |= (latin1Str.charCodeAt(i) & 0xff) << (24 - (i % 4) * 8);
            }

            return new WordArray.init(words, latin1StrLength);
        }
    };

    /**
     * UTF-8 encoding strategy.
     */
    var Utf8 = C_enc.Utf8 = {
        /**
         * Converts a word array to a UTF-8 string.
         *
         * @param {WordArray} wordArray The word array.
         *
         * @return {string} The UTF-8 string.
         *
         * @static
         *
         * @example
         *
         *     var utf8String = CryptoJS.enc.Utf8.stringify(wordArray);
         */
        stringify: function (wordArray) {
            try {
                return decodeURIComponent(escape(Latin1.stringify(wordArray)));
            } catch (e) {
                throw new Error('Malformed UTF-8 data');
            }
        },

        /**
         * Converts a UTF-8 string to a word array.
         *
         * @param {string} utf8Str The UTF-8 string.
         *
         * @return {WordArray} The word array.
         *
         * @static
         *
         * @example
         *
         *     var wordArray = CryptoJS.enc.Utf8.parse(utf8String);
         */
        parse: function (utf8Str) {
            return Latin1.parse(unescape(encodeURIComponent(utf8Str)));
        }
    };

    /**
     * Abstract buffered block algorithm template.
     *
     * The property blockSize must be implemented in a concrete subtype.
     *
     * @property {number} _minBufferSize The number of blocks that should be kept unprocessed in the buffer. Default: 0
     */
    var BufferedBlockAlgorithm = C_lib.BufferedBlockAlgorithm = Base.extend({
        /**
         * Resets this block algorithm's data buffer to its initial state.
         *
         * @example
         *
         *     bufferedBlockAlgorithm.reset();
         */
        reset: function () {
            // Initial values
            this._data = new WordArray.init();
            this._nDataBytes = 0;
        },

        /**
         * Adds new data to this block algorithm's buffer.
         *
         * @param {WordArray|string} data The data to append. Strings are converted to a WordArray using UTF-8.
         *
         * @example
         *
         *     bufferedBlockAlgorithm._append('data');
         *     bufferedBlockAlgorithm._append(wordArray);
         */
        _append: function (data) {
            // Convert string to WordArray, else assume WordArray already
            if (typeof data == 'string') {
                data = Utf8.parse(data);
            }

            // Append
            this._data.concat(data);
            this._nDataBytes += data.sigBytes;
        },

        /**
         * Processes available data blocks.
         *
         * This method invokes _doProcessBlock(offset), which must be implemented by a concrete subtype.
         *
         * @param {boolean} doFlush Whether all blocks and partial blocks should be processed.
         *
         * @return {WordArray} The processed data.
         *
         * @example
         *
         *     var processedData = bufferedBlockAlgorithm._process();
         *     var processedData = bufferedBlockAlgorithm._process(!!'flush');
         */
        _process: function (doFlush) {
            // Shortcuts
            var data = this._data;
            var dataWords = data.words;
            var dataSigBytes = data.sigBytes;
            var blockSize = this.blockSize;
            var blockSizeBytes = blockSize * 4;

            // Count blocks ready
            var nBlocksReady = dataSigBytes / blockSizeBytes;
            if (doFlush) {
                // Round up to include partial blocks
                nBlocksReady = Math.ceil(nBlocksReady);
            } else {
                // Round down to include only full blocks,
                // less the number of blocks that must remain in the buffer
                nBlocksReady = Math.max((nBlocksReady | 0) - this._minBufferSize, 0);
            }

            // Count words ready
            var nWordsReady = nBlocksReady * blockSize;

            // Count bytes ready
            var nBytesReady = Math.min(nWordsReady * 4, dataSigBytes);

            // Process blocks
            if (nWordsReady) {
                for (var offset = 0; offset < nWordsReady; offset += blockSize) {
                    // Perform concrete-algorithm logic
                    this._doProcessBlock(dataWords, offset);
                }

                // Remove processed words
                var processedWords = dataWords.splice(0, nWordsReady);
                data.sigBytes -= nBytesReady;
            }

            // Return processed words
            return new WordArray.init(processedWords, nBytesReady);
        },

        /**
         * Creates a copy of this object.
         *
         * @return {Object} The clone.
         *
         * @example
         *
         *     var clone = bufferedBlockAlgorithm.clone();
         */
        clone: function () {
            var clone = Base.clone.call(this);
            clone._data = this._data.clone();

            return clone;
        },

        _minBufferSize: 0
    });

    /**
     * Abstract hasher template.
     *
     * @property {number} blockSize The number of 32-bit words this hasher operates on. Default: 16 (512 bits)
     */
    var Hasher = C_lib.Hasher = BufferedBlockAlgorithm.extend({
        /**
         * Configuration options.
         */
        cfg: Base.extend(),

        /**
         * Initializes a newly created hasher.
         *
         * @param {Object} cfg (Optional) The configuration options to use for this hash computation.
         *
         * @example
         *
         *     var hasher = CryptoJS.algo.SHA256.create();
         */
        init: function (cfg) {
            // Apply config defaults
            this.cfg = this.cfg.extend(cfg);

            // Set initial values
            this.reset();
        },

        /**
         * Resets this hasher to its initial state.
         *
         * @example
         *
         *     hasher.reset();
         */
        reset: function () {
            // Reset data buffer
            BufferedBlockAlgorithm.reset.call(this);

            // Perform concrete-hasher logic
            this._doReset();
        },

        /**
         * Updates this hasher with a message.
         *
         * @param {WordArray|string} messageUpdate The message to append.
         *
         * @return {Hasher} This hasher.
         *
         * @example
         *
         *     hasher.update('message');
         *     hasher.update(wordArray);
         */
        update: function (messageUpdate) {
            // Append
            this._append(messageUpdate);

            // Update the hash
            this._process();

            // Chainable
            return this;
        },

        /**
         * Finalizes the hash computation.
         * Note that the finalize operation is effectively a destructive, read-once operation.
         *
         * @param {WordArray|string} messageUpdate (Optional) A final message update.
         *
         * @return {WordArray} The hash.
         *
         * @example
         *
         *     var hash = hasher.finalize();
         *     var hash = hasher.finalize('message');
         *     var hash = hasher.finalize(wordArray);
         */
        finalize: function (messageUpdate) {
            // Final message update
            if (messageUpdate) {
                this._append(messageUpdate);
            }

            // Perform concrete-hasher logic
            var hash = this._doFinalize();

            return hash;
        },

        blockSize: 512/32,

        /**
         * Creates a shortcut function to a hasher's object interface.
         *
         * @param {Hasher} hasher The hasher to create a helper for.
         *
         * @return {Function} The shortcut function.
         *
         * @static
         *
         * @example
         *
         *     var SHA256 = CryptoJS.lib.Hasher._createHelper(CryptoJS.algo.SHA256);
         */
        _createHelper: function (hasher) {
            return function (message, cfg) {
                return new hasher.init(cfg).finalize(message);
            };
        },

        /**
         * Creates a shortcut function to the HMAC's object interface.
         *
         * @param {Hasher} hasher The hasher to use in this HMAC helper.
         *
         * @return {Function} The shortcut function.
         *
         * @static
         *
         * @example
         *
         *     var HmacSHA256 = CryptoJS.lib.Hasher._createHmacHelper(CryptoJS.algo.SHA256);
         */
        _createHmacHelper: function (hasher) {
            return function (message, key) {
                return new C_algo.HMAC.init(hasher, key).finalize(message);
            };
        }
    });

    /**
     * Algorithm namespace.
     */
    var C_algo = C.algo = {};

    return C;
}(Math));

(function (Math) {
    // Shortcuts
    var C = CryptoJS;
    var C_lib = C.lib;
    var WordArray = C_lib.WordArray;
    var Hasher = C_lib.Hasher;
    var C_algo = C.algo;

    // Initialization and round constants tables
    var H = [];
    var K = [];

    // Compute constants
    (function () {
        function isPrime(n) {
            var sqrtN = Math.sqrt(n);
            for (var factor = 2; factor <= sqrtN; factor++) {
                if (!(n % factor)) {
                    return false;
                }
            }

            return true;
        }

        function getFractionalBits(n) {
            return ((n - (n | 0)) * 0x100000000) | 0;
        }

        var n = 2;
        var nPrime = 0;
        while (nPrime < 64) {
            if (isPrime(n)) {
                if (nPrime < 8) {
                    H[nPrime] = getFractionalBits(Math.pow(n, 1 / 2));
                }
                K[nPrime] = getFractionalBits(Math.pow(n, 1 / 3));

                nPrime++;
            }

            n++;
        }
    }());

    // Reusable object
    var W = [];

    /**
     * SHA-256 hash algorithm.
     */
    var SHA256 = C_algo.SHA256 = Hasher.extend({
        _doReset: function () {
            this._hash = new WordArray.init(H.slice(0));
        },

        _doProcessBlock: function (M, offset) {
            // Shortcut
            var H = this._hash.words;

            // Working variables
            var a = H[0];
            var b = H[1];
            var c = H[2];
            var d = H[3];
            var e = H[4];
            var f = H[5];
            var g = H[6];
            var h = H[7];

            // Computation
            for (var i = 0; i < 64; i++) {
                if (i < 16) {
                    W[i] = M[offset + i] | 0;
                } else {
                    var gamma0x = W[i - 15];
                    var gamma0  = ((gamma0x << 25) | (gamma0x >>> 7))  ^
                                  ((gamma0x << 14) | (gamma0x >>> 18)) ^
                                   (gamma0x >>> 3);

                    var gamma1x = W[i - 2];
                    var gamma1  = ((gamma1x << 15) | (gamma1x >>> 17)) ^
                                  ((gamma1x << 13) | (gamma1x >>> 19)) ^
                                   (gamma1x >>> 10);

                    W[i] = gamma0 + W[i - 7] + gamma1 + W[i - 16];
                }

                var ch  = (e & f) ^ (~e & g);
                var maj = (a & b) ^ (a & c) ^ (b & c);

                var sigma0 = ((a << 30) | (a >>> 2)) ^ ((a << 19) | (a >>> 13)) ^ ((a << 10) | (a >>> 22));
                var sigma1 = ((e << 26) | (e >>> 6)) ^ ((e << 21) | (e >>> 11)) ^ ((e << 7)  | (e >>> 25));

                var t1 = h + sigma1 + ch + K[i] + W[i];
                var t2 = sigma0 + maj;

                h = g;
                g = f;
                f = e;
                e = (d + t1) | 0;
                d = c;
                c = b;
                b = a;
                a = (t1 + t2) | 0;
            }

            // Intermediate hash value
            H[0] = (H[0] + a) | 0;
            H[1] = (H[1] + b) | 0;
            H[2] = (H[2] + c) | 0;
            H[3] = (H[3] + d) | 0;
            H[4] = (H[4] + e) | 0;
            H[5] = (H[5] + f) | 0;
            H[6] = (H[6] + g) | 0;
            H[7] = (H[7] + h) | 0;
        },

        _doFinalize: function () {
            // Shortcuts
            var data = this._data;
            var dataWords = data.words;

            var nBitsTotal = this._nDataBytes * 8;
            var nBitsLeft = data.sigBytes * 8;

            // Add padding
            dataWords[nBitsLeft >>> 5] |= 0x80 << (24 - nBitsLeft % 32);
            dataWords[(((nBitsLeft + 64) >>> 9) << 4) + 14] = Math.floor(nBitsTotal / 0x100000000);
            dataWords[(((nBitsLeft + 64) >>> 9) << 4) + 15] = nBitsTotal;
            data.sigBytes = dataWords.length * 4;

            // Hash final blocks
            this._process();

            // Return final computed hash
            return this._hash;
        },

        clone: function () {
            var clone = Hasher.clone.call(this);
            clone._hash = this._hash.clone();

            return clone;
        }
    });

    /**
     * Shortcut function to the hasher's object interface.
     *
     * @param {WordArray|string} message The message to hash.
     *
     * @return {WordArray} The hash.
     *
     * @static
     *
     * @example
     *
     *     var hash = CryptoJS.SHA256('message');
     *     var hash = CryptoJS.SHA256(wordArray);
     */
    C.SHA256 = Hasher._createHelper(SHA256);

    /**
     * Shortcut function to the HMAC's object interface.
     *
     * @param {WordArray|string} message The message to hash.
     * @param {WordArray|string} key The secret key.
     *
     * @return {WordArray} The HMAC.
     *
     * @static
     *
     * @example
     *
     *     var hmac = CryptoJS.HmacSHA256(message, key);
     */
    C.HmacSHA256 = Hasher._createHmacHelper(SHA256);
}(Math));

(function () {
    // Shortcuts
    var C = CryptoJS;
    var C_lib = C.lib;
    var Base = C_lib.Base;
    var C_enc = C.enc;
    var Utf8 = C_enc.Utf8;
    var C_algo = C.algo;

    /**
     * HMAC algorithm.
     */
    var HMAC = C_algo.HMAC = Base.extend({
        /**
         * Initializes a newly created HMAC.
         *
         * @param {Hasher} hasher The hash algorithm to use.
         * @param {WordArray|string} key The secret key.
         *
         * @example
         *
         *     var hmacHasher = CryptoJS.algo.HMAC.create(CryptoJS.algo.SHA256, key);
         */
        init: function (hasher, key) {
            // Init hasher
            hasher = this._hasher = new hasher.init();

            // Convert string to WordArray, else assume WordArray already
            if (typeof key == 'string') {
                key = Utf8.parse(key);
            }

            // Shortcuts
            var hasherBlockSize = hasher.blockSize;
            var hasherBlockSizeBytes = hasherBlockSize * 4;

            // Allow arbitrary length keys
            if (key.sigBytes > hasherBlockSizeBytes) {
                key = hasher.finalize(key);
            }

            // Clamp excess bits
            key.clamp();

            // Clone key for inner and outer pads
            var oKey = this._oKey = key.clone();
            var iKey = this._iKey = key.clone();

            // Shortcuts
            var oKeyWords = oKey.words;
            var iKeyWords = iKey.words;

            // XOR keys with pad constants
            for (var i = 0; i < hasherBlockSize; i++) {
                oKeyWords[i] ^= 0x5c5c5c5c;
                iKeyWords[i] ^= 0x36363636;
            }
            oKey.sigBytes = iKey.sigBytes = hasherBlockSizeBytes;

            // Set initial values
            this.reset();
        },

        /**
         * Resets this HMAC to its initial state.
         *
         * @example
         *
         *     hmacHasher.reset();
         */
        reset: function () {
            // Shortcut
            var hasher = this._hasher;

            // Reset
            hasher.reset();
            hasher.update(this._iKey);
        },

        /**
         * Updates this HMAC with a message.
         *
         * @param {WordArray|string} messageUpdate The message to append.
         *
         * @return {HMAC} This HMAC instance.
         *
         * @example
         *
         *     hmacHasher.update('message');
         *     hmacHasher.update(wordArray);
         */
        update: function (messageUpdate) {
            this._hasher.update(messageUpdate);

            // Chainable
            return this;
        },

        /**
         * Finalizes the HMAC computation.
         * Note that the finalize operation is effectively a destructive, read-once operation.
         *
         * @param {WordArray|string} messageUpdate (Optional) A final message update.
         *
         * @return {WordArray} The HMAC.
         *
         * @example
         *
         *     var hmac = hmacHasher.finalize();
         *     var hmac = hmacHasher.finalize('message');
         *     var hmac = hmacHasher.finalize(wordArray);
         */
        finalize: function (messageUpdate) {
            // Shortcut
            var hasher = this._hasher;

            // Compute HMAC
            var innerHash = hasher.finalize(messageUpdate);
            hasher.reset();
            var hmac = hasher.finalize(this._oKey.clone().concat(innerHash));

            return hmac;
        }
    });
}());

(function () {
    // Shortcuts
    var C = CryptoJS;
    var C_lib = C.lib;
    var WordArray = C_lib.WordArray;
    var C_enc = C.enc;

    /**
     * Base64 encoding strategy.
     */
    var Base64 = C_enc.Base64 = {
        /**
         * Converts a word array to a Base64 string.
         *
         * @param {WordArray} wordArray The word array.
         *
         * @return {string} The Base64 string.
         *
         * @static
         *
         * @example
         *
         *     var base64String = CryptoJS.enc.Base64.stringify(wordArray);
         */
        stringify: function (wordArray) {
            // Shortcuts
            var words = wordArray.words;
            var sigBytes = wordArray.sigBytes;
            var map = this._map;

            // Clamp excess bits
            wordArray.clamp();

            // Convert
            var base64Chars = [];
            for (var i = 0; i < sigBytes; i += 3) {
                var byte1 = (words[i >>> 2]       >>> (24 - (i % 4) * 8))       & 0xff;
                var byte2 = (words[(i + 1) >>> 2] >>> (24 - ((i + 1) % 4) * 8)) & 0xff;
                var byte3 = (words[(i + 2) >>> 2] >>> (24 - ((i + 2) % 4) * 8)) & 0xff;

                var triplet = (byte1 << 16) | (byte2 << 8) | byte3;

                for (var j = 0; (j < 4) && (i + j * 0.75 < sigBytes); j++) {
                    base64Chars.push(map.charAt((triplet >>> (6 * (3 - j))) & 0x3f));
                }
            }

            // Add padding
            var paddingChar = map.charAt(64);
            if (paddingChar) {
                while (base64Chars.length % 4) {
                    base64Chars.push(paddingChar);
                }
            }

            return base64Chars.join('');
        },

        /**
         * Converts a Base64 string to a word array.
         *
         * @param {string} base64Str The Base64 string.
         *
         * @return {WordArray} The word array.
         *
         * @static
         *
         * @example
         *
         *     var wordArray = CryptoJS.enc.Base64.parse(base64String);
         */
        parse: function (base64Str) {
            // Shortcuts
            var base64StrLength = base64Str.length;
            var map = this._map;

            // Ignore padding
            var paddingChar = map.charAt(64);
            if (paddingChar) {
                var paddingIndex = base64Str.indexOf(paddingChar);
                if (paddingIndex != -1) {
                    base64StrLength = paddingIndex;
                }
            }

            // Convert
            var words = [];
            var nBytes = 0;
            for (var i = 0; i < base64StrLength; i++) {
                if (i % 4) {
                    var bits1 = map.indexOf(base64Str.charAt(i - 1)) << ((i % 4) * 2);
                    var bits2 = map.indexOf(base64Str.charAt(i)) >>> (6 - (i % 4) * 2);
                    words[nBytes >>> 2] |= (bits1 | bits2) << (24 - (nBytes % 4) * 8);
                    nBytes++;
                }
            }

            return WordArray.create(words, nBytes);
        },

        _map: 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/='
    };
}());

/**
 * Cipher core components.
 */
CryptoJS.lib.Cipher || (function (undefined) {
    // Shortcuts
    var C = CryptoJS;
    var C_lib = C.lib;
    var Base = C_lib.Base;
    var WordArray = C_lib.WordArray;
    var BufferedBlockAlgorithm = C_lib.BufferedBlockAlgorithm;
    var C_enc = C.enc;
    var Utf8 = C_enc.Utf8;
    var Base64 = C_enc.Base64;
    var C_algo = C.algo;
    var EvpKDF = C_algo.EvpKDF;

    /**
     * Abstract base cipher template.
     *
     * @property {number} keySize This cipher's key size. Default: 4 (128 bits)
     * @property {number} ivSize This cipher's IV size. Default: 4 (128 bits)
     * @property {number} _ENC_XFORM_MODE A constant representing encryption mode.
     * @property {number} _DEC_XFORM_MODE A constant representing decryption mode.
     */
    var Cipher = C_lib.Cipher = BufferedBlockAlgorithm.extend({
        /**
         * Configuration options.
         *
         * @property {WordArray} iv The IV to use for this operation.
         */
        cfg: Base.extend(),

        /**
         * Creates this cipher in encryption mode.
         *
         * @param {WordArray} key The key.
         * @param {Object} cfg (Optional) The configuration options to use for this operation.
         *
         * @return {Cipher} A cipher instance.
         *
         * @static
         *
         * @example
         *
         *     var cipher = CryptoJS.algo.AES.createEncryptor(keyWordArray, { iv: ivWordArray });
         */
        createEncryptor: function (key, cfg) {
            return this.create(this._ENC_XFORM_MODE, key, cfg);
        },

        /**
         * Creates this cipher in decryption mode.
         *
         * @param {WordArray} key The key.
         * @param {Object} cfg (Optional) The configuration options to use for this operation.
         *
         * @return {Cipher} A cipher instance.
         *
         * @static
         *
         * @example
         *
         *     var cipher = CryptoJS.algo.AES.createDecryptor(keyWordArray, { iv: ivWordArray });
         */
        createDecryptor: function (key, cfg) {
            return this.create(this._DEC_XFORM_MODE, key, cfg);
        },

        /**
         * Initializes a newly created cipher.
         *
         * @param {number} xformMode Either the encryption or decryption transormation mode constant.
         * @param {WordArray} key The key.
         * @param {Object} cfg (Optional) The configuration options to use for this operation.
         *
         * @example
         *
         *     var cipher = CryptoJS.algo.AES.create(CryptoJS.algo.AES._ENC_XFORM_MODE, keyWordArray, { iv: ivWordArray });
         */
        init: function (xformMode, key, cfg) {
            // Apply config defaults
            this.cfg = this.cfg.extend(cfg);

            // Store transform mode and key
            this._xformMode = xformMode;
            this._key = key;

            // Set initial values
            this.reset();
        },

        /**
         * Resets this cipher to its initial state.
         *
         * @example
         *
         *     cipher.reset();
         */
        reset: function () {
            // Reset data buffer
            BufferedBlockAlgorithm.reset.call(this);

            // Perform concrete-cipher logic
            this._doReset();
        },

        /**
         * Adds data to be encrypted or decrypted.
         *
         * @param {WordArray|string} dataUpdate The data to encrypt or decrypt.
         *
         * @return {WordArray} The data after processing.
         *
         * @example
         *
         *     var encrypted = cipher.process('data');
         *     var encrypted = cipher.process(wordArray);
         */
        process: function (dataUpdate) {
            // Append
            this._append(dataUpdate);

            // Process available blocks
            return this._process();
        },

        /**
         * Finalizes the encryption or decryption process.
         * Note that the finalize operation is effectively a destructive, read-once operation.
         *
         * @param {WordArray|string} dataUpdate The final data to encrypt or decrypt.
         *
         * @return {WordArray} The data after final processing.
         *
         * @example
         *
         *     var encrypted = cipher.finalize();
         *     var encrypted = cipher.finalize('data');
         *     var encrypted = cipher.finalize(wordArray);
         */
        finalize: function (dataUpdate) {
            // Final data update
            if (dataUpdate) {
                this._append(dataUpdate);
            }

            // Perform concrete-cipher logic
            var finalProcessedData = this._doFinalize();

            return finalProcessedData;
        },

        keySize: 128/32,

        ivSize: 128/32,

        _ENC_XFORM_MODE: 1,

        _DEC_XFORM_MODE: 2,

        /**
         * Creates shortcut functions to a cipher's object interface.
         *
         * @param {Cipher} cipher The cipher to create a helper for.
         *
         * @return {Object} An object with encrypt and decrypt shortcut functions.
         *
         * @static
         *
         * @example
         *
         *     var AES = CryptoJS.lib.Cipher._createHelper(CryptoJS.algo.AES);
         */
        _createHelper: (function () {
            function selectCipherStrategy(key) {
                if (typeof key == 'string') {
                    return PasswordBasedCipher;
                } else {
                    return SerializableCipher;
                }
            }

            return function (cipher) {
                return {
                    encrypt: function (message, key, cfg) {
                        return selectCipherStrategy(key).encrypt(cipher, message, key, cfg);
                    },

                    decrypt: function (ciphertext, key, cfg) {
                        return selectCipherStrategy(key).decrypt(cipher, ciphertext, key, cfg);
                    }
                };
            };
        }())
    });

    /**
     * Abstract base stream cipher template.
     *
     * @property {number} blockSize The number of 32-bit words this cipher operates on. Default: 1 (32 bits)
     */
    var StreamCipher = C_lib.StreamCipher = Cipher.extend({
        _doFinalize: function () {
            // Process partial blocks
            var finalProcessedBlocks = this._process(!!'flush');

            return finalProcessedBlocks;
        },

        blockSize: 1
    });

    /**
     * Mode namespace.
     */
    var C_mode = C.mode = {};

    /**
     * Abstract base block cipher mode template.
     */
    var BlockCipherMode = C_lib.BlockCipherMode = Base.extend({
        /**
         * Creates this mode for encryption.
         *
         * @param {Cipher} cipher A block cipher instance.
         * @param {Array} iv The IV words.
         *
         * @static
         *
         * @example
         *
         *     var mode = CryptoJS.mode.CBC.createEncryptor(cipher, iv.words);
         */
        createEncryptor: function (cipher, iv) {
            return this.Encryptor.create(cipher, iv);
        },

        /**
         * Creates this mode for decryption.
         *
         * @param {Cipher} cipher A block cipher instance.
         * @param {Array} iv The IV words.
         *
         * @static
         *
         * @example
         *
         *     var mode = CryptoJS.mode.CBC.createDecryptor(cipher, iv.words);
         */
        createDecryptor: function (cipher, iv) {
            return this.Decryptor.create(cipher, iv);
        },

        /**
         * Initializes a newly created mode.
         *
         * @param {Cipher} cipher A block cipher instance.
         * @param {Array} iv The IV words.
         *
         * @example
         *
         *     var mode = CryptoJS.mode.CBC.Encryptor.create(cipher, iv.words);
         */
        init: function (cipher, iv) {
            this._cipher = cipher;
            this._iv = iv;
        }
    });

    /**
     * Cipher Block Chaining mode.
     */
    var CBC = C_mode.CBC = (function () {
        /**
         * Abstract base CBC mode.
         */
        var CBC = BlockCipherMode.extend();

        /**
         * CBC encryptor.
         */
        CBC.Encryptor = CBC.extend({
            /**
             * Processes the data block at offset.
             *
             * @param {Array} words The data words to operate on.
             * @param {number} offset The offset where the block starts.
             *
             * @example
             *
             *     mode.processBlock(data.words, offset);
             */
            processBlock: function (words, offset) {
                // Shortcuts
                var cipher = this._cipher;
                var blockSize = cipher.blockSize;

                // XOR and encrypt
                xorBlock.call(this, words, offset, blockSize);
                cipher.encryptBlock(words, offset);

                // Remember this block to use with next block
                this._prevBlock = words.slice(offset, offset + blockSize);
            }
        });

        /**
         * CBC decryptor.
         */
        CBC.Decryptor = CBC.extend({
            /**
             * Processes the data block at offset.
             *
             * @param {Array} words The data words to operate on.
             * @param {number} offset The offset where the block starts.
             *
             * @example
             *
             *     mode.processBlock(data.words, offset);
             */
            processBlock: function (words, offset) {
                // Shortcuts
                var cipher = this._cipher;
                var blockSize = cipher.blockSize;

                // Remember this block to use with next block
                var thisBlock = words.slice(offset, offset + blockSize);

                // Decrypt and XOR
                cipher.decryptBlock(words, offset);
                xorBlock.call(this, words, offset, blockSize);

                // This block becomes the previous block
                this._prevBlock = thisBlock;
            }
        });

        function xorBlock(words, offset, blockSize) {
            // Shortcut
            var iv = this._iv;

            // Choose mixing block
            if (iv) {
                var block = iv;

                // Remove IV for subsequent blocks
                this._iv = undefined;
            } else {
                var block = this._prevBlock;
            }

            // XOR blocks
            for (var i = 0; i < blockSize; i++) {
                words[offset + i] ^= block[i];
            }
        }

        return CBC;
    }());

    /**
     * Padding namespace.
     */
    var C_pad = C.pad = {};

    /**
     * PKCS #5/7 padding strategy.
     */
    var Pkcs7 = C_pad.Pkcs7 = {
        /**
         * Pads data using the algorithm defined in PKCS #5/7.
         *
         * @param {WordArray} data The data to pad.
         * @param {number} blockSize The multiple that the data should be padded to.
         *
         * @static
         *
         * @example
         *
         *     CryptoJS.pad.Pkcs7.pad(wordArray, 4);
         */
        pad: function (data, blockSize) {
            // Shortcut
            var blockSizeBytes = blockSize * 4;

            // Count padding bytes
            var nPaddingBytes = blockSizeBytes - data.sigBytes % blockSizeBytes;

            // Create padding word
            var paddingWord = (nPaddingBytes << 24) | (nPaddingBytes << 16) | (nPaddingBytes << 8) | nPaddingBytes;

            // Create padding
            var paddingWords = [];
            for (var i = 0; i < nPaddingBytes; i += 4) {
                paddingWords.push(paddingWord);
            }
            var padding = WordArray.create(paddingWords, nPaddingBytes);

            // Add padding
            data.concat(padding);
        },

        /**
         * Unpads data that had been padded using the algorithm defined in PKCS #5/7.
         *
         * @param {WordArray} data The data to unpad.
         *
         * @static
         *
         * @example
         *
         *     CryptoJS.pad.Pkcs7.unpad(wordArray);
         */
        unpad: function (data) {
            // Get number of padding bytes from last byte
            var nPaddingBytes = data.words[(data.sigBytes - 1) >>> 2] & 0xff;

            // Remove padding
            data.sigBytes -= nPaddingBytes;
        }
    };

    /**
     * Abstract base block cipher template.
     *
     * @property {number} blockSize The number of 32-bit words this cipher operates on. Default: 4 (128 bits)
     */
    var BlockCipher = C_lib.BlockCipher = Cipher.extend({
        /**
         * Configuration options.
         *
         * @property {Mode} mode The block mode to use. Default: CBC
         * @property {Padding} padding The padding strategy to use. Default: Pkcs7
         */
        cfg: Cipher.cfg.extend({
            mode: CBC,
            padding: Pkcs7
        }),

        reset: function () {
            // Reset cipher
            Cipher.reset.call(this);

            // Shortcuts
            var cfg = this.cfg;
            var iv = cfg.iv;
            var mode = cfg.mode;

            // Reset block mode
            if (this._xformMode == this._ENC_XFORM_MODE) {
                var modeCreator = mode.createEncryptor;
            } else /* if (this._xformMode == this._DEC_XFORM_MODE) */ {
                var modeCreator = mode.createDecryptor;

                // Keep at least one block in the buffer for unpadding
                this._minBufferSize = 1;
            }
            this._mode = modeCreator.call(mode, this, iv && iv.words);
        },

        _doProcessBlock: function (words, offset) {
            this._mode.processBlock(words, offset);
        },

        _doFinalize: function () {
            // Shortcut
            var padding = this.cfg.padding;

            // Finalize
            if (this._xformMode == this._ENC_XFORM_MODE) {
                // Pad data
                padding.pad(this._data, this.blockSize);

                // Process final blocks
                var finalProcessedBlocks = this._process(!!'flush');
            } else /* if (this._xformMode == this._DEC_XFORM_MODE) */ {
                // Process final blocks
                var finalProcessedBlocks = this._process(!!'flush');

                // Unpad data
                padding.unpad(finalProcessedBlocks);
            }

            return finalProcessedBlocks;
        },

        blockSize: 128/32
    });

    /**
     * A collection of cipher parameters.
     *
     * @property {WordArray} ciphertext The raw ciphertext.
     * @property {WordArray} key The key to this ciphertext.
     * @property {WordArray} iv The IV used in the ciphering operation.
     * @property {WordArray} salt The salt used with a key derivation function.
     * @property {Cipher} algorithm The cipher algorithm.
     * @property {Mode} mode The block mode used in the ciphering operation.
     * @property {Padding} padding The padding scheme used in the ciphering operation.
     * @property {number} blockSize The block size of the cipher.
     * @property {Format} formatter The default formatting strategy to convert this cipher params object to a string.
     */
    var CipherParams = C_lib.CipherParams = Base.extend({
        /**
         * Initializes a newly created cipher params object.
         *
         * @param {Object} cipherParams An object with any of the possible cipher parameters.
         *
         * @example
         *
         *     var cipherParams = CryptoJS.lib.CipherParams.create({
         *         ciphertext: ciphertextWordArray,
         *         key: keyWordArray,
         *         iv: ivWordArray,
         *         salt: saltWordArray,
         *         algorithm: CryptoJS.algo.AES,
         *         mode: CryptoJS.mode.CBC,
         *         padding: CryptoJS.pad.PKCS7,
         *         blockSize: 4,
         *         formatter: CryptoJS.format.OpenSSL
         *     });
         */
        init: function (cipherParams) {
            this.mixIn(cipherParams);
        },

        /**
         * Converts this cipher params object to a string.
         *
         * @param {Format} formatter (Optional) The formatting strategy to use.
         *
         * @return {string} The stringified cipher params.
         *
         * @throws Error If neither the formatter nor the default formatter is set.
         *
         * @example
         *
         *     var string = cipherParams + '';
         *     var string = cipherParams.toString();
         *     var string = cipherParams.toString(CryptoJS.format.OpenSSL);
         */
        toString: function (formatter) {
            return (formatter || this.formatter).stringify(this);
        }
    });

    /**
     * Format namespace.
     */
    var C_format = C.format = {};

    /**
     * OpenSSL formatting strategy.
     */
    var OpenSSLFormatter = C_format.OpenSSL = {
        /**
         * Converts a cipher params object to an OpenSSL-compatible string.
         *
         * @param {CipherParams} cipherParams The cipher params object.
         *
         * @return {string} The OpenSSL-compatible string.
         *
         * @static
         *
         * @example
         *
         *     var openSSLString = CryptoJS.format.OpenSSL.stringify(cipherParams);
         */
        stringify: function (cipherParams) {
            // Shortcuts
            var ciphertext = cipherParams.ciphertext;
            var salt = cipherParams.salt;

            // Format
            if (salt) {
                var wordArray = WordArray.create([0x53616c74, 0x65645f5f]).concat(salt).concat(ciphertext);
            } else {
                var wordArray = ciphertext;
            }

            return wordArray.toString(Base64);
        },

        /**
         * Converts an OpenSSL-compatible string to a cipher params object.
         *
         * @param {string} openSSLStr The OpenSSL-compatible string.
         *
         * @return {CipherParams} The cipher params object.
         *
         * @static
         *
         * @example
         *
         *     var cipherParams = CryptoJS.format.OpenSSL.parse(openSSLString);
         */
        parse: function (openSSLStr) {
            // Parse base64
            var ciphertext = Base64.parse(openSSLStr);

            // Shortcut
            var ciphertextWords = ciphertext.words;

            // Test for salt
            if (ciphertextWords[0] == 0x53616c74 && ciphertextWords[1] == 0x65645f5f) {
                // Extract salt
                var salt = WordArray.create(ciphertextWords.slice(2, 4));

                // Remove salt from ciphertext
                ciphertextWords.splice(0, 4);
                ciphertext.sigBytes -= 16;
            }

            return CipherParams.create({ ciphertext: ciphertext, salt: salt });
        }
    };

    /**
     * A cipher wrapper that returns ciphertext as a serializable cipher params object.
     */
    var SerializableCipher = C_lib.SerializableCipher = Base.extend({
        /**
         * Configuration options.
         *
         * @property {Formatter} format The formatting strategy to convert cipher param objects to and from a string. Default: OpenSSL
         */
        cfg: Base.extend({
            format: OpenSSLFormatter
        }),

        /**
         * Encrypts a message.
         *
         * @param {Cipher} cipher The cipher algorithm to use.
         * @param {WordArray|string} message The message to encrypt.
         * @param {WordArray} key The key.
         * @param {Object} cfg (Optional) The configuration options to use for this operation.
         *
         * @return {CipherParams} A cipher params object.
         *
         * @static
         *
         * @example
         *
         *     var ciphertextParams = CryptoJS.lib.SerializableCipher.encrypt(CryptoJS.algo.AES, message, key);
         *     var ciphertextParams = CryptoJS.lib.SerializableCipher.encrypt(CryptoJS.algo.AES, message, key, { iv: iv });
         *     var ciphertextParams = CryptoJS.lib.SerializableCipher.encrypt(CryptoJS.algo.AES, message, key, { iv: iv, format: CryptoJS.format.OpenSSL });
         */
        encrypt: function (cipher, message, key, cfg) {
            // Apply config defaults
            cfg = this.cfg.extend(cfg);

            // Encrypt
            var encryptor = cipher.createEncryptor(key, cfg);
            var ciphertext = encryptor.finalize(message);

            // Shortcut
            var cipherCfg = encryptor.cfg;

            // Create and return serializable cipher params
            return CipherParams.create({
                ciphertext: ciphertext,
                key: key,
                iv: cipherCfg.iv,
                algorithm: cipher,
                mode: cipherCfg.mode,
                padding: cipherCfg.padding,
                blockSize: cipher.blockSize,
                formatter: cfg.format
            });
        },

        /**
         * Decrypts serialized ciphertext.
         *
         * @param {Cipher} cipher The cipher algorithm to use.
         * @param {CipherParams|string} ciphertext The ciphertext to decrypt.
         * @param {WordArray} key The key.
         * @param {Object} cfg (Optional) The configuration options to use for this operation.
         *
         * @return {WordArray} The plaintext.
         *
         * @static
         *
         * @example
         *
         *     var plaintext = CryptoJS.lib.SerializableCipher.decrypt(CryptoJS.algo.AES, formattedCiphertext, key, { iv: iv, format: CryptoJS.format.OpenSSL });
         *     var plaintext = CryptoJS.lib.SerializableCipher.decrypt(CryptoJS.algo.AES, ciphertextParams, key, { iv: iv, format: CryptoJS.format.OpenSSL });
         */
        decrypt: function (cipher, ciphertext, key, cfg) {
            // Apply config defaults
            cfg = this.cfg.extend(cfg);

            // Convert string to CipherParams
            ciphertext = this._parse(ciphertext, cfg.format);

            // Decrypt
            var plaintext = cipher.createDecryptor(key, cfg).finalize(ciphertext.ciphertext);

            return plaintext;
        },

        /**
         * Converts serialized ciphertext to CipherParams,
         * else assumed CipherParams already and returns ciphertext unchanged.
         *
         * @param {CipherParams|string} ciphertext The ciphertext.
         * @param {Formatter} format The formatting strategy to use to parse serialized ciphertext.
         *
         * @return {CipherParams} The unserialized ciphertext.
         *
         * @static
         *
         * @example
         *
         *     var ciphertextParams = CryptoJS.lib.SerializableCipher._parse(ciphertextStringOrParams, format);
         */
        _parse: function (ciphertext, format) {
            if (typeof ciphertext == 'string') {
                return format.parse(ciphertext, this);
            } else {
                return ciphertext;
            }
        }
    });

    /**
     * Key derivation function namespace.
     */
    var C_kdf = C.kdf = {};

    /**
     * OpenSSL key derivation function.
     */
    var OpenSSLKdf = C_kdf.OpenSSL = {
        /**
         * Derives a key and IV from a password.
         *
         * @param {string} password The password to derive from.
         * @param {number} keySize The size in words of the key to generate.
         * @param {number} ivSize The size in words of the IV to generate.
         * @param {WordArray|string} salt (Optional) A 64-bit salt to use. If omitted, a salt will be generated randomly.
         *
         * @return {CipherParams} A cipher params object with the key, IV, and salt.
         *
         * @static
         *
         * @example
         *
         *     var derivedParams = CryptoJS.kdf.OpenSSL.execute('Password', 256/32, 128/32);
         *     var derivedParams = CryptoJS.kdf.OpenSSL.execute('Password', 256/32, 128/32, 'saltsalt');
         */
        execute: function (password, keySize, ivSize, salt) {
            // Generate random salt
            if (!salt) {
                salt = WordArray.random(64/8);
            }

            // Derive key and IV
            var key = EvpKDF.create({ keySize: keySize + ivSize }).compute(password, salt);

            // Separate key and IV
            var iv = WordArray.create(key.words.slice(keySize), ivSize * 4);
            key.sigBytes = keySize * 4;

            // Return params
            return CipherParams.create({ key: key, iv: iv, salt: salt });
        }
    };

    /**
     * A serializable cipher wrapper that derives the key from a password,
     * and returns ciphertext as a serializable cipher params object.
     */
    var PasswordBasedCipher = C_lib.PasswordBasedCipher = SerializableCipher.extend({
        /**
         * Configuration options.
         *
         * @property {KDF} kdf The key derivation function to use to generate a key and IV from a password. Default: OpenSSL
         */
        cfg: SerializableCipher.cfg.extend({
            kdf: OpenSSLKdf
        }),

        /**
         * Encrypts a message using a password.
         *
         * @param {Cipher} cipher The cipher algorithm to use.
         * @param {WordArray|string} message The message to encrypt.
         * @param {string} password The password.
         * @param {Object} cfg (Optional) The configuration options to use for this operation.
         *
         * @return {CipherParams} A cipher params object.
         *
         * @static
         *
         * @example
         *
         *     var ciphertextParams = CryptoJS.lib.PasswordBasedCipher.encrypt(CryptoJS.algo.AES, message, 'password');
         *     var ciphertextParams = CryptoJS.lib.PasswordBasedCipher.encrypt(CryptoJS.algo.AES, message, 'password', { format: CryptoJS.format.OpenSSL });
         */
        encrypt: function (cipher, message, password, cfg) {
            // Apply config defaults
            cfg = this.cfg.extend(cfg);

            // Derive key and other params
            var derivedParams = cfg.kdf.execute(password, cipher.keySize, cipher.ivSize);

            // Add IV to config
            cfg.iv = derivedParams.iv;

            // Encrypt
            var ciphertext = SerializableCipher.encrypt.call(this, cipher, message, derivedParams.key, cfg);

            // Mix in derived params
            ciphertext.mixIn(derivedParams);

            return ciphertext;
        },

        /**
         * Decrypts serialized ciphertext using a password.
         *
         * @param {Cipher} cipher The cipher algorithm to use.
         * @param {CipherParams|string} ciphertext The ciphertext to decrypt.
         * @param {string} password The password.
         * @param {Object} cfg (Optional) The configuration options to use for this operation.
         *
         * @return {WordArray} The plaintext.
         *
         * @static
         *
         * @example
         *
         *     var plaintext = CryptoJS.lib.PasswordBasedCipher.decrypt(CryptoJS.algo.AES, formattedCiphertext, 'password', { format: CryptoJS.format.OpenSSL });
         *     var plaintext = CryptoJS.lib.PasswordBasedCipher.decrypt(CryptoJS.algo.AES, ciphertextParams, 'password', { format: CryptoJS.format.OpenSSL });
         */
        decrypt: function (cipher, ciphertext, password, cfg) {
            // Apply config defaults
            cfg = this.cfg.extend(cfg);

            // Convert string to CipherParams
            ciphertext = this._parse(ciphertext, cfg.format);

            // Derive key and other params
            var derivedParams = cfg.kdf.execute(password, cipher.keySize, cipher.ivSize, ciphertext.salt);

            // Add IV to config
            cfg.iv = derivedParams.iv;

            // Decrypt
            var plaintext = SerializableCipher.decrypt.call(this, cipher, ciphertext, derivedParams.key, cfg);

            return plaintext;
        }
    });
}());

(function () {
    // Shortcuts
    var C = CryptoJS;
    var C_lib = C.lib;
    var BlockCipher = C_lib.BlockCipher;
    var C_algo = C.algo;

    // Lookup tables
    var SBOX = [];
    var INV_SBOX = [];
    var SUB_MIX_0 = [];
    var SUB_MIX_1 = [];
    var SUB_MIX_2 = [];
    var SUB_MIX_3 = [];
    var INV_SUB_MIX_0 = [];
    var INV_SUB_MIX_1 = [];
    var INV_SUB_MIX_2 = [];
    var INV_SUB_MIX_3 = [];

    // Compute lookup tables
    (function () {
        // Compute double table
        var d = [];
        for (var i = 0; i < 256; i++) {
            if (i < 128) {
                d[i] = i << 1;
            } else {
                d[i] = (i << 1) ^ 0x11b;
            }
        }

        // Walk GF(2^8)
        var x = 0;
        var xi = 0;
        for (var i = 0; i < 256; i++) {
            // Compute sbox
            var sx = xi ^ (xi << 1) ^ (xi << 2) ^ (xi << 3) ^ (xi << 4);
            sx = (sx >>> 8) ^ (sx & 0xff) ^ 0x63;
            SBOX[x] = sx;
            INV_SBOX[sx] = x;

            // Compute multiplication
            var x2 = d[x];
            var x4 = d[x2];
            var x8 = d[x4];

            // Compute sub bytes, mix columns tables
            var t = (d[sx] * 0x101) ^ (sx * 0x1010100);
            SUB_MIX_0[x] = (t << 24) | (t >>> 8);
            SUB_MIX_1[x] = (t << 16) | (t >>> 16);
            SUB_MIX_2[x] = (t << 8)  | (t >>> 24);
            SUB_MIX_3[x] = t;

            // Compute inv sub bytes, inv mix columns tables
            var t = (x8 * 0x1010101) ^ (x4 * 0x10001) ^ (x2 * 0x101) ^ (x * 0x1010100);
            INV_SUB_MIX_0[sx] = (t << 24) | (t >>> 8);
            INV_SUB_MIX_1[sx] = (t << 16) | (t >>> 16);
            INV_SUB_MIX_2[sx] = (t << 8)  | (t >>> 24);
            INV_SUB_MIX_3[sx] = t;

            // Compute next counter
            if (!x) {
                x = xi = 1;
            } else {
                x = x2 ^ d[d[d[x8 ^ x2]]];
                xi ^= d[d[xi]];
            }
        }
    }());

    // Precomputed Rcon lookup
    var RCON = [0x00, 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80, 0x1b, 0x36];

    /**
     * AES block cipher algorithm.
     */
    var AES = C_algo.AES = BlockCipher.extend({
        _doReset: function () {
            // Shortcuts
            var key = this._key;
            var keyWords = key.words;
            var keySize = key.sigBytes / 4;

            // Compute number of rounds
            var nRounds = this._nRounds = keySize + 6

            // Compute number of key schedule rows
            var ksRows = (nRounds + 1) * 4;

            // Compute key schedule
            var keySchedule = this._keySchedule = [];
            for (var ksRow = 0; ksRow < ksRows; ksRow++) {
                if (ksRow < keySize) {
                    keySchedule[ksRow] = keyWords[ksRow];
                } else {
                    var t = keySchedule[ksRow - 1];

                    if (!(ksRow % keySize)) {
                        // Rot word
                        t = (t << 8) | (t >>> 24);

                        // Sub word
                        t = (SBOX[t >>> 24] << 24) | (SBOX[(t >>> 16) & 0xff] << 16) | (SBOX[(t >>> 8) & 0xff] << 8) | SBOX[t & 0xff];

                        // Mix Rcon
                        t ^= RCON[(ksRow / keySize) | 0] << 24;
                    } else if (keySize > 6 && ksRow % keySize == 4) {
                        // Sub word
                        t = (SBOX[t >>> 24] << 24) | (SBOX[(t >>> 16) & 0xff] << 16) | (SBOX[(t >>> 8) & 0xff] << 8) | SBOX[t & 0xff];
                    }

                    keySchedule[ksRow] = keySchedule[ksRow - keySize] ^ t;
                }
            }

            // Compute inv key schedule
            var invKeySchedule = this._invKeySchedule = [];
            for (var invKsRow = 0; invKsRow < ksRows; invKsRow++) {
                var ksRow = ksRows - invKsRow;

                if (invKsRow % 4) {
                    var t = keySchedule[ksRow];
                } else {
                    var t = keySchedule[ksRow - 4];
                }

                if (invKsRow < 4 || ksRow <= 4) {
                    invKeySchedule[invKsRow] = t;
                } else {
                    invKeySchedule[invKsRow] = INV_SUB_MIX_0[SBOX[t >>> 24]] ^ INV_SUB_MIX_1[SBOX[(t >>> 16) & 0xff]] ^
                                               INV_SUB_MIX_2[SBOX[(t >>> 8) & 0xff]] ^ INV_SUB_MIX_3[SBOX[t & 0xff]];
                }
            }
        },

        encryptBlock: function (M, offset) {
            this._doCryptBlock(M, offset, this._keySchedule, SUB_MIX_0, SUB_MIX_1, SUB_MIX_2, SUB_MIX_3, SBOX);
        },

        decryptBlock: function (M, offset) {
            // Swap 2nd and 4th rows
            var t = M[offset + 1];
            M[offset + 1] = M[offset + 3];
            M[offset + 3] = t;

            this._doCryptBlock(M, offset, this._invKeySchedule, INV_SUB_MIX_0, INV_SUB_MIX_1, INV_SUB_MIX_2, INV_SUB_MIX_3, INV_SBOX);

            // Inv swap 2nd and 4th rows
            var t = M[offset + 1];
            M[offset + 1] = M[offset + 3];
            M[offset + 3] = t;
        },

        _doCryptBlock: function (M, offset, keySchedule, SUB_MIX_0, SUB_MIX_1, SUB_MIX_2, SUB_MIX_3, SBOX) {
            // Shortcut
            var nRounds = this._nRounds;

            // Get input, add round key
            var s0 = M[offset]     ^ keySchedule[0];
            var s1 = M[offset + 1] ^ keySchedule[1];
            var s2 = M[offset + 2] ^ keySchedule[2];
            var s3 = M[offset + 3] ^ keySchedule[3];

            // Key schedule row counter
            var ksRow = 4;

            // Rounds
            for (var round = 1; round < nRounds; round++) {
                // Shift rows, sub bytes, mix columns, add round key
                var t0 = SUB_MIX_0[s0 >>> 24] ^ SUB_MIX_1[(s1 >>> 16) & 0xff] ^ SUB_MIX_2[(s2 >>> 8) & 0xff] ^ SUB_MIX_3[s3 & 0xff] ^ keySchedule[ksRow++];
                var t1 = SUB_MIX_0[s1 >>> 24] ^ SUB_MIX_1[(s2 >>> 16) & 0xff] ^ SUB_MIX_2[(s3 >>> 8) & 0xff] ^ SUB_MIX_3[s0 & 0xff] ^ keySchedule[ksRow++];
                var t2 = SUB_MIX_0[s2 >>> 24] ^ SUB_MIX_1[(s3 >>> 16) & 0xff] ^ SUB_MIX_2[(s0 >>> 8) & 0xff] ^ SUB_MIX_3[s1 & 0xff] ^ keySchedule[ksRow++];
                var t3 = SUB_MIX_0[s3 >>> 24] ^ SUB_MIX_1[(s0 >>> 16) & 0xff] ^ SUB_MIX_2[(s1 >>> 8) & 0xff] ^ SUB_MIX_3[s2 & 0xff] ^ keySchedule[ksRow++];

                // Update state
                s0 = t0;
                s1 = t1;
                s2 = t2;
                s3 = t3;
            }

            // Shift rows, sub bytes, add round key
            var t0 = ((SBOX[s0 >>> 24] << 24) | (SBOX[(s1 >>> 16) & 0xff] << 16) | (SBOX[(s2 >>> 8) & 0xff] << 8) | SBOX[s3 & 0xff]) ^ keySchedule[ksRow++];
            var t1 = ((SBOX[s1 >>> 24] << 24) | (SBOX[(s2 >>> 16) & 0xff] << 16) | (SBOX[(s3 >>> 8) & 0xff] << 8) | SBOX[s0 & 0xff]) ^ keySchedule[ksRow++];
            var t2 = ((SBOX[s2 >>> 24] << 24) | (SBOX[(s3 >>> 16) & 0xff] << 16) | (SBOX[(s0 >>> 8) & 0xff] << 8) | SBOX[s1 & 0xff]) ^ keySchedule[ksRow++];
            var t3 = ((SBOX[s3 >>> 24] << 24) | (SBOX[(s0 >>> 16) & 0xff] << 16) | (SBOX[(s1 >>> 8) & 0xff] << 8) | SBOX[s2 & 0xff]) ^ keySchedule[ksRow++];

            // Set output
            M[offset]     = t0;
            M[offset + 1] = t1;
            M[offset + 2] = t2;
            M[offset + 3] = t3;
        },

        keySize: 256/32
    });

    /**
     * Shortcut functions to the cipher's object interface.
     *
     * @example
     *
     *     var ciphertext = CryptoJS.AES.encrypt(message, key, cfg);
     *     var plaintext  = CryptoJS.AES.decrypt(ciphertext, key, cfg);
     */
    C.AES = BlockCipher._createHelper(AES);
}());

(function () {
    // Check if typed arrays are supported
    if (typeof ArrayBuffer === 'undefined') {
        return;
    }

    // Shortcuts
    var C = CryptoJS;
    var C_lib = C.lib;
    var WordArray = C_lib.WordArray;

    // Reference original init
    var superInit = WordArray.init;

    // Augment WordArray.init to handle typed arrays
    var subInit = WordArray.init = function (typedArray) {
        // Convert buffers to uint8
        if (typedArray instanceof ArrayBuffer) {
            typedArray = new Uint8Array(typedArray);
        }

        // Convert other array views to uint8
        else if (
            typedArray instanceof Int8Array ||
            (typeof Uint8ClampedArray !== 'undefined' && typedArray instanceof Uint8ClampedArray) ||
            typedArray instanceof Int16Array ||
            typedArray instanceof Uint16Array ||
            typedArray instanceof Int32Array ||
            typedArray instanceof Uint32Array ||
            (typeof Float32Array !== 'undefined' && typedArray instanceof Float32Array) ||
            (typeof Float64Array !== 'undefined' && typedArray instanceof Float64Array)
        ) {
            typedArray = new Uint8Array(typedArray.buffer, typedArray.byteOffset, typedArray.byteLength);
        }

        // Handle Uint8Array
        if (typedArray instanceof Uint8Array) {
            // Shortcut
            var typedArrayByteLength = typedArray.byteLength;

            // Extract bytes
            var words = [];
            for (var i = 0; i < typedArrayByteLength; i++) {
                words[i >>> 2] |= typedArray[i] << (24 - (i % 4) * 8);
            }

            // Initialize this word array
            superInit.call(this, words, typedArrayByteLength);
        } else {
            // Else call normal init
            superInit.apply(this, arguments);
        }
    };

    subInit.prototype = WordArray;
}());

var DomEvent = (function() {
	function DomEvent() {}

	DomEvent.addListener = function(target, event, listener) {
		if(target.addEventListener) {
			target.addEventListener(event, listener, false);
		} else {
			target.attachEvent('on'+event, function() { listener.apply(target, arguments); });
		}
	};

	DomEvent.removeListener = function(target, event, listener) {
		if(target.removeEventListener) {
			target.removeEventListener(event, listener, false);
		} else {
			target.detachEvent('on'+event, function() { listener.apply(target, arguments); });
		}
	};

	DomEvent.addMessageListener = function(target, listener) {
		DomEvent.addListener(target, 'message', listener);
	};

	DomEvent.removeMessageListener = function(target, listener) {
		DomEvent.removeListener(target, 'message', listener);
	};

	DomEvent.addUnloadListener = function(listener) {
		DomEvent.addListener(window, 'unload', listener);
	};

	return DomEvent;
})();

var msgpack = (function() {
	"use strict";

	var exports = {};

	exports.inspect = inspect;
	function inspect(buffer) {
		if (buffer === undefined)
			return "undefined";
		var view;
		var type;
		if ( buffer instanceof ArrayBuffer) {
			type = "ArrayBuffer";
			view = new DataView(buffer);
		} else if ( buffer instanceof DataView) {
			type = "DataView";
			view = buffer;
		}
		if (!view)
			return JSON.stringify(buffer);
		var bytes = [];
		for (var i = 0; i < buffer.byteLength; i++) {
			if (i > 20) {
				bytes.push("...");
				break;
			}
			var byte_ = view.getUint8(i).toString(16);
			if (byte_.length === 1)
				byte_ = "0" + byte_;
			bytes.push(byte_);
		}
		return "<" + type + " " + bytes.join(" ") + ">";
	}

	// Encode string as utf8 into dataview at offset
	exports.utf8Write = utf8Write;
	function utf8Write(view, offset, string) {
		var byteLength = view.byteLength;
		for (var i = 0, l = string.length; i < l; i++) {
			var codePoint = string.charCodeAt(i);

			// One byte of UTF-8
			if (codePoint < 0x80) {
				view.setUint8(offset++, codePoint >>> 0 & 0x7f | 0x00);
				continue;
			}

			// Two bytes of UTF-8
			if (codePoint < 0x800) {
				view.setUint8(offset++, codePoint >>> 6 & 0x1f | 0xc0);
				view.setUint8(offset++, codePoint >>> 0 & 0x3f | 0x80);
				continue;
			}

			// Three bytes of UTF-8.
			if (codePoint < 0x10000) {
				view.setUint8(offset++, codePoint >>> 12 & 0x0f | 0xe0);
				view.setUint8(offset++, codePoint >>> 6 & 0x3f | 0x80);
				view.setUint8(offset++, codePoint >>> 0 & 0x3f | 0x80);
				continue;
			}

			// Four bytes of UTF-8
			if (codePoint < 0x110000) {
				view.setUint8(offset++, codePoint >>> 18 & 0x07 | 0xf0);
				view.setUint8(offset++, codePoint >>> 12 & 0x3f | 0x80);
				view.setUint8(offset++, codePoint >>> 6 & 0x3f | 0x80);
				view.setUint8(offset++, codePoint >>> 0 & 0x3f | 0x80);
				continue;
			}
			throw new Error("bad codepoint " + codePoint);
		}
	}


	exports.utf8Read = utf8Read;
	function utf8Read(view, offset, length) {
		var string = "";
		for (var i = offset, end = offset + length; i < end; i++) {
			var byte_ = view.getUint8(i);
			// One byte character
			if ((byte_ & 0x80) === 0x00) {
				string += String.fromCharCode(byte_);
				continue;
			}
			// Two byte character
			if ((byte_ & 0xe0) === 0xc0) {
				string += String.fromCharCode(((byte_ & 0x0f) << 6) | (view.getUint8(++i) & 0x3f));
				continue;
			}
			// Three byte character
			if ((byte_ & 0xf0) === 0xe0) {
				string += String.fromCharCode(((byte_ & 0x0f) << 12) | ((view.getUint8(++i) & 0x3f) << 6) | ((view.getUint8(++i) & 0x3f) << 0));
				continue;
			}
			// Four byte character
			if ((byte_ & 0xf8) === 0xf0) {
				string += String.fromCharCode(((byte_ & 0x07) << 18) | ((view.getUint8(++i) & 0x3f) << 12) | ((view.getUint8(++i) & 0x3f) << 6) | ((view.getUint8(++i) & 0x3f) << 0));
				continue;
			}
			throw new Error("Invalid byte " + byte_.toString(16));
		}
		return string;
	}


	exports.utf8ByteCount = utf8ByteCount;
	function utf8ByteCount(string) {
		var count = 0;
		for (var i = 0, l = string.length; i < l; i++) {
			var codePoint = string.charCodeAt(i);
			if (codePoint < 0x80) {
				count += 1;
				continue;
			}
			if (codePoint < 0x800) {
				count += 2;
				continue;
			}
			if (codePoint < 0x10000) {
				count += 3;
				continue;
			}
			if (codePoint < 0x110000) {
				count += 4;
				continue;
			}
			throw new Error("bad codepoint " + codePoint);
		}
		return count;
	}


	exports.encode = function(value, sparse) {
		var size = sizeof(value, sparse);
		if(size == 0)
			return undefined;
		var buffer = new ArrayBuffer(size);
		var view = new DataView(buffer);
		encode(value, view, 0, sparse);
		return buffer;
	};

	exports.decode = decode;

	var SH_L_32 = (1 << 16) * (1 << 16), SH_R_32 = 1 / SH_L_32;
	function getInt64(view, offset) {
		offset = offset || 0;
		return view.getInt32(offset) * SH_L_32 + view.getUint32(offset + 4);
	}

	function getUint64(view, offset) {
		offset = offset || 0;
		return view.getUint32(offset) * SH_L_32 + view.getUint32(offset + 4);
	}

	function setInt64(view, offset, val) {
		if (val < 0x8000000000000000) {
			view.setInt32(offset, Math.floor(val * SH_R_32));
			view.setInt32(offset + 4, val & -1);
		} else {
			view.setUint32(offset, 0x7fffffff);
			view.setUint32(offset + 4, 0x7fffffff);
		}
	}

	function setUint64(view, offset, val) {
		if (val < 0x10000000000000000) {
			view.setUint32(offset, Math.floor(val * SH_R_32));
			view.setInt32(offset + 4, val & -1);
		} else {
			view.setUint32(offset, 0xffffffff);
			view.setUint32(offset + 4, 0xffffffff);
		}
	}

// https://gist.github.com/frsyuki/5432559 - v5 spec
//
// I've used one extension point from `fixext 1` to store `undefined`. On the wire this
// should translate to exactly 0xd40000
//
// +--------+--------+--------+
// |  0xd4  |  0x00  |  0x00  |
// +--------+--------+--------+
//    ^ fixext |        ^ value part unused (fixed to be 0)
//             ^ indicates undefined value
//

	function Decoder(view, offset) {
		this.offset = offset || 0;
		this.view = view;
	}


	Decoder.prototype.map = function(length) {
		var value = {};
		for (var i = 0; i < length; i++) {
			var key = this.parse();
			value[key] = this.parse();
		}
		return value;
	};

	Decoder.prototype.bin = Decoder.prototype.buf = function(length) {
		var value = new ArrayBuffer(length);
		(new Uint8Array(value)).set(new Uint8Array(this.view.buffer, this.offset, length), 0);
		this.offset += length;
		return value;
	};

	Decoder.prototype.str = function(length) {
		var value = utf8Read(this.view, this.offset, length);
		this.offset += length;
		return value;
	};

	Decoder.prototype.array = function(length) {
		var value = new Array(length);
		for (var i = 0; i < length; i++) {
			value[i] = this.parse();
		}
		return value;
	};

	Decoder.prototype.ext = function(length) {
		var value = {};
		// Get the type byte
		value['type'] = this.view.getInt8(this.offset);
		this.offset++;
		// Get the data array (length)
		value['data'] = this.buf(length);
		this.offset += length;
		return value;
	};

	Decoder.prototype.parse = function() {
		var type = this.view.getUint8(this.offset);
		var value, length;

		// Positive FixInt - 0xxxxxxx
		if ((type & 0x80) === 0x00) {
			this.offset++;
			return type;
		}

		// FixMap - 1000xxxx
		if ((type & 0xf0) === 0x80) {
			length = type & 0x0f;
			this.offset++;
			return this.map(length);
		}

		// FixArray - 1001xxxx
		if ((type & 0xf0) === 0x90) {
			length = type & 0x0f;
			this.offset++;
			return this.array(length);
		}

		// FixStr - 101xxxxx
		if ((type & 0xe0) === 0xa0) {
			length = type & 0x1f;
			this.offset++;
			return this.str(length);
		}

		// Negative FixInt - 111xxxxx
		if ((type & 0xe0) === 0xe0) {
			value = this.view.getInt8(this.offset);
			this.offset++;
			return value;
		}

		switch (type) {

			// nil
			case 0xc0:
				this.offset++;
				return null;

			// 0xc1 never used - use for undefined (NON-STANDARD)
			case 0xc1:
				this.offset++;
				return undefined;

			// false
			case 0xc2:
				this.offset++;
				return false;

			// true
			case 0xc3:
				this.offset++;
				return true;

			// bin 8
			case 0xc4:
				length = this.view.getUint8(this.offset + 1);
				this.offset += 2;
				return this.bin(length);

			// bin 16
			case 0xc5:
				length = this.view.getUint16(this.offset + 1);
				this.offset += 3;
				return this.bin(length);

			// bin 32
			case 0xc6:
				length = this.view.getUint32(this.offset + 1);
				this.offset += 5;
				return this.bin(length);

			// ext 8
			case 0xc7:
				length = this.view.getUint8(this.offset + 1);
				this.offset += 2;
				return this.ext(length);

			// ext 16
			case 0xc8:
				length = this.view.getUint16(this.offset + 1);
				this.offset += 3;
				return this.ext(length);

			// ext 32
			case 0xc9:
				length = this.view.getUint32(this.offset + 1);
				this.offset += 5;
				return this.ext(length);

			// float 32
			case 0xca:
				value = this.view.getFloat32(this.offset + 1);
				this.offset += 5;
				return value;

			// float 64
			case 0xcb:
				value = this.view.getFloat64(this.offset + 1);
				this.offset += 9;
				return value;

			// uint8
			case 0xcc:
				value = this.view.getUint8(this.offset + 1);
				this.offset += 2;
				return value;

			// uint 16
			case 0xcd:
				value = this.view.getUint16(this.offset + 1);
				this.offset += 3;
				return value;

			// uint 32
			case 0xce:
				value = this.view.getUint32(this.offset + 1);
				this.offset += 5;
				return value;

			// uint 64
			case 0xcf:
				value = getUint64(this.view, this.offset + 1);
				this.offset += 9;
				return value;

			// int 8
			case 0xd0:
				value = this.view.getInt8(this.offset + 1);
				this.offset += 2;
				return value;

			// int 16
			case 0xd1:
				value = this.view.getInt16(this.offset + 1);
				this.offset += 3;
				return value;

			// int 32
			case 0xd2:
				value = this.view.getInt32(this.offset + 1);
				this.offset += 5;
				return value;

			// int 64
			case 0xd3:
				value = getInt64(this.view, this.offset + 1);
				this.offset += 9;
				return value;

			// fixext 1
			case 0xd4:
				length = 1;
				this.offset++;
				return this.ext(length);

			// fixext 2
			case 0xd5:
				length = 2;
				this.offset++;
				return this.ext(length);

			// fixext 4
			case 0xd6:
				length = 4;
				this.offset++;
				return this.ext(length);

			// fixext 8
			case 0xd7:
				length = 8;
				this.offset++;
				return this.ext(length);

			// fixext 16
			case 0xd8:
				length = 16;
				this.offset++;
				return this.ext(length);

			// str8
			case 0xd9:
				length = this.view.getUint8(this.offset + 1);
				this.offset += 2;
				return this.str(length);

			// str 16
			case 0xda:
				length = this.view.getUint16(this.offset + 1);
				this.offset += 3;
				return this.str(length);

			// str 32
			case 0xdb:
				length = this.view.getUint32(this.offset + 1);
				this.offset += 5;
				return this.str(length);

			// array 16
			case 0xdc:
				length = this.view.getUint16(this.offset + 1);
				this.offset += 3;
				return this.array(length);

			// array 32
			case 0xdd:
				length = this.view.getUint32(this.offset + 1);
				this.offset += 5;
				return this.array(length);

			// map 16
			case 0xde:
				length = this.view.getUint16(this.offset + 1);
				this.offset += 3;
				return this.map(length);

			// map 32
			case 0xdf:
				length = this.view.getUint32(this.offset + 1);
				this.offset += 5;
				return this.map(length);
		}
		throw new Error("Unknown type 0x" + type.toString(16));
	};

	function decode(buffer) {
		var view = new DataView(buffer);
		var decoder = new Decoder(view);
		var value = decoder.parse();
		if (decoder.offset !== buffer.byteLength)
			throw new Error((buffer.byteLength - decoder.offset) + " trailing bytes");
		return value;
	}

	function encodeableKeys(value, sparse) {
		return Utils.keysArray(value, true).filter(function (e) {
			var val = value[e], type = typeof(val);
			return (!sparse || (val !== undefined && val !== null)) && ('function' !== type || !!val.toJSON);
		})
	}

	function encode(value, view, offset, sparse) {
		var type = typeof value;

		// Strings Bytes
		// There are four string types: fixstr/str8/str16/str32
		if (type === "string") {
			var length = utf8ByteCount(value);

			// fixstr
			if (length < 0x20) {
				view.setUint8(offset, length | 0xa0);
				utf8Write(view, offset + 1, value);
				return 1 + length;
			}

			// str8
			if (length < 0x100) {
				view.setUint8(offset, 0xd9);
				view.setUint8(offset + 1, length);
				utf8Write(view, offset + 2, value);
				return 2 + length;
			}

			// str16
			if (length < 0x10000) {
				view.setUint8(offset, 0xda);
				view.setUint16(offset + 1, length);
				utf8Write(view, offset + 3, value);
				return 3 + length;
			}
			// str32
			if (length < 0x100000000) {
				view.setUint8(offset, 0xdb);
				view.setUint32(offset + 1, length);
				utf8Write(view, offset + 5, value);
				return 5 + length;
			}
		}

		// There are three bin types: bin8/bin16/bin32
		if (value instanceof ArrayBuffer) {
			var length = value.byteLength;

			// bin8
			if (length < 0x100) {
				view.setUint8(offset, 0xc4);
				view.setUint8(offset + 1, length);
				(new Uint8Array(view.buffer)).set(new Uint8Array(value), offset + 2);
				return 2 + length;
			}

			// bin16
			if (length < 0x10000) {
				view.setUint8(offset, 0xc5);
				view.setUint16(offset + 1, length);
				(new Uint8Array(view.buffer)).set(new Uint8Array(value), offset + 3);
				return 3 + length;
			}

			// bin 32
			if (length < 0x100000000) {
				view.setUint8(offset, 0xc6);
				view.setUint32(offset + 1, length);
				(new Uint8Array(view.buffer)).set(new Uint8Array(value), offset + 5);
				return 5 + length;
			}
		}

		if (type === "number") {

			// Floating Point
			// NOTE: We're always using float64
			if (Math.floor(value) !== value) {
				view.setUint8(offset, 0xcb);
				view.setFloat64(offset + 1, value);
				return 9;
			}

			// Integers
			if (value >= 0) {
				// positive fixnum
				if (value < 0x80) {
					view.setUint8(offset, value);
					return 1;
				}
				// uint 8
				if (value < 0x100) {
					view.setUint8(offset, 0xcc);
					view.setUint8(offset + 1, value);
					return 2;
				}
				// uint 16
				if (value < 0x10000) {
					view.setUint8(offset, 0xcd);
					view.setUint16(offset + 1, value);
					return 3;
				}
				// uint 32
				if (value < 0x100000000) {
					view.setUint8(offset, 0xce);
					view.setUint32(offset + 1, value);
					return 5;
				}
				// uint 64
				if (value < 0x10000000000000000) {
					view.setUint8(offset, 0xcf);
					setUint64(view, offset + 1, value);
					return 9;
				}
				throw new Error("Number too big 0x" + value.toString(16));
			}

			// negative fixnum
			if (value >= -0x20) {
				view.setInt8(offset, value);
				return 1;
			}
			// int 8
			if (value >= -0x80) {
				view.setUint8(offset, 0xd0);
				view.setInt8(offset + 1, value);
				return 2;
			}
			// int 16
			if (value >= -0x8000) {
				view.setUint8(offset, 0xd1);
				view.setInt16(offset + 1, value);
				return 3;
			}
			// int 32
			if (value >= -0x80000000) {
				view.setUint8(offset, 0xd2);
				view.setInt32(offset + 1, value);
				return 5;
			}
			// int 64
			if (value >= -0x8000000000000000) {
				view.setUint8(offset, 0xd3);
				setInt64(view, offset + 1, value);
				return 9;
			}
			throw new Error("Number too small -0x" + (-value).toString(16).substr(1));
		}

		// undefined - use d4 (NON-STANDARD)
		if (type === "undefined") {
			if(sparse) return 0;
			view.setUint8(offset, 0xd4);
			view.setUint8(offset + 1, 0x00);
			view.setUint8(offset + 2, 0x00);
			return 3;
		}

		// null
		if (value === null) {
			if(sparse) return 0;
			view.setUint8(offset, 0xc0);
			return 1;
		}

		// Boolean
		if (type === "boolean") {
			view.setUint8(offset, value ? 0xc3 : 0xc2);
			return 1;
		}

		if('function' === typeof value.toJSON)
			return encode(value.toJSON(), view, offset, sparse);

		// Container Types
		if (type === "object") {
			var length, size = 0;
			var isArray = Array.isArray(value);

			if (isArray) {
				length = value.length;
			} else {
				var keys = encodeableKeys(value, sparse);
				length = keys.length;
			}

			var size;
			if (length < 0x10) {
				view.setUint8(offset, length | ( isArray ? 0x90 : 0x80));
				size = 1;
			} else if (length < 0x10000) {
				view.setUint8(offset, isArray ? 0xdc : 0xde);
				view.setUint16(offset + 1, length);
				size = 3;
			} else if (length < 0x100000000) {
				view.setUint8(offset, isArray ? 0xdd : 0xdf);
				view.setUint32(offset + 1, length);
				size = 5;
			}

			if (isArray) {
				for (var i = 0; i < length; i++) {
					size += encode(value[i], view, offset + size, sparse);
				}
			} else {
				for (var i = 0; i < length; i++) {
					var key = keys[i];
					size += encode(key, view, offset + size);
					size += encode(value[key], view, offset + size, sparse);
				}
			}

			return size;
		}
		if(type === "function")
			return 0;

		throw new Error("Unknown type " + type);
	}

	function sizeof(value, sparse) {
		var type = typeof value;

		// fixstr or str8 or str16 or str32
		if (type === "string") {
			var length = utf8ByteCount(value);
			if (length < 0x20) {
				return 1 + length;
			}
			if (length < 0x100) {
				return 2 + length;
			}
			if (length < 0x10000) {
				return 3 + length;
			}
			if (length < 0x100000000) {
				return 5 + length;
			}
		}

		// bin8 or bin16 or bin32
		if (value instanceof ArrayBuffer) {
			var length = value.byteLength;
			if (length < 0x100) {
				return 2 + length;
			}
			if (length < 0x10000) {
				return 3 + length;
			}
			if (length < 0x100000000) {
				return 5 + length;
			}
		}

		if (type === "number") {
			// Floating Point (32 bits)
			// double
			if (Math.floor(value) !== value)
				return 9;

			// Integers
			if (value >= 0) {
				// positive fixint
				if (value < 0x80)
					return 1;
				// uint 8
				if (value < 0x100)
					return 2;
				// uint 16
				if (value < 0x10000)
					return 3;
				// uint 32
				if (value < 0x100000000)
					return 5;
				// uint 64
				if (value < 0x10000000000000000)
					return 9;
				// Too big
				throw new Error("Number too big 0x" + value.toString(16));
			}
			// negative fixint
			if (value >= -0x20)
				return 1;
			// int 8
			if (value >= -0x80)
				return 2;
			// int 16
			if (value >= -0x8000)
				return 3;
			// int 32
			if (value >= -0x80000000)
				return 5;
			// int 64
			if (value >= -0x8000000000000000)
				return 9;
			// Too small
			throw new Error("Number too small -0x" + value.toString(16).substr(1));
		}

		// Boolean
		if (type === "boolean") return 1;

		// undefined, null
		if (value === null) return sparse ? 0 : 1;
		if (value === undefined) return sparse ? 0 : 3;

		if('function' === typeof value.toJSON)
			return sizeof(value.toJSON(), sparse);

		// Container Types
		if (type === "object") {
			var length, size = 0;
			if (Array.isArray(value)) {
				length = value.length;
				for (var i = 0; i < length; i++) {
					size += sizeof(value[i], sparse);
				}
			} else {
				var keys = encodeableKeys(value, sparse)
				length = keys.length;
				for (var i = 0; i < length; i++) {
					var key = keys[i];
					size += sizeof(key) + sizeof(value[key], sparse);
				}
			}
			if (length < 0x10) {
				return 1 + size;
			}
			if (length < 0x10000) {
				return 3 + size;
			}
			if (length < 0x100000000) {
				return 5 + size;
			}
			throw new Error("Array or object too long 0x" + length.toString(16));
		}
		if(type === "function")
			return 0;

		throw new Error("Unknown type " + type);
	}

	return exports;
})();

if(typeof window !== 'object') {
	console.log("Warning: this distribution of Ably is intended for browsers. On nodejs, please use the 'ably' package on npm");
}

function allowComet() {
	/* xhr requests from local files are unreliable in some browsers, such as Chrome 65 and higher -- see eg
	 * https://stackoverflow.com/questions/49256429/chrome-65-unable-to-make-post-requests-from-local-files-to-flask
	 * So if websockets are supported, then just forget about comet transports and use that */
	var loc = window.location;
	return (!window.WebSocket || !loc || !loc.origin || loc.origin.indexOf("http") > -1);
}

var Platform = {
	libver: 'js-web-',
	logTimestamps: true,
	noUpgrade: navigator && navigator.userAgent.toString().match(/MSIE\s8\.0/),
	binaryType: 'arraybuffer',
	WebSocket: window.WebSocket || window.MozWebSocket,
	xhrSupported: window.XMLHttpRequest && 'withCredentials' in new XMLHttpRequest(),
	jsonpSupported: typeof(document) !== 'undefined',
	allowComet: allowComet(),
	streamingSupported: true,
	useProtocolHeartbeats: true,
	createHmac: null,
	msgpack: msgpack,
	supportsBinary: !!window.TextDecoder,
	preferBinary: false,
	ArrayBuffer: window.ArrayBuffer,
	atob: window.atob,
	nextTick: function(f) { setTimeout(f, 0); },
	addEventListener: window.addEventListener,
	inspect: JSON.stringify,
	getRandomValues: (function(crypto) {
		if (crypto === undefined) {
			return undefined;
		}
		return function(arr, callback) {
			crypto.getRandomValues(arr);
			callback(null);
		};
	})(window.crypto || window.msCrypto) // mscrypto for IE11
};

var Crypto = (function() {
	var DEFAULT_ALGORITHM = 'aes';
	var DEFAULT_KEYLENGTH = 256; // bits
	var DEFAULT_MODE = 'cbc';
	var DEFAULT_BLOCKLENGTH = 16; // bytes
	var DEFAULT_BLOCKLENGTH_WORDS = 4; // 32-bit words
	var UINT32_SUP = 0x100000000;
	var INT32_SUP = 0x80000000;
	var WordArray = CryptoJS.lib.WordArray;

	/**
	 * Internal: generate an array of secure random words corresponding to the given length of bytes
	 * @param bytes
	 * @param callback
	 */
	var generateRandom;
	if(typeof Uint32Array !== 'undefined' && Platform.getRandomValues) {
		var blockRandomArray = new Uint32Array(DEFAULT_BLOCKLENGTH_WORDS);
		generateRandom = function(bytes, callback) {
			var words = bytes / 4, nativeArray = (words == DEFAULT_BLOCKLENGTH_WORDS) ? blockRandomArray : new Uint32Array(words);
			Platform.getRandomValues(nativeArray, function(err) {
				callback(err, BufferUtils.toWordArray(nativeArray))
			});
		};
	} else {
		generateRandom = function(bytes, callback) {
			Logger.logAction(Logger.LOG_MAJOR, 'Ably.Crypto.generateRandom()', 'Warning: the browser you are using does not support secure cryptographically secure randomness generation; falling back to insecure Math.random()');
			var words = bytes / 4, array = new Array(words);
			for(var i = 0; i < words; i++) {
				/* cryptojs wordarrays use signed ints. When WordArray.create is fed a
				* Uint32Array unsigned are converted to signed automatically, but when
				* fed a normal array they aren't, so need to do so ourselves by
				* subtracting INT32_SUP */
				array[i] = Math.floor(Math.random() * UINT32_SUP) - INT32_SUP;
			}

			callback(null, WordArray.create(array));
		};
	}

	/**
	 * Internal: calculate the padded length of a given plaintext
	 * using PKCS5.
	 * @param plaintextLength
	 * @return
	 */
	function getPaddedLength(plaintextLength) {
		return (plaintextLength + DEFAULT_BLOCKLENGTH) & -DEFAULT_BLOCKLENGTH;
	}

	/**
	 * Internal: checks that the cipherParams are a valid combination. Currently
	 * just checks that the calculated keyLength is a valid one for aes-cbc
	 */
	function validateCipherParams(params) {
		if(params.algorithm === 'aes' && params.mode === 'cbc') {
			if(params.keyLength === 128 || params.keyLength === 256) {
				return;
			}
			throw new Error('Unsupported key length ' + params.keyLength + ' for aes-cbc encryption. Encryption key must be 128 or 256 bits (16 or 32 ASCII characters)');
		}
	}

	function normaliseBase64(string) {
		/* url-safe base64 strings use _ and - instread of / and + */
		return string.replace('_', '/').replace('-', '+');
	}

	/**
	 * Internal: a block containing zeros
	 */
	var emptyBlock = WordArray.create([0,0,0,0]);

	/**
	 * Internal: obtain the pkcs5 padding string for a given padded length;
	 */
	var pkcs5Padding = [
		WordArray.create([0x10101010,0x10101010,0x10101010,0x10101010], 16),
		WordArray.create([0x01000000], 1),
		WordArray.create([0x02020000], 2),
		WordArray.create([0x03030300], 3),
		WordArray.create([0x04040404], 4),
		WordArray.create([0x05050505,0x05000000], 5),
		WordArray.create([0x06060606,0x06060000], 6),
		WordArray.create([0x07070707,0x07070700], 7),
		WordArray.create([0x08080808,0x08080808], 8),
		WordArray.create([0x09090909,0x09090909,0x09000000], 9),
		WordArray.create([0x0a0a0a0a,0x0a0a0a0a,0x0a0a0000], 10),
		WordArray.create([0x0b0b0b0b,0x0b0b0b0b,0x0b0b0b00], 11),
		WordArray.create([0x0c0c0c0c,0x0c0c0c0c,0x0c0c0c0c], 12),
		WordArray.create([0x0d0d0d0d,0x0d0d0d0d,0x0d0d0d0d,0x0d000000], 13),
		WordArray.create([0x0e0e0e0e,0x0e0e0e0e,0x0e0e0e0e,0x0e0e0000], 14),
		WordArray.create([0x0f0f0f0f,0x0f0f0f0f,0x0f0f0f0f,0x0f0f0f0f], 15),
		WordArray.create([0x10101010,0x10101010,0x10101010,0x10101010], 16)
	];

	/**
	 * Utility classes and interfaces for message payload encryption.
	 *
	 * This class supports AES/CBC/PKCS5 with a default keylength of 128 bits
	 * but supporting other keylengths. Other algorithms and chaining modes are
	 * not supported directly, but supportable by extending/implementing the base
	 * classes and interfaces here.
	 *
	 * Secure random data for creation of Initialization Vectors (IVs) and keys
	 * is obtained from window.crypto.getRandomValues if available, or from
	 * Math.random() if not. Clients who do not want to depend on Math.random()
	 * should polyfill window.crypto.getRandomValues with a library that seeds
	 * a PRNG with real entropy.
	 *
	 * Each message payload is encrypted with an IV in CBC mode, and the IV is
	 * concatenated with the resulting raw ciphertext to construct the "ciphertext"
	 * data passed to the recipient.
	 */
	function Crypto() {}

	/**
	 * A class encapsulating the client-specifiable parameters for
	 * the cipher.
	 *
	 * algorithm is the name of the algorithm in the default system provider,
	 * or the lower-cased version of it; eg "aes" or "AES".
	 *
	 * Clients are recommended to not call this directly, but instead to use the
	 * Crypto.getDefaultParams helper, which will fill in any fields not supplied
	 * with default values and validation the result.
	 */
	function CipherParams() {
		this.algorithm = null;
		this.keyLength = null;
		this.mode = null;
		this.key = null;
	}
	Crypto.CipherParams = CipherParams;

	/**
	 * Obtain a complete CipherParams instance from the provided params, filling
	 * in any not provided with default values, calculating a keyLength from
	 * the supplied key, and validating the result.
	 * @param params an object containing at a minimum a `key` key with value the
	 * key, as either a binary (ArrayBuffer, Array, WordArray) or a
	 * base64-encoded string. May optionally also contain: algorithm (defaults to
	 * AES), mode (defaults to 'cbc')
	 */
	Crypto.getDefaultParams = function(params) {
		var key;
		/* Backward compatibility */
		if((typeof(params) === 'function') || (typeof(params) === 'string')) {
			Logger.deprecated('Crypto.getDefaultParams(key, callback)', 'Crypto.getDefaultParams({key: key})');
			if(typeof(params) === 'function') {
				Crypto.generateRandomKey(function(key) {
					params(null, Crypto.getDefaultParams({key: key}));
				})
			} else if(typeof arguments[1] === 'function') {
				arguments[1](null, Crypto.getDefaultParams({key: params}));
			} else {
				throw new Error('Invalid arguments for Crypto.getDefaultParams');
			}
			return;
		}

		if(!params.key) {
			throw new Error('Crypto.getDefaultParams: a key is required');
		}

		if (typeof(params.key) === 'string') {
			key = CryptoJS.enc.Base64.parse(normaliseBase64(params.key));
		} else {
			key = BufferUtils.toWordArray(params.key); // Expect key to be an Array, ArrayBuffer, or WordArray at this point
		}

		var cipherParams = new CipherParams();
		cipherParams.key = key;
		cipherParams.algorithm = params.algorithm || DEFAULT_ALGORITHM;
		cipherParams.keyLength = key.words.length * (4 * 8);
		cipherParams.mode = params.mode || DEFAULT_MODE;

		if(params.keyLength && params.keyLength !== cipherParams.keyLength) {
			throw new Error('Crypto.getDefaultParams: a keyLength of ' + params.keyLength + ' was specified, but the key actually has length ' + cipherParams.keyLength);
		}

		validateCipherParams(cipherParams);
		return cipherParams;
	};

	/**
	 * Generate a random encryption key from the supplied keylength (or the
	 * default keyLength if none supplied) as a CryptoJS WordArray
	 * @param keyLength (optional) the required keyLength in bits
	 * @param callback (err, key)
	 */
	Crypto.generateRandomKey = function(keyLength, callback) {
		if(arguments.length == 1 && typeof(keyLength) == 'function') {
			callback = keyLength;
			keyLength = undefined;
		}
		generateRandom((keyLength || DEFAULT_KEYLENGTH) / 8, callback);
	};

	/**
	 * Internal; get a ChannelCipher instance based on the given cipherParams
	 * @param params either a CipherParams instance or some subset of its
	 * fields that includes a key
	 */
	Crypto.getCipher = function(params) {
		var cipherParams = (params instanceof CipherParams) ?
		                   params :
		                   Crypto.getDefaultParams(params);

		return {cipherParams: cipherParams, cipher: new CBCCipher(cipherParams, DEFAULT_BLOCKLENGTH_WORDS, params.iv)};
	};

	function CBCCipher(params, blockLengthWords, iv) {
		this.algorithm = params.algorithm + '-' + String(params.keyLength) + '-' + params.mode;
		this.cjsAlgorithm = params.algorithm.toUpperCase().replace(/-\d+$/, '');
		this.key = BufferUtils.toWordArray(params.key);
		if(iv) {
			this.iv = BufferUtils.toWordArray(iv).clone();
		}
		this.blockLengthWords = blockLengthWords;
	}

	CBCCipher.prototype.encrypt = function(plaintext, callback) {
		Logger.logAction(Logger.LOG_MICRO, 'CBCCipher.encrypt()', '');
		plaintext = BufferUtils.toWordArray(plaintext);
		//console.log('encrypt: plaintext:');
		//console.log(CryptoJS.enc.Hex.stringify(plaintext));
		var plaintextLength = plaintext.sigBytes,
			paddedLength = getPaddedLength(plaintextLength),
			self = this;

		var then = function() {
			self.getIv(function(err, iv) {
				if (err) {
					callback(err);
					return;
				}
				var cipherOut = self.encryptCipher.process(plaintext.concat(pkcs5Padding[paddedLength - plaintextLength]));
				var ciphertext = iv.concat(cipherOut);
				//console.log('encrypt: ciphertext:');
				//console.log(CryptoJS.enc.Hex.stringify(ciphertext));
				callback(null, ciphertext);
			});
		};

		if (!this.encryptCipher) {
			if(this.iv) {
				this.encryptCipher = CryptoJS.algo[this.cjsAlgorithm].createEncryptor(this.key, { iv: this.iv });
				then();
			} else {
				generateRandom(DEFAULT_BLOCKLENGTH, function(err, iv) {
					if (err) {
						callback(err);
						return;
					}
					self.encryptCipher = CryptoJS.algo[self.cjsAlgorithm].createEncryptor(self.key, { iv: iv });
					self.iv = iv;
					then();
				});
			}
		} else {
			then();
		}
	};

	CBCCipher.prototype.decrypt = function(ciphertext) {
		Logger.logAction(Logger.LOG_MICRO, 'CBCCipher.decrypt()', '');
		ciphertext = BufferUtils.toWordArray(ciphertext);
		//console.log('decrypt: ciphertext:');
		//console.log(CryptoJS.enc.Hex.stringify(ciphertext));
		var blockLengthWords = this.blockLengthWords,
			ciphertextWords = ciphertext.words,
			iv = WordArray.create(ciphertextWords.slice(0, blockLengthWords)),
			ciphertextBody = WordArray.create(ciphertextWords.slice(blockLengthWords));

		var decryptCipher = CryptoJS.algo[this.cjsAlgorithm].createDecryptor(this.key, { iv: iv });
		var plaintext = decryptCipher.process(ciphertextBody);
		var epilogue = decryptCipher.finalize();
		decryptCipher.reset();
		if(epilogue && epilogue.sigBytes) plaintext.concat(epilogue);
		//console.log('decrypt: plaintext:');
		//console.log(CryptoJS.enc.Hex.stringify(plaintext));
		return plaintext;
	};

	CBCCipher.prototype.getIv = function(callback) {
		if(this.iv) {
			var iv = this.iv;
			this.iv = null;
			callback(null, iv);
			return;
		}

		/* Since the iv for a new block is the ciphertext of the last, this
		* sets a new iv (= aes(randomBlock XOR lastCipherText)) as well as
		* returning it */
		var self = this;
		generateRandom(DEFAULT_BLOCKLENGTH, function(err, randomBlock) {
			if (err) {
				callback(err);
				return;
			}
			callback(null, self.encryptCipher.process(randomBlock));
		});
	};

	return Crypto;
})();

var WebStorage = (function() {
	var sessionSupported,
		localSupported,
		test = 'ablyjs-storage-test';

	/* Even just accessing the session/localStorage object can throw a
	 * security exception in some circumstances with some browsers. In
	 * others, calling setItem will throw. So have to check in this
	 * somewhat roundabout way. (If unsupported or no window object,
	 * will throw on accessing a property of undefined) */
	try {
		window.sessionStorage.setItem(test, test);
		window.sessionStorage.removeItem(test);
		sessionSupported = true;
	} catch(e) {
		sessionSupported = false;
	}

	try {
		window.localStorage.setItem(test, test);
		window.localStorage.removeItem(test);
		localSupported = true;
	} catch(e) {
		localSupported = false;
	}

	function WebStorage() {}

	function storageInterface(session) {
		return session ? window.sessionStorage : window.localStorage;
	}

	function set(name, value, ttl, session) {
		var wrappedValue = {value: value};
		if(ttl) {
			wrappedValue.expires = Utils.now() + ttl;
		}
		return storageInterface(session).setItem(name, JSON.stringify(wrappedValue));
	}

	function get(name, session) {
		var rawItem = storageInterface(session).getItem(name);
		if(!rawItem) return null;
		var wrappedValue = JSON.parse(rawItem);
		if(wrappedValue.expires && (wrappedValue.expires < Utils.now())) {
			storageInterface(session).removeItem(name);
			return null;
		}
		return wrappedValue.value;
	}

	function remove(name, session) {
		return storageInterface(session).removeItem(name);
	}

	if(localSupported) {
		WebStorage.set    = function(name, value, ttl) { return set(name, value, ttl, false); };
		WebStorage.get    = function(name) { return get(name, false); };
		WebStorage.remove = function(name) { return remove(name, false); };
	}

	if(sessionSupported) {
		WebStorage.setSession    = function(name, value, ttl) { return set(name, value, ttl, true); };
		WebStorage.getSession    = function(name) { return get(name, true); };
		WebStorage.removeSession = function(name) { return remove(name, true); };
	}

	return WebStorage;
})();

var Defaults = {
	internetUpUrl: 'https://internet-up.ably-realtime.com/is-the-internet-up.txt',
	jsonpInternetUpUrl: 'https://internet-up.ably-realtime.com/is-the-internet-up-0-9.js',
	/* Order matters here: the base transport is the leftmost one in the
	 * intersection of baseTransportOrder and the transports clientOption that's
	 * supported.  This is not quite the same as the preference order -- e.g.
	 * xhr_polling is preferred to jsonp, but for browsers that support it we want
	 * the base transport to be xhr_polling, not jsonp */
	defaultTransports: ['xhr_polling', 'xhr_streaming', 'jsonp', 'web_socket'],
	baseTransportOrder: ['xhr_polling', 'xhr_streaming', 'jsonp', 'web_socket'],
	transportPreferenceOrder: ['jsonp', 'xhr_polling', 'xhr_streaming', 'web_socket'],
	upgradeTransports: ['xhr_streaming', 'web_socket'],
	minified: !(function _(){}).name
};

/* If using IE8, don't attempt to upgrade from xhr_polling to xhr_streaming -
* while it can do streaming, the low max http-connections-per-host limit means
* that the polling transport is crippled during the upgrade process. So just
* leave it at the base transport */
if(Platform.noUpgrade) {
	Defaults.upgradeTransports = [];
}


var BufferUtils = (function() {
	var WordArray = CryptoJS.lib.WordArray;
	var ArrayBuffer = Platform.ArrayBuffer;
	var atob = Platform.atob;

	function isWordArray(ob) { return ob !== null && ob !== undefined && ob.sigBytes !== undefined; }
	function isArrayBuffer(ob) { return ob !== null && ob !== undefined && ob.constructor === ArrayBuffer; }

	// https://gist.githubusercontent.com/jonleighton/958841/raw/f200e30dfe95212c0165ccf1ae000ca51e9de803/gistfile1.js
	function arrayBufferToBase64(ArrayBuffer) {
		var base64    = ''
		var encodings = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'

		var bytes         = new Uint8Array(ArrayBuffer)
		var byteLength    = bytes.byteLength
		var byteRemainder = byteLength % 3
		var mainLength    = byteLength - byteRemainder

		var a, b, c, d
		var chunk

		// Main loop deals with bytes in chunks of 3
		for (var i = 0; i < mainLength; i = i + 3) {
			// Combine the three bytes into a single integer
			chunk = (bytes[i] << 16) | (bytes[i + 1] << 8) | bytes[i + 2]

			// Use bitmasks to extract 6-bit segments from the triplet
			a = (chunk & 16515072) >> 18 // 16515072 = (2^6 - 1) << 18
			b = (chunk & 258048)   >> 12 // 258048   = (2^6 - 1) << 12
			c = (chunk & 4032)     >>  6 // 4032     = (2^6 - 1) << 6
			d = chunk & 63               // 63       = 2^6 - 1

			// Convert the raw binary segments to the appropriate ASCII encoding
			base64 += encodings[a] + encodings[b] + encodings[c] + encodings[d]
		}

		// Deal with the remaining bytes and padding
		if (byteRemainder == 1) {
			chunk = bytes[mainLength]

			a = (chunk & 252) >> 2 // 252 = (2^6 - 1) << 2

			// Set the 4 least significant bits to zero
			b = (chunk & 3)   << 4 // 3   = 2^2 - 1

			base64 += encodings[a] + encodings[b] + '=='
		} else if (byteRemainder == 2) {
			chunk = (bytes[mainLength] << 8) | bytes[mainLength + 1]

			a = (chunk & 64512) >> 10 // 64512 = (2^6 - 1) << 10
			b = (chunk & 1008)  >>  4 // 1008  = (2^6 - 1) << 4

			// Set the 2 least significant bits to zero
			c = (chunk & 15)    <<  2 // 15    = 2^4 - 1

			base64 += encodings[a] + encodings[b] + encodings[c] + '='
		}

		return base64
	}

	function base64ToArrayBuffer(base64) {
		var binary_string =  atob(base64);
		var len = binary_string.length;
		var bytes = new Uint8Array( len );
		for (var i = 0; i < len; i++)        {
			var ascii = binary_string.charCodeAt(i);
			bytes[i] = ascii;
		}
		return bytes.buffer;
	}

	function BufferUtils() {}

	BufferUtils.isBuffer = function(buf) { return isArrayBuffer(buf) || isWordArray(buf); };

	BufferUtils.toArrayBuffer = function(buf) {
		if(!ArrayBuffer)
			throw new Error("Can't convert to ArrayBuffer: ArrayBuffer not supported");

		if(isArrayBuffer(buf))
			return buf;

		if(isWordArray(buf)) {
			/* Backported from unreleased CryptoJS
			* https://code.google.com/p/crypto-js/source/browse/branches/3.x/src/lib-typedarrays.js?r=661 */
			var arrayBuffer = new ArrayBuffer(buf.sigBytes);
			var uint8View = new Uint8Array(arrayBuffer);

			for (var i = 0; i < buf.sigBytes; i++) {
				uint8View[i] = (buf.words[i >>> 2] >>> (24 - (i % 4) * 8)) & 0xff;
			}

			return arrayBuffer;
		};

		throw new Error("BufferUtils.toArrayBuffer expected a buffer");
	};

	BufferUtils.toWordArray = function(buf) {
		return isWordArray(buf) ? buf : WordArray.create(buf);
	};

	BufferUtils.base64Encode = function(buf) {
		if(isArrayBuffer(buf))
			return arrayBufferToBase64(buf);
		if(isWordArray(buf))
			return CryptoJS.enc.Base64.stringify(buf);
	};

	BufferUtils.base64Decode = function(str) {
		if(ArrayBuffer && atob)
			return base64ToArrayBuffer(str);
		return CryptoJS.enc.Base64.parse(str);
	};

	BufferUtils.hexEncode = function(buf) {
		if(isArrayBuffer(buf)) buf = WordArray.create(buf);
		return CryptoJS.enc.Hex.stringify(buf);
	};

	BufferUtils.utf8Encode = function(string) {
		return CryptoJS.enc.Utf8.parse(string);
	};

	BufferUtils.utf8Decode = function(buf) {
		if(isArrayBuffer(buf))
			buf = BufferUtils.toWordArray(buf) // CryptoJS only works with WordArrays
		if(isWordArray(buf))
			return CryptoJS.enc.Utf8.stringify(buf);
		throw new Error("Expected input of utf8Decode to be a buffer or CryptoJS WordArray");
	};

	BufferUtils.bufferCompare = function(buf1, buf2) {
		if(!buf1) return -1;
		if(!buf2) return 1;
		buf1 = BufferUtils.toWordArray(buf1);
		buf2 = BufferUtils.toWordArray(buf2);
		buf1.clamp(); buf2.clamp();

		var cmp = buf1.sigBytes - buf2.sigBytes;
		if(cmp != 0) return cmp;
		buf1 = buf1.words; buf2 = buf2.words;
		for(var i = 0; i < buf1.length; i++) {
			cmp = buf1[i] - buf2[i];
			if(cmp != 0) return cmp;
		}
		return 0;
	};

	return BufferUtils;
})();

var Http = (function() {
	var noop = function() {};

	function Http() {}

	function shouldFallback(err) {
		var statusCode = err.statusCode;
		/* 400 + no code = a generic xhr onerror. Browser doesn't give us enough
		 * detail to know whether it's fallback-fixable, but it may be (eg if a
		 * network issue), so try just in case */
		return (statusCode === 408 && !err.code) ||
			(statusCode === 400 && !err.code)      ||
			(statusCode >= 500 && statusCode <= 504);
	}

	/**
	 * Perform an HTTP GET request for a given path against prime and fallback Ably hosts
	 * @param rest
	 * @param path the full path of the POST request
	 * @param headers optional hash of headers
	 * @param params optional hash of params
	 * @param callback (err, response)
	 */
	Http.get = function(rest, path, headers, params, callback) {
		callback = callback || noop;
		var uri = (typeof(path) == 'function') ? path : function(host) { return rest.baseUri(host) + path; };
		var binary = (headers && headers.accept != 'application/json');

		var hosts, connection = rest.connection;
		if(connection && connection.state == 'connected')
			hosts = [connection.connectionManager.host];
		else
			hosts = Defaults.getHosts(rest.options);

		/* if there is only one host do it */
		if(hosts.length == 1) {
			Http.getUri(rest, uri(hosts[0]), headers, params, callback);
			return;
		}

		/* so host is an array with preferred host plus at least one fallback */
		var tryAHost = function(candidateHosts) {
			Http.getUri(rest, uri(candidateHosts.shift()), headers, params, function(err) {
				if(err && shouldFallback(err) && candidateHosts.length) {
					/* use a fallback host if available */
					tryAHost(candidateHosts);
					return;
				}
				callback.apply(null, arguments);
			});
		}
		tryAHost(hosts);
	};

	/**
	 * Perform an HTTP GET request for a given resolved URI
	 * @param rest
	 * @param the full path of the POST request
	 * @param headers optional hash of headers
	 * @param params optional hash of params
	 * @param callback (err, response)
	 */
	Http.getUri = function(rest, uri, headers, params, callback) {
		Http.Request(rest, uri, headers, params, null, callback || noop);
	};

	/**
	 * Perform an HTTP POST request
	 * @param rest
	 * @param the full path of the POST request
	 * @param headers optional hash of headers
	 * @param body object or buffer containing request body
	 * @param params optional hash of params
	 * @param callback (err, response)
	 */
	Http.post = function(rest, path, headers, body, params, callback) {
		callback = callback || noop;
		var uri = (typeof(path) == 'function') ? path : function(host) { return rest.baseUri(host) + path; };
		var binary = (headers && headers.accept != 'application/json');

		var hosts, connection = rest.connection;
		if(connection && connection.state == 'connected')
			hosts = [connection.connectionManager.host];
		else
			hosts = Defaults.getHosts(rest.options);

		/* if there is only one host do it */
		if(hosts.length == 1) {
			Http.postUri(rest, uri(hosts[0]), headers, body, params, callback);
			return;
		}

		/* hosts is an array with preferred host plus at least one fallback */
		var tryAHost = function(candidateHosts) {
			Http.postUri(rest, uri(candidateHosts.shift()), headers, body, params, function(err) {
				if(err && shouldFallback(err) && candidateHosts.length) {
					tryAHost(candidateHosts);
					return;
				}
				callback.apply(null, arguments);
			});
		};
		tryAHost(hosts);
	};

	/**
	 * Perform an HTTP POST request for a given resolved URI
	 * @param rest
	 * @param the full path of the POST request
	 * @param headers optional hash of headers
	 * @param body object or buffer containing request body
	 * @param params optional hash of params
	 * @param callback (err, response)
	 */
	Http.postUri = function(rest, uri, headers, body, params, callback) {
		Http.Request(rest, uri, headers, params, body, callback || noop);
	};

	Http.supportsAuthHeaders = false;
	Http.supportsLinkHeaders = false;
	return Http;
})();

/*
 Copyright (c) 2008 Fred Palmer fred.palmer_at_gmail.com

 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 */
var Base64 = (function() {
	function StringBuffer()
	{
		this.buffer = [];
	}

	StringBuffer.prototype.append = function append(string)
	{
		this.buffer.push(string);
		return this;
	};

	StringBuffer.prototype.toString = function toString()
	{
		return this.buffer.join("");
	};

	var Base64 =
	{
		codex : "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",

		encode : function (input)
		{
			var output = new StringBuffer();
			var codex = Base64.codex;

			var enumerator = new Utf8EncodeEnumerator(input);
			while (enumerator.moveNext())
			{
				var chr1 = enumerator.current;

				enumerator.moveNext();
				var chr2 = enumerator.current;

				enumerator.moveNext();
				var chr3 = enumerator.current;

				var enc1 = chr1 >> 2;
				var enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
				var enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
				var enc4 = chr3 & 63;

				if (isNaN(chr2))
				{
					enc3 = enc4 = 64;
				}
				else if (isNaN(chr3))
				{
					enc4 = 64;
				}

				output.append(codex.charAt(enc1) + codex.charAt(enc2) + codex.charAt(enc3) + codex.charAt(enc4));
			}

			return output.toString();
		},

		decode : function (input)
		{
			var output = new StringBuffer();

			var enumerator = new Base64DecodeEnumerator(input);
			while (enumerator.moveNext())
			{
				var charCode = enumerator.current;

				if (charCode < 128)
					output.append(String.fromCharCode(charCode));
				else if ((charCode > 191) && (charCode < 224))
				{
					enumerator.moveNext();
					var charCode2 = enumerator.current;

					output.append(String.fromCharCode(((charCode & 31) << 6) | (charCode2 & 63)));
				}
				else
				{
					enumerator.moveNext();
					var charCode2 = enumerator.current;

					enumerator.moveNext();
					var charCode3 = enumerator.current;

					output.append(String.fromCharCode(((charCode & 15) << 12) | ((charCode2 & 63) << 6) | (charCode3 & 63)));
				}
			}

			return output.toString();
		}
	};

	function Utf8EncodeEnumerator(input)
	{
		this._input = input;
		this._index = -1;
		this._buffer = [];
	}

	Utf8EncodeEnumerator.prototype =
	{
		current: Number.NaN,

		moveNext: function()
		{
			if (this._buffer.length > 0)
			{
				this.current = this._buffer.shift();
				return true;
			}
			else if (this._index >= (this._input.length - 1))
			{
				this.current = Number.NaN;
				return false;
			}
			else
			{
				var charCode = this._input.charCodeAt(++this._index);

				// "\r\n" -> "\n"
				//
				if ((charCode == 13) && (this._input.charCodeAt(this._index + 1) == 10))
				{
					charCode = 10;
					this._index += 2;
				}

				if (charCode < 128)
				{
					this.current = charCode;
				}
				else if ((charCode > 127) && (charCode < 2048))
				{
					this.current = (charCode >> 6) | 192;
					this._buffer.push((charCode & 63) | 128);
				}
				else
				{
					this.current = (charCode >> 12) | 224;
					this._buffer.push(((charCode >> 6) & 63) | 128);
					this._buffer.push((charCode & 63) | 128);
				}

				return true;
			}
		}
	};

	function Base64DecodeEnumerator(input)
	{
		this._input = input;
		this._index = -1;
		this._buffer = [];
	}

	Base64DecodeEnumerator.prototype =
	{
		current: 64,

		moveNext: function()
		{
			if (this._buffer.length > 0)
			{
				this.current = this._buffer.shift();
				return true;
			}
			else if (this._index >= (this._input.length - 1))
			{
				this.current = 64;
				return false;
			}
			else
			{
				var enc1 = Base64.codex.indexOf(this._input.charAt(++this._index));
				var enc2 = Base64.codex.indexOf(this._input.charAt(++this._index));
				var enc3 = Base64.codex.indexOf(this._input.charAt(++this._index));
				var enc4 = Base64.codex.indexOf(this._input.charAt(++this._index));

				var chr1 = (enc1 << 2) | (enc2 >> 4);
				var chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
				var chr3 = ((enc3 & 3) << 6) | enc4;

				this.current = chr1;

				if (enc3 != 64)
					this._buffer.push(chr2);

				if (enc4 != 64)
					this._buffer.push(chr3);

				return true;
			}
		}
	};

	return Base64;
})();

Defaults.ENVIRONMENT              = '';
Defaults.REST_HOST                = 'rest.ably.io';
Defaults.REALTIME_HOST            = 'realtime.ably.io';
Defaults.FALLBACK_HOSTS           = ['A.ably-realtime.com', 'B.ably-realtime.com', 'C.ably-realtime.com', 'D.ably-realtime.com', 'E.ably-realtime.com'];
Defaults.PORT                     = 80;
Defaults.TLS_PORT                 = 443;
Defaults.TIMEOUTS = {
	/* Documented as options params: */
	disconnectedRetryTimeout   : 15000,
	suspendedRetryTimeout      : 30000,
	httpRequestTimeout         : 15000,
	channelRetryTimeout        : 15000,
	/* Not documented: */
	connectionStateTtl         : 120000,
	realtimeRequestTimeout     : 10000,
	recvTimeout                : 90000,
	preferenceConnectTimeout   : 6000,
	parallelUpgradeDelay       : 6000
};
Defaults.httpMaxRetryCount = 3;

Defaults.version          = '1.0.18';
Defaults.libstring        = Platform.libver + Defaults.version;
Defaults.apiVersion       = '1.0';

Defaults.getHost = function(options, host, ws) {
	if(ws)
		host = ((host == options.restHost) && options.realtimeHost) || host || options.realtimeHost;
	else
		host = host || options.restHost;

	return host;
};

Defaults.getPort = function(options, tls) {
	return (tls || options.tls) ? options.tlsPort : options.port;
};

Defaults.getHttpScheme = function(options) {
	return options.tls ? 'https://' : 'http://';
};

Defaults.getHosts = function(options) {
	var hosts = [options.restHost],
		fallbackHosts = options.fallbackHosts,
		httpMaxRetryCount = typeof(options.httpMaxRetryCount) !== 'undefined' ? options.httpMaxRetryCount : Defaults.httpMaxRetryCount;

	if(fallbackHosts) {
		hosts = hosts.concat(Utils.arrChooseN(fallbackHosts, httpMaxRetryCount));
	}
	return hosts;
};

Defaults.normaliseOptions = function(options) {
	/* Deprecated options */
	if(options.host) {
		Logger.deprecated('host', 'restHost');
		options.restHost = options.host;
	}
	if(options.wsHost) {
		Logger.deprecated('wsHost', 'realtimeHost');
		options.realtimeHost = options.wsHost;
	}
	if(options.queueEvents) {
		Logger.deprecated('queueEvents', 'queueMessages');
		options.queueMessages = options.queueEvents;
	}

	if(options.recover === true) {
		Logger.deprecated('{recover: true}', '{recover: function(lastConnectionDetails, cb) { cb(true); }}');
		options.recover = function(lastConnectionDetails, cb) { cb(true); };
	}

	if(typeof options.recover === 'function' && options.closeOnUnload === true) {
		Logger.logAction(Logger.LOG_ERROR, 'Defaults.normaliseOptions', 'closeOnUnload was true and a session recovery function was set - these are mutually exclusive, so unsetting the latter');
		options.recover = null;
	}

	if(!('closeOnUnload' in options)) {
		/* Have closeOnUnload default to true unless we have any indication that
		 * the user may want to recover the connection */
		options.closeOnUnload = !options.recover;
	}

	if(options.transports && Utils.arrIn(options.transports, 'xhr')) {
		Logger.deprecated('transports: ["xhr"]', 'transports: ["xhr_streaming"]');
		Utils.arrDeleteValue(options.transports, 'xhr');
		options.transports.push('xhr_streaming');
	}

	if(!('queueMessages' in options))
		options.queueMessages = true;

	var production = false;
	if(options.restHost) {
		options.realtimeHost = options.realtimeHost || options.restHost;
	} else {
		var environment = (options.environment && String(options.environment).toLowerCase()) || Defaults.ENVIRONMENT;
		production = !environment || (environment === 'production');
		options.restHost = production ? Defaults.REST_HOST : environment + '-' + Defaults.REST_HOST;
		options.realtimeHost = production ? Defaults.REALTIME_HOST : environment + '-' + Defaults.REALTIME_HOST;
	}
	options.fallbackHosts = (production || options.fallbackHostsUseDefault) ? Defaults.FALLBACK_HOSTS : options.fallbackHosts;
	options.port = options.port || Defaults.PORT;
	options.tlsPort = options.tlsPort || Defaults.TLS_PORT;
	if(!('tls' in options)) options.tls = true;

	/* Allow values passed in options to override default timeouts */
	options.timeouts = {};
	for(var prop in Defaults.TIMEOUTS) {
		options.timeouts[prop] = options[prop] || Defaults.TIMEOUTS[prop];
	};

	if('useBinaryProtocol' in options) {
		options.useBinaryProtocol = Platform.supportsBinary && options.useBinaryProtocol;
	} else {
		options.useBinaryProtocol = Platform.preferBinary;
	}

	return options;
};

var EventEmitter = (function() {

	/* public constructor */
	function EventEmitter() {
		this.any = [];
		this.events = {};
		this.anyOnce = [];
		this.eventsOnce = {};
	}

	/* Call the listener, catch any exceptions and log, but continue operation*/
	function callListener(eventThis, listener, args) {
		try {
			listener.apply(eventThis, args);
		} catch(e) {
			Logger.logAction(Logger.LOG_ERROR, 'EventEmitter.emit()', 'Unexpected listener exception: ' + e + '; stack = ' + (e && e.stack));
		}
	}

	/**
	 * Remove listeners that match listener
	 * @param targetListeners is an array of listener arrays or event objects with arrays of listeners
	 * @param listener the listener callback to remove
	 * @param eventFilter (optional) event name instructing the function to only remove listeners for the specified event
	 */
	function removeListener(targetListeners, listener, eventFilter) {
		var listeners, idx, eventName, targetListenersIndex;

		for (targetListenersIndex = 0; targetListenersIndex < targetListeners.length; targetListenersIndex++) {
			listeners = targetListeners[targetListenersIndex];
			if (eventFilter) { listeners = listeners[eventFilter]; }

			if (Utils.isArray(listeners)) {
				while ((idx = Utils.arrIndexOf(listeners, listener)) !== -1) {
					listeners.splice(idx, 1);
				}
				/* If events object has an event name key with no listeners then
				   remove the key to stop the list growing indefinitely */
				if (eventFilter && (listeners.length === 0)) {
					delete targetListeners[targetListenersIndex][eventFilter];
				}
			} else if (Utils.isObject(listeners)) {
				/* events */
				for (eventName in listeners) {
					if (listeners.hasOwnProperty(eventName) && Utils.isArray(listeners[eventName])) {
						removeListener([listeners], listener, eventName);
					}
				}
			}
		}
	}

	/**
	 * Add an event listener
	 * @param event (optional) the name of the event to listen to
	 *        if not supplied, all events trigger a call to the listener
	 * @param listener the listener to be called
	 */
	EventEmitter.prototype.on = function(event, listener) {
		if(arguments.length == 1 && typeof(event) == 'function') {
			this.any.push(event);
		} else if(Utils.isEmptyArg(event)) {
			this.any.push(listener);
		} else if(Utils.isArray(event)) {
			var self = this;
			Utils.arrForEach(event, function(ev) {
				self.on(ev, listener);
			});
		} else {
			var listeners = (this.events[event] || (this.events[event] = []));
			listeners.push(listener);
		}
	};

	/**
	 * Remove one or more event listeners
	 * @param event (optional) the name of the event whose listener
	 *        is to be removed. If not supplied, the listener is
	 *        treated as an 'any' listener
	 * @param listener (optional) the listener to remove. If not
	 *        supplied, all listeners are removed.
	 */
	EventEmitter.prototype.off = function(event, listener) {
		if(arguments.length == 0 || (Utils.isEmptyArg(event) && Utils.isEmptyArg(listener))) {
			this.any = [];
			this.events = {};
			this.anyOnce = [];
			this.eventsOnce = {};
			return;
		}
		if(arguments.length == 1) {
			if(typeof(event) == 'function') {
				/* we take this to be the listener and treat the event as "any" .. */
				listener = event;
				event = null;
			}
			/* ... or we take event to be the actual event name and listener to be all */
		}

		if(listener && Utils.isEmptyArg(event)) {
			removeListener([this.any, this.events, this.anyOnce, this.eventsOnce], listener);
			return;
		}

		if(Utils.isArray(event)) {
			var self = this;
			Utils.arrForEach(event, function(ev) {
				self.off(ev, listener);
			});
		}

		/* "normal" case where event is an actual event */
		if(listener) {
			removeListener([this.events, this.eventsOnce], listener, event);
		} else {
			delete this.events[event];
			delete this.eventsOnce[event];
		}
	};

	/**
	 * Get the array of listeners for a given event; excludes once events
	 * @param event (optional) the name of the event, or none for 'any'
	 * @return array of events, or null if none
	 */
	EventEmitter.prototype.listeners = function(event) {
		if(event) {
			var listeners = (this.events[event] || []);
			if(this.eventsOnce[event])
				Array.prototype.push.apply(listeners, this.eventsOnce[event]);
			return listeners.length ? listeners : null;
		}
		return this.any.length ? this.any : null;
	};

	/**
	 * Emit an event
	 * @param event the event name
	 * @param args the arguments to pass to the listener
	 */
	EventEmitter.prototype.emit = function(event  /* , args... */) {
		var args = Array.prototype.slice.call(arguments, 1);
		var eventThis = {event:event};
		var listeners = [];

		if(this.anyOnce.length) {
			Array.prototype.push.apply(listeners, this.anyOnce);
			this.anyOnce = [];
		}
		if(this.any.length) {
			Array.prototype.push.apply(listeners, this.any);
		}
		var eventsOnceListeners = this.eventsOnce[event];
		if(eventsOnceListeners) {
			Array.prototype.push.apply(listeners, eventsOnceListeners);
			delete this.eventsOnce[event];
		}
		var eventsListeners = this.events[event];
		if(eventsListeners) {
			Array.prototype.push.apply(listeners, eventsListeners);
		}

		Utils.arrForEach(listeners, function(listener) {
			callListener(eventThis, listener, args);
		});
	};

	/**
	 * Listen for a single occurrence of an event
	 * @param event the name of the event to listen to
	 * @param listener the listener to be called
	 */
	EventEmitter.prototype.once = function(event, listener) {
		if(arguments.length == 1 && typeof(event) == 'function') {
			this.anyOnce.push(event);
		} else if(Utils.isEmptyArg(event)) {
			this.anyOnce.push(listener);
		} else if(Utils.isArray(event)){
			throw("Arrays of events can only be used with on(), not once()");
		} else {
			var listeners = (this.eventsOnce[event] || (this.eventsOnce[event] = []));
			listeners.push(listener);
		}
	};

	/**
	 * Private API
	 *
	 * Listen for a single occurrence of a state event and fire immediately if currentState matches targetState
	 * @param targetState the name of the state event to listen to
	 * @param currentState the name of the current state of this object
	 * @param listener the listener to be called
	 */
	EventEmitter.prototype.whenState = function(targetState, currentState, listener /* ...listenerArgs */) {
		var eventThis = {event:targetState},
				listenerArgs = Array.prototype.slice.call(arguments, 3);

		if((typeof(targetState) !== 'string') || (typeof(currentState) !== 'string'))
			throw("whenState requires a valid event String argument");
		if (typeof(listener) !== 'function')
			throw("whenState requires a valid listener argument");

		if(targetState === currentState) {
			callListener(eventThis, listener, listenerArgs);
		} else {
			this.once(targetState, listener);
		}
	}

	return EventEmitter;
})();

var Logger = (function() {
	var consoleLogger;

	/* Can't just check for console && console.log; fails in IE <=9 */
	if((typeof window === 'undefined') /* node */ ||
		 (window.console && window.console.log && (typeof window.console.log.apply === 'function')) /* sensible browsers */) {
		consoleLogger = function() { console.log.apply(console, arguments); };
	} else if(window.console && window.console.log) {
		/* IE <= 9 with the console open -- console.log does not
		 * inherit from Function, so has no apply method */
		consoleLogger = function() { Function.prototype.apply.call(console.log, console, arguments); };
	} else {
		/* IE <= 9 when dev tools are closed - window.console not even defined */
		consoleLogger = function() {};
	}

	function pad(str, three) {
		return ('000' + str).slice(-2-(three || 0));
	}

	var LOG_NONE  = 0,
	LOG_ERROR = 1,
	LOG_MAJOR = 2,
	LOG_MINOR = 3,
	LOG_MICRO = 4;

	var LOG_DEFAULT = LOG_ERROR,
	LOG_DEBUG   = LOG_MICRO;

	var logLevel = LOG_DEFAULT;
	var logHandler = Platform.logTimestamps ?
		function(msg) {
			var time = new Date();
			consoleLogger(pad(time.getHours()) + ':' + pad(time.getMinutes()) + ':' + pad(time.getSeconds()) + '.' + pad(time.getMilliseconds(), true) + ' ' + msg);
		} : consoleLogger;

	/* public constructor */
	function Logger(args) {}

	/* public constants */
	Logger.LOG_NONE    = LOG_NONE,
	Logger.LOG_ERROR   = LOG_ERROR,
	Logger.LOG_MAJOR   = LOG_MAJOR,
	Logger.LOG_MINOR   = LOG_MINOR,
	Logger.LOG_MICRO   = LOG_MICRO;

	Logger.LOG_DEFAULT = LOG_DEFAULT,
	Logger.LOG_DEBUG   = LOG_DEBUG;

	/* public static functions */
	Logger.logAction = function(level, action, message) {
		if (Logger.shouldLog(level)) {
			logHandler('Ably: ' + action + ': ' + message);
		}
	};

	Logger.deprecated = function(original, replacement) {
		if (Logger.shouldLog(LOG_ERROR)) {
			logHandler("Ably: Deprecation warning - '" + original + "' is deprecated and will be removed from a future version. Please use '" + replacement + "' instead.");
		}
	}

	/* Where a logging operation is expensive, such as serialisation of data, use shouldLog will prevent
	   the object being serialised if the log level will not output the message */
	Logger.shouldLog = function(level) {
		return level <= logLevel;
	};

	Logger.setLog = function(level, handler) {
		if(level !== undefined) logLevel = level;
		if(handler !== undefined) logHandler = handler;
	};

	return Logger;
})();

var Utils = (function() {
	function Utils() {}

	/*
	 * Add a set of properties to a target object
	 * target: the target object
	 * props:  an object whose enumerable properties are
	 *         added, by reference only
	 */
	Utils.mixin = function(target, src) {
		if(src) {
			var hasOwnProperty = src.hasOwnProperty;
			for(var key in src) {
				if(!hasOwnProperty || hasOwnProperty.call(src, key)) {
					target[key] = src[key];
				}
			}
		}
		return target;
	};

	/*
	 * Add a set of properties to a target object
	 * target: the target object
	 * props:  an object whose enumerable properties are
	 *         added, by reference only
	 */
	Utils.copy = function(src) {
		return Utils.mixin({}, src);
	};

	/*
	 * Determine whether or not a given object is
	 * an array.
	 */
	Utils.isArray = Array.isArray || function(ob) {
		return Object.prototype.toString.call(ob) == '[object Array]';
	};

	/*
	 * Ensures that an Array object is always returned
	 * returning the original Array of obj is an Array
	 * else wrapping the obj in a single element Array
	 */
	Utils.ensureArray = function(obj) {
		if(Utils.isEmptyArg(obj)) {
			return [];
		}
		if(Utils.isArray(obj)) {
			return obj;
		}
		return [obj];
	}

	/* ...Or an Object (in the narrow sense) */
	Utils.isObject = function(ob) {
		return Object.prototype.toString.call(ob) == '[object Object]';
	};

	/*
	 * Determine whether or not an object contains
	 * any enumerable properties.
	 * ob: the object
	 */
	Utils.isEmpty = function(ob) {
		for(var prop in ob)
			return false;
		return true;
	};

	Utils.isOnlyPropIn = function(ob, property) {
		for(var prop in ob) {
			if(prop !== property) {
				return false;
			}
		}
		return true;
	};

	/*
	 * Determine whether or not an argument to an overloaded function is
	 * undefined (missing) or null.
	 * This method is useful when constructing functions such as (WebIDL terminology):
	 *   off([TreatUndefinedAs=Null] DOMString? event)
	 * as you can then confirm the argument using:
	 *   Utils.isEmptyArg(event)
	 */

	Utils.isEmptyArg = function(arg) {
		return arg === null || arg === undefined;
	}

	/*
	 * Perform a simple shallow clone of an object.
	 * Result is an object irrespective of whether
	 * the input is an object or array. All
	 * enumerable properties are copied.
	 * ob: the object
	 */
	Utils.shallowClone = function(ob) {
		var result = new Object();
		for(var prop in ob)
			result[prop] = ob[prop];
		return result;
	};

	/*
	 * Clone an object by creating a new object with the
	 * given object as its prototype. Optionally
	 * a set of additional own properties can be
	 * supplied to be added to the newly created clone.
	 * ob:            the object to be cloned
	 * ownProperties: optional object with additional
	 *                properties to add
	 */
	Utils.prototypicalClone = function(ob, ownProperties) {
		function F() {}
		F.prototype = ob;
		var result = new F();
		if(ownProperties)
			Utils.mixin(result, ownProperties);
		return result;
	};

	/*
	 * Declare a constructor to represent a subclass
	 * of another constructor
	 * If platform has a built-in version we use that from Platform, else we
	 * define here (so can make use of other Utils fns)
	 * See node.js util.inherits
	 */
	Utils.inherits = Platform.inherits || function(ctor, superCtor) {
		ctor.super_ = superCtor;
		ctor.prototype = Utils.prototypicalClone(superCtor.prototype, { constructor: ctor });
	};

	/*
	 * Determine whether or not an object has an enumerable
	 * property whose value equals a given value.
	 * ob:  the object
	 * val: the value to find
	 */
	Utils.containsValue = function(ob, val) {
		for(var i in ob) {
			if(ob[i] == val)
				return true;
		}
		return false;
	};

	Utils.intersect = function(arr, ob) { return Utils.isArray(ob) ? Utils.arrIntersect(arr, ob) : Utils.arrIntersectOb(arr, ob); };

	Utils.arrIntersect = function(arr1, arr2) {
		var result = [];
		for(var i = 0; i < arr1.length; i++) {
			var member = arr1[i];
			if(Utils.arrIndexOf(arr2, member) != -1)
				result.push(member);
		}
		return result;
	};

	Utils.arrIntersectOb = function(arr, ob) {
		var result = [];
		for(var i = 0; i < arr.length; i++) {
			var member = arr[i];
			if(member in ob)
				result.push(member);
		}
		return result;
	};

	Utils.arrSubtract = function(arr1, arr2) {
		var result = [];
		for(var i = 0; i < arr1.length; i++) {
			var element = arr1[i];
			if(Utils.arrIndexOf(arr2, element) == -1)
				result.push(element);
		}
		return result;
	};

	Utils.arrIndexOf = Array.prototype.indexOf
		? function(arr, elem, fromIndex) {
			return arr.indexOf(elem,  fromIndex);
		}
		: function(arr, elem, fromIndex) {
			fromIndex = fromIndex || 0;
			var len = arr.length;
			for(;fromIndex < len; fromIndex++) {
				if(arr[fromIndex] === elem) {
					return fromIndex;
				}
			}
			return -1;
		};

	Utils.arrIn = function(arr, val) {
		return Utils.arrIndexOf(arr, val) !== -1;
	};

	Utils.arrDeleteValue = function(arr, val) {
		var idx = Utils.arrIndexOf(arr, val);
		var res = (idx != -1);
		if(res)
			arr.splice(idx, 1);
		return res;
	};

	Utils.arrWithoutValue = function(arr, val) {
		var newArr = arr.slice();
		Utils.arrDeleteValue(newArr, val);
		return newArr;
	};

	/*
	 * Construct an array of the keys of the enumerable
	 * properties of a given object, optionally limited
	 * to only the own properties.
	 * ob:      the object
	 * ownOnly: boolean, get own properties only
	 */
	Utils.keysArray = function(ob, ownOnly) {
		var result = [];
		for(var prop in ob) {
			if(ownOnly && !ob.hasOwnProperty(prop)) continue;
			result.push(prop);
		}
		return result;
	};

	/*
	 * Construct an array of the values of the enumerable
	 * properties of a given object, optionally limited
	 * to only the own properties.
	 * ob:      the object
	 * ownOnly: boolean, get own properties only
	 */
	Utils.valuesArray = function(ob, ownOnly) {
		var result = [];
		for(var prop in ob) {
			if(ownOnly && !ob.hasOwnProperty(prop)) continue;
			result.push(ob[prop]);
		}
		return result;
	};

	Utils.arrForEach = Array.prototype.forEach ?
		function(arr, fn) {
			arr.forEach(fn);
		} :
		function(arr, fn) {
			var len = arr.length;
			for(var i = 0; i < len; i++) {
				fn(arr[i], i, arr);
			}
		};

	/* Useful when the function may mutate the array */
	Utils.safeArrForEach = function(arr, fn) {
		return Utils.arrForEach(arr.slice(), fn);
	};

	Utils.arrMap = Array.prototype.map ?
		function(arr, fn) {
			return arr.map(fn);
		} :
		function(arr, fn)	{
			var result = [],
				len = arr.length;
			for(var i = 0; i < len; i++) {
				result.push(fn(arr[i], i, arr));
			}
			return result;
		};

	Utils.arrFilter = Array.prototype.filter ?
		function(arr, fn) {
			return arr.filter(fn);
		} :
		function(arr, fn)	{
			var result = [],
				len = arr.length;
			for(var i = 0; i < len; i++) {
				if(fn(arr[i])) {
					result.push(arr[i]);
				}
			}
			return result;
		};

	Utils.arrEvery = Array.prototype.every ?
		function(arr, fn) {
			return arr.every(fn);
		} : function(arr, fn) {
			var len = arr.length;
			for(var i = 0; i < len; i++) {
				if(!fn(arr[i], i, arr)) {
					return false;
				};
			}
			return true;
		};

	Utils.nextTick = Platform.nextTick;

	var contentTypes = {
		json:   'application/json',
		jsonp:  'application/javascript',
		xml:    'application/xml',
		html:   'text/html',
		msgpack: 'application/x-msgpack'
	};

	Utils.defaultGetHeaders = function(format) {
		var accept = contentTypes[format || 'json'];
		return {
			accept: accept,
			'X-Ably-Version': Defaults.apiVersion,
			'X-Ably-Lib': Defaults.libstring
		};
	};

	Utils.defaultPostHeaders = function(format) {
		var accept, contentType;
		accept = contentType = contentTypes[format || 'json'];

		return {
			accept: accept,
			'content-type': contentType,
			'X-Ably-Version': Defaults.apiVersion,
			'X-Ably-Lib': Defaults.libstring
		};
	};

	Utils.arrPopRandomElement = function(arr) {
		return arr.splice(Math.floor(Math.random() * arr.length), 1)[0];
	};

	Utils.toQueryString = function(params) {
		var parts = [];
		if(params) {
			for(var key in params)
				parts.push(encodeURIComponent(key) + '=' + encodeURIComponent(params[key]));
		}
		return parts.length ? '?' + parts.join('&') : '';
	};

	Utils.parseQueryString = function(query) {
		var match,
			search = /([^?&=]+)=?([^&]*)/g,
			result = {};

		while (match = search.exec(query))
			result[decodeURIComponent(match[1])] = decodeURIComponent(match[2]);

 		return result;
	};

	Utils.now = Date.now || function() {
		/* IE 8 */
		return new Date().getTime();
	};

	Utils.inspect = Platform.inspect;

	Utils.isErrorInfo = function(err) {
		return err.constructor.name == 'ErrorInfo'
	};

	Utils.inspectError = function(x) {
		/* redundant, but node vmcontext issue makes instanceof unreliable, and
		 * can't use just constructor test as could be a TypeError constructor etc. */
		return (x && (Utils.isErrorInfo(x) ||
			x.constructor.name == 'Error' ||
			x instanceof Error)) ?
			x.toString() :
			Utils.inspect(x);
	};

	Utils.randStr = function() {
		return String(Math.random()).substr(2);
	};

	/* Pick n elements at random without replacement from an array */
	Utils.arrChooseN = function(arr, n) {
		var numItems = Math.min(n, arr.length),
			mutableArr = arr.slice(),
			result = [];
		for(var i = 0; i < numItems; i++) {
			result.push(Utils.arrPopRandomElement(mutableArr));
		}
		return result;
	};

	Utils.trim = String.prototype.trim ? function(str) {
		return str.trim();
	} : function(str) {
		return str.replace(/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g, '');
	};

	return Utils;
})();

var Multicaster = (function() {

	function Multicaster(members) {
		members = members || [];

		var handler = function() {
			for(var i = 0; i < members.length; i++) {
				var member = members[i];
				if(member) {
					try {
						member.apply(null, arguments);
					} catch(e){
						Logger.logAction(Logger.LOG_ERROR, 'Multicaster multiple callback handler', 'Unexpected exception: ' + e + '; stack = ' + e.stack);
					}
				}
			}
		};

		handler.push = function() {
			Array.prototype.push.apply(members, arguments);
		};
		return handler;
	}

	return Multicaster;
})();

var ErrorInfo = (function() {

	function ErrorInfo(message, code, statusCode, cause) {
		this.message = message;
		this.code = code;
		this.statusCode = statusCode;
		this.cause = cause;
	}

	ErrorInfo.prototype.toString = function() {
		var result = '[' + this.constructor.name;
		if(this.message) result += ': ' + this.message;
		if(this.statusCode) result += '; statusCode=' + this.statusCode;
		if(this.code) result += '; code=' + this.code;
		if(this.cause) result += '; cause=' + Utils.inspectError(this.cause);
		result += ']';
		return result;
	};

	ErrorInfo.fromValues = function(values) {
		var result = Utils.mixin(new ErrorInfo(), values);
		if (values instanceof Error) {
			/* Error.message is not enumerable, so mixin loses the message */
			result.message = values.message;
		}
		return result;
	};

	return ErrorInfo;
})();

var Message = (function() {
	var msgpack = Platform.msgpack;

	function Message() {
		this.name = undefined;
		this.id = undefined;
		this.timestamp = undefined;
		this.clientId = undefined;
		this.connectionId = undefined;
		this.connectionKey = undefined;
		this.data = undefined;
		this.encoding = undefined;
		this.extras = undefined;
	}

	/**
	 * Overload toJSON() to intercept JSON.stringify()
	 * @return {*}
	 */
	Message.prototype.toJSON = function() {
		var result = {
			name: this.name,
			id: this.id,
			clientId: this.clientId,
			connectionId: this.connectionId,
			connectionKey: this.connectionKey,
			encoding: this.encoding,
			extras: this.extras
		};

		/* encode data to base64 if present and we're returning real JSON;
		 * although msgpack calls toJSON(), we know it is a stringify()
		 * call if it has a non-empty arguments list */
		var data = this.data;
		if(data && BufferUtils.isBuffer(data)) {
			if(arguments.length > 0) {
				/* stringify call */
				var encoding = this.encoding;
				result.encoding = encoding ? (encoding + '/base64') : 'base64';
				data = BufferUtils.base64Encode(data);
			} else {
				/* Called by msgpack. Need to feed it an ArrayBuffer, msgpack doesn't
				* understand WordArrays */
				data = BufferUtils.toArrayBuffer(data);
			}
		}
		result.data = data;
		return result;
	};

	Message.prototype.toString = function() {
		var result = '[Message';
		if(this.name)
			result += '; name=' + this.name;
		if(this.id)
			result += '; id=' + this.id;
		if(this.timestamp)
			result += '; timestamp=' + this.timestamp;
		if(this.clientId)
			result += '; clientId=' + this.clientId;
		if(this.connectionId)
			result += '; connectionId=' + this.connectionId;
		if(this.encoding)
			result += '; encoding=' + this.encoding;
		if(this.extras)
			result += '; extras =' + JSON.stringify(this.extras);
		if(this.data) {
			if (typeof(this.data) == 'string')
				result += '; data=' + this.data;
			else if (BufferUtils.isBuffer(this.data))
				result += '; data (buffer)=' + BufferUtils.base64Encode(this.data);
			else
				result += '; data (json)=' + JSON.stringify(this.data);
		}
		result += ']';
		return result;
	};

	Message.encrypt = function(msg, options, callback) {
		var data = msg.data,
			encoding = msg.encoding,
			cipher = options.channelCipher;

		encoding = encoding ? (encoding + '/') : '';
		if(!BufferUtils.isBuffer(data)) {
			data = BufferUtils.utf8Encode(String(data));
			encoding = encoding + 'utf-8/';
		}
		cipher.encrypt(data, function(err, data) {
			if (err) {
				callback(err);
				return;
			}
			msg.data = data;
			msg.encoding = encoding + 'cipher+' + cipher.algorithm;
			callback(null, msg);
		});
	};

	Message.encode = function(msg, options, callback) {
		var data = msg.data, encoding,
			nativeDataType = typeof(data) == 'string' || BufferUtils.isBuffer(data) || data === null || data === undefined;

		if (!nativeDataType) {
			if (Utils.isObject(data) || Utils.isArray(data)) {
				msg.data = JSON.stringify(data);
				msg.encoding = (encoding = msg.encoding) ? (encoding + '/json') : 'json';
			} else {
				throw new ErrorInfo('Data type is unsupported', 40013, 400);
			}
		}

		if(options != null && options.cipher) {
			Message.encrypt(msg, options, callback);
		} else {
			callback(null, msg);
		}
	};

	Message.encodeArray = function(messages, options, callback) {
		var processed = 0;
		for (var i = 0; i < messages.length; i++) {
			Message.encode(messages[i], options, function(err, msg) {
				if (err) {
					callback(err);
					return;
				}
				processed++;
				if (processed == messages.length) {
					callback(null, messages);
				}
			});
		}
	};

	Message.toRequestBody = function(messages, options, format, callback) {
		Message.encodeArray(messages, options, function(err) {
			if (err) {
				callback(err);
				return;
			}
			callback(null, (format == 'msgpack') ? msgpack.encode(messages, true): JSON.stringify(messages));
		});
	};

	Message.decode = function(message, options) {
		var encoding = message.encoding;
		if(encoding) {
			var xforms = encoding.split('/'),
				i, j = xforms.length,
				data = message.data;

			try {
				while((i = j) > 0) {
					var match = xforms[--j].match(/([\-\w]+)(\+([\w\-]+))?/);
					if(!match) break;
					var xform = match[1];
					switch(xform) {
						case 'base64':
							data = BufferUtils.base64Decode(String(data));
							continue;
						case 'utf-8':
							data = BufferUtils.utf8Decode(data);
							continue;
						case 'json':
							data = JSON.parse(data);
							continue;
						case 'cipher':
							if(options != null && options.cipher) {
								var xformAlgorithm = match[3], cipher = options.channelCipher;
								/* don't attempt to decrypt unless the cipher params are compatible */
								if(xformAlgorithm != cipher.algorithm) {
									throw new Error('Unable to decrypt message with given cipher; incompatible cipher params');
								}
								data = cipher.decrypt(data);
								continue;
							} else {
								throw new Error('Unable to decrypt message; not an encrypted channel');
							}
						default:
							throw new Error("Unknown encoding");
					}
					break;
				}
			} catch(e) {
				throw new ErrorInfo('Error processing the ' + xform + ' encoding, decoder returned ‘' + e.message + '’', 40013, 400);
			} finally {
				message.encoding = (i <= 0) ? null : xforms.slice(0, i).join('/');
				message.data = data;
			}
		}
	};

	Message.fromResponseBody = function(body, options, format) {
		if(format)
			body = (format == 'msgpack') ? msgpack.decode(body) : JSON.parse(String(body));

		for(var i = 0; i < body.length; i++) {
			var msg = body[i] = Message.fromValues(body[i]);
			try {
				Message.decode(msg, options);
			} catch (e) {
				Logger.logAction(Logger.LOG_ERROR, 'Message.fromResponseBody()', e.toString());
			}
		}
		return body;
	};

	Message.fromValues = function(values) {
		return Utils.mixin(new Message(), values);
	};

	Message.fromValuesArray = function(values) {
		var count = values.length, result = new Array(count);
		for(var i = 0; i < count; i++) result[i] = Message.fromValues(values[i]);
		return result;
	};

	function normalizeCipherOptions(options) {
		if(options && options.cipher && !options.cipher.channelCipher) {
			if(!Crypto) throw new Error('Encryption not enabled; use ably.encryption.js instead');
			var cipher = Crypto.getCipher(options.cipher);
			options.cipher = cipher.cipherParams;
			options.channelCipher = cipher.cipher;
		}
	}

	Message.fromEncoded = function(encoded, options) {
		var msg = Message.fromValues(encoded);
		normalizeCipherOptions(options);
		/* if decoding fails at any point, catch and return the message decoded to
		 * the fullest extent possible */
		try {
			Message.decode(msg, options);
		} catch(e) {
			Logger.logAction(Logger.LOG_ERROR, 'Message.fromEncoded()', e.toString());
		}
		return msg;
	};

	Message.fromEncodedArray = function(encodedArray, options) {
		normalizeCipherOptions(options);
		return Utils.arrMap(encodedArray, function(encoded) {
			return Message.fromEncoded(encoded, options);
		});
	};

	return Message;
})();

var PresenceMessage = (function() {
	var msgpack = Platform.msgpack;

	function toActionValue(actionString) {
		return Utils.arrIndexOf(PresenceMessage.Actions, actionString)
	}

	function PresenceMessage() {
		this.action = undefined;
		this.id = undefined;
		this.timestamp = undefined;
		this.clientId = undefined;
		this.connectionId = undefined;
		this.data = undefined;
		this.encoding = undefined;
	}

	PresenceMessage.Actions = [
		'absent',
		'present',
		'enter',
		'leave',
		'update'
	];

	/* Returns whether this presenceMessage is synthesized, i.e. was not actually
	 * sent by the connection (usually means a leave event sent 15s after a
	 * disconnection). This is useful because synthesized messages cannot be
	 * compared for newness by id lexicographically - RTP2b1
	 */
	PresenceMessage.prototype.isSynthesized = function() {
		return this.id.substring(this.connectionId.length, 0) !== this.connectionId;
	};

	/* RTP2b2 */
	PresenceMessage.prototype.parseId = function() {
		var parts = this.id.split(':');
		return {
			connectionId: parts[0],
			msgSerial: parseInt(parts[1], 10),
			index: parseInt(parts[2], 10)
		};
	};

	/**
	 * Overload toJSON() to intercept JSON.stringify()
	 * @return {*}
	 */
	PresenceMessage.prototype.toJSON = function() {
		var result = {
			clientId: this.clientId,
			/* Convert presence action back to an int for sending to Ably */
			action: toActionValue(this.action),
			encoding: this.encoding
		};

		/* encode data to base64 if present and we're returning real JSON;
		 * although msgpack calls toJSON(), we know it is a stringify()
		 * call if it has a non-empty arguments list */
		var data = this.data;
		if(data && BufferUtils.isBuffer(data)) {
			if(arguments.length > 0) {
				/* stringify call */
				var encoding = this.encoding;
				result.encoding = encoding ? (encoding + '/base64') : 'base64';
				data = BufferUtils.base64Encode(data);
			} else {
				/* Called by msgpack. Need to feed it an ArrayBuffer, msgpack doesn't
				* understand WordArrays */
				data = BufferUtils.toArrayBuffer(data);
			}
		}
		result.data = data;
		return result;
	};

	PresenceMessage.prototype.toString = function() {
		var result = '[PresenceMessage';
		result += '; action=' + this.action;
		if(this.id)
			result += '; id=' + this.id;
		if(this.timestamp)
			result += '; timestamp=' + this.timestamp;
		if(this.clientId)
			result += '; clientId=' + this.clientId;
		if(this.connectionId)
			result += '; connectionId=' + this.connectionId;
		if(this.encoding)
			result += '; encoding=' + this.encoding;
		if(this.data) {
			if (typeof(this.data) == 'string')
				result += '; data=' + this.data;
			else if (BufferUtils.isBuffer(this.data))
				result += '; data (buffer)=' + BufferUtils.base64Encode(this.data);
			else
				result += '; data (json)=' + JSON.stringify(this.data);
		}
		result += ']';
		return result;
	};
	PresenceMessage.encode = Message.encode;
	PresenceMessage.decode = Message.decode;

	PresenceMessage.fromResponseBody = function(body, options, format) {
		if(format)
			body = (format == 'msgpack') ? msgpack.decode(body) : JSON.parse(String(body));

		for(var i = 0; i < body.length; i++) {
			var msg = body[i] = PresenceMessage.fromValues(body[i], true);
			try {
				PresenceMessage.decode(msg, options);
			} catch (e) {
				Logger.logAction(Logger.LOG_ERROR, 'PresenceMessage.fromResponseBody()', e.toString());
			}
		}
		return body;
	};

	/* Creates a PresenceMessage from specified values, with a string presence action */
	PresenceMessage.fromValues = function(values, stringifyAction) {
		if(stringifyAction) {
			values.action = PresenceMessage.Actions[values.action]
		}
		return Utils.mixin(new PresenceMessage(), values);
	};

	PresenceMessage.fromValuesArray = function(values) {
		var count = values.length, result = new Array(count);
		for(var i = 0; i < count; i++) result[i] = PresenceMessage.fromValues(values[i]);
		return result;
	};

	PresenceMessage.fromEncoded = function(encoded, options) {
		var msg = PresenceMessage.fromValues(encoded, true);
		/* if decoding fails at any point, catch and return the message decoded to
		 * the fullest extent possible */
		try {
			PresenceMessage.decode(msg, options);
		} catch(e) {
			Logger.logAction(Logger.LOG_ERROR, 'PresenceMessage.fromEncoded()', e.toString());
		}
		return msg;
	};

	PresenceMessage.fromEncodedArray = function(encodedArray, options) {
		return Utils.arrMap(encodedArray, function(encoded) {
			return PresenceMessage.fromEncoded(encoded, options);
		});
	};

	return PresenceMessage;
})();

var ProtocolMessage = (function() {
	var msgpack = Platform.msgpack;

	function ProtocolMessage() {
		this.action = undefined;
		this.flags = undefined;
		this.id = undefined;
		this.timestamp = undefined;
		this.count = undefined;
		this.error = undefined;
		this.connectionId = undefined;
		this.connectionKey = undefined;
		this.connectionSerial = undefined;
		this.channel = undefined;
		this.channelSerial = undefined;
		this.msgSerial = undefined;
		this.messages = undefined;
		this.presence = undefined;
		this.auth = undefined;
	}

	ProtocolMessage.Action = {
		'HEARTBEAT' : 0,
		'ACK' : 1,
		'NACK' : 2,
		'CONNECT' : 3,
		'CONNECTED' : 4,
		'DISCONNECT' : 5,
		'DISCONNECTED' : 6,
		'CLOSE' : 7,
		'CLOSED' : 8,
		'ERROR' : 9,
		'ATTACH' : 10,
		'ATTACHED' : 11,
		'DETACH' : 12,
		'DETACHED' : 13,
		'PRESENCE' : 14,
		'MESSAGE' : 15,
		'SYNC' : 16,
		'AUTH' : 17
	};

	ProtocolMessage.ActionName = [];
	Utils.arrForEach(Utils.keysArray(ProtocolMessage.Action, true), function(name) {
		ProtocolMessage.ActionName[ProtocolMessage.Action[name]] = name;
	});

	var flags = {
		/* Channel attach state flags */
		'HAS_PRESENCE':       1 << 0,
		'HAS_BACKLOG':        1 << 1,
		'RESUMED':            1 << 2,
		'HAS_LOCAL_PRESENCE': 1 << 3,
		'TRANSIENT':          1 << 4,
		/* Channel mode flags */
		'PRESENCE':           1 << 16,
		'PUBLISH':            1 << 17,
		'SUBSCRIBE':          1 << 18,
		'PRESENCE_SUBSCRIBE': 1 << 19
	};
	var flagNames = Utils.keysArray(flags);
	flags.MODE_ALL = flags.PRESENCE | flags.PUBLISH | flags.SUBSCRIBE | flags.PRESENCE_SUBSCRIBE;

	ProtocolMessage.prototype.hasFlag = function(flag) {
		return ((this.flags & flags[flag]) > 0);
	};

	ProtocolMessage.prototype.setFlag = function(flag) {
		return this.flags = this.flags | flags[flag];
	};

	ProtocolMessage.prototype.getMode = function() {
		return this.flags && (this.flags & flags.MODE_ALL);
	};

	ProtocolMessage.serialize = function(msg, format) {
		return (format == 'msgpack') ? msgpack.encode(msg, true): JSON.stringify(msg);
	};

	ProtocolMessage.deserialize = function(serialized, format) {
		var deserialized = (format == 'msgpack') ? msgpack.decode(serialized) : JSON.parse(String(serialized));
		return ProtocolMessage.fromDeserialized(deserialized);
	};

	ProtocolMessage.fromDeserialized = function(deserialized) {
		var error = deserialized.error;
		if(error) deserialized.error = ErrorInfo.fromValues(error);
		var messages = deserialized.messages;
		if(messages) for(var i = 0; i < messages.length; i++) messages[i] = Message.fromValues(messages[i]);
		var presence = deserialized.presence;
		if(presence) for(var i = 0; i < presence.length; i++) presence[i] = PresenceMessage.fromValues(presence[i], true);
		return Utils.mixin(new ProtocolMessage(), deserialized);
	};

	ProtocolMessage.fromValues = function(values) {
		return Utils.mixin(new ProtocolMessage(), values);
	};

	function toStringArray(array) {
		var result = [];
		if (array) {
			for (var i = 0; i < array.length; i++) {
				result.push(array[i].toString());
			}
		}
		return '[ ' + result.join(', ') + ' ]';
	}

	var simpleAttributes = 'id channel channelSerial connectionId connectionKey connectionSerial count msgSerial timestamp'.split(' ');

	ProtocolMessage.stringify = function(msg) {
		var result = '[ProtocolMessage';
		if(msg.action !== undefined)
			result += '; action=' + ProtocolMessage.ActionName[msg.action] || msg.action;

		var attribute;
		for (var attribIndex = 0; attribIndex < simpleAttributes.length; attribIndex++) {
			attribute = simpleAttributes[attribIndex];
			if(msg[attribute] !== undefined)
				result += '; ' + attribute + '=' + msg[attribute];
		}

		if(msg.messages)
			result += '; messages=' + toStringArray(Message.fromValuesArray(msg.messages));
		if(msg.presence)
			result += '; presence=' + toStringArray(PresenceMessage.fromValuesArray(msg.presence));
		if(msg.error)
			result += '; error=' + ErrorInfo.fromValues(msg.error).toString();
		if(msg.auth && msg.auth.accessToken)
			result += '; token=' + msg.auth.accessToken;
		if(msg.flags)
			result += '; flags=' + Utils.arrFilter(flagNames, function(flag) {
				return msg.hasFlag(flag);
			}).join(',');

		result += ']';
		return result;
	};

	return ProtocolMessage;
})();

var Stats = (function() {

	function MessageCount(values) {
		this.count = (values && values.count) || 0;
		this.data = (values && values.data) || 0;
		this.failed = (values && values.failed) || 0;
		this.refused = (values && values.refused) || 0;
	}

	function ResourceCount(values) {
		this.peak = (values && values.peak) || 0;
		this.min = (values && values.min) || 0;
		this.mean = (values && values.mean) || 0;
		this.opened = (values && values.opened) || 0;
		this.refused = (values && values.refused) || 0;
	}

	function RequestCount(values) {
		this.succeeded = (values && values.succeeded) || 0;
		this.failed = (values && values.failed) || 0;
		this.refused = (values && values.refused) || 0;
	}

	function ConnectionTypes(values) {
		this.plain = new ResourceCount(values && values.plain);
		this.tls = new ResourceCount(values && values.tls);
		this.all = new ResourceCount(values && values.all);
	}

	function MessageTypes(values) {
		this.messages = new MessageCount(values && values.messages);
		this.presence = new MessageCount(values && values.presence);
		this.all = new MessageCount(values && values.all);
	}

	function MessageTraffic(values) {
		this.realtime = new MessageTypes(values && values.realtime);
		this.rest = new MessageTypes(values && values.rest);
		this.webhook = new MessageTypes(values && values.webhook);
		this.push = new MessageTypes(values && values.push);
		this.sharedQueue = new MessageTypes(values && values.sharedQueue);
		this.externalQueue = new MessageTypes(values && values.externalQueue);
		this.all = new MessageTypes(values && values.all);
	}

	function Stats(values) {
		this.all           = new MessageTypes(values && values.all);
		this.inbound       = new MessageTraffic(values && values.inbound);
		this.outbound      = new MessageTraffic(values && values.outbound);
		this.persisted     = new MessageTypes(values && values.persisted);
		this.connections   = new ConnectionTypes(values && values.connections);
		this.channels      = new ResourceCount(values && values.channels);
		this.apiRequests   = new RequestCount(values && values.apiRequests);
		this.tokenRequests = new RequestCount(values && values.tokenRequests);
		this.inProgress    = (values && values.inProgress) || undefined;
		this.unit          = (values && values.unit) || undefined;
		this.intervalId    = (values && values.intervalId) || undefined;
	}

	Stats.fromValues = function(values) {
		return new Stats(values);
	};

	return Stats;
})();

var ConnectionError = {
	disconnected: ErrorInfo.fromValues({
		statusCode: 400,
		code: 80003,
		message: 'Connection to server temporarily unavailable'
	}),
	suspended: ErrorInfo.fromValues({
		statusCode: 400,
		code: 80002,
		message: 'Connection to server unavailable'
	}),
	failed: ErrorInfo.fromValues({
		statusCode: 400,
		code: 80000,
		message: 'Connection failed or disconnected by server'
	}),
	closing: ErrorInfo.fromValues({
		statusCode: 400,
		code: 80017,
		message: 'Connection closing'
	}),
	closed: ErrorInfo.fromValues({
		statusCode: 400,
		code: 80017,
		message: 'Connection closed'
	}),
	unknownConnectionErr: ErrorInfo.fromValues({
		statusCode: 500,
		code: 50002,
		message: 'Internal connection error'
	}),
	unknownChannelErr: ErrorInfo.fromValues({
		statusCode: 500,
		code: 50001,
		message: 'Internal channel error'
	})
};

var MessageQueue = (function() {
	function MessageQueue() {
		EventEmitter.call(this);
		this.messages = [];
	}
	Utils.inherits(MessageQueue, EventEmitter);

	MessageQueue.prototype.count = function() {
		return this.messages.length;
	};

	MessageQueue.prototype.push = function(message) {
		this.messages.push(message);
	};

	MessageQueue.prototype.shift = function() {
		return this.messages.shift();
	};

	MessageQueue.prototype.last = function() {
		return this.messages[this.messages.length - 1];
	};

	MessageQueue.prototype.copyAll = function() {
		return this.messages.slice();
	};

	MessageQueue.prototype.append = function(messages) {
		this.messages.push.apply(this.messages, messages);
	};

	MessageQueue.prototype.prepend = function(messages) {
		this.messages.unshift.apply(this.messages, messages);
	};

	MessageQueue.prototype.completeMessages = function(serial, count, err) {
		Logger.logAction(Logger.LOG_MICRO, 'MessageQueue.completeMessages()', 'serial = ' + serial + '; count = ' + count);
		err = err || null;
		var messages = this.messages;
		var first = messages[0];
		if(first) {
			var startSerial = first.message.msgSerial;
			var endSerial = serial + count; /* the serial of the first message that is *not* the subject of this call */
			if(endSerial > startSerial) {
				var completeMessages = messages.splice(0, (endSerial - startSerial));
				for(var i = 0; i < completeMessages.length; i++) {
					completeMessages[i].callback(err);
				}
			}
			if(messages.length == 0)
				this.emit('idle');
		}
	};

	MessageQueue.prototype.completeAllMessages = function(err) {
		this.completeMessages(0, Number.MAX_SAFE_INTEGER || Number.MAX_VALUE, err);
	};

	MessageQueue.prototype.clear = function() {
		Logger.logAction(Logger.LOG_MICRO, 'MessageQueue.clear()', 'clearing ' + this.messages.length + ' messages');
		this.messages = [];
		this.emit('idle');
	};

	return MessageQueue;
})();

var Protocol = (function() {
	var actions = ProtocolMessage.Action;

	function Protocol(transport) {
		EventEmitter.call(this);
		this.transport = transport;
		this.messageQueue = new MessageQueue();
		var self = this;
		transport.on('ack', function(serial, count) { self.onAck(serial, count); });
		transport.on('nack', function(serial, count, err) { self.onNack(serial, count, err); });
	}
	Utils.inherits(Protocol, EventEmitter);

	Protocol.prototype.onAck = function(serial, count) {
		Logger.logAction(Logger.LOG_MICRO, 'Protocol.onAck()', 'serial = ' + serial + '; count = ' + count);
		this.messageQueue.completeMessages(serial, count);
	};

	Protocol.prototype.onNack = function(serial, count, err) {
		Logger.logAction(Logger.LOG_ERROR, 'Protocol.onNack()', 'serial = ' + serial + '; count = ' + count + '; err = ' + Utils.inspectError(err));
		if(!err) {
			err = new ErrorInfo('Unable to send message; channel not responding', 50001, 500);
		}
		this.messageQueue.completeMessages(serial, count, err);
	};

	Protocol.prototype.onceIdle = function(listener) {
		var messageQueue = this.messageQueue;
		if(messageQueue.count() === 0) {
			listener();
			return;
		}
		messageQueue.once('idle', listener);
	};

	Protocol.prototype.send = function(pendingMessage) {
		if(pendingMessage.ackRequired) {
			this.messageQueue.push(pendingMessage);
		}
		if (Logger.shouldLog(Logger.LOG_MICRO)) {
			Logger.logAction(Logger.LOG_MICRO, 'Protocol.send()', 'sending msg; ' + ProtocolMessage.stringify(pendingMessage.message));
		}
		pendingMessage.sendAttempted = true;
		this.transport.send(pendingMessage.message);
	};

	Protocol.prototype.getTransport = function() {
		return this.transport;
	};

	Protocol.prototype.getPendingMessages = function() {
		return this.messageQueue.copyAll();
	};

	Protocol.prototype.clearPendingMessages = function() {
		return this.messageQueue.clear();
	};

	Protocol.prototype.finish = function() {
		var transport = this.transport;
		this.onceIdle(function() {
			transport.disconnect();
		});
	};

	function PendingMessage(message, callback) {
		this.message = message;
		this.callback = callback;
		this.merged = false;
		var action = message.action;
		this.sendAttempted = false;
		this.ackRequired = (action == actions.MESSAGE || action == actions.PRESENCE);
	}
	Protocol.PendingMessage = PendingMessage;

	return Protocol;
})();

var ConnectionManager = (function() {
	var haveWebStorage = !!(typeof(WebStorage) !== 'undefined' && WebStorage.get);
	var haveSessionStorage = !!(typeof(WebStorage) !== 'undefined' && WebStorage.getSession);
	var actions = ProtocolMessage.Action;
	var PendingMessage = Protocol.PendingMessage;
	var noop = function() {};
	var transportPreferenceOrder = Defaults.transportPreferenceOrder;
	var optimalTransport = transportPreferenceOrder[transportPreferenceOrder.length - 1];
	var transportPreferenceName = 'ably-transport-preference';

	var sessionRecoveryName = 'ably-connection-recovery';
	function getSessionRecoverData() {
		return haveSessionStorage && WebStorage.getSession(sessionRecoveryName);
	}
	function setSessionRecoverData(value) {
		return haveSessionStorage && WebStorage.setSession(sessionRecoveryName, value);
	}
	function clearSessionRecoverData() {
		return haveSessionStorage && WebStorage.removeSession(sessionRecoveryName);
	}

	function betterTransportThan(a, b) {
		return Utils.arrIndexOf(transportPreferenceOrder, a.shortName) >
		   Utils.arrIndexOf(transportPreferenceOrder, b.shortName);
	}

	function TransportParams(options, host, mode, connectionKey) {
		this.options = options;
		this.host = host;
		this.mode = mode;
		this.connectionKey = connectionKey;
		this.format = options.useBinaryProtocol ? 'msgpack' : 'json';

		this.connectionSerial = undefined;
		this.timeSerial = undefined;
	}

	TransportParams.prototype.getConnectParams = function(authParams) {
		var params = authParams ? Utils.copy(authParams) : {};
		var options = this.options;
		switch(this.mode) {
			case 'upgrade':
				params.upgrade = this.connectionKey;
				break;
			case 'resume':
				params.resume = this.connectionKey;
				if(this.timeSerial !== undefined) {
					params.timeSerial = this.timeSerial;
				} else if(this.connectionSerial !== undefined) {
					params.connectionSerial = this.connectionSerial;
				}
				break;
			case 'recover':
				var match = options.recover.split(':');
				if(match) {
					params.recover = match[0];
					var recoverSerial = match[1];
					if(isNaN(recoverSerial)) {
						params.timeSerial = recoverSerial;
					} else {
						params.connectionSerial = recoverSerial;
					}
				}
				break;
			default:
		}
		if(options.clientId !== undefined) {
			params.clientId = options.clientId;
		}
		if(options.echoMessages === false) {
			params.echo = 'false';
		}
		if(this.format !== undefined) {
			params.format = this.format;
		}
		if(this.stream !== undefined) {
			params.stream = this.stream;
		}
		if(this.heartbeats !== undefined) {
			params.heartbeats = this.heartbeats;
		}
		params.v = Defaults.apiVersion;
		params.lib = Defaults.libstring;
		if(options.transportParams !== undefined) {
			Utils.mixin(params, options.transportParams);
		}
		return params;
	};

	TransportParams.prototype.toString = function() {
		var result = '[mode=' + this.mode;
		if(this.host) { result += (',host=' + this.host); }
		if(this.connectionKey) { result += (',connectionKey=' + this.connectionKey); }
		if(this.connectionSerial !== undefined) { result += (',connectionSerial=' + this.connectionSerial); }
		if(this.timeSerial) { result += (',timeSerial=' + this.timeSerial); }
		if(this.format) { result += (',format=' + this.format); }
		result += ']';

		return result;
	};

	/* public constructor */
	function ConnectionManager(realtime, options) {
		EventEmitter.call(this);
		this.realtime = realtime;
		this.options = options;
		var timeouts = options.timeouts;
		var self = this;
		/* connectingTimeout: leave preferenceConnectTimeout (~6s) to try the
		 * preference transport, then realtimeRequestTimeout (~10s) to establish
		 * the base transport in case that fails */
		var connectingTimeout = timeouts.preferenceConnectTimeout + timeouts.realtimeRequestTimeout;
		this.states = {
			initialized:   {state: 'initialized',   terminal: false, queueEvents: true,  sendEvents: false, failState: 'disconnected'},
			connecting:    {state: 'connecting',    terminal: false, queueEvents: true,  sendEvents: false, retryDelay: connectingTimeout, failState: 'disconnected'},
			connected:     {state: 'connected',     terminal: false, queueEvents: false, sendEvents: true,  failState: 'disconnected'},
			synchronizing: {state: 'connected',     terminal: false, queueEvents: true,  sendEvents: false, forceQueueEvents: true, failState: 'disconnected'},
			disconnected:  {state: 'disconnected',  terminal: false, queueEvents: true,  sendEvents: false, retryDelay: timeouts.disconnectedRetryTimeout, failState: 'disconnected'},
			suspended:     {state: 'suspended',     terminal: false, queueEvents: false, sendEvents: false, retryDelay: timeouts.suspendedRetryTimeout, failState: 'suspended'},
			closing:       {state: 'closing',       terminal: false, queueEvents: false, sendEvents: false, retryDelay: timeouts.realtimeRequestTimeout, failState: 'closed'},
			closed:        {state: 'closed',        terminal: true,  queueEvents: false, sendEvents: false, failState: 'closed'},
			failed:        {state: 'failed',        terminal: true,  queueEvents: false, sendEvents: false, failState: 'failed'}
		};
		this.state = this.states.initialized;
		this.errorReason = null;

		this.queuedMessages = new MessageQueue();
		this.msgSerial = 0;
		this.connectionDetails = undefined;
		this.connectionId = undefined;
		this.connectionKey = undefined;
		this.timeSerial = undefined;
		this.connectionSerial = undefined;
		this.connectionStateTtl = timeouts.connectionStateTtl;
		this.maxIdleInterval = null;

		this.transports = Utils.intersect((options.transports || Defaults.defaultTransports), ConnectionManager.supportedTransports);
		/* baseTransports selects the leftmost transport in the Defaults.baseTransportOrder list
		* that's both requested and supported. Normally this will be xhr_polling;
		* if xhr isn't supported it will be jsonp. If the user has forced a
		* transport, it'll just be that one. */
		this.baseTransport = Utils.intersect(Defaults.baseTransportOrder, this.transports)[0];
		this.upgradeTransports = Utils.intersect(this.transports, Defaults.upgradeTransports);
		/* Map of hosts to an array of transports to not be tried for that host */
		this.transportHostBlacklist = {};
		this.transportPreference = null;

		this.httpHosts = Defaults.getHosts(options);
		this.activeProtocol = null;
		this.proposedTransports = [];
		this.pendingTransports = [];
		this.host = null;
		this.lastAutoReconnectAttempt = null;
		this.lastActivity = null;
		this.mostRecentMsgId = null;

		Logger.logAction(Logger.LOG_MINOR, 'Realtime.ConnectionManager()', 'started');
		Logger.logAction(Logger.LOG_MICRO, 'Realtime.ConnectionManager()', 'requested transports = [' + (options.transports || Defaults.defaultTransports) + ']');
		Logger.logAction(Logger.LOG_MICRO, 'Realtime.ConnectionManager()', 'available transports = [' + this.transports + ']');
		Logger.logAction(Logger.LOG_MICRO, 'Realtime.ConnectionManager()', 'http hosts = [' + this.httpHosts + ']');

		if(!this.transports.length) {
			var msg = 'no requested transports available';
			Logger.logAction(Logger.LOG_ERROR, 'realtime.ConnectionManager()', msg);
			throw new Error(msg);
		}

		var addEventListener = Platform.addEventListener;
		if(addEventListener) {
			/* intercept close event in browser to persist connection id if requested */
			if(haveSessionStorage && typeof options.recover === 'function') {
				/* Usually can't use bind as not supported in IE8, but IE doesn't support sessionStorage, so... */
				window.addEventListener('beforeunload', this.persistConnection.bind(this));
			}

			if(options.closeOnUnload === true) {
                window.addEventListener('beforeunload', function() {
					Logger.logAction(Logger.LOG_MAJOR, 'Realtime.ConnectionManager()', 'beforeunload event has triggered the connection to close as closeOnUnload is true');
					self.requestState({state: 'closing'});
				});
			}

			/* Listen for online and offline events */
            window.addEventListener('online', function() {
				if(self.state == self.states.disconnected || self.state == self.states.suspended) {
					Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager caught browser ‘online’ event', 'reattempting connection');
					self.requestState({state: 'connecting'});
				}
			});
            window.addEventListener('offline', function() {
				if(self.state == self.states.connected) {
					Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager caught browser ‘offline’ event', 'disconnecting active transport');
					// Not sufficient to just go to the 'disconnected' state, want to
					// force all transports to reattempt the connection. Will immediately
					// retry.
					self.disconnectAllTransports();
				}
			});
		}
	}
	Utils.inherits(ConnectionManager, EventEmitter);

	/*********************
	 * transport management
	 *********************/

	ConnectionManager.supportedTransports = {};

	ConnectionManager.prototype.createTransportParams = function(host, mode) {
		var params = new TransportParams(this.options, host, mode, this.connectionKey);
		if(this.timeSerial) {
			params.timeSerial = this.timeSerial;
		} else if(this.connectionSerial !== undefined) {
			params.connectionSerial = this.connectionSerial;
		}
		return params;
	};

	ConnectionManager.prototype.getTransportParams = function(callback) {
		var self = this;

		function decideMode(modeCb) {
			if(self.connectionKey) {
				modeCb('resume');
				return;
			}

			if(typeof self.options.recover === 'string') {
				modeCb('recover');
				return;
			}

			var recoverFn = self.options.recover,
				lastSessionData = getSessionRecoverData();
			if(lastSessionData && typeof(recoverFn) === 'function') {
				Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.getTransportParams()', 'Calling clientOptions-provided recover function with last session data');
				recoverFn(lastSessionData, function(shouldRecover) {
					if(shouldRecover) {
						self.options.recover = lastSessionData.recoveryKey;
						modeCb('recover');
					} else {
						modeCb('clean');
					}
				});
				return;
			}
			modeCb('clean');
		}

		decideMode(function(mode) {
			var transportParams = self.createTransportParams(null, mode);
			if(mode === 'recover') {
				Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.getTransportParams()', 'Transport recovery mode = recover; recoveryKey = ' + self.options.recover);
				var match = self.options.recover.split(':');
				if(match && match[2]) {
					self.msgSerial = match[2];
				}
			} else {
				Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.getTransportParams()', 'Transport params = ' + transportParams.toString());
			}
			callback(transportParams);
		});
	};

	/**
	 * Attempt to connect using a given transport
	 * @param transportParams
	 * @param candidate, the transport to try
	 * @param callback
	 */
	ConnectionManager.prototype.tryATransport = function(transportParams, candidate, callback) {
		var self = this, host = transportParams.host;
		Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.tryATransport()', 'trying ' + candidate);
		if((host in this.transportHostBlacklist) && Utils.arrIn(this.transportHostBlacklist[host], candidate)) {
			Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.tryATransport()', candidate + ' transport is blacklisted for host ' + transportParams.host);
			return;
		}
		(ConnectionManager.supportedTransports[candidate]).tryConnect(this, this.realtime.auth, transportParams, function(wrappedErr, transport) {
			var state = self.state;
			if(state == self.states.closing || state == self.states.closed || state == self.states.failed) {
				if(transport) {
					Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.tryATransport()', 'connection ' + state.state + ' while we were attempting the transport; closing ' + transport);
					transport.close();
				}
				callback(true);
				return;
			}

			if(wrappedErr) {
				Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.tryATransport()', 'transport ' + candidate + ' ' + wrappedErr.event + ', err: ' + wrappedErr.error.toString());

				/* Comet transport onconnect token errors can be dealt with here.
				* Websocket ones only happen after the transport claims to be viable,
				* so are dealt with as non-onconnect token errors */
				if(Auth.isTokenErr(wrappedErr.error)) {
					/* re-get a token and try again */
					self.realtime.auth._forceNewToken(null, null, function(err) {
						if(err) {
							self.actOnErrorFromAuthorize(err);
							return;
						}
						self.tryATransport(transportParams, candidate, callback);
					});
				} else if(wrappedErr.event === 'failed') {
					/* Error that's fatal to the connection */
					self.notifyState({state: 'failed', error: wrappedErr.error});
					callback(true);
				} else if(wrappedErr.event === 'disconnected') {
					/* Error with that transport only */
					callback(false);
				}
				return;
			}

			Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.chooseTransportForHost()', 'viable transport ' + candidate + '; setting pending');
			self.setTransportPending(transport, transportParams);
			callback(null, transport);
		});
	};


	/**
	 * Called when a transport is indicated to be viable, and the connectionmanager
	 * expects to activate this transport as soon as it is connected.
	 * @param host
	 * @param transportParams
	 */
	ConnectionManager.prototype.setTransportPending = function(transport, transportParams) {
		var mode = transportParams.mode;
		Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.setTransportPending()', 'transport = ' + transport + '; mode = ' + mode);

		Utils.arrDeleteValue(this.proposedTransports, transport);
		this.pendingTransports.push(transport);

		var self = this;
		transport.once('connected', function(error, connectionId, connectionDetails, connectionPosition) {
			if(mode == 'upgrade' && self.activeProtocol) {
				/*  if ws and xhrs are connecting in parallel, delay xhrs activation to let ws go ahead */
				if(transport.shortName !== optimalTransport && Utils.arrIn(self.getUpgradePossibilities(), optimalTransport)) {
					setTimeout(function() {
						self.scheduleTransportActivation(error, transport, connectionId, connectionDetails, connectionPosition);
					}, self.options.timeouts.parallelUpgradeDelay);
				} else {
					self.scheduleTransportActivation(error, transport, connectionId, connectionDetails, connectionPosition);
				}
			} else {
				self.activateTransport(error, transport, connectionId, connectionDetails, connectionPosition);

				/* allow connectImpl to start the upgrade process if needed, but allow
				 * other event handlers, including activating the transport, to run first */
				Utils.nextTick(function() {
					self.connectImpl(transportParams);
				});
			}

			if(mode === 'recover' && self.options.recover) {
				/* After a successful recovery, we unpersist, as a recovery key cannot
				* be used more than once */
				self.options.recover = null;
				self.unpersistConnection();
			}
		});

		transport.on(['disconnected', 'closed', 'failed'], function(error) {
			self.deactivateTransport(transport, this.event, error);
		});

		this.emit('transport.pending', transport);
	};

	/**
	 * Called when an upgrade transport is connected,
	 * to schedule the activation of that transport.
	 * @param error
	 * @param transport
	 * @param connectionId
	 * @param connectionDetails
	 * @param connectedMessage
	 */
	ConnectionManager.prototype.scheduleTransportActivation = function(error, transport, connectionId, connectionDetails, upgradeConnectionPosition) {
		var self = this,
			currentTransport = this.activeProtocol && this.activeProtocol.getTransport(),
			abandon = function() {
				transport.disconnect();
				Utils.arrDeleteValue(self.pendingTransports, transport);
			};

		if(this.state !== this.states.connected && this.state !== this.states.connecting) {
			/* This is most likely to happen for the delayed xhrs, when xhrs and ws are scheduled in parallel*/
			Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.scheduleTransportActivation()', 'Current connection state (' + this.state.state + (this.state === this.states.synchronizing ? ', but with an upgrade already in progress' : '') + ') is not valid to upgrade in; abandoning upgrade to ' + transport.shortName);
			abandon();
			return;
		}

		if(currentTransport && !betterTransportThan(transport, currentTransport)) {
			Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.scheduleTransportActivation()', 'Proposed transport ' + transport.shortName + ' is no better than current active transport ' + currentTransport.shortName + ' - abandoning upgrade');
			abandon();
			return;
		}

		Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.scheduleTransportActivation()', 'Scheduling transport upgrade; transport = ' + transport);

		this.realtime.channels.onceNopending(function(err) {
			var oldProtocol;
			if(err) {
				Logger.logAction(Logger.LOG_ERROR, 'ConnectionManager.scheduleTransportActivation()', 'Unable to activate transport; transport = ' + transport + '; err = ' + err);
				return;
			}

			if(!transport.isConnected) {
				/* This is only possible if the xhr streaming transport was disconnected during the parallelUpgradeDelay */
				Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.scheduleTransportActivation()', 'Proposed transport ' + transport.shortName + 'is no longer connected; abandoning upgrade');
				abandon();
				return;
			}

			if(self.state === self.states.connected) {
				Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.scheduleTransportActivation()', 'Currently connected, so temporarily pausing events until the upgrade is complete');
				self.state = self.states.synchronizing;
				oldProtocol = self.activeProtocol;
			} else if(self.state !== self.states.connecting) {
				/* Note: upgrading from the connecting state is valid if the old active
				* transport was deactivated after the upgrade transport first connected;
				* see logic in deactivateTransport */
				Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.scheduleTransportActivation()', 'Current connection state (' + self.state.state + (self.state === self.states.synchronizing ? ', but with an upgrade already in progress' : '') + ') is not valid to upgrade in; abandoning upgrade to ' + transport.shortName);
				abandon();
				return;
			}

			/* If the connectionId has changed, the upgrade hasn't worked. But as
			* it's still an upgrade, realtime still expects a sync - it just needs to
			* be a sync with the new connection position. (And it
			* needs to be set in the library, which is done by activateTransport). */
			var connectionReset = connectionId !== self.connectionId,
				syncPosition = connectionReset ? upgradeConnectionPosition : self;

			if(connectionReset) {
				Logger.logAction(Logger.LOG_ERROR, 'ConnectionManager.scheduleTransportActivation()', 'Upgrade resulted in new connectionId; resetting library connection position from ' + (self.timeSerial || self.connectionSerial) + ' to ' + (syncPosition.timeSerial || syncPosition.connectionSerial) + '; upgrade error was ' + error);
			}

			Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.scheduleTransportActivation()', 'Syncing transport; transport = ' + transport);
			self.sync(transport, syncPosition, function(syncErr, connectionId, postSyncPosition) {
				/* If there's been some problem with syncing (and the connection hasn't
				 * closed or something in the meantime), we have a problem -- we can't
				 * just fall back on the old transport, as we don't know whether
				 * realtime got the sync -- if it did, the old transport is no longer
				 * valid. To be safe, we disconnect both and start again from scratch. */
				if(syncErr) {
					if(self.state === self.states.synchronizing) {
						Logger.logAction(Logger.LOG_ERROR, 'ConnectionManager.scheduleTransportActivation()', 'Unexpected error attempting to sync transport; transport = ' + transport + '; err = ' + syncErr);
						self.disconnectAllTransports();
					}
					return;
				}
				var finishUpgrade = function() {
					Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.scheduleTransportActivation()', 'Activating transport; transport = ' + transport);
					self.activateTransport(error, transport, connectionId, connectionDetails, postSyncPosition);
					/* Restore pre-sync state. If state has changed in the meantime,
					 * don't touch it -- since the websocket transport waits a tick before
					 * disposing itself, it's possible for it to have happily synced
					 * without err while, unknown to it, the connection has closed in the
					 * meantime and the ws transport is scheduled for death */
					if(self.state === self.states.synchronizing) {
						Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.scheduleTransportActivation()', 'Pre-upgrade protocol idle, sending queued messages on upgraded transport; transport = ' + transport);
						self.state = self.states.connected;
					} else {
						Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.scheduleTransportActivation()', 'Pre-upgrade protocol idle, but state is now ' + self.state.state + ', so leaving unchanged');
					}
					if(self.state.sendEvents) {
						self.sendQueuedMessages();
					}
				};

				/* Wait until sync is done and old transport is idle before activating new transport. This
				 * guarantees that messages arrive at realtime in the same order they are sent.
				 *
				 * If a message times out on the old transport, since it's still the active transport the
				 * message will be requeued. deactivateTransport will see the pending transport and notify
				 * the `connecting` state without starting a new connection, so the new transport can take
				 * over once deactivateTransport clears the old protocol's queue.
				 *
				 * If there is no old protocol, that meant that we weren't in the connected state at the
				 * beginning of the sync - likely the base transport died just before the sync. So can just
				 * finish the upgrade. If we're actually in closing/failed rather than connecting, that's
				 * fine, activatetransport will deal with that. */
				if(oldProtocol) {
				 /* Most of the time this will be already true: the new-transport sync will have given
				 * enough time for in-flight messages on the old transport to complete. */
					oldProtocol.onceIdle(finishUpgrade);
				} else {
					finishUpgrade();
				}
			});
		});
	};

	/**
	 * Called when a transport is connected, and the connectionmanager decides that
	 * it will now be the active transport. Returns whether or not it activated
	 * the transport (if the connection is closing/closed it will choose not to).
	 * @param transport the transport instance
	 * @param connectionId the id of the new active connection
	 * @param connectionDetails the details of the new active connection
	 * @param connectionPosition the position at the point activation; either {connectionSerial: <serial>} or {timeSerial: <serial>}
	 */
	ConnectionManager.prototype.activateTransport = function(error, transport, connectionId, connectionDetails, connectionPosition) {
		Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.activateTransport()', 'transport = ' + transport);
		if(error) {
			Logger.logAction(Logger.LOG_ERROR, 'ConnectionManager.activateTransport()', 'error = ' + error);
		}
		if(connectionId) {
			Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.activateTransport()', 'connectionId =  ' + connectionId);
		}
		if(connectionDetails) {
			Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.activateTransport()', 'connectionDetails =  ' + JSON.stringify(connectionDetails));
		}
		if(connectionPosition) {
			Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.activateTransport()', 'serial =  ' + (connectionPosition.timeSerial || connectionPosition.connectionSerial));
		}

		this.persistTransportPreference(transport);

		/* if the connectionmanager moved to the closing/closed state before this
		 * connection event, then we won't activate this transport */
		var existingState = this.state,
			connectedState = this.states.connected.state;
		Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.activateTransport()', 'current state = ' + existingState.state);
		if(existingState.state == this.states.closing.state || existingState.state == this.states.closed.state || existingState.state == this.states.failed.state) {
			Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.activateTransport()', 'Disconnecting transport and abandoning');
			transport.disconnect();
			return false;
		}

		/* remove this transport from pending transports */
		Utils.arrDeleteValue(this.pendingTransports, transport);

		/* if the transport is not connected (eg because it failed during a
		 * scheduleTransportActivation#onceNoPending wait) then don't activate it */
		if(!transport.isConnected) {
			Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.activateTransport()', 'Declining to activate transport ' + transport + ' since it appears to no longer be connected');
			return false;
		}

		/* the given transport is connected; this will immediately
		 * take over as the active transport */
		var existingActiveProtocol = this.activeProtocol;
		this.activeProtocol = new Protocol(transport);
		this.host = transport.params.host;

		var connectionKey = connectionDetails.connectionKey;
		if(connectionKey && this.connectionKey != connectionKey)  {
			this.setConnection(connectionId, connectionDetails, connectionPosition, true);
		}

		/* Rebroadcast any new connectionDetails from the active transport, which
		 * can come at any time (eg following a reauth), and emit an RTN24 UPDATE
		 * event. (Listener added on nextTick because we're in a transport.on('connected')
		 * callback at the moment; if we add it now we'll be adding it to the end
		 * of the listeners array and it'll be called immediately) */
		this.onConnectionDetailsUpdate(connectionDetails, transport);
		var self = this;
		Utils.nextTick(function() {
			transport.on('connected', function(connectedErr, _connectionId, connectionDetails) {
				self.onConnectionDetailsUpdate(connectionDetails, transport);
				self.emit('update', new ConnectionStateChange(connectedState, connectedState, null, connectedErr));
			});
		})

		/* If previously not connected, notify the state change (including any
		 * error). */
		if(existingState.state === this.states.connected.state) {
			if(error) {
				/* if upgrading without error, leave any existing errorReason alone */
				this.errorReason = this.realtime.connection.errorReason = error;
				/* Only bother emitting an upgrade if there's an error; otherwise it's
				 * just a transport upgrade, so auth details won't have changed */
				this.emit('update', new ConnectionStateChange(connectedState, connectedState, null, error));
			}
		} else {
			this.notifyState({state: 'connected', error: error});
			this.errorReason = this.realtime.connection.errorReason = error || null;
		}

		/* Send after the connection state update, as Channels hooks into this to
		 * resend attaches on a new transport if necessary */
		this.emit('transport.active', transport);

		/* Gracefully terminate existing protocol */
		if(existingActiveProtocol) {
			if(existingActiveProtocol.messageQueue.count() > 0) {
				/* We could just requeue pending messages on the new transport, but
				 * actually this should never happen: transports should only take over
				 * from other active transports when upgrading, and upgrading waits for
				 * the old transport to be idle. So log an error. */
				Logger.logAction(Logger.LOG_ERROR, 'ConnectionManager.activateTransport()', 'Previous active protocol (for transport ' + existingActiveProtocol.transport.shortName + ', new one is ' + transport.shortName + ') finishing with ' + existingActiveProtocol.messageQueue.count() + ' messages still pending');
			}
			existingActiveProtocol.finish();
		}

		/* Terminate any other pending transport(s), and
		 * abort any not-yet-pending transport attempts */
		Utils.safeArrForEach(this.pendingTransports, function(transport) {
			transport.disconnect();
		});
		Utils.safeArrForEach(this.proposedTransports, function(transport) {
			transport.dispose();
		});

		return true;
	};

	/**
	 * Called when a transport is no longer the active transport. This can occur
	 * in any transport connection state.
	 * @param transport
	 */
	ConnectionManager.prototype.deactivateTransport = function(transport, state, error) {
		var currentProtocol = this.activeProtocol,
			wasActive = currentProtocol && currentProtocol.getTransport() === transport,
			wasPending = Utils.arrDeleteValue(this.pendingTransports, transport),
			wasProposed = Utils.arrDeleteValue(this.proposedTransports, transport),
			noTransportsScheduledForActivation = this.noTransportsScheduledForActivation();

		Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.deactivateTransport()', 'transport = ' + transport);
		Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.deactivateTransport()', 'state = ' + state + (wasActive ? '; was active' : wasPending ? '; was pending' : wasProposed ? '; was proposed' : '') + (noTransportsScheduledForActivation ? '' : '; another transport is scheduled for activation'));
		if(error && error.message)
			Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.deactivateTransport()', 'reason =  ' + error.message);

		if(wasActive) {
			Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.deactivateTransport()', 'Getting, clearing, and requeuing ' + this.activeProtocol.messageQueue.count() + ' pending messages');
			this.queuePendingMessages(currentProtocol.getPendingMessages());
			/* Clear any messages we requeue to allow the protocol to become idle.
			 * In case of an upgrade, this will trigger an immediate activation of
			 * the upgrade transport, so delay a tick so this transport can finish
			 * deactivating */
			Utils.nextTick(function() {
				currentProtocol.clearPendingMessages();
			});
			this.activeProtocol = this.host = null;
		}

		this.emit('transport.inactive', transport);

		/* this transport state change is a state change for the connectionmanager if
		 * - the transport was the active transport and there are no transports
		 *   which are connected and scheduled for activation, just waiting for the
		 *   active transport to finish what its doing; or
		 * - the transport was the active transport and the error was fatal (so
		 *   unhealable by another transport); or
		 * - there is no active transport, and this is the last remaining
		 *   pending transport (so we were in the connecting state)
		 */
		if((wasActive && noTransportsScheduledForActivation) ||
			(wasActive && (state === 'failed') || (state === 'closed')) ||
			(currentProtocol === null && wasPending && this.pendingTransports.length === 0)) {
			/* TODO remove below line once realtime sends token errors as DISCONNECTEDs */
			if(state === 'failed' && Auth.isTokenErr(error)) { state = 'disconnected' }
			this.notifyState({state: state, error: error});
		} else if(wasActive && (state === 'disconnected')) {
			/* If we were active but there is another transport scheduled for
			* activation, go into to the connecting state until that transport
			* activates and sets us back to connected. (manually starting the
			* transition timers in case that never happens) */
			Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.deactivateTransport()', 'wasActive but another transport is connected and scheduled for activation, so going into the connecting state until it activates');
			this.startSuspendTimer();
			this.startTransitionTimer(this.states.connecting);
			this.notifyState({state: 'connecting', error: error});
		}
	};

	/* Helper that returns true if there are no transports which are pending,
	* have been connected, and are just waiting for onceNoPending to fire before
	* being activated */
	ConnectionManager.prototype.noTransportsScheduledForActivation = function() {
		return Utils.isEmpty(this.pendingTransports) ||
			this.pendingTransports.every(function(transport) {
				return !transport.isConnected;
			});
	};

	/**
	 * Called when activating a new transport, to ensure message delivery
	 * on the new transport synchronises with the messages already received
	 */
	ConnectionManager.prototype.sync = function(transport, requestedSyncPosition, callback) {
		var timeout = setTimeout(function () {
			transport.off('sync');
			callback(new ErrorInfo('Timeout waiting for sync response', 50000, 500));
		}, this.options.timeouts.realtimeRequestTimeout);

		/* send sync request */
		var syncMessage = ProtocolMessage.fromValues({
			action: actions.SYNC,
			connectionKey: this.connectionKey
		});

		if(requestedSyncPosition.timeSerial) {
			syncMessage.timeSerial = requestedSyncPosition.timeSerial;
		} else if(requestedSyncPosition.connectionSerial !== undefined) {
			syncMessage.connectionSerial = requestedSyncPosition.connectionSerial;
		}
		transport.send(syncMessage);

		transport.once('sync', function(connectionId, syncPosition) {
			clearTimeout(timeout);
			callback(null, connectionId, syncPosition);
		});
	};

	ConnectionManager.prototype.setConnection = function(connectionId, connectionDetails, connectionPosition, forceSetPosition) {
		/* if connectionKey changes but connectionId stays the same, then just a
		 * transport change on the same connection. If connectionId changes, we're
		 * on a new connection, with implications for msgSerial and channel state */
		var self = this;
		/* If no previous connectionId, don't reset the msgSerial as it may have been set by recover data */
		if(this.connectionId && (this.connectionId !== connectionId))  {
			Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.setConnection()', 'Resetting msgSerial');
			this.msgSerial = 0;
		}
		/* but do need to reattach channels, for channels that were previously in
		 * the attached state even though the connection mode was 'clean' due to a
		 * freshness check - see https://github.com/ably/ably-js/issues/394 */
		if(this.connectionId !== connectionId)  {
			Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.setConnection()', 'New connectionId; reattaching any attached channels');
			/* Wait till next tick before reattaching channels, so that connection
			 * state will be updated and so that it will be applied after
			 * Channels#onTransportUpdate, else channels will not have an ATTACHED
			 * sent twice (once from this and once from that). */
			Utils.nextTick(function() {
				self.realtime.channels.reattach();
			});
		}
		this.realtime.connection.id = this.connectionId = connectionId;
		this.realtime.connection.key = this.connectionKey = connectionDetails.connectionKey;
		this.setConnectionSerial(connectionPosition, forceSetPosition);
	};

	ConnectionManager.prototype.clearConnection = function() {
		this.realtime.connection.id = this.connectionId = undefined;
		this.realtime.connection.key = this.connectionKey = undefined;
		this.clearConnectionSerial();
		this.msgSerial = 0;
		this.unpersistConnection();
	};

	/* force: set the connectionSerial even if it's less than the current connectionSerial. Used when
	 * activating a new transport, where the connectionSerial realtime tells us we have must be authoritative */
	ConnectionManager.prototype.setConnectionSerial = function(connectionPosition, force) {
		var timeSerial = connectionPosition.timeSerial;
		if(timeSerial !== undefined) {
			if(timeSerial <= this.timeSerial && !force) {
				Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.setConnectionSerial() received message with timeSerial ' + timeSerial + ', but current timeSerial is ' + this.timeSerial + '; assuming message is a duplicate and discarding it');
				return;
			}
			this.realtime.connection.timeSerial = this.timeSerial = timeSerial;
			this.setRecoveryKey();
			return;
		}
		var connectionSerial = connectionPosition.connectionSerial;
		if(connectionSerial !== undefined) {
			if(connectionSerial <= this.connectionSerial && !force) {
				Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.setConnectionSerial() received message with connectionSerial ' + connectionSerial + ', but current connectionSerial is ' + this.connectionSerial + '; assuming message is a duplicate and discarding it');
				return;
			}
			this.realtime.connection.serial = this.connectionSerial = connectionSerial;
			this.setRecoveryKey();
		}
	};

	ConnectionManager.prototype.clearConnectionSerial = function() {
		this.realtime.connection.serial = this.connectionSerial = undefined;
		this.realtime.connection.timeSerial = this.timeSerial = undefined;
		this.clearRecoveryKey();
	};

	ConnectionManager.prototype.setRecoveryKey = function() {
		this.realtime.connection.recoveryKey = this.connectionKey + ':' + (this.timeSerial || this.connectionSerial) + ':' + this.msgSerial;
	};

	ConnectionManager.prototype.clearRecoveryKey = function() {
		this.realtime.connection.recoveryKey = null;
	};

	ConnectionManager.prototype.checkConnectionStateFreshness = function() {
		if(!this.lastActivity || !this.connectionId) { return; }

		var sinceLast = Utils.now() - this.lastActivity;
		if(sinceLast > this.connectionStateTtl + this.maxIdleInterval) {
			Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.checkConnectionStateFreshness()', 'Last known activity from realtime was ' + sinceLast + 'ms ago; discarding connection state');
			this.clearConnection();
			this.states.connecting.failState = 'suspended';
			this.states.connecting.queueEvents = false;
		}
	};

	/**
	 * Called when the connectionmanager wants to persist transport
	 * state for later recovery. Only applicable in the browser context.
	 */
	ConnectionManager.prototype.persistConnection = function() {
		if(haveSessionStorage) {
			var recoveryKey = this.realtime.connection.recoveryKey;
			if(recoveryKey) {
				setSessionRecoverData({
					recoveryKey: recoveryKey,
					disconnectedAt: Utils.now(),
					location: window.location,
					clientId: this.realtime.auth.clientId
				}, this.connectionStateTtl);
			}
		}
	};

	/**
	 * Called when the connectionmanager wants to persist transport
	 * state for later recovery. Only applicable in the browser context.
	 */
	ConnectionManager.prototype.unpersistConnection = function() {
		clearSessionRecoverData();
	};

	/*********************
	 * state management
	 *********************/

	ConnectionManager.prototype.getStateError = function() {
		return ConnectionError[this.state.state];
	};

	ConnectionManager.prototype.activeState = function() {
		return this.state.queueEvents || this.state.sendEvents;
	};

	ConnectionManager.prototype.enactStateChange = function(stateChange) {
		var logLevel = stateChange.current === 'failed' ? Logger.LOG_ERROR : Logger.LOG_MAJOR;
		Logger.logAction(logLevel, 'Connection state', stateChange.current + (stateChange.reason ? ('; reason: ' + stateChange.reason) : ''));
		Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.enactStateChange', 'setting new state: ' + stateChange.current + '; reason = ' + (stateChange.reason && stateChange.reason.message));
		var newState = this.state = this.states[stateChange.current];
		if(stateChange.reason) {
			this.errorReason = stateChange.reason;
			this.realtime.connection.errorReason = stateChange.reason;
		}
		if(newState.terminal || newState.state === 'suspended') {
			/* suspended is nonterminal, but once in the suspended state, realtime
			 * will have discarded our connection state, so futher connection
			 * attempts should start from scratch */
			this.clearConnection();
		}
		this.emit('connectionstate', stateChange);
	};

	/****************************************
	 * ConnectionManager connection lifecycle
	 ****************************************/

	ConnectionManager.prototype.startTransitionTimer = function(transitionState) {
		Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.startTransitionTimer()', 'transitionState: ' + transitionState.state);

		if(this.transitionTimer) {
			Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.startTransitionTimer()', 'clearing already-running timer');
			clearTimeout(this.transitionTimer);
		}

		var self = this;
		this.transitionTimer = setTimeout(function() {
			if(self.transitionTimer) {
				self.transitionTimer = null;
				Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager ' + transitionState.state + ' timer expired', 'requesting new state: ' + transitionState.failState);
				self.notifyState({state: transitionState.failState});
			}
		}, transitionState.retryDelay);
	};

	ConnectionManager.prototype.cancelTransitionTimer = function() {
		Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.cancelTransitionTimer()', '');
		if(this.transitionTimer) {
			clearTimeout(this.transitionTimer);
			this.transitionTimer = null;
		}
	};

	ConnectionManager.prototype.startSuspendTimer = function() {
		var self = this;
		if(this.suspendTimer)
			return;
		this.suspendTimer = setTimeout(function() {
			if(self.suspendTimer) {
				self.suspendTimer = null;
				Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager suspend timer expired', 'requesting new state: suspended');
				self.states.connecting.failState = 'suspended';
				self.states.connecting.queueEvents = false;
				self.notifyState({state: 'suspended'});
			}
		}, this.connectionStateTtl);
	};

	ConnectionManager.prototype.checkSuspendTimer = function(state) {
		if(state !== 'disconnected' && state !== 'suspended' && state !== 'connecting')
			this.cancelSuspendTimer();
	};

	ConnectionManager.prototype.cancelSuspendTimer = function() {
		this.states.connecting.failState = 'disconnected';
		this.states.connecting.queueEvents = true;
		if(this.suspendTimer) {
			clearTimeout(this.suspendTimer);
			this.suspendTimer = null;
		}
	};

	ConnectionManager.prototype.startRetryTimer = function(interval) {
		var self = this;
		this.retryTimer = setTimeout(function() {
			Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager retry timer expired', 'retrying');
			self.retryTimer = null;
			self.requestState({state: 'connecting'});
		}, interval);
	};

	ConnectionManager.prototype.cancelRetryTimer = function() {
		if(this.retryTimer) {
			clearTimeout(this.retryTimer);
			this.retryTimer = null;
		}
	};

	ConnectionManager.prototype.notifyState = function(indicated) {
		var state = indicated.state,
			self = this;

		/* We retry immediately if:
		 * - something disconnects us while we're connected, or
		 * - a viable (but not yet active) transport fails due to a token error (so
		 *   this.errorReason will be set, and startConnect will do a forced authorize) */
		var retryImmediately = (state === 'disconnected' &&
			(this.state === this.states.connected     ||
			 this.state === this.states.synchronizing ||
				(this.state === this.states.connecting  &&
					indicated.error && Auth.isTokenErr(indicated.error))));

		Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.notifyState()', 'new state: ' + state + (retryImmediately ? '; will retry connection immediately' : ''));
		/* do nothing if we're already in the indicated state */
		if(state == this.state.state)
			return;

		/* kill timers (possibly excepting suspend timer depending on the notified
		* state), as these are superseded by this notification */
		this.cancelTransitionTimer();
		this.cancelRetryTimer();
		this.checkSuspendTimer(indicated.state);

		/* do nothing if we're unable to move from the current state */
		if(this.state.terminal)
			return;

		/* process new state */
		var newState = this.states[indicated.state],
			change = new ConnectionStateChange(this.state.state, newState.state, newState.retryDelay, (indicated.error || ConnectionError[newState.state]));

		if(retryImmediately) {
			var autoReconnect = function() {
				if(self.state === self.states.disconnected) {
					self.lastAutoReconnectAttempt = Utils.now();
					self.requestState({state: 'connecting'});
				}
			};
			var sinceLast = this.lastAutoReconnectAttempt && (Utils.now() - this.lastAutoReconnectAttempt + 1);
			if(sinceLast && (sinceLast < 1000)) {
				Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.notifyState()', 'Last reconnect attempt was only ' + sinceLast + 'ms ago, waiting another ' + (1000 - sinceLast) + 'ms before trying again');
				setTimeout(autoReconnect, 1000 - sinceLast);
			} else {
				Utils.nextTick(autoReconnect);
			}
		} else if(state === 'disconnected' || state === 'suspended') {
			this.startRetryTimer(newState.retryDelay);
		}

		 /* If going into disconnect/suspended (and not retrying immediately), or a
			* terminal state, ensure there are no orphaned transports hanging around. */
		if((state === 'disconnected' && !retryImmediately) ||
			 (state === 'suspended') ||
			 newState.terminal) {
				 /* Wait till the next tick so the connection state change is enacted,
				 * so aborting transports doesn't trigger redundant state changes */
				 Utils.nextTick(function() {
					 self.disconnectAllTransports();
				 });
		 }

		if(state == 'connected' && !this.activeProtocol) {
			Logger.logAction(Logger.LOG_ERROR, 'ConnectionManager.notifyState()', 'Broken invariant: attempted to go into connected state, but there is no active protocol');
		}

		/* implement the change and notify */
		this.enactStateChange(change);
		if(this.state.sendEvents) {
			this.sendQueuedMessages();
		} else if(!this.state.queueEvents) {
			this.realtime.channels.propogateConnectionInterruption(state, change.reason);
			this.failQueuedMessages(change.reason); // RTN7c
		}
	};

	ConnectionManager.prototype.requestState = function(request) {
		var state = request.state, self = this;
		Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.requestState()', 'requested state: ' + state + '; current state: ' + this.state.state);
		if(state == this.state.state)
			return; /* silently do nothing */

		/* kill running timers, as this request supersedes them */
		this.cancelTransitionTimer();
		this.cancelRetryTimer();
		/* for suspend timer check rather than cancel -- eg requesting a connecting
		* state should not reset the suspend timer */
		this.checkSuspendTimer(state);

		if(state == 'connecting' && this.state.state == 'connected') return;
		if(state == 'closing' && this.state.state == 'closed') return;

		var newState = this.states[state],
			change = new ConnectionStateChange(this.state.state, newState.state, null, (request.error || ConnectionError[newState.state]));

		this.enactStateChange(change);

		if(state == 'connecting') {
			Utils.nextTick(function() { self.startConnect(); });
		}
		if(state == 'closing') {
			this.closeImpl();
		}
	};


	ConnectionManager.prototype.startConnect = function() {
		if(this.state !== this.states.connecting) {
			Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.startConnect()', 'Must be in connecting state to connect, but was ' + this.state.state);
			return;
		}

		var auth = this.realtime.auth,
			self = this;

		var connect = function() {
			self.checkConnectionStateFreshness();
			self.getTransportParams(function(transportParams) {
				self.connectImpl(transportParams);
			});
		};

		Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.startConnect()', 'starting connection');
		this.startSuspendTimer();
		this.startTransitionTimer(this.states.connecting);

		if(auth.method === 'basic') {
			connect();
		} else {
			var authCb = function(err) {
				if(err) {
					self.actOnErrorFromAuthorize(err);
				} else {
					connect();
				}
			};
			if(this.errorReason && Auth.isTokenErr(this.errorReason)) {
				/* Force a refetch of a new token */
				auth._forceNewToken(null, null, authCb);
			} else {
				auth._ensureValidAuthCredentials(authCb);
			}
		}
	};

	/**
	 * There are three stages in connecting:
	 * - preference: if there is a cached transport preference, we try to connect
	 *   on that. If that fails or times out we abort the attempt, remove the
	 *   preference and fall back to base. If it succeeds, we try upgrading it if
	 *   needed (will only be in the case where the preference is xhrs and the
	 *   browser supports ws).
	 * - base: we try to connect with the best transport that we think will
	 *   never fail for this browser (usually this is xhr_polling; for very old
	 *   browsers will be jsonp, for node will be comet). If it doesn't work, we
	 *   try fallback hosts.
	 * - upgrade: given a connected transport, we see if there are any better
	 *   ones, and if so, try to upgrade to them.
	 *
	 * connectImpl works out what stage you're at (which is purely a function of
	 * the current connection state and whether there are any stored preferences),
	 * and dispatches accordingly. After a transport has been set pending,
	 * tryATransport calls connectImpl to see if there's another stage to be done.
	 * */
	ConnectionManager.prototype.connectImpl = function(transportParams) {
		var state = this.state.state;

		if(state !== this.states.connecting.state && state !== this.states.connected.state) {
			/* Only keep trying as long as in the 'connecting' state (or 'connected'
			 * for upgrading). Any operation can put us into 'disconnected' to cancel
			 * connection attempts and wait before retrying, or 'failed' to fail. */
			Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.connectImpl()', 'Must be in connecting state to connect (or connected to upgrade), but was ' + state);
		} else if(this.pendingTransports.length) {
			Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.connectImpl()', 'Transports ' + this.pendingTransports[0].toString() + ' currently pending; taking no action');
		} else if(state == this.states.connected.state) {
			this.upgradeIfNeeded(transportParams);
		} else if(this.transports.length > 1 && this.getTransportPreference()) {
			this.connectPreference(transportParams);
		} else {
			this.connectBase(transportParams);
		}
	};


	ConnectionManager.prototype.connectPreference = function(transportParams) {
		var preference = this.getTransportPreference(),
			self = this,
			preferenceTimeoutExpired = false;

		if(!Utils.arrIn(this.transports, preference)) {
			this.unpersistTransportPreference();
			this.connectImpl(transportParams);
		}

		Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.connectPreference()', 'Trying to connect with stored transport preference ' + preference);

		var preferenceTimeout = setTimeout(function() {
			preferenceTimeoutExpired = true;
			if(!(self.state.state === self.states.connected.state)) {
				Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.connectPreference()', 'Shortcircuit connection attempt with ' + preference + ' failed; clearing preference and trying from scratch');
				/* Abort all connection attempts. (This also disconnects the active
				 * protocol, but none exists if we're not in the connected state) */
				self.disconnectAllTransports();
				/* Be quite agressive about clearing the stored preference if ever it doesn't work */
				self.unpersistTransportPreference();
			}
			self.connectImpl(transportParams);
		}, this.options.timeouts.preferenceConnectTimeout);

		/* For connectPreference, just use the main host. If host fallback is needed, do it in connectBase.
		 * The wstransport it will substitute the httphost for an appropriate wshost */
		transportParams.host = self.httpHosts[0];
		self.tryATransport(transportParams, preference, function(fatal, transport) {
			clearTimeout(preferenceTimeout);
			if(preferenceTimeoutExpired && transport) {
				/* Viable, but too late - connectImpl() will already be trying
				* connectBase, and we weren't in upgrade mode. Just remove the
				* onconnected listener and get rid of it */
				transport.off();
				transport.disconnect();
				Utils.arrDeleteValue(this.pendingTransports, transport);
			} else if(!transport && !fatal) {
				/* Preference failed in a transport-specific way. Try more */
				self.unpersistTransportPreference();
				self.connectImpl(transportParams);
			}
			/* If suceeded, or failed fatally, nothing to do */
		});
	};


	/**
	 * Try to establish a transport on the base transport (the best transport
	 * such that if it doesn't work, nothing will work) as determined through
	 * static feature detection, checking for network connectivity and trying
	 * fallback hosts if applicable.
	 * @param transportParams
	 */
	ConnectionManager.prototype.connectBase = function(transportParams) {
		var self = this,
			giveUp = function(err) {
				self.notifyState({state: self.states.connecting.failState, error: err});
			},
			candidateHosts = this.httpHosts.slice(),
			hostAttemptCb = function(fatal, transport) {
				if(!transport && !fatal) {
					tryFallbackHosts();
				}
			};

		Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.connectBase()', 'Trying to connect with base transport ' + this.baseTransport);

		/* first try to establish a connection with the priority host with http transport */
		var host = candidateHosts.shift();
		if(!host) {
			giveUp(new ErrorInfo('Unable to connect (no available host)', 80000, 404));
			return;
		}
		transportParams.host = host;

		/* this is what we'll be doing if the attempt for the main host fails */
		function tryFallbackHosts() {
			/* if there aren't any fallback hosts, fail */
			if(!candidateHosts.length) {
				giveUp(new ErrorInfo('Unable to connect (and no more fallback hosts to try)', 80000, 404));
				return;
			}
			/* before trying any fallback (or any remaining fallback) we decide if
			 * there is a problem with the ably host, or there is a general connectivity
			 * problem */
			Http.checkConnectivity(function(err, connectivity) {
				/* we know err won't happen but handle it here anyway */
				if(err) {
					giveUp(err);
					return;
				}
				if(!connectivity) {
					/* the internet isn't reachable, so don't try the fallback hosts */
					giveUp(new ErrorInfo('Unable to connect (network unreachable)', 80000, 404));
					return;
				}
				/* the network is there, so there's a problem with the main host, or
				 * its dns. Try the fallback hosts. We could try them simultaneously but
				 * that would potentially cause a huge spike in load on the load balancer */
				transportParams.host = Utils.arrPopRandomElement(candidateHosts);
				self.tryATransport(transportParams, self.baseTransport, hostAttemptCb);
			});
		}

		this.tryATransport(transportParams, this.baseTransport, hostAttemptCb);
	};


	ConnectionManager.prototype.getUpgradePossibilities = function() {
		/* returns the subset of upgradeTransports to the right of the current
		 * transport in upgradeTransports (if it's in there - if not, currentPosition
		 * will be -1, so return upgradeTransports.slice(0) == upgradeTransports */
		var current = this.activeProtocol.getTransport().shortName;
		var currentPosition = Utils.arrIndexOf(this.upgradeTransports, current);
		return this.upgradeTransports.slice(currentPosition + 1);
	};


	ConnectionManager.prototype.upgradeIfNeeded = function(transportParams) {
		var upgradePossibilities = this.getUpgradePossibilities(),
			self = this;
		Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.upgradeIfNeeded()', 'upgrade possibilities: ' + Utils.inspect(upgradePossibilities));

		if(!upgradePossibilities.length) {
			return;
		}

		Utils.arrForEach(upgradePossibilities, function(upgradeTransport) {
			/* Note: the transport may mutate the params, so give each transport a fresh one */
			var upgradeTransportParams = self.createTransportParams(transportParams.host, 'upgrade');
			self.tryATransport(upgradeTransportParams, upgradeTransport, noop);
		});
	};


	ConnectionManager.prototype.closeImpl = function() {
		Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.closeImpl()', 'closing connection');
		this.cancelSuspendTimer();
		this.startTransitionTimer(this.states.closing);

		Utils.safeArrForEach(this.pendingTransports, function(transport) {
			Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.closeImpl()', 'Closing pending transport: ' + transport);
			if(transport) transport.close();
		});

		Utils.safeArrForEach(this.proposedTransports, function(transport) {
			Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.closeImpl()', 'Disposing of proposed transport: ' + transport);
			if(transport) transport.dispose();
		});

		if(this.activeProtocol) {
			Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.closeImpl()', 'Closing active transport: ' + this.activeProtocol.getTransport());
			this.activeProtocol.getTransport().close();
		}

		/* If there was an active transport, this will probably be
		 * preempted by the notifyState call in deactivateTransport */
		this.notifyState({state: 'closed'});
	};

	ConnectionManager.prototype.onAuthUpdated = function(tokenDetails, callback) {
		var self = this;
		switch(this.state.state) {
			case 'connected':
				Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.onAuthUpdated()', 'Sending AUTH message on active transport');
				/* If there are any proposed/pending transports (eg an upgrade that
				 * isn't yet scheduled for activation) that hasn't yet started syncing,
				 * just to get rid of them & restart the upgrade with the new token, to
				 * avoid a race condition. (If it has started syncing, the AUTH will be
				 * queued until the upgrade is complete, so everything's fine) */
				if((this.pendingTransports.length || this.proposedTransports.length) &&
					self.state !== self.states.synchronizing) {
					this.disconnectAllTransports(/* exceptActive: */true);
					var transportParams = this.activeProtocol.getTransport().params;
					Utils.nextTick(function() {
						if(self.state.state === 'connected') {
							self.upgradeIfNeeded(transportParams);
						}
					});
				}

				/* Do any transport-specific new-token action */
				this.activeProtocol.getTransport().onAuthUpdated(tokenDetails);

				var authMsg = ProtocolMessage.fromValues({
					action: actions.AUTH,
					auth: {
						accessToken: tokenDetails.token
					}
				});
				this.send(authMsg);

				/* The answer will come back as either a connectiondetails event
				 * (realtime sends a CONNECTED to asknowledge the reauth) or a
				 * statechange to failed */
				var successListener = function() {
					self.off(failureListener);
					callback(null, tokenDetails);
				};
				var failureListener = function(stateChange) {
					if(stateChange.current === 'failed') {
						self.off(successListener);
						self.off(failureListener);
						callback(stateChange.reason || self.getStateError());
					}
				};
				this.once('connectiondetails', successListener);
				this.on('connectionstate', failureListener);
				break;

			case 'connecting':
				Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.onAuthUpdated()',
					'Aborting current connection attempts in order to start again with the new auth details');
				this.disconnectAllTransports();
				/* fallthrough to add statechange listener */

			default:
				Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.onAuthUpdated()',
					'Connection state is ' + this.state.state + '; waiting until either connected or failed');
				var listener = function(stateChange) {
					switch(stateChange.current) {
						case 'connected':
							self.off(listener);
							callback(null, tokenDetails);
							break;
						case 'failed':
						case 'closed':
						case 'suspended':
							self.off(listener);
							callback(stateChange.reason || self.getStateError());
							break;
						default:
							/* ignore till we get either connected or failed */
							break;
					}
				};
				self.on('connectionstate', listener);
				if(this.state.state === 'connecting') {
					/* can happen if in the connecting state but no transport was pending
					 * yet, so disconnectAllTransports did not trigger a disconnected state */
					self.startConnect();
				} else {
					self.requestState({state: 'connecting'});
				}
		}
	};

	ConnectionManager.prototype.disconnectAllTransports = function(exceptActive) {
		Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.disconnectAllTransports()', 'Disconnecting all transports' + (exceptActive ? ' except the active transport' : ''));

		Utils.safeArrForEach(this.pendingTransports, function(transport) {
			Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.disconnectAllTransports()', 'Disconnecting pending transport: ' + transport);
			if(transport) transport.disconnect();
		});
		this.pendingTransports = [];

		Utils.safeArrForEach(this.proposedTransports, function(transport) {
			Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.disconnectAllTransports()', 'Disposing of proposed transport: ' + transport);
			if(transport) transport.dispose();
		});
		this.proposedTransports = [];

		if(this.activeProtocol && !exceptActive) {
			Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.disconnectAllTransports()', 'Disconnecting active transport: ' + this.activeProtocol.getTransport());
			this.activeProtocol.getTransport().disconnect();
		}
		/* No need to notify state disconnected; disconnecting the active transport
		 * will have that effect */
	};

	/******************
	 * event queueing
	 ******************/

	ConnectionManager.prototype.send = function(msg, queueEvent, callback) {
		callback = callback || noop;
		var state = this.state;

		if(state.sendEvents) {
			Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.send()', 'sending event');
			this.sendImpl(new PendingMessage(msg, callback));
			return;
		}
		var shouldQueue = (queueEvent && state.queueEvents) || state.forceQueueEvents;
		if(!shouldQueue) {
			var err = 'rejecting event, queueEvent was ' + queueEvent + ', state was ' + state.state;
			Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.send()', err);
			callback(this.errorReason || new ErrorInfo(err, 90000, 400));
			return;
		}
		if(Logger.shouldLog(Logger.LOG_MICRO)) {
			Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.send()', 'queueing msg; ' + ProtocolMessage.stringify(msg));
		}
		this.queue(msg, callback);
	};

	ConnectionManager.prototype.sendImpl = function(pendingMessage) {
		var msg = pendingMessage.message;
		/* If have already attempted to send this, resend with the same msgSerial,
		 * so Ably can dedup if the previous send succeeded */
		if(pendingMessage.ackRequired && !pendingMessage.sendAttempted) {
			msg.msgSerial = this.msgSerial++;
			this.setRecoveryKey();
		}
		try {
			this.activeProtocol.send(pendingMessage);
		} catch(e) {
			Logger.logAction(Logger.LOG_ERROR, 'ConnectionManager.sendImpl()', 'Unexpected exception in transport.send(): ' + e.stack);
		}
	};

	ConnectionManager.prototype.queue = function(msg, callback) {
		Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.queue()', 'queueing event');
		var lastQueued = this.queuedMessages.last();
		/* If have already attempted to send a message, don't merge more messages
		 * into it, as if the previous send actually succeeded and realtime ignores
		 * the dup, they'll be lost */
		if(lastQueued && !lastQueued.sendAttempted && RealtimeChannel.mergeTo(lastQueued.message, msg)) {
			if(!lastQueued.merged) {
				lastQueued.callback = Multicaster([lastQueued.callback]);
				lastQueued.merged = true;
			}
			lastQueued.callback.push(callback);
		} else {
			this.queuedMessages.push(new PendingMessage(msg, callback));
		}
	};

	ConnectionManager.prototype.sendQueuedMessages = function() {
		Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.sendQueuedMessages()', 'sending ' + this.queuedMessages.count() + ' queued messages');
		var pendingMessage;
		while(pendingMessage = this.queuedMessages.shift())
			this.sendImpl(pendingMessage);
	};

	ConnectionManager.prototype.queuePendingMessages = function(pendingMessages) {
		if(pendingMessages && pendingMessages.length) {
			Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.queuePendingMessages()', 'queueing ' + pendingMessages.length + ' pending messages');
			this.queuedMessages.prepend(pendingMessages);
		}
	};

	ConnectionManager.prototype.failQueuedMessages = function(err) {
		var numQueued = this.queuedMessages.count();
		if(numQueued > 0) {
			Logger.logAction(Logger.LOG_ERROR, 'ConnectionManager.failQueuedMessages()', 'failing ' + numQueued + ' queued messages, err = ' + Utils.inspectError(err));
			this.queuedMessages.completeAllMessages(err);
		}
	};

	ConnectionManager.prototype.onChannelMessage = function(message, transport) {
		var onActiveTransport = this.activeProtocol && transport === this.activeProtocol.getTransport(),
			onUpgradeTransport = Utils.arrIn(this.pendingTransports, transport) && this.state == this.states.synchronizing;

		/* As the lib now has a period where the upgrade transport is synced but
		 * before it's become active (while waiting for the old one to become
		 * idle), message can validly arrive on it even though it isn't active */
		if(onActiveTransport || onUpgradeTransport) {
			this.setConnectionSerial(message);
			var msgId = message.id;
			if(msgId && (msgId === this.mostRecentMsgId)) {
				Logger.logAction(Logger.LOG_ERROR, 'ConnectionManager.onChannelMessage() received message with different connectionSerial, but same message id as a previous; discarding; id = ' + msgId);
				return;
			}
			this.mostRecentMsgId = msgId;
			this.realtime.channels.onChannelMessage(message);
		} else {
			// Message came in on a defunct transport. Allow only acks, nacks, & errors for outstanding
			// messages,  no new messages (as sync has been sent on new transport so new messages will
			// be resent there, or connection has been closed so don't want new messages)
			if(Utils.arrIndexOf([actions.ACK, actions.NACK, actions.ERROR], message.action) > -1) {
				this.realtime.channels.onChannelMessage(message);
			} else {
				Logger.logAction(Logger.LOG_MICRO, 'ConnectionManager.onChannelMessage()', 'received message ' + JSON.stringify(message) + 'on defunct transport; discarding');
			}
		}
	};

	ConnectionManager.prototype.ping = function(transport, callback) {
		/* if transport is specified, try that */
		if(transport) {
			Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.ping()', 'transport = ' + transport);

			var onTimeout = function () {
				transport.off('heartbeat', onHeartbeat);
				callback(new ErrorInfo('Timeout waiting for heartbeat response', 50000, 500));
			};

			var pingStart = Utils.now(),
				id = Utils.randStr();

			var onHeartbeat = function (responseId) {
				if(responseId === id) {
					clearTimeout(timer);
					var responseTime = Utils.now() - pingStart;
					callback(null, responseTime);
				}
			};

			var timer = setTimeout(onTimeout, this.options.timeouts.realtimeRequestTimeout);

			transport.once('heartbeat', onHeartbeat);
			transport.ping(id);
			return;
		}

		/* if we're not connected, don't attempt */
		if(this.state.state !== 'connected') {
			callback(new ErrorInfo('Unable to ping service; not connected', 40000, 400));
			return;
		}

		/* no transport was specified, so use the current (connected) one
		 * but ensure that we retry if the transport is superseded before we complete */
		var completed = false, self = this;

		var onPingComplete = function(err, responseTime) {
			self.off('transport.active', onTransportActive);
			if(!completed) {
				completed = true;
				callback(err, responseTime);
			}
		};

		var onTransportActive = function() {
			if(!completed) {
				/* ensure that no callback happens for the currently outstanding operation */
				completed = true;
				/* repeat but picking up the new transport */
				Utils.nextTick(function() {
					self.ping(null, callback);
				});
			}
		};

		this.on('transport.active', onTransportActive);
		this.ping(this.activeProtocol.getTransport(), onPingComplete);
	};

	ConnectionManager.prototype.abort = function(error) {
		this.activeProtocol.getTransport().fail(error);
	};

	ConnectionManager.prototype.registerProposedTransport = function(transport) {
		this.proposedTransports.push(transport);
	};

	ConnectionManager.prototype.getTransportPreference = function() {
		return this.transportPreference || (haveWebStorage && WebStorage.get(transportPreferenceName));
	};

	ConnectionManager.prototype.persistTransportPreference = function(transport) {
		if(Utils.arrIn(Defaults.upgradeTransports, transport.shortName)) {
			this.transportPreference = transport.shortName;
			if(haveWebStorage) {
				WebStorage.set(transportPreferenceName, transport.shortName);
			}
		}
	};

	ConnectionManager.prototype.unpersistTransportPreference = function() {
		this.transportPreference = null;
		if(haveWebStorage) {
			WebStorage.remove(transportPreferenceName);
		}
	};

	/* This method is only used during connection attempts, so implements RSA4c1,
	 * RSA4c2, and RSA4d. In particular it is not invoked for
	 * serverside-triggered reauths or manual reauths, so RSA4c3 does not apply */
	ConnectionManager.prototype.actOnErrorFromAuthorize = function(err) {
		if(err.statusCode === 403) {
			var msg = 'Client configured authentication provider returned 403; failing the connection';
			Logger.logAction(Logger.LOG_ERROR, 'ConnectionManager.actOnErrorFromAuthorize()', msg);
			this.notifyState({state: 'failed', error: new ErrorInfo(msg, 80019, 403, err)});
		} else {
			var msg = 'Client configured authentication provider request failed';
			Logger.logAction(Logger.LOG_MINOR, 'ConnectionManager.actOnErrorFromAuthorize', msg);
			this.notifyState({state: this.state.failState, error: new ErrorInfo(msg, 80019, 401, err)});
		}
	};

	ConnectionManager.prototype.onConnectionDetailsUpdate = function(connectionDetails, transport) {
		if(!connectionDetails) {
			return;
		}
		this.connectionDetails = connectionDetails;
		var clientId = connectionDetails.clientId;
		if(clientId) {
			var err = this.realtime.auth._uncheckedSetClientId(clientId);
			if(err) {
				Logger.logAction(Logger.LOG_ERROR, 'ConnectionManager.onConnectionDetailsUpdate()', err.message);
				/* Errors setting the clientId are fatal to the connection */
				transport.fail(err);
				return;
			}
		}
		var connectionStateTtl = connectionDetails.connectionStateTtl;
		if(connectionStateTtl) {
			this.connectionStateTtl = connectionStateTtl;
		}
		this.maxIdleInterval = connectionDetails.maxIdleInterval;
		this.emit('connectiondetails', connectionDetails);
	};

	return ConnectionManager;
})();

var Transport = (function() {
	var actions = ProtocolMessage.Action;
	var closeMessage = ProtocolMessage.fromValues({action: actions.CLOSE});
	var disconnectMessage = ProtocolMessage.fromValues({action: actions.DISCONNECT});
	var noop = function() {};

	/*
	 * EventEmitter, generates the following events:
	 *
	 * event name       data
	 * closed           error
	 * failed           error
	 * disposed
	 * connected        null error, connectionSerial, connectionId, connectionDetails
	 * sync             connectionSerial, connectionId
	 * event            channel message object
	 */

	/* public constructor */
	function Transport(connectionManager, auth, params) {
		EventEmitter.call(this);
		this.connectionManager = connectionManager;
		connectionManager.registerProposedTransport(this);
		this.auth = auth;
		this.params = params;
		this.timeouts = params.options.timeouts;
		this.format = params.format;
		this.isConnected = false;
		this.isFinished = false;
		this.maxIdleInterval = null;
		this.idleTimer = null;
		this.lastActivity = null;
	}
	Utils.inherits(Transport, EventEmitter);

	Transport.prototype.connect = function() {};

	Transport.prototype.close = function() {
		if(this.isConnected) {
			this.requestClose();
		}
		this.finish('closed', ConnectionError.closed);
	};

	Transport.prototype.disconnect = function(err) {
		/* Used for network/transport issues that need to result in the transport
		 * being disconnected, but should not affect the connection */
		if(this.isConnected) {
			this.requestDisconnect();
		}
		this.finish('disconnected', err || ConnectionError.disconnected);
	};

	Transport.prototype.fail = function(err) {
		/* Used for client-side-detected fatal connection issues */
		if(this.isConnected) {
			this.requestDisconnect();
		}
		this.finish('failed', err || ConnectionError.failed);
	};

	Transport.prototype.finish = function(event, err) {
		if(this.isFinished) {
			return;
		}

		this.isFinished = true;
		this.isConnected = false;
		this.maxIdleInterval = null;
		clearTimeout(this.idleTimer);
		this.idleTimer = null;
		this.emit(event, err);
		this.dispose();
	};

	Transport.prototype.onProtocolMessage = function(message) {
		if (Logger.shouldLog(Logger.LOG_MICRO)) {
			Logger.logAction(Logger.LOG_MICRO, 'Transport.onProtocolMessage()', 'received on ' + this.shortName + ': ' + ProtocolMessage.stringify(message) + '; connectionId = ' + this.connectionManager.connectionId);
		}
		this.onActivity();

		switch(message.action) {
		case actions.HEARTBEAT:
			Logger.logAction(Logger.LOG_MICRO, 'Transport.onProtocolMessage()', this.shortName + ' heartbeat; connectionId = ' + this.connectionManager.connectionId);
			this.emit('heartbeat', message.id);
			break;
		case actions.CONNECTED:
			this.onConnect(message);
			this.emit('connected', message.error, message.connectionId, message.connectionDetails, message);
			break;
		case actions.CLOSED:
			this.onClose(message);
			break;
		case actions.DISCONNECTED:
			this.onDisconnect(message);
			break;
		case actions.ACK:
			this.emit('ack', message.msgSerial, message.count);
			break;
		case actions.NACK:
			this.emit('nack', message.msgSerial, message.count, message.error);
			break;
		case actions.SYNC:
			if(message.connectionId !== undefined) {
				/* a transport SYNC */
				this.emit('sync', message.connectionId, message);
				break;
			}
			/* otherwise it's a channel SYNC, so handle it in the channel */
			this.connectionManager.onChannelMessage(message, this);
			break;
		case actions.AUTH:
			this.auth.authorize(function(err) {
				if(err) {
					Logger.logAction(Logger.LOG_ERROR, 'Transport.onProtocolMessage()', 'Ably requested re-authentication, but unable to obtain a new token: ' + Utils.inspectError(err));
				}
			});
			break;
		case actions.ERROR:
			Logger.logAction(Logger.LOG_MINOR, 'Transport.onProtocolMessage()', 'received error action; connectionId = ' + this.connectionManager.connectionId + '; err = ' + Utils.inspect(message.error) + (message.channel ? (', channel: ' +  message.channel) : ''));
			if(message.channel === undefined) {
				this.onFatalError(message);
				break;
			}
			/* otherwise it's a channel-specific error, so handle it in the channel */
			this.connectionManager.onChannelMessage(message, this);
			break;
		default:
			/* all other actions are channel-specific */
			this.connectionManager.onChannelMessage(message, this);
		}
	};

	Transport.prototype.onConnect = function(message) {
		this.isConnected = true;
		var maxPromisedIdle = message.connectionDetails.maxIdleInterval;
		if(maxPromisedIdle) {
			this.maxIdleInterval = maxPromisedIdle + this.timeouts.realtimeRequestTimeout;
			this.onActivity();
		}
		/* else Realtime declines to guarantee any maximum idle interval - CD2h */
	};

	Transport.prototype.onDisconnect = function(message) {
		/* Used for when the server has disconnected the client (usually with a
		 * DISCONNECTED action) */
		var err = message && message.error;
		Logger.logAction(Logger.LOG_MINOR, 'Transport.onDisconnect()', 'err = ' + Utils.inspectError(err));
		this.finish('disconnected', err);
	};

	Transport.prototype.onFatalError = function(message) {
		/* On receipt of a fatal connection error, we can assume that the server
		 * will close the connection and the transport, and do not need to request
		 * a disconnection - RTN15i */
		var err = message && message.error;
		Logger.logAction(Logger.LOG_MINOR, 'Transport.onFatalError()', 'err = ' + Utils.inspectError(err));
		this.finish('failed', err);
	};

	Transport.prototype.onClose = function(message) {
		var err = message && message.error;
		Logger.logAction(Logger.LOG_MINOR, 'Transport.onClose()', 'err = ' + Utils.inspectError(err));
		this.finish('closed', err);
	};

	Transport.prototype.requestClose = function() {
		Logger.logAction(Logger.LOG_MINOR, 'Transport.requestClose()', '');
		this.send(closeMessage);
	};

	Transport.prototype.requestDisconnect = function() {
		Logger.logAction(Logger.LOG_MINOR, 'Transport.requestDisconnect()', '');
		this.send(disconnectMessage);
	};

	Transport.prototype.ping = function(id) {
		var msg = {action: ProtocolMessage.Action.HEARTBEAT};
		if(id) msg.id = id;
		this.send(ProtocolMessage.fromValues(msg));
	};

	Transport.prototype.dispose = function() {
		Logger.logAction(Logger.LOG_MINOR, 'Transport.dispose()', '');
		this.off();
	};

	Transport.prototype.onActivity = function() {
		if(!this.maxIdleInterval) { return; }
		this.lastActivity = this.connectionManager.lastActivity = Utils.now();
		this.setIdleTimer(this.maxIdleInterval + 100);
	};

	Transport.prototype.setIdleTimer = function(timeout) {
		var self = this;
		if(!this.idleTimer) {
			this.idleTimer = setTimeout(function() {
				self.onIdleTimerExpire();
			}, timeout);
		}
	};

	Transport.prototype.onIdleTimerExpire = function() {
		this.idleTimer = null;
		var sinceLast = Utils.now() - this.lastActivity,
			timeRemaining = this.maxIdleInterval - sinceLast;
		if(timeRemaining <= 0) {
			var msg = 'No activity seen from realtime in ' + sinceLast + 'ms; assuming connection has dropped';
			Logger.logAction(Logger.LOG_ERROR, 'Transport.onIdleTimerExpire()', msg);
			this.disconnect(new ErrorInfo(msg, 80003, 408));
		} else {
			this.setIdleTimer(timeRemaining + 100);
		}
	};

	Transport.prototype.onAuthUpdated = function() {};

	return Transport;
})();

var WebSocketTransport = (function() {
	var WebSocket = Platform.WebSocket;
	var shortName = 'web_socket';

	/* public constructor */
	function WebSocketTransport(connectionManager, auth, params) {
		this.shortName = shortName;
		/* If is a browser, can't detect pings, so request protocol heartbeats */
		params.heartbeats = Platform.useProtocolHeartbeats;
		Transport.call(this, connectionManager, auth, params);
		this.wsHost = Defaults.getHost(params.options, params.host, true);
	}
	Utils.inherits(WebSocketTransport, Transport);

	WebSocketTransport.isAvailable = function() {
		return !!WebSocket;
	};

	if(WebSocketTransport.isAvailable())
		ConnectionManager.supportedTransports[shortName] = WebSocketTransport;

	WebSocketTransport.tryConnect = function(connectionManager, auth, params, callback) {
		var transport = new WebSocketTransport(connectionManager, auth, params);
		var errorCb = function(err) { callback({event: this.event, error: err}); };
		transport.on(['failed', 'disconnected'], errorCb);
		transport.on('wsopen', function() {
			Logger.logAction(Logger.LOG_MINOR, 'WebSocketTransport.tryConnect()', 'viable transport ' + transport);
			transport.off(['failed', 'disconnected'], errorCb);
			callback(null, transport);
		});
		transport.connect();
	};

	WebSocketTransport.prototype.createWebSocket = function(uri, connectParams) {
		var paramCount = 0;
		if(connectParams) {
			for(var key in connectParams)
				uri += (paramCount++ ? '&' : '?') + key + '=' + connectParams[key];
		}
		this.uri = uri;
		return new WebSocket(uri);
	};

	WebSocketTransport.prototype.toString = function() {
		return 'WebSocketTransport; uri=' + this.uri;
	};

	WebSocketTransport.prototype.connect = function() {
		Logger.logAction(Logger.LOG_MINOR, 'WebSocketTransport.connect()', 'starting');
		Transport.prototype.connect.call(this);
		var self = this, params = this.params, options = params.options;
		var wsScheme = options.tls ? 'wss://' : 'ws://';
		var wsUri = wsScheme + this.wsHost + ':' + Defaults.getPort(options) + '/';
		Logger.logAction(Logger.LOG_MINOR, 'WebSocketTransport.connect()', 'uri: ' + wsUri);
		this.auth.getAuthParams(function(err, authParams) {
			var paramStr = ''; for(var param in authParams) paramStr += ' ' + param + ': ' + authParams[param] + ';';
			Logger.logAction(Logger.LOG_MINOR, 'WebSocketTransport.connect()', 'authParams:' + paramStr + ' err: ' + err);
			if(err) {
				self.disconnect(err);
				return;
			}
			var connectParams = params.getConnectParams(authParams);
			try {
				var wsConnection = self.wsConnection = self.createWebSocket(wsUri, connectParams);
				wsConnection.binaryType = Platform.binaryType;
				wsConnection.onopen = function() { self.onWsOpen(); };
				wsConnection.onclose = function(ev) { self.onWsClose(ev); };
				wsConnection.onmessage = function(ev) { self.onWsData(ev.data); };
				wsConnection.onerror = function(ev) { self.onWsError(ev); };
				if(wsConnection.on) {
					/* node; browsers currently don't have a general eventemitter and can't detect
					 * pings. Also, no need to reply with a pong explicitly, ws lib handles that */
					wsConnection.on('ping', function() { self.onActivity(); });
				}
			} catch(e) {
				Logger.logAction(Logger.LOG_ERROR, 'WebSocketTransport.connect()', 'Unexpected exception creating websocket: err = ' + (e.stack || e.message));
				self.disconnect(e);
			}
		});
	};

	WebSocketTransport.prototype.send = function(message) {
		var wsConnection = this.wsConnection;
		if(!wsConnection) {
			Logger.logAction(Logger.LOG_ERROR, 'WebSocketTransport.send()', 'No socket connection');
			return;
		}
		try {
			wsConnection.send(ProtocolMessage.serialize(message, this.params.format));
		} catch (e) {
			var msg = 'Exception from ws connection when trying to send: ' + Utils.inspectError(e);
			Logger.logAction(Logger.LOG_ERROR, 'WebSocketTransport.send()', msg);
			/* Don't try to request a disconnect, that'll just involve sending data
			 * down the websocket again. Just finish the transport. */
			this.finish('disconnected', new ErrorInfo(msg, 50000, 500));
		}
	};

	WebSocketTransport.prototype.onWsData = function(data) {
		Logger.logAction(Logger.LOG_MICRO, 'WebSocketTransport.onWsData()', 'data received; length = ' + data.length + '; type = ' + typeof(data));
		try {
			this.onProtocolMessage(ProtocolMessage.deserialize(data, this.format));
		} catch (e) {
			Logger.logAction(Logger.LOG_ERROR, 'WebSocketTransport.onWsData()', 'Unexpected exception handing channel message: ' + e.stack);
		}
	};

	WebSocketTransport.prototype.onWsOpen = function() {
		Logger.logAction(Logger.LOG_MINOR, 'WebSocketTransport.onWsOpen()', 'opened WebSocket');
		this.emit('wsopen');
	};

	WebSocketTransport.prototype.onWsClose = function(ev) {
		var wasClean, code, reason;
		if(typeof(ev) == 'object') {
			/* W3C spec-compatible */
			wasClean = ev.wasClean;
			code = ev.code;
		} else /*if(typeof(ev) == 'number')*/ {
			/* ws in node */
			code = ev;
			wasClean = (code == 1000);
		}
		delete this.wsConnection;
		if(wasClean) {
			Logger.logAction(Logger.LOG_MINOR, 'WebSocketTransport.onWsClose()', 'Cleanly closed WebSocket');
			var err = new ErrorInfo('Websocket closed', 80003, 400);
			this.finish('disconnected', err);
		} else {
			var msg = 'Unclean disconnection of WebSocket ; code = ' + code,
				err = new ErrorInfo(msg, 80003, 400);
			Logger.logAction(Logger.LOG_MINOR, 'WebSocketTransport.onWsClose()', msg);
			this.finish('disconnected', err);
		}
		this.emit('disposed');
	};

	WebSocketTransport.prototype.onWsError = function(err) {
		Logger.logAction(Logger.LOG_MINOR, 'WebSocketTransport.onError()', 'Error from WebSocket: ' + err.message);
		/* Wait a tick before aborting: if the websocket was connected, this event
		 * will be immediately followed by an onclose event with a close code. Allow
		 * that to close it (so we see the close code) rather than anticipating it */
		var self = this;
		Utils.nextTick(function() {
			self.disconnect(err);
		});
	};

	WebSocketTransport.prototype.dispose = function() {
		Logger.logAction(Logger.LOG_MINOR, 'WebSocketTransport.dispose()', '');
		var wsConnection = this.wsConnection;
		if(wsConnection) {
			/* Ignore any messages that come through after dispose() is called but before
			 * websocket is actually closed. (mostly would be harmless, but if it's a
			 * CONNECTED, it'll re-tick isConnected and cause all sorts of havoc) */
			wsConnection.onmessage = function() {};
			delete this.wsConnection;
			/* defer until the next event loop cycle before closing the socket,
			 * giving some implementations the opportunity to send any outstanding close message */
			Utils.nextTick(function() {
				Logger.logAction(Logger.LOG_MICRO, 'WebSocketTransport.dispose()', 'closing websocket');
				wsConnection.close();
			});
		}
	};

	return WebSocketTransport;
})();

var CometTransport = (function() {

	var REQ_SEND = 0,
		REQ_RECV = 1,
		REQ_RECV_POLL = 2,
		REQ_RECV_STREAM = 3;

	function actOnConnectHeaders(headers, host, connectionManager) {
		if(headers && headers.server && (headers.server.indexOf('cloudflare') > -1)) {
			/* Cloudflare doesn't support xhr streaming */
			var blacklist = connectionManager.transportHostBlacklist[host];
			if(!blacklist) {
				connectionManager.transportHostBlacklist[host] = ['xhr_streaming'];
				return;
			}
			if(!Utils.arrIn(blacklist, 'xhr_streaming')) {
				blacklist.push('xhr_streaming');
			}
		}
	}

	/* TODO: can remove once realtime sends protocol message responses for comet errors */
	function shouldBeErrorAction(err) {
		var UNRESOLVABLE_ERROR_CODES = [80015, 80017, 80030];
		if(err.code) {
			if(Auth.isTokenErr(err)) return false;
			if(Utils.arrIn(UNRESOLVABLE_ERROR_CODES, err.code)) return true;
			return (err.code >= 40000 && err.code < 50000);
		} else {
			/* Likely a network or transport error of some kind. Certainly not fatal to the connection */
			return false;
		}
	}

	function protocolMessageFromRawError(err) {
		/* err will be either a legacy (non-protocolmessage) comet error response
		 * (which will have an err.code), or a xhr/network error (which won't). */
		if(shouldBeErrorAction(err)) {
			return [ProtocolMessage.fromValues({action: ProtocolMessage.Action.ERROR, error: err})];
		} else {
			return [ProtocolMessage.fromValues({action: ProtocolMessage.Action.DISCONNECTED, error: err})];
		}
	}

	/*
	 * A base comet transport class
	 */
	function CometTransport(connectionManager, auth, params) {
		/* binary not supported for comet, so just fall back to default */
		params.format = undefined;
		params.heartbeats = true;
		Transport.call(this, connectionManager, auth, params);
		/* streaming defaults to true */
		this.stream = ('stream' in params) ? params.stream : true;
		this.sendRequest = null;
		this.recvRequest = null;
		this.pendingCallback = null;
		this.pendingItems = null;
		this.disposed = false;
	}
	Utils.inherits(CometTransport, Transport);

	CometTransport.REQ_SEND = REQ_SEND;
	CometTransport.REQ_RECV = REQ_RECV;
	CometTransport.REQ_RECV_POLL = REQ_RECV_POLL;
	CometTransport.REQ_RECV_STREAM = REQ_RECV_STREAM;

	/* public instance methods */
	CometTransport.prototype.connect = function() {
		Logger.logAction(Logger.LOG_MINOR, 'CometTransport.connect()', 'starting');
		Transport.prototype.connect.call(this);
		var self = this, params = this.params, options = params.options;
		var host = Defaults.getHost(options, params.host);
		var port = Defaults.getPort(options);
		var cometScheme = options.tls ? 'https://' : 'http://';

		this.baseUri = cometScheme + host + ':' + port + '/comet/';
		var connectUri = this.baseUri + 'connect';
		Logger.logAction(Logger.LOG_MINOR, 'CometTransport.connect()', 'uri: ' + connectUri);
		this.auth.getAuthParams(function(err, authParams) {
			if(err) {
				self.disconnect(err);
				return;
			}
			self.authParams = authParams;
			var connectParams = self.params.getConnectParams(authParams);
			if('stream' in connectParams) self.stream = connectParams.stream;
			Logger.logAction(Logger.LOG_MINOR, 'CometTransport.connect()', 'connectParams:' + Utils.toQueryString(connectParams));

			/* this will be the 'recvRequest' so this connection can stream messages */
			var preconnected = false,
				connectRequest = self.recvRequest = self.createRequest(connectUri, null, connectParams, null, (self.stream ? REQ_RECV_STREAM : REQ_RECV));

			connectRequest.on('data', function(data) {
				if(!self.recvRequest) {
					/* the transport was disposed before we connected */
					return;
				}
				if(!preconnected) {
					preconnected = true;
					self.emit('preconnect');
				}
				self.onData(data);
			});
			connectRequest.on('complete', function(err, _body, headers) {
				actOnConnectHeaders(headers, host, self.connectionManager);
				if(!self.recvRequest) {
					/* the transport was disposed before we connected */
					err = err || new ErrorInfo('Request cancelled', 80000, 400);
				}
				self.recvRequest = null;
				self.onActivity();
				if(err) {
					if(err.code) {
						/* A protocol error received from realtime. TODO: once realtime
						 * consistendly sends errors wrapped in protocol messages, should be
						 * able to remove this */
						self.onData(protocolMessageFromRawError(err));
					} else {
						/* A network/xhr error. Don't bother wrapping in a protocol message,
						 * just disconnect the transport */
						self.disconnect(err);
					}
					return;
				}
				Utils.nextTick(function() {
					self.recv();
				});
			});
			connectRequest.exec();
		});
	};

	CometTransport.prototype.requestClose = function() {
		Logger.logAction(Logger.LOG_MINOR, 'CometTransport.requestClose()');
		this._requestCloseOrDisconnect(true);
	};

	CometTransport.prototype.requestDisconnect = function() {
		Logger.logAction(Logger.LOG_MINOR, 'CometTransport.requestDisconnect()');
		this._requestCloseOrDisconnect(false);
	};

	CometTransport.prototype._requestCloseOrDisconnect = function(closing) {
		var closeOrDisconnectUri = closing ? this.closeUri : this.disconnectUri;
		if(closeOrDisconnectUri) {
			var self = this,
				request = this.createRequest(closeOrDisconnectUri, null, this.authParams, null, REQ_SEND);

			request.on('complete', function (err) {
				if(err) {
					Logger.logAction(Logger.LOG_ERROR, 'CometTransport.request' + (closing ? 'Close()' : 'Disconnect()'), 'request returned err = ' + err);
					self.finish('disconnected', err);
				}
			});
			request.exec();
		}
	};

	CometTransport.prototype.dispose = function() {
		Logger.logAction(Logger.LOG_MINOR, 'CometTransport.dispose()', '');
		if(!this.disposed) {
			this.disposed = true;
			if(this.recvRequest) {
				Logger.logAction(Logger.LOG_MINOR, 'CometTransport.dispose()', 'aborting recv request');
				this.recvRequest.abort();
				this.recvRequest = null;
			}
			/* In almost all cases the transport will be finished before it's
			 * disposed. Finish here just to make sure. */
			this.finish('disconnected', ConnectionError.disconnected);
			var self = this;
			Utils.nextTick(function() {
				self.emit('disposed');
			});
		}
	};

	CometTransport.prototype.onConnect = function(message) {
		/* if this transport has been disposed whilst awaiting connection, do nothing */
		if(this.disposed) return;

		/* the connectionKey in a comet connected response is really
		 * <instId>-<connectionKey> */
		var connectionStr = message.connectionKey;
		Transport.prototype.onConnect.call(this, message);

		var baseConnectionUri =  this.baseUri + connectionStr;
		Logger.logAction(Logger.LOG_MICRO, 'CometTransport.onConnect()', 'baseUri = ' + baseConnectionUri + '; connectionKey = ' + message.connectionKey);
		this.sendUri = baseConnectionUri + '/send';
		this.recvUri = baseConnectionUri + '/recv';
		this.closeUri = baseConnectionUri + '/close';
		this.disconnectUri = baseConnectionUri + '/disconnect';
	};

	CometTransport.prototype.send = function(message) {
		if(this.sendRequest) {
			/* there is a pending send, so queue this message */
			this.pendingItems = this.pendingItems || [];
			this.pendingItems.push(message);
			return;
		}
		/* send this, plus any pending, now */
		var pendingItems = this.pendingItems || [];
		pendingItems.push(message);
		this.pendingItems = null;

		this.sendItems(pendingItems);
	};

	CometTransport.prototype.sendAnyPending = function() {
		var pendingItems = this.pendingItems;

		if(!pendingItems) {
			return;
		}

		this.pendingItems = null;
		this.sendItems(pendingItems);
	}

	CometTransport.prototype.sendItems = function(items) {
		var self = this,
			sendRequest = this.sendRequest = self.createRequest(self.sendUri, null, self.authParams, this.encodeRequest(items), REQ_SEND);

		sendRequest.on('complete', function(err, data) {
			if(err) Logger.logAction(Logger.LOG_ERROR, 'CometTransport.sendItems()', 'on complete: err = ' + Utils.inspectError(err));
			self.sendRequest = null;

			/* the results of the request usually get handled as protocol responses instead of send errors */
			if(data) {
				self.onData(data);
			} else if(err && err.code) {
				/* A protocol error received from realtime. TODO: once realtime
				 * consistendly sends errors wrapped in protocol messages, should be
				 * able to remove this */
				self.onData(protocolMessageFromRawError(err));
			} else {
				/* A network/xhr error. Don't bother wrapping in a protocol message,
				 * just disconnect the transport */
				self.disconnect(err);
			}

			if(self.pendingItems) {
				Utils.nextTick(function() {
					/* If there's a new send request by now, any pending items will have
					 * been picked up by that; any new ones added since then will be
					 * picked up after that one completes */
					if(!self.sendRequest) {
						self.sendAnyPending();
					}
				});
			}
		});
		sendRequest.exec();
	};

	CometTransport.prototype.recv = function() {
		/* do nothing if there is an active request, which might be streaming */
		if(this.recvRequest)
			return;

		/* If we're no longer connected, do nothing */
		if(!this.isConnected)
			return;

		var self = this,
			recvRequest = this.recvRequest = this.createRequest(this.recvUri, null, this.authParams, null, (self.stream ? REQ_RECV_STREAM : REQ_RECV_POLL));

		recvRequest.on('data', function(data) {
			self.onData(data);
		});
		recvRequest.on('complete', function(err) {
			self.recvRequest = null;
			/* A request completing must be considered activity, as realtime sends
			 * heartbeats every 15s since a request began, not every 15s absolutely */
			self.onActivity();
			if(err) {
				if(err.code) {
					/* A protocol error received from realtime. TODO: once realtime
					 * consistendly sends errors wrapped in protocol messages, should be
					 * able to remove this */
					self.onData(protocolMessageFromRawError(err));
				} else {
					/* A network/xhr error. Don't bother wrapping in a protocol message,
					 * just disconnect the transport */
					self.disconnect(err);
				}
				return;
			}
			Utils.nextTick(function() {
				self.recv();
			});
		});
		recvRequest.exec();
	};

	CometTransport.prototype.onData = function(responseData) {
		try {
			var items = this.decodeResponse(responseData);
			if(items && items.length)
				for(var i = 0; i < items.length; i++)
					this.onProtocolMessage(ProtocolMessage.fromDeserialized(items[i]));
		} catch (e) {
			Logger.logAction(Logger.LOG_ERROR, 'CometTransport.onData()', 'Unexpected exception handing channel event: ' + e.stack);
		}
	};

	CometTransport.prototype.encodeRequest = function(requestItems) {
		return JSON.stringify(requestItems);
	};

	CometTransport.prototype.decodeResponse = function(responseData) {
		if(typeof(responseData) == 'string')
			responseData = JSON.parse(responseData);
		return responseData;
	};

	/* For comet, we could do the auth update by aborting the current recv and
	 * starting a new one with the new token, that'd be sufficient for realtime.
	 * Problem is JSONP - you can't cancel truly abort a recv once started. So
	 * we need to send an AUTH for jsonp. In which case it's simpler to keep all
	 * comet transports the same and do it for all of them. So we send the AUTH
	 * instead, and don't need to abort the recv */
	CometTransport.prototype.onAuthUpdated = function(tokenDetails) {
		this.authParams = {access_token: tokenDetails.token};
	};

	return CometTransport;
})();

var Presence = (function() {
	function noop() {}
	function Presence(channel) {
		this.channel = channel;
		this.basePath = channel.basePath + '/presence';
	}
	Utils.inherits(Presence, EventEmitter);

	Presence.prototype.get = function(params, callback) {
		Logger.logAction(Logger.LOG_MICRO, 'Presence.get()', 'channel = ' + this.channel.name);
		/* params and callback are optional; see if params contains the callback */
		if(callback === undefined) {
			if(typeof(params) == 'function') {
				callback = params;
				params = null;
			} else {
				callback = noop;
			}
		}
		var rest = this.channel.rest,
			format = rest.options.useBinaryProtocol ? 'msgpack' : 'json',
			envelope = Http.supportsLinkHeaders ? undefined : format,
			headers = Utils.copy(Utils.defaultGetHeaders(format));

		if(rest.options.headers)
			Utils.mixin(headers, rest.options.headers);

		var options = this.channel.channelOptions;
		(new PaginatedResource(rest, this.basePath, headers, envelope, function(body, headers, unpacked) {
			return PresenceMessage.fromResponseBody(body, options, !unpacked && format);
		})).get(params, callback);
	};

	Presence.prototype.history = function(params, callback) {
		Logger.logAction(Logger.LOG_MICRO, 'Presence.history()', 'channel = ' + this.channel.name);
		this._history(params, callback);
	};

	Presence.prototype._history = function(params, callback) {
		/* params and callback are optional; see if params contains the callback */
		if(callback === undefined) {
			if(typeof(params) == 'function') {
				callback = params;
				params = null;
			} else {
				callback = noop;
			}
		}
		var rest = this.channel.rest,
			format = rest.options.useBinaryProtocol ? 'msgpack' : 'json',
			envelope = Http.supportsLinkHeaders ? undefined : format,
			headers = Utils.copy(Utils.defaultGetHeaders(format)),
			channel = this.channel;

		if(rest.options.headers)
			Utils.mixin(headers, rest.options.headers);

		var options = this.channel.channelOptions;
		(new PaginatedResource(rest, this.basePath + '/history', headers, envelope, function(body, headers, unpacked) {
			return PresenceMessage.fromResponseBody(body, options, !unpacked && format);
		})).get(params, callback);
	};

	return Presence;
})();

var Resource = (function() {
	var msgpack = Platform.msgpack;

	function Resource() {}

	function withAuthDetails(rest, headers, params, errCallback, opCallback) {
		if (Http.supportsAuthHeaders) {
			rest.auth.getAuthHeaders(function(err, authHeaders) {
				if(err)
					errCallback(err);
				else
					opCallback(Utils.mixin(authHeaders, headers), params);
			});
		} else {
			rest.auth.getAuthParams(function(err, authParams) {
				if(err)
					errCallback(err);
				else
					opCallback(headers, Utils.mixin(authParams, params));
			});
		}
	}

	function unenvelope(callback, format) {
		return function(err, body, headers, unpacked, statusCode) {
			if(err && !body) {
				callback(err);
				return;
			}

			if(!unpacked) {
				try {
					body = (format == 'msgpack') ? msgpack.decode(body) : JSON.parse(body);
				} catch(e) {
					callback(e);
					return;
				}
			}

			if(body.statusCode === undefined) {
				/* Envelope already unwrapped by the transport */
				callback(err, body, headers, true, statusCode);
				return;
			}

			var statusCode = body.statusCode,
				response = body.response,
				headers = body.headers;

			if(statusCode < 200 || statusCode >= 300) {
				/* handle wrapped errors */
				var wrappedErr = (response && response.error) || err;
				if(!wrappedErr) {
					wrappedErr = new Error("Error in unenveloping " + body);
					wrappedErr.statusCode = statusCode;
				}
				callback(wrappedErr, response, headers, true, statusCode);
				return;
			}

			callback(err, response, headers, true, statusCode);
		};
	}

	function paramString(params) {
		var paramPairs = [];
		if (params) {
			for (var needle in params) {
				paramPairs.push(needle + '=' + params[needle]);
			}
		}
		return paramPairs.join('&');
	}

	function urlFromPathAndParams(path, params) {
		return path + (params ? '?' : '') + paramString(params);
	}

	function logResponseHandler(callback, verb, path, params) {
		return function(err, body, headers, unpacked, statusCode) {
			if (err) {
				Logger.logAction(Logger.LOG_MICRO, 'Resource.' + verb + '()', 'Received Error; ' + urlFromPathAndParams(path, params) + '; Error: ' + Utils.inspectError(err));
			} else {
				Logger.logAction(Logger.LOG_MICRO, 'Resource.' + verb + '()',
					'Received; ' + urlFromPathAndParams(path, params) + '; Headers: ' + paramString(headers) + '; StatusCode: ' + statusCode + '; Body: ' + (BufferUtils.isBuffer(body) ? body.toString() : body));
			}
			if (callback) { callback(err, body, headers, unpacked, statusCode); }
		}
	}

	Resource.get = function(rest, path, origheaders, origparams, envelope, callback) {
		if (Logger.shouldLog(Logger.LOG_MICRO)) {
			callback = logResponseHandler(callback, 'get', path, origparams);
		}

		if(envelope) {
			callback = (callback && unenvelope(callback, envelope));
			(origparams = (origparams || {}))['envelope'] = envelope;
		}

		function doGet(headers, params) {
			if (Logger.shouldLog(Logger.LOG_MICRO)) {
				Logger.logAction(Logger.LOG_MICRO, 'Resource.get()', 'Sending; ' + urlFromPathAndParams(path, params));
			}

			Http.get(rest, path, headers, params, function(err, res, headers, unpacked, statusCode) {
				if(err && Auth.isTokenErr(err)) {
					/* token has expired, so get a new one */
					rest.auth.authorize(null, null, function(err) {
						if(err) {
							callback(err);
							return;
						}
						/* retry ... */
						withAuthDetails(rest, origheaders, origparams, callback, doGet);
					});
					return;
				}
				callback(err, res, headers, unpacked, statusCode);
			});
		}
		withAuthDetails(rest, origheaders, origparams, callback, doGet);
	};

	Resource.post = function(rest, path, body, origheaders, origparams, envelope, callback) {
		if (Logger.shouldLog(Logger.LOG_MICRO)) {
			callback = logResponseHandler(callback, 'post', path, origparams);
		}

		if(envelope) {
			callback = unenvelope(callback, envelope);
			origparams['envelope'] = envelope;
		}

		function doPost(headers, params) {
			if (Logger.shouldLog(Logger.LOG_MICRO)) {
				var decodedBody = body;
				if ((headers['content-type'] || '').indexOf('msgpack') > 0) {
					try {
						decodedBody = msgpack.decode(body);
					} catch (decodeErr) {
						Logger.logAction(Logger.LOG_MICRO, 'Resource.post()', 'Sending MsgPack Decoding Error: ' + Utils.inspectError(decodeErr));
					}
				}
				Logger.logAction(Logger.LOG_MICRO, 'Resource.post()', 'Sending; ' + urlFromPathAndParams(path, params) + '; Body: ' + decodedBody);
			}

			Http.post(rest, path, headers, body, params, function(err, res, headers, unpacked, statusCode) {
				if(err && Auth.isTokenErr(err)) {
					/* token has expired, so get a new one */
					rest.auth.authorize(null, null, function(err) {
						if(err) {
							callback(err);
							return;
						}
						/* retry ... */
						withAuthDetails(rest, origheaders, origparams, callback, doPost);
					});
					return;
				}
				callback(err, res, headers, unpacked, statusCode);
			});
		}
		withAuthDetails(rest, origheaders, origparams, callback, doPost);
	};

	return Resource;
})();

var PaginatedResource = (function() {

	function getRelParams(linkUrl) {
		var urlMatch = linkUrl.match(/^\.\/(\w+)\?(.*)$/);
		return urlMatch && Utils.parseQueryString(urlMatch[2]);
	}

	function parseRelLinks(linkHeader) {
		if(typeof(linkHeader) == 'string')
			linkHeader = linkHeader.split(',');

		var relParams = {};
		for(var i = 0; i < linkHeader.length; i++) {
			var linkMatch = linkHeader[i].match(/^\s*<(.+)>;\s*rel="(\w+)"$/);
			if(linkMatch) {
				var params = getRelParams(linkMatch[1]);
				if(params)
					relParams[linkMatch[2]] = params;
			}
		}
		return relParams;
	}

	function PaginatedResource(rest, path, headers, envelope, bodyHandler, useHttpPaginatedResponse) {
		this.rest = rest;
		this.path = path;
		this.headers = headers;
		this.envelope = envelope;
		this.bodyHandler = bodyHandler;
		this.useHttpPaginatedResponse = useHttpPaginatedResponse || false;
	}

	PaginatedResource.prototype.get = function(params, callback) {
		var self = this;
		Resource.get(self.rest, self.path, self.headers, params, self.envelope, function(err, body, headers, unpacked, statusCode) {
			self.handlePage(err, body, headers, unpacked, statusCode, callback);
		});
	};

	PaginatedResource.prototype.post = function(params, body, callback) {
		var self = this;
		Resource.post(self.rest, self.path, body, self.headers, params, self.envelope, function(err, resbody, headers, unpacked, statusCode) {
			if(callback) self.handlePage(err, resbody, headers, unpacked, statusCode, callback);
		});
	};

	function returnErrOnly(err, body, useHPR) {
		/* If using httpPaginatedResponse, errors from Ably are returned as part of
		 * the HPR, only do callback(err) for network errors etc. which don't
		 * return a body and/or have no ably-originated error code (non-numeric
		 * error codes originate from node) */
		return !(useHPR && (body || typeof err.code === 'number'));
	}

	PaginatedResource.prototype.handlePage = function(err, body, headers, unpacked, statusCode, callback) {
		if(err && returnErrOnly(err, body, this.useHttpPaginatedResponse)) {
			Logger.logAction(Logger.LOG_ERROR, 'PaginatedResource.handlePage()', 'Unexpected error getting resource: err = ' + Utils.inspectError(err));
			callback(err);
			return;
		}
		var items, linkHeader, relParams;
		try {
			items = this.bodyHandler(body, headers, unpacked);
		} catch(e) {
			/* If we got an error, the failure to parse the body is almost certainly
			 * due to that, so cb with that in preference to the parse error */
			callback(err || e);
			return;
		}

		if(headers && (linkHeader = (headers['Link'] || headers['link']))) {
			relParams = parseRelLinks(linkHeader);
		}

		if(this.useHttpPaginatedResponse) {
			callback(null, new HttpPaginatedResponse(this, items, headers, statusCode, relParams, err));
		} else {
			callback(null, new PaginatedResult(this, items, relParams));
		}
	};

	function PaginatedResult(resource, items, relParams) {
		this.resource = resource;
		this.items = items;

		if(relParams) {
			var self = this;
			if('first' in relParams)
				this.first = function(cb) { self.get(relParams.first, cb); };
			if('current' in relParams)
				this.current = function(cb) { self.get(relParams.current, cb); };
			this.next = function(cb) {
				if('next' in relParams)
					self.get(relParams.next, cb);
				else
					cb(null, null);
			};

			this.hasNext = function() { return ('next' in relParams) };
			this.isLast = function() { return !this.hasNext(); }
		}
	}

	/* We assume that only the initial request can be a POST, and that accessing
	 * the rest of a multipage set of results can always be done with GET */
	PaginatedResult.prototype.get = function(params, callback) {
		var res = this.resource;
		Resource.get(res.rest, res.path, res.headers, params, res.envelope, function(err, body, headers, unpacked, statusCode) {
			res.handlePage(err, body, headers, unpacked, statusCode, callback);
		});
	};

	function HttpPaginatedResponse(resource, items, headers, statusCode, relParams, err) {
		PaginatedResult.call(this, resource, items, relParams);
		this.statusCode = statusCode;
		this.success = statusCode < 300 && statusCode >= 200;
		this.headers = headers;
		this.errorCode = err && err.code;
		this.errorMessage = err && err.message;
	}
	Utils.inherits(HttpPaginatedResponse, PaginatedResult);

	return PaginatedResource;
})();

var Auth = (function() {
	var MAX_TOKENOBJECT_LENGTH = Math.pow(2, 17);
	var MAX_TOKENSTRING_LENGTH = 384;
	function noop() {}
	function random() { return ('000000' + Math.floor(Math.random() * 1E16)).slice(-16); }
	function normaliseAuthcallbackError(err) {
		/* A client auth callback may give errors in any number of formats; normalise to an errorinfo */
		if(!Utils.isErrorInfo(err)) {
			return new ErrorInfo(Utils.inspectError(err), err.code || 40170, err.statusCode || 401);
		}
		/* network errors will not have an inherent error code */
		if(!err.code) {
			err.code = (err.statusCode === 403) ? 40300 : 40170;
		}
		return err;
	}

	var hmac, toBase64;
	if(Platform.createHmac) {
		toBase64 = function(str) { return (new Buffer(str, 'ascii')).toString('base64'); };
		hmac = function(text, key) {
			var inst = Platform.createHmac('SHA256', key);
			inst.update(text);
			return inst.digest('base64');
		};
	} else {
		toBase64 = Base64.encode;
		hmac = function(text, key) {
			return CryptoJS.HmacSHA256(text, key).toString(CryptoJS.enc.Base64);
		};
	}

	function c14n(capability) {
		if(!capability)
			return '';

		if(typeof(capability) == 'string')
			capability = JSON.parse(capability);

		var c14nCapability = {};
		var keys = Utils.keysArray(capability, true);
		if(!keys)
			return '';
		keys.sort();
		for(var i = 0; i < keys.length; i++) {
			c14nCapability[keys[i]] = capability[keys[i]].sort();
		}
		return JSON.stringify(c14nCapability);
	}

	function logAndValidateTokenAuthMethod(authOptions) {
		if(authOptions.authCallback) {
			Logger.logAction(Logger.LOG_MINOR, 'Auth()', 'using token auth with authCallback');
		} else if(authOptions.authUrl) {
			Logger.logAction(Logger.LOG_MINOR, 'Auth()', 'using token auth with authUrl');
		} else if(authOptions.key) {
			Logger.logAction(Logger.LOG_MINOR, 'Auth()', 'using token auth with client-side signing');
		} else if(authOptions.tokenDetails) {
			Logger.logAction(Logger.LOG_MINOR, 'Auth()', 'using token auth with supplied token only');
		} else {
			var msg = 'authOptions must include valid authentication parameters';
			Logger.logAction(Logger.LOG_ERROR, 'Auth()', msg);
			throw new Error(msg);
		}
	}

	function basicAuthForced(options) {
		return 'useTokenAuth' in options && !options.useTokenAuth;
	}

	/* RSA4 */
	function useTokenAuth(options) {
		return options.useTokenAuth ||
			(!basicAuthForced(options) &&
			 (options.clientId     ||
			  options.authCallback ||
			  options.authUrl      ||
			  options.token        ||
			  options.tokenDetails))
	}

	function Auth(client, options) {
		this.client = client;
		this.tokenParams = options.defaultTokenParams || {};

		if(useTokenAuth(options)) {
			/* Token auth */
			if(options.key && !hmac) {
				var msg = 'client-side token request signing not supported';
				Logger.logAction(Logger.LOG_ERROR, 'Auth()', msg);
				throw new Error(msg);
			}
			this._saveTokenOptions(options.defaultTokenParams, options);
			logAndValidateTokenAuthMethod(this.authOptions);
		} else {
			/* Basic auth */
			if(options.clientId || !options.key) {
				var msg = 'Cannot authenticate with basic auth' +
					(options.clientId ? ' as a clientId implies token auth' :
					 (!options.key ? ' as no key was given' : ''));
					 Logger.logAction(Logger.LOG_ERROR, 'Auth()', msg);
					 throw new Error(msg);
			}
			Logger.logAction(Logger.LOG_MINOR, 'Auth()', 'anonymous, using basic auth');
			this._saveBasicOptions(options);
		}
	}

	/**
	 * Instructs the library to get a token immediately and ensures Token Auth
	 * is used for all future requests, storing the tokenParams and authOptions
	 * given as the new defaults for subsequent use.
	 *
	 * @param tokenParams
	 * an object containing the parameters for the requested token:
	 *
	 * - ttl:        (optional) the requested life of any new token in ms. If none
	 *               is specified a default of 1 hour is provided. The maximum lifetime
	 *               is 24hours; any request exceeeding that lifetime will be rejected
	 *               with an error.
	 *
	 * - capability: (optional) the capability to associate with the access token.
	 *               If none is specified, a token will be requested with all of the
	 *               capabilities of the specified key.
	 *
	 * - clientId:   (optional) a client Id to associate with the token
	 *
	 * - timestamp:  (optional) the time in ms since the epoch. If none is specified,
	 *               the system will be queried for a time value to use.
	 *
	 * @param authOptions
	 * an object containing auth options relevant to token auth:
	 *
	 * - queryTime   (optional) boolean indicating that the Ably system should be
	 *               queried for the current time when none is specified explicitly.
	 *
	 * - tokenDetails: (optional) object: An authenticated TokenDetails object.
	 *
	 * - token:        (optional) string: the `token` property of a tokenDetails object
	 *
	 * - authCallback:  (optional) a javascript callback to be called to get auth information.
	 *                  authCallback should be a function of (tokenParams, callback) that calls
	 *                  the callback with (err, result), where result is any of:
	 *                  - a tokenRequest object (ie the result of a rest.auth.createTokenRequest call),
	 *                  - a tokenDetails object (ie the result of a rest.auth.requestToken call),
	 *                  - a token string
	 *
	 * - authUrl:       (optional) a URL to be used to GET or POST a set of token request
	 *                  params, to obtain a signed token request.
	 *
	 * - authHeaders:   (optional) a set of application-specific headers to be added to any request
	 *                  made to the authUrl.
	 *
	 * - authParams:    (optional) a set of application-specific query params to be added to any
	 *                  request made to the authUrl.
	 *
	 *
	 * - requestHeaders (optional, unsupported, for testing only) extra headers to add to the
	 *                  requestToken request
	 *
	 * @param callback (err, tokenDetails)
	 */
	Auth.prototype.authorize = function(tokenParams, authOptions, callback) {
		/* shuffle and normalise arguments as necessary */
		if(typeof(tokenParams) == 'function' && !callback) {
			callback = tokenParams;
			authOptions = tokenParams = null;
		} else if(typeof(authOptions) == 'function' && !callback) {
			callback = authOptions;
			authOptions = null;
		}
		callback = callback || noop;
		var self = this;

		/* RSA10a: authorize() call implies token auth. If a key is passed it, we
		 * just check if it doesn't clash and assume we're generating a token from it */
		if(authOptions && authOptions.key && (this.key !== authOptions.key)) {
			throw new ErrorInfo('Unable to update auth options with incompatible key', 40102, 401);
		}

		if(authOptions && ('force' in authOptions)) {
			Logger.logAction(Logger.LOG_ERROR, 'Auth.authorize', 'Deprecation warning: specifying {force: true} in authOptions is no longer necessary, authorize() now always gets a new token. Please remove this, as in version 1.0 and later, having a non-null authOptions will overwrite stored library authOptions, which may not be what you want');
			/* Emulate the old behaviour: if 'force' was the only member of authOptions,
			 * set it to null so it doesn't overwrite stored. TODO: remove in version 1.0 */
			if(Utils.isOnlyPropIn(authOptions, 'force')) {
				authOptions = null;
			}
		}

		this._forceNewToken(tokenParams, authOptions, function(err, tokenDetails) {
			if(err) {
				callback(err);
				return;
			}
			/* RTC8
			 * - When authorize called by an end user and have a realtime connection,
			 * don't call back till new token has taken effect.
			 * - Use self.client.connection as a proxy for (self.client instanceof Realtime),
			 * which doesn't work in node as Realtime isn't part of the vm context for Rest clients */
			if(self.client.connection) {
				self.client.connection.connectionManager.onAuthUpdated(tokenDetails, callback);
			} else {
				callback(null, tokenDetails);
			}
		})
	};

	Auth.prototype.authorise = function() {
		Logger.deprecated('Auth.authorise', 'Auth.authorize');
		this.authorize.apply(this, arguments);
	};

	/* For internal use, eg by connectionManager - useful when want to call back
	 * as soon as we have the new token, rather than waiting for it to take
	 * effect on the connection as #authorize does */
	Auth.prototype._forceNewToken = function(tokenParams, authOptions, callback) {
		var self = this;

		/* get rid of current token even if still valid */
		this.tokenDetails = null;

		/* _save normalises the tokenParams and authOptions and updates the auth
		 * object. All subsequent operations should use the values on `this`,
		 * not the passed in ones. */
		this._saveTokenOptions(tokenParams, authOptions);

		logAndValidateTokenAuthMethod(this.authOptions);

		this._ensureValidAuthCredentials(function(err, tokenDetails) {
			/* RSA10g */
			delete self.tokenParams.timestamp;
			delete self.authOptions.queryTime;
			callback(err, tokenDetails);
		});
	}

	/**
	 * Request an access token
	 * @param authOptions
	 * an object containing the request options:
	 * - key:           the key to use.
	 *
	 * - authCallback:  (optional) a javascript callback to be called to get auth information.
	 *                  authCallback should be a function of (tokenParams, callback) that calls
	 *                  the callback with (err, result), where result is any of:
	 *                  - a tokenRequest object (ie the result of a rest.auth.createTokenRequest call),
	 *                  - a tokenDetails object (ie the result of a rest.auth.requestToken call),
	 *                  - a token string
	 *
	 * - authUrl:       (optional) a URL to be used to GET or POST a set of token request
	 *                  params, to obtain a signed token request.
	 *
	 * - authHeaders:   (optional) a set of application-specific headers to be added to any request
	 *                  made to the authUrl.
	 *
	 * - authParams:    (optional) a set of application-specific query params to be added to any
	 *                  request made to the authUrl.
	 *
	 * - queryTime      (optional) boolean indicating that the ably system should be
	 *                  queried for the current time when none is specified explicitly
	 *
	 * - requestHeaders (optional, unsupported, for testing only) extra headers to add to the
	 *                  requestToken request
	 *
	 * @param tokenParams
	 * an object containing the parameters for the requested token:
	 * - ttl:          (optional) the requested life of the token in milliseconds. If none is specified
	 *                  a default of 1 hour is provided. The maximum lifetime is 24hours; any request
	 *                  exceeeding that lifetime will be rejected with an error.
	 *
	 * - capability:    (optional) the capability to associate with the access token.
	 *                  If none is specified, a token will be requested with all of the
	 *                  capabilities of the specified key.
	 *
	 * - clientId:      (optional) a client Id to associate with the token; if not
	 *                  specified, a clientId passed in constructing the Rest interface will be used
	 *
	 * - timestamp:     (optional) the time in ms since the epoch. If none is specified,
	 *                  the system will be queried for a time value to use.
	 *
	 * @param callback (err, tokenDetails)
	 */
	Auth.prototype.requestToken = function(tokenParams, authOptions, callback) {
		/* shuffle and normalise arguments as necessary */
		if(typeof(tokenParams) == 'function' && !callback) {
			callback = tokenParams;
			authOptions = tokenParams = null;
		}
		else if(typeof(authOptions) == 'function' && !callback) {
			callback = authOptions;
			authOptions = null;
		}

		/* RSA8e: if authOptions passed in, they're used instead of stored, don't merge them */
		authOptions = authOptions || this.authOptions;
		tokenParams = tokenParams || Utils.copy(this.tokenParams);
		callback = callback || noop;

		/* first set up whatever callback will be used to get signed
		 * token requests */
		var tokenRequestCallback, client = this.client;

		if(authOptions.authCallback) {
			Logger.logAction(Logger.LOG_MINOR, 'Auth.requestToken()', 'using token auth with authCallback');
			tokenRequestCallback = authOptions.authCallback;
		} else if(authOptions.authUrl) {
			Logger.logAction(Logger.LOG_MINOR, 'Auth.requestToken()', 'using token auth with authUrl');
			tokenRequestCallback = function(params, cb) {
				var authHeaders = Utils.mixin({accept: 'application/json, text/plain'}, authOptions.authHeaders),
					usePost = authOptions.authMethod && authOptions.authMethod.toLowerCase() === 'post';
				if(!usePost) {
					/* Combine authParams with any qs params given in the authUrl */
					var queryIdx = authOptions.authUrl.indexOf('?');
					if(queryIdx > -1) {
						var providedQsParams = Utils.parseQueryString(authOptions.authUrl.slice(queryIdx));
						authOptions.authUrl = authOptions.authUrl.slice(0, queryIdx);
						/* In case of conflict, authParams take precedence over qs params in the authUrl */
						authOptions.authParams = Utils.mixin(providedQsParams, authOptions.authParams);
					}
				}
				var authParams = Utils.mixin(params, authOptions.authParams);
				var authUrlRequestCallback = function(err, body, headers, unpacked) {
					if (err) {
						Logger.logAction(Logger.LOG_MICRO, 'Auth.requestToken().tokenRequestCallback', 'Received Error; ' + Utils.inspectError(err));
					} else {
						Logger.logAction(Logger.LOG_MICRO, 'Auth.requestToken().tokenRequestCallback', 'Received; body: ' + (BufferUtils.isBuffer(body) ? body.toString() : body));
					}
					if(err || unpacked) return cb(err, body);
					if(BufferUtils.isBuffer(body)) body = body.toString();
					var contentType = headers['content-type'];
					if(!contentType) {
						cb(new ErrorInfo('authUrl response is missing a content-type header', 40170, 401));
						return;
					}
					var json = contentType.indexOf('application/json') > -1,
						text = contentType.indexOf('text/plain') > -1 || contentType.indexOf('application/jwt') > -1;
					if(!json && !text) {
						cb(new ErrorInfo('authUrl responded with unacceptable content-type ' + contentType + ', should be either text/plain, application/jwt or application/json', 40170, 401));
						return;
					}
					if(json) {
						if(body.length > MAX_TOKENOBJECT_LENGTH) {
							cb(new ErrorInfo('authUrl response exceeded max permitted length', 40170, 401));
							return;
						}
						try {
							body = JSON.parse(body);
						} catch(e) {
							cb(new ErrorInfo('Unexpected error processing authURL response; err = ' + e.message, 40170, 401));
							return;
						}
					}
					cb(null, body);
				};
				Logger.logAction(Logger.LOG_MICRO, 'Auth.requestToken().tokenRequestCallback', 'Requesting token from ' + authOptions.authUrl + '; Params: ' + JSON.stringify(authParams) + '; method: ' + (usePost ? 'POST' : 'GET'));
				if(usePost) {
					/* send body form-encoded */
					var headers = authHeaders || {};
					headers['content-type'] = 'application/x-www-form-urlencoded';
					var body = Utils.toQueryString(authParams).slice(1); /* slice is to remove the initial '?' */
					Http.postUri(client, authOptions.authUrl, headers, body, {}, authUrlRequestCallback);
				} else {
					Http.getUri(client, authOptions.authUrl, authHeaders || {}, authParams, authUrlRequestCallback);
				}
			};
		} else if(authOptions.key) {
			var self = this;
			Logger.logAction(Logger.LOG_MINOR, 'Auth.requestToken()', 'using token auth with client-side signing');
			tokenRequestCallback = function(params, cb) { self.createTokenRequest(params, authOptions, cb); };
		} else {
			var msg = "Need a new token, but authOptions does not include any way to request one";
			Logger.logAction(Logger.LOG_ERROR, 'Auth.requestToken()', msg);
			callback(new ErrorInfo(msg, 40101, 403));
			return;
		}

		/* normalise token params */
		if('capability' in tokenParams)
			tokenParams.capability = c14n(tokenParams.capability);

		var client = this.client;
		var tokenRequest = function(signedTokenParams, tokenCb) {
			var keyName = signedTokenParams.keyName,
				tokenUri = function(host) { return client.baseUri(host) + '/keys/' + keyName + '/requestToken'; };

			var requestHeaders = Utils.defaultPostHeaders();
			if(authOptions.requestHeaders) Utils.mixin(requestHeaders, authOptions.requestHeaders);
			Logger.logAction(Logger.LOG_MICRO, 'Auth.requestToken().requestToken', 'Sending POST; ' + tokenUri + '; Token params: ' + JSON.stringify(signedTokenParams));
			signedTokenParams = JSON.stringify(signedTokenParams);
			Http.post(client, tokenUri, requestHeaders, signedTokenParams, null, tokenCb);
		};

		var tokenRequestCallbackTimeoutExpired = false,
			timeoutLength = this.client.options.timeouts.realtimeRequestTimeout,
			tokenRequestCallbackTimeout = setTimeout(function() {
				tokenRequestCallbackTimeoutExpired = true;
				var msg = 'Token request callback timed out after ' + (timeoutLength / 1000) + ' seconds';
				Logger.logAction(Logger.LOG_ERROR, 'Auth.requestToken()', msg);
				callback(new ErrorInfo(msg, 40170, 401));
			}, timeoutLength);

		tokenRequestCallback(tokenParams, function(err, tokenRequestOrDetails) {
			if(tokenRequestCallbackTimeoutExpired) return;
			clearTimeout(tokenRequestCallbackTimeout);

			if(err) {
				Logger.logAction(Logger.LOG_ERROR, 'Auth.requestToken()', 'token request signing call returned error; err = ' + Utils.inspectError(err));
				callback(normaliseAuthcallbackError(err));
				return;
			}
			/* the response from the callback might be a token string, a signed request or a token details */
			if(typeof(tokenRequestOrDetails) === 'string') {
				if(tokenRequestOrDetails.length === 0) {
					callback(new ErrorInfo('Token string is empty', 40170, 401));
				} else if(tokenRequestOrDetails.length > MAX_TOKENSTRING_LENGTH) {
					callback(new ErrorInfo('Token string exceeded max permitted length (was ' + tokenRequestOrDetails.length + ' bytes)', 40170, 401));
				} else {
					callback(null, {token: tokenRequestOrDetails});
				}
				return;
			}
			if(typeof(tokenRequestOrDetails) !== 'object') {
				var msg = 'Expected token request callback to call back with a token string or token request/details object, but got a ' + typeof(tokenRequestOrDetails);
				Logger.logAction(Logger.LOG_ERROR, 'Auth.requestToken()', msg);
				callback(new ErrorInfo(msg, 40170, 401));
				return;
			}
			var objectSize = JSON.stringify(tokenRequestOrDetails).length;
			if(objectSize > MAX_TOKENOBJECT_LENGTH && !authOptions.suppressMaxLengthCheck) {
				callback(new ErrorInfo('Token request/details object exceeded max permitted stringified size (was ' + objectSize + ' bytes)', 40170, 401));
				return;
			}
			if('issued' in tokenRequestOrDetails) {
				/* a tokenDetails object */
				callback(null, tokenRequestOrDetails);
				return;
			}
			if(!('keyName' in tokenRequestOrDetails)) {
				var msg = 'Expected token request callback to call back with a token string, token request object, or token details object';
				Logger.logAction(Logger.LOG_ERROR, 'Auth.requestToken()', msg);
				callback(new ErrorInfo(msg, 40170, 401));
				return;
			}
			/* it's a token request, so make the request */
			tokenRequest(tokenRequestOrDetails, function(err, tokenResponse, headers, unpacked) {
				if(err) {
					Logger.logAction(Logger.LOG_ERROR, 'Auth.requestToken()', 'token request API call returned error; err = ' + Utils.inspectError(err));
					callback(normaliseAuthcallbackError(err));
					return;
				}
				if(!unpacked) tokenResponse = JSON.parse(tokenResponse);
				Logger.logAction(Logger.LOG_MINOR, 'Auth.getToken()', 'token received');
				callback(null, tokenResponse);
			});
		});
	};

	/**
	 * Create and sign a token request based on the given options.
	 * NOTE this can only be used when the key value is available locally.
	 * Otherwise, signed token requests must be obtained from the key
	 * owner (either using the token request callback or url).
	 *
	 * @param authOptions
	 * an object containing the request options:
	 * - key:           the key to use. If not specified, a key passed in constructing
	 *                  the Rest interface will be used
	 *
	 * - queryTime      (optional) boolean indicating that the ably system should be
	 *                  queried for the current time when none is specified explicitly
	 *
	 * - requestHeaders (optional, unsupported, for testing only) extra headers to add to the
	 *                  requestToken request
	 *
	 * @param tokenParams
	 * an object containing the parameters for the requested token:
	 * - ttl:       (optional) the requested life of the token in ms. If none is specified
	 *                  a default of 1 hour is provided. The maximum lifetime is 24hours; any request
	 *                  exceeeding that lifetime will be rejected with an error.
	 *
	 * - capability:    (optional) the capability to associate with the access token.
	 *                  If none is specified, a token will be requested with all of the
	 *                  capabilities of the specified key.
	 *
	 * - clientId:      (optional) a client Id to associate with the token; if not
	 *                  specified, a clientId passed in constructing the Rest interface will be used
	 *
	 * - timestamp:     (optional) the time in ms since the epoch. If none is specified,
	 *                  the system will be queried for a time value to use.
	 *
	 */
	Auth.prototype.createTokenRequest = function(tokenParams, authOptions, callback) {
		/* shuffle and normalise arguments as necessary */
		if(typeof(tokenParams) == 'function' && !callback) {
			callback = tokenParams;
			authOptions = tokenParams = null;
		} else if(typeof(authOptions) == 'function' && !callback) {
			callback = authOptions;
			authOptions = null;
		}

		/* RSA9h: if authOptions passed in, they're used instead of stored, don't merge them */
		authOptions = authOptions || this.authOptions;
		tokenParams = tokenParams || Utils.copy(this.tokenParams);

		var key = authOptions.key;
		if(!key) {
			callback(new ErrorInfo('No key specified', 40101, 403));
			return;
		}
		var keyParts = key.split(':'),
			keyName = keyParts[0],
			keySecret = keyParts[1];

		if(!keySecret) {
			callback(new ErrorInfo('Invalid key specified', 40101, 403));
			return;
		}

		if(tokenParams.clientId === '') {
			callback(new ErrorInfo('clientId can’t be an empty string', 40012, 400));
			return;
		}

		if('capability' in tokenParams) {
			tokenParams.capability = c14n(tokenParams.capability);
		}

		var request = Utils.mixin({ keyName: keyName }, tokenParams),
			clientId = tokenParams.clientId || '',
			ttl = tokenParams.ttl || '',
			capability = tokenParams.capability || '',
			self = this;

		(function(authoriseCb) {
			if(request.timestamp) {
				authoriseCb();
				return;
			};
			self.getTimestamp(authOptions && authOptions.queryTime, function(err, time) {
				if(err) {callback(err); return;}
				request.timestamp = time;
				authoriseCb();
			});
		})(function() {
			/* nonce */
			/* NOTE: there is no expectation that the client
			 * specifies the nonce; this is done by the library
			 * However, this can be overridden by the client
			 * simply for testing purposes. */
			var nonce = request.nonce || (request.nonce = random()),
				timestamp = request.timestamp;

			var signText
			=	request.keyName + '\n'
			+	ttl + '\n'
			+	capability + '\n'
			+	clientId + '\n'
			+	timestamp + '\n'
			+	nonce + '\n';

			/* mac */
			/* NOTE: there is no expectation that the client
			 * specifies the mac; this is done by the library
			 * However, this can be overridden by the client
			 * simply for testing purposes. */
			request.mac = request.mac || hmac(signText, keySecret);

			Logger.logAction(Logger.LOG_MINOR, 'Auth.getTokenRequest()', 'generated signed request');
			callback(null, request);
		});
	};

	/**
	 * Get the auth query params to use for a websocket connection,
	 * based on the current auth parameters
	 */
	Auth.prototype.getAuthParams = function(callback) {
		if(this.method == 'basic')
			callback(null, {key: this.key});
		else
			this._ensureValidAuthCredentials(function(err, tokenDetails) {
				if(err) {
					callback(err);
					return;
				}
				callback(null, {access_token: tokenDetails.token});
			});
	};

	/**
	 * Get the authorization header to use for a REST or comet request,
	 * based on the current auth parameters
	 */
	Auth.prototype.getAuthHeaders = function(callback) {
		if(this.method == 'basic') {
			callback(null, {authorization: 'Basic ' + this.basicKey});
		} else {
			this._ensureValidAuthCredentials(function(err, tokenDetails) {
				if(err) {
					callback(err);
					return;
				}
				callback(null, {authorization: 'Bearer ' + toBase64(tokenDetails.token)});
			});
		}
	};

	/**
	 * Get the current time based on the local clock,
	 * or if the option queryTime is true, return the server time.
	 * The server time offset from the local time is stored so that
	 * only one request to the server to get the time is ever needed
	 */
	Auth.prototype.getTimestamp = function(queryTime, callback) {
		var offsetSet = !isNaN(parseInt(this.client.serverTimeOffset));
		if (!offsetSet && (queryTime || this.authOptions.queryTime)) {
			this.client.time(function(err, time) {
				if(err) {
					callback(err);
					return;
				}
				callback(null, time);
			});
		} else {
			callback(null, Utils.now() + (this.client.serverTimeOffset || 0));
		}
	};

	Auth.prototype._saveBasicOptions = function(authOptions) {
		this.method = 'basic';
		this.key = authOptions.key;
		this.basicKey = toBase64(authOptions.key);
		this.authOptions = authOptions || {};
		if('clientId' in authOptions) {
			this._userSetClientId(authOptions.clientId);
		}
	}

	Auth.prototype._saveTokenOptions = function(tokenParams, authOptions) {
		this.method = 'token';

		if(tokenParams) {
			/* We temporarily persist tokenParams.timestamp in case a new token needs
			 * to be requested, then null it out in the callback of
			 * _ensureValidAuthCredentials for RSA10g compliance */
			this.tokenParams = tokenParams;
		}

		if(authOptions) {
			/* normalise */
			if(authOptions.token) {
				/* options.token may contain a token string or, for convenience, a TokenDetails */
				authOptions.tokenDetails = (typeof(authOptions.token) === 'string') ? {token: authOptions.token} : authOptions.token;
			}

			if(authOptions.tokenDetails) {
				this.tokenDetails = authOptions.tokenDetails;
			}

			if('clientId' in authOptions) {
				this._userSetClientId(authOptions.clientId);
			}

			this.authOptions = authOptions;
		}
	};

	Auth.prototype._ensureValidAuthCredentials = function(callback) {
		var self = this,
			token = this.tokenDetails;

		var requestToken = function() {
			self.requestToken(self.tokenParams, self.authOptions, function(err, tokenResponse) {
				if(err) {
					callback(err);
					return;
				}
				callback(null, (self.tokenDetails = tokenResponse));
			});
		};

		if(token) {
			if(this._tokenClientIdMismatch(token.clientId)) {
				/* 403 to trigger a permanently failed client - RSA15c */
				callback(new ErrorInfo('Mismatch between clientId in token (' + token.clientId + ') and current clientId (' + this.clientId + ')', 40102, 403));
				return;
			}
			this.getTimestamp(self.authOptions && self.authOptions.queryTime, function(err, time) {
				if(err)
					callback(err);

				if(token.expires === undefined || (token.expires >= time)) {
					Logger.logAction(Logger.LOG_MINOR, 'Auth.getToken()', 'using cached token; expires = ' + token.expires);
					callback(null, token);
					return;
				} else {
					/* expired, so remove */
					Logger.logAction(Logger.LOG_MINOR, 'Auth.getToken()', 'deleting expired token');
					self.tokenDetails = null;
				}
				requestToken();
			});
		} else {
			requestToken();
		}
	};


	/* User-set: check types, '*' is disallowed, throw any errors */
	Auth.prototype._userSetClientId = function(clientId) {
		if(!(typeof(clientId) === 'string' || clientId === null)) {
			throw new ErrorInfo('clientId must be either a string or null', 40012, 400);
		} else if(clientId === '*') {
			throw new ErrorInfo('Can’t use "*" as a clientId as that string is reserved. (To change the default token request behaviour to use a wildcard clientId, instantiate the library with {defaultTokenParams: {clientId: "*"}}), or if calling authorize(), pass it in as a tokenParam: authorize({clientId: "*"}, authOptions)', 40012, 400);
		} else {
			var err = this._uncheckedSetClientId(clientId);
			if(err) throw err;
		}
	};

	/* Ably-set: no typechecking, '*' is allowed but not set on this.clientId), return errors to the caller */
	Auth.prototype._uncheckedSetClientId = function(clientId) {
		if(this._tokenClientIdMismatch(clientId)) {
			/* Should never happen in normal circumstances as realtime should
			 * recognise mismatch and return an error */
			var msg = 'Unexpected clientId mismatch: client has ' + this.clientId + ', requested ' + clientId;
			var err = new ErrorInfo(msg, 40102, 401);
			Logger.logAction(Logger.LOG_ERROR, 'Auth._uncheckedSetClientId()', msg);
			return err;
		} else if(clientId === '*') {
			this.tokenParams.clientId = clientId;
		} else {
			/* RSA7a4: if options.clientId is provided and is not
			 * null, it overrides defaultTokenParams.clientId */
			this.clientId = this.tokenParams.clientId = clientId;
			return null;
		}
	};

	Auth.prototype._tokenClientIdMismatch = function(tokenClientId) {
		return this.clientId &&
			tokenClientId &&
			(tokenClientId !== '*') &&
			(this.clientId !== tokenClientId);
	};

	Auth.isTokenErr = function(error) {
		return error.code && (error.code >= 40140) && (error.code < 40150);
	};

	return Auth;
})();

var Rest = (function() {
	var noop = function() {};
	var msgpack = Platform.msgpack;

	function Rest(options) {
		if(!(this instanceof Rest)){
			return new Rest(options);
		}

		/* normalise options */
		if(!options) {
			var msg = 'no options provided';
			Logger.logAction(Logger.LOG_ERROR, 'Rest()', msg);
			throw new Error(msg);
		}
		if(typeof(options) == 'string') {
			options = (options.indexOf(':') == -1) ? {token: options} : {key: options};
		}

		if(options.log) {
			Logger.setLog(options.log.level, options.log.handler);
		}
		Logger.logAction(Logger.LOG_MICRO, 'Rest()', 'initialized with clientOptions ' + Utils.inspect(options));

		this.options = Defaults.normaliseOptions(options);

		/* process options */
		if(options.key) {
			var keyMatch = options.key.match(/^([^:\s]+):([^:.\s]+)$/);
			if(!keyMatch) {
				var msg = 'invalid key parameter';
				Logger.logAction(Logger.LOG_ERROR, 'Rest()', msg);
				throw new Error(msg);
			}
			options.keyName = keyMatch[1];
			options.keySecret = keyMatch[2];
		}

		if('clientId' in options) {
			if(!(typeof(options.clientId) === 'string' || options.clientId === null))
				throw new ErrorInfo('clientId must be either a string or null', 40012, 400);
			else if(options.clientId === '*')
				throw new ErrorInfo('Can’t use "*" as a clientId as that string is reserved. (To change the default token request behaviour to use a wildcard clientId, use {defaultTokenParams: {clientId: "*"}})', 40012, 400);
		}

		Logger.logAction(Logger.LOG_MINOR, 'Rest()', 'started; version = ' + Defaults.libstring);

		this.baseUri = this.authority = function(host) { return Defaults.getHttpScheme(options) + host + ':' + Defaults.getPort(options, false); };

		this.serverTimeOffset = null;
		this.auth = new Auth(this, options);
		this.channels = new Channels(this);
	}

	Rest.prototype.stats = function(params, callback) {
		/* params and callback are optional; see if params contains the callback */
		if(callback === undefined) {
			if(typeof(params) == 'function') {
				callback = params;
				params = null;
			} else {
				callback = noop;
			}
		}
		var headers = Utils.copy(Utils.defaultGetHeaders()),
			format = this.options.useBinaryProtocol ? 'msgpack' : 'json',
			envelope = Http.supportsLinkHeaders ? undefined : format;

		if(this.options.headers)
			Utils.mixin(headers, this.options.headers);

		(new PaginatedResource(this, '/stats', headers, envelope, function(body, headers, unpacked) {
			var statsValues = (unpacked ? body : JSON.parse(body));
			for(var i = 0; i < statsValues.length; i++) statsValues[i] = Stats.fromValues(statsValues[i]);
			return statsValues;
		})).get(params, callback);
	};

	Rest.prototype.time = function(params, callback) {
		/* params and callback are optional; see if params contains the callback */
		if(callback === undefined) {
			if(typeof(params) == 'function') {
				callback = params;
				params = null;
			} else {
				callback = noop;
			}
		}
		var headers = Utils.copy(Utils.defaultGetHeaders());
		if(this.options.headers)
			Utils.mixin(headers, this.options.headers);
		var self = this;
		var timeUri = function(host) { return self.authority(host) + '/time' };
		Http.get(this, timeUri, headers, params, function(err, res, headers, unpacked) {
			if(err) {
				callback(err);
				return;
			}
			if(!unpacked) res = JSON.parse(res);
			var time = res[0];
			if(!time) {
				err = new Error('Internal error (unexpected result type from GET /time)');
				err.statusCode = 500;
				callback(err);
				return;
			}
			/* calculate time offset only once for this device by adding to the prototype */
			self.serverTimeOffset = (time - Utils.now());
			callback(null, time);
		});
	};

	Rest.prototype.request = function(method, path, params, body, customHeaders, callback) {
		var useBinary = this.options.useBinaryProtocol,
			encoder = useBinary ? msgpack.encode: JSON.stringify,
			decoder = useBinary ? msgpack.decode : JSON.parse,
			format = useBinary ? 'msgpack' : 'json',
			method = method.toLowerCase(),
			envelope = Http.supportsLinkHeaders ? undefined : format,
			params = params || {},
			headers = Utils.copy(method == 'get' ? Utils.defaultGetHeaders(format) : Utils.defaultPostHeaders(format));

		if(typeof body !== 'string') {
			body = encoder(body);
		}
		if(this.options.headers) {
			Utils.mixin(headers, this.options.headers);
		}
		if(customHeaders) {
			Utils.mixin(headers, customHeaders);
		}
		var paginatedResource = new PaginatedResource(this, path, headers, envelope, function(resbody, headers, unpacked) {
			return Utils.ensureArray(unpacked ? resbody : decoder(resbody));
		}, /* useHttpPaginatedResponse: */ true);

		switch(method) {
			case 'get':
				paginatedResource.get(params, callback);
				break;
			case 'post':
				paginatedResource.post(params, body, callback);
				break;
			default:
				throw new ErrorInfo('Currently only GET and POST methods are supported', 40500, 405);
		}
	};

	function Channels(rest) {
		this.rest = rest;
		this.attached = {};
	}

	Channels.prototype.get = function(name, channelOptions) {
		name = String(name);
		var channel = this.attached[name];
		if(!channel) {
			this.attached[name] = channel = new Channel(this.rest, name, channelOptions);
		} else if(channelOptions) {
			channel.setOptions(channelOptions);
		}

		return channel;
	};

	Channels.prototype.release = function(name) {
		delete this.attached[String(name)];
	};

	return Rest;
})();

var Realtime = (function() {

	function Realtime(options) {
		if(!(this instanceof Realtime)){
			return new Realtime(options);
		}

		Logger.logAction(Logger.LOG_MINOR, 'Realtime()', '');
		Rest.call(this, options);
		this.connection = new Connection(this, this.options);
		this.channels = new Channels(this);
		if(options.autoConnect !== false)
			this.connect();
	}
	Utils.inherits(Realtime, Rest);

	Realtime.prototype.connect = function() {
		Logger.logAction(Logger.LOG_MINOR, 'Realtime.connect()', '');
		this.connection.connect();
	};

	Realtime.prototype.close = function() {
		Logger.logAction(Logger.LOG_MINOR, 'Realtime.close()', '');
		this.connection.close();
	};

	function Channels(realtime) {
		EventEmitter.call(this);
		this.realtime = realtime;
		this.all = {};
		this.inProgress = {};
		realtime.connection.connectionManager.on('transport.active', this.onTransportActive.bind(this));
	}
	Utils.inherits(Channels, EventEmitter);

	Channels.prototype.onChannelMessage = function(msg) {
		var channelName = msg.channel;
		if(channelName === undefined) {
			Logger.logAction(Logger.LOG_ERROR, 'Channels.onChannelMessage()', 'received event unspecified channel, action = ' + msg.action);
			return;
		}
		var channel = this.all[channelName];
		if(!channel) {
			Logger.logAction(Logger.LOG_ERROR, 'Channels.onChannelMessage()', 'received event for non-existent channel: ' + channelName);
			return;
		}
		channel.onMessage(msg);
	};

	/* called when a transport becomes connected; reattempt attach/detach
	 * for channels that are attaching or detaching.
	 * Note that this does not use inProgress as inProgress is only channels which have already made
	* at least one attempt to attach/detach */
	Channels.prototype.onTransportActive = function() {
		for(var channelName in this.all) {
			var channel = this.all[channelName];
			if(channel.state === 'attaching' || channel.state === 'detaching') {
				channel.checkPendingState();
			} else if(channel.state === 'suspended') {
				channel.attach();
			}
		}
	};

	Channels.prototype.reattach = function(reason) {
		for(var channelId in this.all) {
			var channel = this.all[channelId];
			/* NB this should not trigger for merely attaching channels, as they will
			 * be reattached anyway through the onTransportActive checkPendingState */
			if(channel.state === 'attached') {
				channel.requestState('attaching', reason);
			}
		}
	};

	/* Connection interruptions (ie when the connection will no longer queue
	 * events) imply connection state changes for any channel which is either
	 * attached, pending, or will attempt to become attached in the future */
	Channels.prototype.propogateConnectionInterruption = function(connectionState, reason) {
		var connectionStateToChannelState = {
			'closing'  : 'detached',
			'closed'   : 'detached',
			'failed'   : 'failed',
			'suspended': 'suspended'
		};
		var fromChannelStates = ['attaching', 'attached', 'detaching', 'suspended'];
		var toChannelState = connectionStateToChannelState[connectionState];

		for(var channelId in this.all) {
			var channel = this.all[channelId];
			if(Utils.arrIn(fromChannelStates, channel.state)) {
				 channel.notifyState(toChannelState, reason);
			}
		}
	};

	Channels.prototype.get = function(name, channelOptions) {
		name = String(name);
		var channel = this.all[name];
		if(!channel) {
			channel = this.all[name] = new RealtimeChannel(this.realtime, name, channelOptions);
		} else if(channelOptions) {
			channel.setOptions(channelOptions);
		}
		return channel;
	};

	Channels.prototype.release = function(name) {
		var channel = this.all[name];
		if(channel) {
			delete this.all[name];
		}
	};

	/* Records operations currently pending on a transport; used by connectionManager to decide when
	 * it's safe to upgrade. Note that a channel might be in the attaching state without any pending
	 * operations (eg if attached while the connection state is connecting) - such a channel must not
	 * hold up an upgrade, so is not considered inProgress.
	 * Operation is currently one of either 'statechange' or 'sync' */
	Channels.prototype.setInProgress = function(channel, operation, inProgress) {
		this.inProgress[channel.name] = this.inProgress[channel.name] || {};
		this.inProgress[channel.name][operation] = inProgress;
		if(!inProgress && this.hasNopending()) {
			this.emit('nopending');
		}
	};

	Channels.prototype.onceNopending = function(listener) {
		if(this.hasNopending()) {
			listener();
			return;
		}
		this.once('nopending', listener);
	};

	Channels.prototype.hasNopending = function() {
		return Utils.arrEvery(Utils.valuesArray(this.inProgress, true), function(operations) {
			return !Utils.containsValue(operations, true);
		});
	};

	return Realtime;
})();

var ConnectionStateChange = (function() {

	/* public constructor */
	function ConnectionStateChange(previous, current, retryIn, reason) {
		this.previous = previous;
		this.current = current;
		if(retryIn) this.retryIn = retryIn;
		if(reason) this.reason = reason;
	}

	return ConnectionStateChange;
})();

var ChannelStateChange = (function() {

	/* public constructor */
	function ChannelStateChange(previous, current, resumed, reason) {
		this.previous = previous;
		this.current = current;
		if(current === 'attached') this.resumed = resumed;
		if(reason) this.reason = reason;
	}

	return ChannelStateChange;
})();

var Connection = (function() {

	/* public constructor */
	function Connection(ably, options) {
		EventEmitter.call(this);
		this.ably = ably;
		this.connectionManager = new ConnectionManager(ably, options);
		this.state = this.connectionManager.state.state;
		this.key = undefined;
		this.id = undefined;
		this.serial = undefined;
		this.timeSerial = undefined;
		this.recoveryKey = undefined;
		this.errorReason = null;

		var self = this;
		this.connectionManager.on('connectionstate', function(stateChange) {
			var state = self.state = stateChange.current;
			Utils.nextTick(function() {
				self.emit(state, stateChange);
			});
		});
		this.connectionManager.on('update', function(stateChange) {
			Utils.nextTick(function() {
				self.emit('update', stateChange);
			});
		});
	}
	Utils.inherits(Connection, EventEmitter);

	Connection.prototype.whenState = function(state, listener) {
		EventEmitter.prototype.whenState.call(this, state, this.state, listener, new ConnectionStateChange(undefined, state));
	}

	Connection.prototype.connect = function() {
		Logger.logAction(Logger.LOG_MINOR, 'Connection.connect()', '');
		this.connectionManager.requestState({state: 'connecting'});
	};

	Connection.prototype.ping = function(callback) {
		Logger.logAction(Logger.LOG_MINOR, 'Connection.ping()', '');
		callback = callback || function() {};
		this.connectionManager.ping(null, callback);
	};

	Connection.prototype.close = function() {
		Logger.logAction(Logger.LOG_MINOR, 'Connection.close()', 'connectionKey = ' + this.key);
		this.connectionManager.requestState({state: 'closing'});
	};

	return Connection;
})();

var Channel = (function() {
	function noop() {}

	/* public constructor */
	function Channel(rest, name, channelOptions) {
		Logger.logAction(Logger.LOG_MINOR, 'Channel()', 'started; name = ' + name);
		EventEmitter.call(this);
		this.rest = rest;
		this.name = name;
		this.basePath = '/channels/' + encodeURIComponent(name);
		this.presence = new Presence(this);
		this.setOptions(channelOptions);
	}
	Utils.inherits(Channel, EventEmitter);

	Channel.prototype.setOptions = function(options) {
		this.channelOptions = options = options || {};
		if(options.cipher) {
			if(!Crypto) throw new Error('Encryption not enabled; use ably.encryption.js instead');
			var cipher = Crypto.getCipher(options.cipher);
			options.cipher = cipher.cipherParams;
			options.channelCipher = cipher.cipher;
		} else if('cipher' in options) {
			/* Don't deactivate an existing cipher unless options
			 * has a 'cipher' key that's falsey */
			options.cipher = null;
			options.channelCipher = null;
		}
	};

	Channel.prototype.history = function(params, callback) {
		Logger.logAction(Logger.LOG_MICRO, 'Channel.history()', 'channel = ' + this.name);
		/* params and callback are optional; see if params contains the callback */
		if(callback === undefined) {
			if(typeof(params) == 'function') {
				callback = params;
				params = null;
			} else {
				callback = noop;
			}
		}

		this._history(params, callback);
	};

	Channel.prototype._history = function(params, callback) {
		var rest = this.rest,
			format = rest.options.useBinaryProtocol ? 'msgpack' : 'json',
			envelope = Http.supportsLinkHeaders ? undefined : format,
			headers = Utils.copy(Utils.defaultGetHeaders(format)),
			channel = this;

		if(rest.options.headers)
			Utils.mixin(headers, rest.options.headers);

		var options = this.channelOptions;
		(new PaginatedResource(rest, this.basePath + '/messages', headers, envelope, function(body, headers, unpacked) {
			return Message.fromResponseBody(body, options, !unpacked && format);
		})).get(params, callback);
	};

	Channel.prototype.publish = function() {
		var argCount = arguments.length,
			messages = arguments[0],
			callback = arguments[argCount - 1],
			self = this;

		if(typeof(callback) !== 'function') {
			callback = noop;
			++argCount;
		}
		if(argCount == 2) {
			if(Utils.isObject(messages))
				messages = [Message.fromValues(messages)];
			else if(Utils.isArray(messages))
				messages = Message.fromValuesArray(messages);
			else
				throw new ErrorInfo('The single-argument form of publish() expects a message object or an array of message objects', 40013, 400);
		} else {
			messages = [Message.fromValues({name: arguments[0], data: arguments[1]})];
		}

		var rest = this.rest,
			format = rest.options.useBinaryProtocol ? 'msgpack' : 'json',
			headers = Utils.copy(Utils.defaultPostHeaders(format));

		if(rest.options.headers)
			Utils.mixin(headers, rest.options.headers);

		Message.toRequestBody(messages, this.channelOptions, format, function(err, requestBody) {
			if (err) {
				callback(err);
				return;
			}
			self._publish(requestBody, headers, callback);
		});
	};

	Channel.prototype._publish = function(requestBody, headers, callback) {
		Resource.post(this.rest, this.basePath + '/messages', requestBody, headers, null, false, callback);
	};

	return Channel;
})();

var RealtimeChannel = (function() {
	var actions = ProtocolMessage.Action;
	var noop = function() {};
	var statechangeOp = 'statechange';
	var syncOp = 'sync';

	/* public constructor */
	function RealtimeChannel(realtime, name, options) {
		Logger.logAction(Logger.LOG_MINOR, 'RealtimeChannel()', 'started; name = ' + name);
		Channel.call(this, realtime, name, options);
		this.realtime = realtime;
		this.presence = new RealtimePresence(this, realtime.options);
		this.connectionManager = realtime.connection.connectionManager;
		this.state = 'initialized';
		this.subscriptions = new EventEmitter();
		this.syncChannelSerial = undefined;
		this.attachSerial = undefined;
		this.setOptions(options);
		this.errorReason = null;
		this._requestedFlags = null;
		this._mode = null;
	}
	Utils.inherits(RealtimeChannel, Channel);

	RealtimeChannel.invalidStateError = function(state) {
		return {
			statusCode: 400,
			code: 90001,
			message: 'Channel operation failed as channel state is ' + state
		};
	};

	RealtimeChannel.progressOps = {
		statechange: statechangeOp,
		sync: syncOp
	};

	RealtimeChannel.processListenerArgs = function(args) {
		/* [event], listener, [callback] */
		if(typeof(args[0]) == 'function')
			return [null, args[0], args[1] || noop];
		else
			return [args[0], args[1], (args[2] || noop)];
	}

	RealtimeChannel.prototype.publish = function() {
		var argCount = arguments.length,
			messages = arguments[0],
			callback = arguments[argCount - 1];

		if(typeof(callback) !== 'function') {
			callback = noop;
			++argCount;
		}
		if(!this.connectionManager.activeState()) {
			callback(this.connectionManager.getStateError());
			return;
		}
		if(argCount == 2) {
			if(Utils.isObject(messages))
				messages = [Message.fromValues(messages)];
			else if(Utils.isArray(messages))
				messages = Message.fromValuesArray(messages);
			else
				throw new ErrorInfo('The single-argument form of publish() expects a message object or an array of message objects', 40013, 400);
		} else {
			messages = [Message.fromValues({name: arguments[0], data: arguments[1]})];
		}
		var options = this.channelOptions;
		var self = this;
		Message.encodeArray(messages, options, function(err) {
			if (err) {
				callback(err);
				return;
			}
			self._publish(messages, callback);
		});
	};

	RealtimeChannel.prototype._publish = function(messages, callback) {
		Logger.logAction(Logger.LOG_MICRO, 'RealtimeChannel.publish()', 'message count = ' + messages.length);
		var state = this.state;
		switch(state) {
			case 'failed':
			case 'suspended':
				callback(ErrorInfo.fromValues(RealtimeChannel.invalidStateError(state)));
				break;
			default:
				Logger.logAction(Logger.LOG_MICRO, 'RealtimeChannel.publish()', 'sending message; channel state is ' + state);
				var msg = new ProtocolMessage();
				msg.action = actions.MESSAGE;
				msg.channel = this.name;
				msg.messages = messages;
				this.sendMessage(msg, callback);
				break;
		}
	};

	RealtimeChannel.prototype.onEvent = function(messages) {
		Logger.logAction(Logger.LOG_MICRO, 'RealtimeChannel.onEvent()', 'received message');
		var subscriptions = this.subscriptions;
		for(var i = 0; i < messages.length; i++) {
			var message = messages[i];
			subscriptions.emit(message.name, message);
		}
	};

	RealtimeChannel.prototype.attach = function(flags, callback) {
		if(typeof(flags) === 'function') {
			callback = flags;
			flags = null;
		}
		callback = callback || function(err) {
			if(err) {
				Logger.logAction(Logger.LOG_MAJOR, 'RealtimeChannel.attach()', 'Channel attach failed: ' + err.toString());
			}
		};
		if(flags) {
			this._requestedFlags = flags;
		}
		var connectionManager = this.connectionManager;
		if(!connectionManager.activeState()) {
			callback(connectionManager.getStateError());
			return;
		}
		switch(this.state) {
			case 'attached':
				/* If flags requested, always do a re-attach. TODO only do this if if
				* current mode differs from requested mode */
				if(!flags) {
					callback();
					break;
				} /* else fallthrough */
			default:
				this.requestState('attaching');
			case 'attaching':
				this.once(function(stateChange) {
					switch(this.event) {
						case 'attached':
							callback();
							break;
						case 'detached':
						case 'suspended':
						case 'failed':
							callback(stateChange.reason || connectionManager.getStateError());
					}
				});
			}
    };

	RealtimeChannel.prototype.attachImpl = function() {
		Logger.logAction(Logger.LOG_MICRO, 'RealtimeChannel.attachImpl()', 'sending ATTACH message');
		this.setInProgress(statechangeOp, true);
		var attachMsg = ProtocolMessage.fromValues({action: actions.ATTACH, channel: this.name});
		if(this._requestedFlags) {
			Utils.arrForEach(this._requestedFlags, function(flag) {
				attachMsg.setFlag(flag);
			})
		}
		this.sendMessage(attachMsg, noop);
	};

	RealtimeChannel.prototype.detach = function(callback) {
		callback = callback || noop;
		var connectionManager = this.connectionManager;
		if(!connectionManager.activeState()) {
			callback(connectionManager.getStateError());
			return;
		}
		switch(this.state) {
			case 'detached':
			case 'failed':
				callback();
				break;
			default:
				this.requestState('detaching');
			case 'detaching':
				this.once(function(stateChange) {
					switch(this.event) {
						case 'detached':
							callback();
							break;
						case 'failed':
						case 'attached':
							callback(stateChange.reason || connectionManager.getStateError());
							break;
						default:
							/* this shouldn't happen ... */
							callback(ConnectionError.unknownChannelErr);
							break;
					}
				});
		}
	};

	RealtimeChannel.prototype.detachImpl = function(callback) {
		Logger.logAction(Logger.LOG_MICRO, 'RealtimeChannel.detach()', 'sending DETACH message');
		this.setInProgress(statechangeOp, true);
		var msg = ProtocolMessage.fromValues({action: actions.DETACH, channel: this.name});
		this.sendMessage(msg, (callback || noop));
	};

	RealtimeChannel.prototype.subscribe = function(/* [event], listener, [callback] */) {
		var args = RealtimeChannel.processListenerArgs(arguments);
		var event = args[0];
		var listener = args[1];
		var callback = args[2];
		var subscriptions = this.subscriptions;
		var events;

		if(this.state === 'failed') {
			callback(ErrorInfo.fromValues(RealtimeChannel.invalidStateError('failed')));
			return;
		}

		subscriptions.on(event, listener);

		this.attach(callback);
	};

	RealtimeChannel.prototype.unsubscribe = function(/* [event], listener, [callback] */) {
		var args = RealtimeChannel.processListenerArgs(arguments);
		var event = args[0];
		var listener = args[1];
		var callback = args[2];
		var subscriptions = this.subscriptions;
		var events;

		if(this.state === 'failed') {
			callback(ErrorInfo.fromValues(RealtimeChannel.invalidStateError('failed')));
			return;
		}

		subscriptions.off(event, listener);
	};

	RealtimeChannel.prototype.sync = function() {
		/* check preconditions */
		switch(this.state) {
			case 'initialized':
			case 'detaching':
			case 'detached':
				throw new ErrorInfo("Unable to sync to channel; not attached", 40000);
			default:
		}
		var connectionManager = this.connectionManager;
		if(!connectionManager.activeState()) {
			throw connectionManager.getStateError();
		}

		/* send sync request */
		var syncMessage = ProtocolMessage.fromValues({action: actions.SYNC, channel: this.name});
		if(this.syncChannelSerial) {
			syncMessage.channelSerial = this.syncChannelSerial;
		}
		connectionManager.send(syncMessage);
	};

	RealtimeChannel.prototype.sendMessage = function(msg, callback) {
		this.connectionManager.send(msg, this.realtime.options.queueMessages, callback);
	};

	RealtimeChannel.prototype.sendPresence = function(presence, callback) {
		var msg = ProtocolMessage.fromValues({
			action: actions.PRESENCE,
			channel: this.name,
			presence: (Utils.isArray(presence) ?
				PresenceMessage.fromValuesArray(presence) :
				[PresenceMessage.fromValues(presence)])
		});
		this.sendMessage(msg, callback);
	};

	RealtimeChannel.prototype.onMessage = function(message) {
		var syncChannelSerial, isSync = false;
		switch(message.action) {
		case actions.ATTACHED:
			this.attachSerial = message.channelSerial;
			this._mode = message.getMode();
			if(this.state === 'attached') {
				if(!message.hasFlag('RESUMED')) {
					/* On a loss of continuity, the presence set needs to be re-synced */
					this.presence.onAttached(message.hasFlag('HAS_PRESENCE'))
					var change = new ChannelStateChange(this.state, this.state, false, message.error);
					this.emit('update', change);
				}
			} else {
				this.notifyState('attached', message.error, message.hasFlag('RESUMED'), message.hasFlag('HAS_PRESENCE'));
			}
			break;

		case actions.DETACHED:
			var err = message.error ? ErrorInfo.fromValues(message.error) : new ErrorInfo('Channel detached', 90001, 404);
			if(this.state === 'detaching') {
				this.notifyState('detached', err);
			} else if(this.state === 'attaching') {
				/* Only retry immediately if we were previously attached. If we were
				 * attaching, go into suspended, fail messages, and wait a few seconds
				 * before retrying */
				this.notifyState('suspended', err);
			} else {
				this.requestState('attaching', err);
			}
			break;

		case actions.SYNC:
			/* syncs can have channelSerials, but might not if the sync is one page long */
			isSync = true;
			syncChannelSerial = this.syncChannelSerial = message.channelSerial;
			/* syncs can happen on channels with no presence data as part of connection
			 * resuming, in which case protocol message has no presence property */
			if(!message.presence) break;
		case actions.PRESENCE:
			var presence = message.presence,
				id = message.id,
				connectionId = message.connectionId,
				timestamp = message.timestamp;

			var options = this.channelOptions;
			for(var i = 0; i < presence.length; i++) {
				try {
					var presenceMsg = presence[i];
					PresenceMessage.decode(presenceMsg, options);
				} catch (e) {
					Logger.logAction(Logger.LOG_ERROR, 'RealtimeChannel.onMessage()', e.toString());
				}
				if(!presenceMsg.connectionId) presenceMsg.connectionId = connectionId;
				if(!presenceMsg.timestamp) presenceMsg.timestamp = timestamp;
				if(!presenceMsg.id) presenceMsg.id = id + ':' + i;
			}
			this.presence.setPresence(presence, isSync, syncChannelSerial);
			break;

		case actions.MESSAGE:
			var messages = message.messages,
				id = message.id,
				connectionId = message.connectionId,
				timestamp = message.timestamp;

			var options = this.channelOptions;
			for(var i = 0; i < messages.length; i++) {
				try {
					var msg = messages[i];
					Message.decode(msg, options);
				} catch (e) {
					/* decrypt failed .. the most likely cause is that we have the wrong key */
					Logger.logAction(Logger.LOG_ERROR, 'RealtimeChannel.onMessage()', e.toString());
				}
				if(!msg.connectionId) msg.connectionId = connectionId;
				if(!msg.timestamp) msg.timestamp = timestamp;
				if(!msg.id) msg.id = id + ':' + i;
			}
			this.onEvent(messages);
			break;

		case actions.ERROR:
			/* there was a channel-specific error */
			var err = message.error;
			if(err && err.code == 80016) {
				/* attach/detach operation attempted on superseded transport handle */
				this.checkPendingState();
			} else {
				this.notifyState('failed', ErrorInfo.fromValues(err));
			}
			break;

		default:
			Logger.logAction(Logger.LOG_ERROR, 'RealtimeChannel.onMessage()', 'Fatal protocol error: unrecognised action (' + message.action + ')');
			this.connectionManager.abort(ConnectionError.unknownChannelErr);
		}
	};

	RealtimeChannel.mergeTo = function(dest, src) {
		var result = false;
		var action;
		if(dest.channel == src.channel) {
			if((action = dest.action) == src.action) {
				switch(action) {
				case actions.MESSAGE:
					for(var i = 0; i < src.messages.length; i++)
						dest.messages.push(src.messages[i]);
					result = true;
					break;
				case actions.PRESENCE:
					for(var i = 0; i < src.presence.length; i++)
						dest.presence.push(src.presence[i]);
					result = true;
					break;
				default:
				}
			}
		}
		return result;
	};

	RealtimeChannel.prototype.onAttached = function() {
		Logger.logAction(Logger.LOG_MINOR, 'RealtimeChannel.onAttached', 'activating channel; name = ' + this.name);
	};

	RealtimeChannel.prototype.notifyState = function(state, reason, resumed, hasPresence) {
		Logger.logAction(Logger.LOG_MICRO, 'RealtimeChannel.notifyState', 'name = ' + this.name + ', current state = ' + this.state + ', notifying state ' + state);
		this.clearStateTimer();

		if(state === this.state) {
			return;
		}
		this.presence.actOnChannelState(state, hasPresence, reason);
		if(state === 'suspended' && this.connectionManager.state.sendEvents) {
			this.startRetryTimer();
		} else {
			this.cancelRetryTimer();
		}
		if(reason) {
			this.errorReason = reason;
		}
		var change = new ChannelStateChange(this.state, state, resumed, reason);
		var logLevel = state === 'failed' ? Logger.LOG_ERROR : Logger.LOG_MAJOR;
		Logger.logAction(logLevel, 'Channel state for channel "' + this.name + '"', state + (reason ? ('; reason: ' + reason) : ''));

		/* Note: we don't set inProgress for pending states until the request is actually in progress */
		if(state === 'attached') {
			this.onAttached();
			this.setInProgress(syncOp, hasPresence);
			this.setInProgress(statechangeOp, false);
		} else if(state === 'detached' || state === 'failed' || state === 'suspended') {
			this.setInProgress(statechangeOp, false);
			this.setInProgress(syncOp, false);
		}

		this.state = state;
		this.emit(state, change);
	};

	RealtimeChannel.prototype.requestState = function(state, reason) {
		Logger.logAction(Logger.LOG_MINOR, 'RealtimeChannel.requestState', 'name = ' + this.name + ', state = ' + state);
		this.notifyState(state, reason);
		/* send the event and await response */
		this.checkPendingState();
	};

	RealtimeChannel.prototype.checkPendingState = function() {
		/* if can't send events, do nothing */
		var cmState = this.connectionManager.state;
		/* Allow attach messages to queue up when synchronizing, since this will be
		 * the state we'll be in when upgrade transport.active triggers a checkpendingstate */
		if(!(cmState.sendEvents || cmState.forceQueueEvents)) {
			Logger.logAction(Logger.LOG_MINOR, 'RealtimeChannel.checkPendingState', 'sendEvents is false; state is ' + this.connectionManager.state.state);
			return;
		}

		Logger.logAction(Logger.LOG_MINOR, 'RealtimeChannel.checkPendingState', 'name = ' + this.name + ', state = ' + this.state);
		/* Only start the state timer running when actually sending the event */
		switch(this.state) {
			case 'attaching':
				this.startStateTimerIfNotRunning();
				this.attachImpl();
				break;
			case 'detaching':
				this.startStateTimerIfNotRunning();
				this.detachImpl();
				break;
			case 'attached':
				/* resume any sync operation that was in progress */
				this.sync();
			default:
				break;
		}
	};

	RealtimeChannel.prototype.timeoutPendingState = function() {
		switch(this.state) {
			case 'attaching':
				var err = new ErrorInfo('Channel attach timed out', 90007, 408);
				this.notifyState('suspended', err);
				break;
			case 'detaching':
				var err = new ErrorInfo('Channel detach timed out', 90007, 408);
				this.notifyState('attached', err);
				break;
			default:
				this.checkPendingState();
				break;
		}
	};

	RealtimeChannel.prototype.startStateTimerIfNotRunning = function() {
		var self = this;
		if(!this.stateTimer) {
			this.stateTimer = setTimeout(function() {
				Logger.logAction(Logger.LOG_MINOR, 'RealtimeChannel.startStateTimerIfNotRunning', 'timer expired');
				self.stateTimer = null;
				self.timeoutPendingState();
			}, this.realtime.options.timeouts.realtimeRequestTimeout);
		}
	};

	RealtimeChannel.prototype.clearStateTimer = function() {
		var stateTimer = this.stateTimer;
		if(stateTimer) {
			clearTimeout(stateTimer);
			this.stateTimer = null;
		}
	};

	RealtimeChannel.prototype.startRetryTimer = function() {
		var self = this;
		if(this.retryTimer) return;

		this.retryTimer = setTimeout(function() {
			/* If connection is not connected, just leave in suspended, a reattach
			 * will be triggered once it connects again */
			if(self.state === 'suspended' && self.connectionManager.state.sendEvents) {
				self.retryTimer = null;
				Logger.logAction(Logger.LOG_MINOR, 'RealtimeChannel retry timer expired', 'attempting a new attach');
				self.requestState('attaching');
			}
		}, this.realtime.options.timeouts.channelRetryTimeout);
	};

	RealtimeChannel.prototype.cancelRetryTimer = function() {
		if(this.retryTimer) {
			clearTimeout(this.retryTimer);
			this.suspendTimer = null;
		}
	};

	RealtimeChannel.prototype.setInProgress = function(operation, value) {
		this.rest.channels.setInProgress(this, operation, value);
	};

	RealtimeChannel.prototype.history = function(params, callback) {
		Logger.logAction(Logger.LOG_MICRO, 'RealtimeChannel.history()', 'channel = ' + this.name);
		/* params and callback are optional; see if params contains the callback */
		if(callback === undefined) {
			if(typeof(params) == 'function') {
				callback = params;
				params = null;
			} else {
				callback = noop;
			}
		}

		if(params && params.untilAttach) {
			if(this.state !== 'attached') {
				callback(new ErrorInfo("option untilAttach requires the channel to be attached", 40000, 400));
				return;
			}
			if(!this.attachSerial) {
				callback(new ErrorInfo("untilAttach was specified and channel is attached, but attachSerial is not defined", 40000, 400));
				return;
			}
			delete params.untilAttach;
			params.from_serial = this.attachSerial;
		}

		Channel.prototype._history.call(this, params, callback);
	};

	RealtimeChannel.prototype.whenState = function(state, listener) {
		EventEmitter.prototype.whenState.call(this, state, this.state, listener);
	}

	return RealtimeChannel;
})();

var RealtimePresence = (function() {
	var noop = function() {};

	function memberKey(item) {
		return item.clientId + ':' + item.connectionId;
	}

	function getClientId(realtimePresence) {
		return realtimePresence.channel.realtime.auth.clientId;
	}

	function isAnonymous(realtimePresence) {
		var realtime = realtimePresence.channel.realtime;
		/* If not currently connected, we can't assume that we're an anonymous
		 * client, as realtime may inform us of our clientId in the CONNECTED
		 * message. So assume we're not anonymous and leave it to realtime to
		 * return an error if we are */
		return !realtime.auth.clientId && realtime.connection.state === 'connected';
	}

	/* Callback is called only in the event of an error */
	function waitAttached(channel, callback, action) {
		switch(channel.state) {
			case 'attached':
			case 'suspended':
				action();
				break;
			case 'initialized':
			case 'detached':
			case 'detaching':
			case 'attaching':
				channel.attach(function(err) {
					if(err) callback(err);
					else action();
				});
				break;
			default:
				callback(ErrorInfo.fromValues(RealtimeChannel.invalidStateError(channel.state)));
		}
	}

	function RealtimePresence(channel, options) {
		Presence.call(this, channel);
		this.syncComplete = false;
		this.members = new PresenceMap(this);
		this._myMembers = new PresenceMap(this);
		this.subscriptions = new EventEmitter();
		this.pendingPresence = [];
	}
	Utils.inherits(RealtimePresence, Presence);

	RealtimePresence.prototype.enter = function(data, callback) {
		if(isAnonymous(this))
			throw new ErrorInfo('clientId must be specified to enter a presence channel', 40012, 400);
		this._enterOrUpdateClient(undefined, data, callback, 'enter');
	};

	RealtimePresence.prototype.update = function(data, callback) {
		if(isAnonymous(this))
			throw new ErrorInfo('clientId must be specified to update presence data', 40012, 400);
		this._enterOrUpdateClient(undefined, data, callback, 'update');
	};

	RealtimePresence.prototype.enterClient = function(clientId, data, callback) {
		this._enterOrUpdateClient(clientId, data, callback, 'enter');
	};

	RealtimePresence.prototype.updateClient = function(clientId, data, callback) {
		this._enterOrUpdateClient(clientId, data, callback, 'update');
	};

	RealtimePresence.prototype._enterOrUpdateClient = function(clientId, data, callback, action) {
		if (!callback) {
			if (typeof(data)==='function') {
				callback = data;
				data = null;
			} else {
				callback = noop;
			}
		}

		var channel = this.channel;
		if(!channel.connectionManager.activeState()) {
			callback(channel.connectionManager.getStateError());
			return;
		}

		Logger.logAction(Logger.LOG_MICRO, 'RealtimePresence.' + action + 'Client()',
		  action + 'ing; channel = ' + channel.name + ', client = ' + clientId || '(implicit) ' + getClientId(this));

		var presence = PresenceMessage.fromValues({
			action : action,
			data   : data
		});
		if (clientId) { presence.clientId = clientId; }

		var self = this;
		PresenceMessage.encode(presence, channel.channelOptions, function(err) {
			if (err) {
				callback(err);
				return;
			}
			switch(channel.state) {
				case 'attached':
					channel.sendPresence(presence, callback);
					break;
				case 'initialized':
				case 'detached':
					channel.attach();
				case 'attaching':
					self.pendingPresence.push({
						presence : presence,
						callback : callback
					});
					break;
				default:
					var err = new ErrorInfo('Unable to ' + action + ' presence channel (incompatible state)', 90001);
					err.code = 90001;
					callback(err);
			}
		});
	};

	RealtimePresence.prototype.leave = function(data, callback) {
		if(isAnonymous(this))
			throw new ErrorInfo('clientId must have been specified to enter or leave a presence channel', 40012, 400);
		this.leaveClient(undefined, data, callback);
	};

	RealtimePresence.prototype.leaveClient = function(clientId, data, callback) {
		if (!callback) {
			if (typeof(data)==='function') {
				callback = data;
				data = null;
			} else {
				callback = noop;
			}
		}

		var channel = this.channel;
		if(!channel.connectionManager.activeState()) {
			callback(channel.connectionManager.getStateError());
			return;
		}

		Logger.logAction(Logger.LOG_MICRO, 'RealtimePresence.leaveClient()', 'leaving; channel = ' + this.channel.name + ', client = ' + clientId);
		var presence = PresenceMessage.fromValues({
			action : 'leave',
			data   : data
		});
		if (clientId) { presence.clientId = clientId; }

		switch(channel.state) {
			case 'attached':
				channel.sendPresence(presence, callback);
				break;
			case 'attaching':
				this.pendingPresence.push({
					presence : presence,
					callback : callback
				});
				break;
			case 'initialized':
			case 'failed':
				/* we're not attached; therefore we let any entered status
				 * timeout by itself instead of attaching just in order to leave */
				var err = new ErrorInfo('Unable to leave presence channel (incompatible state)', 90001);
				callback(err);
				break;
			default:
				/* there is no connection; therefore we let
				 * any entered status timeout by itself */
				callback(ConnectionError.failed);
		}
	};

	RealtimePresence.prototype.get = function(/* params, callback */) {
		var args = Array.prototype.slice.call(arguments);
		if(args.length == 1 && typeof(args[0]) == 'function')
			args.unshift(null);

		var params = args[0],
			callback = args[1] || noop,
			waitForSync = !params || ('waitForSync' in params ? params.waitForSync : true)

		function returnMembers(members) {
			callback(null, params ? members.list(params) : members.values());
		}

		/* Special-case the suspended state: can still get (stale) presence set if waitForSync is false */
		if(this.channel.state === 'suspended') {
			if(waitForSync) {
				callback(ErrorInfo.fromValues({
					statusCode: 400,
					code: 91005,
					message: 'Presence state is out of sync due to channel being in the SUSPENDED state'
				}));
			} else {
				returnMembers(this.members);
			}
			return;
		}

		var self = this;
		waitAttached(this.channel, callback, function() {
			var members = self.members;
			if(waitForSync) {
				members.waitSync(function() {
					returnMembers(members);
				});
			} else {
				returnMembers(members);
			}
		});
	};

	RealtimePresence.prototype.history = function(params, callback) {
		Logger.logAction(Logger.LOG_MICRO, 'RealtimePresence.history()', 'channel = ' + this.name);
		/* params and callback are optional; see if params contains the callback */
		if(callback === undefined) {
			if(typeof(params) == 'function') {
				callback = params;
				params = null;
			} else {
				callback = noop;
			}
		}

		if(params && params.untilAttach) {
			if(this.channel.state === 'attached') {
				delete params.untilAttach;
				params.from_serial = this.channel.attachSerial;
			} else {
				callback(new ErrorInfo("option untilAttach requires the channel to be attached, was: " + this.channel.state, 40000, 400));
			}
		}

		Presence.prototype._history.call(this, params, callback);
	};

	RealtimePresence.prototype.setPresence = function(presenceSet, isSync, syncChannelSerial) {
		Logger.logAction(Logger.LOG_MICRO, 'RealtimePresence.setPresence()', 'received presence for ' + presenceSet.length + ' participants; syncChannelSerial = ' + syncChannelSerial);
		var syncCursor, match, members = this.members, myMembers = this._myMembers,
			broadcastMessages = [], connId = this.channel.connectionManager.connectionId;

		if(isSync) {
			this.members.startSync();
			if(syncChannelSerial && (match = syncChannelSerial.match(/^[\w\-]+:(.*)$/))) {
				syncCursor = match[1];
			}
		}

		for(var i = 0; i < presenceSet.length; i++) {
			var presence = PresenceMessage.fromValues(presenceSet[i]);
			switch(presence.action) {
				case 'leave':
					if(members.remove(presence)) {
						broadcastMessages.push(presence);
					}
					if(presence.connectionId === connId && !presence.isSynthesized()) {
						myMembers.remove(presence);
					}
					break;
				case 'enter':
				case 'present':
				case 'update':
					if(members.put(presence)) {
						broadcastMessages.push(presence);
					}
					if(presence.connectionId === connId) {
						myMembers.put(presence);
					}
					break;
			}
		}
		/* if this is the last (or only) message in a sequence of sync updates, end the sync */
		if(isSync && !syncCursor) {
			members.endSync();
			/* RTP5c2: re-enter our own members if they haven't shown up in the sync */
			this._ensureMyMembersPresent();
			this.channel.setInProgress(RealtimeChannel.progressOps.sync, false);
			this.channel.syncChannelSerial = null;
		}

		/* broadcast to listeners */
		for(var i = 0; i < broadcastMessages.length; i++) {
			var presence = broadcastMessages[i];
			this.subscriptions.emit(presence.action, presence);
		}
	};

	RealtimePresence.prototype.onAttached = function(hasPresence) {
		Logger.logAction(Logger.LOG_MINOR, 'RealtimePresence.onAttached()', 'channel = ' + this.channel.name + ', hasPresence = ' + hasPresence);

		if(hasPresence) {
			this.members.startSync();
		} else {
			this._synthesizeLeaves(this.members.values());
			this.members.clear();
			this._ensureMyMembersPresent();
		}

		/* NB this must be after the _ensureMyMembersPresent call, which may add items to pendingPresence */
		var pendingPresence = this.pendingPresence,
			pendingPresCount = pendingPresence.length;

		if(pendingPresCount) {
			this.pendingPresence = [];
			var presenceArray = [];
			var multicaster = Multicaster();
			Logger.logAction(Logger.LOG_MICRO, 'RealtimePresence.onAttached', 'sending ' + pendingPresCount + ' queued presence messages');
			for(var i = 0; i < pendingPresCount; i++) {
				var event = pendingPresence[i];
				presenceArray.push(event.presence);
				multicaster.push(event.callback);
			}
			this.channel.sendPresence(presenceArray, multicaster);
		}
	};

	RealtimePresence.prototype.actOnChannelState = function(state, hasPresence, err) {
		switch(state) {
			case 'attached':
				this.onAttached(hasPresence);
				break;
			case 'detached':
			case 'failed':
				this._clearMyMembers();
				this.members.clear();
				/* falls through */
			case 'suspended':
				this.failPendingPresence(err);
				break;
		}
	};

	RealtimePresence.prototype.failPendingPresence = function(err) {
		if(this.pendingPresence.length) {
			Logger.logAction(Logger.LOG_MINOR, 'RealtimeChannel.failPendingPresence', 'channel; name = ' + this.channel.name + ', err = ' + Utils.inspectError(err));
			for(var i = 0; i < this.pendingPresence.length; i++)
				try {
					this.pendingPresence[i].callback(err);
				} catch(e) {}
			this.pendingPresence = [];
		}
	};

	RealtimePresence.prototype._clearMyMembers = function() {
		this._myMembers.clear();
	};

	RealtimePresence.prototype._ensureMyMembersPresent = function() {
		var self = this, members = this.members, myMembers = this._myMembers,
			reenterCb = function(err) {
				if(err) {
					var msg = 'Presence auto-re-enter failed: ' + err.toString();
					var wrappedErr = new ErrorInfo(msg, 91004, 400);
					Logger.logAction(Logger.LOG_ERROR, 'RealtimePresence._ensureMyMembersPresent()', msg);
					var change = new ChannelStateChange(self.channel.state, self.channel.state, true, wrappedErr);
					self.channel.emit('update', change);
				}
			};

		for(var memberKey in myMembers.map) {
			if(!(memberKey in members.map)) {
				var entry = myMembers.map[memberKey];
				Logger.logAction(Logger.LOG_MICRO, 'RealtimePresence._ensureMyMembersPresent()', 'Auto-reentering clientId "' + entry.clientId + '" into the presence set');
				this._enterOrUpdateClient(entry.clientId, entry.data, reenterCb, 'enter');
				delete myMembers.map[memberKey];
			}
		}
	};

	RealtimePresence.prototype._synthesizeLeaves = function(items) {
		var subscriptions = this.subscriptions;
		Utils.arrForEach(items, function(item) {
			var presence = PresenceMessage.fromValues({
				action: 'leave',
				connectionId: item.connectionId,
				clientId: item.clientId,
				data: item.data,
				encoding: item.encoding,
				timestamp: Utils.now()
			});
			subscriptions.emit('leave', presence);
		});
	};

	/* Deprecated */
	RealtimePresence.prototype.on = function() {
		Logger.deprecated('presence.on', 'presence.subscribe');
		this.subscribe.apply(this, arguments);
	};

	/* Deprecated */
	RealtimePresence.prototype.off = function() {
		Logger.deprecated('presence.off', 'presence.unsubscribe');
		this.unsubscribe.apply(this, arguments);
	};

	RealtimePresence.prototype.subscribe = function(/* [event], listener, [callback] */) {
		var args = RealtimeChannel.processListenerArgs(arguments);
		var event = args[0];
		var listener = args[1];
		var callback = args[2];
		var channel = this.channel;
		var self = this;

		if(channel.state === 'failed') {
			callback(ErrorInfo.fromValues(RealtimeChannel.invalidStateError('failed')));
			return;
		}

		this.subscriptions.on(event, listener);
		channel.attach(callback);
	};

	RealtimePresence.prototype.unsubscribe = function(/* [event], listener, [callback] */) {
		var args = RealtimeChannel.processListenerArgs(arguments);
		var event = args[0];
		var listener = args[1];
		var callback = args[2];

		if(this.channel.state === 'failed') {
			callback(ErrorInfo.fromValues(RealtimeChannel.invalidStateError('failed')));
			return;
		}

		this.subscriptions.off(event, listener);
	};

	function PresenceMap(presence) {
		EventEmitter.call(this);
		this.presence = presence;
		this.map = {};
		this.syncInProgress = false;
		this.residualMembers = null;
	}
	Utils.inherits(PresenceMap, EventEmitter);

	PresenceMap.prototype.get = function(key) {
		return this.map[key];
	};

	PresenceMap.prototype.getClient = function(clientId) {
		var map = this.map, result = [];
		for(var key in map) {
			var item = map[key];
			if(item.clientId == clientId && item.action != 'absent')
				result.push(item);
		}
		return result;
	};

	PresenceMap.prototype.list = function(params) {
		var map = this.map,
			clientId = params && params.clientId,
			connectionId = params && params.connectionId,
			result = [];

		for(var key in map) {
			var item = map[key];
			if(item.action === 'absent') continue;
			if(clientId && clientId != item.clientId) continue;
			if(connectionId && connectionId != item.connectionId) continue;
			result.push(item);
		}
		return result;
	};

	function newerThan(item, existing) {
		/* RTP2b1: if either is synthesised, compare by timestamp */
		if(item.isSynthesized() || existing.isSynthesized()) {
			return item.timestamp > existing.timestamp;
		}

		/* RTP2b2 */
		var itemOrderings = item.parseId(),
			existingOrderings = existing.parseId();
		if(itemOrderings.msgSerial === existingOrderings.msgSerial) {
			return itemOrderings.index > existingOrderings.index;
		} else {
			return itemOrderings.msgSerial > existingOrderings.msgSerial;
		}
	}

	PresenceMap.prototype.put = function(item) {
		if(item.action === 'enter' || item.action === 'update') {
			item = PresenceMessage.fromValues(item);
			item.action = 'present';
		}
		var map = this.map, key = memberKey(item);
		/* we've seen this member, so do not remove it at the end of sync */
		if(this.residualMembers)
			delete this.residualMembers[key];

		/* compare the timestamp of the new item with any existing member (or ABSENT witness) */
		var existingItem = map[key];
		if(existingItem && !newerThan(item, existingItem)) {
			return false;
		}
		map[key] = item;
		return true;

	};

	PresenceMap.prototype.values = function() {
		var map = this.map, result = [];
		for(var key in map) {
			var item = map[key];
			if(item.action != 'absent')
				result.push(item);
		}
		return result;
	};

	PresenceMap.prototype.remove = function(item) {
		var map = this.map, key = memberKey(item);
		var existingItem = map[key];

		if(existingItem && !newerThan(item, existingItem)) {
			return false;
		}

		/* RTP2f */
		if(this.syncInProgress) {
			item = PresenceMessage.fromValues(item);
			item.action = 'absent';
			map[key] = item;
		} else {
			delete map[key];
		}

		return true;
	};

	PresenceMap.prototype.startSync = function() {
		var map = this.map, syncInProgress = this.syncInProgress;
		Logger.logAction(Logger.LOG_MINOR, 'PresenceMap.startSync()', 'channel = ' + this.presence.channel.name + '; syncInProgress = ' + syncInProgress);
		/* we might be called multiple times while a sync is in progress */
		if(!this.syncInProgress) {
			this.residualMembers = Utils.copy(map);
			this.setInProgress(true);
		}
	};

	PresenceMap.prototype.endSync = function() {
		var map = this.map, syncInProgress = this.syncInProgress;
		Logger.logAction(Logger.LOG_MINOR, 'PresenceMap.endSync()', 'channel = ' + this.presence.channel.name + '; syncInProgress = ' + syncInProgress);
		if(syncInProgress) {
			/* we can now strip out the ABSENT members, as we have
			 * received all of the out-of-order sync messages */
			for(var memberKey in map) {
				var entry = map[memberKey];
				if(entry.action === 'absent') {
					delete map[memberKey];
				}
			}
			/* any members that were present at the start of the sync,
			 * and have not been seen in sync, can be removed, and leave events emitted */
			this.presence._synthesizeLeaves(Utils.valuesArray(this.residualMembers));
			for(var memberKey in this.residualMembers) {
				delete map[memberKey];
			}
			this.residualMembers = null;

			/* finish, notifying any waiters */
			this.setInProgress(false);
		}
		this.emit('sync');
	};

	PresenceMap.prototype.waitSync = function(callback) {
		var syncInProgress = this.syncInProgress;
		Logger.logAction(Logger.LOG_MINOR, 'PresenceMap.waitSync()', 'channel = ' + this.presence.channel.name + '; syncInProgress = ' + syncInProgress);
		if(!syncInProgress) {
			callback();
			return;
		}
		this.once('sync', callback);
	};

	PresenceMap.prototype.clear = function(callback) {
		this.map = {};
		this.setInProgress(false);
		this.residualMembers = null;
	};

	PresenceMap.prototype.setInProgress = function(inProgress) {
		Logger.logAction(Logger.LOG_MICRO, 'PresenceMap.setInProgress()', 'inProgress = ' + inProgress);
		this.syncInProgress = inProgress;
		this.presence.syncComplete = !inProgress;
	};

	return RealtimePresence;
})();

var XHRRequest = (function() {
	var noop = function() {};
	var idCounter = 0;
	var pendingRequests = {};

	var REQ_SEND = 0,
		REQ_RECV = 1,
		REQ_RECV_POLL = 2,
		REQ_RECV_STREAM = 3;

	function clearPendingRequests() {
		for(var id in pendingRequests)
			pendingRequests[id].dispose();
	}

	var isIE = typeof window !== 'undefined' && window.XDomainRequest;

	function ieVersion() {
		var match = navigator.userAgent.toString().match(/MSIE\s([\d.]+)/);
		return match && Number(match[1]);
	}

	function needJsonEnvelope() {
		/* IE 10 xhr bug: http://stackoverflow.com/a/16320339 */
		var version;
		return isIE && (version = ieVersion()) && version === 10;
	}

	function getHeader(xhr, header) {
		return xhr.getResponseHeader && xhr.getResponseHeader(header);
	}

	/* Safari mysteriously returns 'Identity' for transfer-encoding
	 * when in fact it is 'chunked'. So instead, decide that it is
	 * chunked when transfer-encoding is present, content-length is absent */
	function isEncodingChunked(xhr) {
		return xhr.getResponseHeader
			&& xhr.getResponseHeader('transfer-encoding')
			&& !xhr.getResponseHeader('content-length');
	}

	function getHeadersAsObject(xhr) {
		var headerPairs = Utils.trim(xhr.getAllResponseHeaders()).split('\r\n'),
			headers = {};
		for (var i = 0; i < headerPairs.length; i++) {
			var parts = Utils.arrMap(headerPairs[i].split(':'), Utils.trim);
			headers[parts[0].toLowerCase()] = parts[1];
		}
		return headers;
	}

	function XHRRequest(uri, headers, params, body, requestMode, timeouts) {
		EventEmitter.call(this);
		params = params || {};
		params.rnd = Utils.randStr();
		if(needJsonEnvelope() && !params.envelope)
			params.envelope = 'json';
		this.uri = uri + Utils.toQueryString(params);
		this.headers = headers || {};
		this.body = body;
		this.requestMode = requestMode;
		this.timeouts = timeouts;
		this.timedOut = false;
		this.requestComplete = false;
		pendingRequests[this.id = String(++idCounter)] = this;
	}
	Utils.inherits(XHRRequest, EventEmitter);

	var createRequest = XHRRequest.createRequest = function(uri, headers, params, body, requestMode, timeouts) {
		/* XHR requests are used either with the context being a realtime
		 * transport, or with timeouts passed in (for when used by a rest client),
		 * or completely standalone.  Use the appropriate timeouts in each case */
		timeouts = (this && this.timeouts) || timeouts || Defaults.TIMEOUTS;
		return new XHRRequest(uri, headers, Utils.copy(params), body, requestMode, timeouts);
	};

	XHRRequest.prototype.complete = function(err, body, headers, unpacked, statusCode) {
		if(!this.requestComplete) {
			this.requestComplete = true;
			if(body)
				this.emit('data', body);
			this.emit('complete', err, body, headers, unpacked, statusCode);
			this.dispose();
		}
	};

	XHRRequest.prototype.abort = function() {
		this.dispose();
	};

	XHRRequest.prototype.exec = function() {
		var timeout = (this.requestMode == REQ_SEND) ? this.timeouts.httpRequestTimeout : this.timeouts.recvTimeout,
			self = this,
			timer = this.timer = setTimeout(function() {
				self.timedOut = true;
				xhr.abort();
			}, timeout),
			body = this.body,
			method = Utils.isEmptyArg(body) ? 'GET' : 'POST',
			headers = this.headers,
			xhr = this.xhr = new XMLHttpRequest(),
			accept = headers['accept'],
			responseType = 'text';

		if(!accept) {
			headers['accept'] = 'application/json';
		} else if(accept.indexOf('application/x-msgpack') === 0) {
			responseType = 'arraybuffer';
		}

		if(body) {
			var contentType = headers['content-type'] || (headers['content-type'] = 'application/json');
			if(contentType.indexOf('application/json') > -1 && typeof(body) != 'string')
				body = JSON.stringify(body);
		}

		xhr.open(method, this.uri, true);
		xhr.responseType = responseType;

		if ('authorization' in headers) {
			xhr.withCredentials = 'true';
		}

		for(var h in headers)
			xhr.setRequestHeader(h, headers[h]);

		var errorHandler = function(errorEvent, message, code, statusCode) {
			var errorMessage = message + ' (event type: ' + errorEvent.type + ')' + (self.xhr.statusText ? ', current statusText is ' + self.xhr.statusText : '');
			Logger.logAction(Logger.LOG_ERROR, 'Request.on' + errorEvent.type + '()', errorMessage);
			self.complete(new ErrorInfo(errorMessage, code, statusCode));
		};
		xhr.onerror = function(errorEvent) {
			errorHandler(errorEvent, 'XHR error occurred', null, 400);
		}
		xhr.onabort = function(errorEvent) {
			if(self.timedOut) {
				errorHandler(errorEvent, 'Request aborted due to request timeout expiring', null, 408);
			} else {
				errorHandler(errorEvent, 'Request cancelled', null, 400);
			}
		};
		xhr.ontimeout = function(errorEvent) {
			errorHandler(errorEvent, 'Request timed out', null, 408);
		};

		var streaming,
			statusCode,
			responseBody,
			contentType,
			successResponse,
			streamPos = 0,
			unpacked = false;

		function onResponse() {
			clearTimeout(timer);
			successResponse = (statusCode < 400);
			if(statusCode == 204) {
				self.complete(null, null, null, null, statusCode);
				return;
			}
			streaming = (self.requestMode == REQ_RECV_STREAM && successResponse && isEncodingChunked(xhr));
		}

		function onEnd() {
			try {
				var contentType = getHeader(xhr, 'content-type'),
					headers,
					responseBody,
					/* Be liberal in what we accept; buggy auth servers may respond
					 * without the correct contenttype, but assume they're still
					 * responding with json */
					json = contentType ? (contentType.indexOf('application/json') >= 0) : (xhr.responseType == 'text');

				if(json) {
					/* If we requested msgpack but server responded with json, then since
					 * we set the responseType expecting msgpack, the response will be
					 * an ArrayBuffer containing json */
					responseBody = (xhr.responseType === 'arraybuffer') ? BufferUtils.utf8Decode(xhr.response) : String(xhr.responseText);
					if(responseBody.length) {
						responseBody = JSON.parse(responseBody);
					}
					unpacked = true;
				} else {
					responseBody = xhr.response;
				}

				if(responseBody.response !== undefined) {
					/* unwrap JSON envelope */
					statusCode = responseBody.statusCode;
					successResponse = (statusCode < 400);
					headers = responseBody.headers;
					responseBody = responseBody.response;
				} else {
					headers = getHeadersAsObject(xhr);
				}
			} catch(e) {
				self.complete(new ErrorInfo('Malformed response body from server: ' + e.message, null, 400));
				return;
			}

			/* If response is an array, it's an array of protocol messages -- even if
			 * is contains an error action (hence the nonsuccess statuscode), we can
			 * consider the request to have succeeded, just pass it on to
			 * onProtocolMessage to decide what to do */
			if(successResponse || Utils.isArray(responseBody)) {
				self.complete(null, responseBody, headers, unpacked, statusCode);
				return;
			}

			var err = responseBody.error;
			if(!err) {
				err = new ErrorInfo('Error response received from server: ' + statusCode + ' body was: ' + Utils.inspect(responseBody), null, statusCode);
			}
			self.complete(err, responseBody, headers, unpacked, statusCode);
		}

		function onProgress() {
			responseBody = xhr.responseText;
			var bodyEnd = responseBody.length - 1, idx, chunk;
			while((streamPos < bodyEnd) && (idx = responseBody.indexOf('\n', streamPos)) > -1) {
				chunk = responseBody.slice(streamPos, idx);
				streamPos = idx + 1;
				onChunk(chunk);
			}
		}

		function onChunk(chunk) {
			try {
				chunk = JSON.parse(chunk);
			} catch(e) {
				self.complete(new ErrorInfo('Malformed response body from server: ' + e.message, null, 400));
				return;
			}
			self.emit('data', chunk);
		}

		function onStreamEnd() {
			onProgress();
			self.streamComplete = true;
			Utils.nextTick(function() {
				self.complete();
			});
		}

		xhr.onreadystatechange = function() {
			var readyState = xhr.readyState;
			if(readyState < 3) return;
			if(xhr.status !== 0) {
				if(statusCode === undefined) {
					statusCode = xhr.status;
					/* IE returns 1223 for 204: http://bugs.jquery.com/ticket/1450 */
					if(statusCode === 1223) statusCode = 204;
					onResponse();
				}
				if(readyState == 3 && streaming) {
					onProgress();
				} else if(readyState == 4) {
					if(streaming)
						onStreamEnd();
					else
						onEnd();
				}
			}
		};
		xhr.send(body);
	};

	XHRRequest.prototype.dispose = function() {
		var xhr = this.xhr;
		if(xhr) {
			xhr.onreadystatechange = xhr.onerror = xhr.onabort = xhr.ontimeout = noop;
			this.xhr = null;
			var timer = this.timer;
			if(timer) {
				clearTimeout(timer);
				this.timer = null;
			}
			if(!this.requestComplete)
				xhr.abort();
		}
		delete pendingRequests[this.id];
	};

	if(Platform.xhrSupported) {
		if(typeof DomEvent === 'object') {
			DomEvent.addUnloadListener(clearPendingRequests);
		}
		if(typeof(Http) !== 'undefined') {
			Http.supportsAuthHeaders = true;
			Http.Request = function(rest, uri, headers, params, body, callback) {
				var req = createRequest(uri, headers, params, body, REQ_SEND, rest && rest.options.timeouts);
				req.once('complete', callback);
				req.exec();
				return req;
			};

			Http.checkConnectivity = function(callback) {
				var upUrl = Defaults.internetUpUrl;
				Logger.logAction(Logger.LOG_MICRO, '(XHRRequest)Http.checkConnectivity()', 'Sending; ' + upUrl);
				Http.Request(null, upUrl, null, null, null, function(err, responseText) {
					var result = (!err && responseText.replace(/\n/, '') == 'yes');
					Logger.logAction(Logger.LOG_MICRO, '(XHRRequest)Http.checkConnectivity()', 'Result: ' + result);
					callback(null, result);
				});
			};
		}
	}

	return XHRRequest;
})();

var XHRStreamingTransport = (function() {
	var shortName = 'xhr_streaming';

	/* public constructor */
	function XHRStreamingTransport(connectionManager, auth, params) {
		CometTransport.call(this, connectionManager, auth, params);
		this.shortName = shortName;
	}
	Utils.inherits(XHRStreamingTransport, CometTransport);

	XHRStreamingTransport.isAvailable = function() {
		return Platform.xhrSupported && Platform.streamingSupported && Platform.allowComet;
	};

	XHRStreamingTransport.tryConnect = function(connectionManager, auth, params, callback) {
		var transport = new XHRStreamingTransport(connectionManager, auth, params);
		var errorCb = function(err) { callback({event: this.event, error: err}); };
		transport.on(['failed', 'disconnected'], errorCb);
		transport.on('preconnect', function() {
			Logger.logAction(Logger.LOG_MINOR, 'XHRStreamingTransport.tryConnect()', 'viable transport ' + transport);
			transport.off(['failed', 'disconnected'], errorCb);
			callback(null, transport);
		});
		transport.connect();
	};

	XHRStreamingTransport.prototype.toString = function() {
		return 'XHRStreamingTransport; uri=' + this.baseUri + '; isConnected=' + this.isConnected;
	};

	XHRStreamingTransport.prototype.createRequest = XHRRequest.createRequest;

	if(typeof(ConnectionManager) !== 'undefined' && XHRStreamingTransport.isAvailable()) {
		ConnectionManager.supportedTransports[shortName] = XHRStreamingTransport;
	}

	return XHRStreamingTransport;
})();

var XHRPollingTransport = (function() {
	var shortName = 'xhr_polling';

	function XHRPollingTransport(connectionManager, auth, params) {
		params.stream = false;
		CometTransport.call(this, connectionManager, auth, params);
		this.shortName = shortName;
	}
	Utils.inherits(XHRPollingTransport, CometTransport);

	XHRPollingTransport.isAvailable = function() {
		return Platform.xhrSupported && Platform.allowComet;
	};

	XHRPollingTransport.tryConnect = function(connectionManager, auth, params, callback) {
		var transport = new XHRPollingTransport(connectionManager, auth, params);
		var errorCb = function(err) { callback({event: this.event, error: err}); };
		transport.on(['failed', 'disconnected'], errorCb);
		transport.on('preconnect', function() {
			Logger.logAction(Logger.LOG_MINOR, 'XHRPollingTransport.tryConnect()', 'viable transport ' + transport);
			transport.off(['failed', 'disconnected'], errorCb);
			callback(null, transport);
		});
		transport.connect();
	};

	XHRPollingTransport.prototype.toString = function() {
		return 'XHRPollingTransport; uri=' + this.baseUri + '; isConnected=' + this.isConnected;
	};

	XHRPollingTransport.prototype.createRequest = XHRRequest.createRequest;

	if(typeof(ConnectionManager) !== 'undefined' && XHRPollingTransport.isAvailable()) {
		ConnectionManager.supportedTransports[shortName] = XHRPollingTransport;
	}

	return XHRPollingTransport;
})();

var JSONPTransport = (function() {
	var noop = function() {};
	/* Can't just use windows.Ably, as that won't exist if using the commonjs version. */
	var _ = window._ablyjs_jsonp = {};

	/* express strips out parantheses from the callback!
	 * Kludge to still alow its responses to work, while not keeping the
	 * function form for normal use and not cluttering window.Ably
	 * https://github.com/strongloop/express/blob/master/lib/response.js#L305
	 */
	_._ = function(id) { return _['_' + id] || noop; };
	var idCounter = 1;
	var head = null;
	var shortName = 'jsonp';

	/* public constructor */
	function JSONPTransport(connectionManager, auth, params) {
		params.stream = false;
		CometTransport.call(this, connectionManager, auth, params);
		this.shortName = shortName;
	}
	Utils.inherits(JSONPTransport, CometTransport);

	JSONPTransport.isAvailable = function() {
		return Platform.jsonpSupported && Platform.allowComet;
	};
	if(JSONPTransport.isAvailable()) {
		ConnectionManager.supportedTransports[shortName] = JSONPTransport;
		head = document.getElementsByTagName('head')[0];
	}

	/* connectivity check; since this has a hard-coded callback id,
	 * we just make sure that we handle concurrent requests (but the
	 * connectionmanager should ensure this doesn't happen anyway */
	var checksInProgress = null;
	window.JSONPTransport = JSONPTransport

	JSONPTransport.tryConnect = function(connectionManager, auth, params, callback) {
		var transport = new JSONPTransport(connectionManager, auth, params);
		var errorCb = function(err) { callback({event: this.event, error: err}); };
		transport.on(['failed', 'disconnected'], errorCb);
		transport.on('preconnect', function() {
			Logger.logAction(Logger.LOG_MINOR, 'JSONPTransport.tryConnect()', 'viable transport ' + transport);
			transport.off(['failed', 'disconnected'], errorCb);
			callback(null, transport);
		});
		transport.connect();
	};

	JSONPTransport.prototype.toString = function() {
		return 'JSONPTransport; uri=' + this.baseUri + '; isConnected=' + this.isConnected;
	};

	var createRequest = JSONPTransport.prototype.createRequest = function(uri, headers, params, body, requestMode, timeouts) {
		/* JSONP requests are used either with the context being a realtime
		 * transport, or with timeouts passed in (for when used by a rest client),
		 * or completely standalone.  Use the appropriate timeouts in each case */
		timeouts = (this && this.timeouts) || timeouts || Defaults.TIMEOUTS;
		return new Request(undefined, uri, headers, Utils.copy(params), body, requestMode, timeouts);
	};

	function Request(id, uri, headers, params, body, requestMode, timeouts) {
		EventEmitter.call(this);
		if(id === undefined) id = idCounter++;
		this.id = id;
		this.uri = uri;
		this.params = params || {};
		this.params.rnd = Utils.randStr();
		if(headers) {
			/* JSONP doesn't allow headers. Cherry-pick a couple to turn into qs params */
			if(headers['X-Ably-Version']) this.params.v = headers['X-Ably-Version'];
			if(headers['X-Ably-Lib']) this.params.lib = headers['X-Ably-Lib'];
		}
		this.body = body;
		this.requestMode = requestMode;
		this.timeouts = timeouts;
		this.requestComplete = false;
	}
	Utils.inherits(Request, EventEmitter);

	Request.prototype.exec = function() {
		var id = this.id,
			body = this.body,
			uri = this.uri,
			params = this.params,
			self = this;

		params.callback = '_ablyjs_jsonp._(' + id + ')';

		params.envelope = 'jsonp';
		if(body)
			params.body = body;

		var script = this.script = document.createElement('script');
		var src = uri + Utils.toQueryString(params);
		script.src = src;
		if(script.src.split('/').slice(-1)[0] !== src.split('/').slice(-1)[0]) {
			/* The src has been truncated. Can't abort, but can at least emit an
			 * error so the user knows what's gone wrong. (Can't compare strings
			 * directly as src may have a port, script.src won't) */
			Logger.logAction(Logger.LOG_ERROR, 'JSONP Request.exec()', 'Warning: the browser appears to have truncated the script URI. This will likely result in the request failing due to an unparseable body param');
		}
		script.async = true;
		script.type = 'text/javascript';
		script.charset = 'UTF-8';
		script.onerror = function(err) {
			self.complete(new ErrorInfo('JSONP script error (event: ' + Utils.inspect(err) + ')', null, 400));
		};

		_['_' + id] = function(message) {
			if(message.statusCode) {
				/* Handle as enveloped jsonp, as all jsonp transport uses should be */
				var response = message.response;
				if(message.statusCode == 204) {
					self.complete(null, null, null, message.statusCode);
				} else if(!response) {
					self.complete(new ErrorInfo('Invalid server response: no envelope detected', null, 500));
				} else if(message.statusCode < 400 || Utils.isArray(response)) {
					/* If response is an array, it's an array of protocol messages -- even if
					 * it contains an error action (hence the nonsuccess statuscode), we can
					 * consider the request to have succeeded, just pass it on to
					 * onProtocolMessage to decide what to do */
					self.complete(null, response, message.headers, message.statusCode);
				} else {
					var err = response.error || new ErrorInfo('Error response received from server', null, message.statusCode);
					self.complete(err);
				}
			} else {
				/* Handle as non-enveloped -- as will be eg from a customer's authUrl server */
				self.complete(null, message);
			}
		};

		var timeout = (this.requestMode == CometTransport.REQ_SEND) ? this.timeouts.httpRequestTimeout : this.timeouts.recvTimeout;
		this.timer = setTimeout(function() { self.abort(); }, timeout);
		head.insertBefore(script, head.firstChild);
	};

	Request.prototype.complete = function(err, body, headers, statusCode) {
		headers = headers || {};
		if(!this.requestComplete) {
			this.requestComplete = true;
			var contentType;
			if(body) {
				contentType = (typeof(body) == 'string') ? 'text/plain' : 'application/json';
				headers['content-type'] = contentType;
				this.emit('data', body);
			}

			this.emit('complete', err, body, headers, /* unpacked: */ true, statusCode);
			this.dispose();
		}
	};

	Request.prototype.abort = function() {
		this.dispose();
	};

	Request.prototype.dispose = function() {
		var timer = this.timer;
		if(timer) {
			clearTimeout(timer);
			this.timer = null;
		}
		var script = this.script;
		if(script.parentNode) script.parentNode.removeChild(script);
		delete _[this.id];
		this.emit('disposed');
	};

	if(!Http.Request) {
		Http.Request = function(rest, uri, headers, params, body, callback) {
			var req = createRequest(uri, headers, params, body, CometTransport.REQ_SEND, rest && rest.options.timeouts);
			req.once('complete', callback);
			Utils.nextTick(function() {
				req.exec();
			});
			return req;
		};

		Http.checkConnectivity = function(callback) {
			var upUrl = Defaults.jsonpInternetUpUrl;

			if(checksInProgress) {
				checksInProgress.push(callback);
				return;
			}
			checksInProgress = [callback];
			Logger.logAction(Logger.LOG_MICRO, '(JSONP)Http.checkConnectivity()', 'Sending; ' + upUrl);

			var req = new Request('isTheInternetUp', upUrl, null, null, null, CometTransport.REQ_SEND, Defaults.TIMEOUTS);
			req.once('complete', function(err, response) {
				var result = !err && response;
				Logger.logAction(Logger.LOG_MICRO, '(JSONP)Http.checkConnectivity()', 'Result: ' + result);
				for(var i = 0; i < checksInProgress.length; i++) checksInProgress[i](null, result);
				checksInProgress = null;
			});
			Utils.nextTick(function() {
				req.exec();
			});
		};
	}

	return JSONPTransport;
})();

	if(typeof Realtime !== 'undefined') {
		Ably.msgpack = msgpack;
		Ably.Rest = Rest;
		Ably.Realtime = Realtime;
		Realtime.ConnectionManager = ConnectionManager;
		Realtime.BufferUtils = Rest.BufferUtils = BufferUtils;
		if(typeof(Crypto) !== 'undefined') Realtime.Crypto = Rest.Crypto = Crypto;
		Realtime.Defaults = Rest.Defaults = Defaults;
		Realtime.Http = Rest.Http = Http;
		Realtime.Utils = Rest.Utils = Utils;
		Realtime.Http = Rest.Http = Http;
		Realtime.Message = Rest.Message = Message;
		Realtime.PresenceMessage = Rest.PresenceMessage = PresenceMessage;
		Realtime.ProtocolMessage = Rest.ProtocolMessage = ProtocolMessage;
	}
}).call({});

/* CommonJS support */
if (typeof exports === "object" && exports) {
	for (var obj in window.Ably) {
		if (window.Ably.hasOwnProperty(obj)) {
			exports[obj] = window.Ably[obj];
		}
	}
	/* SystemJS support for default exports to be added to the root of the module
	   https://github.com/frankwallis/plugin-typescript/issues/185 */
	exports.__esModule = true;
}

/* AMD support */
if (typeof define === "function" && define.amd) {
  define("ably", [], function() {
    return window.Ably;
  });
}

/** @license React v17.0.2
 * react.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
(function(){'use strict';(function(c,x){"object"===typeof exports&&"undefined"!==typeof module?x(exports):"function"===typeof define&&define.amd?define(["exports"],x):(c=c||self,x(c.React={}))})(this,function(c){function x(a){if(null===a||"object"!==typeof a)return null;a=Y&&a[Y]||a["@@iterator"];return"function"===typeof a?a:null}function y(a){for(var b="https://reactjs.org/docs/error-decoder.html?invariant="+a,e=1;e<arguments.length;e++)b+="&args[]="+encodeURIComponent(arguments[e]);return"Minified React error #"+
a+"; visit "+b+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function v(a,b,e){this.props=a;this.context=b;this.refs=Z;this.updater=e||aa}function ba(){}function K(a,b,e){this.props=a;this.context=b;this.refs=Z;this.updater=e||aa}function ca(a,b,e){var l,f={},c=null,da=null;if(null!=b)for(l in void 0!==b.ref&&(da=b.ref),void 0!==b.key&&(c=""+b.key),b)ea.call(b,l)&&!fa.hasOwnProperty(l)&&(f[l]=b[l]);var k=arguments.length-2;if(1===
k)f.children=e;else if(1<k){for(var h=Array(k),d=0;d<k;d++)h[d]=arguments[d+2];f.children=h}if(a&&a.defaultProps)for(l in k=a.defaultProps,k)void 0===f[l]&&(f[l]=k[l]);return{$$typeof:w,type:a,key:c,ref:da,props:f,_owner:L.current}}function va(a,b){return{$$typeof:w,type:a.type,key:b,ref:a.ref,props:a.props,_owner:a._owner}}function M(a){return"object"===typeof a&&null!==a&&a.$$typeof===w}function wa(a){var b={"=":"=0",":":"=2"};return"$"+a.replace(/[=:]/g,function(a){return b[a]})}function N(a,b){return"object"===
typeof a&&null!==a&&null!=a.key?wa(""+a.key):b.toString(36)}function C(a,b,e,l,f){var c=typeof a;if("undefined"===c||"boolean"===c)a=null;var d=!1;if(null===a)d=!0;else switch(c){case "string":case "number":d=!0;break;case "object":switch(a.$$typeof){case w:case ha:d=!0}}if(d)return d=a,f=f(d),a=""===l?"."+N(d,0):l,Array.isArray(f)?(e="",null!=a&&(e=a.replace(ia,"$&/")+"/"),C(f,b,e,"",function(a){return a})):null!=f&&(M(f)&&(f=va(f,e+(!f.key||d&&d.key===f.key?"":(""+f.key).replace(ia,"$&/")+"/")+
a)),b.push(f)),1;d=0;l=""===l?".":l+":";if(Array.isArray(a))for(var k=0;k<a.length;k++){c=a[k];var h=l+N(c,k);d+=C(c,b,e,h,f)}else if(h=x(a),"function"===typeof h)for(a=h.call(a),k=0;!(c=a.next()).done;)c=c.value,h=l+N(c,k++),d+=C(c,b,e,h,f);else if("object"===c)throw b=""+a,Error(y(31,"[object Object]"===b?"object with keys {"+Object.keys(a).join(", ")+"}":b));return d}function D(a,b,e){if(null==a)return a;var l=[],c=0;C(a,l,"","",function(a){return b.call(e,a,c++)});return l}function xa(a){if(-1===
a._status){var b=a._result;b=b();a._status=0;a._result=b;b.then(function(b){0===a._status&&(b=b.default,a._status=1,a._result=b)},function(b){0===a._status&&(a._status=2,a._result=b)})}if(1===a._status)return a._result;throw a._result;}function n(){var a=ja.current;if(null===a)throw Error(y(321));return a}function O(a,b){var e=a.length;a.push(b);a:for(;;){var c=e-1>>>1,f=a[c];if(void 0!==f&&0<E(f,b))a[c]=b,a[e]=f,e=c;else break a}}function p(a){a=a[0];return void 0===a?null:a}function F(a){var b=
a[0];if(void 0!==b){var e=a.pop();if(e!==b){a[0]=e;a:for(var c=0,f=a.length;c<f;){var d=2*(c+1)-1,g=a[d],k=d+1,h=a[k];if(void 0!==g&&0>E(g,e))void 0!==h&&0>E(h,g)?(a[c]=h,a[k]=e,c=k):(a[c]=g,a[d]=e,c=d);else if(void 0!==h&&0>E(h,e))a[c]=h,a[k]=e,c=k;else break a}}return b}return null}function E(a,b){var e=a.sortIndex-b.sortIndex;return 0!==e?e:a.id-b.id}function P(a){for(var b=p(r);null!==b;){if(null===b.callback)F(r);else if(b.startTime<=a)F(r),b.sortIndex=b.expirationTime,O(q,b);else break;b=p(r)}}
function Q(a){z=!1;P(a);if(!u)if(null!==p(q))u=!0,A(R);else{var b=p(r);null!==b&&G(Q,b.startTime-a)}}function R(a,b){u=!1;z&&(z=!1,S());H=!0;var e=g;try{P(b);for(m=p(q);null!==m&&(!(m.expirationTime>b)||a&&!T());){var c=m.callback;if("function"===typeof c){m.callback=null;g=m.priorityLevel;var f=c(m.expirationTime<=b);b=t();"function"===typeof f?m.callback=f:m===p(q)&&F(q);P(b)}else F(q);m=p(q)}if(null!==m)var d=!0;else{var n=p(r);null!==n&&G(Q,n.startTime-b);d=!1}return d}finally{m=null,g=e,H=!1}}
var w=60103,ha=60106;c.Fragment=60107;c.StrictMode=60108;c.Profiler=60114;var ka=60109,la=60110,ma=60112;c.Suspense=60113;var na=60115,oa=60116;if("function"===typeof Symbol&&Symbol.for){var d=Symbol.for;w=d("react.element");ha=d("react.portal");c.Fragment=d("react.fragment");c.StrictMode=d("react.strict_mode");c.Profiler=d("react.profiler");ka=d("react.provider");la=d("react.context");ma=d("react.forward_ref");c.Suspense=d("react.suspense");na=d("react.memo");oa=d("react.lazy")}var Y="function"===
typeof Symbol&&Symbol.iterator,ya=Object.prototype.hasOwnProperty,U=Object.assign||function(a,b){if(null==a)throw new TypeError("Object.assign target cannot be null or undefined");for(var e=Object(a),c=1;c<arguments.length;c++){var d=arguments[c];if(null!=d){var g=void 0;d=Object(d);for(g in d)ya.call(d,g)&&(e[g]=d[g])}}return e},aa={isMounted:function(a){return!1},enqueueForceUpdate:function(a,b,c){},enqueueReplaceState:function(a,b,c,d){},enqueueSetState:function(a,b,c,d){}},Z={};v.prototype.isReactComponent=
{};v.prototype.setState=function(a,b){if("object"!==typeof a&&"function"!==typeof a&&null!=a)throw Error(y(85));this.updater.enqueueSetState(this,a,b,"setState")};v.prototype.forceUpdate=function(a){this.updater.enqueueForceUpdate(this,a,"forceUpdate")};ba.prototype=v.prototype;d=K.prototype=new ba;d.constructor=K;U(d,v.prototype);d.isPureReactComponent=!0;var L={current:null},ea=Object.prototype.hasOwnProperty,fa={key:!0,ref:!0,__self:!0,__source:!0},ia=/\/+/g,ja={current:null},V;if("object"===typeof performance&&
"function"===typeof performance.now){var za=performance;var t=function(){return za.now()}}else{var pa=Date,Aa=pa.now();t=function(){return pa.now()-Aa}}if("undefined"===typeof window||"function"!==typeof MessageChannel){var B=null,qa=null,ra=function(){if(null!==B)try{var a=t();B(!0,a);B=null}catch(b){throw setTimeout(ra,0),b;}};var A=function(a){null!==B?setTimeout(A,0,a):(B=a,setTimeout(ra,0))};var G=function(a,b){qa=setTimeout(a,b)};var S=function(){clearTimeout(qa)};var T=function(){return!1};
d=V=function(){}}else{var Ba=window.setTimeout,Ca=window.clearTimeout;"undefined"!==typeof console&&(d=window.cancelAnimationFrame,"function"!==typeof window.requestAnimationFrame&&console.error("This browser doesn't support requestAnimationFrame. Make sure that you load a polyfill in older browsers. https://reactjs.org/link/react-polyfills"),"function"!==typeof d&&console.error("This browser doesn't support cancelAnimationFrame. Make sure that you load a polyfill in older browsers. https://reactjs.org/link/react-polyfills"));
var I=!1,J=null,W=-1,sa=5,ta=0;T=function(){return t()>=ta};d=function(){};V=function(a){0>a||125<a?console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported"):sa=0<a?Math.floor(1E3/a):5};var ua=new MessageChannel,X=ua.port2;ua.port1.onmessage=function(){if(null!==J){var a=t();ta=a+sa;try{J(!0,a)?X.postMessage(null):(I=!1,J=null)}catch(b){throw X.postMessage(null),b;}}else I=!1};A=function(a){J=a;I||(I=!0,X.postMessage(null))};G=
function(a,b){W=Ba(function(){a(t())},b)};S=function(){Ca(W);W=-1}}var q=[],r=[],Da=1,m=null,g=3,H=!1,u=!1,z=!1,Ea=0;d={ReactCurrentDispatcher:ja,ReactCurrentOwner:L,IsSomeRendererActing:{current:!1},ReactCurrentBatchConfig:{transition:0},assign:U,Scheduler:{__proto__:null,unstable_ImmediatePriority:1,unstable_UserBlockingPriority:2,unstable_NormalPriority:3,unstable_IdlePriority:5,unstable_LowPriority:4,unstable_runWithPriority:function(a,b){switch(a){case 1:case 2:case 3:case 4:case 5:break;default:a=
3}var c=g;g=a;try{return b()}finally{g=c}},unstable_next:function(a){switch(g){case 1:case 2:case 3:var b=3;break;default:b=g}var c=g;g=b;try{return a()}finally{g=c}},unstable_scheduleCallback:function(a,b,c){var d=t();"object"===typeof c&&null!==c?(c=c.delay,c="number"===typeof c&&0<c?d+c:d):c=d;switch(a){case 1:var e=-1;break;case 2:e=250;break;case 5:e=1073741823;break;case 4:e=1E4;break;default:e=5E3}e=c+e;a={id:Da++,callback:b,priorityLevel:a,startTime:c,expirationTime:e,sortIndex:-1};c>d?(a.sortIndex=
c,O(r,a),null===p(q)&&a===p(r)&&(z?S():z=!0,G(Q,c-d))):(a.sortIndex=e,O(q,a),u||H||(u=!0,A(R)));return a},unstable_cancelCallback:function(a){a.callback=null},unstable_wrapCallback:function(a){var b=g;return function(){var c=g;g=b;try{return a.apply(this,arguments)}finally{g=c}}},unstable_getCurrentPriorityLevel:function(){return g},get unstable_shouldYield(){return T},unstable_requestPaint:d,unstable_continueExecution:function(){u||H||(u=!0,A(R))},unstable_pauseExecution:function(){},unstable_getFirstCallbackNode:function(){return p(q)},
get unstable_now(){return t},get unstable_forceFrameRate(){return V},unstable_Profiling:null},SchedulerTracing:{__proto__:null,__interactionsRef:null,__subscriberRef:null,unstable_clear:function(a){return a()},unstable_getCurrent:function(){return null},unstable_getThreadID:function(){return++Ea},unstable_trace:function(a,b,c){return c()},unstable_wrap:function(a){return a},unstable_subscribe:function(a){},unstable_unsubscribe:function(a){}}};c.Children={map:D,forEach:function(a,b,c){D(a,function(){b.apply(this,
arguments)},c)},count:function(a){var b=0;D(a,function(){b++});return b},toArray:function(a){return D(a,function(a){return a})||[]},only:function(a){if(!M(a))throw Error(y(143));return a}};c.Component=v;c.PureComponent=K;c.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED=d;c.cloneElement=function(a,b,c){if(null===a||void 0===a)throw Error(y(267,a));var d=U({},a.props),e=a.key,g=a.ref,n=a._owner;if(null!=b){void 0!==b.ref&&(g=b.ref,n=L.current);void 0!==b.key&&(e=""+b.key);if(a.type&&a.type.defaultProps)var k=
a.type.defaultProps;for(h in b)ea.call(b,h)&&!fa.hasOwnProperty(h)&&(d[h]=void 0===b[h]&&void 0!==k?k[h]:b[h])}var h=arguments.length-2;if(1===h)d.children=c;else if(1<h){k=Array(h);for(var m=0;m<h;m++)k[m]=arguments[m+2];d.children=k}return{$$typeof:w,type:a.type,key:e,ref:g,props:d,_owner:n}};c.createContext=function(a,b){void 0===b&&(b=null);a={$$typeof:la,_calculateChangedBits:b,_currentValue:a,_currentValue2:a,_threadCount:0,Provider:null,Consumer:null};a.Provider={$$typeof:ka,_context:a};return a.Consumer=
a};c.createElement=ca;c.createFactory=function(a){var b=ca.bind(null,a);b.type=a;return b};c.createRef=function(){return{current:null}};c.forwardRef=function(a){return{$$typeof:ma,render:a}};c.isValidElement=M;c.lazy=function(a){return{$$typeof:oa,_payload:{_status:-1,_result:a},_init:xa}};c.memo=function(a,b){return{$$typeof:na,type:a,compare:void 0===b?null:b}};c.useCallback=function(a,b){return n().useCallback(a,b)};c.useContext=function(a,b){return n().useContext(a,b)};c.useDebugValue=function(a,
b){};c.useEffect=function(a,b){return n().useEffect(a,b)};c.useImperativeHandle=function(a,b,c){return n().useImperativeHandle(a,b,c)};c.useLayoutEffect=function(a,b){return n().useLayoutEffect(a,b)};c.useMemo=function(a,b){return n().useMemo(a,b)};c.useReducer=function(a,b,c){return n().useReducer(a,b,c)};c.useRef=function(a){return n().useRef(a)};c.useState=function(a){return n().useState(a)};c.version="17.0.2"});
})();

/** @license React v17.0.2
 * react-dom.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
(function(){/*
 Modernizr 3.0.0pre (Custom Build) | MIT
*/
'use strict';(function(M,ha){"object"===typeof exports&&"undefined"!==typeof module?ha(exports,require("react")):"function"===typeof define&&define.amd?define(["exports","react"],ha):(M=M||self,ha(M.ReactDOM={},M.React))})(this,function(M,ha){function m(a){for(var b="https://reactjs.org/docs/error-decoder.html?invariant="+a,c=1;c<arguments.length;c++)b+="&args[]="+encodeURIComponent(arguments[c]);return"Minified React error #"+a+"; visit "+b+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}
function Ta(a,b){gb(a,b);gb(a+"Capture",b)}function gb(a,b){Ib[a]=b;for(a=0;a<b.length;a++)zf.add(b[a])}function li(a){if(Af.call(Bf,a))return!0;if(Af.call(Cf,a))return!1;if(mi.test(a))return Bf[a]=!0;Cf[a]=!0;return!1}function ni(a,b,c,d){if(null!==c&&0===c.type)return!1;switch(typeof b){case "function":case "symbol":return!0;case "boolean":if(d)return!1;if(null!==c)return!c.acceptsBooleans;a=a.toLowerCase().slice(0,5);return"data-"!==a&&"aria-"!==a;default:return!1}}function oi(a,b,c,d){if(null===
b||"undefined"===typeof b||ni(a,b,c,d))return!0;if(d)return!1;if(null!==c)switch(c.type){case 3:return!b;case 4:return!1===b;case 5:return isNaN(b);case 6:return isNaN(b)||1>b}return!1}function Q(a,b,c,d,e,f,g){this.acceptsBooleans=2===b||3===b||4===b;this.attributeName=d;this.attributeNamespace=e;this.mustUseProperty=c;this.propertyName=a;this.type=b;this.sanitizeURL=f;this.removeEmptyString=g}function Ed(a,b,c,d){var e=I.hasOwnProperty(b)?I[b]:null;var f=null!==e?0===e.type:d?!1:!(2<b.length)||
"o"!==b[0]&&"O"!==b[0]||"n"!==b[1]&&"N"!==b[1]?!1:!0;f||(oi(b,c,e,d)&&(c=null),d||null===e?li(b)&&(null===c?a.removeAttribute(b):a.setAttribute(b,""+c)):e.mustUseProperty?a[e.propertyName]=null===c?3===e.type?!1:"":c:(b=e.attributeName,d=e.attributeNamespace,null===c?a.removeAttribute(b):(e=e.type,c=3===e||4===e&&!0===c?"":""+c,d?a.setAttributeNS(d,b,c):a.setAttribute(b,c))))}function Jb(a){if(null===a||"object"!==typeof a)return null;a=Df&&a[Df]||a["@@iterator"];return"function"===typeof a?a:null}
function Kb(a,b,c){if(void 0===Fd)try{throw Error();}catch(d){Fd=(b=d.stack.trim().match(/\n( *(at )?)/))&&b[1]||""}return"\n"+Fd+a}function Bc(a,b){if(!a||Gd)return"";Gd=!0;var c=Error.prepareStackTrace;Error.prepareStackTrace=void 0;try{if(b)if(b=function(){throw Error();},Object.defineProperty(b.prototype,"props",{set:function(){throw Error();}}),"object"===typeof Reflect&&Reflect.construct){try{Reflect.construct(b,[])}catch(k){var d=k}Reflect.construct(a,[],b)}else{try{b.call()}catch(k){d=k}a.call(b.prototype)}else{try{throw Error();
}catch(k){d=k}a()}}catch(k){if(k&&d&&"string"===typeof k.stack){for(var e=k.stack.split("\n"),f=d.stack.split("\n"),g=e.length-1,h=f.length-1;1<=g&&0<=h&&e[g]!==f[h];)h--;for(;1<=g&&0<=h;g--,h--)if(e[g]!==f[h]){if(1!==g||1!==h){do if(g--,h--,0>h||e[g]!==f[h])return"\n"+e[g].replace(" at new "," at ");while(1<=g&&0<=h)}break}}}finally{Gd=!1,Error.prepareStackTrace=c}return(a=a?a.displayName||a.name:"")?Kb(a):""}function pi(a){switch(a.tag){case 5:return Kb(a.type);case 16:return Kb("Lazy");case 13:return Kb("Suspense");
case 19:return Kb("SuspenseList");case 0:case 2:case 15:return a=Bc(a.type,!1),a;case 11:return a=Bc(a.type.render,!1),a;case 22:return a=Bc(a.type._render,!1),a;case 1:return a=Bc(a.type,!0),a;default:return""}}function hb(a){if(null==a)return null;if("function"===typeof a)return a.displayName||a.name||null;if("string"===typeof a)return a;switch(a){case wa:return"Fragment";case Ua:return"Portal";case Lb:return"Profiler";case Hd:return"StrictMode";case Mb:return"Suspense";case Cc:return"SuspenseList"}if("object"===
typeof a)switch(a.$$typeof){case Id:return(a.displayName||"Context")+".Consumer";case Jd:return(a._context.displayName||"Context")+".Provider";case Dc:var b=a.render;b=b.displayName||b.name||"";return a.displayName||(""!==b?"ForwardRef("+b+")":"ForwardRef");case Ec:return hb(a.type);case Kd:return hb(a._render);case Ld:b=a._payload;a=a._init;try{return hb(a(b))}catch(c){}}return null}function xa(a){switch(typeof a){case "boolean":case "number":case "object":case "string":case "undefined":return a;
default:return""}}function Ef(a){var b=a.type;return(a=a.nodeName)&&"input"===a.toLowerCase()&&("checkbox"===b||"radio"===b)}function qi(a){var b=Ef(a)?"checked":"value",c=Object.getOwnPropertyDescriptor(a.constructor.prototype,b),d=""+a[b];if(!a.hasOwnProperty(b)&&"undefined"!==typeof c&&"function"===typeof c.get&&"function"===typeof c.set){var e=c.get,f=c.set;Object.defineProperty(a,b,{configurable:!0,get:function(){return e.call(this)},set:function(a){d=""+a;f.call(this,a)}});Object.defineProperty(a,
b,{enumerable:c.enumerable});return{getValue:function(){return d},setValue:function(a){d=""+a},stopTracking:function(){a._valueTracker=null;delete a[b]}}}}function Fc(a){a._valueTracker||(a._valueTracker=qi(a))}function Ff(a){if(!a)return!1;var b=a._valueTracker;if(!b)return!0;var c=b.getValue();var d="";a&&(d=Ef(a)?a.checked?"true":"false":a.value);a=d;return a!==c?(b.setValue(a),!0):!1}function Gc(a){a=a||("undefined"!==typeof document?document:void 0);if("undefined"===typeof a)return null;try{return a.activeElement||
a.body}catch(b){return a.body}}function Md(a,b){var c=b.checked;return B({},b,{defaultChecked:void 0,defaultValue:void 0,value:void 0,checked:null!=c?c:a._wrapperState.initialChecked})}function Gf(a,b){var c=null==b.defaultValue?"":b.defaultValue,d=null!=b.checked?b.checked:b.defaultChecked;c=xa(null!=b.value?b.value:c);a._wrapperState={initialChecked:d,initialValue:c,controlled:"checkbox"===b.type||"radio"===b.type?null!=b.checked:null!=b.value}}function Hf(a,b){b=b.checked;null!=b&&Ed(a,"checked",
b,!1)}function Nd(a,b){Hf(a,b);var c=xa(b.value),d=b.type;if(null!=c)if("number"===d){if(0===c&&""===a.value||a.value!=c)a.value=""+c}else a.value!==""+c&&(a.value=""+c);else if("submit"===d||"reset"===d){a.removeAttribute("value");return}b.hasOwnProperty("value")?Od(a,b.type,c):b.hasOwnProperty("defaultValue")&&Od(a,b.type,xa(b.defaultValue));null==b.checked&&null!=b.defaultChecked&&(a.defaultChecked=!!b.defaultChecked)}function If(a,b,c){if(b.hasOwnProperty("value")||b.hasOwnProperty("defaultValue")){var d=
b.type;if(!("submit"!==d&&"reset"!==d||void 0!==b.value&&null!==b.value))return;b=""+a._wrapperState.initialValue;c||b===a.value||(a.value=b);a.defaultValue=b}c=a.name;""!==c&&(a.name="");a.defaultChecked=!!a._wrapperState.initialChecked;""!==c&&(a.name=c)}function Od(a,b,c){if("number"!==b||Gc(a.ownerDocument)!==a)null==c?a.defaultValue=""+a._wrapperState.initialValue:a.defaultValue!==""+c&&(a.defaultValue=""+c)}function ri(a){var b="";ha.Children.forEach(a,function(a){null!=a&&(b+=a)});return b}
function Pd(a,b){a=B({children:void 0},b);if(b=ri(b.children))a.children=b;return a}function ib(a,b,c,d){a=a.options;if(b){b={};for(var e=0;e<c.length;e++)b["$"+c[e]]=!0;for(c=0;c<a.length;c++)e=b.hasOwnProperty("$"+a[c].value),a[c].selected!==e&&(a[c].selected=e),e&&d&&(a[c].defaultSelected=!0)}else{c=""+xa(c);b=null;for(e=0;e<a.length;e++){if(a[e].value===c){a[e].selected=!0;d&&(a[e].defaultSelected=!0);return}null!==b||a[e].disabled||(b=a[e])}null!==b&&(b.selected=!0)}}function Qd(a,b){if(null!=
b.dangerouslySetInnerHTML)throw Error(m(91));return B({},b,{value:void 0,defaultValue:void 0,children:""+a._wrapperState.initialValue})}function Jf(a,b){var c=b.value;if(null==c){c=b.children;b=b.defaultValue;if(null!=c){if(null!=b)throw Error(m(92));if(Array.isArray(c)){if(!(1>=c.length))throw Error(m(93));c=c[0]}b=c}null==b&&(b="");c=b}a._wrapperState={initialValue:xa(c)}}function Kf(a,b){var c=xa(b.value),d=xa(b.defaultValue);null!=c&&(c=""+c,c!==a.value&&(a.value=c),null==b.defaultValue&&a.defaultValue!==
c&&(a.defaultValue=c));null!=d&&(a.defaultValue=""+d)}function Lf(a,b){b=a.textContent;b===a._wrapperState.initialValue&&""!==b&&null!==b&&(a.value=b)}function Mf(a){switch(a){case "svg":return"http://www.w3.org/2000/svg";case "math":return"http://www.w3.org/1998/Math/MathML";default:return"http://www.w3.org/1999/xhtml"}}function Rd(a,b){return null==a||"http://www.w3.org/1999/xhtml"===a?Mf(b):"http://www.w3.org/2000/svg"===a&&"foreignObject"===b?"http://www.w3.org/1999/xhtml":a}function Nf(a,b,c){return null==
b||"boolean"===typeof b||""===b?"":c||"number"!==typeof b||0===b||Nb.hasOwnProperty(a)&&Nb[a]?(""+b).trim():b+"px"}function Of(a,b){a=a.style;for(var c in b)if(b.hasOwnProperty(c)){var d=0===c.indexOf("--"),e=Nf(c,b[c],d);"float"===c&&(c="cssFloat");d?a.setProperty(c,e):a[c]=e}}function Sd(a,b){if(b){if(si[a]&&(null!=b.children||null!=b.dangerouslySetInnerHTML))throw Error(m(137,a));if(null!=b.dangerouslySetInnerHTML){if(null!=b.children)throw Error(m(60));if(!("object"===typeof b.dangerouslySetInnerHTML&&
"__html"in b.dangerouslySetInnerHTML))throw Error(m(61));}if(null!=b.style&&"object"!==typeof b.style)throw Error(m(62));}}function Td(a,b){if(-1===a.indexOf("-"))return"string"===typeof b.is;switch(a){case "annotation-xml":case "color-profile":case "font-face":case "font-face-src":case "font-face-uri":case "font-face-format":case "font-face-name":case "missing-glyph":return!1;default:return!0}}function Ud(a){a=a.target||a.srcElement||window;a.correspondingUseElement&&(a=a.correspondingUseElement);
return 3===a.nodeType?a.parentNode:a}function Pf(a){if(a=Ob(a)){if("function"!==typeof Vd)throw Error(m(280));var b=a.stateNode;b&&(b=Hc(b),Vd(a.stateNode,a.type,b))}}function Qf(a){jb?kb?kb.push(a):kb=[a]:jb=a}function Rf(){if(jb){var a=jb,b=kb;kb=jb=null;Pf(a);if(b)for(a=0;a<b.length;a++)Pf(b[a])}}function Wd(){if(null!==jb||null!==kb)Xd(),Rf()}function ti(a,b,c){if(Yd)return a(b,c);Yd=!0;try{return Sf(a,b,c)}finally{Yd=!1,Wd()}}function Pb(a,b){var c=a.stateNode;if(null===c)return null;var d=Hc(c);
if(null===d)return null;c=d[b];a:switch(b){case "onClick":case "onClickCapture":case "onDoubleClick":case "onDoubleClickCapture":case "onMouseDown":case "onMouseDownCapture":case "onMouseMove":case "onMouseMoveCapture":case "onMouseUp":case "onMouseUpCapture":case "onMouseEnter":(d=!d.disabled)||(a=a.type,d=!("button"===a||"input"===a||"select"===a||"textarea"===a));a=!d;break a;default:a=!1}if(a)return null;if(c&&"function"!==typeof c)throw Error(m(231,b,typeof c));return c}function ui(a,b,c,d,e,
f,g,h,k){Qb=!1;Ic=null;vi.apply(wi,arguments)}function xi(a,b,c,d,e,f,g,h,k){ui.apply(this,arguments);if(Qb){if(Qb){var v=Ic;Qb=!1;Ic=null}else throw Error(m(198));Jc||(Jc=!0,Zd=v)}}function Va(a){var b=a,c=a;if(a.alternate)for(;b.return;)b=b.return;else{a=b;do b=a,0!==(b.flags&1026)&&(c=b.return),a=b.return;while(a)}return 3===b.tag?c:null}function Tf(a){if(13===a.tag){var b=a.memoizedState;null===b&&(a=a.alternate,null!==a&&(b=a.memoizedState));if(null!==b)return b.dehydrated}return null}function Uf(a){if(Va(a)!==
a)throw Error(m(188));}function yi(a){var b=a.alternate;if(!b){b=Va(a);if(null===b)throw Error(m(188));return b!==a?null:a}for(var c=a,d=b;;){var e=c.return;if(null===e)break;var f=e.alternate;if(null===f){d=e.return;if(null!==d){c=d;continue}break}if(e.child===f.child){for(f=e.child;f;){if(f===c)return Uf(e),a;if(f===d)return Uf(e),b;f=f.sibling}throw Error(m(188));}if(c.return!==d.return)c=e,d=f;else{for(var g=!1,h=e.child;h;){if(h===c){g=!0;c=e;d=f;break}if(h===d){g=!0;d=e;c=f;break}h=h.sibling}if(!g){for(h=
f.child;h;){if(h===c){g=!0;c=f;d=e;break}if(h===d){g=!0;d=f;c=e;break}h=h.sibling}if(!g)throw Error(m(189));}}if(c.alternate!==d)throw Error(m(190));}if(3!==c.tag)throw Error(m(188));return c.stateNode.current===c?a:b}function Vf(a){a=yi(a);if(!a)return null;for(var b=a;;){if(5===b.tag||6===b.tag)return b;if(b.child)b.child.return=b,b=b.child;else{if(b===a)break;for(;!b.sibling;){if(!b.return||b.return===a)return null;b=b.return}b.sibling.return=b.return;b=b.sibling}}return null}function Wf(a,b){for(var c=
a.alternate;null!==b;){if(b===a||b===c)return!0;b=b.return}return!1}function $d(a,b,c,d,e){return{blockedOn:a,domEventName:b,eventSystemFlags:c|16,nativeEvent:e,targetContainers:[d]}}function Xf(a,b){switch(a){case "focusin":case "focusout":ya=null;break;case "dragenter":case "dragleave":za=null;break;case "mouseover":case "mouseout":Aa=null;break;case "pointerover":case "pointerout":Rb.delete(b.pointerId);break;case "gotpointercapture":case "lostpointercapture":Sb.delete(b.pointerId)}}function Tb(a,
b,c,d,e,f){if(null===a||a.nativeEvent!==f)return a=$d(b,c,d,e,f),null!==b&&(b=Ob(b),null!==b&&Yf(b)),a;a.eventSystemFlags|=d;b=a.targetContainers;null!==e&&-1===b.indexOf(e)&&b.push(e);return a}function zi(a,b,c,d,e){switch(b){case "focusin":return ya=Tb(ya,a,b,c,d,e),!0;case "dragenter":return za=Tb(za,a,b,c,d,e),!0;case "mouseover":return Aa=Tb(Aa,a,b,c,d,e),!0;case "pointerover":var f=e.pointerId;Rb.set(f,Tb(Rb.get(f)||null,a,b,c,d,e));return!0;case "gotpointercapture":return f=e.pointerId,Sb.set(f,
Tb(Sb.get(f)||null,a,b,c,d,e)),!0}return!1}function Ai(a){var b=Wa(a.target);if(null!==b){var c=Va(b);if(null!==c)if(b=c.tag,13===b){if(b=Tf(c),null!==b){a.blockedOn=b;Bi(a.lanePriority,function(){ae(a.priority,function(){Ci(c)})});return}}else if(3===b&&c.stateNode.hydrate){a.blockedOn=3===c.tag?c.stateNode.containerInfo:null;return}}a.blockedOn=null}function Kc(a){if(null!==a.blockedOn)return!1;for(var b=a.targetContainers;0<b.length;){var c=be(a.domEventName,a.eventSystemFlags,b[0],a.nativeEvent);
if(null!==c)return b=Ob(c),null!==b&&Yf(b),a.blockedOn=c,!1;b.shift()}return!0}function Zf(a,b,c){Kc(a)&&c.delete(b)}function Di(){for(ce=!1;0<ia.length;){var a=ia[0];if(null!==a.blockedOn){a=Ob(a.blockedOn);null!==a&&Ei(a);break}for(var b=a.targetContainers;0<b.length;){var c=be(a.domEventName,a.eventSystemFlags,b[0],a.nativeEvent);if(null!==c){a.blockedOn=c;break}b.shift()}null===a.blockedOn&&ia.shift()}null!==ya&&Kc(ya)&&(ya=null);null!==za&&Kc(za)&&(za=null);null!==Aa&&Kc(Aa)&&(Aa=null);Rb.forEach(Zf);
Sb.forEach(Zf)}function Ub(a,b){a.blockedOn===b&&(a.blockedOn=null,ce||(ce=!0,$f(ag,Di)))}function bg(a){if(0<ia.length){Ub(ia[0],a);for(var b=1;b<ia.length;b++){var c=ia[b];c.blockedOn===a&&(c.blockedOn=null)}}null!==ya&&Ub(ya,a);null!==za&&Ub(za,a);null!==Aa&&Ub(Aa,a);b=function(b){return Ub(b,a)};Rb.forEach(b);Sb.forEach(b);for(b=0;b<Vb.length;b++)c=Vb[b],c.blockedOn===a&&(c.blockedOn=null);for(;0<Vb.length&&(b=Vb[0],null===b.blockedOn);)Ai(b),null===b.blockedOn&&Vb.shift()}function Lc(a,b){var c=
{};c[a.toLowerCase()]=b.toLowerCase();c["Webkit"+a]="webkit"+b;c["Moz"+a]="moz"+b;return c}function Mc(a){if(de[a])return de[a];if(!lb[a])return a;var b=lb[a],c;for(c in b)if(b.hasOwnProperty(c)&&c in cg)return de[a]=b[c];return a}function ee(a,b){for(var c=0;c<a.length;c+=2){var d=a[c],e=a[c+1];e="on"+(e[0].toUpperCase()+e.slice(1));fe.set(d,b);dg.set(d,e);Ta(e,[d])}}function mb(a){if(0!==(1&a))return w=15,1;if(0!==(2&a))return w=14,2;if(0!==(4&a))return w=13,4;var b=24&a;if(0!==b)return w=12,b;
if(0!==(a&32))return w=11,32;b=192&a;if(0!==b)return w=10,b;if(0!==(a&256))return w=9,256;b=3584&a;if(0!==b)return w=8,b;if(0!==(a&4096))return w=7,4096;b=4186112&a;if(0!==b)return w=6,b;b=62914560&a;if(0!==b)return w=5,b;if(a&67108864)return w=4,67108864;if(0!==(a&134217728))return w=3,134217728;b=805306368&a;if(0!==b)return w=2,b;if(0!==(1073741824&a))return w=1,1073741824;w=8;return a}function Fi(a){switch(a){case 99:return 15;case 98:return 10;case 97:case 96:return 8;case 95:return 2;default:return 0}}
function Gi(a){switch(a){case 15:case 14:return 99;case 13:case 12:case 11:case 10:return 98;case 9:case 8:case 7:case 6:case 4:case 5:return 97;case 3:case 2:case 1:return 95;case 0:return 90;default:throw Error(m(358,a));}}function Wb(a,b){var c=a.pendingLanes;if(0===c)return w=0;var d=0,e=0,f=a.expiredLanes,g=a.suspendedLanes,h=a.pingedLanes;if(0!==f)d=f,e=w=15;else if(f=c&134217727,0!==f){var k=f&~g;0!==k?(d=mb(k),e=w):(h&=f,0!==h&&(d=mb(h),e=w))}else f=c&~g,0!==f?(d=mb(f),e=w):0!==h&&(d=mb(h),
e=w);if(0===d)return 0;d=31-Ba(d);d=c&((0>d?0:1<<d)<<1)-1;if(0!==b&&b!==d&&0===(b&g)){mb(b);if(e<=w)return b;w=e}b=a.entangledLanes;if(0!==b)for(a=a.entanglements,b&=d;0<b;)c=31-Ba(b),e=1<<c,d|=a[c],b&=~e;return d}function eg(a){a=a.pendingLanes&-1073741825;return 0!==a?a:a&1073741824?1073741824:0}function Nc(a,b){switch(a){case 15:return 1;case 14:return 2;case 12:return a=nb(24&~b),0===a?Nc(10,b):a;case 10:return a=nb(192&~b),0===a?Nc(8,b):a;case 8:return a=nb(3584&~b),0===a&&(a=nb(4186112&~b),
0===a&&(a=512)),a;case 2:return b=nb(805306368&~b),0===b&&(b=268435456),b}throw Error(m(358,a));}function nb(a){return a&-a}function ge(a){for(var b=[],c=0;31>c;c++)b.push(a);return b}function Oc(a,b,c){a.pendingLanes|=b;var d=b-1;a.suspendedLanes&=d;a.pingedLanes&=d;a=a.eventTimes;b=31-Ba(b);a[b]=c}function Hi(a){return 0===a?32:31-(Ii(a)/Ji|0)|0}function Ki(a,b,c,d){Xa||Xd();var e=he,f=Xa;Xa=!0;try{fg(e,a,b,c,d)}finally{(Xa=f)||Wd()}}function Li(a,b,c,d){Mi(Ni,he.bind(null,a,b,c,d))}function he(a,
b,c,d){if(Pc){var e;if((e=0===(b&4))&&0<ia.length&&-1<gg.indexOf(a))a=$d(null,a,b,c,d),ia.push(a);else{var f=be(a,b,c,d);if(null===f)e&&Xf(a,d);else{if(e){if(-1<gg.indexOf(a)){a=$d(f,a,b,c,d);ia.push(a);return}if(zi(f,a,b,c,d))return;Xf(a,d)}hg(a,b,d,null,c)}}}}function be(a,b,c,d){var e=Ud(d);e=Wa(e);if(null!==e){var f=Va(e);if(null===f)e=null;else{var g=f.tag;if(13===g){e=Tf(f);if(null!==e)return e;e=null}else if(3===g){if(f.stateNode.hydrate)return 3===f.tag?f.stateNode.containerInfo:null;e=null}else f!==
e&&(e=null)}}hg(a,b,d,e,c);return null}function ig(){if(Qc)return Qc;var a,b=ie,c=b.length,d,e="value"in Ca?Ca.value:Ca.textContent,f=e.length;for(a=0;a<c&&b[a]===e[a];a++);var g=c-a;for(d=1;d<=g&&b[c-d]===e[f-d];d++);return Qc=e.slice(a,1<d?1-d:void 0)}function Rc(a){var b=a.keyCode;"charCode"in a?(a=a.charCode,0===a&&13===b&&(a=13)):a=b;10===a&&(a=13);return 32<=a||13===a?a:0}function Sc(){return!0}function jg(){return!1}function V(a){function b(b,d,e,f,g){this._reactName=b;this._targetInst=e;this.type=
d;this.nativeEvent=f;this.target=g;this.currentTarget=null;for(var c in a)a.hasOwnProperty(c)&&(b=a[c],this[c]=b?b(f):f[c]);this.isDefaultPrevented=(null!=f.defaultPrevented?f.defaultPrevented:!1===f.returnValue)?Sc:jg;this.isPropagationStopped=jg;return this}B(b.prototype,{preventDefault:function(){this.defaultPrevented=!0;var a=this.nativeEvent;a&&(a.preventDefault?a.preventDefault():"unknown"!==typeof a.returnValue&&(a.returnValue=!1),this.isDefaultPrevented=Sc)},stopPropagation:function(){var a=
this.nativeEvent;a&&(a.stopPropagation?a.stopPropagation():"unknown"!==typeof a.cancelBubble&&(a.cancelBubble=!0),this.isPropagationStopped=Sc)},persist:function(){},isPersistent:Sc});return b}function Oi(a){var b=this.nativeEvent;return b.getModifierState?b.getModifierState(a):(a=Pi[a])?!!b[a]:!1}function je(a){return Oi}function kg(a,b){switch(a){case "keyup":return-1!==Qi.indexOf(b.keyCode);case "keydown":return 229!==b.keyCode;case "keypress":case "mousedown":case "focusout":return!0;default:return!1}}
function lg(a){a=a.detail;return"object"===typeof a&&"data"in a?a.data:null}function Ri(a,b){switch(a){case "compositionend":return lg(b);case "keypress":if(32!==b.which)return null;mg=!0;return ng;case "textInput":return a=b.data,a===ng&&mg?null:a;default:return null}}function Si(a,b){if(ob)return"compositionend"===a||!ke&&kg(a,b)?(a=ig(),Qc=ie=Ca=null,ob=!1,a):null;switch(a){case "paste":return null;case "keypress":if(!(b.ctrlKey||b.altKey||b.metaKey)||b.ctrlKey&&b.altKey){if(b.char&&1<b.char.length)return b.char;
if(b.which)return String.fromCharCode(b.which)}return null;case "compositionend":return og&&"ko"!==b.locale?null:b.data;default:return null}}function pg(a){var b=a&&a.nodeName&&a.nodeName.toLowerCase();return"input"===b?!!Ti[a.type]:"textarea"===b?!0:!1}function Ui(a){if(!oa)return!1;a="on"+a;var b=a in document;b||(b=document.createElement("div"),b.setAttribute(a,"return;"),b="function"===typeof b[a]);return b}function qg(a,b,c,d){Qf(d);b=Tc(b,"onChange");0<b.length&&(c=new le("onChange","change",
null,c,d),a.push({event:c,listeners:b}))}function Vi(a){rg(a,0)}function Uc(a){var b=pb(a);if(Ff(b))return a}function Wi(a,b){if("change"===a)return b}function sg(){Xb&&(Xb.detachEvent("onpropertychange",tg),Yb=Xb=null)}function tg(a){if("value"===a.propertyName&&Uc(Yb)){var b=[];qg(b,Yb,a,Ud(a));a=Vi;if(Xa)a(b);else{Xa=!0;try{me(a,b)}finally{Xa=!1,Wd()}}}}function Xi(a,b,c){"focusin"===a?(sg(),Xb=b,Yb=c,Xb.attachEvent("onpropertychange",tg)):"focusout"===a&&sg()}function Yi(a,b){if("selectionchange"===
a||"keyup"===a||"keydown"===a)return Uc(Yb)}function Zi(a,b){if("click"===a)return Uc(b)}function $i(a,b){if("input"===a||"change"===a)return Uc(b)}function aj(a,b){return a===b&&(0!==a||1/a===1/b)||a!==a&&b!==b}function Zb(a,b){if(X(a,b))return!0;if("object"!==typeof a||null===a||"object"!==typeof b||null===b)return!1;var c=Object.keys(a),d=Object.keys(b);if(c.length!==d.length)return!1;for(d=0;d<c.length;d++)if(!bj.call(b,c[d])||!X(a[c[d]],b[c[d]]))return!1;return!0}function ug(a){for(;a&&a.firstChild;)a=
a.firstChild;return a}function vg(a,b){var c=ug(a);a=0;for(var d;c;){if(3===c.nodeType){d=a+c.textContent.length;if(a<=b&&d>=b)return{node:c,offset:b-a};a=d}a:{for(;c;){if(c.nextSibling){c=c.nextSibling;break a}c=c.parentNode}c=void 0}c=ug(c)}}function wg(a,b){return a&&b?a===b?!0:a&&3===a.nodeType?!1:b&&3===b.nodeType?wg(a,b.parentNode):"contains"in a?a.contains(b):a.compareDocumentPosition?!!(a.compareDocumentPosition(b)&16):!1:!1}function xg(){for(var a=window,b=Gc();b instanceof a.HTMLIFrameElement;){try{var c=
"string"===typeof b.contentWindow.location.href}catch(d){c=!1}if(c)a=b.contentWindow;else break;b=Gc(a.document)}return b}function ne(a){var b=a&&a.nodeName&&a.nodeName.toLowerCase();return b&&("input"===b&&("text"===a.type||"search"===a.type||"tel"===a.type||"url"===a.type||"password"===a.type)||"textarea"===b||"true"===a.contentEditable)}function yg(a,b,c){var d=c.window===c?c.document:9===c.nodeType?c:c.ownerDocument;oe||null==qb||qb!==Gc(d)||(d=qb,"selectionStart"in d&&ne(d)?d={start:d.selectionStart,
end:d.selectionEnd}:(d=(d.ownerDocument&&d.ownerDocument.defaultView||window).getSelection(),d={anchorNode:d.anchorNode,anchorOffset:d.anchorOffset,focusNode:d.focusNode,focusOffset:d.focusOffset}),$b&&Zb($b,d)||($b=d,d=Tc(pe,"onSelect"),0<d.length&&(b=new le("onSelect","select",null,b,c),a.push({event:b,listeners:d}),b.target=qb)))}function zg(a,b,c){var d=a.type||"unknown-event";a.currentTarget=c;xi(d,b,void 0,a);a.currentTarget=null}function rg(a,b){b=0!==(b&4);for(var c=0;c<a.length;c++){var d=
a[c],e=d.event;d=d.listeners;a:{var f=void 0;if(b)for(var g=d.length-1;0<=g;g--){var h=d[g],k=h.instance,v=h.currentTarget;h=h.listener;if(k!==f&&e.isPropagationStopped())break a;zg(e,h,v);f=k}else for(g=0;g<d.length;g++){h=d[g];k=h.instance;v=h.currentTarget;h=h.listener;if(k!==f&&e.isPropagationStopped())break a;zg(e,h,v);f=k}}}if(Jc)throw a=Zd,Jc=!1,Zd=null,a;}function z(a,b){var c=Ag(b),d=a+"__bubble";c.has(d)||(Bg(b,a,2,!1),c.add(d))}function Cg(a){a[Dg]||(a[Dg]=!0,zf.forEach(function(b){Eg.has(b)||
Fg(b,!1,a,null);Fg(b,!0,a,null)}))}function Fg(a,b,c,d){var e=4<arguments.length&&void 0!==arguments[4]?arguments[4]:0,f=c;"selectionchange"===a&&9!==c.nodeType&&(f=c.ownerDocument);if(null!==d&&!b&&Eg.has(a)){if("scroll"!==a)return;e|=2;f=d}var g=Ag(f),h=a+"__"+(b?"capture":"bubble");g.has(h)||(b&&(e|=4),Bg(f,a,e,b),g.add(h))}function Bg(a,b,c,d,e){e=fe.get(b);switch(void 0===e?2:e){case 0:e=Ki;break;case 1:e=Li;break;default:e=he}c=e.bind(null,b,c,a);e=void 0;!qe||"touchstart"!==b&&"touchmove"!==
b&&"wheel"!==b||(e=!0);d?void 0!==e?a.addEventListener(b,c,{capture:!0,passive:e}):a.addEventListener(b,c,!0):void 0!==e?a.addEventListener(b,c,{passive:e}):a.addEventListener(b,c,!1)}function hg(a,b,c,d,e){var f=d;if(0===(b&1)&&0===(b&2)&&null!==d)a:for(;;){if(null===d)return;var g=d.tag;if(3===g||4===g){var h=d.stateNode.containerInfo;if(h===e||8===h.nodeType&&h.parentNode===e)break;if(4===g)for(g=d.return;null!==g;){var k=g.tag;if(3===k||4===k)if(k=g.stateNode.containerInfo,k===e||8===k.nodeType&&
k.parentNode===e)return;g=g.return}for(;null!==h;){g=Wa(h);if(null===g)return;k=g.tag;if(5===k||6===k){d=f=g;continue a}h=h.parentNode}}d=d.return}ti(function(){var d=f,e=Ud(c),g=[];a:{var h=dg.get(a);if(void 0!==h){var k=le,m=a;switch(a){case "keypress":if(0===Rc(c))break a;case "keydown":case "keyup":k=cj;break;case "focusin":m="focus";k=re;break;case "focusout":m="blur";k=re;break;case "beforeblur":case "afterblur":k=re;break;case "click":if(2===c.button)break a;case "auxclick":case "dblclick":case "mousedown":case "mousemove":case "mouseup":case "mouseout":case "mouseover":case "contextmenu":k=
Gg;break;case "drag":case "dragend":case "dragenter":case "dragexit":case "dragleave":case "dragover":case "dragstart":case "drop":k=dj;break;case "touchcancel":case "touchend":case "touchmove":case "touchstart":k=ej;break;case Hg:case Ig:case Jg:k=fj;break;case Kg:k=gj;break;case "scroll":k=hj;break;case "wheel":k=ij;break;case "copy":case "cut":case "paste":k=jj;break;case "gotpointercapture":case "lostpointercapture":case "pointercancel":case "pointerdown":case "pointermove":case "pointerout":case "pointerover":case "pointerup":k=
Lg}var l=0!==(b&4),C=!l&&"scroll"===a,x=l?null!==h?h+"Capture":null:h;l=[];for(var p=d,q;null!==p;){q=p;var u=q.stateNode;5===q.tag&&null!==u&&(q=u,null!==x&&(u=Pb(p,x),null!=u&&l.push(ac(p,u,q))));if(C)break;p=p.return}0<l.length&&(h=new k(h,m,null,c,e),g.push({event:h,listeners:l}))}}if(0===(b&7)){a:{h="mouseover"===a||"pointerover"===a;k="mouseout"===a||"pointerout"===a;if(h&&0===(b&16)&&(m=c.relatedTarget||c.fromElement)&&(Wa(m)||m[rb]))break a;if(k||h){h=e.window===e?e:(h=e.ownerDocument)?h.defaultView||
h.parentWindow:window;if(k){if(m=c.relatedTarget||c.toElement,k=d,m=m?Wa(m):null,null!==m&&(C=Va(m),m!==C||5!==m.tag&&6!==m.tag))m=null}else k=null,m=d;if(k!==m){l=Gg;u="onMouseLeave";x="onMouseEnter";p="mouse";if("pointerout"===a||"pointerover"===a)l=Lg,u="onPointerLeave",x="onPointerEnter",p="pointer";C=null==k?h:pb(k);q=null==m?h:pb(m);h=new l(u,p+"leave",k,c,e);h.target=C;h.relatedTarget=q;u=null;Wa(e)===d&&(l=new l(x,p+"enter",m,c,e),l.target=q,l.relatedTarget=C,u=l);C=u;if(k&&m)b:{l=k;x=m;p=
0;for(q=l;q;q=sb(q))p++;q=0;for(u=x;u;u=sb(u))q++;for(;0<p-q;)l=sb(l),p--;for(;0<q-p;)x=sb(x),q--;for(;p--;){if(l===x||null!==x&&l===x.alternate)break b;l=sb(l);x=sb(x)}l=null}else l=null;null!==k&&Mg(g,h,k,l,!1);null!==m&&null!==C&&Mg(g,C,m,l,!0)}}}a:{h=d?pb(d):window;k=h.nodeName&&h.nodeName.toLowerCase();if("select"===k||"input"===k&&"file"===h.type)var n=Wi;else if(pg(h))if(Ng)n=$i;else{n=Yi;var da=Xi}else(k=h.nodeName)&&"input"===k.toLowerCase()&&("checkbox"===h.type||"radio"===h.type)&&(n=Zi);
if(n&&(n=n(a,d))){qg(g,n,c,e);break a}da&&da(a,h,d);"focusout"===a&&(da=h._wrapperState)&&da.controlled&&"number"===h.type&&Od(h,"number",h.value)}da=d?pb(d):window;switch(a){case "focusin":if(pg(da)||"true"===da.contentEditable)qb=da,pe=d,$b=null;break;case "focusout":$b=pe=qb=null;break;case "mousedown":oe=!0;break;case "contextmenu":case "mouseup":case "dragend":oe=!1;yg(g,c,e);break;case "selectionchange":if(kj)break;case "keydown":case "keyup":yg(g,c,e)}var Ea;if(ke)b:{switch(a){case "compositionstart":var F=
"onCompositionStart";break b;case "compositionend":F="onCompositionEnd";break b;case "compositionupdate":F="onCompositionUpdate";break b}F=void 0}else ob?kg(a,c)&&(F="onCompositionEnd"):"keydown"===a&&229===c.keyCode&&(F="onCompositionStart");F&&(og&&"ko"!==c.locale&&(ob||"onCompositionStart"!==F?"onCompositionEnd"===F&&ob&&(Ea=ig()):(Ca=e,ie="value"in Ca?Ca.value:Ca.textContent,ob=!0)),da=Tc(d,F),0<da.length&&(F=new Og(F,a,null,c,e),g.push({event:F,listeners:da}),Ea?F.data=Ea:(Ea=lg(c),null!==Ea&&
(F.data=Ea))));if(Ea=lj?Ri(a,c):Si(a,c))d=Tc(d,"onBeforeInput"),0<d.length&&(e=new mj("onBeforeInput","beforeinput",null,c,e),g.push({event:e,listeners:d}),e.data=Ea)}rg(g,b)})}function ac(a,b,c){return{instance:a,listener:b,currentTarget:c}}function Tc(a,b){for(var c=b+"Capture",d=[];null!==a;){var e=a,f=e.stateNode;5===e.tag&&null!==f&&(e=f,f=Pb(a,c),null!=f&&d.unshift(ac(a,f,e)),f=Pb(a,b),null!=f&&d.push(ac(a,f,e)));a=a.return}return d}function sb(a){if(null===a)return null;do a=a.return;while(a&&
5!==a.tag);return a?a:null}function Mg(a,b,c,d,e){for(var f=b._reactName,g=[];null!==c&&c!==d;){var h=c,k=h.alternate,v=h.stateNode;if(null!==k&&k===d)break;5===h.tag&&null!==v&&(h=v,e?(k=Pb(c,f),null!=k&&g.unshift(ac(c,k,h))):e||(k=Pb(c,f),null!=k&&g.push(ac(c,k,h))));c=c.return}0!==g.length&&a.push({event:b,listeners:g})}function Vc(){}function Pg(a,b){switch(a){case "button":case "input":case "select":case "textarea":return!!b.autoFocus}return!1}function se(a,b){return"textarea"===a||"option"===
a||"noscript"===a||"string"===typeof b.children||"number"===typeof b.children||"object"===typeof b.dangerouslySetInnerHTML&&null!==b.dangerouslySetInnerHTML&&null!=b.dangerouslySetInnerHTML.__html}function te(a){1===a.nodeType?a.textContent="":9===a.nodeType&&(a=a.body,null!=a&&(a.textContent=""))}function tb(a){for(;null!=a;a=a.nextSibling){var b=a.nodeType;if(1===b||3===b)break}return a}function Qg(a){a=a.previousSibling;for(var b=0;a;){if(8===a.nodeType){var c=a.data;if("$"===c||"$!"===c||"$?"===
c){if(0===b)return a;b--}else"/$"===c&&b++}a=a.previousSibling}return null}function nj(a){return{$$typeof:ue,toString:a,valueOf:a}}function Wa(a){var b=a[Fa];if(b)return b;for(var c=a.parentNode;c;){if(b=c[rb]||c[Fa]){c=b.alternate;if(null!==b.child||null!==c&&null!==c.child)for(a=Qg(a);null!==a;){if(c=a[Fa])return c;a=Qg(a)}return b}a=c;c=a.parentNode}return null}function Ob(a){a=a[Fa]||a[rb];return!a||5!==a.tag&&6!==a.tag&&13!==a.tag&&3!==a.tag?null:a}function pb(a){if(5===a.tag||6===a.tag)return a.stateNode;
throw Error(m(33));}function Hc(a){return a[Wc]||null}function Ag(a){var b=a[Rg];void 0===b&&(b=a[Rg]=new Set);return b}function Ga(a){return{current:a}}function t(a,b){0>ub||(a.current=ve[ub],ve[ub]=null,ub--)}function A(a,b,c){ub++;ve[ub]=a.current;a.current=b}function vb(a,b){var c=a.type.contextTypes;if(!c)return Ha;var d=a.stateNode;if(d&&d.__reactInternalMemoizedUnmaskedChildContext===b)return d.__reactInternalMemoizedMaskedChildContext;var e={},f;for(f in c)e[f]=b[f];d&&(a=a.stateNode,a.__reactInternalMemoizedUnmaskedChildContext=
b,a.__reactInternalMemoizedMaskedChildContext=e);return e}function S(a){a=a.childContextTypes;return null!==a&&void 0!==a}function Sg(a,b,c){if(D.current!==Ha)throw Error(m(168));A(D,b);A(J,c)}function Tg(a,b,c){var d=a.stateNode;a=b.childContextTypes;if("function"!==typeof d.getChildContext)return c;d=d.getChildContext();for(var e in d)if(!(e in a))throw Error(m(108,hb(b)||"Unknown",e));return B({},c,d)}function Xc(a){a=(a=a.stateNode)&&a.__reactInternalMemoizedMergedChildContext||Ha;Ya=D.current;
A(D,a);A(J,J.current);return!0}function Ug(a,b,c){var d=a.stateNode;if(!d)throw Error(m(169));c?(a=Tg(a,b,Ya),d.__reactInternalMemoizedMergedChildContext=a,t(J),t(D),A(D,a)):t(J);A(J,c)}function wb(){switch(oj()){case Yc:return 99;case Vg:return 98;case Wg:return 97;case Xg:return 96;case Yg:return 95;default:throw Error(m(332));}}function Zg(a){switch(a){case 99:return Yc;case 98:return Vg;case 97:return Wg;case 96:return Xg;case 95:return Yg;default:throw Error(m(332));}}function Za(a,b){a=Zg(a);
return pj(a,b)}function bc(a,b,c){a=Zg(a);return we(a,b,c)}function ja(){if(null!==Zc){var a=Zc;Zc=null;xe(a)}$g()}function $g(){if(!ye&&null!==pa){ye=!0;var a=0;try{var b=pa;Za(99,function(){for(;a<b.length;a++){var c=b[a];do c=c(!0);while(null!==c)}});pa=null}catch(c){throw null!==pa&&(pa=pa.slice(a+1)),we(Yc,ja),c;}finally{ye=!1}}}function ea(a,b){if(a&&a.defaultProps){b=B({},b);a=a.defaultProps;for(var c in a)void 0===b[c]&&(b[c]=a[c]);return b}return b}function ze(){$c=xb=ad=null}function Ae(a){var b=
bd.current;t(bd);a.type._context._currentValue=b}function ah(a,b){for(;null!==a;){var c=a.alternate;if((a.childLanes&b)===b)if(null===c||(c.childLanes&b)===b)break;else c.childLanes|=b;else a.childLanes|=b,null!==c&&(c.childLanes|=b);a=a.return}}function yb(a,b){ad=a;$c=xb=null;a=a.dependencies;null!==a&&null!==a.firstContext&&(0!==(a.lanes&b)&&(fa=!0),a.firstContext=null)}function Y(a,b){if($c!==a&&!1!==b&&0!==b){if("number"!==typeof b||1073741823===b)$c=a,b=1073741823;b={context:a,observedBits:b,
next:null};if(null===xb){if(null===ad)throw Error(m(308));xb=b;ad.dependencies={lanes:0,firstContext:b,responders:null}}else xb=xb.next=b}return a._currentValue}function Be(a){a.updateQueue={baseState:a.memoizedState,firstBaseUpdate:null,lastBaseUpdate:null,shared:{pending:null},effects:null}}function bh(a,b){a=a.updateQueue;b.updateQueue===a&&(b.updateQueue={baseState:a.baseState,firstBaseUpdate:a.firstBaseUpdate,lastBaseUpdate:a.lastBaseUpdate,shared:a.shared,effects:a.effects})}function Ia(a,b){return{eventTime:a,
lane:b,tag:0,payload:null,callback:null,next:null}}function Ja(a,b){a=a.updateQueue;if(null!==a){a=a.shared;var c=a.pending;null===c?b.next=b:(b.next=c.next,c.next=b);a.pending=b}}function ch(a,b){var c=a.updateQueue,d=a.alternate;if(null!==d&&(d=d.updateQueue,c===d)){var e=null,f=null;c=c.firstBaseUpdate;if(null!==c){do{var g={eventTime:c.eventTime,lane:c.lane,tag:c.tag,payload:c.payload,callback:c.callback,next:null};null===f?e=f=g:f=f.next=g;c=c.next}while(null!==c);null===f?e=f=b:f=f.next=b}else e=
f=b;c={baseState:d.baseState,firstBaseUpdate:e,lastBaseUpdate:f,shared:d.shared,effects:d.effects};a.updateQueue=c;return}a=c.lastBaseUpdate;null===a?c.firstBaseUpdate=b:a.next=b;c.lastBaseUpdate=b}function cc(a,b,c,d){var e=a.updateQueue;Ka=!1;var f=e.firstBaseUpdate,g=e.lastBaseUpdate,h=e.shared.pending;if(null!==h){e.shared.pending=null;var k=h,v=k.next;k.next=null;null===g?f=v:g.next=v;g=k;var m=a.alternate;if(null!==m){m=m.updateQueue;var l=m.lastBaseUpdate;l!==g&&(null===l?m.firstBaseUpdate=
v:l.next=v,m.lastBaseUpdate=k)}}if(null!==f){l=e.baseState;g=0;m=v=k=null;do{h=f.lane;var r=f.eventTime;if((d&h)===h){null!==m&&(m=m.next={eventTime:r,lane:0,tag:f.tag,payload:f.payload,callback:f.callback,next:null});a:{var n=a,t=f;h=b;r=c;switch(t.tag){case 1:n=t.payload;if("function"===typeof n){l=n.call(r,l,h);break a}l=n;break a;case 3:n.flags=n.flags&-4097|64;case 0:n=t.payload;h="function"===typeof n?n.call(r,l,h):n;if(null===h||void 0===h)break a;l=B({},l,h);break a;case 2:Ka=!0}}null!==f.callback&&
(a.flags|=32,h=e.effects,null===h?e.effects=[f]:h.push(f))}else r={eventTime:r,lane:h,tag:f.tag,payload:f.payload,callback:f.callback,next:null},null===m?(v=m=r,k=l):m=m.next=r,g|=h;f=f.next;if(null===f)if(h=e.shared.pending,null===h)break;else f=h.next,h.next=null,e.lastBaseUpdate=h,e.shared.pending=null}while(1);null===m&&(k=l);e.baseState=k;e.firstBaseUpdate=v;e.lastBaseUpdate=m;La|=g;a.lanes=g;a.memoizedState=l}}function dh(a,b,c){a=b.effects;b.effects=null;if(null!==a)for(b=0;b<a.length;b++){var d=
a[b],e=d.callback;if(null!==e){d.callback=null;d=c;if("function"!==typeof e)throw Error(m(191,e));e.call(d)}}}function cd(a,b,c,d){b=a.memoizedState;c=c(d,b);c=null===c||void 0===c?b:B({},b,c);a.memoizedState=c;0===a.lanes&&(a.updateQueue.baseState=c)}function eh(a,b,c,d,e,f,g){a=a.stateNode;return"function"===typeof a.shouldComponentUpdate?a.shouldComponentUpdate(d,f,g):b.prototype&&b.prototype.isPureReactComponent?!Zb(c,d)||!Zb(e,f):!0}function fh(a,b,c){var d=!1,e=Ha;var f=b.contextType;"object"===
typeof f&&null!==f?f=Y(f):(e=S(b)?Ya:D.current,d=b.contextTypes,f=(d=null!==d&&void 0!==d)?vb(a,e):Ha);b=new b(c,f);a.memoizedState=null!==b.state&&void 0!==b.state?b.state:null;b.updater=dd;a.stateNode=b;b._reactInternals=a;d&&(a=a.stateNode,a.__reactInternalMemoizedUnmaskedChildContext=e,a.__reactInternalMemoizedMaskedChildContext=f);return b}function gh(a,b,c,d){a=b.state;"function"===typeof b.componentWillReceiveProps&&b.componentWillReceiveProps(c,d);"function"===typeof b.UNSAFE_componentWillReceiveProps&&
b.UNSAFE_componentWillReceiveProps(c,d);b.state!==a&&dd.enqueueReplaceState(b,b.state,null)}function Ce(a,b,c,d){var e=a.stateNode;e.props=c;e.state=a.memoizedState;e.refs=hh;Be(a);var f=b.contextType;"object"===typeof f&&null!==f?e.context=Y(f):(f=S(b)?Ya:D.current,e.context=vb(a,f));cc(a,c,e,d);e.state=a.memoizedState;f=b.getDerivedStateFromProps;"function"===typeof f&&(cd(a,b,f,c),e.state=a.memoizedState);"function"===typeof b.getDerivedStateFromProps||"function"===typeof e.getSnapshotBeforeUpdate||
"function"!==typeof e.UNSAFE_componentWillMount&&"function"!==typeof e.componentWillMount||(b=e.state,"function"===typeof e.componentWillMount&&e.componentWillMount(),"function"===typeof e.UNSAFE_componentWillMount&&e.UNSAFE_componentWillMount(),b!==e.state&&dd.enqueueReplaceState(e,e.state,null),cc(a,c,e,d),e.state=a.memoizedState);"function"===typeof e.componentDidMount&&(a.flags|=4)}function dc(a,b,c){a=c.ref;if(null!==a&&"function"!==typeof a&&"object"!==typeof a){if(c._owner){c=c._owner;if(c){if(1!==
c.tag)throw Error(m(309));var d=c.stateNode}if(!d)throw Error(m(147,a));var e=""+a;if(null!==b&&null!==b.ref&&"function"===typeof b.ref&&b.ref._stringRef===e)return b.ref;b=function(a){var b=d.refs;b===hh&&(b=d.refs={});null===a?delete b[e]:b[e]=a};b._stringRef=e;return b}if("string"!==typeof a)throw Error(m(284));if(!c._owner)throw Error(m(290,a));}return a}function ed(a,b){if("textarea"!==a.type)throw Error(m(31,"[object Object]"===Object.prototype.toString.call(b)?"object with keys {"+Object.keys(b).join(", ")+
"}":b));}function ih(a){function b(b,c){if(a){var d=b.lastEffect;null!==d?(d.nextEffect=c,b.lastEffect=c):b.firstEffect=b.lastEffect=c;c.nextEffect=null;c.flags=8}}function c(c,d){if(!a)return null;for(;null!==d;)b(c,d),d=d.sibling;return null}function d(a,b){for(a=new Map;null!==b;)null!==b.key?a.set(b.key,b):a.set(b.index,b),b=b.sibling;return a}function e(a,b){a=Ma(a,b);a.index=0;a.sibling=null;return a}function f(b,c,d){b.index=d;if(!a)return c;d=b.alternate;if(null!==d)return d=d.index,d<c?(b.flags=
2,c):d;b.flags=2;return c}function g(b){a&&null===b.alternate&&(b.flags=2);return b}function h(a,b,c,d){if(null===b||6!==b.tag)return b=De(c,a.mode,d),b.return=a,b;b=e(b,c);b.return=a;return b}function k(a,b,c,d){if(null!==b&&b.elementType===c.type)return d=e(b,c.props),d.ref=dc(a,b,c),d.return=a,d;d=fd(c.type,c.key,c.props,null,a.mode,d);d.ref=dc(a,b,c);d.return=a;return d}function v(a,b,c,d){if(null===b||4!==b.tag||b.stateNode.containerInfo!==c.containerInfo||b.stateNode.implementation!==c.implementation)return b=
Ee(c,a.mode,d),b.return=a,b;b=e(b,c.children||[]);b.return=a;return b}function l(a,b,c,d,f){if(null===b||7!==b.tag)return b=zb(c,a.mode,d,f),b.return=a,b;b=e(b,c);b.return=a;return b}function n(a,b,c){if("string"===typeof b||"number"===typeof b)return b=De(""+b,a.mode,c),b.return=a,b;if("object"===typeof b&&null!==b){switch(b.$$typeof){case ec:return c=fd(b.type,b.key,b.props,null,a.mode,c),c.ref=dc(a,null,b),c.return=a,c;case Ua:return b=Ee(b,a.mode,c),b.return=a,b}if(gd(b)||Jb(b))return b=zb(b,
a.mode,c,null),b.return=a,b;ed(a,b)}return null}function r(a,b,c,d){var e=null!==b?b.key:null;if("string"===typeof c||"number"===typeof c)return null!==e?null:h(a,b,""+c,d);if("object"===typeof c&&null!==c){switch(c.$$typeof){case ec:return c.key===e?c.type===wa?l(a,b,c.props.children,d,e):k(a,b,c,d):null;case Ua:return c.key===e?v(a,b,c,d):null}if(gd(c)||Jb(c))return null!==e?null:l(a,b,c,d,null);ed(a,c)}return null}function t(a,b,c,d,e){if("string"===typeof d||"number"===typeof d)return a=a.get(c)||
null,h(b,a,""+d,e);if("object"===typeof d&&null!==d){switch(d.$$typeof){case ec:return a=a.get(null===d.key?c:d.key)||null,d.type===wa?l(b,a,d.props.children,e,d.key):k(b,a,d,e);case Ua:return a=a.get(null===d.key?c:d.key)||null,v(b,a,d,e)}if(gd(d)||Jb(d))return a=a.get(c)||null,l(b,a,d,e,null);ed(b,d)}return null}function w(e,g,h,k){for(var m=null,v=null,l=g,p=g=0,x=null;null!==l&&p<h.length;p++){l.index>p?(x=l,l=null):x=l.sibling;var C=r(e,l,h[p],k);if(null===C){null===l&&(l=x);break}a&&l&&null===
C.alternate&&b(e,l);g=f(C,g,p);null===v?m=C:v.sibling=C;v=C;l=x}if(p===h.length)return c(e,l),m;if(null===l){for(;p<h.length;p++)l=n(e,h[p],k),null!==l&&(g=f(l,g,p),null===v?m=l:v.sibling=l,v=l);return m}for(l=d(e,l);p<h.length;p++)x=t(l,e,p,h[p],k),null!==x&&(a&&null!==x.alternate&&l.delete(null===x.key?p:x.key),g=f(x,g,p),null===v?m=x:v.sibling=x,v=x);a&&l.forEach(function(a){return b(e,a)});return m}function z(e,g,h,k){var l=Jb(h);if("function"!==typeof l)throw Error(m(150));h=l.call(h);if(null==
h)throw Error(m(151));for(var v=l=null,p=g,x=g=0,C=null,q=h.next();null!==p&&!q.done;x++,q=h.next()){p.index>x?(C=p,p=null):C=p.sibling;var Da=r(e,p,q.value,k);if(null===Da){null===p&&(p=C);break}a&&p&&null===Da.alternate&&b(e,p);g=f(Da,g,x);null===v?l=Da:v.sibling=Da;v=Da;p=C}if(q.done)return c(e,p),l;if(null===p){for(;!q.done;x++,q=h.next())q=n(e,q.value,k),null!==q&&(g=f(q,g,x),null===v?l=q:v.sibling=q,v=q);return l}for(p=d(e,p);!q.done;x++,q=h.next())q=t(p,e,x,q.value,k),null!==q&&(a&&null!==
q.alternate&&p.delete(null===q.key?x:q.key),g=f(q,g,x),null===v?l=q:v.sibling=q,v=q);a&&p.forEach(function(a){return b(e,a)});return l}return function(a,d,f,h){var k="object"===typeof f&&null!==f&&f.type===wa&&null===f.key;k&&(f=f.props.children);var l="object"===typeof f&&null!==f;if(l)switch(f.$$typeof){case ec:a:{l=f.key;for(k=d;null!==k;){if(k.key===l){switch(k.tag){case 7:if(f.type===wa){c(a,k.sibling);d=e(k,f.props.children);d.return=a;a=d;break a}break;default:if(k.elementType===f.type){c(a,
k.sibling);d=e(k,f.props);d.ref=dc(a,k,f);d.return=a;a=d;break a}}c(a,k);break}else b(a,k);k=k.sibling}f.type===wa?(d=zb(f.props.children,a.mode,h,f.key),d.return=a,a=d):(h=fd(f.type,f.key,f.props,null,a.mode,h),h.ref=dc(a,d,f),h.return=a,a=h)}return g(a);case Ua:a:{for(k=f.key;null!==d;){if(d.key===k)if(4===d.tag&&d.stateNode.containerInfo===f.containerInfo&&d.stateNode.implementation===f.implementation){c(a,d.sibling);d=e(d,f.children||[]);d.return=a;a=d;break a}else{c(a,d);break}else b(a,d);d=
d.sibling}d=Ee(f,a.mode,h);d.return=a;a=d}return g(a)}if("string"===typeof f||"number"===typeof f)return f=""+f,null!==d&&6===d.tag?(c(a,d.sibling),d=e(d,f),d.return=a,a=d):(c(a,d),d=De(f,a.mode,h),d.return=a,a=d),g(a);if(gd(f))return w(a,d,f,h);if(Jb(f))return z(a,d,f,h);l&&ed(a,f);if("undefined"===typeof f&&!k)switch(a.tag){case 1:case 22:case 0:case 11:case 15:throw Error(m(152,hb(a.type)||"Component"));}return c(a,d)}}function $a(a){if(a===fc)throw Error(m(174));return a}function Fe(a,b){A(gc,
b);A(hc,a);A(ka,fc);a=b.nodeType;switch(a){case 9:case 11:b=(b=b.documentElement)?b.namespaceURI:Rd(null,"");break;default:a=8===a?b.parentNode:b,b=a.namespaceURI||null,a=a.tagName,b=Rd(b,a)}t(ka);A(ka,b)}function Ab(a){t(ka);t(hc);t(gc)}function jh(a){$a(gc.current);var b=$a(ka.current);var c=Rd(b,a.type);b!==c&&(A(hc,a),A(ka,c))}function Ge(a){hc.current===a&&(t(ka),t(hc))}function hd(a){for(var b=a;null!==b;){if(13===b.tag){var c=b.memoizedState;if(null!==c&&(c=c.dehydrated,null===c||"$?"===c.data||
"$!"===c.data))return b}else if(19===b.tag&&void 0!==b.memoizedProps.revealOrder){if(0!==(b.flags&64))return b}else if(null!==b.child){b.child.return=b;b=b.child;continue}if(b===a)break;for(;null===b.sibling;){if(null===b.return||b.return===a)return null;b=b.return}b.sibling.return=b.return;b=b.sibling}return null}function kh(a,b){var c=Z(5,null,null,0);c.elementType="DELETED";c.type="DELETED";c.stateNode=b;c.return=a;c.flags=8;null!==a.lastEffect?(a.lastEffect.nextEffect=c,a.lastEffect=c):a.firstEffect=
a.lastEffect=c}function lh(a,b){switch(a.tag){case 5:var c=a.type;b=1!==b.nodeType||c.toLowerCase()!==b.nodeName.toLowerCase()?null:b;return null!==b?(a.stateNode=b,!0):!1;case 6:return b=""===a.pendingProps||3!==b.nodeType?null:b,null!==b?(a.stateNode=b,!0):!1;case 13:return!1;default:return!1}}function He(a){if(la){var b=Na;if(b){var c=b;if(!lh(a,b)){b=tb(c.nextSibling);if(!b||!lh(a,b)){a.flags=a.flags&-1025|2;la=!1;ra=a;return}kh(ra,c)}ra=a;Na=tb(b.firstChild)}else a.flags=a.flags&-1025|2,la=!1,
ra=a}}function mh(a){for(a=a.return;null!==a&&5!==a.tag&&3!==a.tag&&13!==a.tag;)a=a.return;ra=a}function id(a){if(a!==ra)return!1;if(!la)return mh(a),la=!0,!1;var b=a.type;if(5!==a.tag||"head"!==b&&"body"!==b&&!se(b,a.memoizedProps))for(b=Na;b;)kh(a,b),b=tb(b.nextSibling);mh(a);if(13===a.tag){a=a.memoizedState;a=null!==a?a.dehydrated:null;if(!a)throw Error(m(317));a:{a=a.nextSibling;for(b=0;a;){if(8===a.nodeType){var c=a.data;if("/$"===c){if(0===b){Na=tb(a.nextSibling);break a}b--}else"$"!==c&&"$!"!==
c&&"$?"!==c||b++}a=a.nextSibling}Na=null}}else Na=ra?tb(a.stateNode.nextSibling):null;return!0}function Ie(){Na=ra=null;la=!1}function Je(){for(var a=0;a<Bb.length;a++)Bb[a]._workInProgressVersionPrimary=null;Bb.length=0}function T(){throw Error(m(321));}function Ke(a,b){if(null===b)return!1;for(var c=0;c<b.length&&c<a.length;c++)if(!X(a[c],b[c]))return!1;return!0}function Le(a,b,c,d,e,f){ic=f;y=b;b.memoizedState=null;b.updateQueue=null;b.lanes=0;jc.current=null===a||null===a.memoizedState?qj:rj;
a=c(d,e);if(kc){f=0;do{kc=!1;if(!(25>f))throw Error(m(301));f+=1;K=N=null;b.updateQueue=null;jc.current=sj;a=c(d,e)}while(kc)}jc.current=jd;b=null!==N&&null!==N.next;ic=0;K=N=y=null;kd=!1;if(b)throw Error(m(300));return a}function ab(){var a={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};null===K?y.memoizedState=K=a:K=K.next=a;return K}function bb(){if(null===N){var a=y.alternate;a=null!==a?a.memoizedState:null}else a=N.next;var b=null===K?y.memoizedState:K.next;if(null!==
b)K=b,N=a;else{if(null===a)throw Error(m(310));N=a;a={memoizedState:N.memoizedState,baseState:N.baseState,baseQueue:N.baseQueue,queue:N.queue,next:null};null===K?y.memoizedState=K=a:K=K.next=a}return K}function ma(a,b){return"function"===typeof b?b(a):b}function lc(a,b,c){b=bb();c=b.queue;if(null===c)throw Error(m(311));c.lastRenderedReducer=a;var d=N,e=d.baseQueue,f=c.pending;if(null!==f){if(null!==e){var g=e.next;e.next=f.next;f.next=g}d.baseQueue=e=f;c.pending=null}if(null!==e){e=e.next;d=d.baseState;
var h=g=f=null,k=e;do{var l=k.lane;if((ic&l)===l)null!==h&&(h=h.next={lane:0,action:k.action,eagerReducer:k.eagerReducer,eagerState:k.eagerState,next:null}),d=k.eagerReducer===a?k.eagerState:a(d,k.action);else{var n={lane:l,action:k.action,eagerReducer:k.eagerReducer,eagerState:k.eagerState,next:null};null===h?(g=h=n,f=d):h=h.next=n;y.lanes|=l;La|=l}k=k.next}while(null!==k&&k!==e);null===h?f=d:h.next=g;X(d,b.memoizedState)||(fa=!0);b.memoizedState=d;b.baseState=f;b.baseQueue=h;c.lastRenderedState=
d}return[b.memoizedState,c.dispatch]}function mc(a,b,c){b=bb();c=b.queue;if(null===c)throw Error(m(311));c.lastRenderedReducer=a;var d=c.dispatch,e=c.pending,f=b.memoizedState;if(null!==e){c.pending=null;var g=e=e.next;do f=a(f,g.action),g=g.next;while(g!==e);X(f,b.memoizedState)||(fa=!0);b.memoizedState=f;null===b.baseQueue&&(b.baseState=f);c.lastRenderedState=f}return[f,d]}function nh(a,b,c){var d=b._getVersion;d=d(b._source);var e=b._workInProgressVersionPrimary;if(null!==e)a=e===d;else if(a=a.mutableReadLanes,
a=(ic&a)===a)b._workInProgressVersionPrimary=d,Bb.push(b);if(a)return c(b._source);Bb.push(b);throw Error(m(350));}function oh(a,b,c,d){var e=R;if(null===e)throw Error(m(349));var f=b._getVersion,g=f(b._source),h=jc.current,k=h.useState(function(){return nh(e,b,c)}),l=k[1],n=k[0];k=K;var t=a.memoizedState,r=t.refs,w=r.getSnapshot,z=t.source;t=t.subscribe;var B=y;a.memoizedState={refs:r,source:b,subscribe:d};h.useEffect(function(){r.getSnapshot=c;r.setSnapshot=l;var a=f(b._source);if(!X(g,a)){a=c(b._source);
X(n,a)||(l(a),a=Oa(B),e.mutableReadLanes|=a&e.pendingLanes);a=e.mutableReadLanes;e.entangledLanes|=a;for(var d=e.entanglements,h=a;0<h;){var k=31-Ba(h),m=1<<k;d[k]|=a;h&=~m}}},[c,b,d]);h.useEffect(function(){return d(b._source,function(){var a=r.getSnapshot,c=r.setSnapshot;try{c(a(b._source));var d=Oa(B);e.mutableReadLanes|=d&e.pendingLanes}catch(q){c(function(){throw q;})}})},[b,d]);X(w,c)&&X(z,b)&&X(t,d)||(a={pending:null,dispatch:null,lastRenderedReducer:ma,lastRenderedState:n},a.dispatch=l=Me.bind(null,
y,a),k.queue=a,k.baseQueue=null,n=nh(e,b,c),k.memoizedState=k.baseState=n);return n}function ph(a,b,c){var d=bb();return oh(d,a,b,c)}function nc(a){var b=ab();"function"===typeof a&&(a=a());b.memoizedState=b.baseState=a;a=b.queue={pending:null,dispatch:null,lastRenderedReducer:ma,lastRenderedState:a};a=a.dispatch=Me.bind(null,y,a);return[b.memoizedState,a]}function ld(a,b,c,d){a={tag:a,create:b,destroy:c,deps:d,next:null};b=y.updateQueue;null===b?(b={lastEffect:null},y.updateQueue=b,b.lastEffect=
a.next=a):(c=b.lastEffect,null===c?b.lastEffect=a.next=a:(d=c.next,c.next=a,a.next=d,b.lastEffect=a));return a}function qh(a){var b=ab();a={current:a};return b.memoizedState=a}function md(a){return bb().memoizedState}function Ne(a,b,c,d){var e=ab();y.flags|=a;e.memoizedState=ld(1|b,c,void 0,void 0===d?null:d)}function Oe(a,b,c,d){var e=bb();d=void 0===d?null:d;var f=void 0;if(null!==N){var g=N.memoizedState;f=g.destroy;if(null!==d&&Ke(d,g.deps)){ld(b,c,f,d);return}}y.flags|=a;e.memoizedState=ld(1|
b,c,f,d)}function rh(a,b){return Ne(516,4,a,b)}function nd(a,b){return Oe(516,4,a,b)}function sh(a,b){return Oe(4,2,a,b)}function th(a,b){if("function"===typeof b)return a=a(),b(a),function(){b(null)};if(null!==b&&void 0!==b)return a=a(),b.current=a,function(){b.current=null}}function uh(a,b,c){c=null!==c&&void 0!==c?c.concat([a]):null;return Oe(4,2,th.bind(null,b,a),c)}function Pe(a,b){}function vh(a,b){var c=bb();b=void 0===b?null:b;var d=c.memoizedState;if(null!==d&&null!==b&&Ke(b,d[1]))return d[0];
c.memoizedState=[a,b];return a}function wh(a,b){var c=bb();b=void 0===b?null:b;var d=c.memoizedState;if(null!==d&&null!==b&&Ke(b,d[1]))return d[0];a=a();c.memoizedState=[a,b];return a}function tj(a,b){var c=wb();Za(98>c?98:c,function(){a(!0)});Za(97<c?97:c,function(){var c=aa.transition;aa.transition=1;try{a(!1),b()}finally{aa.transition=c}})}function Me(a,b,c){var d=W(),e=Oa(a),f={lane:e,action:c,eagerReducer:null,eagerState:null,next:null},g=b.pending;null===g?f.next=f:(f.next=g.next,g.next=f);
b.pending=f;g=a.alternate;if(a===y||null!==g&&g===y)kc=kd=!0;else{if(0===a.lanes&&(null===g||0===g.lanes)&&(g=b.lastRenderedReducer,null!==g))try{var h=b.lastRenderedState,k=g(h,c);f.eagerReducer=g;f.eagerState=k;if(X(k,h))return}catch(v){}finally{}Pa(a,e,d)}}function U(a,b,c,d){b.child=null===a?xh(b,null,c,d):od(b,a.child,c,d)}function yh(a,b,c,d,e){c=c.render;var f=b.ref;yb(b,e);d=Le(a,b,c,d,f,e);if(null!==a&&!fa)return b.updateQueue=a.updateQueue,b.flags&=-517,a.lanes&=~e,sa(a,b,e);b.flags|=1;
U(a,b,d,e);return b.child}function zh(a,b,c,d,e,f){if(null===a){var g=c.type;if("function"===typeof g&&!Qe(g)&&void 0===g.defaultProps&&null===c.compare&&void 0===c.defaultProps)return b.tag=15,b.type=g,Ah(a,b,g,d,e,f);a=fd(c.type,null,d,b,b.mode,f);a.ref=b.ref;a.return=b;return b.child=a}g=a.child;if(0===(e&f)&&(e=g.memoizedProps,c=c.compare,c=null!==c?c:Zb,c(e,d)&&a.ref===b.ref))return sa(a,b,f);b.flags|=1;a=Ma(g,d);a.ref=b.ref;a.return=b;return b.child=a}function Ah(a,b,c,d,e,f){if(null!==a&&Zb(a.memoizedProps,
d)&&a.ref===b.ref)if(fa=!1,0!==(f&e))0!==(a.flags&16384)&&(fa=!0);else return b.lanes=a.lanes,sa(a,b,f);return Re(a,b,c,d,f)}function Se(a,b,c){var d=b.pendingProps,e=d.children,f=null!==a?a.memoizedState:null;if("hidden"===d.mode||"unstable-defer-without-hiding"===d.mode)if(0===(b.mode&4))b.memoizedState={baseLanes:0},pd(b,c);else if(0!==(c&1073741824))b.memoizedState={baseLanes:0},pd(b,null!==f?f.baseLanes:c);else return a=null!==f?f.baseLanes|c:c,b.lanes=b.childLanes=1073741824,b.memoizedState=
{baseLanes:a},pd(b,a),null;else null!==f?(d=f.baseLanes|c,b.memoizedState=null):d=c,pd(b,d);U(a,b,e,c);return b.child}function Bh(a,b){var c=b.ref;if(null===a&&null!==c||null!==a&&a.ref!==c)b.flags|=128}function Re(a,b,c,d,e){var f=S(c)?Ya:D.current;f=vb(b,f);yb(b,e);c=Le(a,b,c,d,f,e);if(null!==a&&!fa)return b.updateQueue=a.updateQueue,b.flags&=-517,a.lanes&=~e,sa(a,b,e);b.flags|=1;U(a,b,c,e);return b.child}function Ch(a,b,c,d,e){if(S(c)){var f=!0;Xc(b)}else f=!1;yb(b,e);if(null===b.stateNode)null!==
a&&(a.alternate=null,b.alternate=null,b.flags|=2),fh(b,c,d),Ce(b,c,d,e),d=!0;else if(null===a){var g=b.stateNode,h=b.memoizedProps;g.props=h;var k=g.context,l=c.contextType;"object"===typeof l&&null!==l?l=Y(l):(l=S(c)?Ya:D.current,l=vb(b,l));var m=c.getDerivedStateFromProps,n="function"===typeof m||"function"===typeof g.getSnapshotBeforeUpdate;n||"function"!==typeof g.UNSAFE_componentWillReceiveProps&&"function"!==typeof g.componentWillReceiveProps||(h!==d||k!==l)&&gh(b,g,d,l);Ka=!1;var r=b.memoizedState;
g.state=r;cc(b,d,g,e);k=b.memoizedState;h!==d||r!==k||J.current||Ka?("function"===typeof m&&(cd(b,c,m,d),k=b.memoizedState),(h=Ka||eh(b,c,h,d,r,k,l))?(n||"function"!==typeof g.UNSAFE_componentWillMount&&"function"!==typeof g.componentWillMount||("function"===typeof g.componentWillMount&&g.componentWillMount(),"function"===typeof g.UNSAFE_componentWillMount&&g.UNSAFE_componentWillMount()),"function"===typeof g.componentDidMount&&(b.flags|=4)):("function"===typeof g.componentDidMount&&(b.flags|=4),
b.memoizedProps=d,b.memoizedState=k),g.props=d,g.state=k,g.context=l,d=h):("function"===typeof g.componentDidMount&&(b.flags|=4),d=!1)}else{g=b.stateNode;bh(a,b);h=b.memoizedProps;l=b.type===b.elementType?h:ea(b.type,h);g.props=l;n=b.pendingProps;r=g.context;k=c.contextType;"object"===typeof k&&null!==k?k=Y(k):(k=S(c)?Ya:D.current,k=vb(b,k));var t=c.getDerivedStateFromProps;(m="function"===typeof t||"function"===typeof g.getSnapshotBeforeUpdate)||"function"!==typeof g.UNSAFE_componentWillReceiveProps&&
"function"!==typeof g.componentWillReceiveProps||(h!==n||r!==k)&&gh(b,g,d,k);Ka=!1;r=b.memoizedState;g.state=r;cc(b,d,g,e);var w=b.memoizedState;h!==n||r!==w||J.current||Ka?("function"===typeof t&&(cd(b,c,t,d),w=b.memoizedState),(l=Ka||eh(b,c,l,d,r,w,k))?(m||"function"!==typeof g.UNSAFE_componentWillUpdate&&"function"!==typeof g.componentWillUpdate||("function"===typeof g.componentWillUpdate&&g.componentWillUpdate(d,w,k),"function"===typeof g.UNSAFE_componentWillUpdate&&g.UNSAFE_componentWillUpdate(d,
w,k)),"function"===typeof g.componentDidUpdate&&(b.flags|=4),"function"===typeof g.getSnapshotBeforeUpdate&&(b.flags|=256)):("function"!==typeof g.componentDidUpdate||h===a.memoizedProps&&r===a.memoizedState||(b.flags|=4),"function"!==typeof g.getSnapshotBeforeUpdate||h===a.memoizedProps&&r===a.memoizedState||(b.flags|=256),b.memoizedProps=d,b.memoizedState=w),g.props=d,g.state=w,g.context=k,d=l):("function"!==typeof g.componentDidUpdate||h===a.memoizedProps&&r===a.memoizedState||(b.flags|=4),"function"!==
typeof g.getSnapshotBeforeUpdate||h===a.memoizedProps&&r===a.memoizedState||(b.flags|=256),d=!1)}return Te(a,b,c,d,f,e)}function Te(a,b,c,d,e,f){Bh(a,b);var g=0!==(b.flags&64);if(!d&&!g)return e&&Ug(b,c,!1),sa(a,b,f);d=b.stateNode;uj.current=b;var h=g&&"function"!==typeof c.getDerivedStateFromError?null:d.render();b.flags|=1;null!==a&&g?(b.child=od(b,a.child,null,f),b.child=od(b,null,h,f)):U(a,b,h,f);b.memoizedState=d.state;e&&Ug(b,c,!0);return b.child}function Dh(a){var b=a.stateNode;b.pendingContext?
Sg(a,b.pendingContext,b.pendingContext!==b.context):b.context&&Sg(a,b.context,!1);Fe(a,b.containerInfo)}function Eh(a,b,c){var d=b.pendingProps,e=E.current,f=!1,g;(g=0!==(b.flags&64))||(g=null!==a&&null===a.memoizedState?!1:0!==(e&2));g?(f=!0,b.flags&=-65):null!==a&&null===a.memoizedState||void 0===d.fallback||!0===d.unstable_avoidThisFallback||(e|=1);A(E,e&1);if(null===a){void 0!==d.fallback&&He(b);a=d.children;e=d.fallback;if(f)return a=Fh(b,a,e,c),b.child.memoizedState={baseLanes:c},b.memoizedState=
qd,a;if("number"===typeof d.unstable_expectedLoadTime)return a=Fh(b,a,e,c),b.child.memoizedState={baseLanes:c},b.memoizedState=qd,b.lanes=33554432,a;c=Ue({mode:"visible",children:a},b.mode,c,null);c.return=b;return b.child=c}if(null!==a.memoizedState){if(f)return d=Gh(a,b,d.children,d.fallback,c),f=b.child,e=a.child.memoizedState,f.memoizedState=null===e?{baseLanes:c}:{baseLanes:e.baseLanes|c},f.childLanes=a.childLanes&~c,b.memoizedState=qd,d;c=Hh(a,b,d.children,c);b.memoizedState=null;return c}if(f)return d=
Gh(a,b,d.children,d.fallback,c),f=b.child,e=a.child.memoizedState,f.memoizedState=null===e?{baseLanes:c}:{baseLanes:e.baseLanes|c},f.childLanes=a.childLanes&~c,b.memoizedState=qd,d;c=Hh(a,b,d.children,c);b.memoizedState=null;return c}function Fh(a,b,c,d){var e=a.mode,f=a.child;b={mode:"hidden",children:b};0===(e&2)&&null!==f?(f.childLanes=0,f.pendingProps=b):f=Ue(b,e,0,null);c=zb(c,e,d,null);f.return=a;c.return=a;f.sibling=c;a.child=f;return c}function Hh(a,b,c,d){var e=a.child;a=e.sibling;c=Ma(e,
{mode:"visible",children:c});0===(b.mode&2)&&(c.lanes=d);c.return=b;c.sibling=null;null!==a&&(a.nextEffect=null,a.flags=8,b.firstEffect=b.lastEffect=a);return b.child=c}function Gh(a,b,c,d,e){var f=b.mode,g=a.child;a=g.sibling;var h={mode:"hidden",children:c};0===(f&2)&&b.child!==g?(c=b.child,c.childLanes=0,c.pendingProps=h,g=c.lastEffect,null!==g?(b.firstEffect=c.firstEffect,b.lastEffect=g,g.nextEffect=null):b.firstEffect=b.lastEffect=null):c=Ma(g,h);null!==a?d=Ma(a,d):(d=zb(d,f,e,null),d.flags|=
2);d.return=b;c.return=b;c.sibling=d;b.child=c;return d}function Ih(a,b){a.lanes|=b;var c=a.alternate;null!==c&&(c.lanes|=b);ah(a.return,b)}function Ve(a,b,c,d,e,f){var g=a.memoizedState;null===g?a.memoizedState={isBackwards:b,rendering:null,renderingStartTime:0,last:d,tail:c,tailMode:e,lastEffect:f}:(g.isBackwards=b,g.rendering=null,g.renderingStartTime=0,g.last=d,g.tail=c,g.tailMode=e,g.lastEffect=f)}function Jh(a,b,c){var d=b.pendingProps,e=d.revealOrder,f=d.tail;U(a,b,d.children,c);d=E.current;
if(0!==(d&2))d=d&1|2,b.flags|=64;else{if(null!==a&&0!==(a.flags&64))a:for(a=b.child;null!==a;){if(13===a.tag)null!==a.memoizedState&&Ih(a,c);else if(19===a.tag)Ih(a,c);else if(null!==a.child){a.child.return=a;a=a.child;continue}if(a===b)break a;for(;null===a.sibling;){if(null===a.return||a.return===b)break a;a=a.return}a.sibling.return=a.return;a=a.sibling}d&=1}A(E,d);if(0===(b.mode&2))b.memoizedState=null;else switch(e){case "forwards":c=b.child;for(e=null;null!==c;)a=c.alternate,null!==a&&null===
hd(a)&&(e=c),c=c.sibling;c=e;null===c?(e=b.child,b.child=null):(e=c.sibling,c.sibling=null);Ve(b,!1,e,c,f,b.lastEffect);break;case "backwards":c=null;e=b.child;for(b.child=null;null!==e;){a=e.alternate;if(null!==a&&null===hd(a)){b.child=e;break}a=e.sibling;e.sibling=c;c=e;e=a}Ve(b,!0,c,null,f,b.lastEffect);break;case "together":Ve(b,!1,null,null,void 0,b.lastEffect);break;default:b.memoizedState=null}return b.child}function sa(a,b,c){null!==a&&(b.dependencies=a.dependencies);La|=b.lanes;if(0!==(c&
b.childLanes)){if(null!==a&&b.child!==a.child)throw Error(m(153));if(null!==b.child){a=b.child;c=Ma(a,a.pendingProps);b.child=c;for(c.return=b;null!==a.sibling;)a=a.sibling,c=c.sibling=Ma(a,a.pendingProps),c.return=b;c.sibling=null}return b.child}return null}function oc(a,b){if(!la)switch(a.tailMode){case "hidden":b=a.tail;for(var c=null;null!==b;)null!==b.alternate&&(c=b),b=b.sibling;null===c?a.tail=null:c.sibling=null;break;case "collapsed":c=a.tail;for(var d=null;null!==c;)null!==c.alternate&&
(d=c),c=c.sibling;null===d?b||null===a.tail?a.tail=null:a.tail.sibling=null:d.sibling=null}}function vj(a,b,c){var d=b.pendingProps;switch(b.tag){case 2:case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return null;case 1:return S(b.type)&&(t(J),t(D)),null;case 3:Ab();t(J);t(D);Je();d=b.stateNode;d.pendingContext&&(d.context=d.pendingContext,d.pendingContext=null);if(null===a||null===a.child)id(b)?b.flags|=4:d.hydrate||(b.flags|=256);Kh(b);return null;case 5:Ge(b);var e=$a(gc.current);
c=b.type;if(null!==a&&null!=b.stateNode)wj(a,b,c,d,e),a.ref!==b.ref&&(b.flags|=128);else{if(!d){if(null===b.stateNode)throw Error(m(166));return null}a=$a(ka.current);if(id(b)){d=b.stateNode;c=b.type;var f=b.memoizedProps;d[Fa]=b;d[Wc]=f;switch(c){case "dialog":z("cancel",d);z("close",d);break;case "iframe":case "object":case "embed":z("load",d);break;case "video":case "audio":for(a=0;a<pc.length;a++)z(pc[a],d);break;case "source":z("error",d);break;case "img":case "image":case "link":z("error",d);
z("load",d);break;case "details":z("toggle",d);break;case "input":Gf(d,f);z("invalid",d);break;case "select":d._wrapperState={wasMultiple:!!f.multiple};z("invalid",d);break;case "textarea":Jf(d,f),z("invalid",d)}Sd(c,f);a=null;for(var g in f)f.hasOwnProperty(g)&&(e=f[g],"children"===g?"string"===typeof e?d.textContent!==e&&(a=["children",e]):"number"===typeof e&&d.textContent!==""+e&&(a=["children",""+e]):Ib.hasOwnProperty(g)&&null!=e&&"onScroll"===g&&z("scroll",d));switch(c){case "input":Fc(d);If(d,
f,!0);break;case "textarea":Fc(d);Lf(d);break;case "select":case "option":break;default:"function"===typeof f.onClick&&(d.onclick=Vc)}d=a;b.updateQueue=d;null!==d&&(b.flags|=4)}else{g=9===e.nodeType?e:e.ownerDocument;"http://www.w3.org/1999/xhtml"===a&&(a=Mf(c));"http://www.w3.org/1999/xhtml"===a?"script"===c?(a=g.createElement("div"),a.innerHTML="<script>\x3c/script>",a=a.removeChild(a.firstChild)):"string"===typeof d.is?a=g.createElement(c,{is:d.is}):(a=g.createElement(c),"select"===c&&(g=a,d.multiple?
g.multiple=!0:d.size&&(g.size=d.size))):a=g.createElementNS(a,c);a[Fa]=b;a[Wc]=d;xj(a,b,!1,!1);b.stateNode=a;g=Td(c,d);switch(c){case "dialog":z("cancel",a);z("close",a);e=d;break;case "iframe":case "object":case "embed":z("load",a);e=d;break;case "video":case "audio":for(e=0;e<pc.length;e++)z(pc[e],a);e=d;break;case "source":z("error",a);e=d;break;case "img":case "image":case "link":z("error",a);z("load",a);e=d;break;case "details":z("toggle",a);e=d;break;case "input":Gf(a,d);e=Md(a,d);z("invalid",
a);break;case "option":e=Pd(a,d);break;case "select":a._wrapperState={wasMultiple:!!d.multiple};e=B({},d,{value:void 0});z("invalid",a);break;case "textarea":Jf(a,d);e=Qd(a,d);z("invalid",a);break;default:e=d}Sd(c,e);var h=e;for(f in h)if(h.hasOwnProperty(f)){var k=h[f];"style"===f?Of(a,k):"dangerouslySetInnerHTML"===f?(k=k?k.__html:void 0,null!=k&&Lh(a,k)):"children"===f?"string"===typeof k?("textarea"!==c||""!==k)&&qc(a,k):"number"===typeof k&&qc(a,""+k):"suppressContentEditableWarning"!==f&&"suppressHydrationWarning"!==
f&&"autoFocus"!==f&&(Ib.hasOwnProperty(f)?null!=k&&"onScroll"===f&&z("scroll",a):null!=k&&Ed(a,f,k,g))}switch(c){case "input":Fc(a);If(a,d,!1);break;case "textarea":Fc(a);Lf(a);break;case "option":null!=d.value&&a.setAttribute("value",""+xa(d.value));break;case "select":a.multiple=!!d.multiple;f=d.value;null!=f?ib(a,!!d.multiple,f,!1):null!=d.defaultValue&&ib(a,!!d.multiple,d.defaultValue,!0);break;default:"function"===typeof e.onClick&&(a.onclick=Vc)}Pg(c,d)&&(b.flags|=4)}null!==b.ref&&(b.flags|=
128)}return null;case 6:if(a&&null!=b.stateNode)yj(a,b,a.memoizedProps,d);else{if("string"!==typeof d&&null===b.stateNode)throw Error(m(166));c=$a(gc.current);$a(ka.current);id(b)?(d=b.stateNode,c=b.memoizedProps,d[Fa]=b,d.nodeValue!==c&&(b.flags|=4)):(d=(9===c.nodeType?c:c.ownerDocument).createTextNode(d),d[Fa]=b,b.stateNode=d)}return null;case 13:t(E);d=b.memoizedState;if(0!==(b.flags&64))return b.lanes=c,b;d=null!==d;c=!1;null===a?void 0!==b.memoizedProps.fallback&&id(b):c=null!==a.memoizedState;
if(d&&!c&&0!==(b.mode&2))if(null===a&&!0!==b.memoizedProps.unstable_avoidThisFallback||0!==(E.current&1))0===L&&(L=3);else{if(0===L||3===L)L=4;null===R||0===(La&134217727)&&0===(Cb&134217727)||Db(R,O)}if(d||c)b.flags|=4;return null;case 4:return Ab(),Kh(b),null===a&&Cg(b.stateNode.containerInfo),null;case 10:return Ae(b),null;case 17:return S(b.type)&&(t(J),t(D)),null;case 19:t(E);d=b.memoizedState;if(null===d)return null;f=0!==(b.flags&64);g=d.rendering;if(null===g)if(f)oc(d,!1);else{if(0!==L||null!==
a&&0!==(a.flags&64))for(a=b.child;null!==a;){g=hd(a);if(null!==g){b.flags|=64;oc(d,!1);f=g.updateQueue;null!==f&&(b.updateQueue=f,b.flags|=4);null===d.lastEffect&&(b.firstEffect=null);b.lastEffect=d.lastEffect;d=c;for(c=b.child;null!==c;)f=c,a=d,f.flags&=2,f.nextEffect=null,f.firstEffect=null,f.lastEffect=null,g=f.alternate,null===g?(f.childLanes=0,f.lanes=a,f.child=null,f.memoizedProps=null,f.memoizedState=null,f.updateQueue=null,f.dependencies=null,f.stateNode=null):(f.childLanes=g.childLanes,f.lanes=
g.lanes,f.child=g.child,f.memoizedProps=g.memoizedProps,f.memoizedState=g.memoizedState,f.updateQueue=g.updateQueue,f.type=g.type,a=g.dependencies,f.dependencies=null===a?null:{lanes:a.lanes,firstContext:a.firstContext}),c=c.sibling;A(E,E.current&1|2);return b.child}a=a.sibling}null!==d.tail&&P()>We&&(b.flags|=64,f=!0,oc(d,!1),b.lanes=33554432)}else{if(!f)if(a=hd(g),null!==a){if(b.flags|=64,f=!0,c=a.updateQueue,null!==c&&(b.updateQueue=c,b.flags|=4),oc(d,!0),null===d.tail&&"hidden"===d.tailMode&&
!g.alternate&&!la)return b=b.lastEffect=d.lastEffect,null!==b&&(b.nextEffect=null),null}else 2*P()-d.renderingStartTime>We&&1073741824!==c&&(b.flags|=64,f=!0,oc(d,!1),b.lanes=33554432);d.isBackwards?(g.sibling=b.child,b.child=g):(c=d.last,null!==c?c.sibling=g:b.child=g,d.last=g)}return null!==d.tail?(c=d.tail,d.rendering=c,d.tail=c.sibling,d.lastEffect=b.lastEffect,d.renderingStartTime=P(),c.sibling=null,b=E.current,A(E,f?b&1|2:b&1),c):null;case 23:case 24:return ta=cb.current,t(cb),null!==a&&null!==
a.memoizedState!==(null!==b.memoizedState)&&"unstable-defer-without-hiding"!==d.mode&&(b.flags|=4),null}throw Error(m(156,b.tag));}function zj(a,b){switch(a.tag){case 1:return S(a.type)&&(t(J),t(D)),b=a.flags,b&4096?(a.flags=b&-4097|64,a):null;case 3:Ab();t(J);t(D);Je();b=a.flags;if(0!==(b&64))throw Error(m(285));a.flags=b&-4097|64;return a;case 5:return Ge(a),null;case 13:return t(E),b=a.flags,b&4096?(a.flags=b&-4097|64,a):null;case 19:return t(E),null;case 4:return Ab(),null;case 10:return Ae(a),
null;case 23:case 24:return ta=cb.current,t(cb),null;default:return null}}function Xe(a,b){try{var c="",d=b;do c+=pi(d),d=d.return;while(d);var e=c}catch(f){e="\nError generating stack: "+f.message+"\n"+f.stack}return{value:a,source:b,stack:e}}function Ye(a,b){try{console.error(b.value)}catch(c){setTimeout(function(){throw c;})}}function Mh(a,b,c){c=Ia(-1,c);c.tag=3;c.payload={element:null};var d=b.value;c.callback=function(){rd||(rd=!0,Ze=d);Ye(a,b)};return c}function Nh(a,b,c){c=Ia(-1,c);c.tag=
3;var d=a.type.getDerivedStateFromError;if("function"===typeof d){var e=b.value;c.payload=function(){Ye(a,b);return d(e)}}var f=a.stateNode;null!==f&&"function"===typeof f.componentDidCatch&&(c.callback=function(){"function"!==typeof d&&(null===na?na=new Set([this]):na.add(this),Ye(a,b));var c=b.stack;this.componentDidCatch(b.value,{componentStack:null!==c?c:""})});return c}function Oh(a){var b=a.ref;if(null!==b)if("function"===typeof b)try{b(null)}catch(c){Qa(a,c)}else b.current=null}function Aj(a,
b){switch(b.tag){case 0:case 11:case 15:case 22:return;case 1:if(b.flags&256&&null!==a){var c=a.memoizedProps,d=a.memoizedState;a=b.stateNode;b=a.getSnapshotBeforeUpdate(b.elementType===b.type?c:ea(b.type,c),d);a.__reactInternalSnapshotBeforeUpdate=b}return;case 3:b.flags&256&&te(b.stateNode.containerInfo);return;case 5:case 6:case 4:case 17:return}throw Error(m(163));}function Bj(a,b,c,d){switch(c.tag){case 0:case 11:case 15:case 22:b=c.updateQueue;b=null!==b?b.lastEffect:null;if(null!==b){a=b=b.next;
do 3===(a.tag&3)&&(d=a.create,a.destroy=d()),a=a.next;while(a!==b)}b=c.updateQueue;b=null!==b?b.lastEffect:null;if(null!==b){a=b=b.next;do{var e=a;d=e.next;e=e.tag;0!==(e&4)&&0!==(e&1)&&(Ph(c,a),Cj(c,a));a=d}while(a!==b)}return;case 1:a=c.stateNode;c.flags&4&&(null===b?a.componentDidMount():(d=c.elementType===c.type?b.memoizedProps:ea(c.type,b.memoizedProps),a.componentDidUpdate(d,b.memoizedState,a.__reactInternalSnapshotBeforeUpdate)));b=c.updateQueue;null!==b&&dh(c,b,a);return;case 3:b=c.updateQueue;
if(null!==b){a=null;if(null!==c.child)switch(c.child.tag){case 5:a=c.child.stateNode;break;case 1:a=c.child.stateNode}dh(c,b,a)}return;case 5:a=c.stateNode;null===b&&c.flags&4&&Pg(c.type,c.memoizedProps)&&a.focus();return;case 6:return;case 4:return;case 12:return;case 13:null===c.memoizedState&&(c=c.alternate,null!==c&&(c=c.memoizedState,null!==c&&(c=c.dehydrated,null!==c&&bg(c))));return;case 19:case 17:case 20:case 21:case 23:case 24:return}throw Error(m(163));}function Qh(a,b){for(var c=a;;){if(5===
c.tag){var d=c.stateNode;if(b)d=d.style,"function"===typeof d.setProperty?d.setProperty("display","none","important"):d.display="none";else{d=c.stateNode;var e=c.memoizedProps.style;e=void 0!==e&&null!==e&&e.hasOwnProperty("display")?e.display:null;d.style.display=Nf("display",e)}}else if(6===c.tag)c.stateNode.nodeValue=b?"":c.memoizedProps;else if((23!==c.tag&&24!==c.tag||null===c.memoizedState||c===a)&&null!==c.child){c.child.return=c;c=c.child;continue}if(c===a)break;for(;null===c.sibling;){if(null===
c.return||c.return===a)return;c=c.return}c.sibling.return=c.return;c=c.sibling}}function Rh(a,b,c){if(db&&"function"===typeof db.onCommitFiberUnmount)try{db.onCommitFiberUnmount($e,b)}catch(f){}switch(b.tag){case 0:case 11:case 14:case 15:case 22:a=b.updateQueue;if(null!==a&&(a=a.lastEffect,null!==a)){c=a=a.next;do{var d=c,e=d.destroy;d=d.tag;if(void 0!==e)if(0!==(d&4))Ph(b,c);else{d=b;try{e()}catch(f){Qa(d,f)}}c=c.next}while(c!==a)}break;case 1:Oh(b);a=b.stateNode;if("function"===typeof a.componentWillUnmount)try{a.props=
b.memoizedProps,a.state=b.memoizedState,a.componentWillUnmount()}catch(f){Qa(b,f)}break;case 5:Oh(b);break;case 4:Sh(a,b)}}function Th(a){a.alternate=null;a.child=null;a.dependencies=null;a.firstEffect=null;a.lastEffect=null;a.memoizedProps=null;a.memoizedState=null;a.pendingProps=null;a.return=null;a.updateQueue=null}function Uh(a){return 5===a.tag||3===a.tag||4===a.tag}function Vh(a){a:{for(var b=a.return;null!==b;){if(Uh(b))break a;b=b.return}throw Error(m(160));}var c=b;b=c.stateNode;switch(c.tag){case 5:var d=
!1;break;case 3:b=b.containerInfo;d=!0;break;case 4:b=b.containerInfo;d=!0;break;default:throw Error(m(161));}c.flags&16&&(qc(b,""),c.flags&=-17);a:b:for(c=a;;){for(;null===c.sibling;){if(null===c.return||Uh(c.return)){c=null;break a}c=c.return}c.sibling.return=c.return;for(c=c.sibling;5!==c.tag&&6!==c.tag&&18!==c.tag;){if(c.flags&2)continue b;if(null===c.child||4===c.tag)continue b;else c.child.return=c,c=c.child}if(!(c.flags&2)){c=c.stateNode;break a}}d?af(a,c,b):bf(a,c,b)}function af(a,b,c){var d=
a.tag,e=5===d||6===d;if(e)a=e?a.stateNode:a.stateNode.instance,b?8===c.nodeType?c.parentNode.insertBefore(a,b):c.insertBefore(a,b):(8===c.nodeType?(b=c.parentNode,b.insertBefore(a,c)):(b=c,b.appendChild(a)),c=c._reactRootContainer,null!==c&&void 0!==c||null!==b.onclick||(b.onclick=Vc));else if(4!==d&&(a=a.child,null!==a))for(af(a,b,c),a=a.sibling;null!==a;)af(a,b,c),a=a.sibling}function bf(a,b,c){var d=a.tag,e=5===d||6===d;if(e)a=e?a.stateNode:a.stateNode.instance,b?c.insertBefore(a,b):c.appendChild(a);
else if(4!==d&&(a=a.child,null!==a))for(bf(a,b,c),a=a.sibling;null!==a;)bf(a,b,c),a=a.sibling}function Sh(a,b,c){c=b;for(var d=!1,e,f;;){if(!d){e=c.return;a:for(;;){if(null===e)throw Error(m(160));f=e.stateNode;switch(e.tag){case 5:e=f;f=!1;break a;case 3:e=f.containerInfo;f=!0;break a;case 4:e=f.containerInfo;f=!0;break a}e=e.return}d=!0}if(5===c.tag||6===c.tag){a:for(var g=a,h=c,k=h;;)if(Rh(g,k),null!==k.child&&4!==k.tag)k.child.return=k,k=k.child;else{if(k===h)break a;for(;null===k.sibling;){if(null===
k.return||k.return===h)break a;k=k.return}k.sibling.return=k.return;k=k.sibling}f?(g=e,h=c.stateNode,8===g.nodeType?g.parentNode.removeChild(h):g.removeChild(h)):e.removeChild(c.stateNode)}else if(4===c.tag){if(null!==c.child){e=c.stateNode.containerInfo;f=!0;c.child.return=c;c=c.child;continue}}else if(Rh(a,c),null!==c.child){c.child.return=c;c=c.child;continue}if(c===b)break;for(;null===c.sibling;){if(null===c.return||c.return===b)return;c=c.return;4===c.tag&&(d=!1)}c.sibling.return=c.return;c=
c.sibling}}function cf(a,b){switch(b.tag){case 0:case 11:case 14:case 15:case 22:var c=b.updateQueue;c=null!==c?c.lastEffect:null;if(null!==c){var d=c=c.next;do 3===(d.tag&3)&&(a=d.destroy,d.destroy=void 0,void 0!==a&&a()),d=d.next;while(d!==c)}return;case 1:return;case 5:c=b.stateNode;if(null!=c){d=b.memoizedProps;var e=null!==a?a.memoizedProps:d;a=b.type;var f=b.updateQueue;b.updateQueue=null;if(null!==f){c[Wc]=d;"input"===a&&"radio"===d.type&&null!=d.name&&Hf(c,d);Td(a,e);b=Td(a,d);for(e=0;e<f.length;e+=
2){var g=f[e],h=f[e+1];"style"===g?Of(c,h):"dangerouslySetInnerHTML"===g?Lh(c,h):"children"===g?qc(c,h):Ed(c,g,h,b)}switch(a){case "input":Nd(c,d);break;case "textarea":Kf(c,d);break;case "select":a=c._wrapperState.wasMultiple,c._wrapperState.wasMultiple=!!d.multiple,f=d.value,null!=f?ib(c,!!d.multiple,f,!1):a!==!!d.multiple&&(null!=d.defaultValue?ib(c,!!d.multiple,d.defaultValue,!0):ib(c,!!d.multiple,d.multiple?[]:"",!1))}}}return;case 6:if(null===b.stateNode)throw Error(m(162));b.stateNode.nodeValue=
b.memoizedProps;return;case 3:c=b.stateNode;c.hydrate&&(c.hydrate=!1,bg(c.containerInfo));return;case 12:return;case 13:null!==b.memoizedState&&(df=P(),Qh(b.child,!0));Wh(b);return;case 19:Wh(b);return;case 17:return;case 23:case 24:Qh(b,null!==b.memoizedState);return}throw Error(m(163));}function Wh(a){var b=a.updateQueue;if(null!==b){a.updateQueue=null;var c=a.stateNode;null===c&&(c=a.stateNode=new Dj);b.forEach(function(b){var d=Ej.bind(null,a,b);c.has(b)||(c.add(b),b.then(d,d))})}}function Fj(a,
b){return null!==a&&(a=a.memoizedState,null===a||null!==a.dehydrated)?(b=b.memoizedState,null!==b&&null===b.dehydrated):!1}function Eb(){We=P()+500}function W(){return 0!==(n&48)?P():-1!==sd?sd:sd=P()}function Oa(a){a=a.mode;if(0===(a&2))return 1;if(0===(a&4))return 99===wb()?1:2;0===ua&&(ua=Fb);if(0!==Gj.transition){0!==td&&(td=null!==ef?ef.pendingLanes:0);a=ua;var b=4186112&~td;b&=-b;0===b&&(a=4186112&~a,b=a&-a,0===b&&(b=8192));return b}a=wb();0!==(n&4)&&98===a?a=Nc(12,ua):(a=Fi(a),a=Nc(a,ua));
return a}function Pa(a,b,c){if(50<rc)throw rc=0,ff=null,Error(m(185));a=ud(a,b);if(null===a)return null;Oc(a,b,c);a===R&&(Cb|=b,4===L&&Db(a,O));var d=wb();1===b?0!==(n&8)&&0===(n&48)?gf(a):(ba(a,c),0===n&&(Eb(),ja())):(0===(n&4)||98!==d&&99!==d||(null===va?va=new Set([a]):va.add(a)),ba(a,c));ef=a}function ud(a,b){a.lanes|=b;var c=a.alternate;null!==c&&(c.lanes|=b);c=a;for(a=a.return;null!==a;)a.childLanes|=b,c=a.alternate,null!==c&&(c.childLanes|=b),c=a,a=a.return;return 3===c.tag?c.stateNode:null}
function ba(a,b){for(var c=a.callbackNode,d=a.suspendedLanes,e=a.pingedLanes,f=a.expirationTimes,g=a.pendingLanes;0<g;){var h=31-Ba(g),k=1<<h,l=f[h];if(-1===l){if(0===(k&d)||0!==(k&e)){l=b;mb(k);var m=w;f[h]=10<=m?l+250:6<=m?l+5E3:-1}}else l<=b&&(a.expiredLanes|=k);g&=~k}d=Wb(a,a===R?O:0);b=w;if(0===d)null!==c&&(c!==hf&&xe(c),a.callbackNode=null,a.callbackPriority=0);else{if(null!==c){if(a.callbackPriority===b)return;c!==hf&&xe(c)}15===b?(c=gf.bind(null,a),null===pa?(pa=[c],Zc=we(Yc,$g)):pa.push(c),
c=hf):14===b?c=bc(99,gf.bind(null,a)):(c=Gi(b),c=bc(c,Xh.bind(null,a)));a.callbackPriority=b;a.callbackNode=c}}function Xh(a){sd=-1;td=ua=0;if(0!==(n&48))throw Error(m(327));var b=a.callbackNode;if(Ra()&&a.callbackNode!==b)return null;var c=Wb(a,a===R?O:0);if(0===c)return null;var d=c;var e=n;n|=16;var f=Yh();if(R!==a||O!==d)Eb(),Gb(a,d);do try{Hj();break}catch(h){Zh(a,h)}while(1);ze();vd.current=f;n=e;null!==G?d=0:(R=null,O=0,d=L);if(0!==(Fb&Cb))Gb(a,0);else if(0!==d){2===d&&(n|=64,a.hydrate&&(a.hydrate=
!1,te(a.containerInfo)),c=eg(a),0!==c&&(d=sc(a,c)));if(1===d)throw b=wd,Gb(a,0),Db(a,c),ba(a,P()),b;a.finishedWork=a.current.alternate;a.finishedLanes=c;switch(d){case 0:case 1:throw Error(m(345));case 2:eb(a);break;case 3:Db(a,c);if((c&62914560)===c&&(d=df+500-P(),10<d)){if(0!==Wb(a,0))break;e=a.suspendedLanes;if((e&c)!==c){W();a.pingedLanes|=a.suspendedLanes&e;break}a.timeoutHandle=$h(eb.bind(null,a),d);break}eb(a);break;case 4:Db(a,c);if((c&4186112)===c)break;d=a.eventTimes;for(e=-1;0<c;){var g=
31-Ba(c);f=1<<g;g=d[g];g>e&&(e=g);c&=~f}c=e;c=P()-c;c=(120>c?120:480>c?480:1080>c?1080:1920>c?1920:3E3>c?3E3:4320>c?4320:1960*Ij(c/1960))-c;if(10<c){a.timeoutHandle=$h(eb.bind(null,a),c);break}eb(a);break;case 5:eb(a);break;default:throw Error(m(329));}}ba(a,P());return a.callbackNode===b?Xh.bind(null,a):null}function Db(a,b){b&=~jf;b&=~Cb;a.suspendedLanes|=b;a.pingedLanes&=~b;for(a=a.expirationTimes;0<b;){var c=31-Ba(b),d=1<<c;a[c]=-1;b&=~d}}function gf(a){if(0!==(n&48))throw Error(m(327));Ra();
if(a===R&&0!==(a.expiredLanes&O)){var b=O;var c=sc(a,b);0!==(Fb&Cb)&&(b=Wb(a,b),c=sc(a,b))}else b=Wb(a,0),c=sc(a,b);0!==a.tag&&2===c&&(n|=64,a.hydrate&&(a.hydrate=!1,te(a.containerInfo)),b=eg(a),0!==b&&(c=sc(a,b)));if(1===c)throw c=wd,Gb(a,0),Db(a,b),ba(a,P()),c;a.finishedWork=a.current.alternate;a.finishedLanes=b;eb(a);ba(a,P());return null}function Jj(){if(null!==va){var a=va;va=null;a.forEach(function(a){a.expiredLanes|=24&a.pendingLanes;ba(a,P())})}ja()}function ai(a,b){var c=n;n|=1;try{return a(b)}finally{n=
c,0===n&&(Eb(),ja())}}function bi(a,b){var c=n;n&=-2;n|=8;try{return a(b)}finally{n=c,0===n&&(Eb(),ja())}}function pd(a,b){A(cb,ta);ta|=b;Fb|=b}function Gb(a,b){a.finishedWork=null;a.finishedLanes=0;var c=a.timeoutHandle;-1!==c&&(a.timeoutHandle=-1,Kj(c));if(null!==G)for(c=G.return;null!==c;){var d=c;switch(d.tag){case 1:d=d.type.childContextTypes;null!==d&&void 0!==d&&(t(J),t(D));break;case 3:Ab();t(J);t(D);Je();break;case 5:Ge(d);break;case 4:Ab();break;case 13:t(E);break;case 19:t(E);break;case 10:Ae(d);
break;case 23:case 24:ta=cb.current,t(cb)}c=c.return}R=a;G=Ma(a.current,null);O=ta=Fb=b;L=0;wd=null;jf=Cb=La=0}function Zh(a,b){do{var c=G;try{ze();jc.current=jd;if(kd){for(var d=y.memoizedState;null!==d;){var e=d.queue;null!==e&&(e.pending=null);d=d.next}kd=!1}ic=0;K=N=y=null;kc=!1;kf.current=null;if(null===c||null===c.return){L=1;wd=b;G=null;break}a:{var f=a,g=c.return,h=c,k=b;b=O;h.flags|=2048;h.firstEffect=h.lastEffect=null;if(null!==k&&"object"===typeof k&&"function"===typeof k.then){var l=k;
if(0===(h.mode&2)){var m=h.alternate;m?(h.updateQueue=m.updateQueue,h.memoizedState=m.memoizedState,h.lanes=m.lanes):(h.updateQueue=null,h.memoizedState=null)}var n=0!==(E.current&1),r=g;do{var t;if(t=13===r.tag){var w=r.memoizedState;if(null!==w)t=null!==w.dehydrated?!0:!1;else{var z=r.memoizedProps;t=void 0===z.fallback?!1:!0!==z.unstable_avoidThisFallback?!0:n?!1:!0}}if(t){var C=r.updateQueue;if(null===C){var x=new Set;x.add(l);r.updateQueue=x}else C.add(l);if(0===(r.mode&2)){r.flags|=64;h.flags|=
16384;h.flags&=-2981;if(1===h.tag)if(null===h.alternate)h.tag=17;else{var p=Ia(-1,1);p.tag=2;Ja(h,p)}h.lanes|=1;break a}k=void 0;h=b;var q=f.pingCache;null===q?(q=f.pingCache=new Lj,k=new Set,q.set(l,k)):(k=q.get(l),void 0===k&&(k=new Set,q.set(l,k)));if(!k.has(h)){k.add(h);var u=Mj.bind(null,f,l,h);l.then(u,u)}r.flags|=4096;r.lanes=b;break a}r=r.return}while(null!==r);k=Error((hb(h.type)||"A React component")+" suspended while rendering, but no fallback UI was specified.\n\nAdd a <Suspense fallback=...> component higher in the tree to provide a loading indicator or placeholder to display.")}5!==
L&&(L=2);k=Xe(k,h);r=g;do{switch(r.tag){case 3:f=k;r.flags|=4096;b&=-b;r.lanes|=b;var B=Mh(r,f,b);ch(r,B);break a;case 1:f=k;var A=r.type,D=r.stateNode;if(0===(r.flags&64)&&("function"===typeof A.getDerivedStateFromError||null!==D&&"function"===typeof D.componentDidCatch&&(null===na||!na.has(D)))){r.flags|=4096;b&=-b;r.lanes|=b;var F=Nh(r,f,b);ch(r,F);break a}}r=r.return}while(null!==r)}ci(c)}catch(qa){b=qa;G===c&&null!==c&&(G=c=c.return);continue}break}while(1)}function Yh(){var a=vd.current;vd.current=
jd;return null===a?jd:a}function sc(a,b){var c=n;n|=16;var d=Yh();R===a&&O===b||Gb(a,b);do try{Nj();break}catch(e){Zh(a,e)}while(1);ze();n=c;vd.current=d;if(null!==G)throw Error(m(261));R=null;O=0;return L}function Nj(){for(;null!==G;)di(G)}function Hj(){for(;null!==G&&!Oj();)di(G)}function di(a){var b=Pj(a.alternate,a,ta);a.memoizedProps=a.pendingProps;null===b?ci(a):G=b;kf.current=null}function ci(a){var b=a;do{var c=b.alternate;a=b.return;if(0===(b.flags&2048)){c=vj(c,b,ta);if(null!==c){G=c;return}c=
b;if(24!==c.tag&&23!==c.tag||null===c.memoizedState||0!==(ta&1073741824)||0===(c.mode&4)){for(var d=0,e=c.child;null!==e;)d|=e.lanes|e.childLanes,e=e.sibling;c.childLanes=d}null!==a&&0===(a.flags&2048)&&(null===a.firstEffect&&(a.firstEffect=b.firstEffect),null!==b.lastEffect&&(null!==a.lastEffect&&(a.lastEffect.nextEffect=b.firstEffect),a.lastEffect=b.lastEffect),1<b.flags&&(null!==a.lastEffect?a.lastEffect.nextEffect=b:a.firstEffect=b,a.lastEffect=b))}else{c=zj(b);if(null!==c){c.flags&=2047;G=c;
return}null!==a&&(a.firstEffect=a.lastEffect=null,a.flags|=2048)}b=b.sibling;if(null!==b){G=b;return}G=b=a}while(null!==b);0===L&&(L=5)}function eb(a){var b=wb();Za(99,Qj.bind(null,a,b));return null}function Qj(a,b){do Ra();while(null!==tc);if(0!==(n&48))throw Error(m(327));var c=a.finishedWork;if(null===c)return null;a.finishedWork=null;a.finishedLanes=0;if(c===a.current)throw Error(m(177));a.callbackNode=null;var d=c.lanes|c.childLanes,e=d,f=a.pendingLanes&~e;a.pendingLanes=e;a.suspendedLanes=0;
a.pingedLanes=0;a.expiredLanes&=e;a.mutableReadLanes&=e;a.entangledLanes&=e;e=a.entanglements;for(var g=a.eventTimes,h=a.expirationTimes;0<f;){var k=31-Ba(f),v=1<<k;e[k]=0;g[k]=-1;h[k]=-1;f&=~v}null!==va&&0===(d&24)&&va.has(a)&&va.delete(a);a===R&&(G=R=null,O=0);1<c.flags?null!==c.lastEffect?(c.lastEffect.nextEffect=c,d=c.firstEffect):d=c:d=c.firstEffect;if(null!==d){e=n;n|=32;kf.current=null;lf=Pc;g=xg();if(ne(g)){if("selectionStart"in g)h={start:g.selectionStart,end:g.selectionEnd};else a:if(h=
(h=g.ownerDocument)&&h.defaultView||window,(v=h.getSelection&&h.getSelection())&&0!==v.rangeCount){h=v.anchorNode;f=v.anchorOffset;k=v.focusNode;v=v.focusOffset;try{h.nodeType,k.nodeType}catch(qa){h=null;break a}var t=0,w=-1,r=-1,z=0,B=0,y=g,C=null;b:for(;;){for(var x;;){y!==h||0!==f&&3!==y.nodeType||(w=t+f);y!==k||0!==v&&3!==y.nodeType||(r=t+v);3===y.nodeType&&(t+=y.nodeValue.length);if(null===(x=y.firstChild))break;C=y;y=x}for(;;){if(y===g)break b;C===h&&++z===f&&(w=t);C===k&&++B===v&&(r=t);if(null!==
(x=y.nextSibling))break;y=C;C=y.parentNode}y=x}h=-1===w||-1===r?null:{start:w,end:r}}else h=null;h=h||{start:0,end:0}}else h=null;mf={focusedElem:g,selectionRange:h};Pc=!1;uc=null;xd=!1;l=d;do try{Rj()}catch(qa){if(null===l)throw Error(m(330));Qa(l,qa);l=l.nextEffect}while(null!==l);uc=null;l=d;do try{for(g=a;null!==l;){var p=l.flags;p&16&&qc(l.stateNode,"");if(p&128){var q=l.alternate;if(null!==q){var u=q.ref;null!==u&&("function"===typeof u?u(null):u.current=null)}}switch(p&1038){case 2:Vh(l);l.flags&=
-3;break;case 6:Vh(l);l.flags&=-3;cf(l.alternate,l);break;case 1024:l.flags&=-1025;break;case 1028:l.flags&=-1025;cf(l.alternate,l);break;case 4:cf(l.alternate,l);break;case 8:h=l;Sh(g,h);var A=h.alternate;Th(h);null!==A&&Th(A)}l=l.nextEffect}}catch(qa){if(null===l)throw Error(m(330));Qa(l,qa);l=l.nextEffect}while(null!==l);u=mf;q=xg();p=u.focusedElem;g=u.selectionRange;if(q!==p&&p&&p.ownerDocument&&wg(p.ownerDocument.documentElement,p)){null!==g&&ne(p)&&(q=g.start,u=g.end,void 0===u&&(u=q),"selectionStart"in
p?(p.selectionStart=q,p.selectionEnd=Math.min(u,p.value.length)):(u=(q=p.ownerDocument||document)&&q.defaultView||window,u.getSelection&&(u=u.getSelection(),h=p.textContent.length,A=Math.min(g.start,h),g=void 0===g.end?A:Math.min(g.end,h),!u.extend&&A>g&&(h=g,g=A,A=h),h=vg(p,A),f=vg(p,g),h&&f&&(1!==u.rangeCount||u.anchorNode!==h.node||u.anchorOffset!==h.offset||u.focusNode!==f.node||u.focusOffset!==f.offset)&&(q=q.createRange(),q.setStart(h.node,h.offset),u.removeAllRanges(),A>g?(u.addRange(q),u.extend(f.node,
f.offset)):(q.setEnd(f.node,f.offset),u.addRange(q))))));q=[];for(u=p;u=u.parentNode;)1===u.nodeType&&q.push({element:u,left:u.scrollLeft,top:u.scrollTop});"function"===typeof p.focus&&p.focus();for(p=0;p<q.length;p++)u=q[p],u.element.scrollLeft=u.left,u.element.scrollTop=u.top}Pc=!!lf;mf=lf=null;a.current=c;l=d;do try{for(p=a;null!==l;){var D=l.flags;D&36&&Bj(p,l.alternate,l);if(D&128){q=void 0;var E=l.ref;if(null!==E){var F=l.stateNode;switch(l.tag){case 5:q=F;break;default:q=F}"function"===typeof E?
E(q):E.current=q}}l=l.nextEffect}}catch(qa){if(null===l)throw Error(m(330));Qa(l,qa);l=l.nextEffect}while(null!==l);l=null;Sj();n=e}else a.current=c;if(Sa)Sa=!1,tc=a,vc=b;else for(l=d;null!==l;)b=l.nextEffect,l.nextEffect=null,l.flags&8&&(D=l,D.sibling=null,D.stateNode=null),l=b;d=a.pendingLanes;0===d&&(na=null);1===d?a===ff?rc++:(rc=0,ff=a):rc=0;c=c.stateNode;if(db&&"function"===typeof db.onCommitFiberRoot)try{db.onCommitFiberRoot($e,c,void 0,64===(c.current.flags&64))}catch(qa){}ba(a,P());if(rd)throw rd=
!1,a=Ze,Ze=null,a;if(0!==(n&8))return null;ja();return null}function Rj(){for(;null!==l;){var a=l.alternate;xd||null===uc||(0!==(l.flags&8)?Wf(l,uc)&&(xd=!0):13===l.tag&&Fj(a,l)&&Wf(l,uc)&&(xd=!0));var b=l.flags;0!==(b&256)&&Aj(a,l);0===(b&512)||Sa||(Sa=!0,bc(97,function(){Ra();return null}));l=l.nextEffect}}function Ra(){if(90!==vc){var a=97<vc?97:vc;vc=90;return Za(a,Tj)}return!1}function Cj(a,b){nf.push(b,a);Sa||(Sa=!0,bc(97,function(){Ra();return null}))}function Ph(a,b){of.push(b,a);Sa||(Sa=
!0,bc(97,function(){Ra();return null}))}function Tj(){if(null===tc)return!1;var a=tc;tc=null;if(0!==(n&48))throw Error(m(331));var b=n;n|=32;var c=of;of=[];for(var d=0;d<c.length;d+=2){var e=c[d],f=c[d+1],g=e.destroy;e.destroy=void 0;if("function"===typeof g)try{g()}catch(k){if(null===f)throw Error(m(330));Qa(f,k)}}c=nf;nf=[];for(d=0;d<c.length;d+=2){e=c[d];f=c[d+1];try{var h=e.create;e.destroy=h()}catch(k){if(null===f)throw Error(m(330));Qa(f,k)}}for(h=a.current.firstEffect;null!==h;)a=h.nextEffect,
h.nextEffect=null,h.flags&8&&(h.sibling=null,h.stateNode=null),h=a;n=b;ja();return!0}function ei(a,b,c){b=Xe(c,b);b=Mh(a,b,1);Ja(a,b);b=W();a=ud(a,1);null!==a&&(Oc(a,1,b),ba(a,b))}function Qa(a,b){if(3===a.tag)ei(a,a,b);else for(var c=a.return;null!==c;){if(3===c.tag){ei(c,a,b);break}else if(1===c.tag){var d=c.stateNode;if("function"===typeof c.type.getDerivedStateFromError||"function"===typeof d.componentDidCatch&&(null===na||!na.has(d))){a=Xe(b,a);var e=Nh(c,a,1);Ja(c,e);e=W();c=ud(c,1);if(null!==
c)Oc(c,1,e),ba(c,e);else if("function"===typeof d.componentDidCatch&&(null===na||!na.has(d)))try{d.componentDidCatch(b,a)}catch(f){}break}}c=c.return}}function Mj(a,b,c){var d=a.pingCache;null!==d&&d.delete(b);b=W();a.pingedLanes|=a.suspendedLanes&c;R===a&&(O&c)===c&&(4===L||3===L&&(O&62914560)===O&&500>P()-df?Gb(a,0):jf|=c);ba(a,b)}function Ej(a,b){var c=a.stateNode;null!==c&&c.delete(b);b=0;0===b&&(b=a.mode,0===(b&2)?b=1:0===(b&4)?b=99===wb()?1:2:(0===ua&&(ua=Fb),b=nb(62914560&~ua),0===b&&(b=4194304)));
c=W();a=ud(a,b);null!==a&&(Oc(a,b,c),ba(a,c))}function Uj(a,b,c,d){this.tag=a;this.key=c;this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null;this.index=0;this.ref=null;this.pendingProps=b;this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null;this.mode=d;this.flags=0;this.lastEffect=this.firstEffect=this.nextEffect=null;this.childLanes=this.lanes=0;this.alternate=null}function Qe(a){a=a.prototype;return!(!a||!a.isReactComponent)}function Vj(a){if("function"===
typeof a)return Qe(a)?1:0;if(void 0!==a&&null!==a){a=a.$$typeof;if(a===Dc)return 11;if(a===Ec)return 14}return 2}function Ma(a,b){var c=a.alternate;null===c?(c=Z(a.tag,b,a.key,a.mode),c.elementType=a.elementType,c.type=a.type,c.stateNode=a.stateNode,c.alternate=a,a.alternate=c):(c.pendingProps=b,c.type=a.type,c.flags=0,c.nextEffect=null,c.firstEffect=null,c.lastEffect=null);c.childLanes=a.childLanes;c.lanes=a.lanes;c.child=a.child;c.memoizedProps=a.memoizedProps;c.memoizedState=a.memoizedState;c.updateQueue=
a.updateQueue;b=a.dependencies;c.dependencies=null===b?null:{lanes:b.lanes,firstContext:b.firstContext};c.sibling=a.sibling;c.index=a.index;c.ref=a.ref;return c}function fd(a,b,c,d,e,f){var g=2;d=a;if("function"===typeof a)Qe(a)&&(g=1);else if("string"===typeof a)g=5;else a:switch(a){case wa:return zb(c.children,e,f,b);case fi:g=8;e|=16;break;case Hd:g=8;e|=1;break;case Lb:return a=Z(12,c,b,e|8),a.elementType=Lb,a.type=Lb,a.lanes=f,a;case Mb:return a=Z(13,c,b,e),a.type=Mb,a.elementType=Mb,a.lanes=
f,a;case Cc:return a=Z(19,c,b,e),a.elementType=Cc,a.lanes=f,a;case pf:return Ue(c,e,f,b);case qf:return a=Z(24,c,b,e),a.elementType=qf,a.lanes=f,a;default:if("object"===typeof a&&null!==a)switch(a.$$typeof){case Jd:g=10;break a;case Id:g=9;break a;case Dc:g=11;break a;case Ec:g=14;break a;case Ld:g=16;d=null;break a;case Kd:g=22;break a}throw Error(m(130,null==a?a:typeof a,""));}b=Z(g,c,b,e);b.elementType=a;b.type=d;b.lanes=f;return b}function zb(a,b,c,d){a=Z(7,a,d,b);a.lanes=c;return a}function Ue(a,
b,c,d){a=Z(23,a,d,b);a.elementType=pf;a.lanes=c;return a}function De(a,b,c){a=Z(6,a,null,b);a.lanes=c;return a}function Ee(a,b,c){b=Z(4,null!==a.children?a.children:[],a.key,b);b.lanes=c;b.stateNode={containerInfo:a.containerInfo,pendingChildren:null,implementation:a.implementation};return b}function Wj(a,b,c){this.tag=b;this.containerInfo=a;this.finishedWork=this.pingCache=this.current=this.pendingChildren=null;this.timeoutHandle=-1;this.pendingContext=this.context=null;this.hydrate=c;this.callbackNode=
null;this.callbackPriority=0;this.eventTimes=ge(0);this.expirationTimes=ge(-1);this.entangledLanes=this.finishedLanes=this.mutableReadLanes=this.expiredLanes=this.pingedLanes=this.suspendedLanes=this.pendingLanes=0;this.entanglements=ge(0);this.mutableSourceEagerHydrationData=null}function Xj(a,b,c){var d=3<arguments.length&&void 0!==arguments[3]?arguments[3]:null;return{$$typeof:Ua,key:null==d?null:""+d,children:a,containerInfo:b,implementation:c}}function yd(a,b,c,d){var e=b.current,f=W(),g=Oa(e);
a:if(c){c=c._reactInternals;b:{if(Va(c)!==c||1!==c.tag)throw Error(m(170));var h=c;do{switch(h.tag){case 3:h=h.stateNode.context;break b;case 1:if(S(h.type)){h=h.stateNode.__reactInternalMemoizedMergedChildContext;break b}}h=h.return}while(null!==h);throw Error(m(171));}if(1===c.tag){var k=c.type;if(S(k)){c=Tg(c,k,h);break a}}c=h}else c=Ha;null===b.context?b.context=c:b.pendingContext=c;b=Ia(f,g);b.payload={element:a};d=void 0===d?null:d;null!==d&&(b.callback=d);Ja(e,b);Pa(e,g,f);return g}function rf(a){a=
a.current;if(!a.child)return null;switch(a.child.tag){case 5:return a.child.stateNode;default:return a.child.stateNode}}function gi(a,b){a=a.memoizedState;if(null!==a&&null!==a.dehydrated){var c=a.retryLane;a.retryLane=0!==c&&c<b?c:b}}function sf(a,b){gi(a,b);(a=a.alternate)&&gi(a,b)}function Yj(a){a=Vf(a);return null===a?null:a.stateNode}function Zj(a){return null}function tf(a,b,c){var d=null!=c&&null!=c.hydrationOptions&&c.hydrationOptions.mutableSources||null;c=new Wj(a,b,null!=c&&!0===c.hydrate);
b=Z(3,null,null,2===b?7:1===b?3:0);c.current=b;b.stateNode=c;Be(b);a[rb]=c.current;Cg(8===a.nodeType?a.parentNode:a);if(d)for(a=0;a<d.length;a++){b=d[a];var e=b._getVersion;e=e(b._source);null==c.mutableSourceEagerHydrationData?c.mutableSourceEagerHydrationData=[b,e]:c.mutableSourceEagerHydrationData.push(b,e)}this._internalRoot=c}function wc(a){return!(!a||1!==a.nodeType&&9!==a.nodeType&&11!==a.nodeType&&(8!==a.nodeType||" react-mount-point-unstable "!==a.nodeValue))}function ak(a,b){b||(b=a?9===
a.nodeType?a.documentElement:a.firstChild:null,b=!(!b||1!==b.nodeType||!b.hasAttribute("data-reactroot")));if(!b)for(var c;c=a.lastChild;)a.removeChild(c);return new tf(a,0,b?{hydrate:!0}:void 0)}function zd(a,b,c,d,e){var f=c._reactRootContainer;if(f){var g=f._internalRoot;if("function"===typeof e){var h=e;e=function(){var a=rf(g);h.call(a)}}yd(b,g,a,e)}else{f=c._reactRootContainer=ak(c,d);g=f._internalRoot;if("function"===typeof e){var k=e;e=function(){var a=rf(g);k.call(a)}}bi(function(){yd(b,
g,a,e)})}return rf(g)}function hi(a,b){var c=2<arguments.length&&void 0!==arguments[2]?arguments[2]:null;if(!wc(b))throw Error(m(200));return Xj(a,b,null,c)}if(!ha)throw Error(m(227));var zf=new Set,Ib={},oa=!("undefined"===typeof window||"undefined"===typeof window.document||"undefined"===typeof window.document.createElement),mi=/^[:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD][:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD\-.0-9\u00B7\u0300-\u036F\u203F-\u2040]*$/,
Af=Object.prototype.hasOwnProperty,Cf={},Bf={},I={};"children dangerouslySetInnerHTML defaultValue defaultChecked innerHTML suppressContentEditableWarning suppressHydrationWarning style".split(" ").forEach(function(a){I[a]=new Q(a,0,!1,a,null,!1,!1)});[["acceptCharset","accept-charset"],["className","class"],["htmlFor","for"],["httpEquiv","http-equiv"]].forEach(function(a){var b=a[0];I[b]=new Q(b,1,!1,a[1],null,!1,!1)});["contentEditable","draggable","spellCheck","value"].forEach(function(a){I[a]=
new Q(a,2,!1,a.toLowerCase(),null,!1,!1)});["autoReverse","externalResourcesRequired","focusable","preserveAlpha"].forEach(function(a){I[a]=new Q(a,2,!1,a,null,!1,!1)});"allowFullScreen async autoFocus autoPlay controls default defer disabled disablePictureInPicture disableRemotePlayback formNoValidate hidden loop noModule noValidate open playsInline readOnly required reversed scoped seamless itemScope".split(" ").forEach(function(a){I[a]=new Q(a,3,!1,a.toLowerCase(),null,!1,!1)});["checked","multiple",
"muted","selected"].forEach(function(a){I[a]=new Q(a,3,!0,a,null,!1,!1)});["capture","download"].forEach(function(a){I[a]=new Q(a,4,!1,a,null,!1,!1)});["cols","rows","size","span"].forEach(function(a){I[a]=new Q(a,6,!1,a,null,!1,!1)});["rowSpan","start"].forEach(function(a){I[a]=new Q(a,5,!1,a.toLowerCase(),null,!1,!1)});var uf=/[\-:]([a-z])/g,vf=function(a){return a[1].toUpperCase()};"accent-height alignment-baseline arabic-form baseline-shift cap-height clip-path clip-rule color-interpolation color-interpolation-filters color-profile color-rendering dominant-baseline enable-background fill-opacity fill-rule flood-color flood-opacity font-family font-size font-size-adjust font-stretch font-style font-variant font-weight glyph-name glyph-orientation-horizontal glyph-orientation-vertical horiz-adv-x horiz-origin-x image-rendering letter-spacing lighting-color marker-end marker-mid marker-start overline-position overline-thickness paint-order panose-1 pointer-events rendering-intent shape-rendering stop-color stop-opacity strikethrough-position strikethrough-thickness stroke-dasharray stroke-dashoffset stroke-linecap stroke-linejoin stroke-miterlimit stroke-opacity stroke-width text-anchor text-decoration text-rendering underline-position underline-thickness unicode-bidi unicode-range units-per-em v-alphabetic v-hanging v-ideographic v-mathematical vector-effect vert-adv-y vert-origin-x vert-origin-y word-spacing writing-mode xmlns:xlink x-height".split(" ").forEach(function(a){var b=
a.replace(uf,vf);I[b]=new Q(b,1,!1,a,null,!1,!1)});"xlink:actuate xlink:arcrole xlink:role xlink:show xlink:title xlink:type".split(" ").forEach(function(a){var b=a.replace(uf,vf);I[b]=new Q(b,1,!1,a,"http://www.w3.org/1999/xlink",!1,!1)});["xml:base","xml:lang","xml:space"].forEach(function(a){var b=a.replace(uf,vf);I[b]=new Q(b,1,!1,a,"http://www.w3.org/XML/1998/namespace",!1,!1)});["tabIndex","crossOrigin"].forEach(function(a){I[a]=new Q(a,1,!1,a.toLowerCase(),null,!1,!1)});I.xlinkHref=new Q("xlinkHref",
1,!1,"xlink:href","http://www.w3.org/1999/xlink",!0,!1);["src","href","action","formAction"].forEach(function(a){I[a]=new Q(a,1,!1,a.toLowerCase(),null,!0,!0)});var B=ha.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED.assign,fb=ha.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED,ec=60103,Ua=60106,wa=60107,Hd=60108,Lb=60114,Jd=60109,Id=60110,Dc=60112,Mb=60113,Cc=60120,Ec=60115,Ld=60116,Kd=60121,ue=60128,fi=60129,pf=60130,qf=60131;if("function"===typeof Symbol&&Symbol.for){var H=Symbol.for;ec=
H("react.element");Ua=H("react.portal");wa=H("react.fragment");Hd=H("react.strict_mode");Lb=H("react.profiler");Jd=H("react.provider");Id=H("react.context");Dc=H("react.forward_ref");Mb=H("react.suspense");Cc=H("react.suspense_list");Ec=H("react.memo");Ld=H("react.lazy");Kd=H("react.block");H("react.scope");ue=H("react.opaque.id");fi=H("react.debug_trace_mode");pf=H("react.offscreen");qf=H("react.legacy_hidden")}var Df="function"===typeof Symbol&&Symbol.iterator,Fd,Gd=!1,Ad,Lh=function(a){return"undefined"!==
typeof MSApp&&MSApp.execUnsafeLocalFunction?function(b,c,d,e){MSApp.execUnsafeLocalFunction(function(){return a(b,c,d,e)})}:a}(function(a,b){if("http://www.w3.org/2000/svg"!==a.namespaceURI||"innerHTML"in a)a.innerHTML=b;else{Ad=Ad||document.createElement("div");Ad.innerHTML="<svg>"+b.valueOf().toString()+"</svg>";for(b=Ad.firstChild;a.firstChild;)a.removeChild(a.firstChild);for(;b.firstChild;)a.appendChild(b.firstChild)}}),qc=function(a,b){if(b){var c=a.firstChild;if(c&&c===a.lastChild&&3===c.nodeType){c.nodeValue=
b;return}}a.textContent=b},Nb={animationIterationCount:!0,borderImageOutset:!0,borderImageSlice:!0,borderImageWidth:!0,boxFlex:!0,boxFlexGroup:!0,boxOrdinalGroup:!0,columnCount:!0,columns:!0,flex:!0,flexGrow:!0,flexPositive:!0,flexShrink:!0,flexNegative:!0,flexOrder:!0,gridArea:!0,gridRow:!0,gridRowEnd:!0,gridRowSpan:!0,gridRowStart:!0,gridColumn:!0,gridColumnEnd:!0,gridColumnSpan:!0,gridColumnStart:!0,fontWeight:!0,lineClamp:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,tabSize:!0,widows:!0,zIndex:!0,
zoom:!0,fillOpacity:!0,floodOpacity:!0,stopOpacity:!0,strokeDasharray:!0,strokeDashoffset:!0,strokeMiterlimit:!0,strokeOpacity:!0,strokeWidth:!0},bk=["Webkit","ms","Moz","O"];Object.keys(Nb).forEach(function(a){bk.forEach(function(b){b=b+a.charAt(0).toUpperCase()+a.substring(1);Nb[b]=Nb[a]})});var si=B({menuitem:!0},{area:!0,base:!0,br:!0,col:!0,embed:!0,hr:!0,img:!0,input:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0}),Vd=null,jb=null,kb=null,me=function(a,b){return a(b)},fg=function(a,
b,c,d,e){return a(b,c,d,e)},Xd=function(){},Sf=me,Xa=!1,Yd=!1,qe=!1;if(oa)try{var xc={};Object.defineProperty(xc,"passive",{get:function(){qe=!0}});window.addEventListener("test",xc,xc);window.removeEventListener("test",xc,xc)}catch(a){qe=!1}var vi=function(a,b,c,d,e,f,g,h,k){var l=Array.prototype.slice.call(arguments,3);try{b.apply(c,l)}catch(Da){this.onError(Da)}},Qb=!1,Ic=null,Jc=!1,Zd=null,wi={onError:function(a){Qb=!0;Ic=a}},ca=ha.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED.Scheduler,
ck=ca.unstable_cancelCallback,Bd=ca.unstable_now,$f=ca.unstable_scheduleCallback,dk=ca.unstable_shouldYield,ii=ca.unstable_requestPaint,ae=ca.unstable_runWithPriority,ek=ca.unstable_getCurrentPriorityLevel,fk=ca.unstable_ImmediatePriority,ji=ca.unstable_UserBlockingPriority,ag=ca.unstable_NormalPriority,gk=ca.unstable_LowPriority,hk=ca.unstable_IdlePriority,ce=!1,ia=[],ya=null,za=null,Aa=null,Rb=new Map,Sb=new Map,Vb=[],gg="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset submit".split(" "),
lb={animationend:Lc("Animation","AnimationEnd"),animationiteration:Lc("Animation","AnimationIteration"),animationstart:Lc("Animation","AnimationStart"),transitionend:Lc("Transition","TransitionEnd")},de={},cg={};oa&&(cg=document.createElement("div").style,"AnimationEvent"in window||(delete lb.animationend.animation,delete lb.animationiteration.animation,delete lb.animationstart.animation),"TransitionEvent"in window||delete lb.transitionend.transition);var Hg=Mc("animationend"),Ig=Mc("animationiteration"),
Jg=Mc("animationstart"),Kg=Mc("transitionend"),dg=new Map,fe=new Map,ik=["abort","abort",Hg,"animationEnd",Ig,"animationIteration",Jg,"animationStart","canplay","canPlay","canplaythrough","canPlayThrough","durationchange","durationChange","emptied","emptied","encrypted","encrypted","ended","ended","error","error","gotpointercapture","gotPointerCapture","load","load","loadeddata","loadedData","loadedmetadata","loadedMetadata","loadstart","loadStart","lostpointercapture","lostPointerCapture","playing",
"playing","progress","progress","seeking","seeking","stalled","stalled","suspend","suspend","timeupdate","timeUpdate",Kg,"transitionEnd","waiting","waiting"];Bd();var w=8,Ba=Math.clz32?Math.clz32:Hi,Ii=Math.log,Ji=Math.LN2,Ni=ji,Mi=ae,Pc=!0,Ca=null,ie=null,Qc=null,Hb={eventPhase:0,bubbles:0,cancelable:0,timeStamp:function(a){return a.timeStamp||Date.now()},defaultPrevented:0,isTrusted:0},le=V(Hb),yc=B({},Hb,{view:0,detail:0}),hj=V(yc),wf,xf,zc,Cd=B({},yc,{screenX:0,screenY:0,clientX:0,clientY:0,pageX:0,
pageY:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,getModifierState:je,button:0,buttons:0,relatedTarget:function(a){return void 0===a.relatedTarget?a.fromElement===a.srcElement?a.toElement:a.fromElement:a.relatedTarget},movementX:function(a){if("movementX"in a)return a.movementX;a!==zc&&(zc&&"mousemove"===a.type?(wf=a.screenX-zc.screenX,xf=a.screenY-zc.screenY):xf=wf=0,zc=a);return wf},movementY:function(a){return"movementY"in a?a.movementY:xf}}),Gg=V(Cd),jk=B({},Cd,{dataTransfer:0}),dj=V(jk),kk=B({},
yc,{relatedTarget:0}),re=V(kk),lk=B({},Hb,{animationName:0,elapsedTime:0,pseudoElement:0}),fj=V(lk),mk=B({},Hb,{clipboardData:function(a){return"clipboardData"in a?a.clipboardData:window.clipboardData}}),jj=V(mk),nk=B({},Hb,{data:0}),Og=V(nk),mj=Og,ok={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},pk={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",
16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},Pi={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"},qk=B({},yc,{key:function(a){if(a.key){var b=ok[a.key]||a.key;if("Unidentified"!==
b)return b}return"keypress"===a.type?(a=Rc(a),13===a?"Enter":String.fromCharCode(a)):"keydown"===a.type||"keyup"===a.type?pk[a.keyCode]||"Unidentified":""},code:0,location:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,repeat:0,locale:0,getModifierState:je,charCode:function(a){return"keypress"===a.type?Rc(a):0},keyCode:function(a){return"keydown"===a.type||"keyup"===a.type?a.keyCode:0},which:function(a){return"keypress"===a.type?Rc(a):"keydown"===a.type||"keyup"===a.type?a.keyCode:0}}),cj=V(qk),rk=B({},
Cd,{pointerId:0,width:0,height:0,pressure:0,tangentialPressure:0,tiltX:0,tiltY:0,twist:0,pointerType:0,isPrimary:0}),Lg=V(rk),sk=B({},yc,{touches:0,targetTouches:0,changedTouches:0,altKey:0,metaKey:0,ctrlKey:0,shiftKey:0,getModifierState:je}),ej=V(sk),tk=B({},Hb,{propertyName:0,elapsedTime:0,pseudoElement:0}),gj=V(tk),uk=B({},Cd,{deltaX:function(a){return"deltaX"in a?a.deltaX:"wheelDeltaX"in a?-a.wheelDeltaX:0},deltaY:function(a){return"deltaY"in a?a.deltaY:"wheelDeltaY"in a?-a.wheelDeltaY:"wheelDelta"in
a?-a.wheelDelta:0},deltaZ:0,deltaMode:0}),ij=V(uk),Qi=[9,13,27,32],ke=oa&&"CompositionEvent"in window,Ac=null;oa&&"documentMode"in document&&(Ac=document.documentMode);var lj=oa&&"TextEvent"in window&&!Ac,og=oa&&(!ke||Ac&&8<Ac&&11>=Ac),ng=String.fromCharCode(32),mg=!1,ob=!1,Ti={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0},Xb=null,Yb=null,Ng=!1;oa&&(Ng=Ui("input")&&(!document.documentMode||9<document.documentMode));
var X="function"===typeof Object.is?Object.is:aj,bj=Object.prototype.hasOwnProperty,kj=oa&&"documentMode"in document&&11>=document.documentMode,qb=null,pe=null,$b=null,oe=!1;ee("cancel cancel click click close close contextmenu contextMenu copy copy cut cut auxclick auxClick dblclick doubleClick dragend dragEnd dragstart dragStart drop drop focusin focus focusout blur input input invalid invalid keydown keyDown keypress keyPress keyup keyUp mousedown mouseDown mouseup mouseUp paste paste pause pause play play pointercancel pointerCancel pointerdown pointerDown pointerup pointerUp ratechange rateChange reset reset seeked seeked submit submit touchcancel touchCancel touchend touchEnd touchstart touchStart volumechange volumeChange".split(" "),
0);ee("drag drag dragenter dragEnter dragexit dragExit dragleave dragLeave dragover dragOver mousemove mouseMove mouseout mouseOut mouseover mouseOver pointermove pointerMove pointerout pointerOut pointerover pointerOver scroll scroll toggle toggle touchmove touchMove wheel wheel".split(" "),1);ee(ik,2);(function(a,b){for(var c=0;c<a.length;c++)fe.set(a[c],b)})("change selectionchange textInput compositionstart compositionend compositionupdate".split(" "),0);gb("onMouseEnter",["mouseout","mouseover"]);
gb("onMouseLeave",["mouseout","mouseover"]);gb("onPointerEnter",["pointerout","pointerover"]);gb("onPointerLeave",["pointerout","pointerover"]);Ta("onChange","change click focusin focusout input keydown keyup selectionchange".split(" "));Ta("onSelect","focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" "));Ta("onBeforeInput",["compositionend","keypress","textInput","paste"]);Ta("onCompositionEnd","compositionend focusout keydown keypress keyup mousedown".split(" "));
Ta("onCompositionStart","compositionstart focusout keydown keypress keyup mousedown".split(" "));Ta("onCompositionUpdate","compositionupdate focusout keydown keypress keyup mousedown".split(" "));var pc="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),Eg=new Set("cancel close invalid load scroll toggle".split(" ").concat(pc)),
Dg="_reactListening"+Math.random().toString(36).slice(2),lf=null,mf=null,$h="function"===typeof setTimeout?setTimeout:void 0,Kj="function"===typeof clearTimeout?clearTimeout:void 0,yf=0,Dd=Math.random().toString(36).slice(2),Fa="__reactFiber$"+Dd,Wc="__reactProps$"+Dd,rb="__reactContainer$"+Dd,Rg="__reactEvents$"+Dd,ve=[],ub=-1,Ha={},D=Ga(Ha),J=Ga(!1),Ya=Ha,$e=null,db=null,pj=ae,we=$f,xe=ck,oj=ek,Yc=fk,Vg=ji,Wg=ag,Xg=gk,Yg=hk,hf={},Oj=dk,Sj=void 0!==ii?ii:function(){},pa=null,Zc=null,ye=!1,ki=Bd(),
P=1E4>ki?Bd:function(){return Bd()-ki},Gj=fb.ReactCurrentBatchConfig,bd=Ga(null),ad=null,xb=null,$c=null,Ka=!1,hh=(new ha.Component).refs,dd={isMounted:function(a){return(a=a._reactInternals)?Va(a)===a:!1},enqueueSetState:function(a,b,c){a=a._reactInternals;var d=W(),e=Oa(a),f=Ia(d,e);f.payload=b;void 0!==c&&null!==c&&(f.callback=c);Ja(a,f);Pa(a,e,d)},enqueueReplaceState:function(a,b,c){a=a._reactInternals;var d=W(),e=Oa(a),f=Ia(d,e);f.tag=1;f.payload=b;void 0!==c&&null!==c&&(f.callback=c);Ja(a,f);
Pa(a,e,d)},enqueueForceUpdate:function(a,b){a=a._reactInternals;var c=W(),d=Oa(a),e=Ia(c,d);e.tag=2;void 0!==b&&null!==b&&(e.callback=b);Ja(a,e);Pa(a,d,c)}},gd=Array.isArray,od=ih(!0),xh=ih(!1),fc={},ka=Ga(fc),hc=Ga(fc),gc=Ga(fc),E=Ga(0),ra=null,Na=null,la=!1,Bb=[],jc=fb.ReactCurrentDispatcher,aa=fb.ReactCurrentBatchConfig,ic=0,y=null,N=null,K=null,kd=!1,kc=!1,jd={readContext:Y,useCallback:T,useContext:T,useEffect:T,useImperativeHandle:T,useLayoutEffect:T,useMemo:T,useReducer:T,useRef:T,useState:T,
useDebugValue:T,useDeferredValue:T,useTransition:T,useMutableSource:T,useOpaqueIdentifier:T,unstable_isNewReconciler:!1},qj={readContext:Y,useCallback:function(a,b){ab().memoizedState=[a,void 0===b?null:b];return a},useContext:Y,useEffect:rh,useImperativeHandle:function(a,b,c){c=null!==c&&void 0!==c?c.concat([a]):null;return Ne(4,2,th.bind(null,b,a),c)},useLayoutEffect:function(a,b){return Ne(4,2,a,b)},useMemo:function(a,b){var c=ab();b=void 0===b?null:b;a=a();c.memoizedState=[a,b];return a},useReducer:function(a,
b,c){var d=ab();b=void 0!==c?c(b):b;d.memoizedState=d.baseState=b;a=d.queue={pending:null,dispatch:null,lastRenderedReducer:a,lastRenderedState:b};a=a.dispatch=Me.bind(null,y,a);return[d.memoizedState,a]},useRef:qh,useState:nc,useDebugValue:Pe,useDeferredValue:function(a){var b=nc(a),c=b[0],d=b[1];rh(function(){var b=aa.transition;aa.transition=1;try{d(a)}finally{aa.transition=b}},[a]);return c},useTransition:function(){var a=nc(!1),b=a[0];a=tj.bind(null,a[1]);qh(a);return[a,b]},useMutableSource:function(a,
b,c){var d=ab();d.memoizedState={refs:{getSnapshot:b,setSnapshot:null},source:a,subscribe:c};return oh(d,a,b,c)},useOpaqueIdentifier:function(){if(la){var a=!1,b=nj(function(){a||(a=!0,c("r:"+(yf++).toString(36)));throw Error(m(355));}),c=nc(b)[1];0===(y.mode&2)&&(y.flags|=516,ld(5,function(){c("r:"+(yf++).toString(36))},void 0,null));return b}b="r:"+(yf++).toString(36);nc(b);return b},unstable_isNewReconciler:!1},rj={readContext:Y,useCallback:vh,useContext:Y,useEffect:nd,useImperativeHandle:uh,useLayoutEffect:sh,
useMemo:wh,useReducer:lc,useRef:md,useState:function(a){return lc(ma)},useDebugValue:Pe,useDeferredValue:function(a){var b=lc(ma),c=b[0],d=b[1];nd(function(){var b=aa.transition;aa.transition=1;try{d(a)}finally{aa.transition=b}},[a]);return c},useTransition:function(){var a=lc(ma)[0];return[md().current,a]},useMutableSource:ph,useOpaqueIdentifier:function(){return lc(ma)[0]},unstable_isNewReconciler:!1},sj={readContext:Y,useCallback:vh,useContext:Y,useEffect:nd,useImperativeHandle:uh,useLayoutEffect:sh,
useMemo:wh,useReducer:mc,useRef:md,useState:function(a){return mc(ma)},useDebugValue:Pe,useDeferredValue:function(a){var b=mc(ma),c=b[0],d=b[1];nd(function(){var b=aa.transition;aa.transition=1;try{d(a)}finally{aa.transition=b}},[a]);return c},useTransition:function(){var a=mc(ma)[0];return[md().current,a]},useMutableSource:ph,useOpaqueIdentifier:function(){return mc(ma)[0]},unstable_isNewReconciler:!1},uj=fb.ReactCurrentOwner,fa=!1,qd={dehydrated:null,retryLane:0};var xj=function(a,b,c,d){for(c=
b.child;null!==c;){if(5===c.tag||6===c.tag)a.appendChild(c.stateNode);else if(4!==c.tag&&null!==c.child){c.child.return=c;c=c.child;continue}if(c===b)break;for(;null===c.sibling;){if(null===c.return||c.return===b)return;c=c.return}c.sibling.return=c.return;c=c.sibling}};var Kh=function(a){};var wj=function(a,b,c,d,e){var f=a.memoizedProps;if(f!==d){a=b.stateNode;$a(ka.current);e=null;switch(c){case "input":f=Md(a,f);d=Md(a,d);e=[];break;case "option":f=Pd(a,f);d=Pd(a,d);e=[];break;case "select":f=
B({},f,{value:void 0});d=B({},d,{value:void 0});e=[];break;case "textarea":f=Qd(a,f);d=Qd(a,d);e=[];break;default:"function"!==typeof f.onClick&&"function"===typeof d.onClick&&(a.onclick=Vc)}Sd(c,d);var g;c=null;for(l in f)if(!d.hasOwnProperty(l)&&f.hasOwnProperty(l)&&null!=f[l])if("style"===l){var h=f[l];for(g in h)h.hasOwnProperty(g)&&(c||(c={}),c[g]="")}else"dangerouslySetInnerHTML"!==l&&"children"!==l&&"suppressContentEditableWarning"!==l&&"suppressHydrationWarning"!==l&&"autoFocus"!==l&&(Ib.hasOwnProperty(l)?
e||(e=[]):(e=e||[]).push(l,null));for(l in d){var k=d[l];h=null!=f?f[l]:void 0;if(d.hasOwnProperty(l)&&k!==h&&(null!=k||null!=h))if("style"===l)if(h){for(g in h)!h.hasOwnProperty(g)||k&&k.hasOwnProperty(g)||(c||(c={}),c[g]="");for(g in k)k.hasOwnProperty(g)&&h[g]!==k[g]&&(c||(c={}),c[g]=k[g])}else c||(e||(e=[]),e.push(l,c)),c=k;else"dangerouslySetInnerHTML"===l?(k=k?k.__html:void 0,h=h?h.__html:void 0,null!=k&&h!==k&&(e=e||[]).push(l,k)):"children"===l?"string"!==typeof k&&"number"!==typeof k||(e=
e||[]).push(l,""+k):"suppressContentEditableWarning"!==l&&"suppressHydrationWarning"!==l&&(Ib.hasOwnProperty(l)?(null!=k&&"onScroll"===l&&z("scroll",a),e||h===k||(e=[])):"object"===typeof k&&null!==k&&k.$$typeof===ue?k.toString():(e=e||[]).push(l,k))}c&&(e=e||[]).push("style",c);var l=e;if(b.updateQueue=l)b.flags|=4}};var yj=function(a,b,c,d){c!==d&&(b.flags|=4)};var Lj="function"===typeof WeakMap?WeakMap:Map,Dj="function"===typeof WeakSet?WeakSet:Set,Ij=Math.ceil,vd=fb.ReactCurrentDispatcher,kf=
fb.ReactCurrentOwner,n=0,R=null,G=null,O=0,ta=0,cb=Ga(0),L=0,wd=null,Fb=0,La=0,Cb=0,jf=0,ef=null,df=0,We=Infinity,l=null,rd=!1,Ze=null,na=null,Sa=!1,tc=null,vc=90,nf=[],of=[],va=null,rc=0,ff=null,sd=-1,ua=0,td=0,uc=null,xd=!1;var Pj=function(a,b,c){var d=b.lanes;if(null!==a)if(a.memoizedProps!==b.pendingProps||J.current)fa=!0;else if(0!==(c&d))fa=0!==(a.flags&16384)?!0:!1;else{fa=!1;switch(b.tag){case 3:Dh(b);Ie();break;case 5:jh(b);break;case 1:S(b.type)&&Xc(b);break;case 4:Fe(b,b.stateNode.containerInfo);
break;case 10:d=b.memoizedProps.value;var e=b.type._context;A(bd,e._currentValue);e._currentValue=d;break;case 13:if(null!==b.memoizedState){if(0!==(c&b.child.childLanes))return Eh(a,b,c);A(E,E.current&1);b=sa(a,b,c);return null!==b?b.sibling:null}A(E,E.current&1);break;case 19:d=0!==(c&b.childLanes);if(0!==(a.flags&64)){if(d)return Jh(a,b,c);b.flags|=64}e=b.memoizedState;null!==e&&(e.rendering=null,e.tail=null,e.lastEffect=null);A(E,E.current);if(d)break;else return null;case 23:case 24:return b.lanes=
0,Se(a,b,c)}return sa(a,b,c)}else fa=!1;b.lanes=0;switch(b.tag){case 2:d=b.type;null!==a&&(a.alternate=null,b.alternate=null,b.flags|=2);a=b.pendingProps;e=vb(b,D.current);yb(b,c);e=Le(null,b,d,a,e,c);b.flags|=1;if("object"===typeof e&&null!==e&&"function"===typeof e.render&&void 0===e.$$typeof){b.tag=1;b.memoizedState=null;b.updateQueue=null;if(S(d)){var f=!0;Xc(b)}else f=!1;b.memoizedState=null!==e.state&&void 0!==e.state?e.state:null;Be(b);var g=d.getDerivedStateFromProps;"function"===typeof g&&
cd(b,d,g,a);e.updater=dd;b.stateNode=e;e._reactInternals=b;Ce(b,d,a,c);b=Te(null,b,d,!0,f,c)}else b.tag=0,U(null,b,e,c),b=b.child;return b;case 16:e=b.elementType;a:{null!==a&&(a.alternate=null,b.alternate=null,b.flags|=2);a=b.pendingProps;f=e._init;e=f(e._payload);b.type=e;f=b.tag=Vj(e);a=ea(e,a);switch(f){case 0:b=Re(null,b,e,a,c);break a;case 1:b=Ch(null,b,e,a,c);break a;case 11:b=yh(null,b,e,a,c);break a;case 14:b=zh(null,b,e,ea(e.type,a),d,c);break a}throw Error(m(306,e,""));}return b;case 0:return d=
b.type,e=b.pendingProps,e=b.elementType===d?e:ea(d,e),Re(a,b,d,e,c);case 1:return d=b.type,e=b.pendingProps,e=b.elementType===d?e:ea(d,e),Ch(a,b,d,e,c);case 3:Dh(b);d=b.updateQueue;if(null===a||null===d)throw Error(m(282));d=b.pendingProps;e=b.memoizedState;e=null!==e?e.element:null;bh(a,b);cc(b,d,null,c);d=b.memoizedState.element;if(d===e)Ie(),b=sa(a,b,c);else{e=b.stateNode;if(f=e.hydrate)Na=tb(b.stateNode.containerInfo.firstChild),ra=b,f=la=!0;if(f){a=e.mutableSourceEagerHydrationData;if(null!=
a)for(e=0;e<a.length;e+=2)f=a[e],f._workInProgressVersionPrimary=a[e+1],Bb.push(f);c=xh(b,null,d,c);for(b.child=c;c;)c.flags=c.flags&-3|1024,c=c.sibling}else U(a,b,d,c),Ie();b=b.child}return b;case 5:return jh(b),null===a&&He(b),d=b.type,e=b.pendingProps,f=null!==a?a.memoizedProps:null,g=e.children,se(d,e)?g=null:null!==f&&se(d,f)&&(b.flags|=16),Bh(a,b),U(a,b,g,c),b.child;case 6:return null===a&&He(b),null;case 13:return Eh(a,b,c);case 4:return Fe(b,b.stateNode.containerInfo),d=b.pendingProps,null===
a?b.child=od(b,null,d,c):U(a,b,d,c),b.child;case 11:return d=b.type,e=b.pendingProps,e=b.elementType===d?e:ea(d,e),yh(a,b,d,e,c);case 7:return U(a,b,b.pendingProps,c),b.child;case 8:return U(a,b,b.pendingProps.children,c),b.child;case 12:return U(a,b,b.pendingProps.children,c),b.child;case 10:a:{d=b.type._context;e=b.pendingProps;g=b.memoizedProps;f=e.value;var h=b.type._context;A(bd,h._currentValue);h._currentValue=f;if(null!==g)if(h=g.value,f=X(h,f)?0:("function"===typeof d._calculateChangedBits?
d._calculateChangedBits(h,f):1073741823)|0,0===f){if(g.children===e.children&&!J.current){b=sa(a,b,c);break a}}else for(h=b.child,null!==h&&(h.return=b);null!==h;){var k=h.dependencies;if(null!==k){g=h.child;for(var l=k.firstContext;null!==l;){if(l.context===d&&0!==(l.observedBits&f)){1===h.tag&&(l=Ia(-1,c&-c),l.tag=2,Ja(h,l));h.lanes|=c;l=h.alternate;null!==l&&(l.lanes|=c);ah(h.return,c);k.lanes|=c;break}l=l.next}}else g=10===h.tag?h.type===b.type?null:h.child:h.child;if(null!==g)g.return=h;else for(g=
h;null!==g;){if(g===b){g=null;break}h=g.sibling;if(null!==h){h.return=g.return;g=h;break}g=g.return}h=g}U(a,b,e.children,c);b=b.child}return b;case 9:return e=b.type,f=b.pendingProps,d=f.children,yb(b,c),e=Y(e,f.unstable_observedBits),d=d(e),b.flags|=1,U(a,b,d,c),b.child;case 14:return e=b.type,f=ea(e,b.pendingProps),f=ea(e.type,f),zh(a,b,e,f,d,c);case 15:return Ah(a,b,b.type,b.pendingProps,d,c);case 17:return d=b.type,e=b.pendingProps,e=b.elementType===d?e:ea(d,e),null!==a&&(a.alternate=null,b.alternate=
null,b.flags|=2),b.tag=1,S(d)?(a=!0,Xc(b)):a=!1,yb(b,c),fh(b,d,e),Ce(b,d,e,c),Te(null,b,d,!0,a,c);case 19:return Jh(a,b,c);case 23:return Se(a,b,c);case 24:return Se(a,b,c)}throw Error(m(156,b.tag));};var Z=function(a,b,c,d){return new Uj(a,b,c,d)};tf.prototype.render=function(a){yd(a,this._internalRoot,null,null)};tf.prototype.unmount=function(){var a=this._internalRoot,b=a.containerInfo;yd(null,a,null,function(){b[rb]=null})};var Ei=function(a){if(13===a.tag){var b=W();Pa(a,4,b);sf(a,4)}};var Yf=
function(a){if(13===a.tag){var b=W();Pa(a,67108864,b);sf(a,67108864)}};var Ci=function(a){if(13===a.tag){var b=W(),c=Oa(a);Pa(a,c,b);sf(a,c)}};var Bi=function(a,b){return b()};Vd=function(a,b,c){switch(b){case "input":Nd(a,c);b=c.name;if("radio"===c.type&&null!=b){for(c=a;c.parentNode;)c=c.parentNode;c=c.querySelectorAll("input[name="+JSON.stringify(""+b)+'][type="radio"]');for(b=0;b<c.length;b++){var d=c[b];if(d!==a&&d.form===a.form){var e=Hc(d);if(!e)throw Error(m(90));Ff(d);Nd(d,e)}}}break;case "textarea":Kf(a,
c);break;case "select":b=c.value,null!=b&&ib(a,!!c.multiple,b,!1)}};(function(a,b,c,d){me=a;fg=b;Xd=c;Sf=d})(ai,function(a,b,c,d,e){var f=n;n|=4;try{return Za(98,a.bind(null,b,c,d,e))}finally{n=f,0===n&&(Eb(),ja())}},function(){0===(n&49)&&(Jj(),Ra())},function(a,b){var c=n;n|=2;try{return a(b)}finally{n=c,0===n&&(Eb(),ja())}});var vk={Events:[Ob,pb,Hc,Qf,Rf,Ra,{current:!1}]};(function(a){a={bundleType:a.bundleType,version:a.version,rendererPackageName:a.rendererPackageName,rendererConfig:a.rendererConfig,
overrideHookState:null,overrideHookStateDeletePath:null,overrideHookStateRenamePath:null,overrideProps:null,overridePropsDeletePath:null,overridePropsRenamePath:null,setSuspenseHandler:null,scheduleUpdate:null,currentDispatcherRef:fb.ReactCurrentDispatcher,findHostInstanceByFiber:Yj,findFiberByHostInstance:a.findFiberByHostInstance||Zj,findHostInstancesForRefresh:null,scheduleRefresh:null,scheduleRoot:null,setRefreshHandler:null,getCurrentFiber:null};if("undefined"===typeof __REACT_DEVTOOLS_GLOBAL_HOOK__)a=
!1;else{var b=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(!b.isDisabled&&b.supportsFiber)try{$e=b.inject(a),db=b}catch(c){}a=!0}return a})({findFiberByHostInstance:Wa,bundleType:0,version:"17.0.2",rendererPackageName:"react-dom"});M.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED=vk;M.createPortal=hi;M.findDOMNode=function(a){if(null==a)return null;if(1===a.nodeType)return a;var b=a._reactInternals;if(void 0===b){if("function"===typeof a.render)throw Error(m(188));throw Error(m(268,Object.keys(a)));
}a=Vf(b);a=null===a?null:a.stateNode;return a};M.flushSync=function(a,b){var c=n;if(0!==(c&48))return a(b);n|=1;try{if(a)return Za(99,a.bind(null,b))}finally{n=c,ja()}};M.hydrate=function(a,b,c){if(!wc(b))throw Error(m(200));return zd(null,a,b,!0,c)};M.render=function(a,b,c){if(!wc(b))throw Error(m(200));return zd(null,a,b,!1,c)};M.unmountComponentAtNode=function(a){if(!wc(a))throw Error(m(40));return a._reactRootContainer?(bi(function(){zd(null,null,a,!1,function(){a._reactRootContainer=null;a[rb]=
null})}),!0):!1};M.unstable_batchedUpdates=ai;M.unstable_createPortal=function(a,b){return hi(a,b,2<arguments.length&&void 0!==arguments[2]?arguments[2]:null)};M.unstable_renderSubtreeIntoContainer=function(a,b,c,d){if(!wc(c))throw Error(m(200));if(null==a||void 0===a._reactInternals)throw Error(m(38));return zd(a,b,c,!1,d)};M.version="17.0.2"});
})();

window.TimeTrack = {}, ContentScript = {
    v2: {
        Components: {},
    },
    Constants: {
        DAY_TIME_SUBPART_TYPE: {
            HOURS: 0,
            MINUTES: 1,
            MERIDIEM: 2
        },
        DAILY_TIME: 24 * 60 * 60,
        TIME_FORMAT12: 12,
        TIME_FORMAT24: 24
    }
};
window.TeamApplicationsStore = function() {
    this.index = {};
};

TeamApplicationsStore.prototype = {
    load: function() {
        return everhour.teamApplications().then((applications) => this.index = applications);
    },

    get: function() {
        return this.index;
    },

    findByUrl: function(url) {
        // if profile is suspended we skip loading projects/applications/tasks/time etc
        return this.index.length ? this.index.find((application) => url.includes(application.baseUrl)) : null;
    }
};
window.TeamStatusStore = function() {
    this.index = {};
};

TeamStatusStore.prototype = {
    load: function(from, to) {
        const start = from || moment().format('YYYY-MM-DD');
        const end = to || moment().format('YYYY-MM-DD');

        return everhour.teamStatus(start, end)
            .then((teamStatus) => this.index = teamStatus);
    },

    get: function() {
        return this.index;
    },

    getById: function(id) {
        return this.index.filter((user) => user.user.id === id)[0];
    }
};

window.ClientsStore = function() {
    this.indexById = null; // internal id => client details
};

ClientsStore.Act = {
    CREATE_COMPLETED: 'api:clients:create_completed'
};

ClientsStore.prototype = {
    load: function() {
        return !this.indexById ?
            everhour.clients()
                .done((allClients) => {
                    this.indexById = _.indexBy(allClients, 'id');
                }) :
            resolvedPromise();
    },

    reduce: function(action) {
        switch (action.type) {
            case ClientsStore.Act.CREATE_COMPLETED:
                if (this.indexById && !this.has(action.item.id)) {
                    this.indexById[action.item.id] = action.item;
                }
                break;
        }
    },

    /**
     * @param {string} clientId
     * @returns {boolean}
     */
    has: function(clientId) {
        return this.indexById ? this.indexById.hasOwnProperty(clientId) : false;
    },

    /**
     * @param {string} clientId
     * @returns {boolean}
     */
    get: function(clientId) {
        return this.indexById ? this.indexById[clientId] : null;
    },

    all: function() {
        return this.indexById;
    },

    /**
     *
     * @param {Object} data
     * @param {string} requestEvent
     * @returns {*}
     */
    createClient: function(data, requestEvent) {
        return everhour.lock('create_client', () => {
            return everhour.createClient(data, requestEvent)
                .done((newClient) => {
                    store.dispatch({
                        type: ClientsStore.Act.CREATE_COMPLETED,
                        item: newClient
                    });

                    return newClient;
                });
        });
    },

    /**
     * @param {Function} filterFn
     * @returns {[]}
     */
    getAssignable: function(filterFn) {
        filterFn = filterFn || ((client) => client.status === 'active');

        return Object.values(this.all())
            .filter(filterFn)
            .sort((clientA, clientB) =>
                (clientA.name || '').localeCompare(clientB.name || '')
            );
    }
};
window.ProjectsStore = function() {
    this.indexById = {}; // internal id => project details
    this.indexByName = {};
    this.progressById = {};
    this.collaboratorsById = {};
    this.indexByWorkspace = {};
    this.synchronizing = false;
    this.syncInProgressIds = []; // case when sync in progress right now
    this.syncDoneIds = [];
    this.syncFailIds = []; // case when sync failed
    this.scheduledClientAssign = [];
    this.scheduledCopyProjectOptions = {};
};

ProjectsStore.Act = {
    RELOAD: '@api:projects:reload', // we using it for send to front app
    BUDGET_UPDATED: 'api:projects:budget_updated',
    PROJECT_CLIENT_UPDATED: 'api:projects:project_client_updated',
    PROJECT_INVALIDATE_TOTAL: 'api:projects:project_invalidate_total',

    Reload: () => ({
        type: ProjectsStore.Act.RELOAD
    }),

    InvalidateTotals: (projectIds) => ({
        type: ProjectsStore.Act.PROJECT_INVALIDATE_TOTAL,
        projects: projectIds
    })
};

ProjectsStore.prototype = {
    load: function(platform, allowForeign, requestEvent) {
        return everhour.projects(platform, requestEvent)
            .done((allProjects) => {
                this.indexById = {};
                this.indexByName = {};
                this.indexByWorkspace = {};

                const projects = allProjects
                    .filter((project) => allowForeign || !project.foreign)
                    .map(project => {
                        project.archived = project.status === 'archived';
                        return project;
                    });

                this.composeIndexWithResponse(projects);
            })
    },

    reduce: function(action) {
        let projectId, updatedProject;
        switch (action.type) {
            case ProjectsStore.Act.BUDGET_UPDATED:
                updatedProject = action.item;
                projectId = updatedProject.projectId;

                if (this.has(projectId)) {
                    this.indexById[projectId] = Object.assign(
                        {},
                        this.get(projectId),
                        {budget: updatedProject.budget || null},
                        {billing: updatedProject.billing || null},
                        {rate: updatedProject.rate || null}
                    );
                }
                break;

            case ProjectsStore.Act.PROJECT_CLIENT_UPDATED:
                updatedProject = action.project;
                projectId = updatedProject.id;

                if (this.has(projectId)) {
                    this.indexById[projectId] = Object.assign(
                        {},
                        this.get(projectId),
                        {
                            client: updatedProject.client || null,
                            budget: updatedProject.budget || null,
                            rate: updatedProject.rate || null
                        }
                    )
                }
                break;

            case ClientsStore.Act.CREATE_COMPLETED:
                (action.item.projects || []).forEach(projectId => {
                    if (this.has(projectId)) {
                        this.indexById[projectId] = Object.assign(
                            {},
                            this.get(projectId),
                            {
                                client: action.item.id
                            }
                        )
                    }
                });
                break;

            case ProjectsStore.Act.PROJECT_INVALIDATE_TOTAL:
                (action.projects || []).forEach(projectId => {
                    // invalidate progress/collaborators
                    delete this.progressById[projectId];
                    delete this.collaboratorsById[projectId];
                });
                break;
        }
    },

    taskProject: function(projects) {
        if (projects.length === 0) {
            return null;
        }

        for (let i = 0; i < projects.length; i++) {
            if (store.projects.has(projects[i]) && store.projects.get(projects[i]).status === 'open') {
                return projects[i];
            }
        }

        for (let i = 0; i < projects.length; i++) {
            if (store.projects.has(projects[i]) && !store.projects.get(projects[i]).foreign) {
                return projects[i];
            }
        }

        return projects[0];
    },

    /**
     * @param {string} projectId
     * @returns {*|jQuery}
     */
    getProjectById: function(projectId) {
        const promise = $.Deferred();

        if (!projectId) {
            return promise.resolve(null);
        }

        everhour.projectById(projectId)
            .done((project) => {
                // API said this project already synced
                this.syncDoneIds = [].concat(this.syncDoneIds, [projectId]);
                this.syncFailIds = this.syncFailIds.filter(id => projectId !== id);

                promise.resolve(project);
            })
            .fail((err) => {
                this.syncFailIds = [].concat(this.syncFailIds, [projectId]);
                this.syncDoneIds = this.syncDoneIds.filter(id => projectId !== id);

                promise.resolve(null);
            })
            .always(() => {
                this.syncInProgressIds = this.syncInProgressIds.filter(id => projectId !== id);
            });

        return promise.promise();
    },

    /**
     *
     * @param {[]<string>} projectIds
     * @returns {*|jQuery}
     */
    getProjectsById: function(projectIds) {
        let promise = $.Deferred();

        $.when.apply(this, projectIds.map(projectId => this.getProjectById(projectId)))
            .done((...response) => {
                const syncedProjects = response.filter(data => !!data);
                this.composeIndexWithResponse(syncedProjects);

                promise.resolve(syncedProjects);
            });

        return promise.promise();
    },

    /**
     * @param {string} accountId
     * @param {[]|string} projects
     * @param {boolean} theWholeAccountSync
     * @param {number} additionalAttemptsCount
     * @returns {*|jQuery}
     */
    synchronizeAccount: function(accountId, projects, theWholeAccountSync, additionalAttemptsCount) {
        let synchronizeAccountPromise = $.Deferred();
        additionalAttemptsCount = additionalAttemptsCount || 0;

        everhour.synchronizeAccount(accountId, !theWholeAccountSync ? projects : null, !!additionalAttemptsCount)
            .done(() => {
                this.getProjectsById(projects)
                    .done(() => {
                        if (theWholeAccountSync || !projects || projects.length === 0) {
                            store.dispatch(ProjectsStore.Act.Reload());
                            synchronizeAccountPromise.resolve();
                            return;
                        }

                        const failedProjects = projects.filter(project => this.syncFailIds.indexOf(project) !== -1);

                        // all projects synced
                        if (failedProjects.length === 0) {
                            store.dispatch(ProjectsStore.Act.Reload());
                            synchronizeAccountPromise.resolve();
                            return;
                        }

                        if (additionalAttemptsCount <= 0) {

                            this.syncDoneIds = [].concat(this.syncDoneIds, projects);
                            this.syncFailIds = this.syncFailIds.filter(project => failedProjects.indexOf(project) === -1);
                            synchronizeAccountPromise.reject();
                            return;
                        }

                        additionalAttemptsCount--;
                        this.synchronizeAccount(accountId, failedProjects, theWholeAccountSync, additionalAttemptsCount)
                            .done(() => synchronizeAccountPromise.resolve())
                            .fail(() => synchronizeAccountPromise.reject());
                    });
            });

        return synchronizeAccountPromise.promise();
    },

    /**
     *
     * @param {string} accountId
     * @param {[]|string} projects
     * @param {boolean} isIndividual
     * @param {number} additionalAttemptsCount
     * @returns {*|jQuery}
     */
    synchronize: function(accountId, projects, isIndividual = false, additionalAttemptsCount) {
        additionalAttemptsCount = additionalAttemptsCount || 0;
        // not for all integrations we may sync specific project(s) - using isIndividual
        // due API restrictions, for this case we syncing the whole account
        projects = _.isArray(projects) ? projects : [projects];

        const notProcessedProjects = projects
            .filter((projectId) => (
                !this.isProjectSyncing(projectId) &&
                !this.isProjectSynced(projectId) &&
                !this.isProjectNotSynced(projectId)
            ));

        if (notProcessedProjects.length === 0) {
            // immitate behavior like sync process done
            return resolvedPromise();
        }

        Logger.log('projectsStore:', ' synchronize newProjects', notProcessedProjects );

        this.syncInProgressIds = [].concat(
            this.syncInProgressIds,
            notProcessedProjects
        );

        let syncPromise = $.Deferred();

        this.synchronizing = true;

        const theWholeAccountSync = !(isIndividual && projects.length > 0);

        // pre-check that projects isn't synced yet
        // not depends on theWholeAccountSync
        this.getProjectsById(projects)
            .then((syncedProjects) => {
                Logger.log('projectsStore:',' synchronize getProjectsById done', syncedProjects);
                syncedProjects = syncedProjects.filter(project => !project.foreign);

                if (projects.length === syncedProjects.length) {
                    // all projects already synced
                    // then skip sync and dispatch event for reload web app
                    this.synchronizing = false;

                    store.dispatch(ProjectsStore.Act.Reload());
                    syncPromise.resolve();
                    return;
                }

                // sync then
                this.synchronizeAccount(accountId, projects, theWholeAccountSync, additionalAttemptsCount)
                    .done(() => {
                        Logger.log('projectsStore:', 'synchronizeAccount done');

                        syncPromise.resolve();
                    })
                    .catch(() => {
                        Logger.log('projectsStore:', 'synchronizeAccount error');

                        syncPromise.reject();
                    })
                    .always(() => {
                        Logger.log('projectsStore:','synchronizeAccount always');

                        this.synchronizing = false;
                        this.syncInProgressIds = [];
                    });
            });

        return syncPromise.promise();
    },

    /**
     * @param {[]<Object|null>} projectsResponse
     */
    composeIndexWithResponse: function(projectsResponse) {
        projectsResponse
            .filter(responseData => !!responseData)
            .forEach(projectData => {
                const platformPrefix = projectData.id.substr(0, 2);
                const nameIdx = platformPrefix + ':' + projectData.name.toLowerCase();

                this.indexById[projectData.id] = projectData;
                this.indexByName[nameIdx] = projectData;

                if (projectData.workspaceName) {
                    const key = platformPrefix + ':' + projectData.workspaceName + '/' + projectData.name;
                    this.indexByWorkspace[key.toLowerCase()] = projectData;
                }
            });
    },

    /**
     * @param {string} accountId
     * @param {string} targetProjectId
     * @param {string} sourceProjectId
     * @param {boolean} isCopyEstimatesRequired
     * @param {boolean} isCopyBudgetRequired
     * @returns {*|jQuery}
     */
    duplicateProjectSettings: function(
        accountId,
        targetProjectId,
        sourceProjectId,
        isCopyEstimatesRequired,
        isCopyBudgetRequired
    ) {
        return everhour.lock('project_duplication_' + targetProjectId, () => {
            const promise = $.Deferred();
            if (!isCopyBudgetRequired && !isCopyEstimatesRequired) {
                return promise.resolve();
            }
            promise.done(() => store.notify());
            const platform = app.adaptor.platform;
            const data = {
                targetProject: targetProjectId,
                sourceProject: sourceProjectId,
                copyEstimates: isCopyEstimatesRequired,
                copyBudget: isCopyBudgetRequired
            };
            everhour.duplicateProjectSettings(accountId, data)
                .done(() => {
                    // when API call && published message observe finished
                    store.dispatch(ProjectsStore.Act.Reload());
                    // reload tasks and load a new project
                    $.when(this.getProjectsById([targetProjectId]), store.tasks.load(platform)).done(() => {
                        // and then - resolve our promise
                        this.synchronizing = false;
                        promise.resolve();
                    });
                })
                .fail(() => {
                    this.synchronizing = false;
                    promise.reject();
                });

            return promise.promise();
        });
    },

    /**
     * @param {string} projectId
     * @returns {boolean}
     */
    isProjectSyncing: function(projectId) {
        return this.synchronizing && this.syncInProgressIds.indexOf(projectId) !== -1;
    },

    /**
     * @param {string} projectId
     * @returns {boolean}
     */
    isProjectSynced: function(projectId) {
        return this.syncDoneIds.indexOf(projectId) !== -1;
    },

    isProjectNotSynced: function(projectId) {
        return this.syncFailIds.indexOf(projectId) !== -1;
    },

    /**
     * @param {string} projectId
     * @returns {boolean}
     */
    has: function(projectId) {
        return this.indexById.hasOwnProperty(projectId);
    },

    /**
     * @param {string} projectId
     * @returns {boolean}
     */
    get: function(projectId) {
        return this.indexById[projectId];
    },

    /**
     * @param {string} projectId
     * @returns {Object|null}
     */
    getProgress: function(projectId) {
        return this.progressById[projectId] || null;
    },

    /**
     * @param {string} projectId
     * @returns {Object|null}
     */
    getCollaborators: function(projectId) {
        return this.collaboratorsById[projectId] || null;
    },

    /**
     * @param {string} projectId
     * @returns {boolean}
     */
    isArchived: function(projectId) {
        return this.has(projectId) && this.get(projectId).archived;
    },

    /**
     * @param {string} projectName
     * @returns {boolean}
     */
    hasByName: function(projectName) {
        return this.indexByName.hasOwnProperty(projectName);
    },

    /**
     * @param {string} projectName
     * @returns {boolean}
     */
    getByName: function(projectName) {
        return this.indexByName[projectName];
    },

    /**
     * @param {string} workspaceKey
     * @returns {boolean}
     */
    hasByWorkspace: function(workspaceKey) {
        return this.indexByWorkspace.hasOwnProperty(workspaceKey);
    },

    /**
     * @param {string} workspaceKey
     * @returns {boolean}
     */
    getByWorkspace: function(workspaceKey) {
        return this.indexByWorkspace[workspaceKey];
    },

    /**
     * @param {number} applicationId Everhour application id e.g. 459
     * @param {string} taskNumber Task number e.g. SCRUM-15
     */
    findByJiraTask: function(applicationId, taskNumber) {
        var projectCode = taskNumber.split('-')[0];

        for (var id in this.indexById) {
            if (!this.indexById.hasOwnProperty(id)) {
                continue;
            }

            if (id.substr(0, 2) !== 'jr') {
                continue;
            }

            if (id.indexOf('jr:' + applicationId) !== 0) {
                continue;
            }

            if (this.indexById[id].name.indexOf(projectCode + ' ') !== 0) {
                continue;
            }

            return id;
        }

        return null;
    },

    all: function() {
        return this.indexById;
    },

    /**
     *
     * @param {string} projectId
     * @param {Object|null} billingSettings
     * @returns {*}
     */
    updateBillingSettings: function(projectId, billingSettings) {
        return everhour.lock('project_billing', () => {
            return everhour.updateProjectBillingSettings(projectId, billingSettings)
                .done((updatedProject) =>{
                    updatedProject.archived = updatedProject.status === 'archived';

                    this.indexById[projectId] = Object.assign(
                        {},
                        this.get(projectId),
                        updatedProject,
                        {budget: updatedProject.budget || null},
                        {billing: updatedProject.billing || null}
                    );

                    return updatedProject;
                });
        });
    },

    /**
     * @param {string} projectId
     * @param {number} clientId
     * @param {string} requestEvent
     * @returns {*}
     */
    updateClient: function(projectId, clientId, requestEvent) {
        return everhour.lock('project_client_assign_' + projectId, () => {
            return everhour.updateProjectClient(projectId, clientId, requestEvent)
                .done((updatedProject) => {
                    this.indexById[projectId] = Object.assign(
                        {},
                        this.get(projectId),
                        updatedProject,
                        {client: updatedProject.client || null},
                    );

                    return updatedProject;
                });
        });
    },

    /**
     * @param {Object} progressPerProject
     */
    setProjectTotals: function (progressPerProject) {
        progressPerProject
            .forEach(projectProgress => {
                const projectId = projectProgress.project;
                this.progressById[projectId] = {
                    time: projectProgress.time,
                    estimate: projectProgress.estimate
                };
            });
    },

    /**
     *
     * @param {[].<string>} projectIds
     * @returns {*}
     */
    loadTotalsByProjects: function(projectIds) {
        projectIds = _.unique(projectIds).filter(projectId => projectId && !this.getProgress(projectId));
        //request totals for synced projects and if data not loaded yet

        if (!projectIds.length) {
            return resolvedPromise();
        }

        return everhour.lock('projects_progress', () => {
            const deferred = $.Deferred();
            everhour.totalsByProjects(projectIds)
                .done((progressPerProject) => {
                    store.projects.setProjectTotals(progressPerProject);
                    deferred.resolve();
                })
                .fail(() => {
                    // failsafing for avoid infinite loop for render/loading
                    projectIds.forEach(projectId => {
                        this.progressById[projectId] = {
                            time: 0,
                            estimate: 0
                        };
                    });
                    deferred.resolve();
                });

            return deferred.promise();
        });
    },

    /**
     * @param {string} projectId
     * @returns {*}
     */
    loadDetailedProgress: function(projectId) {
        const collaborators = this.getCollaborators(projectId);
        if (collaborators) {
            return resolvedPromise();
        }

        return everhour.lock('project_detailed_progress_' + projectId, () => {
            const deferred = $.Deferred();
            everhour.progressByProject(projectId)
                .done((detailedProjectProgress) => {
                    this.collaboratorsById[projectId] = detailedProjectProgress;
                    deferred.resolve();
                })
                .fail(() => {
                    // failsafing for avoid infinite loop for render/loading
                    this.collaboratorsById[projectId]  = [];
                    deferred.resolve();
                });

            return deferred.promise();
        });
    },

    getGroupsProjects: function(limit = 5) {
        const startWeekMillis = moment().startOf('week').startOf('day')._d.valueOf();
        const endWeekMillis = moment().endOf('week').endOf('day')._d.valueOf();
        const groups = {
            recent: {id: 1, label: 'Recent Projects', projects: []},
            active: {id: 2, label: 'Active Projects', projects: []}
        };
        const recentProjectIds = {};
        const taskTime = store.taskTime.taskTime && store.taskTime.taskTime
            .filter((taskTime) => {
                const taskTimeCreatedAtMillis = moment(taskTime.createdAt)._d.valueOf();

                return taskTimeCreatedAtMillis >= startWeekMillis && taskTimeCreatedAtMillis <= endWeekMillis
            })
            .sort((a, b) => {
                if (b.createdAt < a.createdAt) {
                    return -1;
                }

                if (b.createdAt > a.createdAt) {
                    return 1;
                }
            });

        taskMarker: for (let i = 0; i < taskTime.length; i++) {
            const time = taskTime[i];

            if (!time.task || !time.task.projects) {
                continue;
            }

            for (const projectId of time.task.projects) {
                const project = this.get(projectId);
                const projectClone = project && JSON.parse(JSON.stringify(project));

                if ((!projectClone)
                    || (!!recentProjectIds[projectId])
                    || (projectClone && projectClone.foreign || projectClone.status !== 'open')) {
                    continue;
                }

                if (groups.recent.projects.length >= limit) {
                    break taskMarker;
                }

                groups.recent.projects.push(project);
                recentProjectIds[projectId] = 1;
            }
        }

        Object.keys(this.indexById).forEach((projectId) => {
            const project = this.get(projectId);
            const projectClone = project && JSON.parse(JSON.stringify(project));

            if (projectClone && !projectClone.foreign && projectClone.status === 'open' && !!!recentProjectIds[projectId]) {
                groups.active.projects.push(projectClone);
            }
        });

        groups.active.projects = groups.active.projects.sort((a, b) => {
            const lowerNameA = a.name.toLowerCase();
            const lowerNameB = b.name.toLowerCase();

            if (lowerNameA < lowerNameB) {
                return -1;
            }

            if (lowerNameA > lowerNameB) {
                return 1;
            }
        });

        const handleAddSearchField = (project) => {
            project._searchName = project.workspaceName ? `${project.name} ${project.workspaceName}` : project.name;

            return project;
        };

        groups.active.projects.map((project) => handleAddSearchField(project));
        groups.recent.projects.map((project) => handleAddSearchField(project));

        return groups;
    },

    connectAccountProjects: function(accountId, projects) {
        return everhour.connectAccountProjects(accountId, {projects});
    }
};

window.ProfileStore = function() {
    this.index = {}; // user profile object
};

ProfileStore.Act = {
    TEAM_UPDATED: 'auth:team_updated',
    ACCOUNT_REMOVED: 'auth:account_removed',
    ACCOUNT_UPDATED: 'auth:account_updated'
};

ProfileStore.prototype = {
    timeManagementRoles: ['admin', 'supervisor'],

    load: function() {
        return everhour.profile().then((profile) => {
            // don't override profile totally on reload, we have specific settings for integrations
            // such as clickUpSettings for example
            this.index = Object.assign(this.index || {}, profile);
            if (moment) {
                moment.updateLocale('en', {
                    week: {
                        dow : profile.team.beginningOfWeek
                    }
                });
            }

            const currentTimezone = Math.round((-1 * (new Date().getTimezoneOffset())) / 30) / 2;
            if (!profile.hasOwnProperty('timezone') || profile.timezone !== currentTimezone) {
                this.index.timezone = currentTimezone;
                everhour.setTimezone(currentTimezone);
            }

            browserPort.eventDispatcher.on('*', 'profile:updated', (data) => {
                this.index = data;
                store.notify();
            })
        });
    },

    loadPolicy: function() {
        return everhour.profileTeamPolicy()
            .then((timeTrackingPolicy) => {
                this.index.timeTrackingPolicy = Object.assign(this.index.timeTrackingPolicy, timeTrackingPolicy);
            });
    },

    get: function() {
        return this.index;
    },

    team: function() {
        return this.index ? this.index.team : null;
    },

    /**
     * optional params for prototype.call
     * @param {string}      role
     * @param {string}      lockTimePeriod
     * @param {int}         lockTimeAfter
     * @param {boolean}     lockTimeEnabled
     * @returns {string}
     */
    getMinDateAllowed: function(role, lockTimePeriod, lockTimeAfter, lockTimeEnabled) {
        // 'no-lock','days','day-of-week', 'day-of-month'
        const lockPeriod  = typeof(lockTimePeriod) !== 'undefined' ? lockTimePeriod : this.index.team.lockTimePeriod;
        const lockAfterValue = typeof(lockTimeAfter) !== 'undefined' ? lockTimeAfter : parseInt(this.index.team.lockTimeAfter);
        const lockEnabled = typeof(lockTimeEnabled) !== 'undefined' ? lockTimeEnabled : this.index.team.lockTimeEnabled;

        if (this.timeManagementRoles.indexOf(role || this.index.role) !== -1) {
            return moment().subtract(1, 'years').format('YYYY-MM-DD');
        }

        if (!lockEnabled || lockAfterValue < 0 || lockPeriod === 'no-lock') {
            return moment().subtract(1, 'years').format('YYYY-MM-DD');
        }

        let minDate;
        switch (lockPeriod) {
            case 'days':
                // additional +1  day required for exclude full day, reflects BE logic
                minDate = moment().subtract(lockAfterValue, 'days');
                break;
            case 'day-of-week':
                var days = parseInt(moment().format('E')) - lockAfterValue;
                var daysAdd = days >= 0 ? days : days + 7;
                minDate = moment().subtract(daysAdd, 'days');
                break;
            case 'day-of-month':
                var lockAfterCorrected = lockAfterValue > moment().daysInMonth() ? moment().daysInMonth() : lockAfterValue;
                minDate = moment().startOf('month');
                if (parseInt(moment().format("DD")) < lockAfterCorrected) {
                    minDate.subtract(1,  'months');
                }
                // // correct lock date for previous month.
                // // e.g. lockTimeAfter = 30, today = 03/01 and we need lock time before 02/28
                lockAfterCorrected = lockAfterValue > parseInt(minDate.daysInMonth()) ? parseInt(minDate.daysInMonth()) : lockAfterValue;
                if (lockAfterCorrected > 1) {
                    minDate.add(lockAfterCorrected- 1, 'days');
                }
                break;
            default:
                minDate = moment();
        }

        return minDate.format('YYYY-MM-DD');
    },

    /**
     * @returns {boolean}
     */
    is24TimeFormat: function() {
        return this.index.dateTimeFormat === ContentScript.Constants.TIME_FORMAT24;
    },

    /**
     *
     * @returns {string|null}
     */
    getMaxDateAllowed: function() {
        if (this.index.team.allowFutureTracking) {
            return null;
        }

        // reflect API BE logic, moment.utc()
        return moment().format('YYYY-MM-DD');
    },

    /**
     * @returns {boolean}
     */
    isSuspended: function() {
        return this.index.isSuspended;
    },

    /**
     * @returns {boolean}
     */
    isAdmin: function() {
        return this.index.role === 'admin';
    },

    /**
     * @returns {boolean}
     */
    isSupervisor: function () {
        return this.index.role === 'supervisor'
    },

    /**
     * @returns {boolean}
     */
    isMember: function () {
        return this.index.role === 'member'
    },

    /**
     * @returns {boolean}
     */
    isMemberLimited: function () {
        return this.index.role === 'member_limited'
    },

    /**
     * @returns {boolean}
     */
    canPurchase: function() {
        return this.isAdmin();
    },

    /**
     * @returns {boolean}
     */
    canExtendTrial: function() {
        return this.index.team.billing.allowedTrialExtension;
    },

    /**
     * @returns {string}
     */
    getTrialMessage: function() {
        if (!this.isAdmin() && !this.canExtendTrial()) {
            return 'Unfortunately, your Everhour trial has ended! Contact team admin.';
        }

        return 'Unfortunately, your Everhour trial has ended!';
    },

    getTrialProgressMessage: function() {
        const { status, trialEndedAt, teamPlanTrialUntil } = this.index.team.billing;
        const todayMessage = `Everhour free trial ends today`;
        const getDaysMessage = (message) => `Everhour free trial ends in ${message}`;
        const getCountDays = (date) => moment(date).diff(moment(), 'days') + 1;

        if (status === 'trial_end') {
            return 'Your Everhour free trial has ended';
        }

        if (status === 'trial') {
            const countDays = getCountDays(trialEndedAt);
            if (countDays >= 8) {
                return null;
            }
            if (countDays <= 1) {
                return todayMessage;
            }

            return getDaysMessage(countDays + ' days');
        }

        if (teamPlanTrialUntil && moment.utc(teamPlanTrialUntil).isAfter(moment.utc())) {
            if (getCountDays(teamPlanTrialUntil) >= 8) {
                return null;
            }
            return getDaysMessage(moment.utc(teamPlanTrialUntil).local().fromNow(true));
        }

        return null;
    },

    /**
     * @returns {promise}
     */
    extendTrial: function() {
        return everhour.extendTrial().then(() => {
            browserPort.settings.load().then((options) => {
                // After extend trial we are sending message to background and reload all tabs.
                browserPort.eventDispatcher.broadcast('background', 'settings:save', options);
            });
        });
    },

    /**
     * @returns {boolean}
     */
    isPlatformSupported: function(platformCode) {
        return this.index.accounts && this.index.accounts.filter(function(account) {
            return account.platform === platformCode;
        }).length > 0;
    },

    /**
     * @param {string} id
     * @returns {boolean}
     */
    isAccountConnected: function(id) {
        return this.index.accounts && this.index.accounts.filter(function(account) {
            return account.id === id && !!~['active', 'new'].indexOf(account.status);
        }).length > 0;
    },

    /**
     * @param {string} platformCode
     * @returns {boolean}
     */
    getPlatformAccountId: function(platformCode) {
        const account = this.index.accounts && this.index.accounts.find(function(account) {
            return account.platform === platformCode;
        });

        return account ? account.id : null;
    },

    /**
     * @param {string} accountId
     * @param {object} settings
     * @returns {void | * | never}
     */
    updateTrelloAccountSetting: function(accountId, settings) {
        const account = this.index.accounts.find(userAccount => userAccount.id == accountId);

        return everhour.updateTrelloAccountSetting(accountId, settings)
            .done((updatedAccount) => {
                account.settings = updatedAccount.settings;
                store.notify();
            });
    },

    getPlatformSettings: function() {
        return this.index.platformSettings;
    },

    /**
     *
     * @param {string} platform
     * @param {object} platformSettings
     */

    setPlatformSettings: function(platform, platformSettings) {
        let settings = null;

        switch (platform) {
            case 'cl':
                let clickUpDefaultSettings = {syncProjectsAs: 'folder'};

                if (platformSettings && platformSettings.settings && platformSettings.settings.syncProjectsAs) {
                    clickUpDefaultSettings.syncProjectsAs = platformSettings.settings.syncProjectsAs;
                }

                settings = clickUpDefaultSettings;
                break;
            case 'as':
                settings = platformSettings && platformSettings.settings;
                break;
            default:
                break;
        }

        this.index.platformSettings = settings;
    },

    completeOnboardingStep: function(featureId) {
        return everhour.lock(`complete_onboarding_step_${featureId}`, () =>
            everhour.completeOnboardingStep({extensionSettings: {[featureId]: true}}));
    },

    /**
     * @param {string} accountId
     * @param {[]<string>} hiddenProjects
     * @returns {void | * | never}
     */
    updateAsanaHiddenProjects: function(accountId, hiddenProjects) {
        return everhour.lock(`update_account_${accountId}_onboarding`, () =>
            everhour.updateAsanaAccountSetting(accountId, {hiddenProjects: hiddenProjects || []})
                .done((updatedAccount) => {
                    const account = this.index.accounts.find((account) => account.id === accountId);
                    account.settings = updatedAccount.settings;

                    return updatedAccount;
                })
        );
    },

    /**
     *
     * @returns {boolean}
     */
    isTimeManagementAllowed: function () {
        return this.timeManagementRoles.indexOf(this.index.role) !== -1;
    },

    /**
     *
     * @param {string} date
     * @param userId
     * @param project
     * @returns {string}
     */
    isEditTimeAllowed(date, userId, project)
    {
        let max = this.getMaxDateAllowed();

        if (max && max < date) {
            return 'locked';
        }

        if (this.isTimeManagementAllowed()) {
            return 'permitted';
        }

        if (project.status !== 'open') {
            return 'locked'
        }

        if (this.index.id !== userId) {
            return 'denied';
        }

        let min = this.getMinDateAllowed();

        if (!min) {
            return 'permitted';
        }

        return date < min ? 'locked' : 'permitted';
    },

    /**
     * @param {Object} action
     */
    reduce: function(action) {
        switch (action.type) {
            case ProfileStore.Act.TEAM_UPDATED:
                this.index.team = Object.assign(
                    {},
                    this.index.team,
                    action.team
                );
                break;

            case ProfileStore.Act.ACCOUNT_REMOVED:
                let removedAccount = this.index.accounts.find(userAccount => action.account.id === userAccount.id);
                if (removedAccount && removedAccount.platform === app.adaptor.platform) {
                    // if removed account belongs to active tab integration
                    // then we must reload data
                    window.location.reload();
                }
                break;

            case ProfileStore.Act.ACCOUNT_UPDATED:
                const updatedAccount = action.account;
                let appropriateAccount = this.index.accounts.find(userAccount => userAccount.id === updatedAccount.id);
                if (!appropriateAccount && updatedAccount.platform === app.adaptor.platform) {
                    // new account added
                    // then reload page for get new data(projects tasks etc)
                    window.location.reload();
                    return;
                }

                let isAccountReconnected = false;
                this.index.accounts = this.index.accounts.map(userAccount => {
                    if (updatedAccount.id === userAccount.id) {
                        // case for update account
                        // sync personal boards in trello changed for example
                        isAccountReconnected = userAccount.status === 'disabled' && updatedAccount.status === 'active';
                        return updatedAccount;
                    }

                    return userAccount;
                });
                if (isAccountReconnected) {
                    // too much nuances with sensitive data
                    // reload solve them all
                    window.location.reload();
                }
                // re-render our controls, case when account settings changed for example
                // our contentScript Adaptors subscription, like  `store.subscribe(() => this.render())
                store.notify();
                break;
        }
    },

    isApplicationEnabled: function(applicationName) {
        return this.index.team && !!this.index.team.applications[applicationName];
    },

    isTimeCardsEnabledForCurrentUser: function() {
        return this.index.team
            && this.index.team.applications
            && this.index.team.applications.timecardsUsers
            && this.index.team.applications.timecardsUsers.length
            && !!~this.index.team.applications.timecardsUsers.indexOf(this.index.id);
    },

    isTimecardsEnabled: function() {
        const isTimecardsEnabledForTeam = this.isApplicationEnabled('timecards');
        const timecardsUsers = this.index.team && this.index.team.applications
            && this.index.team.applications.timecardsUsers || null;

        return (isTimecardsEnabledForTeam && (!timecardsUsers || !timecardsUsers.length)) ||
            (isTimecardsEnabledForTeam && timecardsUsers && timecardsUsers.length && this.isTimeCardsEnabledForCurrentUser());
    },

    isAllowBillingAndBudget: function() {
        return this.index.permissions && this.index.permissions.billingBudgets;
    },

    isAllowCosts: function() {
        return this.index.permissions && this.index.permissions.costs;
    }
};

window.TimerStore = function() {
    this.timer = null;
    this.loadedAt = null;
    this.interval = null;
    this.timecard = null;
};

TimerStore.Act = {
    STARTED: 'api:timer:started',
    BEFORE_STARTED: 'api:timer:before_started',
    STOPPED: 'api:timer:stopped',
    BEFORE_STOPPED: 'api:timer:before_stopped',
    FETCHED: 'api:timer:fetched',
    UPDATED: 'api:timer:updated',
    TASK_TIME_APPEND: 'api:task_time:append',
    TIMECARD_UPDATED: 'api:timecards:updated', // we using it for send to front app

    Started: function(timer) {
        return {
            type: TimerStore.Act.STARTED,
            timer: timer
        }
    },

    BeforeStarted: function(task) {
        return {
            type: TimerStore.Act.BEFORE_STARTED,
            task: task
        }
    },


    Stopped: function(timer) {
        return {
            type: TimerStore.Act.STOPPED,
            timer: timer
        }
    },

    Updated: function(timer) {
        return {
            type: TimerStore.Act.UPDATED,
            timer: timer
        }
    },

    BeforeStopped: function() {
        return {
            type: TimerStore.Act.BEFORE_STOPPED
        }
    },

    TimecardUpdated: function(timecard) {
        return {
            type: TimerStore.Act.TIMECARD_UPDATED,
            timecard: timecard
        }
    }
};

TimerStore.prototype = {
    load: function() {
        var self = this;

        return everhour.timer().then((timer) => {
            self.timer = timer;
            self.loadedAt = timestamp();
            this.calculateAllVirtualProperties();
        });
    },

    get: function() {
        return this.timer;
    },

    timerTick: function(handler) {
        this.clearTimerTick();
        this.interval = setInterval(handler, 500);
    },

    clearTimerTick: function() {
        if (this.interval) {
            clearInterval(this.interval);
        }
    },

    activeTask: function() {
        return this.isActive() ? (this.timer.task ? this.timer.task.id : null) : null;
    },

    activeTimeUrl: function() {
        if (!this.isActive()) {
            return null;
        }

        if (this.timer.website && this.timer.website.url) {
            return this.timer.website.url;
        }

        if (this.timer.currentTaskTime && this.timer.currentTaskTime.website && this.timer.currentTaskTime.website.url) {
            return this.timer.currentTaskTime.website.url;
        }

        return null;
    },

    runningWithoutTask: function() {
        return this.isActive() && !this.timer.task;
    },

    isActive: function() {
        return this.timer && this.timer.status === 'active';
    },

    duration: function() {
        if (!this.timer || this.timer.status !== 'active') {
            return 0;
        }
        return (this.timer.duration + timestamp() - this.loadedAt) + (this.timer.today || 0);
    },

    currentDuration: function() {
        if (!this.timer || this.timer.status !== 'active') {
            return 0;
        }

        return this.timer.duration + timestamp() - this.loadedAt;
    },

    reduce: function(action) {
        switch (action.type) {
            case TimerStore.Act.STARTED:
                this.timer = action.timer;
                this.loadedAt = timestamp();

                if (action.timer.taskTime && typeof TasksStore !== 'undefined') {
                    store.reduce(TasksStore.Act.TaskTimeAppend([action.timer.taskTime]), true);
                }
                this.calculateAllVirtualProperties();

                break;

            case TimerStore.Act.STOPPED:
                this.timer = action.timer;
                this.loadedAt = timestamp();
                this.clearTimerTick();

                if (action.timer.taskTime && typeof TasksStore !== 'undefined') {
                    store.reduce(TasksStore.Act.TaskTimeAppend([action.timer.taskTime]), true);
                }

                this.calculateAllVirtualProperties();

                break;

            case TimerStore.Act.UPDATED:
                this.loadedAt = timestamp();
                this.timer = action.timer;
                this.calculateAllVirtualProperties();
                break;

            case TimerStore.Act.TASK_TIME_APPEND:
                if (this.isActive()) {
                    for (let i = 0; i < action.items.length; i++) {
                        let taskTime = action.items[i];
                        if (this.timer.task && taskTime.user === this.timer.user.id && taskTime.date === this.timer.userDate &&
                            (taskTime.task && this.timer.task && taskTime.task.id === this.timer.task.id ||
                            this.timer.currentTaskTime && taskTime.id === this.timer.currentTaskTime.id
                            )
                        ) {
                            this.timer.today = taskTime.time;
                        }
                    }
                }

                break;

            case TimerStore.Act.TIMECARD_UPDATED:
                this.timecard = action.item || action.timecard;
                break;
        }
    },

    calculateAllVirtualProperties: function() {
        this.timer = this.calculateVirtualProperties(this.timer);
    },

    calculateVirtualProperties: function(timer) {
        if (!timer) {
            return null;
        }

        if (timer.task) {
            if (timer.task.projectsObjects) {
                timer.task.project = store.taskTime && store.taskTime.taskProject(timer.task.projectsObjects);
                timer.task.projectId = timer.task.project ? timer.task.projectId : null;
            } else {
                timer.task.projectId = store.projects && store.projects.taskProject(timer.task.projects);
                timer.task.project = timer.task.projectId && store.projects && store.projects.has(timer.task.projectId) ? store.projects.get(timer.task.projectId) : null;
            }
        }

        return timer;
    },

    start: function(taskId, taskDetails, taskTimeId, comment, website) {
        return everhour.lock('timer', () => {
            browserPort.eventDispatcher.broadcast('self', 'timer:started');
            const timerStartedAt = new Date().getTime();
            // start timer from extension popup doesn't require presync
            // also extension popup doesn't use TaskStore
            // let promiseForHandle = store.tasks && taskId ?
            //     store.tasks.presync(taskId, taskDetails) :
            //     resolvedPromise();

            // return promiseForHandle.then(() => {
                return everhour.startTimer(taskId, taskTimeId, comment, website, taskDetails).done((timer) => {
                    store.dispatch(TimerStore.Act.Started(timer));
                    browserPort.analytics.event('timer_started', {latency: Date.now() - timerStartedAt});
                }).fail((response) => {
                    // allow to reverse timer button state, seems like it's affect nothing
                    store.dispatch(TimerStore.Act.Updated(this.isActive() ? this.timer : null));
                    browserPort.analytics.event('timer_started', {latency: Date.now() - timerStartedAt, reason: 'Timer failed'});
                    return $.Deferred().reject(response);
                });
            // });
        })
    },

    stop: function(id, comment) {
        return everhour.lock('timer', () => {
            const timerStoppedAt = new Date().getTime();
            return everhour.stopTimer(id, comment).then((timer) => {
                store.dispatch(TimerStore.Act.Stopped(timer));
                browserPort.analytics.event('timer_stopped', {latency: Date.now() - timerStoppedAt});
            }).fail(() => {
                browserPort.analytics.event('timer_stopped', {latency: Date.now() - timerStoppedAt, reason: 'Timer failed'});
            });
        });
    },

    comment: function(comment) {
        return everhour.commentTimer({comment : comment}).then(function(timer) {
            store.dispatch(TimerStore.Act.Updated(timer));
        });
    },

    setTimecard: function(timecard) {
        this.timecard = timecard;
    },

    startWebsiteTimer: function(url, comment) {
        return everhour.lock('timer', () => {
            return everhour.startWebsiteTimer(url, comment).done((timer) => {
                store.dispatch(TimerStore.Act.Started(timer));
            }).fail((response) => {
                store.dispatch(TimerStore.Act.Updated(this.isActive() ? this.timer : null));

                return $.Deferred().reject(response);
            });
        })
    }
};

window.IndexVisitor = function(index, defaultValue) {
    this.index = index;
    this.get = function(key) {
        if (!this.index.hasOwnProperty(key)) {
            this.index[key] = defaultValue();
        }

        return this.index[key];
    };

    this.set = function(key, value) {
        this.index[key] = value;
    };

    this.all = function() {
        return this.index;
    }
};

window.TasksStore = function() {
    this.createIndex();
};

TasksStore.Act = {
    TASK_TIME_FETCH_COMPLETED: 'api:task_time:fetch_completed',
    TASK_TIME_APPEND: 'api:task_time:append',
    TASK_TIME_UPDATED: 'api:task_time:updated',
    TASK_PRESYNC_COMPLETED: 'api:task:presync_completed',
    TASK_UPDATE_ESTIMATES_COMPLETED: 'api:tasks:update_estimates_completed',
    TASKS_RELOAD: 'api:tasks:reload',

    TaskTimeAppend: function(items) {
        return {
            type: TasksStore.Act.TASK_TIME_APPEND,
            items: items
        }
    },

    TaskTimeUpdated: function(items) {
        return {
            type: TasksStore.Act.TASK_TIME_UPDATED,
            items: items
        }
    },

    TaskTimeFetched: function() {
        return {
            type: TasksStore.Act.TASK_TIME_FETCHED
        }
    },

    TaskPresyncCompleted: function(task) {
        return {
            type: TasksStore.Act.TASK_PRESYNC_COMPLETED,
            id: task
        }
    },

    TaskUpdateCompleted: function(task, estimate, projectIds) {
        return {
            type: TasksStore.Act.TASK_UPDATE_ESTIMATES_COMPLETED,
            id: task,
            estimate: estimate,
            projects: projectIds
        }
    }
};

TasksStore.prototype = {
    index: null,

    createIndex: function() {
        this.index = {
            total: {}, // [internal id] => { time: number, estimate: number }
            progress: {} // [internal id] => {time: object, estimate: object, today: number}
        };

        this.visitor = {
            total: new IndexVisitor(this.index.total, () => ({
                time: 0,
                estimate: 0,
                children: [],
            })),
            progress: new IndexVisitor(this.index.progress, () => ({
                    time: this.getDefaultTime(),
                    today: 0,
                    estimate: this.getDefaultEstimates(),
                    comment: null,
                    isUnbillable: false
                })
            )
        }
    },

    extractEstimate: function(compactedEstimate) {
        if (!compactedEstimate) {
            return null;
        }

        if (!isNaN(parseFloat(compactedEstimate)) && isFinite(compactedEstimate)) {
            return {
                type: 'overall',
                total: compactedEstimate
            };
        }


        return {
            type: 'users',
            total: Object.keys(compactedEstimate).reduce(function(total, user) {
                return total + compactedEstimate[user];
            }, 0),
            users: compactedEstimate
        };
    },

    extractTime: function(compactedTime) {
        if (!compactedTime) {
            return null;
        }

        return {
            type: 'users',
            total: Object.keys(compactedTime).reduce(function(total, user) {
                return total + compactedTime[user];
            }, 0),
            users: compactedTime
        };
    },

    load: function(platform) {
        this.createIndex();

        const allowRate = this.checkAllowedRate();
        // need .then(..) for handle responseData in callback instead of response obj
        const options = allowRate ? ['withRates=1'] : null;
        const tasksDataRequest = everhour.tasks(platform, options).then(dataResponse => dataResponse);
        // need .then(..) for handle responseData in callback instead of response obj
        const todayProgressRequest = everhour.todayProgress(platform).then(dataResponse => dataResponse);
        return $.when(tasksDataRequest, todayProgressRequest)
            .done((tasksResponse, todayProgressResponse) => {
                // ['task', 'parent_id', 'time', 'estimate', 'is_unbillable'] (time and estimate compacted)
                const progress = tasksResponse.result.progress.data;
                // ['task', 'time', 'comment']
                let today = todayProgressResponse.result.today.data.reduce(function(index, item) {
                    index[item[0]] = {
                        time: item[1],
                        comment: item[2]
                    };

                    return index;
                }, {});

                progress.forEach((row) => {
                    let task = row[0];
                    let parent = row[1];

                    // for debug
                    // if (["as:1200171778977656"].indexOf(task) > -1) {
                    //     console.log(row)
                    // }

                    const time = this.extractTime(row[2]);
                    const estimate = this.extractEstimate(row[3]);
                    const isUnbillable = row[4];
                    const rate = allowRate ? row[5] : null;
                    const total = this.visitor.total.get(task);
                    total.time += time ? time.total : 0;
                    total.estimate += estimate ? estimate.total : 0;

                    let todayTask = today.hasOwnProperty(task) ? today[task] : null;

                    this.visitor.progress.set(task, {
                        time: time || this.getDefaultTime(),
                        estimate: estimate || this.getDefaultEstimates(),
                        today: todayTask ? todayTask.time : 0,
                        comment: todayTask ? todayTask.comment : null,
                        isUnbillable: isUnbillable,
                        rate: rate
                    });

                    // parent
                    if (parent) {
                        if (this.visitor.total.get(parent).children.indexOf(task) === -1) {
                            this.visitor.total.get(parent).children.push(task);
                        }
                    }
                });
            });
    },

    has: function(id) {
        return this.index.total.hasOwnProperty(id);
    },

    /**
     * @param {string} id
     * @returns {null|Object}
     */
    total: function(id) {
        let self = this.index.total[id];

        if (!self) {
            return null;
        }

        let time = self.time;
        let estimate = self.estimate;

        if (!self.children || self.children.length === 0) {
            return {time: time, estimate: estimate};
        }

        let child, subTime = 0, subEstimate = 0;
        for (let i = 0; i < self.children.length; i++) {
            if (self.children[i] === id) {
                console.warn('[EV][TasksStore.js] parent has subtask with same name');
                continue;
            }

            child = this.total(self.children[i]);

            if (!child) {
                continue;
            }

            time += child.time;
            subTime += child.time;
            estimate += child.estimate;
            subEstimate += child.estimate;
        }

        if (time === 0 && estimate === 0) {
            return null;
        }

        return {time: time, estimate: estimate, subtasksData: {time: subTime, estimate: subEstimate}, children: self.children};
    },

    /**
     * @param {string} id
     * @returns {int}
     */
    today: function(id) {
        if (!this.index.progress.hasOwnProperty(id)) {
            return 0;
        }

        return this.index.progress[id].today || 0;
    },

    /**
     * @param {string} parentId
     * @returns {null|Object}
     */
    getSubtasksProgress: function (parentId) {
        let total = this.index.total[parentId];
        if (!total || !total.children && !total.children.length) {
            return null;
        }
        let subtaskIds = total.children;
        let subtasksProgress = {time: {users: {}}, estimate: {users: {}}, totalTime: 0, totalEstimate: 0};

        for (let i = 0; i < subtaskIds.length; i++) {
            let childId = subtaskIds[i];
            let childProgress = this.progress(childId);

            if (!childProgress) {
                continue;
            }
            let childProgressUsers =  _.unique(Object.keys(childProgress.time.users || []).concat(Object.keys(childProgress.estimate.users || [])));
            childProgressUsers.forEach( userId => {
                if (childProgress.time.users.hasOwnProperty(userId)) {
                    if (!subtasksProgress.time.users) {
                        subtasksProgress.time.users = {};
                    }
                    if (!subtasksProgress.time.users.hasOwnProperty(userId)) {
                        subtasksProgress.time.users[userId] = childProgress.time.users[userId];
                    } else {
                        subtasksProgress.time.users[userId] += childProgress.time.users[userId];
                    }
                }

                if (childProgress.estimate.type === 'users') {
                    if (childProgress.estimate.users.hasOwnProperty(userId)) {
                        if (!subtasksProgress.estimate.users.hasOwnProperty(userId)) {
                            subtasksProgress.estimate.users[userId] = childProgress.estimate.users[userId];
                        } else {
                            subtasksProgress.estimate.users[userId] += childProgress.estimate.users[userId];
                        }
                    }
                }
            });
        }

        return subtasksProgress
    },

    progress: function(id) {
        return this.index.progress[id];
    },

    /**
     *
     * @param {string} taskId
     * @param {*} date
     * @param {int} time
     * @param {object} taskDetails
     * @param {string} comment
     * @param {number} userId
     * @returns {*}
     */
    addTime: function(taskId, date, time, taskDetails, comment, userId) {
        return everhour.lock('add_time', () => {
            // return store.tasks.presync(taskId, taskDetails).then(function () {
                let data = {
                    date: date,
                    time: time,
                    task: taskId ? taskId : null,
                    taskData: taskDetails
                };

                if (typeof(userId) !== 'undefined' && userId) {
                    data.user = userId;
                }

                if (comment) {
                    data.comment = comment;
                }

                return everhour.createTaskTime(data).done((taskTime) => {
                    store.dispatch(TasksStore.Act.TaskTimeAppend([taskTime]));
                });
            // });
        });
    },

    /**
     *
     * @param {string} taskTimeId
     * @returns {null}
     */
    updateTaskTime: function(taskTimeId, data) {
        return everhour.lock('update_time', () => {
            return everhour.updateTaskTime(taskTimeId, data);
        });
    },

    /**
     *
     * @param {string} taskTimeId
     * @returns {null}
     */
    deleteTaskTime: function(taskTimeId) {
        return everhour.lock('delete_time', () => {
            return everhour.deleteTaskTime(taskTimeId);
        });
    },

    /**
     *
     * @param {string} taskId
     * @param {*} date
     * @param {object} taskDetails
     * @param {string} comment
     * @returns {*}
     */
    updateComment: function(taskId, date, taskDetails, comment) {
        return everhour.lock('update_time_comment', () => {
            return store.tasks.presync(taskId, taskDetails).then(function () {
                let data = {
                    date: date,
                    comment: comment
                };
                return everhour.updateTime(taskId, data).then((taskTime) => {
                    store.dispatch(TasksStore.Act.TaskTimeAppend([taskTime]));
                });
            });
        });
    },

    /**
     *
     * @param {string} taskId
     * @returns {*}
     */
    timeLog: function(taskId) {
        const total = this.total(taskId);

        if (!total || total.time === 0) {
            return $.Deferred().resolve([]);
        }

        return everhour.taskTime(taskId);
    },

    timePartialLog: function(taskId, limit, page) {
        const total = this.total(taskId);

        if (!total || total.time === 0) {
            return $.Deferred().resolve([]);
        }

        return everhour.taskTime(taskId, limit, page);
    },

    /**
     *
     * @param {[]<Object>} taskTimeEntries
     * @param {string} taskId
     * @param {object|null} taskDetails
     * @returns {*}
     */
    bulkUpdateTime: function(taskTimeEntries, taskId, taskDetails) {
        return everhour.lock('bulk_update_time', () => {
            return store.tasks.presync(taskId, taskDetails).then(() => {
                return everhour.bulkUpdateTime(taskTimeEntries).then((history) => {
                    if (!history || history.length === 0) {
                        return;
                    }

                    history.forEach(taskTime => {
                        store.dispatch(TasksStore.Act.TaskTimeAppend([taskTime]));
                    });
                });
            });
        });
    },

    reduce: function(action) {
        switch (action.type) {
            case TasksStore.Act.TASK_TIME_FETCH_COMPLETED:
            case TasksStore.Act.TASK_TIME_APPEND:
                let user = store.profile.get().id;
                let taskTime, progress;
                let projects = [];
                // proceed max 10 items to prevent extension lacks when user refresh everhour.com page
                for (let i = 0; i < action.items.slice(0, 10).length; i++) {
                    taskTime = action.items[i];
                    const taskId = taskTime.task ? taskTime.task.id : null;
                    if (!taskId) {
                        continue;
                    }
                    projects = projects.concat(taskTime.task.projects);
                    // if user may manage time of team members
                    if (parseInt(taskTime.user) === parseInt(user) || store.profile.isTimeManagementAllowed()) {
                        progress = this.visitor.progress.get(taskId);

                        if (!progress) {
                            this.setDefaultTaskProgress(taskId);
                        }

                        progress.time = taskTime.task.time || this.getDefaultTime();
                        progress.estimate = taskTime.task.estimate || this.getDefaultEstimates();

                        if (taskTime.user === user && taskTime.date === moment().format('YYYY-MM-DD')) {
                            progress.today = (taskTime.time || 0);
                            // TODO check case if time was deleted but comments was written before
                            progress.comment = taskTime.time !== 0 ? taskTime.comment || null : null;
                        }
                    }

                    this.rebuildTotal(taskId, taskTime.task.parentId);
                }

                if (action.type === TasksStore.Act.TASK_TIME_APPEND) {
                    store.dispatch(ProjectsStore.Act.InvalidateTotals(_.unique(projects)), true);
                }

                break;

            case TasksStore.Act.TASK_PRESYNC_COMPLETED:
                this.setDefaultTaskProgress(action.id);
                break;

            case TasksStore.Act.TASK_UPDATE_ESTIMATES_COMPLETED:
                this.setTaskEstimate(action.id, action.estimate);
                this.rebuildTotal(action.id);
                store.dispatch(ProjectsStore.Act.InvalidateTotals(action.projects), true);
                break;
        }
    },

    setDefaultTaskProgress: function (taskId) {
        let progress = this.visitor.progress.get(taskId);

        if (!progress) {
            this.visitor.progress.set(taskId, {
                time: this.getDefaultTime(),
                estimate: this.getDefaultEstimates(),
                today: 0
            });
        }
    },

    setTaskEstimate: function(taskId, estimates) {
        if (!estimates) {
            estimates = this.getDefaultEstimates();
        }

        this.visitor.progress.get(taskId).estimate = estimates;
    },

    setTaskBilling: function (taskId, isUnbillable, rate) {
        const task = this.visitor.progress.get(taskId);
        task.isUnbillable = isUnbillable;
        task.rate = rate;
    },

    /**
     * @param {string} taskId
     * @param {string|null} parentId
     */
    rebuildTotal: function(taskId, parentId = null) {
        let progress = this.visitor.progress.get(taskId);
        let total = this.visitor.total.get(taskId);

        total.time = progress.time.total;
        total.estimate = progress.estimate.total;

        if (parentId) {
            let parentTotal = this.visitor.total.get(parentId);

            if (parentTotal.children.indexOf(taskId) === -1) {
                parentTotal.children.push(taskId);
            }
        }
    },

    getDefaultEstimates: function() {
        return {
            total: 0,
            type: 'overall',
            users: {}
        };
    },

    getDefaultTime: function() {
        return {
            total: 0,
            type: 'users',
            users: {}
        };
    },

    /**
     * Can help when subtask progress is exist and we aren't detect parent task from markup
     * @param {string} taskId
     * @returns {string}
     */
    getParentId: function(taskId) {
        let parentId = null;
        Object.keys(this.index.total).forEach((id) => {
            if (this.index.total[id].children.indexOf(taskId) > -1) {
                parentId = id;
            }
        });

        return parentId;
    },

    /**
     * @param {string} taskId
     * @param {Object|null} taskDetails
     * @param {Object} estimate
     * @returns {*}
     */
    setEstimate: function(taskId, taskDetails, estimate) {
        return everhour.lock('set_estimate_' + taskId, () => {
            return store.tasks.presync(taskId, taskDetails).then(() => {
                return everhour.setEstimate(taskId, estimate).then((estimate) => {
                    store.dispatch(TasksStore.Act.TaskUpdateCompleted(taskId, estimate, taskDetails.projects));
                    // dispatch for erase current total for projects
                    // actual for Asana, nothing happens for the rest integrations
                });
            });
        });
    },

    /**
     * @param {string} taskId
     * @param {Object} taskDetails
     * @returns {*|PromiseLike<T>|Promise<T>}
     */
    presync: function(taskId, taskDetails) {
        if (!taskDetails || this.progress(taskId)) {
            return resolvedPromise();
        }

        // disable skip presync logic
        // if (taskId.substring(0, 2) === 'as') {
        //     this.createNewTask(taskId, taskDetails.parentId);
        //     return resolvedPromise();
        // }

        return everhour.lock(taskId, () => {
            return everhour.presync(taskId, taskDetails).then(() => {
                this.createNewTask(taskId, taskDetails.parentId);
                store.reduce(TasksStore.Act.TaskPresyncCompleted(taskId), true);
            });
        });
    },

    /**
     * @param {string} taskId
     * @param {Object} parentId
     * @returns {*}
     */
    createNewTask: function(taskId, parentId) {
        if (!this.has(taskId)) {
            this.index.total[taskId] = {time: 0, estimate: 0, children: []};
        }
        if (parentId) {
            // append new subtask to parent's progress immediately
            if (!this.has(parentId)) {
                this.index.total[parentId] = {time: 0, estimate: 0, children: []}
            }
            if (this.index.total[parentId].children.indexOf(taskId) < 0) {
                this.index.total[parentId].children.push(taskId);
            }
        }

        store.reduce(TasksStore.Act.TaskPresyncCompleted(taskId), true);
    },

    /**
     * @param {string} taskId
     * @param {object} taskDetails
     * @param {boolean} isUnbillable
     * @param {number | null} rate
     * @returns {*}
     */
    updateBillingSettings: function(taskId, taskDetails, isUnbillable, rate) {
        return everhour.lock('update_task_billing', () => {
            return this.presync(taskId, taskDetails).then(() => {
                const requestBody = {unbillable: isUnbillable};

                if (this.checkAllowedRate()) {
                    requestBody.rate = rate;
                }

                return everhour.updateTaskBillingSettings(taskId, requestBody).then((task) => {
                    task.isUnbillable = isUnbillable;

                    if (_.has(requestBody, 'rate') && rate) {
                        task.rate = rate;
                    }

                    this.setTaskBilling(taskId, task.isUnbillable, task.rate);
                    return task;
                })
            });
        });

    },

    checkAllowedRate: function () {
        return store.profile.isAdmin() && !!~['as', 'cl', 'b3', 'mo', 'tr', 'td', 'no'].indexOf(app.adaptor.platform);
    }
};

window.UsersStore = function() {
    this.index = {}; // internal id => user details
    this.loaded = false;
};

UsersStore.prototype = {
    load: function() {
        if (this.loaded) {
            return $.Deferred().resolve();
        }

        let self = this;

        return everhour.users().then(function(users) {
            self.index = _.indexBy(users, 'id');
            self.loaded = true;
        });
    },

    has: function(id) {
        return this.index.hasOwnProperty(id);
    },

    get: function(id) {
        return this.index[id];
    },

    all: function() {
        return this.index;
    },

    /**
     *
     * @param {boolean} isLoadingRequired
     * @returns {*}
     */
    loadBase64: function (isLoadingRequired) {
        if (!isLoadingRequired) {
            return resolvedPromise();
        }

        let usersBase64Loading = Object.values(this.index).map(user => {

            return !user.avatarUrl || user.avatarUrl.match(/https:\/\/avatars\d*\.githubusercontent\.com\//) ?
                resolvedPromise(user) :
                convertImageUrlToBase64Data(user.avatarUrl).then((base64ImgData) => {
                    if (base64ImgData === null) {
                        // if image loading failed for a unknown reason
                        user.avatarUrl = null;
                    } else {
                        user.base64Avatar = base64ImgData;
                    }

                    return user;
                });
        });

        return $.when.apply(this, usersBase64Loading);
    }
};
window.TaskTimeStore = function() { // usage:popup
    this.createIndex();
};

TaskTimeStore.prototype = {
    index: null,

    load: function() {
        this.from = moment().endOf('week').subtract(4, 'week').startOf('week').format('YYYY-MM-DD');
        this.to = moment().endOf('week').format('YYYY-MM-DD');

        return everhour.activity(this.from, this.to, null).then(taskTime => this.rebuild(taskTime));
    },

    createIndex: function() {
        this.taskTime = null;

    },

    rebuild: function(taskTimeData) {
        this.createIndex();

        let applicationUrl = everhour.productionUrl;

        this.taskTime = taskTimeData
            .filter(taskTime => (!taskTime.task
                || taskTime.task && taskTime.task.name !== ''
                || taskTime.task && taskTime.task.name === '' && !!~taskTime.task.id.indexOf('as'))
                && taskTime.time > 0)
            .map(taskTime => {
                const taskId = taskTime.task ? taskTime.task.id : null;

                if (taskId && taskId.split(':')[0] === 'ev' && !taskTime.task.url) {
                    taskTime.task.url = applicationUrl + '/#/projects/' + taskTime.task.projects[0] + '(view:' + taskId + ')';
                }

                return taskTime;
            });
    },

    calculateAllVirtualProperties: function() {
        this.taskTime = this.taskTime.map(taskTime => this.calculateVirtualProperties(taskTime));
    },

    calculateVirtualProperties: function(taskTime) {
        if (taskTime.task) {
            if (taskTime.task.projectsObjects) {
                taskTime.task.projectsObjects = taskTime.task.projectsObjects.filter((project) => !!project);
                taskTime.task.project = this.taskProject(taskTime.task.projectsObjects);
                taskTime.task.projectId = taskTime.task.project ? taskTime.task.project.id : null;
            } else {
                taskTime.task.projectId = store.projects.taskProject(taskTime.task.projects);
                taskTime.task.project = taskTime.task.projectId && store.projects.has(taskTime.task.projectId) ? store.projects.get(taskTime.task.projectId) : null;
            }
        }

        return taskTime;
    },

    taskProject: function(projects) {
        if (!projects || projects && projects.length === 0) {
            return null;
        }

        for (let i = 0; i < projects.length; i++) {
            if (projects[i].status === 'open') {
                return projects[i];
            }
        }

        for (let i = 0; i < projects.length; i++) {
            if (projects[i].foreign) {
                return projects[i];
            }
        }

        return projects[0];
    },

    getRecentTasks: function() {
        let timerTaskId = store.timer.activeTask();

        let tasks = this.taskTime
            .reduce((tasks, taskTime) => {
                const taskId = taskTime.task ? taskTime.task.id : null;

                if (!taskId ) {
                    return tasks;
                }

                if (!tasks.hasOwnProperty(taskId)) {
                    tasks[taskId] = Object.assign({}, taskTime.task, {
                        usedAt: taskTime.date,
                        taskTimeId: taskTime.id
                    })
                } else if (taskTime.date > tasks[taskId].usedAt) {
                    tasks[taskId].usedAt = taskTime.date;
                    tasks[taskId].taskTimeId = taskTime.id;
                }

                return tasks;
            }, {});

        return Object.keys(tasks)
            // .filter(taskId => taskId !== timerTaskId)
            .sort((a, b) => {
                if (tasks[a].usedAt !== tasks[b].usedAt) {
                    return tasks[b].usedAt.localeCompare(tasks[a].usedAt);
                }

                return tasks[b].taskTimeId - tasks[a].taskTimeId;
            })
            .slice(0, 20)
            .map(id => tasks[id]);
    },

    getUserActivity: function() {
        const currentUserId = parseInt(store.profile.index.id);
        let tasksTime = this.taskTime ? [].concat(this.taskTime) : [];
        const timer = Object.assign({}, store.timer.get());

        // add fake taskTime for current timer if timer didn't started for current date
        let taskTimeIndex = tasksTime.findIndex((taskTime) => (timer.currentTaskTime && taskTime.time)
            && (taskTime.id === timer.currentTaskTime.id)
            && (moment().format('YYYY-MM-DD') === taskTime.date));
        let isRunningTimer = timer && timer.status && timer.status !== 'stopped';

        if ((isRunningTimer) && (taskTimeIndex === -1)) {
            timer.user = timer.user.id;
            timer.date = timer.userDate;
            timer.time = store.timer.duration();
            timer.isCurrent = true;
            tasksTime.unshift(timer);
            tasksTime = tasksTime.filter((taskTime) => taskTime.time);
        } else if ((isRunningTimer) && (taskTimeIndex > -1)) {
            timer.isCurrent = true;
            let project = tasksTime[taskTimeIndex] && tasksTime[taskTimeIndex].task && tasksTime[taskTimeIndex].task.project;
            let projectId = tasksTime[taskTimeIndex] && tasksTime[taskTimeIndex].task && tasksTime[taskTimeIndex].task.projectId;
            let taskTime = Object.assign({isCurrent: true}, tasksTime[taskTimeIndex], timer.currentTaskTime);

            if (taskTime.task) {
                taskTime.task.project = project;
                taskTime.task.projectId = projectId;
            }

            tasksTime.splice(taskTimeIndex, 1, taskTime);
        } else {
            tasksTime = tasksTime.filter((taskTime) => taskTime.id && taskTime.time);
        }

        let days = (tasksTime || [])
            .filter(taskTime => parseInt(taskTime.user) === currentUserId)
            .reduce((days, taskTime) => {
                if (!days.hasOwnProperty(taskTime.date)) {
                    days[taskTime.date] = {
                        time: 0,
                        records: [],
                    }
                }

                days[taskTime.date].time += taskTime.time ? taskTime.time : 1;
                days[taskTime.date].records.push(taskTime);

                return days;
            }, {});

        let dates = Object.keys(days);

        dates.sort();
        dates.reverse();

        let bufferWeeks = {};
        let weeks = dates.reduce((activity, date) => {
            let week = moment(date).startOf('week').format('YYYY-MM-DD');

            days[date].records.sort((a, b) => {
                if (a.date === b.date) {
                    return b.id - a.id;
                }

                return b.date > a.date ? 1 : -1;
            });
            if (!bufferWeeks.hasOwnProperty(week)) {
                bufferWeeks[week] = {
                    time: 0,
                    days: [],
                    weekDays: [],
                    daysIndex: {}
                }
            }

            bufferWeeks[week].time += days[date].time;
            bufferWeeks[week].days.push({
                date: date,
                time: days[date].time,
                records: days[date].records,
            });

            bufferWeeks[week].daysIndex[date] = bufferWeeks[week].days.length - 1;
            // skip empty weeks in activity
            if (bufferWeeks[week].time > 0) {
                activity[week] = _.clone(bufferWeeks[week]);
            }

            return activity;
        }, {});


        const workdays = store.profile.team().workdays || ['MON', 'TUE', 'WED', 'THU', 'FRI'];

        Object.keys(weeks).forEach((week) => {
            let days = [];
            let firstDay = moment(week).startOf('week');
            let lastDay = moment(week).endOf('week');

            let activityWeek = weeks[week];

            for (let date = firstDay; date <= lastDay; date = moment(date).add(1, 'day')) {
                let day = date.format('YYYY-MM-DD');
                let weekDay = date.format('ddd').toString().toUpperCase();
                let dayIndex = activityWeek.daysIndex.hasOwnProperty(day) ? activityWeek.daysIndex[day] : -1;

                if (workdays.indexOf(weekDay) !== -1 || dayIndex !== -1) {
                    days.push({
                        day: day,
                        time: dayIndex !== -1 ? activityWeek.days[dayIndex].time : 0
                    });
                }
            }

            return weeks[week].weekDays = days;
        });

        if (store.timer && store.timer.isActive()) {
            weeks = this.mergeTimerInActivity(weeks, store.timer.get());
        }

        return weeks;
    },

    mergeTimerInActivity: function(activity, timer)
    {
        const timerDuration = store.timer.currentDuration();

        if (timerDuration === 0) {
            return activity;
        }

        /**
         * Duration that we include into week/day summary should consider time rounding
         */
        const todayTaskTime = timer.today || 0;
        const duration = _format.timeRound(timerDuration + todayTaskTime, 60) - todayTaskTime;

        if (duration === 0) {
            return activity;
        }

        const taskId = timer.task ? timer.task.id : null;
        const week = moment(timer.userDate).startOf('week').format('YYYY-MM-DD');
        const day = timer.userDate;

        if (!activity.hasOwnProperty(week)) {
            return activity;
        }

        let seconds = timerDuration % 60;
        let secondsCompensation = 0;

        if (!todayTaskTime && duration >= 90 && seconds >= 30) {
            secondsCompensation += 30;
        }

        if (!todayTaskTime && duration <= 60 && timerDuration < 60) {
            secondsCompensation += 60;
        }

        if (todayTaskTime && seconds >= 30) {
            secondsCompensation += duration + 30;
        } else if (todayTaskTime && seconds < 30) {
            secondsCompensation += duration;
        }

        activity[week].time += secondsCompensation;

        if (activity[week].daysIndex.hasOwnProperty(day)) {

            activity[week].weekDays.forEach((weekDay) => {
                if (weekDay.day !== day) {
                    return;
                }

                weekDay.time += secondsCompensation;
            });

            activity[week].days.forEach((weekDay) => {
                if (weekDay.date !== day) {
                    return;
                }

                weekDay.time += secondsCompensation;

                // we have to clone records since they stored in store
                weekDay.records = weekDay.records.map((taskTime) => {
                    if (taskTime.task && taskTime.task.id !== taskId) {
                        return taskTime;
                    }

                    if (!taskTime.task) {
                        return Object.assign({}, taskTime, {
                            time: taskTime.time
                        })
                    }

                    return Object.assign({}, taskTime, {
                        // time: taskTime.time + duration
                        time: taskTime.time
                    });
                })
            });
        }

        return activity;
    },

    /**
     *
     * @param {*} date
     * @param {int} time
     * @param {string} comment
     * @returns {*}
     */
    createTaskTime: function(data) {
        return everhour.lock('create_time', () => {
            return everhour.createTaskTime(data).done(function (taskTime) {
                store.dispatch(TasksStore.Act.TaskTimeAppend([taskTime]));
            });
        });
    },

    updateTaskTIme: function(taskTimeId, data) {
        return everhour.lock('edit_time', () => {
            return everhour.updateTaskTime(taskTimeId, data).then(function (taskTime) {
                store.dispatch(TasksStore.Act.TaskTimeAppend([taskTime]));
            });
        });
    },

    // /**
    //  *
    //  * @param {string} taskId
    //  * @param {*} date
    //  * @param {int} time
    //  * @param {string} comment
    //  * @returns {*}
    //  */
    // addTime: function(taskId, date, time, comment) {
    //     let data = {
    //         date: date,
    //         time: time
    //     };
    //
    //     if (comment) {
    //         data.commentLine = comment;
    //     }
    //
    //     return everhour.lock('add_time', () => {
    //         return everhour.addTime(taskId, data).done(function (taskTime) {
    //             store.dispatch(TasksStore.Act.TaskTimeAppend([taskTime]));
    //         });
    //     });
    // },
    // /**
    //  *
    //  * @param {string} taskId
    //  * @param {Object} updatedTimeEntry
    //  * @returns {*}
    //  */
    // editTime: function(taskId, updatedTimeEntry) {
    //     let data = {
    //         user: updatedTimeEntry.user,
    //         date: updatedTimeEntry.date,
    //         time: updatedTimeEntry.time,
    //         comment: updatedTimeEntry.comment
    //     };
    //
    //     return everhour.lock('edit_time', () => {
    //         return everhour.updateTime(taskId, data).then(function (taskTime) {
    //             store.dispatch(TasksStore.Act.TaskTimeAppend([taskTime]));
    //         });
    //     });
    // },

    deleteTime: function(taskTimeId) {
        const findIndex = this.taskTime.findIndex((taskTime) => taskTime.id === taskTimeId);

        if (findIndex > -1) {
            this.taskTime.splice(findIndex, 1);
            this.rebuild(this.taskTime);
            this.calculateAllVirtualProperties();
        }
    },

    reduce: function(action) {
        switch (action.type) {
            case TasksStore.Act.TASK_TIME_APPEND:
                if (!action.items || action.items.length === 0 || !this.taskTime) {
                    return;
                }

                for (let i = 0; i < action.items.length; i++) {
                    let newTaskTime = action.items[i];

                    // let existsTaskTime = this.taskTime.find(
                    //     taskTime =>
                    //         taskTime.date === newTaskTime.date &&
                    //         taskTime.user === newTaskTime.user && (
                    //             taskTime.task && newTaskTime.task &&
                    //             taskTime.task.id === newTaskTime.task.id ||
                    //             newTaskTime.id === taskTime.id
                    //         )
                    // );

                    let existsTaskTime = this.taskTime.find(taskTime => taskTime.id === newTaskTime.id);

                    if (existsTaskTime) {
                        existsTaskTime.time = newTaskTime.time;
                        existsTaskTime.comment = newTaskTime.comment;

                        if (!existsTaskTime.task && newTaskTime.task) {
                            existsTaskTime.task = newTaskTime.task;
                        }
                    } else if (newTaskTime.date >= this.from && newTaskTime.date <= this.to) {
                        this.taskTime.push(this.calculateVirtualProperties(newTaskTime));
                    }
                }
                break;
            case TasksStore.Act.TASK_TIME_UPDATED:
                if (!action.items || action.items.length === 0 || !this.taskTime) {
                    return;
                }

                for (let i = 0; i < action.items.length; i++) {
                    let newTaskTime = action.items[i];

                    let taskTimeIndex = this.taskTime.findIndex(
                        taskTime =>
                            taskTime.date === newTaskTime.date &&
                            taskTime.user === newTaskTime.user && (
                                taskTime.task && newTaskTime.task &&
                                taskTime.task.id === newTaskTime.task.id ||
                                newTaskTime.id === taskTime.id
                            )
                    );

                    if (taskTimeIndex > -1) {
                        this.taskTime[taskTimeIndex] = newTaskTime;
                    }
                }
                break;
        }
    }
};

window.TimeExportStore = function() { // usage:popup
    this.createIndex();
};

TimeExportStore.prototype = {
    time: null,
    months: null,
    countMonth: 2,

    load: function() {
        if (this.isLoaded()) {
            return $.Deferred().resolve();
        }

        return everhour.exportTime(
            moment().subtract(this.countMonth, 'month').startOf('month').format('YYYY-MM-DD'),
            moment().endOf('month').format('YYYY-MM-DD'),
            store.profile.get().id
        ).then(time => this.rebuild(time));
    },

    isLoaded: function() {
        return this.months !== null;
    },

    createIndex: function() {
        this.time = null;
        this.months = null;
    },

    rebuild: function(timeData) {
        this.createIndex();

        if (store.timer && store.timer.isActive() && store.timer.activeTask()) {
            const duration = _format.timeRound(store.timer.duration(), 60) - (store.timer.get().today || 0);

            if (duration > 0) {
                timeData = timeData.slice(0);
                timeData.push({
                    date: store.timer.get().userDate,
                    time: duration,
                })
            }
        }

        this.time = timeData.reduce((time, item) => {
            time[item.date] = (time[item.date] || 0) + item.time;
            return time;
        }, {});

        this.months = this.buildMonths(this.time);
    },

    buildMonths: function(time) {
        let months = [];
        for (let i = 0; i < this.countMonth + 1; i++) {
            let start = moment().subtract(i, 'month').startOf('month');
            let end = moment().subtract(i, 'month').endOf('month');

            let month ={
                monthName: start.format('MMMM'),
                start: start.format('YYYY-MM-DD'),
                end: end.format('YYYY-MM-DD'),
                weeks: [],
                total: 0
            };

            let week = {
                days: [],
                total: 0,
                period: null
            };

            let weekday = start.weekday();

            // prepend first week
            for (let i = 0; i < weekday; i++) {
                // const date = moment(start).startOf('week').add(i, 'day').format('YYYY-MM-DD');
                //
                // week.days.push({
                //     date: date,
                //     day: moment(date).format('DD'),
                //     time: time[date] || 0
                // });
                // week.total += time[date] || 0;
                week.days.push(null);
            }

            for (let day = moment(start); day <= end; day = day.add(1, 'day')) {
                let date = day.format('YYYY-MM-DD');

                week.days.push({
                    date: date,
                    day: day.format('DD'),
                    time: time[date] || 0
                });
                week.total += time[date] || 0;

                month.total += time[date] || 0;

                if (weekday % 7 === 6) {
                    week.period = this.getWeekPeriod(start, end, week);
                    month.weeks.push(week);
                    week = {
                        days: [],
                        total: 0,
                        period: null
                    };
                }

                weekday++;
            }

            // append last week
            if (week.days.length > 0) {
                for (let i = 0; week.days.length < 7; i++) {
                    const date = moment(end).add(i + 1, 'days').format('YYYY-MM-DD');

                    // week.days.push({
                    //     date: date,
                    //     day: moment(date).format('DD'),
                    //     time: time[date] || 0
                    // });
                    // week.total += time[date] || 0;
                    week.days.push(null)
                }

                week.period = this.getWeekPeriod(start, end, week);

                month.weeks.push(week);
            }

            months.push(month);
        }

        return months;
    },

    getWeekPeriod(start, end, week) {
        const days = [].concat(week.days).filter((day) => day);
        const startWeekMillis = moment(days[0].date).valueOf();
        const endWeekMillis = moment(days[days.length - 1].date).valueOf();
        const startMonthMillis = start.valueOf();
        const endMonthMillis = moment(end).endOf('day').valueOf();
        let period = '';

        if (startWeekMillis >= startMonthMillis && endWeekMillis < endMonthMillis) {
            period = `${moment(startWeekMillis).format('MMM D')} - ${moment(endWeekMillis).format('D')}`
        }

        if (startWeekMillis < startMonthMillis || endWeekMillis > endMonthMillis) {
            period = `${moment(startWeekMillis).format('MMM D')} - ${moment(endWeekMillis).format('MMM D')}`
        }

        if (startMonthMillis === endWeekMillis) {
            period = `${moment(startMonthMillis).format('MMM D')}`;
        }

        if (endMonthMillis === moment(startWeekMillis).endOf('day').valueOf()) {
            period = `${moment(endMonthMillis).format('MMM D')}`
        }

        return period;
    },

    reduce: function(action) {
        switch (action.type) {
            case TasksStore.Act.TASK_TIME_APPEND:
                if (!action.items || action.items.length === 0 || !this.time) {
                    return;
                }

                this.createIndex();
                break;
        }
    },
};
window.TaskDetailsStore = function() { // usage:popup
    this.createIndex();
};

TaskDetailsStore.prototype = {
    tasks: null,

    load: function(taskId) {

        if (this.isLoaded(taskId)) {
            return $.Deferred().resolve(this.tasks[taskId]);
        }

        return everhour.task(taskId).then(task => {
            const requests = [
                store.users.load(),
                everhour.taskTime(taskId)
            ];

            task.projects.forEach(project => {
                requests.push(everhour.projectById(project));
            });

            return $.when.apply(this, requests).then((users, history, ...projects) => {
                let projectsMerged = projects.map((project) => project[0]);
                return this.rebuild(task, history[0], projects[0], projectsMerged);
            });
        });
    },

    getTask(taskId) {
        return this.tasks[taskId];
    },

    removeTask: function(taskId) {
        delete this.tasks[taskId];
    },

    isLoaded: function(taskId) {
        return this.tasks.hasOwnProperty(taskId);
    },

    createIndex: function() {
        this.tasks = {};
    },

    rebuild: function(task, history, projects, projectsAll) {
        let collaborators = [];

        if (task.time && task.time.users) {
            collaborators = collaborators.concat(Object.keys(task.time.users));
        }

        if (task.estimate && task.estimate.users) {
            collaborators = collaborators.concat(Object.keys(task.estimate.users));
        }

        if (!task.url && task.id.split(':')[0] === 'ev') {
            task.url = everhour.productionUrl + '/#/projects/' + task.projects[0] + '/list(view:' + task.id + ')';
        }

        let me = store.profile.get().id;
        let today = moment().format('YYYY-MM-DD');

        this.tasks[task.id] = {
            task: task,
            projects: projects,
            projectsAll: projectsAll,
            history: history,
            collaborators: collaborators
                .filter((value, index, self) => self.indexOf(value) === index)
                .map(id => store.users.get(id)),
            todayHistory: history.find(item => item.user === me && item.date === today && item.time > 0) || null
        };

        return this.tasks[task.id];
    },

    reduce: function(action) {
        switch (action.type) {
            case TasksStore.Act.TASK_TIME_APPEND:
                if (!action.items || action.items.length === 0 || !this.tasks) {
                    return;
                }

                for (let i = 0; i < action.items.length; i++) {
                    const newTaskTime = action.items[i];
                    const taskId = newTaskTime.task ? newTaskTime.task.id : null;
                    if (!taskId || !this.tasks.hasOwnProperty(taskId)) {
                        continue;
                    }

                    let existsTaskTime = this.tasks[taskId].history.find(
                        taskTime =>
                            taskTime.date === newTaskTime.date &&
                            taskTime.user === newTaskTime.user
                    );

                    if (existsTaskTime) {
                        existsTaskTime.time = newTaskTime.time;
                    } else {
                        this.tasks[taskId].history.push(newTaskTime);
                    }

                    this.rebuild(newTaskTime.task,  this.tasks[taskId].history, this.tasks[taskId].projects);
                }
                break;

            case TasksStore.Act.TASK_UPDATE_ESTIMATES_COMPLETED:
                if (!this.tasks[action.id]) {
                    return;
                }

                this.tasks[action.id].task.estimate = action.estimate;
                // we may remove cached data for force invalidate, but not sure that's correct way
                // delete this.tasks[action.id];
                break;
        }
    },

    /**
     *
     * @param {string} taskId
     * @param {Object} estimate
     * @returns {*|PromiseLike<T>|Promise<T>}
     */
    setEstimate: function(taskId, estimate) {
        return everhour.lock('estimate', () => {
            return everhour.setEstimate(taskId, estimate).then(function(estimate) {
                store.dispatch(TasksStore.Act.TaskUpdateCompleted(taskId, estimate));
            });
        });
    }
};

window.Store = function() {
    this.projects = new ProjectsStore();
    this.users = new UsersStore();
    this.profile = new ProfileStore();
    this.tasks = null;
    this.timer = new TimerStore();
    this.teamApplications = new TeamApplicationsStore();
    this.teamStatus = new TeamStatusStore();
    this.taskTime = null;
    this.timeExport = null;
    this.taskDetails = null;
    this.select = new Selectors(this);
    this.clients = new ClientsStore();

    this.listeners = [];
};

Store.prototype = {
    internalActions: ['api:projects:project_invalidate_total'],

    isAuthorized: function() {
        return everhour.isAuthorized();
    },

    loadSelectorsDictionary: function(platform, version) {
        return everhour.loadSelectorsDictionary(platform, version).then(dictionary => Store.dictionary = dictionary || {});
    },

    load: function(platform, isRequireBase64) {
        this.tasks = new TasksStore();

        if (!this.isAuthorized()) {
            console.log('[EV][Store] user isn\'t authorized');
            return resolvedPromise();
        }

        return $.when(
            this.profile.load()
        ).then(() => {
            console.log('[EV][Store] profile loaded');

            if (this.profile.isSuspended()) {
                console.log('[EV][Store] profile is suspend');
                browserPort.eventDispatcher.broadcast('background', 'sync:profile-suspended', null);
                return $.when.apply(this, []);
            }

            if (!store.profile.isPlatformSupported(platform)) {
                console.log('[EV][Store] platform not connected');
                return $.when.apply(this, []);
            }

            browserPort.eventDispatcher.broadcast('background', 'sync:profile-active', null);
            let requests = [
                this.profile.loadPolicy(),
                this.projects.load(platform, false, 'content-script:store-load'),
                this.users.load(),
                this.tasks.load(platform),
                this.timer.load(),
            ];
            if (platform === 'jr') {
                requests.push(this.teamApplications.load());
            }
            if (platform === 'as') {
                requests.push(this.loadSelectorsDictionary(platform, currentExtensionVersion()));
            }

            return $.when.apply(this, requests);
        }).then(() => {
            this.notify();

            if (!this.profile.isSuspended()) {
                this.timer.calculateAllVirtualProperties();
                this.openSocket();
            }

            return this.users.loadBase64(isRequireBase64);
        }).catch((err) => {
            console.log('[EV][Store] user isn\'t authorized ', err);
        });
    },

    loadPopup: function(extraRequestsCallback) {
        this.taskTime = new TaskTimeStore();
        this.timeExport = new TimeExportStore();
        this.taskDetails = new TaskDetailsStore();

        if (!this.isAuthorized()) {
            return resolvedPromise();
        }

        return $.when(
            this.profile.load()
        ).then(() => {
            if (!this.profile.isSuspended()) {
                browserPort.eventDispatcher.broadcast('background', 'sync:profile-active', null);

                let requests = [
                    this.timer.load(),
                    this.taskTime.load()
                ];

                if (extraRequestsCallback) {
                    requests = requests.concat(extraRequestsCallback());
                }

                return $.when.apply(this, requests);
            }
        }).then(() => {
            const isSuspended = this.profile.isSuspended();
            // do not use this.notify() for
            // avoid double render in popup (see store.subscribe)
            if (!isSuspended) {
                this.taskTime.calculateAllVirtualProperties();
                this.timer.calculateAllVirtualProperties();
                this.openSocket();
            }

            if (isSuspended) {
                return rejectedPromise({status: 402});
            }
        });
    },

    openSocket: function() {
        // we always using everhour.socket() not depends where were we call it
        this.channel = everhour.socket(this.profile.get().id);
        this.channel.subscribe('action', (message) => this.reduce(message.data));
    },

    subscribe: function(listener) {
        this.listeners.push(listener);
    },

    /**
     * Unsubscribe
     *
     * @param {function} listener - callback passed to method subscribe
     */
    unsubscribe: function(listener) {
        const removeIndex = this.listeners.findIndex((func) => func === listener);

        if (removeIndex > -1) {
            this.listeners.splice(removeIndex, 1);
        }
    },

    notify: function(action) {
        this.listeners.forEach(listener => listener(action));
    },

    dispatch: function(action, quite) {
        this.reduce(action, quite);

        if (
            !this.profile.isSuspended() &&
            this.internalActions.indexOf(action.type) === -1
        ) {
            this.channel.publish('action', action);
        }
    },

    reduce: function(action, quite) {
        Logger.log('store.action', action);
        if (this.timer) {
            this.timer.reduce(action);
        }

        if (this.projects) {
            this.projects.reduce(action);
        }

        if (this.tasks) {
            this.tasks.reduce(action);
        }

        if (this.taskTime) {
            this.taskTime.reduce(action);
        }

        if (this.timeExport) {
            this.timeExport.reduce(action);
        }

        if (this.taskDetails) {
            this.taskDetails.reduce(action);
        }

        if (this.profile) {
            this.profile.reduce(action);
        }

        if (this.clients) {
            this.clients.reduce(action);
        }

        if (!quite) {
            this.notify(action);
        }
    }
};

window.Selectors = function(store) {
    this.store = store;
};

Selectors.prototype = {
    sortedProgress: function(id, projects, includeEmpty) {
        let me = this.store.profile.get();
        let teamUsers = this.store.users.all();
        let progress = this.store.tasks.progress(id);

        let users = new Set([]
            .concat(progress && progress.estimate && progress.estimate.users ? Object.keys(progress.estimate.users) : [])
            .concat(progress && progress.time && progress.time.users ? Object.keys(progress.time.users) : [])
            .concat((projects || [])
                .reduce((users, id) => users.concat(this.store.projects.get(id).users || []), [])
            )
            // .concat(Object.keys(teamUsers).slice(0, 4))
            .map(id => parseInt(id))
        );

        return Array.from(users)
            .map((id) => ({
                user: id,
                name: teamUsers[id.toString()].name,
                estimate: progress && progress.estimate && progress.estimate.users ? progress.estimate.users[id] || 0 : 0,
                time: progress && progress.time && progress.time.users ? progress.time.users[id] || 0 : 0,
                today: progress && id === me.id ? progress.today : 0,
                comment: progress && id === me.id ? progress.comment : ''
            }))
            .filter(user => !includeEmpty ? user.estimate > 0 || user.time > 0 : true)
            .sort((a, b) => {
                // put current user first
                if (a.user === me.id) {
                    return -1;
                }

                if (b.user === me.id) {
                    return 1;
                }

                // estimate DESC
                if (a.estimate !== b.estimate) {
                    return b.estimate - a.estimate;
                }

                // time DESC
                if (a.time !== b.time) {
                    return b.time - a.time;
                }

                // user name ASC
                return (a.name || '').localeCompare(b.name || '');
            });
    },

    /**
     * @param {string} taskId
     * @returns {Array}
     */
    sortedProgressWithSubtasks: function (taskId) {
        let subProgress = store.tasks.getSubtasksProgress(taskId);
        // collect users from subtasks progress
        let subProgressUsers = [];
        if (subProgress) {
            subProgressUsers = Array.from(new Set(
                Object.keys(subProgress.time.users)
                .concat(Object.keys(subProgress.estimate.users))
                .map(id => parseInt(id))
            ));
        }

        let progress = this.sortedProgress(taskId)
            .map(function (progressRow) {
                progressRow.totalTime = progressRow.time || 0;
                progressRow.totalEstimate = progressRow.estimate || 0;
                // merge time from subtasks
                if (subProgress && subProgressUsers.indexOf(progressRow.user) !== -1) {
                    progressRow.totalTime += subProgress.time.users[progressRow.user] || 0;
                    progressRow.totalEstimate += subProgress.estimate.users[progressRow.user] || 0;
                    progressRow.subtasksData = {time: subProgress.time.users[progressRow.user] || 0, estimate: subProgress.estimate.users[progressRow.user] || 0}
                }

                return progressRow;
            });
        let progressUsers = _.pluck(progress, 'user');

        _.each(subProgressUsers, function (userId) {
            // append users from subtasks to parent's progress
            if (progressUsers.indexOf(userId) === -1) {
                let subtasksTime = subProgress.time.users[userId] || 0;
                let subtasksEstimate = subProgress.estimate.users[userId] || 0;
                let progressItem = {
                    user: userId,
                    time: 0,
                    estimate: 0,
                    totalTime: subtasksTime,
                    totalEstimate: subtasksEstimate,
                    subtasksData: {time: subtasksTime, estimate: subtasksEstimate}
                };

                progress.push(progressItem);
            }
        }, this);

        return progress;
    }
};

Store.dictionary = {};

const DIC = function(key, prefix) {
    return Store.dictionary[prefix ? prefix + '$$' + key : key] || key;
};

window.Logger = {
    debug: false,

    log: function() {
        if (!Logger.debug) {
            return;
        }

        var args = ["[" + (new Date()).toString() + "] "].concat(Array.prototype.slice.call(arguments));

        Logger.Output.Stack.log.apply(Logger.Output.Stack, args);
        if (Logger.debug) {
            Logger.Output.Console.log.apply(Logger.Output.Console, args);
        }
    },

    warn: function() {
        var args = ["[" + (new Date()).toString() + "] "].concat(Array.prototype.slice.call(arguments));

        Logger.Output.Stack.warn.apply(Logger.Output.Stack, args);
        if (Logger.debug) {
            Logger.Output.Console.warn.apply(Logger.Output.Console, args);
        }
    },

    error: function() {
        var args = ["[" + (new Date()).toString() + "] "].concat(Array.prototype.slice.call(arguments));

        Logger.Output.Stack.error.apply(Logger.Output.Stack, args);
        if (Logger.debug) {
            Logger.Output.Console.error.apply(Logger.Output.Console, args);
        }
    },

    startDebug: function() {
        Logger.Output.Console.log('--- PAST MESSAGES ---');
        Logger.Output.Stack.replay();

        Logger.debug = true;
        Logger.Output.Console.log('--- DEBUG STARTED ---');
    }
};

Logger.Output = {
    Console: {
        log: function() {
            console.log.apply(console, arguments);
        },

        warn: function() {
            console.warn.apply(console, arguments);
        },

        error: function() {
            console.error.apply(console, arguments);
        }
    },

    Stack: {
        _messages: [],

        log: function() {
            Logger.Output.Stack._push('log', arguments);
        },

        warn: function() {
            Logger.Output.Stack._push('warn', arguments);
        },

        error: function() {
            Logger.Output.Stack._push('error', arguments);
        },

        replay: function(output) {
            output = output || console;
            Logger.Output.Stack._messages.forEach(function(message) {
                output[message[0]].apply(output, message[1]);
            });
        },

        _push: function(type, args) {
            Logger.Output.Stack._messages.push([type, args]);

            if (Logger.Output.Stack._messages.length > 100) {
                Logger.Output.Stack._messages.shift();
            }
        }
    }
};


window.version = function (apiKey, defaultVersion) {
    let client = new Everhour2(apiKey);
    if (client.endpoint === 'https://t1-api.evh.io' || (apiKey && apiKey.indexOf('@') !== -1)) {
        return 'v2';
    }

    defaultVersion = defaultVersion || 'v1';

    if (!apiKey || apiKey.length === 0) {
        return defaultVersion;
    }

    return apiKey.length === 32 ? 'v2' : 'v1';
};

window.isEverhour2App = function(url) {
    return url.match(/app\.everhour.com/) ||
        url.match(/everhour\.test/) ||
        url.match(/evh\.io/);
};

window.isLegacyEverhourApp = function(url) {
    return url.match(/everhour.com\/track\//);
};

window.isIntegrationPage = function(url) {

    let urls = [
        /https:\/\/app.asana.com\//,
        /https:\/\/3.basecamp.com\//,
        /https:\/\/basecamp.com\//,
        /https:\/\/trello.com\//,
        /https:\/\/github.com\//,
        /https:\/\/.*\.teamwork\.com\//,
        /https:\/\/.*\.eu\.teamwork\.com\//,
        /https:\/\/.*\.teamworkpm\.net\//,
        /https:\/\/bitbucket.org\//,
        /https:\/\/.*\.insight.ly\//,
        /https:\/\/.*\.insightly.com\//,
        /https:\/\/app.clickup.com\//,
        /https:\/\/www.pivotaltracker.com\/n\/(projects|workspaces)\//,
        /https:\/\/.*\.atlassian\.(net|io|com)\//,
        /https:\/\/.*\.jira\.com\//,
        /https:\/\/.*\.atl-paas\.net\//,
        /https:\/\/jira-(\d+)\.evh\.io\//,
        /https:\/\/.*\.monday\.com\//,
        /https:\/\/gitlab.com\//,
        /https:\/\/todoist\.com\//,
        /https:\/\/app.todoist\.com\//,
        /https:\/\/www.notion.so\//,
    ];

    for (let i = 0; i < urls.length; i++) {
        if (urls[i].test(url)) {
            return true;
        }
    }

    return false;
};

window.resolvedPromise = function(data) {
    let promise = $.Deferred();
    promise.resolve(data || null);

    return promise.promise();
};

window.rejectedPromise = function(data) {
    let promise = $.Deferred();
    promise.reject(data || null);

    return promise.promise();
};

window.getCurrency = function() {
    let team = store.profile.index.team;

    return team.currencyDetails;
};

window.getCurrencySymbol = function() {
    let currency = getCurrency();

    if (currency.symbol) {
        return currency.symbol;
    }
    if (currency.symbolNarrow) {
        return currency.symbolNarrow;
    }

    return currency.code;
};

window.toBasicUnit = function(value) {
    return value / 100;
};

window.toCents = (value) => {
    return (value || 0) * 100;
};

window.getMoney = function(value) {
    const currency = getCurrency();
    let sign = '';

    let valueFormatted = toBasicUnit(value || 0).toLocaleString('en-US', {
        style: 'decimal',
        maximumFractionDigits: 2,
        minimumFractionDigits: 2
    });

    if (valueFormatted[0] === '-') {
        valueFormatted = valueFormatted.substr(1);
        sign = '-';
    }
    if (currency.symbol) {
        valueFormatted = sign + currency.symbol + valueFormatted;
    } else {
        valueFormatted = sign + valueFormatted + ' ' + (currency.symbolNarrow || currency.code);
    }

    return valueFormatted;
};

window.FloatParser = function() {
    this.pattern = RegExp;
    this.init();
};

FloatParser.prototype = {
    init: function() {
        this.compilePattern();
    },

    parse: function(value) {
        if (value === undefined || value === null) {
            return null;
        }

        const matches = this.pattern.exec(value.toString().trim());

        if (matches === null) {
            return undefined;
        }

        return toBasicUnit(toCents(parseFloat(value)));
    },

    compilePattern: function() {
        const pattern = '[\-]?(\\d+(\\.\\d{0,2})?)';

        this.pattern = new RegExp('(^)' + pattern + '($)');
    }
};

window.getMoneyRound = function(value) {
    const currency = getCurrency();
    let sign = '';

    let valueFormatted = toBasicUnit(value || 0).toLocaleString('en-US', {
        style: 'decimal',
        maximumFractionDigits: 0,
        minimumFractionDigits: 0
    });

    if (valueFormatted[0] === '-') {
        valueFormatted = valueFormatted.substr(1);
        sign = '-';
    }

    if (currency.symbol) {
        valueFormatted = sign + currency.symbol + valueFormatted;
    } else {
        valueFormatted = sign + valueFormatted + ' ' + (currency.symbolNarrow || currency.code);
    }

    return valueFormatted;
};

window.getUserShortName = function(name) {
    // Mike -> Mike
    // Mike Kulakov -> Mike K.
    // Mike Kulakov 🇨🇾 -> Mike
    // Mike Kulakov🇨🇾 -> Mike K.
    // Mike 🇨🇾 -> Mike 🇨🇾
    // Mike 🇨🇾 Kulakov -> Mike
    // Mike 🇨🇾Kulakov -> Mike 🇨🇾
    // 🇨🇾 Mike ->  🇨🇾 Mike
    // 🇨🇾 Mike Kulakov -> 🇨🇾 Mike K.

    const words = name.trim().split(' ').filter(word => !!word);

    return window.shortName(words);
};

window.getUserInitials = function(name) {
    // Mike -> MI
    // Mike Kulakov -> MK
    // Mike Kulakov 🇨🇾 -> MK
    // Mike Kulakov🇨🇾 -> MK
    // Mike 🇨🇾 -> M🇨🇾
    // Mike 🇨🇾 Kulakov -> M🇨🇾
    // Mike 🇨🇾Kulakov -> M🇨🇾
    // 🇨🇾 Mike ->  🇨🇾M
    // 🇨🇾 Mike Kulakov -> 🇨🇾M

    const words = name.trim().split(' ').filter(word => !!word);

    if (words.length === 0) {
        words.push('');
    }

    if (words.length > 1) {
        return window.getFirstLetter(words[0]) + window.getFirstLetter(words[1]);
    }

    return window.getFirstLetter(words[0], 2);
};

window.shortName = function(words) {
    if (words.length === 1) {
        return words;
    }

    const firstName = words[0];
    const firstNameEmoji = window.containsEmoji(firstName);

    if (firstNameEmoji && firstNameEmoji[0] === firstName) {
        words.shift();

        return `${firstName} ${window.shortName(words)}`;
    } else {
        if (words.length === 2) {
            const startWithEmoji = window.containsEmoji(words[1]);
            const lastName = startWithEmoji ? startWithEmoji[0] : `${words[1][0]}.`;

            return `${firstName} ${lastName}`;
        }

        return firstName;
    }
};

window.containsEmoji = function(word) {
    return word.match(/^\p{Emoji_Presentation}+/gu);
};

window.getFirstLetter = function(word, count = 1) {
    const emoji = window.getFirstEmoji(word);

    if (emoji.length) {
        return emoji;
    }

    return word.substring(0, count).toUpperCase();
};

window.getFirstEmoji = function(word) {
    const wordEmoji = window.containsEmoji(word);

    if (wordEmoji && wordEmoji[0]) {
        return wordEmoji[0];
    }

    return '';
}

window.getAvatarName = function(user) {
    return window.getUserInitials(user.name || user.email);
};

window.getGeneratedAvatarHtml = function(user) {
    let name = getAvatarName(user);
    let color = user.id % 10;

    return `<span class="color${color}">${name}</span>`;
};

window.getUserAvatarHtml = function(url) {
    return `<img src="${url}" />`;
};

window.getAvatar = function(user, src) {
    let html = '';

    if (user && !user.avatarUrl) {
        html = getGeneratedAvatarHtml(user);
    } else if (user && user.avatarUrl && !src) {
        html = getUserAvatarHtml(user.avatarUrl);
    } else if (src) {
        html = getUserAvatarHtml(src);
    }

    return html;
};

function getLockAfterText(team) {
    team = team || store.profile.team();

    switch (team.lockTimePeriod) {
        case 'day-of-week':
            // TODO how this worked before?
            if (this.getLockAfterDayOfWeek) {
                return this.getLockAfterDayOfWeek(team.lockTimeAfter);
            }

            return getLockAfterDayOfWeek(team.lockTimeAfter);
        case 'day-of-month':
            return getLockAfterDayOfMonth(team.lockTimeAfter);
        case'days':
            return getLockAfterDays(team.lockTimeAfter);
    }
}

function collapsedProgressTooltip(progress) {
    return progress.reduce((tooltip, progress) => {
        const timeFormatted = TimeEntryUtils.format(progress.totalTime || progress.time, '0h');
        const estimateFormatted = ( progress.totalEstimate || progress.estimate) ?
            ' of ' + TimeEntryUtils.format(progress.totalEstimate || progress.estimate) :
            '';
        tooltip += _.escape(progress.name) + '&nbsp;&middot;&nbsp;' + timeFormatted + estimateFormatted + '<br/>';

        return tooltip;
    }, '');
}

function getLockAfterDayOfMonth(lockTimeAfter) {
    return 'Time edit is limited after ' + moment().startOf('month').add(lockTimeAfter).format('D MMMM');
}

function getLockAfterDayOfWeek(lockTimeAfter) {
    return 'Time edit is limited after ' + moment().startOf('week').add(lockTimeAfter - 1, 'd').format('dddd');
}

function getLockAfterDays(lockTimeAfter) {
    if (lockTimeAfter === 0) {
        return 'Time edit before today is limited';
    }

    return 'Time edit is limited after ' + lockTimeAfter + (lockTimeAfter == 1 ? ' day' : ' days');
}

function projectSummaryUrl(projectId) {
    if (!projectId || !store.profile.isAdmin()) {
        return null;
    }

    return everhour.productionUrl + `/#/projects/${projectId}/summary`;
}

window._format = {
    money: getMoney,
    lockAfterText: getLockAfterText,
    moneyRound: getMoneyRound,
    totalTimeTooltip: _.template(`
        The sum of team reported time for parent task and its subtasks.
        <hr/>
        <span>
            Parent Task: <strong><%- TimeEntryUtils.format(total.time - total.subtasksData.time, '0h') %></strong>
        </span><br/>
        <span>
            Sub Tasks: <strong><%- TimeEntryUtils.format(total.subtasksData.time, '0h') %></strong>
        </span>
    `),
    totalEstimateTooltip: _.template(`
        The sum of team estimates for parent task and its subtasks.
        <hr/>
        <span>
            Parent Task: <strong><%- TimeEntryUtils.format(total.estimate - total.subtasksData.estimate, '0h') %></strong>
        </span><br/>
        <span>
            Sub Tasks: <strong><%- TimeEntryUtils.format(total.subtasksData.estimate, '0h') %></strong>
        </span>
    `),
    userProgressTemplate: _.template(`
        The sum of personal reported time and estimates for parent task and its subtasks.
        <hr/>
        <span>
            Parent Task: 
            <% if (progress.time || progress.estimate) { %>
                <strong><%- TimeEntryUtils.format(progress.time, '0h') %></strong> 
                <% if (progress.estimate) { %> of 
                    <strong><%- TimeEntryUtils.format(progress.estimate, '0h') %></strong>
                <% } %>
            <% } else { %>
                 <strong>0h</strong>
            <% } %>
        </span><br/>
        <span>
            Sub Tasks:
            <% if (progress.subtasksData && (progress.subtasksData.time || progress.subtasksData.estimate)) { %> 
                <strong><%- TimeEntryUtils.format(progress.subtasksData.time, '0h') %></strong>
                <% if (progress.subtasksData.estimate) { %> of 
                    <strong><%- TimeEntryUtils.format(progress.subtasksData.estimate) %></strong>
                <% } %>
            <% } else { %>
                <strong>0h</strong>
            <% } %>
        </span>
    `),
    projectSummaryLinkTemplate: _.template(`
        <span class="ev-time <%- hideEstimate ? 'ev-time-only' : '' %>">
            Time: 
            <%- time ? TimeEntryUtils.format(time, '0h') : '0h' %>
        </span>
        <% if (!hideEstimate) { %>
            <span class="ev-estimates">
                Estimate: 
                <%- estimate ? TimeEntryUtils.format(estimate, '0h') : '0h' %> 
            </span>
        <% } %>
        <span class="icon-form-info"></span> 
        <span>
            <% if (platform === 'Basecamp 3') { %>
                <%- ContentScript.Constants.SUMMARY_TOOLTIP_TEXT_BC3 %>
            <% } else { %>
                <% if (!hideEstimate) { %>
                    These are totals
                <% } else { %>
                    This is total
                <% } %>
                 for visible tasks only. It may be that <%- platform %> has not loaded all your tasks or you have some filter enabled.
            <% } %>
         </span>
    `),
    getAvatar: getAvatar,
    getAvatarName: getAvatarName,
    getCurrencySymbol: getCurrencySymbol,
    getCurrency: getCurrency,
    getMoney: getMoney,
    projectSummaryUrl: projectSummaryUrl,
    collapsedProgressTooltip: collapsedProgressTooltip
};

$.fn.setCursorPosition = function(pos) {
    this.each(function(index, elem) {
        if (elem.setSelectionRange) {
            elem.setSelectionRange(pos, pos);
        } else if (elem.createTextRange) {
            let range = elem.createTextRange();
            range.collapse(true);
            range.moveEnd('character', pos);
            range.moveStart('character', pos);
            range.select();
        }
    });

    return this;
};

/**
 *
 * @param {object} failedResponse
 * @returns {string}
 */
handleErrorsFromResponse = function(failedResponse) {
    if (!failedResponse) {
        return '';
    }
    let errorMessage = failedResponse.responseJSON.message ||
        failedResponse.responseJSON.error; // v1
    const multiErrorMessages = failedResponse.responseJSON.errors;
    // sometime we have error as `result.message`
    // sometime we have `result.errors` object even with single error and `Bad Request` in message property
    if (multiErrorMessages) {
        let errorMessages = [];
        for (let key in multiErrorMessages) {
            if (multiErrorMessages.hasOwnProperty(key)) {
                errorMessages.push(multiErrorMessages[key]);
            }
        }
        errorMessage = errorMessages.join(' ');
    }

    return errorMessage;
};

window.currentExtensionVersion = function() {
    return chrome.runtime.getManifest().version;
};

window.LocalStorageService = function() {
    this.numberRegexp = new RegExp(/^\d+$/);
    this.jsonRegexp = new RegExp(/[\[\{].*[\}\]]/);

    this.init();
};

LocalStorageService.prototype = {
    init: function() {
        return this;
    },

    getItem: function(key) {
        try {
            const item = localStorage.getItem(key);

            if (!item) {
                return null;
            }

            switch (true) {
                case this.jsonRegexp.test(item):
                    return JSON.parse(item);
                case this.numberRegexp.test(item):
                    return parseFloat(item);
                default:
                    return item;
            }
        } catch (err) {
            return null;
        }

    },

    setItem: function(key, value) {
        if (!key && !value) {
            return;
        }

        if (typeof value === "object") {
            localStorage.setItem(key, JSON.stringify(value));
            return;
        }

        localStorage.setItem(key, value);
    },

    removeItem: function(key) {
        localStorage.removeItem(key);
    }
};

window.CookieStorage = function() {
    this.init();
};

CookieStorage.prototype = {
    init: function() {},
    getCookie: function(name) {
        let matches = document.cookie.match(new RegExp(
            "(?:^|; )" + name.replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, '\\$1') + "=([^;]*)"
        ));

        return matches ? decodeURIComponent(matches[1]) : undefined;
    },
    setCookie: function (name, value, options = {}) {
        options = {
            path: '/',
            ...options
        };

        if (options.expires instanceof Date) {
            options.expires = options.expires.toUTCString();
        }

        let updatedCookie = encodeURIComponent(name) + "=" + encodeURIComponent(value);

        for (let optionKey in options) {
            updatedCookie += "; " + optionKey;
            let optionValue = options[optionKey];

            if (optionValue !== true) {
                updatedCookie += "=" + optionValue;
            }
        }

        document.cookie = updatedCookie;
    },
    deleteCookie: function(name) {
        setCookie(name, "", {'max-age': -1});
    }
};

window.everhourMethods = {
    filterByMatch: (arr, key, searchTerm) => {
        if (!searchTerm || !key) {
            return arr;
        }

        return arr.filter((item) => !!~item[key].toLowerCase().indexOf(searchTerm.toLowerCase()));
    },
    timestamp: () => {
        return Math.ceil(new Date().getTime() / 1000);
    }
};

/**
 * Wrap function call with mutex
 * It will ignore new calls during execution
 * E.g. you have render function. Mutex will ensure that render didn't executed during render execution
 *
 * @param func
 * @param context
 * @returns {Function}
 */
window.mutex = function(func, context) {
    var running = false;
    return function() {
        var result = null;

        if (running) {
            return result;
        }

        running = true;
        try {
            result = func.apply(context, arguments);
        } finally {
            running = false;
        }

        return result;
    }
};

/**
 * Extend class
 */
window.extend = function(protoProps, staticProps) {
    var parent = this;
    var child;

    // The constructor function for the new subclass is either defined by you
    // (the "constructor" property in your `extend` definition), or defaulted
    // by us to simply call the parent's constructor.
    if (protoProps && _.has(protoProps, 'constructor')) {
      child = protoProps.constructor;
    } else {
      child = function(){ return parent.apply(this, arguments); };
    }

    // Add static properties to the constructor function, if supplied.
    _.extend(child, parent, staticProps);

    // Set the prototype chain to inherit from `parent`, without calling
    // `parent`'s constructor function.
    var Surrogate = function(){ this.constructor = child; };
    Surrogate.prototype = parent.prototype;
    child.prototype = new Surrogate;

    var protoPropsToCopy = {};

    for (var key in protoProps) {
        if (Object.getOwnPropertyDescriptor(protoProps, key).get) {
            Object.defineProperty(child.prototype, key, Object.getOwnPropertyDescriptor(protoProps, key));
        } else {
            protoPropsToCopy[key] = protoProps[key];
        }
    }

    // Add prototype properties (instance properties) to the subclass,
    // if supplied.
    if (protoProps) _.extend(child.prototype, protoPropsToCopy);

    // Set a convenience property in case the parent's prototype is needed
    // later.
    child.__super__ = parent.prototype;

    return child;
};

/**
 * Execute js script in main page context
 * @param {string} script
 */
window.embedScript = function(script) {
    // script = 'document.documentElement.removeAttribute("onreset");' + script + ';';
    //
    // document.documentElement.setAttribute('onreset', script);
    // document.documentElement.dispatchEvent(new CustomEvent('reset'));
    // document.documentElement.removeAttribute('onreset');
    //

    var scriptElement = document.createElement('script');
    scriptElement.textContent = script + ';';
    (document.head||document.documentElement).appendChild(scriptElement);
    scriptElement.remove();
};

window.embedFunction = function(name, func) {
    embedScript('document.' + name + ' = ' + func.toString() + ';');
};

window.embedMethodId = 0;
window.embedMethod = function(func, options) {
    options = Object.assign({cache: false}, options);

    var name = 'everhourMethod' + (++embedMethodId);
    var dict = JSON.stringify(Store.dictionary || {});
    var script = `
        document.${name} = function() {
            var storeDictionary = ${dict};
            var DIC = (key) => storeDictionary[key] || key;
            var result = null;
            try {
                result = (${func.toString()}).apply(this, arguments);
            } catch (e) {}
            
            if (result === null) {
                document.documentElement.removeAttribute('${name}');
                return;
            }
            
            document.documentElement.setAttribute('${name}', JSON.stringify(result));
        };
    `;

    embedScript(script);

    var cache = {};

    return function() {
        var args = JSON.stringify(Array.prototype.slice.call(arguments)), value;

        if (cache.hasOwnProperty(args)) {
            return cache[args];
        }

        embedScript(`document.${name}.apply(this, ${args});`);
        value = JSON.parse(document.documentElement.getAttribute(name) || null);

        if (options.cache) {
            cache[args] = value;
        }

        return value;
    };
};

/**
 * Sleep for specified duration in ms
 * @param sleepDuration
 */
window.sleepFor = function(sleepDuration) {
    var now = new Date().getTime();
    while(new Date().getTime() < now + sleepDuration){ /* do nothing */ }
};

if (!NodeList.prototype.hasOwnProperty('forEach')) {
    NodeList.prototype.forEach = Array.prototype.forEach;
}

window.timestamp = function() {
    return Math.ceil(new Date().getTime() / 1000);
};

window.htmlToElement = function(html) {
    var template = document.createElement('template');
    template.innerHTML = html;
    return template.content.firstChild;
};

window.elementSelector = function(element) {
    if (element.id && !/\d/.exec(element.id)) {
        return element.nodeName + '#' + element.id;
    }

    var classList = element.classList.toString().replace(/\s/g, '.');

    if (classList) {
        return element.nodeName + '.' + classList;
    }

    return element.nodeName;
};

window.getLocation = function() {
    return window.location.href.replace(/\#.*?$/, '');
};

window.watch = function(selector) {
    (new ContentScript.MutationPatternFinder(selector)).start();
};

window.nodeIndex = function(node) {
    if (!node) {
        return -1;
    }

    return Array.prototype.indexOf.call(node.parentNode.childNodes, node);
};

window.watchMutations = function() {
    var observer = new MutationObserver(function(mutations) {
        // console.log('mutations', mutations.length);
    });

    observer.observe(document, {
        childList: true,
        attributes: false,
        subtree: true
    });
};

if (!Array.prototype.last){
    Array.prototype.last = function(){
        return this[this.length - 1];
    };
}

/**
 * Load image to data URL
 * Worked with Firefox and Safari and allow to inject images with CSP
 * @param src
 */
function convertImageUrlToBase64Data(src) {
    let deferred = $.Deferred();

    // We have an issue with CloudFront and Safari CORS
    // https://serverfault.com/questions/856904/chrome-s3-cloudfront-no-access-control-allow-origin-header-on-initial-xhr-req/856948#856948
    // if (browserPort.type === 'safari' && src.indexOf('d36887svjhykt4.cloudfront.net') !== -1) {
    //     src = src.replace('d36887svjhykt4.cloudfront.net/', 's3.amazonaws.com/everhour-images/');
    // }

    let xhttp = new XMLHttpRequest();

    xhttp.onload = function() {
        let fileReader = new FileReader();
        fileReader.onloadend = function() {
            deferred.resolve(fileReader.result);
        };

        fileReader.readAsDataURL(xhttp.response);
    };

    xhttp.onerror = function(e) {
        console.log('[content-script][functions.js][convertImageUrlToBase64Data] Avatar doesn\'t load');

        deferred.resolve(null);
    };

    xhttp.responseType = 'blob';
    xhttp.open('GET', src, true);
    xhttp.send();

    return deferred.promise();
};

/**
 * @param encodedId
 * @returns {string|null}
 */
window.composeId = (encodedId) => {
    return encodedId ? [app.adaptor.platform, encodedId].join(':') : null;
};

window.toggleBillingModal = (forceClose, projectId) => {
    const modalSelector = ContentScript.v2.Components.ProjectBillingModal.prototype.modalSelector;
    const attributeName = ContentScript.v2.Components.ProjectBillingModal.prototype.projectIdAttribute;

    if (projectId || forceClose) {
        const modalInstance = document.querySelector(modalSelector);
        if (modalInstance) {
            if (projectId) {
                modalInstance.setAttribute(attributeName, projectId);
            } else {
                modalInstance.removeAttribute(attributeName);
            }
        }
    }
    toggleEverhourModal(modalSelector, forceClose);
};

window.toggleEverhourModal = (modalSelector, forceClose) => {
    const modalInstance = $(modalSelector);
    if (modalInstance) {
        if (forceClose) {
            modalInstance.modal('hide');
        } else {
            modalInstance.modal('toggle');
        }
    }
};

window.clearEverhourTooltips = (selector) => {
    // clear tooltips that may be frozen due component render
    document.querySelectorAll(selector || '.ev-tooltip').forEach(element => element.remove());
};

window.everhourTooltip = (selector, options = {}, additionalClass = '') => {
    const evTemplate = `
        <div class="ev-tooltip ${additionalClass}" role="tooltip">
            <div class="tooltip-arrow"></div>
            <div class="tooltip-inner"></div>
        </div>`;

    $(selector).each((i, elem) => {
        const tooltip = $(elem).data('bs.tooltip');
        if (tooltip) {
            tooltip.destroy();
        }
    });

    $(selector).tooltip(
        Object.assign(
            {},
            {
                sanitize: false,
                html: true,
                template: evTemplate
            },
            options || {},
        )
    );
};

window.isWindows = function() {
    try {
        return !!~navigator.appVersion.indexOf("Win")
    } catch (e) {}
};

window.isMac = function() {
    try {
        return !!~navigator.appVersion.indexOf("Mac")
    } catch (e) {}
};

window.TaskTimeTooltipService = function() {
    const profileStore = store.profile;

    this.profile = profileStore.get();
    this.authUserId = this.profile.id;
    this.authUserIsAdmin = profileStore.isAdmin();
    this.authUserIsMember = profileStore.isMember();
    this.authUserIsSupervisor = profileStore.isSupervisor();
    this.show24Hours = this.profile.dateTimeFormat === 24;
    this.lockReasons = {
        not_mine: {
            admin: 'This time does not belong to you',
            member: 'Your role doesn’t allow editing someone\'s time',
        },
        time_invoiced: {
            admin: 'This time has been invoiced',
            member: 'Invoiced time can’t be changed',
        },
        timesheet_approved: {
            admin: 'This time has been approved',
            member: 'Approved time can’t be changed',
        },
        timesheet_approval_requested: {
            admin: 'This time has been sent for approval',
            member: 'Time waiting for approval can’t be changed',
        },
        future_date: {
            admin: 'Time edit is limited for future dates',
            member: 'Time edit is limited for future dates',
        },
        project_archived: {
            admin: 'This time belongs to an archived project',
            member: 'Time in archived project can’t be changed',
        },
        project_foreign: {
            admin: 'You are not assigned to the project',
            member: 'You are not assigned to the project',
        },
        completed_task: {
            admin: 'You can’t manipulate time in completed tasks per your team policy',
            member: 'You can’t manipulate time in completed tasks per your team policy',
        },
    };
    this.logLimit = 7;
    this.logLimitDays = 3;
}
TaskTimeTooltipService.prototype = {
    order: function(a, b, fallback) {
        if (a < b) {
            return -1;
        }

        if (a > b) {
            return 1;
        }

        return fallback ? fallback() : 0;
    },
    escape: function(string) {
        const escapeMap = {
            '&': '&amp;',
            '<': '&lt;',
            '>': '&gt;',
            '"': '&quot;',
            '\'': '&#x27;',
            '`': '&#x60;',
        };
        const escapeTestRegexp = RegExp('(?:' + Object.keys(escapeMap).join('|') + ')');
        const escapeReplaceRegexp = RegExp('(?:' + Object.keys(escapeMap).join('|') + ')', 'g');

        if (!string) {
            return '';
        }

        return escapeTestRegexp.test(string) ? string.replace(escapeReplaceRegexp, m => escapeMap[m]) : string;
    },
    getNl2Br: function(value) {
        if (!value) {
            return null;
        }

        value = value.toString();

        if (value.length === 0) {
            return null;
        }

        return this.escape(value).replace(/\n/g, '<br>');
    },
    plural: function(count, single, multiple) {
        return (count > 0 && count <= 1) ? single : multiple;
    },
    truncate: function(value, args) {
        const limit = args.length > 0 ? parseInt(args[0], 10) : 10;
        const trail = args.length > 1 ? args[1] : '';

        return value.length > limit ? value.substring(0, limit) + trail : value;
    },
    zeroPrefixed: function(value) {
        if (value === 0) {
            return '00';
        }

        return value >= 10 ? value.toString() : '0' + value.toString();
    },
    timeFormat: function(seconds, format = 'text_short') {
        if (seconds < 0) {
            seconds = -seconds;
        }

        const hours = Math.floor(seconds / 60 / 60);
        const minutes = Math.floor(seconds / 60) % 60;

        switch (format) {
            // 1:00 / 1:05 / 0:05
            case 'clock':
                return (hours ? hours : '0') + ':' + this.zeroPrefixed(minutes);

            // 01:00 / 01:05 / 0:05
            case 'clock_full':
                return zeroPrefixed(hours ? hours : 0) + ':' + this.zeroPrefixed(minutes);

            // 1.00 / 1.08 / 0.08
            case 'hours':
                return (Math.round((seconds / 60 / 60) * 100) / 100).toFixed(2);

            // 1h 00m / 1h 05m / 0h 05m
            case 'text':
                return (hours ? hours + 'h' : '0h') + ' ' + this.zeroPrefixed(minutes) + 'm';

            // 1h / 1h 5m / 5m
            case 'text_short':
                return (
                    (hours ? hours + 'h' : '') +
                    (hours && minutes ? ' ' : '') +
                    (minutes ? minutes + 'm' : '') +
                    (!hours && !minutes ? '0' : '')
                );

            case 'text_hours_only':
                return hours ? hours + 'h' : '0h';

            case 'hours_only':
                return (seconds / 3600).toString();

            default:
                throw new Error('Invalid time format: ' + format);
        }
    },
    formatWhat: function(history) {
        const sign = history.time > 0 ? '+' : '-';
        const value = this.timeFormat(Math.abs(history.time));

        return `${sign}${value}`;
    },
    formatWhen: function(history) {
        const format = this.show24Hours ? 'HH:mm' : 'h:mm a';
        const createdAt = moment.utc(history.createdAt, 'YYYY-MM-DD HH:mm:ss').local();

        if (history.action === 'TIMER' || history.action === 'TIMER_PREVIOUS_DAY') {
            const startAt = createdAt.clone().subtract(history.time, 'seconds');
            return `${startAt.format(format)} - ${createdAt.format(format)}`;
        }

        if (history.action === 'BY_DAY') {
            return 'Updated';
        }

        return createdAt.format(format);
    },
    lockAfter: function (lockTimePeriod, lockTimeAfter) {
        switch (lockTimePeriod) {
            case 'day-of-week':
                return moment().isoWeekday(lockTimeAfter).format('dddd');
            case 'day-of-month':
                return moment().startOf('month').add(lockTimeAfter - 1, 'day').format('D MMMM');
            case'days':
                return lockTimeAfter === 0 ? 'before today' : String(lockTimeAfter) + this.plural(lockTimeAfter, ' day', ' days');
            default:
                return null;
        }
    },
    ordinal: function(n, keepNumber = true) {
        const ordinals = ['th', 'st', 'nd', 'rd'];
        const v = n % 100;

        return (keepNumber ? n : '') + (ordinals[(v - 20) % 10] || ordinals[v] || ordinals[0]);
    },
    generateHtml: function(placeholder, restrictions = [], warnings = [], log = new Map(), comment) {
        let warningsHtml = '';

        warnings.forEach((warning) => {
            warningsHtml += `<div>${this.truncate(warning, [120, '...'])}</div>`;
        });

        let restrictionsHtml = warnings.length > 0 ? `
            <div class="tooltip-time-warnings">
                ${warningsHtml}
            </div>
        ` : '';

        let restrictionLinesHtml = '';

        restrictions.forEach((restriction) => {
            restrictionLinesHtml += `<div>${restriction}</div>`;
        });

        restrictionsHtml += restrictions.length > 0
            ? `
                <div class="tooltip-time-restrictions ${!this.authUserIsAdmin && 'tooltip-time-warnings'}">
                    ${restrictionLinesHtml}
                </div>
            `
            : '';

        const logSet = {
            when: [],
            what: [],
        };

        let todayLimit;
        let daysLimit = 0;
        let logHtml = '';

        log.forEach((lines, key) => {
            if (daysLimit < this.logLimitDays) {
                const createdAt = moment(key, 'YYYY-MM-DD');
                todayLimit = 0;
                daysLimit++;

                if (key !== 'today') {
                    logSet.when.push(`<div class="tooltip-time-log-row log-row-day">${createdAt.format('MMM D')}</div>`);
                    logSet.what.push(`<div class="tooltip-time-log-row log-row-day">&nbsp;</div>`);
                }

                lines.forEach((line) => {
                    if (todayLimit < this.logLimit) {
                        todayLimit++;
                        logSet.when.push(`<div class="tooltip-time-log-row">${line[0]}</div>`);
                        logSet.what.push(`<div class="tooltip-time-log-row">${line[1]}</div>`);
                    }

                    if (todayLimit === this.logLimit && lines.length > this.logLimit) {
                        logSet.when.pop();
                        logSet.what.pop();
                        logSet.when.push(`<div class="tooltip-time-log-row log-row-log-limit">+${lines.length - this.logLimit + 1} more</div>`);
                        logSet.what.push(`<div class="tooltip-time-log-row log-row-log-limit">&nbsp;</div>`);
                    }
                });

                if (daysLimit === this.logLimitDays && log.size > this.logLimitDays) {
                    daysLimit++;
                    logSet.when.push(`<div class="tooltip-time-log-row log-row-day log-row-log-limit">+${log.size - this.logLimitDays} more ${this.plural(log.size - this.logLimitDays, 'day', 'days')}</div>`);
                    logSet.what.push(`<div class="tooltip-time-log-row log-row-day log-row-log-limit">&nbsp;</div>`);
                }
            }
        });

        logHtml += log.size > 0 ? `
            <div class="tooltip-time-log-body">
                <div class="tooltip-time-log-when">
                    ${logSet.when.join('')}
                </div>
                <div class="tooltip-time-log-what">
                    ${logSet.what.join('')}
                </div>
            </div>
        ` : '';

        const commentHtml = comment !== '' && comment !== undefined ?
            `<div class="tooltip-time-comment"><span class="icon-quote icon-quote-small"></span><span>${this.getNl2Br(this.truncate(comment, [80, '...']))}</span></div>`
            : '';

        const htmlParts = [];

        if (restrictionsHtml !== '') {
            htmlParts.push(restrictionsHtml);
        }

        if (logHtml !== '') {
            htmlParts.push(`<div>${logHtml}</div>`);
        }

        if (commentHtml !== '') {
            htmlParts.push(commentHtml);
        }

        return htmlParts.length === 0
            ? placeholder
            : htmlParts.join('<div class="separator"></div>');
    },
    generateLogData: function(taskTime) {
        const history = (taskTime.history || []);
        const historyMap = new Map();
        const historyTodayMap = new Map();
        const historyArrayMap = new Map();

        for (const historyRow of history) {
            let historyLocalDate = moment
                .utc(historyRow.createdAt, 'YYYY-MM-DD HH:mm:ss')
                .local()
                .format('YYYY-MM-DD');

            let action = 'BY_DAY';
            let time = 0;

            if (historyLocalDate === taskTime.date) {
                if (!historyTodayMap.has(historyLocalDate)) {
                    historyTodayMap.set(historyLocalDate, []);
                }

                historyTodayMap.get(historyLocalDate).push(historyRow);

                continue;
            } else if (historyRow.action === 'TIMER') {
                const createdAt = moment.utc(historyRow.createdAt, 'YYYY-MM-DD HH:mm:ss').local();
                const startAt = createdAt.clone().subtract(historyRow.time, 'seconds').format('YYYY-MM-DD');

                historyLocalDate = startAt === taskTime.date ? taskTime.date : startAt;
                action = 'TIMER_PREVIOUS_DAY';
                time = historyRow.time;
            }

            if (historyRow.action !== 'TIMER') {
                time += historyRow.time;
            }

            if (!historyMap.has(historyLocalDate)) {
                historyMap.set(historyLocalDate, []);
            }

            const historyLocalItem = historyMap.get(historyLocalDate);

            const historyRowLocal = {
                action,
                createdAt: historyRow.createdAt,
                time,
            };

            const testByDayIndex = historyLocalItem.findIndex(item => item.action === 'BY_DAY');

            if (historyRowLocal.action === 'BY_DAY' && testByDayIndex !== -1) {
                historyLocalItem[testByDayIndex].time += time;
            } else {
                historyLocalItem.push(historyRowLocal);
            }

            historyMap.set(historyLocalDate, historyLocalItem);
        }

        historyMap.forEach((value, key) => {
            if (!historyTodayMap.has(key)) {
                historyTodayMap.set(key, value);
            } else {
                historyTodayMap.set(key, historyTodayMap.get(key).concat(value));
            }
        });

        Array
            .from(historyTodayMap.entries())
            .sort((a, b) => this.order(b[0], a[0]))
            .forEach(([key, value]) => {
                const nextValue = value
                    .filter(row => row.time > 30 || row.time < -30)
                    .sort((a, b) => this.order(b.createdAt, a.createdAt))
                    .map(row => [this.formatWhen(row), this.formatWhat(row)]);

                if (nextValue.length === 0) {
                    historyArrayMap.delete(key);
                } else {
                    historyArrayMap.set(key, nextValue);
                }
            });

        let showDate = false;

        if (historyMap.has(taskTime.date) && historyMap.get(taskTime.date).filter(item => item.action === 'TIMER_PREVIOUS_DAY').length > 0) {
            showDate = true;
        }

        if (historyArrayMap.size === 1 && historyTodayMap.has(taskTime.date) && showDate === false) {
            historyArrayMap.set('today', historyArrayMap.get(taskTime.date));
            historyArrayMap.delete(taskTime.date);
        }

        return historyArrayMap;
    },
    getLockAfterReason: function() {
        const team = this.profile.team;
        const message = 'You can’t change time as per team settings<br>';

        if (team.lockTimePeriod === 'days') {
            return `${message}(lock prev. entries ${team.lockTimeAfter > 0 ? 'after' : ''} ${this.lockAfter(team.lockTimePeriod, team.lockTimeAfter)})`;
        }

        if (team.lockTimePeriod === 'day-of-week') {
            return `${message}(lock prev. entries every ${this.lockAfter(team.lockTimePeriod, team.lockTimeAfter)})`;
        }

        return `${message}(lock prev. entries every ${this.ordinal(team.lockTimeAfter)} day of  month)`;
    },
    getTooltip(taskTime, canManageProjectTime = false, displayLog = true, displayComment = true, placeholder = 'No history') {
        let lockReasonsBuildSettings = null;

        if (taskTime !== null) {
            lockReasonsBuildSettings = {
                user: taskTime.user,
                lockReasons: taskTime.lockReasons,
                date: taskTime.date,
                task: taskTime.task,
            };
        }
        const lockReasons = this.getLockReasons(canManageProjectTime, lockReasonsBuildSettings);
        const warningReasons = this.getWarningReasons(taskTime);
        let comment = '';
        let log;

        if (displayLog === true && taskTime !== null) {
            log = this.generateLogData(taskTime);
        }

        if (displayComment === true && taskTime !== null) {
            comment = taskTime.comment;
        }

        return this.generateHtml(
            placeholder,
            lockReasons,
            warningReasons,
            log,
            comment,
        );
    },
    hasWarning: function(taskTime, canManageProjectTime) {
        const warningReasons = this.getWarningReasons(taskTime);

        if (warningReasons.length > 0) {
            return true;
        }
        let lockReasonsBuildSettings = null;

        if (taskTime !== null) {
            lockReasonsBuildSettings = {
                user: taskTime.user,
                lockReasons: taskTime.lockReasons,
                date: taskTime.date,
                task: taskTime.task,
            };
        }
        const lockReasons = this.getLockReasons(canManageProjectTime, lockReasonsBuildSettings);

        if (lockReasons.length > 0 && this.authUserIsAdmin === true) {
            return true;
        }
    },
    formatWarning: function(warning) {
        const byPolicy = 'Timer has been stopped as per team policy';
        switch (warning.type) {
            case 'timer_stopped_by_admin':
                const user = store.team.index[warning.createdBy];
                const who = this.escape(user ? (user.name || user.email) : ' admin');
                const comment = warning.comment ? ' – "' + this.escape(warning.comment) + '"' : '';
                return `Timer was stopped by ${who} ${comment}`;
            case 'timer_stopped_by_limit':
                return `Stopped because a limit to track time`;
            case 'timer_stopped_at_day_end':
                return `${byPolicy}<br>(stop all timers at day\'s end)`;
            case 'timer_stopped_by_client_limit':
                return `${byPolicy}<br>(the client to which the project belongs has a time tracking limit)`;
            case 'timer_stopped_by_project_limit':
                return `${byPolicy}<br>(this project has a time tracking limit)`;
            case 'timer_stopped_by_user_limit':
                return `${byPolicy}<br>(an admin set a limit)`;
            case 'timer_stopped_by_timecard':
                return `Timer was stopped after has been clocked out by admin`;
            case 'timer_stopped_by_estimate_reached':
                return `Timer was stopped by estimate limit`;
            case 'timer_stopped_by_time_limit':
                const policy = this.getTimeTrackingPolicy();
                return `Timer was stopped by timer limit<br>(max ${policy.timerAutostopAt}h of non-stop timer)`;
        }

        return null;
    },

    getWarningReasons: function(taskTime = null) {
        if (taskTime !== null && taskTime.warning) {
            return [this.formatWarning(taskTime.warning)];
        }

        return [];
    },
    getTimeTrackingPolicy: function() {
        return this.profile.timeTrackingPolicy;
    },

    calculateMinAllowedDate: function() {
        const format = 'YYYY-MM-DD';
        const timeTrackingPolicy = this.getTimeTrackingPolicy();
        if (!timeTrackingPolicy.lockTimeAfter && timeTrackingPolicy.lockTimeAfter !== 0) {
            return null;
        }

        let lockTimePeriod = timeTrackingPolicy.lockTimePeriod || 'no-lock';
        if (!timeTrackingPolicy.lockTimePeriod && timeTrackingPolicy.lockTimeAfter >= 0) {
            lockTimePeriod = 'days';
        }

        const today = moment.utc();

        switch (lockTimePeriod) {
            case 'days':
                return today.subtract(timeTrackingPolicy.lockTimeAfter, 'days').format(format);
            case 'day-of-week':
                const days = today.day() - timeTrackingPolicy.lockTimeAfter;
                return today.subtract((days >= 0 ? days : days + 7), 'days').format(format);
            case 'day-of-month':
                const daysInMonth = today.daysInMonth();
                const lockTimeAfter = timeTrackingPolicy.lockTimeAfter > daysInMonth ? daysInMonth : timeTrackingPolicy.lockTimeAfter;
                const startOfMonth = today.date() < lockTimeAfter ? today.subtract(1, 'month').startOf('month') : today.startOf('month');

                if (lockTimeAfter > 1) {
                    return startOfMonth.add((lockTimeAfter - 1), 'days').format(format);
                }

                return startOfMonth.format(format);
            default:
                return null;
        }
    },

    minAllowedDate: function({forceUseTeamLockTime = false} = {}) {
        if (forceUseTeamLockTime === false && this.isAdmin) {
            return null;
        }

        return this.calculateMinAllowedDate();
    },

    maxAllowedDate: function() {
        const timeTrackingPolicy = this.getTimeTrackingPolicy();

        if (timeTrackingPolicy.allowFutureTime) {
            return null;
        }

        return moment.utc();
    },
    isLockedFutureDate: function(date) {
        const max = this.maxAllowedDate();

        return max !== null && moment(date).isAfter(max) === true;
    },

    isLockedDate: function(date, {forceUseTeamLockTime = false} = {}) {
        const max = this.maxAllowedDate();

        if (max !== null && moment(date).isAfter(max) === true) {
            return true;
        }

        const min = this.minAllowedDate({forceUseTeamLockTime});

        return min !== null && moment(date).isBefore(min);
    },

    getTaskProject(prjs, projectsObj) {
        const projects = (prjs || [])
            .map(id => {
                if (store.projects.get(id)) {
                    return store.projects.get(id);
                }

                return (projectsObj || []).find((project) => project.id === id)
            })
            .filter(project => !!project);

        if (projects.length === 0) {
            return null;
        }

        for (const project of projects) {
            if (project && project.status === 'open' && !project.foreign) {
                return project;
            }
        }

        for (const project of projects) {
            if (project && !project.foreign) {
                return project;
            }
        }

        return projects[0];
    },

    isTaskProjectForeign: function(task) {
        const project = this.getTaskProject(task.projects, task.projectsObjects);

        return project !== null && project.foreign === true;
    },

    getLockReasons(canManageProjectTime, lockReasonsBuildSettings) {
        const role = this.authUserIsAdmin ? 'admin' : 'member';
        const reasons = [];

        if (lockReasonsBuildSettings !== null) {
            if (this.authUserIsMember === true && this.authUserId !== lockReasonsBuildSettings.user) {
                reasons.push(this.lockReasons.not_mine[role]);
                return reasons;
            }

            const lockReasons = new Set(lockReasonsBuildSettings.lockReasons || []);

            if (lockReasons.has('time_invoiced')) {
                reasons.push(this.lockReasons.time_invoiced[role]);
            }

            if (lockReasons.has('completed_task')) {
                reasons.push(this.lockReasons.completed_task[role]);
            }

            if (lockReasons.has('timesheet_approved')) {
                reasons.push(this.lockReasons.timesheet_approved[role]);
            }

            if (lockReasons.has('timesheet_approval_requested')) {
                reasons.push(this.lockReasons.timesheet_approval_requested[role]);
            }

            if (lockReasons.has('future_date') || this.isLockedFutureDate(lockReasonsBuildSettings.date) === true) {
                reasons.push(this.lockReasons.future_date[role]);
            } else if (
                !this.authUserIsAdmin &&
                !this.authUserIsSupervisor &&
                (
                    lockReasons.has('date') ||
                    this.isLockedDate(lockReasonsBuildSettings.date, {forceUseTeamLockTime: true}) === true
                )
            ) {
                reasons.push(this.getLockAfterReason());
            }

            if (lockReasons.has('project_archived') || canManageProjectTime === false) {
                reasons.push(this.lockReasons.project_archived[role]);
            }

            if (lockReasonsBuildSettings.task && this.isTaskProjectForeign(lockReasonsBuildSettings.task)) {
                reasons.push(this.lockReasons.project_foreign[role]);
            }

            return reasons;
        }

        if (canManageProjectTime === false) {
            reasons.push(this.lockReasons.project_archived[role]);
        }

        return reasons;
    },

    isLockedByReasons(taskTime) {
        let user = store.profile.get();

        if (taskTime.lockReasons === undefined) {
            return false;
        }

        if (user.role === 'admin') {
            return false;
        }

        if (user.role === 'supervisor') {
            return taskTime.lockReasons.filter(reason => reason !== 'date').length > 0;
        }

        return taskTime.lockReasons.length > 0;
    },

    isCanEditTime(date, userId, project) {
        if (date && this.isLockedFutureDate(date)) {
            return 'locked';
        }

        const user = store.profile.get();

        if (user.role === 'admin') {
            return 'permitted';
        }

        if (project && (project.foreign || project.status !== 'open')) {
            return 'locked';
        }

        if (userId !== null && user.id !== userId && user.role !== 'supervisor') {
            return 'denied';
        }

        if (user.role === 'supervisor') {
            return 'permitted';
        }

        const min = this.minAllowedDate();

        if (!min) {
            return 'permitted';
        }

        return date < min ? 'locked' : 'permitted';
    },

    isCanManageIndividualEstimate: function(project) {
        return project ? project.estimatesType === 'any' : true;
    }
}

window.Timer = function(selector) {
    this.selector = selector;

    this.init();
};

Timer.prototype = {

    interval: null,
    initialDuration: 0,
    startedAt: null,

    init: function() {

    },

    start: function() {
        this.stop();

        var element = $(this.selector);
        if (element.length == 0) {
            return ;
        }

        this.initialDuration = element.data('value');
        this.startedAt = Timer.timestamp();

        var self = this;
        this.interval = setInterval(function() {
            self.render();
        }, 500);

        self.render();
    },

    stop: function() {
        if (this.interval !== null) {
            clearInterval(this.interval);
        }
    },

    render: function() {
        var html = TimeEntryUtils.formatTimeSpendSeconds(this.getDuration()).replace(/\:/g, '<span>:</span>');
        $(this.selector).html(html);
    },

    getDuration: function() {
        return this.initialDuration + Timer.timestamp() - this.startedAt;
    }
};

Timer.timestamp = function() {
    return Math.ceil(new Date().getTime() / 1000);
};

window.Template = function(options) {
    this.templateHtml = '';
    this.template = null;
    this.init.apply(this, arguments);
};

Template.prototype = {
    init: function(selector) {
        this.templateHtml =  $(selector).html();
        this.compile();
    },

    render: function(params) {
        return this.template(Object.assign({}, params, {
            base: typeof(browserPort) !== 'undefined' && browserPort.resource ? browserPort.resource.getUrl('') : '/'
        }));
    },

    renderTo: function(element, params) {
        $(element).html(this.render(params));
    },

    compile: function() {
        this.template = _.template(this.templateHtml);
    },

    transform: function(callback) {
        this.templateHtml = callback(this.templateHtml);
        this.compile();
    }
};
window.Everhour = function(config) {
    this.config = config;
    this.baseUrl = 'https://everhour.com/api/v1';
    this.language = null;
    this.init();
};

Everhour.prototype = {
    init: function () {
        this.loadLanguage();
    },

    getAccessToken: function(creds) {
        return $.get(this.baseUrl + '/auth/getAccessToken/', creds);
    },

    validateAccessToken: function(accessToken) {
        return $.get(this.baseUrl + '/auth/validateAccessToken/', {
            accessToken: accessToken
        });
    },

    getTimeEntries: function(params) {
        params.accessToken = this.config.accessToken;
        return $.get(this.baseUrl + '/time_entries/', params);
    },

    getTimeEntry: function(id) {
        return $.get(this.baseUrl + '/time_entries/' + id + '/', {
            accessToken: this.config.accessToken
        });
    },

    getCurrentTimeEntry: function() {
        return $.get(this.baseUrl + '/timer/current/', {
            accessToken: this.config.accessToken
        });
    },

    stopTimer: function() {
        return $.get(this.baseUrl + '/timer/stop/', {
            accessToken: this.config.accessToken
        });
    },

    continueTimeEntry: function(timeEntryId) {
        return $.get(this.baseUrl + '/timer/continue/' + timeEntryId + '/', {
            accessToken: this.config.accessToken
        });
    },

    createTimeEntry: function(entry, today) {
        entry.today = today || moment().format('YYYY-MM-DD');
        return $.ajax({
            type: "POST",
            url: this.baseUrl + '/time_entries/?accessToken=' + this.config.accessToken,
            contentType: "application/json",
            data: JSON.stringify(entry)
        });
    },


    createTimeEntryFromText: function(comment, today) {
        return $.ajax({
            type: "PUT",
            url: this.baseUrl + '/time_entries/?accessToken=' + this.config.accessToken,
            contentType: "application/json",
            // TODO: do we use json params only in that case ?
            data: JSON.stringify({entry: comment, today: today || moment().format('YYYY-MM-DD')})
        });
    },

    addTimeEntry: function(issue, time, date) {
        return this.createTimeEntry({
            'mention': '@' + issue.repository.mention.name,
            'issue': (issue.number || issue.id),
            'comment': issue.name,
            'date': date, // YYYY-MM-DD
            'time': time // in minutes
        }).done(function(entry) {
            if (entry) {
                var isToday = (entry.date && entry.date == moment().format('YYYY-MM-DD'));
                issue.repository.updateTimeEntryStatistic(entry, isToday);
            }
        });
    },

    fetchRepositories: function(params) {
        params.accessToken = this.config.accessToken;
        return $.get(this.baseUrl + '/repositories/fetch/', params);
    },

    createIssue: function(repositoryId, issue) {
        return $.ajax({
            type: "PUT",
            url: this.baseUrl + '/repositories/' + repositoryId + '/issues/?accessToken=' + this.config.accessToken,
            contentType: "application/json",
            data: JSON.stringify(issue)
        });
    },

    loadIssueUserTime: function(repositoryId, issueNumber) {
        var params = {
            accessToken : this.config.accessToken
        };

        return $.get(this.baseUrl + '/repositories/' + repositoryId + '/issues/' + issueNumber + '/user-time/', params);
    },

    updateIssue: function(repositoryId, issueId, params) {
        params = params || {};

        return $.ajax({
            type: "PUT",
            url: this.baseUrl + '/repositories/' + repositoryId + '/issues/' + issueId + '/?accessToken=' + this.config.accessToken,
            contentType: "application/json",
            data: JSON.stringify(params)
        });
    },

    getIssue: function(repositoryId, issueId) {
        var params = {
            accessToken : this.config.accessToken
        };

        return $.get(this.baseUrl + '/repositories/' + repositoryId + '/issues/' + issueId + '/', params);
    },

    getUserReferences: function() {
        var params = {
            accessToken : this.config.accessToken
        };

        return $.get(this.baseUrl + '/users/references/', params);
    },

    getMentionIssues: function(mention) {

        var params = {
            accessToken : this.config.accessToken,
            mention : mention
        };

        return $.get(this.baseUrl + '/repositories/search-issues/', params);
    },

    getDomain: function() {
        var matches = this.baseUrl.match(/^https?\:\/\/([^\/?#]+)(?:[\/?#]|$)/i);
        var domain = matches && matches[1];

        return domain;
    },

    loadLanguage: function() {
        var promise = $.Deferred();
        var self = this;

        //chrome.i18n.getAcceptLanguages(function(result) {
        //    self.language = result[0];
        //    promise.resolve();
        //});

        promise.resolve();

        return promise.promise();
    }
}; 
window.Everhour2 = function(apiKey) {
    this.apiKey = apiKey ? apiKey.split('@')[0] : null;
    this.endpoint = 'https://app.everhour.com/api';
    this.readonlyEndpoint = null;
    this.productionUrl = 'https://app.everhour.com';
    this.channel = null;
    this.onError = null;
    this.socketConnectionId = null;
    this.sessionId = apiKey ? (apiKey.split('@')[1] || null) : null;
    this.loadProjectsInTasks = '';
};

Everhour2.prototype = {
    /**
     * @returns {boolean}
     */
    isAuthorized: function() {
        return this.apiKey && this.apiKey.length > 0;
    },

    /**
     *
     * @param {string} method
     * @param {string} resource
     * @param {object} data
     * @param {function} beforeSendCallback
     * @returns {*}
     */
    request: function(method, resource, data, beforeSendCallback) {
        Logger.log('api.request', method, resource);

        const endpoint = method.toUpperCase() === 'GET' && this.readonlyEndpoint && resource.indexOf('async/') === -1
            ? this.readonlyEndpoint
            : this.endpoint;

        var request = {
            url: endpoint + resource,
            dataType: 'json',
            method: method,
            contentType: 'application/json',
            headers: {
                'X-Api-Key': this.apiKey,
                'X-Accept-Version' : '1.3',
                'X-Device': 'ext'
            }
        };

        if (this.sessionId) {
            request.headers['X-Session-Id'] = this.sessionId;
        }

        if (data) {
            request.data = JSON.stringify(data);
        }

        if (beforeSendCallback) {
            request.beforeSend = beforeSendCallback;
        }

        const response = $.ajax(request);

        if (this.onError) {
            response.fail(this.onError);
        }

        return response;
    },

    /**
     * @param {number} userId
     * @returns {*}
     */
    socket: function(userId) {
        if (this.channel) {
            if (this.channel.name === 'user:' + userId) {
                return this.channel;
            }

            // if APIkey was changed - it means we need to close previous WS channel
            // and  open new one for another user
            if (this.closeSocket) {
                this.closeSocket();
            } else {
                this.channel.realtime.close();
            }
        }

        const authHeaders = {
            'X-Api-Key': this.apiKey
        };

        if (this.sessionId) {
            authHeaders['X-Session-Id'] = this.sessionId;
        }

        const endpoint = this.readonlyEndpoint || this.endpoint;

        /** @type Ably.Realtime **/
        const webscoketClient  = new Ably.Realtime({
            authUrl: endpoint + '/auth/subscribe',
            authHeaders: authHeaders
        });

        webscoketClient.connection.on('connected', () => {
            // Once the connection has been re-established, channels will be automatically re-attached.
            // connection ID may be changed due reconnect
            this.socketConnectionId = webscoketClient.connection.id;
        });

        // onFailed handler used for fatal/non-recoverable errors
        webscoketClient.connection.on('failed', stateChange => {
            // if (stateChange.reason.code === 40101) {
            // sure that we must try to call webscoketClient.connect() in any case on fatal error
            setTimeout(function() {
                webscoketClient.connect();
            }, 2000);

            console.log('WEBSOCKET FAILED WITH REASON:', stateChange.reason);
        });

        this.channel = webscoketClient.channels.get('user:' + userId);
        this.channel.on('detached', () => {
            if (this.channel && webscoketClient.connection.state === 'connected') {
                this.channel.attach();
            }
        });

        this.channel.subscribe('action', message => {
            if (message.connectionId !== this.socketConnectionId) {
                browserPort.eventDispatcher.broadcast('*', 'everhour:websocket', message.data);
            }
        });

        this.closeSocket = () => {
            this.channel = null;
            webscoketClient.close();
        };

        return this.channel;
    },

    /**
     *
     * @param {string} userId
     * @param {string|null} date
     * @returns {*}
     */
    getUserTimecard: function(userId, date) {
        const dateFormat = 'Y-MM-DD';
        const dateFormatted = (date ? moment(date, dateFormat) : moment()).format(dateFormat);

        return this.request('GET', '/users/' + userId + '/timecards/' + dateFormatted);
    },

    /**
     * @returns {[].<object>}
     */
    checkIn: function(userId, isClockIn) {
        return this.request('POST', '/users/' + userId + '/timecards/' + (!isClockIn ? 'clock-in' : 'clock-out'), {});
    },

    /**
     * @returns {[].<object>}
     */
    projects: function(platform, requestEvent) {
        const queryStringParts = [];
        let queryString = (platform || requestEvent) ? '?' : '';
        if (platform) {
            queryStringParts.push('platform=' + platform);
        }
        if (requestEvent) {
            queryStringParts.push('event=' + requestEvent);
        }

        return this.request('GET', '/projects' + queryString + queryStringParts.join('&'));
    },

    /**
     * @returns {[].<object>}
     */
    clients: function() {
        return this.request('GET', '/clients');
    },

    /**
     * @param {object} data
     * @param {string} requestEvent
     * @returns {[].<object>}
     */
    createClient: function(data, requestEvent) {
        const newClient = Object.assign({
            businessDetails: "",
            defaultDiscount: 0,
            defaultTax: {id: 0, rate: 0},
            paymentDueDays: 0,
        }, data);
        const url = '/clients?event=' + requestEvent + '-create';

        return this.request('POST', url, newClient);
    },

    /**
     * @returns {[].<object>}
     */
    projectById: function(projectId) {

        return this.request('GET', '/projects/' + projectId);
    },

    /**
     * @returns {[].<object>}
     */
    users: function() {
        return this.request('GET', '/team/users');
    },

    /**
     * @returns{[].<object>}
     * */
    teamStatus: function(from, to) {
        return this.request('GET', `/team/status?from=${from}&to=${to}`);
    },

    /**
     * @returns {object}
     */
    profile: function() {
        return this.request('GET', '/users/me');
    },


    /**
     * @returns {object}
     */
    profileTeamPolicy: function() {
        return this.request('GET', '/team/time-tracking-policy');
    },

    /**
     * @returns {object}
     */
    setTimezone: function(timezone) {
        return this.request('PUT', '/users/me', {timezone});
    },

    /**
     * @returns {object}
     */
    completeOnboardingStep: function(extensionSettings) {
        return this.request('PUT', '/users/me', extensionSettings);
    },

    /**
     * @returns {[].<object>}
     */
    teamApplications: function() {
        return this.request('GET', '/team/jira-instances');
    },

    /**
     * @returns {[].<object>}
     */
    tasks: function(platform, params) {
        let queryStringParts = ['compact=true'];

        if (platform) {
            queryStringParts.push('platform=' + platform);
        }

        if (params) {
            queryStringParts = queryStringParts.concat(params);
        }

        return this.request('GET', '/reports/progress?' + queryStringParts.join('&'));
    },

    /**
     * @returns {[].<object>}
     */
    todayProgress: function(platform) {
        return this.request('GET', '/reports/today-progress' + (platform ? '?platform=' + platform : ''));
    },

    /**
     * @oaram {[].<string>}
     * @returns {[].<object>}
     */
    totalsByProjects: function(projectIds) {
        let url = '/reports/projects-progress';
        // it will works while < 60 project Ids
        url += '?projects[]=' + projectIds
            .map(encodeURIComponent)
            .join('&projects[]=');

        return this.request('GET', url);
    },

    /**
     * @oaram {<string>}
     * @returns {[].<object>}
     */
    progressByProject: function(projectId) {
        return this.request('GET', `/reports/projects-progress/${projectId}`);
    },

    /**
     * @param {string} taskId
     * @param {object} data
     * @returns {*}
     */
    presync: function(taskId, data) {
        return this.request('POST', '/tasks/' + taskId + '/presync', data);
    },

    /**
     * @param {string} projectId
     * @returns {*}
     */
    pushsync: function(projectId) {
        return this.request('POST', `/projects/${projectId}/push-sync`, {ignoreSyncedAt: false})
            .then(status => this.observeMessage(status, null, false));
    },

    /**
     * @param {string} taskId
     * @returns {*}
     */
    taskTime: function(taskId, limit, page) {
        let params = '';

        if (typeof limit === 'number' && typeof page === 'number') {
            params += `&limit=${limit}&page=${page}`;

            return this.request('GET', '/tasks/' + taskId + '/time?opts_include_projects=1&' + params);
        }

        return this.request('GET', '/tasks/' + taskId + '/time?' + this.loadProjectsInTasks);
    },

    /**
     * @param {string} taskId
     * @returns {*}
     */
    task: function(taskId) {
        return this.request('GET', '/tasks/' + taskId + '?' + this.loadProjectsInTasks);
    },

    /**
     * @param {string} taskId
     * @returns {*}
     */
    taskMetrics: function(taskId) {
        return this.request('GET', `/tasks/${taskId}/metrics`);
    },

    /**
     * @param {string} taskId
     * @param {object} data
     * @returns {*}
     */
    updateTask(taskId, data) {
        return this.request(`PUT`, `/tasks/${taskId}`, data);
    },

    /**
     * @param {string} from
     * @param {string} to
     * @param {number|null} limit
     * @returns {[].<object>}
     */
    activity: function(from, to, limit) {
        return this.request('GET', '/users/me/time?from=' + from + '&to=' + to  + (limit ? '&limit=' + limit : '') + '&' + this.loadProjectsInTasks);
    },

    /**
     * @param {string} from
     * @param {string} to
     * @param {number} userId
     * @returns {[].<object>}
     */
    exportTime: function(from, to, userId) {
        return this.request('GET', '/team/time/export?from=' + from + '&to=' + to  + '&fields=date,projects&user=' + userId + '&allowTimeWithoutTask=true');
    },

    /**
     * @param {object} data
     * @returns {*}
     */
    createTaskTime: function(data) {
        return this.request(`POST`, `/time`, data);
    },

    /**
     * @param {string} taskTimeId
     * @param {object} data
     * @returns {*}
     */
    updateTaskTime: function(taskTimeId, data) {
        return this.request('PUT', `/time/${taskTimeId}?` + this.loadProjectsInTasks, data);
    },

    /**
     * @param {string} taskTimeId
     * @param {object} data
     * @returns {*}
     */
    moveTaskTime: function(taskTimeId, data) {
        return this.request('PUT', `/time/${taskTimeId}?` + this.loadProjectsInTasks, data);
    },

    /**
     * @param {array} data
     * @returns {*}
     */
    batch: function(data) {
        return this.request('PATCH', '/batch', {actions: data});
    },

    /**
     * @param {string} taskId
     * @param {object} data
     */
    setEstimate: function(taskId, data) {
        return this.request('PUT', '/tasks/' + taskId + '/estimate?' + this.loadProjectsInTasks, data);
    },

    /**
     * @returns {[].<object>}
     */
    timer: function() {
        return this.request('GET', '/timers/current?' + this.loadProjectsInTasks);
    },

    bulkUpdateTime: function(taskTimeEntries) {
        return this.request('PUT', '/time?' + this.loadProjectsInTasks, {
            data: taskTimeEntries
        });
    },

    /**
     * @param {string?}  taskId
     * @param {string?}  taskTimeId
     * @param {string?}  comment
     * @param {string?}  website
     * @param {object?}  taskData
     * @returns {*}
     */
    startTimer: function(taskId, taskTimeId, comment, website, taskData) {
        let data = Object.assign({userDate: moment().format('YYYY-MM-DD')},
            taskId ? {task: taskId} : {},
            taskTimeId ? {taskTime: taskTimeId} : {},
            {taskData}
        );

        if (comment || comment === '') {
            data.comment = comment;
        }

        if (website) {
            data.website = website;
        }

        return this.request('POST', '/timers?' + this.loadProjectsInTasks, data);
    },

    /**
     * @param {string?}  url
     * @param {string?}  comment
     * @returns {*}
     */
    startWebsiteTimer(url, comment) {
        let data = Object.assign({
            userDate: moment().format('YYYY-MM-DD'),
            comment,
            website: {
                url,
                title: comment
            }
        });

        return this.request('POST', '/timers?' + this.loadProjectsInTasks, data);
    },

    /**
     * @param {string} projectId
     * @param {Object} billableSetting
     * @returns {*}
     */
    updateProjectBillingSettings: function(projectId, billableSetting) {
        return this.request('PUT', '/projects/' + projectId + '/billing', billableSetting);
    },

    /**
     * @param {string} projectId
     * @param {number} clientId
     * @param {string} requestEvent
     * @returns {*}
     */
    updateProjectClient: function(projectId, clientId, requestEvent) {
        const data = clientId ? {client: clientId} : null;
        const requestMethod = clientId ? 'PUT' : 'DELETE';
        const url = '/projects/' + projectId + '/client?event=' + requestEvent + '-update';

        return this.request(requestMethod, url, data);
    },

    getProjectSections: function(projectId) {
        return this.request(`GET`, `/projects/${projectId}/sections`);
    },

    /**
     * @param {string} taskId
     * @param {boolean} isUnbillable
     * @returns {*}
     */
    updateTaskBillingSettings: function(taskId, isUnbillable) {
        return this.request('PUT', '/tasks/' + taskId + '/billing', isUnbillable);
    },

    stopTimer: function(id, comment) {
        const url = id ? `/timers/${id}?` + this.loadProjectsInTasks : `/timers/current?` + this.loadProjectsInTasks;

        if (comment) {
            return this.request('DELETE', url, {comment});
        }

        return this.request('DELETE', url);
    },

    /**
     * @param {Object} data
     * @returns {*}
     */
    commentTimer: function(data) {
        return this.request('PUT', '/timers/current?' + this.loadProjectsInTasks, data);
    },

    available: function() {
        if (!this.isAuthorized()) {
            return resolvedPromise();
        }

        return this.request('POST', '/users/me/extension-used', {extensionVersion: currentExtensionVersion()});
    },

    /**
     * @param {string} id - user_id + year + week
     * @param {number} limit - count tasks
     * @returns {*}
     */
    getRecentTasks: function(id, limit) {
        let url = `/tasks/recent?opts_include_projects=1`;

        if (limit) {
            url += `&limit=${limit}`;
        }

        return this.request('GET', url);
    },

    /**
     * @param {string} searchTerm
     * @returns {*}
     */
    searchTasks: function(searchTerm, limit, beforeSendCallback) {
        return this.request('GET', '/tasks/search?limit=30&query=' + (encodeURIComponent(searchTerm)) + '&' + this.loadProjectsInTasks, null, beforeSendCallback);
    },

    /**
     * @param {string} searchTerm
     * @returns {*}
     */
    searchRecentTasks: function(searchTerm, limit, withComments, beforeSendCallback) {
        let url = `/search?opts_include_projects=1`;

        if (searchTerm) {
            url += `&query=${searchTerm}&limit=${limit || 30}&includeCompleted=1`;
        }

        url += `&includeComments=${withComments ? 1 : 0}`;

        return this.request('GET', url, null, beforeSendCallback);
    },

    /**
     * @param {string} projectId
     * @returns {*}
     */
    getTasksByProject: function(projectId) {
        return this.request('GET', `/projects/${projectId}/tasks?opts_include_projects=1&exclude-closed=1&limit=30`);
    },

    /**
     * @param {string} projectId
     * @returns {*}
     */
    getRecentTasksByProject: function(projectId) {
        return this.request('GET', `/projects/${projectId}/tasks/recent?opts_include_projects=1`);
    },

    /**
     * @param {string} searchTerm
     * @param {string} projectId
     * @param {function} beforeSendCallback
     * @returns {*}
     */
    searchTasksByProject: function(searchTerm, projectId, beforeSendCallback) {
        return this.request('GET', `/projects/${projectId}/tasks/search?opts_include_projects=1&exclude-closed=1&query=${encodeURIComponent(searchTerm)}&limit=30`, null, beforeSendCallback)
    },

    deleteTaskTime(taskTimeId) {
        return this.request(`DELETE`, `/time/${taskTimeId}`);
    },

    /**
     * @param {string} event
     * @param {Object} payload
     * @returns {*}
     */
    analyticsEvent: function(event, payload) {
        return this.request('POST', '/analytics/events', {
            event: event,
            payload: payload
        });
    },

    /**
     * @param {string} object
     * @param {string} property
     * @param {*} value
     * @returns {*}
     */
    analyticsAttribute: function(object, property, value) {
        return this.request('POST', '/analytics/attributes', {
            object: object,
            property: property,
            value: value
        });
    },

    /**
     * @returns {*}
     */
    exchangeApiKey: function() {
        return this.request('GET', '/users/me/api-key');
    },

    /**
     *
     * @param {Object} status
     * @param {string|null} errorMessage
     * @param {boolean} isErrorSuppressed
     * @returns {*}
     */
    observeMessage: function(status, errorMessage, isErrorSuppressed = false) {
        const deferred = $.Deferred();
        let attempts = 0;
        const sync = () => {
            return this.request('GET', '/async/messages/' + status.messageId).then(status => {
                const failedResponse = {
                    status: 400,
                    responseJSON: {
                        message: errorMessage || 'Account sync process failed, please try again.'
                    }
                };
                if (
                    status.messageStatus === 'missing' || status.messageStatus === 'failed' ||
                    status.messageStatus === 'completed' && status.flushStatus === 'missing' && !status.flushChanges
                ) {
                    // do not throw error immediately
                    deferred.reject(isErrorSuppressed ? null : failedResponse);
                    return;
                }

                if (status.messageStatus === 'completed' && status.flushStatus !== 'scheduled') {
                    // TODO need to rework/remove, seems this case isn't actual now
                    deferred.resolve(status.flushChanges > 0);
                    return;
                }

                attempts++;

                if (attempts > 60) {
                    deferred.reject(isErrorSuppressed ? null : failedResponse);
                    return;
                }

                setTimeout(sync, 1000);
            }).fail(deferred.reject);
        };

        sync();

        return deferred;
    },

    /**
     *
     * @param {string} accountId
     * @param {[].<string>?} projects
     * @param {boolean} isErrorSuppressed
     * @returns {Q.Promise<any> | * | PromiseLike<any>}
     */
    synchronizeAccount: function(accountId, projects, isErrorSuppressed = false) {
        let params = {
            ignoreSyncedAt: true
        };

        if (projects && _.isArray(projects) && projects.length > 0) {
            params.projects = projects;
        }

        return this.request('POST', '/accounts/' + accountId + '/push-sync', params)
            .then(status => this.observeMessage(status, null, isErrorSuppressed));
    },

    /**
     * @param {string} accountId
     * @param {object} settings
     * @returns {*}
     */
    updateTrelloAccountSetting: function(accountId, settings) {
        return this.request('PUT', '/accounts/' + accountId + '/trello-settings', settings);
    },

    /**
     * @param {string} platform
     * @returns {*}
     */
    getPlatformSettings: function(platform) {
        return this.request(`GET`, `/team/integrations/${platform}`);
    },

    /**
     * @param {string} accountId
     * @param {object} state
     * @returns {*}
     */
    updateAsanaAccountSetting: function(accountId, state) {
        return this.request('PUT', '/accounts/' + accountId + '/asana-settings', state);
    },

    /**
     * @param {string} accountId
     * @param {object} data
     * @returns {*|PromiseLike<T | never>|Promise<T | never>}
     */
    duplicateProjectSettings: function(accountId, data) {
        return this.request('POST', '/accounts/' + accountId + '/duplicate-project-settings', data)
            .then(status => this.observeMessage(status, 'Project duplication in Everhour failed.'));
    },


    /**
     * @param {string} accountId
     * @param {object} data
     * @returns {*|PromiseLike<T | never>|Promise<T | never>}
     */
    connectAccountProjects: function(accountId, data) {
        return this.request('POST', '/accounts/' + accountId + '/projects ', data);
    },

    /**
     * @returns {PromiseLike<T>|Promise<T | never>}
     */
    saveIdleState: function(currentState, idleDuration) {
        const data = {
            state: currentState,
            idleDuration: idleDuration
        };

        return this.request('POST', '/users/me/idle', data);
    },

    /**
     * @param platform
     * @param version
     * @returns {object}
     */
    loadSelectorsDictionary: function(platform, version) {
        const response = $.ajax({
            url: 'https://everhour.com/addon/overrides/' + platform + '_' + version + '.json',
            dataType: 'json',
            contentType: 'application/json',
            cache: false
        });

        if (this.onError) {
            response.fail(this.onError);
        }

        return response;
    },

    sendNps: function(score, comment, placement) {
        return this.request(`POST`, `/nps`, {score, comment, placement});
    },

    extendTrial: function() {
        return this.request(`PUT`, `/billing/extend-trial`, null, null);
    },

    locks: {},

    /**
     * Prevents concurrent api calls
     *
     * Usage:
     *
     * const promise = everhour.lock('timer', () => everhour.startTimer(..));
     *
     * It will make sure that you can't start timer multiple times.
     * If you're starting timer it will return current promise without extra API request.
     *
     * You can use this method for any promises, not only AJAX:
     *
     * const promise = everhour.lock('some_long_action', () => {
     *     const def = $.Deferred();
     *     setTimeout(() => def.resolve(), 1000);
     *     return def.promise();
     * });
     *
     * @param {string} lockId
     * @param {() => Promise} promiseFactory
     * @returns {*}
     */
    lock: function(lockId, promiseFactory) {
        if (!this.locks[lockId]) {
            const defer = $.Deferred();
            const promise = promiseFactory();

            promise.then(
                result => defer.resolve(result),
                err => defer.reject(err),
                progress => defer.notify(progress)
            );

            promise.done(() => {
                this.locks[lockId] = null;
            });
            promise.fail(() => this.locks[lockId] = null);

            this.locks[lockId] = defer;
        }

        this.locks[lockId].notify();

        return this.locks[lockId].promise();
    }
};

window.TimeEntryUtils = {
    onlyNumbersPattern: '(\\d{0,10}(\\.\\d{0,2})?)',
    hoursPattern: '(\\d{0,13}(\\.\\d{0,2})?)\\s*(h|hr|hrs|hour|hours)',
    minutesPattern: '(\\d{1,10})\\s*(m|min|mins|minute|minutes)',

    getTimeRegexp: function() {
        let timePattern = '(' + this.hoursPattern + '\\s+' +
            this.minutesPattern + '|' + this.hoursPattern + '|' +
            this.minutesPattern + '|' + this.onlyNumbersPattern + ')';

        return new RegExp('(^)' + timePattern + '($)', 'i');
    },

    /**
     * Format minutes in XXh YYm
     * @param minutes
     * @returns {string}
     */
    formatSpendTime: function (minutes) {
        if (minutes == 0) {
            return 0;
        }

        minutes = Math.round(minutes);
        let hours = parseInt(minutes) / 60;

        if (isNaN(hours)) {
            return '-';
        }

        hours = Math.floor(hours);
        minutes = minutes - hours * 60;

        let parts = [];

        if (hours > 0) {
            parts.push(hours + 'h');
        }

        if (minutes > 0) {
            parts.push(minutes + 'm');
        }

        return parts.join(' ');
    },

    /**
     * Format seconds in XXh YYm
     * @param {number} seconds
     * @returns {string}
     */
    format: function (seconds, placeholder, format) {
        placeholder = typeof placeholder !== 'string' ? '-' : placeholder;

        if (seconds === 0) {
            return placeholder;
        }

        if (seconds < 60) {
            seconds = 60;
        }

        let hours = Math.floor(seconds / 60 / 60);
        let minutes = Math.floor(seconds / 60) % 60;

        switch (format) {
            // 1:00 / 1:05 / 0:05
            case 'clock':
                return (hours ? hours : '0') + ':' + zeroPrefixed(minutes);

            // 01:00 / 01:05 / 0:05
            case 'clock_full':
                return zeroPrefixed(hours ? hours : 0) + ':' + zeroPrefixed(minutes);

            // 1.00 / 1.08 / 0.08
            case 'hours':
                return (Math.round(seconds / 60 / 60 * 100) / 100).toFixed(2);

            // 1h 00m / 1h 05m / 0h 05m
            case 'text':
                return (hours ? hours + 'h' : '0h') + ' ' + zeroPrefixed(minutes) + 'm';

            // 1h / 1h 5m / 5m
            case 'text_short':
                return (hours ? hours + 'h' : '') + (hours && minutes ? ' ' : '') + (minutes ? minutes + 'm' : '') + (!hours && !minutes ? '0' : '');

            // 1h / 1h 5m / 5m
            case 'text_short_rounded':
                if (seconds <= 60) {
                    return '1m';
                }

                if (seconds < 3600) {
                    return Math.floor(seconds / 60) + 'm'
                }

                return Math.round(seconds / 3600) + 'h';

            case 'hours_only':
                return Number.isInteger(seconds / 60 / 60) ? hours.toString() : (Math.round(seconds / 60 / 60 * 100) / 100).toFixed(2);

            case 'hours_only_rounded':
                return (Number.isInteger(seconds / 60 / 60) ? hours.toString() : (Math.round(seconds / 60 / 60 * 100) / 100).toFixed(0)) + 'h';
        }

        if (isNaN(hours) || isNaN(minutes)) {
            return placeholder;
        }

        if (hours && !minutes) {
            return hours + 'h';
        }

        if (hours && minutes) {
            return hours + 'h ' + minutes + 'm';
        }

        if (minutes) {
            return minutes + 'm';
        }

        return placeholder;
    },

    /**
     * Format seconds in HH:MM:SS
     * @param time
     * @returns {string}
     */
    formatTimeSpendSeconds: function(time) {
        let total = [];
        let seconds = Math.round(time % 60);
        let minutes = Math.floor(time / 60);
        let hours = Math.floor(minutes / 60);

        minutes = minutes % 60;
        hours = hours % 24;

        total.push(hours >= 10 ? hours : '0' + hours);
        total.push(minutes >= 10 ? minutes : '0' + minutes);

        let secondsFormatted = seconds === 60 ? 0 : seconds;

        total.push(secondsFormatted >= 10 ? secondsFormatted : '0' + secondsFormatted);

        return total.join(':');
    },

    /**
     * Format seconds in HH:MM
     * @param time
     * @returns {string}
     */
    formatTimeSpendSecondsShort: function(time) {
        let total = [];
        let minutes = Math.floor(time / 60);
        let hours = Math.floor(minutes / 60);

        minutes = minutes % 60;
        hours = hours % 24;

        if (minutes === 0 && hours === 0) {
            minutes = 1;
        }

        total.push(hours >= 10 ? hours : '0' + hours);
        total.push(minutes >= 10 ? minutes : '0' + minutes);

        return total.join(':');
    },

    /**
     * Format minutes in HH:MM:SS
     * @param time
     * @returns {string}
     */
    formatTimeSpendMinutes: function(time) {
        let total = [];
        let minutes = Math.floor(time / 60);
        let hours = Math.floor(minutes / 60);

        minutes = minutes % 60;
        hours = hours % 24;

        total.push(hours >= 10 ? hours : '0' + hours);
        total.push(minutes >= 10 ? minutes : '0' + minutes);

        return total.join(':');
    },

    /**
     * Format time entry to html
     * @param entry
     * @returns {string}
     */
    formatTimeEntry: function(entry) {
        let html = _.escape(entry.comment);

        if ($.trim(html).length == 0) {
            return '<i>without comment</i>';
        }

        if (entry.mention) {
            html = html.replace('@' +  _.escape(entry.mention.name), '<b>@' +  _.escape(entry.mention.name) + '</b>');
        }

        let containsIssueNumber = false;
        html = html.replace(/(\s|^)(\#[^!"#\$%&'()*+,./:;<=>?@\^\[\]`{|}~\s]+)/ig, function(all, prefix, tag) {
            let isNumber = /^#(\d+|[a-z][a-z0-9]+\-\d{1,6})$/ig.test(tag);

            // don't format second issue number
            if (isNumber && containsIssueNumber) {
                return prefix + tag;
            }

            if (isNumber && entry.issue && entry.issue.url) {
                containsIssueNumber = true;
                return prefix + '<a href="' + encodeURI(entry.issue.url) + '" title="' +  _.escape(entry.issue.name ? entry.issue.name : '') + '" class="external">' +  _.escape(tag) + '</a>';
            }
            return prefix + '<i>' +  _.escape(tag) + '</i>';
        });

//        html += ' <b>' + TimeEntryUtils.formatSpendTime(entry.time_spend) + '</b>';

        return html;
    },

    /**
     * Get names of months
     *
     * @returns {string[]}
     */
    getMonthsNames: function() {
        return ["Jan", "Feb", "Mar",
            "Apr", "May", "Jun", "Jul", "Aug", "Sep",
            "Oct", "Nov", "Dec"];
    },

    /**
     * Format date from datepicker
     *
     * @param date
     */
    formatDate: function(date) {
        let monthsNames = this.getMonthsNames();
        let dateObject = new Date(date);
        let month = dateObject.getMonth();

        return monthsNames[month] + ',' + dateObject.getDate();
    },

    /**
     * Validate time
     *
     * @param time
     * @returns {boolean}
     */
    validateTime: function(time) {
        if (time.toString().trim().length > 20) {
            return false;
        }

        const matches = this.getTimeRegexp().exec(time.toString().trim());

        return !!matches;
    },

    /**
     * @param {string} time
     * @returns {string}
     */
    parseTime: function(time) {
        time = $.trim(time);
        if (time.indexOf(',') !== -1) {
            time = time.replace(',', '.')
        }

        if (time.indexOf(':') !== -1) {
            time = time.replace(':', 'h ') + 'm';
        }

        time = $.isNumeric(time) ? (time + 'h') : time;

        if (!TimeEntryUtils.validateTime(time)) {
            return null;
        }

        return time;
    },

    parseTimeToSeconds: function(value) {
        if (!value) {
            return '';
        }

        if (value.indexOf(',') !== -1) {
            value = value.replace(',', '.');
        }

        if (value.indexOf(':') !== -1) {
            value = value.replace(':', 'h ') + 'm';
        }

        const matches = this.getTimeRegexp().exec(value.toString().trim().toLowerCase());

        if (matches === null) {
            return null;
        }

        let hoursString = matches[3] || matches[8] || matches[13];

        // handle hours like .25h (without 0 at the beginning)
        if (hoursString && hoursString.substr(0, 1) === '.') {
            hoursString = '0' + hoursString;
        }

        const hours = parseFloat(hoursString);
        const minutes = parseInt(matches[6] || matches[11], 10);

        let seconds = 0;

        if (minutes && !Number.isNaN(minutes)) {
            seconds += minutes * 60;
        }

        if (hours && !Number.isNaN(hours)) {
            seconds += this.toSeconds(hours);
        }

        return Math.round(seconds);
    },

    toSeconds: function(hours) {
        return hours * 60 * 60;
    },

    /**
     * @param {string} time like `00:35` / `12:00 am`
     * @returns {string|null}
     */
    parseDayTime: function(time) {
        const is24Format = store.profile.is24TimeFormat();
        const format = 'HH:mm' + (is24Format ? ''  : ' a');
        const dayTimePattern = new RegExp(is24Format ?
            /^([01][0-9]|2[0-3]):([0-5][0-9])$/i :
            /^([01][0-9]):([0-5][0-9])\s(am|pm)$/i
        );
        let timeParts = dayTimePattern.exec(time.trim().toLowerCase());

        if (!timeParts) {
            return null;
        }
        const momentDayTime = moment(time, format);

        if (!momentDayTime.isValid()) {
            return null;
        }
        // return time in 24 for parse it later
        return this.parseTime(momentDayTime.format('HH:mm'));
    },

    /**
     * @param {string} date
     * @returns {string}
     */
    parseDate: function(date) {
        date = $.trim(date);
        let momentObj = moment(date, 'MM-DD-YYYY');
        let datePattern = /^((0?[1-9]|1[012])[- /.](0?[1-9]|[12][0-9]|3[01])[- /.](19|20)?[0-9]{2})*$/;

        if (datePattern.test(date) && momentObj && momentObj.isValid()) {
            return momentObj.format('YYYY-MM-DD');
        }

        return null;
    },

    /**
     * Get time spend minutes
     */
    getTimeSpendMinutes: function(time) {
        time = $.trim(time);
        let regex = /(([0-9]*[.]?[0-9]+)\s*(hours|hour|hrs|hr|h))?\s*(([0-9]*[.]?[0-9]+)\s*(minutes|minute|mins|min|m))?/i;
        let matches = time.split(regex);
        let result = 0;

        if (matches[2]) {
            result += parseFloat(matches[2]) * 60;
        }

        if (matches[5]) {
            result += parseFloat(matches[5]);
        }

        return result;
    },

    /**
     * Check if date is today
     */
    isToday: function(date) {
        return date && date == moment().format('YYYY-MM-DD');
    },

    isClockedIn: function(timecard) {
        return timecard.hasOwnProperty('clockIn') && !timecard.hasOwnProperty('clockOut');
    }
};

window.zeroPrefixed = function (value) {
    if (value === 0) {
        return '00';
    }

    return value >= 10 ? value.toString() : '0' + value.toString();
};

window.getPlatformCode = function (id) {
    if (!id) {
        return '';
    }

    switch (id.substr(0, 2)) {
        case 'as': return 'asana';
        case 'jr': return 'jira';
        case 'gh': return 'github';
        case 'bb': return 'bitbucket';
        case 'tw': return 'teamwork';
        case 'tr': return 'trello';
        case 'b2': return 'basecamp2';
        case 'b3': return 'basecamp3';
        case 'pv': return 'pivotal';
        case 'ev': return 'everhour';
        case 'in': return 'insightly';
        case 'sl': return 'slack';
        case 'xr': return 'xero';
        case 'fb': return 'freshbooks';
        case 'qb': return 'quickbooks';
        case 'cl': return 'clickup';
        case 'gl': return 'gitlab';
        case 'mo': return 'monday';
        case 'td': return 'todoist';
        case 'no': return 'notion';
        case 'li': return 'linear';
    }
};

window.getPlatformName = function (code) {
    if (Array.isArray(code)) {
        return code.map(code => getPlatformName(code));
    }

    let platform = getPlatformCode(code);
    if (platform === 'basecamp2' || platform === 'basecamp3') {
        platform = 'basecamp';
    }

    return platform;
};

window.getCapitalizedName = function (platform)
{
    switch (platform) {
        case 'asana': return 'Asana';
        case 'jira': return 'JIRA';
        case 'github': return 'Github';
        case 'teamwork': return 'Teamwork';
        case 'bitbucket': return 'Bitbucket';
        case 'trello': return 'Trello';
        case 'basecamp2': return 'Basecamp 2';
        case 'basecamp3': return 'Basecamp 3';
        case 'pivotal': return 'Pivotal';
        case 'everhour': return 'Everhour';
        case 'insightly': return 'Insightly';
        case 'freshbooks': return 'Freshbooks';
        case 'quickbooks': return 'Quickbooks';
        case 'slack': return 'Slack';
        case 'xero': return 'Xero';
        case 'clickup': return 'ClickUp';
        case 'gitlab': return 'Gitlab';
        case 'todoist': return 'Todoist';
        case 'notion': return 'Notion';
        case 'monday': return 'Monday';
        case 'linear': return 'Linear';
    }
};

window.escapeMap = {
    '&': '&amp;',
    '<': '&lt;',
    '>': '&gt;',
    '"': '&quot;',
    "'": '&#x27;',
    '`': '&#x60;'
};

window.escapeTestRegexp = RegExp('(?:' + Object.keys(escapeMap).join('|') + ')');
window.escapeReplaceRegexp = RegExp('(?:' + Object.keys(escapeMap).join('|') + ')', 'g');

/**
 * Shortcut for TimeEntryUtils in order to simplify templates
 */
_format = $.extend(_format, {
    time: TimeEntryUtils.format,
    isClockedIn: TimeEntryUtils.isClockedIn,
    date: (date, format) => moment(date, 'YYYY-MM-DD').format(format ? format : 'MMM, DD'),
    timeRound: (seconds, roundTo) => {
        seconds = Math.abs(seconds);

        if (roundTo <= 1) {
            return seconds;
        }

        if (seconds <= roundTo) {
            return roundTo;
        }

        return Math.round(seconds / roundTo) * roundTo;
    },
    weekDay: (date) => {
        date = moment(date);

        if (date.isSame(moment(), 'day')) {
            return 'Today';
        }

        if (date.isSame(moment().subtract(1, 'day'), 'day')) {
            return 'Yesterday';
        }

        return date.format('dddd');
    },
    monthDay: (date, withSup) => {
        date = moment(date);

        let monthDay = date.format('MMM D');
        let ordinal = date.format('Do').replace(/[0-9]/g, '');
        if (!withSup) {
            return monthDay;
        }

        return `${monthDay}<sup>${ordinal}</sup>`;
    },
    getDailyProgress: (time) => {
        let overtimeMin = 9 * 3600;
        let overtimeMax = 14 * 3600;
        switch(true) {
            case time <= 0: return 'low';
            case time >= overtimeMin && time <= overtimeMax: return 'overtime';
            case time > overtimeMax: return 'error';
            default: return 'normal';
        }
    },
    week: (date, relative) => {
        let mdate = moment(date, 'YYYY-MM-DD').weekday(0);

        if (relative) {
            let currentWeek = parseInt(moment().endOf('week').format('YYYYWW'));

            let dateWeek = parseInt(mdate.endOf('week').format('YYYYWW'));

            if (currentWeek === dateWeek) {
                return 'This Week';
            }

            let lastWeek = parseInt(moment().subtract(1, 'week').endOf('week').format('YYYYWW'));

            if (lastWeek === dateWeek) {
                return 'Last Week';
            }
        }

        return _format.date(mdate.startOf('week'), 'MMM, DD') + ' - ' + _format.date(mdate.endOf('week'), 'MMM, DD')
    },
    getPlatformName: getPlatformName,
    getCapitalizedName: getCapitalizedName,
    getPlatformCode: getPlatformCode,
    nl2br: text => _.escape(text).replace(/\n/g, '<br>'),
    escape: (stringValue) => {
        if (!stringValue) {
            return '';
        }

        return escapeTestRegexp.test(stringValue) ? stringValue.replace(escapeReplaceRegexp, m => escapeMap[m]) : stringValue;
    }
});

window.Form = function(element, controls) {
    this.element = element;
    this.controls = {};

    if (controls) {
        this.setControls(controls);
    }
};

Form.prototype = {
    setControls: function(controls) {
        let names = Object.keys(controls), name, control;
        for (let i = 0; i < names.length; i++) {
            name = names[i];
            control = controls[name];
            if (typeof(control.validator) !== 'function') {
                control.validator = () => null;
            }

            if (typeof(control.getter) !== 'function') {
                control.getter = (element) => element.value;
            }

            this.controls[name] = control;

            if (control.bind) {
                control.bind(this.element.elements[name]);
            }
        }
    },

    valid: function() {
        var valid = true;
        var elements = this.element.elements;
        var names = Object.keys(this.controls), name, control, errors, element;
        for (var i = 0; i < names.length; i++) {
            name = names[i];
            control = this.controls[name];
            element = elements[name];

            if (element.disabled) {
                continue;
            }

            errors = control.validator(control.getter(element));

            const errorContainer = element.parentElement.classList.contains('form-group')
                ? element.parentElement
                : element;

            // show errors
            if (errors) {
                const errorKeys = Object.keys(errors);
                const error = errors[errorKeys[0]];

                errorContainer.classList.add('everhour-error');
                errorContainer.setAttribute('data-error', error);
                valid = false;
            } else {
                errorContainer.classList.remove('everhour-error');
                errorContainer.removeAttribute('data-error');
            }
        }

        return valid;
    },

    value: function() {
        var value = {};
        var elements = this.element.elements;
        var names = Object.keys(this.controls), name;
        for (var i = 0; i < names.length; i++) {
            name = names[i];

            if (elements[name].disabled) {
                continue;
            }

            value[name] = this.controls[name].getter(elements[name]);
        }

        return value;
    },

    /**
     * @param {String} name
     * @returns {HTMLFormElement}
     */
    get: function(name) {
        return this.element.elements[name];
    },

    isEmpty: function() {
        var elements = this.element.elements;
        var names = Object.keys(this.controls), name;
        for (var i = 0; i < names.length; i++) {
            name = names[i];
            if (this.controls[name].getter(elements[name])) {
                return false;
            }
        }

        return true;
    },

    reset: function() {
        this.element.reset();
        var elements = this.element.elements;
        var names = Object.keys(this.controls);
        for (var i = 0; i < names.length; i++) {
            elements[names[i]].classList.remove('everhour-error');
        }
    }
};

Form.bindInput = function(element) {
    element.addEventListener('keydown', function(event) {
        let handled = false;
        const isTextarea = element.nodeName === 'TEXTAREA';

        if (event.keyCode === 13) {
            if (isTextarea && !event.shiftKey) {
                // allow go to a new line for textarea element
                // submit byt shift + enter
                return;
            }
            handled = true;
            if (app && app.adaptor && app.adaptor.settings.submitFormByEvent) {
                element.form.dispatchEvent(new CustomEvent('submit'));
            } else if (app && app.appType === 'browserAction') {
                const modalElement = element.closest('.modal[data-modal-type="everhour-modal"]');
                const submitButton = modalElement ? modalElement.querySelector('button[name="save"]') : null;

                if (!submitButton) {
                    return
                }

                submitButton.click();
            } else {
                // TODO rework this stub later, FF issue should be fixed soon (I hope)
                element.form.querySelector('[name="save"]').click();
            }
        } else if (event.keyCode === 27) {
            handled = true;
            element.form.dispatchEvent(new CustomEvent('cancel'));
        }

        if (handled) {
            event.preventDefault();
            event.stopPropagation();
            event.stopImmediatePropagation();
        }
    });
};

Form.saveState = function(form) {
    var elements = form.elements, element;
    var state = [];
    for (var i = 0; i < elements.length; i++) {
        element = elements[i];

        if (element.hasAttribute('stateless')) {
            continue;
        }

        state.push({
            element: element.name,
            value: element.value,
            hasError: element.classList.contains('everhour-error')
        });
    }

    return state;
};

Form.restoreState = function(form, state) {
    var elements = form.elements;
    state.forEach(function(controlState) {
        var element = elements[controlState.element];

        if (!element) {
            return;
        }

        element.value = controlState.value;
        if (element.classList) {
            element.classList.toggle('everhour-error', controlState.hasError);
        }
    });
};
window.TimeControl = function(allowEmpty, placeholder, disableFocus) {
    return {
        getter: function(element) {
            const time = TimeEntryUtils.parseTimeToSeconds(element.value);
            if (time === null) {
                return null;
            }

            if (time === '' || time === 0) {
                return 0;
            }

            return time;
        },

        validator: function(value) {
            if (value === null) {
                return {
                    time: 'Invalid format'
                }
            }

            if (!value && !allowEmpty) {
                return {
                    required: 'Required'
                }
            }

            if (value > ContentScript.Constants.DAILY_TIME) {
                return {
                    time: 'Time should less than 24h'
                }
            }

            return null;
        },

        bind: function(element) {
            const lengthPos = element.value + '';
            $(element).setCursorPosition(lengthPos.length);
            $(element).attr('placeholder', placeholder || '0h');
            $(element).attr('autocomplete', 'off');

            if (!disableFocus) {
                element.focus();
            }
        }
    }
};

window.DayTimeControl = function(placeholder) {
    return {
        dayTimeSubPart: ContentScript.Constants.DAY_TIME_SUBPART_TYPE.HOURS,
        currentTimeParts: null,
        dayTimeSubPartSettings: null,
        numberPosition: 0,
        element: null,

        getter: function(element) {
            const time = TimeEntryUtils.parseDayTime(element.value);

            if (time === null) {
                return null;
            }

            if (!time) {
                return '';
            }

            return TimeEntryUtils.getTimeSpendMinutes(time) * 60;
        },

        validator: function(value) {
            if (value === '') {
                return {
                    time: 'Required'
                }
            }

            if (value === null) {
                return {
                    time: 'Invalid format'
                }
            }

            if (value > ContentScript.Constants.DAILY_TIME) {
                return {
                    time: 'Time should less than 24h'
                }
            }

            return null;
        },

        bind: function(element) {
            this.init();
            this.element = element;
            const lengthPos = element.value + '';
            $(element).setCursorPosition(lengthPos.length);
            $(element).attr('placeholder', placeholder || ('--:--' + (this.is12TimeFormat ? ' --' : '' )));
            $(element).attr('autocomplete', 'off');

            $(element)
                .off('keypress keydown click')
                .on('keypress', (event) => this.onKeyPress(event))
                .on('keydown', (event) => this.onKeyDown(event))
                .on('click', (event) => this.onClick(event))
        },

        init: function() {
            this.is12TimeFormat = !store.profile.is24TimeFormat();
            this.currentTimeParts = new Map();
            this.currentTimeParts.set(ContentScript.Constants.DAY_TIME_SUBPART_TYPE.HOURS, '');
            this.currentTimeParts.set(ContentScript.Constants.DAY_TIME_SUBPART_TYPE.MINUTES, '');

            this.dayTimeSubPartSettings = new Map();
            this.dayTimeSubPartSettings.set(ContentScript.Constants.DAY_TIME_SUBPART_TYPE.HOURS, {
                format: 'HH',
                initial: '00',
                separator: ':',
                upperLimit: this.is12TimeFormat ? 12 : 23,
                lowerLimit: 0,
            });

            this.dayTimeSubPartSettings.set(ContentScript.Constants.DAY_TIME_SUBPART_TYPE.MINUTES, {
                format: 'mm',
                initial: '00',
                separator: this.is12TimeFormat ? ' ' : '',
                upperLimit: 59,
                lowerLimit: 0,
            });
            if (this.is12TimeFormat) {
                this.currentTimeParts.set(ContentScript.Constants.DAY_TIME_SUBPART_TYPE.MERIDIEM, '');
                this.dayTimeSubPartSettings.set(ContentScript.Constants.DAY_TIME_SUBPART_TYPE.MERIDIEM, {
                    format: 'A',
                    initial: 'am',
                    separator: '',
                });
            }
        },

        /**
         * @param {KeyboardEvent} event
         * @returns {*|boolean|boolean}
         */
        onKeyPress: function(event) {
            this.numberPosition++;
            const chr = String.fromCharCode(event.charCode === undefined ? event.keyCode : event.charCode).toLowerCase();
            const isLimit = this.dayTimeSubPartSettings.get(this.dayTimeSubPart).upperLimit !== undefined;
            const format = this.dayTimeSubPartSettings.get(this.dayTimeSubPart).format;

            if (chr >= '0' && chr <= '9') {
                const currentValue = this.currentTimeParts.get(this.dayTimeSubPart);
                let newValue = this.composeFieldValue(currentValue, format, chr);
                let shouldChangeSelection = false;

                if (isLimit && currentValue) {
                    const upperLimit = this.dayTimeSubPartSettings.get(this.dayTimeSubPart).upperLimit;
                    const tempValue = parseInt(newValue.toString(), 0);

                    if (tempValue > upperLimit) {
                        newValue = '0' + event.key;
                        shouldChangeSelection = true;
                    }
                }
                if (newValue !== currentValue) {
                    this.initCurrentTimeParts();
                    this.currentTimeParts.set(this.dayTimeSubPart, newValue);
                    if (this.numberPosition === format.length) {
                        this.changeSelection(1);
                        shouldChangeSelection = false;
                    }
                }
                if (shouldChangeSelection) {
                    this.changeSelection(1);
                }
            } else if (this.is12TimeFormat) {
                const meridiemPart = this.currentTimeParts.get(ContentScript.Constants.DAY_TIME_SUBPART_TYPE.MERIDIEM);
                const hoursPart = this.currentTimeParts.get(ContentScript.Constants.DAY_TIME_SUBPART_TYPE.HOURS);
                const hours = hoursPart.length > 0 ? parseInt(hoursPart, 10) : null;

                if (
                    hours !== null &&
                    (chr === 'a' || chr === 'p')
                ) {
                    let newMeridiemPart = meridiemPart.slice();

                    if (meridiemPart === 'am' && chr === 'p') {
                        newMeridiemPart = 'pm';
                    }

                    if (meridiemPart === 'pm' && chr === 'a') {
                        newMeridiemPart = 'am';
                    }

                    this.currentTimeParts.set(ContentScript.Constants.DAY_TIME_SUBPART_TYPE.MERIDIEM, newMeridiemPart);
                }
            }

            this.updateViewValue();

            return false;
        },

        /**
         * @param {KeyboardEvent} event
         * @returns {*|boolean|boolean}
         */
        onKeyDown: function(event) {
            let eventKey = event.key;

            switch (eventKey) {
                case 'Backspace': return this.clearCurrentFieldPart();
                case 'Tab': return this.changeSelection(event.shiftKey ? -1 : 1, true);
                case 'ArrowLeft': return this.changeSelection(-1);
                case 'ArrowUp': return this.changeFieldPart(1);
                case 'ArrowRight': return this.changeSelection(1);
                case 'ArrowDown': return this.changeFieldPart(-1);
                case 'Delete': return this.clearField();
                default: return true;
            }
        },

        onClick: function (event) {
            const cursorPosition = event.currentTarget.selectionStart;
            let position = 0, stop = false, selection = ContentScript.Constants.DAY_TIME_SUBPART_TYPE.HOURS;

            this.currentTimeParts.forEach((value, key) => {
                if (stop) {
                    return false;
                }
                position += value.length;

                if (cursorPosition <= position) {
                    selection = key;
                    stop = true;
                }
                position += this.dayTimeSubPartSettings.get(key).separator.length;
            });

            this.dayTimeSubPart = selection;

            return false;
        },

        composeFieldValue: function (currentValue, format, chr) {
            let tempValue = (currentValue + chr).slice(-2);
            let tempMoment = moment(tempValue, format);

            if (tempMoment.isValid()) {
                return tempMoment.format(format);
            }

            tempValue = '0' + chr;
            tempMoment = moment(tempValue, format);

            if (tempMoment.isValid()) {
                return tempMoment.format(format);
            }

            return currentValue;
        },

        changeSelection: function(subPartTypeShift, isExit) {
            isExit = typeof isExit === 'undefined' ? false : isExit;
            const newSelection = this.dayTimeSubPart + subPartTypeShift;

            if (this.currentTimeParts.has(newSelection)) {
                this.dayTimeSubPart = newSelection;
                this.numberPosition = 0;
                isExit = false;
            }

            this.setSelection();

            return isExit;
        },

        toggleMeridiemPart: function() {
            const currentValue = this.currentTimeParts.get(ContentScript.Constants.DAY_TIME_SUBPART_TYPE.MERIDIEM);

            this.currentTimeParts.set(ContentScript.Constants.DAY_TIME_SUBPART_TYPE.MERIDIEM, (currentValue === 'am') ? 'pm' : 'am');
        },

        changeTime: function(shiftValue) {
            const hasLimit = this.dayTimeSubPartSettings.get(this.dayTimeSubPart).upperLimit !== undefined;
            const hasValues = this.dayTimeSubPartSettings.get(this.dayTimeSubPart).values !== undefined;
            const format = this.dayTimeSubPartSettings.get(this.dayTimeSubPart).format;
            const currentValue = this.currentTimeParts.get(this.dayTimeSubPart);

            if (hasLimit) {
                const lowerLimit = this.dayTimeSubPartSettings.get(this.dayTimeSubPart).lowerLimit;
                const upperLimit = this.dayTimeSubPartSettings.get(this.dayTimeSubPart).upperLimit;
                let tempValue = (parseInt(currentValue.toString(), 0) + shiftValue);

                if (tempValue > upperLimit) {
                    tempValue = lowerLimit;
                }

                if (tempValue < lowerLimit) {
                    tempValue = upperLimit;
                }

                const tempMoment = moment(tempValue, format);

                this.currentTimeParts.set(this.dayTimeSubPart, tempMoment.format(format));
            }

            if (hasValues) {
                const values = this.dayTimeSubPartSettings.get(this.dayTimeSubPart).values;
                let tempKey = values.indexOf(currentValue.toString()) + shiftValue;

                if (tempKey < 0) {
                    tempKey = values.length - 1;
                } else if (values[tempKey] === undefined) {
                    tempKey = 0;
                }

                this.currentTimeParts.set(this.dayTimeSubPart, values[tempKey]);
            }

            this.updateViewValue();

            return false;
        },

        /**
         * @param adjustValue
         */
        changeFieldPart: function(adjustValue) {
            this.initCurrentTimeParts();

            if (this.dayTimeSubPart === ContentScript.Constants.DAY_TIME_SUBPART_TYPE.MERIDIEM) {
                this.toggleMeridiemPart();
            } else {
                this.changeTime(adjustValue);
            }
            this.updateViewValue();

        },

        initCurrentTimeParts: function() {
            this.currentTimeParts.forEach((value, key) => {
                if (value === '') {
                    this.currentTimeParts.set(key, this.dayTimeSubPartSettings.get(key).initial);
                }
            });
        },

        updateViewValue: function() {
            let formattedValue = '';let isAllEmpty = true;
            this.currentTimeParts.forEach((value, key) => {
                if (this.currentTimeParts.get(key) !== '') {
                    formattedValue += this.currentTimeParts.get(key) + this.dayTimeSubPartSettings.get(key).separator;
                }
            });

            this.element.value = formattedValue;

            this.setSelection();
        },
        
        setSelection: function () {
            let start = 0, end = 0, stop = false;

            this.currentTimeParts.forEach((value, key) => {
                if (stop) {
                    return;
                }

                if (key === this.dayTimeSubPart) {
                    end += value.length;
                    stop = true;

                    return;
                }
                const partLength = value.length + this.dayTimeSubPartSettings.get(key).separator.length;
                start += partLength;
                end += partLength;
            });
            this.element.dispatchEvent(new CustomEvent('change'));

            this.element.setSelectionRange(start, end);
        },

        clearField: function() {
            this.currentTimeParts.forEach((value, key) => {
                this.currentTimeParts.set(key, '');
            });
            this.changeSelection(-1);
            this.updateViewValue();
            this.numberPosition = 0;

            return false;
        },

        clearCurrentFieldPart: function() {
            if (this.currentTimeParts.get(this.dayTimeSubPart) === '') {
                this.changeSelection(-1);
            } else {
                this.currentTimeParts.set(this.dayTimeSubPart, '');
            }

            this.updateViewValue();

            return false;
        }
    }
};
window.EstimateControl = function(isRequired, skipFocus, placeholder) {
    return {
        getter: function(element) {
            const time = TimeEntryUtils.parseTimeToSeconds(element.value);

            if (time === null) { // invalid value
                return null;
            }

            if (time === '' || time === 0) { // reset prev value
                return 0;
            }

            return time;
        },

        validator: function(value) {
            if (value === null) {
                return {
                    estimate: 'Invalid format'
                }
            }

            if (isRequired) {
                if (value === 0) {
                    return {
                        estimate: 'isRequired'
                    }
                }
            }

            if (value > 3600 * 5000) {
                return {
                    estimate: 'Too long'
                }
            }

            return null;
        },

        bind: function(element) {
            if (!element || $(element).attr('disabled')) {
                return;
            }
            if (element && !skipFocus) {
                const lengthPos = element.value + '';
                $(element).setCursorPosition(lengthPos.length);
                element.focus();
            }

            $(element).attr('autocomplete', 'off');
            $(element).attr('placeholder', placeholder ? placeholder : '-');
        }
    }
};

window.DateControl = function(format, beginningOfWeek, minDate, maxDate, customClass, datepickerOptions) {
    return new DateControlClass(format, beginningOfWeek, minDate, maxDate, customClass, datepickerOptions);
};
window.DateControlClass = function (format, beginningOfWeek, minDate, maxDate, customClass, datepickerOptions) {
    this.init(format, beginningOfWeek, minDate, maxDate, customClass, datepickerOptions)
};
DateControlClass.prototype = {
    minDate: null,
    maxDate: null,
    format: null,
    beginningOfWeek: null,
    customClass: null,

    init: function (format, beginningOfWeek, minDate, maxDate, customClass, datepickerOptions) {
        this.maxDate = typeof(maxDate) !== 'undefined' ? maxDate : store.profile.getMaxDateAllowed();
        this.minDate = typeof(minDate) !== 'undefined' ? minDate : store.profile.getMinDateAllowed();
        this.datepickerOptions = datepickerOptions || {};
        this.customClass = customClass;
        this.format = format || "d M yy";
        this.beginningOfWeek = typeof(beginningOfWeek) !== 'undefined' ? beginningOfWeek : store.profile.get().team.beginningOfWeek;
    },

    getter: function(element) {
        var value = $(element).datepicker('getDate');
        if (!value) {
            return null;
        }
        var date = moment(value);

        return date.format('YYYY-MM-DD');
    },

    validator: function(value) {
        if (!value) {
            return {
                required: 'Required'
            }
        }

        if (this.maxDate && value > this.maxDate) {
            return {
                date: 'No future dates allowed'
            }
        }

        if (value < this.minDate) {
            return {
                date: 'Time locked for these dates'
            }
        }

        return null;
    },

    bind: function(element) {
        const minDate = moment(this.minDate, 'YYYY-MM-DD').toDate();
        const maxDate = this.maxDate ? moment(this.maxDate, 'YYYY-MM-DD').toDate() : null;
        const customClass = this.customClass;

        $(element).datepicker(Object.assign(
            {
                beforeShow: function(input, datepicker) {
                    if (customClass) {
                        datepicker.dpDiv.addClass(customClass);
                    }
                },
                hideIfNoPrevNext: true,
                firstDay: this.beginningOfWeek,
                maxDate: maxDate,
                minDate: minDate,
                dateFormat: this.format
            },
            this.datepickerOptions,
        ));
    }
};

window.CommentControl = function() {
    return {
        getter: function(element) {
            if (!element.value) {
                return "";
            }

            return element.value;
        },

        validator: function(value) {
            if (value && value.length > 1000 ) {
                return {
                    commentLine: 'max length limited by 1000'
                }
            }

            return null;
        },

        bind: function(element) {
            var lengthPos = element.value + '';
            $(element).setCursorPosition(lengthPos.length);
        }
    };
};
window.Component = function() {
    this.init.apply(this, arguments);
};

Component.prototype = {
    destructed: false,
    renderType: 'normal',

    anchor: '',
    
    placement: 'append',
    
    elementConstruct: '',

    selector: '',

    output: [],

    input: [],

    components: {},

    activeComponents: null,

    init: function(parent) {
        this.element = null;

        if (parent && parent.hasOwnProperty('element')) {
            this.parent = parent;
            this.parentElement = parent.element || document;
        }

        if (parent && parent instanceof Node) {
            this.parent = null;
            this.parentElement = parent;
        }

        if (!this.parentElement) {
            this.parentElement = document;
        }

        this.activeComponents = {};

        this.onLoad();
    },

    onLoad: function() {

    },

    template: function() {},

    getElement: function() {
        const element = this.parentElement.querySelector(this.selector);

        if (element) {
            return element;
        }
        
        if (!this.elementConstruct) {
            return ;
        }

        const createElement = () => htmlToElement(this.elementConstruct);

        if (this.anchor) {
            return this.placeElement(createElement, this.anchor, this.placement);
        }

        if (this.anchors) {
            for (let i = 0; i < this.anchors.length; i++) {
                let placedElement = this.placeElement(createElement, this.anchors[i].anchor, this.anchors[i].placement);

                if (placedElement) {
                    return placedElement;
                }
            }
            return;
        }

        if (this.placement === Component.APPEND) {
            return this.parentElement.appendChild(createElement());
        }

        console.error('invalid placement for parent: ' + this.placement + '. Only append supported.');
        return ;
    },

    placeElement(createElement, anchor, placement) {
        let anchorElement = this.parentElement.querySelector(anchor);

        if (!anchorElement) {
            return ;
        }

        if (placement === Component.APPEND) {
            return anchorElement.appendChild(createElement());
        }

        if (placement === Component.APPEND_FIRST) {
            return anchorElement.insertBefore(createElement(), anchorElement.firstChild);
        }

        if (placement === Component.BEFORE) {
            return anchorElement.parentNode.insertBefore(createElement(), anchorElement);
        }

        if (placement === Component.AFTER) {
            return anchorElement.parentNode.insertBefore(createElement(), anchorElement.nextSibling);
        }

        console.error('invalid placement for anchor (' + anchor + '): ' + placement + '. Supported: append, after, before.');
        return ;
    },

    onInit: function() {

    },

    beforeGetElement: function() {

    },

    beforeRender: function() {

    },

    afterRender: function() {

    },

    afterRestoreFormState: function() {

    },

    render: function() {
        if (this.renderType === Component.SIMPLIFIED_RENDER) {
            this.renderSimplified();
            return;
        }

        if (this.destructed) {
            return ;
        }

        this.beforeGetElement();

        if (this.destructed) {
            return ;
        }

        if (!this.template) {
            throw new Error('Component template is not defined');
        }

        this.element = this.getElement();

        if (this.destructed) {
            return ;
        }

        if (!this.element) {
            Logger.log('component', 'render failed -> NO ELEMENT,', ' anchor: ', this.anchor, ',  selector: ', this.selector);
            return ;
        }

        var form = this.element.querySelector('form'), formState;
        if (form && !form.hasAttribute('disable-restore')) {
            formState = Form.saveState(form);
        }

        this.beforeRender();

        if (this.destructed) {
            return ;
        }

        var html = this.template(this);
        if (html) {
            if (this.element.innerHTML === html) {
                return;
            }

            this.element.innerHTML = html;

            // update element reference
            this.element = this.getElement();
        }

        // bind event listeners
        this.bindOutput();

        // bind input attributes
        this.bindComponents();

        this.afterRender();

        if (this.destructed) {
            return ;
        }

        if (formState) {
            form = this.element.querySelector('form');
            if (form) {
                Form.restoreState(form, formState);
            }
        }

        this.afterRestoreFormState();
    },

    renderSimplified: function() {
        if (this.destructed) {
            return ;
        }

        this.beforeGetElement();

        if (this.destructed) {
            return ;
        }

        this.element = this.getElement();

        if (this.destructed) {
            return ;
        }

        this.beforeRender();

        if (this.destructed) {
            return ;
        }

        if (this.element && this.template) {
            var html = this.template(this);
            if (html) {
                this.element.innerHTML = html;

                // update element reference
                this.element = this.getElement();
            }
        }

        if (this.destructed) {
            return ;
        }

        this.afterRender();
    },

    bindComponents: function() {

        var component, element, attr, event, i, j;

        for (i in this.components) {
            if (!this.components.hasOwnProperty(i)) {
                return;
            }

            component = new this.components[i](this);

            element = component.getElement();

            if (!element) {
                continue;
            }

            for (j = 0; j < component.input.length; j++) {
                attr = component.input[j];
                if (element.hasAttribute('[' + attr + ']')) {
                    component[attr] = (new Function('with (this) { return ' + element.getAttribute('[' + attr + ']') + '; }')).apply(this);
                }
            }

            for (j = 0; j < component.output.length; j++) {
                event = component.output[j];
                if (element.hasAttribute('(' + event + ')')) {
                    bindEvent(this, element, event);
                }
            }

            component.onInit();
            component.render();

            this.activeComponents[i] = component;
        }
    },

    getComponent: function(id) {
        return this.activeComponents.hasOwnProperty(id) ? this.activeComponents[id] : null;
    },

    bindOutput: function() {
        const events = ['click', 'submit', 'cancel', 'change', 'outclick', 'blur', 'focus'];
        let eventName;

        for (var i = 0; i < events.length; i++) {
            eventName = events[i];

            var elements = this.element.querySelectorAll('[\\(' + eventName + '\\)]');

            for (var j = 0; j < elements.length; j++) {
                bindEvent(this, elements[j], eventName);
            }

            if (this.element.hasAttribute('(' + eventName + ')')) {
                bindEvent(this, this.element, eventName);
            }
        }
    },

    emit: function(event, data) {
        this.element.dispatchEvent(new CustomEvent(event, {
            detail: data
        }));
    },

    destruct: function() {
        if (this.element) {
            this.element.remove();
        }

        this.destructed = true;
    }
};

Component.APPEND = 'append';
Component.BEFORE = 'before';
Component.AFTER = 'after';
Component.APPEND_FIRST = 'append_first';

Component.SIMPLIFIED_RENDER = 'simplified';


Component.outclick = {
    initialized: false,
    onlyTrusted: false,
    elements: [],
    initialize: function() {
        if (Component.outclick.initialized) {
            return;
        }

        Component.outclick.initialized = true;
        document.addEventListener('click', function(e) {
            if (Component.outclick.onlyTrusted && !e.isTrusted) {
                return;
            }

            var element;
            for (var i = 0; i < Component.outclick.elements.length; i++) {
                element = Component.outclick.elements[i];

                if (!element) {
                    continue;
                }

                if (!document.contains(element)) {
                    delete Component.outclick.elements[i];
                    continue;
                }

                if (!element.contains(e.target)) {
                    element.dispatchEvent(new CustomEvent('outclick', {
                        target: e.target
                    }));
                }
            }
        });
    },

    bind: function(element) {
        if (Component.outclick.elements.indexOf(element) !== -1) {
            return;
        }

        Component.outclick.elements.push(element);

        if (!Component.outclick.initialized) {
            Component.outclick.initialize();
        }
    }
};

Component.extend = extend;

function bindEvent(context, element, event) {
    var callback = new Function('data', 'event', 'with (this) { return ' + element.getAttribute('(' + event + ')') + '; }');
    element.addEventListener(event, function(e) {
        e.preventDefault();
        e.stopPropagation();
        e.stopImmediatePropagation();
        callback.apply(context, [e.detail, e]);

        return false;
    });

    if (event === 'outclick') {
        Component.outclick.bind(element);
    }
}

ContentScript.v2.Components.AddTime = Component.extend({
    form: null,
    lockActions: false,
    dateFormat: 'YYYY-MM-DD',
    isUserSelectionAllowed: false,
    withUserManagement: false,
    selectedUser: null,
    task: null,
    users: null,
    isPeriodMode: false,
    timeFieldLabel: '.ev-time-label',
    useShortMyself: true,

    onLoad: function() {
        this.isUserSelectionAllowed = this.withUserManagement ? store.profile.isTimeManagementAllowed() : false;
        if (this.isUserSelectionAllowed && this.withUserManagement) {
            this.currentUser = store.profile.get();
            this.allUsers = _.chain(store.users.all()) // like a frontend logic
                .values()
                // prevent to report time for removed users
                .filter(user => user.status !== 'removed')
                .indexBy('id')
                .value();
            this.selectedUser = this.currentUser;
        }
    },

    beforeRender: function() {
        if (!this.task) {
            !this.disabledTemplate ?
                this.destruct() :
                this.template = this.disabledTemplate;

            return;
        }

        if (!this.users && this.isUserSelectionAllowed && this.withUserManagement) {
            let users = ContentScript.v2.Components.ManageEstimates.prototype.getTaskUsers.apply(this);
            this.users = this.getSortedUserList(Object.keys(users).length > 0 ? users : this.allUsers);
        }
        this.isPeriodMode = this.getComponent('addTimeInput') ?
            this.getComponent('addTimeInput').isPeriodMode : false;
    },

    getSortedUserList: function (users) {
        return _.chain(users)
            .values()
            .compact()
            .map(user => {
                let name = user.name;
                if (this.useShortMyself && user.id == store.profile.get().id) {
                    name = 'Myself';
                } else if (!name) {
                    name = user.email;
                }

                return Object.assign({}, user, {name: name});
            })
            .sort((a, b) =>  {
                if (this.useShortMyself && a.id == store.profile.get().id) {
                    return -1;
                } else {
                    return (a.name || '').localeCompare(b.name || '');
                }
            })
            .sort((a, b) => {
                if (!a.taskTotalTime && !b.taskTotalTime) {
                    return 0;
                }

                return a.taskTotalTime > b.taskTotalTime ? -1 : 1;
            })
            .value();
    },

    afterRender: function() {
        this.buildForm();
        this.bindForm();
    },

    buildForm: function() {
        let controls = {
            date: DateInputControl(this.dateFormat)
        };

        if (this.isPeriodMode) {
            controls['time_from'] = DayTimeControl();
            controls['time_to'] = DayTimeControl();
        } else {
            controls['time'] = TimeControl();
        }

        this.form = new Form(this.element.querySelector('form'), controls);
    },

    bindForm: function() {
        if (this.isPeriodMode) {
            Form.bindInput(this.form.element.elements.time_from);
            Form.bindInput(this.form.element.elements.time_to);
        } else {
            Form.bindInput(this.form.element.elements.time);
        }
        this.form.element.elements[this.isPeriodMode ? 'time_from' : 'time'].focus();
    },

    save: function() {
        if (this.lockActions) {
            return false;
        }

        if (this.form.valid() && this.task) {
            let formValue = this.form.value(), totalTime;
            if (this.withUserManagement && this.isUserSelectionAllowed) {
                if (!this.selectedUser) {
                    throw Error('form should provide user selection elements')
                }
                formValue.user = this.selectedUser.id;
            }

            if (!this.isPeriodMode) {
                totalTime = formValue.time;
            } else {
                if (
                    !formValue.time_from && !formValue.time_to ||
                    formValue.time_from === formValue.time_to
                ) {
                    totalTime = ContentScript.Constants.DAILY_TIME;
                } else if (formValue.time_from > formValue.time_to) {
                    // case for input like 19.00 to 3.00
                    totalTime = ContentScript.Constants.DAILY_TIME - formValue.time_from + formValue.time_to;
                } else {
                    totalTime = formValue.time_to - formValue.time_from;
                }
            }

            this.lockActions = true;
            this.beforeSave();

            let dataArgs = [
                this.task.id,
                formValue.date,
                totalTime,
            ];

            if (this.task.hasOwnProperty('details') && typeof this.task.details === 'function') {
                dataArgs.push(this.task.details());
            }
            dataArgs = dataArgs.concat([
                formValue.commentLine,
                formValue.user || null
            ]);
            // taskTime - case for extension popup
            return (store.tasks || store.taskTime).addTime.call(null, ...dataArgs)
                .done(() => {
                    if (this.isUserSelectionAllowed && this.withUserManagement) {
                        // reset selectedUser
                        this.selectedUser = this.currentUser;
                    }
                    this.afterSave();
                })
                .fail((response) => this.onError(response));
        }

        return $.Deferred().reject();
    },

    beforeSave: function() {

    },

    afterSave: function() {

    },

    onError: function(error) {

    },

    cancel: function() {

    },

    selectUser: function(userId) {
        if (this.isUserSelectionAllowed && this.withUserManagement) {
            this.selectedUser = this.allUsers[userId];
            const isCurrentUser = this.selectedUser.id === this.currentUser.id;

            if (this.element.querySelector('textarea[name="commentLine"]')) {
                let placeholderText = ContentScript.Constants.COMMENT_PLACEHOLDER;
                this.element.querySelector('textarea[name="commentLine"]').placeholder = placeholderText.replace('your', !isCurrentUser ? 'their' : 'your');
            }
        }
    },

    toggleInputMode: function () {
        if (!this.getComponent('addTimeInput')) {
            return;
        }

        this.getComponent('addTimeInput').toggle();
        this.isPeriodMode = !this.isPeriodMode;
        this.element.querySelector('.toggle-with-icon').classList.toggle('toggle-enabled');
        this.toggleTimeLabelFor();
        this.buildForm();
        this.bindForm();
        this.onModeToggled();
    },

    toggleTimeLabelFor: function () {
        this.element.querySelector(this.timeFieldLabel).htmlFor = this.isPeriodMode ? 'ev_time_from' : 'ev_time';
    },

    onTimeChange: function () {
    },

    onModeToggled: function() {
    }
});

ContentScript.v2.Components.AddTime.extend = extend;

ContentScript.v2.Components.AddTimeInput = Component.extend({
    selector: '.everhour-time-input-component',
    elementConstruct: '<div class="everhour-time-input-component"></div>',
    input: ['isPeriodMode'],
    output: ['changed'],
    isPeriodMode: false,

    template: _.template(`
        <% if (!isPeriodMode) { %>
            <span class="icon-minus " (click)="changeTime('minus')"><i class="icon-minus-image"></i></span>
            <input type="text" id="ev_time"
                class="generic-input time-input"
                name="time" placeholder="0h"
            />
            <span class="icon-plus" (click)="changeTime('plus')"><i class="icon-plus-image"></i></span>
        <% } else { %>
            <div class="form-control-group">
                <input id="ev_time_from"
                    class="generic-input time-input <%- is12TimeFormat ? 'meridiem' : '' %>" 
                    type="text" name="time_from" 
                    autocomplete="off"
                    tabindex="1"
                />
            </div>
            <span class="ev-period-divider"> - </span>
            <div class="form-control-group">
                <input id="ev_time_to" 
                    class="generic-input time-input <%- is12TimeFormat ? 'meridiem' : '' %>" 
                    type="text" name="time_to" 
                    autocomplete="off"
                    tabindex="2"
                />
            </div>
        <% } %>
    `),

    beforeRender: function() {
        this.is12TimeFormat = store.profile && !store.profile.is24TimeFormat();
    },

    afterRender: function() {
        const escEvent = new Event('escEvent');
        this.element.classList.toggle( 'period-mode', this.isPeriodMode);
        this.element.classList.toggle( 'time-mode', !this.isPeriodMode);
        if (!this.isPeriodMode) {
            $('.time-input', this.parentElement)
                .off('keydown input click')
                .on('keydown', (event) => {
                    let element = event.currentTarget || event.target;
                    switch (event.key) {
                        case 'ArrowUp':
                            return this.changeTime('plus', element); // Increment date field on up
                        case 'ArrowDown':
                            return this.changeTime('minus', element); // Decrement date field on down
                        case 'Escape':
                            const modalOverlay = event.target.closest('#modal-overlay');

                            if (modalOverlay) {
                                return modalOverlay.dispatchEvent(escEvent);
                            }

                            return null;
                    }
                })
                .on('input', _.debounce(() => this.validate(), 300, false))
        } else {
            $('.time-input', this.parentElement)
                .off('change')
                .on('change', _.debounce(() => this.onPeriodChanged(), 400 ,false));
        }
    },

    changeTime: function(operationType, element) {
        const maxTime = 24 * 3600;
        const minStep = 15 * 60;
        element = element || this.element.querySelector('[name="time"]');

        const control = TimeControl();
        const time = control.getter(element);
        let value = !operationType ? time : 0;

        if (operationType === 'plus') {
            value = time + minStep <= maxTime ? time + minStep : maxTime;
        }

        if (operationType === 'minus') {
            value = time - minStep > 0 ? time - minStep : time;
        }

        if (control.validator(value) !== null) {
            return;
        }
        element.classList.remove('everhour-error');

        element.value = TimeEntryUtils.format(value);
    },

    validate: function() {
        const element = this.element.querySelector('[name="time"]');
        const control = TimeControl();
        const time = control.getter(element);
        element.classList.toggle('everhour-error', control.validator(time) !== null);
    },

    toggle: function () {
        this.isPeriodMode = !this.isPeriodMode;
        this.emit('toggleTimeView');
        this.render();
    },

    onPeriodChanged: function() {
        this.emit('changed');
    }
});

ContentScript.v2.Components.EditTime = Component.extend({
    state: 'hidden', // hidden, loading, form
    lockActions: false,
    form: null,
    task: null,
    isUserSelectionAllowed: false,
    withUserManagement: true,
    week: 0,
    weeks: [],
    inputPlaceholder: null,
    sortUsersByTotalTime: false,
    useShortMyself: true,
    hasTime: true,

    days: ['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday'],

    onLoad: function() {
        this._days = this.days;
        this.isUserSelectionAllowed = store.profile.isTimeManagementAllowed() && this.withUserManagement;
        if (this.isUserSelectionAllowed) {
            this.currentUser = store.profile.get();
            this.allUsers = _.chain(store.users.all()) // like a frontend logic
                .values()
                // allow to edit time for all users, active|invited|removed
                .indexBy('id')
                .value();
            this.selectedUser = this.currentUser;
        }
    },

    show: function() {

    },

    hide: function() {

    },

    reset: function() {
        this.week = 0;
        this.lockActions = false;
        this.weeks = [];
        this.state = 'hidden';
    },

    start: function() {
        this.reset();
        this.state = 'loading';
        this.load();
        this.render();
    },

    /**
     *
     * @param {string?} userId
     */
    load: function(userId) {
        // force Monday to be first day of the week
        moment.updateLocale('en', {week: { dow : store.profile.team().beginningOfWeek}});
        const dateFormat = 'YYYY-MM-DD';
        // we always need current and prev weeks
        const mandatoryDates = [moment().weekday(0).format(dateFormat), moment().weekday(0).subtract(1, 'weeks').format(dateFormat)];
        const predefinedWeeks = [];

        const minDate = store.profile.getMinDateAllowed(); // resolve max/min date using current user from store.profile
        const maxDate = store.profile.getMaxDateAllowed();

        const currentUser = store.profile.get();
        userId = (typeof(userId) !== 'undefined' ? parseInt(userId) : currentUser.id); // userId for filtering task time history

        mandatoryDates.forEach(function(date) {
            var days = [], dayDate, i;
            for (i = 0; i < 7; i++) {
                dayDate = moment(date, dateFormat).add(i, 'day').format(dateFormat);
                days.push({
                    date: dayDate,
                    time: 0,
                    locked: this.isLockedByDate(dayDate, minDate, maxDate)
                });
            }
            predefinedWeeks[date] = {
                date: moment(date, dateFormat).weekday(0).format(dateFormat),
                time: 0,
                days: days
            }
        }, this);

        const isSortByTimeRequired = this.isUserSelectionAllowed && this.sortUsersByTotalTime;

        store.tasks.timeLog(this.task.id).then((history) => {
            if (isSortByTimeRequired) {
                // for Asana we sorting users in select by total Time spend and then alphabetically
                Object.keys(this.allUsers || {}).forEach(allUserId => {
                    this.allUsers[allUserId].taskTotalTime = 0;
                });
            }

            const activity = history
                .filter(taskTime => {
                    if (isSortByTimeRequired) {
                        let teamUser = this.allUsers[taskTime.user];
                        if (teamUser) {
                            teamUser.taskTotalTime += taskTime.time;
                        }
                    }

                    return taskTime.user === userId;
                })
                .reduce((activity, taskTime) => {
                    var week = moment(taskTime.date).weekday(0).format(dateFormat);

                    if (!activity.hasOwnProperty(week)) {
                        activity[week] = {
                            time: 0,
                            days: {}
                        }
                    }

                    activity[week].time += taskTime.time;
                    activity[week].days[taskTime.date] = {
                        time: taskTime.time,
                        isLocked: taskTime.isLocked,
                        lockReasons: taskTime.lockReasons
                    };

                    return activity;
                }, {});

            this.composeUsers();

            // if there are no time reported show this week
            if (Object.keys(activity).length === 0) {
                activity[moment().weekday(0).format(dateFormat)] = {time: 0,days: {}};
            }

            Object.keys(activity).forEach(date => {
                const days = [];
                let dayDate, i;
                for (i = 0; i < 7; i++) {
                    dayDate = moment(date, dateFormat).add(i, 'day').format(dateFormat);

                    days.push({
                        date: dayDate,
                        time: (activity[date].days[dayDate] || {}).time || 0,
                        locked: this.isLockedByDate(dayDate, minDate, maxDate) ||
                            !this.isTimeEditableByRole(activity[date].days[dayDate]),
                        isToday: moment().isSame(moment(dayDate, dateFormat), 'day')
                    });
                }

                predefinedWeeks[date] = {
                    date: date,
                    time: activity[date].time,
                    days: days
                }
            });

            this.weeks = _.values(predefinedWeeks).sort((w1, w2) => w2.date.localeCompare(w1.date));

            this.state = 'form';
            this.render();

            // restore profile settings
            moment.updateLocale('en', {
                week: {
                    dow : store.profile.team().beginningOfWeek
                }
            });
        }).fail(() => {
            // restore profile settings
            moment.updateLocale('en', {
                week: {
                    dow : store.profile.team().beginningOfWeek
                }
            });
        });
    },

    selectWeek: function(week) {
        if (this.week !== week) {
            this.week = week;
            this.render();
        }
    },

    isLockedByDate: function (dayDate, minDate, maxDate) {
        // resolve max/min date using current user from store.profile
        minDate = minDate ? minDate : store.profile.getMinDateAllowed();
        maxDate = maxDate ? maxDate : store.profile.getMaxDateAllowed();

        return dayDate < minDate || (maxDate && dayDate > maxDate);
    },

    isTimeEditableByRole: function (timeEntry) {
        if (store.profile.isAdmin()) {
            // allow everything
            return true;
        }

        if (!store.profile.isTimeManagementAllowed() && (timeEntry || {}).isLocked) {
            // allow if isLocked flag false
            return false;
        }

        const supervisorLockReasons = ['time_approved', 'time_invoiced', 'timesheet_approved'];
        // allow if no specific lockReasons for supervisor
        return ((timeEntry || {}).lockReasons || [])
            .reduce((lockCount, lockReason) => {
                return (lockCount += supervisorLockReasons.includes(lockReason) ? 1 : 0);
            }, 0) === 0;
    },

    getWeekDays: function() {
        // const days = this.weeks[this.week].days;
        // const isFullWeekForIntegration = !!~['as', 'cl', 'mo', 'no', 'td', 'tr'].indexOf(ContentScript.v2.ApplicationAdaptor.prototype.platform);
        //
        // if (store.profile.team().beginningOfWeek === 0 && isFullWeekForIntegration
        //     || store.profile.team().beginningOfWeek === 0 && this.hasWeekendTime()) {
        //     const days = [].concat(this._days);
        //     days.unshift(days.pop());
        //
        //     this.days = days;
        // } else {
        //     this.days = [].concat(this._days);
        // }
        //
        // return this.hasWeekendTime() || isFullWeekForIntegration ? days : days.slice(0, 5);

        const days = this.weeks[this.week].days;

        if (store.profile.team().beginningOfWeek === 0) {
            const days = [].concat(this._days);
            days.unshift(days.pop());

            this.days = days;
        }

        // return this.hasWeekendTime() ? days : days.slice(0, 5);

        return days;
    },

    hasWeekendTime: function() {
        const days = this.weeks[this.week].days;

        return days[5].time > 0 || days[6].time > 0;
    },

    getChanges: function() {
        var value = this.form.value();
        var changes = [];

        this.getWeekDays().forEach((day, index) => {
            if (!value.hasOwnProperty('day_' + index)) {
                return;
            }

            if (_format.time(day.time) !== _format.time(value['day_' + index])) {
                changes.push({
                    date: day.date,
                    time: value['day_' + index],
                    taskId: this.task.id,
                    user: {id: value.user || store.profile.get().id}
                })
            }
        });

        return changes;
    },

    beforeRender: function() {
        if (!this.task) {
            this.destruct();
        }
    },

    afterRender: function() {
        if (this.state === 'form') {
            var formElement = this.element.querySelector('form');
            var controls = {}, focusElementIndex, days = this.getWeekDays();
            days.forEach((day, index) => {
                // find and keep first non locked time input index
                // for add it as last for set focus on it
                if (typeof(focusElementIndex) == 'undefined' && !day.locked) {
                    focusElementIndex = index;
                }
                if (index !== focusElementIndex) {
                    controls['day_' + index] = TimeControl(true, this.inputPlaceholder);
                    Form.bindInput(formElement.elements['day_' + index]);
                }
                if (index == (days.length -1) && typeof(focusElementIndex) !== 'undefined') {
                    controls['day_' + focusElementIndex] = TimeControl(true, this.inputPlaceholder);
                    Form.bindInput(formElement.elements['day_' + focusElementIndex]);
                }
            });

            this.form = new Form(formElement, controls);
            if (
                this.isUserSelectionAllowed && this.users && Object.keys(this.users).length > 1 &&
                this.element.querySelector('[name="user"]')
                // in Asana we using pseudo select
            ) {
                this.form.setControls({
                    user: UserControl()
                });
            }
        }
    },

    save: function() {
        if (this.lockActions) {
            return false;
        }

        if (this.form && this.form.valid()) {
            var changes = this.getChanges();

            if (changes.length === 0) {
                this.beforeSave();
                this.afterSave();
                return;
            }

            this.lockActions = true;
            this.beforeSave();

            store.tasks.bulkUpdateTime(changes, this.task.id, this.task.details())
                .done(() => this.afterSave())
                .fail(() => this.onError());
        }

        return false;
    },

    beforeSave: function() {

    },

    afterSave: function() {
        this.lockActions = false;
        this.hide();
        return false;
    },

    onError: function() {
        this.lockActions = false;
        this.hide();
        return false;
    },

    composeUsers: function() {
        if (this.isUserSelectionAllowed) {
            if (!this.users) {
                // all users that have access to project
                const users = ContentScript.v2.Components.ManageEstimates.prototype.getTaskUsers.apply(this);
                // all users that have progress but may lost access to project
                const progress = this.getTaskProgress();
                progress.forEach(progressPerMember => {
                    if (!users.hasOwnProperty(progressPerMember.user)) {
                        // progress may contains subtasks progress or parent estimate only(we editing time)
                        if (progressPerMember.time) {
                            // we need only members with time for current task
                            users[progressPerMember.user] = Object.assign(
                                {},
                                {id: progressPerMember.user},
                                progressPerMember
                            );
                        }
                    }
                });
                if (this.selectedUser && !users[this.selectedUser.id]) {
                    users[this.selectedUser.id] = this.selectedUser;
                }


                if (this.sortUsersByTotalTime) {
                    Object.keys(users).forEach(taskUserId => {
                        users[taskUserId] = Object.assign(users[taskUserId] || {}, this.allUsers[taskUserId] || {});
                    });
                }
                this.users = ContentScript.v2.Components.AddTime.prototype.getSortedUserList.call(this, Object.keys(users).length > 0 ? users : this.allUsers);
            }

            if (this.sortUsersByTotalTime) {
                this.hasTime = !!(store.tasks.total(this.task.id) || {}).time;
            }
        }
    },

    getTaskProgress: function() {
        return store.select.sortedProgress(this.task.id)
    },

    selectUser: function(userId) {
        if (this.isUserSelectionAllowed) {
            this.selectedUser = this.allUsers[userId];
            this.reset();
            this.state = 'loading';
            this.load(userId);
            this.render();
        }
    }
});

ContentScript.v2.Components.EditTime.extend = extend;

window.BrowserAction = {};

BrowserAction.Application = function() {
    this.controllers = {};
    this.appType = 'browserAction';
    this.init();
};

BrowserAction.Application.prototype = {
    init: function() {
        let self = this;

        browserPort.settings.load().then(function(options) {
            self.everhour = new Everhour(options);
            self.boot();
        });

        browserPort.eventDispatcher.on('!self', 'popup:action:show-everhour-error', function(response) {
            browserPort.eventDispatcher.broadcast('browser-button', 'render-errors', response)
        });
    },

    boot: function() {
        let accessToken = browserPort.settings.get('accessToken');

        if (!accessToken) {
            ReactDOM.render(
                React.createElement(LoginController),
                document.getElementById('popup-controller-v2')
            );
            return;
        }

        window.everhour = new Everhour2(accessToken);
        this.everhour = window.everhour;

        ReactDOM.render(
            React.createElement(PopupController),
            document.getElementById('popup-controller-v2')
        );
    }
};
/**
 * Declare permission
 *
 * {
 *  id - use for display website icon for permission list
 *  name - use for display website name for permission list
 *  className - use for replace base styles for specific website
 *  anchors - use for add embed placement
 *  selector, removeSelector - use for DomObserver mutations trigger
 *  urlFormat - use to replace default gluing. PATH - url.origin + url.pathname(without query params) or default - window.location.href(full url)
 *  renderDebounceTimeout - use for replace default debounce(5ms)
 * */

window.Websites = [
    {
        id: 'airtable',
        name: 'Airtable - airtable.com',
        domains: ['https://airtable.com/'],
        anchors: [
            {
                anchor: '.DetailViewWithActivityFeed > div:nth-child(1) > div:nth-child(1) > div:nth-child(1) > div:nth-child(1) > div:nth-child(1)', // evil, but doesn't exist any normally selector
                placement: Component.AFTER
            }
        ],
        urlFormat: 'PATH',
        title: () => {
            return document.querySelector('.DetailViewWithActivityFeed .labelCellPair textarea').value;
        }
    },
    {
        id: 'breeze',
        name: 'Breeze - breeze.pm',
        domains: ['https://app.breeze.pm/'],
        anchors: [
            {
                anchor: '#card_modal .card-side',
                placement: Component.APPEND_FIRST
            }
        ],
        urlFormat: 'PATH',
        title: () => {
            return document.querySelector('.card-content .card-name').innerText.trim();
        }
    },
    {
        id: 'canva',
        name: 'Canva - canva.com',
        domains: ['https://*.canva.com/'],
        anchors: [{
            anchor: 'main div[role="toolbar"]',
            placement: Component.BEFORE
        }],
        urlFormat: 'PATH',
        title: () => {
            return document.querySelector('nav[role="menubar"] div[aria-describedby] input[type="text"]').value;
        }
    },
    {
        id: 'coda',
        name: 'Coda - coda.io',
        domains: ['https://coda.io/'],
        anchors: [{
            anchor: 'header[data-kr-interactive][tabindex] > div:nth-child(2)',
            placement: Component.APPEND_FIRST
        }],
        title: () => {
            return document.querySelector('div[data-coda-ui-id="documentTitle"]').innerText.trim();
        },
        url: () => {
            return /https:\/\/coda.io\/.*\/.*\//.exec(window.location.href)[0].replace(/\/+$/, '');
        }
    },
    {
        id: 'figma',
        name: 'Figma - figma.com',
        domains: ['https://*.figma.com/'],
        anchors: [{
            anchor: 'div[class*="toolbar_view--toolbar"] div[class*="toolbar_view--rightButtonGroup"]',
            placement: Component.APPEND_FIRST
        }],
        urlFormat: 'PATH',
        title: () => {
            return document.querySelector('div[class*="toolbar_view--toolbar"] span[class*="filename_view--titleWithToggleContainer"]').innerText.trim();
        }
    },
    {
        id: 'freshdesk',
        name: 'Freshdesk - freshdesk.com',
        domains: ['https://*.freshdesk.com/'],
        anchors: [{
            anchor: '#mainactionbar .page-actions__left button[data-test-id="top-navigation-delete"] + .app-dropdown-group.ember-basic-dropdown.ember-view',
            placement: Component.AFTER
        }],
        selector: 'body.ember-application .ember-view',
        urlFormat: 'PATH',
        renderDebounceTimeout: 50
    },
    {
        id: 'google-docs',
        name: 'Google Docs - docs.google.com',
        domains: ['https://docs.google.com/'],
        anchors: [
            {
                anchor: '#docs-titlebar-container .docs-titlebar-buttons',
                placement: Component.APPEND_FIRST
            }
        ],
        urlFormat: 'PATH',
        renderDebounceTimeout: 50,
        title: () => {
            return document.querySelector('#docs-titlebar-container .docs-title-input-label .docs-title-input-label-inner').innerText.trim();
        }
    },
    {
        id: 'google-mail',
        name: 'Google Mail - mail.google.com',
        domains: ['https://mail.google.com/'],
        anchors: [
            {
                anchor: 'div[role="main"] table[role="presentation"] > tr:nth-child(1) > td:nth-child(1) > div:nth-child(2) > div:nth-child(1) > div:nth-child(1) > div:nth-child(1)',
                placement: Component.APPEND_FIRST
            },
            {
                anchor: 'div[id=":4"] > div[jsaction] > div[gh]',
                placement: Component.APPEND
            }
        ],
        urlFormat: 'HASH',
        renderDebounceTimeout: 50
    },
    {
        id: 'helpscout',
        name: 'Help Scout - helpscout.net',
        domains: ['https://secure.helpscout.net/'],
        anchors: [
            {
                anchor: '.convo-actions.ConvoActionList',
                placement: Component.APPEND,
            },
            {
                anchor: 'div[class*="StatusAssigneecss__ContainerUI"] button[data-testid="StatusDropListToggler"]',
                placement: Component.BEFORE,
            }
        ],
        urlFormat: 'PATH'
    },
    {
        id: 'hubspot',
        name: 'Hubspot - hubspot.com',
        domains: ['https://*.hubspot.com/'],
        anchors: [
            {
                anchor: 'div[data-measured-element="timeline-container"] div[class*="TimelineSearchAndActionsBar"] div[class*="TimelineSearchInput"] + div',
                placement: Component.APPEND_FIRST,
            }
        ],
        urlFormat: 'PATH',
        title: () => {
            return `[${document.querySelector('.private-breadcrumbs a.private-link > span').innerText.trim()}] ${document.title}`;
        },
        removeSelector: 'div[class*="AppSkeleton"]'
    },
    {
        id: 'ladesk',
        name: 'LiveAgent - liveagent.com',
        domains: ['https://*.ladesk.com/'],
        anchors: [
            {
                anchor: '.ConversationHeaderView .ConversationHeaderSubjectPanel',
                placement: Component.APPEND,
            }
        ],
        selector: '.OfflineSmallIcon',
        urlFormat: 'HASH',
        title: () => {
            const ownerElement = document.querySelector('.ExtendedOwnerName');

            if (ownerElement) {
                return document.querySelector('.ConversationHeaderView .ConversationHeaderSubject').innerText.trim() + ' - ' + ownerElement.title.trim();
            }

            return document.querySelector('.ConversationHeaderView .ConversationHeaderSubject').innerText.trim();
        },
    },
    {
        id: 'startinfinity',
        name: 'Infinity - app.startinfinity.com',
        domains: ['https://app.startinfinity.com/'],
        anchors: [
            {
                anchor: '.item-sidebar-panel.right-panel .hbox.item-sidebar-header',
                placement: Component.AFTER
            },
            {
                anchor: '.item-sidebar-header .hbox.item-navigate',
                placement: Component.BEFORE
            }
        ],
        selector: '.menubar, .list-view, .draggable-item',
        removeSelector: '.v-popper__backdrop, .sidebar-links-link',
        urlFormat: 'PATH',
        title: () => {
            return document.querySelector('.item-sidebar .item-name').innerText.trim();
        }
    },
    {
        id: 'intercom',
        name: 'Intercom - intercom.com',
        domains: ['https://app.intercom.com/'],
        anchors: [
            {
                anchor: '.inbox2__conversation-details-sidebar-sheet [data-intercom-target-sidebar-conversation-state]',
                placement: Component.BEFORE
            },
            {
                anchor: 'div[data-intercom-target="conversation-sidebar"] .ember-view',
                placement: Component.BEFORE
            }
        ],
        selector: '.intercom-interblocks-html, .embercom-prosemirror-composer-attachment',
        urlFormat: 'PATH',
        title: () => {
            // old, new
            const titleElement = document.querySelector('.js__inbox-list-item a.o__active.ember-view .conversation__list__item__title__text,' +
                ' .inbox2__animated-list-item-container a.bg-blue-light.ember-view.group > div:nth-child(2) > div:nth-child(2) div.line-clamp-1 div.line-clamp-1,' +
                ' .inbox2__conversation-stream div[data-intercom-target="conversation-toolbar"] div.inbox-2__conversation-header div[data-conversation-title] span');
            const subtitleElement = document.querySelector('.inbox2__conversation-title .intercom-interblocks-html');

            return (subtitleElement ? subtitleElement.innerText.trim() + ' - ' : '') + titleElement.innerText.trim();
        }
    },
    {
        id: 'miro',
        name: 'Miro - miro.com',
        domains: ['https://miro.com/'],
        anchors: [
            {
                anchor: 'div[data-name="APP_BAR"]',
                placement: Component.BEFORE
            },
        ],
        selector: '.intercom-interblocks-html, .embercom-prosemirror-composer-attachment',
        urlFormat: 'PATH',
        title: () => {
            const titleElement = document.querySelector('[data-testid="board-header__title"] > div');
            return titleElement.innerText.trim();
        }
    },
    {
        id: 'pipedrive',
        name: 'Pipedrive - pipedrive.com',
        domains: ['https://*.pipedrive.com/'],
        anchors: [
            {
                anchor: 'div[data-test="contextual-view"] div[data-testid="SidebarWrapper"] section[data-testid="DealSection"]',
                placement: Component.BEFORE
            },
            {
                anchor: 'div[data-testid="header"] div[class*="WrapGroupWrapper-my-app"]',
                placement: Component.APPEND_FIRST
            },
            {
                anchor: '.viewContainer:not([style="display: none;"]) .infoBlock .actions',
                placement: Component.APPEND_FIRST,
            },
            {
                anchor: '.viewContainer:not([style="display: none;"]) .infoBlock .content',
                placement: Component.APPEND_FIRST,
            },
            {
                anchor: 'div[data-testid="header"] div[class*="Actions-my-app"]',
                placement: Component.APPEND_FIRST
            },
        ],
        urlFormat: 'PATH',
        title: () => {
            const pageTypeElement = document.querySelector('#froot-nav a.active');
            const pageTypeElementAriaLabel = pageTypeElement.getAttribute('aria-label');
            const pageType = pageTypeElementAriaLabel.slice(0, pageTypeElementAriaLabel.length - 1);
            let titleElement = document.querySelector('div[data-test="contextual-view"] div[data-testid="SidebarWrapper"] [data-testid="SidebarLeadTitle"] div[class*="EditFieldstyles"] > div');

            if (!titleElement) {
                titleElement = document.querySelector('div[data-test="contextual-view"] div[data-testid="SidebarWrapper"] [data-testid="SidebarLeadTitle"] div[class*="EditFieldstyles"]');
            }

            if (!titleElement) {
                titleElement = document.querySelector('.viewContainer:not([style="display: none;"]) .content h1 a')
            }

            if (!titleElement) {
                titleElement = document.querySelector('div[data-testid="header-title"] textarea');
            }

            return `[${pageType}] ${(titleElement.innerText || titleElement.innerHTML).trim()}`;

        },
        selector: 'div[data-testid="FlowToolbarTab-notes"], div[class*="tabs__tabs"], div[class*="tabs__content"], button[data-test="support-center"], span[class*="SidebarBlock"], div[data-testid="lead-info-wrapper"]',
        removeSelector: 'div[class*="spinner"], a[class*="scheduleButton"]',
        renderDebounceTimeout: 250,
    },
    {
        id: 'force',
        name: 'Salesforce - salesforce.com',
        domains: ['https://*.force.com/'],
        anchors: [
            {
                anchor: '.windowViewMode-normal.oneContent.active .slds-page-header .primaryFieldRow div.slds-media__body',
                placement: Component.AFTER
            }
        ],
        selector: '.forcePageBlockSection, .region-header, .slds-card, .forcegenerated-flexipage-module',
        urlFormat: 'PATH',
        renderDebounceTimeout: 500,
        title: () => {
            const titleElement = document.querySelector('.windowViewMode-normal.oneContent.active .slds-page-header h1');
            const pageType = titleElement.querySelector('.entityNameTitle').innerText.trim();
            const commentElement = titleElement.querySelector('.slds-page-header__title .custom-truncate, lightning-formatted-text[slot="primaryField"], lightning-formatted-name[slot="primaryField"], .slds-page-header__title > span');
            let comment = commentElement && commentElement.innerText.trim();

            if (!comment) {
                comment = document.title.split(' | ')[0];
            }

            return `[${pageType}] ${comment}`;
        }
    },
    {
        id: 'shortcut',
        name: 'Shortcut - shortcut.com',
        domains: ['https://app.shortcut.com/'],
        anchors: [
            {
                anchor: '.dialog-content-portals + .story-container .story-attributes .attribute-spacer',
                placement: Component.BEFORE
            }
        ],
        urlFormat: 'PATH',
        title: () => {
            return document.querySelector('h2.story-name').innerText.trim();
        }
    },
    {
        id: 'redbooth',
        name: 'Redbooth - redbooth.com',
        domains: ['https://redbooth.com/'],
        anchors: [
            {
                anchor: '.tb-modal-header .js-dialog-close + tb-modal-title',
                placement: Component.AFTER
            },
            {
                anchor: '.tb-element-title .tb-meta-components',
                placement: Component.APPEND_FIRST
            },
            {
                anchor: '.l-panes-view [class*="two_pane"] [class*="right_pane"] .cb__header',
                placement: Component.BEFORE
            }
        ],
        urlFormat: 'HASH',
        title: () => {
            return document.querySelector('.tb-modal-body .tb-element-title-inner h3 a,' +
                ' .l-panes-view [class*="two_pane"] [class*="right_pane"] .cb__header .title-link').innerText.trim();
        }
    },
    {
        id: 'zendesk',
        name: 'Zendesk - zendesk.com',
        domains: ['https://*.zendesk.com/'],
        anchors: [
            {
                anchor: '.ember-view.workspace.has-play:not([style*="display:none"]):not([style*="opacity: 0;"]) header.ember-view nav.ember-view span[data-test-id="tabs-section-nav-item-ticket"]',
                placement: Component.AFTER
            },
            {
                anchor: '.ember-view.workspace:not([style*="display:none"]):not([style*="opacity: 0;"]) header.ember-view nav.ember-view span[data-test-id="tabs-section-nav-item-ticket"]',
                placement: Component.AFTER
            }
        ],
        selector: '.zd-menu-list-holder, article[data-test-id="omni-log-comment-item"]',
        removeSelector: '[data-garden-id="modals.tooltip_modal"], div[data-garden-id="tooltip.tooltip"], [data-garden-container-id="containers.keyboardfocus"], figure.thumbnail-container, button[data-test-id="close-button"]',
        urlFormat: 'PATH',
        title: () => {
            return document.querySelector('div[data-test-id="header-toolbar"] div[aria-label="Tabs"] > div:not([data-garden-container-id="containers.tooltip"]) div[data-test-id="header-tab-subtitle"] > span').innerText.trim() + ' ' + document.title;
        },
        renderDebounceTimeout: 500
    },
    {
        id: 'zenkit',
        name: 'Zenkit - zenkit.com',
        domains: ['https://*.zenkit.com/'],
        anchors: [
            {
                anchor: '.zenkit-entry-detail-popup-subheader-wrapper .zenkit-entry-detail-popup-subheader-right',
                placement: Component.APPEND_FIRST
            },
        ],
        urlFormat: 'PATH',
        title: () => {
            return document.querySelector('.zenkit-ui-dialog__page .zenkit-ui-list__row__main .zenkit-details-view__display-string').innerText.trim();
        },
        renderDebounceTimeout: 500
    }
];

"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var ErrorWrapper = /*#__PURE__*/function (_React$Component) {
  function ErrorWrapper(props) {
    var _this;
    _classCallCheck(this, ErrorWrapper);
    _this = _callSuper(this, ErrorWrapper, [props]);
    _this.state = {
      error: null
    };
    return _this;
  }
  _inherits(ErrorWrapper, _React$Component);
  return _createClass(ErrorWrapper, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      var _this2 = this;
      browserPort.eventDispatcher.on('browser-button', 'render-errors', function (errorResponse) {
        _this2.props.reactAppStateObj.lockActions = false;
        _this2.setState({
          error: handleErrorsFromResponse(errorResponse)
        });
      });
    }
  }, {
    key: "handleCloseError",
    value: function handleCloseError() {
      this.setState({
        error: null
      });
    }
  }, {
    key: "render",
    value: function render() {
      var error = this.state.error;
      return /*#__PURE__*/React.createElement("div", {
        className: "ev-popover__errors-block"
      }, error ? /*#__PURE__*/React.createElement("div", {
        className: "message message--error",
        id: "everhour_banner_error"
      }, /*#__PURE__*/React.createElement("div", {
        className: "message__description"
      }, error), /*#__PURE__*/React.createElement("div", {
        className: "message__ctrl",
        onClick: this.handleCloseError.bind(this)
      }, /*#__PURE__*/React.createElement("span", {
        className: "icon-cancel"
      }))) : '');
    }
  }]);
}(React.Component);
{
  ErrorWrapper;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var BillingMessageWrapper = /*#__PURE__*/function (_React$Component) {
  function BillingMessageWrapper(props) {
    var _this;
    _classCallCheck(this, BillingMessageWrapper);
    _this = _callSuper(this, BillingMessageWrapper, [props]);
    _this.state = {
      billingMessage: null,
      owner: null,
      canPurchase: false
    };
    return _this;
  }
  _inherits(BillingMessageWrapper, _React$Component);
  return _createClass(BillingMessageWrapper, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      var props = this.props;
      var profile = props.store.profile.get();
      var billingMessage = null;
      var billing = profile.team.billing;
      if (billing.status === 'trial' && moment(billing.trialEndedAt).diff(moment(), 'days') <= 0) {
        billingMessage = 'trial_expires_today';
      } else if (billing.status === 'trial' && moment(billing.trialEndedAt).diff(moment(), 'days') === 1) {
        billingMessage = 'trial_expires_in_2_days';
      }
      this.setState({
        billingMessage: billingMessage,
        owner: profile.team ? profile.team.owner.name : '',
        canPurchase: props.store.profile.canPurchase()
      });
    }
  }, {
    key: "render",
    value: function render() {
      var state = this.state;
      var billingMessage = this.state.billingMessage;
      return /*#__PURE__*/React.createElement(React.Fragment, null, billingMessage ? /*#__PURE__*/React.createElement("div", {
        className: "message message--warning"
      }, /*#__PURE__*/React.createElement("div", {
        className: "message__description message__description-trial"
      }, /*#__PURE__*/React.createElement("span", null, billingMessage === 'trial_expires_today' ? 'Your 2-week trial expires today.' : 'Your 2-week trial expires in two short days.', state.canPurchase ? 'Take a few moments now to activate your full subscription.' : "To continue using Everhour, please contact any team admin to activate the full subscription."), state.canPurchase ? /*#__PURE__*/React.createElement("a", {
        href: "https://app.everhour.com/#/account/billing/settings",
        className: "button button--orange",
        target: "_blank"
      }, "Subscribe") : '')) : '');
    }
  }]);
}(React.Component);
{
  BillingMessageWrapper;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var ToolbarMenu = /*#__PURE__*/function (_React$Component) {
  function ToolbarMenu(props) {
    var _this;
    _classCallCheck(this, ToolbarMenu);
    _this = _callSuper(this, ToolbarMenu, [props]);
    _this.state = {
      user: _this.getUser(),
      isOpenMenu: false,
      todayTimecard: _this.getTimeCard(),
      isEnabledTimecard: _this.checkEnabledTimecard()
    };
    browserPort.eventDispatcher.on('browser-button', 'websocket-update', function () {
      _this.setState({
        todayTimecard: _this.getTimeCard(),
        isEnabledTimecard: _this.checkEnabledTimecard()
      });
    });
    return _this;
  }
  _inherits(ToolbarMenu, _React$Component);
  return _createClass(ToolbarMenu, [{
    key: "getUser",
    value: function getUser() {
      return this.props.store.profile.get();
    }
  }, {
    key: "getTimeCard",
    value: function getTimeCard() {
      return this.props.store.timer.timecard;
    }
  }, {
    key: "checkEnabledTimecard",
    value: function checkEnabledTimecard() {
      var user = this.getUser();
      return user.team.applications.timecards && !user.team.applications.timecardsUsers || user.team.applications.timecards && user.team.applications.timecardsUsers && (!!~user.team.applications.timecardsUsers.indexOf(user.id) || !user.team.applications.timecardsUsers.length);
    }
  }, {
    key: "onToggleMenu",
    value: function onToggleMenu(isOpen) {
      if (browserPort.type === 'safari' && !this.state.isEnabledTimecard) {
        return;
      }
      this.setState({
        isOpenMenu: isOpen
      });
    }
  }, {
    key: "toggleCheckin",
    value: function toggleCheckin() {
      var _this2 = this;
      var store = this.props.store;
      var isClockedIn = this.getClockStateIn();
      this.props.api.checkIn(this.state.user.id, isClockedIn).done(function (timecard) {
        _this2.props.reactAppStateObj.todayTimecard = timecard;
        store.timer.setTimecard(timecard);
        var timer = timecard.timerStatus;
        // handle timer stop
        if (timer && timer.taskTime) {
          store.dispatch(TimerStore.Act.Stopped(timer));
        }
        // notify front app
        store.dispatch(TimerStore.Act.TimecardUpdated(timecard));
      }).fail(function (response) {
        return browserPort.eventDispatcher.broadcast('browser-button', 'render-errors', response);
      });
    }
  }, {
    key: "getClockStateIn",
    value: function getClockStateIn() {
      return !this.state.todayTimecard || _format.isClockedIn(this.state.todayTimecard);
    }
  }, {
    key: "formatTimecardTime",
    value: function formatTimecardTime(time) {
      var isClockedIn = this.getClockStateIn();
      if (!this.state.todayTimecard) {
        return null;
      }
      return moment(isClockedIn ? this.state.todayTimecard.clockIn : this.state.todayTimecard.clockOut, 'h:mm').format('h:mm a');
    }
  }, {
    key: "handleOpenIntegrationsTab",
    value: function handleOpenIntegrationsTab() {
      browserPort.eventDispatcher.broadcast('browser-button', 'change-tab', {
        tab: 'integrations'
      });
    }
  }, {
    key: "render",
    value: function render() {
      var user = this.state.user;
      var isSafari = browserPort.type === 'safari';
      var isClockedIn = this.getClockStateIn();
      var time = this.formatTimecardTime();
      var isNedPlanUpdate = this.props.isNedPlanUpdate;
      var isEnabledTimecard = this.state.isEnabledTimecard;
      return /*#__PURE__*/React.createElement(Popover2, {
        triggerNode: /*#__PURE__*/React.createElement("div", {
          className: "toolbar-trigger ".concat(isSafari && !isEnabledTimecard ? 'toolbar-trigger--inactive' : '')
        }, /*#__PURE__*/React.createElement("div", {
          className: "toolbar-trigger__avatar"
        }, /*#__PURE__*/React.createElement(UserAvatar, {
          user: user
        }), !this.state.isEnabledTimecard && isNedPlanUpdate && /*#__PURE__*/React.createElement("div", {
          className: "toolbar-trigger__integrations-upgrade"
        }, "Upgrade"), this.state.isEnabledTimecard ? /*#__PURE__*/React.createElement("div", {
          className: "toolbar-trigger__timecard-status\n                                        ".concat(isClockedIn ? 'toolbar-trigger__timecard-status--active' : '')
        }, isClockedIn ? 'in' : 'out') : ''), (!isSafari || isSafari && isEnabledTimecard) && /*#__PURE__*/React.createElement("div", {
          className: "icon-arrow ".concat(this.state.isOpenMenu ? 'icon-arrow-up' : '')
        })),
        trigger: !isSafari || isSafari && isEnabledTimecard ? 'click' : null,
        placement: "bottom-right",
        onToggle: this.onToggleMenu.bind(this),
        onClose: this.onToggleMenu.bind(this, false),
        popoverClasses: 'toolbar-popover',
        isCanClose: false,
        width: 192
      }, /*#__PURE__*/React.createElement("div", {
        className: "toolbar-menu"
      }, this.state.isEnabledTimecard ? /*#__PURE__*/React.createElement("div", {
        className: "toolbar-menu__time-card ".concat(isSafari ? 'toolbar-menu__time-card--safari' : '')
      }, isClockedIn ? /*#__PURE__*/React.createElement("button", {
        type: "button",
        className: "button button--orange",
        onClick: this.toggleCheckin.bind(this)
      }, "Clock-Out") : /*#__PURE__*/React.createElement("button", {
        type: "button",
        className: "button button--green",
        onClick: this.toggleCheckin.bind(this)
      }, "Clock-In"), this.state.todayTimecard && (this.state.todayTimecard.clockIn || this.state.todayTimecard.clockOut) ? /*#__PURE__*/React.createElement("div", {
        className: "time-card__log"
      }, /*#__PURE__*/React.createElement("div", {
        className: "icon-help"
      }), /*#__PURE__*/React.createElement("div", {
        className: "log-text"
      }, /*#__PURE__*/React.createElement("span", null, "You've clocked ", isClockedIn ? 'in' : 'out', " at\xA0", /*#__PURE__*/React.createElement("span", {
        className: "text-bold"
      }, this.formatTimecardTime())))) : /*#__PURE__*/React.createElement("div", {
        className: "time-card__log"
      }, /*#__PURE__*/React.createElement("div", {
        className: "log-text"
      }, /*#__PURE__*/React.createElement("span", null, "You haven\u2019t clocked in today")))) : '', !isSafari ? /*#__PURE__*/React.createElement("div", {
        className: "toolbar-menu__item",
        onClick: this.handleOpenIntegrationsTab.bind(this)
      }, "Integrations") : '', !isSafari && /*#__PURE__*/React.createElement("div", {
        className: "toolbar-menu__item"
      }, /*#__PURE__*/React.createElement("a", {
        href: "https://support.everhour.com/",
        target: "_blank"
      }, "Help Docs")), !isSafari && isNedPlanUpdate && /*#__PURE__*/React.createElement("div", {
        className: "toolbar-menu__item toolbar-menu__item--upgrade-plan"
      }, /*#__PURE__*/React.createElement("a", {
        href: everhour.productionUrl + '/?upgrade',
        target: "_blank"
      }, "Upgrade", /*#__PURE__*/React.createElement("span", {
        className: "icon-lightning"
      }))), !isSafari ? /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("div", {
        className: "toolbar-menu__separator"
      }), /*#__PURE__*/React.createElement("div", {
        className: "toolbar-menu__item"
      }, /*#__PURE__*/React.createElement("a", {
        href: "https://app.everhour.com/",
        target: "_blank"
      }, "Go to\xA0", /*#__PURE__*/React.createElement("span", {
        className: "text-bold"
      }, "everhour.com")))) : ''));
    }
  }]);
}(React.Component);
{
  ToolbarMenu;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var Toolbar = /*#__PURE__*/function (_React$Component) {
  function Toolbar(props) {
    var _this;
    _classCallCheck(this, Toolbar);
    _this = _callSuper(this, Toolbar, [props]);
    _this.isSafari = browserPort.type === 'safari';
    return _this;
  }
  _inherits(Toolbar, _React$Component);
  return _createClass(Toolbar, [{
    key: "handleChangeTab",
    value: function handleChangeTab(tab, isSearchOpenFromLens) {
      browserPort.eventDispatcher.broadcast('browser-button', 'change-tab', {
        tab: tab,
        isSearchOpenFromLens: isSearchOpenFromLens
      });
    }
  }, {
    key: "render",
    value: function render() {
      var _this2 = this;
      var props = this.props;
      var isNedPlanUpdate = this.props.isNedPlanUpdate;
      var href = isNedPlanUpdate ? everhour.productionUrl + '/?upgrade' : everhour.productionUrl;
      return /*#__PURE__*/React.createElement("header", {
        className: "ev-popover__toolbar"
      }, /*#__PURE__*/React.createElement("span", {
        className: "toolbar__logo-wrapper"
      }, this.isSafari ? /*#__PURE__*/React.createElement("span", {
        className: "toolbar__logo"
      }) : /*#__PURE__*/React.createElement("a", {
        className: "toolbar__logo js-open-link",
        href: href,
        target: "_blank"
      }, isNedPlanUpdate && /*#__PURE__*/React.createElement("span", {
        className: "toolbar__logo-update-plan-wrapper"
      }, /*#__PURE__*/React.createElement("span", {
        className: "toolbar__logo-update-plan icon-lightning"
      })))), /*#__PURE__*/React.createElement("nav", null, /*#__PURE__*/React.createElement("ul", {
        className: "toolbar__nav"
      }, props.tabs.map(function (tab) {
        return /*#__PURE__*/React.createElement("li", {
          className: "toolbar__nav-item\n                                            ".concat(props.activeTab === tab.id || tab.id === 'time' && props.activeTab === 'search' && !props.isSearchOpenFromLens ? 'toolbar__nav-item--active' : ''),
          onClick: _this2.handleChangeTab.bind(_this2, tab.id, false)
        }, tab.name);
      }))), /*#__PURE__*/React.createElement("div", {
        className: "toolbar__search ".concat(props.isSearchOpenFromLens ? 'toolbar__search--active' : ''),
        onClick: this.handleChangeTab.bind(this, 'search', true)
      }, /*#__PURE__*/React.createElement("div", {
        className: "icon-search"
      })), /*#__PURE__*/React.createElement("div", {
        className: "toolbar__menu"
      }, /*#__PURE__*/React.createElement(ToolbarMenu, {
        isNedPlanUpdate: isNedPlanUpdate,
        store: props.store,
        api: props.api,
        reactAppStateObj: props.reactAppStateObj
      })));
    }
  }]);
}(React.Component);
{
  Toolbar;
}
;
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var PopoverContent = /*#__PURE__*/function (_React$Component) {
  function PopoverContent() {
    _classCallCheck(this, PopoverContent);
    return _callSuper(this, PopoverContent, arguments);
  }
  _inherits(PopoverContent, _React$Component);
  return _createClass(PopoverContent, [{
    key: "handleClickOutSide",
    value: function handleClickOutSide(e) {
      if (this.node && this.props.closeOnClick) {
        this.props.onClose();
      }
      if (this.node && !this.props.closeOnClick && !this.node.contains(e.target)) {
        this.props.onClickOutSide();
      }
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      this.node && this.node.focus();
      this.node && this.node.addEventListener('blur', this.handleClickOutSide.bind(this), false);
      window.document.addEventListener('click', this.handleClickOutSide.bind(this), false);
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.node && this.node.blur && this.node.blur();
      this.node && this.node.removeEventListener('blur', this.handleClickOutSide.bind(this), false);
      window.document.removeEventListener('click', this.handleClickOutSide.bind(this), false);
    }
  }, {
    key: "onKeyDown",
    value: function onKeyDown(e) {
      if (e.keyCode === 27) {
        e.stopPropagation();
        this.props.onClose();
      }
    }
  }, {
    key: "render",
    value: function render() {
      var _this = this;
      return /*#__PURE__*/React.createElement("div", {
        ref: function ref(node) {
          return _this.node = node;
        },
        tabIndex: "-1",
        onKeyDown: this.onKeyDown.bind(this),
        className: "popover-content \n                    ".concat(this.props.className ? ' ' + this.props.className : '', " \n                    ").concat(this.props.placement ? ' ' + this.props.placement : '', "\n                "),
        style: this.props.style
      }, /*#__PURE__*/React.createElement("div", {
        className: "popover-inner"
      }, this.props.children));
    }
  }]);
}(React.Component);
var Popover = /*#__PURE__*/function (_React$Component2) {
  function Popover(props) {
    var _this2;
    _classCallCheck(this, Popover);
    _this2 = _callSuper(this, Popover, [props]);
    _this2.state = {
      visible: !!_this2.props.visible
    };
    return _this2;
  }
  _inherits(Popover, _React$Component2);
  return _createClass(Popover, [{
    key: "onShow",
    value: function onShow(e) {
      e.stopPropagation();
      this.setState({
        visible: true
      });
    }
  }, {
    key: "onClose",
    value: function onClose() {
      this.setState({
        visible: false
      });
      this.props.onClose && this.props.onClose();
    }
  }, {
    key: "onToggle",
    value: function onToggle(e) {
      e.stopPropagation();
      this.setState({
        visible: !this.state.visible
      });
      this.props.onToggle && this.props.onToggle(!this.state.visible);
    }
  }, {
    key: "onClickOutSide",
    value: function onClickOutSide() {
      this.onClose();
      this.props.onClickOutSide && this.props.onClickOutSide();
    }
  }, {
    key: "render",
    value: function render() {
      return /*#__PURE__*/React.createElement("div", {
        className: "popover"
      }, this.props.triggerNode && React.cloneElement(this.props.triggerNode, {
        onClick: this.props.trigger === 'click' || this.props.trigger === 'hover' ? this.onToggle.bind(this) : null,
        onMouseOver: this.props.trigger === 'hover' ? this.onShow.bind(this) : null
      }), this.state.visible && /*#__PURE__*/React.createElement(PopoverContent, {
        className: this.props.className,
        style: this.props.style,
        placement: this.props.placement,
        closeOnClick: this.props.closeOnClick,
        onClickOutSide: this.onClickOutSide.bind(this),
        onClose: this.onClose.bind(this)
      }, this.props.children));
    }
  }]);
}(React.Component);
{
  Popover;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var PopoverPortal = /*#__PURE__*/function (_React$PureComponent) {
  function PopoverPortal(props) {
    var _this;
    _classCallCheck(this, PopoverPortal);
    _this = _callSuper(this, PopoverPortal, [props]);
    _this.el = document.createElement('div');
    _this.el.classList.add('popover-portal');
    _this.el.setAttribute('data-popover-id', _this.props.randomId);
    return _this;
  }
  _inherits(PopoverPortal, _React$PureComponent);
  return _createClass(PopoverPortal, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this.props.trigger.appendChild(this.el);
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.props.trigger.removeChild(this.el);
    }
  }, {
    key: "render",
    value: function render() {
      return ReactDOM.createPortal(this.props.children, this.el);
    }
  }]);
}(React.PureComponent);
var Popover2 = /*#__PURE__*/function (_React$PureComponent2) {
  function Popover2(props) {
    var _this2;
    _classCallCheck(this, Popover2);
    _this2 = _callSuper(this, Popover2, [props]);
    _this2.state = {
      visible: false
    };
    _this2.randomId = Math.random().toString(36).substr(2, 55);
    _this2.width = props.width || 296;
    _this2.space = props.space || 0;
    _this2.isShowClose = _this2.props.hasOwnProperty('isShowClose') ? _this2.props.isShowClose : true;
    _this2.isCanClose = _this2.props.isCanClose !== undefined ? _this2.props.isCanClose : true;
    _this2.isShowArrow = _this2.props.isShowArrow !== undefined ? _this2.props.isShowArrow : false;
    _this2.placement = _this2.props.placement || 'bottom';
    _this2.popverService = new Popover2Service();
    _this2.popverService.addPopover(_this2.randomId, _this2);
    return _this2;
  }
  _inherits(Popover2, _React$PureComponent2);
  return _createClass(Popover2, [{
    key: "handleClickOutSide",
    value: function handleClickOutSide(e) {
      if (this.node && this.props.closeOnClick) {
        this.onClose();
      }
      if (this.node && !this.props.closeOnClick && !this.node.el.children[0].contains(e.target)) {
        this.onClickOutSide();
      }
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      this.node && this.node.focus();
      this.node && this.node.addEventListener('blur', this.handleClickOutSide.bind(this), false);
      window.document.addEventListener('click', this.handleClickOutSide.bind(this), false);
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.node && this.node.blur && this.node.blur();
      this.node && this.node.removeEventListener && this.node.removeEventListener('blur', this.handleClickOutSide.bind(this), false);
      window.document.removeEventListener('click', this.handleClickOutSide.bind(this), false);
    }
  }, {
    key: "onClose",
    value: function onClose() {
      this.hidePopover();
      this.props.onClose && this.props.onClose();
      this.props.onToggle && this.props.onToggle(false);
    }
  }, {
    key: "onToggle",
    value: function onToggle(e) {
      e.stopPropagation();
      this.popverService.closeNotCurrent(this.randomId);
      var isVisible = this.state.visible;
      if (isVisible) {
        this.hidePopover();
        this.props.onClose && this.props.onClose();
      } else {
        this.showPopover();
      }
      this.props.onToggle && this.props.onToggle(!isVisible);
    }
  }, {
    key: "showPopover",
    value: function showPopover() {
      var style = {
        width: this.width
      };
      var styleArrow = {};
      var dimensions = this.el.getBoundingClientRect();
      var arrowHeight = 5;
      var arrowWidth = 22;
      style.left = dimensions.width / 2 - this.width / 2;
      style.left = Math.max(this.space, style.left);
      if (this.placement === 'bottom') {
        style.left = -(this.width / 2 - dimensions.width / 2);
        styleArrow.left = dimensions.width / 2 - arrowWidth / 2;
      } else if (this.placement === 'bottom-right') {
        style.left = dimensions.width - this.width;
        styleArrow.left = dimensions.width / 2 - arrowWidth / 2;
      } else if (this.placement === 'bottom-left') {
        style.left = 0;
        styleArrow.left = dimensions.width / 2 - arrowWidth / 2;
      }
      if (dimensions.left + this.width >= document.body.clientWidth) {
        delete style.left;
        style.right = 0;
      }
      if (dimensions.bottom < window.innerHeight / 1.25) {
        style.top = dimensions.height + this.space + arrowHeight;
        styleArrow.top = dimensions.height + this.space;
      } else {
        style.bottom = dimensions.height + this.space + arrowHeight;
        styleArrow.bottom = dimensions.height + this.space - 1;
      }
      this.setState({
        visible: true,
        style: style,
        styleArrow: styleArrow
      });
    }
  }, {
    key: "hidePopover",
    value: function hidePopover() {
      this.setState({
        visible: false
      });
    }
  }, {
    key: "onClickOutSide",
    value: function onClickOutSide() {
      this.onClose();
      this.props.onClickOutSide && this.props.onClickOutSide();
    }
  }, {
    key: "onKeyDown",
    value: function onKeyDown(e) {
      if (e.keyCode === 27) {
        e.stopPropagation();
        this.props.onClose();
      }
    }
  }, {
    key: "render",
    value: function render() {
      var _this3 = this;
      return /*#__PURE__*/React.createElement("div", {
        className: "popover-trigger",
        "data-popover-id": this.randomId,
        ref: function ref(el) {
          return _this3.el = el;
        }
      }, this.props.triggerNode && this.props.trigger === 'click' ? React.cloneElement(this.props.triggerNode, {
        onClick: this.props.trigger === 'click' || this.props.trigger === 'hover' ? this.onToggle.bind(this) : null
      }) : '', this.props.triggerNode && this.props.trigger === 'hover' ? React.cloneElement(this.props.triggerNode, {
        onMouseOver: this.props.trigger === 'hover' ? this.showPopover.bind(this) : null,
        onMouseOut: this.props.trigger === 'hover' ? this.onClose.bind(this) : null
      }) : '', this.props.triggerNode && !this.props.trigger ? this.props.triggerNode : '', this.state.visible && /*#__PURE__*/React.createElement(PopoverPortal, {
        ref: function ref(node) {
          return _this3.node = node;
        },
        tabIndex: "-1",
        onKeyDown: this.onKeyDown.bind(this),
        onClickOutSide: this.onClickOutSide.bind(this),
        onClose: this.onClose.bind(this),
        randomId: this.randomId,
        trigger: this.el
      }, this.isShowArrow ? /*#__PURE__*/React.createElement("div", {
        className: "popover-arrow ".concat(this.state.styleArrow.bottom ? 'popover-arrow--down' : ''),
        style: this.state.styleArrow
      }) : '', /*#__PURE__*/React.createElement("div", {
        className: "popover-body",
        style: this.state.style
      }, this.props.title || this.props.isShowClose ? /*#__PURE__*/React.createElement("div", {
        className: "popover__header"
      }, /*#__PURE__*/React.createElement("div", {
        className: "popover__header-title"
      }, this.props.title), /*#__PURE__*/React.createElement("div", {
        className: "popover__header-close"
      }, /*#__PURE__*/React.createElement("div", {
        className: "icon-cancel",
        onClick: this.onClose.bind(this)
      }))) : '', /*#__PURE__*/React.createElement("div", {
        className: "popover__content ".concat(this.props.popoverClasses ? this.props.popoverClasses : '')
      }, this.props.children), this.props.actionsNode ? /*#__PURE__*/React.createElement("div", {
        className: "popover__actions"
      }, this.props.actionsNode) : '')));
    }
  }]);
}(React.PureComponent);
{
  Popover2;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
var popover2instance = null;
var Popover2Service = /*#__PURE__*/function () {
  function Popover2Service() {
    _classCallCheck(this, Popover2Service);
    if (!popover2instance) {
      this.popovers = {};
      popover2instance = this;
    }
    return popover2instance;
  }
  return _createClass(Popover2Service, [{
    key: "addPopover",
    value: function addPopover(key, popover) {
      this.popovers[key] = popover;
    }
  }, {
    key: "closeAll",
    value: function closeAll() {
      for (var k in this.popovers) {
        var popover = this.popovers[k];
        if (!document.querySelector(".popover-trigger[data-popover-id=\"".concat(k, "\"]"))) {
          delete this.popovers[k];
          continue;
        }
        if (popover.isCanClose) {
          popover.onClose();
        }
      }
    }
  }, {
    key: "closeNotCurrent",
    value: function closeNotCurrent(key) {
      for (var k in this.popovers) {
        var popover = this.popovers[k];
        if (!document.querySelector(".popover-trigger[data-popover-id=\"".concat(k, "\"]"))) {
          delete this.popovers[k];
          continue;
        }
        if (popover.isCanClose && popover.randomId !== key) {
          popover.onClose();
        }
      }
    }
  }]);
}();
{
  Popover2Service;
}
;
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TooltipPortal = /*#__PURE__*/function (_React$PureComponent) {
  function TooltipPortal(props) {
    var _this;
    _classCallCheck(this, TooltipPortal);
    _this = _callSuper(this, TooltipPortal, [props]);
    _this.el = document.createElement('div');
    return _this;
  }
  _inherits(TooltipPortal, _React$PureComponent);
  return _createClass(TooltipPortal, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      document.body.appendChild(this.el);
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      document.body.removeChild(this.el);
    }
  }, {
    key: "render",
    value: function render() {
      return ReactDOM.createPortal(this.props.children, this.el);
    }
  }]);
}(React.PureComponent);
var Tooltip = /*#__PURE__*/function (_React$PureComponent2) {
  function Tooltip(props) {
    var _this2;
    _classCallCheck(this, Tooltip);
    _this2 = _callSuper(this, Tooltip, [props]);
    _this2.state = {
      visible: false
    };
    _this2.width = props.width || 200;
    _this2.space = props.space || 0;
    _this2.placement = props.placement || 'auto';
    _this2.showTooltip = _this2.showTooltip.bind(_this2);
    _this2.hideTooltip = _this2.hideTooltip.bind(_this2);
    return _this2;
  }
  _inherits(Tooltip, _React$PureComponent2);
  return _createClass(Tooltip, [{
    key: "showTooltip",
    value: function showTooltip() {
      var style = {
        width: this.width
      };
      var styleArrow = {};
      var dimensions = this.el.getBoundingClientRect();
      var arrowWidth = 5;
      var leftCompensation = 8;
      var placement = this.placement;
      style.left = dimensions.left + dimensions.width / 2 - this.width / 2;
      style.left = Math.max(this.space, style.left);
      style.left = Math.min(style.left, document.body.clientWidth - this.width - this.space);
      styleArrow.left = dimensions.left + dimensions.width / 2 - arrowWidth;
      if (style.left === 0) {
        style.left = leftCompensation;
      }
      if (style.left + this.width === document.body.clientWidth) {
        style.left -= leftCompensation;
      }
      if (placement !== 'auto') {
        switch (placement) {
          case 'top':
            style.bottom = window.innerHeight - dimensions.top + this.space;
            styleArrow.bottom = window.innerHeight - dimensions.top + this.space - arrowWidth;
            break;
          case 'bottom':
            style.top = dimensions.top + dimensions.height + this.space;
            styleArrow.top = dimensions.bottom + this.space - arrowWidth;
            break;
          default:
            break;
        }
        this.setState({
          visible: true,
          style: style,
          styleArrow: styleArrow
        });
        return;
      }
      if (dimensions.top - dimensions.height < 0) {
        style.top = dimensions.top + dimensions.height + this.space;
        styleArrow.top = dimensions.bottom + this.space - arrowWidth;
      } else if (dimensions.top < window.innerHeight / 2) {
        // style.top = dimensions.top + dimensions.height + this.space;
        // styleArrow.top = dimensions.bottom + this.space - arrowWidth;
        style.bottom = window.innerHeight - dimensions.top + this.space;
        styleArrow.bottom = window.innerHeight - dimensions.top + this.space - arrowWidth;
      } else {
        style.bottom = window.innerHeight - dimensions.top + this.space;
        styleArrow.bottom = window.innerHeight - dimensions.top + this.space - arrowWidth;
      }
      this.setState({
        visible: true,
        style: style,
        styleArrow: styleArrow
      });
    }
  }, {
    key: "hideTooltip",
    value: function hideTooltip() {
      this.setState({
        visible: false
      });
    }
  }, {
    key: "render",
    value: function render() {
      var _this3 = this;
      return /*#__PURE__*/React.createElement("div", {
        onMouseOver: this.showTooltip,
        onMouseOut: this.hideTooltip,
        className: "tooltip-trigger",
        ref: function ref(el) {
          return _this3.el = el;
        }
      }, this.props.triggerTemplate ? this.props.triggerTemplate() : this.props.text, this.state.visible && /*#__PURE__*/React.createElement(TooltipPortal, null, /*#__PURE__*/React.createElement("div", {
        className: "tooltip-arrow \n                                         ".concat(this.state.styleArrow.bottom ? 'tooltip-arrow--down' : ''),
        style: this.state.styleArrow
      }), /*#__PURE__*/React.createElement("div", {
        className: "tooltip-body ".concat(this.props.classes || ''),
        style: this.state.style
      }, this.props.children)));
    }
  }]);
}(React.PureComponent);
{
  Tooltip;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var Modal = /*#__PURE__*/function (_React$Component) {
  function Modal(props) {
    var _this;
    _classCallCheck(this, Modal);
    _this = _callSuper(this, Modal, [props]);
    _this.defaultButtonElement = null;
    _this.firstTabStop = null;
    _this.lastTabStop = null;
    return _this;
  }
  _inherits(Modal, _React$Component);
  return _createClass(Modal, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      var _this2 = this;
      var modalOverlayElement = document.getElementById('everhour-modal-overlay');
      var modalElement = modalOverlayElement && modalOverlayElement.querySelector('.modal.modal--show');
      setTimeout(function () {
        var focusableElementsString = 'a[href], area[href], input:not([disable]), select:not([disabled]), textarea:not([disabled]), button:not([disabled]), iframe, object, embed, [tabindex="0"], [contenteditable]';
        var focusableElements = modalOverlayElement.querySelectorAll(focusableElementsString);
        _this2.defaultFocusElement = focusableElements[0];
        _this2.firstTabStop = focusableElements[1];
        _this2.lastTabStop = focusableElements[focusableElements.length - 1];
        _this2.subscribeEvents(modalOverlayElement, modalElement);
      });
    }
  }, {
    key: "subscribeEvents",
    value: function subscribeEvents(modalOverlayElement, modalElement) {
      var _this3 = this;
      modalOverlayElement.onclick = function (event) {
        if (event.target === modalElement) {
          _this3.props.onClose();
        }
        if (event.target !== modalElement && document.activeElement === document.body) {
          _this3.defaultFocusElement.focus();
        }
      };
      modalOverlayElement.addEventListener('escEvent', function (e) {
        _this3.props.onClose();
      });
      modalOverlayElement.onkeydown = function (event) {
        if (event.keyCode === 27) {
          _this3.props.onClose();
        }
        if (event.keyCode === 9 && event.shiftKey && document.activeElement === _this3.firstTabStop) {
          event.preventDefault();
          _this3.lastTabStop.focus();
        }
        if (event.keyCode === 9 && !event.shiftKey && document.activeElement === _this3.lastTabStop) {
          event.preventDefault();
          _this3.firstTabStop.focus();
        }
      };
    }
  }, {
    key: "render",
    value: function render() {
      return /*#__PURE__*/React.createElement("div", {
        "data-modal-type": "everhour-modal",
        className: "modal modal--show ".concat(this.props.classes || '')
      }, /*#__PURE__*/React.createElement("button", {
        type: "button",
        "data-type": 'autofocus',
        className: "button button--custom-focus"
      }), /*#__PURE__*/React.createElement("div", {
        className: "modal__wrapper"
      }, this.props.title ? /*#__PURE__*/React.createElement("div", {
        className: "modal__header"
      }, /*#__PURE__*/React.createElement("div", {
        className: "modal__header-title"
      }, this.props.title), /*#__PURE__*/React.createElement("div", {
        className: "modal__header-close",
        onClick: this.props.onClose
      }, /*#__PURE__*/React.createElement("svg", {
        xmlns: "http://www.w3.org/2000/svg"
      }, /*#__PURE__*/React.createElement("path", {
        d: "M7.6 6.45l5.176-5.175.275-.275.549.55-.275.274L8.15 7l5.176 5.176.275.275-.55.549-.274-.275L7.6 7.55l-5.176 5.176-.275.275-.549-.55.275-.274L7.05 7 1.875 1.824 1.6 1.549 2.15 1l.274.275L7.6 6.45z",
        fill: "none",
        stroke: "#C7CACD",
        "stroke-linecap": "square",
        "stroke-width": "1.5"
      })))) : '', /*#__PURE__*/React.createElement("div", {
        className: "modal__content"
      })));
    }
  }]);
}(React.Component);
{
  Modal;
}
;
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
var ModalService = /*#__PURE__*/function () {
  function ModalService() {
    _classCallCheck(this, ModalService);
    this.rootModalSelectorName = 'everhour-modal-overlay';
    this.rootModalElement = null;
    this.modalContentSelector = '.modal__content';
    this.modalContentElement = null;
    this.component = null;
    this.componentProps = null;
    this.modalNameMap = {
      'AddTimeModal': 'Add Time',
      'AddCommentModal': 'Update Comments',
      'AssignTimeModal': 'Assign Time',
      'DeleteTimeModal': 'Confirmation',
      'StopTimerModal': 'Force Stop Timer',
      'EditTimeModal': 'Edit Time'
    };
  }
  return _createClass(ModalService, [{
    key: "showModal",
    value: function showModal(component, props, options) {
      var rootModalWrapper = document.createElement('div');
      rootModalWrapper.setAttribute('id', this.rootModalSelectorName);
      document.body.appendChild(rootModalWrapper);
      this.rootModalElement = rootModalWrapper;
      this.component = component;
      this.componentProps = props;
      var modalComponent = React.createElement(component, Object.assign({}, props, {
        onClose: this.closeModal.bind(this)
      }));
      ReactDOM.render(React.createElement(Modal, {
        title: options && options.title || this.modalNameMap[modalComponent.type.name],
        onClose: this.closeModal.bind(this),
        classes: options && options.classes || ''
      }), this.rootModalElement);
      this.modalContentElement = rootModalWrapper.querySelector(this.modalContentSelector);
      ReactDOM.render(modalComponent, this.modalContentElement);
    }
  }, {
    key: "closeModal",
    value: function closeModal(data) {
      this.componentProps.onClose && this.componentProps.onClose(data);
      if (this.modalContentElement) {
        ReactDOM.unmountComponentAtNode(this.modalContentElement);
      }
      if (this.rootModalElement) {
        document.body.removeChild(this.rootModalElement);
      }
      this.component = null;
      this.componentProps = null;
    }
  }]);
}();
{
  ModalService;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var Skeleton = /*#__PURE__*/function (_React$Component) {
  function Skeleton(props) {
    _classCallCheck(this, Skeleton);
    return _callSuper(this, Skeleton, [props]);
  }
  _inherits(Skeleton, _React$Component);
  return _createClass(Skeleton, [{
    key: "getCounts",
    value: function getCounts(count) {
      return new Array(count).fill(count);
    }
  }, {
    key: "renderSkeletonHeader",
    value: function renderSkeletonHeader() {
      return /*#__PURE__*/React.createElement("div", {
        className: "skeleton__header"
      }, /*#__PURE__*/React.createElement("a", {
        className: "skeleton__header-logo",
        href: "https://app.everhour.com/"
      }));
    }
  }, {
    key: "renderTimeSkeleton",
    value: function renderTimeSkeleton() {
      var _this = this;
      return /*#__PURE__*/React.createElement("div", {
        className: "skeleton"
      }, this.renderSkeletonHeader(), /*#__PURE__*/React.createElement("div", {
        className: "skeleton__content skeleton__content-time"
      }, /*#__PURE__*/React.createElement("div", {
        className: "content-time__search"
      }, /*#__PURE__*/React.createElement("div", {
        className: "content-time__search-left"
      }, /*#__PURE__*/React.createElement("div", {
        className: "skeleton__rect skeleton__rect--w-215 skeleton__rect--h-16"
      })), /*#__PURE__*/React.createElement("div", {
        className: "content-time__search-right"
      }, /*#__PURE__*/React.createElement("div", {
        className: "skeleton__rect skeleton__rect--w-88 skeleton__rect--h-32"
      }), /*#__PURE__*/React.createElement("div", {
        className: "skeleton__rect skeleton__rect--w-88 skeleton__rect--h-32"
      }))), /*#__PURE__*/React.createElement("div", {
        className: "content-time__week-row"
      }, /*#__PURE__*/React.createElement("div", {
        className: "skeleton__rect skeleton__rect--w-192 skeleton__rect--h-16"
      })), this.getCounts(2).map(function (dayGroup) {
        return /*#__PURE__*/React.createElement("div", {
          className: "content-time__day-group"
        }, /*#__PURE__*/React.createElement("div", {
          className: "day-group__title skeleton__rect"
        }), _this.getCounts(4).map(function (row) {
          return /*#__PURE__*/React.createElement("div", {
            className: "day-group__row"
          }, /*#__PURE__*/React.createElement("div", {
            className: "row__time skeleton__rect skeleton__rect--w-40 skeleton__rect--h-14"
          }), /*#__PURE__*/React.createElement("div", {
            className: "row__description"
          }, /*#__PURE__*/React.createElement("div", {
            className: "description__top skeleton__rect skeleton__rect--w-304 skeleton__rect--h-14"
          }), /*#__PURE__*/React.createElement("div", {
            className: "description__bottom skeleton__rect skeleton__rect--w-184 skeleton__rect--h-14"
          })));
        }));
      })));
    }
  }, {
    key: "renderEditTimeSkeleton",
    value: function renderEditTimeSkeleton() {
      return /*#__PURE__*/React.createElement("div", {
        className: "skeleton skeleton-task-details__edit-time"
      }, this.getCounts(6).map(function (row) {
        return /*#__PURE__*/React.createElement("div", {
          className: "content-task-details__row"
        }, /*#__PURE__*/React.createElement("div", {
          className: "content-task-details__row-date skeleton__rect skeleton__rect--w-50 skeleton__rect--h-14"
        }), /*#__PURE__*/React.createElement("div", {
          className: "content-task-details__row-assignee"
        }, /*#__PURE__*/React.createElement("div", {
          className: "content-task-details__row-assignee-avatar skeleton__rect"
        }), /*#__PURE__*/React.createElement("div", {
          className: "content-task-details__row-assignee-name skeleton__rect skeleton__rect--w-110 skeleton__rect--h-14"
        })), /*#__PURE__*/React.createElement("div", {
          className: "content-task-details__row-time skeleton__rect skeleton__rect--w-64 skeleton__rect--h-14"
        }));
      }));
    }
  }, {
    key: "renderRecentSkeleton",
    value: function renderRecentSkeleton() {
      return /*#__PURE__*/React.createElement("div", {
        className: "skeleton"
      }, /*#__PURE__*/React.createElement("div", {
        className: "skeleton__content skeleton__content-recent"
      }, this.props.options && this.props.options.withTitle ? /*#__PURE__*/React.createElement("div", {
        className: "content-recent__title"
      }, /*#__PURE__*/React.createElement("div", {
        className: "skeleton__rect skeleton__rect--w-88 skeleton__rect--h-14"
      })) : '', this.getCounts(10).map(function (row) {
        return /*#__PURE__*/React.createElement("div", {
          className: "content-recent__row"
        }, /*#__PURE__*/React.createElement("div", {
          className: "row__left"
        }, /*#__PURE__*/React.createElement("div", {
          className: "row__status skeleton__rect"
        }), /*#__PURE__*/React.createElement("div", {
          className: "row__description"
        }, /*#__PURE__*/React.createElement("div", {
          className: "description__top skeleton__rect skeleton__rect--w-304 skeleton__rect--h-14"
        }), /*#__PURE__*/React.createElement("div", {
          className: "description__bottom skeleton__rect skeleton__rect--w-184 skeleton__rect--h-14"
        }))), /*#__PURE__*/React.createElement("div", {
          className: "row__time skeleton__rect skeleton__rect--w-48 skeleton__rect--h-14"
        }));
      })));
    }
  }, {
    key: "renderSelectTaskSkeleton",
    value: function renderSelectTaskSkeleton() {
      return /*#__PURE__*/React.createElement("div", {
        className: "skeleton"
      }, /*#__PURE__*/React.createElement("div", {
        className: "skeleton__content skeleton__select-task"
      }, this.props.options && this.props.options.withTitle ? /*#__PURE__*/React.createElement("div", {
        className: "content-recent__title"
      }, /*#__PURE__*/React.createElement("div", {
        className: "skeleton__rect skeleton__rect--w-88 skeleton__rect--h-14"
      })) : '', this.getCounts(10).map(function (row) {
        return /*#__PURE__*/React.createElement("div", {
          className: "content-recent__row"
        }, /*#__PURE__*/React.createElement("div", {
          className: "row__left"
        }, /*#__PURE__*/React.createElement("div", {
          className: "row__status skeleton__rect"
        }), /*#__PURE__*/React.createElement("div", {
          className: "row__description"
        }, /*#__PURE__*/React.createElement("div", {
          className: "description__top skeleton__rect skeleton__rect--w-304 skeleton__rect--h-14"
        }))));
      })));
    }
  }, {
    key: "renderTimesheetSkeleton",
    value: function renderTimesheetSkeleton() {
      var _this2 = this;
      return /*#__PURE__*/React.createElement("div", {
        className: "skeleton"
      }, /*#__PURE__*/React.createElement("div", {
        className: "skeleton__content skeleton__content-timesheet"
      }, this.getCounts(2).map(function (week) {
        return /*#__PURE__*/React.createElement("div", {
          className: "content-timesheet__week"
        }, /*#__PURE__*/React.createElement("div", {
          className: "content-timesheet__title"
        }, /*#__PURE__*/React.createElement("div", {
          className: "skeleton__rect skeleton__rect--w-204 skeleton__rect--h-14"
        })), /*#__PURE__*/React.createElement("div", {
          className: "content-timesheet__day-group"
        }, /*#__PURE__*/React.createElement("div", {
          className: "day-group__title"
        }, /*#__PURE__*/React.createElement("div", {
          className: "title__left"
        }), /*#__PURE__*/React.createElement("div", {
          className: "title__right"
        })), _this2.getCounts(5).map(function (row) {
          return /*#__PURE__*/React.createElement("div", {
            className: "content-timesheet__row"
          }, /*#__PURE__*/React.createElement("div", {
            className: "row__left"
          }, /*#__PURE__*/React.createElement("div", {
            className: "skeleton__rect skeleton__rect--light skeleton__rect--w-367 skeleton__rect--h-30"
          })), /*#__PURE__*/React.createElement("div", {
            className: "row__right"
          }));
        })));
      })));
    }
  }, {
    key: "renderDetailsSkeleton",
    value: function renderDetailsSkeleton() {
      return /*#__PURE__*/React.createElement("div", {
        className: "skeleton"
      }, /*#__PURE__*/React.createElement("div", {
        className: "skeleton__content skeleton__content-details"
      }, /*#__PURE__*/React.createElement("div", {
        className: "skeleton__content-details-header"
      }), /*#__PURE__*/React.createElement("div", {
        className: "skeleton__content-details-info"
      }, /*#__PURE__*/React.createElement("div", {
        className: "skeleton__content-details-info-name skeleton__rect skeleton__rect--w-352 skeleton__rect--h-18"
      }), /*#__PURE__*/React.createElement("div", {
        className: "skeleton__content-details-info-total skeleton__rect skeleton__rect--w-296 skeleton__rect--h-14"
      }), /*#__PURE__*/React.createElement("div", {
        className: "skeleton__content-details-info-timer"
      }), this.getCounts(2).map(function (row) {
        return /*#__PURE__*/React.createElement("div", {
          className: "skeleton__content-details-info-row skeleton__rect skeleton__rect--w-200 skeleton__rect--h-14"
        });
      }))));
    }
  }, {
    key: "renderTimersSkeleton",
    value: function renderTimersSkeleton() {
      return /*#__PURE__*/React.createElement("div", {
        className: "skeleton"
      }, /*#__PURE__*/React.createElement("div", {
        className: "skeleton__content skeleton__timers"
      }, this.getCounts(8).map(function (row) {
        return /*#__PURE__*/React.createElement("div", {
          className: "skeleton__timers-row"
        }, /*#__PURE__*/React.createElement("div", {
          className: "timers-row__avatar skeleton__rect"
        }), /*#__PURE__*/React.createElement("div", {
          className: "timers-row__user-description"
        }, /*#__PURE__*/React.createElement("div", {
          className: "user-description__name skeleton__rect skeleton__rect--w-64 skeleton__rect--h-14"
        }), /*#__PURE__*/React.createElement("div", {
          className: "skeleton__rect skeleton__rect--w-32 skeleton__rect--h-14"
        })), /*#__PURE__*/React.createElement("div", {
          className: "timers-row__task-description"
        }, /*#__PURE__*/React.createElement("div", {
          className: "task-description__top skeleton__rect skeleton__rect--w-288 skeleton__rect--h-14"
        }), /*#__PURE__*/React.createElement("div", {
          className: "skeleton__rect skeleton__rect--w-208 skeleton__rect--h-14"
        })));
      })));
    }
  }, {
    key: "render",
    value: function render() {
      var view = this.props.view;
      if (view === 'time') {
        return this.renderTimeSkeleton();
      }
      if (view === 'editTime') {
        return this.renderEditTimeSkeleton();
      }
      if (view === 'recent') {
        return this.renderRecentSkeleton();
      }
      if (view === 'timesheet') {
        return this.renderTimesheetSkeleton();
      }
      if (view === 'details') {
        return this.renderDetailsSkeleton();
      }
      if (view === 'timers') {
        return this.renderTimersSkeleton();
      }
      if (view === 'select-task') {
        return this.renderSelectTaskSkeleton();
      }
      return '';
    }
  }]);
}(React.Component);
{
  Skeleton;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var HighlightText = /*#__PURE__*/function (_React$Component) {
  function HighlightText(props) {
    _classCallCheck(this, HighlightText);
    return _callSuper(this, HighlightText, [props]);
  }
  _inherits(HighlightText, _React$Component);
  return _createClass(HighlightText, [{
    key: "searchStr",
    value: function searchStr(str, searchTerm, pos) {
      return str.indexOf(searchTerm, pos);
    }
  }, {
    key: "render",
    value: function render() {
      var children = this.props.children;
      var searchTerm = this.props.searchTerm;
      if (!searchTerm) {
        return children;
      }

      // const regexp = new RegExp(searchTerm, 'gi');
      // const matches = children.match(regexp);
      // let parts = children.split(new RegExp(`${searchTerm.replace()}`, 'gi'));
      //
      // for (let i = 0; i < parts.length; i++) {
      //     if (i !== parts.length - 1) {
      //         let match = matches[i];
      //         // While the next part is an empty string, merge the corresponding match with the current
      //         // match into a single <span/> to avoid consequent spans with nothing between them.
      //         while(parts[i + 1] === '') {
      //             if (matches[++i]) {
      //                 match += matches[++i];
      //             }
      //         }
      //
      //         parts[i] = (
      //             <React.Fragment key={i}>
      //                 {parts[i]}<span className="highlighted">{match}</span>
      //             </React.Fragment>
      //         );
      //     }
      // }

      // const matches = children.split(searchTerm);
      // let parts = [];
      //
      // if (matches.length <= 1) {
      //     parts = parts.concat(matches);
      //
      //     return <span className="task-row__description-name">{parts}</span>;
      // }
      //
      // for (let k = 0; k < matches.length; k++) {
      //     const part = matches[k];
      //
      //     if (k === matches.length - 1) {
      //         parts.push(<React.Fragment key={k}>
      //             {part}
      //         </React.Fragment>);
      //
      //         continue;
      //     }
      //
      //     parts.push(<React.Fragment key={k}>
      //         {part}<span className="highlighted">{searchTerm}</span>
      //     </React.Fragment>)
      // }
      //
      // return <span className="task-row__description-name">{parts}</span>;
      var searchTermStr = searchTerm.toLowerCase();
      var searchTermLength = searchTermStr.length;
      var searchStr = children.toLowerCase();
      var searchStrLength = searchStr.length;
      var parts = [];
      var pos = 0;
      var isEndSearch = false;
      do {
        var searchedPosition = this.searchStr(searchStr, searchTermStr, pos);

        // find in first
        if (searchedPosition === 0 && searchedPosition <= searchStrLength) {
          parts.push( /*#__PURE__*/React.createElement(React.Fragment, {
            key: searchedPosition
          }, /*#__PURE__*/React.createElement("span", {
            className: "highlighted"
          }, children.slice(pos, searchTermLength))));
          pos = searchedPosition + searchTermLength;
          continue;
        }

        // find in middle
        if (searchedPosition > 0 && searchedPosition <= searchStrLength) {
          parts.push( /*#__PURE__*/React.createElement(React.Fragment, {
            key: searchedPosition
          }, children.slice(pos, searchedPosition), /*#__PURE__*/React.createElement("span", {
            className: "highlighted"
          }, children.slice(searchedPosition, searchedPosition + searchTermLength))));
          pos = searchedPosition + searchTermLength;
          continue;
        }

        // merge end
        if (pos > 0 && searchedPosition === -1) {
          parts.push(children.slice(pos));
        }

        // not match
        if (pos === 0 && searchedPosition === -1) {
          parts.push(children);
        }
        isEndSearch = true;
      } while (!isEndSearch);
      return /*#__PURE__*/React.createElement("span", {
        className: "task-row__description-name"
      }, parts);
    }
  }]);
}(React.Component);
{
  HighlightText;
}
;
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var ScrollingContainer = /*#__PURE__*/function (_React$Component) {
  function ScrollingContainer(props) {
    var _this;
    _classCallCheck(this, ScrollingContainer);
    _this = _callSuper(this, ScrollingContainer, [props]);
    _this.state = {
      activeKeyId: null
    };
    _this._subscribeKeyboardEvents = _this.subscribeKeyboardEvents.bind(_this);
    browserPort.eventDispatcher.on('browser-button', 'scrolling-container-reset-events', function () {
      _this.resetEvents();
    });
    return _this;
  }
  _inherits(ScrollingContainer, _React$Component);
  return _createClass(ScrollingContainer, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this.setEvents();
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate() {
      var eventSelector = document.getElementById(this.props.eventSelectorId);
      var activeElement = eventSelector && eventSelector.querySelector("".concat(this.props.itemSelector, ".sc-active"));
      var initActiveElement = eventSelector && eventSelector.querySelector("".concat(this.props.itemSelector));
      if (eventSelector && initActiveElement && !activeElement) {
        this.resetEvents();
        this.setEvents();
        initActiveElement.classList.add("sc-active");
        this.setState({
          activeKeyId: initActiveElement.dataset.keyId
        });
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.resetEvents();
    }
  }, {
    key: "setEvents",
    value: function setEvents() {
      var eventSelector = document.getElementById(this.props.eventSelectorId);
      var initActiveElement = eventSelector && eventSelector.querySelector("".concat(this.props.itemSelector));
      if (eventSelector && initActiveElement) {
        eventSelector.addEventListener('keydown', this._subscribeKeyboardEvents);
        initActiveElement.classList.add("sc-active");
        this.setState({
          activeKeyId: initActiveElement.dataset.keyId
        });
      }
    }
  }, {
    key: "resetEvents",
    value: function resetEvents() {
      var eventSelector = document.getElementById(this.props.eventSelectorId);
      if (eventSelector) {
        eventSelector.removeEventListener('keydown', this._subscribeKeyboardEvents);
      }
    }
  }, {
    key: "subscribeKeyboardEvents",
    value: function subscribeKeyboardEvents(event) {
      var keyCode = event && event.keyCode;
      if (!keyCode) {
        return;
      }
      switch (keyCode) {
        case 13:
          // Enter
          this.props.onSelect(this.state.activeKeyId);
          break;
        case 38:
          // ArrowUp
          this.setActiveElement(38);
          break;
        case 40:
          // ArrowDown
          this.setActiveElement(40);
          break;
        default:
          break;
      }
    }
  }, {
    key: "setActiveElement",
    value: function setActiveElement(code) {
      var eventElement = document.getElementById(this.props.eventSelectorId);
      var listElements = eventElement && eventElement.querySelectorAll(this.props.itemSelector);
      var activeListElement = eventElement && eventElement.querySelector("".concat(this.props.itemSelector, ".sc-active"));
      var activeProjectElementPosition = 0;
      var newActiveProjectElementPosition = 0;
      if (!listElements || !activeListElement) {
        return;
      }
      for (var i = 0; i < listElements.length; i++) {
        var listElement = listElements[i];
        if (listElement.dataset.keyId === activeListElement.dataset.keyId) {
          activeProjectElementPosition = i;
          break;
        }
      }
      var newActiveListElement = null;
      if (code === 38) {
        newActiveListElement = !activeProjectElementPosition ? listElements[listElements.length - 1] : listElements[activeProjectElementPosition - 1];
        newActiveProjectElementPosition = !activeProjectElementPosition ? listElements.length - 1 : activeProjectElementPosition - 1;
      }
      if (code === 40) {
        newActiveListElement = activeProjectElementPosition === listElements.length - 1 ? listElements[0] : listElements[activeProjectElementPosition + 1];
        newActiveProjectElementPosition = activeProjectElementPosition === listElements.length - 1 ? 0 : activeProjectElementPosition + 1;
      }
      var isFirstElement = newActiveProjectElementPosition === 0;
      var isLastElement = newActiveProjectElementPosition === listElements.length - 1;
      var containerElements = document.querySelector(this.props.listContainerSelector);
      var containerElementsOffsetTop = containerElements.offsetTop;
      var containerElementsScrollTop = containerElements.scrollTop;
      var containerElementsScrollBottom = containerElements.scrollTop + containerElements.offsetHeight;
      var newActiveElementTop = newActiveListElement.offsetTop - containerElements.offsetTop;
      var newActiveElementBottom = newActiveListElement.offsetTop + newActiveListElement.offsetHeight - containerElements.offsetTop;
      switch (true) {
        case isFirstElement:
          containerElements.scrollTop = 0;
          break;
        case isLastElement:
          containerElements.scrollTop = newActiveElementTop;
          break;
        case newActiveElementTop < containerElementsScrollTop:
          containerElements.scrollTop = newActiveListElement.offsetTop - containerElementsOffsetTop;
          break;
        case newActiveElementBottom > containerElementsScrollBottom:
          containerElements.scrollTop = newActiveListElement.offsetTop + newActiveListElement.offsetHeight - containerElements.offsetHeight - containerElementsOffsetTop;
          break;
      }
      activeListElement.classList.remove('sc-active');
      newActiveListElement.classList.add('sc-active');
      this.setState({
        activeKeyId: newActiveListElement.dataset.keyId
      });
    }
  }, {
    key: "render",
    value: function render() {
      return /*#__PURE__*/React.createElement(React.Fragment, null, this.props.children);
    }
  }]);
}(React.Component);
{
  ScrollingContainer;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TasksSelectField = /*#__PURE__*/function (_React$Component) {
  function TasksSelectField(props) {
    var _this;
    _classCallCheck(this, TasksSelectField);
    _this = _callSuper(this, TasksSelectField, [props]);
    _this.LocalStorageService = new window.LocalStorageService();
    var everhourPopupSearchTab = _this.LocalStorageService.getItem('everhour-popup-search-tab');
    if (!everhourPopupSearchTab) {
      everhourPopupSearchTab = {
        mode: 'tasks'
      };
      _this.LocalStorageService.setItem('everhour-popup-search-tab', everhourPopupSearchTab);
    }
    _this.state = {
      task: props.task ? props.task : null,
      isSearchOpened: false,
      isFetchTasks: false,
      searchedRecentTasks: null,
      weekRecentTask: null,
      isError: false,
      errorMessage: null,
      tasks: null,
      // tasks for render
      recentTasks: null,
      // recent tasks for default(!searchTerm)
      tasksByProject: null,
      // merge tasks for default(!searchTerm)
      modified: null,
      mode: everhourPopupSearchTab.mode,
      projects: null,
      // projects for render
      _projects: null,
      // projects for default(!searchTerm or filter by searchTerm)
      projectForSearchTasks: null
    };
    _this.searchInput = null;
    _this.selectId = "tasks-select--".concat(+new Date());
    _this._request = null;
    _this._cancelRequest = _this.cancelRequest.bind(_this);
    _this.searchTerm = '';
    _this._searchWithDelay = _.debounce(_this.searchWithDelay, 250, false);
    _this._outClick = _this.outClick.bind(_this);
    _this.popupControllerService = new PopupControllerService();
    return _this;
  }
  _inherits(TasksSelectField, _React$Component);
  return _createClass(TasksSelectField, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this.getProjects();
      if (this.state.mode === 'tasks') {
        this.fetchRecentTasks();
      }
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate() {
      if (this.state.isSearchOpened && this.searchInput) {
        this.searchInput.focus();
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      document.removeEventListener('click', this._outClick);
    }
  }, {
    key: "outClick",
    value: function outClick(event) {
      if (!event) {
        return;
      }
      var isTaskSelectWrapper = event.target.classList.contains('tasks-select__wrapper-placeholder');
      if (!isTaskSelectWrapper && !event.target.closest('.tasks-select__wrapper') && this.state.isSearchOpened) {
        this.setState({
          isSearchOpened: false
        });
        document.removeEventListener('click', this._outClick);
      }
    }
  }, {
    key: "getProjects",
    value: function getProjects() {
      var projects = this.props.store.projects.getGroupsProjects();
      this.setState({
        projects: projects,
        _projects: projects
      });
    }
  }, {
    key: "search",
    value: function search(event) {
      var state = this.state;
      var searchTerm = this.searchTerm = event.target.value;
      if (!searchTerm) {
        this.setState({
          isFetchTasks: false,
          tasks: state.mode === 'tasks' ? state.recentTasks : state.tasksByProject,
          projects: state._projects
        });
        this.searchTerm = '';
        this._cancelRequest();
        return;
      }
      this._searchWithDelay(searchTerm);
    }
  }, {
    key: "searchWithDelay",
    value: function searchWithDelay(searchTerm) {
      if (this.state.mode === 'tasks') {
        this.searchRecentTasks(searchTerm);
      } else if (this.state.mode === 'projects' && this.state.projectForSearchTasks) {
        this.searchRecentTasksByProject(searchTerm, this.state.projectForSearchTasks);
      } else if (this.state.mode === 'projects' && !this.state.projectForSearchTasks) {
        this.handleFilterProjects();
      }
    }
  }, {
    key: "cancelRequest",
    value: function cancelRequest() {
      if (this._request && this._request.abort) {
        this._request.abort();
      }
    }
  }, {
    key: "checkWithComments",
    value: function checkWithComments() {
      return this.props.withComments;
    }
  }, {
    key: "fetchRecentTasks",
    value: function fetchRecentTasks() {
      var _this2 = this;
      this.setState({
        isFetchTasks: true
      });
      this.props.api.searchRecentTasks(null, null, this.checkWithComments(), null).then(function (tasks) {
        var modifiedTasks = {
          search: {
            label: '',
            tasks: []
          },
          recent: {
            label: 'recent',
            tasks: _this2.modifyTasks(tasks)
          },
          opened: {
            label: 'opened',
            tasks: []
          }
        };
        _this2.setState({
          tasks: modifiedTasks,
          recentTasks: modifiedTasks,
          isFetchTasks: false
        });
      });
    }
  }, {
    key: "searchRecentTasks",
    value: function searchRecentTasks(searchTerm) {
      var _this3 = this;
      if (!this.searchTerm) {
        return;
      }
      this.setState({
        isFetchTasks: true,
        tasks: null
      });
      this._request = this.props.api.searchRecentTasks(searchTerm, null, this.checkWithComments(), this._cancelRequest);
      this._request.then(function (tasks) {
        var modifiedTasks = {
          search: {
            label: "".concat(tasks.length, " result").concat(tasks.length > 1 ? 's' : ''),
            tasks: _this3.modifyTasks(tasks)
          },
          recent: {
            label: 'recent',
            tasks: []
          },
          opened: {
            label: 'opened',
            tasks: []
          }
        };
        _this3.setState({
          tasks: modifiedTasks,
          isFetchTasks: false
        });
        _this3._request = null;
      });
    }
  }, {
    key: "fetchTasksByProject",
    value: function fetchTasksByProject(projectId) {
      var _this4 = this;
      this.setState({
        isFetchTasks: true
      });
      return this.props.api.getTasksByProject(projectId).then(function (tasks) {
        _this4.projectTasks = tasks;
      });
    }
  }, {
    key: "fetchRecentTaskByProject",
    value: function fetchRecentTaskByProject(projectId) {
      var _this5 = this;
      this.setState({
        isFetchTasks: true
      });
      return this.props.api.getRecentTasksByProject(projectId).then(function (tasks) {
        _this5.recentProjectTasks = tasks;
      });
    }
  }, {
    key: "searchRecentTasksByProject",
    value: function searchRecentTasksByProject(searchTerm, projectId) {
      var _this6 = this;
      if (!this.searchTerm) {
        return;
      }
      this.setState({
        isFetchTasks: true,
        tasks: null
      });
      this._request = this.props.api.searchTasksByProject(searchTerm || '', projectId, this._cancelRequest);
      this._request.then(function (tasks) {
        _this6.setState({
          isFetchTasks: false,
          tasks: {
            search: {
              label: "".concat(tasks.length, " result").concat(tasks.length > 1 ? 's' : ''),
              tasks: tasks
            },
            recent: {
              label: 'recent',
              tasks: []
            },
            opened: {
              label: 'opened',
              tasks: []
            }
          }
        });
        _this6._request = null;
      });
    }
  }, {
    key: "modifyTasks",
    value: function modifyTasks(tasks) {
      return tasks.map(function (task) {
        if ((task._type || task.type) && task._type === 'comment' || task.type === 'comment') {
          task._id = Math.floor(Math.random() * 1000000);
        }
        return task;
      });
    }
  }, {
    key: "getProjectTasks",
    value: function getProjectTasks(projectId) {
      var _this7 = this;
      if (!projectId) {
        return;
      }
      this.fetchRecentTaskByProject(projectId).then(function () {
        _this7.setState({
          tasks: {
            search: {
              label: '',
              tasks: []
            },
            recent: {
              label: 'recent',
              tasks: _this7.recentProjectTasks
            },
            opened: {
              label: 'opened',
              tasks: []
            }
          }
        });
        if (!_this7.popupControllerService.isSyncedProjectByTimestamp(projectId)) {
          return _this7.props.api.pushsync(projectId).then(function () {
            _this7.popupControllerService.setSyncedProjectTimestamp(projectId);
          });
        }
      }).then(function () {
        if (!_this7.popupControllerService.isSyncedProjectByTimestamp(projectId)) {
          return Promise.all([_this7.fetchRecentTaskByProject(projectId), _this7.fetchTasksByProject(projectId)]);
        }
        return _this7.fetchTasksByProject(projectId);
      }).then(function () {
        _this7.mergeTasks();
      });
    }
  }, {
    key: "mergeTasks",
    value: function mergeTasks() {
      var _this8 = this;
      var recentTasksIds = [];
      var recentTasks = this.recentProjectTasks.filter(function (recentTask) {
        if (_this8.projectTasks.find(function (task) {
          return task.id === recentTask.id;
        })) {
          recentTasksIds.push(recentTask.id);
          return true;
        }
        return false;
      });
      var projectTasks = this.projectTasks.filter(function (task) {
        return recentTasksIds.indexOf(task.id) === -1;
      }).sort(function (taskA, taskB) {
        var taskAName = taskA.name.toLowerCase();
        var taskBName = taskB.name.toLowerCase();
        if (taskAName < taskBName) {
          return -1;
        }
        if (taskBName > taskAName) {
          return 1;
        }
      });
      this.setState({
        isFetchTasks: false,
        tasksByProject: {
          search: {
            label: '',
            tasks: []
          },
          recent: {
            label: 'recent',
            tasks: recentTasks
          },
          opened: {
            label: 'opened',
            tasks: projectTasks
          }
        },
        tasks: {
          search: {
            label: '',
            tasks: []
          },
          recent: {
            label: 'recent',
            tasks: recentTasks
          },
          opened: {
            label: 'opened',
            tasks: projectTasks
          }
        }
      });
    }
  }, {
    key: "handleFilterProjects",
    value: function handleFilterProjects() {
      var searchTerm = this.searchTerm;
      var _projects = this.state._projects;
      var recentProjects = everhourMethods.filterByMatch(_projects.recent.projects, '_searchName', searchTerm);
      var activeProjects = everhourMethods.filterByMatch(_projects.active.projects, '_searchName', searchTerm);
      this.setState({
        projects: {
          recent: Object.assign({}, _projects.recent, {
            projects: recentProjects
          }),
          active: Object.assign({}, _projects.active, {
            projects: activeProjects
          })
        }
      });
    }
  }, {
    key: "toggleSearchMode",
    value: function toggleSearchMode(event) {
      event.preventDefault();
      event.stopPropagation();
      browserPort.eventDispatcher.broadcast('browser-button', 'scrolling-container-reset-events');
      var mode = this.state.mode === 'projects' ? 'tasks' : 'projects';
      this.LocalStorageService.setItem('everhour-popup-search-tab', {
        mode: mode
      });
      this.setState({
        mode: mode,
        projects: this.state._projects
      });
      this.searchTerm = '';
      if (this.searchInput) {
        this.searchInput.value = '';
        this.searchInput.focus();
      }
      if (mode === 'tasks') {
        this.fetchRecentTasks();
      }
    }
  }, {
    key: "handleSelectProject",
    value: function handleSelectProject(projectId, event) {
      event && event.preventDefault();
      event && event.stopPropagation();
      browserPort.eventDispatcher.broadcast('browser-button', 'scrolling-container-reset-events');
      this.setState({
        projectForSearchTasks: projectId
      });
      this.searchTerm = '';
      if (this.searchInput) {
        this.searchInput.value = '';
      }
      this.getProjectTasks(projectId);
    }
  }, {
    key: "handleResetProject",
    value: function handleResetProject(event) {
      event.preventDefault();
      event.stopPropagation();
      this.setState({
        projectForSearchTasks: null,
        projects: this.state._projects,
        isFetchTasks: false
      });
      this.searchTerm = '';
      if (this.searchInput) {
        this.searchInput.value = '';
      }
    }
  }, {
    key: "toggleSearch",
    value: function toggleSearch(isOpened) {
      this.setState({
        isSearchOpened: isOpened
      });
      if (isOpened) {
        document.removeEventListener('click', this._outClick);
        document.addEventListener('click', this._outClick);
      } else {
        document.removeEventListener('click', this._outClick);
      }
    }
  }, {
    key: "handleSelectTask",
    value: function handleSelectTask(taskId) {
      var task = null;
      var tasks = this.state.tasks;
      if (taskId) {
        var mergedTasks = [].concat(tasks.opened.tasks || []).concat(tasks.recent.tasks || []).concat(tasks.search.tasks || []);
        task = mergedTasks.find(function (task) {
          return task.id === taskId || task._id === taskId;
        });
      }
      this.setState({
        task: task,
        isSearchOpened: false,
        isError: false,
        errorMessage: null
      });
      this.searchTerm = '';
      if (this.searchInput) {
        this.searchInput.value = '';
      }
      document.removeEventListener('click', this._outClick);
      if (this.props.onSelect && typeof this.props.onSelect === 'function') {
        this.props.onSelect(task);
      }
    }
  }, {
    key: "resetTask",
    value: function resetTask(event) {
      event.preventDefault();
      event.stopPropagation();
      this.handleSelectTask(null);
    }
  }, {
    key: "toggleError",
    value: function toggleError(error) {
      this.setState({
        isError: !!error,
        errorMessage: error
      });
    }
  }, {
    key: "renderProjects",
    value: function renderProjects(projects) {
      var _this9 = this;
      return /*#__PURE__*/React.createElement(ScrollingContainer, {
        eventSelectorId: this.selectId,
        listContainerSelector: '.group__projects.group__projects-scrolling-container-tasks-field',
        itemSelector: '.group__projects-row:not(.group__projects-row--title)',
        onSelect: this.handleSelectProject.bind(this)
      }, /*#__PURE__*/React.createElement("div", {
        className: "group__projects group__projects-scrolling-container-tasks-field"
      }, Object.keys(projects).map(function (projectType) {
        var projectsArray = projects[projectType] && projects[projectType].projects;
        if (projectsArray && projectsArray.length) {
          return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
            className: "group__projects-row group__projects-row--title",
            key: 'project-row-recent'
          }, projects[projectType].label), projectsArray.map(function (project) {
            var client = project.client ? _this9.props.store.clients.get(project.client) : null;
            var clientIntegrationIcon = client ? getPlatformName(client.externalId || 'ev') : null;
            return /*#__PURE__*/React.createElement("div", {
              className: "group__projects-row",
              "data-key-id": project.id,
              onClick: _this9.handleSelectProject.bind(_this9, project.id)
            }, /*#__PURE__*/React.createElement("div", {
              className: "project-row__icon"
            }, /*#__PURE__*/React.createElement("div", {
              className: "integration-icon integration-".concat(getPlatformName(project.platform))
            })), /*#__PURE__*/React.createElement("div", {
              className: "project-row__wrapper"
            }, /*#__PURE__*/React.createElement("span", {
              className: "project-row__name"
            }, /*#__PURE__*/React.createElement(HighlightText, {
              searchTerm: _this9.searchTerm
            }, project.name)), project.workspaceName ? /*#__PURE__*/React.createElement("span", {
              className: "project-row__name-workspace"
            }, /*#__PURE__*/React.createElement(HighlightText, {
              searchTerm: _this9.searchTerm
            }, project.workspaceName)) : ''), client && _this9.isAdmin ? /*#__PURE__*/React.createElement("div", {
              className: "project-row__client"
            }, /*#__PURE__*/React.createElement("div", {
              className: "project-row__client-name"
            }, client.name), clientIntegrationIcon ? /*#__PURE__*/React.createElement("div", {
              className: "project-row__client-icon integration-icon\n                                              integration-".concat(clientIntegrationIcon)
            }) : '') : '');
          }));
        }
        return '';
      })));
    }
  }, {
    key: "renderTasks",
    value: function renderTasks(tasks) {
      var _this10 = this;
      return /*#__PURE__*/React.createElement(ScrollingContainer, {
        eventSelectorId: this.selectId,
        listContainerSelector: '.group__tasks.group__tasks-scrolling-container-tasks-select',
        itemSelector: '.task-row:not(.task-row--title)',
        onSelect: this.handleSelectTask.bind(this)
      }, /*#__PURE__*/React.createElement("div", {
        className: "group__tasks group__tasks-scrolling-container-tasks-select"
      }, Object.keys(tasks).map(function (taskType) {
        var tasksArray = tasks[taskType] && tasks[taskType].tasks;
        if (tasksArray && tasksArray.length) {
          return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
            className: "task-row task-row--title",
            key: 'task-row-recent'
          }, tasks[taskType].label), tasksArray.map(function (task) {
            return /*#__PURE__*/React.createElement("article", {
              className: "task-row task-row--recent task-time",
              "data-task-id": task ? task.id : null,
              "data-task-time-id": null,
              "data-key-id": task ? task.id || task._id : null,
              onClick: _this10.handleSelectTask.bind(_this10, task.id || task._id)
            }, task && (task._type === 'task' || task.type === 'task') || !task._type && !task.type ? /*#__PURE__*/React.createElement(TaskStatus, {
              task: task,
              store: _this10.props.store
            }) : '', task && (task._type === 'comment' || task.type === 'comment') ? /*#__PURE__*/React.createElement("div", {
              className: "task-row__comment-icon icon-comment"
            }) : '', /*#__PURE__*/React.createElement("div", {
              className: "task-row__description"
            }, /*#__PURE__*/React.createElement("div", {
              className: "task-row__description-top"
            }, /*#__PURE__*/React.createElement("span", {
              className: "task-row__description-name",
              style: {
                flex: (task.comment || task.name).length < 30 ? '0 0 auto' : '0 1 auto'
              }
            }, task.name || task.comment ? /*#__PURE__*/React.createElement(HighlightText, {
              searchTerm: _this10.searchTerm
            }, task.name || task.comment) : /*#__PURE__*/React.createElement("span", {
              className: "task-row__description-name task-row__description-name--empty"
            }, "Task without title")), task && task.parentId && /*#__PURE__*/React.createElement("span", {
              className: "task-row__description-parent"
            }, task.parentName || 'Task without title'), task && task.number && /*#__PURE__*/React.createElement("span", {
              className: "task-row__description-number task-number"
            }, "#", task.number))));
          }));
        }
        return '';
      })));
    }
  }, {
    key: "render",
    value: function render() {
      var _this11 = this;
      var state = this.state;
      var task = state.task;
      var tasks = state.tasks;
      var project = state.projectForSearchTasks && this.props.store.projects.get(state.projectForSearchTasks);
      var isShowModeSwitch = !state.projectForSearchTasks;
      return /*#__PURE__*/React.createElement("div", {
        className: "field__wrapper tasks-select",
        id: this.selectId
      }, /*#__PURE__*/React.createElement("button", {
        className: "button--custom-focus",
        type: "button",
        onFocus: this.toggleSearch.bind(this, true)
      }), /*#__PURE__*/React.createElement("label", {
        className: "field__label"
      }, "Task"), /*#__PURE__*/React.createElement("div", {
        className: "field__body"
      }, /*#__PURE__*/React.createElement("div", {
        className: "tasks-select__wrapper ".concat(this.state.isSearchOpened ? 'tasks-select--open' : '')
      }, !task ? /*#__PURE__*/React.createElement("div", {
        className: "tasks-select__wrapper-placeholder ".concat(this.state.isError ? 'everhour-error' : ''),
        "data-error": this.state.isError ? this.state.errorMessage : '',
        onClick: this.toggleSearch.bind(this, true)
      }, /*#__PURE__*/React.createElement("div", {
        className: "icon-lens"
      }), /*#__PURE__*/React.createElement("div", {
        className: "tasks-select__placeholder"
      }, this.task ? '' : 'Select task...'), /*#__PURE__*/React.createElement("div", {
        className: "icon-arrow"
      })) : /*#__PURE__*/React.createElement("div", {
        className: "tasks-select__wrapper-exist-placeholder",
        onClick: this.toggleSearch.bind(this, true)
      }, (task._type || task.type) && (task._type === 'comment' || task.type === 'comment') ? /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
        className: "task-row__comment-icon icon-comment"
      }), /*#__PURE__*/React.createElement("div", {
        className: "task-row__name"
      }, /*#__PURE__*/React.createElement("span", {
        className: "task-row__description-name"
      }, task.comment)), /*#__PURE__*/React.createElement("div", {
        className: "icon-delete-circle",
        onClick: this.resetTask.bind(this)
      })) : /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(TaskStatus, {
        task: task,
        store: this.props.store
      }), /*#__PURE__*/React.createElement("div", {
        className: "task-row__name"
      }, task.number ? /*#__PURE__*/React.createElement("span", {
        className: "task-row__number"
      }, "#", task.number) : '', /*#__PURE__*/React.createElement("span", {
        className: "task-row__description-name"
      }, task.name || 'Task without title')), /*#__PURE__*/React.createElement("div", {
        className: "icon-delete-circle",
        onClick: this.resetTask.bind(this)
      }))), this.state.isSearchOpened ? /*#__PURE__*/React.createElement("div", {
        className: "tasks-select__search-wrapper",
        id: "tasks-select__search-wrapper"
      }, /*#__PURE__*/React.createElement("div", {
        className: "search-wrapper"
      }, /*#__PURE__*/React.createElement("div", {
        className: "icon-lens"
      }), /*#__PURE__*/React.createElement("input", {
        type: "text",
        className: "tasks-select__input",
        ref: function ref(input) {
          _this11.searchInput = input;
        },
        onChange: this.search.bind(this)
      }), /*#__PURE__*/React.createElement("div", {
        className: "icon-arrow"
      })), state.projectForSearchTasks ? /*#__PURE__*/React.createElement("div", {
        className: "search__project-active-container"
      }, /*#__PURE__*/React.createElement("div", {
        className: "search__project-active"
      }, /*#__PURE__*/React.createElement("div", {
        className: "integration-icon integration-".concat(getPlatformName(project.platform))
      }), /*#__PURE__*/React.createElement("div", {
        className: "project-active__name"
      }, project.name), /*#__PURE__*/React.createElement("div", {
        className: "project-active__reset",
        onClick: this.handleResetProject.bind(this)
      }, /*#__PURE__*/React.createElement("div", {
        className: "icon-remove-tag"
      })))) : '', /*#__PURE__*/React.createElement("div", {
        className: "search__results"
      }, !this.state.isFetchTasks && (tasks && tasks.search && tasks.search.tasks.length || tasks && tasks.recent && tasks.recent.tasks.length || tasks && tasks.opened && tasks.opened.tasks.length) && (state.mode === 'tasks' || state.mode === 'projects' && state.projectForSearchTasks) ? this.renderTasks(state.tasks) : '', !this.state.isFetchTasks && tasks && tasks.search && !tasks.search.tasks.length && tasks && tasks.recent && !tasks.recent.tasks.length && tasks && tasks.opened && !tasks.opened.tasks.length && (state.mode === 'tasks' || state.mode === 'projects' && state.projectForSearchTasks) ? /*#__PURE__*/React.createElement("div", {
        className: "empty-text"
      }, "No tasks found") : '', state.mode === 'projects' && !state.projectForSearchTasks && state.projects && (state.projects.recent.projects.length || state.projects.active.projects.length) ? this.renderProjects(state.projects) : '', state.mode === 'projects' && !state.projectForSearchTasks && state.projects && !state.projects.recent.projects.length && !state.projects.active.projects.length ? /*#__PURE__*/React.createElement("div", {
        className: "empty-text"
      }, "No projects found") : '', this.state.isFetchTasks ? /*#__PURE__*/React.createElement(Skeleton, {
        view: 'select-task',
        options: {
          withTitle: true
        }
      }) : ''), isShowModeSwitch ? /*#__PURE__*/React.createElement("div", {
        className: "search__mode-switch"
      }, /*#__PURE__*/React.createElement("label", {
        className: "checkbox checkbox--slider",
        onClick: this.toggleSearchMode.bind(this)
      }, /*#__PURE__*/React.createElement("span", {
        className: "slider__label"
      }, "Search in a specific project"), /*#__PURE__*/React.createElement("input", {
        type: "checkbox",
        checked: state.mode === 'projects'
      }), /*#__PURE__*/React.createElement("span", {
        className: "toggle-with-icon slider round"
      }))) : '') : '')));
    }
  }]);
}(React.Component);
{
  TasksSelectField;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var AddTimeMenu = /*#__PURE__*/function (_React$Component) {
  function AddTimeMenu(props) {
    var _this;
    _classCallCheck(this, AddTimeMenu);
    _this = _callSuper(this, AddTimeMenu, [props]);
    _this.modalService = new ModalService();
    return _this;
  }
  _inherits(AddTimeMenu, _React$Component);
  return _createClass(AddTimeMenu, [{
    key: "onToggleAddTimeMenu",
    value: function onToggleAddTimeMenu(isShowMenu) {
      this.props.onToggleAddTimeMenu && this.props.onToggleAddTimeMenu(isShowMenu);
    }
  }, {
    key: "showAddTimeModal",
    value: function showAddTimeModal() {
      this.modalService.showModal(AddTimeModal, {
        store: this.props.store,
        api: this.props.api,
        weekDate: this.props.addTimeDate,
        isShowMoreButton: true,
        isAbstract: true
      });
    }
  }, {
    key: "render",
    value: function render() {
      return /*#__PURE__*/React.createElement(Popover2, {
        triggerNode: /*#__PURE__*/React.createElement("div", {
          className: "add-time-menu"
        }, /*#__PURE__*/React.createElement("div", {
          className: "icon-plus-circle"
        })),
        trigger: "click",
        placement: "bottom-left",
        onToggle: this.onToggleAddTimeMenu.bind(this),
        onClose: this.onToggleAddTimeMenu.bind(this, false),
        closeOnClick: true,
        width: 125,
        key: "add-time-menu-".concat(this.props.addTimeDate)
      }, /*#__PURE__*/React.createElement("div", {
        className: "actions-menu"
      }, /*#__PURE__*/React.createElement("div", {
        className: "actions-menu__item",
        onClick: this.showAddTimeModal.bind(this)
      }, "Add Time...")));
    }
  }]);
}(React.Component);
{
  AddTimeMenu;
}
;
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var AddTimeModal = /*#__PURE__*/function (_React$Component) {
  function AddTimeModal(props) {
    var _this;
    _classCallCheck(this, AddTimeModal);
    _this = _callSuper(this, AddTimeModal, [props]);
    _this.state = {
      error: {
        isShowError: false,
        message: null
      },
      isFetchAddTime: false
    };
    _this.timeInputComponent = null;
    _this.isAddTimeOpened = false;
    _this.tasksSelectInstance = null;
    _this.tasksSelectRef = React.createRef();
    return _this;
  }
  _inherits(AddTimeModal, _React$Component);
  return _createClass(AddTimeModal, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      var _this2 = this;
      this.timeInputComponent = new BrowserAction.AddTimeComponent();
      if (this.props.isAbstract || this.props.isMoveTime) {
        this.timeInputComponent.isAddTimeForAbstractTask = true;
      } else {
        this.timeInputComponent.task = this.props.task;
        // this.timeInputComponent.isAddTimeForAbstractTask = true;
      }
      if (this.props.isEditTime) {
        this.timeInputComponent.isEditTime = true;
        this.timeInputComponent.taskTimeId = this.props.taskTimeId;
        this.timeInputComponent.taskId = this.props.task && this.props.task.id;
      }
      this.timeInputComponent.tasksSelectRef = this.tasksSelectRef;
      this.timeInputComponent.afterSave = function () {
        _this2.isAddTimeOpened = false;
      };
      this.timeInputComponent.onError = function (error) {
        var _error$responseJSON, _error$responseJSON2;
        if (error !== null && error !== void 0 && (_error$responseJSON = error.responseJSON) !== null && _error$responseJSON !== void 0 && (_error$responseJSON = _error$responseJSON.errors) !== null && _error$responseJSON !== void 0 && _error$responseJSON.date) {
          _this2.setState({
            error: {
              isShowError: true,
              message: error.responseJSON.errors.date
            }
          });
          return;
        }
        var message = error === null || error === void 0 || (_error$responseJSON2 = error.responseJSON) === null || _error$responseJSON2 === void 0 ? void 0 : _error$responseJSON2.message;
        if (message) {
          _this2.setState({
            error: {
              isShowError: true,
              message: message
            }
          });
        }
      };
      this.timeInputComponent.render();
      var taskSelectElement = document.getElementById('tasks-select');
      if (taskSelectElement) {
        ReactDOM.render(React.createElement(TasksSelectField, {
          api: this.props.api,
          store: this.props.store,
          reactAppStateObj: this.props.reactAppStateObj,
          onSelect: this.onSelectTask.bind(this),
          ref: this.tasksSelectRef,
          withComments: this.props.withComments,
          task: this.props.task
        }), taskSelectElement);
      }
      var _submit = this.onSubmitForm.bind(this);
      // need refactoring
      setTimeout(function () {
        var form = document.querySelector('form[name="everhourAddTimeForm"]');
        if (form) {
          form.addEventListener('submit', _submit);
          if (_this2.props.time) {
            form.querySelector('.time-input').value = TimeEntryUtils.format(_this2.props.time);
          }
          form.querySelector('.time-input').focus();
          var todayDate = moment().startOf('day')._d.valueOf();
          var yesterdayDate = moment().subtract(1, 'day').startOf('day')._d.valueOf();
          var taskTimeDate = moment(_this2.props.weekDate)._d.valueOf();
          var dateInput = form.querySelector('.add-time__date-input');
          // const formatDate = taskTimeDate === todayDate ?
          //     'Today' : taskTimeDate === yesterdayDate ? 'Yesterday' : moment(taskTimeDate).format('D MMM YYYY');
          var formatDate = taskTimeDate === todayDate ? 'Today' : moment(taskTimeDate).format('D MMM YYYY');
          dateInput.value = formatDate;
          var commentLine = form.querySelector('textarea#textarea-comment');
          var task = _this2.props.task;
          if (_this2.props.isAbstract && task && (task._type === 'comment' || task.type === 'comment') && task.comment) {
            commentLine.value = task.comment;
          }
          if (_this2.props.isTimeTab && task && (task._type === 'comment' || task.type === 'comment') && (task.comment || _this2.props.comment)) {
            commentLine.value = task.comment || _this2.props.comment;
          }
          if (_this2.props.isTimeTab && _this2.props.isEditTime && task && (task._type === 'task' || task.type === 'task') && (task.comment || _this2.props.comment)) {
            commentLine.value = task.comment || _this2.props.comment;
          }
          if (!_this2.props.task && _this2.props.taskTimeId) {
            dateInput.disabled = true;
          }
        }
      }, 100);
    }
  }, {
    key: "onSelectTask",
    value: function onSelectTask(task) {
      this.timeInputComponent.task = task;
      if (task && task._type && (task._type === 'task' || task.type === 'task') || !task) {
        this.timeInputComponent.showComment();
      }
      if (task && task._type && (task._type === 'comment' || task.type === 'comment')) {
        this.timeInputComponent.hideComment(task.comment);
      }
    }
  }, {
    key: "onSubmitForm",
    value: function onSubmitForm(buttonType) {
      var _this3 = this;
      this.setState({
        warning: {
          isShowWarning: false,
          message: null
        }
      });
      if (this.props.task || this.props.isAbstract) {
        this.setState({
          isFetchAddTime: true
        });
        this.timeInputComponent.save().then(function () {
          var timeInputElement = document.querySelector('form[name="everhourAddTimeForm"] .add-time__ctrl-input');
          if (_this3.props.isShowMoreButton && timeInputElement && buttonType === 'saveMore') {
            _this3.timeInputComponent.lockActions = false;
            _this3.setState({
              isFetchAddTime: false
            });
            timeInputElement.value = '';
            _this3.props.onSuccess && _this3.props.onSuccess({
              isSuccessEditTime: true
            });
            return;
          }
          _this3.props.onClose({
            isSuccessEditTime: true
          });
        })["catch"](function () {
          _this3.setState({
            isFetchAddTime: false
          });
        });
        return;
      }
      if (this.props.isMoveTime && this.timeInputComponent.task) {
        var formValue = this.timeInputComponent.form.value();
        var date = formValue.date;
        var time = null;
        var comment = formValue.commentLine;
        var user = this.props.store.profile.get().id;
        this.moveTaskTime(time, comment, user, date);
      }
    }
  }, {
    key: "moveTaskTime",
    value: function moveTaskTime(time, comment, user, date) {
      var _this4 = this;
      this.setState({
        isFetchAddTime: true
      });
      this.props.api.updateTaskTime(this.props.taskTimeId, {
        time: time,
        comment: comment,
        user: user,
        date: date
      }).then(function (taskTime1) {
        _this4.props.api.moveTaskTime(_this4.props.taskTimeId, {
          task: _this4.timeInputComponent.task.id
        }).then(function (taskTime2) {
          store.dispatch(TasksStore.Act.TaskTimeAppend(taskTime2));
          _this4.props.onClose({
            isSuccessEditTime: true
          });
        })["catch"](function (error) {
          _this4.parseError(error);
        });
      })["catch"](function (error) {
        _this4.parseError(error);
      });
    }
  }, {
    key: "parseError",
    value: function parseError(error) {
      var _error$responseJSON3;
      var message = error === null || error === void 0 || (_error$responseJSON3 = error.responseJSON) === null || _error$responseJSON3 === void 0 ? void 0 : _error$responseJSON3.message;
      if (message) {
        this.setState({
          isFetchAddTime: false,
          error: {
            isShowError: true,
            message: message
          }
        });
      }
    }
  }, {
    key: "render",
    value: function render() {
      return /*#__PURE__*/React.createElement("div", null, this.state.error.isShowError && /*#__PURE__*/React.createElement("div", {
        className: "modal__error"
      }, this.state.error.message), /*#__PURE__*/React.createElement("div", {
        className: "everhour-time-add-holder"
      }), /*#__PURE__*/React.createElement("div", {
        className: "modal__content-actions"
      }, /*#__PURE__*/React.createElement("button", {
        type: "button",
        name: "save",
        className: "content-actions__button content-actions__button--submit",
        disabled: this.state.isFetchAddTime,
        onClick: this.onSubmitForm.bind(this, 'save')
      }, "Save"), this.props.isShowMoreButton ? /*#__PURE__*/React.createElement("button", {
        type: "button",
        name: "saveAndSubmit",
        className: "content-actions__button content-actions__button--submit content-actions__button--submit-grey",
        disabled: this.state.isFetchAddTime,
        onClick: this.onSubmitForm.bind(this, 'saveMore')
      }, "Save & Add More") : '', /*#__PURE__*/React.createElement("button", {
        type: "button",
        className: "content-actions__button content-actions__button--close",
        onClick: this.props.onClose
      }, "Cancel")));
    }
  }]);
}(React.Component);
{
  AddTimeModal;
}
;
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var EditTimeModal = /*#__PURE__*/function (_React$Component) {
  function EditTimeModal(props) {
    var _this;
    _classCallCheck(this, EditTimeModal);
    _this = _callSuper(this, EditTimeModal, [props]);
    _this.state = {
      isFetchEditTime: true,
      error: {
        message: null,
        isShowError: false
      }
    };
    _this.fetchTime();
    browserPort.analytics.event('time_edit_open');
    return _this;
  }
  _inherits(EditTimeModal, _React$Component);
  return _createClass(EditTimeModal, [{
    key: "fetchTime",
    value: function fetchTime() {
      var _this2 = this;
      this.props.api.taskTime(this.props.task.id).then(function (response) {
        return _this2.buildTime(response);
      });
    }
  }, {
    key: "buildTime",
    value: function buildTime(times) {
      var _this3 = this;
      var timesForUserCount = 0;
      this.setState({
        times: times,
        timesForUser: times.filter(function (time) {
          if (!!~_this3.props.userIds.indexOf(time.user) && time.time) {
            timesForUserCount += time.time || 0;
            return true;
          }
          return false;
        }).sort(function (a, b) {
          if (b.date < a.date) {
            return -1;
          }
          if (b.date > a.date) {
            return 1;
          }
        }),
        timesForUserCount: timesForUserCount,
        isFetchEditTime: false
      });
      var titleElement = document.querySelector('.modal--edit-time .modal__header-title');
      if (titleElement) {
        titleElement.innerText = "Edit time \xB7 ".concat(TimeEntryUtils.format(timesForUserCount, '0h'));
      }
    }
  }, {
    key: "handleUpdateTimeDone",
    value: function handleUpdateTimeDone() {
      this.props.onUpdateTimeDone({
        isSuccessEditTime: true
      });
      this.setState({
        error: {
          isShowError: false,
          message: null
        }
      });
      this.fetchTime();
    }
  }, {
    key: "handleUpdateTimeError",
    value: function handleUpdateTimeError(error) {
      var _error$responseJSON, _error$responseJSON2;
      var message = (error === null || error === void 0 || (_error$responseJSON = error.responseJSON) === null || _error$responseJSON === void 0 || (_error$responseJSON = _error$responseJSON.errors) === null || _error$responseJSON === void 0 ? void 0 : _error$responseJSON.date) || (error === null || error === void 0 || (_error$responseJSON2 = error.responseJSON) === null || _error$responseJSON2 === void 0 ? void 0 : _error$responseJSON2.message);
      if (message) {
        this.setState({
          error: {
            isShowError: true,
            message: message
          }
        });
      }
    }
  }, {
    key: "render",
    value: function render() {
      var _this4 = this;
      var state = this.state;
      var props = this.props;
      return /*#__PURE__*/React.createElement("div", null, state.error.isShowError && /*#__PURE__*/React.createElement("div", {
        className: "modal__error"
      }, this.state.error.message), /*#__PURE__*/React.createElement("div", {
        className: "modal-edit-time"
      }, state.isFetchEditTime ? /*#__PURE__*/React.createElement(Skeleton, {
        view: 'editTime'
      }) : '', !state.isFetchEditTime && (!state.timesForUser || state.timesForUser && !state.timesForUser.length) ? /*#__PURE__*/React.createElement("div", {
        className: "modal-edit-time__empty"
      }, /*#__PURE__*/React.createElement("span", {
        className: "text-placeholder"
      }, "No time reported yet.")) : '', !state.isFetchEditTime && state.timesForUser ? state.timesForUser.map(function (history) {
        return /*#__PURE__*/React.createElement(TaskDetailsEditTimeRow, {
          task: props.task,
          history: history,
          api: props.api,
          reactAppStateObj: props.reactAppStateObj,
          store: props.store,
          onUpdateTimeDone: _this4.handleUpdateTimeDone.bind(_this4),
          onUpdateTimeError: _this4.handleUpdateTimeError.bind(_this4)
        });
      }) : ''));
    }
  }]);
}(React.Component);
{
  EditTimeModal;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var AddCommentModal = /*#__PURE__*/function (_React$Component) {
  function AddCommentModal(props) {
    var _this;
    _classCallCheck(this, AddCommentModal);
    _this = _callSuper(this, AddCommentModal, [props]);
    _this.state = {
      comment: _this.props.comment,
      error: {
        isShowError: false,
        message: null
      }
    };
    _this.timeInputComponent = null;
    _this.isAddTimeOpened = false;
    _this.commentArea = null;
    return _this;
  }
  _inherits(AddCommentModal, _React$Component);
  return _createClass(AddCommentModal, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this.commentArea.focus();
      var pos = this.state.comment && this.state.comment.length || 0;
      this.commentArea.selectionStart = pos;
    }
  }, {
    key: "onChangeComment",
    value: function onChangeComment(event) {
      this.setState({
        comment: event.target.value
      });
    }
  }, {
    key: "isValidCommentMaxLength",
    value: function isValidCommentMaxLength() {
      return !this.state.comment || this.state.comment && this.state.comment.length <= 1000;
    }
  }, {
    key: "onSubmitForm",
    value: function onSubmitForm() {
      var _this2 = this;
      var request = null;
      this.setState({
        warning: {
          isShowWarning: false,
          message: null
        }
      });
      if (!this.isValidCommentMaxLength()) {
        return;
      }
      if (this.props.task && !this.props.isCurrent) {
        request = this.updateTaskComment();
      }
      if (!this.props.task && this.props.isCurrent || this.props.task && this.props.isCurrent) {
        request = this.updateCurrentComment();
      }
      if (!this.props.task && this.props.taskTimeId) {
        request = this.updateTaskTime();
      }
      if (!request) {
        return;
      }
      request.then(function (taskTime) {
        var taskTimeIndex = _this2.props.store.taskTime.taskTime.findIndex(function (t) {
          return t.id === _this2.props.taskTimeId;
        });

        // update comment for timer with task
        if (taskTimeIndex > -1) {
          var _taskTime = _this2.props.store.taskTime.taskTime[taskTimeIndex];
          _taskTime.comment = taskTime.comment;
          if (_this2.props.isCurrent) {
            _this2.props.store.dispatch(TimerStore.Act.Updated(taskTime));
          } else {
            _this2.props.store.dispatch(TasksStore.Act.TaskTimeAppend([_taskTime]));
          }
        }

        // update comment for active timer with task for current date
        if (taskTimeIndex === -1 && _this2.props.task && _this2.props.isCurrent) {
          _this2.props.store.dispatch(TimerStore.Act.Updated(taskTime));
        }

        // update comment for timer without task
        if (taskTimeIndex === -1 && !_this2.props.task && _this2.props.isCurrent) {
          _this2.props.store.dispatch(TimerStore.Act.Updated(taskTime));
        }
        if (taskTimeIndex === -1 && !_this2.props.task && !_this2.props.isCurrent) {
          _this2.props.store.dispatch(TimerStore.Act.Updated(taskTime));
        }
        _this2.props.onClose && _this2.props.onClose({
          isSuccessEditComment: true
        });
      }).fail(function (error) {
        var _error$responseJSON, _error$responseJSON2;
        var message = (error === null || error === void 0 || (_error$responseJSON = error.responseJSON) === null || _error$responseJSON === void 0 || (_error$responseJSON = _error$responseJSON.errors) === null || _error$responseJSON === void 0 ? void 0 : _error$responseJSON.date) || (error === null || error === void 0 || (_error$responseJSON2 = error.responseJSON) === null || _error$responseJSON2 === void 0 ? void 0 : _error$responseJSON2.message);
        if (message) {
          _this2.setState({
            error: {
              isShowError: true,
              message: message
            }
          });
        }
      });
    }
  }, {
    key: "updateCurrentComment",
    value: function updateCurrentComment() {
      return this.props.api.commentTimer({
        comment: this.state.comment || ''
      });
    }
  }, {
    key: "updateTaskComment",
    value: function updateTaskComment() {
      return this.props.api.updateTaskTime(this.props.taskTimeId, {
        comment: this.state.comment || '',
        date: this.props.weekDate || moment().format('YYYY-MM-DD'),
        user: this.props.userId || store.profile.get().id
      });
    }
  }, {
    key: "updateTaskTime",
    value: function updateTaskTime() {
      return this.props.api.updateTaskTime(this.props.taskTimeId, {
        comment: this.state.comment || '',
        date: this.props.weekDate || moment().format('YYYY-MM-DD'),
        user: this.props.userId || store.profile.get().id
      });
    }
  }, {
    key: "render",
    value: function render() {
      var _this3 = this;
      return /*#__PURE__*/React.createElement("div", null, this.state.error.isShowError && /*#__PURE__*/React.createElement("div", {
        className: "modal__error"
      }, this.state.error.message), /*#__PURE__*/React.createElement("div", {
        className: "add-comment"
      }, /*#__PURE__*/React.createElement("div", {
        className: "field__wrapper"
      }, /*#__PURE__*/React.createElement("label", {
        className: "field__label",
        htmlFor: "textarea-comment"
      }, "Comments"), /*#__PURE__*/React.createElement("div", {
        className: "field__body ".concat(this.isValidCommentMaxLength() ? '' : 'everhour-error'),
        "data-error": this.isValidCommentMaxLength() ? '' : '1000 MAX'
      }, /*#__PURE__*/React.createElement("textarea", {
        id: "textarea-comment",
        className: "comment field__textarea add-time__comment-textarea\n                                  ".concat(this.isValidCommentMaxLength() ? '' : 'everhour-error'),
        name: "commentLine",
        placeholder: "Explain your progress\u2026",
        ref: function ref(input) {
          _this3.commentArea = input;
        },
        onChange: this.onChangeComment.bind(this),
        value: this.state.comment
      })))), /*#__PURE__*/React.createElement("div", {
        className: "modal__content-actions"
      }, /*#__PURE__*/React.createElement("button", {
        type: "button",
        className: "content-actions__button content-actions__button--submit",
        onClick: this.onSubmitForm.bind(this)
      }, "Save"), /*#__PURE__*/React.createElement("button", {
        type: "button",
        className: "content-actions__button content-actions__button--close",
        onClick: this.props.onClose
      }, "Cancel")));
    }
  }]);
}(React.Component);
{
  AddCommentModal;
}
;
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var AssignTimeModal = /*#__PURE__*/function (_React$Component) {
  function AssignTimeModal(props) {
    var _this;
    _classCallCheck(this, AssignTimeModal);
    _this = _callSuper(this, AssignTimeModal, [props]);
    _this.state = {
      task: null,
      time: {
        value: null,
        isValid: true
      },
      comment: _this.props.comment,
      error: {
        isShowError: false,
        message: null
      }
    };
    _this.tasksSelectRef = React.createRef();
    _this.popoverControllerService = new PopupControllerService();
    return _this;
  }
  _inherits(AssignTimeModal, _React$Component);
  return _createClass(AssignTimeModal, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      var time = TimeEntryUtils.format(this.props.time);
      this.setState({
        time: {
          value: time ? time.replace('-', '') : null,
          isValid: true
        }
      });
      this.tasksSelectRef.current.toggleSearch(true);
    }
  }, {
    key: "getTimeMillis",
    value: function getTimeMillis(value) {
      return TimeEntryUtils.parseTimeToSeconds(value || this.state.time.value || null);
    }
  }, {
    key: "checkTimeValid",
    value: function checkTimeValid(value) {
      var millis = this.getTimeMillis(value);
      return !(millis === null || !millis || millis > ContentScript.Constants.DAILY_TIME);
    }
  }, {
    key: "onChangeComment",
    value: function onChangeComment(event) {
      this.setState({
        comment: event.target.value
      });
    }
  }, {
    key: "onChangeTimeValue",
    value: function onChangeTimeValue(event) {
      var timeValue = event.target.value;
      this.setState({
        time: {
          value: timeValue,
          isValid: this.checkTimeValid(timeValue)
        }
      });
    }
  }, {
    key: "onSubmitForm",
    value: function onSubmitForm() {
      var isCurrent = this.props.isCurrent;
      this.setState({
        warning: {
          isShowWarning: false,
          message: null
        }
      });
      var task = this.props.task || this.state.task;
      var isValid = this.state.time.isValid;
      if (!task) {
        this.tasksSelectRef.current.toggleError('Required');
      }
      if (!this.checkTimeValid()) {
        this.setState({
          time: Object.assign({}, this.state.time, {
            isValid: false
          })
        });
        isValid = false;
      }
      if (!task || !isValid) {
        return;
      }
      if (isCurrent) {
        this.assignTimeToCurrent();
        return;
      }
      this.assignTimeToTask();
    }
  }, {
    key: "assignTimeToCurrent",
    value: function assignTimeToCurrent() {
      var _this2 = this;
      return this.props.api.commentTimer({
        comment: this.state.comment || '',
        task: this.state.task.id
      }).then(function (taskTime) {
        var taskTimeIndex = _this2.props.store.taskTime.taskTime.findIndex(function (t) {
          return t.id === _this2.props.taskTimeId;
        });
        if (taskTimeIndex === -1 && _this2.props.isCurrent) {
          _this2.props.store.dispatch(TimerStore.Act.Updated(taskTime));
        }
        _this2.props.onClose();
      }).fail(function (err) {
        return _this2.handleError(err);
      });
    }
  }, {
    key: "assignTimeToTask",
    value: function assignTimeToTask() {
      var _this3 = this;
      return this.props.api.updateTaskTime(this.props.taskTimeId, {
        time: this.getTimeMillis(),
        comment: this.state.comment || null,
        task: this.state.task.id
      }).then(function (taskTime) {
        taskTime.task.project = _this3.popoverControllerService.taskProject(taskTime.task.projects, taskTime.task.projectsObjects);
        _this3.props.store.dispatch(TasksStore.Act.TaskTimeAppend([taskTime]));
        _this3.props.onClose();
      }).fail(function (err) {
        return _this3.handleError(err);
      });
    }
  }, {
    key: "handleError",
    value: function handleError(error) {
      var _error$responseJSON;
      var message = error === null || error === void 0 || (_error$responseJSON = error.responseJSON) === null || _error$responseJSON === void 0 ? void 0 : _error$responseJSON.message;
      if (message) {
        this.setState({
          error: {
            isShowError: true,
            message: message
          }
        });
      }
    }
  }, {
    key: "onSelectTask",
    value: function onSelectTask(task) {
      this.setState({
        task: task
      });
    }
  }, {
    key: "resetTime",
    value: function resetTime() {
      this.setState({
        time: {
          value: '',
          isValid: this.checkTimeValid()
        }
      });
    }
  }, {
    key: "render",
    value: function render() {
      return /*#__PURE__*/React.createElement("div", null, this.state.error.isShowError && /*#__PURE__*/React.createElement("div", {
        className: "modal__error"
      }, this.state.error.message), /*#__PURE__*/React.createElement("div", {
        className: "assign-time"
      }, /*#__PURE__*/React.createElement("div", {
        className: "assign-time__row"
      }, /*#__PURE__*/React.createElement(TasksSelectField, {
        api: this.props.api,
        store: this.props.store,
        reactAppStateObj: this.props.reactAppStateObj,
        onSelect: this.onSelectTask.bind(this),
        ref: this.tasksSelectRef,
        withComments: this.props.withComments
      })), /*#__PURE__*/React.createElement("div", {
        className: "assign-time__row assign-time__group"
      }, !this.props.isCurrent ? /*#__PURE__*/React.createElement("div", {
        className: "field__wrapper field__wrapper--assign-time"
      }, /*#__PURE__*/React.createElement("label", {
        className: "field__label",
        htmlFor: "input-time"
      }, "Time"), /*#__PURE__*/React.createElement("div", {
        className: "field__body"
      }, /*#__PURE__*/React.createElement("input", {
        className: "field__input ".concat(!this.state.time.isValid ? 'everhour-error' : ''),
        disabled: this.props.isCurrent,
        type: "text",
        onChange: this.onChangeTimeValue.bind(this),
        value: this.state.time.value
      }), /*#__PURE__*/React.createElement("div", {
        className: "icon-delete",
        onClick: this.resetTime.bind(this)
      }))) : '', /*#__PURE__*/React.createElement("div", {
        className: "".concat(this.props.isCurrent ? 'field__wrapper' : 'field__wrapper field__wrapper--assign-comment')
      }, /*#__PURE__*/React.createElement("label", {
        className: "field__label",
        htmlFor: "input-comment"
      }, "Comments"), /*#__PURE__*/React.createElement("div", {
        className: "field__body"
      }, /*#__PURE__*/React.createElement("input", {
        className: "field__input",
        type: "text",
        id: "input-comment",
        placeholder: "Explain your progress\u2026",
        onChange: this.onChangeComment.bind(this),
        value: this.state.comment
      }))))), /*#__PURE__*/React.createElement("div", {
        className: "modal__content-actions"
      }, /*#__PURE__*/React.createElement("button", {
        type: "button",
        className: "content-actions__button content-actions__button--submit",
        onClick: this.onSubmitForm.bind(this)
      }, "Save"), /*#__PURE__*/React.createElement("button", {
        type: "button",
        className: "content-actions__button content-actions__button--close",
        onClick: this.props.onClose
      }, "Cancel")), /*#__PURE__*/React.createElement("div", {
        className: "modal__warning"
      }, "Please note that this time without a task will be excluded from team reports. So no one except you will see it."));
    }
  }]);
}(React.Component);
{
  AssignTimeModal;
}
;
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var DeleteTimeModal = /*#__PURE__*/function (_React$Component) {
  function DeleteTimeModal(props) {
    var _this;
    _classCallCheck(this, DeleteTimeModal);
    _this = _callSuper(this, DeleteTimeModal, [props]);
    _this.state = {
      error: {
        isShowError: false,
        message: null
      }
    };
    return _this;
  }
  _inherits(DeleteTimeModal, _React$Component);
  return _createClass(DeleteTimeModal, [{
    key: "getCurrentUser",
    value: function getCurrentUser() {
      return this.props.store.profile.get().id;
    }
  }, {
    key: "deleteTaskTime",
    value: function deleteTaskTime() {
      var props = this.props;
      return props.api.deleteTaskTime(props.taskTimeId);
    }
  }, {
    key: "onSubmitForm",
    value: function onSubmitForm() {
      var _this2 = this;
      this.setState({
        warning: {
          isShowWarning: false,
          message: null
        }
      });
      this.deleteTaskTime().then(function (taskTime) {
        _this2.props.store.taskTime.deleteTime(_this2.props.taskTimeId);
        _this2.props.store.timer.calculateAllVirtualProperties();
        _this2.props.onClose();
        _this2.props.store.dispatch(TasksStore.Act.TaskTimeAppend([taskTime]));
        browserPort.eventDispatcher.broadcast('browser-button', 'websocket-update');
      }).fail(function (error) {
        var _error$responseJSON;
        var message = error === null || error === void 0 || (_error$responseJSON = error.responseJSON) === null || _error$responseJSON === void 0 ? void 0 : _error$responseJSON.message;
        if (message) {
          _this2.setState({
            error: {
              isShowError: true,
              message: message
            }
          });
        }
      });
    }
  }, {
    key: "render",
    value: function render() {
      return /*#__PURE__*/React.createElement("div", null, this.state.error.isShowError && /*#__PURE__*/React.createElement("div", {
        className: "modal__error"
      }, this.state.error.message), /*#__PURE__*/React.createElement("div", {
        className: "delete-time-modal"
      }, "Are you sure you want to delete this time?"), /*#__PURE__*/React.createElement("div", {
        className: "modal__content-actions"
      }, /*#__PURE__*/React.createElement("button", {
        type: "button",
        className: "content-actions__button content-actions__button--submit-red",
        onClick: this.onSubmitForm.bind(this)
      }, "Delete"), /*#__PURE__*/React.createElement("button", {
        type: "button",
        className: "content-actions__button content-actions__button--close",
        onClick: this.props.onClose
      }, "Cancel")));
    }
  }]);
}(React.Component);
{
  DeleteTimeModal;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TaskTimeInput = /*#__PURE__*/function (_React$Component) {
  function TaskTimeInput(props) {
    var _this;
    _classCallCheck(this, TaskTimeInput);
    _this = _callSuper(this, TaskTimeInput, [props]);
    _this.state = {
      inputValue: TimeEntryUtils.format(_this.props.taskTime.time),
      isInputValid: true
    };
    _this.timeInput = null;
    _this.isSendRequest = false;
    return _this;
  }
  _inherits(TaskTimeInput, _React$Component);
  return _createClass(TaskTimeInput, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this.timeInput.focus();
    }
  }, {
    key: "onChangeValue",
    value: function onChangeValue(value) {
      var _this2 = this;
      this.setState({
        inputValue: value
      });
      var timerId = setTimeout(function () {
        _this2.setState({
          isInputValid: _this2.checkValid(true)
        });
        clearTimeout(timerId);
      });
    }
  }, {
    key: "onKeyDown",
    value: function onKeyDown(event) {
      if (event.keyCode === 27) {
        this.props.hideTimeInput();
      }
      if (event.keyCode === 13) {
        this.saveTime();
      }
    }
  }, {
    key: "onBlurInput",
    value: function onBlurInput() {
      this.saveTime();
    }
  }, {
    key: "getTimeMillis",
    value: function getTimeMillis() {
      return this.state.inputValue ? TimeEntryUtils.parseTimeToSeconds(this.state.inputValue) : 0;
    }
  }, {
    key: "checkValid",
    value: function checkValid(allowEmpty) {
      var millis = this.getTimeMillis();
      return !(millis === null || !millis && !allowEmpty || millis > ContentScript.Constants.DAILY_TIME);
    }
  }, {
    key: "saveTime",
    value: function saveTime() {
      var _this3 = this;
      if (!this.checkValid(true)) {
        this.setState({
          isInputValid: true
        });
        this.props.hideTimeInput();
        this.isSendRequest = false;
        return null;
      }
      var time = this.getTimeMillis();
      var taskTime = this.props.taskTime;
      if (taskTime.time === time) {
        this.props.hideTimeInput();
        this.isSendRequest = false;
        return null;
      }
      this.isSendRequest = true;
      var store = this.props.store;
      var taskTimeEntries = {
        date: taskTime.date,
        time: time,
        // task: taskTime.task ? taskTime.task.id : null,
        user: {
          id: this.props.userId || store.profile.get().id
        }
      };
      browserPort.analytics.event('time_edited');
      this.props.api.updateTaskTime(taskTime.id, taskTimeEntries).done(function (taskTime) {
        _this3.props.store.dispatch(TasksStore.Act.TaskTimeAppend([taskTime]));
        _this3.props.hideTimeInput();
        _this3.props.onUpdateTimeDone && _this3.props.onUpdateTimeDone();
        _this3.isSendRequest = false;
      }).fail(function (error) {
        _this3.broadcastError(error);
        _this3.isSendRequest = false;
      });
    }
  }, {
    key: "broadcastError",
    value: function broadcastError(error) {
      this.setState({
        isInputValid: false
      });
      if (this.props.onUpdateTimeError) {
        this.props.onUpdateTimeError(error);
        return null;
      }
      browserPort.eventDispatcher.broadcast('browser-button', 'render-errors', error);
    }
  }, {
    key: "render",
    value: function render() {
      var _this4 = this;
      return /*#__PURE__*/React.createElement("input", {
        ref: function ref(input) {
          _this4.timeInput = input;
        },
        type: "text",
        className: "task-row__time-input ".concat(!this.state.isInputValid ? 'task-row__time-input--invalid' : ''),
        onBlur: this.onBlurInput.bind(this),
        onChange: function onChange(e) {
          return _this4.onChangeValue(e.target.value);
        },
        onKeyDown: function onKeyDown(e) {
          return _this4.onKeyDown(e);
        },
        value: this.state.inputValue
      });
    }
  }]);
}(React.Component);
{
  TaskTimeInput;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TaskTimer = /*#__PURE__*/function (_React$Component) {
  function TaskTimer(props) {
    var _this;
    _classCallCheck(this, TaskTimer);
    _this = _callSuper(this, TaskTimer, [props]);
    _this.state = {
      timerButtonText: 'Start Timer',
      duration: null
    };
    _this.popupControllerService = new PopupControllerService();
    // console.log('[TaskTimer.js] props: ', this);
    return _this;
  }
  _inherits(TaskTimer, _React$Component);
  return _createClass(TaskTimer, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      var _this2 = this;
      this.handleUpdateTimer();
      browserPort.eventDispatcher.on('browser-button', 'timer-tick', function () {
        _this2.handleUpdateTimer();
      });
    }
  }, {
    key: "isTimecardsEnabled",
    value: function isTimecardsEnabled() {
      return this.props.store.profile.isApplicationEnabled('timecards');
    }
  }, {
    key: "handleUpdateTimer",
    value: function handleUpdateTimer() {
      var props = this.props;
      this.setState({
        duration: props.isRunningTimer ? props.store.timer.duration() : null
      });
    }
  }, {
    key: "stopTimer",
    value: function stopTimer(event) {
      var _this3 = this;
      event.preventDefault();
      event.stopPropagation();
      this.props.reactAppStateObj.lockActions = true;
      this.props.store.timer.stop().done(function () {
        _this3.props.reactAppStateObj.lockActions = false;
        _this3.setState({
          duration: null
        });
        _this3.props.onStopTimer && _this3.props.onStopTimer();
      }).fail(function (response) {
        return browserPort.eventDispatcher.broadcast('browser-button', 'render-errors', response);
      });
    }
  }, {
    key: "startTimer",
    value: function startTimer(event) {
      var _this4 = this;
      event.preventDefault();
      event.stopPropagation();

      // const taskTime = this.props.taskTime;
      //
      // const taskId = taskTime && taskTime.task && taskTime.task.id ? taskTime.task.id :  null;
      // const taskTimeId = !taskId && taskTime && taskTime.date === moment().format('YYYY-MM-DD') ? taskTime.id : null;
      // const website = taskTime && taskTime.website ? taskTime.website : null;
      //
      // let comment = null;
      //
      // if (this.props.comment) {
      //     comment = this.props.comment;
      // }
      //
      // if (!comment && !taskTimeId && !taskId && taskTime && taskTime.comment) {
      //     comment = taskTime.comment;
      // }
      //
      // if (!comment && !taskTime && !this.props.hasOwnProperty('comment')
      //     || !comment && taskTimeId && !taskId && taskTime && !taskTime.comment
      //     || !comment && !taskTime && this.props.hasOwnProperty('comment') && !this.props.comment) {
      //     comment = '';
      // }
      var props = this.props;
      var taskId = props.taskId ? props.taskId : null;
      var website = props.website ? props.website : null;
      var comment = props.comment ? props.comment : null;
      if (this.props.reactAppStateObj.lockActions) {
        return;
      }
      this.props.reactAppStateObj.lockActions = true;
      this.props.store.timer.start(taskId, null, null, comment, website).done(function (timer) {
        _this4.props.reactAppStateObj.lockActions = false;
        if (_this4.isTimecardsEnabled()) {
          var todayTimecard = _this4.props.reactAppStateObj.todayTimecard;
          var isToggleNeeded = !todayTimecard || todayTimecard && timer.timecard && todayTimecard.clockOut !== timer.timecard.clockOut;
          if (timer.timecard) {
            _this4.props.reactAppStateObj.todayTimecard = timer.timecard;
          }
          if (isToggleNeeded) {
            browserPort.eventDispatcher.broadcast('browser-button', 'toggle-clockin-class-state');
          }
        }
        _this4.props.onStartTimer && _this4.props.onStartTimer();
      }).fail(function (response) {
        return browserPort.eventDispatcher.broadcast('browser-button', 'render-errors', response);
      });
    }
  }, {
    key: "render",
    value: function render() {
      var props = this.props;
      var state = this.state;
      var isRunningTimer = props.isRunningTimer;
      if (!isRunningTimer) {
        return /*#__PURE__*/React.createElement("button", {
          type: "button",
          className: "timer timer-v2 timer--start",
          onClick: this.startTimer.bind(this)
        }, /*#__PURE__*/React.createElement("span", {
          className: "timer-text"
        }, state.timerButtonText));
      }
      if (props.isCircleMode) {
        return /*#__PURE__*/React.createElement("button", {
          type: "button",
          className: "timer timer-v2 timer--running timer--circle",
          onClick: this.stopTimer.bind(this)
        });
      }
      return /*#__PURE__*/React.createElement("button", {
        type: "button",
        className: "timer timer-v2 timer--running",
        onClick: this.stopTimer.bind(this)
      }, /*#__PURE__*/React.createElement("span", {
        className: "timer-icon"
      }), /*#__PURE__*/React.createElement("span", {
        className: "timer-text timer-counter"
      }, TimeEntryUtils.formatTimeSpendSeconds(state.duration)));
    }
  }]);
}(React.Component);
{
  TaskTimer;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TaskRowDescription = /*#__PURE__*/function (_React$Component) {
  function TaskRowDescription(props) {
    var _this;
    _classCallCheck(this, TaskRowDescription);
    _this = _callSuper(this, TaskRowDescription, [props]);
    _this.state = {
      isShowComment: false
    };
    _this.modalService = new ModalService();
    _this.popoverService = new Popover2Service();
    return _this;
  }
  _inherits(TaskRowDescription, _React$Component);
  return _createClass(TaskRowDescription, [{
    key: "showAddCommentModal",
    value: function showAddCommentModal(event) {
      event.preventDefault();
      event.stopPropagation();
      this.popoverService.closeAll();
      this.modalService.showModal(AddCommentModal, {
        task: this.props.task,
        comment: this.props.comment,
        api: this.props.api,
        store: this.props.store,
        isCurrent: this.props.isCurrent,
        taskTimeId: this.props.taskTimeId
      });
    }
  }, {
    key: "openAssignTimeModal",
    value: function openAssignTimeModal(event) {
      event.preventDefault();
      event.stopPropagation();
      this.popoverService.closeAll();
      this.modalService.showModal(AssignTimeModal, {
        taskTimeId: this.props.taskTimeId,
        time: this.props.time,
        comment: this.props.comment,
        api: this.props.api,
        store: this.props.store,
        isCurrent: this.props.isCurrent
      });
    }
  }, {
    key: "handleDescriptionTopClick",
    value: function handleDescriptionTopClick(event) {
      if (this.props.isNotClikableDescription) {
        return;
      }
      event.preventDefault();
      event.stopPropagation();
      var timer = this.props.store.timer.get();
      var task = this.props.task;
      if (timer && timer.status === 'active' && !timer.task && task && this.props.isSearchTab) {
        var requestBody = {};
        if (task && task.id) {
          requestBody.taskId = task.id;
        }
        if (task._type === 'comment' || task.type === 'comment') {
          requestBody.comment = task.comment;
        }
        browserPort.eventDispatcher.broadcast('browser-button', 'select-task', requestBody);
        return;
      }
      var taskId = this.props.task && this.props.task.id;
      if (!taskId) {
        return null;
      }
      browserPort.eventDispatcher.broadcast('browser-button', 'change-tab', {
        tab: 'taskDetails',
        taskId: taskId
      });
    }
  }, {
    key: "handleToggleComment",
    value: function handleToggleComment(isShow, event) {
      event && event.preventDefault();
      event && event.stopPropagation();
      if (this.props.isSearchTab || this.props.isNotClikableDescription || !this.props.isCanEditComment && !this.props.task) {
        return;
      }
      this.setState({
        isShowComment: isShow
      });
    }
  }, {
    key: "render",
    value: function render() {
      var state = this.state;
      var task = this.props.task;
      var comment = this.props.comment;
      var isTask = task && task._type !== 'comment';
      return /*#__PURE__*/React.createElement("div", {
        className: "task-row__description"
      }, /*#__PURE__*/React.createElement("div", {
        className: "task-row__description-top",
        onClick: this.handleDescriptionTopClick.bind(this)
      }, isTask ? /*#__PURE__*/React.createElement("span", {
        className: "task-row__description-name",
        style: {
          flex: task.name.length < 30 ? '0 0 auto' : '0 1 auto'
        }
      }, task.name ? /*#__PURE__*/React.createElement(HighlightText, {
        searchTerm: this.props.searchTerm
      }, task.name) : /*#__PURE__*/React.createElement("span", {
        className: "task-row__description-name task-row__description-name--empty"
      }, "Task without title")) : /*#__PURE__*/React.createElement(React.Fragment, null, state.isShowComment ? /*#__PURE__*/React.createElement(TaskCommentInput, {
        isCurrent: this.props.isCurrent,
        taskTimeId: this.props.isCurrent ? null : this.props.taskTimeId,
        comment: comment,
        onHideCommentInput: this.handleToggleComment.bind(this, false),
        weekDate: this.props.weekDate,
        api: this.props.api,
        store: this.props.store
      }) : /*#__PURE__*/React.createElement("span", {
        className: "task-row__comment-static\n                                                    ".concat(!comment ? 'task-row__comment-static--without-comment' : '', "\n                                                    ").concat(this.props.isSearchTab || this.props.isNotClikableDescription || !this.props.isCanEditComment ? 'task-row__comment-static--disabled' : ''),
        onClick: this.handleToggleComment.bind(this, true)
      }, comment || 'No details')), task && task.parentId && /*#__PURE__*/React.createElement("span", {
        className: "task-row__description-parent"
      }, task.parentName ? task.parentName : 'Task without title'), task && task.number && task.url && !this.props.isNotClikableDescription && /*#__PURE__*/React.createElement("span", {
        className: "task-row__description-number"
      }, /*#__PURE__*/React.createElement("a", {
        href: task.url,
        className: "external2 task-number"
      }, "#", task.number)), task && task.number && task.url && this.props.isNotClikableDescription && /*#__PURE__*/React.createElement("span", {
        className: "task-row__description-number task-number"
      }, "#", task.number), task && task.number && !task.url && /*#__PURE__*/React.createElement("span", {
        className: "task-row__description-number task-number"
      }, "#", task.number), isTask && comment && /*#__PURE__*/React.createElement("span", {
        className: "task-row__description-comment",
        onClick: this.showAddCommentModal.bind(this)
      })), /*#__PURE__*/React.createElement("div", {
        className: "task-row__description-bottom"
      }, isTask && task.project && /*#__PURE__*/React.createElement("div", {
        className: "task-row__description-project"
      }, task.project.name), isTask && !task.project && /*#__PURE__*/React.createElement("div", {
        className: "task-row__description-placeholder"
      }, "Without project")));
    }
  }]);
}(React.Component);
{
  TaskRowDescription;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TaskStatus = /*#__PURE__*/function (_React$Component) {
  function TaskStatus(props) {
    var _this;
    _classCallCheck(this, TaskStatus);
    _this = _callSuper(this, TaskStatus, [props]);
    _this.popoverControllerService = new PopupControllerService();
    _this.state = {
      status: _this.transformStatus()
    };
    return _this;
  }
  _inherits(TaskStatus, _React$Component);
  return _createClass(TaskStatus, [{
    key: "transformStatus",
    value: function transformStatus() {
      var task = this.props.task;
      if (!task) {
        return;
      }
      return this.popoverControllerService.transformStatus(task);
    }
  }, {
    key: "render",
    value: function render() {
      var task = this.props.task;
      if (!task) {
        return /*#__PURE__*/React.createElement("div", {
          className: "task-row__status"
        });
      }
      return /*#__PURE__*/React.createElement("div", {
        className: "task-row__status ".concat(this.state.status ? "task-row__status--".concat(this.state.status) : '')
      });
    }
  }]);
}(React.Component);
{
  TaskStatus;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TaskCommentInput = /*#__PURE__*/function (_React$Component) {
  function TaskCommentInput(props) {
    var _this;
    _classCallCheck(this, TaskCommentInput);
    _this = _callSuper(this, TaskCommentInput, [props]);
    _this.state = {
      inputValue: _this.props.comment,
      isInputValid: true
    };
    _this.commentInput = null;
    return _this;
  }
  _inherits(TaskCommentInput, _React$Component);
  return _createClass(TaskCommentInput, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      if (!this.props.isMultiLine) {
        this.commentInput.focus();
      }
    }
  }, {
    key: "onBlurInput",
    value: function onBlurInput() {
      this.saveComment();
    }
  }, {
    key: "onChangeValue",
    value: function onChangeValue(event) {
      this.setState({
        inputValue: event.target.value
      });
    }
  }, {
    key: "onKeyDown",
    value: function onKeyDown(event) {
      if (event.keyCode === 27) {
        this.hideCommentInput();
      }
      if (event.keyCode === 13) {
        event.preventDefault();
        event.stopPropagation();
        this.saveComment();
      }
    }
  }, {
    key: "hideCommentInput",
    value: function hideCommentInput() {
      this.setState({
        isInputValid: true
      });
      this.props.onHideCommentInput && this.props.onHideCommentInput();
      if (this.props.isMultiLine) {
        this.commentInput && this.commentInput.blur();
      }
    }
  }, {
    key: "saveComment",
    value: function saveComment() {
      var _this2 = this;
      var comment = this.props.isMultiLine ? this.commentInput && this.commentInput.innerText.trim() : this.state.inputValue;
      var isTaskTimeComment = !!this.props.taskTimeId;
      var isCurrent = this.props.isCurrent;
      if (this.props.comment === comment) {
        this.hideCommentInput();
        return;
      }
      var request = null;
      if (isTaskTimeComment) {
        request = this.saveTaskTimeComment();
      } else {
        request = this.saveCurrentComment();
      }
      request.then(function (response) {
        if (!isTaskTimeComment || isCurrent) {
          if (response && response.currentTaskTime) {
            response.currentTaskTime.comment = response.comment;
          }
          _this2.props.store.dispatch(TimerStore.Act.Updated(response));
        }
        if (isTaskTimeComment && !isCurrent) {
          var taskTimeIndex = _this2.props.store.taskTime.taskTime.findIndex(function (t) {
            return t.id === _this2.props.taskTimeId;
          });
          var _taskTime = _this2.props.store.taskTime.taskTime[taskTimeIndex];
          _taskTime.comment = response.comment;
          _this2.props.store.dispatch(TasksStore.Act.TaskTimeAppend([_taskTime]));
        }
        _this2.hideCommentInput();
      })["catch"](function (err) {
        _this2.broadcastError({
          responseJSON: err
        });
      });
    }
  }, {
    key: "getComment",
    value: function getComment() {
      return this.props.isMultiLine ? this.commentInput && this.commentInput.innerText.trim() : this.state.inputValue || '';
    }
  }, {
    key: "saveTaskTimeComment",
    value: function saveTaskTimeComment() {
      return this.props.api.updateTaskTime(this.props.taskTimeId, {
        comment: this.getComment()
      });
    }
  }, {
    key: "saveCurrentComment",
    value: function saveCurrentComment() {
      return this.props.api.commentTimer({
        comment: this.getComment(),
        date: this.props.weekDate || moment().format('YYYY-MM-DD'),
        user: this.props.userId || store.profile.get().id
      });
    }
  }, {
    key: "broadcastError",
    value: function broadcastError(error) {
      this.setState({
        isInputValid: false
      });
      if (this.props.onUpdateCommentError) {
        this.props.onUpdateCommentError(error);
        return null;
      }
      browserPort.eventDispatcher.broadcast('browser-button', 'render-errors', error);
    }
  }, {
    key: "render",
    value: function render() {
      var _this3 = this;
      if (this.props.isMultiLine) {
        return /*#__PURE__*/React.createElement("div", {
          contentEditable: true,
          ref: function ref(input) {
            _this3.commentInput = input;
          },
          className: "task-row__comment-input task-row__comment-input--multiline",
          onBlur: this.onBlurInput.bind(this)
          // onInput={this.onChangeValue.bind(this)}
          ,
          onKeyDown: this.onKeyDown.bind(this)
        }, this.state.inputValue);
      }
      return /*#__PURE__*/React.createElement("input", {
        ref: function ref(input) {
          _this3.commentInput = input;
        },
        maxLength: 1000,
        placeholder: 'No details',
        type: "text",
        className: "task-row__comment-input ".concat(!this.state.isInputValid ? 'task-row__comment-input--invalid' : ''),
        onBlur: this.onBlurInput.bind(this),
        onChange: this.onChangeValue.bind(this),
        onKeyDown: this.onKeyDown.bind(this),
        value: this.state.inputValue
      });
    }
  }]);
}(React.Component);
{
  TaskCommentInput;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var WebsiteIcon = /*#__PURE__*/function (_React$Component) {
  function WebsiteIcon(props) {
    _classCallCheck(this, WebsiteIcon);
    return _callSuper(this, WebsiteIcon, [props]);
  }
  _inherits(WebsiteIcon, _React$Component);
  return _createClass(WebsiteIcon, [{
    key: "getWebsiteIcon",
    value: function getWebsiteIcon() {
      var website = this.props.website;
      if (website.url.indexOf('docs.google.com/document') !== -1) {
        return 'google-docs';
      }
      if (website.url.indexOf('docs.google.com/spreadsheets') !== -1) {
        return 'google-sheets';
      }
      if (website.url.indexOf('docs.google.com/presentation') !== -1) {
        return 'google-slides';
      }
      if (website.url.indexOf('mail.google.com') !== -1) {
        return 'google-mail';
      }
      return website.domain.split('.')[0];
    }
  }, {
    key: "render",
    value: function render() {
      return /*#__PURE__*/React.createElement("div", {
        className: "website-icon website-icon--".concat(this.getWebsiteIcon())
      });
    }
  }]);
}(React.Component);
{
  WebsiteIcon;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TaskRowMenu = /*#__PURE__*/function (_React$Component) {
  function TaskRowMenu(props) {
    var _this;
    _classCallCheck(this, TaskRowMenu);
    _this = _callSuper(this, TaskRowMenu, [props]);
    _this.modalService = new ModalService();
    return _this;
  }
  _inherits(TaskRowMenu, _React$Component);
  return _createClass(TaskRowMenu, [{
    key: "showAddTimeModal",
    value: function showAddTimeModal() {
      this.modalService.showModal(AddTimeModal, {
        task: this.props.task || {
          _type: 'comment',
          comment: this.props.comment
        },
        store: this.props.store,
        api: this.props.api,
        taskTimeId: this.props.taskTimeId,
        weekDate: this.props.weekDate,
        isTimeTab: true,
        withComments: true,
        isShowMoreButton: true
      });
    }
  }, {
    key: "showAddCommentModal",
    value: function showAddCommentModal() {
      this.modalService.showModal(AddCommentModal, {
        task: this.props.task,
        comment: this.props.comment,
        api: this.props.api,
        store: this.props.store,
        isCurrent: this.props.isCurrent,
        taskTimeId: this.props.taskTimeId,
        weekDate: this.props.weekDate,
        userId: this.props.userId || null
      });
    }
  }, {
    key: "showDeleteTimeModal",
    value: function showDeleteTimeModal() {
      this.modalService.showModal(DeleteTimeModal, {
        task: this.props.task,
        api: this.props.api,
        store: this.props.store,
        taskTimeId: this.props.taskTimeId,
        weekDate: this.props.weekDate,
        userId: this.props.userId || null
      }, {
        title: 'Delete Time'
      });
    }
  }, {
    key: "showAssignTaskModal",
    value: function showAssignTaskModal() {
      this.modalService.showModal(AssignTimeModal, {
        task: this.props.task,
        api: this.props.api,
        store: this.props.store,
        isCurrent: this.props.isCurrent,
        time: this.props.time,
        taskTimeId: this.props.taskTimeId,
        weekDate: this.props.weekDate,
        userId: this.props.userId || null,
        withsComments: false,
        comment: this.props.comment
      }, {
        title: 'Select Task'
      });
    }
  }, {
    key: "showEditTimeModal",
    value: function showEditTimeModal() {
      this.modalService.showModal(AddTimeModal, {
        task: this.props.task || {
          _type: 'comment',
          comment: this.props.comment
        },
        store: this.props.store,
        api: this.props.api,
        taskTimeId: this.props.taskTimeId,
        weekDate: this.props.weekDate,
        isTimeTab: true,
        isEditTime: true,
        withComments: true,
        time: this.props.time,
        comment: this.props.comment
      }, {
        title: 'Edit time'
      });
    }
  }, {
    key: "onToggleMenu",
    value: function onToggleMenu(isShowMenu) {
      this.props.onToggleMenu(isShowMenu);
    }
  }, {
    key: "render",
    value: function render() {
      if (this.props.isCurrent && !this.props.task) {
        return null;
      }
      return /*#__PURE__*/React.createElement(Popover2, {
        triggerNode: /*#__PURE__*/React.createElement("div", {
          className: "task-row__actions-menu"
        }, /*#__PURE__*/React.createElement("div", {
          className: "icon-menu"
        })),
        trigger: "click",
        placement: "bottom-right",
        onToggle: this.onToggleMenu.bind(this),
        onClose: this.onToggleMenu.bind(this, false),
        closeOnClick: true,
        width: 125
      }, this.props.isCurrent ? /*#__PURE__*/React.createElement("div", {
        className: "actions-menu"
      }, /*#__PURE__*/React.createElement("div", {
        className: "actions-menu__item",
        onClick: this.showAddCommentModal.bind(this)
      }, "Comment...")) : /*#__PURE__*/React.createElement("div", {
        className: "actions-menu"
      }, !this.props.isEditTime && this.props.task && /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
        className: "actions-menu__item",
        onClick: this.showAddTimeModal.bind(this)
      }, "Add Time..."), /*#__PURE__*/React.createElement("div", {
        className: "actions-menu__item",
        onClick: this.showEditTimeModal.bind(this)
      }, "Edit Time...")), !this.props.isEditTime && !this.props.task && /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
        className: "actions-menu__item",
        onClick: this.showAssignTaskModal.bind(this)
      }, "Select Task...")), this.props.task ? /*#__PURE__*/React.createElement("div", {
        className: "actions-menu__item",
        onClick: this.showAddCommentModal.bind(this)
      }, "Comment...") : '', /*#__PURE__*/React.createElement("div", {
        className: "actions-menu__item",
        onClick: this.showDeleteTimeModal.bind(this)
      }, "Delete Time...")));
    }
  }]);
}(React.Component);
{
  TaskRowMenu;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TimeSearchTaskWrapper = /*#__PURE__*/function (_React$Component) {
  function TimeSearchTaskWrapper(props) {
    var _this;
    _classCallCheck(this, TimeSearchTaskWrapper);
    _this = _callSuper(this, TimeSearchTaskWrapper, [props]);
    _this.modalService = new ModalService();
    return _this;
  }
  _inherits(TimeSearchTaskWrapper, _React$Component);
  return _createClass(TimeSearchTaskWrapper, [{
    key: "showAddTimeModal",
    value: function showAddTimeModal() {
      this.modalService.showModal(AddTimeModal, {
        task: null,
        store: this.props.store,
        api: this.props.api,
        reactAppStateObj: this.props.reactAppStateObj,
        isAbstract: true,
        withComments: false,
        isShowMoreButton: true
      });
    }
  }, {
    key: "handleNavigateToSearchTab",
    value: function handleNavigateToSearchTab() {
      browserPort.eventDispatcher.broadcast('browser-button', 'change-tab', {
        tab: 'search'
      });
    }
  }, {
    key: "render",
    value: function render() {
      return /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("div", {
        className: "search__field-wrapper"
      }, /*#__PURE__*/React.createElement("div", {
        className: "icon-lens",
        onClick: this.handleNavigateToSearchTab.bind(this)
      }), /*#__PURE__*/React.createElement("div", {
        className: "search__field",
        onClick: this.handleNavigateToSearchTab.bind(this)
      }, "What are you working on..."), /*#__PURE__*/React.createElement("div", {
        className: "task-row__actions task-row__actions--current"
      }, /*#__PURE__*/React.createElement("button", {
        onClick: this.showAddTimeModal.bind(this),
        type: "button",
        className: "button button--default button--add-time"
      }, "Add Time"), /*#__PURE__*/React.createElement(TaskTimer, {
        isRunningTimer: this.props.store.timer.isActive(),
        store: this.props.store,
        reactAppStateObj: this.props.reactAppStateObj
      }))));
    }
  }]);
}(React.Component);
{
  TimeSearchTaskWrapper;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TimeWeekGroup = /*#__PURE__*/function (_React$Component) {
  function TimeWeekGroup(props) {
    var _this;
    _classCallCheck(this, TimeWeekGroup);
    _this = _callSuper(this, TimeWeekGroup, [props]);
    _this.state = {
      isShowAddTimeMenu: false
    };
    _this.popupControllerService = new PopupControllerService();
    return _this;
  }
  _inherits(TimeWeekGroup, _React$Component);
  return _createClass(TimeWeekGroup, [{
    key: "onToggleAddTimeMenu",
    value: function onToggleAddTimeMenu(isShowMenu) {
      this.setState({
        isShowAddTimeMenu: isShowMenu
      });
    }
  }, {
    key: "changeExpanded",
    value: function changeExpanded(week, event) {
      if (event.target && event.target.classList.contains('actions-menu__item')) {
        return;
      }
      this.props.changeExpanded(week, event);
    }
  }, {
    key: "render",
    value: function render() {
      var _this2 = this;
      var weekDate = this.props.weekDate;
      var week = this.props.week;
      var profile = this.props.store.profile;
      var user = profile.get();
      var isAdmin = profile.isAdmin();
      var isSupervisor = profile.isSupervisor();
      var lockTimeAfter = profile.get().team.lockTimeAfter;
      var lockTimeEnabled = profile.get().team.lockTimeEnabled;
      var minDate = !user.beginningOfWeek ? moment().subtract(lockTimeAfter, 'days').valueOf() : moment().subtract(lockTimeAfter + 1, 'days').valueOf();
      var minWeekDate = !user.beginningOfWeek ? moment().subtract(lockTimeAfter, 'days').startOf('week').valueOf() : moment().subtract(lockTimeAfter + 1, 'days').startOf('week').valueOf();
      var allowManualTimeInput = this.props.store.profile.get().timeTrackingPolicy.allowManualTimeInput;
      var isShowAddTime = (isAdmin || isSupervisor) && allowManualTimeInput || !isAdmin && !isSupervisor && moment(weekDate).valueOf() >= minWeekDate;
      var addTimeDate = lockTimeEnabled && moment(weekDate).valueOf() < minDate && !isAdmin && !isSupervisor ? moment(minDate).format('YYYY-MM-DD') : moment(weekDate).format('YYYY-MM-DD');
      return /*#__PURE__*/React.createElement("div", {
        key: "".concat(weekDate),
        className: "task-activity my-activity-week my-activity-week-tooltiped\n                        ".concat(week.isExpanded ? 'expanded' : '', "\n                        ").concat(this.state.isShowAddTimeMenu ? 'add-time-menu-show' : '', "\n                    ")
      }, /*#__PURE__*/React.createElement("div", {
        className: "page-title expandable",
        onClick: this.changeExpanded.bind(this, week)
      }, /*#__PURE__*/React.createElement("div", {
        className: "icon-arrow"
      }), /*#__PURE__*/React.createElement("h2", {
        className: "page-title__header"
      }, /*#__PURE__*/React.createElement("div", {
        className: "page-title__header-week"
      }, _format.week(weekDate, true, 'MMM DD')), /*#__PURE__*/React.createElement("div", {
        className: "page-title__activity-week-start my-activity-week-stats ev-tooltip-holder"
      }, week.weekDays.map(function (day) {
        return /*#__PURE__*/React.createElement("div", {
          className: "activity-week-stats__dot\n                                                        activity-week-stats__dot--".concat(_format.getDailyProgress(day.time))
        });
      })), /*#__PURE__*/React.createElement("div", {
        className: "page-title__header-time"
      }, _format.time(week.time))), allowManualTimeInput && isShowAddTime ? /*#__PURE__*/React.createElement(AddTimeMenu, {
        api: this.props.api,
        store: this.props.store,
        addTimeDate: addTimeDate,
        onToggleAddTimeMenu: this.onToggleAddTimeMenu.bind(this)
      }) : ''), week.days.map(function (item) {
        return /*#__PURE__*/React.createElement(TimeWeekDayGroup, {
          key: item.date,
          day: item,
          store: _this2.props.store,
          api: _this2.props.api,
          reactAppStateObj: _this2.props.reactAppStateObj
        });
      }));
    }
  }]);
}(React.Component);
{
  TimeWeekGroup;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TimeWeekDayGroup = /*#__PURE__*/function (_React$Component) {
  function TimeWeekDayGroup(props) {
    var _this;
    _classCallCheck(this, TimeWeekDayGroup);
    _this = _callSuper(this, TimeWeekDayGroup, [props]);
    _this.state = {
      isShowAddTimeMenu: false
    };
    _this.popupControllerService = new PopupControllerService();
    return _this;
  }
  _inherits(TimeWeekDayGroup, _React$Component);
  return _createClass(TimeWeekDayGroup, [{
    key: "onToggleAddTimeMenu",
    value: function onToggleAddTimeMenu(isShowMenu) {
      this.setState({
        isShowAddTimeMenu: isShowMenu
      });
    }
  }, {
    key: "render",
    value: function render() {
      var _this2 = this;
      var day = this.props.day;
      var allowManualTimeInput = this.props.store.profile.get().timeTrackingPolicy.allowManualTimeInput;
      var availableAddTimeByDate = this.popupControllerService.isLockedDate(day.date);
      return /*#__PURE__*/React.createElement("section", {
        className: "page-section"
      }, /*#__PURE__*/React.createElement("header", {
        className: "page-section__header\n                        ".concat(this.state.isShowAddTimeMenu ? 'add-time-menu-show' : '', "\n                    ")
      }, day.time > 0 && /*#__PURE__*/React.createElement("div", {
        className: "page-section__header-time-wrapper"
      }, /*#__PURE__*/React.createElement("div", {
        className: "page-section__header-time\n                                     page-section__header-time--".concat(_format.getDailyProgress(day.time))
      }, TimeEntryUtils.format(day.time))), /*#__PURE__*/React.createElement("div", {
        className: "page-section__header-day"
      }, /*#__PURE__*/React.createElement("div", {
        className: "header-day__name"
      }, _format.weekDay(day.date)), /*#__PURE__*/React.createElement("div", {
        className: "header-day__dot"
      }), /*#__PURE__*/React.createElement("div", {
        className: "header-day__date"
      }, _format.monthDay(day.date))), allowManualTimeInput && !availableAddTimeByDate ? /*#__PURE__*/React.createElement(AddTimeMenu, {
        api: this.props.api,
        store: this.props.store,
        addTimeDate: day.date,
        onToggleAddTimeMenu: this.onToggleAddTimeMenu.bind(this)
      }) : ''), day.records.filter(function (taskTime) {
        return taskTime.time;
      }).map(function (taskTime) {
        return /*#__PURE__*/React.createElement(TimeDayGroupTaskRow, {
          taskTime: taskTime,
          key: taskTime.id || taskTime.createdAt || taskTime.startedAt,
          store: _this2.props.store,
          api: _this2.props.api,
          reactAppStateObj: _this2.props.reactAppStateObj,
          weekDate: taskTime.date
        });
      }));
    }
  }]);
}(React.Component);
{
  TimeWeekDayGroup;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TimeDayGroupTaskRow = /*#__PURE__*/function (_React$Component) {
  function TimeDayGroupTaskRow(props) {
    var _this;
    _classCallCheck(this, TimeDayGroupTaskRow);
    _this = _callSuper(this, TimeDayGroupTaskRow, [props]);
    _this.popupControllerService = new PopupControllerService();
    _this.TaskTimeTooltipService = new TaskTimeTooltipService();
    _this.canStartTimer = true;
    _this.state = {
      isTimeInputFocus: false,
      isEditTimeByDate: _this.canEditTimeByDate(_this.props.taskTime.date),
      isEditTime: _this.calculateCanEditTime(),
      isManageProjectTime: _this.canManageProjectTime(),
      isShowTimer: _this.canShowTimer(),
      isShowMenu: false
    };

    // console.log('[EV][TimeDayGroupTaskRow] ', this.state, this.props)
    return _this;
  }
  _inherits(TimeDayGroupTaskRow, _React$Component);
  return _createClass(TimeDayGroupTaskRow, [{
    key: "componentDidUpdate",
    value: function componentDidUpdate() {
      var isShowTimer = this.canShowTimer();
      if (this.state.isShowTimer !== isShowTimer) {
        this.setState({
          isShowTimer: isShowTimer
        });
      }
    }
  }, {
    key: "canEditTimeByDate",
    value: function canEditTimeByDate(dayDate) {
      var store = this.props.store;
      return store.profile.isAdmin() || store.profile.isSupervisor() ? this.popupControllerService.isLockedFutureDate(dayDate) === false : this.popupControllerService.isLockedDate(dayDate, {
        forceUseTeamLockTime: true
      }) === false;
    }
  }, {
    key: "canManageProjectTime",
    value: function canManageProjectTime() {
      if (this.props.taskTime.task && !this.props.taskTime.task.project) {
        this.props.taskTime.task.project = this.TaskTimeTooltipService.getTaskProject(this.props.taskTime.task.projects, this.props.taskTime.task.projectsObjects);
      }
      return !!this.props.taskTime.task && this.props.taskTime.task.project !== null && this.props.taskTime.task.project.status === 'open';
    }
  }, {
    key: "canShowTimer",
    value: function canShowTimer() {
      return this.canManageProjectTime() === true && this.props.taskTime.task && this.props.taskTime.task.project !== null && this.popupControllerService.checkArchivedOrForeignTask(this.props.taskTime.task, this.props.taskTime.date) || !this.props.taskTime.task && this.popupControllerService.checkShowTimerWithoutTask(this.props.taskTime) && this.popupControllerService.checkArchivedOrForeignTask(this.props.taskTime.task || {
        _type: 'comment'
      }, this.props.taskTime.date);
    }
  }, {
    key: "isLockedFor",
    value: function isLockedFor(data, user) {
      if (user.role === 'admin') {
        return false;
      }
      if (data.lockReasons === undefined) {
        return false;
      }
      if (user.role === 'supervisor') {
        return data.lockReasons.filter(function (reason) {
          return reason !== 'date';
        }).length > 0;
      }
      return data.lockReasons.length > 0;
    }
  }, {
    key: "calculateCanEditTime",
    value: function calculateCanEditTime() {
      // if (store.profile.isAdmin()) {
      //     return true;
      // }
      //
      // if (!this.canEditTimeByDate(this.props.taskTime.date)) {
      //     return false;
      // }
      //
      // if (this.isLockedFor(this.props.taskTime, store.profile.get())) {
      //     return false;
      // }
      //
      // if (this.props.taskTime && this.props.taskTime.task) {
      //     return this.canManageProjectTime();
      // }
      //
      // return true;

      var _ref = this.props && this.props.taskTime,
        date = _ref.date,
        user = _ref.user;
      var project = this.props && this.props.taskTime && this.props.taskTime.task && this.props.taskTime.task.project;
      var editTimePermission = this.TaskTimeTooltipService.isCanEditTime(date, user, project);
      var isLockedByReasons = this.TaskTimeTooltipService.isLockedByReasons(this.props.taskTime);
      return editTimePermission === 'permitted' && isLockedByReasons === false;
    }
  }, {
    key: "showTimeInput",
    value: function showTimeInput() {
      if (!this.state.isEditTime) {
        return;
      }
      this.setState({
        isTimeInputFocus: true
      });
    }
  }, {
    key: "hideTimeInput",
    value: function hideTimeInput() {
      this.setState({
        isTimeInputFocus: false
      });
    }
  }, {
    key: "onToggleMenu",
    value: function onToggleMenu(isShowMenu) {
      this.setState({
        isShowMenu: isShowMenu
      });
    }
  }, {
    key: "render",
    value: function render() {
      var _this$props$taskTime = this.props.taskTime,
        task = _this$props$taskTime.task,
        time = _this$props$taskTime.time,
        id = _this$props$taskTime.id,
        website = _this$props$taskTime.website;
      var store = this.props.store;
      var isShowActions = task && (!task.project || !task.project.foreign && task.project && task.project.status === 'open') || store.profile.isAdmin() || !task;
      var isRunningTimer = this.props.taskTime.isCurrent;
      var timer = store.timer.get();
      var duration = store.timer.duration();
      var isShowTimer = this.state.isShowTimer;
      if (isRunningTimer) {
        var todayTaskTime = timer.today || 0;
        time = _format.timeRound(duration + todayTaskTime, 60) - todayTaskTime;
        this.props.taskTime.comment = timer.comment;
      }

      /**
       * We are hiding start timer button for other task times with same task.id
       * */
      if (!isRunningTimer && timer && timer.task && timer.task.id && task && timer.task.id === task.id) {
        isShowTimer = false;
      }
      return /*#__PURE__*/React.createElement("article", {
        className: "task-row task-time\n                ".concat(this.props.taskTime.comment ? 'task-row--with-comment' : '', "\n                ").concat(task && task.number ? 'task-row--with-number' : '', "\n                ").concat(!this.state.isEditTime ? 'task-row--locked' : '', "\n                ").concat(this.state.isEditTime ? 'task-row--editable' : 'task-row--without-editable', "\n                ").concat(this.state.isShowMenu ? 'task-row--menu-show' : '', "\n                ").concat(this.state.isShowTimer ? 'task-row--with-timer' : '', " \n                ").concat(isRunningTimer ? 'with-timer hover-timer task-row--active-timer' : '', "\n                ").concat(this.props.taskTime && this.props.taskTime.task ? 'task-row--with-task' : 'task-row--without-task'),
        "data-task-id": task ? task.id : null,
        "data-task-time-id": id || null
      }, /*#__PURE__*/React.createElement("div", {
        className: "task-row__time time ".concat(task && task.project && task.project.status !== 'open' ? '' : '')
      }, !isRunningTimer && this.state.isEditTime && this.state.isTimeInputFocus ? /*#__PURE__*/React.createElement(TaskTimeInput, {
        taskTime: this.props.taskTime,
        api: this.props.api,
        store: this.props.store,
        hideTimeInput: this.hideTimeInput.bind(this)
      }) : /*#__PURE__*/React.createElement("div", {
        className: "task-row__time-static",
        onClick: this.showTimeInput.bind(this)
      }, TimeEntryUtils.format(time, '1m'))), task ? /*#__PURE__*/React.createElement(TaskStatus, {
        task: task,
        store: this.props.store
      }) : '', website ? /*#__PURE__*/React.createElement(WebsiteIcon, {
        website: website
      }) : '', !task && !website && this.props.taskTime.comment ? /*#__PURE__*/React.createElement("div", {
        className: 'icon-comment'
      }) : '', /*#__PURE__*/React.createElement(TaskRowDescription, {
        task: task,
        taskTimeId: this.props.taskTime.id,
        time: this.props.taskTime.time,
        comment: this.props.taskTime.comment,
        api: this.props.api,
        store: this.props.store,
        weekDate: this.props.weekDate,
        isCurrent: isRunningTimer,
        isCanEditComment: this.state.isEditTime
      }), isShowActions ? /*#__PURE__*/React.createElement("div", {
        className: "task-row__actions"
      }, this.state.isEditTime && /*#__PURE__*/React.createElement(TaskRowMenu, {
        task: task,
        comment: this.props.taskTime.comment,
        store: this.props.store,
        time: this.props.taskTime.time,
        api: this.props.api,
        isCurrent: isRunningTimer,
        reactAppStateObj: this.props.reactAppStateObj,
        onToggleMenu: this.onToggleMenu.bind(this),
        taskTimeId: this.props.taskTime.id,
        withComments: true,
        weekDate: this.props.weekDate
      }), isShowTimer && /*#__PURE__*/React.createElement(TaskTimer, {
        isCircleMode: true,
        isRunningTimer: isRunningTimer,
        taskId: this.props.taskTime.task ? this.props.taskTime.task.id : null,
        website: this.props.taskTime.website ? this.props.taskTime.website : null,
        comment: this.props.taskTime.comment && !this.props.taskTime.task ? this.props.taskTime.comment : null,
        store: this.props.store,
        reactAppStateObj: this.props.reactAppStateObj
      })) : '');
    }
  }]);
}(React.Component);
{
  TimeDayGroupTaskRow;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var UserActivityRow = /*#__PURE__*/function (_React$Component) {
  function UserActivityRow(props) {
    var _this;
    _classCallCheck(this, UserActivityRow);
    _this = _callSuper(this, UserActivityRow, [props]);
    var user = _this.props.user;
    var storeTimer = _this.props.store.timer.get();
    var taskTimeId = storeTimer && !storeTimer.task && storeTimer.currentTaskTime ? storeTimer.currentTaskTime.id : null;
    var timer = user.timer && user.timer.task || user.timer && !user.timer.task ? user.timer : null;
    var userTotalTime = 0;
    var clockTime = 0;
    if (timer && timer.today && timer.task) {
      userTotalTime = user.timeTotal + timer.today + timer.duration;
      clockTime = timer.today + timer.duration;
    }
    if (timer && !timer.today && timer.task) {
      userTotalTime = timer.duration < 60 ? user.timeTotal + timer.duration + 60 : user.timeTotal + timer.duration;
      clockTime = timer.duration < 60 ? timer.duration + 60 : timer.duration;
    }
    if (timer && !timer.task && taskTimeId) {
      var timerCommentIndex = user.tasks && user.tasks.findIndex(function (task) {
        return !task.task && task.taskTimeId === taskTimeId;
      });
      var isTimerComment = timerCommentIndex > -1;
      var timerComment = isTimerComment ? user.tasks[timerCommentIndex] : {};
      userTotalTime = (timerComment.time || 0) < 60 ? user.timeTotal + 60 : user.timeTotal;
      clockTime = (timerComment.time || 0) < 60 ? (timerComment.time || 0) + timer.duration + 60 : (timerComment.time || 0) + timer.duration;
      if (isTimerComment) {
        user.tasks.splice(timerCommentIndex, 1);
      }
    }
    if (timer && !timer.task && !taskTimeId) {
      userTotalTime = timer.duration < 60 ? user.timeTotal + 60 : user.timeTotal + timer.duration;
      clockTime = timer.duration < 60 ? timer.duration + 60 : timer.duration;
    }
    if (!timer) {
      userTotalTime = user.timeTotal;
      clockTime = 0;
    }
    _this.state = {
      userTotalTime: userTotalTime,
      clockTime: clockTime,
      today: timer && timer.today || 0,
      isExpandedTasks: false
    };
    _this.timerId = null;
    _this.modalService = new ModalService();
    return _this;
  }
  _inherits(UserActivityRow, _React$Component);
  return _createClass(UserActivityRow, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      if (this.props.user.timer) {
        this.timerTick();
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      if (this.timerId) {
        clearInterval(this.timerId);
      }
    }
  }, {
    key: "timerTick",
    value: function timerTick() {
      var _this2 = this;
      var seconds = this.state.clockTime % 60;
      this.timerId = setInterval(function () {
        seconds++;
        var state = _this2.state;
        if (seconds === 60) {
          _this2.setState({
            userTotalTime: state.userTotalTime + 60,
            clockTime: state.clockTime + 60
          });
          seconds = 0;
        }
      }, 1000);
    }
  }, {
    key: "checkTasksStatuses",
    value: function checkTasksStatuses() {
      var user = this.props.user;
      var tasks = user.tasks && user.tasks.length ? user.tasks.filter(function (task) {
        return task.time;
      }) : null;
      return {
        hasExpanded: !tasks || tasks && tasks.length > 4,
        count: tasks ? tasks.length : 0
      };
    }
  }, {
    key: "toggleTasks",
    value: function toggleTasks() {
      if (!this.checkTasksStatuses().hasExpanded) {
        return false;
      }
      this.setState({
        isExpandedTasks: !this.state.isExpandedTasks
      });
    }
  }, {
    key: "clickByTaskName",
    value: function clickByTaskName(taskId, event) {
      event.preventDefault();
      event.stopPropagation();
      browserPort.eventDispatcher.broadcast('browser-button', 'change-tab', {
        tab: 'taskDetails',
        taskId: taskId
      });
    }
  }, {
    key: "renderTask",
    value: function renderTask(task) {
      return /*#__PURE__*/React.createElement("div", {
        className: "user-activity__task",
        key: task && task.task && task.task.id
      }, /*#__PURE__*/React.createElement("div", {
        className: "task__time"
      }, _format.time(task.time, null, 'clock')), task.isFake ? /*#__PURE__*/React.createElement("div", {
        className: "task__name task__name--fake"
      }, task.task.name || 'Task without title') : task && task._type && (task._type === 'comment' || task.type === 'comment') ? /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
        className: "task-row__comment-icon icon-comment"
      }), /*#__PURE__*/React.createElement("div", {
        className: "task__name task__name--comment",
        "data-task-id": task && task._id
      }, task.comment || 'No details')) : /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(TaskStatus, {
        task: task.task,
        store: this.props.store
      }), /*#__PURE__*/React.createElement("div", {
        className: "task__name",
        "data-task-id": task && task.task && task.task.id,
        onClick: this.clickByTaskName.bind(this, task.task.id)
      }, task.task.name || 'Task without title')));
    }
  }, {
    key: "renderTasksList",
    value: function renderTasksList(user) {
      var _this3 = this;
      var tasks = user.tasks.filter(function (task) {
        return task.time;
      });
      if (tasks.length <= 4 || tasks.length > 4 && this.state.isExpandedTasks) {
        return tasks.map(function (task) {
          return _this3.renderTask(task);
        });
      }
      var firstPartTasks = [].concat(tasks).splice(0, 3);
      var endPartTasks = [].concat(tasks).splice(3);
      var endPartTasksTotalTime = endPartTasks.reduce(function (total, task) {
        return total + task.time;
      }, 0);
      firstPartTasks.push({
        time: endPartTasksTotalTime,
        task: {
          name: "+".concat(endPartTasks.length, " more tasks...")
        },
        isFake: true
      });
      return firstPartTasks.map(function (task) {
        return _this3.renderTask(task);
      });
    }
  }, {
    key: "stopTimer",
    value: function stopTimer(event) {
      event.preventDefault();
      event.stopPropagation();
      this.props.store.timer.stop(null, null);
    }
  }, {
    key: "openStopTimerModal",
    value: function openStopTimerModal(event) {
      event.preventDefault();
      event.stopPropagation();
      this.modalService.showModal(StopTimerModal, {
        user: this.props.user,
        store: this.props.store
      });
    }
  }, {
    key: "render",
    value: function render() {
      var user = this.props.user;
      var timer = user.timer;
      var tasksStatuses = this.checkTasksStatuses();
      var currentUser = this.props.store.profile.get();
      var isComment = timer && !timer.task;
      return /*#__PURE__*/React.createElement("div", {
        className: "timers__list-row \n                                ".concat(tasksStatuses.hasExpanded ? 'timers__list-row--interactive' : '', "\n                                ").concat(user.timer ? 'timers__list-row--active-timer' : 'timers__list-row--inactive-timer'),
        onClick: this.toggleTasks.bind(this)
      }, /*#__PURE__*/React.createElement("div", {
        className: "list-row__user"
      }, /*#__PURE__*/React.createElement("div", {
        className: "list-row__user-avatar"
      }, /*#__PURE__*/React.createElement("a", {
        href: "https://app.everhour.com/#/members/".concat(user.user.id, "/timesheet"),
        target: "_blank"
      }, /*#__PURE__*/React.createElement(UserAvatar, {
        user: user.user
      }))), /*#__PURE__*/React.createElement("div", {
        className: "list-row__user-description"
      }, /*#__PURE__*/React.createElement("div", {
        className: "user-description__short-name",
        title: user.user.name
      }, user.user.name), /*#__PURE__*/React.createElement("div", {
        className: "user-description__total-time"
      }, _format.time(this.state.userTotalTime, null, 'clock')))), /*#__PURE__*/React.createElement("div", {
        className: "list-row__user-activity"
      }, timer && timer.task || isComment ? /*#__PURE__*/React.createElement("div", {
        className: "user-activity__task user-activity__task--current"
      }, /*#__PURE__*/React.createElement("div", {
        className: "task__time"
      }, /*#__PURE__*/React.createElement("span", {
        className: "time__hours"
      }, Math.floor(_format.time(this.state.clockTime, '00', 'hours_only') * 100 / 100)), /*#__PURE__*/React.createElement("span", {
        className: "time__blink"
      }, ":"), /*#__PURE__*/React.createElement("span", {
        className: "time__minutes"
      }, moment().startOf('day').add(this.state.clockTime, 'seconds').format("mm"))), isComment ? /*#__PURE__*/React.createElement("div", {
        className: "task-row__comment-icon icon-comment"
      }) : /*#__PURE__*/React.createElement(TaskStatus, {
        task: timer.task,
        store: this.props.store
      }), isComment ? /*#__PURE__*/React.createElement("div", {
        className: "task__name",
        "data-task-id": timer._id
      }, timer.comment || 'No details') : /*#__PURE__*/React.createElement("div", {
        className: "task__name",
        "data-task-id": timer.task.id,
        onClick: this.clickByTaskName.bind(this, timer.task.id)
      }, timer.task.name || 'Task without title'), currentUser.id === user.user.id ? /*#__PURE__*/React.createElement("span", {
        className: "timer-stop",
        onClick: this.stopTimer.bind(this)
      }, /*#__PURE__*/React.createElement("span", null)) : /*#__PURE__*/React.createElement("span", {
        className: "timer-stop",
        onClick: this.openStopTimerModal.bind(this)
      }, /*#__PURE__*/React.createElement("span", null))) : '', user.tasks && user.tasks.length ? this.renderTasksList(user) : ''));
    }
  }]);
}(React.Component);
{
  UserActivityRow;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TimeTab = /*#__PURE__*/function (_React$Component) {
  function TimeTab(props) {
    var _this;
    _classCallCheck(this, TimeTab);
    _this = _callSuper(this, TimeTab, [props]);
    _this.state = {
      activity: null,
      isShowComment: false,
      unpermittedDomains: props.unpermittedDomains,
      modified: +new Date()
    };
    _this.modalService = new ModalService();
    return _this;
  }
  _inherits(TimeTab, _React$Component);
  return _createClass(TimeTab, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      var _this2 = this;
      this.setState({
        activity: this.getActivity()
      });
      this.setWeeklyTooltips(this.props.store.taskTime.getUserActivity());
      browserPort.eventDispatcher.on('browser-button', 'websocket-update', function () {
        _this2.synchronizeState();
      });
      browserPort.eventDispatcher.on('browser-button', 'manual-update', function () {
        _this2.synchronizeState();
      });
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate() {
      var _this3 = this;
      setTimeout(function () {
        _this3.setWeeklyTooltips(_this3.props.store.taskTime.getUserActivity());
      });
    }
  }, {
    key: "synchronizeState",
    value: function synchronizeState() {
      var newActivity = this.getActivity();
      var stateActivity = this.state.activity;
      var newActivityKeys = newActivity && Object.keys(newActivity) || [];
      var stateActivityKeys = stateActivity && Object.keys(stateActivity) || [];
      if (newActivityKeys.length && stateActivityKeys.length && newActivityKeys[0] !== stateActivityKeys[0]) {
        newActivity[newActivityKeys[0]].isExpanded = true;
      }
      this.setState({
        activity: Object.assign({}, newActivity),
        modified: +new Date()
      });
    }
  }, {
    key: "getActivity",
    value: function getActivity() {
      var activity = this.props.store.taskTime.getUserActivity();
      var stateActivity = this.state.activity;
      var stateActivityExists = stateActivity && stateActivity.constructor === Object && Object.keys(stateActivity).length;
      var activityKeys = activity && Object.keys(activity) || [];
      if (stateActivity && stateActivityExists && activityKeys.length) {
        for (var key in activity) {
          if (activity.hasOwnProperty(key)) {
            activity[key].isExpanded = stateActivity[key] && stateActivity[key].isExpanded || false;
          }
        }
      }
      if ((!stateActivity || !stateActivityExists) && activityKeys.length) {
        activity[activityKeys[0]].isExpanded = true;
      }
      return activity;
    }
  }, {
    key: "changeExpanded",
    value: function changeExpanded(week) {
      week.isExpanded = !week.isExpanded;
      this.setState({
        activity: this.getActivity()
      });
    }
  }, {
    key: "setWeeklyTooltips",
    value: function setWeeklyTooltips(activity) {
      Object.keys(activity).forEach(function (week, weekIndex) {
        var activityWeek = activity[week];
        var table = activityWeek.weekDays.map(function (day) {
          var dayOfWeek = moment(day.day).format('dd');
          var time = day.time > 0 ? _format.time(_format.timeRound(day.time, 60), null, 'clock') : '-';
          return "<div><span>".concat(dayOfWeek, "</span><span>").concat(time, "</span></div>");
        }).join('');
        everhourTooltip("#time-tab .my-activity-week-tooltiped:eq(".concat(weekIndex, ") .ev-tooltip-holder"), {
          placement: weekIndex ? 'top' : 'bottom',
          title: function title() {
            return table;
          }
        }, 'my-activity-week-tooltip');
      });
    }
  }, {
    key: "showAddCommentModal",
    value: function showAddCommentModal(timer) {
      var taskTimeIndex = this.props.store.taskTime.taskTime.findIndex(function (t) {
        return t.task && t.task.id && timer.task && t.task.id === timer.task.id && t.date === timer.userDate && t.user === timer.user.id;
      });
      var taskTime = taskTimeIndex > -1 ? this.props.store.taskTime.taskTime[taskTimeIndex] : null;
      this.modalService.showModal(AddCommentModal, {
        task: timer.task,
        comment: timer.comment || taskTime && taskTime.comment,
        api: this.props.api,
        store: this.props.store,
        isCurrent: true,
        taskTimeId: taskTime && taskTime.id || null
      });
    }
  }, {
    key: "handleNavigateToSearchTab",
    value: function handleNavigateToSearchTab() {
      var timer = this.props.store.timer.get();
      if (timer && timer.status && timer.status === 'active' && !timer.task && timer.comment || timer && timer.status && timer.status === 'active' && timer.task) {
        return;
      }
      browserPort.eventDispatcher.broadcast('browser-button', 'change-tab', {
        tab: 'search'
      });
    }
  }, {
    key: "handleToggleComment",
    value: function handleToggleComment(isShow) {
      this.setState({
        isShowComment: isShow
      });
    }
  }, {
    key: "renderActiveTimer",
    value: function renderActiveTimer(timer) {
      var timerClone = _.clone(timer);
      timerClone.isCurrent = true;
      return /*#__PURE__*/React.createElement("div", {
        className: "task-timer__wrapper"
      }, /*#__PURE__*/React.createElement("section", {
        className: "search__field-wrapper js-search__field-wrapper"
      }, /*#__PURE__*/React.createElement("article", {
        className: "task-row task-row--current-running\n                ".concat(!timer.task ? 'task-row--without-task' : 'task-row--with-timer'),
        "data-task-id": timer.task ? timer.task.id : null
      }, timer.task ? /*#__PURE__*/React.createElement(TaskRowDescription, {
        task: timer.task,
        comment: timer.comment,
        api: this.props.api,
        store: this.props.store,
        isCurrent: true
      }) : /*#__PURE__*/React.createElement("div", null, timer.comment ? /*#__PURE__*/React.createElement("div", {
        className: "search__task-comment",
        key: timer.comment
      }, timer.currentTaskTime && timer.currentTaskTime.website || timer.website ? /*#__PURE__*/React.createElement(WebsiteIcon, {
        website: timer.currentTaskTime ? timer.currentTaskTime.website : timer.website
      }) : /*#__PURE__*/React.createElement("div", {
        className: "icon-comment"
      }), /*#__PURE__*/React.createElement("div", {
        className: "task-comment__text"
      }, /*#__PURE__*/React.createElement(TaskCommentInput, {
        isMultiLine: true,
        taskTimeId: null,
        comment: timer.comment,
        isCurrent: true,
        api: this.props.api,
        store: this.props.store
      }))) : /*#__PURE__*/React.createElement("div", {
        className: "search__task-comment-empty",
        onClick: this.handleNavigateToSearchTab.bind(this)
      }, /*#__PURE__*/React.createElement("div", {
        className: "icon-lens"
      }), /*#__PURE__*/React.createElement("div", {
        className: "task-comment__text"
      }, "What are you working on..."))), /*#__PURE__*/React.createElement("div", {
        className: "task-row__actions task-row__actions--current"
      }, timer.task ? /*#__PURE__*/React.createElement("div", {
        className: "task-row__description-comment",
        onClick: this.showAddCommentModal.bind(this, timer)
      }) : '', /*#__PURE__*/React.createElement(TaskTimer, {
        isRunningTimer: this.props.store.timer.isActive(),
        store: this.props.store,
        reactAppStateObj: this.props.reactAppStateObj
      })))));
    }
  }, {
    key: "renderTasksList",
    value: function renderTasksList(activity, hasTimer) {
      var _this4 = this;
      if ((!activity || activity && _.keys(activity).length === 0) && !hasTimer) {
        return /*#__PURE__*/React.createElement("div", {
          className: "application-loading"
        }, /*#__PURE__*/React.createElement("div", {
          className: "application-loading-header"
        }, "Your activity will appear here"), /*#__PURE__*/React.createElement("div", {
          className: "application-loading-message"
        }, "Start tracking time and all your time", /*#__PURE__*/React.createElement("br", null), "entries will be displayed here."));
      }
      return Object.keys(activity).map(function (weekDate) {
        var week = activity[weekDate];
        return /*#__PURE__*/React.createElement(TimeWeekGroup, {
          key: weekDate,
          week: week,
          weekDate: weekDate,
          store: _this4.props.store,
          api: _this4.props.api,
          reactAppStateObj: _this4.props.reactAppStateObj,
          changeExpanded: _this4.changeExpanded.bind(_this4)
        });
      });
    }
  }, {
    key: "renderUnpermittedDomains",
    value: function renderUnpermittedDomains(unpermittedDomains) {
      return /*#__PURE__*/React.createElement("div", {
        className: "message message--warning message--permission-domains",
        id: "permissions"
      }, /*#__PURE__*/React.createElement("div", {
        className: "message__description"
      }, /*#__PURE__*/React.createElement("b", null, "Allow access to your domains:"), unpermittedDomains.map(function (domain) {
        return /*#__PURE__*/React.createElement("div", {
          className: "permissions-domain"
        }, domain);
      })), /*#__PURE__*/React.createElement("button", {
        className: "js-permissions-grant-access button button--orange",
        onClick: this.handleGrandAccessClick.bind(this)
      }, "Grant Access"));
    }
  }, {
    key: "handleGrandAccessClick",
    value: function handleGrandAccessClick() {
      var _this5 = this;
      // firefox can't ask permissions from popover
      // https://bugzilla.mozilla.org/show_bug.cgi?id=1432083
      // the task is in progress, most probably it get fixed in 73+ versions
      // if (browserPort.type === 'firefox') {
      //     browserPort.tab.open(browserPort.resource.getUrl('options.html'));
      //     window.close();
      //     return;
      // }
      browserPort.tab.requestCustomDomainPermissions(this.props.unpermittedDomains, function (isGranted) {
        if (isGranted) {
          _this5.setState({
            unpermittedDomains: null
          });
        }
      });
    }
  }, {
    key: "render",
    value: function render() {
      var _this$props = this.props,
        store = _this$props.store,
        api = _this$props.api,
        reactAppStateObj = _this$props.reactAppStateObj;
      var unpermittedDomains = this.state.unpermittedDomains;
      var hasTimer = store.timer.isActive();
      var duration = store.timer.duration();
      var timer = store.timer.get();
      return /*#__PURE__*/React.createElement("div", {
        className: "activity-wrapper"
      }, unpermittedDomains && unpermittedDomains.length ? this.renderUnpermittedDomains(unpermittedDomains) : '', !hasTimer ? /*#__PURE__*/React.createElement(TimeSearchTaskWrapper, {
        store: store,
        api: api,
        reactAppStateObj: reactAppStateObj
      }) : '', hasTimer && this.renderActiveTimer(timer, duration), /*#__PURE__*/React.createElement("div", {
        className: "time-tab__group-list"
      }, this.state.activity && this.renderTasksList(this.state.activity, hasTimer)));
    }
  }]);
}(React.Component);
{
  TimeTab;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TimersTab = /*#__PURE__*/function (_React$Component) {
  function TimersTab(props) {
    var _this;
    _classCallCheck(this, TimersTab);
    _this = _callSuper(this, TimersTab, [props]);
    browserPort.analytics.event('popover_timers');
    _this.sortPopover = [{
      id: 1,
      name: 'Today',
      from: moment().format('YYYY-MM-DD'),
      to: moment().format('YYYY-MM-DD')
    }, {
      id: 2,
      name: 'Yesterday',
      from: moment().subtract(1, 'days').format('YYYY-MM-DD'),
      to: moment().subtract(1, 'days').format('YYYY-MM-DD')
    }, {
      id: 3,
      name: 'This Week',
      from: moment().startOf('week').format('YYYY-MM-DD'),
      to: moment().endOf('week').format('YYYY-MM-DD')
    }, {
      id: 4,
      name: 'Last Week',
      from: moment().startOf('week').subtract(7, 'days').format('YYYY-MM-DD'),
      to: moment().startOf('week').subtract(1, 'days').format('YYYY-MM-DD')
    }];
    _this.state = {
      isFetchTeamStatus: false,
      teamStatus: [],
      teamUsersWithActivity: [],
      teamUsersWithoutActivity: [],
      searchTerm: null,
      searchSortSelected: _this.sortPopover[0],
      isOpenSortPopover: false
    };
    browserPort.eventDispatcher.on('browser-button', 'websocket-update', function () {
      _this.fetchTimers();
    });
    return _this;
  }
  _inherits(TimersTab, _React$Component);
  return _createClass(TimersTab, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this.fetchTimers();
    }
  }, {
    key: "fetchTimers",
    value: function fetchTimers() {
      var _this2 = this;
      var searchSortSelected = this.state.searchSortSelected;
      this.setState({
        isFetchTeamStatus: true
      });
      this.props.store.teamStatus.load(searchSortSelected.from, searchSortSelected.to).then(function () {
        var teamStatus = _this2.props.store.teamStatus.get();
        _this2.setState({
          isFetchTeamStatus: false,
          teamStatus: teamStatus,
          teamUsersWithActivity: _this2.groupUsersActivity(teamStatus),
          teamUsersWithoutActivity: _this2.groupUsersInactive(teamStatus)
        });
      });
    }
  }, {
    key: "groupUsersActivity",
    value: function groupUsersActivity(team) {
      var activeUsers = team.filter(function (user) {
        user.tasks = user.tasks && user.tasks.length ? user.tasks.filter(function (task) {
          return user.timer && user.timer.task && task.task ? user.timer.task.id !== task.task.id : true;
        }) : [];
        return user.tasks && user.tasks.length && user.tasks.filter(function (task) {
          return task.time;
        }).length || user.timer;
      });

      // We are merging time without task
      activeUsers = activeUsers.map(function (user) {
        var tasks = {};
        if (user.tasks) {
          user.tasks.forEach(function (time) {
            if (time.task && time.task.id && !tasks[time.task && time.task.id]) {
              tasks[time.task.id] = time;
            } else if (time.task && time.task.id && tasks[time.task && time.task.id]) {
              tasks[time.task.id].time += time.time;
            }
            if (!time.task && !tasks[time.taskTimeId]) {
              tasks[time.taskTimeId] = time;
            } else if (!time.task && tasks[time.taskTimeId]) {
              tasks[time.taskTimeId].time += time.time;
            }
          });
        }
        user.tasks = Object.values(tasks);
        return user;
      });
      function sortUsers(users) {
        return users.map(function (user) {
          user.timeTotal = 0;
          user.timeSortTotal = 0;
          if (user.timer && user.timer.task) {
            user.timeSortTotal += (user.timer.today || 0) + user.timer.duration;
          }
          user.user.shortName = (user.user.name || user.user.email).replace('&nbsp;', ' ').split(/(\s+)/)[0];
          user.tasks.sort(function (taskA, taskB) {
            return taskB.time - taskA.time;
          }).forEach(function (task) {
            user.timeTotal += task.time;
            user.timeSortTotal += task.time;
          });
          return user;
        }).sort(function (userA, userB) {
          return userB.timeSortTotal - userA.timeSortTotal;
        });
      }
      var activeUsersWithCurrentTimer = sortUsers(activeUsers.filter(function (user) {
        return user.timer;
      }));
      var activeUsersWithoutCurrentTimer = sortUsers(activeUsers.filter(function (user) {
        return !user.timer;
      }));
      return activeUsersWithCurrentTimer.concat(activeUsersWithoutCurrentTimer);
    }
  }, {
    key: "groupUsersInactive",
    value: function groupUsersInactive(team) {
      var usersTimeOff = [];
      var usersWithoutTimeOff = [];
      team.forEach(function (user) {
        if ((!user.tasks || !user.tasks.length || !user.tasks.filter(function (task) {
          return task.time;
        }).length) && !user.timer) {
          if (user.timeOff && user.timeOff.length) {
            usersTimeOff.push(user);
          } else {
            usersWithoutTimeOff.push(user);
          }
        }
      });
      usersTimeOff = usersTimeOff.sort(function (userA, userB) {
        return (userA.user.name || userA.user.email).localeCompare(userB.user.name || userB.user.email);
      });
      usersWithoutTimeOff = usersWithoutTimeOff.sort(function (userA, userB) {
        return userA.user.name || userA.user.email.localeCompare(userB.user.name || userB.user.email);
      });
      return usersTimeOff.concat(usersWithoutTimeOff);
    }
  }, {
    key: "onToggleMenu",
    value: function onToggleMenu(isOpen) {
      this.setState({
        isOpenSortPopover: isOpen
      });
    }
  }, {
    key: "selectSortPeriod",
    value: function selectSortPeriod(item) {
      var _this3 = this;
      this.setState({
        searchSortSelected: item,
        isFetchTeamStatus: false
      });
      setTimeout(function () {
        _this3.fetchTimers();
      });
    }
  }, {
    key: "filterUsers",
    value: function filterUsers(e) {
      var val = e.target.value;
      var teamStatus = val ? this.state.teamStatus.filter(function (user) {
        return !!~(user.user.name || user.user.email).toLowerCase().indexOf(val.toLowerCase());
      }) : [].concat(this.state.teamStatus);
      this.setState({
        searchTerm: val,
        teamUsersWithActivity: this.groupUsersActivity(teamStatus),
        teamUsersWithoutActivity: this.groupUsersInactive(teamStatus)
      });
    }
  }, {
    key: "formatPeriodDate",
    value: function formatPeriodDate(date) {
      return moment(date).format('MMM D');
    }
  }, {
    key: "differenceDays",
    value: function differenceDays(start, end) {
      if (!start && !end) {
        return 0;
      }
      if (start && !end) {
        return 1;
      }
      if (start && end && start === end) {
        return 1;
      }
      var startDate = moment(start);
      var endDate = moment(end);
      return endDate.diff(startDate, 'days') + 1;
    }
  }, {
    key: "renderUserTooltipTrigger",
    value: function renderUserTooltipTrigger(user) {
      return /*#__PURE__*/React.createElement("div", {
        className: "inactive-list__user"
      }, /*#__PURE__*/React.createElement("a", {
        href: "https://app.everhour.com/#/members/".concat(user.user.id, "/timesheet"),
        target: "_blank"
      }, /*#__PURE__*/React.createElement(UserAvatar, {
        user: user.user
      })), user.timeOff && user.timeOff.length ? /*#__PURE__*/React.createElement("div", {
        className: "user__time-off-label"
      }, "off") : '');
    }
  }, {
    key: "renderUserTooltipActivity",
    value: function renderUserTooltipActivity(user) {
      var _this4 = this;
      var groupPeriod = function groupPeriod(timeOff, isShowCountDays) {
        if (timeOff.startDate && !timeOff.endDate || timeOff.startDate && timeOff.endDate && timeOff.startDate === timeOff.endDate) {
          if (isShowCountDays) {
            return "".concat(_this4.formatPeriodDate(timeOff.startDate), " (").concat(_this4.differenceDays(timeOff.startDate), " day)");
          }
          return "".concat(_this4.formatPeriodDate(timeOff.startDate));
        }
        if (timeOff.startDate && timeOff.endDate) {
          if (isShowCountDays) {
            return "".concat(_this4.formatPeriodDate(timeOff.startDate), " - ").concat(_this4.formatPeriodDate(timeOff.endDate), " \n                       (").concat(_this4.differenceDays(timeOff.startDate, timeOff.endDate), " days)");
          }
          return "".concat(_this4.formatPeriodDate(timeOff.startDate), " - ").concat(_this4.formatPeriodDate(timeOff.endDate));
        }
      };
      var concatType = function concatType(timeOff) {
        return /*#__PURE__*/React.createElement("div", {
          className: "description__time-off-type"
        }, timeOff.timeOffType.name, /*#__PURE__*/React.createElement("span", {
          className: "time-off-type__status",
          style: {
            backgroundColor: timeOff.timeOffType.color
          }
        }));
      };
      var concatCreatedMember = function concatCreatedMember(timeOff) {
        var user = _this4.props.store.teamStatus.getById(timeOff.createdBy);
        return user.user.name || user.user.email;
      };
      if (user.timeOff && user.timeOff.length === 1) {
        return /*#__PURE__*/React.createElement("div", {
          className: "user__time-off"
        }, /*#__PURE__*/React.createElement("div", {
          className: "time-off__row"
        }, /*#__PURE__*/React.createElement("div", {
          className: "time-off__label"
        }, "Created by:"), /*#__PURE__*/React.createElement("div", {
          className: "time-off__description"
        }, concatCreatedMember(user.timeOff[0]))), /*#__PURE__*/React.createElement("div", {
          className: "time-off__row"
        }, /*#__PURE__*/React.createElement("div", {
          className: "time-off__label"
        }, "Member:"), /*#__PURE__*/React.createElement("div", {
          className: "time-off__description"
        }, user.user.name || user.user.email)), /*#__PURE__*/React.createElement("div", {
          className: "time-off__row"
        }, /*#__PURE__*/React.createElement("div", {
          className: "time-off__label"
        }, "Period:"), /*#__PURE__*/React.createElement("div", {
          className: "time-off__description"
        }, groupPeriod(user.timeOff[0], true))), /*#__PURE__*/React.createElement("div", {
          className: "time-off__row"
        }, /*#__PURE__*/React.createElement("div", {
          className: "time-off__label"
        }, "Type:"), /*#__PURE__*/React.createElement("div", {
          className: "time-off__description"
        }, concatType(user.timeOff[0]))), user.timeOff[0].status && user.timeOff[0].status !== 'approved' ? /*#__PURE__*/React.createElement("div", {
          className: "time-off__row"
        }, /*#__PURE__*/React.createElement("div", {
          className: "time-off__label"
        }, "Status:"), /*#__PURE__*/React.createElement("div", {
          className: "time-off__description"
        }, user.timeOff[0].status === 'pending' ? /*#__PURE__*/React.createElement("div", {
          className: "description__pending"
        }, "Not approved") : '', user.timeOff[0].status === 'approved' && !!user.timeOff[0].approvedBy ? /*#__PURE__*/React.createElement("div", {
          className: "description__approved"
        }, "Approved") : '')) : '', user.timeOff[0].approvedBy ? /*#__PURE__*/React.createElement("div", {
          className: "time-off__row"
        }, /*#__PURE__*/React.createElement("div", {
          className: "time-off__label"
        }, "By:"), /*#__PURE__*/React.createElement("div", {
          className: "time-off__description"
        }, user.timeOff[0].approvedBy.name)) : '');
      }
      if (user.timeOff && user.timeOff.length > 1) {
        return /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("div", {
          className: "time-off__row"
        }, /*#__PURE__*/React.createElement("div", {
          className: "time-off__description"
        }, user.user.name || user.user.email)), user.timeOff.map(function (timeOff) {
          return /*#__PURE__*/React.createElement("div", {
            className: "time-off__row"
          }, /*#__PURE__*/React.createElement("div", {
            className: "time-off__label"
          }, groupPeriod(timeOff)), /*#__PURE__*/React.createElement("div", {
            className: "time-off__description"
          }, concatType(timeOff)));
        }));
      }
      return /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("div", {
        className: "inactive-list__user-name"
      }, user.user.name || user.user.email), /*#__PURE__*/React.createElement("div", {
        className: "inactive-list__user-description"
      }, "No activity"));
    }
  }, {
    key: "render",
    value: function render() {
      var _this5 = this;
      return /*#__PURE__*/React.createElement("div", {
        className: "timers"
      }, /*#__PURE__*/React.createElement("div", {
        className: "timers__search"
      }, /*#__PURE__*/React.createElement("input", {
        type: "text",
        className: "timers__search-field",
        placeholder: "Search member\u2026",
        onInput: this.filterUsers.bind(this)
      }), /*#__PURE__*/React.createElement(Popover2, {
        triggerNode: /*#__PURE__*/React.createElement("div", {
          className: "timers__sort-trigger"
        }, /*#__PURE__*/React.createElement("div", {
          className: "sort-trigger__period-name"
        }, this.state.searchSortSelected.name), /*#__PURE__*/React.createElement("div", {
          className: "icon-arrow ".concat(this.state.isOpenSortPopover ? 'icon-arrow-up' : '')
        })),
        trigger: "click",
        placement: "bottom-right",
        onToggle: this.onToggleMenu.bind(this),
        onClose: this.onToggleMenu.bind(this, false),
        closeOnClick: true,
        width: 192
      }, /*#__PURE__*/React.createElement("div", {
        className: "sort-menu"
      }, this.sortPopover.map(function (item) {
        return /*#__PURE__*/React.createElement("div", {
          className: "sort-menu__item",
          onClick: _this5.selectSortPeriod.bind(_this5, item)
        }, /*#__PURE__*/React.createElement("span", {
          className: "icon-radio ".concat(item.id === _this5.state.searchSortSelected.id ? 'icon-radio-selected' : 'icon-radio-normal')
        }), item.name);
      })))), !this.state.isFetchTeamStatus && this.state.teamUsersWithActivity && this.state.teamUsersWithActivity.length ? /*#__PURE__*/React.createElement("div", {
        className: "timers__list"
      }, this.state.teamUsersWithActivity.map(function (user) {
        return /*#__PURE__*/React.createElement(UserActivityRow, {
          key: user.user.id,
          user: user,
          store: _this5.props.store
        });
      })) : '', !this.state.isFetchTeamStatus && !this.state.teamUsersWithActivity || !this.state.teamUsersWithActivity.length && this.state.teamUsersWithoutActivity && this.state.teamUsersWithoutActivity.length ? /*#__PURE__*/React.createElement("div", {
        className: "timers__not-exist"
      }, "NO ACTIVITY") : '', !this.state.isFetchTeamStatus && this.state.teamUsersWithActivity && !this.state.teamUsersWithActivity.length && this.state.teamUsersWithoutActivity && !this.state.teamUsersWithoutActivity.length ? /*#__PURE__*/React.createElement("div", {
        className: "timers__not-exist"
      }, "No timers to display. Try a different filter.") : '', !this.state.isFetchTeamStatus && this.state.teamUsersWithoutActivity && this.state.teamUsersWithoutActivity.length ? /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("div", {
        className: "timers__inactive-list"
      }, this.state.teamUsersWithoutActivity.map(function (user) {
        return /*#__PURE__*/React.createElement(Tooltip, {
          key: user.user.id,
          space: 3,
          triggerTemplate: _this5.renderUserTooltipTrigger.bind(_this5, user)
        }, _this5.renderUserTooltipActivity(user));
      }))) : '', this.state.isFetchTeamStatus ? /*#__PURE__*/React.createElement(Skeleton, {
        view: 'timers'
      }) : '');
    }
  }]);
}(React.Component);
{
  TimersTab;
}
;
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var RecentTaskRow = /*#__PURE__*/function (_React$Component) {
  function RecentTaskRow(props) {
    var _this;
    _classCallCheck(this, RecentTaskRow);
    _this = _callSuper(this, RecentTaskRow, [props]);
    _this.state = {
      isTimeEditableByRole: _this.isTimeEditableByRole(null)
    };
    _this.modalService = new ModalService();
    _this.popupControllerService = new PopupControllerService();
    return _this;
  }
  _inherits(RecentTaskRow, _React$Component);
  return _createClass(RecentTaskRow, [{
    key: "isTimeEditableByRole",
    value: function isTimeEditableByRole(timeEntry) {
      return true;
    }
  }, {
    key: "showAddTimeModal",
    value: function showAddTimeModal(event) {
      event.preventDefault();
      event.stopPropagation();
      var task = this.props.task;
      this.modalService.showModal(AddTimeModal, {
        task: task,
        isAbstract: task && (task._type === 'comment' || task.type === 'comment'),
        onClose: this.onCloseAddTimeModal.bind(this),
        store: this.props.store,
        api: this.props.api
      });
    }
  }, {
    key: "onCloseAddTimeModal",
    value: function onCloseAddTimeModal(data) {
      if (data && data.isSuccessEditTime) {
        this.navigateToTimeTab();
      }
    }
  }, {
    key: "onStartTimer",
    value: function onStartTimer() {
      this.navigateToTimeTab();
    }
  }, {
    key: "navigateToTimeTab",
    value: function navigateToTimeTab() {
      this.props.navigateToTimeTab();
    }
  }, {
    key: "handleSelectTask",
    value: function handleSelectTask() {
      var task = this.props.task;
      var request = {};
      if (task.id) {
        request.taskId = task.id;
      }
      if (task._type === 'comment' || task.type === 'comment') {
        request.comment = task.comment;
      }
      browserPort.eventDispatcher.broadcast('browser-button', 'select-task', request);
    }
  }, {
    key: "render",
    value: function render() {
      var task = this.props.task;
      var time = task.time && task.time.users ? task.time.users[this.props.store.profile.get().id] || 0 : task.time || 0;
      var timeFormatted = TimeEntryUtils.format(time);
      var store = this.props.store;
      var timer = store.timer.get();
      var isActiveTask = timer && timer.task && timer.task.id == task.id || timer && timer.currentTaskTime && timer.currentTaskTime.id === task.id || task.isTimerWithOutTask;
      timeFormatted = timeFormatted ? timeFormatted.replace('-', '') : '';
      return /*#__PURE__*/React.createElement("article", {
        className: "task-row task-row--recent \n                ".concat(!this.props.isSearchTab && isActiveTask ? 'task-row--recent-active' : '', "\n                task-time \n            "),
        "data-task-id": task ? task.id : null,
        "data-task-time-id": null,
        "data-key-id": task ? task.id || task._id : null
      }, task && task.status ? /*#__PURE__*/React.createElement(TaskStatus, {
        task: task,
        store: this.props.store
      }) : '', task && (task._type === 'comment' || task.type === 'comment') ? /*#__PURE__*/React.createElement("div", {
        className: "task-row__comment-icon icon-comment"
      }) : '', /*#__PURE__*/React.createElement(TaskRowDescription, {
        task: task && !task.isTimerWithOutTask ? task : null,
        taskTimeId: null,
        time: null,
        searchTerm: this.props.searchTerm,
        api: this.props.api,
        store: this.props.store,
        isCurrent: !!isActiveTask,
        isSearchTab: this.props.isSearchTab,
        isNotClikableDescription: this.props.isNotClikableDescription,
        comment: task && task.comment
      }), timeFormatted ? /*#__PURE__*/React.createElement("div", {
        className: "task-row__time time\n                                ".concat(task && task.project && task.project.status !== 'open' ? ' archived' : '')
      }, /*#__PURE__*/React.createElement("div", {
        className: "task-row__time-static"
      }, timeFormatted)) : '', (task && (!task.project || !task.project.foreign && task.project.status === 'open') || store.profile.isAdmin() || task && task.isTimerWithOutTask || !task) && /*#__PURE__*/React.createElement("div", {
        className: "task-row__actions"
      }, this.state.isTimeEditableByRole && /*#__PURE__*/React.createElement("button", {
        onClick: this.showAddTimeModal.bind(this),
        type: "button",
        className: "button button--default button--add-time"
      }, "Add Time"), /*#__PURE__*/React.createElement(TaskTimer, {
        taskId: this.props.taskTime && this.props.taskTime.task ? this.props.taskTime.task.id : null,
        comment: this.props.comment,
        store: this.props.store,
        reactAppStateObj: this.props.reactAppStateObj,
        onStartTimer: this.onStartTimer.bind(this),
        isRunningTimer: !this.props.isSearchTab && isActiveTask
      })));
    }
  }]);
}(React.Component);
{
  RecentTaskRow;
}
;
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var UserAvatar = /*#__PURE__*/function (_React$Component) {
  function UserAvatar(props) {
    var _this;
    _classCallCheck(this, UserAvatar);
    _this = _callSuper(this, UserAvatar, [props]);
    _this.colorsCount = 10;
    return _this;
  }
  _inherits(UserAvatar, _React$Component);
  return _createClass(UserAvatar, [{
    key: "getColor",
    value: function getColor(id) {
      if (typeof id === 'string') {
        return parseInt(id.substring(3), 10) % this.colorsCount;
      }
      return id % this.colorsCount;
    }
  }, {
    key: "getUserAvatarName",
    value: function getUserAvatarName(user) {
      return window.getUserInitials(user.name || user.email);
    }
  }, {
    key: "getUserAvatarPlaceholder",
    value: function getUserAvatarPlaceholder(user) {
      var name = this.getUserAvatarName(user);
      var color = this.getColor(user.id);
      return /*#__PURE__*/React.createElement("span", {
        className: "popup__user-avatar"
      }, /*#__PURE__*/React.createElement("span", {
        className: "color".concat(color)
      }, name));
    }
  }, {
    key: "getUserAvatarImage",
    value: function getUserAvatarImage(url) {
      return /*#__PURE__*/React.createElement("span", {
        className: "popup__user-avatar"
      }, /*#__PURE__*/React.createElement("img", {
        src: url
      }));
    }
  }, {
    key: "getUserAvatar",
    value: function getUserAvatar(user, src) {
      var html = "";
      if (user && !user.avatarUrl) {
        html = this.getUserAvatarPlaceholder(user);
      } else if (user && user.avatarUrl && !src) {
        html = this.getUserAvatarImage(user.avatarUrl);
      } else if (src) {
        html = this.getUserAvatarImage(src);
      }
      return html;
    }
  }, {
    key: "render",
    value: function render() {
      if (this.props.src) {
        return this.getUserAvatar(null, this.props.src);
      }
      return this.props.user ? this.getUserAvatar(this.props.user, null) : '';
    }
  }]);
}(React.Component);
{
  UserAvatar;
}
;
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var StopTimerModal = /*#__PURE__*/function (_React$Component) {
  function StopTimerModal(props) {
    var _this;
    _classCallCheck(this, StopTimerModal);
    _this = _callSuper(this, StopTimerModal, [props]);
    _this.state = {
      comment: null,
      error: {
        isShowArrow: false,
        message: null
      }
    };
    _this.textareaCommentRef = React.createRef();
    return _this;
  }
  _inherits(StopTimerModal, _React$Component);
  return _createClass(StopTimerModal, [{
    key: "onChangeComment",
    value: function onChangeComment(e) {
      this.setState({
        comment: e.target.value
      });
    }
  }, {
    key: "setValidationComment",
    value: function setValidationComment(e) {
      var isValidComment = this.checkValidComment();
      this.textareaCommentRef.current.parentNode.classList.toggle('everhour-error', !isValidComment);
      this.textareaCommentRef.current.classList.toggle('everhour-error', !isValidComment);
      if (!isValidComment) {
        this.textareaCommentRef.current.parentNode.setAttribute('data-error', '250 MAX');
        return;
      }
      this.textareaCommentRef.current.parentNode.removeAttribute('data-error');
    }
  }, {
    key: "checkValidComment",
    value: function checkValidComment() {
      return !this.state.comment || this.state.comment && this.state.comment.length <= 250;
    }
  }, {
    key: "stopTimer",
    value: function stopTimer() {
      var _this2 = this;
      var currentUserId = this.props.store.profile.get().id;
      var editedUserId = this.props.user.user.id;
      var comment = this.state.comment || null;
      if (currentUserId !== editedUserId) {
        this.setValidationComment();
      }
      if (currentUserId !== editedUserId && comment && !this.checkValidComment()) {
        return;
      }
      this.props.store.timer.stop(editedUserId, comment).then(function () {
        _this2.props.onClose();
      });
    }
  }, {
    key: "render",
    value: function render() {
      return /*#__PURE__*/React.createElement("div", null, this.state.error.isShowError && /*#__PURE__*/React.createElement("div", {
        className: "modal__error"
      }, this.state.error.message), /*#__PURE__*/React.createElement("div", {
        className: "stop-confirm"
      }, /*#__PURE__*/React.createElement("div", {
        className: "stop-confirm__description"
      }, "Timer will be stopped and we will notify ", this.props.user.user.name, " about it, mentioning who did it and why."), /*#__PURE__*/React.createElement("div", {
        className: "stop-confirm__reason"
      }, /*#__PURE__*/React.createElement("div", {
        className: "field__wrapper"
      }, /*#__PURE__*/React.createElement("label", {
        className: "field__label",
        htmlFor: "textarea-comment"
      }, "Reason For Stopping"), /*#__PURE__*/React.createElement("div", {
        className: "field__body"
      }, /*#__PURE__*/React.createElement("textarea", {
        id: "textarea-comment",
        ref: this.textareaCommentRef,
        className: "field__textarea",
        name: "commentLine",
        placeholder: "e.g. You left home but left the timer running. Remember it next time.",
        rows: "3",
        onChange: this.onChangeComment.bind(this),
        value: this.comment
      }))))), /*#__PURE__*/React.createElement("div", {
        className: "modal__content-actions"
      }, /*#__PURE__*/React.createElement("button", {
        type: "button",
        className: "content-actions__button content-actions__button--submit-red",
        onClick: this.stopTimer.bind(this)
      }, "Stop Timer"), /*#__PURE__*/React.createElement("button", {
        type: "button",
        className: "content-actions__button content-actions__button--close",
        onClick: this.props.onClose
      }, "Cancel")));
    }
  }]);
}(React.Component);
{
  StopTimerModal;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var GroupProjectRow = /*#__PURE__*/function (_React$Component) {
  function GroupProjectRow(props) {
    var _this;
    _classCallCheck(this, GroupProjectRow);
    _this = _callSuper(this, GroupProjectRow, [props]);
    _this.isAdmin = _this.props.store.profile.isAdmin();
    return _this;
  }
  _inherits(GroupProjectRow, _React$Component);
  return _createClass(GroupProjectRow, [{
    key: "handleProjectClick",
    value: function handleProjectClick() {
      this.props.onSelectProject(this.props.project.id);
    }
  }, {
    key: "render",
    value: function render() {
      var project = this.props.project;
      var client = project.client ? this.props.store.clients.get(project.client) : null;
      var clientIntegrationIcon = client ? getPlatformName(client.externalId || 'ev') : null;
      return /*#__PURE__*/React.createElement("div", {
        className: "group__projects-row",
        "data-key-id": project.id,
        onClick: this.handleProjectClick.bind(this)
      }, /*#__PURE__*/React.createElement("div", {
        className: "project-row__icon"
      }, /*#__PURE__*/React.createElement("div", {
        className: "integration-icon integration-".concat(getPlatformName(project.platform))
      })), /*#__PURE__*/React.createElement("div", {
        className: "project-row__wrapper"
      }, /*#__PURE__*/React.createElement("span", {
        className: "project-row__name"
      }, /*#__PURE__*/React.createElement(HighlightText, {
        searchTerm: this.props.searchTerm
      }, project.name)), project.workspaceName ? /*#__PURE__*/React.createElement("span", {
        className: "project-row__name-workspace"
      }, /*#__PURE__*/React.createElement(HighlightText, {
        searchTerm: this.props.searchTerm
      }, project.workspaceName)) : ''), client && this.isAdmin ? /*#__PURE__*/React.createElement("div", {
        className: "project-row__client"
      }, /*#__PURE__*/React.createElement("div", {
        className: "project-row__client-name"
      }, client.name), clientIntegrationIcon ? /*#__PURE__*/React.createElement("div", {
        className: "project-row__client-icon integration-icon\n                                              integration-".concat(clientIntegrationIcon)
      }) : '') : '');
    }
  }]);
}(React.Component);
{
  GroupProjectRow;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var GroupProjects = /*#__PURE__*/function (_React$Component) {
  function GroupProjects(props) {
    _classCallCheck(this, GroupProjects);
    return _callSuper(this, GroupProjects, [props]);
  }
  _inherits(GroupProjects, _React$Component);
  return _createClass(GroupProjects, [{
    key: "handleSelectProject",
    value: function handleSelectProject(projectId) {
      this.props.onSelectProject(projectId);
    }
  }, {
    key: "render",
    value: function render() {
      var _this = this;
      var projects = this.props.projects;
      return /*#__PURE__*/React.createElement(ScrollingContainer, {
        eventSelectorId: 'search-tab',
        listContainerSelector: '.group__projects.group__projects-scrolling-container-group-projects',
        itemSelector: '.group__projects-row:not(.group__projects-row--title)',
        onSelect: this.handleSelectProject.bind(this)
      }, /*#__PURE__*/React.createElement("div", {
        className: "group__projects group__projects-scrolling-container-group-projects"
      }, Object.keys(projects).map(function (projectType) {
        var projectsArray = projects[projectType] && projects[projectType].projects;
        if (projectsArray && projectsArray.length) {
          return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
            className: "group__projects-row group__projects-row--title",
            key: 'project-row-recent'
          }, projects[projectType].label), projectsArray.map(function (project) {
            return /*#__PURE__*/React.createElement(GroupProjectRow, {
              project: project,
              store: _this.props.store,
              searchTerm: _this.props.searchTerm,
              key: "project-row-recent-".concat(project.id),
              onSelectProject: _this.handleSelectProject.bind(_this)
            });
          }));
        }
        return '';
      })));
    }
  }]);
}(React.Component);
{
  GroupProjects;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var GroupTasks = /*#__PURE__*/function (_React$Component) {
  function GroupTasks(props) {
    _classCallCheck(this, GroupTasks);
    return _callSuper(this, GroupTasks, [props]); // console.log('[EV][GroupTasks] ', this.props)
  }
  _inherits(GroupTasks, _React$Component);
  return _createClass(GroupTasks, [{
    key: "handleSelectTask",
    value: function handleSelectTask(taskId) {
      this.props.onSelectTask(taskId);
    }
  }, {
    key: "handleNavigateToTimeTab",
    value: function handleNavigateToTimeTab() {
      this.props.onNavigateToTimeTab();
    }
  }, {
    key: "render",
    value: function render() {
      var _this = this;
      var tasks = this.props.tasks;
      return /*#__PURE__*/React.createElement(ScrollingContainer, {
        eventSelectorId: 'search-tab',
        listContainerSelector: '.group__tasks.group__tasks-scrolling-container-group-tasks',
        itemSelector: '.task-row:not(.task-row--title)',
        onSelect: this.handleSelectTask.bind(this)
      }, /*#__PURE__*/React.createElement("div", {
        className: "group__tasks group__tasks-scrolling-container-group-tasks"
      }, Object.keys(tasks).map(function (taskType) {
        var tasksArray = tasks[taskType] && tasks[taskType].tasks;
        if (tasksArray && tasksArray.length) {
          return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
            className: "task-row task-row--title",
            key: 'task-row-recent'
          }, tasks[taskType].label), tasksArray.map(function (task) {
            var isComment = task._type === 'comment' || task.type === 'comment';
            return /*#__PURE__*/React.createElement(RecentTaskRow, {
              isSearchTab: false,
              task: task,
              taskTime: !isComment ? {
                task: task
              } : null,
              taskTimeId: !isComment && task.id,
              time: task.time,
              comment: task.comment,
              key: "task-row-recent-".concat(task.id || task._id),
              searchTerm: _this.props.searchTerm,
              isTaskOnly: true,
              api: _this.props.api,
              store: _this.props.store,
              reactAppStateObj: _this.props.reactAppStateObj,
              navigateToTimeTab: _this.handleNavigateToTimeTab.bind(_this)
            });
          }));
        }
        return '';
      })));
    }
  }]);
}(React.Component);
{
  GroupTasks;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TaskDetailsBilling = /*#__PURE__*/function (_React$Component) {
  function TaskDetailsBilling(props) {
    var _this;
    _classCallCheck(this, TaskDetailsBilling);
    _this = _callSuper(this, TaskDetailsBilling, [props]);
    var currency = _format.getCurrency();
    _this.position = currency.symbol || !currency.symbolNarrow ? 'left' : 'right';
    _this.popoverControllerService = new PopupControllerService();
    _this.popover2Service = new Popover2Service();
    _this.state = {
      isShowBilling: false,
      isCanBillable: false,
      hasAbilityCustomRate: false,
      billingType: null,
      customRateValue: '',
      customRateError: '',
      isValidCustomRate: true,
      isProjectBillable: false
    };
    _this.billings = [{
      id: 'project_rate',
      name: 'Use rate from project settings'
    }, {
      id: 'user_rate',
      name: 'Custom task rate'
    }, {
      id: 'non_billable',
      name: 'Non-billable'
    }];
    _this.isSaveByEnter = false;
    return _this;
  }
  _inherits(TaskDetailsBilling, _React$Component);
  return _createClass(TaskDetailsBilling, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this.initBillingPopover();
    }
  }, {
    key: "initBillingPopover",
    value: function initBillingPopover() {
      var task = this.props.task;
      var store = this.props.store;
      this.setState({
        isShowBilling: store.profile.isAdmin() && store.profile.isAllowBillingAndBudget(),
        isCanBillable: this.popoverControllerService.checkIsBillable(task.projectsObjects),
        hasAbilityCustomRate: this.popoverControllerService.hasAbilityCustomRate(task.projectsObjects),
        billingType: this.getCurrentBillingType(),
        customRateValue: this.getCurrentRate(),
        customRateError: '',
        isValidCustomRate: true,
        isProjectBillable: this.popoverControllerService.isProjectBillable(this.props.project)
      });
    }
  }, {
    key: "getCurrentBillingType",
    value: function getCurrentBillingType() {
      var task = this.props.task;
      return task.unbillable ? 'non_billable' : !task.unbillable && task.rate ? 'user_rate' : 'project_rate';
    }
  }, {
    key: "getCurrentRate",
    value: function getCurrentRate() {
      var task = this.props.task;
      return !task.unbillable && task.rate ? task.rate / 100 + '' : '';
    }
  }, {
    key: "billableTooltipText",
    value: function billableTooltipText() {
      var task = this.props.task;
      var state = this.state;
      if (this.state.isCanBillable === false) {
        return null;
      }
      if (task.unbillable) {
        return 'This is a non-billable task. The time that you track to it will be excluded from the project\'s billable amount';
      }
      if (task.rate && state.hasAbilityCustomRate) {
        return 'This is a custom rate that applies only to time of this task';
      }
      if (state.hasAbilityCustomRate) {
        return 'This is a task in a billable project. Click to make it non-billable or set a different rate from the project billing rates';
      }
      return 'This is a task in a billable project. Click to mark it as non-billable to exclude from the project billable amount';
    }
  }, {
    key: "billableText",
    value: function billableText() {
      var task = this.props.task;
      var state = this.state;
      if (task.unbillable === true || state.isCanBillable === false) {
        return 'Non-billable';
      }
      if (task.rate && state.hasAbilityCustomRate) {
        return "Billable ".concat(this.popoverControllerService.getMoneyWithSymbol(task.rate, 'narrow'), "/h");
      }
      return 'Billable';
    }
  }, {
    key: "handleClickBilling",
    value: function handleClickBilling() {
      var state = this.state;
      var task = this.props.task;
      if (!state.isCanBillable) {
        return;
      }
      var requestObj = {
        unbillable: !task.unbillable
      };
      if (!task.unbillable && task.rate) {
        requestObj.rate = 0;
      }
      return this.updateBillableTask(task.id, requestObj);
    }
  }, {
    key: "handleChangeBillingType",
    value: function handleChangeBillingType(billingType) {
      this.setState({
        billingType: billingType.id
      });
      this.subscribeCustomRateKeydown();
    }
  }, {
    key: "onToggleBillingPopover",
    value: function onToggleBillingPopover(isOpen) {
      if (!isOpen) {
        this.save();
        return;
      }
      this.initBillingPopover();
    }
  }, {
    key: "save",
    value: function save() {
      var state = this.state;
      var currentBillingType = this.getCurrentBillingType();
      var currentRate = this.getCurrentRate();
      if (state.billingType === 'user_rate' && !state.isValidCustomRate) {
        return;
      }
      var rate = state.billingType === 'user_rate' && state.customRateValue ? state.customRateValue * 100 : null;
      if (state.billingType === 'user_rate' && state.isValidCustomRate && currentRate === state.customRateValue) {
        return;
      }
      if (state.billingType !== 'user_rate' && currentBillingType === state.billingType) {
        return;
      }
      var isUnbillable = state.billingType === 'non_billable' || state.billingType === 'user_rate' && !!~['', '0', null].indexOf(state.customRateValue);
      var requestBody = {
        unbillable: isUnbillable
      };
      var taskId = this.props.task.id;
      if (rate) {
        requestBody.rate = rate;
      }
      if (isUnbillable) {
        requestBody.rate = 0;
      }
      if (!isUnbillable && state.billingType === 'project_rate') {
        requestBody.rate = null;
      }
      return this.updateBillableTask(taskId, requestBody);
    }
  }, {
    key: "updateBillableTask",
    value: function updateBillableTask(taskId, requestBody) {
      var _this2 = this;
      return this.props.api.updateTaskBillingSettings(taskId, requestBody).then(function (task) {
        _this2.props.onUpdateTask();
      });
    }
  }, {
    key: "checkValidCustomRate",
    value: function checkValidCustomRate(val) {
      if (!val) {
        return {
          isValid: true,
          message: ''
        };
      }
      var isValidNumber = /^\d*(\.\d+)?$/.test(val);
      if (!isValidNumber) {
        return {
          isValid: false,
          message: ''
        };
      }
      if (isValidNumber && val > 20000000) {
        return {
          isValid: false,
          message: '20000000 MAX'
        };
      }
      return {
        isValid: isValidNumber,
        message: ''
      };
    }
  }, {
    key: "handleChangeCustomRate",
    value: function handleChangeCustomRate(event) {
      var val = event.target.value;
      var validationState = this.checkValidCustomRate(val);
      this.setState({
        customRateValue: val,
        customRateError: validationState.message,
        isValidCustomRate: validationState.isValid
      });
    }
  }, {
    key: "handleCloseBillingPopover",
    value: function handleCloseBillingPopover() {
      this.popover2Service.closeAll();
    }
  }, {
    key: "subscribeCustomRateKeydown",
    value: function subscribeCustomRateKeydown() {
      var self = this;
      var customRateElement = document.querySelector('.custom-rate__input');
      if (customRateElement) {
        customRateElement.addEventListener('keydown', function (event) {
          if (event.keyCode === 13 && self.checkValidCustomRate().isValid && !self.isSaveByEnter) {
            self.handleCloseBillingPopover();
          }
        });
      }
    }
  }, {
    key: "renderBilling",
    value: function renderBilling() {
      var state = this.state;
      var billableText = this.billableText();
      return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
        className: "header-billing__icon\n                    header-billing__icon--".concat(!!~billableText.toLocaleLowerCase().indexOf('non-billable') ? 'non-billable' : 'billable')
      }, /*#__PURE__*/React.createElement("span", {
        className: "icon-dollar"
      })), /*#__PURE__*/React.createElement("div", {
        className: "\n                header-billing__text\n                header-billing__project--".concat(!state.isProjectBillable ? 'non-billable' : 'billable', "\n                header-billing__text--").concat(!!~billableText.toLocaleLowerCase().indexOf('non-billable') ? 'non-billable' : 'billable')
      }, billableText));
    }
  }, {
    key: "render",
    value: function render() {
      var _this3 = this;
      var state = this.state;
      if (!state.isShowBilling) {
        return '';
      }
      var billingTooltipText = this.billableTooltipText();
      var currencySymbol = _format.getCurrencySymbol();
      if (state.hasAbilityCustomRate) {
        return /*#__PURE__*/React.createElement(Popover2, {
          triggerNode: /*#__PURE__*/React.createElement("div", {
            className: "task-details__header-billing ".concat(state.isCanBillable ? 'interactive' : 'locked-visible')
          }, /*#__PURE__*/React.createElement(Tooltip, {
            space: 5,
            placement: 'bottom',
            triggerTemplate: function triggerTemplate() {
              return _this3.renderBilling();
            }
          }, billingTooltipText)),
          trigger: "click",
          placement: "bottom-left",
          onToggle: this.onToggleBillingPopover.bind(this),
          width: 296
        }, /*#__PURE__*/React.createElement("div", {
          className: "billing-popover"
        }, /*#__PURE__*/React.createElement("div", {
          className: "billing-popover__header"
        }, /*#__PURE__*/React.createElement("div", {
          className: "billing-popover__header-title"
        }, "Billing"), /*#__PURE__*/React.createElement("div", {
          className: "billing-popover__header-close icon-close-small",
          onClick: this.handleCloseBillingPopover.bind(this)
        })), /*#__PURE__*/React.createElement("div", {
          className: "billing-popover__content"
        }, this.billings.map(function (billing) {
          return /*#__PURE__*/React.createElement("div", {
            className: "billing-popover__content-item",
            onClick: _this3.handleChangeBillingType.bind(_this3, billing)
          }, /*#__PURE__*/React.createElement("div", {
            className: "content-item__radio"
          }, /*#__PURE__*/React.createElement("div", {
            className: "icon-radio ".concat(state.billingType === billing.id ? 'icon-radio-selected' : 'icon-radio-normal')
          }), /*#__PURE__*/React.createElement("div", {
            className: "content-item__radio-text"
          }, billing.name)), state.billingType === billing.id && state.billingType === 'user_rate' ? /*#__PURE__*/React.createElement("div", {
            className: "custom-rate__wrapper\n                                                 custom-rate__wrapper--rate-position-".concat(_this3.position, "\n                                                 ").concat(!state.isValidCustomRate ? 'custom-rate__wrapper--invalid' : ''),
            "data-error": state.customRateError
          }, _this3.position === 'left' ? /*#__PURE__*/React.createElement("span", {
            className: "icon-rate"
          }, currencySymbol) : '', /*#__PURE__*/React.createElement("input", {
            type: "text",
            name: "rate",
            autoComplete: "off",
            className: "custom-rate__input",
            value: state.customRateValue,
            onChange: _this3.handleChangeCustomRate.bind(_this3)
          }), _this3.position === 'right' ? /*#__PURE__*/React.createElement("span", {
            className: "icon-rate"
          }, currencySymbol) : '') : '');
        }))));
      }
      if (billingTooltipText) {
        return /*#__PURE__*/React.createElement("div", {
          className: "task-details__header-billing ".concat(state.isCanBillable ? 'interactive' : 'locked-visible'),
          onClick: this.handleClickBilling.bind(this)
        }, /*#__PURE__*/React.createElement(Tooltip, {
          space: 5,
          placement: 'bottom',
          triggerTemplate: function triggerTemplate() {
            return _this3.renderBilling();
          }
        }, billingTooltipText));
      }
      return /*#__PURE__*/React.createElement("div", {
        className: "task-details__header-billing ".concat(state.isCanBillable ? 'interactive' : 'locked-visible'),
        onClick: this.handleClickBilling.bind(this)
      }, this.renderBilling());
    }
  }]);
}(React.Component);
{
  TaskDetailsBilling;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TaskDetailsUserProgress = /*#__PURE__*/function (_React$Component) {
  function TaskDetailsUserProgress(props) {
    var _this;
    _classCallCheck(this, TaskDetailsUserProgress);
    _this = _callSuper(this, TaskDetailsUserProgress, [props]);
    _this.isSafari = browserPort.type === 'safari';
    return _this;
  }
  _inherits(TaskDetailsUserProgress, _React$Component);
  return _createClass(TaskDetailsUserProgress, [{
    key: "handleOpenEditTimeModal",
    value: function handleOpenEditTimeModal() {
      var props = this.props;
      if (props.isViewOnly || !props.total.time) {
        return;
      }
      props.onOpenEditTimeModal(this.props.user.id);
    }
  }, {
    key: "renderAvatar",
    value: function renderAvatar(user) {
      return /*#__PURE__*/React.createElement(React.Fragment, null, user.avatarUrl ? /*#__PURE__*/React.createElement("img", {
        src: user.avatarUrl,
        className: "avatar avatar--image"
      }) : '', !user.avatarUrl ? /*#__PURE__*/React.createElement("span", {
        className: "avatar avatar--text"
      }, _format.getAvatarName(user)) : '');
    }
  }, {
    key: "renderUserName",
    value: function renderUserName(user) {
      return /*#__PURE__*/React.createElement(React.Fragment, null, user.name || user.email);
    }
  }, {
    key: "render",
    value: function render() {
      var props = this.props;
      var user = props.user;
      var taskId = props.taskId;
      var total = props.total;
      var isAdmin = this.props.store.profile.isAdmin();
      return /*#__PURE__*/React.createElement("div", {
        className: "timer__progress"
      }, /*#__PURE__*/React.createElement("div", {
        className: "progress__avatar"
      }, !this.isSafari && !props.isViewOnly && isAdmin ? /*#__PURE__*/React.createElement("a", {
        href: "".concat(everhour.productionUrl, "/#/members/").concat(user.id, "(view:").concat(taskId, ")"),
        className: "text-link",
        target: "_blank"
      }, /*#__PURE__*/React.createElement(UserAvatar, {
        user: user
      })) : /*#__PURE__*/React.createElement(UserAvatar, {
        user: user
      })), /*#__PURE__*/React.createElement("div", {
        className: "progress__summary"
      }, /*#__PURE__*/React.createElement("div", {
        className: "summary__user-name"
      }, !this.isSafari && !props.isViewOnly && isAdmin ? /*#__PURE__*/React.createElement("a", {
        href: "".concat(everhour.productionUrl, "/#/members/").concat(user.id, "(view:").concat(taskId, ")"),
        className: "text-link",
        target: "_blank"
      }, this.renderUserName(user)) : /*#__PURE__*/React.createElement("span", null, this.renderUserName(user))), total.time > 0 || total.estimate > 0 ? /*#__PURE__*/React.createElement("div", {
        className: "summary__user-hours\n                                       ".concat(total.time > 0 && !props.isViewOnly ? ' interactive' : '', "\n                                       ").concat(total.time && total.estimate && total.estimate < total.time ? ' overtime' : ''),
        onClick: this.handleOpenEditTimeModal.bind(this)
      }, /*#__PURE__*/React.createElement("span", {
        className: "user-hours__time"
      }, total.time > 0 ? TimeEntryUtils.format(total.time) : '0h'), total.estimate ? /*#__PURE__*/React.createElement("span", {
        className: "user-hours__estimate"
      }, "\xA0of ", total.estimate > 0 ? TimeEntryUtils.format(total.estimate) : '0h') : '') : ''));
    }
  }]);
}(React.Component);
{
  TaskDetailsUserProgress;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TaskDetailsInfo = /*#__PURE__*/function (_React$Component) {
  function TaskDetailsInfo(props) {
    var _this;
    _classCallCheck(this, TaskDetailsInfo);
    _this = _callSuper(this, TaskDetailsInfo, [props]);
    _this.popoverControllerService = new PopupControllerService();
    _this.state = _this.getInitState();
    _this.initDetails();
    _this.isSafari = browserPort.type === 'safari';
    browserPort.eventDispatcher.on('browser-button', 'details-updated', function () {
      _this.initDetails();
    });
    return _this;
  }
  _inherits(TaskDetailsInfo, _React$Component);
  return _createClass(TaskDetailsInfo, [{
    key: "initDetails",
    value: function initDetails() {
      this.setState(this.getInitState());
    }
  }, {
    key: "getInitState",
    value: function getInitState() {
      var props = this.props;
      var taskDetails = props.store.taskDetails.getTask(props.taskId);
      var task = taskDetails.task;
      var taskId = props.taskId;
      var platform = getPlatformCode(taskId);
      var project = this.popoverControllerService.taskProject(task.projects, task.projectsObjects);
      var taskType = this.popoverControllerService.getTaskType(taskId, task.type);
      var isExternal = platform === 'everhour';
      this.position = this.props.currencyPosition;
      return {
        isAdmin: props.store.profile.isAdmin(),
        isMember: !!~['member', 'member_limited'].indexOf(props.store.profile.get().role),
        task: task,
        project: project,
        taskName: taskDetails.task.name,
        taskType: taskType,
        taskTypeName: platform === 'bitbucket' || platform === 'jira' ? task.type : taskType,
        taskTypeLabel: taskType === 'pull' || taskType === 'pull_request' ? 'pull request' : taskType,
        taskLabels: taskDetails.task.labels,
        taskTags: taskDetails.task.tags,
        isExternal: isExternal,
        isEditable: props.isEditable,
        assignee: taskDetails.task.assignees,
        status: this.popoverControllerService.transformStatus(task) !== 'closed' ? 'open' : 'closed',
        accuracy: task.time ? Math.round(task.time.timerTime / task.time.total * 100) : 100,
        billableAmount: this.popoverControllerService.getMoneyRound(this.getBillableAmount(task, taskDetails.collaborators, task.projectsObjects)),
        cost: this.popoverControllerService.getMoneyRound(props.cost),
        canManageCosts: store.profile.isAllowCosts(),
        canManageBilling: store.profile.isAllowBillingAndBudget()
      };
    }
  }, {
    key: "getBillableAmount",
    value: function getBillableAmount(task, collaborators) {
      var _this2 = this;
      if (this.props.store.profile.isAdmin()) {
        var rate = 0;
        var totalBillableAmount = 0;
        var hasAbilityCustomRate = this.popoverControllerService.hasAbilityCustomRate(task.projectsObjects);
        collaborators = collaborators.map(function (c) {
          c.progress = {
            time: task.time && task.time.users && task.time.users[c.id] || 0,
            estimate: task.estimate && task.estimate.users && task.estimate[c.id] || 0
          };
          return c;
        });
        collaborators.forEach(function (participant) {
          var hours = participant.progress.time / 3600;
          if (hasAbilityCustomRate && task.rate) {
            rate = task.rate;
          } else {
            var user = _this2.props.store.users.get(participant.id);
            if (user) {
              var billing = _this2.popoverControllerService.billing({
                user: participant.id,
                task: task
              }, null, task.projectsObjects);
              rate = billing.rate;
            }
          }
          totalBillableAmount += rate * hours;
        });
        return totalBillableAmount;
      }
      return 0;
    }
  }, {
    key: "renderAssigneeUser",
    value: function renderAssigneeUser(user) {
      var isAccountUser = _typeof(user) === 'object';
      var userObj = null;
      if (isAccountUser && !user.userId) {
        userObj = user;
        userObj.id = userObj.accountId;
        userObj.name = userObj.accountName;
      }
      if (isAccountUser && user.userId) {
        userObj = this.props.store.users.get(user.userId);
      }
      if (!isAccountUser) {
        userObj = this.props.store.users.get(user);
      }
      return /*#__PURE__*/React.createElement("div", {
        className: "timer__progress assignee__user"
      }, /*#__PURE__*/React.createElement("div", {
        className: "progress__avatar"
      }, /*#__PURE__*/React.createElement(UserAvatar, {
        user: userObj
      })), /*#__PURE__*/React.createElement("div", {
        className: "progress__summary"
      }, /*#__PURE__*/React.createElement("div", {
        className: "summary__user-name"
      }, userObj.name || userObj.email)));
    }
  }, {
    key: "isNumeric",
    value: function isNumeric(str) {
      if (typeof str !== "string") return false;
      return !isNaN(str) && !isNaN(parseFloat(str));
    }
  }, {
    key: "render",
    value: function render() {
      var _this3 = this;
      var state = this.state;
      var currencySymbol = _format.getCurrencySymbol();
      var dueDate = this.popoverControllerService.getDueDateLabel(state.task.dueOn);
      return /*#__PURE__*/React.createElement("div", {
        className: "task-details__description"
      }, state.assignee && state.assignee.length ? /*#__PURE__*/React.createElement("div", {
        className: "task-details__description-row task-details__description-row--va-middle"
      }, /*#__PURE__*/React.createElement("div", {
        className: "description-row__label"
      }, "Assignee"), /*#__PURE__*/React.createElement("div", {
        className: "description-row__column description-row__column-assignee\n                               ".concat(state.assignee.length > 1 ? 'description-row__column-assignee--collapsed' : '')
      }, state.assignee.slice(0, 10).map(function (user) {
        return _this3.renderAssigneeUser(user);
      }), state.assignee.length > 10 ? /*#__PURE__*/React.createElement("div", {
        className: "description-row__column-assignee-count"
      }, state.assignee.length) : '')) : '', state.task.dueOn ? /*#__PURE__*/React.createElement("div", {
        className: "task-details__description-row task-details__description-row--va-middle"
      }, /*#__PURE__*/React.createElement("div", {
        className: "description-row__label"
      }, "Due date"), /*#__PURE__*/React.createElement("div", {
        className: "description-row__column ".concat(dueDate.color ? 'description-row__column--task-duedate-' + dueDate.color : '')
      }, dueDate.label)) : '', /*#__PURE__*/React.createElement("div", {
        className: "task-details__description-row"
      }, /*#__PURE__*/React.createElement("div", {
        className: "description-row__label"
      }, "Project"), /*#__PURE__*/React.createElement("div", {
        className: "description-row__column description-row__column-project"
      }, !state.isExternal ? state.task.projectsObjects.map(function (project) {
        return /*#__PURE__*/React.createElement("div", {
          className: "column-project__row"
        }, /*#__PURE__*/React.createElement("div", {
          className: "column-project__integration-icon integration-icon\n                                     integration-".concat(getPlatformName(project.id))
        }), /*#__PURE__*/React.createElement("div", {
          className: "column-project__external-project-name"
        }, !_this3.isSafari ? /*#__PURE__*/React.createElement("a", {
          href: "".concat(everhour.productionUrl, "/#/projects/").concat(project.id, "/").concat(_this3.isAdmin ? '/summary' : '/tasks'),
          className: "text-link",
          target: "_blank"
        }, project.name) : project.name), state.task.iteration ? /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
          className: "project-name__separator"
        }, "\u203A"), /*#__PURE__*/React.createElement("div", {
          className: "column-project__project-name"
        }, state.task.iteration)) : '');
      }) : '', state.isExternal ? /*#__PURE__*/React.createElement("div", {
        className: "column-project__row"
      }, /*#__PURE__*/React.createElement("div", {
        className: "column-project__integration-icon integration-icon\n                                     integration-".concat(getPlatformName(state.project.id))
      }), /*#__PURE__*/React.createElement("div", {
        className: "column-project__external-project-name"
      }, !this.isSafari ? /*#__PURE__*/React.createElement("a", {
        href: "".concat(everhour.productionUrl, "/#/projects/").concat(state.project.id),
        className: "text-link",
        target: "_blank"
      }, state.project.name) : state.project.name), state.task.iteration ? /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
        className: "project-name__separator"
      }, "\u203A"), /*#__PURE__*/React.createElement("div", {
        className: "column-project__project-name"
      }, state.task.iteration)) : /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
        className: "project-name__separator"
      }, "\u203A"), /*#__PURE__*/React.createElement("div", {
        className: "column-project__project-name"
      }, "Untitled"))) : '')), state.taskType && !state.isExternal ? /*#__PURE__*/React.createElement("div", {
        className: "task-details__description-row task-details__description-row--va-middle"
      }, /*#__PURE__*/React.createElement("div", {
        className: "description-row__label"
      }, "Type"), /*#__PURE__*/React.createElement("div", {
        className: "description-row__column description-row__column-task-type"
      }, /*#__PURE__*/React.createElement("span", {
        className: "task-type task-type-inline ".concat(state.taskType, " ").concat(state.status)
      }), state.taskTypeLabel)) : '', state.taskLabels && state.taskLabels.length && !state.isExternal ? /*#__PURE__*/React.createElement("div", {
        className: "task-details__description-row \n                                     ".concat(state.taskLabels && state.taskLabels.length ? 'task-details__description-row--tags' : '')
      }, /*#__PURE__*/React.createElement("div", {
        className: "description-row__label"
      }, "Tags"), /*#__PURE__*/React.createElement("div", {
        className: "description-row__column description-row__column-tags"
      }, state.taskLabels && state.taskLabels.length ? state.taskLabels.map(function (label) {
        return /*#__PURE__*/React.createElement("div", {
          className: "column-label"
        }, label);
      }) : '-')) : '', state.taskTags && state.taskTags.length || state.isExternal && state.isEditable ? /*#__PURE__*/React.createElement("div", {
        className: "task-details__description-row \n                                     ".concat(state.taskTags && state.taskTags.length ? 'task-details__description-row--tags' : '')
      }, /*#__PURE__*/React.createElement("div", {
        className: "description-row__label"
      }, "Tags"), /*#__PURE__*/React.createElement("div", {
        className: "description-row__column description-row__column-tags"
      }, state.taskTags && state.taskTags.length ? /*#__PURE__*/React.createElement(React.Fragment, null, state.taskTags && state.taskTags.length ? state.taskTags.map(function (tag) {
        return /*#__PURE__*/React.createElement("div", {
          className: "column-tag",
          style: {
            backgroundColor: tag.color
          }
        }, tag.name);
      }) : '') : '-')) : '', state.task.time && state.task.time.total ? /*#__PURE__*/React.createElement("div", {
        className: "task-details__description-row task-details__description-row--va-middle"
      }, /*#__PURE__*/React.createElement("div", {
        className: "description-row__label"
      }, "Timer"), /*#__PURE__*/React.createElement("div", {
        className: "description-row__column"
      }, state.accuracy, "%")) : '', state.billableAmount && state.billableAmount !== '0' && state.isAdmin && !state.task.unbillable && state.canManageBilling ? /*#__PURE__*/React.createElement("div", {
        className: "task-details__description-row task-details__description-row--va-middle"
      }, /*#__PURE__*/React.createElement("div", {
        className: "description-row__label"
      }, "Billable"), /*#__PURE__*/React.createElement("div", {
        className: "description-row__column"
      }, this.position === 'left' ? "".concat(currencySymbol).concat(state.billableAmount) : "".concat(state.billableAmount, " ").concat(currencySymbol))) : '', state.cost && state.cost !== '0' && state.isAdmin && state.canManageCosts ? /*#__PURE__*/React.createElement("div", {
        className: "task-details__description-row task-details__description-row--va-middle"
      }, /*#__PURE__*/React.createElement("div", {
        className: "description-row__label"
      }, "Costs"), /*#__PURE__*/React.createElement("div", {
        className: "description-row__column"
      }, this.position === 'left' ? "".concat(currencySymbol).concat(state.cost) : "".concat(state.cost, " ").concat(currencySymbol))) : '', state.task.description ? /*#__PURE__*/React.createElement("div", {
        className: "task-details__description-row"
      }, /*#__PURE__*/React.createElement("div", {
        className: "description-row__label"
      }, "Description"), /*#__PURE__*/React.createElement("div", {
        className: "description-row__column description-row__column--task-description"
      }, state.task.description)) : '');
    }
  }]);
}(React.Component);
{
  TaskDetailsInfo;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TaskDetailsAssigneeUser = /*#__PURE__*/function (_React$Component) {
  function TaskDetailsAssigneeUser(props) {
    _classCallCheck(this, TaskDetailsAssigneeUser);
    return _callSuper(this, TaskDetailsAssigneeUser, [props]);
  }
  _inherits(TaskDetailsAssigneeUser, _React$Component);
  return _createClass(TaskDetailsAssigneeUser, [{
    key: "render",
    value: function render() {
      var user = this.props.store.users.get(this.props.userId);
      return /*#__PURE__*/React.createElement("div", {
        className: "timer__progress assignee__user"
      }, /*#__PURE__*/React.createElement("div", {
        className: "progress__avatar"
      }, /*#__PURE__*/React.createElement(UserAvatar, {
        user: user
      })), /*#__PURE__*/React.createElement("div", {
        className: "progress__summary"
      }, /*#__PURE__*/React.createElement("div", {
        className: "summary__user-name"
      }, user.name || user.email)));
    }
  }]);
}(React.Component);
{
  TaskDetailsAssigneeUser;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TaskDetailsAssigneeUserTime = /*#__PURE__*/function (_React$Component) {
  function TaskDetailsAssigneeUserTime(props) {
    var _this;
    _classCallCheck(this, TaskDetailsAssigneeUserTime);
    _this = _callSuper(this, TaskDetailsAssigneeUserTime, [props]);
    _this.state = {
      isTimeInputFocus: false
    };
    _this.popupControllerService = new PopupControllerService();
    return _this;
  }
  _inherits(TaskDetailsAssigneeUserTime, _React$Component);
  return _createClass(TaskDetailsAssigneeUserTime, [{
    key: "showTimeInput",
    value: function showTimeInput(isCanEdit) {
      if (!isCanEdit) {
        return;
      }
      this.setState({
        isTimeInputFocus: true
      });
    }
  }, {
    key: "hideTimeInput",
    value: function hideTimeInput() {
      this.setState({
        isTimeInputFocus: false
      });
    }
  }, {
    key: "onToggleMenu",
    value: function onToggleMenu(isShowMenu) {
      this.setState({
        isShowMenu: isShowMenu
      });
    }
  }, {
    key: "handleUpdateTimeDone",
    value: function handleUpdateTimeDone() {
      this.props.onUpdateTimeDone();
    }
  }, {
    key: "handleUpdateTimeError",
    value: function handleUpdateTimeError(error) {
      this.props.onUpdateTimeError(error);
    }
  }, {
    key: "handleOpenDeleteTimeModal",
    value: function handleOpenDeleteTimeModal() {
      this.props.onOpenDeleteTimeModal();
    }
  }, {
    key: "render",
    value: function render() {
      var _this2 = this;
      var props = this.props;
      var lockReasons = this.popupControllerService.getLocktimeReasons(this.props.taskTime);
      var isCanEdit = props.isCanEdit;
      var tooltip = props.tooltip;
      return /*#__PURE__*/React.createElement(React.Fragment, null, this.state.isTimeInputFocus && isCanEdit ? /*#__PURE__*/React.createElement(TaskTimeInput, {
        taskTime: this.props.taskTime,
        userId: this.props.userId,
        api: this.props.api,
        store: this.props.store,
        hideTimeInput: this.hideTimeInput.bind(this),
        onUpdateTimeError: this.handleUpdateTimeError.bind(this),
        onUpdateTimeDone: this.handleUpdateTimeDone.bind(this)
      }) : /*#__PURE__*/React.createElement("div", {
        className: "task-row__time-static ".concat(!isCanEdit ? 'task-row__time-static--locked' : ''),
        onClick: this.showTimeInput.bind(this, isCanEdit)
      }, !isCanEdit && tooltip ? /*#__PURE__*/React.createElement(Tooltip, {
        key: this.props.taskTime.id,
        space: 10,
        triggerTemplate: function triggerTemplate() {
          return TimeEntryUtils.format(_this2.props.taskTime.time);
        }
      }, /*#__PURE__*/React.createElement("div", {
        className: "ev-task-time-tooltip",
        dangerouslySetInnerHTML: {
          __html: tooltip
        }
      })) : TimeEntryUtils.format(this.props.taskTime.time)), isCanEdit ? /*#__PURE__*/React.createElement("div", {
        className: "history-row__remove-time icon-delete",
        onClick: this.handleOpenDeleteTimeModal.bind(this)
      }) : /*#__PURE__*/React.createElement("div", {
        className: "history-row__remove-time history-row__remove-time--empty"
      }));
    }
  }]);
}(React.Component);
{
  TaskDetailsAssigneeUserTime;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TaskDetailsEditTimeRow = /*#__PURE__*/function (_React$Component) {
  function TaskDetailsEditTimeRow(props) {
    var _this;
    _classCallCheck(this, TaskDetailsEditTimeRow);
    _this = _callSuper(this, TaskDetailsEditTimeRow, [props]);
    _this.modalService = new ModalService();
    _this.popupControllerService = new PopupControllerService();
    _this.TaskTimeTooltipService = new TaskTimeTooltipService();
    _this.state = {
      isShowMenu: false,
      isCanEdit: _this.canEditTime(),
      tooltip: _this.getTooltip()
    };
    return _this;
  }
  _inherits(TaskDetailsEditTimeRow, _React$Component);
  return _createClass(TaskDetailsEditTimeRow, [{
    key: "onToggleMenu",
    value: function onToggleMenu(isShowMenu) {
      this.setState({
        isShowMenu: isShowMenu
      });
    }
  }, {
    key: "handleUpdateTimeDone",
    value: function handleUpdateTimeDone() {
      this.props.onUpdateTimeDone();
    }
  }, {
    key: "onUpdateTimeError",
    value: function onUpdateTimeError(error) {
      this.props.onUpdateTimeError(error);
    }
  }, {
    key: "handleOpenAddCommentModal",
    value: function handleOpenAddCommentModal() {
      var props = this.props;
      this.modalService.showModal(AddCommentModal, {
        task: props.task,
        comment: props.history.comment,
        api: props.api,
        store: props.store,
        isCurrent: props.task.id === props.store.timer.activeTask(),
        taskTimeId: props.history.id,
        weekDate: props.history.date,
        userId: props.history.user || null,
        onClose: this.handleUpdateTimeDone.bind(this)
      });
    }
  }, {
    key: "handleOpenDeleteTimeModal",
    value: function handleOpenDeleteTimeModal() {
      var props = this.props;
      this.modalService.showModal(DeleteTimeModal, {
        task: props.task,
        api: props.api,
        store: props.store,
        taskTimeId: props.history.id,
        weekDate: props.history.date,
        userId: props.history.user || null,
        onClose: this.handleUpdateTimeDone.bind(this)
      }, {
        title: 'Delete Time'
      });
    }
  }, {
    key: "canEditTime",
    value: function canEditTime() {
      var _this$props$history = this.props.history,
        task = _this$props$history.task,
        user = _this$props$history.user,
        date = _this$props$history.date;
      var project = this.TaskTimeTooltipService.getTaskProject(task.projects, task.projectsObjects, user);
      var editTimePermission = this.TaskTimeTooltipService.isCanEditTime(date, user, project);
      var isLockedByReasons = this.TaskTimeTooltipService.isLockedByReasons(this.props.history);
      return editTimePermission === 'permitted' && isLockedByReasons === false;
    }
  }, {
    key: "getTooltip",
    value: function getTooltip() {
      var history = this.props.history;
      var project = this.TaskTimeTooltipService.getTaskProject(history.task.projects, history.task.projectsObjects);
      var canManageProjectTime = project !== null && project.status === 'open';
      return this.TaskTimeTooltipService.getTooltip(this.props.history, canManageProjectTime, false, false);
    }
  }, {
    key: "render",
    value: function render() {
      var props = this.props;
      var history = props.history;
      var me = props.store.profile.get().id;
      var isAdmin = props.store.profile.isAdmin();
      var taskTime = Object.assign({}, history, {
        task: props.task
      });
      var isCanEdit = this.state.isCanEdit;
      return /*#__PURE__*/React.createElement("div", {
        className: "modal-edit-time__history-row\n                              ".concat(this.state.isShowMenu && isAdmin || history.user === me ? 'modal-edit-time__history-row--menu-show' : '', "\n                              ").concat(isAdmin || history.user === me ? 'interactive' : '')
      }, /*#__PURE__*/React.createElement("div", {
        className: "history-row__user-date"
      }, moment(history.date).format('MMM D')), /*#__PURE__*/React.createElement(TaskDetailsAssigneeUser, {
        userId: history.user,
        store: this.props.store
      }), isCanEdit ? /*#__PURE__*/React.createElement("div", {
        className: "history-row__add-comment icon-comment",
        onClick: this.handleOpenAddCommentModal.bind(this)
      }) : '', /*#__PURE__*/React.createElement("div", {
        className: "history-row__user-actions"
      }, /*#__PURE__*/React.createElement(TaskDetailsAssigneeUserTime, {
        userId: history.user,
        store: props.store,
        api: props.api,
        taskTime: taskTime,
        isCanEdit: isCanEdit,
        tooltip: this.state.tooltip,
        onUpdateTimeDone: this.handleUpdateTimeDone.bind(this),
        onUpdateTimeError: this.onUpdateTimeError.bind(this),
        onOpenDeleteTimeModal: this.handleOpenDeleteTimeModal.bind(this)
      })));
    }
  }]);
}(React.Component);
{
  TaskDetailsEditTimeRow;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TaskDetailsEstimatePopover = /*#__PURE__*/function (_React$Component) {
  function TaskDetailsEstimatePopover(props) {
    var _this;
    _classCallCheck(this, TaskDetailsEstimatePopover);
    _this = _callSuper(this, TaskDetailsEstimatePopover, [props]);
    _this.TaskTimeTooltipService = new TaskTimeTooltipService();
    _this.state = {
      estimateLabel: props.estimate && props.estimate.total ? TimeEntryUtils.format(props.estimate.total, '0h') : '0h',
      allEstimateInput: '',
      allEstimateInputMinutes: '',
      isValidAllEstimateInput: true,
      users: [],
      isEstimateUsers: false,
      searchTerm: '',
      isEmptyAnyUserBySearch: false,
      enableIndividual: _this.TaskTimeTooltipService.isCanManageIndividualEstimate(props.project)
    };
    _this.maxEstimateValue = 18000000; // 5000h
    _this.popover2Service = new Popover2Service();
    _this.initEstimatePopover();
    return _this;
  }
  _inherits(TaskDetailsEstimatePopover, _React$Component);
  return _createClass(TaskDetailsEstimatePopover, [{
    key: "initEstimatePopover",
    value: function initEstimatePopover() {
      var props = this.props;
      this.setState({
        allEstimateInput: props.estimate && props.estimate.total ? TimeEntryUtils.format(props.estimate.total) : '',
        allEstimateInputMinutes: props.estimate && props.estimate.total || 0,
        isValidAllEstimateInput: this.checkValidEstimate(props.estimate && props.estimate.total || 0),
        users: this.buildUsersEstimate(),
        isEstimateUsers: props.estimate && props.estimate.type !== 'overall'
      });
    }
  }, {
    key: "buildUsersEstimate",
    value: function buildUsersEstimate() {
      var _this2 = this;
      var props = this.props;

      /**
       * If user add estimate or time then was deactivated, we are needing display your time or estimate
       * */
      var ids = props.userIds ? _.clone(props.userIds) : [];
      if (props.estimate && props.estimate.users) {
        Object.keys(props.estimate.users).forEach(function (userId) {
          if (!~ids.indexOf(Number(userId))) {
            ids.push(userId);
          }
        });
      }
      var users = ids && ids.map(function (id) {
        var user = _.clone(props.store.users.get(id));
        // time, estimate fields for only visible in user progress block
        // estimateInput, estimateInputMinutes fields for input typed value(display and validation)
        user.progress = {
          time: props.time && props.time.users && props.time.users[id] || '',
          estimate: props.estimate && props.estimate.users && TimeEntryUtils.format(props.estimate.users[id]) || '0h',
          estimateInput: props.estimate && props.estimate.users && props.estimate.users[id] ? TimeEntryUtils.format(props.estimate.users[id]) : '',
          estimateInputMinutes: props.estimate && props.estimate.users && props.estimate.users[id] || 0,
          isValidEstimate: _this2.checkValidEstimate(props.estimate && props.estimate.users && props.estimate.users[id] || 0)
        };
        user.isVisible = true;
        return user;
      });
      var me = props.store.profile.get();
      var currentUserIndex = (users || []).findIndex(function (user) {
        return user.id === me.id;
      });
      if (currentUserIndex === -1) {
        return users;
      }
      var currentUser = users.splice(currentUserIndex, 1);
      users.unshift(currentUser[0]);
      return users;
    }
  }, {
    key: "onToggleEstimatePopover",
    value: function onToggleEstimatePopover(isOpen) {
      this.initEstimatePopover();
      if (isOpen) {
        browserPort.analytics.event('estimate_open');
      }
    }
  }, {
    key: "handleCloseEstimatePopover",
    value: function handleCloseEstimatePopover() {
      this.popover2Service.closeAll();
    }
  }, {
    key: "handleChangeEstimateType",
    value: function handleChangeEstimateType(event) {
      var props = this.props;
      var estimateType = props.estimate && props.estimate.type || 'overall';
      var isChecked = event.target.checked;
      var isValidAllEstimateInput = null;
      if (estimateType === 'overall') {
        isValidAllEstimateInput = isChecked ? true : this.checkValidEstimate(props.estimate && props.estimate.total || 0);
        this.setState({
          isEstimateUsers: isChecked,
          allEstimateInput: isChecked ? '' : props.estimate && props.estimate.total && TimeEntryUtils.format(props.estimate.total) || '',
          allEstimateInputMinutes: isChecked ? '' : props.estimate && props.estimate.total || 0,
          isValidAllEstimateInput: isValidAllEstimateInput,
          allEstimateValidationError: this.getEstimateValidationError(isChecked ? '' : props.estimate && props.estimate.total || 0),
          users: this.buildUsersEstimate(),
          searchTerm: '',
          isEmptyAnyUserBySearch: false
        });
        return;
      }
      this.setState({
        isEstimateUsers: isChecked,
        allEstimateInput: props.estimate && props.estimate.total && TimeEntryUtils.format(props.estimate.total),
        allEstimateInputMinutes: props.estimate && props.estimate.total || 0,
        isValidAllEstimateInput: this.checkValidEstimate(props.estimate && props.estimate.total || 0),
        allEstimateValidationError: this.getEstimateValidationError(isValidAllEstimateInput, props.estimate && props.estimate.total || 0),
        users: this.buildUsersEstimate(),
        searchTerm: '',
        isEmptyAnyUserBySearch: false
      });
    }
  }, {
    key: "handleChangeAllEstimate",
    value: function handleChangeAllEstimate(event) {
      var parsedValue = TimeEntryUtils.parseTimeToSeconds(event.target.value, '');
      var isValidAllEstimateInput = this.checkValidEstimate(parsedValue);
      this.setState({
        allEstimateInput: event.target.value,
        allEstimateInputMinutes: parsedValue || 0,
        isValidAllEstimateInput: isValidAllEstimateInput,
        allEstimateValidationError: this.getEstimateValidationError(parsedValue)
      });
    }
  }, {
    key: "handleChangeUserEstimate",
    value: function handleChangeUserEstimate(user, event) {
      var state = this.state;
      var parsedValue = TimeEntryUtils.parseTimeToSeconds(event.target.value);
      var isValidEstimate = this.checkValidEstimate(parsedValue);
      user.progress.estimateInput = event.target.value || '';
      user.progress.estimateInputMinutes = parsedValue || 0;
      user.progress.isValidEstimate = isValidEstimate;
      user.progress.validationError = this.getEstimateValidationError(parsedValue);
      var allEstimate = this.calculateEstimate();
      var isValidAllEstimate = this.checkValidEstimate(allEstimate);
      this.setState({
        users: [].concat(state.users),
        allEstimateInput: this.formatEstimate(allEstimate),
        isValidAllEstimateInput: isValidAllEstimate,
        allEstimateValidationError: this.getEstimateValidationError(allEstimate)
      });
    }
  }, {
    key: "handleClearUserEstimate",
    value: function handleClearUserEstimate(user) {
      var state = this.state;
      user.progress.estimateInput = '';
      user.progress.estimateInputMinutes = 0;
      user.progress.isValidEstimate = true;
      this.setState({
        users: [].concat(state.users),
        allEstimateInput: this.formatEstimate(this.calculateEstimate())
      });
    }
  }, {
    key: "calculateEstimate",
    value: function calculateEstimate() {
      return this.state.users.reduce(function (count, user) {
        return count + user.progress.estimateInputMinutes || 0;
      }, 0);
    }
  }, {
    key: "formatEstimate",
    value: function formatEstimate(estimate) {
      return TimeEntryUtils.format(estimate, '');
    }
  }, {
    key: "checkValidEstimate",
    value: function checkValidEstimate(estimate) {
      return estimate === '' || estimate === 0 || !!estimate && estimate < this.maxEstimateValue;
    }
  }, {
    key: "getEstimateValidationError",
    value: function getEstimateValidationError(estimate) {
      return !this.checkValidEstimate(estimate) && !!estimate && estimate > this.maxEstimateValue ? '5000H MAX' : null;
    }
  }, {
    key: "handleClearEstimate",
    value: function handleClearEstimate() {
      this.setState({
        allEstimateInput: '',
        allEstimateInputMinutes: '',
        isValidAllEstimateInput: true
      });
    }
  }, {
    key: "handleSearchUsers",
    value: function handleSearchUsers(event) {
      var searchTerm = event.target.value;
      var state = this.state;
      var users = state.users;
      var isEmptyAnyUserBySearch = true;
      users.forEach(function (user) {
        if (!searchTerm) {
          user.isVisible = true;
          isEmptyAnyUserBySearch = false;
          return;
        }
        var searchVal = searchTerm.toLocaleLowerCase();
        var isVisible = !!~(user.name || user.email).toLocaleLowerCase().indexOf(searchVal);
        if (isVisible) {
          isEmptyAnyUserBySearch = false;
        }
        user.isVisible = isVisible;
      });
      this.setState({
        users: [].concat(users),
        searchTerm: searchTerm,
        isEmptyAnyUserBySearch: isEmptyAnyUserBySearch
      });
    }
  }, {
    key: "handleClearSearch",
    value: function handleClearSearch(event) {
      event.stopPropagation();
      event.preventDefault();
      this.handleSearchUsers({
        target: {
          value: ''
        }
      });
    }
  }, {
    key: "checkValidForm",
    value: function checkValidForm() {
      var state = this.state;
      return state.isValidAllEstimateInput && !state.users.find(function (user) {
        return user.progress && !user.progress.isValidEstimate;
      });
    }
  }, {
    key: "isJiraTask",
    value: function isJiraTask() {
      return !!~this.props.taskId.indexOf('jr');
    }
  }, {
    key: "handleSubmitForm",
    value: function handleSubmitForm() {
      var _this3 = this;
      var isValidForm = this.checkValidForm();
      if (!isValidForm) {
        return;
      }
      var usersEstimate = {};
      var total = 0;
      if (this.state.isEstimateUsers) {
        this.state.users.forEach(function (user) {
          if (user.progress && user.progress.estimateInputMinutes) {
            usersEstimate[user.id] = user.progress.estimateInputMinutes;
            total += user.progress.estimateInputMinutes;
          }
        });
      }
      if (!this.state.isEstimateUsers) {
        total = +this.state.allEstimateInputMinutes;
      }
      this.props.store.taskDetails.setEstimate(this.props.taskId, {
        total: total,
        type: this.state.isEstimateUsers ? 'users' : 'overall',
        users: usersEstimate
      }).then(function (estimate) {
        browserPort.analytics.event('estimate_saved');
        _this3.setState({
          estimateLabel: TimeEntryUtils.format(total, '0h')
        });
        _this3.handleCloseEstimatePopover();
        _this3.props.onUpdateTask();
      })["catch"](function (error) {
        browserPort.eventDispatcher.broadcast('browser-button', 'render-errors', error);
      });
    }
  }, {
    key: "render",
    value: function render() {
      var _this4 = this;
      var state = this.state;
      var isEmptyEstimateUsers = !state.users.length;
      if (!this.props.canEditEstimate) {
        return /*#__PURE__*/React.createElement("div", {
          className: "task-details__total-estimate"
        }, /*#__PURE__*/React.createElement("div", {
          className: "total-estimate__text"
        }, "Estimate:\xA0"), /*#__PURE__*/React.createElement("div", {
          className: "total-estimate__hours"
        }, state.estimateLabel));
      }
      return /*#__PURE__*/React.createElement(Popover2, {
        triggerNode: /*#__PURE__*/React.createElement("div", {
          className: "task-details__total-estimate interactive"
        }, /*#__PURE__*/React.createElement("div", {
          className: "total-estimate__text"
        }, "Estimate:\xA0"), /*#__PURE__*/React.createElement("div", {
          className: "total-estimate__hours"
        }, state.estimateLabel)),
        trigger: "click",
        placement: "bottom-left",
        onToggle: this.onToggleEstimatePopover.bind(this),
        width: 296
      }, /*#__PURE__*/React.createElement("div", {
        className: "estimate-popover"
      }, /*#__PURE__*/React.createElement("div", {
        className: "estimate-popover__header"
      }, /*#__PURE__*/React.createElement("div", {
        className: "estimate-popover__header-title"
      }, this.props.estimate ? 'EDIT ESTIMATE' : 'ADD ESTIMATE'), /*#__PURE__*/React.createElement("div", {
        className: "estimate-popover-close icon-cancel",
        onClick: this.handleCloseEstimatePopover.bind(this)
      })), /*#__PURE__*/React.createElement("div", {
        className: "estimate-popover__content"
      }, /*#__PURE__*/React.createElement("div", {
        className: "estimate-popover__summary"
      }, /*#__PURE__*/React.createElement("div", {
        className: "".concat(!state.isValidAllEstimateInput ? 'everhour-error' : '', " input-estimate-all__wrapper"),
        "data-error": state.allEstimateValidationError || ''
      }, /*#__PURE__*/React.createElement("input", {
        type: "text",
        className: "field__input ".concat(!state.isValidAllEstimateInput ? 'field__input--invalid' : '', " input-estimate-all"),
        onChange: this.handleChangeAllEstimate.bind(this),
        value: state.allEstimateInput,
        disabled: state.isEstimateUsers,
        placeholder: "0h"
      }), !state.isEstimateUsers ? /*#__PURE__*/React.createElement("span", {
        className: "input-estimate-all__remove icon-delete",
        onClick: this.handleClearEstimate.bind(this)
      }) : ''), !this.isJiraTask() && this.state.enableIndividual ? /*#__PURE__*/React.createElement("label", {
        className: "checkbox"
      }, /*#__PURE__*/React.createElement("input", {
        type: "checkbox",
        className: "checkbox",
        onChange: this.handleChangeEstimateType.bind(this),
        checked: state.isEstimateUsers
      }), /*#__PURE__*/React.createElement("span", {
        className: "checkbox__label"
      }, "Set individual estimates")) : ''), state.isEstimateUsers ? /*#__PURE__*/React.createElement("div", {
        className: "estimate-popover__users"
      }, !isEmptyEstimateUsers ? /*#__PURE__*/React.createElement("div", {
        className: "estimate-popover__users-search"
      }, /*#__PURE__*/React.createElement("input", {
        type: "text",
        className: "field__input input-estimate-users",
        placeholder: "Search...",
        value: state.searchTerm,
        onChange: this.handleSearchUsers.bind(this)
      }), /*#__PURE__*/React.createElement("span", {
        className: "icon-search"
      }), state.searchTerm ? /*#__PURE__*/React.createElement("span", {
        className: "icon-cancel",
        onClick: this.handleClearSearch.bind(this)
      }) : '') : '', /*#__PURE__*/React.createElement("div", {
        className: "estimate-popover__users-list"
      }, state.users && state.users.length ? state.users.map(function (user) {
        return /*#__PURE__*/React.createElement("div", {
          className: "estimate-popover__users-list-item\n                                                             ".concat(user.isVisible ? 'estimate-popover__users-list-item--visible' : '')
        }, /*#__PURE__*/React.createElement(TaskDetailsUserProgress, {
          user: user,
          taskId: _this4.props.taskId,
          total: user.progress,
          isViewOnly: true,
          store: _this4.props.store
        }), /*#__PURE__*/React.createElement("div", {
          className: "users-list-item__clear"
        }, /*#__PURE__*/React.createElement("span", {
          className: "icon-delete",
          onClick: _this4.handleClearUserEstimate.bind(_this4, user)
        })), /*#__PURE__*/React.createElement("span", {
          className: !user.progress.isValidEstimate ? 'everhour-error' : '',
          "data-error": user.progress.validationError || ''
        }, /*#__PURE__*/React.createElement("input", {
          type: "text",
          value: user.progress.estimateInput,
          onChange: _this4.handleChangeUserEstimate.bind(_this4, user),
          className: "field__input ".concat(!user.progress.isValidEstimate ? 'field__input--invalid' : ''),
          placeholder: "\u2014"
        })));
      }) : '', state.isEmptyAnyUserBySearch ? /*#__PURE__*/React.createElement("div", {
        className: "estimate-popover__users-list--empty"
      }, "Whoops!", /*#__PURE__*/React.createElement("br", null), "We have not found anything.", /*#__PURE__*/React.createElement("br", null), "Try to change your search.") : '', isEmptyEstimateUsers ? /*#__PURE__*/React.createElement("div", {
        className: "estimate-popover__users-list--empty"
      }, "Whoops!", /*#__PURE__*/React.createElement("br", null), " We have nothing to select from.") : '')) : ''), /*#__PURE__*/React.createElement("div", {
        className: "estimate-popover__actions"
      }, /*#__PURE__*/React.createElement("button", {
        type: "button",
        className: "content-actions__button content-actions__button--submit",
        onClick: this.handleSubmitForm.bind(this)
      }, "Save"), /*#__PURE__*/React.createElement("button", {
        type: "button",
        className: "content-actions__button content-actions__button--close",
        onClick: this.handleCloseEstimatePopover.bind(this)
      }, "Cancel"))));
    }
  }]);
}(React.Component);
{
  TaskDetailsEstimatePopover;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TaskDetailsProgress = /*#__PURE__*/function (_React$Component) {
  function TaskDetailsProgress(props) {
    _classCallCheck(this, TaskDetailsProgress);
    return _callSuper(this, TaskDetailsProgress, [props]);
  }
  _inherits(TaskDetailsProgress, _React$Component);
  return _createClass(TaskDetailsProgress, [{
    key: "getUserTotal",
    value: function getUserTotal(userId) {
      var taskDetails = this.props.task;
      var time = taskDetails.time;
      var estimate = taskDetails.estimate;
      return {
        time: time && time.users && time.users[userId] || 0,
        estimate: estimate && estimate.users && estimate.users[userId] || 0
      };
    }
  }, {
    key: "handleOpenEditTimeModal",
    value: function handleOpenEditTimeModal(userId) {
      this.props.onOpenEditTimeModal(userId);
    }
  }, {
    key: "renderUserProgress",
    value: function renderUserProgress(user, taskId, total, count, withTooltip) {
      var _this = this;
      if (withTooltip) {
        return /*#__PURE__*/React.createElement(Tooltip, {
          space: 5,
          triggerTemplate: function triggerTemplate() {
            return /*#__PURE__*/React.createElement(TaskDetailsUserProgress, {
              user: user,
              taskId: taskId,
              total: total,
              store: _this.props.store,
              onOpenEditTimeModal: _this.handleOpenEditTimeModal.bind(_this),
              isViewOnly: count > 3
            });
          }
        }, /*#__PURE__*/React.createElement("div", {
          className: "tooltip-progress__user-name"
        }, user.name || user.email), /*#__PURE__*/React.createElement("div", {
          className: "tooltip-progress__user-total"
        }, total && total.time || total && total.estimate ? /*#__PURE__*/React.createElement("div", null, total.time ? TimeEntryUtils.format(total.time, '0h') : '0h', total.estimate ? " \xB7 ".concat(TimeEntryUtils.format(total.estimate, '0h')) : '') : ''));
      }
      return /*#__PURE__*/React.createElement(TaskDetailsUserProgress, {
        user: user,
        taskId: taskId,
        total: total,
        store: this.props.store,
        onOpenEditTimeModal: this.handleOpenEditTimeModal.bind(this),
        isViewOnly: count > 3
      });
    }
  }, {
    key: "render",
    value: function render() {
      var _this2 = this;
      var collaborators = this.props.collaborators;
      var me = this.props.store.profile.get().id;
      if (!collaborators || collaborators && !collaborators.length) {
        return '';
      }

      // if (collaborators.length <= 4) {
      collaborators = collaborators.filter(function (c) {
        return c.id !== me;
      });
      // }

      return /*#__PURE__*/React.createElement("div", {
        className: "task-details__progress\n                                 task-details__progress--".concat(collaborators.length, "\n                                 ").concat(collaborators.length > 3 ? 'task-details__progress--collapsed' : '')
      }, collaborators.map(function (user) {
        return _this2.renderUserProgress(user, _this2.props.task.id, _this2.getUserTotal(user.id), collaborators.length, collaborators.length > 3);
      }));
    }
  }]);
}(React.Component);
{
  TaskDetailsProgress;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var SearchTab = /*#__PURE__*/function (_React$Component) {
  function SearchTab(props) {
    var _this;
    _classCallCheck(this, SearchTab);
    _this = _callSuper(this, SearchTab, [props]);
    browserPort.analytics.event('popover_search');
    _this.LocalStorageService = new window.LocalStorageService();
    var everhourPopupSearchTab = _this.LocalStorageService.getItem('everhour-popup-search-tab');
    if (!everhourPopupSearchTab) {
      everhourPopupSearchTab = {
        mode: 'tasks'
      };
      _this.LocalStorageService.setItem('everhour-popup-search-tab', everhourPopupSearchTab);
    }

    // let timer = this.props.store.timer.get();

    _this.state = {
      // searchTerm: props.searchTerm && timer && timer.status === 'active' && !timer.task ? timer.comment : null,
      // searchTerm: null,
      isFetchTasks: false,
      tasks: null,
      // tasks for render
      recentTasks: null,
      // recent tasks for default(!searchTerm)
      tasksByProject: null,
      // merge tasks for default(!searchTerm)
      modified: null,
      mode: everhourPopupSearchTab.mode,
      projects: null,
      // projects for render
      _projects: null,
      // projects for default(!searchTerm or filter by searchTerm)
      projectForSearchTasks: null
    };
    _this._searchWithDelay = _.debounce(_this.searchWithDelay, 250, false);
    _this.searchTerm = '';
    _this.searchInput = null;
    _this._request = null;
    _this._cancelRequest = _this.cancelRequest.bind(_this);
    _this.popupControllerService = new PopupControllerService();
    browserPort.eventDispatcher.on('browser-button', 'websocket-update', function () {
      _this.setState({
        modified: +new Date()
      });
    });
    browserPort.eventDispatcher.on('browser-button', 'select-task', function (data) {
      if (!data || data && !data.taskId && !data.comment) {
        return;
      }
      _this.handleSelectTask(data.taskId, data.comment);
    });
    return _this;
  }
  _inherits(SearchTab, _React$Component);
  return _createClass(SearchTab, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this.searchInput.focus();
      this.getProjects();
      var searchTerm = this.searchTerm;
      if (searchTerm && this.state.mode === 'tasks') {
        this.searchRecentTasks(searchTerm, null, true);
      } else if (this.state.mode === 'tasks') {
        this.fetchRecentTasks();
      }
    }
  }, {
    key: "search",
    value: function search(event) {
      var state = this.state;
      var searchTerm = this.searchTerm = event.target.value;
      browserPort.eventDispatcher.broadcast('browser-button', 'scrolling-container-reset-events');
      if (!searchTerm) {
        this._cancelRequest();
        this.setState({
          isFetchTasks: false,
          tasks: state.mode === 'tasks' ? state.recentTasks : state.tasksByProject,
          projects: state._projects
        });
        this.searchTerm = '';
        return;
      }
      this._searchWithDelay(searchTerm);
    }
  }, {
    key: "searchWithDelay",
    value: function searchWithDelay(searchTerm) {
      if (this.state.mode === 'tasks') {
        this.searchRecentTasks(searchTerm, null, true);
      } else if (this.state.mode === 'projects' && this.state.projectForSearchTasks) {
        this.searchRecentTasksByProject(searchTerm, this.state.projectForSearchTasks);
      } else if (this.state.mode === 'projects' && !this.state.projectForSearchTasks) {
        this.handleFilterProjects();
      }
    }
  }, {
    key: "cancelRequest",
    value: function cancelRequest() {
      if (this._request && this._request.abort) {
        this._request.abort();
      }
    }
  }, {
    key: "getProjects",
    value: function getProjects() {
      var projects = this.props.store.projects.getGroupsProjects();
      this.setState({
        projects: projects,
        _projects: projects
      });
    }
  }, {
    key: "fetchRecentTasks",
    value: function fetchRecentTasks() {
      var _this2 = this;
      this.setState({
        isFetchTasks: true
      });
      this.props.api.searchRecentTasks(null, null, true, null).then(function (tasks) {
        var modifiedTasks = {
          search: {
            label: '',
            tasks: []
          },
          recent: {
            label: 'recent',
            tasks: _this2.modifyTasks(tasks)
          },
          opened: {
            label: 'opened',
            tasks: []
          }
        };
        _this2.setState({
          tasks: modifiedTasks,
          recentTasks: modifiedTasks,
          isFetchTasks: false
        });
      });
    }
  }, {
    key: "searchRecentTasks",
    value: function searchRecentTasks(searchTerm) {
      var _this3 = this;
      if (!this.searchTerm) {
        return;
      }
      this.setState({
        isFetchTasks: true,
        tasks: null
      });
      this._request = this.props.api.searchRecentTasks(searchTerm, null, true, this._cancelRequest);
      this._request.then(function (tasks) {
        var modifiedTasks = {
          search: {
            label: "".concat(tasks.length, " EXISTING TASK").concat(tasks.length > 1 ? 's' : ''),
            tasks: _this3.modifyTasks(tasks)
          },
          recent: {
            label: 'recent',
            tasks: []
          },
          opened: {
            label: 'opened',
            tasks: []
          }
        };
        _this3.setState({
          tasks: modifiedTasks,
          isFetchTasks: false
        });
        _this3._request = null;
      });
    }
  }, {
    key: "fetchTasksByProject",
    value: function fetchTasksByProject(projectId) {
      var _this4 = this;
      this.setState({
        isFetchTasks: true,
        tasks: null
      });
      return this.props.api.getTasksByProject(projectId).then(function (tasks) {
        _this4.projectTasks = tasks;
      });
    }
  }, {
    key: "fetchRecentTaskByProject",
    value: function fetchRecentTaskByProject(projectId) {
      var _this5 = this;
      this.setState({
        isFetchTasks: true
      });
      return this.props.api.getRecentTasksByProject(projectId).then(function (tasks) {
        _this5.recentProjectTasks = _this5.modifyTasks(tasks);
      });
    }
  }, {
    key: "searchRecentTasksByProject",
    value: function searchRecentTasksByProject(searchTerm, projectId) {
      var _this6 = this;
      if (!this.searchTerm) {
        return;
      }
      this.setState({
        isFetchTasks: true,
        tasks: null
      });
      this._request = this.props.api.searchTasksByProject(searchTerm || '', projectId, this._cancelRequest);
      this._request.then(function (tasks) {
        _this6.setState({
          isFetchTasks: false,
          tasks: {
            search: {
              label: "".concat(tasks.length, " EXISTING TASK").concat(tasks.length > 1 ? 's' : ''),
              tasks: _this6.modifyTasks(tasks)
            },
            recent: {
              label: 'recent',
              tasks: []
            },
            opened: {
              label: 'opened',
              tasks: []
            }
          }
        });
        _this6._request = null;
      });
    }
  }, {
    key: "getProjectTasks",
    value: function getProjectTasks(projectId) {
      var _this7 = this;
      if (!projectId) {
        return;
      }
      this.fetchRecentTaskByProject(projectId).then(function () {
        _this7.setState({
          tasks: {
            search: {
              label: '',
              tasks: []
            },
            recent: {
              label: 'recent',
              tasks: _this7.modifyTasks(_this7.recentProjectTasks)
            },
            opened: {
              label: 'opened',
              tasks: []
            }
          }
        });
        if (!_this7.popupControllerService.isSyncedProjectByTimestamp(projectId)) {
          return _this7.props.api.pushsync(projectId).then(function () {
            _this7.popupControllerService.setSyncedProjectTimestamp(projectId);
          });
        }
      }).then(function () {
        if (!_this7.popupControllerService.isSyncedProjectByTimestamp(projectId)) {
          return Promise.all([_this7.fetchRecentTaskByProject(projectId), _this7.fetchTasksByProject(projectId)]);
        }
        return _this7.fetchTasksByProject(projectId);
      }).then(function () {
        _this7.mergeTasks();
      });
    }
  }, {
    key: "mergeTasks",
    value: function mergeTasks() {
      var _this8 = this;
      var recentTasksIds = [];
      var recentTasks = this.recentProjectTasks.filter(function (recentTask) {
        if (_this8.projectTasks.find(function (task) {
          return task.id === recentTask.id;
        })) {
          recentTasksIds.push(recentTask.id);
          return true;
        }
        return false;
      });
      var projectTasks = this.projectTasks.filter(function (task) {
        return recentTasksIds.indexOf(task.id) === -1;
      }).sort(function (taskA, taskB) {
        var taskAName = taskA.name.toLowerCase();
        var taskBName = taskB.name.toLowerCase();
        if (taskAName < taskBName) {
          return -1;
        }
        if (taskBName > taskAName) {
          return 1;
        }
      });
      this.setState({
        isFetchTasks: false,
        tasksByProject: {
          search: {
            label: '',
            tasks: []
          },
          recent: {
            label: 'recent',
            tasks: this.modifyTasks(recentTasks)
          },
          opened: {
            label: 'opened',
            tasks: this.modifyTasks(projectTasks)
          }
        },
        tasks: {
          search: {
            label: '',
            tasks: []
          },
          recent: {
            label: 'recent',
            tasks: this.modifyTasks(recentTasks)
          },
          opened: {
            label: 'opened',
            tasks: this.modifyTasks(projectTasks)
          }
        }
      });
    }
  }, {
    key: "handleFilterProjects",
    value: function handleFilterProjects() {
      var searchTerm = this.searchTerm;
      var _projects = this.state._projects;
      var recentProjects = everhourMethods.filterByMatch(_projects.recent.projects, '_searchName', searchTerm);
      var activeProjects = everhourMethods.filterByMatch(_projects.active.projects, '_searchName', searchTerm);
      this.setState({
        projects: {
          recent: Object.assign({}, _projects.recent, {
            projects: recentProjects
          }),
          active: Object.assign({}, _projects.active, {
            projects: activeProjects
          })
        }
      });
    }
  }, {
    key: "toggleSearchMode",
    value: function toggleSearchMode(event) {
      event.preventDefault();
      event.stopPropagation();
      browserPort.eventDispatcher.broadcast('browser-button', 'scrolling-container-reset-events');
      var mode = this.state.mode === 'projects' ? 'tasks' : 'projects';
      this.LocalStorageService.setItem('everhour-popup-search-tab', {
        mode: mode
      });
      this.setState({
        mode: mode,
        projects: this.state._projects
      });
      this.searchTerm = '';
      this.searchInput.value = '';
      this.searchInput.focus();
      if (mode === 'tasks') {
        this.fetchRecentTasks();
      }
    }
  }, {
    key: "modifyTasks",
    value: function modifyTasks(tasks) {
      var _this9 = this;
      if (!tasks || !tasks.length) {
        return [];
      }
      return tasks.map(function (task) {
        if (task._type === 'comment' || task.type === 'comment') {
          task._id = String(Math.floor(Math.random() * 1000000));
        }
        task.project = _this9.props.store.taskTime.taskProject(task.projectsObjects);
        task.projectId = task.project ? task.project.id : null;
        return task;
      });
    }
  }, {
    key: "handleHideSearchTab",
    value: function handleHideSearchTab() {
      browserPort.eventDispatcher.broadcast('browser-button', 'change-tab', {
        tab: 'time'
      });
    }
  }, {
    key: "handleNavigateToTimeTab",
    value: function handleNavigateToTimeTab() {
      browserPort.eventDispatcher.broadcast('browser-button', 'change-tab', {
        tab: 'time'
      });
    }
  }, {
    key: "handleSelectProject",
    value: function handleSelectProject(projectId) {
      browserPort.eventDispatcher.broadcast('browser-button', 'scrolling-container-reset-events');
      this.setState({
        projectForSearchTasks: projectId
      });
      this.searchTerm = '';
      this.searchInput.value = '';
      this.getProjectTasks(projectId);
    }
  }, {
    key: "handleSelectTask",
    value: function handleSelectTask(taskId, comment) {
      var _this10 = this;
      var timer = this.props.store.timer.get();
      browserPort.eventDispatcher.broadcast('browser-button', 'scrolling-container-reset-events');
      if (timer && timer.status === 'active' && !timer.task) {
        var tasks = [].concat(this.state.tasks.opened.tasks || []).concat(this.state.tasks.recent.tasks || []).concat(this.state.tasks.search.tasks || []);
        var requestBody = {};

        // We are checking if taskId is real task
        if (taskId && tasks.find(function (task) {
          return task.id === taskId;
        })) {
          requestBody.task = taskId;
        }
        if (comment) {
          requestBody.comment = comment;
        }
        this.props.api.commentTimer(requestBody).then(function (timer) {
          store.dispatch(TimerStore.Act.Updated(timer));
          _this10.handleNavigateToTimeTab();
        });
        return;
      }
      if (timer && timer.status === 'active' && taskId === timer.task.id) {
        this.stopTimer();
        return;
      }
      if (timer && timer.status === 'stopped') {
        var task = this.findTask(taskId);
        var isComment = task && task._type && (task._type === 'comment' || task.type === 'comment');
        this.startTimer(isComment ? null : taskId, isComment ? task.comment : null);
      }
    }
  }, {
    key: "handleResetProject",
    value: function handleResetProject() {
      this.setState({
        projectForSearchTasks: null,
        projects: this.state._projects,
        isFetchTasks: false
      });
      this.searchTerm = '';
      this.searchInput.value = '';
    }
  }, {
    key: "isTimecardsEnabled",
    value: function isTimecardsEnabled() {
      return this.props.store.profile.isApplicationEnabled('timecards');
    }
  }, {
    key: "startTimer",
    value: function startTimer(taskId, comment) {
      var _this11 = this;
      this.props.reactAppStateObj.lockActions = true;
      this.props.store.timer.start(taskId, null, null, comment).done(function (timer) {
        _this11.props.reactAppStateObj.lockActions = false;
        if (_this11.isTimecardsEnabled()) {
          var todayTimecard = _this11.props.reactAppStateObj.todayTimecard;
          var isToggleNeeded = !todayTimecard || todayTimecard && timer.timecard && todayTimecard.clockOut !== timer.timecard.clockOut;
          if (timer.timecard) {
            _this11.props.reactAppStateObj.todayTimecard = timer.timecard;
          }
          if (isToggleNeeded) {
            browserPort.eventDispatcher.broadcast('browser-button', 'toggle-clockin-class-state');
          }
        }
        _this11.handleNavigateToTimeTab();
      }).fail(function (response) {
        return browserPort.eventDispatcher.broadcast('browser-button', 'render-errors', response);
      });
    }
  }, {
    key: "stopTimer",
    value: function stopTimer() {
      var _this12 = this;
      this.props.reactAppStateObj.lockActions = true;
      this.props.store.timer.stop().done(function () {
        _this12.props.reactAppStateObj.lockActions = false;
        _this12.handleNavigateToTimeTab();
      }).fail(function (response) {
        return browserPort.eventDispatcher.broadcast('browser-button', 'render-errors', response);
      });
    }
  }, {
    key: "findTask",
    value: function findTask(taskId) {
      var tasks = [];
      var stateTasks = this.state.tasks;
      if (stateTasks && stateTasks.recent && stateTasks.recent.tasks) {
        tasks = tasks.concat(stateTasks.recent.tasks);
      }
      if (stateTasks && stateTasks.search && stateTasks.search.tasks) {
        tasks = tasks.concat(stateTasks.search.tasks);
      }
      if (stateTasks && stateTasks.opened && stateTasks.opened.tasks) {
        tasks = tasks.concat(stateTasks.opened.tasks);
      }
      return tasks.find(function (task) {
        return task.id && task.id === taskId || task._id && task._id === taskId;
      });
    }
  }, {
    key: "handleCreateAbstractTaskTime",
    value: function handleCreateAbstractTaskTime() {
      var _this13 = this;
      this.props.store.timer.start(null, null, null, this.searchTerm).then(function () {
        _this13.handleNavigateToTimeTab();
      });
    }
  }, {
    key: "render",
    value: function render() {
      var _this14 = this;
      var state = this.state;
      var tasks = state.tasks;
      var project = state.projectForSearchTasks && this.props.store.projects.get(state.projectForSearchTasks);
      var searchTerm = this.searchTerm;
      var isShowModeSwitch = !state.projectForSearchTasks;
      var hasTimer = store.timer.isActive();
      var timerClone = this.props.store.timer.get();
      if (timerClone) {
        timerClone.isCurrent = true;
      }
      return /*#__PURE__*/React.createElement("div", {
        className: "dynamic-height"
      }, /*#__PURE__*/React.createElement("div", {
        className: "search__field-wrapper"
      }, /*#__PURE__*/React.createElement("div", {
        className: "icon-lens"
      }), /*#__PURE__*/React.createElement("input", {
        type: "text",
        className: "search__field",
        placeholder: "What are you working on\u2026",
        ref: function ref(input) {
          _this14.searchInput = input;
        },
        onChange: this.search.bind(this)
      }), /*#__PURE__*/React.createElement("div", {
        className: "search__field-close",
        onClick: this.handleHideSearchTab
      }, /*#__PURE__*/React.createElement("div", {
        className: "icon-close-small"
      })), !this.props.isSearchOpenFromLens ? /*#__PURE__*/React.createElement(TaskTimer, {
        taskTime: hasTimer ? timerClone : null,
        store: this.props.store,
        comment: searchTerm,
        reactAppStateObj: this.props.reactAppStateObj,
        onStartTimer: this.handleHideSearchTab.bind(this)
      }) : ''), state.projectForSearchTasks ? /*#__PURE__*/React.createElement("div", {
        className: "search__project-acrive-container"
      }, /*#__PURE__*/React.createElement("div", {
        className: "search__project-active"
      }, /*#__PURE__*/React.createElement("div", {
        className: "integration-icon integration-".concat(getPlatformName(project.platform))
      }), /*#__PURE__*/React.createElement("div", {
        className: "project-active__name"
      }, project.name), /*#__PURE__*/React.createElement("div", {
        className: "project-active__reset",
        onClick: this.handleResetProject.bind(this)
      }, /*#__PURE__*/React.createElement("div", {
        className: "icon-remove-tag"
      })))) : '', !this.state.isFetchTasks && (tasks && tasks.search && tasks.search.tasks.length || tasks && tasks.recent && tasks.recent.tasks.length || tasks && tasks.opened && tasks.opened.tasks.length) && (state.mode === 'tasks' || state.mode === 'projects' && state.projectForSearchTasks) ? /*#__PURE__*/React.createElement(GroupTasks, {
        tasks: this.state.tasks,
        searchTerm: this.searchTerm,
        api: this.props.api,
        store: this.props.store,
        reactAppStateObj: this.props.reactAppStateObj,
        onSelectTask: this.handleSelectTask.bind(this),
        onNavigateToTimeTab: this.handleNavigateToTimeTab.bind(this)
      }) : '', state.mode === 'projects' && !state.projectForSearchTasks && state.projects ? /*#__PURE__*/React.createElement(GroupProjects, {
        projects: state.projects,
        store: this.props.store,
        searchTerm: this.searchTerm,
        onSelectProject: this.handleSelectProject.bind(this)
      }) : '', state.isFetchTasks ? /*#__PURE__*/React.createElement(Skeleton, {
        view: 'recent',
        options: {
          withTitle: true
        }
      }) : '', !state.isFetchTasks && !searchTerm && state.mode === 'tasks' && state.tasks && !state.tasks.search.tasks.length && !state.tasks.recent.tasks.length ? /*#__PURE__*/React.createElement("div", {
        className: "nothing-found"
      }, "No tasks found") : '', !state.isFetchTasks && searchTerm && state.mode === 'tasks' && state.tasks && !state.tasks.search.tasks.length && !state.tasks.recent.tasks.length ? /*#__PURE__*/React.createElement("div", {
        className: "nothing-found"
      }, "No tasks found") : '', !state.isFetchTasks && searchTerm && state.mode === 'projects' && state.projectForSearchTasks && state.tasks && !state.tasks.search.tasks.length && !state.tasks.recent.tasks.length && !state.tasks.opened.tasks.length ? /*#__PURE__*/React.createElement("div", {
        className: "nothing-found"
      }, "Hmm... nothing matches your search") : '', !state.isFetchTasks && !searchTerm && state.mode === 'projects' && state.projectForSearchTasks && state.tasks && !state.tasks.search.tasks.length && !state.tasks.recent.tasks.length && !state.tasks.opened.tasks.length ? /*#__PURE__*/React.createElement("div", {
        className: "nothing-found"
      }, "No task found") : '', !state.isFetchTasks && state.mode === 'projects' && !state.projectForSearchTasks && state.projects && !state.projects.recent.projects.length && !state.projects.active.projects.length ? /*#__PURE__*/React.createElement("div", {
        className: "nothing-found"
      }, "Hmm... nothing matches your search") : '', searchTerm && state.mode === 'tasks' || searchTerm && state.mode === 'projects' && state.projectForSearchTasks ? /*#__PURE__*/React.createElement("div", {
        className: "group__free"
      }, /*#__PURE__*/React.createElement("div", {
        className: "task-row task-row--title"
      }, "FREE TEXT ENTRY"), /*#__PURE__*/React.createElement("div", {
        className: "search__comment-line",
        onClick: this.handleCreateAbstractTaskTime.bind(this)
      }, /*#__PURE__*/React.createElement("div", {
        className: "comment-line__icon icon-comment"
      }), /*#__PURE__*/React.createElement("div", {
        className: "comment-line__text"
      }, searchTerm))) : '', isShowModeSwitch ? /*#__PURE__*/React.createElement("div", {
        className: "search__mode-switch"
      }, /*#__PURE__*/React.createElement("label", {
        className: "checkbox checkbox--slider",
        onClick: this.toggleSearchMode.bind(this)
      }, /*#__PURE__*/React.createElement("span", {
        className: "slider__label"
      }, "Search in a specific project"), /*#__PURE__*/React.createElement("input", {
        type: "checkbox",
        checked: state.mode === 'projects'
      }), /*#__PURE__*/React.createElement("span", {
        className: "toggle-with-icon slider round"
      }))) : '');
    }
  }]);
}(React.Component);
{
  SearchTab;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var GroupRecentTask = /*#__PURE__*/function (_React$Component) {
  function GroupRecentTask(props) {
    var _this;
    _classCallCheck(this, GroupRecentTask);
    _this = _callSuper(this, GroupRecentTask, [props]);
    _this.popupControllerService = new PopupControllerService();
    return _this;
  }
  _inherits(GroupRecentTask, _React$Component);
  return _createClass(GroupRecentTask, [{
    key: "handleSelectTask",
    value: function handleSelectTask(taskId) {
      var task = this.props.tasks.find(function (task) {
        return task.id && task.id === taskId || task._id && task._id === taskId;
      });
      if (!task || this.props.reactAppStateObj && this.props.reactAppStateObj.lockActions) {
        return;
      }
      if (this.props.onSelect) {
        this.onSelectTask(task);
        return;
      }
      if (this.props.store.timer.activeTask() === task.id || task.isTimerWithOutTask) {
        this.stopTimer();
      } else {
        this.startTimer(task);
      }
    }
  }, {
    key: "stopTimer",
    value: function stopTimer() {
      var _this2 = this;
      this.props.reactAppStateObj.lockActions = true;
      this.props.store.timer.stop().done(function () {
        _this2.props.reactAppStateObj.lockActions = false;
        _this2.navigateToTimeTab();
      }).fail(function (response) {
        return browserPort.eventDispatcher.broadcast('browser-button', 'render-errors', response);
      });
    }
  }, {
    key: "isTimecardsEnabled",
    value: function isTimecardsEnabled() {
      return this.props.store.profile.isApplicationEnabled('timecards');
    }
  }, {
    key: "startTimer",
    value: function startTimer(task) {
      var _this3 = this;
      if (!task || !this.popupControllerService.checkArchivedOrForeignTask(task)) {
        return;
      }
      this.props.reactAppStateObj.lockActions = true;
      this.props.store.timer.start(task.id, null, null, task._type === 'comment' || task.type === 'comment' ? task.comment : null).done(function (timer) {
        _this3.props.reactAppStateObj.lockActions = false;
        if (_this3.isTimecardsEnabled()) {
          var todayTimecard = _this3.props.reactAppStateObj.todayTimecard;
          var isToggleNeeded = !todayTimecard || todayTimecard && timer.timecard && todayTimecard.clockOut !== timer.timecard.clockOut;
          if (timer.timecard) {
            _this3.props.reactAppStateObj.todayTimecard = timer.timecard;
          }
          if (isToggleNeeded) {
            browserPort.eventDispatcher.broadcast('browser-button', 'toggle-clockin-class-state');
          }
        }
        _this3.navigateToTimeTab();
      }).fail(function (response) {
        return browserPort.eventDispatcher.broadcast('browser-button', 'render-errors', response);
      });
    }
  }, {
    key: "navigateToTimeTab",
    value: function navigateToTimeTab() {
      if (this.props.eventsSelector === 'recent-tab') {
        // return;
      }

      // browserPort.eventDispatcher.broadcast('browser-button', 'render-tab', 'time');
      browserPort.eventDispatcher.broadcast('browser-button', 'change-tab', {
        tab: 'time'
      });
    }
  }, {
    key: "render",
    value: function render() {
      var _this4 = this;
      return /*#__PURE__*/React.createElement(ScrollingContainer, {
        eventSelectorId: this.props.eventsSelector,
        listContainerSelector: '.recent__tasks',
        itemSelector: '.task-row.task-row--recent',
        onSelect: this.handleSelectTask.bind(this)
      }, /*#__PURE__*/React.createElement("div", {
        className: "recent__tasks"
      }, this.props.tasks.map(function (task) {
        var isComment = task._type === 'comment' || task.type === 'comment';
        return /*#__PURE__*/React.createElement("div", {
          onClick: _this4.handleSelectTask.bind(_this4, task)
        }, /*#__PURE__*/React.createElement(RecentTaskRow, {
          isSearchTab: false,
          task: task,
          taskTime: !isComment ? {
            task: task
          } : null,
          taskTimeId: !isComment && task.id,
          time: task.time,
          comment: task.comment,
          key: "task-row-recent-".concat(task.id || task._id),
          "data-key-id": task ? task.id || task._id : null,
          searchTerm: _this4.props.searchTerm,
          api: _this4.props.api,
          store: _this4.props.store,
          reactAppStateObj: _this4.props.reactAppStateObj,
          isNotClikableDescription: isComment,
          isTaskOnly: true,
          navigateToTimeTab: _this4.navigateToTimeTab.bind(_this4)
        }));
      })));
    }
  }]);
}(React.Component);
{
  GroupRecentTask;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var RecentTab = /*#__PURE__*/function (_React$Component) {
  function RecentTab(props) {
    var _this;
    _classCallCheck(this, RecentTab);
    _this = _callSuper(this, RecentTab, [props]);
    _this.state = {
      modified: null,
      tasks: null,
      isFetchTasks: false
    };
    browserPort.analytics.event('popover_recent');
    browserPort.eventDispatcher.on('browser-button', 'websocket-update', function () {
      _this.setState({
        modified: +new Date()
      });
    });
    _this._recentTabInput = null;
    _this._recentTabFocusSubscribe = _this.recentTabFocusSubscribe.bind(_this);
    return _this;
  }
  _inherits(RecentTab, _React$Component);
  return _createClass(RecentTab, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this.getRecentTasks();
      document.getElementById('recent-tab').addEventListener('click', this._recentTabFocusSubscribe);
      this._recentTabInput.focus();
    }
  }, {
    key: "recentTabFocusSubscribe",
    value: function recentTabFocusSubscribe() {
      this._recentTabInput && this._recentTabInput.focus();
    }
  }, {
    key: "getRecentTasks",
    value: function getRecentTasks() {
      var _this2 = this;
      if (this.state.isFetchTasks) {
        return;
      }
      this.setState({
        isFetchTasks: true,
        tasks: []
      });
      this.props.api.searchRecentTasks(null, null, true, null).then(function (response) {
        var store = _this2.props.store;
        var isActiveTask = store.timer.isActive();
        var timer = store.timer.get();
        var activeTask = timer && timer.task && timer.task.id;
        var tasks = _this2.modifyTasks(response || []);
        if (isActiveTask && timer.task && tasks.findIndex(function (task) {
          return task && task.id === activeTask;
        }) === -1) {
          tasks.unshift(_this2.modifyTask(timer.task));
        }
        _this2.setState({
          isFetchTasks: false,
          tasks: tasks,
          modified: +new Date()
        });
      })["catch"](function (err) {
        _this2.setState({
          isFetchTasks: false,
          tasks: []
        });
      });
    }
  }, {
    key: "modifyTasks",
    value: function modifyTasks(tasks) {
      var _this3 = this;
      if (!tasks || !tasks.length) {
        return [];
      }
      return tasks.map(function (task) {
        return _this3.modifyTask(task);
      });
    }
  }, {
    key: "modifyTask",
    value: function modifyTask(task) {
      if (task._type === 'comment' || task.type === 'comment') {
        task._id = String(Math.floor(Math.random() * 1000000));
        return task;
      }
      task.project = task.projectsObjects ? this.props.store.taskTime.taskProject(task.projectsObjects) : null;
      task.projectId = task.project ? task.project.id : null;
      return task;
    }
  }, {
    key: "render",
    value: function render() {
      var _this4 = this;
      var tasks = this.state.tasks;
      var isActiveTask = store.timer.isActive();
      return /*#__PURE__*/React.createElement("div", {
        className: "dynamic-height"
      }, /*#__PURE__*/React.createElement("input", {
        className: "input-hidden-focus",
        type: "text",
        ref: function ref(input) {
          _this4._recentTabInput = input;
        }
      }), !this.state.isFetchTasks ? /*#__PURE__*/React.createElement("div", {
        className: "tab__title"
      }, "recent") : '', !this.state.isFetchTasks && (!tasks || !tasks.length) && !isActiveTask && /*#__PURE__*/React.createElement("div", {
        className: "application-loading"
      }, /*#__PURE__*/React.createElement("div", {
        className: "application-loading-header"
      }, "Your recent tasks will appear here"), /*#__PURE__*/React.createElement("div", {
        className: "application-loading-message"
      }, "Start tracking time and all your recent", /*#__PURE__*/React.createElement("br", null), "tasks will be displayed here.")), tasks && tasks.length > 0 && /*#__PURE__*/React.createElement(GroupRecentTask, {
        tasks: tasks,
        eventsSelector: 'recent-tab',
        api: this.props.api,
        store: this.props.store,
        reactAppStateObj: this.props.reactAppStateObj
      }), this.state.isFetchTasks ? /*#__PURE__*/React.createElement(Skeleton, {
        view: 'recent',
        options: {
          withTitle: true
        }
      }) : '');
    }
  }]);
}(React.Component);
{
  RecentTab;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var IntegrationsTab = /*#__PURE__*/function (_React$Component) {
  function IntegrationsTab(props) {
    var _this;
    _classCallCheck(this, IntegrationsTab);
    _this = _callSuper(this, IntegrationsTab, [props]);
    _this.state = {
      searchTerm: '',
      websites: []
    };
    _this.websitesMap = {};
    return _this;
  }
  _inherits(IntegrationsTab, _React$Component);
  return _createClass(IntegrationsTab, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this.initWebsites();
    }
  }, {
    key: "initWebsites",
    value: function initWebsites() {
      var _this2 = this;
      var self = this;
      var websites = [].concat(window.Websites);
      chrome.permissions.getAll(function (permissions) {
        var origins = permissions.origins;
        websites.forEach(function (website) {
          website.isChecked = !!origins.find(function (origin) {
            return !!~origin.indexOf(website.domains[0]);
          });
          website.isShow = true;
          _this2.websitesMap[website.domains[0]] = website;
        });
        self.setState({
          websites: websites,
          isAnyIntegrationsVisible: true
        });
      });
    }
  }, {
    key: "handleNavigateToTimeTab",
    value: function handleNavigateToTimeTab() {
      browserPort.eventDispatcher.broadcast('browser-button', 'change-tab', {
        tab: 'time'
      });
    }
  }, {
    key: "handleFilterIntegrations",
    value: function handleFilterIntegrations(event) {
      var self = this;
      var searchTerm = event.target.value;
      var websites = self.state.websites.map(function (website) {
        website.isShow = !!~website.name.toLowerCase().indexOf(searchTerm.toLowerCase());
        return website;
      });
      this.setState({
        searchTerm: event.target.value,
        websites: websites,
        isAnyIntegrationsVisible: !!websites.find(function (website) {
          return website.isShow;
        })
      });
    }
  }, {
    key: "getAllDomains",
    value: function getAllDomains() {
      return this.state.websites.reduce(function (domains, website) {
        return domains.concat(website.domains);
      }, []);
    }
  }, {
    key: "handleChangePermissionState",
    value: function handleChangePermissionState(website, event) {
      var domains = website.domains;
      if (event.target.checked) {
        this.handleEnablePermissions(domains);
        return;
      }
      this.handleDisablePermissions(domains);
    }
  }, {
    key: "handleEnablePermissions",
    value: function handleEnablePermissions(domains) {
      var _this3 = this;
      browserPort.tab.requestCustomDomainPermissions(domains, function (isGranted) {
        if (isGranted) {
          _this3.initWebsites();
          browserPort.eventDispatcher.broadcast('background', 'custom-integrations-domains:refresh');
        }
      });
    }
  }, {
    key: "handleDisablePermissions",
    value: function handleDisablePermissions(domains) {
      var _this4 = this;
      domains.forEach(function (domain) {
        try {
          /**
           * domain can be https://google.mail.com/ or https://google.mail.com/*
           */
          var domainReplaced = domain.replace(/\/\*$/gm, '/');

          /**
           * For reload tab we are saving tabId before remove permission.
           * Look https://developer.chrome.com/docs/extensions/reference/tabs/#method-query url parameter
           * for details.
           */
          chrome.tabs.query({
            url: domainReplaced + '*'
          }, function (tabs) {
            browserPort.tab.removeContentScriptsForCustomDomains("everhour_".concat(_this4.websitesMap[domainReplaced].id, "_cs_websites"));
            browserPort.tab.removeWebsitesPermissions(domains, function () {
              _this4.initWebsites();
            });
            tabs.forEach(function (tab) {
              return chrome.tabs.reload(tab.id, {
                bypassCache: false
              });
            });
          });
        } catch (err) {
          console.log('[Integrations.js] removeContentScriptsForCustomDomains is fail with reason: ', err);
        }
      });
    }
  }, {
    key: "render",
    value: function render() {
      var _this5 = this;
      return /*#__PURE__*/React.createElement("div", {
        className: "dynamic-height"
      }, /*#__PURE__*/React.createElement("div", {
        className: "integrations__header"
      }, /*#__PURE__*/React.createElement("div", {
        className: "integrations__header-title"
      }, "Integrations"), /*#__PURE__*/React.createElement("div", {
        className: "integrations__close",
        onClick: this.handleNavigateToTimeTab.bind(this)
      }, /*#__PURE__*/React.createElement("svg", {
        xmlns: "http://www.w3.org/2000/svg"
      }, /*#__PURE__*/React.createElement("path", {
        d: "M7.6 6.45l5.176-5.175.275-.275.549.55-.275.274L8.15 7l5.176 5.176.275.275-.55.549-.274-.275L7.6 7.55l-5.176 5.176-.275.275-.549-.55.275-.274L7.05 7 1.875 1.824 1.6 1.549 2.15 1l.274.275L7.6 6.45z",
        fill: "none",
        stroke: "#C7CACD",
        "stroke-linecap": "square",
        "stroke-width": "1.5"
      })))), /*#__PURE__*/React.createElement("div", {
        className: "integrations__search-wrapper"
      }, /*#__PURE__*/React.createElement("div", {
        className: "integrations__search-wrapper-description"
      }, "Enable the websites you want to use with ", /*#__PURE__*/React.createElement("strong", null, "Everhour Browser Extension."), /*#__PURE__*/React.createElement("br", null), "You can enable everything by clicking the \"Enable all\" button."), /*#__PURE__*/React.createElement("div", {
        className: "integrations__search-wrapper-input"
      }, /*#__PURE__*/React.createElement("div", {
        className: "field__wrapper"
      }, /*#__PURE__*/React.createElement("div", {
        className: "field__body"
      }, /*#__PURE__*/React.createElement("input", {
        className: "field__input",
        type: "text",
        placeholder: "Filter integrations...",
        onChange: this.handleFilterIntegrations.bind(this),
        value: this.state.searchTerm
      }))))), /*#__PURE__*/React.createElement("div", {
        className: "integrations__list-wrapper"
      }, /*#__PURE__*/React.createElement("div", {
        className: "integrations__list"
      }, this.state.websites.map(function (website) {
        return /*#__PURE__*/React.createElement("div", {
          className: "integrations__list-item",
          style: {
            display: website.isShow ? 'flex' : 'none'
          }
        }, /*#__PURE__*/React.createElement("label", {
          className: "checkbox checkbox--slider"
        }, /*#__PURE__*/React.createElement("span", {
          className: "slider__label"
        }, /*#__PURE__*/React.createElement("span", {
          className: "integrations__list-item-icon website-icon website-icon--".concat(website.id)
        }), /*#__PURE__*/React.createElement("span", {
          className: "integrations__list-item-name"
        }, /*#__PURE__*/React.createElement(HighlightText, {
          searchTerm: _this5.state.searchTerm
        }, website.name))), /*#__PURE__*/React.createElement("input", {
          type: "checkbox",
          checked: website.isChecked,
          onChange: _this5.handleChangePermissionState.bind(_this5, website)
        }), /*#__PURE__*/React.createElement("span", {
          className: "toggle-with-icon slider round"
        })));
      }), !this.state.isAnyIntegrationsVisible ? /*#__PURE__*/React.createElement("div", {
        className: "empty-text"
      }, "No integrations to display") : '')), /*#__PURE__*/React.createElement("div", {
        className: "integrations__actions"
      }, /*#__PURE__*/React.createElement("button", {
        type: "button",
        className: "button button--default",
        onClick: this.handleEnablePermissions.bind(this, this.getAllDomains())
      }, "Enable All"), /*#__PURE__*/React.createElement("button", {
        type: "button",
        className: "button button--default",
        onClick: this.handleDisablePermissions.bind(this, this.getAllDomains())
      }, "Disable All")));
    }
  }]);
}(React.Component);
{
  IntegrationsTab;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TimesheetTab = /*#__PURE__*/function (_React$Component) {
  function TimesheetTab(props) {
    var _this;
    _classCallCheck(this, TimesheetTab);
    _this = _callSuper(this, TimesheetTab, [props]);
    browserPort.analytics.event('popover_timesheet');
    _this.state = {
      today: moment().format('YYYY-MM-DD'),
      months: null,
      weekDays: null,
      isFetchTimesheet: false
    };
    return _this;
  }
  _inherits(TimesheetTab, _React$Component);
  return _createClass(TimesheetTab, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      if (!this.props.store.timeExport.isLoaded()) {
        this.getTimesheet();
      } else {
        this.setTimesheetState();
      }
    }
  }, {
    key: "setTimesheetState",
    value: function setTimesheetState() {
      this.setState({
        months: this.props.store.timeExport.months.filter(function (month) {
          return month.total > 0;
        }).length > 0 ? this.props.store.timeExport.months : [],
        weekDays: this.props.store.profile.get().team.beginningOfWeek ? ['MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT', 'SUN'] : ['SUN', 'MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT'],
        isFetchTimesheet: false
      });
    }
  }, {
    key: "getTimesheet",
    value: function getTimesheet() {
      var _this2 = this;
      this.setState({
        isFetchTimesheet: true
      });
      this.props.store.timeExport.load().then(function () {
        _this2.setTimesheetState();
      });
    }
  }, {
    key: "render",
    value: function render() {
      var _this3 = this;
      var months = this.state.months;
      var today = this.state.today;
      var isFetchTimesheet = this.state.isFetchTimesheet;
      return /*#__PURE__*/React.createElement("div", {
        className: "timesheet"
      }, !isFetchTimesheet && months ? months.map(function (month) {
        return /*#__PURE__*/React.createElement("div", {
          className: "timesheet__month",
          key: "".concat(month.monthName, "_").concat(month.start)
        }, /*#__PURE__*/React.createElement("div", {
          className: "month-header"
        }, /*#__PURE__*/React.createElement("div", {
          className: "month-header__name"
        }, month.monthName), /*#__PURE__*/React.createElement("div", {
          className: "month-header__separator"
        }), /*#__PURE__*/React.createElement("div", {
          className: "month-header__total"
        }, _format.time(month.total, '0h', 'clock'))), /*#__PURE__*/React.createElement("section", {
          className: "month__activity"
        }, /*#__PURE__*/React.createElement("header", {
          className: "activity__week-days"
        }, _this3.state.weekDays.map(function (day) {
          return /*#__PURE__*/React.createElement("div", {
            className: "week-days__day",
            key: "".concat(month.monthName, "_").concat(month.start, "_").concat(day)
          }, /*#__PURE__*/React.createElement("div", {
            className: "week-days__day-wrapper"
          }, day));
        }), /*#__PURE__*/React.createElement("div", {
          className: "week-days__total"
        }, /*#__PURE__*/React.createElement("div", {
          className: "week-days__total-wrapper"
        }, "Totals"))), month.weeks.map(function (week) {
          return /*#__PURE__*/React.createElement("article", {
            className: "activity__week",
            key: "".concat(month.monthName, "_").concat(month.start, "_").concat(week.period)
          }, week.days.map(function (day, index) {
            return /*#__PURE__*/React.createElement("div", {
              className: "activity__week-day",
              key: "".concat(month.monthName, "_").concat(month.start, "_").concat(week.period, "_").concat(day ? day.date : index)
            }, /*#__PURE__*/React.createElement("div", {
              className: "week-day \n                                                        ".concat(day && day.date === today ? 'week-day--today' : '', "\n                                                        ").concat(day && day.date > today ? 'week-day--future' : '')
            }, day ? day.day : ''), /*#__PURE__*/React.createElement("div", {
              "data-time": day ? day.time : null,
              className: "week-day__time \n                                                              ".concat(day && day.time ? _format.getDailyProgress(day.time) : '', "\n                                                              ").concat(day && day.date > today ? 'week-day__time--future' : '', "\n                                                              ").concat(!day || !day.time ? 'week-day__time--empty' : '')
            }, day ? day.time > 0 ? _format.time(day.time, null, 'clock') : '-' : ''));
          }), /*#__PURE__*/React.createElement("div", {
            className: "activity__week-total"
          }, /*#__PURE__*/React.createElement("div", {
            className: "activity__week-total-wrapper"
          }, /*#__PURE__*/React.createElement("div", {
            className: "week-total__period"
          }, week.period), /*#__PURE__*/React.createElement("div", {
            className: "week-total__time ".concat(!week.total ? 'week-total__time--empty' : '')
          }, week.total ? _format.time(week.total, null, 'clock') : '-'))));
        })));
      }) : '', !isFetchTimesheet && months && !months.length ? /*#__PURE__*/React.createElement("div", {
        className: "application-loading"
      }, /*#__PURE__*/React.createElement("div", {
        className: "application-loading-header"
      }, "Your timesheet will appear here"), /*#__PURE__*/React.createElement("div", {
        className: "application-loading-message"
      }, "Start tracking time and your ", /*#__PURE__*/React.createElement("br", null), "timesheet will be displayed here.")) : '', isFetchTimesheet ? /*#__PURE__*/React.createElement(Skeleton, {
        view: 'timesheet'
      }) : '');
    }
  }]);
}(React.Component);
{
  TimesheetTab;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var TaskDetailsTab = /*#__PURE__*/function (_React$Component) {
  function TaskDetailsTab(props) {
    var _this;
    _classCallCheck(this, TaskDetailsTab);
    _this = _callSuper(this, TaskDetailsTab, [props]);
    browserPort.analytics.event('popover_task');
    _this.state = {
      isFetchTaskDetails: true,
      modified: +new Date()
    };
    _this.taskDetails = null;
    _this.popoverControllerService = new PopupControllerService();
    _this.modalService = new ModalService();
    _this.currency = _format.getCurrency();
    _this.position = _this.currency.symbol ? 'left' : 'right';
    return _this;
  }
  _inherits(TaskDetailsTab, _React$Component);
  return _createClass(TaskDetailsTab, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      var self = this;
      self.fetchDetails();
      browserPort.eventDispatcher.on('browser-button', 'timer-started', function () {
        self.setState({
          isRenderCommentActual: true,
          modified: +new Date()
        });
      });
      browserPort.eventDispatcher.on('browser-button', 'websocket-update', function () {
        self.setState({
          modified: +new Date()
        });
      });
    }
  }, {
    key: "handleUpdateTask",
    value: function handleUpdateTask() {
      this.props.store.taskDetails.removeTask(this.props.taskId);
      this.fetchDetails().then(function () {
        setTimeout(function () {
          browserPort.eventDispatcher.broadcast('browser-button', 'details-updated');
        });
      });
    }
  }, {
    key: "navigateToParentTask",
    value: function navigateToParentTask(id) {
      if (!id) {
        return;
      }
      this.props.taskId = id;
      this.setState({
        isFetchTaskDetails: true
      });
      this.fetchDetails();
    }
  }, {
    key: "fetchDetails",
    value: function fetchDetails() {
      var _this2 = this;
      var requests = [this.fetchTaskDetails()];
      if (this.props.store.profile.isAdmin()) {
        requests.push(this.fetchMetrics());
      }
      return Promise.all(requests).then(function (result) {
        var taskDetails = result[0];
        var metrics = result[1];
        var popoverControllerService = _this2.popoverControllerService;
        var task = taskDetails.task,
          projectsAll = taskDetails.projectsAll;
        task.projectsObjects = _this2.mergeProjects(task.projectsObjects, projectsAll || []);
        var platform = getPlatformCode(task.id);
        var project = _this2.popoverControllerService.taskProject(task.projects, task.projectsObjects);
        // TODO Fix me. Need invert logic
        var isExternal = platform === 'everhour';
        var profile = _this2.props.store.profile;
        var self = _this2;
        self.setState({
          taskDetails: taskDetails,
          status: popoverControllerService.transformStatus(task),
          isExternal: isExternal,
          isEditable: popoverControllerService.checkIsEditable(project, isExternal),
          canStartTimer: popoverControllerService.canStartTimer(project),
          canManageTime: popoverControllerService.canManageTime(project),
          canEditEstimate: popoverControllerService.canEditEstimate(project, task),
          project: project,
          time: task.time ? task.time.total || 0 : 0,
          isRenderCommentActual: self.isRenderCommentActual(taskDetails.todayHistory),
          isFetchTaskDetails: false,
          cost: metrics && metrics.cost,
          isArchived: project && project.archived,
          isAdmin: profile.isAdmin() || profile.isSupervisor(),
          isMemberLimited: profile.get().role === 'member_limited',
          currentUserId: profile.get().id
        });
      });
    }
  }, {
    key: "fetchTaskDetails",
    value: function fetchTaskDetails() {
      return this.props.store.taskDetails.load(this.props.taskId);
    }
  }, {
    key: "fetchMetrics",
    value: function fetchMetrics() {
      return this.props.api.taskMetrics(this.props.taskId);
    }
  }, {
    key: "mergeProjects",
    value: function mergeProjects(projectsA, projectsB) {
      var projects = [];
      projectsA.forEach(function (projectA) {
        var projectB = projectsB.find(function (project) {
          return project.id === projectA.id;
        });
        if (projectB) {
          projectA = Object.assign(projectA, projectB);
        }
        projects.push(projectA);
      });
      return projects;
    }
  }, {
    key: "showAddTimeModal",
    value: function showAddTimeModal() {
      this.modalService.showModal(AddTimeModal, {
        task: this.state.taskDetails.task,
        store: this.props.store,
        api: this.props.api,
        reactAppStateObj: this.props.reactAppStateObj,
        onClose: this.handleFetchAfterUpdate.bind(this),
        onSuccess: this.handleFetchAfterUpdate.bind(this),
        isShowMoreButton: true
      });
    }
  }, {
    key: "showEditCommentModal",
    value: function showEditCommentModal() {
      var props = this.props;
      var state = this.state;
      var timer = props.store.timer.get();
      this.modalService.showModal(AddCommentModal, {
        task: state.taskDetails.task,
        comment: timer.comment,
        api: props.api,
        store: props.store,
        isCurrent: props.store.timer.activeTask() === state.taskDetails.task.id,
        onClose: this.handleFetchAfterUpdate.bind(this)
      });
    }
  }, {
    key: "showEditTimeModal",
    value: function showEditTimeModal(userIds) {
      var props = this.props;
      var state = this.state;
      this.modalService.showModal(EditTimeModal, {
        task: state.taskDetails.task,
        userIds: userIds,
        api: props.api,
        store: props.store,
        onClose: this.handleFetchAfterUpdate.bind(this),
        onUpdateTimeDone: this.handleFetchAfterUpdate.bind(this)
      }, {
        classes: 'modal--edit-time'
      });
    }
  }, {
    key: "handleFetchAfterUpdate",
    value: function handleFetchAfterUpdate(data) {
      if (data && data.isSuccessEditComment || data && data.isSuccessEditTime) {
        this.handleUpdateTask();
      }
    }
  }, {
    key: "getTodayTaskTime",
    value: function getTodayTaskTime(history) {
      var currentUserId = this.props.store.profile.get().id;
      var props = this.props;
      var state = this.state;
      return (history || state.taskDetails && state.taskDetails.history || []).find(function (h) {
        return h.user === currentUserId && h.date === moment().format('YYYY-MM-DD');
      }) || props.store.timer.isActive() && props.store.timer.activeTask() === props.taskId && props.store.timer.get() || null;
    }
  }, {
    key: "isRenderCommentActual",
    value: function isRenderCommentActual(todayHistory) {
      return this.props.store.timer.activeTask() === this.props.taskId;
    }
  }, {
    key: "handleCloseTaskDetails",
    value: function handleCloseTaskDetails() {
      browserPort.eventDispatcher.broadcast('browser-button', 'change-tab', {
        tab: null,
        taskId: null
      });
    }
  }, {
    key: "getStatusTooltip",
    value: function getStatusTooltip() {
      var state = this.state;
      var platform = getPlatformCode(state.taskDetails.task.id);
      if (!state.isExternal) {
        return {
          width: 100,
          text: 'Task status in ' + getCapitalizedName(platform)
        };
      }
      if (state.project && state.project.status === 'archived' && state.isEditable) {
        return {
          width: 60,
          text: 'Task status'
        };
      }
      if (!state.isEditable) {
        return {
          width: 200,
          text: 'Current task status (Editing available to assigned admins only)'
        };
      }
      return {
        width: 125,
        text: 'Click to ' + (state.status !== 'closed' ? 'close' : 'reopen') + ' task'
      };
    }
  }, {
    key: "handleUpdateTaskStatus",
    value: function handleUpdateTaskStatus() {
      var state = this.state;
      if (!state.isEditable) {
        return;
      }
      this.updateTask({
        status: state.status === 'closed' ? 'open' : 'closed'
      });
    }
  }, {
    key: "renderTaskStatusTemplate",
    value: function renderTaskStatusTemplate() {
      var state = this.state;
      var status = state.status.toLocaleLowerCase();
      return /*#__PURE__*/React.createElement("div", {
        className: "task-details__header-status ".concat(state.isEditable ? 'interactive' : 'locked-visible'),
        onClick: this.handleUpdateTaskStatus.bind(this)
      }, /*#__PURE__*/React.createElement("div", {
        className: "header-status__icon header-status__icon--".concat(status !== 'closed' ? 'open' : 'closed')
      }, /*#__PURE__*/React.createElement("svg", {
        height: "10",
        width: "12",
        xmlns: "http://www.w3.org/2000/svg"
      }, /*#__PURE__*/React.createElement("path", {
        d: "M12 2l-8 8-4-4 1.5-1.5L4 7 10.5.5z",
        "fill-rule": "evenodd"
      }))), /*#__PURE__*/React.createElement("div", {
        className: "header-status__text"
      }, state.taskDetails.task && state.taskDetails.task.status));
    }
  }, {
    key: "renderUserProgress",
    value: function renderUserProgress(user, taskId, total, count) {
      return /*#__PURE__*/React.createElement(TaskDetailsUserProgress, {
        user: user,
        taskId: taskId,
        total: total,
        store: this.props.store,
        onOpenEditTimeModal: this.handleOpenEditTimeModal.bind(this),
        isViewOnly: count <= 3
      });
    }
  }, {
    key: "handleOpenEditTimeModalAllProgress",
    value: function handleOpenEditTimeModalAllProgress() {
      this.showEditTimeModal(Object.keys(this.state.taskDetails.task.time.users).map(function (id) {
        return Number(id);
      }));
    }
  }, {
    key: "handleOpenEditTimeModal",
    value: function handleOpenEditTimeModal(userId) {
      this.showEditTimeModal([userId]);
    }
  }, {
    key: "getUserTotal",
    value: function getUserTotal(userId) {
      var state = this.state;
      var taskDetails = state.taskDetails.task;
      var time = taskDetails.time;
      var estimate = taskDetails.estimate;
      return {
        time: time && time.users && time.users[userId] || 0,
        estimate: estimate && estimate.users && estimate.users[userId] || 0
      };
    }
  }, {
    key: "updateTask",
    value: function updateTask(data) {
      var _this3 = this;
      this.props.api.updateTask(this.props.taskId, data).then(function () {
        _this3.handleUpdateTask();
      });
    }
  }, {
    key: "render",
    value: function render() {
      var _this4 = this;
      var isSafari = browserPort.type === 'safari';
      var state = this.state;
      var taskNameExternalSpace = 85;
      var tooltipCompensationSpace = 5;
      if (state.isFetchTaskDetails) {
        return /*#__PURE__*/React.createElement(Skeleton, {
          view: 'details'
        });
      }
      var statusTooltip = this.getStatusTooltip();
      var me = this.props.store.profile.get();
      var taskId = state.taskDetails.task.id;
      var collaborators = (state.taskDetails.collaborators || []).sort(function (userA, userB) {
        var userATotal = _this4.getUserTotal(userA.id);
        var userBTotal = _this4.getUserTotal(userB.id);
        return userBTotal.time - userATotal.time;
      });
      return /*#__PURE__*/React.createElement(React.Fragment, null, state.isArchived || state.project.foreign ? /*#__PURE__*/React.createElement("div", {
        className: "message message--warning"
      }, /*#__PURE__*/React.createElement("div", {
        className: "message__description message__description-trial"
      }, /*#__PURE__*/React.createElement("span", null, state.isArchived ? 'This project is archived.' : '', state.project.foreign ? 'You are not assigned to this project.' : ''))) : '', /*#__PURE__*/React.createElement("div", {
        className: "task-details__header"
      }, /*#__PURE__*/React.createElement(Tooltip, {
        space: tooltipCompensationSpace,
        placement: 'bottom',
        triggerTemplate: this.renderTaskStatusTemplate.bind(this),
        width: statusTooltip.width,
        classes: "tooltip-body--centered"
      }, statusTooltip.text), /*#__PURE__*/React.createElement(TaskDetailsBilling, {
        currencyPosition: this.position,
        task: state.taskDetails && state.taskDetails.task,
        project: state.project,
        store: this.props.store,
        api: this.props.api,
        onUpdateTask: this.handleUpdateTask.bind(this)
      }), /*#__PURE__*/React.createElement("div", {
        className: "task-details__close",
        onClick: this.handleCloseTaskDetails.bind(this)
      }, /*#__PURE__*/React.createElement("svg", {
        xmlns: "http://www.w3.org/2000/svg"
      }, /*#__PURE__*/React.createElement("path", {
        d: "M7.6 6.45l5.176-5.175.275-.275.549.55-.275.274L8.15 7l5.176 5.176.275.275-.55.549-.274-.275L7.6 7.55l-5.176 5.176-.275.275-.549-.55.275-.274L7.05 7 1.875 1.824 1.6 1.549 2.15 1l.274.275L7.6 6.45z",
        fill: "none",
        stroke: "#C7CACD",
        "stroke-linecap": "square",
        "stroke-width": "1.5"
      })))), /*#__PURE__*/React.createElement("div", {
        className: "task-details__name"
      }, state.taskDetails.task.url ? /*#__PURE__*/React.createElement(Tooltip, {
        space: tooltipCompensationSpace,
        width: !state.isExternal ? taskNameExternalSpace : null,
        placement: 'bottom',
        triggerTemplate: function triggerTemplate() {
          return /*#__PURE__*/React.createElement("a", {
            className: "text-link text-link__task",
            target: "_blank",
            href: state.taskDetails.task.url
          }, state.taskDetails.task.name || 'Task without title');
        },
        classes: "tooltip-body--centered"
      }, !state.isExternal ? this.popoverControllerService.numberTooltip(state.taskDetails.task.id) : 'Open task details in a new tab') : state.taskDetails.task.name || 'Task without title', state.taskDetails.task.parentId ? /*#__PURE__*/React.createElement("a", {
        className: "text-link text-link__task-parent",
        onClick: this.navigateToParentTask.bind(this, state.taskDetails.task.parentId)
      }, state.taskDetails.task.parentName || 'Task without title') : '', state.taskDetails.task.url && state.taskDetails.task.number ? /*#__PURE__*/React.createElement("span", {
        className: "name-number"
      }, /*#__PURE__*/React.createElement(Tooltip, {
        space: tooltipCompensationSpace,
        placement: 'bottom',
        width: taskNameExternalSpace,
        triggerTemplate: function triggerTemplate() {
          return /*#__PURE__*/React.createElement("a", {
            className: "text-link text-link__task-number",
            target: "_blank",
            href: state.taskDetails.task.url
          }, _this4.popoverControllerService.getTaskNumber(state.taskDetails.task));
        },
        classes: "tooltip-body--centered"
      }, this.popoverControllerService.numberTooltip(state.taskDetails.task.id))) : '', !state.taskDetails.task.url && state.taskDetails.task.number ? /*#__PURE__*/React.createElement("span", {
        className: "text-link text-link__task-number-static",
        target: "_blank"
      }, this.popoverControllerService.getTaskNumber(state.taskDetails.task)) : ''), /*#__PURE__*/React.createElement("div", {
        className: "task-details__total"
      }, /*#__PURE__*/React.createElement("div", {
        className: "task-details__total-time\n                              ".concat(state.time ? 'interactive' : ''),
        onClick: this.handleOpenEditTimeModalAllProgress.bind(this)
      }, /*#__PURE__*/React.createElement("div", {
        className: "total-time__text"
      }, "Time:\xA0"), /*#__PURE__*/React.createElement("div", {
        className: "total-time__hours"
      }, state.time ? TimeEntryUtils.format(state.time, '0h') : '0h')), /*#__PURE__*/React.createElement(TaskDetailsEstimatePopover, {
        store: this.props.store,
        canEditEstimate: state.canEditEstimate,
        taskId: state.taskDetails.task.id,
        estimate: state.taskDetails.task.estimate,
        time: state.taskDetails.task.time,
        userIds: !state.isMemberLimited ? state.project.users : state.project.users.filter(function (id) {
          return id === state.currentUserId;
        }),
        project: state.project,
        onUpdateTask: this.handleUpdateTask.bind(this)
      })), /*#__PURE__*/React.createElement("div", {
        className: "task-details__timer"
      }, this.renderUserProgress(me, taskId, this.getUserTotal(me.id)), /*#__PURE__*/React.createElement("div", {
        className: "timer__actions"
      }, state.canStartTimer && state.isRenderCommentActual ? /*#__PURE__*/React.createElement("div", {
        className: "timer__comment-icon",
        onClick: this.showEditCommentModal.bind(this)
      }, /*#__PURE__*/React.createElement("div", {
        className: "icon-task-comment"
      })) : '', state.canStartTimer ? /*#__PURE__*/React.createElement(TaskTimer, {
        taskId: state.taskDetails.task.id,
        isRunningTimer: state.taskDetails.task.id === this.props.store.timer.activeTask(),
        store: this.props.store,
        reactAppStateObj: this.props.reactAppStateObj,
        onStopTimer: this.handleUpdateTask.bind(this)
      }) : '', state.canManageTime ? /*#__PURE__*/React.createElement("button", {
        className: "button button--default",
        onClick: this.showAddTimeModal.bind(this)
      }, "Add Time") : '')), /*#__PURE__*/React.createElement(TaskDetailsProgress, {
        store: this.props.store,
        task: state.taskDetails && state.taskDetails.task,
        collaborators: collaborators,
        onOpenEditTimeModal: this.handleOpenEditTimeModal.bind(this)
      }), /*#__PURE__*/React.createElement(TaskDetailsInfo, {
        currency: this.currency,
        currencyPosition: this.position,
        isEditable: state.isEditable,
        taskId: taskId,
        store: this.props.store,
        cost: state.cost
      }), !isSafari && this.props.isNedPlanUpdate && /*#__PURE__*/React.createElement("div", {
        className: "task-details__plan-upgrade"
      }, /*#__PURE__*/React.createElement("a", {
        href: everhour.productionUrl + '/?upgrade',
        target: "_blank"
      }, /*#__PURE__*/React.createElement("span", {
        className: "icon-lightning"
      }), "Upgrade plan")));
    }
  }]);
}(React.Component);
{
  TaskDetailsTab;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var LoginController = /*#__PURE__*/function (_React$Component) {
  function LoginController(props) {
    var _this;
    _classCallCheck(this, LoginController);
    _this = _callSuper(this, LoginController, [props]);
    _this.state = {
      accessToken: '',
      error: props.error || null,
      isSubmitForm: false
    };
    return _this;
  }
  _inherits(LoginController, _React$Component);
  return _createClass(LoginController, [{
    key: "handleSubmitForm",
    value: function handleSubmitForm(event) {
      var _this2 = this;
      event.stopPropagation();
      event.preventDefault();
      this.setState({
        error: null,
        isSubmitForm: true
      });
      var everhourInstance = new Everhour2(this.state.accessToken);
      everhourInstance.profile().then(function (user) {
        // for disabled user we may access only to profile data
        if (user.status === 'removed') {
          _this2.setState({
            error: 'Your current account was suspended by team administrator.'
          });
          return;
        }
        var finalPromise = everhourInstance.exchangeApiKey();

        // we must exchange public key to private for avoid issues related to BE limits
        finalPromise.done(function (response) {
          var token = response.apiKey;
          browserPort.settings.set('accessToken', token);

          // required for Store -> isAuthorized method
          window.everhour = new Everhour2(token);
          ReactDOM.render(React.createElement(PopupController), document.getElementById('popup-controller-v2'));
        });
      })["catch"](function (err) {
        var error = null;
        if (err.status === 403) {
          error = 'The session has been invalidated because the API key has been changed';
        }
        if (err.status === 500) {
          error = 'Invalid API key';
        }
        if (!error) {
          return;
        }
        _this2.setState({
          error: error,
          isSubmitForm: false
        });
      });
    }
  }, {
    key: "handleUpdateToken",
    value: function handleUpdateToken(event) {
      this.setState({
        accessToken: event.target.value
      });
    }
  }, {
    key: "render",
    value: function render() {
      var state = this.state;
      var error = state.error;
      var isSafari = browserPort.type === 'safari';
      return /*#__PURE__*/React.createElement("div", {
        className: "user application-login"
      }, /*#__PURE__*/React.createElement("form", {
        onSubmit: this.handleSubmitForm.bind(this),
        autoComplete: 'off'
      }, /*#__PURE__*/React.createElement("div", {
        className: "application-login-logo"
      }, /*#__PURE__*/React.createElement("img", {
        src: "icons/logo/ev_48_logo.svg",
        alt: "everhour",
        title: "everhour"
      })), /*#__PURE__*/React.createElement("div", {
        className: "application-login-header"
      }, "Log in ", /*#__PURE__*/React.createElement("b", null, "Everhour")), /*#__PURE__*/React.createElement("div", {
        className: "application-login-message"
      }, "You need your Everhour API key to authorize. Find it by visiting your ", /*#__PURE__*/React.createElement("a", {
        href: "https://app.everhour.com/#/account/profile",
        target: "_blank",
        className: "external"
      }, "profile settings"), "."), error ? /*#__PURE__*/React.createElement("div", {
        className: "application-login-error"
      }, error) : '', /*#__PURE__*/React.createElement("div", {
        className: "application-login-input"
      }, /*#__PURE__*/React.createElement("input", {
        disabled: state.isSubmitForm,
        type: "text",
        name: "accessToken",
        placeholder: "API Key",
        value: state.accessToken,
        onChange: this.handleUpdateToken.bind(this),
        autoFocus: true
      })), /*#__PURE__*/React.createElement("div", {
        className: "application-login-button"
      }, /*#__PURE__*/React.createElement("button", {
        disabled: state.isSubmitForm,
        type: "submit",
        id: "login"
      }, "Log In")), /*#__PURE__*/React.createElement("div", {
        className: "application-login-footer"
      }, !isSafari ? /*#__PURE__*/React.createElement(React.Fragment, null, "Don\u2019t have an account? ", /*#__PURE__*/React.createElement("a", {
        href: "https://everhour.com",
        target: "_blank",
        className: "external"
      }, "Sign Up"), /*#__PURE__*/React.createElement("br", null)) : '', !isSafari ? /*#__PURE__*/React.createElement("a", {
        className: "help-center",
        target: "_blank",
        href: "https://support.everhour.com/knowledge-base/why-browser-extension-is-essential/"
      }, "Help Center") : '')));
    }
  }]);
}(React.Component);
{
  LoginController;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
var TrialController = /*#__PURE__*/function (_React$Component) {
  function TrialController(props) {
    var _this;
    _classCallCheck(this, TrialController);
    _this = _callSuper(this, TrialController, [props]);
    _defineProperty(_this, "extendTrial", function (event) {
      event.preventDefault();
      event.stopPropagation();
      _this.setState({
        isFetch: true
      });
      _this.props.store.profile.extendTrial().then(function () {
        window.browserPort = new BrowserPort('browser-button');
        window.app = new BrowserAction.Application();
      })["catch"](function (err) {
        _this.setState({
          isFetch: false,
          extendTrialError: 'Sorry, something went wrong. Could you please try again?'
        });
      });
    });
    var profile = props.store.profile.get();
    _this.isSafari = browserPort.type === 'safari';
    _this.state = {
      canPurchase: props.store.profile.canPurchase(),
      currentUserEmail: profile.email,
      owner: profile.team ? profile.team.owner.name : '',
      canExtendTrial: props.store.profile.canExtendTrial(),
      extendTrialError: null,
      isFetch: false
    };
    return _this;
  }
  _inherits(TrialController, _React$Component);
  return _createClass(TrialController, [{
    key: "render",
    value: function render() {
      var state = this.state;
      if (state.canExtendTrial) {
        return /*#__PURE__*/React.createElement("div", {
          className: "application-loading application-loading-trial"
        }, /*#__PURE__*/React.createElement("div", {
          className: "application-logo"
        }, /*#__PURE__*/React.createElement("img", {
          src: "icons/logo/ev_48_logo.svg",
          alt: "everhour",
          title: "everhour"
        })), /*#__PURE__*/React.createElement("div", {
          className: "application-loading-header application-error-color"
        }, "Your free trial has ended.", /*#__PURE__*/React.createElement("br", null), "Need more time?"), state.extendTrialError && /*#__PURE__*/React.createElement("div", {
          className: "application-login-error"
        }, state.extendTrialError), /*#__PURE__*/React.createElement("div", {
          className: "application-login-button"
        }, /*#__PURE__*/React.createElement("button", {
          disabled: state.isFetch,
          type: "button",
          onClick: this.extendTrial
        }, "Extend trial for 7 days")), state.canPurchase && !this.isSafari && /*#__PURE__*/React.createElement("div", {
          className: "external-subscribe"
        }, /*#__PURE__*/React.createElement("a", {
          className: "external",
          href: "https://app.everhour.com/",
          target: "_blank"
        }, "Subscribe today")));
      }
      return /*#__PURE__*/React.createElement("div", {
        className: "application-loading"
      }, /*#__PURE__*/React.createElement("div", {
        className: "application-logo"
      }, /*#__PURE__*/React.createElement("img", {
        src: "icons/logo/ev_48_logo.svg",
        alt: "everhour",
        title: "everhour"
      })), state.canPurchase ? /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
        className: "application-loading-header application-error-color"
      }, "Your trial has expired!"), /*#__PURE__*/React.createElement("div", {
        className: "application-loading-message"
      }, "Don\u2019t worry - all your data is safe.", /*#__PURE__*/React.createElement("br", null), "To continue using Everhour - please subscribe now."), !this.isSafari && /*#__PURE__*/React.createElement("div", {
        className: "application-login-button"
      }, /*#__PURE__*/React.createElement("a", {
        href: "https://app.everhour.com/",
        target: "_blank"
      }, "Subscribe"))) : /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
        className: "application-loading-header application-error-color"
      }, "Your trial has expired!"), /*#__PURE__*/React.createElement("div", {
        className: "application-loading-message"
      }, "The trial period for ", /*#__PURE__*/React.createElement("b", null, state.currentUserEmail), " has ended. Don't worry - all your data is safe."), /*#__PURE__*/React.createElement("div", {
        className: "application-loading-message"
      }, "To continue using Everhour, please contact your team admin to activate the full subscription.")));
    }
  }]);
}(React.Component);
{
  TrialController;
}
;
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var ErrorController = /*#__PURE__*/function (_React$Component) {
  function ErrorController(props) {
    var _this;
    _classCallCheck(this, ErrorController);
    _this = _callSuper(this, ErrorController, [props]);
    _this.state = {
      error: props.error
    };
    return _this;
  }
  _inherits(ErrorController, _React$Component);
  return _createClass(ErrorController, [{
    key: "render",
    value: function render() {
      return /*#__PURE__*/React.createElement("div", {
        className: "application-loading"
      }, /*#__PURE__*/React.createElement("div", {
        className: "application-loading-logo"
      }, /*#__PURE__*/React.createElement("img", {
        src: "icons/logo/ev_48_logo.svg",
        alt: "everhour",
        title: "everhour"
      })), /*#__PURE__*/React.createElement("div", {
        className: "application-loading-header application-error-color"
      }, this.state.error), /*#__PURE__*/React.createElement("div", {
        className: "application-loading-message"
      }, "Seems like a temporary glitch!", /*#__PURE__*/React.createElement("br", null), "Please re-open the extension or refresh the page."), /*#__PURE__*/React.createElement("div", {
        className: "application-login-button"
      }, /*#__PURE__*/React.createElement("button", {
        className: "refresh",
        type: "button",
        onClick: function onClick() {
          return window.location.reload();
        }
      }, "Refresh Page")));
    }
  }]);
}(React.Component);
{
  ErrorController;
}
;
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
var PopupControllerServiceInstance = null;
var PopupControllerService = /*#__PURE__*/function () {
  function PopupControllerService() {
    _classCallCheck(this, PopupControllerService);
    if (!PopupControllerServiceInstance) {
      this.isAdmin = store.profile.isAdmin();
      this.isSupervisor = store.profile.isSupervisor();
      this.searchProjectsSyncTimestamps = {};
      this.store = window.store;
      this.taskClosedStatuses = ['closed', 'archived', 'completed', 'deleted', 'done', 'accepted', 'complete'];
      this.taskTypes = ['task', 'issue', 'pull', 'pull_request', 'feature', 'epic', 'chore', 'bug', 'release'];
      this.taskTypesByPlatform = {
        'github': {
          'task': 'issue',
          'pull_request': 'pull request'
        },
        'bitbucket': {
          'task': 'bb-task',
          'proposal': 'bb-proposal',
          'enhancement': 'bb-enhancement',
          'bug': 'bb-bug'
        }
      };
      PopupControllerServiceInstance = this;
    }
    return PopupControllerServiceInstance;
  }
  return _createClass(PopupControllerService, [{
    key: "isSyncedProjectByTimestamp",
    value: function isSyncedProjectByTimestamp(projectId) {
      if (!this.searchProjectsSyncTimestamps[projectId]) {
        return false;
      }

      // sync project every 30m
      if (everhourMethods.timestamp() - this.searchProjectsSyncTimestamps[projectId] > 30 * 60) {
        return false;
      }
      return true;
    }
  }, {
    key: "setSyncedProjectTimestamp",
    value: function setSyncedProjectTimestamp(projectId) {
      this.searchProjectsSyncTimestamps[projectId] = everhourMethods.timestamp();
    }
  }, {
    key: "checkTaskArchived",
    value: function checkTaskArchived(task) {
      return task && task.project.status === 'archived';
    }
  }, {
    key: "checkArchivedOrForeignTask",
    value: function checkArchivedOrForeignTask(task, date) {
      if (task && (task._type || task.type) && (task._type === 'comment' || task.type === 'comment')) {
        return true;
      }
      var isForeign = true;
      if (task.projectsObjects) {
        isForeign = !(task.projectsObjects.findIndex(function (project) {
          return project.foreign === false;
        }) > -1);
      } else {
        isForeign = task.project.foreign;
      }
      return task && task.project && !this.checkTaskArchived(task) && (!isForeign || this.isAdmin && !isForeign) || !task && date && date === moment().format('YYYY-MM-DD');
    }
  }, {
    key: "checkShowTimerWithoutTask",
    value: function checkShowTimerWithoutTask(taskTime) {
      return true;

      // const date = taskTime.date ? taskTime.date : null;

      // return date && date === moment().format('YYYY-MM-DD')
    }
  }, {
    key: "getTimeTrackingPolicy",
    value: function getTimeTrackingPolicy() {
      return store.profile.index.timeTrackingPolicy;
    }
  }, {
    key: "calculateMinAllowedDate",
    value: function calculateMinAllowedDate() {
      var format = 'YYYY-MM-DD';
      var timeTrackingPolicy = this.getTimeTrackingPolicy();
      if (!timeTrackingPolicy.lockTimeAfter && timeTrackingPolicy.lockTimeAfter !== 0) {
        return null;
      }
      var lockTimePeriod = timeTrackingPolicy.lockTimePeriod || 'no-lock';
      if (!timeTrackingPolicy.lockTimePeriod && timeTrackingPolicy.lockTimeAfter >= 0) {
        lockTimePeriod = 'days';
      }
      var today = moment.utc();
      switch (lockTimePeriod) {
        case 'days':
          return today.subtract(timeTrackingPolicy.lockTimeAfter, 'days').format(format);
        case 'day-of-week':
          var days = today.day() - timeTrackingPolicy.lockTimeAfter;
          return today.subtract(days >= 0 ? days : days + 7, 'days').format(format);
        case 'day-of-month':
          var daysInMonth = today.daysInMonth();
          var lockTimeAfter = timeTrackingPolicy.lockTimeAfter > daysInMonth ? daysInMonth : timeTrackingPolicy.lockTimeAfter;
          var startOfMonth = today.date() < lockTimeAfter ? today.subtract(1, 'month').startOf('month') : today.startOf('month');
          if (lockTimeAfter > 1) {
            return startOfMonth.add(lockTimeAfter - 1, 'days').format(format);
          }
          return startOfMonth.format(format);
        default:
          return null;
      }
    }
  }, {
    key: "minAllowedDate",
    value: function minAllowedDate() {
      var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        _ref$forceUseTeamLock = _ref.forceUseTeamLockTime,
        forceUseTeamLockTime = _ref$forceUseTeamLock === void 0 ? false : _ref$forceUseTeamLock;
      if (forceUseTeamLockTime === false && this.isAdmin) {
        return null;
      }
      return this.calculateMinAllowedDate();
    }
  }, {
    key: "maxAllowedDate",
    value: function maxAllowedDate() {
      var timeTrackingPolicy = this.getTimeTrackingPolicy;
      if (timeTrackingPolicy.allowFutureTime) {
        return null;
      }
      return moment.utc();
    }
  }, {
    key: "isLockedFutureDate",
    value: function isLockedFutureDate(date) {
      var max = this.maxAllowedDate();
      return max !== null && moment(date).isAfter(max) === true;
    }
  }, {
    key: "isLockedDate",
    value: function isLockedDate(date) {
      var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        _ref2$forceUseTeamLoc = _ref2.forceUseTeamLockTime,
        forceUseTeamLockTime = _ref2$forceUseTeamLoc === void 0 ? false : _ref2$forceUseTeamLoc;
      var max = this.maxAllowedDate();
      if (max !== null && moment(date).isAfter(max) === true) {
        return true;
      }
      var min = this.minAllowedDate({
        forceUseTeamLockTime: forceUseTeamLockTime
      });
      return min !== null && moment(date).isBefore(min);
    }
  }, {
    key: "editEstimate",
    value: function editEstimate(project) {
      if (!project) {
        return false;
      }
      var isActiveProject = project.status === 'open';
      var isProjectEditable = getPlatformCode(project.id) === 'everhour' ? project.editable : true;
      var isAdmin = this.isAdmin;
      var isSupervisor = this.isSupervisor;

      // check project availability
      return isAdmin || project && isProjectEditable && (isSupervisor || isActiveProject);
    }
  }, {
    key: "editTime",
    value: function editTime(date, userId, project) {
      if (date && this.isLockedFutureDate(date)) {
        return 'locked';
      }
      var user = store.profile.get();
      if (user.role === 'admin') {
        return 'permitted';
      }
      if (project && (project.foreign || project.status !== 'open')) {
        return 'locked';
      }
      if (userId !== null && user.id !== userId && user.role !== 'supervisor') {
        return 'denied';
      }
      if (user.role === 'supervisor') {
        return 'permitted';
      }
      var min = this.minAllowedDate();
      if (!min) {
        return 'permitted';
      }
      return date < min ? 'locked' : 'permitted';
    }
  }, {
    key: "editTask",
    value: function editTask(project) {
      var user = store.profile.get();
      return project.editable && !project.foreign && (project.status !== 'archived' || user.role === 'admin');
    }
  }, {
    key: "runningTimerForTask",
    value: function runningTimerForTask(timerStore, taskTimeObj) {
      var task = taskTimeObj.task;
      var timer = timerStore.get();
      if (timer === null) {
        return false;
      }
      if (task === null) {
        return false;
      }
      if (timer.task === null) {
        return false;
      }
      if (timerStore.activeTask() && timerStore.activeTask() === task.id) {
        return true;
      }
      return false;
    }
  }, {
    key: "checkActiveTimer",
    value: function checkActiveTimer(taskTimeObj) {
      if (!taskTimeObj) {
        return false;
      }
      var task = taskTimeObj.task;
      var taskTime = taskTimeObj.taskTime || taskTimeObj.currentTaskTime;
      var id = taskTimeObj.id || taskTime && taskTime.id;
      var date = taskTimeObj.date || taskTime && taskTime.date || taskTimeObj.userDate;
      var timerStore = this.store.timer;
      var timer = timerStore.get();
      return task && timerStore.activeTask() === task.id && timer && date === timer.userDate || !task && timer && date === timer.userDate && timer && timer.currentTaskTime && id === timer.currentTaskTime.id || !task && timer && date === timer.userDate && timer && !timer.currentTaskTime && taskTimeObj.isCurrent;
    }
  }, {
    key: "checkActiveTimerForTaskDetails",
    value: function checkActiveTimerForTaskDetails(taskTimeObj) {
      if (!taskTimeObj) {
        return false;
      }
      var task = taskTimeObj.task;
      var timerStore = this.store.timer;
      return task && timerStore.activeTask() === task.id;
    }
  }, {
    key: "transformStatus",
    value: function transformStatus(task) {
      var userId = this.store.profile.get().id;
      if (!task || !task.status) {
        return '';
      }
      var status = this.taskClosedStatuses.indexOf(task.status.toLowerCase()) >= 0 ? 'closed' : 'open';
      if (status === 'closed' || !task.estimate || !task.time) {
        return status;
      }
      var useUserEstimate = userId && task.estimate.type !== 'overall';
      var estimate = useUserEstimate ? task.estimate.users[userId] || 0 : task.estimate.total;
      var time = userId && useUserEstimate ? task.time.users[userId] || 0 : task.time.total;
      if (status === 'open' && estimate > 0 && estimate < time) {
        return 'exceeded-estimate';
      }
      return status;
    }
  }, {
    key: "taskProject",
    value: function taskProject(prjs, projectsObj) {
      var _this = this;
      var projects = (prjs || []).map(function (id) {
        if (_this.store.projects.get(id)) {
          return _this.store.projects.get(id);
        }
        return projectsObj.find(function (project) {
          return project.id === id;
        });
      }).filter(function (project) {
        return !!project;
      });
      if (projects.length === 0) {
        return null;
      }
      var _iterator = _createForOfIteratorHelper(projects),
        _step;
      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var project = _step.value;
          if (project && project.status === 'open' && !project.foreign) {
            return project;
          }
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }
      var _iterator2 = _createForOfIteratorHelper(projects),
        _step2;
      try {
        for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
          var _project = _step2.value;
          if (_project && !_project.foreign) {
            return _project;
          }
        }
      } catch (err) {
        _iterator2.e(err);
      } finally {
        _iterator2.f();
      }
      return projects[0];
    }
  }, {
    key: "getTaskType",
    value: function getTaskType(id, typeKey) {
      var taskTypesByPlatform = this.taskTypesByPlatform;
      var typeName = typeKey ? typeKey.toLowerCase() : '';
      var platform = id ? getPlatformCode(id) : null;
      if (platform && taskTypesByPlatform[platform] && taskTypesByPlatform[platform][typeName]) {
        return taskTypesByPlatform[platform][typeName];
      }
      if (this.taskTypes.indexOf(typeName) < 0) {
        return '';
      }
      return typeName;
    }
  }, {
    key: "checkIsEditable",
    value: function checkIsEditable(project, isInternal) {
      if (!project || !isInternal) {
        return false;
      }
      if (this.isAdmin && !project.foreign) {
        return true;
      }
      return this.editTask(project);
    }
  }, {
    key: "canStartTimer",
    value: function canStartTimer(project) {
      return project && project.status === 'open' && !project.foreign;
    }
  }, {
    key: "canManageTime",
    value: function canManageTime(project) {
      var isActiveProject = this.isActiveProject(project);
      return this.isAdmin || this.isSupervisor && project && this.isProjectEditable(project) && isActiveProject || project && isActiveProject;
    }
  }, {
    key: "canManageEstimate",
    value: function canManageEstimate(project) {
      return this.isAdmin || this.isSupervisor && project && this.isProjectEditable(project) || project && this.isActiveProject(project);
    }
  }, {
    key: "canEditEstimate",
    value: function canEditEstimate(project, task) {
      return this.canManageEstimate(project) === true && (project.estimatesType !== 'none' || task.estimate && task.estimate.total > 0);
    }
  }, {
    key: "checkIsBillable",
    value: function checkIsBillable(projects) {
      var _this2 = this;
      var billableProject = (projects || []
      // .map(id => this.store.projects.get(id))
      ).find(function (project) {
        return _this2.isProjectBillable(project);
      });
      return !!billableProject;
    }
  }, {
    key: "isProjectEditable",
    value: function isProjectEditable(project) {
      return getPlatformCode(project.id) === 'everhour' ? project.editable : true;
    }
  }, {
    key: "isActiveProject",
    value: function isActiveProject(project) {
      return project.status === 'open';
    }
  }, {
    key: "isProjectBillable",
    value: function isProjectBillable(project) {
      if (!project || !project.billing) {
        return false;
      }
      return project.billing && project.billing.type !== 'non_billable';
    }
  }, {
    key: "isProjectFixed",
    value: function isProjectFixed(project) {
      if (!project.billing) {
        return false;
      }
      return project.billing && project.billing.type === 'fixed_fee';
    }
  }, {
    key: "hasAbilityCustomRate",
    value: function hasAbilityCustomRate(projects) {
      if (!projects) {
        return false;
      }
      return projects.filter(function (project) {
        if (!project) {
          return null;
        }
        return project.billing && project.billing.type === 'hourly';
      }).length > 0;
    }
  }, {
    key: "formatNumber",
    value: function formatNumber(minimumFractionDigits, maximumFractionDigits, value) {
      return value.toLocaleString('en-US', {
        style: 'decimal',
        minimumFractionDigits: minimumFractionDigits,
        maximumFractionDigits: maximumFractionDigits
      });
    }
  }, {
    key: "getMoney",
    value: function getMoney(value) {
      // using 'en-US' for general number format like 1,000.00
      var zero = 0;
      var formattedValue = this.formatNumber(2, 2, toBasicUnit(value || 0));
      if (parseFloat(formattedValue) !== 0) {
        return formattedValue;
      }
      return this.formatNumber(2, 2, zero);
    }
  }, {
    key: "getMoneyRound",
    value: function getMoneyRound(value) {
      // using 'en-US' for general number format like 1,000
      var negative = false;
      if (value < 0) {
        negative = true;
      }
      var rounded = Math.round(Math.abs(toBasicUnit(value))) || 0;
      return this.formatNumber(0, 0, negative ? -rounded : rounded);
    }
  }, {
    key: "getMoneyWithSymbol",
    value: function getMoneyWithSymbol(value) {
      var format = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'default';
      var thousandPrefix = arguments.length > 2 ? arguments[2] : undefined;
      var isRoundRequired = format === 'narrow';
      var valueFormatted = isRoundRequired ? this.getMoneyRound(value) : this.getMoney(value);
      return this.adjustWithSymbol(valueFormatted, format, false, thousandPrefix);
    }
  }, {
    key: "adjustWithSymbol",
    value: function adjustWithSymbol(valueFormatted, format, isCurrencyUSD, thousandPrefix) {
      var isFinancialFormat = format === 'financial';
      var currency = isCurrencyUSD ? CurrencyUSD : getCurrency();
      var sign = '';
      if (valueFormatted[0] === '-') {
        valueFormatted = valueFormatted.substr(1);
        sign = '–';
      }
      if (thousandPrefix || thousandPrefix === null) {
        valueFormatted += thousandPrefix || '';
      }
      var currencyPrefix;
      if (currency.symbol) {
        valueFormatted = currency.symbol + valueFormatted;
      } else if (currency.symbolNarrow && !isFinancialFormat && (!thousandPrefix || thousandPrefix && !currency.symbolNarrow.includes('.'))) {
        currencyPrefix = {
          prefix: currency.symbolNarrow,
          withSpace: currency.symbolNarrow.length > 1
        };
      } else if (!isFinancialFormat) {
        currencyPrefix = {
          prefix: currency.code,
          withSpace: true
        };
      }
      if (currencyPrefix) {
        if (thousandPrefix || thousandPrefix === null) {
          valueFormatted = currencyPrefix.prefix + (currencyPrefix.withSpace ? ' ' : '') + valueFormatted;
        } else {
          valueFormatted = valueFormatted + ' ' + currencyPrefix.prefix;
        }
      }

      // for invoice(isFinancialFormat) format we always use ISO currency code at the end
      // we add only prefix symbol! before value
      // invoice format like $1,000.54 USD (has symbol)|| 1,000.54 HRK (has symbolNarrow) || 1,000.54 TZS (no symbol either symbolNarrow)
      if (isFinancialFormat) {
        valueFormatted = valueFormatted + ' ' + currency.code;
      }
      return sign + valueFormatted;
    }
  }, {
    key: "getProjects",
    value: function getProjects(taskTime) {
      var projects = [];
      if (taskTime.project && taskTime.project.id) {
        projects = [taskTime.project.id];
      }
      if (projects.length === 0) {
        projects = taskTime.task && taskTime.task.projects ? taskTime.task.projects : taskTime.projects || [];
      }
      return projects;
    }
  }, {
    key: "getUserRate",
    value: function getUserRate(project, userId) {
      var user = this.store.users.get(userId);
      if (!project || !user) {
        return 0;
      }
      var billing = project.billing;
      var rate = project.rate;
      if (!this.isProjectBillable(project)) {
        return 0;
      }
      if (billing.type === 'hourly' && rate && rate.type === 'project_rate') {
        return rate.rate || 0;
      }
      if (rate && rate.userRateOverrides && rate.userRateOverrides.hasOwnProperty(userId)) {
        return rate.userRateOverrides[user.id] || 0;
      }
      return user.rate || 0;
    }
  }, {
    key: "billing",
    value: function billing(taskTime, projectId, projectsArr) {
      var _this3 = this;
      var userId = taskTime.user && taskTime.user.id ? taskTime.user.id : taskTime.user;
      var projects = projectId ? [projectId] : this.getProjects(taskTime);
      var billableProjects = projects.map(function (id) {
        var project = _this3.store.projects.get(id) || projectsArr.find(function (project) {
          return project.id === id;
        });
        var isBillable = false;
        var rate = _this3.getUserRate(project, userId);
        if (project) {
          if (_this3.isProjectBillable(project) && rate > 0) {
            isBillable = true;
          }
          if (_this3.isProjectFixed(project)) {
            rate = 0;
          }
        }
        return {
          isBillable: isBillable,
          project: project,
          rate: rate
        };
      }).filter(function (item) {
        return item.isBillable === true;
      });
      var maxRate = 0;
      var itemWithMaxRate = null;
      billableProjects.forEach(function (item) {
        if (item.rate >= maxRate) {
          itemWithMaxRate = item;
        }
      });
      return {
        isBillable: billableProjects.length > 0,
        rate: itemWithMaxRate ? itemWithMaxRate.rate : 0,
        project: itemWithMaxRate ? itemWithMaxRate.project : null
      };
    }
  }, {
    key: "getDueDateLabel",
    value: function getDueDateLabel(dueDate) {
      var todayMillis = moment().startOf('day')._d.valueOf();
      var yesterdayMillis = moment().startOf('day').subtract(1, 'day')._d.valueOf();
      var dueDateMillis = moment(dueDate).startOf('day')._d.valueOf();
      if (dueDateMillis === todayMillis) {
        return {
          label: 'Today',
          color: 'green'
        };
      }
      if (dueDateMillis === yesterdayMillis) {
        return {
          label: 'Yesterday',
          color: 'red'
        };
      }
      if (dueDateMillis < yesterdayMillis) {
        return {
          label: moment(dueDate).format('MMM D'),
          color: 'red'
        };
      }
      return {
        label: moment(dueDate).format('MMM D'),
        color: null
      };
    }
  }, {
    key: "getLocktimeReasons",
    value: function getLocktimeReasons(timeEntry) {
      var reasons = [];
      var lockReasons = timeEntry.lockReasons || [];
      var isTimeManagementAllowed = this.store.profile.isTimeManagementAllowed();
      var currentUserId = this.store.profile.index.id;
      if (lockReasons.indexOf('time_invoiced') !== -1) {
        reasons.push('This time has been invoiced.');
      }
      if (lockReasons.indexOf('time_approved') !== -1) {
        reasons.push('This time has been approved.');
      }
      if (lockReasons.indexOf('date') !== -1 && !isTimeManagementAllowed) {
        reasons.push("".concat(_format.lockAfterText(), "."));
      }
      if (lockReasons.indexOf('future_date') !== -1) {
        reasons.push('Time edit is limited for future dates.');
      }
      if (!isTimeManagementAllowed && currentUserId !== timeEntry.user) {
        reasons.push('This time does not belong to you.');
      }
      if (lockReasons.indexOf('project_archived') !== -1) {
        reasons.push('Time edit is restricted in archived projects.');
      }
      if (lockReasons.indexOf('timesheet_approved') !== -1) {
        reasons.push('This time has been approved.');
      }
      if (lockReasons.indexOf('timesheet_approval_requested') !== -1) {
        reasons.push('This time has been sent for approval.');
      }
      return reasons;
    }
  }, {
    key: "getTaskNumber",
    value: function getTaskNumber(task) {
      var number = task.number || this.getPlatformId(task.id);
      return number ? '#' + number : '';
    }
  }, {
    key: "getPlatformId",
    value: function getPlatformId(id) {
      if (!id) {
        return '';
      }
      return id.substr(3);
    }
  }, {
    key: "numberTooltip",
    value: function numberTooltip(taskId) {
      var platform = getPlatformCode(taskId);
      return 'Open in ' + getCapitalizedName(platform);
    }
  }]);
}();
{
  PopupControllerService;
}
"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _callSuper(t, o, e) { return o = _getPrototypeOf(o), _possibleConstructorReturn(t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf(t).constructor) : o.apply(t, e)); }
function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } else if (call !== void 0) { throw new TypeError("Derived constructors may only return object or undefined"); } return _assertThisInitialized(self); }
function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); Object.defineProperty(subClass, "prototype", { writable: false }); if (superClass) _setPrototypeOf(subClass, superClass); }
function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }
var PopupController = /*#__PURE__*/function (_React$Component) {
  function PopupController(props) {
    var _this;
    _classCallCheck(this, PopupController);
    _this = _callSuper(this, PopupController, [props]);
    _this.state = {
      isFetchStoreData: true,
      currentActiveTab: 'time',
      tabs: [{
        id: 'time',
        name: 'Time'
      }, {
        id: 'recent',
        name: 'Recent'
      }, {
        id: 'timesheet',
        name: 'Timesheet'
      }],
      activeTaskId: null,
      unpermittedDomains: null,
      isTimerRunning: false
    };
    _this.modalService = new ModalService();
    _this.reactAppStateObj = {
      todayTimecard: null,
      lockActions: false
    };
    window.everhour.loadProjectsInTasks = 'opts_include_projects=1';
    window.everhour.onError = _this.handleEverhourError.bind(_this);
    _this.store = window.store = new Store();
    _this.api = window.everhour;
    _this.tabsComponentProps = {
      store: _this.store,
      api: _this.api,
      reactAppStateObj: _this.reactAppStateObj
    };
    _this.tabsComponent = {
      time: {
        id: 'time-tab',
        component: TimeTab,
        props: _this.createProps()
      },
      recent: {
        id: 'recent-tab',
        component: RecentTab,
        props: _this.createProps()
      },
      timesheet: {
        id: 'timesheet-tab',
        component: TimesheetTab,
        props: _this.createProps()
      },
      timers: {
        id: 'timers-tab',
        component: TimersTab,
        props: _this.createProps()
      },
      search: {
        id: 'search-tab',
        component: SearchTab,
        props: _this.createProps()
      },
      taskDetails: {
        id: 'task-details-tab',
        component: TaskDetailsTab,
        props: _this.createProps()
      },
      integrations: {
        id: 'integrations-tab',
        component: IntegrationsTab,
        props: _this.createProps()
      }
    };
    _this._handleTimerTick = _this.handleTimerTick.bind(_this);
    _this.minutes = 0;
    return _this;
  }
  _inherits(PopupController, _React$Component);
  return _createClass(PopupController, [{
    key: "handleEverhourError",
    value: function handleEverhourError(error) {
      console.log('handleEverhourError ', error);
      var self = this;
      switch (true) {
        // backend validation error. We have to handle some error for own context and
        // broadcast message to ErrorWrapper component
        case error.status === 400:
          return;
        // trial expired. We have to render trial expired template
        case error.status === 402:
          // self.controllers.error.trialAction();
          self.renderTrialTemplate();
          return;
        // invalid access token. We have to render login template and show error
        case error.status === 403:
          browserPort.settings.set('accessToken', '').done(function () {
            // self.controllers.user.loginFailAction('The session has been invalidated because the API key has been changed');
            self.renderLoginTemplate('The session has been invalidated because the API key has been changed');
          });
          return;
        // Invalid API key. We have to render login template and show error
        case error.status === 500 && error.responseJSON && error.responseJSON.error === 'Invalid API key':
          return;
        // manual abort(search by typing)
        case error.status === 0 && error.statusText === 'abort':
          return;
      }
      self.renderErrorTemplate('Sorry, an error has just occurred.');
    }
  }, {
    key: "renderLoginTemplate",
    value: function renderLoginTemplate(error) {
      ReactDOM.render(React.createElement(LoginController, {
        error: error
      }), document.getElementById('popup-controller-v2'));
    }
  }, {
    key: "renderTrialTemplate",
    value: function renderTrialTemplate() {
      ReactDOM.render(React.createElement(TrialController, {
        store: this.store,
        api: this.api
      }), document.getElementById('popup-controller-v2'));
    }
  }, {
    key: "renderErrorTemplate",
    value: function renderErrorTemplate(error) {
      ReactDOM.render(React.createElement(ErrorController, {
        error: error
      }), document.getElementById('popup-controller-v2'));
    }
  }, {
    key: "createProps",
    value: function createProps() {
      return Object.assign({}, this.tabsComponentProps);
    }
  }, {
    key: "setBrowserIcon",
    value: function setBrowserIcon() {
      browserPort.action.changeState(!this.store.profile.isSuspended() ? this.store.timer.isActive() ? 'stop' : 'start' : null);
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      var _this2 = this;
      this.store.subscribe(function () {
        var isTimerRunning = _this2.checkTimerRunning();
        if (isTimerRunning && !_this2.state.isTimerRunning) {
          _this2.setState({
            isTimerRunning: true
          });
          _this2.store.timer.timerTick(_this2._handleTimerTick);
          browserPort.eventDispatcher.broadcast('browser-button', 'timer-started');
          browserPort.eventDispatcher.broadcast('browser-button', 'timer-tick');
          _this2.setBrowserIcon();
          return;
        }
        if (!isTimerRunning && _this2.state.isTimerRunning) {
          _this2.setState({
            isTimerRunning: false
          });
          _this2.store.timer.clearTimerTick();
          browserPort.eventDispatcher.broadcast('browser-button', 'timer-tick');
          _this2.setBrowserIcon();
          return;
        }
        browserPort.eventDispatcher.broadcast('browser-button', 'websocket-update');
      });
      browserPort.eventDispatcher.on('browser-button', 'show-modal', function (data) {
        if (data && data.modalName === 'add-time') {
          _this2.modalService.showModal(AddTimeModal, {
            task: data.task
          });
        }
      });
      browserPort.eventDispatcher.on('browser-button', 'change-tab', function (data) {
        _this2.setState({
          activeTaskId: data && data.taskId || null,
          currentActiveTab: data && data.taskId ? _this2.state.currentActiveTab : data.tab || _this2.state.currentActiveTab,
          isSearchOpenFromLens: data && data.isSearchOpenFromLens || false
        });
        if (data && data.tab) {
          _this2.tabsComponent[data.tab].props = _this2.createProps();
        }
      });
      document.addEventListener('keydown', function (event) {
        if (event.keyCode === 27 && _this2.state.currentActiveTab === 'search') {
          event.preventDefault();
          event.stopPropagation();
          browserPort.eventDispatcher.broadcast('browser-button', 'change-tab', {
            tab: 'time'
          });
        }
      });
      this.loadPopup();
    }
  }, {
    key: "checkTimerRunning",
    value: function checkTimerRunning() {
      return this.store.timer.activeTask() || this.store.timer.runningWithoutTask();
    }
  }, {
    key: "handleTimerTick",
    value: function handleTimerTick() {
      var duration = this.store.timer.duration();

      // We are needing update all tabs once per minute(update total time, time for row when running timer)
      var minutes = duration && Math.floor(store.timer.duration() / 30);
      if (minutes !== this.minutes) {
        this.minutes = minutes;
        browserPort.eventDispatcher.broadcast('browser-button', 'manual-update');
      }
      browserPort.eventDispatcher.broadcast('browser-button', 'timer-tick');
    }
  }, {
    key: "loadPopup",
    value: function loadPopup() {
      var _this3 = this;
      this.store.loadPopup(function () {
        return [_this3.loadTimeCard(), _this3.loadUnpermittedDomains()];
      }).then(function () {
        browserPort.analytics.event('popover');
        if (_this3.checkTimerRunning()) {
          _this3.store.timer.timerTick(_this3._handleTimerTick);
          browserPort.eventDispatcher.broadcast('browser-button', 'timer-tick');
        }
        _this3.setState({
          isFetchStoreData: false,
          tabs: _this3.store.profile.isAdmin() || _this3.store.profile.isSupervisor() ? _this3.state.tabs.concat([{
            id: 'timers',
            name: 'Timers'
          }]) : _this3.state.tabs,
          isTimerRunning: _this3.checkTimerRunning()
        });
        _this3.setBrowserIcon();
      }).then(function () {
        _this3.store.projects.load();
        _this3.store.clients.load();
      })["catch"](function (error) {
        // We are triggering manual error
        // $(document).trigger('ajaxError', error);
        _this3.handleEverhourError(error);
      });
    }
  }, {
    key: "loadUnpermittedDomains",
    value: function loadUnpermittedDomains() {
      var _this4 = this;
      var deferred = $.Deferred();
      this.unpermittedDomains = [];
      browserPort.eventDispatcher.broadcast('background', 'custom-domains:get-unpermitted-domains', null, function (domains) {
        _this4.setState({
          unpermittedDomains: domains
        });
        deferred.resolve();
      });
      return deferred.promise();
    }
  }, {
    key: "loadTimeCard",
    value: function loadTimeCard() {
      var _this5 = this;
      if (this.isTimecardsEnabled()) {
        return window.everhour.getUserTimecard(this.getCurrentUserId()).done(function (todayTimecard) {
          _this5.reactAppStateObj.todayTimecard = todayTimecard;
          _this5.store.timer.setTimecard(todayTimecard);
        });
      }
      var deferred = $.Deferred();
      deferred.resolve();
      return deferred.promise();
    }
  }, {
    key: "isTimecardsEnabled",
    value: function isTimecardsEnabled() {
      return window.store.profile.isTimecardsEnabled();
    }
  }, {
    key: "getCurrentUserId",
    value: function getCurrentUserId() {
      return window.store.profile.get().id;
    }
  }, {
    key: "render",
    value: function render() {
      var state = this.state;
      var team = this.store.profile.get().team;
      var billing = team && team.billing;
      // const isNedPlanUpdate = user && user.team && user.team.planFeatures
      //     && !user.team.planFeatures.integrations && this.store.profile.isAdmin();
      var isNedPlanUpdate = browserPort.type !== 'safari' && billing && team.billing.plan && (team.billing.plan.deprecated || !!~['lite', 'basic', 'free', 'lite_monthly', 'lite_yearly', 'basic_monthly', 'basic_yearly'].indexOf(team.billing.plan)) && !team.billing.teamPlanTrialUntil && this.store.profile.isAdmin();
      var componentObj = this.tabsComponent[state.activeTaskId ? 'taskDetails' : state.currentActiveTab];
      var Component = componentObj.component;
      var componentProps = componentObj.props;
      if (state.currentActiveTab === 'time' && !state.activeTaskId) {
        componentProps.unpermittedDomains = state.unpermittedDomains;
      }
      if (state.activeTaskId) {
        componentProps.taskId = state.activeTaskId;
        componentProps.isNedPlanUpdate = isNedPlanUpdate;
      }
      if (state.currentActiveTab === 'search' && state.isSearchOpenFromLens) {
        componentProps.isSearchOpenFromLens = true;
      }
      return /*#__PURE__*/React.createElement("div", {
        className: "ev-popover"
      }, state.isFetchStoreData ? /*#__PURE__*/React.createElement(Skeleton, {
        view: "time"
      }) : '', !state.isFetchStoreData ? /*#__PURE__*/React.createElement(React.Fragment, null, !state.activeTaskId && state.currentActiveTab !== 'integrations' ? /*#__PURE__*/React.createElement(Toolbar, {
        isSearchOpenFromLens: state.isSearchOpenFromLens,
        isNedPlanUpdate: isNedPlanUpdate,
        tabs: state.tabs,
        activeTab: state.currentActiveTab,
        api: this.api,
        store: this.store,
        reactAppStateObj: this.reactAppStateObj
      }) : '', /*#__PURE__*/React.createElement(ErrorWrapper, {
        reactAppStateObj: this.reactAppStateObj
      }), !state.activeTaskId && state.currentActiveTab !== 'integrations' ? /*#__PURE__*/React.createElement(BillingMessageWrapper, {
        store: this.store
      }) : '', /*#__PURE__*/React.createElement("main", {
        className: "ev-popover__tab",
        id: componentObj.id
      }, /*#__PURE__*/React.createElement(Component, componentProps))) : '');
    }
  }]);
}(React.Component);
{
  PopupController;
}
BrowserAction.AddTimeInputComponent = ContentScript.v2.Components.AddTimeInput.extend({
    template: _.template(`
        <% if (!isPeriodMode) { %>
            <div class="add-time__ctrl">
                <label class="ev-time-label field__label" for="ev_time">Time</label>
                <div class="add-time__ctrl-wrapper">
                    <div class="add-time__ctrl-down" (click)="changeTime('minus')"></div>
                    <input type="text" id="ev_time" class="time-input field__input add-time__ctrl-input" name="time" placeholder="0h"/>
                    <div class="add-time__ctrl-up" (click)="changeTime('plus')"></div>
                </div>
            </div>
        <% } else { %>
            <div class="add-time__period">
                <label class="ev-time-label field__label" for="ev_time_from">Time</label>
                <div class="add-time__period-wrapper">
                    <input 
                        type="text" 
                        id="ev_time_from" 
                        class="time-input field__input <%- is12TimeFormat ? 'meridiem' : '' %> add-time__period-start-input" 
                        name="time_from" 
                        autocomplete="off"
                        tabindex="1"
                    />
                    <span>-</span>
                    <input 
                        type="text" 
                        id="ev_time_to" 
                        class="time-input field__input <%- is12TimeFormat ? 'meridiem' : '' %> add-time__period-end-input"
                        name="time_to"
                        autocomplete="off"
                        tabindex="2"
                    />
                </div>
            </div>
        <% } %>
    `)
})
BrowserAction.AddTimeComponent = ContentScript.v2.Components.AddTime.extend({
    selector: '.ev-time-add',
    anchor: '.everhour-time-add-holder',
    placement: Component.APPEND,
    elementConstruct: '<div class="ev-time-add"></div>',
    dateFormat: 'MM/DD/YYYY',
    withUserManagement: false,
    components: {
        addTimeInput: BrowserAction.AddTimeInputComponent
    },

    template: _.template(`
        <div class="add-time">
            <form class="ev-form" name="everhourAddTimeForm">
                <% if (this.isAddTimeForAbstractTask && this.isEditTime) { %>
                    <div class="add-time__row add-time__row-tasks-select" id="tasks-select"></div>
                <% } %>
                <div class="add-time__row">
                    <div class="everhour-time-input-component"></div>
                    <div class="add-time__date">
                        <label class="field__label" for="input-date">Date</label>
                        <div class="add-time__date-wrapper everhour-datepicker">
                            <input type="text" 
                                id="input-date" 
                                class="field__input add-time__date-input" 
                                autocomplete="off" 
                                name="date" 
                                readonly="readonly" 
                                value="Today"/>
                            <div class="icon-calendar"></div>
                        </div>
                    </div>
                </div>
                <div class="add-time__row add-time__row-period">
                    <label class="checkbox checkbox--slider" (click)="toggleInputMode()">
                        <span class="slider__label">
                            Enter time as start/end
                        </span>
                        <input type="checkbox"/>
                        <span class="toggle-with-icon slider round" (click)="toggleInputMode()"></span>
                    </label>
                </div>
                <% if (this.isAddTimeForAbstractTask && !this.isEditTime) { %>
                    <div class="add-time__row add-time__row-tasks-select" id="tasks-select"></div>
                <% } %>
                <div class="add-time__row add-time__row-comment">
                    <div class="add-time__comment">
                        <label class="field__label" for="textarea-comment">Comments</label>
                        <div class="add-time__comment-wrapper">
                            <textarea 
                                id="textarea-comment" 
                                class="comment field__textarea add-time__comment-textarea" 
                                name="commentLine" 
                                placeholder="Explain your progress…"></textarea>
                        </div>
                    </div>
                </div>
            </form>
        </div>
    `),

    beforeRender: function() {
        if (!this.users && this.isUserSelectionAllowed && this.withUserManagement) {
            let users = ContentScript.v2.Components.ManageEstimates.prototype.getTaskUsers.apply(this);
            this.users = this.getSortedUserList(Object.keys(users).length > 0 ? users : this.allUsers);
        }
        this.isPeriodMode = this.getComponent('addTimeInput') ?
            this.getComponent('addTimeInput').isPeriodMode : false;
    },

    buildForm: function () {
        let controls = {
            date: DateControl(),
            commentLine: CommentControl()
        };
        if (this.isPeriodMode) {
            controls['time_from'] = DayTimeControl();
            controls['time_to'] = DayTimeControl();
        } else {
            controls['time'] = TimeControl();
        }

        this.form = new Form(this.element.querySelector('form'), controls);
    },

    onModeToggled: function() {
        const periodCheckboxElement = this.element.querySelector('.add-time__row-period input[type="checkbox"]');

        periodCheckboxElement.checked = this.isPeriodMode;
    },

    getCommentElement: function() {
        return this.element.querySelector('.add-time__row.add-time__row-comment');
    },

    getCommentField: function() {
        return this.element.querySelector('.add-time__row.add-time__row-comment').querySelector('textarea');
    },

    hideComment: function(commentValue) {
        const comment = this.getCommentElement();
        const commentField = this.getCommentField();

        commentField.value = commentValue;
        comment.style.display = 'none';
    },

    showComment: function() {
        const comment = this.getCommentElement();
        const commentField = this.getCommentField();

        commentField.value = '';
        comment.style.display = 'flex';
    },

    save: function() {
        if (this.lockActions) {
            return false;
        }

        if (!this.form.valid()) {
            return $.Deferred().reject();
        }

        let formValue = this.form.value(), totalTime;
        if (this.withUserManagement && this.isUserSelectionAllowed) {
            if (!this.selectedUser) {
                throw Error('form should provide user selection elements')
            }
            formValue.user = this.selectedUser.id;
        }

        if (!this.isPeriodMode) {
            totalTime = formValue.time;
        } else {
            if (
                !formValue.time_from && !formValue.time_to ||
                formValue.time_from === formValue.time_to
            ) {
                totalTime = ContentScript.Constants.DAILY_TIME;
            } else if (formValue.time_from > formValue.time_to) {
                // case for input like 19.00 to 3.00
                totalTime = ContentScript.Constants.DAILY_TIME - formValue.time_from + formValue.time_to;
            } else {
                totalTime = formValue.time_to - formValue.time_from;
            }
        }

        this.lockActions = true;
        this.beforeSave();

        if (this.isEditTime && this.taskTimeId) {
            let request = {
                date: formValue.date,
                time: totalTime,
                user: store.profile.get().id,
                comment: formValue.commentLine || null
            };
            let isTaskВifferent = (this.taskId && this.task && this.task.id) && (this.taskId !== this.task.id);

            if (isTaskВifferent && this.task && this.task.id) {
                request.task = this.task && this.task.id
            }

            browserPort.analytics.event('time_edited');
            return store.taskTime.updateTaskTIme.call(null, this.taskTimeId, request).done(() => {
                if (this.isUserSelectionAllowed && this.withUserManagement) {
                    this.selectedUser = this.currentUser;
                }

                this.afterSave();
            }).fail((response) => this.onError(response));
        }

        browserPort.analytics.event('time_added');
        return store.taskTime.createTaskTime.call(null, {
            date: formValue.date,
            time: totalTime,
            user: store.profile.get().id,
            comment: formValue.commentLine || null,
            task: this.task && this.task.id || null
        }).done(() => {
            if (this.isUserSelectionAllowed && this.withUserManagement) {
                this.selectedUser = this.currentUser;
            }

            this.afterSave();
        }).fail((response) => this.onError(response));
    },
});

$(document).on('DOMContentLoaded', function () {
    window.browserPort = new BrowserPort('browser-button');
    window.app = new BrowserAction.Application();
});

$.fn.setCursorPosition = function (pos) {
    this.each(function (index, elem) {
        if (elem.setSelectionRange) {
            elem.setSelectionRange(pos, pos);
        } else if (elem.createTextRange) {
            var range = elem.createTextRange();
            range.collapse(true);
            range.moveEnd('character', pos);
            range.moveStart('character', pos);
            range.select();
        }
    });
    return this;
};
